#region --- License ---
/*
Copyright 2010 Daniel Filonik

Permission is hereby granted, free of charge, to any person obtaining a copy of
this software and associated documentation files (the "Software"), to deal in
the Software without restriction, including without limitation the rights to
use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
of the Software, and to permit persons to whom the Software is furnished to do
so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all
copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
SOFTWARE.
*/
#endregion

namespace TKM {
    using System;
    
    
    public partial class dmath {
        
        public static double LengthSquared(dvec2 v0) {
            return dmath.MagnitudeSquared(ref v0);
        }
        
        public static double LengthSquared(ref dvec2 v0) {
            return dmath.MagnitudeSquared(ref v0);
        }
        
        public static double Length(dvec2 v0) {
            return dmath.Magnitude(ref v0);
        }
        
        public static double Length(ref dvec2 v0) {
            return dmath.Magnitude(ref v0);
        }
        
        public static double DistanceSquared(dvec2 v0, dvec2 v1) {
            return dmath.MagnitudeSquared(v1 - v0);
        }
        
        public static double DistanceSquared(ref dvec2 v0, ref dvec2 v1) {
            return dmath.MagnitudeSquared(v1 - v0);
        }
        
        public static double Distance(dvec2 v0, dvec2 v1) {
            return dmath.Magnitude(v1 - v0);
        }
        
        public static double Distance(ref dvec2 v0, ref dvec2 v1) {
            return dmath.Magnitude(v1 - v0);
        }
        
        public static double Dot(dvec2 t0, dvec2 t1) {
            return dmath.InnerProduct1(ref t0, ref t1);
        }
        
        public static double Dot(ref dvec2 t0, ref dvec2 t1) {
            return dmath.InnerProduct1(ref t0, ref t1);
        }
        
        public static dvec2 Dot(dvec2 t0, dmat2 t1) {
            return dmath.InnerProduct1(ref t0, ref t1);
        }
        
        public static dvec2 Dot(ref dvec2 t0, ref dmat2 t1) {
            return dmath.InnerProduct1(ref t0, ref t1);
        }
        
        public static dvec3 Dot(dvec2 t0, dmat2x3 t1) {
            return dmath.InnerProduct1(ref t0, ref t1);
        }
        
        public static dvec3 Dot(ref dvec2 t0, ref dmat2x3 t1) {
            return dmath.InnerProduct1(ref t0, ref t1);
        }
        
        public static dvec4 Dot(dvec2 t0, dmat2x4 t1) {
            return dmath.InnerProduct1(ref t0, ref t1);
        }
        
        public static dvec4 Dot(ref dvec2 t0, ref dmat2x4 t1) {
            return dmath.InnerProduct1(ref t0, ref t1);
        }
        
        public static dvec2 Cross(dvec2 v0) {
            return new dvec2((-1 * v0.v1), (v0.v0));
        }
        
        public static dvec2 Cross(ref dvec2 v0) {
            return new dvec2((-1 * v0.v1), (v0.v0));
        }
        
        public static dvec2 Normalize(dvec2 v0) {
            dmath.Divide(ref v0, ((double)(dmath.Magnitude(ref v0))), out v0);
            return v0;
        }
        
        public static dvec2 Normalize(ref dvec2 v0) {
            dmath.Divide(ref v0, ((double)(dmath.Magnitude(ref v0))), out v0);
            return v0;
        }
        
        public static dvec2 FaceForward(dvec2 v0, dvec2 v1, dvec2 v2) {
            if ((dmath.InnerProduct1(ref v2, ref v1) < 0D)) {
                return v0;
            }
            else {
                return -v0;
            }
        }
        
        public static dvec2 FaceForward(ref dvec2 v0, ref dvec2 v1, ref dvec2 v2) {
            if ((dmath.InnerProduct1(ref v2, ref v1) < 0D)) {
                return v0;
            }
            else {
                return -v0;
            }
        }
        
        public static dvec2 Reflect(dvec2 v0, dvec2 v1) {
            return (v0 
                        - ((dmath.InnerProduct1(ref v1, ref v0) * 2D) 
                        * v1));
        }
        
        public static dvec2 Reflect(ref dvec2 v0, ref dvec2 v1) {
            return (v0 
                        - ((dmath.InnerProduct1(ref v1, ref v0) * 2D) 
                        * v1));
        }
        
        public static dvec2 Refract(dvec2 v0, dvec2 v1, double eta) {
            double d = dmath.InnerProduct1(ref v1, ref v0);
            double k = (1D 
                        - ((eta * eta) 
                        * (1D 
                        - (d * d))));
            if ((k < 0D)) {
                return new dvec2();
            }
            else {
                return ((eta * v0) 
                            - (((double)(((eta * d) 
                            + System.Math.Sqrt(k)))) * v1));
            }
        }
        
        public static dvec2 Refract(ref dvec2 v0, ref dvec2 v1, double eta) {
            double d = dmath.InnerProduct1(ref v1, ref v0);
            double k = (1D 
                        - ((eta * eta) 
                        * (1D 
                        - (d * d))));
            if ((k < 0D)) {
                return new dvec2();
            }
            else {
                return ((eta * v0) 
                            - (((double)(((eta * d) 
                            + System.Math.Sqrt(k)))) * v1));
            }
        }
        
        public static double LengthSquared(dvec3 v0) {
            return dmath.MagnitudeSquared(ref v0);
        }
        
        public static double LengthSquared(ref dvec3 v0) {
            return dmath.MagnitudeSquared(ref v0);
        }
        
        public static double Length(dvec3 v0) {
            return dmath.Magnitude(ref v0);
        }
        
        public static double Length(ref dvec3 v0) {
            return dmath.Magnitude(ref v0);
        }
        
        public static double DistanceSquared(dvec3 v0, dvec3 v1) {
            return dmath.MagnitudeSquared(v1 - v0);
        }
        
        public static double DistanceSquared(ref dvec3 v0, ref dvec3 v1) {
            return dmath.MagnitudeSquared(v1 - v0);
        }
        
        public static double Distance(dvec3 v0, dvec3 v1) {
            return dmath.Magnitude(v1 - v0);
        }
        
        public static double Distance(ref dvec3 v0, ref dvec3 v1) {
            return dmath.Magnitude(v1 - v0);
        }
        
        public static double Dot(dvec3 t0, dvec3 t1) {
            return dmath.InnerProduct1(ref t0, ref t1);
        }
        
        public static double Dot(ref dvec3 t0, ref dvec3 t1) {
            return dmath.InnerProduct1(ref t0, ref t1);
        }
        
        public static dvec2 Dot(dvec3 t0, dmat3x2 t1) {
            return dmath.InnerProduct1(ref t0, ref t1);
        }
        
        public static dvec2 Dot(ref dvec3 t0, ref dmat3x2 t1) {
            return dmath.InnerProduct1(ref t0, ref t1);
        }
        
        public static dvec3 Dot(dvec3 t0, dmat3 t1) {
            return dmath.InnerProduct1(ref t0, ref t1);
        }
        
        public static dvec3 Dot(ref dvec3 t0, ref dmat3 t1) {
            return dmath.InnerProduct1(ref t0, ref t1);
        }
        
        public static dvec4 Dot(dvec3 t0, dmat3x4 t1) {
            return dmath.InnerProduct1(ref t0, ref t1);
        }
        
        public static dvec4 Dot(ref dvec3 t0, ref dmat3x4 t1) {
            return dmath.InnerProduct1(ref t0, ref t1);
        }
        
        public static dvec3 Cross(dvec3 v0, dvec3 v1) {
            return new dvec3((v0.v1 * v1.v2) + (-1 * v0.v2 * v1.v1), (-1 * v0.v0 * v1.v2) + (v0.v2 * v1.v0), (v0.v0 * v1.v1) + (-1 * v0.v1 * v1.v0));
        }
        
        public static dvec3 Cross(ref dvec3 v0, ref dvec3 v1) {
            return new dvec3((v0.v1 * v1.v2) + (-1 * v0.v2 * v1.v1), (-1 * v0.v0 * v1.v2) + (v0.v2 * v1.v0), (v0.v0 * v1.v1) + (-1 * v0.v1 * v1.v0));
        }
        
        public static dvec3 Normalize(dvec3 v0) {
            dmath.Divide(ref v0, ((double)(dmath.Magnitude(ref v0))), out v0);
            return v0;
        }
        
        public static dvec3 Normalize(ref dvec3 v0) {
            dmath.Divide(ref v0, ((double)(dmath.Magnitude(ref v0))), out v0);
            return v0;
        }
        
        public static dvec3 FaceForward(dvec3 v0, dvec3 v1, dvec3 v2) {
            if ((dmath.InnerProduct1(ref v2, ref v1) < 0D)) {
                return v0;
            }
            else {
                return -v0;
            }
        }
        
        public static dvec3 FaceForward(ref dvec3 v0, ref dvec3 v1, ref dvec3 v2) {
            if ((dmath.InnerProduct1(ref v2, ref v1) < 0D)) {
                return v0;
            }
            else {
                return -v0;
            }
        }
        
        public static dvec3 Reflect(dvec3 v0, dvec3 v1) {
            return (v0 
                        - ((dmath.InnerProduct1(ref v1, ref v0) * 2D) 
                        * v1));
        }
        
        public static dvec3 Reflect(ref dvec3 v0, ref dvec3 v1) {
            return (v0 
                        - ((dmath.InnerProduct1(ref v1, ref v0) * 2D) 
                        * v1));
        }
        
        public static dvec3 Refract(dvec3 v0, dvec3 v1, double eta) {
            double d = dmath.InnerProduct1(ref v1, ref v0);
            double k = (1D 
                        - ((eta * eta) 
                        * (1D 
                        - (d * d))));
            if ((k < 0D)) {
                return new dvec3();
            }
            else {
                return ((eta * v0) 
                            - (((double)(((eta * d) 
                            + System.Math.Sqrt(k)))) * v1));
            }
        }
        
        public static dvec3 Refract(ref dvec3 v0, ref dvec3 v1, double eta) {
            double d = dmath.InnerProduct1(ref v1, ref v0);
            double k = (1D 
                        - ((eta * eta) 
                        * (1D 
                        - (d * d))));
            if ((k < 0D)) {
                return new dvec3();
            }
            else {
                return ((eta * v0) 
                            - (((double)(((eta * d) 
                            + System.Math.Sqrt(k)))) * v1));
            }
        }
        
        public static double LengthSquared(dvec4 v0) {
            return dmath.MagnitudeSquared(ref v0);
        }
        
        public static double LengthSquared(ref dvec4 v0) {
            return dmath.MagnitudeSquared(ref v0);
        }
        
        public static double Length(dvec4 v0) {
            return dmath.Magnitude(ref v0);
        }
        
        public static double Length(ref dvec4 v0) {
            return dmath.Magnitude(ref v0);
        }
        
        public static double DistanceSquared(dvec4 v0, dvec4 v1) {
            return dmath.MagnitudeSquared(v1 - v0);
        }
        
        public static double DistanceSquared(ref dvec4 v0, ref dvec4 v1) {
            return dmath.MagnitudeSquared(v1 - v0);
        }
        
        public static double Distance(dvec4 v0, dvec4 v1) {
            return dmath.Magnitude(v1 - v0);
        }
        
        public static double Distance(ref dvec4 v0, ref dvec4 v1) {
            return dmath.Magnitude(v1 - v0);
        }
        
        public static double Dot(dvec4 t0, dvec4 t1) {
            return dmath.InnerProduct1(ref t0, ref t1);
        }
        
        public static double Dot(ref dvec4 t0, ref dvec4 t1) {
            return dmath.InnerProduct1(ref t0, ref t1);
        }
        
        public static dvec2 Dot(dvec4 t0, dmat4x2 t1) {
            return dmath.InnerProduct1(ref t0, ref t1);
        }
        
        public static dvec2 Dot(ref dvec4 t0, ref dmat4x2 t1) {
            return dmath.InnerProduct1(ref t0, ref t1);
        }
        
        public static dvec3 Dot(dvec4 t0, dmat4x3 t1) {
            return dmath.InnerProduct1(ref t0, ref t1);
        }
        
        public static dvec3 Dot(ref dvec4 t0, ref dmat4x3 t1) {
            return dmath.InnerProduct1(ref t0, ref t1);
        }
        
        public static dvec4 Dot(dvec4 t0, dmat4 t1) {
            return dmath.InnerProduct1(ref t0, ref t1);
        }
        
        public static dvec4 Dot(ref dvec4 t0, ref dmat4 t1) {
            return dmath.InnerProduct1(ref t0, ref t1);
        }
        
        public static dvec4 Cross(dvec4 v0, dvec4 v1, dvec4 v2) {
            return new dvec4((-1 * v0.v1 * v1.v2 * v2.v3) + (v0.v1 * v1.v3 * v2.v2) + (v0.v2 * v1.v1 * v2.v3) + (-1 * v0.v2 * v1.v3 * v2.v1) + (-1 * v0.v3 * v1.v1 * v2.v2) + (v0.v3 * v1.v2 * v2.v1), (v0.v0 * v1.v2 * v2.v3) + (-1 * v0.v0 * v1.v3 * v2.v2) + (-1 * v0.v2 * v1.v0 * v2.v3) + (v0.v2 * v1.v3 * v2.v0) + (v0.v3 * v1.v0 * v2.v2) + (-1 * v0.v3 * v1.v2 * v2.v0), (-1 * v0.v0 * v1.v1 * v2.v3) + (v0.v0 * v1.v3 * v2.v1) + (v0.v1 * v1.v0 * v2.v3) + (-1 * v0.v1 * v1.v3 * v2.v0) + (-1 * v0.v3 * v1.v0 * v2.v1) + (v0.v3 * v1.v1 * v2.v0), (v0.v0 * v1.v1 * v2.v2) + (-1 * v0.v0 * v1.v2 * v2.v1) + (-1 * v0.v1 * v1.v0 * v2.v2) + (v0.v1 * v1.v2 * v2.v0) + (v0.v2 * v1.v0 * v2.v1) + (-1 * v0.v2 * v1.v1 * v2.v0));
        }
        
        public static dvec4 Cross(ref dvec4 v0, ref dvec4 v1, ref dvec4 v2) {
            return new dvec4((-1 * v0.v1 * v1.v2 * v2.v3) + (v0.v1 * v1.v3 * v2.v2) + (v0.v2 * v1.v1 * v2.v3) + (-1 * v0.v2 * v1.v3 * v2.v1) + (-1 * v0.v3 * v1.v1 * v2.v2) + (v0.v3 * v1.v2 * v2.v1), (v0.v0 * v1.v2 * v2.v3) + (-1 * v0.v0 * v1.v3 * v2.v2) + (-1 * v0.v2 * v1.v0 * v2.v3) + (v0.v2 * v1.v3 * v2.v0) + (v0.v3 * v1.v0 * v2.v2) + (-1 * v0.v3 * v1.v2 * v2.v0), (-1 * v0.v0 * v1.v1 * v2.v3) + (v0.v0 * v1.v3 * v2.v1) + (v0.v1 * v1.v0 * v2.v3) + (-1 * v0.v1 * v1.v3 * v2.v0) + (-1 * v0.v3 * v1.v0 * v2.v1) + (v0.v3 * v1.v1 * v2.v0), (v0.v0 * v1.v1 * v2.v2) + (-1 * v0.v0 * v1.v2 * v2.v1) + (-1 * v0.v1 * v1.v0 * v2.v2) + (v0.v1 * v1.v2 * v2.v0) + (v0.v2 * v1.v0 * v2.v1) + (-1 * v0.v2 * v1.v1 * v2.v0));
        }
        
        public static dvec4 Normalize(dvec4 v0) {
            dmath.Divide(ref v0, ((double)(dmath.Magnitude(ref v0))), out v0);
            return v0;
        }
        
        public static dvec4 Normalize(ref dvec4 v0) {
            dmath.Divide(ref v0, ((double)(dmath.Magnitude(ref v0))), out v0);
            return v0;
        }
        
        public static dvec4 FaceForward(dvec4 v0, dvec4 v1, dvec4 v2) {
            if ((dmath.InnerProduct1(ref v2, ref v1) < 0D)) {
                return v0;
            }
            else {
                return -v0;
            }
        }
        
        public static dvec4 FaceForward(ref dvec4 v0, ref dvec4 v1, ref dvec4 v2) {
            if ((dmath.InnerProduct1(ref v2, ref v1) < 0D)) {
                return v0;
            }
            else {
                return -v0;
            }
        }
        
        public static dvec4 Reflect(dvec4 v0, dvec4 v1) {
            return (v0 
                        - ((dmath.InnerProduct1(ref v1, ref v0) * 2D) 
                        * v1));
        }
        
        public static dvec4 Reflect(ref dvec4 v0, ref dvec4 v1) {
            return (v0 
                        - ((dmath.InnerProduct1(ref v1, ref v0) * 2D) 
                        * v1));
        }
        
        public static dvec4 Refract(dvec4 v0, dvec4 v1, double eta) {
            double d = dmath.InnerProduct1(ref v1, ref v0);
            double k = (1D 
                        - ((eta * eta) 
                        * (1D 
                        - (d * d))));
            if ((k < 0D)) {
                return new dvec4();
            }
            else {
                return ((eta * v0) 
                            - (((double)(((eta * d) 
                            + System.Math.Sqrt(k)))) * v1));
            }
        }
        
        public static dvec4 Refract(ref dvec4 v0, ref dvec4 v1, double eta) {
            double d = dmath.InnerProduct1(ref v1, ref v0);
            double k = (1D 
                        - ((eta * eta) 
                        * (1D 
                        - (d * d))));
            if ((k < 0D)) {
                return new dvec4();
            }
            else {
                return ((eta * v0) 
                            - (((double)(((eta * d) 
                            + System.Math.Sqrt(k)))) * v1));
            }
        }
    }
}
