#region --- License ---
/*
Copyright 2010 Daniel Filonik

Permission is hereby granted, free of charge, to any person obtaining a copy of
this software and associated documentation files (the "Software"), to deal in
the Software without restriction, including without limitation the rights to
use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
of the Software, and to permit persons to whom the Software is furnished to do
so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all
copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
SOFTWARE.
*/
#endregion

namespace TKM {
    using System;
    
    
    public partial class imath {
        
        public static int LengthSquared(ivec2 v0) {
            return imath.MagnitudeSquared(ref v0);
        }
        
        public static int LengthSquared(ref ivec2 v0) {
            return imath.MagnitudeSquared(ref v0);
        }
        
        public static double Length(ivec2 v0) {
            return imath.Magnitude(ref v0);
        }
        
        public static double Length(ref ivec2 v0) {
            return imath.Magnitude(ref v0);
        }
        
        public static int DistanceSquared(ivec2 v0, ivec2 v1) {
            return imath.MagnitudeSquared(v1 - v0);
        }
        
        public static int DistanceSquared(ref ivec2 v0, ref ivec2 v1) {
            return imath.MagnitudeSquared(v1 - v0);
        }
        
        public static double Distance(ivec2 v0, ivec2 v1) {
            return imath.Magnitude(v1 - v0);
        }
        
        public static double Distance(ref ivec2 v0, ref ivec2 v1) {
            return imath.Magnitude(v1 - v0);
        }
        
        public static int Dot(ivec2 t0, ivec2 t1) {
            return imath.InnerProduct1(ref t0, ref t1);
        }
        
        public static int Dot(ref ivec2 t0, ref ivec2 t1) {
            return imath.InnerProduct1(ref t0, ref t1);
        }
        
        public static ivec2 Dot(ivec2 t0, imat2 t1) {
            return imath.InnerProduct1(ref t0, ref t1);
        }
        
        public static ivec2 Dot(ref ivec2 t0, ref imat2 t1) {
            return imath.InnerProduct1(ref t0, ref t1);
        }
        
        public static ivec3 Dot(ivec2 t0, imat2x3 t1) {
            return imath.InnerProduct1(ref t0, ref t1);
        }
        
        public static ivec3 Dot(ref ivec2 t0, ref imat2x3 t1) {
            return imath.InnerProduct1(ref t0, ref t1);
        }
        
        public static ivec4 Dot(ivec2 t0, imat2x4 t1) {
            return imath.InnerProduct1(ref t0, ref t1);
        }
        
        public static ivec4 Dot(ref ivec2 t0, ref imat2x4 t1) {
            return imath.InnerProduct1(ref t0, ref t1);
        }
        
        public static ivec2 Cross(ivec2 v0) {
            return new ivec2((-1 * v0.v1), (v0.v0));
        }
        
        public static ivec2 Cross(ref ivec2 v0) {
            return new ivec2((-1 * v0.v1), (v0.v0));
        }
        
        public static ivec2 FaceForward(ivec2 v0, ivec2 v1, ivec2 v2) {
            if ((imath.InnerProduct1(ref v2, ref v1) < 0)) {
                return v0;
            }
            else {
                return -v0;
            }
        }
        
        public static ivec2 FaceForward(ref ivec2 v0, ref ivec2 v1, ref ivec2 v2) {
            if ((imath.InnerProduct1(ref v2, ref v1) < 0)) {
                return v0;
            }
            else {
                return -v0;
            }
        }
        
        public static ivec2 Reflect(ivec2 v0, ivec2 v1) {
            return (v0 
                        - ((imath.InnerProduct1(ref v1, ref v0) * 2) 
                        * v1));
        }
        
        public static ivec2 Reflect(ref ivec2 v0, ref ivec2 v1) {
            return (v0 
                        - ((imath.InnerProduct1(ref v1, ref v0) * 2) 
                        * v1));
        }
        
        public static int LengthSquared(ivec3 v0) {
            return imath.MagnitudeSquared(ref v0);
        }
        
        public static int LengthSquared(ref ivec3 v0) {
            return imath.MagnitudeSquared(ref v0);
        }
        
        public static double Length(ivec3 v0) {
            return imath.Magnitude(ref v0);
        }
        
        public static double Length(ref ivec3 v0) {
            return imath.Magnitude(ref v0);
        }
        
        public static int DistanceSquared(ivec3 v0, ivec3 v1) {
            return imath.MagnitudeSquared(v1 - v0);
        }
        
        public static int DistanceSquared(ref ivec3 v0, ref ivec3 v1) {
            return imath.MagnitudeSquared(v1 - v0);
        }
        
        public static double Distance(ivec3 v0, ivec3 v1) {
            return imath.Magnitude(v1 - v0);
        }
        
        public static double Distance(ref ivec3 v0, ref ivec3 v1) {
            return imath.Magnitude(v1 - v0);
        }
        
        public static int Dot(ivec3 t0, ivec3 t1) {
            return imath.InnerProduct1(ref t0, ref t1);
        }
        
        public static int Dot(ref ivec3 t0, ref ivec3 t1) {
            return imath.InnerProduct1(ref t0, ref t1);
        }
        
        public static ivec2 Dot(ivec3 t0, imat3x2 t1) {
            return imath.InnerProduct1(ref t0, ref t1);
        }
        
        public static ivec2 Dot(ref ivec3 t0, ref imat3x2 t1) {
            return imath.InnerProduct1(ref t0, ref t1);
        }
        
        public static ivec3 Dot(ivec3 t0, imat3 t1) {
            return imath.InnerProduct1(ref t0, ref t1);
        }
        
        public static ivec3 Dot(ref ivec3 t0, ref imat3 t1) {
            return imath.InnerProduct1(ref t0, ref t1);
        }
        
        public static ivec4 Dot(ivec3 t0, imat3x4 t1) {
            return imath.InnerProduct1(ref t0, ref t1);
        }
        
        public static ivec4 Dot(ref ivec3 t0, ref imat3x4 t1) {
            return imath.InnerProduct1(ref t0, ref t1);
        }
        
        public static ivec3 Cross(ivec3 v0, ivec3 v1) {
            return new ivec3((v0.v1 * v1.v2) + (-1 * v0.v2 * v1.v1), (-1 * v0.v0 * v1.v2) + (v0.v2 * v1.v0), (v0.v0 * v1.v1) + (-1 * v0.v1 * v1.v0));
        }
        
        public static ivec3 Cross(ref ivec3 v0, ref ivec3 v1) {
            return new ivec3((v0.v1 * v1.v2) + (-1 * v0.v2 * v1.v1), (-1 * v0.v0 * v1.v2) + (v0.v2 * v1.v0), (v0.v0 * v1.v1) + (-1 * v0.v1 * v1.v0));
        }
        
        public static ivec3 FaceForward(ivec3 v0, ivec3 v1, ivec3 v2) {
            if ((imath.InnerProduct1(ref v2, ref v1) < 0)) {
                return v0;
            }
            else {
                return -v0;
            }
        }
        
        public static ivec3 FaceForward(ref ivec3 v0, ref ivec3 v1, ref ivec3 v2) {
            if ((imath.InnerProduct1(ref v2, ref v1) < 0)) {
                return v0;
            }
            else {
                return -v0;
            }
        }
        
        public static ivec3 Reflect(ivec3 v0, ivec3 v1) {
            return (v0 
                        - ((imath.InnerProduct1(ref v1, ref v0) * 2) 
                        * v1));
        }
        
        public static ivec3 Reflect(ref ivec3 v0, ref ivec3 v1) {
            return (v0 
                        - ((imath.InnerProduct1(ref v1, ref v0) * 2) 
                        * v1));
        }
        
        public static int LengthSquared(ivec4 v0) {
            return imath.MagnitudeSquared(ref v0);
        }
        
        public static int LengthSquared(ref ivec4 v0) {
            return imath.MagnitudeSquared(ref v0);
        }
        
        public static double Length(ivec4 v0) {
            return imath.Magnitude(ref v0);
        }
        
        public static double Length(ref ivec4 v0) {
            return imath.Magnitude(ref v0);
        }
        
        public static int DistanceSquared(ivec4 v0, ivec4 v1) {
            return imath.MagnitudeSquared(v1 - v0);
        }
        
        public static int DistanceSquared(ref ivec4 v0, ref ivec4 v1) {
            return imath.MagnitudeSquared(v1 - v0);
        }
        
        public static double Distance(ivec4 v0, ivec4 v1) {
            return imath.Magnitude(v1 - v0);
        }
        
        public static double Distance(ref ivec4 v0, ref ivec4 v1) {
            return imath.Magnitude(v1 - v0);
        }
        
        public static int Dot(ivec4 t0, ivec4 t1) {
            return imath.InnerProduct1(ref t0, ref t1);
        }
        
        public static int Dot(ref ivec4 t0, ref ivec4 t1) {
            return imath.InnerProduct1(ref t0, ref t1);
        }
        
        public static ivec2 Dot(ivec4 t0, imat4x2 t1) {
            return imath.InnerProduct1(ref t0, ref t1);
        }
        
        public static ivec2 Dot(ref ivec4 t0, ref imat4x2 t1) {
            return imath.InnerProduct1(ref t0, ref t1);
        }
        
        public static ivec3 Dot(ivec4 t0, imat4x3 t1) {
            return imath.InnerProduct1(ref t0, ref t1);
        }
        
        public static ivec3 Dot(ref ivec4 t0, ref imat4x3 t1) {
            return imath.InnerProduct1(ref t0, ref t1);
        }
        
        public static ivec4 Dot(ivec4 t0, imat4 t1) {
            return imath.InnerProduct1(ref t0, ref t1);
        }
        
        public static ivec4 Dot(ref ivec4 t0, ref imat4 t1) {
            return imath.InnerProduct1(ref t0, ref t1);
        }
        
        public static ivec4 Cross(ivec4 v0, ivec4 v1, ivec4 v2) {
            return new ivec4((-1 * v0.v1 * v1.v2 * v2.v3) + (v0.v1 * v1.v3 * v2.v2) + (v0.v2 * v1.v1 * v2.v3) + (-1 * v0.v2 * v1.v3 * v2.v1) + (-1 * v0.v3 * v1.v1 * v2.v2) + (v0.v3 * v1.v2 * v2.v1), (v0.v0 * v1.v2 * v2.v3) + (-1 * v0.v0 * v1.v3 * v2.v2) + (-1 * v0.v2 * v1.v0 * v2.v3) + (v0.v2 * v1.v3 * v2.v0) + (v0.v3 * v1.v0 * v2.v2) + (-1 * v0.v3 * v1.v2 * v2.v0), (-1 * v0.v0 * v1.v1 * v2.v3) + (v0.v0 * v1.v3 * v2.v1) + (v0.v1 * v1.v0 * v2.v3) + (-1 * v0.v1 * v1.v3 * v2.v0) + (-1 * v0.v3 * v1.v0 * v2.v1) + (v0.v3 * v1.v1 * v2.v0), (v0.v0 * v1.v1 * v2.v2) + (-1 * v0.v0 * v1.v2 * v2.v1) + (-1 * v0.v1 * v1.v0 * v2.v2) + (v0.v1 * v1.v2 * v2.v0) + (v0.v2 * v1.v0 * v2.v1) + (-1 * v0.v2 * v1.v1 * v2.v0));
        }
        
        public static ivec4 Cross(ref ivec4 v0, ref ivec4 v1, ref ivec4 v2) {
            return new ivec4((-1 * v0.v1 * v1.v2 * v2.v3) + (v0.v1 * v1.v3 * v2.v2) + (v0.v2 * v1.v1 * v2.v3) + (-1 * v0.v2 * v1.v3 * v2.v1) + (-1 * v0.v3 * v1.v1 * v2.v2) + (v0.v3 * v1.v2 * v2.v1), (v0.v0 * v1.v2 * v2.v3) + (-1 * v0.v0 * v1.v3 * v2.v2) + (-1 * v0.v2 * v1.v0 * v2.v3) + (v0.v2 * v1.v3 * v2.v0) + (v0.v3 * v1.v0 * v2.v2) + (-1 * v0.v3 * v1.v2 * v2.v0), (-1 * v0.v0 * v1.v1 * v2.v3) + (v0.v0 * v1.v3 * v2.v1) + (v0.v1 * v1.v0 * v2.v3) + (-1 * v0.v1 * v1.v3 * v2.v0) + (-1 * v0.v3 * v1.v0 * v2.v1) + (v0.v3 * v1.v1 * v2.v0), (v0.v0 * v1.v1 * v2.v2) + (-1 * v0.v0 * v1.v2 * v2.v1) + (-1 * v0.v1 * v1.v0 * v2.v2) + (v0.v1 * v1.v2 * v2.v0) + (v0.v2 * v1.v0 * v2.v1) + (-1 * v0.v2 * v1.v1 * v2.v0));
        }
        
        public static ivec4 FaceForward(ivec4 v0, ivec4 v1, ivec4 v2) {
            if ((imath.InnerProduct1(ref v2, ref v1) < 0)) {
                return v0;
            }
            else {
                return -v0;
            }
        }
        
        public static ivec4 FaceForward(ref ivec4 v0, ref ivec4 v1, ref ivec4 v2) {
            if ((imath.InnerProduct1(ref v2, ref v1) < 0)) {
                return v0;
            }
            else {
                return -v0;
            }
        }
        
        public static ivec4 Reflect(ivec4 v0, ivec4 v1) {
            return (v0 
                        - ((imath.InnerProduct1(ref v1, ref v0) * 2) 
                        * v1));
        }
        
        public static ivec4 Reflect(ref ivec4 v0, ref ivec4 v1) {
            return (v0 
                        - ((imath.InnerProduct1(ref v1, ref v0) * 2) 
                        * v1));
        }
    }
}
