#region --- License ---
/*
Copyright 2010 Daniel Filonik

Permission is hereby granted, free of charge, to any person obtaining a copy of
this software and associated documentation files (the "Software"), to deal in
the Software without restriction, including without limitation the rights to
use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
of the Software, and to permit persons to whom the Software is furnished to do
so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all
copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
SOFTWARE.
*/
#endregion

namespace TKM {
    using System;
    
    
    public partial class math {
        
        public static float LengthSquared(vec2 v0) {
            return math.MagnitudeSquared(ref v0);
        }
        
        public static float LengthSquared(ref vec2 v0) {
            return math.MagnitudeSquared(ref v0);
        }
        
        public static double Length(vec2 v0) {
            return math.Magnitude(ref v0);
        }
        
        public static double Length(ref vec2 v0) {
            return math.Magnitude(ref v0);
        }
        
        public static float DistanceSquared(vec2 v0, vec2 v1) {
            return math.MagnitudeSquared(v1 - v0);
        }
        
        public static float DistanceSquared(ref vec2 v0, ref vec2 v1) {
            return math.MagnitudeSquared(v1 - v0);
        }
        
        public static double Distance(vec2 v0, vec2 v1) {
            return math.Magnitude(v1 - v0);
        }
        
        public static double Distance(ref vec2 v0, ref vec2 v1) {
            return math.Magnitude(v1 - v0);
        }
        
        public static float Dot(vec2 t0, vec2 t1) {
            return math.InnerProduct1(ref t0, ref t1);
        }
        
        public static float Dot(ref vec2 t0, ref vec2 t1) {
            return math.InnerProduct1(ref t0, ref t1);
        }
        
        public static vec2 Dot(vec2 t0, mat2 t1) {
            return math.InnerProduct1(ref t0, ref t1);
        }
        
        public static vec2 Dot(ref vec2 t0, ref mat2 t1) {
            return math.InnerProduct1(ref t0, ref t1);
        }
        
        public static vec3 Dot(vec2 t0, mat2x3 t1) {
            return math.InnerProduct1(ref t0, ref t1);
        }
        
        public static vec3 Dot(ref vec2 t0, ref mat2x3 t1) {
            return math.InnerProduct1(ref t0, ref t1);
        }
        
        public static vec4 Dot(vec2 t0, mat2x4 t1) {
            return math.InnerProduct1(ref t0, ref t1);
        }
        
        public static vec4 Dot(ref vec2 t0, ref mat2x4 t1) {
            return math.InnerProduct1(ref t0, ref t1);
        }
        
        public static vec2 Cross(vec2 v0) {
            return new vec2((-1 * v0.v1), (v0.v0));
        }
        
        public static vec2 Cross(ref vec2 v0) {
            return new vec2((-1 * v0.v1), (v0.v0));
        }
        
        public static vec2 Normalize(vec2 v0) {
            math.Divide(ref v0, ((float)(math.Magnitude(ref v0))), out v0);
            return v0;
        }
        
        public static vec2 Normalize(ref vec2 v0) {
            math.Divide(ref v0, ((float)(math.Magnitude(ref v0))), out v0);
            return v0;
        }
        
        public static vec2 FaceForward(vec2 v0, vec2 v1, vec2 v2) {
            if ((math.InnerProduct1(ref v2, ref v1) < 0F)) {
                return v0;
            }
            else {
                return -v0;
            }
        }
        
        public static vec2 FaceForward(ref vec2 v0, ref vec2 v1, ref vec2 v2) {
            if ((math.InnerProduct1(ref v2, ref v1) < 0F)) {
                return v0;
            }
            else {
                return -v0;
            }
        }
        
        public static vec2 Reflect(vec2 v0, vec2 v1) {
            return (v0 
                        - ((math.InnerProduct1(ref v1, ref v0) * 2F) 
                        * v1));
        }
        
        public static vec2 Reflect(ref vec2 v0, ref vec2 v1) {
            return (v0 
                        - ((math.InnerProduct1(ref v1, ref v0) * 2F) 
                        * v1));
        }
        
        public static vec2 Refract(vec2 v0, vec2 v1, float eta) {
            float d = math.InnerProduct1(ref v1, ref v0);
            float k = (1F 
                        - ((eta * eta) 
                        * (1F 
                        - (d * d))));
            if ((k < 0F)) {
                return new vec2();
            }
            else {
                return ((eta * v0) 
                            - (((float)(((eta * d) 
                            + System.Math.Sqrt(k)))) * v1));
            }
        }
        
        public static vec2 Refract(ref vec2 v0, ref vec2 v1, float eta) {
            float d = math.InnerProduct1(ref v1, ref v0);
            float k = (1F 
                        - ((eta * eta) 
                        * (1F 
                        - (d * d))));
            if ((k < 0F)) {
                return new vec2();
            }
            else {
                return ((eta * v0) 
                            - (((float)(((eta * d) 
                            + System.Math.Sqrt(k)))) * v1));
            }
        }
        
        public static float LengthSquared(vec3 v0) {
            return math.MagnitudeSquared(ref v0);
        }
        
        public static float LengthSquared(ref vec3 v0) {
            return math.MagnitudeSquared(ref v0);
        }
        
        public static double Length(vec3 v0) {
            return math.Magnitude(ref v0);
        }
        
        public static double Length(ref vec3 v0) {
            return math.Magnitude(ref v0);
        }
        
        public static float DistanceSquared(vec3 v0, vec3 v1) {
            return math.MagnitudeSquared(v1 - v0);
        }
        
        public static float DistanceSquared(ref vec3 v0, ref vec3 v1) {
            return math.MagnitudeSquared(v1 - v0);
        }
        
        public static double Distance(vec3 v0, vec3 v1) {
            return math.Magnitude(v1 - v0);
        }
        
        public static double Distance(ref vec3 v0, ref vec3 v1) {
            return math.Magnitude(v1 - v0);
        }
        
        public static float Dot(vec3 t0, vec3 t1) {
            return math.InnerProduct1(ref t0, ref t1);
        }
        
        public static float Dot(ref vec3 t0, ref vec3 t1) {
            return math.InnerProduct1(ref t0, ref t1);
        }
        
        public static vec2 Dot(vec3 t0, mat3x2 t1) {
            return math.InnerProduct1(ref t0, ref t1);
        }
        
        public static vec2 Dot(ref vec3 t0, ref mat3x2 t1) {
            return math.InnerProduct1(ref t0, ref t1);
        }
        
        public static vec3 Dot(vec3 t0, mat3 t1) {
            return math.InnerProduct1(ref t0, ref t1);
        }
        
        public static vec3 Dot(ref vec3 t0, ref mat3 t1) {
            return math.InnerProduct1(ref t0, ref t1);
        }
        
        public static vec4 Dot(vec3 t0, mat3x4 t1) {
            return math.InnerProduct1(ref t0, ref t1);
        }
        
        public static vec4 Dot(ref vec3 t0, ref mat3x4 t1) {
            return math.InnerProduct1(ref t0, ref t1);
        }
        
        public static vec3 Cross(vec3 v0, vec3 v1) {
            return new vec3((v0.v1 * v1.v2) + (-1 * v0.v2 * v1.v1), (-1 * v0.v0 * v1.v2) + (v0.v2 * v1.v0), (v0.v0 * v1.v1) + (-1 * v0.v1 * v1.v0));
        }
        
        public static vec3 Cross(ref vec3 v0, ref vec3 v1) {
            return new vec3((v0.v1 * v1.v2) + (-1 * v0.v2 * v1.v1), (-1 * v0.v0 * v1.v2) + (v0.v2 * v1.v0), (v0.v0 * v1.v1) + (-1 * v0.v1 * v1.v0));
        }
        
        public static vec3 Normalize(vec3 v0) {
            math.Divide(ref v0, ((float)(math.Magnitude(ref v0))), out v0);
            return v0;
        }
        
        public static vec3 Normalize(ref vec3 v0) {
            math.Divide(ref v0, ((float)(math.Magnitude(ref v0))), out v0);
            return v0;
        }
        
        public static vec3 FaceForward(vec3 v0, vec3 v1, vec3 v2) {
            if ((math.InnerProduct1(ref v2, ref v1) < 0F)) {
                return v0;
            }
            else {
                return -v0;
            }
        }
        
        public static vec3 FaceForward(ref vec3 v0, ref vec3 v1, ref vec3 v2) {
            if ((math.InnerProduct1(ref v2, ref v1) < 0F)) {
                return v0;
            }
            else {
                return -v0;
            }
        }
        
        public static vec3 Reflect(vec3 v0, vec3 v1) {
            return (v0 
                        - ((math.InnerProduct1(ref v1, ref v0) * 2F) 
                        * v1));
        }
        
        public static vec3 Reflect(ref vec3 v0, ref vec3 v1) {
            return (v0 
                        - ((math.InnerProduct1(ref v1, ref v0) * 2F) 
                        * v1));
        }
        
        public static vec3 Refract(vec3 v0, vec3 v1, float eta) {
            float d = math.InnerProduct1(ref v1, ref v0);
            float k = (1F 
                        - ((eta * eta) 
                        * (1F 
                        - (d * d))));
            if ((k < 0F)) {
                return new vec3();
            }
            else {
                return ((eta * v0) 
                            - (((float)(((eta * d) 
                            + System.Math.Sqrt(k)))) * v1));
            }
        }
        
        public static vec3 Refract(ref vec3 v0, ref vec3 v1, float eta) {
            float d = math.InnerProduct1(ref v1, ref v0);
            float k = (1F 
                        - ((eta * eta) 
                        * (1F 
                        - (d * d))));
            if ((k < 0F)) {
                return new vec3();
            }
            else {
                return ((eta * v0) 
                            - (((float)(((eta * d) 
                            + System.Math.Sqrt(k)))) * v1));
            }
        }
        
        public static float LengthSquared(vec4 v0) {
            return math.MagnitudeSquared(ref v0);
        }
        
        public static float LengthSquared(ref vec4 v0) {
            return math.MagnitudeSquared(ref v0);
        }
        
        public static double Length(vec4 v0) {
            return math.Magnitude(ref v0);
        }
        
        public static double Length(ref vec4 v0) {
            return math.Magnitude(ref v0);
        }
        
        public static float DistanceSquared(vec4 v0, vec4 v1) {
            return math.MagnitudeSquared(v1 - v0);
        }
        
        public static float DistanceSquared(ref vec4 v0, ref vec4 v1) {
            return math.MagnitudeSquared(v1 - v0);
        }
        
        public static double Distance(vec4 v0, vec4 v1) {
            return math.Magnitude(v1 - v0);
        }
        
        public static double Distance(ref vec4 v0, ref vec4 v1) {
            return math.Magnitude(v1 - v0);
        }
        
        public static float Dot(vec4 t0, vec4 t1) {
            return math.InnerProduct1(ref t0, ref t1);
        }
        
        public static float Dot(ref vec4 t0, ref vec4 t1) {
            return math.InnerProduct1(ref t0, ref t1);
        }
        
        public static vec2 Dot(vec4 t0, mat4x2 t1) {
            return math.InnerProduct1(ref t0, ref t1);
        }
        
        public static vec2 Dot(ref vec4 t0, ref mat4x2 t1) {
            return math.InnerProduct1(ref t0, ref t1);
        }
        
        public static vec3 Dot(vec4 t0, mat4x3 t1) {
            return math.InnerProduct1(ref t0, ref t1);
        }
        
        public static vec3 Dot(ref vec4 t0, ref mat4x3 t1) {
            return math.InnerProduct1(ref t0, ref t1);
        }
        
        public static vec4 Dot(vec4 t0, mat4 t1) {
            return math.InnerProduct1(ref t0, ref t1);
        }
        
        public static vec4 Dot(ref vec4 t0, ref mat4 t1) {
            return math.InnerProduct1(ref t0, ref t1);
        }
        
        public static vec4 Cross(vec4 v0, vec4 v1, vec4 v2) {
            return new vec4((-1 * v0.v1 * v1.v2 * v2.v3) + (v0.v1 * v1.v3 * v2.v2) + (v0.v2 * v1.v1 * v2.v3) + (-1 * v0.v2 * v1.v3 * v2.v1) + (-1 * v0.v3 * v1.v1 * v2.v2) + (v0.v3 * v1.v2 * v2.v1), (v0.v0 * v1.v2 * v2.v3) + (-1 * v0.v0 * v1.v3 * v2.v2) + (-1 * v0.v2 * v1.v0 * v2.v3) + (v0.v2 * v1.v3 * v2.v0) + (v0.v3 * v1.v0 * v2.v2) + (-1 * v0.v3 * v1.v2 * v2.v0), (-1 * v0.v0 * v1.v1 * v2.v3) + (v0.v0 * v1.v3 * v2.v1) + (v0.v1 * v1.v0 * v2.v3) + (-1 * v0.v1 * v1.v3 * v2.v0) + (-1 * v0.v3 * v1.v0 * v2.v1) + (v0.v3 * v1.v1 * v2.v0), (v0.v0 * v1.v1 * v2.v2) + (-1 * v0.v0 * v1.v2 * v2.v1) + (-1 * v0.v1 * v1.v0 * v2.v2) + (v0.v1 * v1.v2 * v2.v0) + (v0.v2 * v1.v0 * v2.v1) + (-1 * v0.v2 * v1.v1 * v2.v0));
        }
        
        public static vec4 Cross(ref vec4 v0, ref vec4 v1, ref vec4 v2) {
            return new vec4((-1 * v0.v1 * v1.v2 * v2.v3) + (v0.v1 * v1.v3 * v2.v2) + (v0.v2 * v1.v1 * v2.v3) + (-1 * v0.v2 * v1.v3 * v2.v1) + (-1 * v0.v3 * v1.v1 * v2.v2) + (v0.v3 * v1.v2 * v2.v1), (v0.v0 * v1.v2 * v2.v3) + (-1 * v0.v0 * v1.v3 * v2.v2) + (-1 * v0.v2 * v1.v0 * v2.v3) + (v0.v2 * v1.v3 * v2.v0) + (v0.v3 * v1.v0 * v2.v2) + (-1 * v0.v3 * v1.v2 * v2.v0), (-1 * v0.v0 * v1.v1 * v2.v3) + (v0.v0 * v1.v3 * v2.v1) + (v0.v1 * v1.v0 * v2.v3) + (-1 * v0.v1 * v1.v3 * v2.v0) + (-1 * v0.v3 * v1.v0 * v2.v1) + (v0.v3 * v1.v1 * v2.v0), (v0.v0 * v1.v1 * v2.v2) + (-1 * v0.v0 * v1.v2 * v2.v1) + (-1 * v0.v1 * v1.v0 * v2.v2) + (v0.v1 * v1.v2 * v2.v0) + (v0.v2 * v1.v0 * v2.v1) + (-1 * v0.v2 * v1.v1 * v2.v0));
        }
        
        public static vec4 Normalize(vec4 v0) {
            math.Divide(ref v0, ((float)(math.Magnitude(ref v0))), out v0);
            return v0;
        }
        
        public static vec4 Normalize(ref vec4 v0) {
            math.Divide(ref v0, ((float)(math.Magnitude(ref v0))), out v0);
            return v0;
        }
        
        public static vec4 FaceForward(vec4 v0, vec4 v1, vec4 v2) {
            if ((math.InnerProduct1(ref v2, ref v1) < 0F)) {
                return v0;
            }
            else {
                return -v0;
            }
        }
        
        public static vec4 FaceForward(ref vec4 v0, ref vec4 v1, ref vec4 v2) {
            if ((math.InnerProduct1(ref v2, ref v1) < 0F)) {
                return v0;
            }
            else {
                return -v0;
            }
        }
        
        public static vec4 Reflect(vec4 v0, vec4 v1) {
            return (v0 
                        - ((math.InnerProduct1(ref v1, ref v0) * 2F) 
                        * v1));
        }
        
        public static vec4 Reflect(ref vec4 v0, ref vec4 v1) {
            return (v0 
                        - ((math.InnerProduct1(ref v1, ref v0) * 2F) 
                        * v1));
        }
        
        public static vec4 Refract(vec4 v0, vec4 v1, float eta) {
            float d = math.InnerProduct1(ref v1, ref v0);
            float k = (1F 
                        - ((eta * eta) 
                        * (1F 
                        - (d * d))));
            if ((k < 0F)) {
                return new vec4();
            }
            else {
                return ((eta * v0) 
                            - (((float)(((eta * d) 
                            + System.Math.Sqrt(k)))) * v1));
            }
        }
        
        public static vec4 Refract(ref vec4 v0, ref vec4 v1, float eta) {
            float d = math.InnerProduct1(ref v1, ref v0);
            float k = (1F 
                        - ((eta * eta) 
                        * (1F 
                        - (d * d))));
            if ((k < 0F)) {
                return new vec4();
            }
            else {
                return ((eta * v0) 
                            - (((float)(((eta * d) 
                            + System.Math.Sqrt(k)))) * v1));
            }
        }
    }
}
