#region --- License ---
/*
Copyright 2010 Daniel Filonik

Permission is hereby granted, free of charge, to any person obtaining a copy of
this software and associated documentation files (the "Software"), to deal in
the Software without restriction, including without limitation the rights to
use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
of the Software, and to permit persons to whom the Software is furnished to do
so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all
copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
SOFTWARE.
*/
#endregion

namespace TKM {
    using System;
    
    
    public partial class imath {
        
        public static void UnaryPlus(ref ivec2 t, out ivec2 r) {
            r.v0 = +t.v0;
            r.v1 = +t.v1;
        }
        
        public static void Negate(ref ivec2 t, out ivec2 r) {
            r.v0 = -t.v0;
            r.v1 = -t.v1;
        }
        
        public static void OnesComplement(ref ivec2 t, out ivec2 r) {
            r.v0 = ~t.v0;
            r.v1 = ~t.v1;
        }
        
        public static void Increment(ref ivec2 t, out ivec2 r) {
            r.v0 = ++t.v0;
            r.v1 = ++t.v1;
        }
        
        public static void Decrement(ref ivec2 t, out ivec2 r) {
            r.v0 = --t.v0;
            r.v1 = --t.v1;
        }
        
        public static void Add(int s1, ref ivec2 t2, out ivec2 r) {
            r.v0 = s1 + t2.v0;
            r.v1 = s1 + t2.v1;
        }
        
        public static void Subtract(int s1, ref ivec2 t2, out ivec2 r) {
            r.v0 = s1 - t2.v0;
            r.v1 = s1 - t2.v1;
        }
        
        public static void Multiply(int s1, ref ivec2 t2, out ivec2 r) {
            r.v0 = s1 * t2.v0;
            r.v1 = s1 * t2.v1;
        }
        
        public static void Divide(int s1, ref ivec2 t2, out ivec2 r) {
            r.v0 = s1 / t2.v0;
            r.v1 = s1 / t2.v1;
        }
        
        public static void Modulo(int s1, ref ivec2 t2, out ivec2 r) {
            r.v0 = s1 % t2.v0;
            r.v1 = s1 % t2.v1;
        }
        
        public static void And(int s1, ref ivec2 t2, out ivec2 r) {
            r.v0 = s1 & t2.v0;
            r.v1 = s1 & t2.v1;
        }
        
        public static void Or(int s1, ref ivec2 t2, out ivec2 r) {
            r.v0 = s1 | t2.v0;
            r.v1 = s1 | t2.v1;
        }
        
        public static void ExclusiveOr(int s1, ref ivec2 t2, out ivec2 r) {
            r.v0 = s1 ^ t2.v0;
            r.v1 = s1 ^ t2.v1;
        }
        
        public static void Add(ref ivec2 t1, int s2, out ivec2 r) {
            r.v0 = t1.v0 + s2;
            r.v1 = t1.v1 + s2;
        }
        
        public static void Subtract(ref ivec2 t1, int s2, out ivec2 r) {
            r.v0 = t1.v0 - s2;
            r.v1 = t1.v1 - s2;
        }
        
        public static void Multiply(ref ivec2 t1, int s2, out ivec2 r) {
            r.v0 = t1.v0 * s2;
            r.v1 = t1.v1 * s2;
        }
        
        public static void Divide(ref ivec2 t1, int s2, out ivec2 r) {
            r.v0 = t1.v0 / s2;
            r.v1 = t1.v1 / s2;
        }
        
        public static void Modulo(ref ivec2 t1, int s2, out ivec2 r) {
            r.v0 = t1.v0 % s2;
            r.v1 = t1.v1 % s2;
        }
        
        public static void And(ref ivec2 t1, int s2, out ivec2 r) {
            r.v0 = t1.v0 & s2;
            r.v1 = t1.v1 & s2;
        }
        
        public static void Or(ref ivec2 t1, int s2, out ivec2 r) {
            r.v0 = t1.v0 | s2;
            r.v1 = t1.v1 | s2;
        }
        
        public static void ExclusiveOr(ref ivec2 t1, int s2, out ivec2 r) {
            r.v0 = t1.v0 ^ s2;
            r.v1 = t1.v1 ^ s2;
        }
        
        public static void Add(ref ivec2 t1, ref ivec2 t2, out ivec2 r) {
            r.v0 = t1.v0 + t2.v0;
            r.v1 = t1.v1 + t2.v1;
        }
        
        public static void Subtract(ref ivec2 t1, ref ivec2 t2, out ivec2 r) {
            r.v0 = t1.v0 - t2.v0;
            r.v1 = t1.v1 - t2.v1;
        }
        
        public static void Multiply(ref ivec2 t1, ref ivec2 t2, out ivec2 r) {
            r.v0 = t1.v0 * t2.v0;
            r.v1 = t1.v1 * t2.v1;
        }
        
        public static void Divide(ref ivec2 t1, ref ivec2 t2, out ivec2 r) {
            r.v0 = t1.v0 / t2.v0;
            r.v1 = t1.v1 / t2.v1;
        }
        
        public static void Modulo(ref ivec2 t1, ref ivec2 t2, out ivec2 r) {
            r.v0 = t1.v0 % t2.v0;
            r.v1 = t1.v1 % t2.v1;
        }
        
        public static void And(ref ivec2 t1, ref ivec2 t2, out ivec2 r) {
            r.v0 = t1.v0 & t2.v0;
            r.v1 = t1.v1 & t2.v1;
        }
        
        public static void Or(ref ivec2 t1, ref ivec2 t2, out ivec2 r) {
            r.v0 = t1.v0 | t2.v0;
            r.v1 = t1.v1 | t2.v1;
        }
        
        public static void ExclusiveOr(ref ivec2 t1, ref ivec2 t2, out ivec2 r) {
            r.v0 = t1.v0 ^ t2.v0;
            r.v1 = t1.v1 ^ t2.v1;
        }
        
        public static void Equal(ref ivec2 t1, ref ivec2 t2, out bvec2 r) {
            r.v0 = t1.v0 == t2.v0;
            r.v1 = t1.v1 == t2.v1;
        }
        
        public static void GreaterThan(ref ivec2 t1, ref ivec2 t2, out bvec2 r) {
            r.v0 = t1.v0 > t2.v0;
            r.v1 = t1.v1 > t2.v1;
        }
        
        public static void GreaterThanEqual(ref ivec2 t1, ref ivec2 t2, out bvec2 r) {
            r.v0 = t1.v0 >= t2.v0;
            r.v1 = t1.v1 >= t2.v1;
        }
        
        public static void LessThan(ref ivec2 t1, ref ivec2 t2, out bvec2 r) {
            r.v0 = t1.v0 < t2.v0;
            r.v1 = t1.v1 < t2.v1;
        }
        
        public static void LessThanEqual(ref ivec2 t1, ref ivec2 t2, out bvec2 r) {
            r.v0 = t1.v0 <= t2.v0;
            r.v1 = t1.v1 <= t2.v1;
        }
        
        public static void NotEqual(ref ivec2 t1, ref ivec2 t2, out bvec2 r) {
            r.v0 = t1.v0 != t2.v0;
            r.v1 = t1.v1 != t2.v1;
        }
        
        public static void UnaryPlus(ref ivec3 t, out ivec3 r) {
            r.v0 = +t.v0;
            r.v1 = +t.v1;
            r.v2 = +t.v2;
        }
        
        public static void Negate(ref ivec3 t, out ivec3 r) {
            r.v0 = -t.v0;
            r.v1 = -t.v1;
            r.v2 = -t.v2;
        }
        
        public static void OnesComplement(ref ivec3 t, out ivec3 r) {
            r.v0 = ~t.v0;
            r.v1 = ~t.v1;
            r.v2 = ~t.v2;
        }
        
        public static void Increment(ref ivec3 t, out ivec3 r) {
            r.v0 = ++t.v0;
            r.v1 = ++t.v1;
            r.v2 = ++t.v2;
        }
        
        public static void Decrement(ref ivec3 t, out ivec3 r) {
            r.v0 = --t.v0;
            r.v1 = --t.v1;
            r.v2 = --t.v2;
        }
        
        public static void Add(int s1, ref ivec3 t2, out ivec3 r) {
            r.v0 = s1 + t2.v0;
            r.v1 = s1 + t2.v1;
            r.v2 = s1 + t2.v2;
        }
        
        public static void Subtract(int s1, ref ivec3 t2, out ivec3 r) {
            r.v0 = s1 - t2.v0;
            r.v1 = s1 - t2.v1;
            r.v2 = s1 - t2.v2;
        }
        
        public static void Multiply(int s1, ref ivec3 t2, out ivec3 r) {
            r.v0 = s1 * t2.v0;
            r.v1 = s1 * t2.v1;
            r.v2 = s1 * t2.v2;
        }
        
        public static void Divide(int s1, ref ivec3 t2, out ivec3 r) {
            r.v0 = s1 / t2.v0;
            r.v1 = s1 / t2.v1;
            r.v2 = s1 / t2.v2;
        }
        
        public static void Modulo(int s1, ref ivec3 t2, out ivec3 r) {
            r.v0 = s1 % t2.v0;
            r.v1 = s1 % t2.v1;
            r.v2 = s1 % t2.v2;
        }
        
        public static void And(int s1, ref ivec3 t2, out ivec3 r) {
            r.v0 = s1 & t2.v0;
            r.v1 = s1 & t2.v1;
            r.v2 = s1 & t2.v2;
        }
        
        public static void Or(int s1, ref ivec3 t2, out ivec3 r) {
            r.v0 = s1 | t2.v0;
            r.v1 = s1 | t2.v1;
            r.v2 = s1 | t2.v2;
        }
        
        public static void ExclusiveOr(int s1, ref ivec3 t2, out ivec3 r) {
            r.v0 = s1 ^ t2.v0;
            r.v1 = s1 ^ t2.v1;
            r.v2 = s1 ^ t2.v2;
        }
        
        public static void Add(ref ivec3 t1, int s2, out ivec3 r) {
            r.v0 = t1.v0 + s2;
            r.v1 = t1.v1 + s2;
            r.v2 = t1.v2 + s2;
        }
        
        public static void Subtract(ref ivec3 t1, int s2, out ivec3 r) {
            r.v0 = t1.v0 - s2;
            r.v1 = t1.v1 - s2;
            r.v2 = t1.v2 - s2;
        }
        
        public static void Multiply(ref ivec3 t1, int s2, out ivec3 r) {
            r.v0 = t1.v0 * s2;
            r.v1 = t1.v1 * s2;
            r.v2 = t1.v2 * s2;
        }
        
        public static void Divide(ref ivec3 t1, int s2, out ivec3 r) {
            r.v0 = t1.v0 / s2;
            r.v1 = t1.v1 / s2;
            r.v2 = t1.v2 / s2;
        }
        
        public static void Modulo(ref ivec3 t1, int s2, out ivec3 r) {
            r.v0 = t1.v0 % s2;
            r.v1 = t1.v1 % s2;
            r.v2 = t1.v2 % s2;
        }
        
        public static void And(ref ivec3 t1, int s2, out ivec3 r) {
            r.v0 = t1.v0 & s2;
            r.v1 = t1.v1 & s2;
            r.v2 = t1.v2 & s2;
        }
        
        public static void Or(ref ivec3 t1, int s2, out ivec3 r) {
            r.v0 = t1.v0 | s2;
            r.v1 = t1.v1 | s2;
            r.v2 = t1.v2 | s2;
        }
        
        public static void ExclusiveOr(ref ivec3 t1, int s2, out ivec3 r) {
            r.v0 = t1.v0 ^ s2;
            r.v1 = t1.v1 ^ s2;
            r.v2 = t1.v2 ^ s2;
        }
        
        public static void Add(ref ivec3 t1, ref ivec3 t2, out ivec3 r) {
            r.v0 = t1.v0 + t2.v0;
            r.v1 = t1.v1 + t2.v1;
            r.v2 = t1.v2 + t2.v2;
        }
        
        public static void Subtract(ref ivec3 t1, ref ivec3 t2, out ivec3 r) {
            r.v0 = t1.v0 - t2.v0;
            r.v1 = t1.v1 - t2.v1;
            r.v2 = t1.v2 - t2.v2;
        }
        
        public static void Multiply(ref ivec3 t1, ref ivec3 t2, out ivec3 r) {
            r.v0 = t1.v0 * t2.v0;
            r.v1 = t1.v1 * t2.v1;
            r.v2 = t1.v2 * t2.v2;
        }
        
        public static void Divide(ref ivec3 t1, ref ivec3 t2, out ivec3 r) {
            r.v0 = t1.v0 / t2.v0;
            r.v1 = t1.v1 / t2.v1;
            r.v2 = t1.v2 / t2.v2;
        }
        
        public static void Modulo(ref ivec3 t1, ref ivec3 t2, out ivec3 r) {
            r.v0 = t1.v0 % t2.v0;
            r.v1 = t1.v1 % t2.v1;
            r.v2 = t1.v2 % t2.v2;
        }
        
        public static void And(ref ivec3 t1, ref ivec3 t2, out ivec3 r) {
            r.v0 = t1.v0 & t2.v0;
            r.v1 = t1.v1 & t2.v1;
            r.v2 = t1.v2 & t2.v2;
        }
        
        public static void Or(ref ivec3 t1, ref ivec3 t2, out ivec3 r) {
            r.v0 = t1.v0 | t2.v0;
            r.v1 = t1.v1 | t2.v1;
            r.v2 = t1.v2 | t2.v2;
        }
        
        public static void ExclusiveOr(ref ivec3 t1, ref ivec3 t2, out ivec3 r) {
            r.v0 = t1.v0 ^ t2.v0;
            r.v1 = t1.v1 ^ t2.v1;
            r.v2 = t1.v2 ^ t2.v2;
        }
        
        public static void Equal(ref ivec3 t1, ref ivec3 t2, out bvec3 r) {
            r.v0 = t1.v0 == t2.v0;
            r.v1 = t1.v1 == t2.v1;
            r.v2 = t1.v2 == t2.v2;
        }
        
        public static void GreaterThan(ref ivec3 t1, ref ivec3 t2, out bvec3 r) {
            r.v0 = t1.v0 > t2.v0;
            r.v1 = t1.v1 > t2.v1;
            r.v2 = t1.v2 > t2.v2;
        }
        
        public static void GreaterThanEqual(ref ivec3 t1, ref ivec3 t2, out bvec3 r) {
            r.v0 = t1.v0 >= t2.v0;
            r.v1 = t1.v1 >= t2.v1;
            r.v2 = t1.v2 >= t2.v2;
        }
        
        public static void LessThan(ref ivec3 t1, ref ivec3 t2, out bvec3 r) {
            r.v0 = t1.v0 < t2.v0;
            r.v1 = t1.v1 < t2.v1;
            r.v2 = t1.v2 < t2.v2;
        }
        
        public static void LessThanEqual(ref ivec3 t1, ref ivec3 t2, out bvec3 r) {
            r.v0 = t1.v0 <= t2.v0;
            r.v1 = t1.v1 <= t2.v1;
            r.v2 = t1.v2 <= t2.v2;
        }
        
        public static void NotEqual(ref ivec3 t1, ref ivec3 t2, out bvec3 r) {
            r.v0 = t1.v0 != t2.v0;
            r.v1 = t1.v1 != t2.v1;
            r.v2 = t1.v2 != t2.v2;
        }
        
        public static void UnaryPlus(ref ivec4 t, out ivec4 r) {
            r.v0 = +t.v0;
            r.v1 = +t.v1;
            r.v2 = +t.v2;
            r.v3 = +t.v3;
        }
        
        public static void Negate(ref ivec4 t, out ivec4 r) {
            r.v0 = -t.v0;
            r.v1 = -t.v1;
            r.v2 = -t.v2;
            r.v3 = -t.v3;
        }
        
        public static void OnesComplement(ref ivec4 t, out ivec4 r) {
            r.v0 = ~t.v0;
            r.v1 = ~t.v1;
            r.v2 = ~t.v2;
            r.v3 = ~t.v3;
        }
        
        public static void Increment(ref ivec4 t, out ivec4 r) {
            r.v0 = ++t.v0;
            r.v1 = ++t.v1;
            r.v2 = ++t.v2;
            r.v3 = ++t.v3;
        }
        
        public static void Decrement(ref ivec4 t, out ivec4 r) {
            r.v0 = --t.v0;
            r.v1 = --t.v1;
            r.v2 = --t.v2;
            r.v3 = --t.v3;
        }
        
        public static void Add(int s1, ref ivec4 t2, out ivec4 r) {
            r.v0 = s1 + t2.v0;
            r.v1 = s1 + t2.v1;
            r.v2 = s1 + t2.v2;
            r.v3 = s1 + t2.v3;
        }
        
        public static void Subtract(int s1, ref ivec4 t2, out ivec4 r) {
            r.v0 = s1 - t2.v0;
            r.v1 = s1 - t2.v1;
            r.v2 = s1 - t2.v2;
            r.v3 = s1 - t2.v3;
        }
        
        public static void Multiply(int s1, ref ivec4 t2, out ivec4 r) {
            r.v0 = s1 * t2.v0;
            r.v1 = s1 * t2.v1;
            r.v2 = s1 * t2.v2;
            r.v3 = s1 * t2.v3;
        }
        
        public static void Divide(int s1, ref ivec4 t2, out ivec4 r) {
            r.v0 = s1 / t2.v0;
            r.v1 = s1 / t2.v1;
            r.v2 = s1 / t2.v2;
            r.v3 = s1 / t2.v3;
        }
        
        public static void Modulo(int s1, ref ivec4 t2, out ivec4 r) {
            r.v0 = s1 % t2.v0;
            r.v1 = s1 % t2.v1;
            r.v2 = s1 % t2.v2;
            r.v3 = s1 % t2.v3;
        }
        
        public static void And(int s1, ref ivec4 t2, out ivec4 r) {
            r.v0 = s1 & t2.v0;
            r.v1 = s1 & t2.v1;
            r.v2 = s1 & t2.v2;
            r.v3 = s1 & t2.v3;
        }
        
        public static void Or(int s1, ref ivec4 t2, out ivec4 r) {
            r.v0 = s1 | t2.v0;
            r.v1 = s1 | t2.v1;
            r.v2 = s1 | t2.v2;
            r.v3 = s1 | t2.v3;
        }
        
        public static void ExclusiveOr(int s1, ref ivec4 t2, out ivec4 r) {
            r.v0 = s1 ^ t2.v0;
            r.v1 = s1 ^ t2.v1;
            r.v2 = s1 ^ t2.v2;
            r.v3 = s1 ^ t2.v3;
        }
        
        public static void Add(ref ivec4 t1, int s2, out ivec4 r) {
            r.v0 = t1.v0 + s2;
            r.v1 = t1.v1 + s2;
            r.v2 = t1.v2 + s2;
            r.v3 = t1.v3 + s2;
        }
        
        public static void Subtract(ref ivec4 t1, int s2, out ivec4 r) {
            r.v0 = t1.v0 - s2;
            r.v1 = t1.v1 - s2;
            r.v2 = t1.v2 - s2;
            r.v3 = t1.v3 - s2;
        }
        
        public static void Multiply(ref ivec4 t1, int s2, out ivec4 r) {
            r.v0 = t1.v0 * s2;
            r.v1 = t1.v1 * s2;
            r.v2 = t1.v2 * s2;
            r.v3 = t1.v3 * s2;
        }
        
        public static void Divide(ref ivec4 t1, int s2, out ivec4 r) {
            r.v0 = t1.v0 / s2;
            r.v1 = t1.v1 / s2;
            r.v2 = t1.v2 / s2;
            r.v3 = t1.v3 / s2;
        }
        
        public static void Modulo(ref ivec4 t1, int s2, out ivec4 r) {
            r.v0 = t1.v0 % s2;
            r.v1 = t1.v1 % s2;
            r.v2 = t1.v2 % s2;
            r.v3 = t1.v3 % s2;
        }
        
        public static void And(ref ivec4 t1, int s2, out ivec4 r) {
            r.v0 = t1.v0 & s2;
            r.v1 = t1.v1 & s2;
            r.v2 = t1.v2 & s2;
            r.v3 = t1.v3 & s2;
        }
        
        public static void Or(ref ivec4 t1, int s2, out ivec4 r) {
            r.v0 = t1.v0 | s2;
            r.v1 = t1.v1 | s2;
            r.v2 = t1.v2 | s2;
            r.v3 = t1.v3 | s2;
        }
        
        public static void ExclusiveOr(ref ivec4 t1, int s2, out ivec4 r) {
            r.v0 = t1.v0 ^ s2;
            r.v1 = t1.v1 ^ s2;
            r.v2 = t1.v2 ^ s2;
            r.v3 = t1.v3 ^ s2;
        }
        
        public static void Add(ref ivec4 t1, ref ivec4 t2, out ivec4 r) {
            r.v0 = t1.v0 + t2.v0;
            r.v1 = t1.v1 + t2.v1;
            r.v2 = t1.v2 + t2.v2;
            r.v3 = t1.v3 + t2.v3;
        }
        
        public static void Subtract(ref ivec4 t1, ref ivec4 t2, out ivec4 r) {
            r.v0 = t1.v0 - t2.v0;
            r.v1 = t1.v1 - t2.v1;
            r.v2 = t1.v2 - t2.v2;
            r.v3 = t1.v3 - t2.v3;
        }
        
        public static void Multiply(ref ivec4 t1, ref ivec4 t2, out ivec4 r) {
            r.v0 = t1.v0 * t2.v0;
            r.v1 = t1.v1 * t2.v1;
            r.v2 = t1.v2 * t2.v2;
            r.v3 = t1.v3 * t2.v3;
        }
        
        public static void Divide(ref ivec4 t1, ref ivec4 t2, out ivec4 r) {
            r.v0 = t1.v0 / t2.v0;
            r.v1 = t1.v1 / t2.v1;
            r.v2 = t1.v2 / t2.v2;
            r.v3 = t1.v3 / t2.v3;
        }
        
        public static void Modulo(ref ivec4 t1, ref ivec4 t2, out ivec4 r) {
            r.v0 = t1.v0 % t2.v0;
            r.v1 = t1.v1 % t2.v1;
            r.v2 = t1.v2 % t2.v2;
            r.v3 = t1.v3 % t2.v3;
        }
        
        public static void And(ref ivec4 t1, ref ivec4 t2, out ivec4 r) {
            r.v0 = t1.v0 & t2.v0;
            r.v1 = t1.v1 & t2.v1;
            r.v2 = t1.v2 & t2.v2;
            r.v3 = t1.v3 & t2.v3;
        }
        
        public static void Or(ref ivec4 t1, ref ivec4 t2, out ivec4 r) {
            r.v0 = t1.v0 | t2.v0;
            r.v1 = t1.v1 | t2.v1;
            r.v2 = t1.v2 | t2.v2;
            r.v3 = t1.v3 | t2.v3;
        }
        
        public static void ExclusiveOr(ref ivec4 t1, ref ivec4 t2, out ivec4 r) {
            r.v0 = t1.v0 ^ t2.v0;
            r.v1 = t1.v1 ^ t2.v1;
            r.v2 = t1.v2 ^ t2.v2;
            r.v3 = t1.v3 ^ t2.v3;
        }
        
        public static void Equal(ref ivec4 t1, ref ivec4 t2, out bvec4 r) {
            r.v0 = t1.v0 == t2.v0;
            r.v1 = t1.v1 == t2.v1;
            r.v2 = t1.v2 == t2.v2;
            r.v3 = t1.v3 == t2.v3;
        }
        
        public static void GreaterThan(ref ivec4 t1, ref ivec4 t2, out bvec4 r) {
            r.v0 = t1.v0 > t2.v0;
            r.v1 = t1.v1 > t2.v1;
            r.v2 = t1.v2 > t2.v2;
            r.v3 = t1.v3 > t2.v3;
        }
        
        public static void GreaterThanEqual(ref ivec4 t1, ref ivec4 t2, out bvec4 r) {
            r.v0 = t1.v0 >= t2.v0;
            r.v1 = t1.v1 >= t2.v1;
            r.v2 = t1.v2 >= t2.v2;
            r.v3 = t1.v3 >= t2.v3;
        }
        
        public static void LessThan(ref ivec4 t1, ref ivec4 t2, out bvec4 r) {
            r.v0 = t1.v0 < t2.v0;
            r.v1 = t1.v1 < t2.v1;
            r.v2 = t1.v2 < t2.v2;
            r.v3 = t1.v3 < t2.v3;
        }
        
        public static void LessThanEqual(ref ivec4 t1, ref ivec4 t2, out bvec4 r) {
            r.v0 = t1.v0 <= t2.v0;
            r.v1 = t1.v1 <= t2.v1;
            r.v2 = t1.v2 <= t2.v2;
            r.v3 = t1.v3 <= t2.v3;
        }
        
        public static void NotEqual(ref ivec4 t1, ref ivec4 t2, out bvec4 r) {
            r.v0 = t1.v0 != t2.v0;
            r.v1 = t1.v1 != t2.v1;
            r.v2 = t1.v2 != t2.v2;
            r.v3 = t1.v3 != t2.v3;
        }
    }
}
