#region --- License ---
/*
Copyright 2010 Daniel Filonik

Permission is hereby granted, free of charge, to any person obtaining a copy of
this software and associated documentation files (the "Software"), to deal in
the Software without restriction, including without limitation the rights to
use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
of the Software, and to permit persons to whom the Software is furnished to do
so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all
copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
SOFTWARE.
*/
#endregion

namespace TKM {
    using System;
    
    
    public partial class umath {
        
        public static void UnaryPlus(ref uvec2 t, out uvec2 r) {
            r.v0 = +t.v0;
            r.v1 = +t.v1;
        }
        
        public static void OnesComplement(ref uvec2 t, out uvec2 r) {
            r.v0 = ~t.v0;
            r.v1 = ~t.v1;
        }
        
        public static void Increment(ref uvec2 t, out uvec2 r) {
            r.v0 = ++t.v0;
            r.v1 = ++t.v1;
        }
        
        public static void Decrement(ref uvec2 t, out uvec2 r) {
            r.v0 = --t.v0;
            r.v1 = --t.v1;
        }
        
        public static void Add(uint s1, ref uvec2 t2, out uvec2 r) {
            r.v0 = s1 + t2.v0;
            r.v1 = s1 + t2.v1;
        }
        
        public static void Subtract(uint s1, ref uvec2 t2, out uvec2 r) {
            r.v0 = s1 - t2.v0;
            r.v1 = s1 - t2.v1;
        }
        
        public static void Multiply(uint s1, ref uvec2 t2, out uvec2 r) {
            r.v0 = s1 * t2.v0;
            r.v1 = s1 * t2.v1;
        }
        
        public static void Divide(uint s1, ref uvec2 t2, out uvec2 r) {
            r.v0 = s1 / t2.v0;
            r.v1 = s1 / t2.v1;
        }
        
        public static void Modulo(uint s1, ref uvec2 t2, out uvec2 r) {
            r.v0 = s1 % t2.v0;
            r.v1 = s1 % t2.v1;
        }
        
        public static void And(uint s1, ref uvec2 t2, out uvec2 r) {
            r.v0 = s1 & t2.v0;
            r.v1 = s1 & t2.v1;
        }
        
        public static void Or(uint s1, ref uvec2 t2, out uvec2 r) {
            r.v0 = s1 | t2.v0;
            r.v1 = s1 | t2.v1;
        }
        
        public static void ExclusiveOr(uint s1, ref uvec2 t2, out uvec2 r) {
            r.v0 = s1 ^ t2.v0;
            r.v1 = s1 ^ t2.v1;
        }
        
        public static void Add(ref uvec2 t1, uint s2, out uvec2 r) {
            r.v0 = t1.v0 + s2;
            r.v1 = t1.v1 + s2;
        }
        
        public static void Subtract(ref uvec2 t1, uint s2, out uvec2 r) {
            r.v0 = t1.v0 - s2;
            r.v1 = t1.v1 - s2;
        }
        
        public static void Multiply(ref uvec2 t1, uint s2, out uvec2 r) {
            r.v0 = t1.v0 * s2;
            r.v1 = t1.v1 * s2;
        }
        
        public static void Divide(ref uvec2 t1, uint s2, out uvec2 r) {
            r.v0 = t1.v0 / s2;
            r.v1 = t1.v1 / s2;
        }
        
        public static void Modulo(ref uvec2 t1, uint s2, out uvec2 r) {
            r.v0 = t1.v0 % s2;
            r.v1 = t1.v1 % s2;
        }
        
        public static void And(ref uvec2 t1, uint s2, out uvec2 r) {
            r.v0 = t1.v0 & s2;
            r.v1 = t1.v1 & s2;
        }
        
        public static void Or(ref uvec2 t1, uint s2, out uvec2 r) {
            r.v0 = t1.v0 | s2;
            r.v1 = t1.v1 | s2;
        }
        
        public static void ExclusiveOr(ref uvec2 t1, uint s2, out uvec2 r) {
            r.v0 = t1.v0 ^ s2;
            r.v1 = t1.v1 ^ s2;
        }
        
        public static void Add(ref uvec2 t1, ref uvec2 t2, out uvec2 r) {
            r.v0 = t1.v0 + t2.v0;
            r.v1 = t1.v1 + t2.v1;
        }
        
        public static void Subtract(ref uvec2 t1, ref uvec2 t2, out uvec2 r) {
            r.v0 = t1.v0 - t2.v0;
            r.v1 = t1.v1 - t2.v1;
        }
        
        public static void Multiply(ref uvec2 t1, ref uvec2 t2, out uvec2 r) {
            r.v0 = t1.v0 * t2.v0;
            r.v1 = t1.v1 * t2.v1;
        }
        
        public static void Divide(ref uvec2 t1, ref uvec2 t2, out uvec2 r) {
            r.v0 = t1.v0 / t2.v0;
            r.v1 = t1.v1 / t2.v1;
        }
        
        public static void Modulo(ref uvec2 t1, ref uvec2 t2, out uvec2 r) {
            r.v0 = t1.v0 % t2.v0;
            r.v1 = t1.v1 % t2.v1;
        }
        
        public static void And(ref uvec2 t1, ref uvec2 t2, out uvec2 r) {
            r.v0 = t1.v0 & t2.v0;
            r.v1 = t1.v1 & t2.v1;
        }
        
        public static void Or(ref uvec2 t1, ref uvec2 t2, out uvec2 r) {
            r.v0 = t1.v0 | t2.v0;
            r.v1 = t1.v1 | t2.v1;
        }
        
        public static void ExclusiveOr(ref uvec2 t1, ref uvec2 t2, out uvec2 r) {
            r.v0 = t1.v0 ^ t2.v0;
            r.v1 = t1.v1 ^ t2.v1;
        }
        
        public static void Equal(ref uvec2 t1, ref uvec2 t2, out bvec2 r) {
            r.v0 = t1.v0 == t2.v0;
            r.v1 = t1.v1 == t2.v1;
        }
        
        public static void GreaterThan(ref uvec2 t1, ref uvec2 t2, out bvec2 r) {
            r.v0 = t1.v0 > t2.v0;
            r.v1 = t1.v1 > t2.v1;
        }
        
        public static void GreaterThanEqual(ref uvec2 t1, ref uvec2 t2, out bvec2 r) {
            r.v0 = t1.v0 >= t2.v0;
            r.v1 = t1.v1 >= t2.v1;
        }
        
        public static void LessThan(ref uvec2 t1, ref uvec2 t2, out bvec2 r) {
            r.v0 = t1.v0 < t2.v0;
            r.v1 = t1.v1 < t2.v1;
        }
        
        public static void LessThanEqual(ref uvec2 t1, ref uvec2 t2, out bvec2 r) {
            r.v0 = t1.v0 <= t2.v0;
            r.v1 = t1.v1 <= t2.v1;
        }
        
        public static void NotEqual(ref uvec2 t1, ref uvec2 t2, out bvec2 r) {
            r.v0 = t1.v0 != t2.v0;
            r.v1 = t1.v1 != t2.v1;
        }
        
        public static void UnaryPlus(ref uvec3 t, out uvec3 r) {
            r.v0 = +t.v0;
            r.v1 = +t.v1;
            r.v2 = +t.v2;
        }
        
        public static void OnesComplement(ref uvec3 t, out uvec3 r) {
            r.v0 = ~t.v0;
            r.v1 = ~t.v1;
            r.v2 = ~t.v2;
        }
        
        public static void Increment(ref uvec3 t, out uvec3 r) {
            r.v0 = ++t.v0;
            r.v1 = ++t.v1;
            r.v2 = ++t.v2;
        }
        
        public static void Decrement(ref uvec3 t, out uvec3 r) {
            r.v0 = --t.v0;
            r.v1 = --t.v1;
            r.v2 = --t.v2;
        }
        
        public static void Add(uint s1, ref uvec3 t2, out uvec3 r) {
            r.v0 = s1 + t2.v0;
            r.v1 = s1 + t2.v1;
            r.v2 = s1 + t2.v2;
        }
        
        public static void Subtract(uint s1, ref uvec3 t2, out uvec3 r) {
            r.v0 = s1 - t2.v0;
            r.v1 = s1 - t2.v1;
            r.v2 = s1 - t2.v2;
        }
        
        public static void Multiply(uint s1, ref uvec3 t2, out uvec3 r) {
            r.v0 = s1 * t2.v0;
            r.v1 = s1 * t2.v1;
            r.v2 = s1 * t2.v2;
        }
        
        public static void Divide(uint s1, ref uvec3 t2, out uvec3 r) {
            r.v0 = s1 / t2.v0;
            r.v1 = s1 / t2.v1;
            r.v2 = s1 / t2.v2;
        }
        
        public static void Modulo(uint s1, ref uvec3 t2, out uvec3 r) {
            r.v0 = s1 % t2.v0;
            r.v1 = s1 % t2.v1;
            r.v2 = s1 % t2.v2;
        }
        
        public static void And(uint s1, ref uvec3 t2, out uvec3 r) {
            r.v0 = s1 & t2.v0;
            r.v1 = s1 & t2.v1;
            r.v2 = s1 & t2.v2;
        }
        
        public static void Or(uint s1, ref uvec3 t2, out uvec3 r) {
            r.v0 = s1 | t2.v0;
            r.v1 = s1 | t2.v1;
            r.v2 = s1 | t2.v2;
        }
        
        public static void ExclusiveOr(uint s1, ref uvec3 t2, out uvec3 r) {
            r.v0 = s1 ^ t2.v0;
            r.v1 = s1 ^ t2.v1;
            r.v2 = s1 ^ t2.v2;
        }
        
        public static void Add(ref uvec3 t1, uint s2, out uvec3 r) {
            r.v0 = t1.v0 + s2;
            r.v1 = t1.v1 + s2;
            r.v2 = t1.v2 + s2;
        }
        
        public static void Subtract(ref uvec3 t1, uint s2, out uvec3 r) {
            r.v0 = t1.v0 - s2;
            r.v1 = t1.v1 - s2;
            r.v2 = t1.v2 - s2;
        }
        
        public static void Multiply(ref uvec3 t1, uint s2, out uvec3 r) {
            r.v0 = t1.v0 * s2;
            r.v1 = t1.v1 * s2;
            r.v2 = t1.v2 * s2;
        }
        
        public static void Divide(ref uvec3 t1, uint s2, out uvec3 r) {
            r.v0 = t1.v0 / s2;
            r.v1 = t1.v1 / s2;
            r.v2 = t1.v2 / s2;
        }
        
        public static void Modulo(ref uvec3 t1, uint s2, out uvec3 r) {
            r.v0 = t1.v0 % s2;
            r.v1 = t1.v1 % s2;
            r.v2 = t1.v2 % s2;
        }
        
        public static void And(ref uvec3 t1, uint s2, out uvec3 r) {
            r.v0 = t1.v0 & s2;
            r.v1 = t1.v1 & s2;
            r.v2 = t1.v2 & s2;
        }
        
        public static void Or(ref uvec3 t1, uint s2, out uvec3 r) {
            r.v0 = t1.v0 | s2;
            r.v1 = t1.v1 | s2;
            r.v2 = t1.v2 | s2;
        }
        
        public static void ExclusiveOr(ref uvec3 t1, uint s2, out uvec3 r) {
            r.v0 = t1.v0 ^ s2;
            r.v1 = t1.v1 ^ s2;
            r.v2 = t1.v2 ^ s2;
        }
        
        public static void Add(ref uvec3 t1, ref uvec3 t2, out uvec3 r) {
            r.v0 = t1.v0 + t2.v0;
            r.v1 = t1.v1 + t2.v1;
            r.v2 = t1.v2 + t2.v2;
        }
        
        public static void Subtract(ref uvec3 t1, ref uvec3 t2, out uvec3 r) {
            r.v0 = t1.v0 - t2.v0;
            r.v1 = t1.v1 - t2.v1;
            r.v2 = t1.v2 - t2.v2;
        }
        
        public static void Multiply(ref uvec3 t1, ref uvec3 t2, out uvec3 r) {
            r.v0 = t1.v0 * t2.v0;
            r.v1 = t1.v1 * t2.v1;
            r.v2 = t1.v2 * t2.v2;
        }
        
        public static void Divide(ref uvec3 t1, ref uvec3 t2, out uvec3 r) {
            r.v0 = t1.v0 / t2.v0;
            r.v1 = t1.v1 / t2.v1;
            r.v2 = t1.v2 / t2.v2;
        }
        
        public static void Modulo(ref uvec3 t1, ref uvec3 t2, out uvec3 r) {
            r.v0 = t1.v0 % t2.v0;
            r.v1 = t1.v1 % t2.v1;
            r.v2 = t1.v2 % t2.v2;
        }
        
        public static void And(ref uvec3 t1, ref uvec3 t2, out uvec3 r) {
            r.v0 = t1.v0 & t2.v0;
            r.v1 = t1.v1 & t2.v1;
            r.v2 = t1.v2 & t2.v2;
        }
        
        public static void Or(ref uvec3 t1, ref uvec3 t2, out uvec3 r) {
            r.v0 = t1.v0 | t2.v0;
            r.v1 = t1.v1 | t2.v1;
            r.v2 = t1.v2 | t2.v2;
        }
        
        public static void ExclusiveOr(ref uvec3 t1, ref uvec3 t2, out uvec3 r) {
            r.v0 = t1.v0 ^ t2.v0;
            r.v1 = t1.v1 ^ t2.v1;
            r.v2 = t1.v2 ^ t2.v2;
        }
        
        public static void Equal(ref uvec3 t1, ref uvec3 t2, out bvec3 r) {
            r.v0 = t1.v0 == t2.v0;
            r.v1 = t1.v1 == t2.v1;
            r.v2 = t1.v2 == t2.v2;
        }
        
        public static void GreaterThan(ref uvec3 t1, ref uvec3 t2, out bvec3 r) {
            r.v0 = t1.v0 > t2.v0;
            r.v1 = t1.v1 > t2.v1;
            r.v2 = t1.v2 > t2.v2;
        }
        
        public static void GreaterThanEqual(ref uvec3 t1, ref uvec3 t2, out bvec3 r) {
            r.v0 = t1.v0 >= t2.v0;
            r.v1 = t1.v1 >= t2.v1;
            r.v2 = t1.v2 >= t2.v2;
        }
        
        public static void LessThan(ref uvec3 t1, ref uvec3 t2, out bvec3 r) {
            r.v0 = t1.v0 < t2.v0;
            r.v1 = t1.v1 < t2.v1;
            r.v2 = t1.v2 < t2.v2;
        }
        
        public static void LessThanEqual(ref uvec3 t1, ref uvec3 t2, out bvec3 r) {
            r.v0 = t1.v0 <= t2.v0;
            r.v1 = t1.v1 <= t2.v1;
            r.v2 = t1.v2 <= t2.v2;
        }
        
        public static void NotEqual(ref uvec3 t1, ref uvec3 t2, out bvec3 r) {
            r.v0 = t1.v0 != t2.v0;
            r.v1 = t1.v1 != t2.v1;
            r.v2 = t1.v2 != t2.v2;
        }
        
        public static void UnaryPlus(ref uvec4 t, out uvec4 r) {
            r.v0 = +t.v0;
            r.v1 = +t.v1;
            r.v2 = +t.v2;
            r.v3 = +t.v3;
        }
        
        public static void OnesComplement(ref uvec4 t, out uvec4 r) {
            r.v0 = ~t.v0;
            r.v1 = ~t.v1;
            r.v2 = ~t.v2;
            r.v3 = ~t.v3;
        }
        
        public static void Increment(ref uvec4 t, out uvec4 r) {
            r.v0 = ++t.v0;
            r.v1 = ++t.v1;
            r.v2 = ++t.v2;
            r.v3 = ++t.v3;
        }
        
        public static void Decrement(ref uvec4 t, out uvec4 r) {
            r.v0 = --t.v0;
            r.v1 = --t.v1;
            r.v2 = --t.v2;
            r.v3 = --t.v3;
        }
        
        public static void Add(uint s1, ref uvec4 t2, out uvec4 r) {
            r.v0 = s1 + t2.v0;
            r.v1 = s1 + t2.v1;
            r.v2 = s1 + t2.v2;
            r.v3 = s1 + t2.v3;
        }
        
        public static void Subtract(uint s1, ref uvec4 t2, out uvec4 r) {
            r.v0 = s1 - t2.v0;
            r.v1 = s1 - t2.v1;
            r.v2 = s1 - t2.v2;
            r.v3 = s1 - t2.v3;
        }
        
        public static void Multiply(uint s1, ref uvec4 t2, out uvec4 r) {
            r.v0 = s1 * t2.v0;
            r.v1 = s1 * t2.v1;
            r.v2 = s1 * t2.v2;
            r.v3 = s1 * t2.v3;
        }
        
        public static void Divide(uint s1, ref uvec4 t2, out uvec4 r) {
            r.v0 = s1 / t2.v0;
            r.v1 = s1 / t2.v1;
            r.v2 = s1 / t2.v2;
            r.v3 = s1 / t2.v3;
        }
        
        public static void Modulo(uint s1, ref uvec4 t2, out uvec4 r) {
            r.v0 = s1 % t2.v0;
            r.v1 = s1 % t2.v1;
            r.v2 = s1 % t2.v2;
            r.v3 = s1 % t2.v3;
        }
        
        public static void And(uint s1, ref uvec4 t2, out uvec4 r) {
            r.v0 = s1 & t2.v0;
            r.v1 = s1 & t2.v1;
            r.v2 = s1 & t2.v2;
            r.v3 = s1 & t2.v3;
        }
        
        public static void Or(uint s1, ref uvec4 t2, out uvec4 r) {
            r.v0 = s1 | t2.v0;
            r.v1 = s1 | t2.v1;
            r.v2 = s1 | t2.v2;
            r.v3 = s1 | t2.v3;
        }
        
        public static void ExclusiveOr(uint s1, ref uvec4 t2, out uvec4 r) {
            r.v0 = s1 ^ t2.v0;
            r.v1 = s1 ^ t2.v1;
            r.v2 = s1 ^ t2.v2;
            r.v3 = s1 ^ t2.v3;
        }
        
        public static void Add(ref uvec4 t1, uint s2, out uvec4 r) {
            r.v0 = t1.v0 + s2;
            r.v1 = t1.v1 + s2;
            r.v2 = t1.v2 + s2;
            r.v3 = t1.v3 + s2;
        }
        
        public static void Subtract(ref uvec4 t1, uint s2, out uvec4 r) {
            r.v0 = t1.v0 - s2;
            r.v1 = t1.v1 - s2;
            r.v2 = t1.v2 - s2;
            r.v3 = t1.v3 - s2;
        }
        
        public static void Multiply(ref uvec4 t1, uint s2, out uvec4 r) {
            r.v0 = t1.v0 * s2;
            r.v1 = t1.v1 * s2;
            r.v2 = t1.v2 * s2;
            r.v3 = t1.v3 * s2;
        }
        
        public static void Divide(ref uvec4 t1, uint s2, out uvec4 r) {
            r.v0 = t1.v0 / s2;
            r.v1 = t1.v1 / s2;
            r.v2 = t1.v2 / s2;
            r.v3 = t1.v3 / s2;
        }
        
        public static void Modulo(ref uvec4 t1, uint s2, out uvec4 r) {
            r.v0 = t1.v0 % s2;
            r.v1 = t1.v1 % s2;
            r.v2 = t1.v2 % s2;
            r.v3 = t1.v3 % s2;
        }
        
        public static void And(ref uvec4 t1, uint s2, out uvec4 r) {
            r.v0 = t1.v0 & s2;
            r.v1 = t1.v1 & s2;
            r.v2 = t1.v2 & s2;
            r.v3 = t1.v3 & s2;
        }
        
        public static void Or(ref uvec4 t1, uint s2, out uvec4 r) {
            r.v0 = t1.v0 | s2;
            r.v1 = t1.v1 | s2;
            r.v2 = t1.v2 | s2;
            r.v3 = t1.v3 | s2;
        }
        
        public static void ExclusiveOr(ref uvec4 t1, uint s2, out uvec4 r) {
            r.v0 = t1.v0 ^ s2;
            r.v1 = t1.v1 ^ s2;
            r.v2 = t1.v2 ^ s2;
            r.v3 = t1.v3 ^ s2;
        }
        
        public static void Add(ref uvec4 t1, ref uvec4 t2, out uvec4 r) {
            r.v0 = t1.v0 + t2.v0;
            r.v1 = t1.v1 + t2.v1;
            r.v2 = t1.v2 + t2.v2;
            r.v3 = t1.v3 + t2.v3;
        }
        
        public static void Subtract(ref uvec4 t1, ref uvec4 t2, out uvec4 r) {
            r.v0 = t1.v0 - t2.v0;
            r.v1 = t1.v1 - t2.v1;
            r.v2 = t1.v2 - t2.v2;
            r.v3 = t1.v3 - t2.v3;
        }
        
        public static void Multiply(ref uvec4 t1, ref uvec4 t2, out uvec4 r) {
            r.v0 = t1.v0 * t2.v0;
            r.v1 = t1.v1 * t2.v1;
            r.v2 = t1.v2 * t2.v2;
            r.v3 = t1.v3 * t2.v3;
        }
        
        public static void Divide(ref uvec4 t1, ref uvec4 t2, out uvec4 r) {
            r.v0 = t1.v0 / t2.v0;
            r.v1 = t1.v1 / t2.v1;
            r.v2 = t1.v2 / t2.v2;
            r.v3 = t1.v3 / t2.v3;
        }
        
        public static void Modulo(ref uvec4 t1, ref uvec4 t2, out uvec4 r) {
            r.v0 = t1.v0 % t2.v0;
            r.v1 = t1.v1 % t2.v1;
            r.v2 = t1.v2 % t2.v2;
            r.v3 = t1.v3 % t2.v3;
        }
        
        public static void And(ref uvec4 t1, ref uvec4 t2, out uvec4 r) {
            r.v0 = t1.v0 & t2.v0;
            r.v1 = t1.v1 & t2.v1;
            r.v2 = t1.v2 & t2.v2;
            r.v3 = t1.v3 & t2.v3;
        }
        
        public static void Or(ref uvec4 t1, ref uvec4 t2, out uvec4 r) {
            r.v0 = t1.v0 | t2.v0;
            r.v1 = t1.v1 | t2.v1;
            r.v2 = t1.v2 | t2.v2;
            r.v3 = t1.v3 | t2.v3;
        }
        
        public static void ExclusiveOr(ref uvec4 t1, ref uvec4 t2, out uvec4 r) {
            r.v0 = t1.v0 ^ t2.v0;
            r.v1 = t1.v1 ^ t2.v1;
            r.v2 = t1.v2 ^ t2.v2;
            r.v3 = t1.v3 ^ t2.v3;
        }
        
        public static void Equal(ref uvec4 t1, ref uvec4 t2, out bvec4 r) {
            r.v0 = t1.v0 == t2.v0;
            r.v1 = t1.v1 == t2.v1;
            r.v2 = t1.v2 == t2.v2;
            r.v3 = t1.v3 == t2.v3;
        }
        
        public static void GreaterThan(ref uvec4 t1, ref uvec4 t2, out bvec4 r) {
            r.v0 = t1.v0 > t2.v0;
            r.v1 = t1.v1 > t2.v1;
            r.v2 = t1.v2 > t2.v2;
            r.v3 = t1.v3 > t2.v3;
        }
        
        public static void GreaterThanEqual(ref uvec4 t1, ref uvec4 t2, out bvec4 r) {
            r.v0 = t1.v0 >= t2.v0;
            r.v1 = t1.v1 >= t2.v1;
            r.v2 = t1.v2 >= t2.v2;
            r.v3 = t1.v3 >= t2.v3;
        }
        
        public static void LessThan(ref uvec4 t1, ref uvec4 t2, out bvec4 r) {
            r.v0 = t1.v0 < t2.v0;
            r.v1 = t1.v1 < t2.v1;
            r.v2 = t1.v2 < t2.v2;
            r.v3 = t1.v3 < t2.v3;
        }
        
        public static void LessThanEqual(ref uvec4 t1, ref uvec4 t2, out bvec4 r) {
            r.v0 = t1.v0 <= t2.v0;
            r.v1 = t1.v1 <= t2.v1;
            r.v2 = t1.v2 <= t2.v2;
            r.v3 = t1.v3 <= t2.v3;
        }
        
        public static void NotEqual(ref uvec4 t1, ref uvec4 t2, out bvec4 r) {
            r.v0 = t1.v0 != t2.v0;
            r.v1 = t1.v1 != t2.v1;
            r.v2 = t1.v2 != t2.v2;
            r.v3 = t1.v3 != t2.v3;
        }
    }
}
