#region --- License ---
/*
Copyright 2010 Daniel Filonik

Permission is hereby granted, free of charge, to any person obtaining a copy of
this software and associated documentation files (the "Software"), to deal in
the Software without restriction, including without limitation the rights to
use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
of the Software, and to permit persons to whom the Software is furnished to do
so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all
copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
SOFTWARE.
*/
#endregion

namespace TKM {
    using System;
    using System.Runtime.InteropServices;
    using System.Runtime.Serialization;
    
    
    [DataContract()]
    [StructLayout(LayoutKind.Sequential)]
    public struct bmat2 {
        
        [DataMember()]
        internal bool m00;
        
        [DataMember()]
        internal bool m01;
        
        [DataMember()]
        internal bool m10;
        
        [DataMember()]
        internal bool m11;
        
        public bmat2(bool s) {
            this.m00 = s;
            this.m01 = false;
            this.m10 = false;
            this.m11 = s;
        }
        
        public bmat2(bmat2 m00) {
            this.m00 = m00.m00;
            this.m01 = m00.m01;
            this.m10 = m00.m10;
            this.m11 = m00.m11;
        }
        
        public bmat2(bvec2 m00, bvec2 m10) {
            this.m00 = m00.v0;
            this.m01 = m00.v1;
            this.m10 = m10.v0;
            this.m11 = m10.v1;
        }
        
        public bmat2(bool m00, bool m01, bool m10, bool m11) {
            this.m00 = m00;
            this.m01 = m01;
            this.m10 = m10;
            this.m11 = m11;
        }
        
        public bmat2(bmat2x3 s) {
            this.m00 = s.m00;
            this.m01 = s.m01;
            this.m10 = s.m10;
            this.m11 = s.m11;
        }
        
        public bmat2(bmat2x4 s) {
            this.m00 = s.m00;
            this.m01 = s.m01;
            this.m10 = s.m10;
            this.m11 = s.m11;
        }
        
        public bmat2(bmat3x2 s) {
            this.m00 = s.m00;
            this.m01 = s.m01;
            this.m10 = s.m10;
            this.m11 = s.m11;
        }
        
        public bmat2(bmat3 s) {
            this.m00 = s.m00;
            this.m01 = s.m01;
            this.m10 = s.m10;
            this.m11 = s.m11;
        }
        
        public bmat2(bmat3x4 s) {
            this.m00 = s.m00;
            this.m01 = s.m01;
            this.m10 = s.m10;
            this.m11 = s.m11;
        }
        
        public bmat2(bmat4x2 s) {
            this.m00 = s.m00;
            this.m01 = s.m01;
            this.m10 = s.m10;
            this.m11 = s.m11;
        }
        
        public bmat2(bmat4x3 s) {
            this.m00 = s.m00;
            this.m01 = s.m01;
            this.m10 = s.m10;
            this.m11 = s.m11;
        }
        
        public bmat2(bmat4 s) {
            this.m00 = s.m00;
            this.m01 = s.m01;
            this.m10 = s.m10;
            this.m11 = s.m11;
        }
        
        public bool this[int i0, int i1] {
            get {
                if (((i0 < 0) 
                            || (1 < i0))) {
                    throw new System.IndexOutOfRangeException("i0");
                }
                if (((i1 < 0) 
                            || (1 < i1))) {
                    throw new System.IndexOutOfRangeException("i1");
                }
                
                unsafe
                {
                    fixed (Boolean* v_ptr = &this.m00)
                    {
                        return *(v_ptr + (i1 + i0 * 2));
                    }
                };
            }
            set {
                if (((i0 < 0) 
                            || (1 < i0))) {
                    throw new System.IndexOutOfRangeException("i0");
                }
                if (((i1 < 0) 
                            || (1 < i1))) {
                    throw new System.IndexOutOfRangeException("i1");
                }
                
                unsafe
                {
                    fixed (Boolean* v_ptr = &this.m00)
                    {
                        *(v_ptr  + (i1 + i0 * 2)) = value;
                    }
                };
            }
        }
        
        public bvec2 this[int i0] {
            get {
                return new bvec2(this[i0, 0], this[i0, 1]);
            }
            set {
                this[i0, 0] = value[0];
                this[i0, 1] = value[1];
            }
        }
        
        public static bmat2 operator!(bmat2 v) {
            return new bmat2(!v.m00, !v.m01, !v.m10, !v.m11);
        }
        
        public static bmat2 operator&(bool s, bmat2 v2) {
            return new bmat2(s & v2.m00, s & v2.m01, s & v2.m10, s & v2.m11);
        }
        
        public static bmat2 operator|(bool s, bmat2 v2) {
            return new bmat2(s | v2.m00, s | v2.m01, s | v2.m10, s | v2.m11);
        }
        
        public static bmat2 operator^(bool s, bmat2 v2) {
            return new bmat2(s ^ v2.m00, s ^ v2.m01, s ^ v2.m10, s ^ v2.m11);
        }
        
        public static bmat2 operator&(bmat2 v1, bool s) {
            return new bmat2(v1.m00 & s, v1.m01 & s, v1.m10 & s, v1.m11 & s);
        }
        
        public static bmat2 operator|(bmat2 v1, bool s) {
            return new bmat2(v1.m00 | s, v1.m01 | s, v1.m10 | s, v1.m11 | s);
        }
        
        public static bmat2 operator^(bmat2 v1, bool s) {
            return new bmat2(v1.m00 ^ s, v1.m01 ^ s, v1.m10 ^ s, v1.m11 ^ s);
        }
        
        public static bmat2 operator&(bmat2 v1, bmat2 v2) {
            return new bmat2(v1.m00 & v2.m00, v1.m01 & v2.m01, v1.m10 & v2.m10, v1.m11 & v2.m11);
        }
        
        public static bmat2 operator|(bmat2 v1, bmat2 v2) {
            return new bmat2(v1.m00 | v2.m00, v1.m01 | v2.m01, v1.m10 | v2.m10, v1.m11 | v2.m11);
        }
        
        public static bmat2 operator^(bmat2 v1, bmat2 v2) {
            return new bmat2(v1.m00 ^ v2.m00, v1.m01 ^ v2.m01, v1.m10 ^ v2.m10, v1.m11 ^ v2.m11);
        }
        
        public static bmat2 operator==(bmat2 v1, bmat2 v2) {
            return new bmat2(v1.m00 == v2.m00, v1.m01 == v2.m01, v1.m10 == v2.m10, v1.m11 == v2.m11);
        }
        
        public static bmat2 operator!=(bmat2 v1, bmat2 v2) {
            return new bmat2(v1.m00 != v2.m00, v1.m01 != v2.m01, v1.m10 != v2.m10, v1.m11 != v2.m11);
        }
        
        public override bool Equals(object obj) {
            return (obj is bmat2) ? bmath.All(this == (bmat2)obj) : false;
        }
        
        public override int GetHashCode() {
            return this.m00.GetHashCode() ^ this.m01.GetHashCode() ^ this.m10.GetHashCode() ^ this.m11.GetHashCode();
        }
        
        public override string ToString() {
            return "bmat2(" + this.m00 + "," + this.m01 + "," + this.m10 + "," + this.m11 + ")";
        }
    }
    
    [DataContract()]
    [StructLayout(LayoutKind.Sequential)]
    public struct bmat2x3 {
        
        [DataMember()]
        internal bool m00;
        
        [DataMember()]
        internal bool m01;
        
        [DataMember()]
        internal bool m02;
        
        [DataMember()]
        internal bool m10;
        
        [DataMember()]
        internal bool m11;
        
        [DataMember()]
        internal bool m12;
        
        public bmat2x3(bool s) {
            this.m00 = s;
            this.m01 = false;
            this.m02 = false;
            this.m10 = false;
            this.m11 = s;
            this.m12 = false;
        }
        
        public bmat2x3(bmat2x3 m00) {
            this.m00 = m00.m00;
            this.m01 = m00.m01;
            this.m02 = m00.m02;
            this.m10 = m00.m10;
            this.m11 = m00.m11;
            this.m12 = m00.m12;
        }
        
        public bmat2x3(bvec3 m00, bvec3 m10) {
            this.m00 = m00.v0;
            this.m01 = m00.v1;
            this.m02 = m00.v2;
            this.m10 = m10.v0;
            this.m11 = m10.v1;
            this.m12 = m10.v2;
        }
        
        public bmat2x3(bvec2 m00, bool m01, bvec2 m10, bool m11) {
            this.m00 = m00.v0;
            this.m01 = m00.v1;
            this.m02 = m01;
            this.m10 = m10.v0;
            this.m11 = m10.v1;
            this.m12 = m11;
        }
        
        public bmat2x3(bool m00, bvec2 m01, bool m10, bvec2 m11) {
            this.m00 = m00;
            this.m01 = m01.v0;
            this.m02 = m01.v1;
            this.m10 = m10;
            this.m11 = m11.v0;
            this.m12 = m11.v1;
        }
        
        public bmat2x3(bool m00, bool m01, bool m02, bool m10, bool m11, bool m12) {
            this.m00 = m00;
            this.m01 = m01;
            this.m02 = m02;
            this.m10 = m10;
            this.m11 = m11;
            this.m12 = m12;
        }
        
        public bmat2x3(bmat2 s) {
            this.m00 = s.m00;
            this.m01 = s.m01;
            this.m02 = false;
            this.m10 = s.m10;
            this.m11 = s.m11;
            this.m12 = false;
        }
        
        public bmat2x3(bmat2x4 s) {
            this.m00 = s.m00;
            this.m01 = s.m01;
            this.m02 = s.m02;
            this.m10 = s.m10;
            this.m11 = s.m11;
            this.m12 = s.m12;
        }
        
        public bmat2x3(bmat3x2 s) {
            this.m00 = s.m00;
            this.m01 = s.m01;
            this.m02 = false;
            this.m10 = s.m10;
            this.m11 = s.m11;
            this.m12 = false;
        }
        
        public bmat2x3(bmat3 s) {
            this.m00 = s.m00;
            this.m01 = s.m01;
            this.m02 = s.m02;
            this.m10 = s.m10;
            this.m11 = s.m11;
            this.m12 = s.m12;
        }
        
        public bmat2x3(bmat3x4 s) {
            this.m00 = s.m00;
            this.m01 = s.m01;
            this.m02 = s.m02;
            this.m10 = s.m10;
            this.m11 = s.m11;
            this.m12 = s.m12;
        }
        
        public bmat2x3(bmat4x2 s) {
            this.m00 = s.m00;
            this.m01 = s.m01;
            this.m02 = false;
            this.m10 = s.m10;
            this.m11 = s.m11;
            this.m12 = false;
        }
        
        public bmat2x3(bmat4x3 s) {
            this.m00 = s.m00;
            this.m01 = s.m01;
            this.m02 = s.m02;
            this.m10 = s.m10;
            this.m11 = s.m11;
            this.m12 = s.m12;
        }
        
        public bmat2x3(bmat4 s) {
            this.m00 = s.m00;
            this.m01 = s.m01;
            this.m02 = s.m02;
            this.m10 = s.m10;
            this.m11 = s.m11;
            this.m12 = s.m12;
        }
        
        public bool this[int i0, int i1] {
            get {
                if (((i0 < 0) 
                            || (1 < i0))) {
                    throw new System.IndexOutOfRangeException("i0");
                }
                if (((i1 < 0) 
                            || (2 < i1))) {
                    throw new System.IndexOutOfRangeException("i1");
                }
                
                unsafe
                {
                    fixed (Boolean* v_ptr = &this.m00)
                    {
                        return *(v_ptr + (i1 + i0 * 3));
                    }
                };
            }
            set {
                if (((i0 < 0) 
                            || (1 < i0))) {
                    throw new System.IndexOutOfRangeException("i0");
                }
                if (((i1 < 0) 
                            || (2 < i1))) {
                    throw new System.IndexOutOfRangeException("i1");
                }
                
                unsafe
                {
                    fixed (Boolean* v_ptr = &this.m00)
                    {
                        *(v_ptr  + (i1 + i0 * 3)) = value;
                    }
                };
            }
        }
        
        public bvec3 this[int i0] {
            get {
                return new bvec3(this[i0, 0], this[i0, 1], this[i0, 2]);
            }
            set {
                this[i0, 0] = value[0];
                this[i0, 1] = value[1];
                this[i0, 2] = value[2];
            }
        }
        
        public static bmat2x3 operator!(bmat2x3 v) {
            return new bmat2x3(!v.m00, !v.m01, !v.m02, !v.m10, !v.m11, !v.m12);
        }
        
        public static bmat2x3 operator&(bool s, bmat2x3 v2) {
            return new bmat2x3(s & v2.m00, s & v2.m01, s & v2.m02, s & v2.m10, s & v2.m11, s & v2.m12);
        }
        
        public static bmat2x3 operator|(bool s, bmat2x3 v2) {
            return new bmat2x3(s | v2.m00, s | v2.m01, s | v2.m02, s | v2.m10, s | v2.m11, s | v2.m12);
        }
        
        public static bmat2x3 operator^(bool s, bmat2x3 v2) {
            return new bmat2x3(s ^ v2.m00, s ^ v2.m01, s ^ v2.m02, s ^ v2.m10, s ^ v2.m11, s ^ v2.m12);
        }
        
        public static bmat2x3 operator&(bmat2x3 v1, bool s) {
            return new bmat2x3(v1.m00 & s, v1.m01 & s, v1.m02 & s, v1.m10 & s, v1.m11 & s, v1.m12 & s);
        }
        
        public static bmat2x3 operator|(bmat2x3 v1, bool s) {
            return new bmat2x3(v1.m00 | s, v1.m01 | s, v1.m02 | s, v1.m10 | s, v1.m11 | s, v1.m12 | s);
        }
        
        public static bmat2x3 operator^(bmat2x3 v1, bool s) {
            return new bmat2x3(v1.m00 ^ s, v1.m01 ^ s, v1.m02 ^ s, v1.m10 ^ s, v1.m11 ^ s, v1.m12 ^ s);
        }
        
        public static bmat2x3 operator&(bmat2x3 v1, bmat2x3 v2) {
            return new bmat2x3(v1.m00 & v2.m00, v1.m01 & v2.m01, v1.m02 & v2.m02, v1.m10 & v2.m10, v1.m11 & v2.m11, v1.m12 & v2.m12);
        }
        
        public static bmat2x3 operator|(bmat2x3 v1, bmat2x3 v2) {
            return new bmat2x3(v1.m00 | v2.m00, v1.m01 | v2.m01, v1.m02 | v2.m02, v1.m10 | v2.m10, v1.m11 | v2.m11, v1.m12 | v2.m12);
        }
        
        public static bmat2x3 operator^(bmat2x3 v1, bmat2x3 v2) {
            return new bmat2x3(v1.m00 ^ v2.m00, v1.m01 ^ v2.m01, v1.m02 ^ v2.m02, v1.m10 ^ v2.m10, v1.m11 ^ v2.m11, v1.m12 ^ v2.m12);
        }
        
        public static bmat2x3 operator==(bmat2x3 v1, bmat2x3 v2) {
            return new bmat2x3(v1.m00 == v2.m00, v1.m01 == v2.m01, v1.m02 == v2.m02, v1.m10 == v2.m10, v1.m11 == v2.m11, v1.m12 == v2.m12);
        }
        
        public static bmat2x3 operator!=(bmat2x3 v1, bmat2x3 v2) {
            return new bmat2x3(v1.m00 != v2.m00, v1.m01 != v2.m01, v1.m02 != v2.m02, v1.m10 != v2.m10, v1.m11 != v2.m11, v1.m12 != v2.m12);
        }
        
        public override bool Equals(object obj) {
            return (obj is bmat2x3) ? bmath.All(this == (bmat2x3)obj) : false;
        }
        
        public override int GetHashCode() {
            return this.m00.GetHashCode() ^ this.m01.GetHashCode() ^ this.m02.GetHashCode() ^ this.m10.GetHashCode() ^ this.m11.GetHashCode() ^ this.m12.GetHashCode();
        }
        
        public override string ToString() {
            return "bmat2x3(" + this.m00 + "," + this.m01 + "," + this.m02 + "," + this.m10 + "," + this.m11 + "," + this.m12 + ")";
        }
    }
    
    [DataContract()]
    [StructLayout(LayoutKind.Sequential)]
    public struct bmat2x4 {
        
        [DataMember()]
        internal bool m00;
        
        [DataMember()]
        internal bool m01;
        
        [DataMember()]
        internal bool m02;
        
        [DataMember()]
        internal bool m03;
        
        [DataMember()]
        internal bool m10;
        
        [DataMember()]
        internal bool m11;
        
        [DataMember()]
        internal bool m12;
        
        [DataMember()]
        internal bool m13;
        
        public bmat2x4(bool s) {
            this.m00 = s;
            this.m01 = false;
            this.m02 = false;
            this.m03 = false;
            this.m10 = false;
            this.m11 = s;
            this.m12 = false;
            this.m13 = false;
        }
        
        public bmat2x4(bmat2x4 m00) {
            this.m00 = m00.m00;
            this.m01 = m00.m01;
            this.m02 = m00.m02;
            this.m03 = m00.m03;
            this.m10 = m00.m10;
            this.m11 = m00.m11;
            this.m12 = m00.m12;
            this.m13 = m00.m13;
        }
        
        public bmat2x4(bmat2 m00, bmat2 m01) {
            this.m00 = m00.m00;
            this.m01 = m00.m01;
            this.m02 = m01.m00;
            this.m03 = m01.m01;
            this.m10 = m00.m10;
            this.m11 = m00.m11;
            this.m12 = m01.m10;
            this.m13 = m01.m11;
        }
        
        public bmat2x4(bvec4 m00, bvec4 m10) {
            this.m00 = m00.v0;
            this.m01 = m00.v1;
            this.m02 = m00.v2;
            this.m03 = m00.v3;
            this.m10 = m10.v0;
            this.m11 = m10.v1;
            this.m12 = m10.v2;
            this.m13 = m10.v3;
        }
        
        public bmat2x4(bvec2 m00, bvec2 m01, bvec2 m10, bvec2 m11) {
            this.m00 = m00.v0;
            this.m01 = m00.v1;
            this.m02 = m01.v0;
            this.m03 = m01.v1;
            this.m10 = m10.v0;
            this.m11 = m10.v1;
            this.m12 = m11.v0;
            this.m13 = m11.v1;
        }
        
        public bmat2x4(bvec3 m00, bool m01, bvec3 m10, bool m11) {
            this.m00 = m00.v0;
            this.m01 = m00.v1;
            this.m02 = m00.v2;
            this.m03 = m01;
            this.m10 = m10.v0;
            this.m11 = m10.v1;
            this.m12 = m10.v2;
            this.m13 = m11;
        }
        
        public bmat2x4(bool m00, bvec3 m01, bool m10, bvec3 m11) {
            this.m00 = m00;
            this.m01 = m01.v0;
            this.m02 = m01.v1;
            this.m03 = m01.v2;
            this.m10 = m10;
            this.m11 = m11.v0;
            this.m12 = m11.v1;
            this.m13 = m11.v2;
        }
        
        public bmat2x4(bvec2 m00, bool m01, bool m02, bvec2 m10, bool m11, bool m12) {
            this.m00 = m00.v0;
            this.m01 = m00.v1;
            this.m02 = m01;
            this.m03 = m02;
            this.m10 = m10.v0;
            this.m11 = m10.v1;
            this.m12 = m11;
            this.m13 = m12;
        }
        
        public bmat2x4(bool m00, bvec2 m01, bool m02, bool m10, bvec2 m11, bool m12) {
            this.m00 = m00;
            this.m01 = m01.v0;
            this.m02 = m01.v1;
            this.m03 = m02;
            this.m10 = m10;
            this.m11 = m11.v0;
            this.m12 = m11.v1;
            this.m13 = m12;
        }
        
        public bmat2x4(bool m00, bool m01, bvec2 m02, bool m10, bool m11, bvec2 m12) {
            this.m00 = m00;
            this.m01 = m01;
            this.m02 = m02.v0;
            this.m03 = m02.v1;
            this.m10 = m10;
            this.m11 = m11;
            this.m12 = m12.v0;
            this.m13 = m12.v1;
        }
        
        public bmat2x4(bool m00, bool m01, bool m02, bool m03, bool m10, bool m11, bool m12, bool m13) {
            this.m00 = m00;
            this.m01 = m01;
            this.m02 = m02;
            this.m03 = m03;
            this.m10 = m10;
            this.m11 = m11;
            this.m12 = m12;
            this.m13 = m13;
        }
        
        public bmat2x4(bmat2 s) {
            this.m00 = s.m00;
            this.m01 = s.m01;
            this.m02 = false;
            this.m03 = false;
            this.m10 = s.m10;
            this.m11 = s.m11;
            this.m12 = false;
            this.m13 = false;
        }
        
        public bmat2x4(bmat2x3 s) {
            this.m00 = s.m00;
            this.m01 = s.m01;
            this.m02 = s.m02;
            this.m03 = false;
            this.m10 = s.m10;
            this.m11 = s.m11;
            this.m12 = s.m12;
            this.m13 = false;
        }
        
        public bmat2x4(bmat3x2 s) {
            this.m00 = s.m00;
            this.m01 = s.m01;
            this.m02 = false;
            this.m03 = false;
            this.m10 = s.m10;
            this.m11 = s.m11;
            this.m12 = false;
            this.m13 = false;
        }
        
        public bmat2x4(bmat3 s) {
            this.m00 = s.m00;
            this.m01 = s.m01;
            this.m02 = s.m02;
            this.m03 = false;
            this.m10 = s.m10;
            this.m11 = s.m11;
            this.m12 = s.m12;
            this.m13 = false;
        }
        
        public bmat2x4(bmat3x4 s) {
            this.m00 = s.m00;
            this.m01 = s.m01;
            this.m02 = s.m02;
            this.m03 = s.m03;
            this.m10 = s.m10;
            this.m11 = s.m11;
            this.m12 = s.m12;
            this.m13 = s.m13;
        }
        
        public bmat2x4(bmat4x2 s) {
            this.m00 = s.m00;
            this.m01 = s.m01;
            this.m02 = false;
            this.m03 = false;
            this.m10 = s.m10;
            this.m11 = s.m11;
            this.m12 = false;
            this.m13 = false;
        }
        
        public bmat2x4(bmat4x3 s) {
            this.m00 = s.m00;
            this.m01 = s.m01;
            this.m02 = s.m02;
            this.m03 = false;
            this.m10 = s.m10;
            this.m11 = s.m11;
            this.m12 = s.m12;
            this.m13 = false;
        }
        
        public bmat2x4(bmat4 s) {
            this.m00 = s.m00;
            this.m01 = s.m01;
            this.m02 = s.m02;
            this.m03 = s.m03;
            this.m10 = s.m10;
            this.m11 = s.m11;
            this.m12 = s.m12;
            this.m13 = s.m13;
        }
        
        public bool this[int i0, int i1] {
            get {
                if (((i0 < 0) 
                            || (1 < i0))) {
                    throw new System.IndexOutOfRangeException("i0");
                }
                if (((i1 < 0) 
                            || (3 < i1))) {
                    throw new System.IndexOutOfRangeException("i1");
                }
                
                unsafe
                {
                    fixed (Boolean* v_ptr = &this.m00)
                    {
                        return *(v_ptr + (i1 + i0 * 4));
                    }
                };
            }
            set {
                if (((i0 < 0) 
                            || (1 < i0))) {
                    throw new System.IndexOutOfRangeException("i0");
                }
                if (((i1 < 0) 
                            || (3 < i1))) {
                    throw new System.IndexOutOfRangeException("i1");
                }
                
                unsafe
                {
                    fixed (Boolean* v_ptr = &this.m00)
                    {
                        *(v_ptr  + (i1 + i0 * 4)) = value;
                    }
                };
            }
        }
        
        public bvec4 this[int i0] {
            get {
                return new bvec4(this[i0, 0], this[i0, 1], this[i0, 2], this[i0, 3]);
            }
            set {
                this[i0, 0] = value[0];
                this[i0, 1] = value[1];
                this[i0, 2] = value[2];
                this[i0, 3] = value[3];
            }
        }
        
        public static bmat2x4 operator!(bmat2x4 v) {
            return new bmat2x4(!v.m00, !v.m01, !v.m02, !v.m03, !v.m10, !v.m11, !v.m12, !v.m13);
        }
        
        public static bmat2x4 operator&(bool s, bmat2x4 v2) {
            return new bmat2x4(s & v2.m00, s & v2.m01, s & v2.m02, s & v2.m03, s & v2.m10, s & v2.m11, s & v2.m12, s & v2.m13);
        }
        
        public static bmat2x4 operator|(bool s, bmat2x4 v2) {
            return new bmat2x4(s | v2.m00, s | v2.m01, s | v2.m02, s | v2.m03, s | v2.m10, s | v2.m11, s | v2.m12, s | v2.m13);
        }
        
        public static bmat2x4 operator^(bool s, bmat2x4 v2) {
            return new bmat2x4(s ^ v2.m00, s ^ v2.m01, s ^ v2.m02, s ^ v2.m03, s ^ v2.m10, s ^ v2.m11, s ^ v2.m12, s ^ v2.m13);
        }
        
        public static bmat2x4 operator&(bmat2x4 v1, bool s) {
            return new bmat2x4(v1.m00 & s, v1.m01 & s, v1.m02 & s, v1.m03 & s, v1.m10 & s, v1.m11 & s, v1.m12 & s, v1.m13 & s);
        }
        
        public static bmat2x4 operator|(bmat2x4 v1, bool s) {
            return new bmat2x4(v1.m00 | s, v1.m01 | s, v1.m02 | s, v1.m03 | s, v1.m10 | s, v1.m11 | s, v1.m12 | s, v1.m13 | s);
        }
        
        public static bmat2x4 operator^(bmat2x4 v1, bool s) {
            return new bmat2x4(v1.m00 ^ s, v1.m01 ^ s, v1.m02 ^ s, v1.m03 ^ s, v1.m10 ^ s, v1.m11 ^ s, v1.m12 ^ s, v1.m13 ^ s);
        }
        
        public static bmat2x4 operator&(bmat2x4 v1, bmat2x4 v2) {
            return new bmat2x4(v1.m00 & v2.m00, v1.m01 & v2.m01, v1.m02 & v2.m02, v1.m03 & v2.m03, v1.m10 & v2.m10, v1.m11 & v2.m11, v1.m12 & v2.m12, v1.m13 & v2.m13);
        }
        
        public static bmat2x4 operator|(bmat2x4 v1, bmat2x4 v2) {
            return new bmat2x4(v1.m00 | v2.m00, v1.m01 | v2.m01, v1.m02 | v2.m02, v1.m03 | v2.m03, v1.m10 | v2.m10, v1.m11 | v2.m11, v1.m12 | v2.m12, v1.m13 | v2.m13);
        }
        
        public static bmat2x4 operator^(bmat2x4 v1, bmat2x4 v2) {
            return new bmat2x4(v1.m00 ^ v2.m00, v1.m01 ^ v2.m01, v1.m02 ^ v2.m02, v1.m03 ^ v2.m03, v1.m10 ^ v2.m10, v1.m11 ^ v2.m11, v1.m12 ^ v2.m12, v1.m13 ^ v2.m13);
        }
        
        public static bmat2x4 operator==(bmat2x4 v1, bmat2x4 v2) {
            return new bmat2x4(v1.m00 == v2.m00, v1.m01 == v2.m01, v1.m02 == v2.m02, v1.m03 == v2.m03, v1.m10 == v2.m10, v1.m11 == v2.m11, v1.m12 == v2.m12, v1.m13 == v2.m13);
        }
        
        public static bmat2x4 operator!=(bmat2x4 v1, bmat2x4 v2) {
            return new bmat2x4(v1.m00 != v2.m00, v1.m01 != v2.m01, v1.m02 != v2.m02, v1.m03 != v2.m03, v1.m10 != v2.m10, v1.m11 != v2.m11, v1.m12 != v2.m12, v1.m13 != v2.m13);
        }
        
        public override bool Equals(object obj) {
            return (obj is bmat2x4) ? bmath.All(this == (bmat2x4)obj) : false;
        }
        
        public override int GetHashCode() {
            return this.m00.GetHashCode() ^ this.m01.GetHashCode() ^ this.m02.GetHashCode() ^ this.m03.GetHashCode() ^ this.m10.GetHashCode() ^ this.m11.GetHashCode() ^ this.m12.GetHashCode() ^ this.m13.GetHashCode();
        }
        
        public override string ToString() {
            return "bmat2x4(" + this.m00 + "," + this.m01 + "," + this.m02 + "," + this.m03 + "," + this.m10 + "," + this.m11 + "," + this.m12 + "," + this.m13 + ")";
        }
    }
    
    [DataContract()]
    [StructLayout(LayoutKind.Sequential)]
    public struct bmat3x2 {
        
        [DataMember()]
        internal bool m00;
        
        [DataMember()]
        internal bool m01;
        
        [DataMember()]
        internal bool m10;
        
        [DataMember()]
        internal bool m11;
        
        [DataMember()]
        internal bool m20;
        
        [DataMember()]
        internal bool m21;
        
        public bmat3x2(bool s) {
            this.m00 = s;
            this.m01 = false;
            this.m10 = false;
            this.m11 = s;
            this.m20 = false;
            this.m21 = false;
        }
        
        public bmat3x2(bmat3x2 m00) {
            this.m00 = m00.m00;
            this.m01 = m00.m01;
            this.m10 = m00.m10;
            this.m11 = m00.m11;
            this.m20 = m00.m20;
            this.m21 = m00.m21;
        }
        
        public bmat3x2(bmat2 m00, bvec2 m10) {
            this.m00 = m00.m00;
            this.m01 = m00.m01;
            this.m10 = m00.m10;
            this.m11 = m00.m11;
            this.m20 = m10.v0;
            this.m21 = m10.v1;
        }
        
        public bmat3x2(bvec2 m00, bmat2 m10) {
            this.m00 = m00.v0;
            this.m01 = m00.v1;
            this.m10 = m10.m00;
            this.m11 = m10.m01;
            this.m20 = m10.m10;
            this.m21 = m10.m11;
        }
        
        public bmat3x2(bvec2 m00, bvec2 m10, bvec2 m20) {
            this.m00 = m00.v0;
            this.m01 = m00.v1;
            this.m10 = m10.v0;
            this.m11 = m10.v1;
            this.m20 = m20.v0;
            this.m21 = m20.v1;
        }
        
        public bmat3x2(bool m00, bool m01, bool m10, bool m11, bool m20, bool m21) {
            this.m00 = m00;
            this.m01 = m01;
            this.m10 = m10;
            this.m11 = m11;
            this.m20 = m20;
            this.m21 = m21;
        }
        
        public bmat3x2(bmat2 s) {
            this.m00 = s.m00;
            this.m01 = s.m01;
            this.m10 = s.m10;
            this.m11 = s.m11;
            this.m20 = false;
            this.m21 = false;
        }
        
        public bmat3x2(bmat2x3 s) {
            this.m00 = s.m00;
            this.m01 = s.m01;
            this.m10 = s.m10;
            this.m11 = s.m11;
            this.m20 = false;
            this.m21 = false;
        }
        
        public bmat3x2(bmat2x4 s) {
            this.m00 = s.m00;
            this.m01 = s.m01;
            this.m10 = s.m10;
            this.m11 = s.m11;
            this.m20 = false;
            this.m21 = false;
        }
        
        public bmat3x2(bmat3 s) {
            this.m00 = s.m00;
            this.m01 = s.m01;
            this.m10 = s.m10;
            this.m11 = s.m11;
            this.m20 = s.m20;
            this.m21 = s.m21;
        }
        
        public bmat3x2(bmat3x4 s) {
            this.m00 = s.m00;
            this.m01 = s.m01;
            this.m10 = s.m10;
            this.m11 = s.m11;
            this.m20 = s.m20;
            this.m21 = s.m21;
        }
        
        public bmat3x2(bmat4x2 s) {
            this.m00 = s.m00;
            this.m01 = s.m01;
            this.m10 = s.m10;
            this.m11 = s.m11;
            this.m20 = s.m20;
            this.m21 = s.m21;
        }
        
        public bmat3x2(bmat4x3 s) {
            this.m00 = s.m00;
            this.m01 = s.m01;
            this.m10 = s.m10;
            this.m11 = s.m11;
            this.m20 = s.m20;
            this.m21 = s.m21;
        }
        
        public bmat3x2(bmat4 s) {
            this.m00 = s.m00;
            this.m01 = s.m01;
            this.m10 = s.m10;
            this.m11 = s.m11;
            this.m20 = s.m20;
            this.m21 = s.m21;
        }
        
        public bool this[int i0, int i1] {
            get {
                if (((i0 < 0) 
                            || (2 < i0))) {
                    throw new System.IndexOutOfRangeException("i0");
                }
                if (((i1 < 0) 
                            || (1 < i1))) {
                    throw new System.IndexOutOfRangeException("i1");
                }
                
                unsafe
                {
                    fixed (Boolean* v_ptr = &this.m00)
                    {
                        return *(v_ptr + (i1 + i0 * 2));
                    }
                };
            }
            set {
                if (((i0 < 0) 
                            || (2 < i0))) {
                    throw new System.IndexOutOfRangeException("i0");
                }
                if (((i1 < 0) 
                            || (1 < i1))) {
                    throw new System.IndexOutOfRangeException("i1");
                }
                
                unsafe
                {
                    fixed (Boolean* v_ptr = &this.m00)
                    {
                        *(v_ptr  + (i1 + i0 * 2)) = value;
                    }
                };
            }
        }
        
        public bvec2 this[int i0] {
            get {
                return new bvec2(this[i0, 0], this[i0, 1]);
            }
            set {
                this[i0, 0] = value[0];
                this[i0, 1] = value[1];
            }
        }
        
        public static bmat3x2 operator!(bmat3x2 v) {
            return new bmat3x2(!v.m00, !v.m01, !v.m10, !v.m11, !v.m20, !v.m21);
        }
        
        public static bmat3x2 operator&(bool s, bmat3x2 v2) {
            return new bmat3x2(s & v2.m00, s & v2.m01, s & v2.m10, s & v2.m11, s & v2.m20, s & v2.m21);
        }
        
        public static bmat3x2 operator|(bool s, bmat3x2 v2) {
            return new bmat3x2(s | v2.m00, s | v2.m01, s | v2.m10, s | v2.m11, s | v2.m20, s | v2.m21);
        }
        
        public static bmat3x2 operator^(bool s, bmat3x2 v2) {
            return new bmat3x2(s ^ v2.m00, s ^ v2.m01, s ^ v2.m10, s ^ v2.m11, s ^ v2.m20, s ^ v2.m21);
        }
        
        public static bmat3x2 operator&(bmat3x2 v1, bool s) {
            return new bmat3x2(v1.m00 & s, v1.m01 & s, v1.m10 & s, v1.m11 & s, v1.m20 & s, v1.m21 & s);
        }
        
        public static bmat3x2 operator|(bmat3x2 v1, bool s) {
            return new bmat3x2(v1.m00 | s, v1.m01 | s, v1.m10 | s, v1.m11 | s, v1.m20 | s, v1.m21 | s);
        }
        
        public static bmat3x2 operator^(bmat3x2 v1, bool s) {
            return new bmat3x2(v1.m00 ^ s, v1.m01 ^ s, v1.m10 ^ s, v1.m11 ^ s, v1.m20 ^ s, v1.m21 ^ s);
        }
        
        public static bmat3x2 operator&(bmat3x2 v1, bmat3x2 v2) {
            return new bmat3x2(v1.m00 & v2.m00, v1.m01 & v2.m01, v1.m10 & v2.m10, v1.m11 & v2.m11, v1.m20 & v2.m20, v1.m21 & v2.m21);
        }
        
        public static bmat3x2 operator|(bmat3x2 v1, bmat3x2 v2) {
            return new bmat3x2(v1.m00 | v2.m00, v1.m01 | v2.m01, v1.m10 | v2.m10, v1.m11 | v2.m11, v1.m20 | v2.m20, v1.m21 | v2.m21);
        }
        
        public static bmat3x2 operator^(bmat3x2 v1, bmat3x2 v2) {
            return new bmat3x2(v1.m00 ^ v2.m00, v1.m01 ^ v2.m01, v1.m10 ^ v2.m10, v1.m11 ^ v2.m11, v1.m20 ^ v2.m20, v1.m21 ^ v2.m21);
        }
        
        public static bmat3x2 operator==(bmat3x2 v1, bmat3x2 v2) {
            return new bmat3x2(v1.m00 == v2.m00, v1.m01 == v2.m01, v1.m10 == v2.m10, v1.m11 == v2.m11, v1.m20 == v2.m20, v1.m21 == v2.m21);
        }
        
        public static bmat3x2 operator!=(bmat3x2 v1, bmat3x2 v2) {
            return new bmat3x2(v1.m00 != v2.m00, v1.m01 != v2.m01, v1.m10 != v2.m10, v1.m11 != v2.m11, v1.m20 != v2.m20, v1.m21 != v2.m21);
        }
        
        public override bool Equals(object obj) {
            return (obj is bmat3x2) ? bmath.All(this == (bmat3x2)obj) : false;
        }
        
        public override int GetHashCode() {
            return this.m00.GetHashCode() ^ this.m01.GetHashCode() ^ this.m10.GetHashCode() ^ this.m11.GetHashCode() ^ this.m20.GetHashCode() ^ this.m21.GetHashCode();
        }
        
        public override string ToString() {
            return "bmat3x2(" + this.m00 + "," + this.m01 + "," + this.m10 + "," + this.m11 + "," + this.m20 + "," + this.m21 + ")";
        }
    }
    
    [DataContract()]
    [StructLayout(LayoutKind.Sequential)]
    public struct bmat3 {
        
        [DataMember()]
        internal bool m00;
        
        [DataMember()]
        internal bool m01;
        
        [DataMember()]
        internal bool m02;
        
        [DataMember()]
        internal bool m10;
        
        [DataMember()]
        internal bool m11;
        
        [DataMember()]
        internal bool m12;
        
        [DataMember()]
        internal bool m20;
        
        [DataMember()]
        internal bool m21;
        
        [DataMember()]
        internal bool m22;
        
        public bmat3(bool s) {
            this.m00 = s;
            this.m01 = false;
            this.m02 = false;
            this.m10 = false;
            this.m11 = s;
            this.m12 = false;
            this.m20 = false;
            this.m21 = false;
            this.m22 = s;
        }
        
        public bmat3(bmat3 m00) {
            this.m00 = m00.m00;
            this.m01 = m00.m01;
            this.m02 = m00.m02;
            this.m10 = m00.m10;
            this.m11 = m00.m11;
            this.m12 = m00.m12;
            this.m20 = m00.m20;
            this.m21 = m00.m21;
            this.m22 = m00.m22;
        }
        
        public bmat3(bmat2x3 m00, bvec3 m10) {
            this.m00 = m00.m00;
            this.m01 = m00.m01;
            this.m02 = m00.m02;
            this.m10 = m00.m10;
            this.m11 = m00.m11;
            this.m12 = m00.m12;
            this.m20 = m10.v0;
            this.m21 = m10.v1;
            this.m22 = m10.v2;
        }
        
        public bmat3(bvec3 m00, bmat2x3 m10) {
            this.m00 = m00.v0;
            this.m01 = m00.v1;
            this.m02 = m00.v2;
            this.m10 = m10.m00;
            this.m11 = m10.m01;
            this.m12 = m10.m02;
            this.m20 = m10.m10;
            this.m21 = m10.m11;
            this.m22 = m10.m12;
        }
        
        public bmat3(bvec3 m00, bvec3 m10, bvec3 m20) {
            this.m00 = m00.v0;
            this.m01 = m00.v1;
            this.m02 = m00.v2;
            this.m10 = m10.v0;
            this.m11 = m10.v1;
            this.m12 = m10.v2;
            this.m20 = m20.v0;
            this.m21 = m20.v1;
            this.m22 = m20.v2;
        }
        
        public bmat3(bvec2 m00, bool m01, bvec2 m10, bool m11, bvec2 m20, bool m21) {
            this.m00 = m00.v0;
            this.m01 = m00.v1;
            this.m02 = m01;
            this.m10 = m10.v0;
            this.m11 = m10.v1;
            this.m12 = m11;
            this.m20 = m20.v0;
            this.m21 = m20.v1;
            this.m22 = m21;
        }
        
        public bmat3(bool m00, bvec2 m01, bool m10, bvec2 m11, bool m20, bvec2 m21) {
            this.m00 = m00;
            this.m01 = m01.v0;
            this.m02 = m01.v1;
            this.m10 = m10;
            this.m11 = m11.v0;
            this.m12 = m11.v1;
            this.m20 = m20;
            this.m21 = m21.v0;
            this.m22 = m21.v1;
        }
        
        public bmat3(bool m00, bool m01, bool m02, bool m10, bool m11, bool m12, bool m20, bool m21, bool m22) {
            this.m00 = m00;
            this.m01 = m01;
            this.m02 = m02;
            this.m10 = m10;
            this.m11 = m11;
            this.m12 = m12;
            this.m20 = m20;
            this.m21 = m21;
            this.m22 = m22;
        }
        
        public bmat3(bmat2 s) {
            this.m00 = s.m00;
            this.m01 = s.m01;
            this.m02 = false;
            this.m10 = s.m10;
            this.m11 = s.m11;
            this.m12 = false;
            this.m20 = false;
            this.m21 = false;
            this.m22 = false;
        }
        
        public bmat3(bmat2x3 s) {
            this.m00 = s.m00;
            this.m01 = s.m01;
            this.m02 = s.m02;
            this.m10 = s.m10;
            this.m11 = s.m11;
            this.m12 = s.m12;
            this.m20 = false;
            this.m21 = false;
            this.m22 = false;
        }
        
        public bmat3(bmat2x4 s) {
            this.m00 = s.m00;
            this.m01 = s.m01;
            this.m02 = s.m02;
            this.m10 = s.m10;
            this.m11 = s.m11;
            this.m12 = s.m12;
            this.m20 = false;
            this.m21 = false;
            this.m22 = false;
        }
        
        public bmat3(bmat3x2 s) {
            this.m00 = s.m00;
            this.m01 = s.m01;
            this.m02 = false;
            this.m10 = s.m10;
            this.m11 = s.m11;
            this.m12 = false;
            this.m20 = s.m20;
            this.m21 = s.m21;
            this.m22 = false;
        }
        
        public bmat3(bmat3x4 s) {
            this.m00 = s.m00;
            this.m01 = s.m01;
            this.m02 = s.m02;
            this.m10 = s.m10;
            this.m11 = s.m11;
            this.m12 = s.m12;
            this.m20 = s.m20;
            this.m21 = s.m21;
            this.m22 = s.m22;
        }
        
        public bmat3(bmat4x2 s) {
            this.m00 = s.m00;
            this.m01 = s.m01;
            this.m02 = false;
            this.m10 = s.m10;
            this.m11 = s.m11;
            this.m12 = false;
            this.m20 = s.m20;
            this.m21 = s.m21;
            this.m22 = false;
        }
        
        public bmat3(bmat4x3 s) {
            this.m00 = s.m00;
            this.m01 = s.m01;
            this.m02 = s.m02;
            this.m10 = s.m10;
            this.m11 = s.m11;
            this.m12 = s.m12;
            this.m20 = s.m20;
            this.m21 = s.m21;
            this.m22 = s.m22;
        }
        
        public bmat3(bmat4 s) {
            this.m00 = s.m00;
            this.m01 = s.m01;
            this.m02 = s.m02;
            this.m10 = s.m10;
            this.m11 = s.m11;
            this.m12 = s.m12;
            this.m20 = s.m20;
            this.m21 = s.m21;
            this.m22 = s.m22;
        }
        
        public bool this[int i0, int i1] {
            get {
                if (((i0 < 0) 
                            || (2 < i0))) {
                    throw new System.IndexOutOfRangeException("i0");
                }
                if (((i1 < 0) 
                            || (2 < i1))) {
                    throw new System.IndexOutOfRangeException("i1");
                }
                
                unsafe
                {
                    fixed (Boolean* v_ptr = &this.m00)
                    {
                        return *(v_ptr + (i1 + i0 * 3));
                    }
                };
            }
            set {
                if (((i0 < 0) 
                            || (2 < i0))) {
                    throw new System.IndexOutOfRangeException("i0");
                }
                if (((i1 < 0) 
                            || (2 < i1))) {
                    throw new System.IndexOutOfRangeException("i1");
                }
                
                unsafe
                {
                    fixed (Boolean* v_ptr = &this.m00)
                    {
                        *(v_ptr  + (i1 + i0 * 3)) = value;
                    }
                };
            }
        }
        
        public bvec3 this[int i0] {
            get {
                return new bvec3(this[i0, 0], this[i0, 1], this[i0, 2]);
            }
            set {
                this[i0, 0] = value[0];
                this[i0, 1] = value[1];
                this[i0, 2] = value[2];
            }
        }
        
        public static bmat3 operator!(bmat3 v) {
            return new bmat3(!v.m00, !v.m01, !v.m02, !v.m10, !v.m11, !v.m12, !v.m20, !v.m21, !v.m22);
        }
        
        public static bmat3 operator&(bool s, bmat3 v2) {
            return new bmat3(s & v2.m00, s & v2.m01, s & v2.m02, s & v2.m10, s & v2.m11, s & v2.m12, s & v2.m20, s & v2.m21, s & v2.m22);
        }
        
        public static bmat3 operator|(bool s, bmat3 v2) {
            return new bmat3(s | v2.m00, s | v2.m01, s | v2.m02, s | v2.m10, s | v2.m11, s | v2.m12, s | v2.m20, s | v2.m21, s | v2.m22);
        }
        
        public static bmat3 operator^(bool s, bmat3 v2) {
            return new bmat3(s ^ v2.m00, s ^ v2.m01, s ^ v2.m02, s ^ v2.m10, s ^ v2.m11, s ^ v2.m12, s ^ v2.m20, s ^ v2.m21, s ^ v2.m22);
        }
        
        public static bmat3 operator&(bmat3 v1, bool s) {
            return new bmat3(v1.m00 & s, v1.m01 & s, v1.m02 & s, v1.m10 & s, v1.m11 & s, v1.m12 & s, v1.m20 & s, v1.m21 & s, v1.m22 & s);
        }
        
        public static bmat3 operator|(bmat3 v1, bool s) {
            return new bmat3(v1.m00 | s, v1.m01 | s, v1.m02 | s, v1.m10 | s, v1.m11 | s, v1.m12 | s, v1.m20 | s, v1.m21 | s, v1.m22 | s);
        }
        
        public static bmat3 operator^(bmat3 v1, bool s) {
            return new bmat3(v1.m00 ^ s, v1.m01 ^ s, v1.m02 ^ s, v1.m10 ^ s, v1.m11 ^ s, v1.m12 ^ s, v1.m20 ^ s, v1.m21 ^ s, v1.m22 ^ s);
        }
        
        public static bmat3 operator&(bmat3 v1, bmat3 v2) {
            return new bmat3(v1.m00 & v2.m00, v1.m01 & v2.m01, v1.m02 & v2.m02, v1.m10 & v2.m10, v1.m11 & v2.m11, v1.m12 & v2.m12, v1.m20 & v2.m20, v1.m21 & v2.m21, v1.m22 & v2.m22);
        }
        
        public static bmat3 operator|(bmat3 v1, bmat3 v2) {
            return new bmat3(v1.m00 | v2.m00, v1.m01 | v2.m01, v1.m02 | v2.m02, v1.m10 | v2.m10, v1.m11 | v2.m11, v1.m12 | v2.m12, v1.m20 | v2.m20, v1.m21 | v2.m21, v1.m22 | v2.m22);
        }
        
        public static bmat3 operator^(bmat3 v1, bmat3 v2) {
            return new bmat3(v1.m00 ^ v2.m00, v1.m01 ^ v2.m01, v1.m02 ^ v2.m02, v1.m10 ^ v2.m10, v1.m11 ^ v2.m11, v1.m12 ^ v2.m12, v1.m20 ^ v2.m20, v1.m21 ^ v2.m21, v1.m22 ^ v2.m22);
        }
        
        public static bmat3 operator==(bmat3 v1, bmat3 v2) {
            return new bmat3(v1.m00 == v2.m00, v1.m01 == v2.m01, v1.m02 == v2.m02, v1.m10 == v2.m10, v1.m11 == v2.m11, v1.m12 == v2.m12, v1.m20 == v2.m20, v1.m21 == v2.m21, v1.m22 == v2.m22);
        }
        
        public static bmat3 operator!=(bmat3 v1, bmat3 v2) {
            return new bmat3(v1.m00 != v2.m00, v1.m01 != v2.m01, v1.m02 != v2.m02, v1.m10 != v2.m10, v1.m11 != v2.m11, v1.m12 != v2.m12, v1.m20 != v2.m20, v1.m21 != v2.m21, v1.m22 != v2.m22);
        }
        
        public override bool Equals(object obj) {
            return (obj is bmat3) ? bmath.All(this == (bmat3)obj) : false;
        }
        
        public override int GetHashCode() {
            return this.m00.GetHashCode() ^ this.m01.GetHashCode() ^ this.m02.GetHashCode() ^ this.m10.GetHashCode() ^ this.m11.GetHashCode() ^ this.m12.GetHashCode() ^ this.m20.GetHashCode() ^ this.m21.GetHashCode() ^ this.m22.GetHashCode();
        }
        
        public override string ToString() {
            return "bmat3(" + this.m00 + "," + this.m01 + "," + this.m02 + "," + this.m10 + "," + this.m11 + "," + this.m12 + "," + this.m20 + "," + this.m21 + "," + this.m22 + ")";
        }
    }
    
    [DataContract()]
    [StructLayout(LayoutKind.Sequential)]
    public struct bmat3x4 {
        
        [DataMember()]
        internal bool m00;
        
        [DataMember()]
        internal bool m01;
        
        [DataMember()]
        internal bool m02;
        
        [DataMember()]
        internal bool m03;
        
        [DataMember()]
        internal bool m10;
        
        [DataMember()]
        internal bool m11;
        
        [DataMember()]
        internal bool m12;
        
        [DataMember()]
        internal bool m13;
        
        [DataMember()]
        internal bool m20;
        
        [DataMember()]
        internal bool m21;
        
        [DataMember()]
        internal bool m22;
        
        [DataMember()]
        internal bool m23;
        
        public bmat3x4(bool s) {
            this.m00 = s;
            this.m01 = false;
            this.m02 = false;
            this.m03 = false;
            this.m10 = false;
            this.m11 = s;
            this.m12 = false;
            this.m13 = false;
            this.m20 = false;
            this.m21 = false;
            this.m22 = s;
            this.m23 = false;
        }
        
        public bmat3x4(bmat3x4 m00) {
            this.m00 = m00.m00;
            this.m01 = m00.m01;
            this.m02 = m00.m02;
            this.m03 = m00.m03;
            this.m10 = m00.m10;
            this.m11 = m00.m11;
            this.m12 = m00.m12;
            this.m13 = m00.m13;
            this.m20 = m00.m20;
            this.m21 = m00.m21;
            this.m22 = m00.m22;
            this.m23 = m00.m23;
        }
        
        public bmat3x4(bmat3x2 m00, bmat3x2 m01) {
            this.m00 = m00.m00;
            this.m01 = m00.m01;
            this.m02 = m01.m00;
            this.m03 = m01.m01;
            this.m10 = m00.m10;
            this.m11 = m00.m11;
            this.m12 = m01.m10;
            this.m13 = m01.m11;
            this.m20 = m00.m20;
            this.m21 = m00.m21;
            this.m22 = m01.m20;
            this.m23 = m01.m21;
        }
        
        public bmat3x4(bmat2x4 m00, bvec4 m10) {
            this.m00 = m00.m00;
            this.m01 = m00.m01;
            this.m02 = m00.m02;
            this.m03 = m00.m03;
            this.m10 = m00.m10;
            this.m11 = m00.m11;
            this.m12 = m00.m12;
            this.m13 = m00.m13;
            this.m20 = m10.v0;
            this.m21 = m10.v1;
            this.m22 = m10.v2;
            this.m23 = m10.v3;
        }
        
        public bmat3x4(bmat2 m00, bmat2 m01, bvec2 m10, bvec2 m11) {
            this.m00 = m00.m00;
            this.m01 = m00.m01;
            this.m02 = m01.m00;
            this.m03 = m01.m01;
            this.m10 = m00.m10;
            this.m11 = m00.m11;
            this.m12 = m01.m10;
            this.m13 = m01.m11;
            this.m20 = m10.v0;
            this.m21 = m10.v1;
            this.m22 = m11.v0;
            this.m23 = m11.v1;
        }
        
        public bmat3x4(bvec4 m00, bmat2x4 m10) {
            this.m00 = m00.v0;
            this.m01 = m00.v1;
            this.m02 = m00.v2;
            this.m03 = m00.v3;
            this.m10 = m10.m00;
            this.m11 = m10.m01;
            this.m12 = m10.m02;
            this.m13 = m10.m03;
            this.m20 = m10.m10;
            this.m21 = m10.m11;
            this.m22 = m10.m12;
            this.m23 = m10.m13;
        }
        
        public bmat3x4(bvec2 m00, bvec2 m01, bmat2 m10, bmat2 m11) {
            this.m00 = m00.v0;
            this.m01 = m00.v1;
            this.m02 = m01.v0;
            this.m03 = m01.v1;
            this.m10 = m10.m00;
            this.m11 = m10.m01;
            this.m12 = m11.m00;
            this.m13 = m11.m01;
            this.m20 = m10.m10;
            this.m21 = m10.m11;
            this.m22 = m11.m10;
            this.m23 = m11.m11;
        }
        
        public bmat3x4(bvec4 m00, bvec4 m10, bvec4 m20) {
            this.m00 = m00.v0;
            this.m01 = m00.v1;
            this.m02 = m00.v2;
            this.m03 = m00.v3;
            this.m10 = m10.v0;
            this.m11 = m10.v1;
            this.m12 = m10.v2;
            this.m13 = m10.v3;
            this.m20 = m20.v0;
            this.m21 = m20.v1;
            this.m22 = m20.v2;
            this.m23 = m20.v3;
        }
        
        public bmat3x4(bvec2 m00, bvec2 m01, bvec2 m10, bvec2 m11, bvec2 m20, bvec2 m21) {
            this.m00 = m00.v0;
            this.m01 = m00.v1;
            this.m02 = m01.v0;
            this.m03 = m01.v1;
            this.m10 = m10.v0;
            this.m11 = m10.v1;
            this.m12 = m11.v0;
            this.m13 = m11.v1;
            this.m20 = m20.v0;
            this.m21 = m20.v1;
            this.m22 = m21.v0;
            this.m23 = m21.v1;
        }
        
        public bmat3x4(bvec3 m00, bool m01, bvec3 m10, bool m11, bvec3 m20, bool m21) {
            this.m00 = m00.v0;
            this.m01 = m00.v1;
            this.m02 = m00.v2;
            this.m03 = m01;
            this.m10 = m10.v0;
            this.m11 = m10.v1;
            this.m12 = m10.v2;
            this.m13 = m11;
            this.m20 = m20.v0;
            this.m21 = m20.v1;
            this.m22 = m20.v2;
            this.m23 = m21;
        }
        
        public bmat3x4(bool m00, bvec3 m01, bool m10, bvec3 m11, bool m20, bvec3 m21) {
            this.m00 = m00;
            this.m01 = m01.v0;
            this.m02 = m01.v1;
            this.m03 = m01.v2;
            this.m10 = m10;
            this.m11 = m11.v0;
            this.m12 = m11.v1;
            this.m13 = m11.v2;
            this.m20 = m20;
            this.m21 = m21.v0;
            this.m22 = m21.v1;
            this.m23 = m21.v2;
        }
        
        public bmat3x4(bvec2 m00, bool m01, bool m02, bvec2 m10, bool m11, bool m12, bvec2 m20, bool m21, bool m22) {
            this.m00 = m00.v0;
            this.m01 = m00.v1;
            this.m02 = m01;
            this.m03 = m02;
            this.m10 = m10.v0;
            this.m11 = m10.v1;
            this.m12 = m11;
            this.m13 = m12;
            this.m20 = m20.v0;
            this.m21 = m20.v1;
            this.m22 = m21;
            this.m23 = m22;
        }
        
        public bmat3x4(bool m00, bvec2 m01, bool m02, bool m10, bvec2 m11, bool m12, bool m20, bvec2 m21, bool m22) {
            this.m00 = m00;
            this.m01 = m01.v0;
            this.m02 = m01.v1;
            this.m03 = m02;
            this.m10 = m10;
            this.m11 = m11.v0;
            this.m12 = m11.v1;
            this.m13 = m12;
            this.m20 = m20;
            this.m21 = m21.v0;
            this.m22 = m21.v1;
            this.m23 = m22;
        }
        
        public bmat3x4(bool m00, bool m01, bvec2 m02, bool m10, bool m11, bvec2 m12, bool m20, bool m21, bvec2 m22) {
            this.m00 = m00;
            this.m01 = m01;
            this.m02 = m02.v0;
            this.m03 = m02.v1;
            this.m10 = m10;
            this.m11 = m11;
            this.m12 = m12.v0;
            this.m13 = m12.v1;
            this.m20 = m20;
            this.m21 = m21;
            this.m22 = m22.v0;
            this.m23 = m22.v1;
        }
        
        public bmat3x4(bool m00, bool m01, bool m02, bool m03, bool m10, bool m11, bool m12, bool m13, bool m20, bool m21, bool m22, bool m23) {
            this.m00 = m00;
            this.m01 = m01;
            this.m02 = m02;
            this.m03 = m03;
            this.m10 = m10;
            this.m11 = m11;
            this.m12 = m12;
            this.m13 = m13;
            this.m20 = m20;
            this.m21 = m21;
            this.m22 = m22;
            this.m23 = m23;
        }
        
        public bmat3x4(bmat2 s) {
            this.m00 = s.m00;
            this.m01 = s.m01;
            this.m02 = false;
            this.m03 = false;
            this.m10 = s.m10;
            this.m11 = s.m11;
            this.m12 = false;
            this.m13 = false;
            this.m20 = false;
            this.m21 = false;
            this.m22 = false;
            this.m23 = false;
        }
        
        public bmat3x4(bmat2x3 s) {
            this.m00 = s.m00;
            this.m01 = s.m01;
            this.m02 = s.m02;
            this.m03 = false;
            this.m10 = s.m10;
            this.m11 = s.m11;
            this.m12 = s.m12;
            this.m13 = false;
            this.m20 = false;
            this.m21 = false;
            this.m22 = false;
            this.m23 = false;
        }
        
        public bmat3x4(bmat2x4 s) {
            this.m00 = s.m00;
            this.m01 = s.m01;
            this.m02 = s.m02;
            this.m03 = s.m03;
            this.m10 = s.m10;
            this.m11 = s.m11;
            this.m12 = s.m12;
            this.m13 = s.m13;
            this.m20 = false;
            this.m21 = false;
            this.m22 = false;
            this.m23 = false;
        }
        
        public bmat3x4(bmat3x2 s) {
            this.m00 = s.m00;
            this.m01 = s.m01;
            this.m02 = false;
            this.m03 = false;
            this.m10 = s.m10;
            this.m11 = s.m11;
            this.m12 = false;
            this.m13 = false;
            this.m20 = s.m20;
            this.m21 = s.m21;
            this.m22 = false;
            this.m23 = false;
        }
        
        public bmat3x4(bmat3 s) {
            this.m00 = s.m00;
            this.m01 = s.m01;
            this.m02 = s.m02;
            this.m03 = false;
            this.m10 = s.m10;
            this.m11 = s.m11;
            this.m12 = s.m12;
            this.m13 = false;
            this.m20 = s.m20;
            this.m21 = s.m21;
            this.m22 = s.m22;
            this.m23 = false;
        }
        
        public bmat3x4(bmat4x2 s) {
            this.m00 = s.m00;
            this.m01 = s.m01;
            this.m02 = false;
            this.m03 = false;
            this.m10 = s.m10;
            this.m11 = s.m11;
            this.m12 = false;
            this.m13 = false;
            this.m20 = s.m20;
            this.m21 = s.m21;
            this.m22 = false;
            this.m23 = false;
        }
        
        public bmat3x4(bmat4x3 s) {
            this.m00 = s.m00;
            this.m01 = s.m01;
            this.m02 = s.m02;
            this.m03 = false;
            this.m10 = s.m10;
            this.m11 = s.m11;
            this.m12 = s.m12;
            this.m13 = false;
            this.m20 = s.m20;
            this.m21 = s.m21;
            this.m22 = s.m22;
            this.m23 = false;
        }
        
        public bmat3x4(bmat4 s) {
            this.m00 = s.m00;
            this.m01 = s.m01;
            this.m02 = s.m02;
            this.m03 = s.m03;
            this.m10 = s.m10;
            this.m11 = s.m11;
            this.m12 = s.m12;
            this.m13 = s.m13;
            this.m20 = s.m20;
            this.m21 = s.m21;
            this.m22 = s.m22;
            this.m23 = s.m23;
        }
        
        public bool this[int i0, int i1] {
            get {
                if (((i0 < 0) 
                            || (2 < i0))) {
                    throw new System.IndexOutOfRangeException("i0");
                }
                if (((i1 < 0) 
                            || (3 < i1))) {
                    throw new System.IndexOutOfRangeException("i1");
                }
                
                unsafe
                {
                    fixed (Boolean* v_ptr = &this.m00)
                    {
                        return *(v_ptr + (i1 + i0 * 4));
                    }
                };
            }
            set {
                if (((i0 < 0) 
                            || (2 < i0))) {
                    throw new System.IndexOutOfRangeException("i0");
                }
                if (((i1 < 0) 
                            || (3 < i1))) {
                    throw new System.IndexOutOfRangeException("i1");
                }
                
                unsafe
                {
                    fixed (Boolean* v_ptr = &this.m00)
                    {
                        *(v_ptr  + (i1 + i0 * 4)) = value;
                    }
                };
            }
        }
        
        public bvec4 this[int i0] {
            get {
                return new bvec4(this[i0, 0], this[i0, 1], this[i0, 2], this[i0, 3]);
            }
            set {
                this[i0, 0] = value[0];
                this[i0, 1] = value[1];
                this[i0, 2] = value[2];
                this[i0, 3] = value[3];
            }
        }
        
        public static bmat3x4 operator!(bmat3x4 v) {
            return new bmat3x4(!v.m00, !v.m01, !v.m02, !v.m03, !v.m10, !v.m11, !v.m12, !v.m13, !v.m20, !v.m21, !v.m22, !v.m23);
        }
        
        public static bmat3x4 operator&(bool s, bmat3x4 v2) {
            return new bmat3x4(s & v2.m00, s & v2.m01, s & v2.m02, s & v2.m03, s & v2.m10, s & v2.m11, s & v2.m12, s & v2.m13, s & v2.m20, s & v2.m21, s & v2.m22, s & v2.m23);
        }
        
        public static bmat3x4 operator|(bool s, bmat3x4 v2) {
            return new bmat3x4(s | v2.m00, s | v2.m01, s | v2.m02, s | v2.m03, s | v2.m10, s | v2.m11, s | v2.m12, s | v2.m13, s | v2.m20, s | v2.m21, s | v2.m22, s | v2.m23);
        }
        
        public static bmat3x4 operator^(bool s, bmat3x4 v2) {
            return new bmat3x4(s ^ v2.m00, s ^ v2.m01, s ^ v2.m02, s ^ v2.m03, s ^ v2.m10, s ^ v2.m11, s ^ v2.m12, s ^ v2.m13, s ^ v2.m20, s ^ v2.m21, s ^ v2.m22, s ^ v2.m23);
        }
        
        public static bmat3x4 operator&(bmat3x4 v1, bool s) {
            return new bmat3x4(v1.m00 & s, v1.m01 & s, v1.m02 & s, v1.m03 & s, v1.m10 & s, v1.m11 & s, v1.m12 & s, v1.m13 & s, v1.m20 & s, v1.m21 & s, v1.m22 & s, v1.m23 & s);
        }
        
        public static bmat3x4 operator|(bmat3x4 v1, bool s) {
            return new bmat3x4(v1.m00 | s, v1.m01 | s, v1.m02 | s, v1.m03 | s, v1.m10 | s, v1.m11 | s, v1.m12 | s, v1.m13 | s, v1.m20 | s, v1.m21 | s, v1.m22 | s, v1.m23 | s);
        }
        
        public static bmat3x4 operator^(bmat3x4 v1, bool s) {
            return new bmat3x4(v1.m00 ^ s, v1.m01 ^ s, v1.m02 ^ s, v1.m03 ^ s, v1.m10 ^ s, v1.m11 ^ s, v1.m12 ^ s, v1.m13 ^ s, v1.m20 ^ s, v1.m21 ^ s, v1.m22 ^ s, v1.m23 ^ s);
        }
        
        public static bmat3x4 operator&(bmat3x4 v1, bmat3x4 v2) {
            return new bmat3x4(v1.m00 & v2.m00, v1.m01 & v2.m01, v1.m02 & v2.m02, v1.m03 & v2.m03, v1.m10 & v2.m10, v1.m11 & v2.m11, v1.m12 & v2.m12, v1.m13 & v2.m13, v1.m20 & v2.m20, v1.m21 & v2.m21, v1.m22 & v2.m22, v1.m23 & v2.m23);
        }
        
        public static bmat3x4 operator|(bmat3x4 v1, bmat3x4 v2) {
            return new bmat3x4(v1.m00 | v2.m00, v1.m01 | v2.m01, v1.m02 | v2.m02, v1.m03 | v2.m03, v1.m10 | v2.m10, v1.m11 | v2.m11, v1.m12 | v2.m12, v1.m13 | v2.m13, v1.m20 | v2.m20, v1.m21 | v2.m21, v1.m22 | v2.m22, v1.m23 | v2.m23);
        }
        
        public static bmat3x4 operator^(bmat3x4 v1, bmat3x4 v2) {
            return new bmat3x4(v1.m00 ^ v2.m00, v1.m01 ^ v2.m01, v1.m02 ^ v2.m02, v1.m03 ^ v2.m03, v1.m10 ^ v2.m10, v1.m11 ^ v2.m11, v1.m12 ^ v2.m12, v1.m13 ^ v2.m13, v1.m20 ^ v2.m20, v1.m21 ^ v2.m21, v1.m22 ^ v2.m22, v1.m23 ^ v2.m23);
        }
        
        public static bmat3x4 operator==(bmat3x4 v1, bmat3x4 v2) {
            return new bmat3x4(v1.m00 == v2.m00, v1.m01 == v2.m01, v1.m02 == v2.m02, v1.m03 == v2.m03, v1.m10 == v2.m10, v1.m11 == v2.m11, v1.m12 == v2.m12, v1.m13 == v2.m13, v1.m20 == v2.m20, v1.m21 == v2.m21, v1.m22 == v2.m22, v1.m23 == v2.m23);
        }
        
        public static bmat3x4 operator!=(bmat3x4 v1, bmat3x4 v2) {
            return new bmat3x4(v1.m00 != v2.m00, v1.m01 != v2.m01, v1.m02 != v2.m02, v1.m03 != v2.m03, v1.m10 != v2.m10, v1.m11 != v2.m11, v1.m12 != v2.m12, v1.m13 != v2.m13, v1.m20 != v2.m20, v1.m21 != v2.m21, v1.m22 != v2.m22, v1.m23 != v2.m23);
        }
        
        public override bool Equals(object obj) {
            return (obj is bmat3x4) ? bmath.All(this == (bmat3x4)obj) : false;
        }
        
        public override int GetHashCode() {
            return this.m00.GetHashCode() ^ this.m01.GetHashCode() ^ this.m02.GetHashCode() ^ this.m03.GetHashCode() ^ this.m10.GetHashCode() ^ this.m11.GetHashCode() ^ this.m12.GetHashCode() ^ this.m13.GetHashCode() ^ this.m20.GetHashCode() ^ this.m21.GetHashCode() ^ this.m22.GetHashCode() ^ this.m23.GetHashCode();
        }
        
        public override string ToString() {
            return "bmat3x4(" + this.m00 + "," + this.m01 + "," + this.m02 + "," + this.m03 + "," + this.m10 + "," + this.m11 + "," + this.m12 + "," + this.m13 + "," + this.m20 + "," + this.m21 + "," + this.m22 + "," + this.m23 + ")";
        }
    }
    
    [DataContract()]
    [StructLayout(LayoutKind.Sequential)]
    public struct bmat4x2 {
        
        [DataMember()]
        internal bool m00;
        
        [DataMember()]
        internal bool m01;
        
        [DataMember()]
        internal bool m10;
        
        [DataMember()]
        internal bool m11;
        
        [DataMember()]
        internal bool m20;
        
        [DataMember()]
        internal bool m21;
        
        [DataMember()]
        internal bool m30;
        
        [DataMember()]
        internal bool m31;
        
        public bmat4x2(bool s) {
            this.m00 = s;
            this.m01 = false;
            this.m10 = false;
            this.m11 = s;
            this.m20 = false;
            this.m21 = false;
            this.m30 = false;
            this.m31 = false;
        }
        
        public bmat4x2(bmat4x2 m00) {
            this.m00 = m00.m00;
            this.m01 = m00.m01;
            this.m10 = m00.m10;
            this.m11 = m00.m11;
            this.m20 = m00.m20;
            this.m21 = m00.m21;
            this.m30 = m00.m30;
            this.m31 = m00.m31;
        }
        
        public bmat4x2(bmat2 m00, bmat2 m10) {
            this.m00 = m00.m00;
            this.m01 = m00.m01;
            this.m10 = m00.m10;
            this.m11 = m00.m11;
            this.m20 = m10.m00;
            this.m21 = m10.m01;
            this.m30 = m10.m10;
            this.m31 = m10.m11;
        }
        
        public bmat4x2(bmat3x2 m00, bvec2 m10) {
            this.m00 = m00.m00;
            this.m01 = m00.m01;
            this.m10 = m00.m10;
            this.m11 = m00.m11;
            this.m20 = m00.m20;
            this.m21 = m00.m21;
            this.m30 = m10.v0;
            this.m31 = m10.v1;
        }
        
        public bmat4x2(bvec2 m00, bmat3x2 m10) {
            this.m00 = m00.v0;
            this.m01 = m00.v1;
            this.m10 = m10.m00;
            this.m11 = m10.m01;
            this.m20 = m10.m10;
            this.m21 = m10.m11;
            this.m30 = m10.m20;
            this.m31 = m10.m21;
        }
        
        public bmat4x2(bmat2 m00, bvec2 m10, bvec2 m20) {
            this.m00 = m00.m00;
            this.m01 = m00.m01;
            this.m10 = m00.m10;
            this.m11 = m00.m11;
            this.m20 = m10.v0;
            this.m21 = m10.v1;
            this.m30 = m20.v0;
            this.m31 = m20.v1;
        }
        
        public bmat4x2(bvec2 m00, bmat2 m10, bvec2 m20) {
            this.m00 = m00.v0;
            this.m01 = m00.v1;
            this.m10 = m10.m00;
            this.m11 = m10.m01;
            this.m20 = m10.m10;
            this.m21 = m10.m11;
            this.m30 = m20.v0;
            this.m31 = m20.v1;
        }
        
        public bmat4x2(bvec2 m00, bvec2 m10, bmat2 m20) {
            this.m00 = m00.v0;
            this.m01 = m00.v1;
            this.m10 = m10.v0;
            this.m11 = m10.v1;
            this.m20 = m20.m00;
            this.m21 = m20.m01;
            this.m30 = m20.m10;
            this.m31 = m20.m11;
        }
        
        public bmat4x2(bvec2 m00, bvec2 m10, bvec2 m20, bvec2 m30) {
            this.m00 = m00.v0;
            this.m01 = m00.v1;
            this.m10 = m10.v0;
            this.m11 = m10.v1;
            this.m20 = m20.v0;
            this.m21 = m20.v1;
            this.m30 = m30.v0;
            this.m31 = m30.v1;
        }
        
        public bmat4x2(bool m00, bool m01, bool m10, bool m11, bool m20, bool m21, bool m30, bool m31) {
            this.m00 = m00;
            this.m01 = m01;
            this.m10 = m10;
            this.m11 = m11;
            this.m20 = m20;
            this.m21 = m21;
            this.m30 = m30;
            this.m31 = m31;
        }
        
        public bmat4x2(bmat2 s) {
            this.m00 = s.m00;
            this.m01 = s.m01;
            this.m10 = s.m10;
            this.m11 = s.m11;
            this.m20 = false;
            this.m21 = false;
            this.m30 = false;
            this.m31 = false;
        }
        
        public bmat4x2(bmat2x3 s) {
            this.m00 = s.m00;
            this.m01 = s.m01;
            this.m10 = s.m10;
            this.m11 = s.m11;
            this.m20 = false;
            this.m21 = false;
            this.m30 = false;
            this.m31 = false;
        }
        
        public bmat4x2(bmat2x4 s) {
            this.m00 = s.m00;
            this.m01 = s.m01;
            this.m10 = s.m10;
            this.m11 = s.m11;
            this.m20 = false;
            this.m21 = false;
            this.m30 = false;
            this.m31 = false;
        }
        
        public bmat4x2(bmat3x2 s) {
            this.m00 = s.m00;
            this.m01 = s.m01;
            this.m10 = s.m10;
            this.m11 = s.m11;
            this.m20 = s.m20;
            this.m21 = s.m21;
            this.m30 = false;
            this.m31 = false;
        }
        
        public bmat4x2(bmat3 s) {
            this.m00 = s.m00;
            this.m01 = s.m01;
            this.m10 = s.m10;
            this.m11 = s.m11;
            this.m20 = s.m20;
            this.m21 = s.m21;
            this.m30 = false;
            this.m31 = false;
        }
        
        public bmat4x2(bmat3x4 s) {
            this.m00 = s.m00;
            this.m01 = s.m01;
            this.m10 = s.m10;
            this.m11 = s.m11;
            this.m20 = s.m20;
            this.m21 = s.m21;
            this.m30 = false;
            this.m31 = false;
        }
        
        public bmat4x2(bmat4x3 s) {
            this.m00 = s.m00;
            this.m01 = s.m01;
            this.m10 = s.m10;
            this.m11 = s.m11;
            this.m20 = s.m20;
            this.m21 = s.m21;
            this.m30 = s.m30;
            this.m31 = s.m31;
        }
        
        public bmat4x2(bmat4 s) {
            this.m00 = s.m00;
            this.m01 = s.m01;
            this.m10 = s.m10;
            this.m11 = s.m11;
            this.m20 = s.m20;
            this.m21 = s.m21;
            this.m30 = s.m30;
            this.m31 = s.m31;
        }
        
        public bool this[int i0, int i1] {
            get {
                if (((i0 < 0) 
                            || (3 < i0))) {
                    throw new System.IndexOutOfRangeException("i0");
                }
                if (((i1 < 0) 
                            || (1 < i1))) {
                    throw new System.IndexOutOfRangeException("i1");
                }
                
                unsafe
                {
                    fixed (Boolean* v_ptr = &this.m00)
                    {
                        return *(v_ptr + (i1 + i0 * 2));
                    }
                };
            }
            set {
                if (((i0 < 0) 
                            || (3 < i0))) {
                    throw new System.IndexOutOfRangeException("i0");
                }
                if (((i1 < 0) 
                            || (1 < i1))) {
                    throw new System.IndexOutOfRangeException("i1");
                }
                
                unsafe
                {
                    fixed (Boolean* v_ptr = &this.m00)
                    {
                        *(v_ptr  + (i1 + i0 * 2)) = value;
                    }
                };
            }
        }
        
        public bvec2 this[int i0] {
            get {
                return new bvec2(this[i0, 0], this[i0, 1]);
            }
            set {
                this[i0, 0] = value[0];
                this[i0, 1] = value[1];
            }
        }
        
        public static bmat4x2 operator!(bmat4x2 v) {
            return new bmat4x2(!v.m00, !v.m01, !v.m10, !v.m11, !v.m20, !v.m21, !v.m30, !v.m31);
        }
        
        public static bmat4x2 operator&(bool s, bmat4x2 v2) {
            return new bmat4x2(s & v2.m00, s & v2.m01, s & v2.m10, s & v2.m11, s & v2.m20, s & v2.m21, s & v2.m30, s & v2.m31);
        }
        
        public static bmat4x2 operator|(bool s, bmat4x2 v2) {
            return new bmat4x2(s | v2.m00, s | v2.m01, s | v2.m10, s | v2.m11, s | v2.m20, s | v2.m21, s | v2.m30, s | v2.m31);
        }
        
        public static bmat4x2 operator^(bool s, bmat4x2 v2) {
            return new bmat4x2(s ^ v2.m00, s ^ v2.m01, s ^ v2.m10, s ^ v2.m11, s ^ v2.m20, s ^ v2.m21, s ^ v2.m30, s ^ v2.m31);
        }
        
        public static bmat4x2 operator&(bmat4x2 v1, bool s) {
            return new bmat4x2(v1.m00 & s, v1.m01 & s, v1.m10 & s, v1.m11 & s, v1.m20 & s, v1.m21 & s, v1.m30 & s, v1.m31 & s);
        }
        
        public static bmat4x2 operator|(bmat4x2 v1, bool s) {
            return new bmat4x2(v1.m00 | s, v1.m01 | s, v1.m10 | s, v1.m11 | s, v1.m20 | s, v1.m21 | s, v1.m30 | s, v1.m31 | s);
        }
        
        public static bmat4x2 operator^(bmat4x2 v1, bool s) {
            return new bmat4x2(v1.m00 ^ s, v1.m01 ^ s, v1.m10 ^ s, v1.m11 ^ s, v1.m20 ^ s, v1.m21 ^ s, v1.m30 ^ s, v1.m31 ^ s);
        }
        
        public static bmat4x2 operator&(bmat4x2 v1, bmat4x2 v2) {
            return new bmat4x2(v1.m00 & v2.m00, v1.m01 & v2.m01, v1.m10 & v2.m10, v1.m11 & v2.m11, v1.m20 & v2.m20, v1.m21 & v2.m21, v1.m30 & v2.m30, v1.m31 & v2.m31);
        }
        
        public static bmat4x2 operator|(bmat4x2 v1, bmat4x2 v2) {
            return new bmat4x2(v1.m00 | v2.m00, v1.m01 | v2.m01, v1.m10 | v2.m10, v1.m11 | v2.m11, v1.m20 | v2.m20, v1.m21 | v2.m21, v1.m30 | v2.m30, v1.m31 | v2.m31);
        }
        
        public static bmat4x2 operator^(bmat4x2 v1, bmat4x2 v2) {
            return new bmat4x2(v1.m00 ^ v2.m00, v1.m01 ^ v2.m01, v1.m10 ^ v2.m10, v1.m11 ^ v2.m11, v1.m20 ^ v2.m20, v1.m21 ^ v2.m21, v1.m30 ^ v2.m30, v1.m31 ^ v2.m31);
        }
        
        public static bmat4x2 operator==(bmat4x2 v1, bmat4x2 v2) {
            return new bmat4x2(v1.m00 == v2.m00, v1.m01 == v2.m01, v1.m10 == v2.m10, v1.m11 == v2.m11, v1.m20 == v2.m20, v1.m21 == v2.m21, v1.m30 == v2.m30, v1.m31 == v2.m31);
        }
        
        public static bmat4x2 operator!=(bmat4x2 v1, bmat4x2 v2) {
            return new bmat4x2(v1.m00 != v2.m00, v1.m01 != v2.m01, v1.m10 != v2.m10, v1.m11 != v2.m11, v1.m20 != v2.m20, v1.m21 != v2.m21, v1.m30 != v2.m30, v1.m31 != v2.m31);
        }
        
        public override bool Equals(object obj) {
            return (obj is bmat4x2) ? bmath.All(this == (bmat4x2)obj) : false;
        }
        
        public override int GetHashCode() {
            return this.m00.GetHashCode() ^ this.m01.GetHashCode() ^ this.m10.GetHashCode() ^ this.m11.GetHashCode() ^ this.m20.GetHashCode() ^ this.m21.GetHashCode() ^ this.m30.GetHashCode() ^ this.m31.GetHashCode();
        }
        
        public override string ToString() {
            return "bmat4x2(" + this.m00 + "," + this.m01 + "," + this.m10 + "," + this.m11 + "," + this.m20 + "," + this.m21 + "," + this.m30 + "," + this.m31 + ")";
        }
    }
    
    [DataContract()]
    [StructLayout(LayoutKind.Sequential)]
    public struct bmat4x3 {
        
        [DataMember()]
        internal bool m00;
        
        [DataMember()]
        internal bool m01;
        
        [DataMember()]
        internal bool m02;
        
        [DataMember()]
        internal bool m10;
        
        [DataMember()]
        internal bool m11;
        
        [DataMember()]
        internal bool m12;
        
        [DataMember()]
        internal bool m20;
        
        [DataMember()]
        internal bool m21;
        
        [DataMember()]
        internal bool m22;
        
        [DataMember()]
        internal bool m30;
        
        [DataMember()]
        internal bool m31;
        
        [DataMember()]
        internal bool m32;
        
        public bmat4x3(bool s) {
            this.m00 = s;
            this.m01 = false;
            this.m02 = false;
            this.m10 = false;
            this.m11 = s;
            this.m12 = false;
            this.m20 = false;
            this.m21 = false;
            this.m22 = s;
            this.m30 = false;
            this.m31 = false;
            this.m32 = false;
        }
        
        public bmat4x3(bmat4x3 m00) {
            this.m00 = m00.m00;
            this.m01 = m00.m01;
            this.m02 = m00.m02;
            this.m10 = m00.m10;
            this.m11 = m00.m11;
            this.m12 = m00.m12;
            this.m20 = m00.m20;
            this.m21 = m00.m21;
            this.m22 = m00.m22;
            this.m30 = m00.m30;
            this.m31 = m00.m31;
            this.m32 = m00.m32;
        }
        
        public bmat4x3(bmat2x3 m00, bmat2x3 m10) {
            this.m00 = m00.m00;
            this.m01 = m00.m01;
            this.m02 = m00.m02;
            this.m10 = m00.m10;
            this.m11 = m00.m11;
            this.m12 = m00.m12;
            this.m20 = m10.m00;
            this.m21 = m10.m01;
            this.m22 = m10.m02;
            this.m30 = m10.m10;
            this.m31 = m10.m11;
            this.m32 = m10.m12;
        }
        
        public bmat4x3(bmat3 m00, bvec3 m10) {
            this.m00 = m00.m00;
            this.m01 = m00.m01;
            this.m02 = m00.m02;
            this.m10 = m00.m10;
            this.m11 = m00.m11;
            this.m12 = m00.m12;
            this.m20 = m00.m20;
            this.m21 = m00.m21;
            this.m22 = m00.m22;
            this.m30 = m10.v0;
            this.m31 = m10.v1;
            this.m32 = m10.v2;
        }
        
        public bmat4x3(bvec3 m00, bmat3 m10) {
            this.m00 = m00.v0;
            this.m01 = m00.v1;
            this.m02 = m00.v2;
            this.m10 = m10.m00;
            this.m11 = m10.m01;
            this.m12 = m10.m02;
            this.m20 = m10.m10;
            this.m21 = m10.m11;
            this.m22 = m10.m12;
            this.m30 = m10.m20;
            this.m31 = m10.m21;
            this.m32 = m10.m22;
        }
        
        public bmat4x3(bmat2x3 m00, bvec3 m10, bvec3 m20) {
            this.m00 = m00.m00;
            this.m01 = m00.m01;
            this.m02 = m00.m02;
            this.m10 = m00.m10;
            this.m11 = m00.m11;
            this.m12 = m00.m12;
            this.m20 = m10.v0;
            this.m21 = m10.v1;
            this.m22 = m10.v2;
            this.m30 = m20.v0;
            this.m31 = m20.v1;
            this.m32 = m20.v2;
        }
        
        public bmat4x3(bvec3 m00, bmat2x3 m10, bvec3 m20) {
            this.m00 = m00.v0;
            this.m01 = m00.v1;
            this.m02 = m00.v2;
            this.m10 = m10.m00;
            this.m11 = m10.m01;
            this.m12 = m10.m02;
            this.m20 = m10.m10;
            this.m21 = m10.m11;
            this.m22 = m10.m12;
            this.m30 = m20.v0;
            this.m31 = m20.v1;
            this.m32 = m20.v2;
        }
        
        public bmat4x3(bvec3 m00, bvec3 m10, bmat2x3 m20) {
            this.m00 = m00.v0;
            this.m01 = m00.v1;
            this.m02 = m00.v2;
            this.m10 = m10.v0;
            this.m11 = m10.v1;
            this.m12 = m10.v2;
            this.m20 = m20.m00;
            this.m21 = m20.m01;
            this.m22 = m20.m02;
            this.m30 = m20.m10;
            this.m31 = m20.m11;
            this.m32 = m20.m12;
        }
        
        public bmat4x3(bvec3 m00, bvec3 m10, bvec3 m20, bvec3 m30) {
            this.m00 = m00.v0;
            this.m01 = m00.v1;
            this.m02 = m00.v2;
            this.m10 = m10.v0;
            this.m11 = m10.v1;
            this.m12 = m10.v2;
            this.m20 = m20.v0;
            this.m21 = m20.v1;
            this.m22 = m20.v2;
            this.m30 = m30.v0;
            this.m31 = m30.v1;
            this.m32 = m30.v2;
        }
        
        public bmat4x3(bvec2 m00, bool m01, bvec2 m10, bool m11, bvec2 m20, bool m21, bvec2 m30, bool m31) {
            this.m00 = m00.v0;
            this.m01 = m00.v1;
            this.m02 = m01;
            this.m10 = m10.v0;
            this.m11 = m10.v1;
            this.m12 = m11;
            this.m20 = m20.v0;
            this.m21 = m20.v1;
            this.m22 = m21;
            this.m30 = m30.v0;
            this.m31 = m30.v1;
            this.m32 = m31;
        }
        
        public bmat4x3(bool m00, bvec2 m01, bool m10, bvec2 m11, bool m20, bvec2 m21, bool m30, bvec2 m31) {
            this.m00 = m00;
            this.m01 = m01.v0;
            this.m02 = m01.v1;
            this.m10 = m10;
            this.m11 = m11.v0;
            this.m12 = m11.v1;
            this.m20 = m20;
            this.m21 = m21.v0;
            this.m22 = m21.v1;
            this.m30 = m30;
            this.m31 = m31.v0;
            this.m32 = m31.v1;
        }
        
        public bmat4x3(bool m00, bool m01, bool m02, bool m10, bool m11, bool m12, bool m20, bool m21, bool m22, bool m30, bool m31, bool m32) {
            this.m00 = m00;
            this.m01 = m01;
            this.m02 = m02;
            this.m10 = m10;
            this.m11 = m11;
            this.m12 = m12;
            this.m20 = m20;
            this.m21 = m21;
            this.m22 = m22;
            this.m30 = m30;
            this.m31 = m31;
            this.m32 = m32;
        }
        
        public bmat4x3(bmat2 s) {
            this.m00 = s.m00;
            this.m01 = s.m01;
            this.m02 = false;
            this.m10 = s.m10;
            this.m11 = s.m11;
            this.m12 = false;
            this.m20 = false;
            this.m21 = false;
            this.m22 = false;
            this.m30 = false;
            this.m31 = false;
            this.m32 = false;
        }
        
        public bmat4x3(bmat2x3 s) {
            this.m00 = s.m00;
            this.m01 = s.m01;
            this.m02 = s.m02;
            this.m10 = s.m10;
            this.m11 = s.m11;
            this.m12 = s.m12;
            this.m20 = false;
            this.m21 = false;
            this.m22 = false;
            this.m30 = false;
            this.m31 = false;
            this.m32 = false;
        }
        
        public bmat4x3(bmat2x4 s) {
            this.m00 = s.m00;
            this.m01 = s.m01;
            this.m02 = s.m02;
            this.m10 = s.m10;
            this.m11 = s.m11;
            this.m12 = s.m12;
            this.m20 = false;
            this.m21 = false;
            this.m22 = false;
            this.m30 = false;
            this.m31 = false;
            this.m32 = false;
        }
        
        public bmat4x3(bmat3x2 s) {
            this.m00 = s.m00;
            this.m01 = s.m01;
            this.m02 = false;
            this.m10 = s.m10;
            this.m11 = s.m11;
            this.m12 = false;
            this.m20 = s.m20;
            this.m21 = s.m21;
            this.m22 = false;
            this.m30 = false;
            this.m31 = false;
            this.m32 = false;
        }
        
        public bmat4x3(bmat3 s) {
            this.m00 = s.m00;
            this.m01 = s.m01;
            this.m02 = s.m02;
            this.m10 = s.m10;
            this.m11 = s.m11;
            this.m12 = s.m12;
            this.m20 = s.m20;
            this.m21 = s.m21;
            this.m22 = s.m22;
            this.m30 = false;
            this.m31 = false;
            this.m32 = false;
        }
        
        public bmat4x3(bmat3x4 s) {
            this.m00 = s.m00;
            this.m01 = s.m01;
            this.m02 = s.m02;
            this.m10 = s.m10;
            this.m11 = s.m11;
            this.m12 = s.m12;
            this.m20 = s.m20;
            this.m21 = s.m21;
            this.m22 = s.m22;
            this.m30 = false;
            this.m31 = false;
            this.m32 = false;
        }
        
        public bmat4x3(bmat4x2 s) {
            this.m00 = s.m00;
            this.m01 = s.m01;
            this.m02 = false;
            this.m10 = s.m10;
            this.m11 = s.m11;
            this.m12 = false;
            this.m20 = s.m20;
            this.m21 = s.m21;
            this.m22 = false;
            this.m30 = s.m30;
            this.m31 = s.m31;
            this.m32 = false;
        }
        
        public bmat4x3(bmat4 s) {
            this.m00 = s.m00;
            this.m01 = s.m01;
            this.m02 = s.m02;
            this.m10 = s.m10;
            this.m11 = s.m11;
            this.m12 = s.m12;
            this.m20 = s.m20;
            this.m21 = s.m21;
            this.m22 = s.m22;
            this.m30 = s.m30;
            this.m31 = s.m31;
            this.m32 = s.m32;
        }
        
        public bool this[int i0, int i1] {
            get {
                if (((i0 < 0) 
                            || (3 < i0))) {
                    throw new System.IndexOutOfRangeException("i0");
                }
                if (((i1 < 0) 
                            || (2 < i1))) {
                    throw new System.IndexOutOfRangeException("i1");
                }
                
                unsafe
                {
                    fixed (Boolean* v_ptr = &this.m00)
                    {
                        return *(v_ptr + (i1 + i0 * 3));
                    }
                };
            }
            set {
                if (((i0 < 0) 
                            || (3 < i0))) {
                    throw new System.IndexOutOfRangeException("i0");
                }
                if (((i1 < 0) 
                            || (2 < i1))) {
                    throw new System.IndexOutOfRangeException("i1");
                }
                
                unsafe
                {
                    fixed (Boolean* v_ptr = &this.m00)
                    {
                        *(v_ptr  + (i1 + i0 * 3)) = value;
                    }
                };
            }
        }
        
        public bvec3 this[int i0] {
            get {
                return new bvec3(this[i0, 0], this[i0, 1], this[i0, 2]);
            }
            set {
                this[i0, 0] = value[0];
                this[i0, 1] = value[1];
                this[i0, 2] = value[2];
            }
        }
        
        public static bmat4x3 operator!(bmat4x3 v) {
            return new bmat4x3(!v.m00, !v.m01, !v.m02, !v.m10, !v.m11, !v.m12, !v.m20, !v.m21, !v.m22, !v.m30, !v.m31, !v.m32);
        }
        
        public static bmat4x3 operator&(bool s, bmat4x3 v2) {
            return new bmat4x3(s & v2.m00, s & v2.m01, s & v2.m02, s & v2.m10, s & v2.m11, s & v2.m12, s & v2.m20, s & v2.m21, s & v2.m22, s & v2.m30, s & v2.m31, s & v2.m32);
        }
        
        public static bmat4x3 operator|(bool s, bmat4x3 v2) {
            return new bmat4x3(s | v2.m00, s | v2.m01, s | v2.m02, s | v2.m10, s | v2.m11, s | v2.m12, s | v2.m20, s | v2.m21, s | v2.m22, s | v2.m30, s | v2.m31, s | v2.m32);
        }
        
        public static bmat4x3 operator^(bool s, bmat4x3 v2) {
            return new bmat4x3(s ^ v2.m00, s ^ v2.m01, s ^ v2.m02, s ^ v2.m10, s ^ v2.m11, s ^ v2.m12, s ^ v2.m20, s ^ v2.m21, s ^ v2.m22, s ^ v2.m30, s ^ v2.m31, s ^ v2.m32);
        }
        
        public static bmat4x3 operator&(bmat4x3 v1, bool s) {
            return new bmat4x3(v1.m00 & s, v1.m01 & s, v1.m02 & s, v1.m10 & s, v1.m11 & s, v1.m12 & s, v1.m20 & s, v1.m21 & s, v1.m22 & s, v1.m30 & s, v1.m31 & s, v1.m32 & s);
        }
        
        public static bmat4x3 operator|(bmat4x3 v1, bool s) {
            return new bmat4x3(v1.m00 | s, v1.m01 | s, v1.m02 | s, v1.m10 | s, v1.m11 | s, v1.m12 | s, v1.m20 | s, v1.m21 | s, v1.m22 | s, v1.m30 | s, v1.m31 | s, v1.m32 | s);
        }
        
        public static bmat4x3 operator^(bmat4x3 v1, bool s) {
            return new bmat4x3(v1.m00 ^ s, v1.m01 ^ s, v1.m02 ^ s, v1.m10 ^ s, v1.m11 ^ s, v1.m12 ^ s, v1.m20 ^ s, v1.m21 ^ s, v1.m22 ^ s, v1.m30 ^ s, v1.m31 ^ s, v1.m32 ^ s);
        }
        
        public static bmat4x3 operator&(bmat4x3 v1, bmat4x3 v2) {
            return new bmat4x3(v1.m00 & v2.m00, v1.m01 & v2.m01, v1.m02 & v2.m02, v1.m10 & v2.m10, v1.m11 & v2.m11, v1.m12 & v2.m12, v1.m20 & v2.m20, v1.m21 & v2.m21, v1.m22 & v2.m22, v1.m30 & v2.m30, v1.m31 & v2.m31, v1.m32 & v2.m32);
        }
        
        public static bmat4x3 operator|(bmat4x3 v1, bmat4x3 v2) {
            return new bmat4x3(v1.m00 | v2.m00, v1.m01 | v2.m01, v1.m02 | v2.m02, v1.m10 | v2.m10, v1.m11 | v2.m11, v1.m12 | v2.m12, v1.m20 | v2.m20, v1.m21 | v2.m21, v1.m22 | v2.m22, v1.m30 | v2.m30, v1.m31 | v2.m31, v1.m32 | v2.m32);
        }
        
        public static bmat4x3 operator^(bmat4x3 v1, bmat4x3 v2) {
            return new bmat4x3(v1.m00 ^ v2.m00, v1.m01 ^ v2.m01, v1.m02 ^ v2.m02, v1.m10 ^ v2.m10, v1.m11 ^ v2.m11, v1.m12 ^ v2.m12, v1.m20 ^ v2.m20, v1.m21 ^ v2.m21, v1.m22 ^ v2.m22, v1.m30 ^ v2.m30, v1.m31 ^ v2.m31, v1.m32 ^ v2.m32);
        }
        
        public static bmat4x3 operator==(bmat4x3 v1, bmat4x3 v2) {
            return new bmat4x3(v1.m00 == v2.m00, v1.m01 == v2.m01, v1.m02 == v2.m02, v1.m10 == v2.m10, v1.m11 == v2.m11, v1.m12 == v2.m12, v1.m20 == v2.m20, v1.m21 == v2.m21, v1.m22 == v2.m22, v1.m30 == v2.m30, v1.m31 == v2.m31, v1.m32 == v2.m32);
        }
        
        public static bmat4x3 operator!=(bmat4x3 v1, bmat4x3 v2) {
            return new bmat4x3(v1.m00 != v2.m00, v1.m01 != v2.m01, v1.m02 != v2.m02, v1.m10 != v2.m10, v1.m11 != v2.m11, v1.m12 != v2.m12, v1.m20 != v2.m20, v1.m21 != v2.m21, v1.m22 != v2.m22, v1.m30 != v2.m30, v1.m31 != v2.m31, v1.m32 != v2.m32);
        }
        
        public override bool Equals(object obj) {
            return (obj is bmat4x3) ? bmath.All(this == (bmat4x3)obj) : false;
        }
        
        public override int GetHashCode() {
            return this.m00.GetHashCode() ^ this.m01.GetHashCode() ^ this.m02.GetHashCode() ^ this.m10.GetHashCode() ^ this.m11.GetHashCode() ^ this.m12.GetHashCode() ^ this.m20.GetHashCode() ^ this.m21.GetHashCode() ^ this.m22.GetHashCode() ^ this.m30.GetHashCode() ^ this.m31.GetHashCode() ^ this.m32.GetHashCode();
        }
        
        public override string ToString() {
            return "bmat4x3(" + this.m00 + "," + this.m01 + "," + this.m02 + "," + this.m10 + "," + this.m11 + "," + this.m12 + "," + this.m20 + "," + this.m21 + "," + this.m22 + "," + this.m30 + "," + this.m31 + "," + this.m32 + ")";
        }
    }
    
    [DataContract()]
    [StructLayout(LayoutKind.Sequential)]
    public struct bmat4 {
        
        [DataMember()]
        internal bool m00;
        
        [DataMember()]
        internal bool m01;
        
        [DataMember()]
        internal bool m02;
        
        [DataMember()]
        internal bool m03;
        
        [DataMember()]
        internal bool m10;
        
        [DataMember()]
        internal bool m11;
        
        [DataMember()]
        internal bool m12;
        
        [DataMember()]
        internal bool m13;
        
        [DataMember()]
        internal bool m20;
        
        [DataMember()]
        internal bool m21;
        
        [DataMember()]
        internal bool m22;
        
        [DataMember()]
        internal bool m23;
        
        [DataMember()]
        internal bool m30;
        
        [DataMember()]
        internal bool m31;
        
        [DataMember()]
        internal bool m32;
        
        [DataMember()]
        internal bool m33;
        
        public bmat4(bool s) {
            this.m00 = s;
            this.m01 = false;
            this.m02 = false;
            this.m03 = false;
            this.m10 = false;
            this.m11 = s;
            this.m12 = false;
            this.m13 = false;
            this.m20 = false;
            this.m21 = false;
            this.m22 = s;
            this.m23 = false;
            this.m30 = false;
            this.m31 = false;
            this.m32 = false;
            this.m33 = s;
        }
        
        public bmat4(bmat4 m00) {
            this.m00 = m00.m00;
            this.m01 = m00.m01;
            this.m02 = m00.m02;
            this.m03 = m00.m03;
            this.m10 = m00.m10;
            this.m11 = m00.m11;
            this.m12 = m00.m12;
            this.m13 = m00.m13;
            this.m20 = m00.m20;
            this.m21 = m00.m21;
            this.m22 = m00.m22;
            this.m23 = m00.m23;
            this.m30 = m00.m30;
            this.m31 = m00.m31;
            this.m32 = m00.m32;
            this.m33 = m00.m33;
        }
        
        public bmat4(bmat4x2 m00, bmat4x2 m01) {
            this.m00 = m00.m00;
            this.m01 = m00.m01;
            this.m02 = m01.m00;
            this.m03 = m01.m01;
            this.m10 = m00.m10;
            this.m11 = m00.m11;
            this.m12 = m01.m10;
            this.m13 = m01.m11;
            this.m20 = m00.m20;
            this.m21 = m00.m21;
            this.m22 = m01.m20;
            this.m23 = m01.m21;
            this.m30 = m00.m30;
            this.m31 = m00.m31;
            this.m32 = m01.m30;
            this.m33 = m01.m31;
        }
        
        public bmat4(bmat2x4 m00, bmat2x4 m10) {
            this.m00 = m00.m00;
            this.m01 = m00.m01;
            this.m02 = m00.m02;
            this.m03 = m00.m03;
            this.m10 = m00.m10;
            this.m11 = m00.m11;
            this.m12 = m00.m12;
            this.m13 = m00.m13;
            this.m20 = m10.m00;
            this.m21 = m10.m01;
            this.m22 = m10.m02;
            this.m23 = m10.m03;
            this.m30 = m10.m10;
            this.m31 = m10.m11;
            this.m32 = m10.m12;
            this.m33 = m10.m13;
        }
        
        public bmat4(bmat2 m00, bmat2 m01, bmat2 m10, bmat2 m11) {
            this.m00 = m00.m00;
            this.m01 = m00.m01;
            this.m02 = m01.m00;
            this.m03 = m01.m01;
            this.m10 = m00.m10;
            this.m11 = m00.m11;
            this.m12 = m01.m10;
            this.m13 = m01.m11;
            this.m20 = m10.m00;
            this.m21 = m10.m01;
            this.m22 = m11.m00;
            this.m23 = m11.m01;
            this.m30 = m10.m10;
            this.m31 = m10.m11;
            this.m32 = m11.m10;
            this.m33 = m11.m11;
        }
        
        public bmat4(bmat3x4 m00, bvec4 m10) {
            this.m00 = m00.m00;
            this.m01 = m00.m01;
            this.m02 = m00.m02;
            this.m03 = m00.m03;
            this.m10 = m00.m10;
            this.m11 = m00.m11;
            this.m12 = m00.m12;
            this.m13 = m00.m13;
            this.m20 = m00.m20;
            this.m21 = m00.m21;
            this.m22 = m00.m22;
            this.m23 = m00.m23;
            this.m30 = m10.v0;
            this.m31 = m10.v1;
            this.m32 = m10.v2;
            this.m33 = m10.v3;
        }
        
        public bmat4(bmat3x2 m00, bmat3x2 m01, bvec2 m10, bvec2 m11) {
            this.m00 = m00.m00;
            this.m01 = m00.m01;
            this.m02 = m01.m00;
            this.m03 = m01.m01;
            this.m10 = m00.m10;
            this.m11 = m00.m11;
            this.m12 = m01.m10;
            this.m13 = m01.m11;
            this.m20 = m00.m20;
            this.m21 = m00.m21;
            this.m22 = m01.m20;
            this.m23 = m01.m21;
            this.m30 = m10.v0;
            this.m31 = m10.v1;
            this.m32 = m11.v0;
            this.m33 = m11.v1;
        }
        
        public bmat4(bvec4 m00, bmat3x4 m10) {
            this.m00 = m00.v0;
            this.m01 = m00.v1;
            this.m02 = m00.v2;
            this.m03 = m00.v3;
            this.m10 = m10.m00;
            this.m11 = m10.m01;
            this.m12 = m10.m02;
            this.m13 = m10.m03;
            this.m20 = m10.m10;
            this.m21 = m10.m11;
            this.m22 = m10.m12;
            this.m23 = m10.m13;
            this.m30 = m10.m20;
            this.m31 = m10.m21;
            this.m32 = m10.m22;
            this.m33 = m10.m23;
        }
        
        public bmat4(bvec2 m00, bvec2 m01, bmat3x2 m10, bmat3x2 m11) {
            this.m00 = m00.v0;
            this.m01 = m00.v1;
            this.m02 = m01.v0;
            this.m03 = m01.v1;
            this.m10 = m10.m00;
            this.m11 = m10.m01;
            this.m12 = m11.m00;
            this.m13 = m11.m01;
            this.m20 = m10.m10;
            this.m21 = m10.m11;
            this.m22 = m11.m10;
            this.m23 = m11.m11;
            this.m30 = m10.m20;
            this.m31 = m10.m21;
            this.m32 = m11.m20;
            this.m33 = m11.m21;
        }
        
        public bmat4(bmat2x4 m00, bvec4 m10, bvec4 m20) {
            this.m00 = m00.m00;
            this.m01 = m00.m01;
            this.m02 = m00.m02;
            this.m03 = m00.m03;
            this.m10 = m00.m10;
            this.m11 = m00.m11;
            this.m12 = m00.m12;
            this.m13 = m00.m13;
            this.m20 = m10.v0;
            this.m21 = m10.v1;
            this.m22 = m10.v2;
            this.m23 = m10.v3;
            this.m30 = m20.v0;
            this.m31 = m20.v1;
            this.m32 = m20.v2;
            this.m33 = m20.v3;
        }
        
        public bmat4(bmat2 m00, bmat2 m01, bvec2 m10, bvec2 m11, bvec2 m20, bvec2 m21) {
            this.m00 = m00.m00;
            this.m01 = m00.m01;
            this.m02 = m01.m00;
            this.m03 = m01.m01;
            this.m10 = m00.m10;
            this.m11 = m00.m11;
            this.m12 = m01.m10;
            this.m13 = m01.m11;
            this.m20 = m10.v0;
            this.m21 = m10.v1;
            this.m22 = m11.v0;
            this.m23 = m11.v1;
            this.m30 = m20.v0;
            this.m31 = m20.v1;
            this.m32 = m21.v0;
            this.m33 = m21.v1;
        }
        
        public bmat4(bvec4 m00, bmat2x4 m10, bvec4 m20) {
            this.m00 = m00.v0;
            this.m01 = m00.v1;
            this.m02 = m00.v2;
            this.m03 = m00.v3;
            this.m10 = m10.m00;
            this.m11 = m10.m01;
            this.m12 = m10.m02;
            this.m13 = m10.m03;
            this.m20 = m10.m10;
            this.m21 = m10.m11;
            this.m22 = m10.m12;
            this.m23 = m10.m13;
            this.m30 = m20.v0;
            this.m31 = m20.v1;
            this.m32 = m20.v2;
            this.m33 = m20.v3;
        }
        
        public bmat4(bvec2 m00, bvec2 m01, bmat2 m10, bmat2 m11, bvec2 m20, bvec2 m21) {
            this.m00 = m00.v0;
            this.m01 = m00.v1;
            this.m02 = m01.v0;
            this.m03 = m01.v1;
            this.m10 = m10.m00;
            this.m11 = m10.m01;
            this.m12 = m11.m00;
            this.m13 = m11.m01;
            this.m20 = m10.m10;
            this.m21 = m10.m11;
            this.m22 = m11.m10;
            this.m23 = m11.m11;
            this.m30 = m20.v0;
            this.m31 = m20.v1;
            this.m32 = m21.v0;
            this.m33 = m21.v1;
        }
        
        public bmat4(bvec4 m00, bvec4 m10, bmat2x4 m20) {
            this.m00 = m00.v0;
            this.m01 = m00.v1;
            this.m02 = m00.v2;
            this.m03 = m00.v3;
            this.m10 = m10.v0;
            this.m11 = m10.v1;
            this.m12 = m10.v2;
            this.m13 = m10.v3;
            this.m20 = m20.m00;
            this.m21 = m20.m01;
            this.m22 = m20.m02;
            this.m23 = m20.m03;
            this.m30 = m20.m10;
            this.m31 = m20.m11;
            this.m32 = m20.m12;
            this.m33 = m20.m13;
        }
        
        public bmat4(bvec2 m00, bvec2 m01, bvec2 m10, bvec2 m11, bmat2 m20, bmat2 m21) {
            this.m00 = m00.v0;
            this.m01 = m00.v1;
            this.m02 = m01.v0;
            this.m03 = m01.v1;
            this.m10 = m10.v0;
            this.m11 = m10.v1;
            this.m12 = m11.v0;
            this.m13 = m11.v1;
            this.m20 = m20.m00;
            this.m21 = m20.m01;
            this.m22 = m21.m00;
            this.m23 = m21.m01;
            this.m30 = m20.m10;
            this.m31 = m20.m11;
            this.m32 = m21.m10;
            this.m33 = m21.m11;
        }
        
        public bmat4(bvec4 m00, bvec4 m10, bvec4 m20, bvec4 m30) {
            this.m00 = m00.v0;
            this.m01 = m00.v1;
            this.m02 = m00.v2;
            this.m03 = m00.v3;
            this.m10 = m10.v0;
            this.m11 = m10.v1;
            this.m12 = m10.v2;
            this.m13 = m10.v3;
            this.m20 = m20.v0;
            this.m21 = m20.v1;
            this.m22 = m20.v2;
            this.m23 = m20.v3;
            this.m30 = m30.v0;
            this.m31 = m30.v1;
            this.m32 = m30.v2;
            this.m33 = m30.v3;
        }
        
        public bmat4(bvec2 m00, bvec2 m01, bvec2 m10, bvec2 m11, bvec2 m20, bvec2 m21, bvec2 m30, bvec2 m31) {
            this.m00 = m00.v0;
            this.m01 = m00.v1;
            this.m02 = m01.v0;
            this.m03 = m01.v1;
            this.m10 = m10.v0;
            this.m11 = m10.v1;
            this.m12 = m11.v0;
            this.m13 = m11.v1;
            this.m20 = m20.v0;
            this.m21 = m20.v1;
            this.m22 = m21.v0;
            this.m23 = m21.v1;
            this.m30 = m30.v0;
            this.m31 = m30.v1;
            this.m32 = m31.v0;
            this.m33 = m31.v1;
        }
        
        public bmat4(bvec3 m00, bool m01, bvec3 m10, bool m11, bvec3 m20, bool m21, bvec3 m30, bool m31) {
            this.m00 = m00.v0;
            this.m01 = m00.v1;
            this.m02 = m00.v2;
            this.m03 = m01;
            this.m10 = m10.v0;
            this.m11 = m10.v1;
            this.m12 = m10.v2;
            this.m13 = m11;
            this.m20 = m20.v0;
            this.m21 = m20.v1;
            this.m22 = m20.v2;
            this.m23 = m21;
            this.m30 = m30.v0;
            this.m31 = m30.v1;
            this.m32 = m30.v2;
            this.m33 = m31;
        }
        
        public bmat4(bool m00, bvec3 m01, bool m10, bvec3 m11, bool m20, bvec3 m21, bool m30, bvec3 m31) {
            this.m00 = m00;
            this.m01 = m01.v0;
            this.m02 = m01.v1;
            this.m03 = m01.v2;
            this.m10 = m10;
            this.m11 = m11.v0;
            this.m12 = m11.v1;
            this.m13 = m11.v2;
            this.m20 = m20;
            this.m21 = m21.v0;
            this.m22 = m21.v1;
            this.m23 = m21.v2;
            this.m30 = m30;
            this.m31 = m31.v0;
            this.m32 = m31.v1;
            this.m33 = m31.v2;
        }
        
        public bmat4(bvec2 m00, bool m01, bool m02, bvec2 m10, bool m11, bool m12, bvec2 m20, bool m21, bool m22, bvec2 m30, bool m31, bool m32) {
            this.m00 = m00.v0;
            this.m01 = m00.v1;
            this.m02 = m01;
            this.m03 = m02;
            this.m10 = m10.v0;
            this.m11 = m10.v1;
            this.m12 = m11;
            this.m13 = m12;
            this.m20 = m20.v0;
            this.m21 = m20.v1;
            this.m22 = m21;
            this.m23 = m22;
            this.m30 = m30.v0;
            this.m31 = m30.v1;
            this.m32 = m31;
            this.m33 = m32;
        }
        
        public bmat4(bool m00, bvec2 m01, bool m02, bool m10, bvec2 m11, bool m12, bool m20, bvec2 m21, bool m22, bool m30, bvec2 m31, bool m32) {
            this.m00 = m00;
            this.m01 = m01.v0;
            this.m02 = m01.v1;
            this.m03 = m02;
            this.m10 = m10;
            this.m11 = m11.v0;
            this.m12 = m11.v1;
            this.m13 = m12;
            this.m20 = m20;
            this.m21 = m21.v0;
            this.m22 = m21.v1;
            this.m23 = m22;
            this.m30 = m30;
            this.m31 = m31.v0;
            this.m32 = m31.v1;
            this.m33 = m32;
        }
        
        public bmat4(bool m00, bool m01, bvec2 m02, bool m10, bool m11, bvec2 m12, bool m20, bool m21, bvec2 m22, bool m30, bool m31, bvec2 m32) {
            this.m00 = m00;
            this.m01 = m01;
            this.m02 = m02.v0;
            this.m03 = m02.v1;
            this.m10 = m10;
            this.m11 = m11;
            this.m12 = m12.v0;
            this.m13 = m12.v1;
            this.m20 = m20;
            this.m21 = m21;
            this.m22 = m22.v0;
            this.m23 = m22.v1;
            this.m30 = m30;
            this.m31 = m31;
            this.m32 = m32.v0;
            this.m33 = m32.v1;
        }
        
        public bmat4(
                    bool m00, 
                    bool m01, 
                    bool m02, 
                    bool m03, 
                    bool m10, 
                    bool m11, 
                    bool m12, 
                    bool m13, 
                    bool m20, 
                    bool m21, 
                    bool m22, 
                    bool m23, 
                    bool m30, 
                    bool m31, 
                    bool m32, 
                    bool m33) {
            this.m00 = m00;
            this.m01 = m01;
            this.m02 = m02;
            this.m03 = m03;
            this.m10 = m10;
            this.m11 = m11;
            this.m12 = m12;
            this.m13 = m13;
            this.m20 = m20;
            this.m21 = m21;
            this.m22 = m22;
            this.m23 = m23;
            this.m30 = m30;
            this.m31 = m31;
            this.m32 = m32;
            this.m33 = m33;
        }
        
        public bmat4(bmat2 s) {
            this.m00 = s.m00;
            this.m01 = s.m01;
            this.m02 = false;
            this.m03 = false;
            this.m10 = s.m10;
            this.m11 = s.m11;
            this.m12 = false;
            this.m13 = false;
            this.m20 = false;
            this.m21 = false;
            this.m22 = false;
            this.m23 = false;
            this.m30 = false;
            this.m31 = false;
            this.m32 = false;
            this.m33 = false;
        }
        
        public bmat4(bmat2x3 s) {
            this.m00 = s.m00;
            this.m01 = s.m01;
            this.m02 = s.m02;
            this.m03 = false;
            this.m10 = s.m10;
            this.m11 = s.m11;
            this.m12 = s.m12;
            this.m13 = false;
            this.m20 = false;
            this.m21 = false;
            this.m22 = false;
            this.m23 = false;
            this.m30 = false;
            this.m31 = false;
            this.m32 = false;
            this.m33 = false;
        }
        
        public bmat4(bmat2x4 s) {
            this.m00 = s.m00;
            this.m01 = s.m01;
            this.m02 = s.m02;
            this.m03 = s.m03;
            this.m10 = s.m10;
            this.m11 = s.m11;
            this.m12 = s.m12;
            this.m13 = s.m13;
            this.m20 = false;
            this.m21 = false;
            this.m22 = false;
            this.m23 = false;
            this.m30 = false;
            this.m31 = false;
            this.m32 = false;
            this.m33 = false;
        }
        
        public bmat4(bmat3x2 s) {
            this.m00 = s.m00;
            this.m01 = s.m01;
            this.m02 = false;
            this.m03 = false;
            this.m10 = s.m10;
            this.m11 = s.m11;
            this.m12 = false;
            this.m13 = false;
            this.m20 = s.m20;
            this.m21 = s.m21;
            this.m22 = false;
            this.m23 = false;
            this.m30 = false;
            this.m31 = false;
            this.m32 = false;
            this.m33 = false;
        }
        
        public bmat4(bmat3 s) {
            this.m00 = s.m00;
            this.m01 = s.m01;
            this.m02 = s.m02;
            this.m03 = false;
            this.m10 = s.m10;
            this.m11 = s.m11;
            this.m12 = s.m12;
            this.m13 = false;
            this.m20 = s.m20;
            this.m21 = s.m21;
            this.m22 = s.m22;
            this.m23 = false;
            this.m30 = false;
            this.m31 = false;
            this.m32 = false;
            this.m33 = false;
        }
        
        public bmat4(bmat3x4 s) {
            this.m00 = s.m00;
            this.m01 = s.m01;
            this.m02 = s.m02;
            this.m03 = s.m03;
            this.m10 = s.m10;
            this.m11 = s.m11;
            this.m12 = s.m12;
            this.m13 = s.m13;
            this.m20 = s.m20;
            this.m21 = s.m21;
            this.m22 = s.m22;
            this.m23 = s.m23;
            this.m30 = false;
            this.m31 = false;
            this.m32 = false;
            this.m33 = false;
        }
        
        public bmat4(bmat4x2 s) {
            this.m00 = s.m00;
            this.m01 = s.m01;
            this.m02 = false;
            this.m03 = false;
            this.m10 = s.m10;
            this.m11 = s.m11;
            this.m12 = false;
            this.m13 = false;
            this.m20 = s.m20;
            this.m21 = s.m21;
            this.m22 = false;
            this.m23 = false;
            this.m30 = s.m30;
            this.m31 = s.m31;
            this.m32 = false;
            this.m33 = false;
        }
        
        public bmat4(bmat4x3 s) {
            this.m00 = s.m00;
            this.m01 = s.m01;
            this.m02 = s.m02;
            this.m03 = false;
            this.m10 = s.m10;
            this.m11 = s.m11;
            this.m12 = s.m12;
            this.m13 = false;
            this.m20 = s.m20;
            this.m21 = s.m21;
            this.m22 = s.m22;
            this.m23 = false;
            this.m30 = s.m30;
            this.m31 = s.m31;
            this.m32 = s.m32;
            this.m33 = false;
        }
        
        public bool this[int i0, int i1] {
            get {
                if (((i0 < 0) 
                            || (3 < i0))) {
                    throw new System.IndexOutOfRangeException("i0");
                }
                if (((i1 < 0) 
                            || (3 < i1))) {
                    throw new System.IndexOutOfRangeException("i1");
                }
                
                unsafe
                {
                    fixed (Boolean* v_ptr = &this.m00)
                    {
                        return *(v_ptr + (i1 + i0 * 4));
                    }
                };
            }
            set {
                if (((i0 < 0) 
                            || (3 < i0))) {
                    throw new System.IndexOutOfRangeException("i0");
                }
                if (((i1 < 0) 
                            || (3 < i1))) {
                    throw new System.IndexOutOfRangeException("i1");
                }
                
                unsafe
                {
                    fixed (Boolean* v_ptr = &this.m00)
                    {
                        *(v_ptr  + (i1 + i0 * 4)) = value;
                    }
                };
            }
        }
        
        public bvec4 this[int i0] {
            get {
                return new bvec4(this[i0, 0], this[i0, 1], this[i0, 2], this[i0, 3]);
            }
            set {
                this[i0, 0] = value[0];
                this[i0, 1] = value[1];
                this[i0, 2] = value[2];
                this[i0, 3] = value[3];
            }
        }
        
        public static bmat4 operator!(bmat4 v) {
            return new bmat4(!v.m00, !v.m01, !v.m02, !v.m03, !v.m10, !v.m11, !v.m12, !v.m13, !v.m20, !v.m21, !v.m22, !v.m23, !v.m30, !v.m31, !v.m32, !v.m33);
        }
        
        public static bmat4 operator&(bool s, bmat4 v2) {
            return new bmat4(s & v2.m00, s & v2.m01, s & v2.m02, s & v2.m03, s & v2.m10, s & v2.m11, s & v2.m12, s & v2.m13, s & v2.m20, s & v2.m21, s & v2.m22, s & v2.m23, s & v2.m30, s & v2.m31, s & v2.m32, s & v2.m33);
        }
        
        public static bmat4 operator|(bool s, bmat4 v2) {
            return new bmat4(s | v2.m00, s | v2.m01, s | v2.m02, s | v2.m03, s | v2.m10, s | v2.m11, s | v2.m12, s | v2.m13, s | v2.m20, s | v2.m21, s | v2.m22, s | v2.m23, s | v2.m30, s | v2.m31, s | v2.m32, s | v2.m33);
        }
        
        public static bmat4 operator^(bool s, bmat4 v2) {
            return new bmat4(s ^ v2.m00, s ^ v2.m01, s ^ v2.m02, s ^ v2.m03, s ^ v2.m10, s ^ v2.m11, s ^ v2.m12, s ^ v2.m13, s ^ v2.m20, s ^ v2.m21, s ^ v2.m22, s ^ v2.m23, s ^ v2.m30, s ^ v2.m31, s ^ v2.m32, s ^ v2.m33);
        }
        
        public static bmat4 operator&(bmat4 v1, bool s) {
            return new bmat4(v1.m00 & s, v1.m01 & s, v1.m02 & s, v1.m03 & s, v1.m10 & s, v1.m11 & s, v1.m12 & s, v1.m13 & s, v1.m20 & s, v1.m21 & s, v1.m22 & s, v1.m23 & s, v1.m30 & s, v1.m31 & s, v1.m32 & s, v1.m33 & s);
        }
        
        public static bmat4 operator|(bmat4 v1, bool s) {
            return new bmat4(v1.m00 | s, v1.m01 | s, v1.m02 | s, v1.m03 | s, v1.m10 | s, v1.m11 | s, v1.m12 | s, v1.m13 | s, v1.m20 | s, v1.m21 | s, v1.m22 | s, v1.m23 | s, v1.m30 | s, v1.m31 | s, v1.m32 | s, v1.m33 | s);
        }
        
        public static bmat4 operator^(bmat4 v1, bool s) {
            return new bmat4(v1.m00 ^ s, v1.m01 ^ s, v1.m02 ^ s, v1.m03 ^ s, v1.m10 ^ s, v1.m11 ^ s, v1.m12 ^ s, v1.m13 ^ s, v1.m20 ^ s, v1.m21 ^ s, v1.m22 ^ s, v1.m23 ^ s, v1.m30 ^ s, v1.m31 ^ s, v1.m32 ^ s, v1.m33 ^ s);
        }
        
        public static bmat4 operator&(bmat4 v1, bmat4 v2) {
            return new bmat4(v1.m00 & v2.m00, v1.m01 & v2.m01, v1.m02 & v2.m02, v1.m03 & v2.m03, v1.m10 & v2.m10, v1.m11 & v2.m11, v1.m12 & v2.m12, v1.m13 & v2.m13, v1.m20 & v2.m20, v1.m21 & v2.m21, v1.m22 & v2.m22, v1.m23 & v2.m23, v1.m30 & v2.m30, v1.m31 & v2.m31, v1.m32 & v2.m32, v1.m33 & v2.m33);
        }
        
        public static bmat4 operator|(bmat4 v1, bmat4 v2) {
            return new bmat4(v1.m00 | v2.m00, v1.m01 | v2.m01, v1.m02 | v2.m02, v1.m03 | v2.m03, v1.m10 | v2.m10, v1.m11 | v2.m11, v1.m12 | v2.m12, v1.m13 | v2.m13, v1.m20 | v2.m20, v1.m21 | v2.m21, v1.m22 | v2.m22, v1.m23 | v2.m23, v1.m30 | v2.m30, v1.m31 | v2.m31, v1.m32 | v2.m32, v1.m33 | v2.m33);
        }
        
        public static bmat4 operator^(bmat4 v1, bmat4 v2) {
            return new bmat4(v1.m00 ^ v2.m00, v1.m01 ^ v2.m01, v1.m02 ^ v2.m02, v1.m03 ^ v2.m03, v1.m10 ^ v2.m10, v1.m11 ^ v2.m11, v1.m12 ^ v2.m12, v1.m13 ^ v2.m13, v1.m20 ^ v2.m20, v1.m21 ^ v2.m21, v1.m22 ^ v2.m22, v1.m23 ^ v2.m23, v1.m30 ^ v2.m30, v1.m31 ^ v2.m31, v1.m32 ^ v2.m32, v1.m33 ^ v2.m33);
        }
        
        public static bmat4 operator==(bmat4 v1, bmat4 v2) {
            return new bmat4(v1.m00 == v2.m00, v1.m01 == v2.m01, v1.m02 == v2.m02, v1.m03 == v2.m03, v1.m10 == v2.m10, v1.m11 == v2.m11, v1.m12 == v2.m12, v1.m13 == v2.m13, v1.m20 == v2.m20, v1.m21 == v2.m21, v1.m22 == v2.m22, v1.m23 == v2.m23, v1.m30 == v2.m30, v1.m31 == v2.m31, v1.m32 == v2.m32, v1.m33 == v2.m33);
        }
        
        public static bmat4 operator!=(bmat4 v1, bmat4 v2) {
            return new bmat4(v1.m00 != v2.m00, v1.m01 != v2.m01, v1.m02 != v2.m02, v1.m03 != v2.m03, v1.m10 != v2.m10, v1.m11 != v2.m11, v1.m12 != v2.m12, v1.m13 != v2.m13, v1.m20 != v2.m20, v1.m21 != v2.m21, v1.m22 != v2.m22, v1.m23 != v2.m23, v1.m30 != v2.m30, v1.m31 != v2.m31, v1.m32 != v2.m32, v1.m33 != v2.m33);
        }
        
        public override bool Equals(object obj) {
            return (obj is bmat4) ? bmath.All(this == (bmat4)obj) : false;
        }
        
        public override int GetHashCode() {
            return this.m00.GetHashCode() ^ this.m01.GetHashCode() ^ this.m02.GetHashCode() ^ this.m03.GetHashCode() ^ this.m10.GetHashCode() ^ this.m11.GetHashCode() ^ this.m12.GetHashCode() ^ this.m13.GetHashCode() ^ this.m20.GetHashCode() ^ this.m21.GetHashCode() ^ this.m22.GetHashCode() ^ this.m23.GetHashCode() ^ this.m30.GetHashCode() ^ this.m31.GetHashCode() ^ this.m32.GetHashCode() ^ this.m33.GetHashCode();
        }
        
        public override string ToString() {
            return "bmat4(" + this.m00 + "," + this.m01 + "," + this.m02 + "," + this.m03 + "," + this.m10 + "," + this.m11 + "," + this.m12 + "," + this.m13 + "," + this.m20 + "," + this.m21 + "," + this.m22 + "," + this.m23 + "," + this.m30 + "," + this.m31 + "," + this.m32 + "," + this.m33 + ")";
        }
    }
}
