#region --- License ---
/*
Copyright 2010 Daniel Filonik

Permission is hereby granted, free of charge, to any person obtaining a copy of
this software and associated documentation files (the "Software"), to deal in
the Software without restriction, including without limitation the rights to
use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
of the Software, and to permit persons to whom the Software is furnished to do
so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all
copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
SOFTWARE.
*/
#endregion

namespace TKM {
    using System;
    using System.Runtime.InteropServices;
    using System.Runtime.Serialization;
    
    
    [DataContract()]
    [StructLayout(LayoutKind.Sequential)]
    public struct bvec2 {
        
        [DataMember()]
        internal bool v0;
        
        [DataMember()]
        internal bool v1;
        
        public bvec2(bool s) {
            this.v0 = s;
            this.v1 = s;
        }
        
        public bvec2(bvec2 v0) {
            this.v0 = v0.v0;
            this.v1 = v0.v1;
        }
        
        public bvec2(bool v0, bool v1) {
            this.v0 = v0;
            this.v1 = v1;
        }
        
        public bvec2(bvec3 s) {
            this.v0 = s.v0;
            this.v1 = s.v1;
        }
        
        public bvec2(bvec4 s) {
            this.v0 = s.v0;
            this.v1 = s.v1;
        }
        
        public bool this[int i0] {
            get {
                if (((i0 < 0) 
                            || (1 < i0))) {
                    throw new System.IndexOutOfRangeException("i0");
                }
                
                unsafe
                {
                    fixed (Boolean* v_ptr = &this.v0)
                    {
                        return *(v_ptr + (i0));
                    }
                };
            }
            set {
                if (((i0 < 0) 
                            || (1 < i0))) {
                    throw new System.IndexOutOfRangeException("i0");
                }
                
                unsafe
                {
                    fixed (Boolean* v_ptr = &this.v0)
                    {
                        *(v_ptr  + (i0)) = value;
                    }
                };
            }
        }
        
        public bool x {
            get {
                return this.v0;
            }
            set {
                this.v0 = value;
            }
        }
        
        public bool r {
            get {
                return this.v0;
            }
            set {
                this.v0 = value;
            }
        }
        
        public bool s {
            get {
                return this.v0;
            }
            set {
                this.v0 = value;
            }
        }
        
        public bool y {
            get {
                return this.v1;
            }
            set {
                this.v1 = value;
            }
        }
        
        public bool g {
            get {
                return this.v1;
            }
            set {
                this.v1 = value;
            }
        }
        
        public bool t {
            get {
                return this.v1;
            }
            set {
                this.v1 = value;
            }
        }
        
        public bvec2 xx {
            get {
                return new bvec2(this.v0, this.v0);
            }
        }
        
        public bvec2 rr {
            get {
                return new bvec2(this.v0, this.v0);
            }
        }
        
        public bvec2 ss {
            get {
                return new bvec2(this.v0, this.v0);
            }
        }
        
        public bvec2 xy {
            get {
                return new bvec2(this.v0, this.v1);
            }
            set {
                this.v0 = value.v0;
                this.v1 = value.v1;
            }
        }
        
        public bvec2 rg {
            get {
                return new bvec2(this.v0, this.v1);
            }
            set {
                this.v0 = value.v0;
                this.v1 = value.v1;
            }
        }
        
        public bvec2 st {
            get {
                return new bvec2(this.v0, this.v1);
            }
            set {
                this.v0 = value.v0;
                this.v1 = value.v1;
            }
        }
        
        public bvec2 yx {
            get {
                return new bvec2(this.v1, this.v0);
            }
            set {
                this.v1 = value.v0;
                this.v0 = value.v1;
            }
        }
        
        public bvec2 gr {
            get {
                return new bvec2(this.v1, this.v0);
            }
            set {
                this.v1 = value.v0;
                this.v0 = value.v1;
            }
        }
        
        public bvec2 ts {
            get {
                return new bvec2(this.v1, this.v0);
            }
            set {
                this.v1 = value.v0;
                this.v0 = value.v1;
            }
        }
        
        public bvec2 yy {
            get {
                return new bvec2(this.v1, this.v1);
            }
        }
        
        public bvec2 gg {
            get {
                return new bvec2(this.v1, this.v1);
            }
        }
        
        public bvec2 tt {
            get {
                return new bvec2(this.v1, this.v1);
            }
        }
        
        public static bvec2 operator!(bvec2 v) {
            return new bvec2(!v.v0, !v.v1);
        }
        
        public static bvec2 operator&(bool s, bvec2 v2) {
            return new bvec2(s & v2.v0, s & v2.v1);
        }
        
        public static bvec2 operator|(bool s, bvec2 v2) {
            return new bvec2(s | v2.v0, s | v2.v1);
        }
        
        public static bvec2 operator^(bool s, bvec2 v2) {
            return new bvec2(s ^ v2.v0, s ^ v2.v1);
        }
        
        public static bvec2 operator&(bvec2 v1, bool s) {
            return new bvec2(v1.v0 & s, v1.v1 & s);
        }
        
        public static bvec2 operator|(bvec2 v1, bool s) {
            return new bvec2(v1.v0 | s, v1.v1 | s);
        }
        
        public static bvec2 operator^(bvec2 v1, bool s) {
            return new bvec2(v1.v0 ^ s, v1.v1 ^ s);
        }
        
        public static bvec2 operator&(bvec2 v1, bvec2 v2) {
            return new bvec2(v1.v0 & v2.v0, v1.v1 & v2.v1);
        }
        
        public static bvec2 operator|(bvec2 v1, bvec2 v2) {
            return new bvec2(v1.v0 | v2.v0, v1.v1 | v2.v1);
        }
        
        public static bvec2 operator^(bvec2 v1, bvec2 v2) {
            return new bvec2(v1.v0 ^ v2.v0, v1.v1 ^ v2.v1);
        }
        
        public static bvec2 operator==(bvec2 v1, bvec2 v2) {
            return new bvec2(v1.v0 == v2.v0, v1.v1 == v2.v1);
        }
        
        public static bvec2 operator!=(bvec2 v1, bvec2 v2) {
            return new bvec2(v1.v0 != v2.v0, v1.v1 != v2.v1);
        }
        
        public override bool Equals(object obj) {
            return (obj is bvec2) ? bmath.All(this == (bvec2)obj) : false;
        }
        
        public override int GetHashCode() {
            return this.v0.GetHashCode() ^ this.v1.GetHashCode();
        }
        
        public override string ToString() {
            return "bvec2(" + this.v0 + "," + this.v1 + ")";
        }
    }
    
    [DataContract()]
    [StructLayout(LayoutKind.Sequential)]
    public struct bvec3 {
        
        [DataMember()]
        internal bool v0;
        
        [DataMember()]
        internal bool v1;
        
        [DataMember()]
        internal bool v2;
        
        public bvec3(bool s) {
            this.v0 = s;
            this.v1 = s;
            this.v2 = s;
        }
        
        public bvec3(bvec3 v0) {
            this.v0 = v0.v0;
            this.v1 = v0.v1;
            this.v2 = v0.v2;
        }
        
        public bvec3(bvec2 v0, bool v1) {
            this.v0 = v0.v0;
            this.v1 = v0.v1;
            this.v2 = v1;
        }
        
        public bvec3(bool v0, bvec2 v1) {
            this.v0 = v0;
            this.v1 = v1.v0;
            this.v2 = v1.v1;
        }
        
        public bvec3(bool v0, bool v1, bool v2) {
            this.v0 = v0;
            this.v1 = v1;
            this.v2 = v2;
        }
        
        public bvec3(bvec2 s) {
            this.v0 = s.v0;
            this.v1 = s.v1;
            this.v2 = false;
        }
        
        public bvec3(bvec4 s) {
            this.v0 = s.v0;
            this.v1 = s.v1;
            this.v2 = s.v2;
        }
        
        public bool this[int i0] {
            get {
                if (((i0 < 0) 
                            || (2 < i0))) {
                    throw new System.IndexOutOfRangeException("i0");
                }
                
                unsafe
                {
                    fixed (Boolean* v_ptr = &this.v0)
                    {
                        return *(v_ptr + (i0));
                    }
                };
            }
            set {
                if (((i0 < 0) 
                            || (2 < i0))) {
                    throw new System.IndexOutOfRangeException("i0");
                }
                
                unsafe
                {
                    fixed (Boolean* v_ptr = &this.v0)
                    {
                        *(v_ptr  + (i0)) = value;
                    }
                };
            }
        }
        
        public bool x {
            get {
                return this.v0;
            }
            set {
                this.v0 = value;
            }
        }
        
        public bool r {
            get {
                return this.v0;
            }
            set {
                this.v0 = value;
            }
        }
        
        public bool s {
            get {
                return this.v0;
            }
            set {
                this.v0 = value;
            }
        }
        
        public bool y {
            get {
                return this.v1;
            }
            set {
                this.v1 = value;
            }
        }
        
        public bool g {
            get {
                return this.v1;
            }
            set {
                this.v1 = value;
            }
        }
        
        public bool t {
            get {
                return this.v1;
            }
            set {
                this.v1 = value;
            }
        }
        
        public bool z {
            get {
                return this.v2;
            }
            set {
                this.v2 = value;
            }
        }
        
        public bool b {
            get {
                return this.v2;
            }
            set {
                this.v2 = value;
            }
        }
        
        public bool p {
            get {
                return this.v2;
            }
            set {
                this.v2 = value;
            }
        }
        
        public bvec2 xx {
            get {
                return new bvec2(this.v0, this.v0);
            }
        }
        
        public bvec2 rr {
            get {
                return new bvec2(this.v0, this.v0);
            }
        }
        
        public bvec2 ss {
            get {
                return new bvec2(this.v0, this.v0);
            }
        }
        
        public bvec2 xy {
            get {
                return new bvec2(this.v0, this.v1);
            }
            set {
                this.v0 = value.v0;
                this.v1 = value.v1;
            }
        }
        
        public bvec2 rg {
            get {
                return new bvec2(this.v0, this.v1);
            }
            set {
                this.v0 = value.v0;
                this.v1 = value.v1;
            }
        }
        
        public bvec2 st {
            get {
                return new bvec2(this.v0, this.v1);
            }
            set {
                this.v0 = value.v0;
                this.v1 = value.v1;
            }
        }
        
        public bvec2 xz {
            get {
                return new bvec2(this.v0, this.v2);
            }
            set {
                this.v0 = value.v0;
                this.v2 = value.v1;
            }
        }
        
        public bvec2 rb {
            get {
                return new bvec2(this.v0, this.v2);
            }
            set {
                this.v0 = value.v0;
                this.v2 = value.v1;
            }
        }
        
        public bvec2 sp {
            get {
                return new bvec2(this.v0, this.v2);
            }
            set {
                this.v0 = value.v0;
                this.v2 = value.v1;
            }
        }
        
        public bvec2 yx {
            get {
                return new bvec2(this.v1, this.v0);
            }
            set {
                this.v1 = value.v0;
                this.v0 = value.v1;
            }
        }
        
        public bvec2 gr {
            get {
                return new bvec2(this.v1, this.v0);
            }
            set {
                this.v1 = value.v0;
                this.v0 = value.v1;
            }
        }
        
        public bvec2 ts {
            get {
                return new bvec2(this.v1, this.v0);
            }
            set {
                this.v1 = value.v0;
                this.v0 = value.v1;
            }
        }
        
        public bvec2 yy {
            get {
                return new bvec2(this.v1, this.v1);
            }
        }
        
        public bvec2 gg {
            get {
                return new bvec2(this.v1, this.v1);
            }
        }
        
        public bvec2 tt {
            get {
                return new bvec2(this.v1, this.v1);
            }
        }
        
        public bvec2 yz {
            get {
                return new bvec2(this.v1, this.v2);
            }
            set {
                this.v1 = value.v0;
                this.v2 = value.v1;
            }
        }
        
        public bvec2 gb {
            get {
                return new bvec2(this.v1, this.v2);
            }
            set {
                this.v1 = value.v0;
                this.v2 = value.v1;
            }
        }
        
        public bvec2 tp {
            get {
                return new bvec2(this.v1, this.v2);
            }
            set {
                this.v1 = value.v0;
                this.v2 = value.v1;
            }
        }
        
        public bvec2 zx {
            get {
                return new bvec2(this.v2, this.v0);
            }
            set {
                this.v2 = value.v0;
                this.v0 = value.v1;
            }
        }
        
        public bvec2 br {
            get {
                return new bvec2(this.v2, this.v0);
            }
            set {
                this.v2 = value.v0;
                this.v0 = value.v1;
            }
        }
        
        public bvec2 ps {
            get {
                return new bvec2(this.v2, this.v0);
            }
            set {
                this.v2 = value.v0;
                this.v0 = value.v1;
            }
        }
        
        public bvec2 zy {
            get {
                return new bvec2(this.v2, this.v1);
            }
            set {
                this.v2 = value.v0;
                this.v1 = value.v1;
            }
        }
        
        public bvec2 bg {
            get {
                return new bvec2(this.v2, this.v1);
            }
            set {
                this.v2 = value.v0;
                this.v1 = value.v1;
            }
        }
        
        public bvec2 pt {
            get {
                return new bvec2(this.v2, this.v1);
            }
            set {
                this.v2 = value.v0;
                this.v1 = value.v1;
            }
        }
        
        public bvec2 zz {
            get {
                return new bvec2(this.v2, this.v2);
            }
        }
        
        public bvec2 bb {
            get {
                return new bvec2(this.v2, this.v2);
            }
        }
        
        public bvec2 pp {
            get {
                return new bvec2(this.v2, this.v2);
            }
        }
        
        public bvec3 xxx {
            get {
                return new bvec3(this.v0, this.v0, this.v0);
            }
        }
        
        public bvec3 rrr {
            get {
                return new bvec3(this.v0, this.v0, this.v0);
            }
        }
        
        public bvec3 sss {
            get {
                return new bvec3(this.v0, this.v0, this.v0);
            }
        }
        
        public bvec3 xxy {
            get {
                return new bvec3(this.v0, this.v0, this.v1);
            }
        }
        
        public bvec3 rrg {
            get {
                return new bvec3(this.v0, this.v0, this.v1);
            }
        }
        
        public bvec3 sst {
            get {
                return new bvec3(this.v0, this.v0, this.v1);
            }
        }
        
        public bvec3 xxz {
            get {
                return new bvec3(this.v0, this.v0, this.v2);
            }
        }
        
        public bvec3 rrb {
            get {
                return new bvec3(this.v0, this.v0, this.v2);
            }
        }
        
        public bvec3 ssp {
            get {
                return new bvec3(this.v0, this.v0, this.v2);
            }
        }
        
        public bvec3 xyx {
            get {
                return new bvec3(this.v0, this.v1, this.v0);
            }
        }
        
        public bvec3 rgr {
            get {
                return new bvec3(this.v0, this.v1, this.v0);
            }
        }
        
        public bvec3 sts {
            get {
                return new bvec3(this.v0, this.v1, this.v0);
            }
        }
        
        public bvec3 xyy {
            get {
                return new bvec3(this.v0, this.v1, this.v1);
            }
        }
        
        public bvec3 rgg {
            get {
                return new bvec3(this.v0, this.v1, this.v1);
            }
        }
        
        public bvec3 stt {
            get {
                return new bvec3(this.v0, this.v1, this.v1);
            }
        }
        
        public bvec3 xyz {
            get {
                return new bvec3(this.v0, this.v1, this.v2);
            }
            set {
                this.v0 = value.v0;
                this.v1 = value.v1;
                this.v2 = value.v2;
            }
        }
        
        public bvec3 rgb {
            get {
                return new bvec3(this.v0, this.v1, this.v2);
            }
            set {
                this.v0 = value.v0;
                this.v1 = value.v1;
                this.v2 = value.v2;
            }
        }
        
        public bvec3 stp {
            get {
                return new bvec3(this.v0, this.v1, this.v2);
            }
            set {
                this.v0 = value.v0;
                this.v1 = value.v1;
                this.v2 = value.v2;
            }
        }
        
        public bvec3 xzx {
            get {
                return new bvec3(this.v0, this.v2, this.v0);
            }
        }
        
        public bvec3 rbr {
            get {
                return new bvec3(this.v0, this.v2, this.v0);
            }
        }
        
        public bvec3 sps {
            get {
                return new bvec3(this.v0, this.v2, this.v0);
            }
        }
        
        public bvec3 xzy {
            get {
                return new bvec3(this.v0, this.v2, this.v1);
            }
            set {
                this.v0 = value.v0;
                this.v2 = value.v1;
                this.v1 = value.v2;
            }
        }
        
        public bvec3 rbg {
            get {
                return new bvec3(this.v0, this.v2, this.v1);
            }
            set {
                this.v0 = value.v0;
                this.v2 = value.v1;
                this.v1 = value.v2;
            }
        }
        
        public bvec3 spt {
            get {
                return new bvec3(this.v0, this.v2, this.v1);
            }
            set {
                this.v0 = value.v0;
                this.v2 = value.v1;
                this.v1 = value.v2;
            }
        }
        
        public bvec3 xzz {
            get {
                return new bvec3(this.v0, this.v2, this.v2);
            }
        }
        
        public bvec3 rbb {
            get {
                return new bvec3(this.v0, this.v2, this.v2);
            }
        }
        
        public bvec3 spp {
            get {
                return new bvec3(this.v0, this.v2, this.v2);
            }
        }
        
        public bvec3 yxx {
            get {
                return new bvec3(this.v1, this.v0, this.v0);
            }
        }
        
        public bvec3 grr {
            get {
                return new bvec3(this.v1, this.v0, this.v0);
            }
        }
        
        public bvec3 tss {
            get {
                return new bvec3(this.v1, this.v0, this.v0);
            }
        }
        
        public bvec3 yxy {
            get {
                return new bvec3(this.v1, this.v0, this.v1);
            }
        }
        
        public bvec3 grg {
            get {
                return new bvec3(this.v1, this.v0, this.v1);
            }
        }
        
        public bvec3 tst {
            get {
                return new bvec3(this.v1, this.v0, this.v1);
            }
        }
        
        public bvec3 yxz {
            get {
                return new bvec3(this.v1, this.v0, this.v2);
            }
            set {
                this.v1 = value.v0;
                this.v0 = value.v1;
                this.v2 = value.v2;
            }
        }
        
        public bvec3 grb {
            get {
                return new bvec3(this.v1, this.v0, this.v2);
            }
            set {
                this.v1 = value.v0;
                this.v0 = value.v1;
                this.v2 = value.v2;
            }
        }
        
        public bvec3 tsp {
            get {
                return new bvec3(this.v1, this.v0, this.v2);
            }
            set {
                this.v1 = value.v0;
                this.v0 = value.v1;
                this.v2 = value.v2;
            }
        }
        
        public bvec3 yyx {
            get {
                return new bvec3(this.v1, this.v1, this.v0);
            }
        }
        
        public bvec3 ggr {
            get {
                return new bvec3(this.v1, this.v1, this.v0);
            }
        }
        
        public bvec3 tts {
            get {
                return new bvec3(this.v1, this.v1, this.v0);
            }
        }
        
        public bvec3 yyy {
            get {
                return new bvec3(this.v1, this.v1, this.v1);
            }
        }
        
        public bvec3 ggg {
            get {
                return new bvec3(this.v1, this.v1, this.v1);
            }
        }
        
        public bvec3 ttt {
            get {
                return new bvec3(this.v1, this.v1, this.v1);
            }
        }
        
        public bvec3 yyz {
            get {
                return new bvec3(this.v1, this.v1, this.v2);
            }
        }
        
        public bvec3 ggb {
            get {
                return new bvec3(this.v1, this.v1, this.v2);
            }
        }
        
        public bvec3 ttp {
            get {
                return new bvec3(this.v1, this.v1, this.v2);
            }
        }
        
        public bvec3 yzx {
            get {
                return new bvec3(this.v1, this.v2, this.v0);
            }
            set {
                this.v1 = value.v0;
                this.v2 = value.v1;
                this.v0 = value.v2;
            }
        }
        
        public bvec3 gbr {
            get {
                return new bvec3(this.v1, this.v2, this.v0);
            }
            set {
                this.v1 = value.v0;
                this.v2 = value.v1;
                this.v0 = value.v2;
            }
        }
        
        public bvec3 tps {
            get {
                return new bvec3(this.v1, this.v2, this.v0);
            }
            set {
                this.v1 = value.v0;
                this.v2 = value.v1;
                this.v0 = value.v2;
            }
        }
        
        public bvec3 yzy {
            get {
                return new bvec3(this.v1, this.v2, this.v1);
            }
        }
        
        public bvec3 gbg {
            get {
                return new bvec3(this.v1, this.v2, this.v1);
            }
        }
        
        public bvec3 tpt {
            get {
                return new bvec3(this.v1, this.v2, this.v1);
            }
        }
        
        public bvec3 yzz {
            get {
                return new bvec3(this.v1, this.v2, this.v2);
            }
        }
        
        public bvec3 gbb {
            get {
                return new bvec3(this.v1, this.v2, this.v2);
            }
        }
        
        public bvec3 tpp {
            get {
                return new bvec3(this.v1, this.v2, this.v2);
            }
        }
        
        public bvec3 zxx {
            get {
                return new bvec3(this.v2, this.v0, this.v0);
            }
        }
        
        public bvec3 brr {
            get {
                return new bvec3(this.v2, this.v0, this.v0);
            }
        }
        
        public bvec3 pss {
            get {
                return new bvec3(this.v2, this.v0, this.v0);
            }
        }
        
        public bvec3 zxy {
            get {
                return new bvec3(this.v2, this.v0, this.v1);
            }
            set {
                this.v2 = value.v0;
                this.v0 = value.v1;
                this.v1 = value.v2;
            }
        }
        
        public bvec3 brg {
            get {
                return new bvec3(this.v2, this.v0, this.v1);
            }
            set {
                this.v2 = value.v0;
                this.v0 = value.v1;
                this.v1 = value.v2;
            }
        }
        
        public bvec3 pst {
            get {
                return new bvec3(this.v2, this.v0, this.v1);
            }
            set {
                this.v2 = value.v0;
                this.v0 = value.v1;
                this.v1 = value.v2;
            }
        }
        
        public bvec3 zxz {
            get {
                return new bvec3(this.v2, this.v0, this.v2);
            }
        }
        
        public bvec3 brb {
            get {
                return new bvec3(this.v2, this.v0, this.v2);
            }
        }
        
        public bvec3 psp {
            get {
                return new bvec3(this.v2, this.v0, this.v2);
            }
        }
        
        public bvec3 zyx {
            get {
                return new bvec3(this.v2, this.v1, this.v0);
            }
            set {
                this.v2 = value.v0;
                this.v1 = value.v1;
                this.v0 = value.v2;
            }
        }
        
        public bvec3 bgr {
            get {
                return new bvec3(this.v2, this.v1, this.v0);
            }
            set {
                this.v2 = value.v0;
                this.v1 = value.v1;
                this.v0 = value.v2;
            }
        }
        
        public bvec3 pts {
            get {
                return new bvec3(this.v2, this.v1, this.v0);
            }
            set {
                this.v2 = value.v0;
                this.v1 = value.v1;
                this.v0 = value.v2;
            }
        }
        
        public bvec3 zyy {
            get {
                return new bvec3(this.v2, this.v1, this.v1);
            }
        }
        
        public bvec3 bgg {
            get {
                return new bvec3(this.v2, this.v1, this.v1);
            }
        }
        
        public bvec3 ptt {
            get {
                return new bvec3(this.v2, this.v1, this.v1);
            }
        }
        
        public bvec3 zyz {
            get {
                return new bvec3(this.v2, this.v1, this.v2);
            }
        }
        
        public bvec3 bgb {
            get {
                return new bvec3(this.v2, this.v1, this.v2);
            }
        }
        
        public bvec3 ptp {
            get {
                return new bvec3(this.v2, this.v1, this.v2);
            }
        }
        
        public bvec3 zzx {
            get {
                return new bvec3(this.v2, this.v2, this.v0);
            }
        }
        
        public bvec3 bbr {
            get {
                return new bvec3(this.v2, this.v2, this.v0);
            }
        }
        
        public bvec3 pps {
            get {
                return new bvec3(this.v2, this.v2, this.v0);
            }
        }
        
        public bvec3 zzy {
            get {
                return new bvec3(this.v2, this.v2, this.v1);
            }
        }
        
        public bvec3 bbg {
            get {
                return new bvec3(this.v2, this.v2, this.v1);
            }
        }
        
        public bvec3 ppt {
            get {
                return new bvec3(this.v2, this.v2, this.v1);
            }
        }
        
        public bvec3 zzz {
            get {
                return new bvec3(this.v2, this.v2, this.v2);
            }
        }
        
        public bvec3 bbb {
            get {
                return new bvec3(this.v2, this.v2, this.v2);
            }
        }
        
        public bvec3 ppp {
            get {
                return new bvec3(this.v2, this.v2, this.v2);
            }
        }
        
        public static bvec3 operator!(bvec3 v) {
            return new bvec3(!v.v0, !v.v1, !v.v2);
        }
        
        public static bvec3 operator&(bool s, bvec3 v2) {
            return new bvec3(s & v2.v0, s & v2.v1, s & v2.v2);
        }
        
        public static bvec3 operator|(bool s, bvec3 v2) {
            return new bvec3(s | v2.v0, s | v2.v1, s | v2.v2);
        }
        
        public static bvec3 operator^(bool s, bvec3 v2) {
            return new bvec3(s ^ v2.v0, s ^ v2.v1, s ^ v2.v2);
        }
        
        public static bvec3 operator&(bvec3 v1, bool s) {
            return new bvec3(v1.v0 & s, v1.v1 & s, v1.v2 & s);
        }
        
        public static bvec3 operator|(bvec3 v1, bool s) {
            return new bvec3(v1.v0 | s, v1.v1 | s, v1.v2 | s);
        }
        
        public static bvec3 operator^(bvec3 v1, bool s) {
            return new bvec3(v1.v0 ^ s, v1.v1 ^ s, v1.v2 ^ s);
        }
        
        public static bvec3 operator&(bvec3 v1, bvec3 v2) {
            return new bvec3(v1.v0 & v2.v0, v1.v1 & v2.v1, v1.v2 & v2.v2);
        }
        
        public static bvec3 operator|(bvec3 v1, bvec3 v2) {
            return new bvec3(v1.v0 | v2.v0, v1.v1 | v2.v1, v1.v2 | v2.v2);
        }
        
        public static bvec3 operator^(bvec3 v1, bvec3 v2) {
            return new bvec3(v1.v0 ^ v2.v0, v1.v1 ^ v2.v1, v1.v2 ^ v2.v2);
        }
        
        public static bvec3 operator==(bvec3 v1, bvec3 v2) {
            return new bvec3(v1.v0 == v2.v0, v1.v1 == v2.v1, v1.v2 == v2.v2);
        }
        
        public static bvec3 operator!=(bvec3 v1, bvec3 v2) {
            return new bvec3(v1.v0 != v2.v0, v1.v1 != v2.v1, v1.v2 != v2.v2);
        }
        
        public override bool Equals(object obj) {
            return (obj is bvec3) ? bmath.All(this == (bvec3)obj) : false;
        }
        
        public override int GetHashCode() {
            return this.v0.GetHashCode() ^ this.v1.GetHashCode() ^ this.v2.GetHashCode();
        }
        
        public override string ToString() {
            return "bvec3(" + this.v0 + "," + this.v1 + "," + this.v2 + ")";
        }
    }
    
    [DataContract()]
    [StructLayout(LayoutKind.Sequential)]
    public struct bvec4 {
        
        [DataMember()]
        internal bool v0;
        
        [DataMember()]
        internal bool v1;
        
        [DataMember()]
        internal bool v2;
        
        [DataMember()]
        internal bool v3;
        
        public bvec4(bool s) {
            this.v0 = s;
            this.v1 = s;
            this.v2 = s;
            this.v3 = s;
        }
        
        public bvec4(bvec4 v0) {
            this.v0 = v0.v0;
            this.v1 = v0.v1;
            this.v2 = v0.v2;
            this.v3 = v0.v3;
        }
        
        public bvec4(bvec2 v0, bvec2 v1) {
            this.v0 = v0.v0;
            this.v1 = v0.v1;
            this.v2 = v1.v0;
            this.v3 = v1.v1;
        }
        
        public bvec4(bvec3 v0, bool v1) {
            this.v0 = v0.v0;
            this.v1 = v0.v1;
            this.v2 = v0.v2;
            this.v3 = v1;
        }
        
        public bvec4(bool v0, bvec3 v1) {
            this.v0 = v0;
            this.v1 = v1.v0;
            this.v2 = v1.v1;
            this.v3 = v1.v2;
        }
        
        public bvec4(bvec2 v0, bool v1, bool v2) {
            this.v0 = v0.v0;
            this.v1 = v0.v1;
            this.v2 = v1;
            this.v3 = v2;
        }
        
        public bvec4(bool v0, bvec2 v1, bool v2) {
            this.v0 = v0;
            this.v1 = v1.v0;
            this.v2 = v1.v1;
            this.v3 = v2;
        }
        
        public bvec4(bool v0, bool v1, bvec2 v2) {
            this.v0 = v0;
            this.v1 = v1;
            this.v2 = v2.v0;
            this.v3 = v2.v1;
        }
        
        public bvec4(bool v0, bool v1, bool v2, bool v3) {
            this.v0 = v0;
            this.v1 = v1;
            this.v2 = v2;
            this.v3 = v3;
        }
        
        public bvec4(bvec2 s) {
            this.v0 = s.v0;
            this.v1 = s.v1;
            this.v2 = false;
            this.v3 = false;
        }
        
        public bvec4(bvec3 s) {
            this.v0 = s.v0;
            this.v1 = s.v1;
            this.v2 = s.v2;
            this.v3 = false;
        }
        
        public bool this[int i0] {
            get {
                if (((i0 < 0) 
                            || (3 < i0))) {
                    throw new System.IndexOutOfRangeException("i0");
                }
                
                unsafe
                {
                    fixed (Boolean* v_ptr = &this.v0)
                    {
                        return *(v_ptr + (i0));
                    }
                };
            }
            set {
                if (((i0 < 0) 
                            || (3 < i0))) {
                    throw new System.IndexOutOfRangeException("i0");
                }
                
                unsafe
                {
                    fixed (Boolean* v_ptr = &this.v0)
                    {
                        *(v_ptr  + (i0)) = value;
                    }
                };
            }
        }
        
        public bool x {
            get {
                return this.v0;
            }
            set {
                this.v0 = value;
            }
        }
        
        public bool r {
            get {
                return this.v0;
            }
            set {
                this.v0 = value;
            }
        }
        
        public bool s {
            get {
                return this.v0;
            }
            set {
                this.v0 = value;
            }
        }
        
        public bool y {
            get {
                return this.v1;
            }
            set {
                this.v1 = value;
            }
        }
        
        public bool g {
            get {
                return this.v1;
            }
            set {
                this.v1 = value;
            }
        }
        
        public bool t {
            get {
                return this.v1;
            }
            set {
                this.v1 = value;
            }
        }
        
        public bool z {
            get {
                return this.v2;
            }
            set {
                this.v2 = value;
            }
        }
        
        public bool b {
            get {
                return this.v2;
            }
            set {
                this.v2 = value;
            }
        }
        
        public bool p {
            get {
                return this.v2;
            }
            set {
                this.v2 = value;
            }
        }
        
        public bool w {
            get {
                return this.v3;
            }
            set {
                this.v3 = value;
            }
        }
        
        public bool a {
            get {
                return this.v3;
            }
            set {
                this.v3 = value;
            }
        }
        
        public bool q {
            get {
                return this.v3;
            }
            set {
                this.v3 = value;
            }
        }
        
        public bvec2 xx {
            get {
                return new bvec2(this.v0, this.v0);
            }
        }
        
        public bvec2 rr {
            get {
                return new bvec2(this.v0, this.v0);
            }
        }
        
        public bvec2 ss {
            get {
                return new bvec2(this.v0, this.v0);
            }
        }
        
        public bvec2 xy {
            get {
                return new bvec2(this.v0, this.v1);
            }
            set {
                this.v0 = value.v0;
                this.v1 = value.v1;
            }
        }
        
        public bvec2 rg {
            get {
                return new bvec2(this.v0, this.v1);
            }
            set {
                this.v0 = value.v0;
                this.v1 = value.v1;
            }
        }
        
        public bvec2 st {
            get {
                return new bvec2(this.v0, this.v1);
            }
            set {
                this.v0 = value.v0;
                this.v1 = value.v1;
            }
        }
        
        public bvec2 xz {
            get {
                return new bvec2(this.v0, this.v2);
            }
            set {
                this.v0 = value.v0;
                this.v2 = value.v1;
            }
        }
        
        public bvec2 rb {
            get {
                return new bvec2(this.v0, this.v2);
            }
            set {
                this.v0 = value.v0;
                this.v2 = value.v1;
            }
        }
        
        public bvec2 sp {
            get {
                return new bvec2(this.v0, this.v2);
            }
            set {
                this.v0 = value.v0;
                this.v2 = value.v1;
            }
        }
        
        public bvec2 xw {
            get {
                return new bvec2(this.v0, this.v3);
            }
            set {
                this.v0 = value.v0;
                this.v3 = value.v1;
            }
        }
        
        public bvec2 ra {
            get {
                return new bvec2(this.v0, this.v3);
            }
            set {
                this.v0 = value.v0;
                this.v3 = value.v1;
            }
        }
        
        public bvec2 sq {
            get {
                return new bvec2(this.v0, this.v3);
            }
            set {
                this.v0 = value.v0;
                this.v3 = value.v1;
            }
        }
        
        public bvec2 yx {
            get {
                return new bvec2(this.v1, this.v0);
            }
            set {
                this.v1 = value.v0;
                this.v0 = value.v1;
            }
        }
        
        public bvec2 gr {
            get {
                return new bvec2(this.v1, this.v0);
            }
            set {
                this.v1 = value.v0;
                this.v0 = value.v1;
            }
        }
        
        public bvec2 ts {
            get {
                return new bvec2(this.v1, this.v0);
            }
            set {
                this.v1 = value.v0;
                this.v0 = value.v1;
            }
        }
        
        public bvec2 yy {
            get {
                return new bvec2(this.v1, this.v1);
            }
        }
        
        public bvec2 gg {
            get {
                return new bvec2(this.v1, this.v1);
            }
        }
        
        public bvec2 tt {
            get {
                return new bvec2(this.v1, this.v1);
            }
        }
        
        public bvec2 yz {
            get {
                return new bvec2(this.v1, this.v2);
            }
            set {
                this.v1 = value.v0;
                this.v2 = value.v1;
            }
        }
        
        public bvec2 gb {
            get {
                return new bvec2(this.v1, this.v2);
            }
            set {
                this.v1 = value.v0;
                this.v2 = value.v1;
            }
        }
        
        public bvec2 tp {
            get {
                return new bvec2(this.v1, this.v2);
            }
            set {
                this.v1 = value.v0;
                this.v2 = value.v1;
            }
        }
        
        public bvec2 yw {
            get {
                return new bvec2(this.v1, this.v3);
            }
            set {
                this.v1 = value.v0;
                this.v3 = value.v1;
            }
        }
        
        public bvec2 ga {
            get {
                return new bvec2(this.v1, this.v3);
            }
            set {
                this.v1 = value.v0;
                this.v3 = value.v1;
            }
        }
        
        public bvec2 tq {
            get {
                return new bvec2(this.v1, this.v3);
            }
            set {
                this.v1 = value.v0;
                this.v3 = value.v1;
            }
        }
        
        public bvec2 zx {
            get {
                return new bvec2(this.v2, this.v0);
            }
            set {
                this.v2 = value.v0;
                this.v0 = value.v1;
            }
        }
        
        public bvec2 br {
            get {
                return new bvec2(this.v2, this.v0);
            }
            set {
                this.v2 = value.v0;
                this.v0 = value.v1;
            }
        }
        
        public bvec2 ps {
            get {
                return new bvec2(this.v2, this.v0);
            }
            set {
                this.v2 = value.v0;
                this.v0 = value.v1;
            }
        }
        
        public bvec2 zy {
            get {
                return new bvec2(this.v2, this.v1);
            }
            set {
                this.v2 = value.v0;
                this.v1 = value.v1;
            }
        }
        
        public bvec2 bg {
            get {
                return new bvec2(this.v2, this.v1);
            }
            set {
                this.v2 = value.v0;
                this.v1 = value.v1;
            }
        }
        
        public bvec2 pt {
            get {
                return new bvec2(this.v2, this.v1);
            }
            set {
                this.v2 = value.v0;
                this.v1 = value.v1;
            }
        }
        
        public bvec2 zz {
            get {
                return new bvec2(this.v2, this.v2);
            }
        }
        
        public bvec2 bb {
            get {
                return new bvec2(this.v2, this.v2);
            }
        }
        
        public bvec2 pp {
            get {
                return new bvec2(this.v2, this.v2);
            }
        }
        
        public bvec2 zw {
            get {
                return new bvec2(this.v2, this.v3);
            }
            set {
                this.v2 = value.v0;
                this.v3 = value.v1;
            }
        }
        
        public bvec2 ba {
            get {
                return new bvec2(this.v2, this.v3);
            }
            set {
                this.v2 = value.v0;
                this.v3 = value.v1;
            }
        }
        
        public bvec2 pq {
            get {
                return new bvec2(this.v2, this.v3);
            }
            set {
                this.v2 = value.v0;
                this.v3 = value.v1;
            }
        }
        
        public bvec2 wx {
            get {
                return new bvec2(this.v3, this.v0);
            }
            set {
                this.v3 = value.v0;
                this.v0 = value.v1;
            }
        }
        
        public bvec2 ar {
            get {
                return new bvec2(this.v3, this.v0);
            }
            set {
                this.v3 = value.v0;
                this.v0 = value.v1;
            }
        }
        
        public bvec2 qs {
            get {
                return new bvec2(this.v3, this.v0);
            }
            set {
                this.v3 = value.v0;
                this.v0 = value.v1;
            }
        }
        
        public bvec2 wy {
            get {
                return new bvec2(this.v3, this.v1);
            }
            set {
                this.v3 = value.v0;
                this.v1 = value.v1;
            }
        }
        
        public bvec2 ag {
            get {
                return new bvec2(this.v3, this.v1);
            }
            set {
                this.v3 = value.v0;
                this.v1 = value.v1;
            }
        }
        
        public bvec2 qt {
            get {
                return new bvec2(this.v3, this.v1);
            }
            set {
                this.v3 = value.v0;
                this.v1 = value.v1;
            }
        }
        
        public bvec2 wz {
            get {
                return new bvec2(this.v3, this.v2);
            }
            set {
                this.v3 = value.v0;
                this.v2 = value.v1;
            }
        }
        
        public bvec2 ab {
            get {
                return new bvec2(this.v3, this.v2);
            }
            set {
                this.v3 = value.v0;
                this.v2 = value.v1;
            }
        }
        
        public bvec2 qp {
            get {
                return new bvec2(this.v3, this.v2);
            }
            set {
                this.v3 = value.v0;
                this.v2 = value.v1;
            }
        }
        
        public bvec2 ww {
            get {
                return new bvec2(this.v3, this.v3);
            }
        }
        
        public bvec2 aa {
            get {
                return new bvec2(this.v3, this.v3);
            }
        }
        
        public bvec2 qq {
            get {
                return new bvec2(this.v3, this.v3);
            }
        }
        
        public bvec3 xxx {
            get {
                return new bvec3(this.v0, this.v0, this.v0);
            }
        }
        
        public bvec3 rrr {
            get {
                return new bvec3(this.v0, this.v0, this.v0);
            }
        }
        
        public bvec3 sss {
            get {
                return new bvec3(this.v0, this.v0, this.v0);
            }
        }
        
        public bvec3 xxy {
            get {
                return new bvec3(this.v0, this.v0, this.v1);
            }
        }
        
        public bvec3 rrg {
            get {
                return new bvec3(this.v0, this.v0, this.v1);
            }
        }
        
        public bvec3 sst {
            get {
                return new bvec3(this.v0, this.v0, this.v1);
            }
        }
        
        public bvec3 xxz {
            get {
                return new bvec3(this.v0, this.v0, this.v2);
            }
        }
        
        public bvec3 rrb {
            get {
                return new bvec3(this.v0, this.v0, this.v2);
            }
        }
        
        public bvec3 ssp {
            get {
                return new bvec3(this.v0, this.v0, this.v2);
            }
        }
        
        public bvec3 xxw {
            get {
                return new bvec3(this.v0, this.v0, this.v3);
            }
        }
        
        public bvec3 rra {
            get {
                return new bvec3(this.v0, this.v0, this.v3);
            }
        }
        
        public bvec3 ssq {
            get {
                return new bvec3(this.v0, this.v0, this.v3);
            }
        }
        
        public bvec3 xyx {
            get {
                return new bvec3(this.v0, this.v1, this.v0);
            }
        }
        
        public bvec3 rgr {
            get {
                return new bvec3(this.v0, this.v1, this.v0);
            }
        }
        
        public bvec3 sts {
            get {
                return new bvec3(this.v0, this.v1, this.v0);
            }
        }
        
        public bvec3 xyy {
            get {
                return new bvec3(this.v0, this.v1, this.v1);
            }
        }
        
        public bvec3 rgg {
            get {
                return new bvec3(this.v0, this.v1, this.v1);
            }
        }
        
        public bvec3 stt {
            get {
                return new bvec3(this.v0, this.v1, this.v1);
            }
        }
        
        public bvec3 xyz {
            get {
                return new bvec3(this.v0, this.v1, this.v2);
            }
            set {
                this.v0 = value.v0;
                this.v1 = value.v1;
                this.v2 = value.v2;
            }
        }
        
        public bvec3 rgb {
            get {
                return new bvec3(this.v0, this.v1, this.v2);
            }
            set {
                this.v0 = value.v0;
                this.v1 = value.v1;
                this.v2 = value.v2;
            }
        }
        
        public bvec3 stp {
            get {
                return new bvec3(this.v0, this.v1, this.v2);
            }
            set {
                this.v0 = value.v0;
                this.v1 = value.v1;
                this.v2 = value.v2;
            }
        }
        
        public bvec3 xyw {
            get {
                return new bvec3(this.v0, this.v1, this.v3);
            }
            set {
                this.v0 = value.v0;
                this.v1 = value.v1;
                this.v3 = value.v2;
            }
        }
        
        public bvec3 rga {
            get {
                return new bvec3(this.v0, this.v1, this.v3);
            }
            set {
                this.v0 = value.v0;
                this.v1 = value.v1;
                this.v3 = value.v2;
            }
        }
        
        public bvec3 stq {
            get {
                return new bvec3(this.v0, this.v1, this.v3);
            }
            set {
                this.v0 = value.v0;
                this.v1 = value.v1;
                this.v3 = value.v2;
            }
        }
        
        public bvec3 xzx {
            get {
                return new bvec3(this.v0, this.v2, this.v0);
            }
        }
        
        public bvec3 rbr {
            get {
                return new bvec3(this.v0, this.v2, this.v0);
            }
        }
        
        public bvec3 sps {
            get {
                return new bvec3(this.v0, this.v2, this.v0);
            }
        }
        
        public bvec3 xzy {
            get {
                return new bvec3(this.v0, this.v2, this.v1);
            }
            set {
                this.v0 = value.v0;
                this.v2 = value.v1;
                this.v1 = value.v2;
            }
        }
        
        public bvec3 rbg {
            get {
                return new bvec3(this.v0, this.v2, this.v1);
            }
            set {
                this.v0 = value.v0;
                this.v2 = value.v1;
                this.v1 = value.v2;
            }
        }
        
        public bvec3 spt {
            get {
                return new bvec3(this.v0, this.v2, this.v1);
            }
            set {
                this.v0 = value.v0;
                this.v2 = value.v1;
                this.v1 = value.v2;
            }
        }
        
        public bvec3 xzz {
            get {
                return new bvec3(this.v0, this.v2, this.v2);
            }
        }
        
        public bvec3 rbb {
            get {
                return new bvec3(this.v0, this.v2, this.v2);
            }
        }
        
        public bvec3 spp {
            get {
                return new bvec3(this.v0, this.v2, this.v2);
            }
        }
        
        public bvec3 xzw {
            get {
                return new bvec3(this.v0, this.v2, this.v3);
            }
            set {
                this.v0 = value.v0;
                this.v2 = value.v1;
                this.v3 = value.v2;
            }
        }
        
        public bvec3 rba {
            get {
                return new bvec3(this.v0, this.v2, this.v3);
            }
            set {
                this.v0 = value.v0;
                this.v2 = value.v1;
                this.v3 = value.v2;
            }
        }
        
        public bvec3 spq {
            get {
                return new bvec3(this.v0, this.v2, this.v3);
            }
            set {
                this.v0 = value.v0;
                this.v2 = value.v1;
                this.v3 = value.v2;
            }
        }
        
        public bvec3 xwx {
            get {
                return new bvec3(this.v0, this.v3, this.v0);
            }
        }
        
        public bvec3 rar {
            get {
                return new bvec3(this.v0, this.v3, this.v0);
            }
        }
        
        public bvec3 sqs {
            get {
                return new bvec3(this.v0, this.v3, this.v0);
            }
        }
        
        public bvec3 xwy {
            get {
                return new bvec3(this.v0, this.v3, this.v1);
            }
            set {
                this.v0 = value.v0;
                this.v3 = value.v1;
                this.v1 = value.v2;
            }
        }
        
        public bvec3 rag {
            get {
                return new bvec3(this.v0, this.v3, this.v1);
            }
            set {
                this.v0 = value.v0;
                this.v3 = value.v1;
                this.v1 = value.v2;
            }
        }
        
        public bvec3 sqt {
            get {
                return new bvec3(this.v0, this.v3, this.v1);
            }
            set {
                this.v0 = value.v0;
                this.v3 = value.v1;
                this.v1 = value.v2;
            }
        }
        
        public bvec3 xwz {
            get {
                return new bvec3(this.v0, this.v3, this.v2);
            }
            set {
                this.v0 = value.v0;
                this.v3 = value.v1;
                this.v2 = value.v2;
            }
        }
        
        public bvec3 rab {
            get {
                return new bvec3(this.v0, this.v3, this.v2);
            }
            set {
                this.v0 = value.v0;
                this.v3 = value.v1;
                this.v2 = value.v2;
            }
        }
        
        public bvec3 sqp {
            get {
                return new bvec3(this.v0, this.v3, this.v2);
            }
            set {
                this.v0 = value.v0;
                this.v3 = value.v1;
                this.v2 = value.v2;
            }
        }
        
        public bvec3 xww {
            get {
                return new bvec3(this.v0, this.v3, this.v3);
            }
        }
        
        public bvec3 raa {
            get {
                return new bvec3(this.v0, this.v3, this.v3);
            }
        }
        
        public bvec3 sqq {
            get {
                return new bvec3(this.v0, this.v3, this.v3);
            }
        }
        
        public bvec3 yxx {
            get {
                return new bvec3(this.v1, this.v0, this.v0);
            }
        }
        
        public bvec3 grr {
            get {
                return new bvec3(this.v1, this.v0, this.v0);
            }
        }
        
        public bvec3 tss {
            get {
                return new bvec3(this.v1, this.v0, this.v0);
            }
        }
        
        public bvec3 yxy {
            get {
                return new bvec3(this.v1, this.v0, this.v1);
            }
        }
        
        public bvec3 grg {
            get {
                return new bvec3(this.v1, this.v0, this.v1);
            }
        }
        
        public bvec3 tst {
            get {
                return new bvec3(this.v1, this.v0, this.v1);
            }
        }
        
        public bvec3 yxz {
            get {
                return new bvec3(this.v1, this.v0, this.v2);
            }
            set {
                this.v1 = value.v0;
                this.v0 = value.v1;
                this.v2 = value.v2;
            }
        }
        
        public bvec3 grb {
            get {
                return new bvec3(this.v1, this.v0, this.v2);
            }
            set {
                this.v1 = value.v0;
                this.v0 = value.v1;
                this.v2 = value.v2;
            }
        }
        
        public bvec3 tsp {
            get {
                return new bvec3(this.v1, this.v0, this.v2);
            }
            set {
                this.v1 = value.v0;
                this.v0 = value.v1;
                this.v2 = value.v2;
            }
        }
        
        public bvec3 yxw {
            get {
                return new bvec3(this.v1, this.v0, this.v3);
            }
            set {
                this.v1 = value.v0;
                this.v0 = value.v1;
                this.v3 = value.v2;
            }
        }
        
        public bvec3 gra {
            get {
                return new bvec3(this.v1, this.v0, this.v3);
            }
            set {
                this.v1 = value.v0;
                this.v0 = value.v1;
                this.v3 = value.v2;
            }
        }
        
        public bvec3 tsq {
            get {
                return new bvec3(this.v1, this.v0, this.v3);
            }
            set {
                this.v1 = value.v0;
                this.v0 = value.v1;
                this.v3 = value.v2;
            }
        }
        
        public bvec3 yyx {
            get {
                return new bvec3(this.v1, this.v1, this.v0);
            }
        }
        
        public bvec3 ggr {
            get {
                return new bvec3(this.v1, this.v1, this.v0);
            }
        }
        
        public bvec3 tts {
            get {
                return new bvec3(this.v1, this.v1, this.v0);
            }
        }
        
        public bvec3 yyy {
            get {
                return new bvec3(this.v1, this.v1, this.v1);
            }
        }
        
        public bvec3 ggg {
            get {
                return new bvec3(this.v1, this.v1, this.v1);
            }
        }
        
        public bvec3 ttt {
            get {
                return new bvec3(this.v1, this.v1, this.v1);
            }
        }
        
        public bvec3 yyz {
            get {
                return new bvec3(this.v1, this.v1, this.v2);
            }
        }
        
        public bvec3 ggb {
            get {
                return new bvec3(this.v1, this.v1, this.v2);
            }
        }
        
        public bvec3 ttp {
            get {
                return new bvec3(this.v1, this.v1, this.v2);
            }
        }
        
        public bvec3 yyw {
            get {
                return new bvec3(this.v1, this.v1, this.v3);
            }
        }
        
        public bvec3 gga {
            get {
                return new bvec3(this.v1, this.v1, this.v3);
            }
        }
        
        public bvec3 ttq {
            get {
                return new bvec3(this.v1, this.v1, this.v3);
            }
        }
        
        public bvec3 yzx {
            get {
                return new bvec3(this.v1, this.v2, this.v0);
            }
            set {
                this.v1 = value.v0;
                this.v2 = value.v1;
                this.v0 = value.v2;
            }
        }
        
        public bvec3 gbr {
            get {
                return new bvec3(this.v1, this.v2, this.v0);
            }
            set {
                this.v1 = value.v0;
                this.v2 = value.v1;
                this.v0 = value.v2;
            }
        }
        
        public bvec3 tps {
            get {
                return new bvec3(this.v1, this.v2, this.v0);
            }
            set {
                this.v1 = value.v0;
                this.v2 = value.v1;
                this.v0 = value.v2;
            }
        }
        
        public bvec3 yzy {
            get {
                return new bvec3(this.v1, this.v2, this.v1);
            }
        }
        
        public bvec3 gbg {
            get {
                return new bvec3(this.v1, this.v2, this.v1);
            }
        }
        
        public bvec3 tpt {
            get {
                return new bvec3(this.v1, this.v2, this.v1);
            }
        }
        
        public bvec3 yzz {
            get {
                return new bvec3(this.v1, this.v2, this.v2);
            }
        }
        
        public bvec3 gbb {
            get {
                return new bvec3(this.v1, this.v2, this.v2);
            }
        }
        
        public bvec3 tpp {
            get {
                return new bvec3(this.v1, this.v2, this.v2);
            }
        }
        
        public bvec3 yzw {
            get {
                return new bvec3(this.v1, this.v2, this.v3);
            }
            set {
                this.v1 = value.v0;
                this.v2 = value.v1;
                this.v3 = value.v2;
            }
        }
        
        public bvec3 gba {
            get {
                return new bvec3(this.v1, this.v2, this.v3);
            }
            set {
                this.v1 = value.v0;
                this.v2 = value.v1;
                this.v3 = value.v2;
            }
        }
        
        public bvec3 tpq {
            get {
                return new bvec3(this.v1, this.v2, this.v3);
            }
            set {
                this.v1 = value.v0;
                this.v2 = value.v1;
                this.v3 = value.v2;
            }
        }
        
        public bvec3 ywx {
            get {
                return new bvec3(this.v1, this.v3, this.v0);
            }
            set {
                this.v1 = value.v0;
                this.v3 = value.v1;
                this.v0 = value.v2;
            }
        }
        
        public bvec3 gar {
            get {
                return new bvec3(this.v1, this.v3, this.v0);
            }
            set {
                this.v1 = value.v0;
                this.v3 = value.v1;
                this.v0 = value.v2;
            }
        }
        
        public bvec3 tqs {
            get {
                return new bvec3(this.v1, this.v3, this.v0);
            }
            set {
                this.v1 = value.v0;
                this.v3 = value.v1;
                this.v0 = value.v2;
            }
        }
        
        public bvec3 ywy {
            get {
                return new bvec3(this.v1, this.v3, this.v1);
            }
        }
        
        public bvec3 gag {
            get {
                return new bvec3(this.v1, this.v3, this.v1);
            }
        }
        
        public bvec3 tqt {
            get {
                return new bvec3(this.v1, this.v3, this.v1);
            }
        }
        
        public bvec3 ywz {
            get {
                return new bvec3(this.v1, this.v3, this.v2);
            }
            set {
                this.v1 = value.v0;
                this.v3 = value.v1;
                this.v2 = value.v2;
            }
        }
        
        public bvec3 gab {
            get {
                return new bvec3(this.v1, this.v3, this.v2);
            }
            set {
                this.v1 = value.v0;
                this.v3 = value.v1;
                this.v2 = value.v2;
            }
        }
        
        public bvec3 tqp {
            get {
                return new bvec3(this.v1, this.v3, this.v2);
            }
            set {
                this.v1 = value.v0;
                this.v3 = value.v1;
                this.v2 = value.v2;
            }
        }
        
        public bvec3 yww {
            get {
                return new bvec3(this.v1, this.v3, this.v3);
            }
        }
        
        public bvec3 gaa {
            get {
                return new bvec3(this.v1, this.v3, this.v3);
            }
        }
        
        public bvec3 tqq {
            get {
                return new bvec3(this.v1, this.v3, this.v3);
            }
        }
        
        public bvec3 zxx {
            get {
                return new bvec3(this.v2, this.v0, this.v0);
            }
        }
        
        public bvec3 brr {
            get {
                return new bvec3(this.v2, this.v0, this.v0);
            }
        }
        
        public bvec3 pss {
            get {
                return new bvec3(this.v2, this.v0, this.v0);
            }
        }
        
        public bvec3 zxy {
            get {
                return new bvec3(this.v2, this.v0, this.v1);
            }
            set {
                this.v2 = value.v0;
                this.v0 = value.v1;
                this.v1 = value.v2;
            }
        }
        
        public bvec3 brg {
            get {
                return new bvec3(this.v2, this.v0, this.v1);
            }
            set {
                this.v2 = value.v0;
                this.v0 = value.v1;
                this.v1 = value.v2;
            }
        }
        
        public bvec3 pst {
            get {
                return new bvec3(this.v2, this.v0, this.v1);
            }
            set {
                this.v2 = value.v0;
                this.v0 = value.v1;
                this.v1 = value.v2;
            }
        }
        
        public bvec3 zxz {
            get {
                return new bvec3(this.v2, this.v0, this.v2);
            }
        }
        
        public bvec3 brb {
            get {
                return new bvec3(this.v2, this.v0, this.v2);
            }
        }
        
        public bvec3 psp {
            get {
                return new bvec3(this.v2, this.v0, this.v2);
            }
        }
        
        public bvec3 zxw {
            get {
                return new bvec3(this.v2, this.v0, this.v3);
            }
            set {
                this.v2 = value.v0;
                this.v0 = value.v1;
                this.v3 = value.v2;
            }
        }
        
        public bvec3 bra {
            get {
                return new bvec3(this.v2, this.v0, this.v3);
            }
            set {
                this.v2 = value.v0;
                this.v0 = value.v1;
                this.v3 = value.v2;
            }
        }
        
        public bvec3 psq {
            get {
                return new bvec3(this.v2, this.v0, this.v3);
            }
            set {
                this.v2 = value.v0;
                this.v0 = value.v1;
                this.v3 = value.v2;
            }
        }
        
        public bvec3 zyx {
            get {
                return new bvec3(this.v2, this.v1, this.v0);
            }
            set {
                this.v2 = value.v0;
                this.v1 = value.v1;
                this.v0 = value.v2;
            }
        }
        
        public bvec3 bgr {
            get {
                return new bvec3(this.v2, this.v1, this.v0);
            }
            set {
                this.v2 = value.v0;
                this.v1 = value.v1;
                this.v0 = value.v2;
            }
        }
        
        public bvec3 pts {
            get {
                return new bvec3(this.v2, this.v1, this.v0);
            }
            set {
                this.v2 = value.v0;
                this.v1 = value.v1;
                this.v0 = value.v2;
            }
        }
        
        public bvec3 zyy {
            get {
                return new bvec3(this.v2, this.v1, this.v1);
            }
        }
        
        public bvec3 bgg {
            get {
                return new bvec3(this.v2, this.v1, this.v1);
            }
        }
        
        public bvec3 ptt {
            get {
                return new bvec3(this.v2, this.v1, this.v1);
            }
        }
        
        public bvec3 zyz {
            get {
                return new bvec3(this.v2, this.v1, this.v2);
            }
        }
        
        public bvec3 bgb {
            get {
                return new bvec3(this.v2, this.v1, this.v2);
            }
        }
        
        public bvec3 ptp {
            get {
                return new bvec3(this.v2, this.v1, this.v2);
            }
        }
        
        public bvec3 zyw {
            get {
                return new bvec3(this.v2, this.v1, this.v3);
            }
            set {
                this.v2 = value.v0;
                this.v1 = value.v1;
                this.v3 = value.v2;
            }
        }
        
        public bvec3 bga {
            get {
                return new bvec3(this.v2, this.v1, this.v3);
            }
            set {
                this.v2 = value.v0;
                this.v1 = value.v1;
                this.v3 = value.v2;
            }
        }
        
        public bvec3 ptq {
            get {
                return new bvec3(this.v2, this.v1, this.v3);
            }
            set {
                this.v2 = value.v0;
                this.v1 = value.v1;
                this.v3 = value.v2;
            }
        }
        
        public bvec3 zzx {
            get {
                return new bvec3(this.v2, this.v2, this.v0);
            }
        }
        
        public bvec3 bbr {
            get {
                return new bvec3(this.v2, this.v2, this.v0);
            }
        }
        
        public bvec3 pps {
            get {
                return new bvec3(this.v2, this.v2, this.v0);
            }
        }
        
        public bvec3 zzy {
            get {
                return new bvec3(this.v2, this.v2, this.v1);
            }
        }
        
        public bvec3 bbg {
            get {
                return new bvec3(this.v2, this.v2, this.v1);
            }
        }
        
        public bvec3 ppt {
            get {
                return new bvec3(this.v2, this.v2, this.v1);
            }
        }
        
        public bvec3 zzz {
            get {
                return new bvec3(this.v2, this.v2, this.v2);
            }
        }
        
        public bvec3 bbb {
            get {
                return new bvec3(this.v2, this.v2, this.v2);
            }
        }
        
        public bvec3 ppp {
            get {
                return new bvec3(this.v2, this.v2, this.v2);
            }
        }
        
        public bvec3 zzw {
            get {
                return new bvec3(this.v2, this.v2, this.v3);
            }
        }
        
        public bvec3 bba {
            get {
                return new bvec3(this.v2, this.v2, this.v3);
            }
        }
        
        public bvec3 ppq {
            get {
                return new bvec3(this.v2, this.v2, this.v3);
            }
        }
        
        public bvec3 zwx {
            get {
                return new bvec3(this.v2, this.v3, this.v0);
            }
            set {
                this.v2 = value.v0;
                this.v3 = value.v1;
                this.v0 = value.v2;
            }
        }
        
        public bvec3 bar {
            get {
                return new bvec3(this.v2, this.v3, this.v0);
            }
            set {
                this.v2 = value.v0;
                this.v3 = value.v1;
                this.v0 = value.v2;
            }
        }
        
        public bvec3 pqs {
            get {
                return new bvec3(this.v2, this.v3, this.v0);
            }
            set {
                this.v2 = value.v0;
                this.v3 = value.v1;
                this.v0 = value.v2;
            }
        }
        
        public bvec3 zwy {
            get {
                return new bvec3(this.v2, this.v3, this.v1);
            }
            set {
                this.v2 = value.v0;
                this.v3 = value.v1;
                this.v1 = value.v2;
            }
        }
        
        public bvec3 bag {
            get {
                return new bvec3(this.v2, this.v3, this.v1);
            }
            set {
                this.v2 = value.v0;
                this.v3 = value.v1;
                this.v1 = value.v2;
            }
        }
        
        public bvec3 pqt {
            get {
                return new bvec3(this.v2, this.v3, this.v1);
            }
            set {
                this.v2 = value.v0;
                this.v3 = value.v1;
                this.v1 = value.v2;
            }
        }
        
        public bvec3 zwz {
            get {
                return new bvec3(this.v2, this.v3, this.v2);
            }
        }
        
        public bvec3 bab {
            get {
                return new bvec3(this.v2, this.v3, this.v2);
            }
        }
        
        public bvec3 pqp {
            get {
                return new bvec3(this.v2, this.v3, this.v2);
            }
        }
        
        public bvec3 zww {
            get {
                return new bvec3(this.v2, this.v3, this.v3);
            }
        }
        
        public bvec3 baa {
            get {
                return new bvec3(this.v2, this.v3, this.v3);
            }
        }
        
        public bvec3 pqq {
            get {
                return new bvec3(this.v2, this.v3, this.v3);
            }
        }
        
        public bvec3 wxx {
            get {
                return new bvec3(this.v3, this.v0, this.v0);
            }
        }
        
        public bvec3 arr {
            get {
                return new bvec3(this.v3, this.v0, this.v0);
            }
        }
        
        public bvec3 qss {
            get {
                return new bvec3(this.v3, this.v0, this.v0);
            }
        }
        
        public bvec3 wxy {
            get {
                return new bvec3(this.v3, this.v0, this.v1);
            }
            set {
                this.v3 = value.v0;
                this.v0 = value.v1;
                this.v1 = value.v2;
            }
        }
        
        public bvec3 arg {
            get {
                return new bvec3(this.v3, this.v0, this.v1);
            }
            set {
                this.v3 = value.v0;
                this.v0 = value.v1;
                this.v1 = value.v2;
            }
        }
        
        public bvec3 qst {
            get {
                return new bvec3(this.v3, this.v0, this.v1);
            }
            set {
                this.v3 = value.v0;
                this.v0 = value.v1;
                this.v1 = value.v2;
            }
        }
        
        public bvec3 wxz {
            get {
                return new bvec3(this.v3, this.v0, this.v2);
            }
            set {
                this.v3 = value.v0;
                this.v0 = value.v1;
                this.v2 = value.v2;
            }
        }
        
        public bvec3 arb {
            get {
                return new bvec3(this.v3, this.v0, this.v2);
            }
            set {
                this.v3 = value.v0;
                this.v0 = value.v1;
                this.v2 = value.v2;
            }
        }
        
        public bvec3 qsp {
            get {
                return new bvec3(this.v3, this.v0, this.v2);
            }
            set {
                this.v3 = value.v0;
                this.v0 = value.v1;
                this.v2 = value.v2;
            }
        }
        
        public bvec3 wxw {
            get {
                return new bvec3(this.v3, this.v0, this.v3);
            }
        }
        
        public bvec3 ara {
            get {
                return new bvec3(this.v3, this.v0, this.v3);
            }
        }
        
        public bvec3 qsq {
            get {
                return new bvec3(this.v3, this.v0, this.v3);
            }
        }
        
        public bvec3 wyx {
            get {
                return new bvec3(this.v3, this.v1, this.v0);
            }
            set {
                this.v3 = value.v0;
                this.v1 = value.v1;
                this.v0 = value.v2;
            }
        }
        
        public bvec3 agr {
            get {
                return new bvec3(this.v3, this.v1, this.v0);
            }
            set {
                this.v3 = value.v0;
                this.v1 = value.v1;
                this.v0 = value.v2;
            }
        }
        
        public bvec3 qts {
            get {
                return new bvec3(this.v3, this.v1, this.v0);
            }
            set {
                this.v3 = value.v0;
                this.v1 = value.v1;
                this.v0 = value.v2;
            }
        }
        
        public bvec3 wyy {
            get {
                return new bvec3(this.v3, this.v1, this.v1);
            }
        }
        
        public bvec3 agg {
            get {
                return new bvec3(this.v3, this.v1, this.v1);
            }
        }
        
        public bvec3 qtt {
            get {
                return new bvec3(this.v3, this.v1, this.v1);
            }
        }
        
        public bvec3 wyz {
            get {
                return new bvec3(this.v3, this.v1, this.v2);
            }
            set {
                this.v3 = value.v0;
                this.v1 = value.v1;
                this.v2 = value.v2;
            }
        }
        
        public bvec3 agb {
            get {
                return new bvec3(this.v3, this.v1, this.v2);
            }
            set {
                this.v3 = value.v0;
                this.v1 = value.v1;
                this.v2 = value.v2;
            }
        }
        
        public bvec3 qtp {
            get {
                return new bvec3(this.v3, this.v1, this.v2);
            }
            set {
                this.v3 = value.v0;
                this.v1 = value.v1;
                this.v2 = value.v2;
            }
        }
        
        public bvec3 wyw {
            get {
                return new bvec3(this.v3, this.v1, this.v3);
            }
        }
        
        public bvec3 aga {
            get {
                return new bvec3(this.v3, this.v1, this.v3);
            }
        }
        
        public bvec3 qtq {
            get {
                return new bvec3(this.v3, this.v1, this.v3);
            }
        }
        
        public bvec3 wzx {
            get {
                return new bvec3(this.v3, this.v2, this.v0);
            }
            set {
                this.v3 = value.v0;
                this.v2 = value.v1;
                this.v0 = value.v2;
            }
        }
        
        public bvec3 abr {
            get {
                return new bvec3(this.v3, this.v2, this.v0);
            }
            set {
                this.v3 = value.v0;
                this.v2 = value.v1;
                this.v0 = value.v2;
            }
        }
        
        public bvec3 qps {
            get {
                return new bvec3(this.v3, this.v2, this.v0);
            }
            set {
                this.v3 = value.v0;
                this.v2 = value.v1;
                this.v0 = value.v2;
            }
        }
        
        public bvec3 wzy {
            get {
                return new bvec3(this.v3, this.v2, this.v1);
            }
            set {
                this.v3 = value.v0;
                this.v2 = value.v1;
                this.v1 = value.v2;
            }
        }
        
        public bvec3 abg {
            get {
                return new bvec3(this.v3, this.v2, this.v1);
            }
            set {
                this.v3 = value.v0;
                this.v2 = value.v1;
                this.v1 = value.v2;
            }
        }
        
        public bvec3 qpt {
            get {
                return new bvec3(this.v3, this.v2, this.v1);
            }
            set {
                this.v3 = value.v0;
                this.v2 = value.v1;
                this.v1 = value.v2;
            }
        }
        
        public bvec3 wzz {
            get {
                return new bvec3(this.v3, this.v2, this.v2);
            }
        }
        
        public bvec3 abb {
            get {
                return new bvec3(this.v3, this.v2, this.v2);
            }
        }
        
        public bvec3 qpp {
            get {
                return new bvec3(this.v3, this.v2, this.v2);
            }
        }
        
        public bvec3 wzw {
            get {
                return new bvec3(this.v3, this.v2, this.v3);
            }
        }
        
        public bvec3 aba {
            get {
                return new bvec3(this.v3, this.v2, this.v3);
            }
        }
        
        public bvec3 qpq {
            get {
                return new bvec3(this.v3, this.v2, this.v3);
            }
        }
        
        public bvec3 wwx {
            get {
                return new bvec3(this.v3, this.v3, this.v0);
            }
        }
        
        public bvec3 aar {
            get {
                return new bvec3(this.v3, this.v3, this.v0);
            }
        }
        
        public bvec3 qqs {
            get {
                return new bvec3(this.v3, this.v3, this.v0);
            }
        }
        
        public bvec3 wwy {
            get {
                return new bvec3(this.v3, this.v3, this.v1);
            }
        }
        
        public bvec3 aag {
            get {
                return new bvec3(this.v3, this.v3, this.v1);
            }
        }
        
        public bvec3 qqt {
            get {
                return new bvec3(this.v3, this.v3, this.v1);
            }
        }
        
        public bvec3 wwz {
            get {
                return new bvec3(this.v3, this.v3, this.v2);
            }
        }
        
        public bvec3 aab {
            get {
                return new bvec3(this.v3, this.v3, this.v2);
            }
        }
        
        public bvec3 qqp {
            get {
                return new bvec3(this.v3, this.v3, this.v2);
            }
        }
        
        public bvec3 www {
            get {
                return new bvec3(this.v3, this.v3, this.v3);
            }
        }
        
        public bvec3 aaa {
            get {
                return new bvec3(this.v3, this.v3, this.v3);
            }
        }
        
        public bvec3 qqq {
            get {
                return new bvec3(this.v3, this.v3, this.v3);
            }
        }
        
        public bvec4 xxxx {
            get {
                return new bvec4(this.v0, this.v0, this.v0, this.v0);
            }
        }
        
        public bvec4 rrrr {
            get {
                return new bvec4(this.v0, this.v0, this.v0, this.v0);
            }
        }
        
        public bvec4 ssss {
            get {
                return new bvec4(this.v0, this.v0, this.v0, this.v0);
            }
        }
        
        public bvec4 xxxy {
            get {
                return new bvec4(this.v0, this.v0, this.v0, this.v1);
            }
        }
        
        public bvec4 rrrg {
            get {
                return new bvec4(this.v0, this.v0, this.v0, this.v1);
            }
        }
        
        public bvec4 ssst {
            get {
                return new bvec4(this.v0, this.v0, this.v0, this.v1);
            }
        }
        
        public bvec4 xxxz {
            get {
                return new bvec4(this.v0, this.v0, this.v0, this.v2);
            }
        }
        
        public bvec4 rrrb {
            get {
                return new bvec4(this.v0, this.v0, this.v0, this.v2);
            }
        }
        
        public bvec4 sssp {
            get {
                return new bvec4(this.v0, this.v0, this.v0, this.v2);
            }
        }
        
        public bvec4 xxxw {
            get {
                return new bvec4(this.v0, this.v0, this.v0, this.v3);
            }
        }
        
        public bvec4 rrra {
            get {
                return new bvec4(this.v0, this.v0, this.v0, this.v3);
            }
        }
        
        public bvec4 sssq {
            get {
                return new bvec4(this.v0, this.v0, this.v0, this.v3);
            }
        }
        
        public bvec4 xxyx {
            get {
                return new bvec4(this.v0, this.v0, this.v1, this.v0);
            }
        }
        
        public bvec4 rrgr {
            get {
                return new bvec4(this.v0, this.v0, this.v1, this.v0);
            }
        }
        
        public bvec4 ssts {
            get {
                return new bvec4(this.v0, this.v0, this.v1, this.v0);
            }
        }
        
        public bvec4 xxyy {
            get {
                return new bvec4(this.v0, this.v0, this.v1, this.v1);
            }
        }
        
        public bvec4 rrgg {
            get {
                return new bvec4(this.v0, this.v0, this.v1, this.v1);
            }
        }
        
        public bvec4 sstt {
            get {
                return new bvec4(this.v0, this.v0, this.v1, this.v1);
            }
        }
        
        public bvec4 xxyz {
            get {
                return new bvec4(this.v0, this.v0, this.v1, this.v2);
            }
        }
        
        public bvec4 rrgb {
            get {
                return new bvec4(this.v0, this.v0, this.v1, this.v2);
            }
        }
        
        public bvec4 sstp {
            get {
                return new bvec4(this.v0, this.v0, this.v1, this.v2);
            }
        }
        
        public bvec4 xxyw {
            get {
                return new bvec4(this.v0, this.v0, this.v1, this.v3);
            }
        }
        
        public bvec4 rrga {
            get {
                return new bvec4(this.v0, this.v0, this.v1, this.v3);
            }
        }
        
        public bvec4 sstq {
            get {
                return new bvec4(this.v0, this.v0, this.v1, this.v3);
            }
        }
        
        public bvec4 xxzx {
            get {
                return new bvec4(this.v0, this.v0, this.v2, this.v0);
            }
        }
        
        public bvec4 rrbr {
            get {
                return new bvec4(this.v0, this.v0, this.v2, this.v0);
            }
        }
        
        public bvec4 ssps {
            get {
                return new bvec4(this.v0, this.v0, this.v2, this.v0);
            }
        }
        
        public bvec4 xxzy {
            get {
                return new bvec4(this.v0, this.v0, this.v2, this.v1);
            }
        }
        
        public bvec4 rrbg {
            get {
                return new bvec4(this.v0, this.v0, this.v2, this.v1);
            }
        }
        
        public bvec4 sspt {
            get {
                return new bvec4(this.v0, this.v0, this.v2, this.v1);
            }
        }
        
        public bvec4 xxzz {
            get {
                return new bvec4(this.v0, this.v0, this.v2, this.v2);
            }
        }
        
        public bvec4 rrbb {
            get {
                return new bvec4(this.v0, this.v0, this.v2, this.v2);
            }
        }
        
        public bvec4 sspp {
            get {
                return new bvec4(this.v0, this.v0, this.v2, this.v2);
            }
        }
        
        public bvec4 xxzw {
            get {
                return new bvec4(this.v0, this.v0, this.v2, this.v3);
            }
        }
        
        public bvec4 rrba {
            get {
                return new bvec4(this.v0, this.v0, this.v2, this.v3);
            }
        }
        
        public bvec4 sspq {
            get {
                return new bvec4(this.v0, this.v0, this.v2, this.v3);
            }
        }
        
        public bvec4 xxwx {
            get {
                return new bvec4(this.v0, this.v0, this.v3, this.v0);
            }
        }
        
        public bvec4 rrar {
            get {
                return new bvec4(this.v0, this.v0, this.v3, this.v0);
            }
        }
        
        public bvec4 ssqs {
            get {
                return new bvec4(this.v0, this.v0, this.v3, this.v0);
            }
        }
        
        public bvec4 xxwy {
            get {
                return new bvec4(this.v0, this.v0, this.v3, this.v1);
            }
        }
        
        public bvec4 rrag {
            get {
                return new bvec4(this.v0, this.v0, this.v3, this.v1);
            }
        }
        
        public bvec4 ssqt {
            get {
                return new bvec4(this.v0, this.v0, this.v3, this.v1);
            }
        }
        
        public bvec4 xxwz {
            get {
                return new bvec4(this.v0, this.v0, this.v3, this.v2);
            }
        }
        
        public bvec4 rrab {
            get {
                return new bvec4(this.v0, this.v0, this.v3, this.v2);
            }
        }
        
        public bvec4 ssqp {
            get {
                return new bvec4(this.v0, this.v0, this.v3, this.v2);
            }
        }
        
        public bvec4 xxww {
            get {
                return new bvec4(this.v0, this.v0, this.v3, this.v3);
            }
        }
        
        public bvec4 rraa {
            get {
                return new bvec4(this.v0, this.v0, this.v3, this.v3);
            }
        }
        
        public bvec4 ssqq {
            get {
                return new bvec4(this.v0, this.v0, this.v3, this.v3);
            }
        }
        
        public bvec4 xyxx {
            get {
                return new bvec4(this.v0, this.v1, this.v0, this.v0);
            }
        }
        
        public bvec4 rgrr {
            get {
                return new bvec4(this.v0, this.v1, this.v0, this.v0);
            }
        }
        
        public bvec4 stss {
            get {
                return new bvec4(this.v0, this.v1, this.v0, this.v0);
            }
        }
        
        public bvec4 xyxy {
            get {
                return new bvec4(this.v0, this.v1, this.v0, this.v1);
            }
        }
        
        public bvec4 rgrg {
            get {
                return new bvec4(this.v0, this.v1, this.v0, this.v1);
            }
        }
        
        public bvec4 stst {
            get {
                return new bvec4(this.v0, this.v1, this.v0, this.v1);
            }
        }
        
        public bvec4 xyxz {
            get {
                return new bvec4(this.v0, this.v1, this.v0, this.v2);
            }
        }
        
        public bvec4 rgrb {
            get {
                return new bvec4(this.v0, this.v1, this.v0, this.v2);
            }
        }
        
        public bvec4 stsp {
            get {
                return new bvec4(this.v0, this.v1, this.v0, this.v2);
            }
        }
        
        public bvec4 xyxw {
            get {
                return new bvec4(this.v0, this.v1, this.v0, this.v3);
            }
        }
        
        public bvec4 rgra {
            get {
                return new bvec4(this.v0, this.v1, this.v0, this.v3);
            }
        }
        
        public bvec4 stsq {
            get {
                return new bvec4(this.v0, this.v1, this.v0, this.v3);
            }
        }
        
        public bvec4 xyyx {
            get {
                return new bvec4(this.v0, this.v1, this.v1, this.v0);
            }
        }
        
        public bvec4 rggr {
            get {
                return new bvec4(this.v0, this.v1, this.v1, this.v0);
            }
        }
        
        public bvec4 stts {
            get {
                return new bvec4(this.v0, this.v1, this.v1, this.v0);
            }
        }
        
        public bvec4 xyyy {
            get {
                return new bvec4(this.v0, this.v1, this.v1, this.v1);
            }
        }
        
        public bvec4 rggg {
            get {
                return new bvec4(this.v0, this.v1, this.v1, this.v1);
            }
        }
        
        public bvec4 sttt {
            get {
                return new bvec4(this.v0, this.v1, this.v1, this.v1);
            }
        }
        
        public bvec4 xyyz {
            get {
                return new bvec4(this.v0, this.v1, this.v1, this.v2);
            }
        }
        
        public bvec4 rggb {
            get {
                return new bvec4(this.v0, this.v1, this.v1, this.v2);
            }
        }
        
        public bvec4 sttp {
            get {
                return new bvec4(this.v0, this.v1, this.v1, this.v2);
            }
        }
        
        public bvec4 xyyw {
            get {
                return new bvec4(this.v0, this.v1, this.v1, this.v3);
            }
        }
        
        public bvec4 rgga {
            get {
                return new bvec4(this.v0, this.v1, this.v1, this.v3);
            }
        }
        
        public bvec4 sttq {
            get {
                return new bvec4(this.v0, this.v1, this.v1, this.v3);
            }
        }
        
        public bvec4 xyzx {
            get {
                return new bvec4(this.v0, this.v1, this.v2, this.v0);
            }
        }
        
        public bvec4 rgbr {
            get {
                return new bvec4(this.v0, this.v1, this.v2, this.v0);
            }
        }
        
        public bvec4 stps {
            get {
                return new bvec4(this.v0, this.v1, this.v2, this.v0);
            }
        }
        
        public bvec4 xyzy {
            get {
                return new bvec4(this.v0, this.v1, this.v2, this.v1);
            }
        }
        
        public bvec4 rgbg {
            get {
                return new bvec4(this.v0, this.v1, this.v2, this.v1);
            }
        }
        
        public bvec4 stpt {
            get {
                return new bvec4(this.v0, this.v1, this.v2, this.v1);
            }
        }
        
        public bvec4 xyzz {
            get {
                return new bvec4(this.v0, this.v1, this.v2, this.v2);
            }
        }
        
        public bvec4 rgbb {
            get {
                return new bvec4(this.v0, this.v1, this.v2, this.v2);
            }
        }
        
        public bvec4 stpp {
            get {
                return new bvec4(this.v0, this.v1, this.v2, this.v2);
            }
        }
        
        public bvec4 xyzw {
            get {
                return new bvec4(this.v0, this.v1, this.v2, this.v3);
            }
            set {
                this.v0 = value.v0;
                this.v1 = value.v1;
                this.v2 = value.v2;
                this.v3 = value.v3;
            }
        }
        
        public bvec4 rgba {
            get {
                return new bvec4(this.v0, this.v1, this.v2, this.v3);
            }
            set {
                this.v0 = value.v0;
                this.v1 = value.v1;
                this.v2 = value.v2;
                this.v3 = value.v3;
            }
        }
        
        public bvec4 stpq {
            get {
                return new bvec4(this.v0, this.v1, this.v2, this.v3);
            }
            set {
                this.v0 = value.v0;
                this.v1 = value.v1;
                this.v2 = value.v2;
                this.v3 = value.v3;
            }
        }
        
        public bvec4 xywx {
            get {
                return new bvec4(this.v0, this.v1, this.v3, this.v0);
            }
        }
        
        public bvec4 rgar {
            get {
                return new bvec4(this.v0, this.v1, this.v3, this.v0);
            }
        }
        
        public bvec4 stqs {
            get {
                return new bvec4(this.v0, this.v1, this.v3, this.v0);
            }
        }
        
        public bvec4 xywy {
            get {
                return new bvec4(this.v0, this.v1, this.v3, this.v1);
            }
        }
        
        public bvec4 rgag {
            get {
                return new bvec4(this.v0, this.v1, this.v3, this.v1);
            }
        }
        
        public bvec4 stqt {
            get {
                return new bvec4(this.v0, this.v1, this.v3, this.v1);
            }
        }
        
        public bvec4 xywz {
            get {
                return new bvec4(this.v0, this.v1, this.v3, this.v2);
            }
            set {
                this.v0 = value.v0;
                this.v1 = value.v1;
                this.v3 = value.v2;
                this.v2 = value.v3;
            }
        }
        
        public bvec4 rgab {
            get {
                return new bvec4(this.v0, this.v1, this.v3, this.v2);
            }
            set {
                this.v0 = value.v0;
                this.v1 = value.v1;
                this.v3 = value.v2;
                this.v2 = value.v3;
            }
        }
        
        public bvec4 stqp {
            get {
                return new bvec4(this.v0, this.v1, this.v3, this.v2);
            }
            set {
                this.v0 = value.v0;
                this.v1 = value.v1;
                this.v3 = value.v2;
                this.v2 = value.v3;
            }
        }
        
        public bvec4 xyww {
            get {
                return new bvec4(this.v0, this.v1, this.v3, this.v3);
            }
        }
        
        public bvec4 rgaa {
            get {
                return new bvec4(this.v0, this.v1, this.v3, this.v3);
            }
        }
        
        public bvec4 stqq {
            get {
                return new bvec4(this.v0, this.v1, this.v3, this.v3);
            }
        }
        
        public bvec4 xzxx {
            get {
                return new bvec4(this.v0, this.v2, this.v0, this.v0);
            }
        }
        
        public bvec4 rbrr {
            get {
                return new bvec4(this.v0, this.v2, this.v0, this.v0);
            }
        }
        
        public bvec4 spss {
            get {
                return new bvec4(this.v0, this.v2, this.v0, this.v0);
            }
        }
        
        public bvec4 xzxy {
            get {
                return new bvec4(this.v0, this.v2, this.v0, this.v1);
            }
        }
        
        public bvec4 rbrg {
            get {
                return new bvec4(this.v0, this.v2, this.v0, this.v1);
            }
        }
        
        public bvec4 spst {
            get {
                return new bvec4(this.v0, this.v2, this.v0, this.v1);
            }
        }
        
        public bvec4 xzxz {
            get {
                return new bvec4(this.v0, this.v2, this.v0, this.v2);
            }
        }
        
        public bvec4 rbrb {
            get {
                return new bvec4(this.v0, this.v2, this.v0, this.v2);
            }
        }
        
        public bvec4 spsp {
            get {
                return new bvec4(this.v0, this.v2, this.v0, this.v2);
            }
        }
        
        public bvec4 xzxw {
            get {
                return new bvec4(this.v0, this.v2, this.v0, this.v3);
            }
        }
        
        public bvec4 rbra {
            get {
                return new bvec4(this.v0, this.v2, this.v0, this.v3);
            }
        }
        
        public bvec4 spsq {
            get {
                return new bvec4(this.v0, this.v2, this.v0, this.v3);
            }
        }
        
        public bvec4 xzyx {
            get {
                return new bvec4(this.v0, this.v2, this.v1, this.v0);
            }
        }
        
        public bvec4 rbgr {
            get {
                return new bvec4(this.v0, this.v2, this.v1, this.v0);
            }
        }
        
        public bvec4 spts {
            get {
                return new bvec4(this.v0, this.v2, this.v1, this.v0);
            }
        }
        
        public bvec4 xzyy {
            get {
                return new bvec4(this.v0, this.v2, this.v1, this.v1);
            }
        }
        
        public bvec4 rbgg {
            get {
                return new bvec4(this.v0, this.v2, this.v1, this.v1);
            }
        }
        
        public bvec4 sptt {
            get {
                return new bvec4(this.v0, this.v2, this.v1, this.v1);
            }
        }
        
        public bvec4 xzyz {
            get {
                return new bvec4(this.v0, this.v2, this.v1, this.v2);
            }
        }
        
        public bvec4 rbgb {
            get {
                return new bvec4(this.v0, this.v2, this.v1, this.v2);
            }
        }
        
        public bvec4 sptp {
            get {
                return new bvec4(this.v0, this.v2, this.v1, this.v2);
            }
        }
        
        public bvec4 xzyw {
            get {
                return new bvec4(this.v0, this.v2, this.v1, this.v3);
            }
            set {
                this.v0 = value.v0;
                this.v2 = value.v1;
                this.v1 = value.v2;
                this.v3 = value.v3;
            }
        }
        
        public bvec4 rbga {
            get {
                return new bvec4(this.v0, this.v2, this.v1, this.v3);
            }
            set {
                this.v0 = value.v0;
                this.v2 = value.v1;
                this.v1 = value.v2;
                this.v3 = value.v3;
            }
        }
        
        public bvec4 sptq {
            get {
                return new bvec4(this.v0, this.v2, this.v1, this.v3);
            }
            set {
                this.v0 = value.v0;
                this.v2 = value.v1;
                this.v1 = value.v2;
                this.v3 = value.v3;
            }
        }
        
        public bvec4 xzzx {
            get {
                return new bvec4(this.v0, this.v2, this.v2, this.v0);
            }
        }
        
        public bvec4 rbbr {
            get {
                return new bvec4(this.v0, this.v2, this.v2, this.v0);
            }
        }
        
        public bvec4 spps {
            get {
                return new bvec4(this.v0, this.v2, this.v2, this.v0);
            }
        }
        
        public bvec4 xzzy {
            get {
                return new bvec4(this.v0, this.v2, this.v2, this.v1);
            }
        }
        
        public bvec4 rbbg {
            get {
                return new bvec4(this.v0, this.v2, this.v2, this.v1);
            }
        }
        
        public bvec4 sppt {
            get {
                return new bvec4(this.v0, this.v2, this.v2, this.v1);
            }
        }
        
        public bvec4 xzzz {
            get {
                return new bvec4(this.v0, this.v2, this.v2, this.v2);
            }
        }
        
        public bvec4 rbbb {
            get {
                return new bvec4(this.v0, this.v2, this.v2, this.v2);
            }
        }
        
        public bvec4 sppp {
            get {
                return new bvec4(this.v0, this.v2, this.v2, this.v2);
            }
        }
        
        public bvec4 xzzw {
            get {
                return new bvec4(this.v0, this.v2, this.v2, this.v3);
            }
        }
        
        public bvec4 rbba {
            get {
                return new bvec4(this.v0, this.v2, this.v2, this.v3);
            }
        }
        
        public bvec4 sppq {
            get {
                return new bvec4(this.v0, this.v2, this.v2, this.v3);
            }
        }
        
        public bvec4 xzwx {
            get {
                return new bvec4(this.v0, this.v2, this.v3, this.v0);
            }
        }
        
        public bvec4 rbar {
            get {
                return new bvec4(this.v0, this.v2, this.v3, this.v0);
            }
        }
        
        public bvec4 spqs {
            get {
                return new bvec4(this.v0, this.v2, this.v3, this.v0);
            }
        }
        
        public bvec4 xzwy {
            get {
                return new bvec4(this.v0, this.v2, this.v3, this.v1);
            }
            set {
                this.v0 = value.v0;
                this.v2 = value.v1;
                this.v3 = value.v2;
                this.v1 = value.v3;
            }
        }
        
        public bvec4 rbag {
            get {
                return new bvec4(this.v0, this.v2, this.v3, this.v1);
            }
            set {
                this.v0 = value.v0;
                this.v2 = value.v1;
                this.v3 = value.v2;
                this.v1 = value.v3;
            }
        }
        
        public bvec4 spqt {
            get {
                return new bvec4(this.v0, this.v2, this.v3, this.v1);
            }
            set {
                this.v0 = value.v0;
                this.v2 = value.v1;
                this.v3 = value.v2;
                this.v1 = value.v3;
            }
        }
        
        public bvec4 xzwz {
            get {
                return new bvec4(this.v0, this.v2, this.v3, this.v2);
            }
        }
        
        public bvec4 rbab {
            get {
                return new bvec4(this.v0, this.v2, this.v3, this.v2);
            }
        }
        
        public bvec4 spqp {
            get {
                return new bvec4(this.v0, this.v2, this.v3, this.v2);
            }
        }
        
        public bvec4 xzww {
            get {
                return new bvec4(this.v0, this.v2, this.v3, this.v3);
            }
        }
        
        public bvec4 rbaa {
            get {
                return new bvec4(this.v0, this.v2, this.v3, this.v3);
            }
        }
        
        public bvec4 spqq {
            get {
                return new bvec4(this.v0, this.v2, this.v3, this.v3);
            }
        }
        
        public bvec4 xwxx {
            get {
                return new bvec4(this.v0, this.v3, this.v0, this.v0);
            }
        }
        
        public bvec4 rarr {
            get {
                return new bvec4(this.v0, this.v3, this.v0, this.v0);
            }
        }
        
        public bvec4 sqss {
            get {
                return new bvec4(this.v0, this.v3, this.v0, this.v0);
            }
        }
        
        public bvec4 xwxy {
            get {
                return new bvec4(this.v0, this.v3, this.v0, this.v1);
            }
        }
        
        public bvec4 rarg {
            get {
                return new bvec4(this.v0, this.v3, this.v0, this.v1);
            }
        }
        
        public bvec4 sqst {
            get {
                return new bvec4(this.v0, this.v3, this.v0, this.v1);
            }
        }
        
        public bvec4 xwxz {
            get {
                return new bvec4(this.v0, this.v3, this.v0, this.v2);
            }
        }
        
        public bvec4 rarb {
            get {
                return new bvec4(this.v0, this.v3, this.v0, this.v2);
            }
        }
        
        public bvec4 sqsp {
            get {
                return new bvec4(this.v0, this.v3, this.v0, this.v2);
            }
        }
        
        public bvec4 xwxw {
            get {
                return new bvec4(this.v0, this.v3, this.v0, this.v3);
            }
        }
        
        public bvec4 rara {
            get {
                return new bvec4(this.v0, this.v3, this.v0, this.v3);
            }
        }
        
        public bvec4 sqsq {
            get {
                return new bvec4(this.v0, this.v3, this.v0, this.v3);
            }
        }
        
        public bvec4 xwyx {
            get {
                return new bvec4(this.v0, this.v3, this.v1, this.v0);
            }
        }
        
        public bvec4 ragr {
            get {
                return new bvec4(this.v0, this.v3, this.v1, this.v0);
            }
        }
        
        public bvec4 sqts {
            get {
                return new bvec4(this.v0, this.v3, this.v1, this.v0);
            }
        }
        
        public bvec4 xwyy {
            get {
                return new bvec4(this.v0, this.v3, this.v1, this.v1);
            }
        }
        
        public bvec4 ragg {
            get {
                return new bvec4(this.v0, this.v3, this.v1, this.v1);
            }
        }
        
        public bvec4 sqtt {
            get {
                return new bvec4(this.v0, this.v3, this.v1, this.v1);
            }
        }
        
        public bvec4 xwyz {
            get {
                return new bvec4(this.v0, this.v3, this.v1, this.v2);
            }
            set {
                this.v0 = value.v0;
                this.v3 = value.v1;
                this.v1 = value.v2;
                this.v2 = value.v3;
            }
        }
        
        public bvec4 ragb {
            get {
                return new bvec4(this.v0, this.v3, this.v1, this.v2);
            }
            set {
                this.v0 = value.v0;
                this.v3 = value.v1;
                this.v1 = value.v2;
                this.v2 = value.v3;
            }
        }
        
        public bvec4 sqtp {
            get {
                return new bvec4(this.v0, this.v3, this.v1, this.v2);
            }
            set {
                this.v0 = value.v0;
                this.v3 = value.v1;
                this.v1 = value.v2;
                this.v2 = value.v3;
            }
        }
        
        public bvec4 xwyw {
            get {
                return new bvec4(this.v0, this.v3, this.v1, this.v3);
            }
        }
        
        public bvec4 raga {
            get {
                return new bvec4(this.v0, this.v3, this.v1, this.v3);
            }
        }
        
        public bvec4 sqtq {
            get {
                return new bvec4(this.v0, this.v3, this.v1, this.v3);
            }
        }
        
        public bvec4 xwzx {
            get {
                return new bvec4(this.v0, this.v3, this.v2, this.v0);
            }
        }
        
        public bvec4 rabr {
            get {
                return new bvec4(this.v0, this.v3, this.v2, this.v0);
            }
        }
        
        public bvec4 sqps {
            get {
                return new bvec4(this.v0, this.v3, this.v2, this.v0);
            }
        }
        
        public bvec4 xwzy {
            get {
                return new bvec4(this.v0, this.v3, this.v2, this.v1);
            }
            set {
                this.v0 = value.v0;
                this.v3 = value.v1;
                this.v2 = value.v2;
                this.v1 = value.v3;
            }
        }
        
        public bvec4 rabg {
            get {
                return new bvec4(this.v0, this.v3, this.v2, this.v1);
            }
            set {
                this.v0 = value.v0;
                this.v3 = value.v1;
                this.v2 = value.v2;
                this.v1 = value.v3;
            }
        }
        
        public bvec4 sqpt {
            get {
                return new bvec4(this.v0, this.v3, this.v2, this.v1);
            }
            set {
                this.v0 = value.v0;
                this.v3 = value.v1;
                this.v2 = value.v2;
                this.v1 = value.v3;
            }
        }
        
        public bvec4 xwzz {
            get {
                return new bvec4(this.v0, this.v3, this.v2, this.v2);
            }
        }
        
        public bvec4 rabb {
            get {
                return new bvec4(this.v0, this.v3, this.v2, this.v2);
            }
        }
        
        public bvec4 sqpp {
            get {
                return new bvec4(this.v0, this.v3, this.v2, this.v2);
            }
        }
        
        public bvec4 xwzw {
            get {
                return new bvec4(this.v0, this.v3, this.v2, this.v3);
            }
        }
        
        public bvec4 raba {
            get {
                return new bvec4(this.v0, this.v3, this.v2, this.v3);
            }
        }
        
        public bvec4 sqpq {
            get {
                return new bvec4(this.v0, this.v3, this.v2, this.v3);
            }
        }
        
        public bvec4 xwwx {
            get {
                return new bvec4(this.v0, this.v3, this.v3, this.v0);
            }
        }
        
        public bvec4 raar {
            get {
                return new bvec4(this.v0, this.v3, this.v3, this.v0);
            }
        }
        
        public bvec4 sqqs {
            get {
                return new bvec4(this.v0, this.v3, this.v3, this.v0);
            }
        }
        
        public bvec4 xwwy {
            get {
                return new bvec4(this.v0, this.v3, this.v3, this.v1);
            }
        }
        
        public bvec4 raag {
            get {
                return new bvec4(this.v0, this.v3, this.v3, this.v1);
            }
        }
        
        public bvec4 sqqt {
            get {
                return new bvec4(this.v0, this.v3, this.v3, this.v1);
            }
        }
        
        public bvec4 xwwz {
            get {
                return new bvec4(this.v0, this.v3, this.v3, this.v2);
            }
        }
        
        public bvec4 raab {
            get {
                return new bvec4(this.v0, this.v3, this.v3, this.v2);
            }
        }
        
        public bvec4 sqqp {
            get {
                return new bvec4(this.v0, this.v3, this.v3, this.v2);
            }
        }
        
        public bvec4 xwww {
            get {
                return new bvec4(this.v0, this.v3, this.v3, this.v3);
            }
        }
        
        public bvec4 raaa {
            get {
                return new bvec4(this.v0, this.v3, this.v3, this.v3);
            }
        }
        
        public bvec4 sqqq {
            get {
                return new bvec4(this.v0, this.v3, this.v3, this.v3);
            }
        }
        
        public bvec4 yxxx {
            get {
                return new bvec4(this.v1, this.v0, this.v0, this.v0);
            }
        }
        
        public bvec4 grrr {
            get {
                return new bvec4(this.v1, this.v0, this.v0, this.v0);
            }
        }
        
        public bvec4 tsss {
            get {
                return new bvec4(this.v1, this.v0, this.v0, this.v0);
            }
        }
        
        public bvec4 yxxy {
            get {
                return new bvec4(this.v1, this.v0, this.v0, this.v1);
            }
        }
        
        public bvec4 grrg {
            get {
                return new bvec4(this.v1, this.v0, this.v0, this.v1);
            }
        }
        
        public bvec4 tsst {
            get {
                return new bvec4(this.v1, this.v0, this.v0, this.v1);
            }
        }
        
        public bvec4 yxxz {
            get {
                return new bvec4(this.v1, this.v0, this.v0, this.v2);
            }
        }
        
        public bvec4 grrb {
            get {
                return new bvec4(this.v1, this.v0, this.v0, this.v2);
            }
        }
        
        public bvec4 tssp {
            get {
                return new bvec4(this.v1, this.v0, this.v0, this.v2);
            }
        }
        
        public bvec4 yxxw {
            get {
                return new bvec4(this.v1, this.v0, this.v0, this.v3);
            }
        }
        
        public bvec4 grra {
            get {
                return new bvec4(this.v1, this.v0, this.v0, this.v3);
            }
        }
        
        public bvec4 tssq {
            get {
                return new bvec4(this.v1, this.v0, this.v0, this.v3);
            }
        }
        
        public bvec4 yxyx {
            get {
                return new bvec4(this.v1, this.v0, this.v1, this.v0);
            }
        }
        
        public bvec4 grgr {
            get {
                return new bvec4(this.v1, this.v0, this.v1, this.v0);
            }
        }
        
        public bvec4 tsts {
            get {
                return new bvec4(this.v1, this.v0, this.v1, this.v0);
            }
        }
        
        public bvec4 yxyy {
            get {
                return new bvec4(this.v1, this.v0, this.v1, this.v1);
            }
        }
        
        public bvec4 grgg {
            get {
                return new bvec4(this.v1, this.v0, this.v1, this.v1);
            }
        }
        
        public bvec4 tstt {
            get {
                return new bvec4(this.v1, this.v0, this.v1, this.v1);
            }
        }
        
        public bvec4 yxyz {
            get {
                return new bvec4(this.v1, this.v0, this.v1, this.v2);
            }
        }
        
        public bvec4 grgb {
            get {
                return new bvec4(this.v1, this.v0, this.v1, this.v2);
            }
        }
        
        public bvec4 tstp {
            get {
                return new bvec4(this.v1, this.v0, this.v1, this.v2);
            }
        }
        
        public bvec4 yxyw {
            get {
                return new bvec4(this.v1, this.v0, this.v1, this.v3);
            }
        }
        
        public bvec4 grga {
            get {
                return new bvec4(this.v1, this.v0, this.v1, this.v3);
            }
        }
        
        public bvec4 tstq {
            get {
                return new bvec4(this.v1, this.v0, this.v1, this.v3);
            }
        }
        
        public bvec4 yxzx {
            get {
                return new bvec4(this.v1, this.v0, this.v2, this.v0);
            }
        }
        
        public bvec4 grbr {
            get {
                return new bvec4(this.v1, this.v0, this.v2, this.v0);
            }
        }
        
        public bvec4 tsps {
            get {
                return new bvec4(this.v1, this.v0, this.v2, this.v0);
            }
        }
        
        public bvec4 yxzy {
            get {
                return new bvec4(this.v1, this.v0, this.v2, this.v1);
            }
        }
        
        public bvec4 grbg {
            get {
                return new bvec4(this.v1, this.v0, this.v2, this.v1);
            }
        }
        
        public bvec4 tspt {
            get {
                return new bvec4(this.v1, this.v0, this.v2, this.v1);
            }
        }
        
        public bvec4 yxzz {
            get {
                return new bvec4(this.v1, this.v0, this.v2, this.v2);
            }
        }
        
        public bvec4 grbb {
            get {
                return new bvec4(this.v1, this.v0, this.v2, this.v2);
            }
        }
        
        public bvec4 tspp {
            get {
                return new bvec4(this.v1, this.v0, this.v2, this.v2);
            }
        }
        
        public bvec4 yxzw {
            get {
                return new bvec4(this.v1, this.v0, this.v2, this.v3);
            }
            set {
                this.v1 = value.v0;
                this.v0 = value.v1;
                this.v2 = value.v2;
                this.v3 = value.v3;
            }
        }
        
        public bvec4 grba {
            get {
                return new bvec4(this.v1, this.v0, this.v2, this.v3);
            }
            set {
                this.v1 = value.v0;
                this.v0 = value.v1;
                this.v2 = value.v2;
                this.v3 = value.v3;
            }
        }
        
        public bvec4 tspq {
            get {
                return new bvec4(this.v1, this.v0, this.v2, this.v3);
            }
            set {
                this.v1 = value.v0;
                this.v0 = value.v1;
                this.v2 = value.v2;
                this.v3 = value.v3;
            }
        }
        
        public bvec4 yxwx {
            get {
                return new bvec4(this.v1, this.v0, this.v3, this.v0);
            }
        }
        
        public bvec4 grar {
            get {
                return new bvec4(this.v1, this.v0, this.v3, this.v0);
            }
        }
        
        public bvec4 tsqs {
            get {
                return new bvec4(this.v1, this.v0, this.v3, this.v0);
            }
        }
        
        public bvec4 yxwy {
            get {
                return new bvec4(this.v1, this.v0, this.v3, this.v1);
            }
        }
        
        public bvec4 grag {
            get {
                return new bvec4(this.v1, this.v0, this.v3, this.v1);
            }
        }
        
        public bvec4 tsqt {
            get {
                return new bvec4(this.v1, this.v0, this.v3, this.v1);
            }
        }
        
        public bvec4 yxwz {
            get {
                return new bvec4(this.v1, this.v0, this.v3, this.v2);
            }
            set {
                this.v1 = value.v0;
                this.v0 = value.v1;
                this.v3 = value.v2;
                this.v2 = value.v3;
            }
        }
        
        public bvec4 grab {
            get {
                return new bvec4(this.v1, this.v0, this.v3, this.v2);
            }
            set {
                this.v1 = value.v0;
                this.v0 = value.v1;
                this.v3 = value.v2;
                this.v2 = value.v3;
            }
        }
        
        public bvec4 tsqp {
            get {
                return new bvec4(this.v1, this.v0, this.v3, this.v2);
            }
            set {
                this.v1 = value.v0;
                this.v0 = value.v1;
                this.v3 = value.v2;
                this.v2 = value.v3;
            }
        }
        
        public bvec4 yxww {
            get {
                return new bvec4(this.v1, this.v0, this.v3, this.v3);
            }
        }
        
        public bvec4 graa {
            get {
                return new bvec4(this.v1, this.v0, this.v3, this.v3);
            }
        }
        
        public bvec4 tsqq {
            get {
                return new bvec4(this.v1, this.v0, this.v3, this.v3);
            }
        }
        
        public bvec4 yyxx {
            get {
                return new bvec4(this.v1, this.v1, this.v0, this.v0);
            }
        }
        
        public bvec4 ggrr {
            get {
                return new bvec4(this.v1, this.v1, this.v0, this.v0);
            }
        }
        
        public bvec4 ttss {
            get {
                return new bvec4(this.v1, this.v1, this.v0, this.v0);
            }
        }
        
        public bvec4 yyxy {
            get {
                return new bvec4(this.v1, this.v1, this.v0, this.v1);
            }
        }
        
        public bvec4 ggrg {
            get {
                return new bvec4(this.v1, this.v1, this.v0, this.v1);
            }
        }
        
        public bvec4 ttst {
            get {
                return new bvec4(this.v1, this.v1, this.v0, this.v1);
            }
        }
        
        public bvec4 yyxz {
            get {
                return new bvec4(this.v1, this.v1, this.v0, this.v2);
            }
        }
        
        public bvec4 ggrb {
            get {
                return new bvec4(this.v1, this.v1, this.v0, this.v2);
            }
        }
        
        public bvec4 ttsp {
            get {
                return new bvec4(this.v1, this.v1, this.v0, this.v2);
            }
        }
        
        public bvec4 yyxw {
            get {
                return new bvec4(this.v1, this.v1, this.v0, this.v3);
            }
        }
        
        public bvec4 ggra {
            get {
                return new bvec4(this.v1, this.v1, this.v0, this.v3);
            }
        }
        
        public bvec4 ttsq {
            get {
                return new bvec4(this.v1, this.v1, this.v0, this.v3);
            }
        }
        
        public bvec4 yyyx {
            get {
                return new bvec4(this.v1, this.v1, this.v1, this.v0);
            }
        }
        
        public bvec4 gggr {
            get {
                return new bvec4(this.v1, this.v1, this.v1, this.v0);
            }
        }
        
        public bvec4 ttts {
            get {
                return new bvec4(this.v1, this.v1, this.v1, this.v0);
            }
        }
        
        public bvec4 yyyy {
            get {
                return new bvec4(this.v1, this.v1, this.v1, this.v1);
            }
        }
        
        public bvec4 gggg {
            get {
                return new bvec4(this.v1, this.v1, this.v1, this.v1);
            }
        }
        
        public bvec4 tttt {
            get {
                return new bvec4(this.v1, this.v1, this.v1, this.v1);
            }
        }
        
        public bvec4 yyyz {
            get {
                return new bvec4(this.v1, this.v1, this.v1, this.v2);
            }
        }
        
        public bvec4 gggb {
            get {
                return new bvec4(this.v1, this.v1, this.v1, this.v2);
            }
        }
        
        public bvec4 tttp {
            get {
                return new bvec4(this.v1, this.v1, this.v1, this.v2);
            }
        }
        
        public bvec4 yyyw {
            get {
                return new bvec4(this.v1, this.v1, this.v1, this.v3);
            }
        }
        
        public bvec4 ggga {
            get {
                return new bvec4(this.v1, this.v1, this.v1, this.v3);
            }
        }
        
        public bvec4 tttq {
            get {
                return new bvec4(this.v1, this.v1, this.v1, this.v3);
            }
        }
        
        public bvec4 yyzx {
            get {
                return new bvec4(this.v1, this.v1, this.v2, this.v0);
            }
        }
        
        public bvec4 ggbr {
            get {
                return new bvec4(this.v1, this.v1, this.v2, this.v0);
            }
        }
        
        public bvec4 ttps {
            get {
                return new bvec4(this.v1, this.v1, this.v2, this.v0);
            }
        }
        
        public bvec4 yyzy {
            get {
                return new bvec4(this.v1, this.v1, this.v2, this.v1);
            }
        }
        
        public bvec4 ggbg {
            get {
                return new bvec4(this.v1, this.v1, this.v2, this.v1);
            }
        }
        
        public bvec4 ttpt {
            get {
                return new bvec4(this.v1, this.v1, this.v2, this.v1);
            }
        }
        
        public bvec4 yyzz {
            get {
                return new bvec4(this.v1, this.v1, this.v2, this.v2);
            }
        }
        
        public bvec4 ggbb {
            get {
                return new bvec4(this.v1, this.v1, this.v2, this.v2);
            }
        }
        
        public bvec4 ttpp {
            get {
                return new bvec4(this.v1, this.v1, this.v2, this.v2);
            }
        }
        
        public bvec4 yyzw {
            get {
                return new bvec4(this.v1, this.v1, this.v2, this.v3);
            }
        }
        
        public bvec4 ggba {
            get {
                return new bvec4(this.v1, this.v1, this.v2, this.v3);
            }
        }
        
        public bvec4 ttpq {
            get {
                return new bvec4(this.v1, this.v1, this.v2, this.v3);
            }
        }
        
        public bvec4 yywx {
            get {
                return new bvec4(this.v1, this.v1, this.v3, this.v0);
            }
        }
        
        public bvec4 ggar {
            get {
                return new bvec4(this.v1, this.v1, this.v3, this.v0);
            }
        }
        
        public bvec4 ttqs {
            get {
                return new bvec4(this.v1, this.v1, this.v3, this.v0);
            }
        }
        
        public bvec4 yywy {
            get {
                return new bvec4(this.v1, this.v1, this.v3, this.v1);
            }
        }
        
        public bvec4 ggag {
            get {
                return new bvec4(this.v1, this.v1, this.v3, this.v1);
            }
        }
        
        public bvec4 ttqt {
            get {
                return new bvec4(this.v1, this.v1, this.v3, this.v1);
            }
        }
        
        public bvec4 yywz {
            get {
                return new bvec4(this.v1, this.v1, this.v3, this.v2);
            }
        }
        
        public bvec4 ggab {
            get {
                return new bvec4(this.v1, this.v1, this.v3, this.v2);
            }
        }
        
        public bvec4 ttqp {
            get {
                return new bvec4(this.v1, this.v1, this.v3, this.v2);
            }
        }
        
        public bvec4 yyww {
            get {
                return new bvec4(this.v1, this.v1, this.v3, this.v3);
            }
        }
        
        public bvec4 ggaa {
            get {
                return new bvec4(this.v1, this.v1, this.v3, this.v3);
            }
        }
        
        public bvec4 ttqq {
            get {
                return new bvec4(this.v1, this.v1, this.v3, this.v3);
            }
        }
        
        public bvec4 yzxx {
            get {
                return new bvec4(this.v1, this.v2, this.v0, this.v0);
            }
        }
        
        public bvec4 gbrr {
            get {
                return new bvec4(this.v1, this.v2, this.v0, this.v0);
            }
        }
        
        public bvec4 tpss {
            get {
                return new bvec4(this.v1, this.v2, this.v0, this.v0);
            }
        }
        
        public bvec4 yzxy {
            get {
                return new bvec4(this.v1, this.v2, this.v0, this.v1);
            }
        }
        
        public bvec4 gbrg {
            get {
                return new bvec4(this.v1, this.v2, this.v0, this.v1);
            }
        }
        
        public bvec4 tpst {
            get {
                return new bvec4(this.v1, this.v2, this.v0, this.v1);
            }
        }
        
        public bvec4 yzxz {
            get {
                return new bvec4(this.v1, this.v2, this.v0, this.v2);
            }
        }
        
        public bvec4 gbrb {
            get {
                return new bvec4(this.v1, this.v2, this.v0, this.v2);
            }
        }
        
        public bvec4 tpsp {
            get {
                return new bvec4(this.v1, this.v2, this.v0, this.v2);
            }
        }
        
        public bvec4 yzxw {
            get {
                return new bvec4(this.v1, this.v2, this.v0, this.v3);
            }
            set {
                this.v1 = value.v0;
                this.v2 = value.v1;
                this.v0 = value.v2;
                this.v3 = value.v3;
            }
        }
        
        public bvec4 gbra {
            get {
                return new bvec4(this.v1, this.v2, this.v0, this.v3);
            }
            set {
                this.v1 = value.v0;
                this.v2 = value.v1;
                this.v0 = value.v2;
                this.v3 = value.v3;
            }
        }
        
        public bvec4 tpsq {
            get {
                return new bvec4(this.v1, this.v2, this.v0, this.v3);
            }
            set {
                this.v1 = value.v0;
                this.v2 = value.v1;
                this.v0 = value.v2;
                this.v3 = value.v3;
            }
        }
        
        public bvec4 yzyx {
            get {
                return new bvec4(this.v1, this.v2, this.v1, this.v0);
            }
        }
        
        public bvec4 gbgr {
            get {
                return new bvec4(this.v1, this.v2, this.v1, this.v0);
            }
        }
        
        public bvec4 tpts {
            get {
                return new bvec4(this.v1, this.v2, this.v1, this.v0);
            }
        }
        
        public bvec4 yzyy {
            get {
                return new bvec4(this.v1, this.v2, this.v1, this.v1);
            }
        }
        
        public bvec4 gbgg {
            get {
                return new bvec4(this.v1, this.v2, this.v1, this.v1);
            }
        }
        
        public bvec4 tptt {
            get {
                return new bvec4(this.v1, this.v2, this.v1, this.v1);
            }
        }
        
        public bvec4 yzyz {
            get {
                return new bvec4(this.v1, this.v2, this.v1, this.v2);
            }
        }
        
        public bvec4 gbgb {
            get {
                return new bvec4(this.v1, this.v2, this.v1, this.v2);
            }
        }
        
        public bvec4 tptp {
            get {
                return new bvec4(this.v1, this.v2, this.v1, this.v2);
            }
        }
        
        public bvec4 yzyw {
            get {
                return new bvec4(this.v1, this.v2, this.v1, this.v3);
            }
        }
        
        public bvec4 gbga {
            get {
                return new bvec4(this.v1, this.v2, this.v1, this.v3);
            }
        }
        
        public bvec4 tptq {
            get {
                return new bvec4(this.v1, this.v2, this.v1, this.v3);
            }
        }
        
        public bvec4 yzzx {
            get {
                return new bvec4(this.v1, this.v2, this.v2, this.v0);
            }
        }
        
        public bvec4 gbbr {
            get {
                return new bvec4(this.v1, this.v2, this.v2, this.v0);
            }
        }
        
        public bvec4 tpps {
            get {
                return new bvec4(this.v1, this.v2, this.v2, this.v0);
            }
        }
        
        public bvec4 yzzy {
            get {
                return new bvec4(this.v1, this.v2, this.v2, this.v1);
            }
        }
        
        public bvec4 gbbg {
            get {
                return new bvec4(this.v1, this.v2, this.v2, this.v1);
            }
        }
        
        public bvec4 tppt {
            get {
                return new bvec4(this.v1, this.v2, this.v2, this.v1);
            }
        }
        
        public bvec4 yzzz {
            get {
                return new bvec4(this.v1, this.v2, this.v2, this.v2);
            }
        }
        
        public bvec4 gbbb {
            get {
                return new bvec4(this.v1, this.v2, this.v2, this.v2);
            }
        }
        
        public bvec4 tppp {
            get {
                return new bvec4(this.v1, this.v2, this.v2, this.v2);
            }
        }
        
        public bvec4 yzzw {
            get {
                return new bvec4(this.v1, this.v2, this.v2, this.v3);
            }
        }
        
        public bvec4 gbba {
            get {
                return new bvec4(this.v1, this.v2, this.v2, this.v3);
            }
        }
        
        public bvec4 tppq {
            get {
                return new bvec4(this.v1, this.v2, this.v2, this.v3);
            }
        }
        
        public bvec4 yzwx {
            get {
                return new bvec4(this.v1, this.v2, this.v3, this.v0);
            }
            set {
                this.v1 = value.v0;
                this.v2 = value.v1;
                this.v3 = value.v2;
                this.v0 = value.v3;
            }
        }
        
        public bvec4 gbar {
            get {
                return new bvec4(this.v1, this.v2, this.v3, this.v0);
            }
            set {
                this.v1 = value.v0;
                this.v2 = value.v1;
                this.v3 = value.v2;
                this.v0 = value.v3;
            }
        }
        
        public bvec4 tpqs {
            get {
                return new bvec4(this.v1, this.v2, this.v3, this.v0);
            }
            set {
                this.v1 = value.v0;
                this.v2 = value.v1;
                this.v3 = value.v2;
                this.v0 = value.v3;
            }
        }
        
        public bvec4 yzwy {
            get {
                return new bvec4(this.v1, this.v2, this.v3, this.v1);
            }
        }
        
        public bvec4 gbag {
            get {
                return new bvec4(this.v1, this.v2, this.v3, this.v1);
            }
        }
        
        public bvec4 tpqt {
            get {
                return new bvec4(this.v1, this.v2, this.v3, this.v1);
            }
        }
        
        public bvec4 yzwz {
            get {
                return new bvec4(this.v1, this.v2, this.v3, this.v2);
            }
        }
        
        public bvec4 gbab {
            get {
                return new bvec4(this.v1, this.v2, this.v3, this.v2);
            }
        }
        
        public bvec4 tpqp {
            get {
                return new bvec4(this.v1, this.v2, this.v3, this.v2);
            }
        }
        
        public bvec4 yzww {
            get {
                return new bvec4(this.v1, this.v2, this.v3, this.v3);
            }
        }
        
        public bvec4 gbaa {
            get {
                return new bvec4(this.v1, this.v2, this.v3, this.v3);
            }
        }
        
        public bvec4 tpqq {
            get {
                return new bvec4(this.v1, this.v2, this.v3, this.v3);
            }
        }
        
        public bvec4 ywxx {
            get {
                return new bvec4(this.v1, this.v3, this.v0, this.v0);
            }
        }
        
        public bvec4 garr {
            get {
                return new bvec4(this.v1, this.v3, this.v0, this.v0);
            }
        }
        
        public bvec4 tqss {
            get {
                return new bvec4(this.v1, this.v3, this.v0, this.v0);
            }
        }
        
        public bvec4 ywxy {
            get {
                return new bvec4(this.v1, this.v3, this.v0, this.v1);
            }
        }
        
        public bvec4 garg {
            get {
                return new bvec4(this.v1, this.v3, this.v0, this.v1);
            }
        }
        
        public bvec4 tqst {
            get {
                return new bvec4(this.v1, this.v3, this.v0, this.v1);
            }
        }
        
        public bvec4 ywxz {
            get {
                return new bvec4(this.v1, this.v3, this.v0, this.v2);
            }
            set {
                this.v1 = value.v0;
                this.v3 = value.v1;
                this.v0 = value.v2;
                this.v2 = value.v3;
            }
        }
        
        public bvec4 garb {
            get {
                return new bvec4(this.v1, this.v3, this.v0, this.v2);
            }
            set {
                this.v1 = value.v0;
                this.v3 = value.v1;
                this.v0 = value.v2;
                this.v2 = value.v3;
            }
        }
        
        public bvec4 tqsp {
            get {
                return new bvec4(this.v1, this.v3, this.v0, this.v2);
            }
            set {
                this.v1 = value.v0;
                this.v3 = value.v1;
                this.v0 = value.v2;
                this.v2 = value.v3;
            }
        }
        
        public bvec4 ywxw {
            get {
                return new bvec4(this.v1, this.v3, this.v0, this.v3);
            }
        }
        
        public bvec4 gara {
            get {
                return new bvec4(this.v1, this.v3, this.v0, this.v3);
            }
        }
        
        public bvec4 tqsq {
            get {
                return new bvec4(this.v1, this.v3, this.v0, this.v3);
            }
        }
        
        public bvec4 ywyx {
            get {
                return new bvec4(this.v1, this.v3, this.v1, this.v0);
            }
        }
        
        public bvec4 gagr {
            get {
                return new bvec4(this.v1, this.v3, this.v1, this.v0);
            }
        }
        
        public bvec4 tqts {
            get {
                return new bvec4(this.v1, this.v3, this.v1, this.v0);
            }
        }
        
        public bvec4 ywyy {
            get {
                return new bvec4(this.v1, this.v3, this.v1, this.v1);
            }
        }
        
        public bvec4 gagg {
            get {
                return new bvec4(this.v1, this.v3, this.v1, this.v1);
            }
        }
        
        public bvec4 tqtt {
            get {
                return new bvec4(this.v1, this.v3, this.v1, this.v1);
            }
        }
        
        public bvec4 ywyz {
            get {
                return new bvec4(this.v1, this.v3, this.v1, this.v2);
            }
        }
        
        public bvec4 gagb {
            get {
                return new bvec4(this.v1, this.v3, this.v1, this.v2);
            }
        }
        
        public bvec4 tqtp {
            get {
                return new bvec4(this.v1, this.v3, this.v1, this.v2);
            }
        }
        
        public bvec4 ywyw {
            get {
                return new bvec4(this.v1, this.v3, this.v1, this.v3);
            }
        }
        
        public bvec4 gaga {
            get {
                return new bvec4(this.v1, this.v3, this.v1, this.v3);
            }
        }
        
        public bvec4 tqtq {
            get {
                return new bvec4(this.v1, this.v3, this.v1, this.v3);
            }
        }
        
        public bvec4 ywzx {
            get {
                return new bvec4(this.v1, this.v3, this.v2, this.v0);
            }
            set {
                this.v1 = value.v0;
                this.v3 = value.v1;
                this.v2 = value.v2;
                this.v0 = value.v3;
            }
        }
        
        public bvec4 gabr {
            get {
                return new bvec4(this.v1, this.v3, this.v2, this.v0);
            }
            set {
                this.v1 = value.v0;
                this.v3 = value.v1;
                this.v2 = value.v2;
                this.v0 = value.v3;
            }
        }
        
        public bvec4 tqps {
            get {
                return new bvec4(this.v1, this.v3, this.v2, this.v0);
            }
            set {
                this.v1 = value.v0;
                this.v3 = value.v1;
                this.v2 = value.v2;
                this.v0 = value.v3;
            }
        }
        
        public bvec4 ywzy {
            get {
                return new bvec4(this.v1, this.v3, this.v2, this.v1);
            }
        }
        
        public bvec4 gabg {
            get {
                return new bvec4(this.v1, this.v3, this.v2, this.v1);
            }
        }
        
        public bvec4 tqpt {
            get {
                return new bvec4(this.v1, this.v3, this.v2, this.v1);
            }
        }
        
        public bvec4 ywzz {
            get {
                return new bvec4(this.v1, this.v3, this.v2, this.v2);
            }
        }
        
        public bvec4 gabb {
            get {
                return new bvec4(this.v1, this.v3, this.v2, this.v2);
            }
        }
        
        public bvec4 tqpp {
            get {
                return new bvec4(this.v1, this.v3, this.v2, this.v2);
            }
        }
        
        public bvec4 ywzw {
            get {
                return new bvec4(this.v1, this.v3, this.v2, this.v3);
            }
        }
        
        public bvec4 gaba {
            get {
                return new bvec4(this.v1, this.v3, this.v2, this.v3);
            }
        }
        
        public bvec4 tqpq {
            get {
                return new bvec4(this.v1, this.v3, this.v2, this.v3);
            }
        }
        
        public bvec4 ywwx {
            get {
                return new bvec4(this.v1, this.v3, this.v3, this.v0);
            }
        }
        
        public bvec4 gaar {
            get {
                return new bvec4(this.v1, this.v3, this.v3, this.v0);
            }
        }
        
        public bvec4 tqqs {
            get {
                return new bvec4(this.v1, this.v3, this.v3, this.v0);
            }
        }
        
        public bvec4 ywwy {
            get {
                return new bvec4(this.v1, this.v3, this.v3, this.v1);
            }
        }
        
        public bvec4 gaag {
            get {
                return new bvec4(this.v1, this.v3, this.v3, this.v1);
            }
        }
        
        public bvec4 tqqt {
            get {
                return new bvec4(this.v1, this.v3, this.v3, this.v1);
            }
        }
        
        public bvec4 ywwz {
            get {
                return new bvec4(this.v1, this.v3, this.v3, this.v2);
            }
        }
        
        public bvec4 gaab {
            get {
                return new bvec4(this.v1, this.v3, this.v3, this.v2);
            }
        }
        
        public bvec4 tqqp {
            get {
                return new bvec4(this.v1, this.v3, this.v3, this.v2);
            }
        }
        
        public bvec4 ywww {
            get {
                return new bvec4(this.v1, this.v3, this.v3, this.v3);
            }
        }
        
        public bvec4 gaaa {
            get {
                return new bvec4(this.v1, this.v3, this.v3, this.v3);
            }
        }
        
        public bvec4 tqqq {
            get {
                return new bvec4(this.v1, this.v3, this.v3, this.v3);
            }
        }
        
        public bvec4 zxxx {
            get {
                return new bvec4(this.v2, this.v0, this.v0, this.v0);
            }
        }
        
        public bvec4 brrr {
            get {
                return new bvec4(this.v2, this.v0, this.v0, this.v0);
            }
        }
        
        public bvec4 psss {
            get {
                return new bvec4(this.v2, this.v0, this.v0, this.v0);
            }
        }
        
        public bvec4 zxxy {
            get {
                return new bvec4(this.v2, this.v0, this.v0, this.v1);
            }
        }
        
        public bvec4 brrg {
            get {
                return new bvec4(this.v2, this.v0, this.v0, this.v1);
            }
        }
        
        public bvec4 psst {
            get {
                return new bvec4(this.v2, this.v0, this.v0, this.v1);
            }
        }
        
        public bvec4 zxxz {
            get {
                return new bvec4(this.v2, this.v0, this.v0, this.v2);
            }
        }
        
        public bvec4 brrb {
            get {
                return new bvec4(this.v2, this.v0, this.v0, this.v2);
            }
        }
        
        public bvec4 pssp {
            get {
                return new bvec4(this.v2, this.v0, this.v0, this.v2);
            }
        }
        
        public bvec4 zxxw {
            get {
                return new bvec4(this.v2, this.v0, this.v0, this.v3);
            }
        }
        
        public bvec4 brra {
            get {
                return new bvec4(this.v2, this.v0, this.v0, this.v3);
            }
        }
        
        public bvec4 pssq {
            get {
                return new bvec4(this.v2, this.v0, this.v0, this.v3);
            }
        }
        
        public bvec4 zxyx {
            get {
                return new bvec4(this.v2, this.v0, this.v1, this.v0);
            }
        }
        
        public bvec4 brgr {
            get {
                return new bvec4(this.v2, this.v0, this.v1, this.v0);
            }
        }
        
        public bvec4 psts {
            get {
                return new bvec4(this.v2, this.v0, this.v1, this.v0);
            }
        }
        
        public bvec4 zxyy {
            get {
                return new bvec4(this.v2, this.v0, this.v1, this.v1);
            }
        }
        
        public bvec4 brgg {
            get {
                return new bvec4(this.v2, this.v0, this.v1, this.v1);
            }
        }
        
        public bvec4 pstt {
            get {
                return new bvec4(this.v2, this.v0, this.v1, this.v1);
            }
        }
        
        public bvec4 zxyz {
            get {
                return new bvec4(this.v2, this.v0, this.v1, this.v2);
            }
        }
        
        public bvec4 brgb {
            get {
                return new bvec4(this.v2, this.v0, this.v1, this.v2);
            }
        }
        
        public bvec4 pstp {
            get {
                return new bvec4(this.v2, this.v0, this.v1, this.v2);
            }
        }
        
        public bvec4 zxyw {
            get {
                return new bvec4(this.v2, this.v0, this.v1, this.v3);
            }
            set {
                this.v2 = value.v0;
                this.v0 = value.v1;
                this.v1 = value.v2;
                this.v3 = value.v3;
            }
        }
        
        public bvec4 brga {
            get {
                return new bvec4(this.v2, this.v0, this.v1, this.v3);
            }
            set {
                this.v2 = value.v0;
                this.v0 = value.v1;
                this.v1 = value.v2;
                this.v3 = value.v3;
            }
        }
        
        public bvec4 pstq {
            get {
                return new bvec4(this.v2, this.v0, this.v1, this.v3);
            }
            set {
                this.v2 = value.v0;
                this.v0 = value.v1;
                this.v1 = value.v2;
                this.v3 = value.v3;
            }
        }
        
        public bvec4 zxzx {
            get {
                return new bvec4(this.v2, this.v0, this.v2, this.v0);
            }
        }
        
        public bvec4 brbr {
            get {
                return new bvec4(this.v2, this.v0, this.v2, this.v0);
            }
        }
        
        public bvec4 psps {
            get {
                return new bvec4(this.v2, this.v0, this.v2, this.v0);
            }
        }
        
        public bvec4 zxzy {
            get {
                return new bvec4(this.v2, this.v0, this.v2, this.v1);
            }
        }
        
        public bvec4 brbg {
            get {
                return new bvec4(this.v2, this.v0, this.v2, this.v1);
            }
        }
        
        public bvec4 pspt {
            get {
                return new bvec4(this.v2, this.v0, this.v2, this.v1);
            }
        }
        
        public bvec4 zxzz {
            get {
                return new bvec4(this.v2, this.v0, this.v2, this.v2);
            }
        }
        
        public bvec4 brbb {
            get {
                return new bvec4(this.v2, this.v0, this.v2, this.v2);
            }
        }
        
        public bvec4 pspp {
            get {
                return new bvec4(this.v2, this.v0, this.v2, this.v2);
            }
        }
        
        public bvec4 zxzw {
            get {
                return new bvec4(this.v2, this.v0, this.v2, this.v3);
            }
        }
        
        public bvec4 brba {
            get {
                return new bvec4(this.v2, this.v0, this.v2, this.v3);
            }
        }
        
        public bvec4 pspq {
            get {
                return new bvec4(this.v2, this.v0, this.v2, this.v3);
            }
        }
        
        public bvec4 zxwx {
            get {
                return new bvec4(this.v2, this.v0, this.v3, this.v0);
            }
        }
        
        public bvec4 brar {
            get {
                return new bvec4(this.v2, this.v0, this.v3, this.v0);
            }
        }
        
        public bvec4 psqs {
            get {
                return new bvec4(this.v2, this.v0, this.v3, this.v0);
            }
        }
        
        public bvec4 zxwy {
            get {
                return new bvec4(this.v2, this.v0, this.v3, this.v1);
            }
            set {
                this.v2 = value.v0;
                this.v0 = value.v1;
                this.v3 = value.v2;
                this.v1 = value.v3;
            }
        }
        
        public bvec4 brag {
            get {
                return new bvec4(this.v2, this.v0, this.v3, this.v1);
            }
            set {
                this.v2 = value.v0;
                this.v0 = value.v1;
                this.v3 = value.v2;
                this.v1 = value.v3;
            }
        }
        
        public bvec4 psqt {
            get {
                return new bvec4(this.v2, this.v0, this.v3, this.v1);
            }
            set {
                this.v2 = value.v0;
                this.v0 = value.v1;
                this.v3 = value.v2;
                this.v1 = value.v3;
            }
        }
        
        public bvec4 zxwz {
            get {
                return new bvec4(this.v2, this.v0, this.v3, this.v2);
            }
        }
        
        public bvec4 brab {
            get {
                return new bvec4(this.v2, this.v0, this.v3, this.v2);
            }
        }
        
        public bvec4 psqp {
            get {
                return new bvec4(this.v2, this.v0, this.v3, this.v2);
            }
        }
        
        public bvec4 zxww {
            get {
                return new bvec4(this.v2, this.v0, this.v3, this.v3);
            }
        }
        
        public bvec4 braa {
            get {
                return new bvec4(this.v2, this.v0, this.v3, this.v3);
            }
        }
        
        public bvec4 psqq {
            get {
                return new bvec4(this.v2, this.v0, this.v3, this.v3);
            }
        }
        
        public bvec4 zyxx {
            get {
                return new bvec4(this.v2, this.v1, this.v0, this.v0);
            }
        }
        
        public bvec4 bgrr {
            get {
                return new bvec4(this.v2, this.v1, this.v0, this.v0);
            }
        }
        
        public bvec4 ptss {
            get {
                return new bvec4(this.v2, this.v1, this.v0, this.v0);
            }
        }
        
        public bvec4 zyxy {
            get {
                return new bvec4(this.v2, this.v1, this.v0, this.v1);
            }
        }
        
        public bvec4 bgrg {
            get {
                return new bvec4(this.v2, this.v1, this.v0, this.v1);
            }
        }
        
        public bvec4 ptst {
            get {
                return new bvec4(this.v2, this.v1, this.v0, this.v1);
            }
        }
        
        public bvec4 zyxz {
            get {
                return new bvec4(this.v2, this.v1, this.v0, this.v2);
            }
        }
        
        public bvec4 bgrb {
            get {
                return new bvec4(this.v2, this.v1, this.v0, this.v2);
            }
        }
        
        public bvec4 ptsp {
            get {
                return new bvec4(this.v2, this.v1, this.v0, this.v2);
            }
        }
        
        public bvec4 zyxw {
            get {
                return new bvec4(this.v2, this.v1, this.v0, this.v3);
            }
            set {
                this.v2 = value.v0;
                this.v1 = value.v1;
                this.v0 = value.v2;
                this.v3 = value.v3;
            }
        }
        
        public bvec4 bgra {
            get {
                return new bvec4(this.v2, this.v1, this.v0, this.v3);
            }
            set {
                this.v2 = value.v0;
                this.v1 = value.v1;
                this.v0 = value.v2;
                this.v3 = value.v3;
            }
        }
        
        public bvec4 ptsq {
            get {
                return new bvec4(this.v2, this.v1, this.v0, this.v3);
            }
            set {
                this.v2 = value.v0;
                this.v1 = value.v1;
                this.v0 = value.v2;
                this.v3 = value.v3;
            }
        }
        
        public bvec4 zyyx {
            get {
                return new bvec4(this.v2, this.v1, this.v1, this.v0);
            }
        }
        
        public bvec4 bggr {
            get {
                return new bvec4(this.v2, this.v1, this.v1, this.v0);
            }
        }
        
        public bvec4 ptts {
            get {
                return new bvec4(this.v2, this.v1, this.v1, this.v0);
            }
        }
        
        public bvec4 zyyy {
            get {
                return new bvec4(this.v2, this.v1, this.v1, this.v1);
            }
        }
        
        public bvec4 bggg {
            get {
                return new bvec4(this.v2, this.v1, this.v1, this.v1);
            }
        }
        
        public bvec4 pttt {
            get {
                return new bvec4(this.v2, this.v1, this.v1, this.v1);
            }
        }
        
        public bvec4 zyyz {
            get {
                return new bvec4(this.v2, this.v1, this.v1, this.v2);
            }
        }
        
        public bvec4 bggb {
            get {
                return new bvec4(this.v2, this.v1, this.v1, this.v2);
            }
        }
        
        public bvec4 pttp {
            get {
                return new bvec4(this.v2, this.v1, this.v1, this.v2);
            }
        }
        
        public bvec4 zyyw {
            get {
                return new bvec4(this.v2, this.v1, this.v1, this.v3);
            }
        }
        
        public bvec4 bgga {
            get {
                return new bvec4(this.v2, this.v1, this.v1, this.v3);
            }
        }
        
        public bvec4 pttq {
            get {
                return new bvec4(this.v2, this.v1, this.v1, this.v3);
            }
        }
        
        public bvec4 zyzx {
            get {
                return new bvec4(this.v2, this.v1, this.v2, this.v0);
            }
        }
        
        public bvec4 bgbr {
            get {
                return new bvec4(this.v2, this.v1, this.v2, this.v0);
            }
        }
        
        public bvec4 ptps {
            get {
                return new bvec4(this.v2, this.v1, this.v2, this.v0);
            }
        }
        
        public bvec4 zyzy {
            get {
                return new bvec4(this.v2, this.v1, this.v2, this.v1);
            }
        }
        
        public bvec4 bgbg {
            get {
                return new bvec4(this.v2, this.v1, this.v2, this.v1);
            }
        }
        
        public bvec4 ptpt {
            get {
                return new bvec4(this.v2, this.v1, this.v2, this.v1);
            }
        }
        
        public bvec4 zyzz {
            get {
                return new bvec4(this.v2, this.v1, this.v2, this.v2);
            }
        }
        
        public bvec4 bgbb {
            get {
                return new bvec4(this.v2, this.v1, this.v2, this.v2);
            }
        }
        
        public bvec4 ptpp {
            get {
                return new bvec4(this.v2, this.v1, this.v2, this.v2);
            }
        }
        
        public bvec4 zyzw {
            get {
                return new bvec4(this.v2, this.v1, this.v2, this.v3);
            }
        }
        
        public bvec4 bgba {
            get {
                return new bvec4(this.v2, this.v1, this.v2, this.v3);
            }
        }
        
        public bvec4 ptpq {
            get {
                return new bvec4(this.v2, this.v1, this.v2, this.v3);
            }
        }
        
        public bvec4 zywx {
            get {
                return new bvec4(this.v2, this.v1, this.v3, this.v0);
            }
            set {
                this.v2 = value.v0;
                this.v1 = value.v1;
                this.v3 = value.v2;
                this.v0 = value.v3;
            }
        }
        
        public bvec4 bgar {
            get {
                return new bvec4(this.v2, this.v1, this.v3, this.v0);
            }
            set {
                this.v2 = value.v0;
                this.v1 = value.v1;
                this.v3 = value.v2;
                this.v0 = value.v3;
            }
        }
        
        public bvec4 ptqs {
            get {
                return new bvec4(this.v2, this.v1, this.v3, this.v0);
            }
            set {
                this.v2 = value.v0;
                this.v1 = value.v1;
                this.v3 = value.v2;
                this.v0 = value.v3;
            }
        }
        
        public bvec4 zywy {
            get {
                return new bvec4(this.v2, this.v1, this.v3, this.v1);
            }
        }
        
        public bvec4 bgag {
            get {
                return new bvec4(this.v2, this.v1, this.v3, this.v1);
            }
        }
        
        public bvec4 ptqt {
            get {
                return new bvec4(this.v2, this.v1, this.v3, this.v1);
            }
        }
        
        public bvec4 zywz {
            get {
                return new bvec4(this.v2, this.v1, this.v3, this.v2);
            }
        }
        
        public bvec4 bgab {
            get {
                return new bvec4(this.v2, this.v1, this.v3, this.v2);
            }
        }
        
        public bvec4 ptqp {
            get {
                return new bvec4(this.v2, this.v1, this.v3, this.v2);
            }
        }
        
        public bvec4 zyww {
            get {
                return new bvec4(this.v2, this.v1, this.v3, this.v3);
            }
        }
        
        public bvec4 bgaa {
            get {
                return new bvec4(this.v2, this.v1, this.v3, this.v3);
            }
        }
        
        public bvec4 ptqq {
            get {
                return new bvec4(this.v2, this.v1, this.v3, this.v3);
            }
        }
        
        public bvec4 zzxx {
            get {
                return new bvec4(this.v2, this.v2, this.v0, this.v0);
            }
        }
        
        public bvec4 bbrr {
            get {
                return new bvec4(this.v2, this.v2, this.v0, this.v0);
            }
        }
        
        public bvec4 ppss {
            get {
                return new bvec4(this.v2, this.v2, this.v0, this.v0);
            }
        }
        
        public bvec4 zzxy {
            get {
                return new bvec4(this.v2, this.v2, this.v0, this.v1);
            }
        }
        
        public bvec4 bbrg {
            get {
                return new bvec4(this.v2, this.v2, this.v0, this.v1);
            }
        }
        
        public bvec4 ppst {
            get {
                return new bvec4(this.v2, this.v2, this.v0, this.v1);
            }
        }
        
        public bvec4 zzxz {
            get {
                return new bvec4(this.v2, this.v2, this.v0, this.v2);
            }
        }
        
        public bvec4 bbrb {
            get {
                return new bvec4(this.v2, this.v2, this.v0, this.v2);
            }
        }
        
        public bvec4 ppsp {
            get {
                return new bvec4(this.v2, this.v2, this.v0, this.v2);
            }
        }
        
        public bvec4 zzxw {
            get {
                return new bvec4(this.v2, this.v2, this.v0, this.v3);
            }
        }
        
        public bvec4 bbra {
            get {
                return new bvec4(this.v2, this.v2, this.v0, this.v3);
            }
        }
        
        public bvec4 ppsq {
            get {
                return new bvec4(this.v2, this.v2, this.v0, this.v3);
            }
        }
        
        public bvec4 zzyx {
            get {
                return new bvec4(this.v2, this.v2, this.v1, this.v0);
            }
        }
        
        public bvec4 bbgr {
            get {
                return new bvec4(this.v2, this.v2, this.v1, this.v0);
            }
        }
        
        public bvec4 ppts {
            get {
                return new bvec4(this.v2, this.v2, this.v1, this.v0);
            }
        }
        
        public bvec4 zzyy {
            get {
                return new bvec4(this.v2, this.v2, this.v1, this.v1);
            }
        }
        
        public bvec4 bbgg {
            get {
                return new bvec4(this.v2, this.v2, this.v1, this.v1);
            }
        }
        
        public bvec4 pptt {
            get {
                return new bvec4(this.v2, this.v2, this.v1, this.v1);
            }
        }
        
        public bvec4 zzyz {
            get {
                return new bvec4(this.v2, this.v2, this.v1, this.v2);
            }
        }
        
        public bvec4 bbgb {
            get {
                return new bvec4(this.v2, this.v2, this.v1, this.v2);
            }
        }
        
        public bvec4 pptp {
            get {
                return new bvec4(this.v2, this.v2, this.v1, this.v2);
            }
        }
        
        public bvec4 zzyw {
            get {
                return new bvec4(this.v2, this.v2, this.v1, this.v3);
            }
        }
        
        public bvec4 bbga {
            get {
                return new bvec4(this.v2, this.v2, this.v1, this.v3);
            }
        }
        
        public bvec4 pptq {
            get {
                return new bvec4(this.v2, this.v2, this.v1, this.v3);
            }
        }
        
        public bvec4 zzzx {
            get {
                return new bvec4(this.v2, this.v2, this.v2, this.v0);
            }
        }
        
        public bvec4 bbbr {
            get {
                return new bvec4(this.v2, this.v2, this.v2, this.v0);
            }
        }
        
        public bvec4 ppps {
            get {
                return new bvec4(this.v2, this.v2, this.v2, this.v0);
            }
        }
        
        public bvec4 zzzy {
            get {
                return new bvec4(this.v2, this.v2, this.v2, this.v1);
            }
        }
        
        public bvec4 bbbg {
            get {
                return new bvec4(this.v2, this.v2, this.v2, this.v1);
            }
        }
        
        public bvec4 pppt {
            get {
                return new bvec4(this.v2, this.v2, this.v2, this.v1);
            }
        }
        
        public bvec4 zzzz {
            get {
                return new bvec4(this.v2, this.v2, this.v2, this.v2);
            }
        }
        
        public bvec4 bbbb {
            get {
                return new bvec4(this.v2, this.v2, this.v2, this.v2);
            }
        }
        
        public bvec4 pppp {
            get {
                return new bvec4(this.v2, this.v2, this.v2, this.v2);
            }
        }
        
        public bvec4 zzzw {
            get {
                return new bvec4(this.v2, this.v2, this.v2, this.v3);
            }
        }
        
        public bvec4 bbba {
            get {
                return new bvec4(this.v2, this.v2, this.v2, this.v3);
            }
        }
        
        public bvec4 pppq {
            get {
                return new bvec4(this.v2, this.v2, this.v2, this.v3);
            }
        }
        
        public bvec4 zzwx {
            get {
                return new bvec4(this.v2, this.v2, this.v3, this.v0);
            }
        }
        
        public bvec4 bbar {
            get {
                return new bvec4(this.v2, this.v2, this.v3, this.v0);
            }
        }
        
        public bvec4 ppqs {
            get {
                return new bvec4(this.v2, this.v2, this.v3, this.v0);
            }
        }
        
        public bvec4 zzwy {
            get {
                return new bvec4(this.v2, this.v2, this.v3, this.v1);
            }
        }
        
        public bvec4 bbag {
            get {
                return new bvec4(this.v2, this.v2, this.v3, this.v1);
            }
        }
        
        public bvec4 ppqt {
            get {
                return new bvec4(this.v2, this.v2, this.v3, this.v1);
            }
        }
        
        public bvec4 zzwz {
            get {
                return new bvec4(this.v2, this.v2, this.v3, this.v2);
            }
        }
        
        public bvec4 bbab {
            get {
                return new bvec4(this.v2, this.v2, this.v3, this.v2);
            }
        }
        
        public bvec4 ppqp {
            get {
                return new bvec4(this.v2, this.v2, this.v3, this.v2);
            }
        }
        
        public bvec4 zzww {
            get {
                return new bvec4(this.v2, this.v2, this.v3, this.v3);
            }
        }
        
        public bvec4 bbaa {
            get {
                return new bvec4(this.v2, this.v2, this.v3, this.v3);
            }
        }
        
        public bvec4 ppqq {
            get {
                return new bvec4(this.v2, this.v2, this.v3, this.v3);
            }
        }
        
        public bvec4 zwxx {
            get {
                return new bvec4(this.v2, this.v3, this.v0, this.v0);
            }
        }
        
        public bvec4 barr {
            get {
                return new bvec4(this.v2, this.v3, this.v0, this.v0);
            }
        }
        
        public bvec4 pqss {
            get {
                return new bvec4(this.v2, this.v3, this.v0, this.v0);
            }
        }
        
        public bvec4 zwxy {
            get {
                return new bvec4(this.v2, this.v3, this.v0, this.v1);
            }
            set {
                this.v2 = value.v0;
                this.v3 = value.v1;
                this.v0 = value.v2;
                this.v1 = value.v3;
            }
        }
        
        public bvec4 barg {
            get {
                return new bvec4(this.v2, this.v3, this.v0, this.v1);
            }
            set {
                this.v2 = value.v0;
                this.v3 = value.v1;
                this.v0 = value.v2;
                this.v1 = value.v3;
            }
        }
        
        public bvec4 pqst {
            get {
                return new bvec4(this.v2, this.v3, this.v0, this.v1);
            }
            set {
                this.v2 = value.v0;
                this.v3 = value.v1;
                this.v0 = value.v2;
                this.v1 = value.v3;
            }
        }
        
        public bvec4 zwxz {
            get {
                return new bvec4(this.v2, this.v3, this.v0, this.v2);
            }
        }
        
        public bvec4 barb {
            get {
                return new bvec4(this.v2, this.v3, this.v0, this.v2);
            }
        }
        
        public bvec4 pqsp {
            get {
                return new bvec4(this.v2, this.v3, this.v0, this.v2);
            }
        }
        
        public bvec4 zwxw {
            get {
                return new bvec4(this.v2, this.v3, this.v0, this.v3);
            }
        }
        
        public bvec4 bara {
            get {
                return new bvec4(this.v2, this.v3, this.v0, this.v3);
            }
        }
        
        public bvec4 pqsq {
            get {
                return new bvec4(this.v2, this.v3, this.v0, this.v3);
            }
        }
        
        public bvec4 zwyx {
            get {
                return new bvec4(this.v2, this.v3, this.v1, this.v0);
            }
            set {
                this.v2 = value.v0;
                this.v3 = value.v1;
                this.v1 = value.v2;
                this.v0 = value.v3;
            }
        }
        
        public bvec4 bagr {
            get {
                return new bvec4(this.v2, this.v3, this.v1, this.v0);
            }
            set {
                this.v2 = value.v0;
                this.v3 = value.v1;
                this.v1 = value.v2;
                this.v0 = value.v3;
            }
        }
        
        public bvec4 pqts {
            get {
                return new bvec4(this.v2, this.v3, this.v1, this.v0);
            }
            set {
                this.v2 = value.v0;
                this.v3 = value.v1;
                this.v1 = value.v2;
                this.v0 = value.v3;
            }
        }
        
        public bvec4 zwyy {
            get {
                return new bvec4(this.v2, this.v3, this.v1, this.v1);
            }
        }
        
        public bvec4 bagg {
            get {
                return new bvec4(this.v2, this.v3, this.v1, this.v1);
            }
        }
        
        public bvec4 pqtt {
            get {
                return new bvec4(this.v2, this.v3, this.v1, this.v1);
            }
        }
        
        public bvec4 zwyz {
            get {
                return new bvec4(this.v2, this.v3, this.v1, this.v2);
            }
        }
        
        public bvec4 bagb {
            get {
                return new bvec4(this.v2, this.v3, this.v1, this.v2);
            }
        }
        
        public bvec4 pqtp {
            get {
                return new bvec4(this.v2, this.v3, this.v1, this.v2);
            }
        }
        
        public bvec4 zwyw {
            get {
                return new bvec4(this.v2, this.v3, this.v1, this.v3);
            }
        }
        
        public bvec4 baga {
            get {
                return new bvec4(this.v2, this.v3, this.v1, this.v3);
            }
        }
        
        public bvec4 pqtq {
            get {
                return new bvec4(this.v2, this.v3, this.v1, this.v3);
            }
        }
        
        public bvec4 zwzx {
            get {
                return new bvec4(this.v2, this.v3, this.v2, this.v0);
            }
        }
        
        public bvec4 babr {
            get {
                return new bvec4(this.v2, this.v3, this.v2, this.v0);
            }
        }
        
        public bvec4 pqps {
            get {
                return new bvec4(this.v2, this.v3, this.v2, this.v0);
            }
        }
        
        public bvec4 zwzy {
            get {
                return new bvec4(this.v2, this.v3, this.v2, this.v1);
            }
        }
        
        public bvec4 babg {
            get {
                return new bvec4(this.v2, this.v3, this.v2, this.v1);
            }
        }
        
        public bvec4 pqpt {
            get {
                return new bvec4(this.v2, this.v3, this.v2, this.v1);
            }
        }
        
        public bvec4 zwzz {
            get {
                return new bvec4(this.v2, this.v3, this.v2, this.v2);
            }
        }
        
        public bvec4 babb {
            get {
                return new bvec4(this.v2, this.v3, this.v2, this.v2);
            }
        }
        
        public bvec4 pqpp {
            get {
                return new bvec4(this.v2, this.v3, this.v2, this.v2);
            }
        }
        
        public bvec4 zwzw {
            get {
                return new bvec4(this.v2, this.v3, this.v2, this.v3);
            }
        }
        
        public bvec4 baba {
            get {
                return new bvec4(this.v2, this.v3, this.v2, this.v3);
            }
        }
        
        public bvec4 pqpq {
            get {
                return new bvec4(this.v2, this.v3, this.v2, this.v3);
            }
        }
        
        public bvec4 zwwx {
            get {
                return new bvec4(this.v2, this.v3, this.v3, this.v0);
            }
        }
        
        public bvec4 baar {
            get {
                return new bvec4(this.v2, this.v3, this.v3, this.v0);
            }
        }
        
        public bvec4 pqqs {
            get {
                return new bvec4(this.v2, this.v3, this.v3, this.v0);
            }
        }
        
        public bvec4 zwwy {
            get {
                return new bvec4(this.v2, this.v3, this.v3, this.v1);
            }
        }
        
        public bvec4 baag {
            get {
                return new bvec4(this.v2, this.v3, this.v3, this.v1);
            }
        }
        
        public bvec4 pqqt {
            get {
                return new bvec4(this.v2, this.v3, this.v3, this.v1);
            }
        }
        
        public bvec4 zwwz {
            get {
                return new bvec4(this.v2, this.v3, this.v3, this.v2);
            }
        }
        
        public bvec4 baab {
            get {
                return new bvec4(this.v2, this.v3, this.v3, this.v2);
            }
        }
        
        public bvec4 pqqp {
            get {
                return new bvec4(this.v2, this.v3, this.v3, this.v2);
            }
        }
        
        public bvec4 zwww {
            get {
                return new bvec4(this.v2, this.v3, this.v3, this.v3);
            }
        }
        
        public bvec4 baaa {
            get {
                return new bvec4(this.v2, this.v3, this.v3, this.v3);
            }
        }
        
        public bvec4 pqqq {
            get {
                return new bvec4(this.v2, this.v3, this.v3, this.v3);
            }
        }
        
        public bvec4 wxxx {
            get {
                return new bvec4(this.v3, this.v0, this.v0, this.v0);
            }
        }
        
        public bvec4 arrr {
            get {
                return new bvec4(this.v3, this.v0, this.v0, this.v0);
            }
        }
        
        public bvec4 qsss {
            get {
                return new bvec4(this.v3, this.v0, this.v0, this.v0);
            }
        }
        
        public bvec4 wxxy {
            get {
                return new bvec4(this.v3, this.v0, this.v0, this.v1);
            }
        }
        
        public bvec4 arrg {
            get {
                return new bvec4(this.v3, this.v0, this.v0, this.v1);
            }
        }
        
        public bvec4 qsst {
            get {
                return new bvec4(this.v3, this.v0, this.v0, this.v1);
            }
        }
        
        public bvec4 wxxz {
            get {
                return new bvec4(this.v3, this.v0, this.v0, this.v2);
            }
        }
        
        public bvec4 arrb {
            get {
                return new bvec4(this.v3, this.v0, this.v0, this.v2);
            }
        }
        
        public bvec4 qssp {
            get {
                return new bvec4(this.v3, this.v0, this.v0, this.v2);
            }
        }
        
        public bvec4 wxxw {
            get {
                return new bvec4(this.v3, this.v0, this.v0, this.v3);
            }
        }
        
        public bvec4 arra {
            get {
                return new bvec4(this.v3, this.v0, this.v0, this.v3);
            }
        }
        
        public bvec4 qssq {
            get {
                return new bvec4(this.v3, this.v0, this.v0, this.v3);
            }
        }
        
        public bvec4 wxyx {
            get {
                return new bvec4(this.v3, this.v0, this.v1, this.v0);
            }
        }
        
        public bvec4 argr {
            get {
                return new bvec4(this.v3, this.v0, this.v1, this.v0);
            }
        }
        
        public bvec4 qsts {
            get {
                return new bvec4(this.v3, this.v0, this.v1, this.v0);
            }
        }
        
        public bvec4 wxyy {
            get {
                return new bvec4(this.v3, this.v0, this.v1, this.v1);
            }
        }
        
        public bvec4 argg {
            get {
                return new bvec4(this.v3, this.v0, this.v1, this.v1);
            }
        }
        
        public bvec4 qstt {
            get {
                return new bvec4(this.v3, this.v0, this.v1, this.v1);
            }
        }
        
        public bvec4 wxyz {
            get {
                return new bvec4(this.v3, this.v0, this.v1, this.v2);
            }
            set {
                this.v3 = value.v0;
                this.v0 = value.v1;
                this.v1 = value.v2;
                this.v2 = value.v3;
            }
        }
        
        public bvec4 argb {
            get {
                return new bvec4(this.v3, this.v0, this.v1, this.v2);
            }
            set {
                this.v3 = value.v0;
                this.v0 = value.v1;
                this.v1 = value.v2;
                this.v2 = value.v3;
            }
        }
        
        public bvec4 qstp {
            get {
                return new bvec4(this.v3, this.v0, this.v1, this.v2);
            }
            set {
                this.v3 = value.v0;
                this.v0 = value.v1;
                this.v1 = value.v2;
                this.v2 = value.v3;
            }
        }
        
        public bvec4 wxyw {
            get {
                return new bvec4(this.v3, this.v0, this.v1, this.v3);
            }
        }
        
        public bvec4 arga {
            get {
                return new bvec4(this.v3, this.v0, this.v1, this.v3);
            }
        }
        
        public bvec4 qstq {
            get {
                return new bvec4(this.v3, this.v0, this.v1, this.v3);
            }
        }
        
        public bvec4 wxzx {
            get {
                return new bvec4(this.v3, this.v0, this.v2, this.v0);
            }
        }
        
        public bvec4 arbr {
            get {
                return new bvec4(this.v3, this.v0, this.v2, this.v0);
            }
        }
        
        public bvec4 qsps {
            get {
                return new bvec4(this.v3, this.v0, this.v2, this.v0);
            }
        }
        
        public bvec4 wxzy {
            get {
                return new bvec4(this.v3, this.v0, this.v2, this.v1);
            }
            set {
                this.v3 = value.v0;
                this.v0 = value.v1;
                this.v2 = value.v2;
                this.v1 = value.v3;
            }
        }
        
        public bvec4 arbg {
            get {
                return new bvec4(this.v3, this.v0, this.v2, this.v1);
            }
            set {
                this.v3 = value.v0;
                this.v0 = value.v1;
                this.v2 = value.v2;
                this.v1 = value.v3;
            }
        }
        
        public bvec4 qspt {
            get {
                return new bvec4(this.v3, this.v0, this.v2, this.v1);
            }
            set {
                this.v3 = value.v0;
                this.v0 = value.v1;
                this.v2 = value.v2;
                this.v1 = value.v3;
            }
        }
        
        public bvec4 wxzz {
            get {
                return new bvec4(this.v3, this.v0, this.v2, this.v2);
            }
        }
        
        public bvec4 arbb {
            get {
                return new bvec4(this.v3, this.v0, this.v2, this.v2);
            }
        }
        
        public bvec4 qspp {
            get {
                return new bvec4(this.v3, this.v0, this.v2, this.v2);
            }
        }
        
        public bvec4 wxzw {
            get {
                return new bvec4(this.v3, this.v0, this.v2, this.v3);
            }
        }
        
        public bvec4 arba {
            get {
                return new bvec4(this.v3, this.v0, this.v2, this.v3);
            }
        }
        
        public bvec4 qspq {
            get {
                return new bvec4(this.v3, this.v0, this.v2, this.v3);
            }
        }
        
        public bvec4 wxwx {
            get {
                return new bvec4(this.v3, this.v0, this.v3, this.v0);
            }
        }
        
        public bvec4 arar {
            get {
                return new bvec4(this.v3, this.v0, this.v3, this.v0);
            }
        }
        
        public bvec4 qsqs {
            get {
                return new bvec4(this.v3, this.v0, this.v3, this.v0);
            }
        }
        
        public bvec4 wxwy {
            get {
                return new bvec4(this.v3, this.v0, this.v3, this.v1);
            }
        }
        
        public bvec4 arag {
            get {
                return new bvec4(this.v3, this.v0, this.v3, this.v1);
            }
        }
        
        public bvec4 qsqt {
            get {
                return new bvec4(this.v3, this.v0, this.v3, this.v1);
            }
        }
        
        public bvec4 wxwz {
            get {
                return new bvec4(this.v3, this.v0, this.v3, this.v2);
            }
        }
        
        public bvec4 arab {
            get {
                return new bvec4(this.v3, this.v0, this.v3, this.v2);
            }
        }
        
        public bvec4 qsqp {
            get {
                return new bvec4(this.v3, this.v0, this.v3, this.v2);
            }
        }
        
        public bvec4 wxww {
            get {
                return new bvec4(this.v3, this.v0, this.v3, this.v3);
            }
        }
        
        public bvec4 araa {
            get {
                return new bvec4(this.v3, this.v0, this.v3, this.v3);
            }
        }
        
        public bvec4 qsqq {
            get {
                return new bvec4(this.v3, this.v0, this.v3, this.v3);
            }
        }
        
        public bvec4 wyxx {
            get {
                return new bvec4(this.v3, this.v1, this.v0, this.v0);
            }
        }
        
        public bvec4 agrr {
            get {
                return new bvec4(this.v3, this.v1, this.v0, this.v0);
            }
        }
        
        public bvec4 qtss {
            get {
                return new bvec4(this.v3, this.v1, this.v0, this.v0);
            }
        }
        
        public bvec4 wyxy {
            get {
                return new bvec4(this.v3, this.v1, this.v0, this.v1);
            }
        }
        
        public bvec4 agrg {
            get {
                return new bvec4(this.v3, this.v1, this.v0, this.v1);
            }
        }
        
        public bvec4 qtst {
            get {
                return new bvec4(this.v3, this.v1, this.v0, this.v1);
            }
        }
        
        public bvec4 wyxz {
            get {
                return new bvec4(this.v3, this.v1, this.v0, this.v2);
            }
            set {
                this.v3 = value.v0;
                this.v1 = value.v1;
                this.v0 = value.v2;
                this.v2 = value.v3;
            }
        }
        
        public bvec4 agrb {
            get {
                return new bvec4(this.v3, this.v1, this.v0, this.v2);
            }
            set {
                this.v3 = value.v0;
                this.v1 = value.v1;
                this.v0 = value.v2;
                this.v2 = value.v3;
            }
        }
        
        public bvec4 qtsp {
            get {
                return new bvec4(this.v3, this.v1, this.v0, this.v2);
            }
            set {
                this.v3 = value.v0;
                this.v1 = value.v1;
                this.v0 = value.v2;
                this.v2 = value.v3;
            }
        }
        
        public bvec4 wyxw {
            get {
                return new bvec4(this.v3, this.v1, this.v0, this.v3);
            }
        }
        
        public bvec4 agra {
            get {
                return new bvec4(this.v3, this.v1, this.v0, this.v3);
            }
        }
        
        public bvec4 qtsq {
            get {
                return new bvec4(this.v3, this.v1, this.v0, this.v3);
            }
        }
        
        public bvec4 wyyx {
            get {
                return new bvec4(this.v3, this.v1, this.v1, this.v0);
            }
        }
        
        public bvec4 aggr {
            get {
                return new bvec4(this.v3, this.v1, this.v1, this.v0);
            }
        }
        
        public bvec4 qtts {
            get {
                return new bvec4(this.v3, this.v1, this.v1, this.v0);
            }
        }
        
        public bvec4 wyyy {
            get {
                return new bvec4(this.v3, this.v1, this.v1, this.v1);
            }
        }
        
        public bvec4 aggg {
            get {
                return new bvec4(this.v3, this.v1, this.v1, this.v1);
            }
        }
        
        public bvec4 qttt {
            get {
                return new bvec4(this.v3, this.v1, this.v1, this.v1);
            }
        }
        
        public bvec4 wyyz {
            get {
                return new bvec4(this.v3, this.v1, this.v1, this.v2);
            }
        }
        
        public bvec4 aggb {
            get {
                return new bvec4(this.v3, this.v1, this.v1, this.v2);
            }
        }
        
        public bvec4 qttp {
            get {
                return new bvec4(this.v3, this.v1, this.v1, this.v2);
            }
        }
        
        public bvec4 wyyw {
            get {
                return new bvec4(this.v3, this.v1, this.v1, this.v3);
            }
        }
        
        public bvec4 agga {
            get {
                return new bvec4(this.v3, this.v1, this.v1, this.v3);
            }
        }
        
        public bvec4 qttq {
            get {
                return new bvec4(this.v3, this.v1, this.v1, this.v3);
            }
        }
        
        public bvec4 wyzx {
            get {
                return new bvec4(this.v3, this.v1, this.v2, this.v0);
            }
            set {
                this.v3 = value.v0;
                this.v1 = value.v1;
                this.v2 = value.v2;
                this.v0 = value.v3;
            }
        }
        
        public bvec4 agbr {
            get {
                return new bvec4(this.v3, this.v1, this.v2, this.v0);
            }
            set {
                this.v3 = value.v0;
                this.v1 = value.v1;
                this.v2 = value.v2;
                this.v0 = value.v3;
            }
        }
        
        public bvec4 qtps {
            get {
                return new bvec4(this.v3, this.v1, this.v2, this.v0);
            }
            set {
                this.v3 = value.v0;
                this.v1 = value.v1;
                this.v2 = value.v2;
                this.v0 = value.v3;
            }
        }
        
        public bvec4 wyzy {
            get {
                return new bvec4(this.v3, this.v1, this.v2, this.v1);
            }
        }
        
        public bvec4 agbg {
            get {
                return new bvec4(this.v3, this.v1, this.v2, this.v1);
            }
        }
        
        public bvec4 qtpt {
            get {
                return new bvec4(this.v3, this.v1, this.v2, this.v1);
            }
        }
        
        public bvec4 wyzz {
            get {
                return new bvec4(this.v3, this.v1, this.v2, this.v2);
            }
        }
        
        public bvec4 agbb {
            get {
                return new bvec4(this.v3, this.v1, this.v2, this.v2);
            }
        }
        
        public bvec4 qtpp {
            get {
                return new bvec4(this.v3, this.v1, this.v2, this.v2);
            }
        }
        
        public bvec4 wyzw {
            get {
                return new bvec4(this.v3, this.v1, this.v2, this.v3);
            }
        }
        
        public bvec4 agba {
            get {
                return new bvec4(this.v3, this.v1, this.v2, this.v3);
            }
        }
        
        public bvec4 qtpq {
            get {
                return new bvec4(this.v3, this.v1, this.v2, this.v3);
            }
        }
        
        public bvec4 wywx {
            get {
                return new bvec4(this.v3, this.v1, this.v3, this.v0);
            }
        }
        
        public bvec4 agar {
            get {
                return new bvec4(this.v3, this.v1, this.v3, this.v0);
            }
        }
        
        public bvec4 qtqs {
            get {
                return new bvec4(this.v3, this.v1, this.v3, this.v0);
            }
        }
        
        public bvec4 wywy {
            get {
                return new bvec4(this.v3, this.v1, this.v3, this.v1);
            }
        }
        
        public bvec4 agag {
            get {
                return new bvec4(this.v3, this.v1, this.v3, this.v1);
            }
        }
        
        public bvec4 qtqt {
            get {
                return new bvec4(this.v3, this.v1, this.v3, this.v1);
            }
        }
        
        public bvec4 wywz {
            get {
                return new bvec4(this.v3, this.v1, this.v3, this.v2);
            }
        }
        
        public bvec4 agab {
            get {
                return new bvec4(this.v3, this.v1, this.v3, this.v2);
            }
        }
        
        public bvec4 qtqp {
            get {
                return new bvec4(this.v3, this.v1, this.v3, this.v2);
            }
        }
        
        public bvec4 wyww {
            get {
                return new bvec4(this.v3, this.v1, this.v3, this.v3);
            }
        }
        
        public bvec4 agaa {
            get {
                return new bvec4(this.v3, this.v1, this.v3, this.v3);
            }
        }
        
        public bvec4 qtqq {
            get {
                return new bvec4(this.v3, this.v1, this.v3, this.v3);
            }
        }
        
        public bvec4 wzxx {
            get {
                return new bvec4(this.v3, this.v2, this.v0, this.v0);
            }
        }
        
        public bvec4 abrr {
            get {
                return new bvec4(this.v3, this.v2, this.v0, this.v0);
            }
        }
        
        public bvec4 qpss {
            get {
                return new bvec4(this.v3, this.v2, this.v0, this.v0);
            }
        }
        
        public bvec4 wzxy {
            get {
                return new bvec4(this.v3, this.v2, this.v0, this.v1);
            }
            set {
                this.v3 = value.v0;
                this.v2 = value.v1;
                this.v0 = value.v2;
                this.v1 = value.v3;
            }
        }
        
        public bvec4 abrg {
            get {
                return new bvec4(this.v3, this.v2, this.v0, this.v1);
            }
            set {
                this.v3 = value.v0;
                this.v2 = value.v1;
                this.v0 = value.v2;
                this.v1 = value.v3;
            }
        }
        
        public bvec4 qpst {
            get {
                return new bvec4(this.v3, this.v2, this.v0, this.v1);
            }
            set {
                this.v3 = value.v0;
                this.v2 = value.v1;
                this.v0 = value.v2;
                this.v1 = value.v3;
            }
        }
        
        public bvec4 wzxz {
            get {
                return new bvec4(this.v3, this.v2, this.v0, this.v2);
            }
        }
        
        public bvec4 abrb {
            get {
                return new bvec4(this.v3, this.v2, this.v0, this.v2);
            }
        }
        
        public bvec4 qpsp {
            get {
                return new bvec4(this.v3, this.v2, this.v0, this.v2);
            }
        }
        
        public bvec4 wzxw {
            get {
                return new bvec4(this.v3, this.v2, this.v0, this.v3);
            }
        }
        
        public bvec4 abra {
            get {
                return new bvec4(this.v3, this.v2, this.v0, this.v3);
            }
        }
        
        public bvec4 qpsq {
            get {
                return new bvec4(this.v3, this.v2, this.v0, this.v3);
            }
        }
        
        public bvec4 wzyx {
            get {
                return new bvec4(this.v3, this.v2, this.v1, this.v0);
            }
            set {
                this.v3 = value.v0;
                this.v2 = value.v1;
                this.v1 = value.v2;
                this.v0 = value.v3;
            }
        }
        
        public bvec4 abgr {
            get {
                return new bvec4(this.v3, this.v2, this.v1, this.v0);
            }
            set {
                this.v3 = value.v0;
                this.v2 = value.v1;
                this.v1 = value.v2;
                this.v0 = value.v3;
            }
        }
        
        public bvec4 qpts {
            get {
                return new bvec4(this.v3, this.v2, this.v1, this.v0);
            }
            set {
                this.v3 = value.v0;
                this.v2 = value.v1;
                this.v1 = value.v2;
                this.v0 = value.v3;
            }
        }
        
        public bvec4 wzyy {
            get {
                return new bvec4(this.v3, this.v2, this.v1, this.v1);
            }
        }
        
        public bvec4 abgg {
            get {
                return new bvec4(this.v3, this.v2, this.v1, this.v1);
            }
        }
        
        public bvec4 qptt {
            get {
                return new bvec4(this.v3, this.v2, this.v1, this.v1);
            }
        }
        
        public bvec4 wzyz {
            get {
                return new bvec4(this.v3, this.v2, this.v1, this.v2);
            }
        }
        
        public bvec4 abgb {
            get {
                return new bvec4(this.v3, this.v2, this.v1, this.v2);
            }
        }
        
        public bvec4 qptp {
            get {
                return new bvec4(this.v3, this.v2, this.v1, this.v2);
            }
        }
        
        public bvec4 wzyw {
            get {
                return new bvec4(this.v3, this.v2, this.v1, this.v3);
            }
        }
        
        public bvec4 abga {
            get {
                return new bvec4(this.v3, this.v2, this.v1, this.v3);
            }
        }
        
        public bvec4 qptq {
            get {
                return new bvec4(this.v3, this.v2, this.v1, this.v3);
            }
        }
        
        public bvec4 wzzx {
            get {
                return new bvec4(this.v3, this.v2, this.v2, this.v0);
            }
        }
        
        public bvec4 abbr {
            get {
                return new bvec4(this.v3, this.v2, this.v2, this.v0);
            }
        }
        
        public bvec4 qpps {
            get {
                return new bvec4(this.v3, this.v2, this.v2, this.v0);
            }
        }
        
        public bvec4 wzzy {
            get {
                return new bvec4(this.v3, this.v2, this.v2, this.v1);
            }
        }
        
        public bvec4 abbg {
            get {
                return new bvec4(this.v3, this.v2, this.v2, this.v1);
            }
        }
        
        public bvec4 qppt {
            get {
                return new bvec4(this.v3, this.v2, this.v2, this.v1);
            }
        }
        
        public bvec4 wzzz {
            get {
                return new bvec4(this.v3, this.v2, this.v2, this.v2);
            }
        }
        
        public bvec4 abbb {
            get {
                return new bvec4(this.v3, this.v2, this.v2, this.v2);
            }
        }
        
        public bvec4 qppp {
            get {
                return new bvec4(this.v3, this.v2, this.v2, this.v2);
            }
        }
        
        public bvec4 wzzw {
            get {
                return new bvec4(this.v3, this.v2, this.v2, this.v3);
            }
        }
        
        public bvec4 abba {
            get {
                return new bvec4(this.v3, this.v2, this.v2, this.v3);
            }
        }
        
        public bvec4 qppq {
            get {
                return new bvec4(this.v3, this.v2, this.v2, this.v3);
            }
        }
        
        public bvec4 wzwx {
            get {
                return new bvec4(this.v3, this.v2, this.v3, this.v0);
            }
        }
        
        public bvec4 abar {
            get {
                return new bvec4(this.v3, this.v2, this.v3, this.v0);
            }
        }
        
        public bvec4 qpqs {
            get {
                return new bvec4(this.v3, this.v2, this.v3, this.v0);
            }
        }
        
        public bvec4 wzwy {
            get {
                return new bvec4(this.v3, this.v2, this.v3, this.v1);
            }
        }
        
        public bvec4 abag {
            get {
                return new bvec4(this.v3, this.v2, this.v3, this.v1);
            }
        }
        
        public bvec4 qpqt {
            get {
                return new bvec4(this.v3, this.v2, this.v3, this.v1);
            }
        }
        
        public bvec4 wzwz {
            get {
                return new bvec4(this.v3, this.v2, this.v3, this.v2);
            }
        }
        
        public bvec4 abab {
            get {
                return new bvec4(this.v3, this.v2, this.v3, this.v2);
            }
        }
        
        public bvec4 qpqp {
            get {
                return new bvec4(this.v3, this.v2, this.v3, this.v2);
            }
        }
        
        public bvec4 wzww {
            get {
                return new bvec4(this.v3, this.v2, this.v3, this.v3);
            }
        }
        
        public bvec4 abaa {
            get {
                return new bvec4(this.v3, this.v2, this.v3, this.v3);
            }
        }
        
        public bvec4 qpqq {
            get {
                return new bvec4(this.v3, this.v2, this.v3, this.v3);
            }
        }
        
        public bvec4 wwxx {
            get {
                return new bvec4(this.v3, this.v3, this.v0, this.v0);
            }
        }
        
        public bvec4 aarr {
            get {
                return new bvec4(this.v3, this.v3, this.v0, this.v0);
            }
        }
        
        public bvec4 qqss {
            get {
                return new bvec4(this.v3, this.v3, this.v0, this.v0);
            }
        }
        
        public bvec4 wwxy {
            get {
                return new bvec4(this.v3, this.v3, this.v0, this.v1);
            }
        }
        
        public bvec4 aarg {
            get {
                return new bvec4(this.v3, this.v3, this.v0, this.v1);
            }
        }
        
        public bvec4 qqst {
            get {
                return new bvec4(this.v3, this.v3, this.v0, this.v1);
            }
        }
        
        public bvec4 wwxz {
            get {
                return new bvec4(this.v3, this.v3, this.v0, this.v2);
            }
        }
        
        public bvec4 aarb {
            get {
                return new bvec4(this.v3, this.v3, this.v0, this.v2);
            }
        }
        
        public bvec4 qqsp {
            get {
                return new bvec4(this.v3, this.v3, this.v0, this.v2);
            }
        }
        
        public bvec4 wwxw {
            get {
                return new bvec4(this.v3, this.v3, this.v0, this.v3);
            }
        }
        
        public bvec4 aara {
            get {
                return new bvec4(this.v3, this.v3, this.v0, this.v3);
            }
        }
        
        public bvec4 qqsq {
            get {
                return new bvec4(this.v3, this.v3, this.v0, this.v3);
            }
        }
        
        public bvec4 wwyx {
            get {
                return new bvec4(this.v3, this.v3, this.v1, this.v0);
            }
        }
        
        public bvec4 aagr {
            get {
                return new bvec4(this.v3, this.v3, this.v1, this.v0);
            }
        }
        
        public bvec4 qqts {
            get {
                return new bvec4(this.v3, this.v3, this.v1, this.v0);
            }
        }
        
        public bvec4 wwyy {
            get {
                return new bvec4(this.v3, this.v3, this.v1, this.v1);
            }
        }
        
        public bvec4 aagg {
            get {
                return new bvec4(this.v3, this.v3, this.v1, this.v1);
            }
        }
        
        public bvec4 qqtt {
            get {
                return new bvec4(this.v3, this.v3, this.v1, this.v1);
            }
        }
        
        public bvec4 wwyz {
            get {
                return new bvec4(this.v3, this.v3, this.v1, this.v2);
            }
        }
        
        public bvec4 aagb {
            get {
                return new bvec4(this.v3, this.v3, this.v1, this.v2);
            }
        }
        
        public bvec4 qqtp {
            get {
                return new bvec4(this.v3, this.v3, this.v1, this.v2);
            }
        }
        
        public bvec4 wwyw {
            get {
                return new bvec4(this.v3, this.v3, this.v1, this.v3);
            }
        }
        
        public bvec4 aaga {
            get {
                return new bvec4(this.v3, this.v3, this.v1, this.v3);
            }
        }
        
        public bvec4 qqtq {
            get {
                return new bvec4(this.v3, this.v3, this.v1, this.v3);
            }
        }
        
        public bvec4 wwzx {
            get {
                return new bvec4(this.v3, this.v3, this.v2, this.v0);
            }
        }
        
        public bvec4 aabr {
            get {
                return new bvec4(this.v3, this.v3, this.v2, this.v0);
            }
        }
        
        public bvec4 qqps {
            get {
                return new bvec4(this.v3, this.v3, this.v2, this.v0);
            }
        }
        
        public bvec4 wwzy {
            get {
                return new bvec4(this.v3, this.v3, this.v2, this.v1);
            }
        }
        
        public bvec4 aabg {
            get {
                return new bvec4(this.v3, this.v3, this.v2, this.v1);
            }
        }
        
        public bvec4 qqpt {
            get {
                return new bvec4(this.v3, this.v3, this.v2, this.v1);
            }
        }
        
        public bvec4 wwzz {
            get {
                return new bvec4(this.v3, this.v3, this.v2, this.v2);
            }
        }
        
        public bvec4 aabb {
            get {
                return new bvec4(this.v3, this.v3, this.v2, this.v2);
            }
        }
        
        public bvec4 qqpp {
            get {
                return new bvec4(this.v3, this.v3, this.v2, this.v2);
            }
        }
        
        public bvec4 wwzw {
            get {
                return new bvec4(this.v3, this.v3, this.v2, this.v3);
            }
        }
        
        public bvec4 aaba {
            get {
                return new bvec4(this.v3, this.v3, this.v2, this.v3);
            }
        }
        
        public bvec4 qqpq {
            get {
                return new bvec4(this.v3, this.v3, this.v2, this.v3);
            }
        }
        
        public bvec4 wwwx {
            get {
                return new bvec4(this.v3, this.v3, this.v3, this.v0);
            }
        }
        
        public bvec4 aaar {
            get {
                return new bvec4(this.v3, this.v3, this.v3, this.v0);
            }
        }
        
        public bvec4 qqqs {
            get {
                return new bvec4(this.v3, this.v3, this.v3, this.v0);
            }
        }
        
        public bvec4 wwwy {
            get {
                return new bvec4(this.v3, this.v3, this.v3, this.v1);
            }
        }
        
        public bvec4 aaag {
            get {
                return new bvec4(this.v3, this.v3, this.v3, this.v1);
            }
        }
        
        public bvec4 qqqt {
            get {
                return new bvec4(this.v3, this.v3, this.v3, this.v1);
            }
        }
        
        public bvec4 wwwz {
            get {
                return new bvec4(this.v3, this.v3, this.v3, this.v2);
            }
        }
        
        public bvec4 aaab {
            get {
                return new bvec4(this.v3, this.v3, this.v3, this.v2);
            }
        }
        
        public bvec4 qqqp {
            get {
                return new bvec4(this.v3, this.v3, this.v3, this.v2);
            }
        }
        
        public bvec4 wwww {
            get {
                return new bvec4(this.v3, this.v3, this.v3, this.v3);
            }
        }
        
        public bvec4 aaaa {
            get {
                return new bvec4(this.v3, this.v3, this.v3, this.v3);
            }
        }
        
        public bvec4 qqqq {
            get {
                return new bvec4(this.v3, this.v3, this.v3, this.v3);
            }
        }
        
        public static bvec4 operator!(bvec4 v) {
            return new bvec4(!v.v0, !v.v1, !v.v2, !v.v3);
        }
        
        public static bvec4 operator&(bool s, bvec4 v2) {
            return new bvec4(s & v2.v0, s & v2.v1, s & v2.v2, s & v2.v3);
        }
        
        public static bvec4 operator|(bool s, bvec4 v2) {
            return new bvec4(s | v2.v0, s | v2.v1, s | v2.v2, s | v2.v3);
        }
        
        public static bvec4 operator^(bool s, bvec4 v2) {
            return new bvec4(s ^ v2.v0, s ^ v2.v1, s ^ v2.v2, s ^ v2.v3);
        }
        
        public static bvec4 operator&(bvec4 v1, bool s) {
            return new bvec4(v1.v0 & s, v1.v1 & s, v1.v2 & s, v1.v3 & s);
        }
        
        public static bvec4 operator|(bvec4 v1, bool s) {
            return new bvec4(v1.v0 | s, v1.v1 | s, v1.v2 | s, v1.v3 | s);
        }
        
        public static bvec4 operator^(bvec4 v1, bool s) {
            return new bvec4(v1.v0 ^ s, v1.v1 ^ s, v1.v2 ^ s, v1.v3 ^ s);
        }
        
        public static bvec4 operator&(bvec4 v1, bvec4 v2) {
            return new bvec4(v1.v0 & v2.v0, v1.v1 & v2.v1, v1.v2 & v2.v2, v1.v3 & v2.v3);
        }
        
        public static bvec4 operator|(bvec4 v1, bvec4 v2) {
            return new bvec4(v1.v0 | v2.v0, v1.v1 | v2.v1, v1.v2 | v2.v2, v1.v3 | v2.v3);
        }
        
        public static bvec4 operator^(bvec4 v1, bvec4 v2) {
            return new bvec4(v1.v0 ^ v2.v0, v1.v1 ^ v2.v1, v1.v2 ^ v2.v2, v1.v3 ^ v2.v3);
        }
        
        public static bvec4 operator==(bvec4 v1, bvec4 v2) {
            return new bvec4(v1.v0 == v2.v0, v1.v1 == v2.v1, v1.v2 == v2.v2, v1.v3 == v2.v3);
        }
        
        public static bvec4 operator!=(bvec4 v1, bvec4 v2) {
            return new bvec4(v1.v0 != v2.v0, v1.v1 != v2.v1, v1.v2 != v2.v2, v1.v3 != v2.v3);
        }
        
        public override bool Equals(object obj) {
            return (obj is bvec4) ? bmath.All(this == (bvec4)obj) : false;
        }
        
        public override int GetHashCode() {
            return this.v0.GetHashCode() ^ this.v1.GetHashCode() ^ this.v2.GetHashCode() ^ this.v3.GetHashCode();
        }
        
        public override string ToString() {
            return "bvec4(" + this.v0 + "," + this.v1 + "," + this.v2 + "," + this.v3 + ")";
        }
    }
}
