#region --- License ---
/*
Copyright 2010 Daniel Filonik

Permission is hereby granted, free of charge, to any person obtaining a copy of
this software and associated documentation files (the "Software"), to deal in
the Software without restriction, including without limitation the rights to
use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
of the Software, and to permit persons to whom the Software is furnished to do
so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all
copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
SOFTWARE.
*/
#endregion

namespace TKM {
    using System;
    using System.Runtime.InteropServices;
    using System.Runtime.Serialization;
    
    
    [DataContract()]
    [StructLayout(LayoutKind.Sequential)]
    public struct dmat2 {
        
        [DataMember()]
        internal double m00;
        
        [DataMember()]
        internal double m01;
        
        [DataMember()]
        internal double m10;
        
        [DataMember()]
        internal double m11;
        
        public dmat2(double s) {
            this.m00 = s;
            this.m01 = 0D;
            this.m10 = 0D;
            this.m11 = s;
        }
        
        public dmat2(dmat2 m00) {
            this.m00 = m00.m00;
            this.m01 = m00.m01;
            this.m10 = m00.m10;
            this.m11 = m00.m11;
        }
        
        public dmat2(dvec2 m00, dvec2 m10) {
            this.m00 = m00.v0;
            this.m01 = m00.v1;
            this.m10 = m10.v0;
            this.m11 = m10.v1;
        }
        
        public dmat2(double m00, double m01, double m10, double m11) {
            this.m00 = m00;
            this.m01 = m01;
            this.m10 = m10;
            this.m11 = m11;
        }
        
        public dmat2(dmat2x3 s) {
            this.m00 = s.m00;
            this.m01 = s.m01;
            this.m10 = s.m10;
            this.m11 = s.m11;
        }
        
        public dmat2(dmat2x4 s) {
            this.m00 = s.m00;
            this.m01 = s.m01;
            this.m10 = s.m10;
            this.m11 = s.m11;
        }
        
        public dmat2(dmat3x2 s) {
            this.m00 = s.m00;
            this.m01 = s.m01;
            this.m10 = s.m10;
            this.m11 = s.m11;
        }
        
        public dmat2(dmat3 s) {
            this.m00 = s.m00;
            this.m01 = s.m01;
            this.m10 = s.m10;
            this.m11 = s.m11;
        }
        
        public dmat2(dmat3x4 s) {
            this.m00 = s.m00;
            this.m01 = s.m01;
            this.m10 = s.m10;
            this.m11 = s.m11;
        }
        
        public dmat2(dmat4x2 s) {
            this.m00 = s.m00;
            this.m01 = s.m01;
            this.m10 = s.m10;
            this.m11 = s.m11;
        }
        
        public dmat2(dmat4x3 s) {
            this.m00 = s.m00;
            this.m01 = s.m01;
            this.m10 = s.m10;
            this.m11 = s.m11;
        }
        
        public dmat2(dmat4 s) {
            this.m00 = s.m00;
            this.m01 = s.m01;
            this.m10 = s.m10;
            this.m11 = s.m11;
        }
        
        public double this[int i0, int i1] {
            get {
                if (((i0 < 0) 
                            || (1 < i0))) {
                    throw new System.IndexOutOfRangeException("i0");
                }
                if (((i1 < 0) 
                            || (1 < i1))) {
                    throw new System.IndexOutOfRangeException("i1");
                }
                
                unsafe
                {
                    fixed (Double* v_ptr = &this.m00)
                    {
                        return *(v_ptr + (i1 + i0 * 2));
                    }
                };
            }
            set {
                if (((i0 < 0) 
                            || (1 < i0))) {
                    throw new System.IndexOutOfRangeException("i0");
                }
                if (((i1 < 0) 
                            || (1 < i1))) {
                    throw new System.IndexOutOfRangeException("i1");
                }
                
                unsafe
                {
                    fixed (Double* v_ptr = &this.m00)
                    {
                        *(v_ptr  + (i1 + i0 * 2)) = value;
                    }
                };
            }
        }
        
        public dvec2 this[int i0] {
            get {
                return new dvec2(this[i0, 0], this[i0, 1]);
            }
            set {
                this[i0, 0] = value[0];
                this[i0, 1] = value[1];
            }
        }
        
        public static dmat2 operator+(dmat2 v) {
            return new dmat2(+v.m00, +v.m01, +v.m10, +v.m11);
        }
        
        public static dmat2 operator-(dmat2 v) {
            return new dmat2(-v.m00, -v.m01, -v.m10, -v.m11);
        }
        
        public static dmat2 operator++(dmat2 v) {
            return new dmat2(++v.m00, ++v.m01, ++v.m10, ++v.m11);
        }
        
        public static dmat2 operator--(dmat2 v) {
            return new dmat2(--v.m00, --v.m01, --v.m10, --v.m11);
        }
        
        public static dmat2 operator+(double s, dmat2 v2) {
            return new dmat2(s + v2.m00, s + v2.m01, s + v2.m10, s + v2.m11);
        }
        
        public static dmat2 operator-(double s, dmat2 v2) {
            return new dmat2(s - v2.m00, s - v2.m01, s - v2.m10, s - v2.m11);
        }
        
        public static dmat2 operator*(double s, dmat2 v2) {
            return new dmat2(s * v2.m00, s * v2.m01, s * v2.m10, s * v2.m11);
        }
        
        public static dmat2 operator/(double s, dmat2 v2) {
            return new dmat2(s / v2.m00, s / v2.m01, s / v2.m10, s / v2.m11);
        }
        
        public static dmat2 operator%(double s, dmat2 v2) {
            return new dmat2(s % v2.m00, s % v2.m01, s % v2.m10, s % v2.m11);
        }
        
        public static dmat2 operator+(dmat2 v1, double s) {
            return new dmat2(v1.m00 + s, v1.m01 + s, v1.m10 + s, v1.m11 + s);
        }
        
        public static dmat2 operator-(dmat2 v1, double s) {
            return new dmat2(v1.m00 - s, v1.m01 - s, v1.m10 - s, v1.m11 - s);
        }
        
        public static dmat2 operator*(dmat2 v1, double s) {
            return new dmat2(v1.m00 * s, v1.m01 * s, v1.m10 * s, v1.m11 * s);
        }
        
        public static dmat2 operator/(dmat2 v1, double s) {
            return new dmat2(v1.m00 / s, v1.m01 / s, v1.m10 / s, v1.m11 / s);
        }
        
        public static dmat2 operator%(dmat2 v1, double s) {
            return new dmat2(v1.m00 % s, v1.m01 % s, v1.m10 % s, v1.m11 % s);
        }
        
        public static dmat2 operator+(dmat2 v1, dmat2 v2) {
            return new dmat2(v1.m00 + v2.m00, v1.m01 + v2.m01, v1.m10 + v2.m10, v1.m11 + v2.m11);
        }
        
        public static dmat2 operator-(dmat2 v1, dmat2 v2) {
            return new dmat2(v1.m00 - v2.m00, v1.m01 - v2.m01, v1.m10 - v2.m10, v1.m11 - v2.m11);
        }
        
        public static dmat2 operator/(dmat2 v1, dmat2 v2) {
            return new dmat2(v1.m00 / v2.m00, v1.m01 / v2.m01, v1.m10 / v2.m10, v1.m11 / v2.m11);
        }
        
        public static dmat2 operator%(dmat2 v1, dmat2 v2) {
            return new dmat2(v1.m00 % v2.m00, v1.m01 % v2.m01, v1.m10 % v2.m10, v1.m11 % v2.m11);
        }
        
        public static bmat2 operator==(dmat2 v1, dmat2 v2) {
            return new bmat2(v1.m00 == v2.m00, v1.m01 == v2.m01, v1.m10 == v2.m10, v1.m11 == v2.m11);
        }
        
        public static bmat2 operator>(dmat2 v1, dmat2 v2) {
            return new bmat2(v1.m00 > v2.m00, v1.m01 > v2.m01, v1.m10 > v2.m10, v1.m11 > v2.m11);
        }
        
        public static bmat2 operator>=(dmat2 v1, dmat2 v2) {
            return new bmat2(v1.m00 >= v2.m00, v1.m01 >= v2.m01, v1.m10 >= v2.m10, v1.m11 >= v2.m11);
        }
        
        public static bmat2 operator<(dmat2 v1, dmat2 v2) {
            return new bmat2(v1.m00 < v2.m00, v1.m01 < v2.m01, v1.m10 < v2.m10, v1.m11 < v2.m11);
        }
        
        public static bmat2 operator<=(dmat2 v1, dmat2 v2) {
            return new bmat2(v1.m00 <= v2.m00, v1.m01 <= v2.m01, v1.m10 <= v2.m10, v1.m11 <= v2.m11);
        }
        
        public static bmat2 operator!=(dmat2 v1, dmat2 v2) {
            return new bmat2(v1.m00 != v2.m00, v1.m01 != v2.m01, v1.m10 != v2.m10, v1.m11 != v2.m11);
        }
        
        public static dvec2 operator*(dvec2 v0, dmat2 m1) {
            return dmath.InnerProduct1(ref v0, ref m1);
        }
        
        public static dvec2 operator*(dmat2 m0, dvec2 v1) {
            return dmath.InnerProduct1(ref m0, ref v1);
        }
        
        public static dmat2 operator*(dmat2 m0, dmat2 m1) {
            return dmath.InnerProduct1(ref m0, ref m1);
        }
        
        public static dmat2x3 operator*(dmat2 m0, dmat2x3 m1) {
            return dmath.InnerProduct1(ref m0, ref m1);
        }
        
        public static dmat2x4 operator*(dmat2 m0, dmat2x4 m1) {
            return dmath.InnerProduct1(ref m0, ref m1);
        }
        
        public override bool Equals(object obj) {
            return (obj is dmat2) ? bmath.All(this == (dmat2)obj) : false;
        }
        
        public override int GetHashCode() {
            return this.m00.GetHashCode() ^ this.m01.GetHashCode() ^ this.m10.GetHashCode() ^ this.m11.GetHashCode();
        }
        
        public override string ToString() {
            return "dmat2(" + this.m00 + "," + this.m01 + "," + this.m10 + "," + this.m11 + ")";
        }
    }
    
    [DataContract()]
    [StructLayout(LayoutKind.Sequential)]
    public struct dmat2x3 {
        
        [DataMember()]
        internal double m00;
        
        [DataMember()]
        internal double m01;
        
        [DataMember()]
        internal double m02;
        
        [DataMember()]
        internal double m10;
        
        [DataMember()]
        internal double m11;
        
        [DataMember()]
        internal double m12;
        
        public dmat2x3(double s) {
            this.m00 = s;
            this.m01 = 0D;
            this.m02 = 0D;
            this.m10 = 0D;
            this.m11 = s;
            this.m12 = 0D;
        }
        
        public dmat2x3(dmat2x3 m00) {
            this.m00 = m00.m00;
            this.m01 = m00.m01;
            this.m02 = m00.m02;
            this.m10 = m00.m10;
            this.m11 = m00.m11;
            this.m12 = m00.m12;
        }
        
        public dmat2x3(dvec3 m00, dvec3 m10) {
            this.m00 = m00.v0;
            this.m01 = m00.v1;
            this.m02 = m00.v2;
            this.m10 = m10.v0;
            this.m11 = m10.v1;
            this.m12 = m10.v2;
        }
        
        public dmat2x3(dvec2 m00, double m01, dvec2 m10, double m11) {
            this.m00 = m00.v0;
            this.m01 = m00.v1;
            this.m02 = m01;
            this.m10 = m10.v0;
            this.m11 = m10.v1;
            this.m12 = m11;
        }
        
        public dmat2x3(double m00, dvec2 m01, double m10, dvec2 m11) {
            this.m00 = m00;
            this.m01 = m01.v0;
            this.m02 = m01.v1;
            this.m10 = m10;
            this.m11 = m11.v0;
            this.m12 = m11.v1;
        }
        
        public dmat2x3(double m00, double m01, double m02, double m10, double m11, double m12) {
            this.m00 = m00;
            this.m01 = m01;
            this.m02 = m02;
            this.m10 = m10;
            this.m11 = m11;
            this.m12 = m12;
        }
        
        public dmat2x3(dmat2 s) {
            this.m00 = s.m00;
            this.m01 = s.m01;
            this.m02 = 0D;
            this.m10 = s.m10;
            this.m11 = s.m11;
            this.m12 = 0D;
        }
        
        public dmat2x3(dmat2x4 s) {
            this.m00 = s.m00;
            this.m01 = s.m01;
            this.m02 = s.m02;
            this.m10 = s.m10;
            this.m11 = s.m11;
            this.m12 = s.m12;
        }
        
        public dmat2x3(dmat3x2 s) {
            this.m00 = s.m00;
            this.m01 = s.m01;
            this.m02 = 0D;
            this.m10 = s.m10;
            this.m11 = s.m11;
            this.m12 = 0D;
        }
        
        public dmat2x3(dmat3 s) {
            this.m00 = s.m00;
            this.m01 = s.m01;
            this.m02 = s.m02;
            this.m10 = s.m10;
            this.m11 = s.m11;
            this.m12 = s.m12;
        }
        
        public dmat2x3(dmat3x4 s) {
            this.m00 = s.m00;
            this.m01 = s.m01;
            this.m02 = s.m02;
            this.m10 = s.m10;
            this.m11 = s.m11;
            this.m12 = s.m12;
        }
        
        public dmat2x3(dmat4x2 s) {
            this.m00 = s.m00;
            this.m01 = s.m01;
            this.m02 = 0D;
            this.m10 = s.m10;
            this.m11 = s.m11;
            this.m12 = 0D;
        }
        
        public dmat2x3(dmat4x3 s) {
            this.m00 = s.m00;
            this.m01 = s.m01;
            this.m02 = s.m02;
            this.m10 = s.m10;
            this.m11 = s.m11;
            this.m12 = s.m12;
        }
        
        public dmat2x3(dmat4 s) {
            this.m00 = s.m00;
            this.m01 = s.m01;
            this.m02 = s.m02;
            this.m10 = s.m10;
            this.m11 = s.m11;
            this.m12 = s.m12;
        }
        
        public double this[int i0, int i1] {
            get {
                if (((i0 < 0) 
                            || (1 < i0))) {
                    throw new System.IndexOutOfRangeException("i0");
                }
                if (((i1 < 0) 
                            || (2 < i1))) {
                    throw new System.IndexOutOfRangeException("i1");
                }
                
                unsafe
                {
                    fixed (Double* v_ptr = &this.m00)
                    {
                        return *(v_ptr + (i1 + i0 * 3));
                    }
                };
            }
            set {
                if (((i0 < 0) 
                            || (1 < i0))) {
                    throw new System.IndexOutOfRangeException("i0");
                }
                if (((i1 < 0) 
                            || (2 < i1))) {
                    throw new System.IndexOutOfRangeException("i1");
                }
                
                unsafe
                {
                    fixed (Double* v_ptr = &this.m00)
                    {
                        *(v_ptr  + (i1 + i0 * 3)) = value;
                    }
                };
            }
        }
        
        public dvec3 this[int i0] {
            get {
                return new dvec3(this[i0, 0], this[i0, 1], this[i0, 2]);
            }
            set {
                this[i0, 0] = value[0];
                this[i0, 1] = value[1];
                this[i0, 2] = value[2];
            }
        }
        
        public static dmat2x3 operator+(dmat2x3 v) {
            return new dmat2x3(+v.m00, +v.m01, +v.m02, +v.m10, +v.m11, +v.m12);
        }
        
        public static dmat2x3 operator-(dmat2x3 v) {
            return new dmat2x3(-v.m00, -v.m01, -v.m02, -v.m10, -v.m11, -v.m12);
        }
        
        public static dmat2x3 operator++(dmat2x3 v) {
            return new dmat2x3(++v.m00, ++v.m01, ++v.m02, ++v.m10, ++v.m11, ++v.m12);
        }
        
        public static dmat2x3 operator--(dmat2x3 v) {
            return new dmat2x3(--v.m00, --v.m01, --v.m02, --v.m10, --v.m11, --v.m12);
        }
        
        public static dmat2x3 operator+(double s, dmat2x3 v2) {
            return new dmat2x3(s + v2.m00, s + v2.m01, s + v2.m02, s + v2.m10, s + v2.m11, s + v2.m12);
        }
        
        public static dmat2x3 operator-(double s, dmat2x3 v2) {
            return new dmat2x3(s - v2.m00, s - v2.m01, s - v2.m02, s - v2.m10, s - v2.m11, s - v2.m12);
        }
        
        public static dmat2x3 operator*(double s, dmat2x3 v2) {
            return new dmat2x3(s * v2.m00, s * v2.m01, s * v2.m02, s * v2.m10, s * v2.m11, s * v2.m12);
        }
        
        public static dmat2x3 operator/(double s, dmat2x3 v2) {
            return new dmat2x3(s / v2.m00, s / v2.m01, s / v2.m02, s / v2.m10, s / v2.m11, s / v2.m12);
        }
        
        public static dmat2x3 operator%(double s, dmat2x3 v2) {
            return new dmat2x3(s % v2.m00, s % v2.m01, s % v2.m02, s % v2.m10, s % v2.m11, s % v2.m12);
        }
        
        public static dmat2x3 operator+(dmat2x3 v1, double s) {
            return new dmat2x3(v1.m00 + s, v1.m01 + s, v1.m02 + s, v1.m10 + s, v1.m11 + s, v1.m12 + s);
        }
        
        public static dmat2x3 operator-(dmat2x3 v1, double s) {
            return new dmat2x3(v1.m00 - s, v1.m01 - s, v1.m02 - s, v1.m10 - s, v1.m11 - s, v1.m12 - s);
        }
        
        public static dmat2x3 operator*(dmat2x3 v1, double s) {
            return new dmat2x3(v1.m00 * s, v1.m01 * s, v1.m02 * s, v1.m10 * s, v1.m11 * s, v1.m12 * s);
        }
        
        public static dmat2x3 operator/(dmat2x3 v1, double s) {
            return new dmat2x3(v1.m00 / s, v1.m01 / s, v1.m02 / s, v1.m10 / s, v1.m11 / s, v1.m12 / s);
        }
        
        public static dmat2x3 operator%(dmat2x3 v1, double s) {
            return new dmat2x3(v1.m00 % s, v1.m01 % s, v1.m02 % s, v1.m10 % s, v1.m11 % s, v1.m12 % s);
        }
        
        public static dmat2x3 operator+(dmat2x3 v1, dmat2x3 v2) {
            return new dmat2x3(v1.m00 + v2.m00, v1.m01 + v2.m01, v1.m02 + v2.m02, v1.m10 + v2.m10, v1.m11 + v2.m11, v1.m12 + v2.m12);
        }
        
        public static dmat2x3 operator-(dmat2x3 v1, dmat2x3 v2) {
            return new dmat2x3(v1.m00 - v2.m00, v1.m01 - v2.m01, v1.m02 - v2.m02, v1.m10 - v2.m10, v1.m11 - v2.m11, v1.m12 - v2.m12);
        }
        
        public static dmat2x3 operator/(dmat2x3 v1, dmat2x3 v2) {
            return new dmat2x3(v1.m00 / v2.m00, v1.m01 / v2.m01, v1.m02 / v2.m02, v1.m10 / v2.m10, v1.m11 / v2.m11, v1.m12 / v2.m12);
        }
        
        public static dmat2x3 operator%(dmat2x3 v1, dmat2x3 v2) {
            return new dmat2x3(v1.m00 % v2.m00, v1.m01 % v2.m01, v1.m02 % v2.m02, v1.m10 % v2.m10, v1.m11 % v2.m11, v1.m12 % v2.m12);
        }
        
        public static bmat2x3 operator==(dmat2x3 v1, dmat2x3 v2) {
            return new bmat2x3(v1.m00 == v2.m00, v1.m01 == v2.m01, v1.m02 == v2.m02, v1.m10 == v2.m10, v1.m11 == v2.m11, v1.m12 == v2.m12);
        }
        
        public static bmat2x3 operator>(dmat2x3 v1, dmat2x3 v2) {
            return new bmat2x3(v1.m00 > v2.m00, v1.m01 > v2.m01, v1.m02 > v2.m02, v1.m10 > v2.m10, v1.m11 > v2.m11, v1.m12 > v2.m12);
        }
        
        public static bmat2x3 operator>=(dmat2x3 v1, dmat2x3 v2) {
            return new bmat2x3(v1.m00 >= v2.m00, v1.m01 >= v2.m01, v1.m02 >= v2.m02, v1.m10 >= v2.m10, v1.m11 >= v2.m11, v1.m12 >= v2.m12);
        }
        
        public static bmat2x3 operator<(dmat2x3 v1, dmat2x3 v2) {
            return new bmat2x3(v1.m00 < v2.m00, v1.m01 < v2.m01, v1.m02 < v2.m02, v1.m10 < v2.m10, v1.m11 < v2.m11, v1.m12 < v2.m12);
        }
        
        public static bmat2x3 operator<=(dmat2x3 v1, dmat2x3 v2) {
            return new bmat2x3(v1.m00 <= v2.m00, v1.m01 <= v2.m01, v1.m02 <= v2.m02, v1.m10 <= v2.m10, v1.m11 <= v2.m11, v1.m12 <= v2.m12);
        }
        
        public static bmat2x3 operator!=(dmat2x3 v1, dmat2x3 v2) {
            return new bmat2x3(v1.m00 != v2.m00, v1.m01 != v2.m01, v1.m02 != v2.m02, v1.m10 != v2.m10, v1.m11 != v2.m11, v1.m12 != v2.m12);
        }
        
        public static dvec3 operator*(dvec2 v0, dmat2x3 m1) {
            return dmath.InnerProduct1(ref v0, ref m1);
        }
        
        public static dvec2 operator*(dmat2x3 m0, dvec3 v1) {
            return dmath.InnerProduct1(ref m0, ref v1);
        }
        
        public static dmat2 operator*(dmat2x3 m0, dmat3x2 m1) {
            return dmath.InnerProduct1(ref m0, ref m1);
        }
        
        public static dmat2x3 operator*(dmat2x3 m0, dmat3 m1) {
            return dmath.InnerProduct1(ref m0, ref m1);
        }
        
        public static dmat2x4 operator*(dmat2x3 m0, dmat3x4 m1) {
            return dmath.InnerProduct1(ref m0, ref m1);
        }
        
        public override bool Equals(object obj) {
            return (obj is dmat2x3) ? bmath.All(this == (dmat2x3)obj) : false;
        }
        
        public override int GetHashCode() {
            return this.m00.GetHashCode() ^ this.m01.GetHashCode() ^ this.m02.GetHashCode() ^ this.m10.GetHashCode() ^ this.m11.GetHashCode() ^ this.m12.GetHashCode();
        }
        
        public override string ToString() {
            return "dmat2x3(" + this.m00 + "," + this.m01 + "," + this.m02 + "," + this.m10 + "," + this.m11 + "," + this.m12 + ")";
        }
    }
    
    [DataContract()]
    [StructLayout(LayoutKind.Sequential)]
    public struct dmat2x4 {
        
        [DataMember()]
        internal double m00;
        
        [DataMember()]
        internal double m01;
        
        [DataMember()]
        internal double m02;
        
        [DataMember()]
        internal double m03;
        
        [DataMember()]
        internal double m10;
        
        [DataMember()]
        internal double m11;
        
        [DataMember()]
        internal double m12;
        
        [DataMember()]
        internal double m13;
        
        public dmat2x4(double s) {
            this.m00 = s;
            this.m01 = 0D;
            this.m02 = 0D;
            this.m03 = 0D;
            this.m10 = 0D;
            this.m11 = s;
            this.m12 = 0D;
            this.m13 = 0D;
        }
        
        public dmat2x4(dmat2x4 m00) {
            this.m00 = m00.m00;
            this.m01 = m00.m01;
            this.m02 = m00.m02;
            this.m03 = m00.m03;
            this.m10 = m00.m10;
            this.m11 = m00.m11;
            this.m12 = m00.m12;
            this.m13 = m00.m13;
        }
        
        public dmat2x4(dmat2 m00, dmat2 m01) {
            this.m00 = m00.m00;
            this.m01 = m00.m01;
            this.m02 = m01.m00;
            this.m03 = m01.m01;
            this.m10 = m00.m10;
            this.m11 = m00.m11;
            this.m12 = m01.m10;
            this.m13 = m01.m11;
        }
        
        public dmat2x4(dvec4 m00, dvec4 m10) {
            this.m00 = m00.v0;
            this.m01 = m00.v1;
            this.m02 = m00.v2;
            this.m03 = m00.v3;
            this.m10 = m10.v0;
            this.m11 = m10.v1;
            this.m12 = m10.v2;
            this.m13 = m10.v3;
        }
        
        public dmat2x4(dvec2 m00, dvec2 m01, dvec2 m10, dvec2 m11) {
            this.m00 = m00.v0;
            this.m01 = m00.v1;
            this.m02 = m01.v0;
            this.m03 = m01.v1;
            this.m10 = m10.v0;
            this.m11 = m10.v1;
            this.m12 = m11.v0;
            this.m13 = m11.v1;
        }
        
        public dmat2x4(dvec3 m00, double m01, dvec3 m10, double m11) {
            this.m00 = m00.v0;
            this.m01 = m00.v1;
            this.m02 = m00.v2;
            this.m03 = m01;
            this.m10 = m10.v0;
            this.m11 = m10.v1;
            this.m12 = m10.v2;
            this.m13 = m11;
        }
        
        public dmat2x4(double m00, dvec3 m01, double m10, dvec3 m11) {
            this.m00 = m00;
            this.m01 = m01.v0;
            this.m02 = m01.v1;
            this.m03 = m01.v2;
            this.m10 = m10;
            this.m11 = m11.v0;
            this.m12 = m11.v1;
            this.m13 = m11.v2;
        }
        
        public dmat2x4(dvec2 m00, double m01, double m02, dvec2 m10, double m11, double m12) {
            this.m00 = m00.v0;
            this.m01 = m00.v1;
            this.m02 = m01;
            this.m03 = m02;
            this.m10 = m10.v0;
            this.m11 = m10.v1;
            this.m12 = m11;
            this.m13 = m12;
        }
        
        public dmat2x4(double m00, dvec2 m01, double m02, double m10, dvec2 m11, double m12) {
            this.m00 = m00;
            this.m01 = m01.v0;
            this.m02 = m01.v1;
            this.m03 = m02;
            this.m10 = m10;
            this.m11 = m11.v0;
            this.m12 = m11.v1;
            this.m13 = m12;
        }
        
        public dmat2x4(double m00, double m01, dvec2 m02, double m10, double m11, dvec2 m12) {
            this.m00 = m00;
            this.m01 = m01;
            this.m02 = m02.v0;
            this.m03 = m02.v1;
            this.m10 = m10;
            this.m11 = m11;
            this.m12 = m12.v0;
            this.m13 = m12.v1;
        }
        
        public dmat2x4(double m00, double m01, double m02, double m03, double m10, double m11, double m12, double m13) {
            this.m00 = m00;
            this.m01 = m01;
            this.m02 = m02;
            this.m03 = m03;
            this.m10 = m10;
            this.m11 = m11;
            this.m12 = m12;
            this.m13 = m13;
        }
        
        public dmat2x4(dmat2 s) {
            this.m00 = s.m00;
            this.m01 = s.m01;
            this.m02 = 0D;
            this.m03 = 0D;
            this.m10 = s.m10;
            this.m11 = s.m11;
            this.m12 = 0D;
            this.m13 = 0D;
        }
        
        public dmat2x4(dmat2x3 s) {
            this.m00 = s.m00;
            this.m01 = s.m01;
            this.m02 = s.m02;
            this.m03 = 0D;
            this.m10 = s.m10;
            this.m11 = s.m11;
            this.m12 = s.m12;
            this.m13 = 0D;
        }
        
        public dmat2x4(dmat3x2 s) {
            this.m00 = s.m00;
            this.m01 = s.m01;
            this.m02 = 0D;
            this.m03 = 0D;
            this.m10 = s.m10;
            this.m11 = s.m11;
            this.m12 = 0D;
            this.m13 = 0D;
        }
        
        public dmat2x4(dmat3 s) {
            this.m00 = s.m00;
            this.m01 = s.m01;
            this.m02 = s.m02;
            this.m03 = 0D;
            this.m10 = s.m10;
            this.m11 = s.m11;
            this.m12 = s.m12;
            this.m13 = 0D;
        }
        
        public dmat2x4(dmat3x4 s) {
            this.m00 = s.m00;
            this.m01 = s.m01;
            this.m02 = s.m02;
            this.m03 = s.m03;
            this.m10 = s.m10;
            this.m11 = s.m11;
            this.m12 = s.m12;
            this.m13 = s.m13;
        }
        
        public dmat2x4(dmat4x2 s) {
            this.m00 = s.m00;
            this.m01 = s.m01;
            this.m02 = 0D;
            this.m03 = 0D;
            this.m10 = s.m10;
            this.m11 = s.m11;
            this.m12 = 0D;
            this.m13 = 0D;
        }
        
        public dmat2x4(dmat4x3 s) {
            this.m00 = s.m00;
            this.m01 = s.m01;
            this.m02 = s.m02;
            this.m03 = 0D;
            this.m10 = s.m10;
            this.m11 = s.m11;
            this.m12 = s.m12;
            this.m13 = 0D;
        }
        
        public dmat2x4(dmat4 s) {
            this.m00 = s.m00;
            this.m01 = s.m01;
            this.m02 = s.m02;
            this.m03 = s.m03;
            this.m10 = s.m10;
            this.m11 = s.m11;
            this.m12 = s.m12;
            this.m13 = s.m13;
        }
        
        public double this[int i0, int i1] {
            get {
                if (((i0 < 0) 
                            || (1 < i0))) {
                    throw new System.IndexOutOfRangeException("i0");
                }
                if (((i1 < 0) 
                            || (3 < i1))) {
                    throw new System.IndexOutOfRangeException("i1");
                }
                
                unsafe
                {
                    fixed (Double* v_ptr = &this.m00)
                    {
                        return *(v_ptr + (i1 + i0 * 4));
                    }
                };
            }
            set {
                if (((i0 < 0) 
                            || (1 < i0))) {
                    throw new System.IndexOutOfRangeException("i0");
                }
                if (((i1 < 0) 
                            || (3 < i1))) {
                    throw new System.IndexOutOfRangeException("i1");
                }
                
                unsafe
                {
                    fixed (Double* v_ptr = &this.m00)
                    {
                        *(v_ptr  + (i1 + i0 * 4)) = value;
                    }
                };
            }
        }
        
        public dvec4 this[int i0] {
            get {
                return new dvec4(this[i0, 0], this[i0, 1], this[i0, 2], this[i0, 3]);
            }
            set {
                this[i0, 0] = value[0];
                this[i0, 1] = value[1];
                this[i0, 2] = value[2];
                this[i0, 3] = value[3];
            }
        }
        
        public static dmat2x4 operator+(dmat2x4 v) {
            return new dmat2x4(+v.m00, +v.m01, +v.m02, +v.m03, +v.m10, +v.m11, +v.m12, +v.m13);
        }
        
        public static dmat2x4 operator-(dmat2x4 v) {
            return new dmat2x4(-v.m00, -v.m01, -v.m02, -v.m03, -v.m10, -v.m11, -v.m12, -v.m13);
        }
        
        public static dmat2x4 operator++(dmat2x4 v) {
            return new dmat2x4(++v.m00, ++v.m01, ++v.m02, ++v.m03, ++v.m10, ++v.m11, ++v.m12, ++v.m13);
        }
        
        public static dmat2x4 operator--(dmat2x4 v) {
            return new dmat2x4(--v.m00, --v.m01, --v.m02, --v.m03, --v.m10, --v.m11, --v.m12, --v.m13);
        }
        
        public static dmat2x4 operator+(double s, dmat2x4 v2) {
            return new dmat2x4(s + v2.m00, s + v2.m01, s + v2.m02, s + v2.m03, s + v2.m10, s + v2.m11, s + v2.m12, s + v2.m13);
        }
        
        public static dmat2x4 operator-(double s, dmat2x4 v2) {
            return new dmat2x4(s - v2.m00, s - v2.m01, s - v2.m02, s - v2.m03, s - v2.m10, s - v2.m11, s - v2.m12, s - v2.m13);
        }
        
        public static dmat2x4 operator*(double s, dmat2x4 v2) {
            return new dmat2x4(s * v2.m00, s * v2.m01, s * v2.m02, s * v2.m03, s * v2.m10, s * v2.m11, s * v2.m12, s * v2.m13);
        }
        
        public static dmat2x4 operator/(double s, dmat2x4 v2) {
            return new dmat2x4(s / v2.m00, s / v2.m01, s / v2.m02, s / v2.m03, s / v2.m10, s / v2.m11, s / v2.m12, s / v2.m13);
        }
        
        public static dmat2x4 operator%(double s, dmat2x4 v2) {
            return new dmat2x4(s % v2.m00, s % v2.m01, s % v2.m02, s % v2.m03, s % v2.m10, s % v2.m11, s % v2.m12, s % v2.m13);
        }
        
        public static dmat2x4 operator+(dmat2x4 v1, double s) {
            return new dmat2x4(v1.m00 + s, v1.m01 + s, v1.m02 + s, v1.m03 + s, v1.m10 + s, v1.m11 + s, v1.m12 + s, v1.m13 + s);
        }
        
        public static dmat2x4 operator-(dmat2x4 v1, double s) {
            return new dmat2x4(v1.m00 - s, v1.m01 - s, v1.m02 - s, v1.m03 - s, v1.m10 - s, v1.m11 - s, v1.m12 - s, v1.m13 - s);
        }
        
        public static dmat2x4 operator*(dmat2x4 v1, double s) {
            return new dmat2x4(v1.m00 * s, v1.m01 * s, v1.m02 * s, v1.m03 * s, v1.m10 * s, v1.m11 * s, v1.m12 * s, v1.m13 * s);
        }
        
        public static dmat2x4 operator/(dmat2x4 v1, double s) {
            return new dmat2x4(v1.m00 / s, v1.m01 / s, v1.m02 / s, v1.m03 / s, v1.m10 / s, v1.m11 / s, v1.m12 / s, v1.m13 / s);
        }
        
        public static dmat2x4 operator%(dmat2x4 v1, double s) {
            return new dmat2x4(v1.m00 % s, v1.m01 % s, v1.m02 % s, v1.m03 % s, v1.m10 % s, v1.m11 % s, v1.m12 % s, v1.m13 % s);
        }
        
        public static dmat2x4 operator+(dmat2x4 v1, dmat2x4 v2) {
            return new dmat2x4(v1.m00 + v2.m00, v1.m01 + v2.m01, v1.m02 + v2.m02, v1.m03 + v2.m03, v1.m10 + v2.m10, v1.m11 + v2.m11, v1.m12 + v2.m12, v1.m13 + v2.m13);
        }
        
        public static dmat2x4 operator-(dmat2x4 v1, dmat2x4 v2) {
            return new dmat2x4(v1.m00 - v2.m00, v1.m01 - v2.m01, v1.m02 - v2.m02, v1.m03 - v2.m03, v1.m10 - v2.m10, v1.m11 - v2.m11, v1.m12 - v2.m12, v1.m13 - v2.m13);
        }
        
        public static dmat2x4 operator/(dmat2x4 v1, dmat2x4 v2) {
            return new dmat2x4(v1.m00 / v2.m00, v1.m01 / v2.m01, v1.m02 / v2.m02, v1.m03 / v2.m03, v1.m10 / v2.m10, v1.m11 / v2.m11, v1.m12 / v2.m12, v1.m13 / v2.m13);
        }
        
        public static dmat2x4 operator%(dmat2x4 v1, dmat2x4 v2) {
            return new dmat2x4(v1.m00 % v2.m00, v1.m01 % v2.m01, v1.m02 % v2.m02, v1.m03 % v2.m03, v1.m10 % v2.m10, v1.m11 % v2.m11, v1.m12 % v2.m12, v1.m13 % v2.m13);
        }
        
        public static bmat2x4 operator==(dmat2x4 v1, dmat2x4 v2) {
            return new bmat2x4(v1.m00 == v2.m00, v1.m01 == v2.m01, v1.m02 == v2.m02, v1.m03 == v2.m03, v1.m10 == v2.m10, v1.m11 == v2.m11, v1.m12 == v2.m12, v1.m13 == v2.m13);
        }
        
        public static bmat2x4 operator>(dmat2x4 v1, dmat2x4 v2) {
            return new bmat2x4(v1.m00 > v2.m00, v1.m01 > v2.m01, v1.m02 > v2.m02, v1.m03 > v2.m03, v1.m10 > v2.m10, v1.m11 > v2.m11, v1.m12 > v2.m12, v1.m13 > v2.m13);
        }
        
        public static bmat2x4 operator>=(dmat2x4 v1, dmat2x4 v2) {
            return new bmat2x4(v1.m00 >= v2.m00, v1.m01 >= v2.m01, v1.m02 >= v2.m02, v1.m03 >= v2.m03, v1.m10 >= v2.m10, v1.m11 >= v2.m11, v1.m12 >= v2.m12, v1.m13 >= v2.m13);
        }
        
        public static bmat2x4 operator<(dmat2x4 v1, dmat2x4 v2) {
            return new bmat2x4(v1.m00 < v2.m00, v1.m01 < v2.m01, v1.m02 < v2.m02, v1.m03 < v2.m03, v1.m10 < v2.m10, v1.m11 < v2.m11, v1.m12 < v2.m12, v1.m13 < v2.m13);
        }
        
        public static bmat2x4 operator<=(dmat2x4 v1, dmat2x4 v2) {
            return new bmat2x4(v1.m00 <= v2.m00, v1.m01 <= v2.m01, v1.m02 <= v2.m02, v1.m03 <= v2.m03, v1.m10 <= v2.m10, v1.m11 <= v2.m11, v1.m12 <= v2.m12, v1.m13 <= v2.m13);
        }
        
        public static bmat2x4 operator!=(dmat2x4 v1, dmat2x4 v2) {
            return new bmat2x4(v1.m00 != v2.m00, v1.m01 != v2.m01, v1.m02 != v2.m02, v1.m03 != v2.m03, v1.m10 != v2.m10, v1.m11 != v2.m11, v1.m12 != v2.m12, v1.m13 != v2.m13);
        }
        
        public static dvec4 operator*(dvec2 v0, dmat2x4 m1) {
            return dmath.InnerProduct1(ref v0, ref m1);
        }
        
        public static dvec2 operator*(dmat2x4 m0, dvec4 v1) {
            return dmath.InnerProduct1(ref m0, ref v1);
        }
        
        public static dmat2 operator*(dmat2x4 m0, dmat4x2 m1) {
            return dmath.InnerProduct1(ref m0, ref m1);
        }
        
        public static dmat2x3 operator*(dmat2x4 m0, dmat4x3 m1) {
            return dmath.InnerProduct1(ref m0, ref m1);
        }
        
        public static dmat2x4 operator*(dmat2x4 m0, dmat4 m1) {
            return dmath.InnerProduct1(ref m0, ref m1);
        }
        
        public override bool Equals(object obj) {
            return (obj is dmat2x4) ? bmath.All(this == (dmat2x4)obj) : false;
        }
        
        public override int GetHashCode() {
            return this.m00.GetHashCode() ^ this.m01.GetHashCode() ^ this.m02.GetHashCode() ^ this.m03.GetHashCode() ^ this.m10.GetHashCode() ^ this.m11.GetHashCode() ^ this.m12.GetHashCode() ^ this.m13.GetHashCode();
        }
        
        public override string ToString() {
            return "dmat2x4(" + this.m00 + "," + this.m01 + "," + this.m02 + "," + this.m03 + "," + this.m10 + "," + this.m11 + "," + this.m12 + "," + this.m13 + ")";
        }
    }
    
    [DataContract()]
    [StructLayout(LayoutKind.Sequential)]
    public struct dmat3x2 {
        
        [DataMember()]
        internal double m00;
        
        [DataMember()]
        internal double m01;
        
        [DataMember()]
        internal double m10;
        
        [DataMember()]
        internal double m11;
        
        [DataMember()]
        internal double m20;
        
        [DataMember()]
        internal double m21;
        
        public dmat3x2(double s) {
            this.m00 = s;
            this.m01 = 0D;
            this.m10 = 0D;
            this.m11 = s;
            this.m20 = 0D;
            this.m21 = 0D;
        }
        
        public dmat3x2(dmat3x2 m00) {
            this.m00 = m00.m00;
            this.m01 = m00.m01;
            this.m10 = m00.m10;
            this.m11 = m00.m11;
            this.m20 = m00.m20;
            this.m21 = m00.m21;
        }
        
        public dmat3x2(dmat2 m00, dvec2 m10) {
            this.m00 = m00.m00;
            this.m01 = m00.m01;
            this.m10 = m00.m10;
            this.m11 = m00.m11;
            this.m20 = m10.v0;
            this.m21 = m10.v1;
        }
        
        public dmat3x2(dvec2 m00, dmat2 m10) {
            this.m00 = m00.v0;
            this.m01 = m00.v1;
            this.m10 = m10.m00;
            this.m11 = m10.m01;
            this.m20 = m10.m10;
            this.m21 = m10.m11;
        }
        
        public dmat3x2(dvec2 m00, dvec2 m10, dvec2 m20) {
            this.m00 = m00.v0;
            this.m01 = m00.v1;
            this.m10 = m10.v0;
            this.m11 = m10.v1;
            this.m20 = m20.v0;
            this.m21 = m20.v1;
        }
        
        public dmat3x2(double m00, double m01, double m10, double m11, double m20, double m21) {
            this.m00 = m00;
            this.m01 = m01;
            this.m10 = m10;
            this.m11 = m11;
            this.m20 = m20;
            this.m21 = m21;
        }
        
        public dmat3x2(dmat2 s) {
            this.m00 = s.m00;
            this.m01 = s.m01;
            this.m10 = s.m10;
            this.m11 = s.m11;
            this.m20 = 0D;
            this.m21 = 0D;
        }
        
        public dmat3x2(dmat2x3 s) {
            this.m00 = s.m00;
            this.m01 = s.m01;
            this.m10 = s.m10;
            this.m11 = s.m11;
            this.m20 = 0D;
            this.m21 = 0D;
        }
        
        public dmat3x2(dmat2x4 s) {
            this.m00 = s.m00;
            this.m01 = s.m01;
            this.m10 = s.m10;
            this.m11 = s.m11;
            this.m20 = 0D;
            this.m21 = 0D;
        }
        
        public dmat3x2(dmat3 s) {
            this.m00 = s.m00;
            this.m01 = s.m01;
            this.m10 = s.m10;
            this.m11 = s.m11;
            this.m20 = s.m20;
            this.m21 = s.m21;
        }
        
        public dmat3x2(dmat3x4 s) {
            this.m00 = s.m00;
            this.m01 = s.m01;
            this.m10 = s.m10;
            this.m11 = s.m11;
            this.m20 = s.m20;
            this.m21 = s.m21;
        }
        
        public dmat3x2(dmat4x2 s) {
            this.m00 = s.m00;
            this.m01 = s.m01;
            this.m10 = s.m10;
            this.m11 = s.m11;
            this.m20 = s.m20;
            this.m21 = s.m21;
        }
        
        public dmat3x2(dmat4x3 s) {
            this.m00 = s.m00;
            this.m01 = s.m01;
            this.m10 = s.m10;
            this.m11 = s.m11;
            this.m20 = s.m20;
            this.m21 = s.m21;
        }
        
        public dmat3x2(dmat4 s) {
            this.m00 = s.m00;
            this.m01 = s.m01;
            this.m10 = s.m10;
            this.m11 = s.m11;
            this.m20 = s.m20;
            this.m21 = s.m21;
        }
        
        public double this[int i0, int i1] {
            get {
                if (((i0 < 0) 
                            || (2 < i0))) {
                    throw new System.IndexOutOfRangeException("i0");
                }
                if (((i1 < 0) 
                            || (1 < i1))) {
                    throw new System.IndexOutOfRangeException("i1");
                }
                
                unsafe
                {
                    fixed (Double* v_ptr = &this.m00)
                    {
                        return *(v_ptr + (i1 + i0 * 2));
                    }
                };
            }
            set {
                if (((i0 < 0) 
                            || (2 < i0))) {
                    throw new System.IndexOutOfRangeException("i0");
                }
                if (((i1 < 0) 
                            || (1 < i1))) {
                    throw new System.IndexOutOfRangeException("i1");
                }
                
                unsafe
                {
                    fixed (Double* v_ptr = &this.m00)
                    {
                        *(v_ptr  + (i1 + i0 * 2)) = value;
                    }
                };
            }
        }
        
        public dvec2 this[int i0] {
            get {
                return new dvec2(this[i0, 0], this[i0, 1]);
            }
            set {
                this[i0, 0] = value[0];
                this[i0, 1] = value[1];
            }
        }
        
        public static dmat3x2 operator+(dmat3x2 v) {
            return new dmat3x2(+v.m00, +v.m01, +v.m10, +v.m11, +v.m20, +v.m21);
        }
        
        public static dmat3x2 operator-(dmat3x2 v) {
            return new dmat3x2(-v.m00, -v.m01, -v.m10, -v.m11, -v.m20, -v.m21);
        }
        
        public static dmat3x2 operator++(dmat3x2 v) {
            return new dmat3x2(++v.m00, ++v.m01, ++v.m10, ++v.m11, ++v.m20, ++v.m21);
        }
        
        public static dmat3x2 operator--(dmat3x2 v) {
            return new dmat3x2(--v.m00, --v.m01, --v.m10, --v.m11, --v.m20, --v.m21);
        }
        
        public static dmat3x2 operator+(double s, dmat3x2 v2) {
            return new dmat3x2(s + v2.m00, s + v2.m01, s + v2.m10, s + v2.m11, s + v2.m20, s + v2.m21);
        }
        
        public static dmat3x2 operator-(double s, dmat3x2 v2) {
            return new dmat3x2(s - v2.m00, s - v2.m01, s - v2.m10, s - v2.m11, s - v2.m20, s - v2.m21);
        }
        
        public static dmat3x2 operator*(double s, dmat3x2 v2) {
            return new dmat3x2(s * v2.m00, s * v2.m01, s * v2.m10, s * v2.m11, s * v2.m20, s * v2.m21);
        }
        
        public static dmat3x2 operator/(double s, dmat3x2 v2) {
            return new dmat3x2(s / v2.m00, s / v2.m01, s / v2.m10, s / v2.m11, s / v2.m20, s / v2.m21);
        }
        
        public static dmat3x2 operator%(double s, dmat3x2 v2) {
            return new dmat3x2(s % v2.m00, s % v2.m01, s % v2.m10, s % v2.m11, s % v2.m20, s % v2.m21);
        }
        
        public static dmat3x2 operator+(dmat3x2 v1, double s) {
            return new dmat3x2(v1.m00 + s, v1.m01 + s, v1.m10 + s, v1.m11 + s, v1.m20 + s, v1.m21 + s);
        }
        
        public static dmat3x2 operator-(dmat3x2 v1, double s) {
            return new dmat3x2(v1.m00 - s, v1.m01 - s, v1.m10 - s, v1.m11 - s, v1.m20 - s, v1.m21 - s);
        }
        
        public static dmat3x2 operator*(dmat3x2 v1, double s) {
            return new dmat3x2(v1.m00 * s, v1.m01 * s, v1.m10 * s, v1.m11 * s, v1.m20 * s, v1.m21 * s);
        }
        
        public static dmat3x2 operator/(dmat3x2 v1, double s) {
            return new dmat3x2(v1.m00 / s, v1.m01 / s, v1.m10 / s, v1.m11 / s, v1.m20 / s, v1.m21 / s);
        }
        
        public static dmat3x2 operator%(dmat3x2 v1, double s) {
            return new dmat3x2(v1.m00 % s, v1.m01 % s, v1.m10 % s, v1.m11 % s, v1.m20 % s, v1.m21 % s);
        }
        
        public static dmat3x2 operator+(dmat3x2 v1, dmat3x2 v2) {
            return new dmat3x2(v1.m00 + v2.m00, v1.m01 + v2.m01, v1.m10 + v2.m10, v1.m11 + v2.m11, v1.m20 + v2.m20, v1.m21 + v2.m21);
        }
        
        public static dmat3x2 operator-(dmat3x2 v1, dmat3x2 v2) {
            return new dmat3x2(v1.m00 - v2.m00, v1.m01 - v2.m01, v1.m10 - v2.m10, v1.m11 - v2.m11, v1.m20 - v2.m20, v1.m21 - v2.m21);
        }
        
        public static dmat3x2 operator/(dmat3x2 v1, dmat3x2 v2) {
            return new dmat3x2(v1.m00 / v2.m00, v1.m01 / v2.m01, v1.m10 / v2.m10, v1.m11 / v2.m11, v1.m20 / v2.m20, v1.m21 / v2.m21);
        }
        
        public static dmat3x2 operator%(dmat3x2 v1, dmat3x2 v2) {
            return new dmat3x2(v1.m00 % v2.m00, v1.m01 % v2.m01, v1.m10 % v2.m10, v1.m11 % v2.m11, v1.m20 % v2.m20, v1.m21 % v2.m21);
        }
        
        public static bmat3x2 operator==(dmat3x2 v1, dmat3x2 v2) {
            return new bmat3x2(v1.m00 == v2.m00, v1.m01 == v2.m01, v1.m10 == v2.m10, v1.m11 == v2.m11, v1.m20 == v2.m20, v1.m21 == v2.m21);
        }
        
        public static bmat3x2 operator>(dmat3x2 v1, dmat3x2 v2) {
            return new bmat3x2(v1.m00 > v2.m00, v1.m01 > v2.m01, v1.m10 > v2.m10, v1.m11 > v2.m11, v1.m20 > v2.m20, v1.m21 > v2.m21);
        }
        
        public static bmat3x2 operator>=(dmat3x2 v1, dmat3x2 v2) {
            return new bmat3x2(v1.m00 >= v2.m00, v1.m01 >= v2.m01, v1.m10 >= v2.m10, v1.m11 >= v2.m11, v1.m20 >= v2.m20, v1.m21 >= v2.m21);
        }
        
        public static bmat3x2 operator<(dmat3x2 v1, dmat3x2 v2) {
            return new bmat3x2(v1.m00 < v2.m00, v1.m01 < v2.m01, v1.m10 < v2.m10, v1.m11 < v2.m11, v1.m20 < v2.m20, v1.m21 < v2.m21);
        }
        
        public static bmat3x2 operator<=(dmat3x2 v1, dmat3x2 v2) {
            return new bmat3x2(v1.m00 <= v2.m00, v1.m01 <= v2.m01, v1.m10 <= v2.m10, v1.m11 <= v2.m11, v1.m20 <= v2.m20, v1.m21 <= v2.m21);
        }
        
        public static bmat3x2 operator!=(dmat3x2 v1, dmat3x2 v2) {
            return new bmat3x2(v1.m00 != v2.m00, v1.m01 != v2.m01, v1.m10 != v2.m10, v1.m11 != v2.m11, v1.m20 != v2.m20, v1.m21 != v2.m21);
        }
        
        public static dvec2 operator*(dvec3 v0, dmat3x2 m1) {
            return dmath.InnerProduct1(ref v0, ref m1);
        }
        
        public static dvec3 operator*(dmat3x2 m0, dvec2 v1) {
            return dmath.InnerProduct1(ref m0, ref v1);
        }
        
        public static dmat3x2 operator*(dmat3x2 m0, dmat2 m1) {
            return dmath.InnerProduct1(ref m0, ref m1);
        }
        
        public static dmat3 operator*(dmat3x2 m0, dmat2x3 m1) {
            return dmath.InnerProduct1(ref m0, ref m1);
        }
        
        public static dmat3x4 operator*(dmat3x2 m0, dmat2x4 m1) {
            return dmath.InnerProduct1(ref m0, ref m1);
        }
        
        public override bool Equals(object obj) {
            return (obj is dmat3x2) ? bmath.All(this == (dmat3x2)obj) : false;
        }
        
        public override int GetHashCode() {
            return this.m00.GetHashCode() ^ this.m01.GetHashCode() ^ this.m10.GetHashCode() ^ this.m11.GetHashCode() ^ this.m20.GetHashCode() ^ this.m21.GetHashCode();
        }
        
        public override string ToString() {
            return "dmat3x2(" + this.m00 + "," + this.m01 + "," + this.m10 + "," + this.m11 + "," + this.m20 + "," + this.m21 + ")";
        }
    }
    
    [DataContract()]
    [StructLayout(LayoutKind.Sequential)]
    public struct dmat3 {
        
        [DataMember()]
        internal double m00;
        
        [DataMember()]
        internal double m01;
        
        [DataMember()]
        internal double m02;
        
        [DataMember()]
        internal double m10;
        
        [DataMember()]
        internal double m11;
        
        [DataMember()]
        internal double m12;
        
        [DataMember()]
        internal double m20;
        
        [DataMember()]
        internal double m21;
        
        [DataMember()]
        internal double m22;
        
        public dmat3(double s) {
            this.m00 = s;
            this.m01 = 0D;
            this.m02 = 0D;
            this.m10 = 0D;
            this.m11 = s;
            this.m12 = 0D;
            this.m20 = 0D;
            this.m21 = 0D;
            this.m22 = s;
        }
        
        public dmat3(dmat3 m00) {
            this.m00 = m00.m00;
            this.m01 = m00.m01;
            this.m02 = m00.m02;
            this.m10 = m00.m10;
            this.m11 = m00.m11;
            this.m12 = m00.m12;
            this.m20 = m00.m20;
            this.m21 = m00.m21;
            this.m22 = m00.m22;
        }
        
        public dmat3(dmat2x3 m00, dvec3 m10) {
            this.m00 = m00.m00;
            this.m01 = m00.m01;
            this.m02 = m00.m02;
            this.m10 = m00.m10;
            this.m11 = m00.m11;
            this.m12 = m00.m12;
            this.m20 = m10.v0;
            this.m21 = m10.v1;
            this.m22 = m10.v2;
        }
        
        public dmat3(dvec3 m00, dmat2x3 m10) {
            this.m00 = m00.v0;
            this.m01 = m00.v1;
            this.m02 = m00.v2;
            this.m10 = m10.m00;
            this.m11 = m10.m01;
            this.m12 = m10.m02;
            this.m20 = m10.m10;
            this.m21 = m10.m11;
            this.m22 = m10.m12;
        }
        
        public dmat3(dvec3 m00, dvec3 m10, dvec3 m20) {
            this.m00 = m00.v0;
            this.m01 = m00.v1;
            this.m02 = m00.v2;
            this.m10 = m10.v0;
            this.m11 = m10.v1;
            this.m12 = m10.v2;
            this.m20 = m20.v0;
            this.m21 = m20.v1;
            this.m22 = m20.v2;
        }
        
        public dmat3(dvec2 m00, double m01, dvec2 m10, double m11, dvec2 m20, double m21) {
            this.m00 = m00.v0;
            this.m01 = m00.v1;
            this.m02 = m01;
            this.m10 = m10.v0;
            this.m11 = m10.v1;
            this.m12 = m11;
            this.m20 = m20.v0;
            this.m21 = m20.v1;
            this.m22 = m21;
        }
        
        public dmat3(double m00, dvec2 m01, double m10, dvec2 m11, double m20, dvec2 m21) {
            this.m00 = m00;
            this.m01 = m01.v0;
            this.m02 = m01.v1;
            this.m10 = m10;
            this.m11 = m11.v0;
            this.m12 = m11.v1;
            this.m20 = m20;
            this.m21 = m21.v0;
            this.m22 = m21.v1;
        }
        
        public dmat3(double m00, double m01, double m02, double m10, double m11, double m12, double m20, double m21, double m22) {
            this.m00 = m00;
            this.m01 = m01;
            this.m02 = m02;
            this.m10 = m10;
            this.m11 = m11;
            this.m12 = m12;
            this.m20 = m20;
            this.m21 = m21;
            this.m22 = m22;
        }
        
        public dmat3(dmat2 s) {
            this.m00 = s.m00;
            this.m01 = s.m01;
            this.m02 = 0D;
            this.m10 = s.m10;
            this.m11 = s.m11;
            this.m12 = 0D;
            this.m20 = 0D;
            this.m21 = 0D;
            this.m22 = 0D;
        }
        
        public dmat3(dmat2x3 s) {
            this.m00 = s.m00;
            this.m01 = s.m01;
            this.m02 = s.m02;
            this.m10 = s.m10;
            this.m11 = s.m11;
            this.m12 = s.m12;
            this.m20 = 0D;
            this.m21 = 0D;
            this.m22 = 0D;
        }
        
        public dmat3(dmat2x4 s) {
            this.m00 = s.m00;
            this.m01 = s.m01;
            this.m02 = s.m02;
            this.m10 = s.m10;
            this.m11 = s.m11;
            this.m12 = s.m12;
            this.m20 = 0D;
            this.m21 = 0D;
            this.m22 = 0D;
        }
        
        public dmat3(dmat3x2 s) {
            this.m00 = s.m00;
            this.m01 = s.m01;
            this.m02 = 0D;
            this.m10 = s.m10;
            this.m11 = s.m11;
            this.m12 = 0D;
            this.m20 = s.m20;
            this.m21 = s.m21;
            this.m22 = 0D;
        }
        
        public dmat3(dmat3x4 s) {
            this.m00 = s.m00;
            this.m01 = s.m01;
            this.m02 = s.m02;
            this.m10 = s.m10;
            this.m11 = s.m11;
            this.m12 = s.m12;
            this.m20 = s.m20;
            this.m21 = s.m21;
            this.m22 = s.m22;
        }
        
        public dmat3(dmat4x2 s) {
            this.m00 = s.m00;
            this.m01 = s.m01;
            this.m02 = 0D;
            this.m10 = s.m10;
            this.m11 = s.m11;
            this.m12 = 0D;
            this.m20 = s.m20;
            this.m21 = s.m21;
            this.m22 = 0D;
        }
        
        public dmat3(dmat4x3 s) {
            this.m00 = s.m00;
            this.m01 = s.m01;
            this.m02 = s.m02;
            this.m10 = s.m10;
            this.m11 = s.m11;
            this.m12 = s.m12;
            this.m20 = s.m20;
            this.m21 = s.m21;
            this.m22 = s.m22;
        }
        
        public dmat3(dmat4 s) {
            this.m00 = s.m00;
            this.m01 = s.m01;
            this.m02 = s.m02;
            this.m10 = s.m10;
            this.m11 = s.m11;
            this.m12 = s.m12;
            this.m20 = s.m20;
            this.m21 = s.m21;
            this.m22 = s.m22;
        }
        
        public double this[int i0, int i1] {
            get {
                if (((i0 < 0) 
                            || (2 < i0))) {
                    throw new System.IndexOutOfRangeException("i0");
                }
                if (((i1 < 0) 
                            || (2 < i1))) {
                    throw new System.IndexOutOfRangeException("i1");
                }
                
                unsafe
                {
                    fixed (Double* v_ptr = &this.m00)
                    {
                        return *(v_ptr + (i1 + i0 * 3));
                    }
                };
            }
            set {
                if (((i0 < 0) 
                            || (2 < i0))) {
                    throw new System.IndexOutOfRangeException("i0");
                }
                if (((i1 < 0) 
                            || (2 < i1))) {
                    throw new System.IndexOutOfRangeException("i1");
                }
                
                unsafe
                {
                    fixed (Double* v_ptr = &this.m00)
                    {
                        *(v_ptr  + (i1 + i0 * 3)) = value;
                    }
                };
            }
        }
        
        public dvec3 this[int i0] {
            get {
                return new dvec3(this[i0, 0], this[i0, 1], this[i0, 2]);
            }
            set {
                this[i0, 0] = value[0];
                this[i0, 1] = value[1];
                this[i0, 2] = value[2];
            }
        }
        
        public static dmat3 operator+(dmat3 v) {
            return new dmat3(+v.m00, +v.m01, +v.m02, +v.m10, +v.m11, +v.m12, +v.m20, +v.m21, +v.m22);
        }
        
        public static dmat3 operator-(dmat3 v) {
            return new dmat3(-v.m00, -v.m01, -v.m02, -v.m10, -v.m11, -v.m12, -v.m20, -v.m21, -v.m22);
        }
        
        public static dmat3 operator++(dmat3 v) {
            return new dmat3(++v.m00, ++v.m01, ++v.m02, ++v.m10, ++v.m11, ++v.m12, ++v.m20, ++v.m21, ++v.m22);
        }
        
        public static dmat3 operator--(dmat3 v) {
            return new dmat3(--v.m00, --v.m01, --v.m02, --v.m10, --v.m11, --v.m12, --v.m20, --v.m21, --v.m22);
        }
        
        public static dmat3 operator+(double s, dmat3 v2) {
            return new dmat3(s + v2.m00, s + v2.m01, s + v2.m02, s + v2.m10, s + v2.m11, s + v2.m12, s + v2.m20, s + v2.m21, s + v2.m22);
        }
        
        public static dmat3 operator-(double s, dmat3 v2) {
            return new dmat3(s - v2.m00, s - v2.m01, s - v2.m02, s - v2.m10, s - v2.m11, s - v2.m12, s - v2.m20, s - v2.m21, s - v2.m22);
        }
        
        public static dmat3 operator*(double s, dmat3 v2) {
            return new dmat3(s * v2.m00, s * v2.m01, s * v2.m02, s * v2.m10, s * v2.m11, s * v2.m12, s * v2.m20, s * v2.m21, s * v2.m22);
        }
        
        public static dmat3 operator/(double s, dmat3 v2) {
            return new dmat3(s / v2.m00, s / v2.m01, s / v2.m02, s / v2.m10, s / v2.m11, s / v2.m12, s / v2.m20, s / v2.m21, s / v2.m22);
        }
        
        public static dmat3 operator%(double s, dmat3 v2) {
            return new dmat3(s % v2.m00, s % v2.m01, s % v2.m02, s % v2.m10, s % v2.m11, s % v2.m12, s % v2.m20, s % v2.m21, s % v2.m22);
        }
        
        public static dmat3 operator+(dmat3 v1, double s) {
            return new dmat3(v1.m00 + s, v1.m01 + s, v1.m02 + s, v1.m10 + s, v1.m11 + s, v1.m12 + s, v1.m20 + s, v1.m21 + s, v1.m22 + s);
        }
        
        public static dmat3 operator-(dmat3 v1, double s) {
            return new dmat3(v1.m00 - s, v1.m01 - s, v1.m02 - s, v1.m10 - s, v1.m11 - s, v1.m12 - s, v1.m20 - s, v1.m21 - s, v1.m22 - s);
        }
        
        public static dmat3 operator*(dmat3 v1, double s) {
            return new dmat3(v1.m00 * s, v1.m01 * s, v1.m02 * s, v1.m10 * s, v1.m11 * s, v1.m12 * s, v1.m20 * s, v1.m21 * s, v1.m22 * s);
        }
        
        public static dmat3 operator/(dmat3 v1, double s) {
            return new dmat3(v1.m00 / s, v1.m01 / s, v1.m02 / s, v1.m10 / s, v1.m11 / s, v1.m12 / s, v1.m20 / s, v1.m21 / s, v1.m22 / s);
        }
        
        public static dmat3 operator%(dmat3 v1, double s) {
            return new dmat3(v1.m00 % s, v1.m01 % s, v1.m02 % s, v1.m10 % s, v1.m11 % s, v1.m12 % s, v1.m20 % s, v1.m21 % s, v1.m22 % s);
        }
        
        public static dmat3 operator+(dmat3 v1, dmat3 v2) {
            return new dmat3(v1.m00 + v2.m00, v1.m01 + v2.m01, v1.m02 + v2.m02, v1.m10 + v2.m10, v1.m11 + v2.m11, v1.m12 + v2.m12, v1.m20 + v2.m20, v1.m21 + v2.m21, v1.m22 + v2.m22);
        }
        
        public static dmat3 operator-(dmat3 v1, dmat3 v2) {
            return new dmat3(v1.m00 - v2.m00, v1.m01 - v2.m01, v1.m02 - v2.m02, v1.m10 - v2.m10, v1.m11 - v2.m11, v1.m12 - v2.m12, v1.m20 - v2.m20, v1.m21 - v2.m21, v1.m22 - v2.m22);
        }
        
        public static dmat3 operator/(dmat3 v1, dmat3 v2) {
            return new dmat3(v1.m00 / v2.m00, v1.m01 / v2.m01, v1.m02 / v2.m02, v1.m10 / v2.m10, v1.m11 / v2.m11, v1.m12 / v2.m12, v1.m20 / v2.m20, v1.m21 / v2.m21, v1.m22 / v2.m22);
        }
        
        public static dmat3 operator%(dmat3 v1, dmat3 v2) {
            return new dmat3(v1.m00 % v2.m00, v1.m01 % v2.m01, v1.m02 % v2.m02, v1.m10 % v2.m10, v1.m11 % v2.m11, v1.m12 % v2.m12, v1.m20 % v2.m20, v1.m21 % v2.m21, v1.m22 % v2.m22);
        }
        
        public static bmat3 operator==(dmat3 v1, dmat3 v2) {
            return new bmat3(v1.m00 == v2.m00, v1.m01 == v2.m01, v1.m02 == v2.m02, v1.m10 == v2.m10, v1.m11 == v2.m11, v1.m12 == v2.m12, v1.m20 == v2.m20, v1.m21 == v2.m21, v1.m22 == v2.m22);
        }
        
        public static bmat3 operator>(dmat3 v1, dmat3 v2) {
            return new bmat3(v1.m00 > v2.m00, v1.m01 > v2.m01, v1.m02 > v2.m02, v1.m10 > v2.m10, v1.m11 > v2.m11, v1.m12 > v2.m12, v1.m20 > v2.m20, v1.m21 > v2.m21, v1.m22 > v2.m22);
        }
        
        public static bmat3 operator>=(dmat3 v1, dmat3 v2) {
            return new bmat3(v1.m00 >= v2.m00, v1.m01 >= v2.m01, v1.m02 >= v2.m02, v1.m10 >= v2.m10, v1.m11 >= v2.m11, v1.m12 >= v2.m12, v1.m20 >= v2.m20, v1.m21 >= v2.m21, v1.m22 >= v2.m22);
        }
        
        public static bmat3 operator<(dmat3 v1, dmat3 v2) {
            return new bmat3(v1.m00 < v2.m00, v1.m01 < v2.m01, v1.m02 < v2.m02, v1.m10 < v2.m10, v1.m11 < v2.m11, v1.m12 < v2.m12, v1.m20 < v2.m20, v1.m21 < v2.m21, v1.m22 < v2.m22);
        }
        
        public static bmat3 operator<=(dmat3 v1, dmat3 v2) {
            return new bmat3(v1.m00 <= v2.m00, v1.m01 <= v2.m01, v1.m02 <= v2.m02, v1.m10 <= v2.m10, v1.m11 <= v2.m11, v1.m12 <= v2.m12, v1.m20 <= v2.m20, v1.m21 <= v2.m21, v1.m22 <= v2.m22);
        }
        
        public static bmat3 operator!=(dmat3 v1, dmat3 v2) {
            return new bmat3(v1.m00 != v2.m00, v1.m01 != v2.m01, v1.m02 != v2.m02, v1.m10 != v2.m10, v1.m11 != v2.m11, v1.m12 != v2.m12, v1.m20 != v2.m20, v1.m21 != v2.m21, v1.m22 != v2.m22);
        }
        
        public static dvec3 operator*(dvec3 v0, dmat3 m1) {
            return dmath.InnerProduct1(ref v0, ref m1);
        }
        
        public static dvec3 operator*(dmat3 m0, dvec3 v1) {
            return dmath.InnerProduct1(ref m0, ref v1);
        }
        
        public static dmat3x2 operator*(dmat3 m0, dmat3x2 m1) {
            return dmath.InnerProduct1(ref m0, ref m1);
        }
        
        public static dmat3 operator*(dmat3 m0, dmat3 m1) {
            return dmath.InnerProduct1(ref m0, ref m1);
        }
        
        public static dmat3x4 operator*(dmat3 m0, dmat3x4 m1) {
            return dmath.InnerProduct1(ref m0, ref m1);
        }
        
        public override bool Equals(object obj) {
            return (obj is dmat3) ? bmath.All(this == (dmat3)obj) : false;
        }
        
        public override int GetHashCode() {
            return this.m00.GetHashCode() ^ this.m01.GetHashCode() ^ this.m02.GetHashCode() ^ this.m10.GetHashCode() ^ this.m11.GetHashCode() ^ this.m12.GetHashCode() ^ this.m20.GetHashCode() ^ this.m21.GetHashCode() ^ this.m22.GetHashCode();
        }
        
        public override string ToString() {
            return "dmat3(" + this.m00 + "," + this.m01 + "," + this.m02 + "," + this.m10 + "," + this.m11 + "," + this.m12 + "," + this.m20 + "," + this.m21 + "," + this.m22 + ")";
        }
    }
    
    [DataContract()]
    [StructLayout(LayoutKind.Sequential)]
    public struct dmat3x4 {
        
        [DataMember()]
        internal double m00;
        
        [DataMember()]
        internal double m01;
        
        [DataMember()]
        internal double m02;
        
        [DataMember()]
        internal double m03;
        
        [DataMember()]
        internal double m10;
        
        [DataMember()]
        internal double m11;
        
        [DataMember()]
        internal double m12;
        
        [DataMember()]
        internal double m13;
        
        [DataMember()]
        internal double m20;
        
        [DataMember()]
        internal double m21;
        
        [DataMember()]
        internal double m22;
        
        [DataMember()]
        internal double m23;
        
        public dmat3x4(double s) {
            this.m00 = s;
            this.m01 = 0D;
            this.m02 = 0D;
            this.m03 = 0D;
            this.m10 = 0D;
            this.m11 = s;
            this.m12 = 0D;
            this.m13 = 0D;
            this.m20 = 0D;
            this.m21 = 0D;
            this.m22 = s;
            this.m23 = 0D;
        }
        
        public dmat3x4(dmat3x4 m00) {
            this.m00 = m00.m00;
            this.m01 = m00.m01;
            this.m02 = m00.m02;
            this.m03 = m00.m03;
            this.m10 = m00.m10;
            this.m11 = m00.m11;
            this.m12 = m00.m12;
            this.m13 = m00.m13;
            this.m20 = m00.m20;
            this.m21 = m00.m21;
            this.m22 = m00.m22;
            this.m23 = m00.m23;
        }
        
        public dmat3x4(dmat3x2 m00, dmat3x2 m01) {
            this.m00 = m00.m00;
            this.m01 = m00.m01;
            this.m02 = m01.m00;
            this.m03 = m01.m01;
            this.m10 = m00.m10;
            this.m11 = m00.m11;
            this.m12 = m01.m10;
            this.m13 = m01.m11;
            this.m20 = m00.m20;
            this.m21 = m00.m21;
            this.m22 = m01.m20;
            this.m23 = m01.m21;
        }
        
        public dmat3x4(dmat2x4 m00, dvec4 m10) {
            this.m00 = m00.m00;
            this.m01 = m00.m01;
            this.m02 = m00.m02;
            this.m03 = m00.m03;
            this.m10 = m00.m10;
            this.m11 = m00.m11;
            this.m12 = m00.m12;
            this.m13 = m00.m13;
            this.m20 = m10.v0;
            this.m21 = m10.v1;
            this.m22 = m10.v2;
            this.m23 = m10.v3;
        }
        
        public dmat3x4(dmat2 m00, dmat2 m01, dvec2 m10, dvec2 m11) {
            this.m00 = m00.m00;
            this.m01 = m00.m01;
            this.m02 = m01.m00;
            this.m03 = m01.m01;
            this.m10 = m00.m10;
            this.m11 = m00.m11;
            this.m12 = m01.m10;
            this.m13 = m01.m11;
            this.m20 = m10.v0;
            this.m21 = m10.v1;
            this.m22 = m11.v0;
            this.m23 = m11.v1;
        }
        
        public dmat3x4(dvec4 m00, dmat2x4 m10) {
            this.m00 = m00.v0;
            this.m01 = m00.v1;
            this.m02 = m00.v2;
            this.m03 = m00.v3;
            this.m10 = m10.m00;
            this.m11 = m10.m01;
            this.m12 = m10.m02;
            this.m13 = m10.m03;
            this.m20 = m10.m10;
            this.m21 = m10.m11;
            this.m22 = m10.m12;
            this.m23 = m10.m13;
        }
        
        public dmat3x4(dvec2 m00, dvec2 m01, dmat2 m10, dmat2 m11) {
            this.m00 = m00.v0;
            this.m01 = m00.v1;
            this.m02 = m01.v0;
            this.m03 = m01.v1;
            this.m10 = m10.m00;
            this.m11 = m10.m01;
            this.m12 = m11.m00;
            this.m13 = m11.m01;
            this.m20 = m10.m10;
            this.m21 = m10.m11;
            this.m22 = m11.m10;
            this.m23 = m11.m11;
        }
        
        public dmat3x4(dvec4 m00, dvec4 m10, dvec4 m20) {
            this.m00 = m00.v0;
            this.m01 = m00.v1;
            this.m02 = m00.v2;
            this.m03 = m00.v3;
            this.m10 = m10.v0;
            this.m11 = m10.v1;
            this.m12 = m10.v2;
            this.m13 = m10.v3;
            this.m20 = m20.v0;
            this.m21 = m20.v1;
            this.m22 = m20.v2;
            this.m23 = m20.v3;
        }
        
        public dmat3x4(dvec2 m00, dvec2 m01, dvec2 m10, dvec2 m11, dvec2 m20, dvec2 m21) {
            this.m00 = m00.v0;
            this.m01 = m00.v1;
            this.m02 = m01.v0;
            this.m03 = m01.v1;
            this.m10 = m10.v0;
            this.m11 = m10.v1;
            this.m12 = m11.v0;
            this.m13 = m11.v1;
            this.m20 = m20.v0;
            this.m21 = m20.v1;
            this.m22 = m21.v0;
            this.m23 = m21.v1;
        }
        
        public dmat3x4(dvec3 m00, double m01, dvec3 m10, double m11, dvec3 m20, double m21) {
            this.m00 = m00.v0;
            this.m01 = m00.v1;
            this.m02 = m00.v2;
            this.m03 = m01;
            this.m10 = m10.v0;
            this.m11 = m10.v1;
            this.m12 = m10.v2;
            this.m13 = m11;
            this.m20 = m20.v0;
            this.m21 = m20.v1;
            this.m22 = m20.v2;
            this.m23 = m21;
        }
        
        public dmat3x4(double m00, dvec3 m01, double m10, dvec3 m11, double m20, dvec3 m21) {
            this.m00 = m00;
            this.m01 = m01.v0;
            this.m02 = m01.v1;
            this.m03 = m01.v2;
            this.m10 = m10;
            this.m11 = m11.v0;
            this.m12 = m11.v1;
            this.m13 = m11.v2;
            this.m20 = m20;
            this.m21 = m21.v0;
            this.m22 = m21.v1;
            this.m23 = m21.v2;
        }
        
        public dmat3x4(dvec2 m00, double m01, double m02, dvec2 m10, double m11, double m12, dvec2 m20, double m21, double m22) {
            this.m00 = m00.v0;
            this.m01 = m00.v1;
            this.m02 = m01;
            this.m03 = m02;
            this.m10 = m10.v0;
            this.m11 = m10.v1;
            this.m12 = m11;
            this.m13 = m12;
            this.m20 = m20.v0;
            this.m21 = m20.v1;
            this.m22 = m21;
            this.m23 = m22;
        }
        
        public dmat3x4(double m00, dvec2 m01, double m02, double m10, dvec2 m11, double m12, double m20, dvec2 m21, double m22) {
            this.m00 = m00;
            this.m01 = m01.v0;
            this.m02 = m01.v1;
            this.m03 = m02;
            this.m10 = m10;
            this.m11 = m11.v0;
            this.m12 = m11.v1;
            this.m13 = m12;
            this.m20 = m20;
            this.m21 = m21.v0;
            this.m22 = m21.v1;
            this.m23 = m22;
        }
        
        public dmat3x4(double m00, double m01, dvec2 m02, double m10, double m11, dvec2 m12, double m20, double m21, dvec2 m22) {
            this.m00 = m00;
            this.m01 = m01;
            this.m02 = m02.v0;
            this.m03 = m02.v1;
            this.m10 = m10;
            this.m11 = m11;
            this.m12 = m12.v0;
            this.m13 = m12.v1;
            this.m20 = m20;
            this.m21 = m21;
            this.m22 = m22.v0;
            this.m23 = m22.v1;
        }
        
        public dmat3x4(double m00, double m01, double m02, double m03, double m10, double m11, double m12, double m13, double m20, double m21, double m22, double m23) {
            this.m00 = m00;
            this.m01 = m01;
            this.m02 = m02;
            this.m03 = m03;
            this.m10 = m10;
            this.m11 = m11;
            this.m12 = m12;
            this.m13 = m13;
            this.m20 = m20;
            this.m21 = m21;
            this.m22 = m22;
            this.m23 = m23;
        }
        
        public dmat3x4(dmat2 s) {
            this.m00 = s.m00;
            this.m01 = s.m01;
            this.m02 = 0D;
            this.m03 = 0D;
            this.m10 = s.m10;
            this.m11 = s.m11;
            this.m12 = 0D;
            this.m13 = 0D;
            this.m20 = 0D;
            this.m21 = 0D;
            this.m22 = 0D;
            this.m23 = 0D;
        }
        
        public dmat3x4(dmat2x3 s) {
            this.m00 = s.m00;
            this.m01 = s.m01;
            this.m02 = s.m02;
            this.m03 = 0D;
            this.m10 = s.m10;
            this.m11 = s.m11;
            this.m12 = s.m12;
            this.m13 = 0D;
            this.m20 = 0D;
            this.m21 = 0D;
            this.m22 = 0D;
            this.m23 = 0D;
        }
        
        public dmat3x4(dmat2x4 s) {
            this.m00 = s.m00;
            this.m01 = s.m01;
            this.m02 = s.m02;
            this.m03 = s.m03;
            this.m10 = s.m10;
            this.m11 = s.m11;
            this.m12 = s.m12;
            this.m13 = s.m13;
            this.m20 = 0D;
            this.m21 = 0D;
            this.m22 = 0D;
            this.m23 = 0D;
        }
        
        public dmat3x4(dmat3x2 s) {
            this.m00 = s.m00;
            this.m01 = s.m01;
            this.m02 = 0D;
            this.m03 = 0D;
            this.m10 = s.m10;
            this.m11 = s.m11;
            this.m12 = 0D;
            this.m13 = 0D;
            this.m20 = s.m20;
            this.m21 = s.m21;
            this.m22 = 0D;
            this.m23 = 0D;
        }
        
        public dmat3x4(dmat3 s) {
            this.m00 = s.m00;
            this.m01 = s.m01;
            this.m02 = s.m02;
            this.m03 = 0D;
            this.m10 = s.m10;
            this.m11 = s.m11;
            this.m12 = s.m12;
            this.m13 = 0D;
            this.m20 = s.m20;
            this.m21 = s.m21;
            this.m22 = s.m22;
            this.m23 = 0D;
        }
        
        public dmat3x4(dmat4x2 s) {
            this.m00 = s.m00;
            this.m01 = s.m01;
            this.m02 = 0D;
            this.m03 = 0D;
            this.m10 = s.m10;
            this.m11 = s.m11;
            this.m12 = 0D;
            this.m13 = 0D;
            this.m20 = s.m20;
            this.m21 = s.m21;
            this.m22 = 0D;
            this.m23 = 0D;
        }
        
        public dmat3x4(dmat4x3 s) {
            this.m00 = s.m00;
            this.m01 = s.m01;
            this.m02 = s.m02;
            this.m03 = 0D;
            this.m10 = s.m10;
            this.m11 = s.m11;
            this.m12 = s.m12;
            this.m13 = 0D;
            this.m20 = s.m20;
            this.m21 = s.m21;
            this.m22 = s.m22;
            this.m23 = 0D;
        }
        
        public dmat3x4(dmat4 s) {
            this.m00 = s.m00;
            this.m01 = s.m01;
            this.m02 = s.m02;
            this.m03 = s.m03;
            this.m10 = s.m10;
            this.m11 = s.m11;
            this.m12 = s.m12;
            this.m13 = s.m13;
            this.m20 = s.m20;
            this.m21 = s.m21;
            this.m22 = s.m22;
            this.m23 = s.m23;
        }
        
        public double this[int i0, int i1] {
            get {
                if (((i0 < 0) 
                            || (2 < i0))) {
                    throw new System.IndexOutOfRangeException("i0");
                }
                if (((i1 < 0) 
                            || (3 < i1))) {
                    throw new System.IndexOutOfRangeException("i1");
                }
                
                unsafe
                {
                    fixed (Double* v_ptr = &this.m00)
                    {
                        return *(v_ptr + (i1 + i0 * 4));
                    }
                };
            }
            set {
                if (((i0 < 0) 
                            || (2 < i0))) {
                    throw new System.IndexOutOfRangeException("i0");
                }
                if (((i1 < 0) 
                            || (3 < i1))) {
                    throw new System.IndexOutOfRangeException("i1");
                }
                
                unsafe
                {
                    fixed (Double* v_ptr = &this.m00)
                    {
                        *(v_ptr  + (i1 + i0 * 4)) = value;
                    }
                };
            }
        }
        
        public dvec4 this[int i0] {
            get {
                return new dvec4(this[i0, 0], this[i0, 1], this[i0, 2], this[i0, 3]);
            }
            set {
                this[i0, 0] = value[0];
                this[i0, 1] = value[1];
                this[i0, 2] = value[2];
                this[i0, 3] = value[3];
            }
        }
        
        public static dmat3x4 operator+(dmat3x4 v) {
            return new dmat3x4(+v.m00, +v.m01, +v.m02, +v.m03, +v.m10, +v.m11, +v.m12, +v.m13, +v.m20, +v.m21, +v.m22, +v.m23);
        }
        
        public static dmat3x4 operator-(dmat3x4 v) {
            return new dmat3x4(-v.m00, -v.m01, -v.m02, -v.m03, -v.m10, -v.m11, -v.m12, -v.m13, -v.m20, -v.m21, -v.m22, -v.m23);
        }
        
        public static dmat3x4 operator++(dmat3x4 v) {
            return new dmat3x4(++v.m00, ++v.m01, ++v.m02, ++v.m03, ++v.m10, ++v.m11, ++v.m12, ++v.m13, ++v.m20, ++v.m21, ++v.m22, ++v.m23);
        }
        
        public static dmat3x4 operator--(dmat3x4 v) {
            return new dmat3x4(--v.m00, --v.m01, --v.m02, --v.m03, --v.m10, --v.m11, --v.m12, --v.m13, --v.m20, --v.m21, --v.m22, --v.m23);
        }
        
        public static dmat3x4 operator+(double s, dmat3x4 v2) {
            return new dmat3x4(s + v2.m00, s + v2.m01, s + v2.m02, s + v2.m03, s + v2.m10, s + v2.m11, s + v2.m12, s + v2.m13, s + v2.m20, s + v2.m21, s + v2.m22, s + v2.m23);
        }
        
        public static dmat3x4 operator-(double s, dmat3x4 v2) {
            return new dmat3x4(s - v2.m00, s - v2.m01, s - v2.m02, s - v2.m03, s - v2.m10, s - v2.m11, s - v2.m12, s - v2.m13, s - v2.m20, s - v2.m21, s - v2.m22, s - v2.m23);
        }
        
        public static dmat3x4 operator*(double s, dmat3x4 v2) {
            return new dmat3x4(s * v2.m00, s * v2.m01, s * v2.m02, s * v2.m03, s * v2.m10, s * v2.m11, s * v2.m12, s * v2.m13, s * v2.m20, s * v2.m21, s * v2.m22, s * v2.m23);
        }
        
        public static dmat3x4 operator/(double s, dmat3x4 v2) {
            return new dmat3x4(s / v2.m00, s / v2.m01, s / v2.m02, s / v2.m03, s / v2.m10, s / v2.m11, s / v2.m12, s / v2.m13, s / v2.m20, s / v2.m21, s / v2.m22, s / v2.m23);
        }
        
        public static dmat3x4 operator%(double s, dmat3x4 v2) {
            return new dmat3x4(s % v2.m00, s % v2.m01, s % v2.m02, s % v2.m03, s % v2.m10, s % v2.m11, s % v2.m12, s % v2.m13, s % v2.m20, s % v2.m21, s % v2.m22, s % v2.m23);
        }
        
        public static dmat3x4 operator+(dmat3x4 v1, double s) {
            return new dmat3x4(v1.m00 + s, v1.m01 + s, v1.m02 + s, v1.m03 + s, v1.m10 + s, v1.m11 + s, v1.m12 + s, v1.m13 + s, v1.m20 + s, v1.m21 + s, v1.m22 + s, v1.m23 + s);
        }
        
        public static dmat3x4 operator-(dmat3x4 v1, double s) {
            return new dmat3x4(v1.m00 - s, v1.m01 - s, v1.m02 - s, v1.m03 - s, v1.m10 - s, v1.m11 - s, v1.m12 - s, v1.m13 - s, v1.m20 - s, v1.m21 - s, v1.m22 - s, v1.m23 - s);
        }
        
        public static dmat3x4 operator*(dmat3x4 v1, double s) {
            return new dmat3x4(v1.m00 * s, v1.m01 * s, v1.m02 * s, v1.m03 * s, v1.m10 * s, v1.m11 * s, v1.m12 * s, v1.m13 * s, v1.m20 * s, v1.m21 * s, v1.m22 * s, v1.m23 * s);
        }
        
        public static dmat3x4 operator/(dmat3x4 v1, double s) {
            return new dmat3x4(v1.m00 / s, v1.m01 / s, v1.m02 / s, v1.m03 / s, v1.m10 / s, v1.m11 / s, v1.m12 / s, v1.m13 / s, v1.m20 / s, v1.m21 / s, v1.m22 / s, v1.m23 / s);
        }
        
        public static dmat3x4 operator%(dmat3x4 v1, double s) {
            return new dmat3x4(v1.m00 % s, v1.m01 % s, v1.m02 % s, v1.m03 % s, v1.m10 % s, v1.m11 % s, v1.m12 % s, v1.m13 % s, v1.m20 % s, v1.m21 % s, v1.m22 % s, v1.m23 % s);
        }
        
        public static dmat3x4 operator+(dmat3x4 v1, dmat3x4 v2) {
            return new dmat3x4(v1.m00 + v2.m00, v1.m01 + v2.m01, v1.m02 + v2.m02, v1.m03 + v2.m03, v1.m10 + v2.m10, v1.m11 + v2.m11, v1.m12 + v2.m12, v1.m13 + v2.m13, v1.m20 + v2.m20, v1.m21 + v2.m21, v1.m22 + v2.m22, v1.m23 + v2.m23);
        }
        
        public static dmat3x4 operator-(dmat3x4 v1, dmat3x4 v2) {
            return new dmat3x4(v1.m00 - v2.m00, v1.m01 - v2.m01, v1.m02 - v2.m02, v1.m03 - v2.m03, v1.m10 - v2.m10, v1.m11 - v2.m11, v1.m12 - v2.m12, v1.m13 - v2.m13, v1.m20 - v2.m20, v1.m21 - v2.m21, v1.m22 - v2.m22, v1.m23 - v2.m23);
        }
        
        public static dmat3x4 operator/(dmat3x4 v1, dmat3x4 v2) {
            return new dmat3x4(v1.m00 / v2.m00, v1.m01 / v2.m01, v1.m02 / v2.m02, v1.m03 / v2.m03, v1.m10 / v2.m10, v1.m11 / v2.m11, v1.m12 / v2.m12, v1.m13 / v2.m13, v1.m20 / v2.m20, v1.m21 / v2.m21, v1.m22 / v2.m22, v1.m23 / v2.m23);
        }
        
        public static dmat3x4 operator%(dmat3x4 v1, dmat3x4 v2) {
            return new dmat3x4(v1.m00 % v2.m00, v1.m01 % v2.m01, v1.m02 % v2.m02, v1.m03 % v2.m03, v1.m10 % v2.m10, v1.m11 % v2.m11, v1.m12 % v2.m12, v1.m13 % v2.m13, v1.m20 % v2.m20, v1.m21 % v2.m21, v1.m22 % v2.m22, v1.m23 % v2.m23);
        }
        
        public static bmat3x4 operator==(dmat3x4 v1, dmat3x4 v2) {
            return new bmat3x4(v1.m00 == v2.m00, v1.m01 == v2.m01, v1.m02 == v2.m02, v1.m03 == v2.m03, v1.m10 == v2.m10, v1.m11 == v2.m11, v1.m12 == v2.m12, v1.m13 == v2.m13, v1.m20 == v2.m20, v1.m21 == v2.m21, v1.m22 == v2.m22, v1.m23 == v2.m23);
        }
        
        public static bmat3x4 operator>(dmat3x4 v1, dmat3x4 v2) {
            return new bmat3x4(v1.m00 > v2.m00, v1.m01 > v2.m01, v1.m02 > v2.m02, v1.m03 > v2.m03, v1.m10 > v2.m10, v1.m11 > v2.m11, v1.m12 > v2.m12, v1.m13 > v2.m13, v1.m20 > v2.m20, v1.m21 > v2.m21, v1.m22 > v2.m22, v1.m23 > v2.m23);
        }
        
        public static bmat3x4 operator>=(dmat3x4 v1, dmat3x4 v2) {
            return new bmat3x4(v1.m00 >= v2.m00, v1.m01 >= v2.m01, v1.m02 >= v2.m02, v1.m03 >= v2.m03, v1.m10 >= v2.m10, v1.m11 >= v2.m11, v1.m12 >= v2.m12, v1.m13 >= v2.m13, v1.m20 >= v2.m20, v1.m21 >= v2.m21, v1.m22 >= v2.m22, v1.m23 >= v2.m23);
        }
        
        public static bmat3x4 operator<(dmat3x4 v1, dmat3x4 v2) {
            return new bmat3x4(v1.m00 < v2.m00, v1.m01 < v2.m01, v1.m02 < v2.m02, v1.m03 < v2.m03, v1.m10 < v2.m10, v1.m11 < v2.m11, v1.m12 < v2.m12, v1.m13 < v2.m13, v1.m20 < v2.m20, v1.m21 < v2.m21, v1.m22 < v2.m22, v1.m23 < v2.m23);
        }
        
        public static bmat3x4 operator<=(dmat3x4 v1, dmat3x4 v2) {
            return new bmat3x4(v1.m00 <= v2.m00, v1.m01 <= v2.m01, v1.m02 <= v2.m02, v1.m03 <= v2.m03, v1.m10 <= v2.m10, v1.m11 <= v2.m11, v1.m12 <= v2.m12, v1.m13 <= v2.m13, v1.m20 <= v2.m20, v1.m21 <= v2.m21, v1.m22 <= v2.m22, v1.m23 <= v2.m23);
        }
        
        public static bmat3x4 operator!=(dmat3x4 v1, dmat3x4 v2) {
            return new bmat3x4(v1.m00 != v2.m00, v1.m01 != v2.m01, v1.m02 != v2.m02, v1.m03 != v2.m03, v1.m10 != v2.m10, v1.m11 != v2.m11, v1.m12 != v2.m12, v1.m13 != v2.m13, v1.m20 != v2.m20, v1.m21 != v2.m21, v1.m22 != v2.m22, v1.m23 != v2.m23);
        }
        
        public static dvec4 operator*(dvec3 v0, dmat3x4 m1) {
            return dmath.InnerProduct1(ref v0, ref m1);
        }
        
        public static dvec3 operator*(dmat3x4 m0, dvec4 v1) {
            return dmath.InnerProduct1(ref m0, ref v1);
        }
        
        public static dmat3x2 operator*(dmat3x4 m0, dmat4x2 m1) {
            return dmath.InnerProduct1(ref m0, ref m1);
        }
        
        public static dmat3 operator*(dmat3x4 m0, dmat4x3 m1) {
            return dmath.InnerProduct1(ref m0, ref m1);
        }
        
        public static dmat3x4 operator*(dmat3x4 m0, dmat4 m1) {
            return dmath.InnerProduct1(ref m0, ref m1);
        }
        
        public override bool Equals(object obj) {
            return (obj is dmat3x4) ? bmath.All(this == (dmat3x4)obj) : false;
        }
        
        public override int GetHashCode() {
            return this.m00.GetHashCode() ^ this.m01.GetHashCode() ^ this.m02.GetHashCode() ^ this.m03.GetHashCode() ^ this.m10.GetHashCode() ^ this.m11.GetHashCode() ^ this.m12.GetHashCode() ^ this.m13.GetHashCode() ^ this.m20.GetHashCode() ^ this.m21.GetHashCode() ^ this.m22.GetHashCode() ^ this.m23.GetHashCode();
        }
        
        public override string ToString() {
            return "dmat3x4(" + this.m00 + "," + this.m01 + "," + this.m02 + "," + this.m03 + "," + this.m10 + "," + this.m11 + "," + this.m12 + "," + this.m13 + "," + this.m20 + "," + this.m21 + "," + this.m22 + "," + this.m23 + ")";
        }
    }
    
    [DataContract()]
    [StructLayout(LayoutKind.Sequential)]
    public struct dmat4x2 {
        
        [DataMember()]
        internal double m00;
        
        [DataMember()]
        internal double m01;
        
        [DataMember()]
        internal double m10;
        
        [DataMember()]
        internal double m11;
        
        [DataMember()]
        internal double m20;
        
        [DataMember()]
        internal double m21;
        
        [DataMember()]
        internal double m30;
        
        [DataMember()]
        internal double m31;
        
        public dmat4x2(double s) {
            this.m00 = s;
            this.m01 = 0D;
            this.m10 = 0D;
            this.m11 = s;
            this.m20 = 0D;
            this.m21 = 0D;
            this.m30 = 0D;
            this.m31 = 0D;
        }
        
        public dmat4x2(dmat4x2 m00) {
            this.m00 = m00.m00;
            this.m01 = m00.m01;
            this.m10 = m00.m10;
            this.m11 = m00.m11;
            this.m20 = m00.m20;
            this.m21 = m00.m21;
            this.m30 = m00.m30;
            this.m31 = m00.m31;
        }
        
        public dmat4x2(dmat2 m00, dmat2 m10) {
            this.m00 = m00.m00;
            this.m01 = m00.m01;
            this.m10 = m00.m10;
            this.m11 = m00.m11;
            this.m20 = m10.m00;
            this.m21 = m10.m01;
            this.m30 = m10.m10;
            this.m31 = m10.m11;
        }
        
        public dmat4x2(dmat3x2 m00, dvec2 m10) {
            this.m00 = m00.m00;
            this.m01 = m00.m01;
            this.m10 = m00.m10;
            this.m11 = m00.m11;
            this.m20 = m00.m20;
            this.m21 = m00.m21;
            this.m30 = m10.v0;
            this.m31 = m10.v1;
        }
        
        public dmat4x2(dvec2 m00, dmat3x2 m10) {
            this.m00 = m00.v0;
            this.m01 = m00.v1;
            this.m10 = m10.m00;
            this.m11 = m10.m01;
            this.m20 = m10.m10;
            this.m21 = m10.m11;
            this.m30 = m10.m20;
            this.m31 = m10.m21;
        }
        
        public dmat4x2(dmat2 m00, dvec2 m10, dvec2 m20) {
            this.m00 = m00.m00;
            this.m01 = m00.m01;
            this.m10 = m00.m10;
            this.m11 = m00.m11;
            this.m20 = m10.v0;
            this.m21 = m10.v1;
            this.m30 = m20.v0;
            this.m31 = m20.v1;
        }
        
        public dmat4x2(dvec2 m00, dmat2 m10, dvec2 m20) {
            this.m00 = m00.v0;
            this.m01 = m00.v1;
            this.m10 = m10.m00;
            this.m11 = m10.m01;
            this.m20 = m10.m10;
            this.m21 = m10.m11;
            this.m30 = m20.v0;
            this.m31 = m20.v1;
        }
        
        public dmat4x2(dvec2 m00, dvec2 m10, dmat2 m20) {
            this.m00 = m00.v0;
            this.m01 = m00.v1;
            this.m10 = m10.v0;
            this.m11 = m10.v1;
            this.m20 = m20.m00;
            this.m21 = m20.m01;
            this.m30 = m20.m10;
            this.m31 = m20.m11;
        }
        
        public dmat4x2(dvec2 m00, dvec2 m10, dvec2 m20, dvec2 m30) {
            this.m00 = m00.v0;
            this.m01 = m00.v1;
            this.m10 = m10.v0;
            this.m11 = m10.v1;
            this.m20 = m20.v0;
            this.m21 = m20.v1;
            this.m30 = m30.v0;
            this.m31 = m30.v1;
        }
        
        public dmat4x2(double m00, double m01, double m10, double m11, double m20, double m21, double m30, double m31) {
            this.m00 = m00;
            this.m01 = m01;
            this.m10 = m10;
            this.m11 = m11;
            this.m20 = m20;
            this.m21 = m21;
            this.m30 = m30;
            this.m31 = m31;
        }
        
        public dmat4x2(dmat2 s) {
            this.m00 = s.m00;
            this.m01 = s.m01;
            this.m10 = s.m10;
            this.m11 = s.m11;
            this.m20 = 0D;
            this.m21 = 0D;
            this.m30 = 0D;
            this.m31 = 0D;
        }
        
        public dmat4x2(dmat2x3 s) {
            this.m00 = s.m00;
            this.m01 = s.m01;
            this.m10 = s.m10;
            this.m11 = s.m11;
            this.m20 = 0D;
            this.m21 = 0D;
            this.m30 = 0D;
            this.m31 = 0D;
        }
        
        public dmat4x2(dmat2x4 s) {
            this.m00 = s.m00;
            this.m01 = s.m01;
            this.m10 = s.m10;
            this.m11 = s.m11;
            this.m20 = 0D;
            this.m21 = 0D;
            this.m30 = 0D;
            this.m31 = 0D;
        }
        
        public dmat4x2(dmat3x2 s) {
            this.m00 = s.m00;
            this.m01 = s.m01;
            this.m10 = s.m10;
            this.m11 = s.m11;
            this.m20 = s.m20;
            this.m21 = s.m21;
            this.m30 = 0D;
            this.m31 = 0D;
        }
        
        public dmat4x2(dmat3 s) {
            this.m00 = s.m00;
            this.m01 = s.m01;
            this.m10 = s.m10;
            this.m11 = s.m11;
            this.m20 = s.m20;
            this.m21 = s.m21;
            this.m30 = 0D;
            this.m31 = 0D;
        }
        
        public dmat4x2(dmat3x4 s) {
            this.m00 = s.m00;
            this.m01 = s.m01;
            this.m10 = s.m10;
            this.m11 = s.m11;
            this.m20 = s.m20;
            this.m21 = s.m21;
            this.m30 = 0D;
            this.m31 = 0D;
        }
        
        public dmat4x2(dmat4x3 s) {
            this.m00 = s.m00;
            this.m01 = s.m01;
            this.m10 = s.m10;
            this.m11 = s.m11;
            this.m20 = s.m20;
            this.m21 = s.m21;
            this.m30 = s.m30;
            this.m31 = s.m31;
        }
        
        public dmat4x2(dmat4 s) {
            this.m00 = s.m00;
            this.m01 = s.m01;
            this.m10 = s.m10;
            this.m11 = s.m11;
            this.m20 = s.m20;
            this.m21 = s.m21;
            this.m30 = s.m30;
            this.m31 = s.m31;
        }
        
        public double this[int i0, int i1] {
            get {
                if (((i0 < 0) 
                            || (3 < i0))) {
                    throw new System.IndexOutOfRangeException("i0");
                }
                if (((i1 < 0) 
                            || (1 < i1))) {
                    throw new System.IndexOutOfRangeException("i1");
                }
                
                unsafe
                {
                    fixed (Double* v_ptr = &this.m00)
                    {
                        return *(v_ptr + (i1 + i0 * 2));
                    }
                };
            }
            set {
                if (((i0 < 0) 
                            || (3 < i0))) {
                    throw new System.IndexOutOfRangeException("i0");
                }
                if (((i1 < 0) 
                            || (1 < i1))) {
                    throw new System.IndexOutOfRangeException("i1");
                }
                
                unsafe
                {
                    fixed (Double* v_ptr = &this.m00)
                    {
                        *(v_ptr  + (i1 + i0 * 2)) = value;
                    }
                };
            }
        }
        
        public dvec2 this[int i0] {
            get {
                return new dvec2(this[i0, 0], this[i0, 1]);
            }
            set {
                this[i0, 0] = value[0];
                this[i0, 1] = value[1];
            }
        }
        
        public static dmat4x2 operator+(dmat4x2 v) {
            return new dmat4x2(+v.m00, +v.m01, +v.m10, +v.m11, +v.m20, +v.m21, +v.m30, +v.m31);
        }
        
        public static dmat4x2 operator-(dmat4x2 v) {
            return new dmat4x2(-v.m00, -v.m01, -v.m10, -v.m11, -v.m20, -v.m21, -v.m30, -v.m31);
        }
        
        public static dmat4x2 operator++(dmat4x2 v) {
            return new dmat4x2(++v.m00, ++v.m01, ++v.m10, ++v.m11, ++v.m20, ++v.m21, ++v.m30, ++v.m31);
        }
        
        public static dmat4x2 operator--(dmat4x2 v) {
            return new dmat4x2(--v.m00, --v.m01, --v.m10, --v.m11, --v.m20, --v.m21, --v.m30, --v.m31);
        }
        
        public static dmat4x2 operator+(double s, dmat4x2 v2) {
            return new dmat4x2(s + v2.m00, s + v2.m01, s + v2.m10, s + v2.m11, s + v2.m20, s + v2.m21, s + v2.m30, s + v2.m31);
        }
        
        public static dmat4x2 operator-(double s, dmat4x2 v2) {
            return new dmat4x2(s - v2.m00, s - v2.m01, s - v2.m10, s - v2.m11, s - v2.m20, s - v2.m21, s - v2.m30, s - v2.m31);
        }
        
        public static dmat4x2 operator*(double s, dmat4x2 v2) {
            return new dmat4x2(s * v2.m00, s * v2.m01, s * v2.m10, s * v2.m11, s * v2.m20, s * v2.m21, s * v2.m30, s * v2.m31);
        }
        
        public static dmat4x2 operator/(double s, dmat4x2 v2) {
            return new dmat4x2(s / v2.m00, s / v2.m01, s / v2.m10, s / v2.m11, s / v2.m20, s / v2.m21, s / v2.m30, s / v2.m31);
        }
        
        public static dmat4x2 operator%(double s, dmat4x2 v2) {
            return new dmat4x2(s % v2.m00, s % v2.m01, s % v2.m10, s % v2.m11, s % v2.m20, s % v2.m21, s % v2.m30, s % v2.m31);
        }
        
        public static dmat4x2 operator+(dmat4x2 v1, double s) {
            return new dmat4x2(v1.m00 + s, v1.m01 + s, v1.m10 + s, v1.m11 + s, v1.m20 + s, v1.m21 + s, v1.m30 + s, v1.m31 + s);
        }
        
        public static dmat4x2 operator-(dmat4x2 v1, double s) {
            return new dmat4x2(v1.m00 - s, v1.m01 - s, v1.m10 - s, v1.m11 - s, v1.m20 - s, v1.m21 - s, v1.m30 - s, v1.m31 - s);
        }
        
        public static dmat4x2 operator*(dmat4x2 v1, double s) {
            return new dmat4x2(v1.m00 * s, v1.m01 * s, v1.m10 * s, v1.m11 * s, v1.m20 * s, v1.m21 * s, v1.m30 * s, v1.m31 * s);
        }
        
        public static dmat4x2 operator/(dmat4x2 v1, double s) {
            return new dmat4x2(v1.m00 / s, v1.m01 / s, v1.m10 / s, v1.m11 / s, v1.m20 / s, v1.m21 / s, v1.m30 / s, v1.m31 / s);
        }
        
        public static dmat4x2 operator%(dmat4x2 v1, double s) {
            return new dmat4x2(v1.m00 % s, v1.m01 % s, v1.m10 % s, v1.m11 % s, v1.m20 % s, v1.m21 % s, v1.m30 % s, v1.m31 % s);
        }
        
        public static dmat4x2 operator+(dmat4x2 v1, dmat4x2 v2) {
            return new dmat4x2(v1.m00 + v2.m00, v1.m01 + v2.m01, v1.m10 + v2.m10, v1.m11 + v2.m11, v1.m20 + v2.m20, v1.m21 + v2.m21, v1.m30 + v2.m30, v1.m31 + v2.m31);
        }
        
        public static dmat4x2 operator-(dmat4x2 v1, dmat4x2 v2) {
            return new dmat4x2(v1.m00 - v2.m00, v1.m01 - v2.m01, v1.m10 - v2.m10, v1.m11 - v2.m11, v1.m20 - v2.m20, v1.m21 - v2.m21, v1.m30 - v2.m30, v1.m31 - v2.m31);
        }
        
        public static dmat4x2 operator/(dmat4x2 v1, dmat4x2 v2) {
            return new dmat4x2(v1.m00 / v2.m00, v1.m01 / v2.m01, v1.m10 / v2.m10, v1.m11 / v2.m11, v1.m20 / v2.m20, v1.m21 / v2.m21, v1.m30 / v2.m30, v1.m31 / v2.m31);
        }
        
        public static dmat4x2 operator%(dmat4x2 v1, dmat4x2 v2) {
            return new dmat4x2(v1.m00 % v2.m00, v1.m01 % v2.m01, v1.m10 % v2.m10, v1.m11 % v2.m11, v1.m20 % v2.m20, v1.m21 % v2.m21, v1.m30 % v2.m30, v1.m31 % v2.m31);
        }
        
        public static bmat4x2 operator==(dmat4x2 v1, dmat4x2 v2) {
            return new bmat4x2(v1.m00 == v2.m00, v1.m01 == v2.m01, v1.m10 == v2.m10, v1.m11 == v2.m11, v1.m20 == v2.m20, v1.m21 == v2.m21, v1.m30 == v2.m30, v1.m31 == v2.m31);
        }
        
        public static bmat4x2 operator>(dmat4x2 v1, dmat4x2 v2) {
            return new bmat4x2(v1.m00 > v2.m00, v1.m01 > v2.m01, v1.m10 > v2.m10, v1.m11 > v2.m11, v1.m20 > v2.m20, v1.m21 > v2.m21, v1.m30 > v2.m30, v1.m31 > v2.m31);
        }
        
        public static bmat4x2 operator>=(dmat4x2 v1, dmat4x2 v2) {
            return new bmat4x2(v1.m00 >= v2.m00, v1.m01 >= v2.m01, v1.m10 >= v2.m10, v1.m11 >= v2.m11, v1.m20 >= v2.m20, v1.m21 >= v2.m21, v1.m30 >= v2.m30, v1.m31 >= v2.m31);
        }
        
        public static bmat4x2 operator<(dmat4x2 v1, dmat4x2 v2) {
            return new bmat4x2(v1.m00 < v2.m00, v1.m01 < v2.m01, v1.m10 < v2.m10, v1.m11 < v2.m11, v1.m20 < v2.m20, v1.m21 < v2.m21, v1.m30 < v2.m30, v1.m31 < v2.m31);
        }
        
        public static bmat4x2 operator<=(dmat4x2 v1, dmat4x2 v2) {
            return new bmat4x2(v1.m00 <= v2.m00, v1.m01 <= v2.m01, v1.m10 <= v2.m10, v1.m11 <= v2.m11, v1.m20 <= v2.m20, v1.m21 <= v2.m21, v1.m30 <= v2.m30, v1.m31 <= v2.m31);
        }
        
        public static bmat4x2 operator!=(dmat4x2 v1, dmat4x2 v2) {
            return new bmat4x2(v1.m00 != v2.m00, v1.m01 != v2.m01, v1.m10 != v2.m10, v1.m11 != v2.m11, v1.m20 != v2.m20, v1.m21 != v2.m21, v1.m30 != v2.m30, v1.m31 != v2.m31);
        }
        
        public static dvec2 operator*(dvec4 v0, dmat4x2 m1) {
            return dmath.InnerProduct1(ref v0, ref m1);
        }
        
        public static dvec4 operator*(dmat4x2 m0, dvec2 v1) {
            return dmath.InnerProduct1(ref m0, ref v1);
        }
        
        public static dmat4x2 operator*(dmat4x2 m0, dmat2 m1) {
            return dmath.InnerProduct1(ref m0, ref m1);
        }
        
        public static dmat4x3 operator*(dmat4x2 m0, dmat2x3 m1) {
            return dmath.InnerProduct1(ref m0, ref m1);
        }
        
        public static dmat4 operator*(dmat4x2 m0, dmat2x4 m1) {
            return dmath.InnerProduct1(ref m0, ref m1);
        }
        
        public override bool Equals(object obj) {
            return (obj is dmat4x2) ? bmath.All(this == (dmat4x2)obj) : false;
        }
        
        public override int GetHashCode() {
            return this.m00.GetHashCode() ^ this.m01.GetHashCode() ^ this.m10.GetHashCode() ^ this.m11.GetHashCode() ^ this.m20.GetHashCode() ^ this.m21.GetHashCode() ^ this.m30.GetHashCode() ^ this.m31.GetHashCode();
        }
        
        public override string ToString() {
            return "dmat4x2(" + this.m00 + "," + this.m01 + "," + this.m10 + "," + this.m11 + "," + this.m20 + "," + this.m21 + "," + this.m30 + "," + this.m31 + ")";
        }
    }
    
    [DataContract()]
    [StructLayout(LayoutKind.Sequential)]
    public struct dmat4x3 {
        
        [DataMember()]
        internal double m00;
        
        [DataMember()]
        internal double m01;
        
        [DataMember()]
        internal double m02;
        
        [DataMember()]
        internal double m10;
        
        [DataMember()]
        internal double m11;
        
        [DataMember()]
        internal double m12;
        
        [DataMember()]
        internal double m20;
        
        [DataMember()]
        internal double m21;
        
        [DataMember()]
        internal double m22;
        
        [DataMember()]
        internal double m30;
        
        [DataMember()]
        internal double m31;
        
        [DataMember()]
        internal double m32;
        
        public dmat4x3(double s) {
            this.m00 = s;
            this.m01 = 0D;
            this.m02 = 0D;
            this.m10 = 0D;
            this.m11 = s;
            this.m12 = 0D;
            this.m20 = 0D;
            this.m21 = 0D;
            this.m22 = s;
            this.m30 = 0D;
            this.m31 = 0D;
            this.m32 = 0D;
        }
        
        public dmat4x3(dmat4x3 m00) {
            this.m00 = m00.m00;
            this.m01 = m00.m01;
            this.m02 = m00.m02;
            this.m10 = m00.m10;
            this.m11 = m00.m11;
            this.m12 = m00.m12;
            this.m20 = m00.m20;
            this.m21 = m00.m21;
            this.m22 = m00.m22;
            this.m30 = m00.m30;
            this.m31 = m00.m31;
            this.m32 = m00.m32;
        }
        
        public dmat4x3(dmat2x3 m00, dmat2x3 m10) {
            this.m00 = m00.m00;
            this.m01 = m00.m01;
            this.m02 = m00.m02;
            this.m10 = m00.m10;
            this.m11 = m00.m11;
            this.m12 = m00.m12;
            this.m20 = m10.m00;
            this.m21 = m10.m01;
            this.m22 = m10.m02;
            this.m30 = m10.m10;
            this.m31 = m10.m11;
            this.m32 = m10.m12;
        }
        
        public dmat4x3(dmat3 m00, dvec3 m10) {
            this.m00 = m00.m00;
            this.m01 = m00.m01;
            this.m02 = m00.m02;
            this.m10 = m00.m10;
            this.m11 = m00.m11;
            this.m12 = m00.m12;
            this.m20 = m00.m20;
            this.m21 = m00.m21;
            this.m22 = m00.m22;
            this.m30 = m10.v0;
            this.m31 = m10.v1;
            this.m32 = m10.v2;
        }
        
        public dmat4x3(dvec3 m00, dmat3 m10) {
            this.m00 = m00.v0;
            this.m01 = m00.v1;
            this.m02 = m00.v2;
            this.m10 = m10.m00;
            this.m11 = m10.m01;
            this.m12 = m10.m02;
            this.m20 = m10.m10;
            this.m21 = m10.m11;
            this.m22 = m10.m12;
            this.m30 = m10.m20;
            this.m31 = m10.m21;
            this.m32 = m10.m22;
        }
        
        public dmat4x3(dmat2x3 m00, dvec3 m10, dvec3 m20) {
            this.m00 = m00.m00;
            this.m01 = m00.m01;
            this.m02 = m00.m02;
            this.m10 = m00.m10;
            this.m11 = m00.m11;
            this.m12 = m00.m12;
            this.m20 = m10.v0;
            this.m21 = m10.v1;
            this.m22 = m10.v2;
            this.m30 = m20.v0;
            this.m31 = m20.v1;
            this.m32 = m20.v2;
        }
        
        public dmat4x3(dvec3 m00, dmat2x3 m10, dvec3 m20) {
            this.m00 = m00.v0;
            this.m01 = m00.v1;
            this.m02 = m00.v2;
            this.m10 = m10.m00;
            this.m11 = m10.m01;
            this.m12 = m10.m02;
            this.m20 = m10.m10;
            this.m21 = m10.m11;
            this.m22 = m10.m12;
            this.m30 = m20.v0;
            this.m31 = m20.v1;
            this.m32 = m20.v2;
        }
        
        public dmat4x3(dvec3 m00, dvec3 m10, dmat2x3 m20) {
            this.m00 = m00.v0;
            this.m01 = m00.v1;
            this.m02 = m00.v2;
            this.m10 = m10.v0;
            this.m11 = m10.v1;
            this.m12 = m10.v2;
            this.m20 = m20.m00;
            this.m21 = m20.m01;
            this.m22 = m20.m02;
            this.m30 = m20.m10;
            this.m31 = m20.m11;
            this.m32 = m20.m12;
        }
        
        public dmat4x3(dvec3 m00, dvec3 m10, dvec3 m20, dvec3 m30) {
            this.m00 = m00.v0;
            this.m01 = m00.v1;
            this.m02 = m00.v2;
            this.m10 = m10.v0;
            this.m11 = m10.v1;
            this.m12 = m10.v2;
            this.m20 = m20.v0;
            this.m21 = m20.v1;
            this.m22 = m20.v2;
            this.m30 = m30.v0;
            this.m31 = m30.v1;
            this.m32 = m30.v2;
        }
        
        public dmat4x3(dvec2 m00, double m01, dvec2 m10, double m11, dvec2 m20, double m21, dvec2 m30, double m31) {
            this.m00 = m00.v0;
            this.m01 = m00.v1;
            this.m02 = m01;
            this.m10 = m10.v0;
            this.m11 = m10.v1;
            this.m12 = m11;
            this.m20 = m20.v0;
            this.m21 = m20.v1;
            this.m22 = m21;
            this.m30 = m30.v0;
            this.m31 = m30.v1;
            this.m32 = m31;
        }
        
        public dmat4x3(double m00, dvec2 m01, double m10, dvec2 m11, double m20, dvec2 m21, double m30, dvec2 m31) {
            this.m00 = m00;
            this.m01 = m01.v0;
            this.m02 = m01.v1;
            this.m10 = m10;
            this.m11 = m11.v0;
            this.m12 = m11.v1;
            this.m20 = m20;
            this.m21 = m21.v0;
            this.m22 = m21.v1;
            this.m30 = m30;
            this.m31 = m31.v0;
            this.m32 = m31.v1;
        }
        
        public dmat4x3(double m00, double m01, double m02, double m10, double m11, double m12, double m20, double m21, double m22, double m30, double m31, double m32) {
            this.m00 = m00;
            this.m01 = m01;
            this.m02 = m02;
            this.m10 = m10;
            this.m11 = m11;
            this.m12 = m12;
            this.m20 = m20;
            this.m21 = m21;
            this.m22 = m22;
            this.m30 = m30;
            this.m31 = m31;
            this.m32 = m32;
        }
        
        public dmat4x3(dmat2 s) {
            this.m00 = s.m00;
            this.m01 = s.m01;
            this.m02 = 0D;
            this.m10 = s.m10;
            this.m11 = s.m11;
            this.m12 = 0D;
            this.m20 = 0D;
            this.m21 = 0D;
            this.m22 = 0D;
            this.m30 = 0D;
            this.m31 = 0D;
            this.m32 = 0D;
        }
        
        public dmat4x3(dmat2x3 s) {
            this.m00 = s.m00;
            this.m01 = s.m01;
            this.m02 = s.m02;
            this.m10 = s.m10;
            this.m11 = s.m11;
            this.m12 = s.m12;
            this.m20 = 0D;
            this.m21 = 0D;
            this.m22 = 0D;
            this.m30 = 0D;
            this.m31 = 0D;
            this.m32 = 0D;
        }
        
        public dmat4x3(dmat2x4 s) {
            this.m00 = s.m00;
            this.m01 = s.m01;
            this.m02 = s.m02;
            this.m10 = s.m10;
            this.m11 = s.m11;
            this.m12 = s.m12;
            this.m20 = 0D;
            this.m21 = 0D;
            this.m22 = 0D;
            this.m30 = 0D;
            this.m31 = 0D;
            this.m32 = 0D;
        }
        
        public dmat4x3(dmat3x2 s) {
            this.m00 = s.m00;
            this.m01 = s.m01;
            this.m02 = 0D;
            this.m10 = s.m10;
            this.m11 = s.m11;
            this.m12 = 0D;
            this.m20 = s.m20;
            this.m21 = s.m21;
            this.m22 = 0D;
            this.m30 = 0D;
            this.m31 = 0D;
            this.m32 = 0D;
        }
        
        public dmat4x3(dmat3 s) {
            this.m00 = s.m00;
            this.m01 = s.m01;
            this.m02 = s.m02;
            this.m10 = s.m10;
            this.m11 = s.m11;
            this.m12 = s.m12;
            this.m20 = s.m20;
            this.m21 = s.m21;
            this.m22 = s.m22;
            this.m30 = 0D;
            this.m31 = 0D;
            this.m32 = 0D;
        }
        
        public dmat4x3(dmat3x4 s) {
            this.m00 = s.m00;
            this.m01 = s.m01;
            this.m02 = s.m02;
            this.m10 = s.m10;
            this.m11 = s.m11;
            this.m12 = s.m12;
            this.m20 = s.m20;
            this.m21 = s.m21;
            this.m22 = s.m22;
            this.m30 = 0D;
            this.m31 = 0D;
            this.m32 = 0D;
        }
        
        public dmat4x3(dmat4x2 s) {
            this.m00 = s.m00;
            this.m01 = s.m01;
            this.m02 = 0D;
            this.m10 = s.m10;
            this.m11 = s.m11;
            this.m12 = 0D;
            this.m20 = s.m20;
            this.m21 = s.m21;
            this.m22 = 0D;
            this.m30 = s.m30;
            this.m31 = s.m31;
            this.m32 = 0D;
        }
        
        public dmat4x3(dmat4 s) {
            this.m00 = s.m00;
            this.m01 = s.m01;
            this.m02 = s.m02;
            this.m10 = s.m10;
            this.m11 = s.m11;
            this.m12 = s.m12;
            this.m20 = s.m20;
            this.m21 = s.m21;
            this.m22 = s.m22;
            this.m30 = s.m30;
            this.m31 = s.m31;
            this.m32 = s.m32;
        }
        
        public double this[int i0, int i1] {
            get {
                if (((i0 < 0) 
                            || (3 < i0))) {
                    throw new System.IndexOutOfRangeException("i0");
                }
                if (((i1 < 0) 
                            || (2 < i1))) {
                    throw new System.IndexOutOfRangeException("i1");
                }
                
                unsafe
                {
                    fixed (Double* v_ptr = &this.m00)
                    {
                        return *(v_ptr + (i1 + i0 * 3));
                    }
                };
            }
            set {
                if (((i0 < 0) 
                            || (3 < i0))) {
                    throw new System.IndexOutOfRangeException("i0");
                }
                if (((i1 < 0) 
                            || (2 < i1))) {
                    throw new System.IndexOutOfRangeException("i1");
                }
                
                unsafe
                {
                    fixed (Double* v_ptr = &this.m00)
                    {
                        *(v_ptr  + (i1 + i0 * 3)) = value;
                    }
                };
            }
        }
        
        public dvec3 this[int i0] {
            get {
                return new dvec3(this[i0, 0], this[i0, 1], this[i0, 2]);
            }
            set {
                this[i0, 0] = value[0];
                this[i0, 1] = value[1];
                this[i0, 2] = value[2];
            }
        }
        
        public static dmat4x3 operator+(dmat4x3 v) {
            return new dmat4x3(+v.m00, +v.m01, +v.m02, +v.m10, +v.m11, +v.m12, +v.m20, +v.m21, +v.m22, +v.m30, +v.m31, +v.m32);
        }
        
        public static dmat4x3 operator-(dmat4x3 v) {
            return new dmat4x3(-v.m00, -v.m01, -v.m02, -v.m10, -v.m11, -v.m12, -v.m20, -v.m21, -v.m22, -v.m30, -v.m31, -v.m32);
        }
        
        public static dmat4x3 operator++(dmat4x3 v) {
            return new dmat4x3(++v.m00, ++v.m01, ++v.m02, ++v.m10, ++v.m11, ++v.m12, ++v.m20, ++v.m21, ++v.m22, ++v.m30, ++v.m31, ++v.m32);
        }
        
        public static dmat4x3 operator--(dmat4x3 v) {
            return new dmat4x3(--v.m00, --v.m01, --v.m02, --v.m10, --v.m11, --v.m12, --v.m20, --v.m21, --v.m22, --v.m30, --v.m31, --v.m32);
        }
        
        public static dmat4x3 operator+(double s, dmat4x3 v2) {
            return new dmat4x3(s + v2.m00, s + v2.m01, s + v2.m02, s + v2.m10, s + v2.m11, s + v2.m12, s + v2.m20, s + v2.m21, s + v2.m22, s + v2.m30, s + v2.m31, s + v2.m32);
        }
        
        public static dmat4x3 operator-(double s, dmat4x3 v2) {
            return new dmat4x3(s - v2.m00, s - v2.m01, s - v2.m02, s - v2.m10, s - v2.m11, s - v2.m12, s - v2.m20, s - v2.m21, s - v2.m22, s - v2.m30, s - v2.m31, s - v2.m32);
        }
        
        public static dmat4x3 operator*(double s, dmat4x3 v2) {
            return new dmat4x3(s * v2.m00, s * v2.m01, s * v2.m02, s * v2.m10, s * v2.m11, s * v2.m12, s * v2.m20, s * v2.m21, s * v2.m22, s * v2.m30, s * v2.m31, s * v2.m32);
        }
        
        public static dmat4x3 operator/(double s, dmat4x3 v2) {
            return new dmat4x3(s / v2.m00, s / v2.m01, s / v2.m02, s / v2.m10, s / v2.m11, s / v2.m12, s / v2.m20, s / v2.m21, s / v2.m22, s / v2.m30, s / v2.m31, s / v2.m32);
        }
        
        public static dmat4x3 operator%(double s, dmat4x3 v2) {
            return new dmat4x3(s % v2.m00, s % v2.m01, s % v2.m02, s % v2.m10, s % v2.m11, s % v2.m12, s % v2.m20, s % v2.m21, s % v2.m22, s % v2.m30, s % v2.m31, s % v2.m32);
        }
        
        public static dmat4x3 operator+(dmat4x3 v1, double s) {
            return new dmat4x3(v1.m00 + s, v1.m01 + s, v1.m02 + s, v1.m10 + s, v1.m11 + s, v1.m12 + s, v1.m20 + s, v1.m21 + s, v1.m22 + s, v1.m30 + s, v1.m31 + s, v1.m32 + s);
        }
        
        public static dmat4x3 operator-(dmat4x3 v1, double s) {
            return new dmat4x3(v1.m00 - s, v1.m01 - s, v1.m02 - s, v1.m10 - s, v1.m11 - s, v1.m12 - s, v1.m20 - s, v1.m21 - s, v1.m22 - s, v1.m30 - s, v1.m31 - s, v1.m32 - s);
        }
        
        public static dmat4x3 operator*(dmat4x3 v1, double s) {
            return new dmat4x3(v1.m00 * s, v1.m01 * s, v1.m02 * s, v1.m10 * s, v1.m11 * s, v1.m12 * s, v1.m20 * s, v1.m21 * s, v1.m22 * s, v1.m30 * s, v1.m31 * s, v1.m32 * s);
        }
        
        public static dmat4x3 operator/(dmat4x3 v1, double s) {
            return new dmat4x3(v1.m00 / s, v1.m01 / s, v1.m02 / s, v1.m10 / s, v1.m11 / s, v1.m12 / s, v1.m20 / s, v1.m21 / s, v1.m22 / s, v1.m30 / s, v1.m31 / s, v1.m32 / s);
        }
        
        public static dmat4x3 operator%(dmat4x3 v1, double s) {
            return new dmat4x3(v1.m00 % s, v1.m01 % s, v1.m02 % s, v1.m10 % s, v1.m11 % s, v1.m12 % s, v1.m20 % s, v1.m21 % s, v1.m22 % s, v1.m30 % s, v1.m31 % s, v1.m32 % s);
        }
        
        public static dmat4x3 operator+(dmat4x3 v1, dmat4x3 v2) {
            return new dmat4x3(v1.m00 + v2.m00, v1.m01 + v2.m01, v1.m02 + v2.m02, v1.m10 + v2.m10, v1.m11 + v2.m11, v1.m12 + v2.m12, v1.m20 + v2.m20, v1.m21 + v2.m21, v1.m22 + v2.m22, v1.m30 + v2.m30, v1.m31 + v2.m31, v1.m32 + v2.m32);
        }
        
        public static dmat4x3 operator-(dmat4x3 v1, dmat4x3 v2) {
            return new dmat4x3(v1.m00 - v2.m00, v1.m01 - v2.m01, v1.m02 - v2.m02, v1.m10 - v2.m10, v1.m11 - v2.m11, v1.m12 - v2.m12, v1.m20 - v2.m20, v1.m21 - v2.m21, v1.m22 - v2.m22, v1.m30 - v2.m30, v1.m31 - v2.m31, v1.m32 - v2.m32);
        }
        
        public static dmat4x3 operator/(dmat4x3 v1, dmat4x3 v2) {
            return new dmat4x3(v1.m00 / v2.m00, v1.m01 / v2.m01, v1.m02 / v2.m02, v1.m10 / v2.m10, v1.m11 / v2.m11, v1.m12 / v2.m12, v1.m20 / v2.m20, v1.m21 / v2.m21, v1.m22 / v2.m22, v1.m30 / v2.m30, v1.m31 / v2.m31, v1.m32 / v2.m32);
        }
        
        public static dmat4x3 operator%(dmat4x3 v1, dmat4x3 v2) {
            return new dmat4x3(v1.m00 % v2.m00, v1.m01 % v2.m01, v1.m02 % v2.m02, v1.m10 % v2.m10, v1.m11 % v2.m11, v1.m12 % v2.m12, v1.m20 % v2.m20, v1.m21 % v2.m21, v1.m22 % v2.m22, v1.m30 % v2.m30, v1.m31 % v2.m31, v1.m32 % v2.m32);
        }
        
        public static bmat4x3 operator==(dmat4x3 v1, dmat4x3 v2) {
            return new bmat4x3(v1.m00 == v2.m00, v1.m01 == v2.m01, v1.m02 == v2.m02, v1.m10 == v2.m10, v1.m11 == v2.m11, v1.m12 == v2.m12, v1.m20 == v2.m20, v1.m21 == v2.m21, v1.m22 == v2.m22, v1.m30 == v2.m30, v1.m31 == v2.m31, v1.m32 == v2.m32);
        }
        
        public static bmat4x3 operator>(dmat4x3 v1, dmat4x3 v2) {
            return new bmat4x3(v1.m00 > v2.m00, v1.m01 > v2.m01, v1.m02 > v2.m02, v1.m10 > v2.m10, v1.m11 > v2.m11, v1.m12 > v2.m12, v1.m20 > v2.m20, v1.m21 > v2.m21, v1.m22 > v2.m22, v1.m30 > v2.m30, v1.m31 > v2.m31, v1.m32 > v2.m32);
        }
        
        public static bmat4x3 operator>=(dmat4x3 v1, dmat4x3 v2) {
            return new bmat4x3(v1.m00 >= v2.m00, v1.m01 >= v2.m01, v1.m02 >= v2.m02, v1.m10 >= v2.m10, v1.m11 >= v2.m11, v1.m12 >= v2.m12, v1.m20 >= v2.m20, v1.m21 >= v2.m21, v1.m22 >= v2.m22, v1.m30 >= v2.m30, v1.m31 >= v2.m31, v1.m32 >= v2.m32);
        }
        
        public static bmat4x3 operator<(dmat4x3 v1, dmat4x3 v2) {
            return new bmat4x3(v1.m00 < v2.m00, v1.m01 < v2.m01, v1.m02 < v2.m02, v1.m10 < v2.m10, v1.m11 < v2.m11, v1.m12 < v2.m12, v1.m20 < v2.m20, v1.m21 < v2.m21, v1.m22 < v2.m22, v1.m30 < v2.m30, v1.m31 < v2.m31, v1.m32 < v2.m32);
        }
        
        public static bmat4x3 operator<=(dmat4x3 v1, dmat4x3 v2) {
            return new bmat4x3(v1.m00 <= v2.m00, v1.m01 <= v2.m01, v1.m02 <= v2.m02, v1.m10 <= v2.m10, v1.m11 <= v2.m11, v1.m12 <= v2.m12, v1.m20 <= v2.m20, v1.m21 <= v2.m21, v1.m22 <= v2.m22, v1.m30 <= v2.m30, v1.m31 <= v2.m31, v1.m32 <= v2.m32);
        }
        
        public static bmat4x3 operator!=(dmat4x3 v1, dmat4x3 v2) {
            return new bmat4x3(v1.m00 != v2.m00, v1.m01 != v2.m01, v1.m02 != v2.m02, v1.m10 != v2.m10, v1.m11 != v2.m11, v1.m12 != v2.m12, v1.m20 != v2.m20, v1.m21 != v2.m21, v1.m22 != v2.m22, v1.m30 != v2.m30, v1.m31 != v2.m31, v1.m32 != v2.m32);
        }
        
        public static dvec3 operator*(dvec4 v0, dmat4x3 m1) {
            return dmath.InnerProduct1(ref v0, ref m1);
        }
        
        public static dvec4 operator*(dmat4x3 m0, dvec3 v1) {
            return dmath.InnerProduct1(ref m0, ref v1);
        }
        
        public static dmat4x2 operator*(dmat4x3 m0, dmat3x2 m1) {
            return dmath.InnerProduct1(ref m0, ref m1);
        }
        
        public static dmat4x3 operator*(dmat4x3 m0, dmat3 m1) {
            return dmath.InnerProduct1(ref m0, ref m1);
        }
        
        public static dmat4 operator*(dmat4x3 m0, dmat3x4 m1) {
            return dmath.InnerProduct1(ref m0, ref m1);
        }
        
        public override bool Equals(object obj) {
            return (obj is dmat4x3) ? bmath.All(this == (dmat4x3)obj) : false;
        }
        
        public override int GetHashCode() {
            return this.m00.GetHashCode() ^ this.m01.GetHashCode() ^ this.m02.GetHashCode() ^ this.m10.GetHashCode() ^ this.m11.GetHashCode() ^ this.m12.GetHashCode() ^ this.m20.GetHashCode() ^ this.m21.GetHashCode() ^ this.m22.GetHashCode() ^ this.m30.GetHashCode() ^ this.m31.GetHashCode() ^ this.m32.GetHashCode();
        }
        
        public override string ToString() {
            return "dmat4x3(" + this.m00 + "," + this.m01 + "," + this.m02 + "," + this.m10 + "," + this.m11 + "," + this.m12 + "," + this.m20 + "," + this.m21 + "," + this.m22 + "," + this.m30 + "," + this.m31 + "," + this.m32 + ")";
        }
    }
    
    [DataContract()]
    [StructLayout(LayoutKind.Sequential)]
    public struct dmat4 {
        
        [DataMember()]
        internal double m00;
        
        [DataMember()]
        internal double m01;
        
        [DataMember()]
        internal double m02;
        
        [DataMember()]
        internal double m03;
        
        [DataMember()]
        internal double m10;
        
        [DataMember()]
        internal double m11;
        
        [DataMember()]
        internal double m12;
        
        [DataMember()]
        internal double m13;
        
        [DataMember()]
        internal double m20;
        
        [DataMember()]
        internal double m21;
        
        [DataMember()]
        internal double m22;
        
        [DataMember()]
        internal double m23;
        
        [DataMember()]
        internal double m30;
        
        [DataMember()]
        internal double m31;
        
        [DataMember()]
        internal double m32;
        
        [DataMember()]
        internal double m33;
        
        public dmat4(double s) {
            this.m00 = s;
            this.m01 = 0D;
            this.m02 = 0D;
            this.m03 = 0D;
            this.m10 = 0D;
            this.m11 = s;
            this.m12 = 0D;
            this.m13 = 0D;
            this.m20 = 0D;
            this.m21 = 0D;
            this.m22 = s;
            this.m23 = 0D;
            this.m30 = 0D;
            this.m31 = 0D;
            this.m32 = 0D;
            this.m33 = s;
        }
        
        public dmat4(dmat4 m00) {
            this.m00 = m00.m00;
            this.m01 = m00.m01;
            this.m02 = m00.m02;
            this.m03 = m00.m03;
            this.m10 = m00.m10;
            this.m11 = m00.m11;
            this.m12 = m00.m12;
            this.m13 = m00.m13;
            this.m20 = m00.m20;
            this.m21 = m00.m21;
            this.m22 = m00.m22;
            this.m23 = m00.m23;
            this.m30 = m00.m30;
            this.m31 = m00.m31;
            this.m32 = m00.m32;
            this.m33 = m00.m33;
        }
        
        public dmat4(dmat4x2 m00, dmat4x2 m01) {
            this.m00 = m00.m00;
            this.m01 = m00.m01;
            this.m02 = m01.m00;
            this.m03 = m01.m01;
            this.m10 = m00.m10;
            this.m11 = m00.m11;
            this.m12 = m01.m10;
            this.m13 = m01.m11;
            this.m20 = m00.m20;
            this.m21 = m00.m21;
            this.m22 = m01.m20;
            this.m23 = m01.m21;
            this.m30 = m00.m30;
            this.m31 = m00.m31;
            this.m32 = m01.m30;
            this.m33 = m01.m31;
        }
        
        public dmat4(dmat2x4 m00, dmat2x4 m10) {
            this.m00 = m00.m00;
            this.m01 = m00.m01;
            this.m02 = m00.m02;
            this.m03 = m00.m03;
            this.m10 = m00.m10;
            this.m11 = m00.m11;
            this.m12 = m00.m12;
            this.m13 = m00.m13;
            this.m20 = m10.m00;
            this.m21 = m10.m01;
            this.m22 = m10.m02;
            this.m23 = m10.m03;
            this.m30 = m10.m10;
            this.m31 = m10.m11;
            this.m32 = m10.m12;
            this.m33 = m10.m13;
        }
        
        public dmat4(dmat2 m00, dmat2 m01, dmat2 m10, dmat2 m11) {
            this.m00 = m00.m00;
            this.m01 = m00.m01;
            this.m02 = m01.m00;
            this.m03 = m01.m01;
            this.m10 = m00.m10;
            this.m11 = m00.m11;
            this.m12 = m01.m10;
            this.m13 = m01.m11;
            this.m20 = m10.m00;
            this.m21 = m10.m01;
            this.m22 = m11.m00;
            this.m23 = m11.m01;
            this.m30 = m10.m10;
            this.m31 = m10.m11;
            this.m32 = m11.m10;
            this.m33 = m11.m11;
        }
        
        public dmat4(dmat3x4 m00, dvec4 m10) {
            this.m00 = m00.m00;
            this.m01 = m00.m01;
            this.m02 = m00.m02;
            this.m03 = m00.m03;
            this.m10 = m00.m10;
            this.m11 = m00.m11;
            this.m12 = m00.m12;
            this.m13 = m00.m13;
            this.m20 = m00.m20;
            this.m21 = m00.m21;
            this.m22 = m00.m22;
            this.m23 = m00.m23;
            this.m30 = m10.v0;
            this.m31 = m10.v1;
            this.m32 = m10.v2;
            this.m33 = m10.v3;
        }
        
        public dmat4(dmat3x2 m00, dmat3x2 m01, dvec2 m10, dvec2 m11) {
            this.m00 = m00.m00;
            this.m01 = m00.m01;
            this.m02 = m01.m00;
            this.m03 = m01.m01;
            this.m10 = m00.m10;
            this.m11 = m00.m11;
            this.m12 = m01.m10;
            this.m13 = m01.m11;
            this.m20 = m00.m20;
            this.m21 = m00.m21;
            this.m22 = m01.m20;
            this.m23 = m01.m21;
            this.m30 = m10.v0;
            this.m31 = m10.v1;
            this.m32 = m11.v0;
            this.m33 = m11.v1;
        }
        
        public dmat4(dvec4 m00, dmat3x4 m10) {
            this.m00 = m00.v0;
            this.m01 = m00.v1;
            this.m02 = m00.v2;
            this.m03 = m00.v3;
            this.m10 = m10.m00;
            this.m11 = m10.m01;
            this.m12 = m10.m02;
            this.m13 = m10.m03;
            this.m20 = m10.m10;
            this.m21 = m10.m11;
            this.m22 = m10.m12;
            this.m23 = m10.m13;
            this.m30 = m10.m20;
            this.m31 = m10.m21;
            this.m32 = m10.m22;
            this.m33 = m10.m23;
        }
        
        public dmat4(dvec2 m00, dvec2 m01, dmat3x2 m10, dmat3x2 m11) {
            this.m00 = m00.v0;
            this.m01 = m00.v1;
            this.m02 = m01.v0;
            this.m03 = m01.v1;
            this.m10 = m10.m00;
            this.m11 = m10.m01;
            this.m12 = m11.m00;
            this.m13 = m11.m01;
            this.m20 = m10.m10;
            this.m21 = m10.m11;
            this.m22 = m11.m10;
            this.m23 = m11.m11;
            this.m30 = m10.m20;
            this.m31 = m10.m21;
            this.m32 = m11.m20;
            this.m33 = m11.m21;
        }
        
        public dmat4(dmat2x4 m00, dvec4 m10, dvec4 m20) {
            this.m00 = m00.m00;
            this.m01 = m00.m01;
            this.m02 = m00.m02;
            this.m03 = m00.m03;
            this.m10 = m00.m10;
            this.m11 = m00.m11;
            this.m12 = m00.m12;
            this.m13 = m00.m13;
            this.m20 = m10.v0;
            this.m21 = m10.v1;
            this.m22 = m10.v2;
            this.m23 = m10.v3;
            this.m30 = m20.v0;
            this.m31 = m20.v1;
            this.m32 = m20.v2;
            this.m33 = m20.v3;
        }
        
        public dmat4(dmat2 m00, dmat2 m01, dvec2 m10, dvec2 m11, dvec2 m20, dvec2 m21) {
            this.m00 = m00.m00;
            this.m01 = m00.m01;
            this.m02 = m01.m00;
            this.m03 = m01.m01;
            this.m10 = m00.m10;
            this.m11 = m00.m11;
            this.m12 = m01.m10;
            this.m13 = m01.m11;
            this.m20 = m10.v0;
            this.m21 = m10.v1;
            this.m22 = m11.v0;
            this.m23 = m11.v1;
            this.m30 = m20.v0;
            this.m31 = m20.v1;
            this.m32 = m21.v0;
            this.m33 = m21.v1;
        }
        
        public dmat4(dvec4 m00, dmat2x4 m10, dvec4 m20) {
            this.m00 = m00.v0;
            this.m01 = m00.v1;
            this.m02 = m00.v2;
            this.m03 = m00.v3;
            this.m10 = m10.m00;
            this.m11 = m10.m01;
            this.m12 = m10.m02;
            this.m13 = m10.m03;
            this.m20 = m10.m10;
            this.m21 = m10.m11;
            this.m22 = m10.m12;
            this.m23 = m10.m13;
            this.m30 = m20.v0;
            this.m31 = m20.v1;
            this.m32 = m20.v2;
            this.m33 = m20.v3;
        }
        
        public dmat4(dvec2 m00, dvec2 m01, dmat2 m10, dmat2 m11, dvec2 m20, dvec2 m21) {
            this.m00 = m00.v0;
            this.m01 = m00.v1;
            this.m02 = m01.v0;
            this.m03 = m01.v1;
            this.m10 = m10.m00;
            this.m11 = m10.m01;
            this.m12 = m11.m00;
            this.m13 = m11.m01;
            this.m20 = m10.m10;
            this.m21 = m10.m11;
            this.m22 = m11.m10;
            this.m23 = m11.m11;
            this.m30 = m20.v0;
            this.m31 = m20.v1;
            this.m32 = m21.v0;
            this.m33 = m21.v1;
        }
        
        public dmat4(dvec4 m00, dvec4 m10, dmat2x4 m20) {
            this.m00 = m00.v0;
            this.m01 = m00.v1;
            this.m02 = m00.v2;
            this.m03 = m00.v3;
            this.m10 = m10.v0;
            this.m11 = m10.v1;
            this.m12 = m10.v2;
            this.m13 = m10.v3;
            this.m20 = m20.m00;
            this.m21 = m20.m01;
            this.m22 = m20.m02;
            this.m23 = m20.m03;
            this.m30 = m20.m10;
            this.m31 = m20.m11;
            this.m32 = m20.m12;
            this.m33 = m20.m13;
        }
        
        public dmat4(dvec2 m00, dvec2 m01, dvec2 m10, dvec2 m11, dmat2 m20, dmat2 m21) {
            this.m00 = m00.v0;
            this.m01 = m00.v1;
            this.m02 = m01.v0;
            this.m03 = m01.v1;
            this.m10 = m10.v0;
            this.m11 = m10.v1;
            this.m12 = m11.v0;
            this.m13 = m11.v1;
            this.m20 = m20.m00;
            this.m21 = m20.m01;
            this.m22 = m21.m00;
            this.m23 = m21.m01;
            this.m30 = m20.m10;
            this.m31 = m20.m11;
            this.m32 = m21.m10;
            this.m33 = m21.m11;
        }
        
        public dmat4(dvec4 m00, dvec4 m10, dvec4 m20, dvec4 m30) {
            this.m00 = m00.v0;
            this.m01 = m00.v1;
            this.m02 = m00.v2;
            this.m03 = m00.v3;
            this.m10 = m10.v0;
            this.m11 = m10.v1;
            this.m12 = m10.v2;
            this.m13 = m10.v3;
            this.m20 = m20.v0;
            this.m21 = m20.v1;
            this.m22 = m20.v2;
            this.m23 = m20.v3;
            this.m30 = m30.v0;
            this.m31 = m30.v1;
            this.m32 = m30.v2;
            this.m33 = m30.v3;
        }
        
        public dmat4(dvec2 m00, dvec2 m01, dvec2 m10, dvec2 m11, dvec2 m20, dvec2 m21, dvec2 m30, dvec2 m31) {
            this.m00 = m00.v0;
            this.m01 = m00.v1;
            this.m02 = m01.v0;
            this.m03 = m01.v1;
            this.m10 = m10.v0;
            this.m11 = m10.v1;
            this.m12 = m11.v0;
            this.m13 = m11.v1;
            this.m20 = m20.v0;
            this.m21 = m20.v1;
            this.m22 = m21.v0;
            this.m23 = m21.v1;
            this.m30 = m30.v0;
            this.m31 = m30.v1;
            this.m32 = m31.v0;
            this.m33 = m31.v1;
        }
        
        public dmat4(dvec3 m00, double m01, dvec3 m10, double m11, dvec3 m20, double m21, dvec3 m30, double m31) {
            this.m00 = m00.v0;
            this.m01 = m00.v1;
            this.m02 = m00.v2;
            this.m03 = m01;
            this.m10 = m10.v0;
            this.m11 = m10.v1;
            this.m12 = m10.v2;
            this.m13 = m11;
            this.m20 = m20.v0;
            this.m21 = m20.v1;
            this.m22 = m20.v2;
            this.m23 = m21;
            this.m30 = m30.v0;
            this.m31 = m30.v1;
            this.m32 = m30.v2;
            this.m33 = m31;
        }
        
        public dmat4(double m00, dvec3 m01, double m10, dvec3 m11, double m20, dvec3 m21, double m30, dvec3 m31) {
            this.m00 = m00;
            this.m01 = m01.v0;
            this.m02 = m01.v1;
            this.m03 = m01.v2;
            this.m10 = m10;
            this.m11 = m11.v0;
            this.m12 = m11.v1;
            this.m13 = m11.v2;
            this.m20 = m20;
            this.m21 = m21.v0;
            this.m22 = m21.v1;
            this.m23 = m21.v2;
            this.m30 = m30;
            this.m31 = m31.v0;
            this.m32 = m31.v1;
            this.m33 = m31.v2;
        }
        
        public dmat4(dvec2 m00, double m01, double m02, dvec2 m10, double m11, double m12, dvec2 m20, double m21, double m22, dvec2 m30, double m31, double m32) {
            this.m00 = m00.v0;
            this.m01 = m00.v1;
            this.m02 = m01;
            this.m03 = m02;
            this.m10 = m10.v0;
            this.m11 = m10.v1;
            this.m12 = m11;
            this.m13 = m12;
            this.m20 = m20.v0;
            this.m21 = m20.v1;
            this.m22 = m21;
            this.m23 = m22;
            this.m30 = m30.v0;
            this.m31 = m30.v1;
            this.m32 = m31;
            this.m33 = m32;
        }
        
        public dmat4(double m00, dvec2 m01, double m02, double m10, dvec2 m11, double m12, double m20, dvec2 m21, double m22, double m30, dvec2 m31, double m32) {
            this.m00 = m00;
            this.m01 = m01.v0;
            this.m02 = m01.v1;
            this.m03 = m02;
            this.m10 = m10;
            this.m11 = m11.v0;
            this.m12 = m11.v1;
            this.m13 = m12;
            this.m20 = m20;
            this.m21 = m21.v0;
            this.m22 = m21.v1;
            this.m23 = m22;
            this.m30 = m30;
            this.m31 = m31.v0;
            this.m32 = m31.v1;
            this.m33 = m32;
        }
        
        public dmat4(double m00, double m01, dvec2 m02, double m10, double m11, dvec2 m12, double m20, double m21, dvec2 m22, double m30, double m31, dvec2 m32) {
            this.m00 = m00;
            this.m01 = m01;
            this.m02 = m02.v0;
            this.m03 = m02.v1;
            this.m10 = m10;
            this.m11 = m11;
            this.m12 = m12.v0;
            this.m13 = m12.v1;
            this.m20 = m20;
            this.m21 = m21;
            this.m22 = m22.v0;
            this.m23 = m22.v1;
            this.m30 = m30;
            this.m31 = m31;
            this.m32 = m32.v0;
            this.m33 = m32.v1;
        }
        
        public dmat4(
                    double m00, 
                    double m01, 
                    double m02, 
                    double m03, 
                    double m10, 
                    double m11, 
                    double m12, 
                    double m13, 
                    double m20, 
                    double m21, 
                    double m22, 
                    double m23, 
                    double m30, 
                    double m31, 
                    double m32, 
                    double m33) {
            this.m00 = m00;
            this.m01 = m01;
            this.m02 = m02;
            this.m03 = m03;
            this.m10 = m10;
            this.m11 = m11;
            this.m12 = m12;
            this.m13 = m13;
            this.m20 = m20;
            this.m21 = m21;
            this.m22 = m22;
            this.m23 = m23;
            this.m30 = m30;
            this.m31 = m31;
            this.m32 = m32;
            this.m33 = m33;
        }
        
        public dmat4(dmat2 s) {
            this.m00 = s.m00;
            this.m01 = s.m01;
            this.m02 = 0D;
            this.m03 = 0D;
            this.m10 = s.m10;
            this.m11 = s.m11;
            this.m12 = 0D;
            this.m13 = 0D;
            this.m20 = 0D;
            this.m21 = 0D;
            this.m22 = 0D;
            this.m23 = 0D;
            this.m30 = 0D;
            this.m31 = 0D;
            this.m32 = 0D;
            this.m33 = 0D;
        }
        
        public dmat4(dmat2x3 s) {
            this.m00 = s.m00;
            this.m01 = s.m01;
            this.m02 = s.m02;
            this.m03 = 0D;
            this.m10 = s.m10;
            this.m11 = s.m11;
            this.m12 = s.m12;
            this.m13 = 0D;
            this.m20 = 0D;
            this.m21 = 0D;
            this.m22 = 0D;
            this.m23 = 0D;
            this.m30 = 0D;
            this.m31 = 0D;
            this.m32 = 0D;
            this.m33 = 0D;
        }
        
        public dmat4(dmat2x4 s) {
            this.m00 = s.m00;
            this.m01 = s.m01;
            this.m02 = s.m02;
            this.m03 = s.m03;
            this.m10 = s.m10;
            this.m11 = s.m11;
            this.m12 = s.m12;
            this.m13 = s.m13;
            this.m20 = 0D;
            this.m21 = 0D;
            this.m22 = 0D;
            this.m23 = 0D;
            this.m30 = 0D;
            this.m31 = 0D;
            this.m32 = 0D;
            this.m33 = 0D;
        }
        
        public dmat4(dmat3x2 s) {
            this.m00 = s.m00;
            this.m01 = s.m01;
            this.m02 = 0D;
            this.m03 = 0D;
            this.m10 = s.m10;
            this.m11 = s.m11;
            this.m12 = 0D;
            this.m13 = 0D;
            this.m20 = s.m20;
            this.m21 = s.m21;
            this.m22 = 0D;
            this.m23 = 0D;
            this.m30 = 0D;
            this.m31 = 0D;
            this.m32 = 0D;
            this.m33 = 0D;
        }
        
        public dmat4(dmat3 s) {
            this.m00 = s.m00;
            this.m01 = s.m01;
            this.m02 = s.m02;
            this.m03 = 0D;
            this.m10 = s.m10;
            this.m11 = s.m11;
            this.m12 = s.m12;
            this.m13 = 0D;
            this.m20 = s.m20;
            this.m21 = s.m21;
            this.m22 = s.m22;
            this.m23 = 0D;
            this.m30 = 0D;
            this.m31 = 0D;
            this.m32 = 0D;
            this.m33 = 0D;
        }
        
        public dmat4(dmat3x4 s) {
            this.m00 = s.m00;
            this.m01 = s.m01;
            this.m02 = s.m02;
            this.m03 = s.m03;
            this.m10 = s.m10;
            this.m11 = s.m11;
            this.m12 = s.m12;
            this.m13 = s.m13;
            this.m20 = s.m20;
            this.m21 = s.m21;
            this.m22 = s.m22;
            this.m23 = s.m23;
            this.m30 = 0D;
            this.m31 = 0D;
            this.m32 = 0D;
            this.m33 = 0D;
        }
        
        public dmat4(dmat4x2 s) {
            this.m00 = s.m00;
            this.m01 = s.m01;
            this.m02 = 0D;
            this.m03 = 0D;
            this.m10 = s.m10;
            this.m11 = s.m11;
            this.m12 = 0D;
            this.m13 = 0D;
            this.m20 = s.m20;
            this.m21 = s.m21;
            this.m22 = 0D;
            this.m23 = 0D;
            this.m30 = s.m30;
            this.m31 = s.m31;
            this.m32 = 0D;
            this.m33 = 0D;
        }
        
        public dmat4(dmat4x3 s) {
            this.m00 = s.m00;
            this.m01 = s.m01;
            this.m02 = s.m02;
            this.m03 = 0D;
            this.m10 = s.m10;
            this.m11 = s.m11;
            this.m12 = s.m12;
            this.m13 = 0D;
            this.m20 = s.m20;
            this.m21 = s.m21;
            this.m22 = s.m22;
            this.m23 = 0D;
            this.m30 = s.m30;
            this.m31 = s.m31;
            this.m32 = s.m32;
            this.m33 = 0D;
        }
        
        public double this[int i0, int i1] {
            get {
                if (((i0 < 0) 
                            || (3 < i0))) {
                    throw new System.IndexOutOfRangeException("i0");
                }
                if (((i1 < 0) 
                            || (3 < i1))) {
                    throw new System.IndexOutOfRangeException("i1");
                }
                
                unsafe
                {
                    fixed (Double* v_ptr = &this.m00)
                    {
                        return *(v_ptr + (i1 + i0 * 4));
                    }
                };
            }
            set {
                if (((i0 < 0) 
                            || (3 < i0))) {
                    throw new System.IndexOutOfRangeException("i0");
                }
                if (((i1 < 0) 
                            || (3 < i1))) {
                    throw new System.IndexOutOfRangeException("i1");
                }
                
                unsafe
                {
                    fixed (Double* v_ptr = &this.m00)
                    {
                        *(v_ptr  + (i1 + i0 * 4)) = value;
                    }
                };
            }
        }
        
        public dvec4 this[int i0] {
            get {
                return new dvec4(this[i0, 0], this[i0, 1], this[i0, 2], this[i0, 3]);
            }
            set {
                this[i0, 0] = value[0];
                this[i0, 1] = value[1];
                this[i0, 2] = value[2];
                this[i0, 3] = value[3];
            }
        }
        
        public static dmat4 operator+(dmat4 v) {
            return new dmat4(+v.m00, +v.m01, +v.m02, +v.m03, +v.m10, +v.m11, +v.m12, +v.m13, +v.m20, +v.m21, +v.m22, +v.m23, +v.m30, +v.m31, +v.m32, +v.m33);
        }
        
        public static dmat4 operator-(dmat4 v) {
            return new dmat4(-v.m00, -v.m01, -v.m02, -v.m03, -v.m10, -v.m11, -v.m12, -v.m13, -v.m20, -v.m21, -v.m22, -v.m23, -v.m30, -v.m31, -v.m32, -v.m33);
        }
        
        public static dmat4 operator++(dmat4 v) {
            return new dmat4(++v.m00, ++v.m01, ++v.m02, ++v.m03, ++v.m10, ++v.m11, ++v.m12, ++v.m13, ++v.m20, ++v.m21, ++v.m22, ++v.m23, ++v.m30, ++v.m31, ++v.m32, ++v.m33);
        }
        
        public static dmat4 operator--(dmat4 v) {
            return new dmat4(--v.m00, --v.m01, --v.m02, --v.m03, --v.m10, --v.m11, --v.m12, --v.m13, --v.m20, --v.m21, --v.m22, --v.m23, --v.m30, --v.m31, --v.m32, --v.m33);
        }
        
        public static dmat4 operator+(double s, dmat4 v2) {
            return new dmat4(s + v2.m00, s + v2.m01, s + v2.m02, s + v2.m03, s + v2.m10, s + v2.m11, s + v2.m12, s + v2.m13, s + v2.m20, s + v2.m21, s + v2.m22, s + v2.m23, s + v2.m30, s + v2.m31, s + v2.m32, s + v2.m33);
        }
        
        public static dmat4 operator-(double s, dmat4 v2) {
            return new dmat4(s - v2.m00, s - v2.m01, s - v2.m02, s - v2.m03, s - v2.m10, s - v2.m11, s - v2.m12, s - v2.m13, s - v2.m20, s - v2.m21, s - v2.m22, s - v2.m23, s - v2.m30, s - v2.m31, s - v2.m32, s - v2.m33);
        }
        
        public static dmat4 operator*(double s, dmat4 v2) {
            return new dmat4(s * v2.m00, s * v2.m01, s * v2.m02, s * v2.m03, s * v2.m10, s * v2.m11, s * v2.m12, s * v2.m13, s * v2.m20, s * v2.m21, s * v2.m22, s * v2.m23, s * v2.m30, s * v2.m31, s * v2.m32, s * v2.m33);
        }
        
        public static dmat4 operator/(double s, dmat4 v2) {
            return new dmat4(s / v2.m00, s / v2.m01, s / v2.m02, s / v2.m03, s / v2.m10, s / v2.m11, s / v2.m12, s / v2.m13, s / v2.m20, s / v2.m21, s / v2.m22, s / v2.m23, s / v2.m30, s / v2.m31, s / v2.m32, s / v2.m33);
        }
        
        public static dmat4 operator%(double s, dmat4 v2) {
            return new dmat4(s % v2.m00, s % v2.m01, s % v2.m02, s % v2.m03, s % v2.m10, s % v2.m11, s % v2.m12, s % v2.m13, s % v2.m20, s % v2.m21, s % v2.m22, s % v2.m23, s % v2.m30, s % v2.m31, s % v2.m32, s % v2.m33);
        }
        
        public static dmat4 operator+(dmat4 v1, double s) {
            return new dmat4(v1.m00 + s, v1.m01 + s, v1.m02 + s, v1.m03 + s, v1.m10 + s, v1.m11 + s, v1.m12 + s, v1.m13 + s, v1.m20 + s, v1.m21 + s, v1.m22 + s, v1.m23 + s, v1.m30 + s, v1.m31 + s, v1.m32 + s, v1.m33 + s);
        }
        
        public static dmat4 operator-(dmat4 v1, double s) {
            return new dmat4(v1.m00 - s, v1.m01 - s, v1.m02 - s, v1.m03 - s, v1.m10 - s, v1.m11 - s, v1.m12 - s, v1.m13 - s, v1.m20 - s, v1.m21 - s, v1.m22 - s, v1.m23 - s, v1.m30 - s, v1.m31 - s, v1.m32 - s, v1.m33 - s);
        }
        
        public static dmat4 operator*(dmat4 v1, double s) {
            return new dmat4(v1.m00 * s, v1.m01 * s, v1.m02 * s, v1.m03 * s, v1.m10 * s, v1.m11 * s, v1.m12 * s, v1.m13 * s, v1.m20 * s, v1.m21 * s, v1.m22 * s, v1.m23 * s, v1.m30 * s, v1.m31 * s, v1.m32 * s, v1.m33 * s);
        }
        
        public static dmat4 operator/(dmat4 v1, double s) {
            return new dmat4(v1.m00 / s, v1.m01 / s, v1.m02 / s, v1.m03 / s, v1.m10 / s, v1.m11 / s, v1.m12 / s, v1.m13 / s, v1.m20 / s, v1.m21 / s, v1.m22 / s, v1.m23 / s, v1.m30 / s, v1.m31 / s, v1.m32 / s, v1.m33 / s);
        }
        
        public static dmat4 operator%(dmat4 v1, double s) {
            return new dmat4(v1.m00 % s, v1.m01 % s, v1.m02 % s, v1.m03 % s, v1.m10 % s, v1.m11 % s, v1.m12 % s, v1.m13 % s, v1.m20 % s, v1.m21 % s, v1.m22 % s, v1.m23 % s, v1.m30 % s, v1.m31 % s, v1.m32 % s, v1.m33 % s);
        }
        
        public static dmat4 operator+(dmat4 v1, dmat4 v2) {
            return new dmat4(v1.m00 + v2.m00, v1.m01 + v2.m01, v1.m02 + v2.m02, v1.m03 + v2.m03, v1.m10 + v2.m10, v1.m11 + v2.m11, v1.m12 + v2.m12, v1.m13 + v2.m13, v1.m20 + v2.m20, v1.m21 + v2.m21, v1.m22 + v2.m22, v1.m23 + v2.m23, v1.m30 + v2.m30, v1.m31 + v2.m31, v1.m32 + v2.m32, v1.m33 + v2.m33);
        }
        
        public static dmat4 operator-(dmat4 v1, dmat4 v2) {
            return new dmat4(v1.m00 - v2.m00, v1.m01 - v2.m01, v1.m02 - v2.m02, v1.m03 - v2.m03, v1.m10 - v2.m10, v1.m11 - v2.m11, v1.m12 - v2.m12, v1.m13 - v2.m13, v1.m20 - v2.m20, v1.m21 - v2.m21, v1.m22 - v2.m22, v1.m23 - v2.m23, v1.m30 - v2.m30, v1.m31 - v2.m31, v1.m32 - v2.m32, v1.m33 - v2.m33);
        }
        
        public static dmat4 operator/(dmat4 v1, dmat4 v2) {
            return new dmat4(v1.m00 / v2.m00, v1.m01 / v2.m01, v1.m02 / v2.m02, v1.m03 / v2.m03, v1.m10 / v2.m10, v1.m11 / v2.m11, v1.m12 / v2.m12, v1.m13 / v2.m13, v1.m20 / v2.m20, v1.m21 / v2.m21, v1.m22 / v2.m22, v1.m23 / v2.m23, v1.m30 / v2.m30, v1.m31 / v2.m31, v1.m32 / v2.m32, v1.m33 / v2.m33);
        }
        
        public static dmat4 operator%(dmat4 v1, dmat4 v2) {
            return new dmat4(v1.m00 % v2.m00, v1.m01 % v2.m01, v1.m02 % v2.m02, v1.m03 % v2.m03, v1.m10 % v2.m10, v1.m11 % v2.m11, v1.m12 % v2.m12, v1.m13 % v2.m13, v1.m20 % v2.m20, v1.m21 % v2.m21, v1.m22 % v2.m22, v1.m23 % v2.m23, v1.m30 % v2.m30, v1.m31 % v2.m31, v1.m32 % v2.m32, v1.m33 % v2.m33);
        }
        
        public static bmat4 operator==(dmat4 v1, dmat4 v2) {
            return new bmat4(v1.m00 == v2.m00, v1.m01 == v2.m01, v1.m02 == v2.m02, v1.m03 == v2.m03, v1.m10 == v2.m10, v1.m11 == v2.m11, v1.m12 == v2.m12, v1.m13 == v2.m13, v1.m20 == v2.m20, v1.m21 == v2.m21, v1.m22 == v2.m22, v1.m23 == v2.m23, v1.m30 == v2.m30, v1.m31 == v2.m31, v1.m32 == v2.m32, v1.m33 == v2.m33);
        }
        
        public static bmat4 operator>(dmat4 v1, dmat4 v2) {
            return new bmat4(v1.m00 > v2.m00, v1.m01 > v2.m01, v1.m02 > v2.m02, v1.m03 > v2.m03, v1.m10 > v2.m10, v1.m11 > v2.m11, v1.m12 > v2.m12, v1.m13 > v2.m13, v1.m20 > v2.m20, v1.m21 > v2.m21, v1.m22 > v2.m22, v1.m23 > v2.m23, v1.m30 > v2.m30, v1.m31 > v2.m31, v1.m32 > v2.m32, v1.m33 > v2.m33);
        }
        
        public static bmat4 operator>=(dmat4 v1, dmat4 v2) {
            return new bmat4(v1.m00 >= v2.m00, v1.m01 >= v2.m01, v1.m02 >= v2.m02, v1.m03 >= v2.m03, v1.m10 >= v2.m10, v1.m11 >= v2.m11, v1.m12 >= v2.m12, v1.m13 >= v2.m13, v1.m20 >= v2.m20, v1.m21 >= v2.m21, v1.m22 >= v2.m22, v1.m23 >= v2.m23, v1.m30 >= v2.m30, v1.m31 >= v2.m31, v1.m32 >= v2.m32, v1.m33 >= v2.m33);
        }
        
        public static bmat4 operator<(dmat4 v1, dmat4 v2) {
            return new bmat4(v1.m00 < v2.m00, v1.m01 < v2.m01, v1.m02 < v2.m02, v1.m03 < v2.m03, v1.m10 < v2.m10, v1.m11 < v2.m11, v1.m12 < v2.m12, v1.m13 < v2.m13, v1.m20 < v2.m20, v1.m21 < v2.m21, v1.m22 < v2.m22, v1.m23 < v2.m23, v1.m30 < v2.m30, v1.m31 < v2.m31, v1.m32 < v2.m32, v1.m33 < v2.m33);
        }
        
        public static bmat4 operator<=(dmat4 v1, dmat4 v2) {
            return new bmat4(v1.m00 <= v2.m00, v1.m01 <= v2.m01, v1.m02 <= v2.m02, v1.m03 <= v2.m03, v1.m10 <= v2.m10, v1.m11 <= v2.m11, v1.m12 <= v2.m12, v1.m13 <= v2.m13, v1.m20 <= v2.m20, v1.m21 <= v2.m21, v1.m22 <= v2.m22, v1.m23 <= v2.m23, v1.m30 <= v2.m30, v1.m31 <= v2.m31, v1.m32 <= v2.m32, v1.m33 <= v2.m33);
        }
        
        public static bmat4 operator!=(dmat4 v1, dmat4 v2) {
            return new bmat4(v1.m00 != v2.m00, v1.m01 != v2.m01, v1.m02 != v2.m02, v1.m03 != v2.m03, v1.m10 != v2.m10, v1.m11 != v2.m11, v1.m12 != v2.m12, v1.m13 != v2.m13, v1.m20 != v2.m20, v1.m21 != v2.m21, v1.m22 != v2.m22, v1.m23 != v2.m23, v1.m30 != v2.m30, v1.m31 != v2.m31, v1.m32 != v2.m32, v1.m33 != v2.m33);
        }
        
        public static dvec4 operator*(dvec4 v0, dmat4 m1) {
            return dmath.InnerProduct1(ref v0, ref m1);
        }
        
        public static dvec4 operator*(dmat4 m0, dvec4 v1) {
            return dmath.InnerProduct1(ref m0, ref v1);
        }
        
        public static dmat4x2 operator*(dmat4 m0, dmat4x2 m1) {
            return dmath.InnerProduct1(ref m0, ref m1);
        }
        
        public static dmat4x3 operator*(dmat4 m0, dmat4x3 m1) {
            return dmath.InnerProduct1(ref m0, ref m1);
        }
        
        public static dmat4 operator*(dmat4 m0, dmat4 m1) {
            return dmath.InnerProduct1(ref m0, ref m1);
        }
        
        public override bool Equals(object obj) {
            return (obj is dmat4) ? bmath.All(this == (dmat4)obj) : false;
        }
        
        public override int GetHashCode() {
            return this.m00.GetHashCode() ^ this.m01.GetHashCode() ^ this.m02.GetHashCode() ^ this.m03.GetHashCode() ^ this.m10.GetHashCode() ^ this.m11.GetHashCode() ^ this.m12.GetHashCode() ^ this.m13.GetHashCode() ^ this.m20.GetHashCode() ^ this.m21.GetHashCode() ^ this.m22.GetHashCode() ^ this.m23.GetHashCode() ^ this.m30.GetHashCode() ^ this.m31.GetHashCode() ^ this.m32.GetHashCode() ^ this.m33.GetHashCode();
        }
        
        public override string ToString() {
            return "dmat4(" + this.m00 + "," + this.m01 + "," + this.m02 + "," + this.m03 + "," + this.m10 + "," + this.m11 + "," + this.m12 + "," + this.m13 + "," + this.m20 + "," + this.m21 + "," + this.m22 + "," + this.m23 + "," + this.m30 + "," + this.m31 + "," + this.m32 + "," + this.m33 + ")";
        }
    }
}
