#region --- License ---
/*
Copyright 2010 Daniel Filonik

Permission is hereby granted, free of charge, to any person obtaining a copy of
this software and associated documentation files (the "Software"), to deal in
the Software without restriction, including without limitation the rights to
use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
of the Software, and to permit persons to whom the Software is furnished to do
so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all
copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
SOFTWARE.
*/
#endregion

namespace TKM {
    using System;
    using System.Runtime.InteropServices;
    using System.Runtime.Serialization;
    
    
    [DataContract()]
    [StructLayout(LayoutKind.Sequential)]
    public struct dvec2 {
        
        [DataMember()]
        internal double v0;
        
        [DataMember()]
        internal double v1;
        
        public dvec2(double s) {
            this.v0 = s;
            this.v1 = s;
        }
        
        public dvec2(dvec2 v0) {
            this.v0 = v0.v0;
            this.v1 = v0.v1;
        }
        
        public dvec2(double v0, double v1) {
            this.v0 = v0;
            this.v1 = v1;
        }
        
        public dvec2(dvec3 s) {
            this.v0 = s.v0;
            this.v1 = s.v1;
        }
        
        public dvec2(dvec4 s) {
            this.v0 = s.v0;
            this.v1 = s.v1;
        }
        
        public double this[int i0] {
            get {
                if (((i0 < 0) 
                            || (1 < i0))) {
                    throw new System.IndexOutOfRangeException("i0");
                }
                
                unsafe
                {
                    fixed (Double* v_ptr = &this.v0)
                    {
                        return *(v_ptr + (i0));
                    }
                };
            }
            set {
                if (((i0 < 0) 
                            || (1 < i0))) {
                    throw new System.IndexOutOfRangeException("i0");
                }
                
                unsafe
                {
                    fixed (Double* v_ptr = &this.v0)
                    {
                        *(v_ptr  + (i0)) = value;
                    }
                };
            }
        }
        
        public double x {
            get {
                return this.v0;
            }
            set {
                this.v0 = value;
            }
        }
        
        public double r {
            get {
                return this.v0;
            }
            set {
                this.v0 = value;
            }
        }
        
        public double s {
            get {
                return this.v0;
            }
            set {
                this.v0 = value;
            }
        }
        
        public double y {
            get {
                return this.v1;
            }
            set {
                this.v1 = value;
            }
        }
        
        public double g {
            get {
                return this.v1;
            }
            set {
                this.v1 = value;
            }
        }
        
        public double t {
            get {
                return this.v1;
            }
            set {
                this.v1 = value;
            }
        }
        
        public dvec2 xx {
            get {
                return new dvec2(this.v0, this.v0);
            }
        }
        
        public dvec2 rr {
            get {
                return new dvec2(this.v0, this.v0);
            }
        }
        
        public dvec2 ss {
            get {
                return new dvec2(this.v0, this.v0);
            }
        }
        
        public dvec2 xy {
            get {
                return new dvec2(this.v0, this.v1);
            }
            set {
                this.v0 = value.v0;
                this.v1 = value.v1;
            }
        }
        
        public dvec2 rg {
            get {
                return new dvec2(this.v0, this.v1);
            }
            set {
                this.v0 = value.v0;
                this.v1 = value.v1;
            }
        }
        
        public dvec2 st {
            get {
                return new dvec2(this.v0, this.v1);
            }
            set {
                this.v0 = value.v0;
                this.v1 = value.v1;
            }
        }
        
        public dvec2 yx {
            get {
                return new dvec2(this.v1, this.v0);
            }
            set {
                this.v1 = value.v0;
                this.v0 = value.v1;
            }
        }
        
        public dvec2 gr {
            get {
                return new dvec2(this.v1, this.v0);
            }
            set {
                this.v1 = value.v0;
                this.v0 = value.v1;
            }
        }
        
        public dvec2 ts {
            get {
                return new dvec2(this.v1, this.v0);
            }
            set {
                this.v1 = value.v0;
                this.v0 = value.v1;
            }
        }
        
        public dvec2 yy {
            get {
                return new dvec2(this.v1, this.v1);
            }
        }
        
        public dvec2 gg {
            get {
                return new dvec2(this.v1, this.v1);
            }
        }
        
        public dvec2 tt {
            get {
                return new dvec2(this.v1, this.v1);
            }
        }
        
        public static dvec2 operator+(dvec2 v) {
            return new dvec2(+v.v0, +v.v1);
        }
        
        public static dvec2 operator-(dvec2 v) {
            return new dvec2(-v.v0, -v.v1);
        }
        
        public static dvec2 operator++(dvec2 v) {
            return new dvec2(++v.v0, ++v.v1);
        }
        
        public static dvec2 operator--(dvec2 v) {
            return new dvec2(--v.v0, --v.v1);
        }
        
        public static dvec2 operator+(double s, dvec2 v2) {
            return new dvec2(s + v2.v0, s + v2.v1);
        }
        
        public static dvec2 operator-(double s, dvec2 v2) {
            return new dvec2(s - v2.v0, s - v2.v1);
        }
        
        public static dvec2 operator*(double s, dvec2 v2) {
            return new dvec2(s * v2.v0, s * v2.v1);
        }
        
        public static dvec2 operator/(double s, dvec2 v2) {
            return new dvec2(s / v2.v0, s / v2.v1);
        }
        
        public static dvec2 operator%(double s, dvec2 v2) {
            return new dvec2(s % v2.v0, s % v2.v1);
        }
        
        public static dvec2 operator+(dvec2 v1, double s) {
            return new dvec2(v1.v0 + s, v1.v1 + s);
        }
        
        public static dvec2 operator-(dvec2 v1, double s) {
            return new dvec2(v1.v0 - s, v1.v1 - s);
        }
        
        public static dvec2 operator*(dvec2 v1, double s) {
            return new dvec2(v1.v0 * s, v1.v1 * s);
        }
        
        public static dvec2 operator/(dvec2 v1, double s) {
            return new dvec2(v1.v0 / s, v1.v1 / s);
        }
        
        public static dvec2 operator%(dvec2 v1, double s) {
            return new dvec2(v1.v0 % s, v1.v1 % s);
        }
        
        public static dvec2 operator+(dvec2 v1, dvec2 v2) {
            return new dvec2(v1.v0 + v2.v0, v1.v1 + v2.v1);
        }
        
        public static dvec2 operator-(dvec2 v1, dvec2 v2) {
            return new dvec2(v1.v0 - v2.v0, v1.v1 - v2.v1);
        }
        
        public static dvec2 operator*(dvec2 v1, dvec2 v2) {
            return new dvec2(v1.v0 * v2.v0, v1.v1 * v2.v1);
        }
        
        public static dvec2 operator/(dvec2 v1, dvec2 v2) {
            return new dvec2(v1.v0 / v2.v0, v1.v1 / v2.v1);
        }
        
        public static dvec2 operator%(dvec2 v1, dvec2 v2) {
            return new dvec2(v1.v0 % v2.v0, v1.v1 % v2.v1);
        }
        
        public static bvec2 operator==(dvec2 v1, dvec2 v2) {
            return new bvec2(v1.v0 == v2.v0, v1.v1 == v2.v1);
        }
        
        public static bvec2 operator>(dvec2 v1, dvec2 v2) {
            return new bvec2(v1.v0 > v2.v0, v1.v1 > v2.v1);
        }
        
        public static bvec2 operator>=(dvec2 v1, dvec2 v2) {
            return new bvec2(v1.v0 >= v2.v0, v1.v1 >= v2.v1);
        }
        
        public static bvec2 operator<(dvec2 v1, dvec2 v2) {
            return new bvec2(v1.v0 < v2.v0, v1.v1 < v2.v1);
        }
        
        public static bvec2 operator<=(dvec2 v1, dvec2 v2) {
            return new bvec2(v1.v0 <= v2.v0, v1.v1 <= v2.v1);
        }
        
        public static bvec2 operator!=(dvec2 v1, dvec2 v2) {
            return new bvec2(v1.v0 != v2.v0, v1.v1 != v2.v1);
        }
        
        public override bool Equals(object obj) {
            return (obj is dvec2) ? bmath.All(this == (dvec2)obj) : false;
        }
        
        public override int GetHashCode() {
            return this.v0.GetHashCode() ^ this.v1.GetHashCode();
        }
        
        public override string ToString() {
            return "dvec2(" + this.v0 + "," + this.v1 + ")";
        }
    }
    
    [DataContract()]
    [StructLayout(LayoutKind.Sequential)]
    public struct dvec3 {
        
        [DataMember()]
        internal double v0;
        
        [DataMember()]
        internal double v1;
        
        [DataMember()]
        internal double v2;
        
        public dvec3(double s) {
            this.v0 = s;
            this.v1 = s;
            this.v2 = s;
        }
        
        public dvec3(dvec3 v0) {
            this.v0 = v0.v0;
            this.v1 = v0.v1;
            this.v2 = v0.v2;
        }
        
        public dvec3(dvec2 v0, double v1) {
            this.v0 = v0.v0;
            this.v1 = v0.v1;
            this.v2 = v1;
        }
        
        public dvec3(double v0, dvec2 v1) {
            this.v0 = v0;
            this.v1 = v1.v0;
            this.v2 = v1.v1;
        }
        
        public dvec3(double v0, double v1, double v2) {
            this.v0 = v0;
            this.v1 = v1;
            this.v2 = v2;
        }
        
        public dvec3(dvec2 s) {
            this.v0 = s.v0;
            this.v1 = s.v1;
            this.v2 = 0D;
        }
        
        public dvec3(dvec4 s) {
            this.v0 = s.v0;
            this.v1 = s.v1;
            this.v2 = s.v2;
        }
        
        public double this[int i0] {
            get {
                if (((i0 < 0) 
                            || (2 < i0))) {
                    throw new System.IndexOutOfRangeException("i0");
                }
                
                unsafe
                {
                    fixed (Double* v_ptr = &this.v0)
                    {
                        return *(v_ptr + (i0));
                    }
                };
            }
            set {
                if (((i0 < 0) 
                            || (2 < i0))) {
                    throw new System.IndexOutOfRangeException("i0");
                }
                
                unsafe
                {
                    fixed (Double* v_ptr = &this.v0)
                    {
                        *(v_ptr  + (i0)) = value;
                    }
                };
            }
        }
        
        public double x {
            get {
                return this.v0;
            }
            set {
                this.v0 = value;
            }
        }
        
        public double r {
            get {
                return this.v0;
            }
            set {
                this.v0 = value;
            }
        }
        
        public double s {
            get {
                return this.v0;
            }
            set {
                this.v0 = value;
            }
        }
        
        public double y {
            get {
                return this.v1;
            }
            set {
                this.v1 = value;
            }
        }
        
        public double g {
            get {
                return this.v1;
            }
            set {
                this.v1 = value;
            }
        }
        
        public double t {
            get {
                return this.v1;
            }
            set {
                this.v1 = value;
            }
        }
        
        public double z {
            get {
                return this.v2;
            }
            set {
                this.v2 = value;
            }
        }
        
        public double b {
            get {
                return this.v2;
            }
            set {
                this.v2 = value;
            }
        }
        
        public double p {
            get {
                return this.v2;
            }
            set {
                this.v2 = value;
            }
        }
        
        public dvec2 xx {
            get {
                return new dvec2(this.v0, this.v0);
            }
        }
        
        public dvec2 rr {
            get {
                return new dvec2(this.v0, this.v0);
            }
        }
        
        public dvec2 ss {
            get {
                return new dvec2(this.v0, this.v0);
            }
        }
        
        public dvec2 xy {
            get {
                return new dvec2(this.v0, this.v1);
            }
            set {
                this.v0 = value.v0;
                this.v1 = value.v1;
            }
        }
        
        public dvec2 rg {
            get {
                return new dvec2(this.v0, this.v1);
            }
            set {
                this.v0 = value.v0;
                this.v1 = value.v1;
            }
        }
        
        public dvec2 st {
            get {
                return new dvec2(this.v0, this.v1);
            }
            set {
                this.v0 = value.v0;
                this.v1 = value.v1;
            }
        }
        
        public dvec2 xz {
            get {
                return new dvec2(this.v0, this.v2);
            }
            set {
                this.v0 = value.v0;
                this.v2 = value.v1;
            }
        }
        
        public dvec2 rb {
            get {
                return new dvec2(this.v0, this.v2);
            }
            set {
                this.v0 = value.v0;
                this.v2 = value.v1;
            }
        }
        
        public dvec2 sp {
            get {
                return new dvec2(this.v0, this.v2);
            }
            set {
                this.v0 = value.v0;
                this.v2 = value.v1;
            }
        }
        
        public dvec2 yx {
            get {
                return new dvec2(this.v1, this.v0);
            }
            set {
                this.v1 = value.v0;
                this.v0 = value.v1;
            }
        }
        
        public dvec2 gr {
            get {
                return new dvec2(this.v1, this.v0);
            }
            set {
                this.v1 = value.v0;
                this.v0 = value.v1;
            }
        }
        
        public dvec2 ts {
            get {
                return new dvec2(this.v1, this.v0);
            }
            set {
                this.v1 = value.v0;
                this.v0 = value.v1;
            }
        }
        
        public dvec2 yy {
            get {
                return new dvec2(this.v1, this.v1);
            }
        }
        
        public dvec2 gg {
            get {
                return new dvec2(this.v1, this.v1);
            }
        }
        
        public dvec2 tt {
            get {
                return new dvec2(this.v1, this.v1);
            }
        }
        
        public dvec2 yz {
            get {
                return new dvec2(this.v1, this.v2);
            }
            set {
                this.v1 = value.v0;
                this.v2 = value.v1;
            }
        }
        
        public dvec2 gb {
            get {
                return new dvec2(this.v1, this.v2);
            }
            set {
                this.v1 = value.v0;
                this.v2 = value.v1;
            }
        }
        
        public dvec2 tp {
            get {
                return new dvec2(this.v1, this.v2);
            }
            set {
                this.v1 = value.v0;
                this.v2 = value.v1;
            }
        }
        
        public dvec2 zx {
            get {
                return new dvec2(this.v2, this.v0);
            }
            set {
                this.v2 = value.v0;
                this.v0 = value.v1;
            }
        }
        
        public dvec2 br {
            get {
                return new dvec2(this.v2, this.v0);
            }
            set {
                this.v2 = value.v0;
                this.v0 = value.v1;
            }
        }
        
        public dvec2 ps {
            get {
                return new dvec2(this.v2, this.v0);
            }
            set {
                this.v2 = value.v0;
                this.v0 = value.v1;
            }
        }
        
        public dvec2 zy {
            get {
                return new dvec2(this.v2, this.v1);
            }
            set {
                this.v2 = value.v0;
                this.v1 = value.v1;
            }
        }
        
        public dvec2 bg {
            get {
                return new dvec2(this.v2, this.v1);
            }
            set {
                this.v2 = value.v0;
                this.v1 = value.v1;
            }
        }
        
        public dvec2 pt {
            get {
                return new dvec2(this.v2, this.v1);
            }
            set {
                this.v2 = value.v0;
                this.v1 = value.v1;
            }
        }
        
        public dvec2 zz {
            get {
                return new dvec2(this.v2, this.v2);
            }
        }
        
        public dvec2 bb {
            get {
                return new dvec2(this.v2, this.v2);
            }
        }
        
        public dvec2 pp {
            get {
                return new dvec2(this.v2, this.v2);
            }
        }
        
        public dvec3 xxx {
            get {
                return new dvec3(this.v0, this.v0, this.v0);
            }
        }
        
        public dvec3 rrr {
            get {
                return new dvec3(this.v0, this.v0, this.v0);
            }
        }
        
        public dvec3 sss {
            get {
                return new dvec3(this.v0, this.v0, this.v0);
            }
        }
        
        public dvec3 xxy {
            get {
                return new dvec3(this.v0, this.v0, this.v1);
            }
        }
        
        public dvec3 rrg {
            get {
                return new dvec3(this.v0, this.v0, this.v1);
            }
        }
        
        public dvec3 sst {
            get {
                return new dvec3(this.v0, this.v0, this.v1);
            }
        }
        
        public dvec3 xxz {
            get {
                return new dvec3(this.v0, this.v0, this.v2);
            }
        }
        
        public dvec3 rrb {
            get {
                return new dvec3(this.v0, this.v0, this.v2);
            }
        }
        
        public dvec3 ssp {
            get {
                return new dvec3(this.v0, this.v0, this.v2);
            }
        }
        
        public dvec3 xyx {
            get {
                return new dvec3(this.v0, this.v1, this.v0);
            }
        }
        
        public dvec3 rgr {
            get {
                return new dvec3(this.v0, this.v1, this.v0);
            }
        }
        
        public dvec3 sts {
            get {
                return new dvec3(this.v0, this.v1, this.v0);
            }
        }
        
        public dvec3 xyy {
            get {
                return new dvec3(this.v0, this.v1, this.v1);
            }
        }
        
        public dvec3 rgg {
            get {
                return new dvec3(this.v0, this.v1, this.v1);
            }
        }
        
        public dvec3 stt {
            get {
                return new dvec3(this.v0, this.v1, this.v1);
            }
        }
        
        public dvec3 xyz {
            get {
                return new dvec3(this.v0, this.v1, this.v2);
            }
            set {
                this.v0 = value.v0;
                this.v1 = value.v1;
                this.v2 = value.v2;
            }
        }
        
        public dvec3 rgb {
            get {
                return new dvec3(this.v0, this.v1, this.v2);
            }
            set {
                this.v0 = value.v0;
                this.v1 = value.v1;
                this.v2 = value.v2;
            }
        }
        
        public dvec3 stp {
            get {
                return new dvec3(this.v0, this.v1, this.v2);
            }
            set {
                this.v0 = value.v0;
                this.v1 = value.v1;
                this.v2 = value.v2;
            }
        }
        
        public dvec3 xzx {
            get {
                return new dvec3(this.v0, this.v2, this.v0);
            }
        }
        
        public dvec3 rbr {
            get {
                return new dvec3(this.v0, this.v2, this.v0);
            }
        }
        
        public dvec3 sps {
            get {
                return new dvec3(this.v0, this.v2, this.v0);
            }
        }
        
        public dvec3 xzy {
            get {
                return new dvec3(this.v0, this.v2, this.v1);
            }
            set {
                this.v0 = value.v0;
                this.v2 = value.v1;
                this.v1 = value.v2;
            }
        }
        
        public dvec3 rbg {
            get {
                return new dvec3(this.v0, this.v2, this.v1);
            }
            set {
                this.v0 = value.v0;
                this.v2 = value.v1;
                this.v1 = value.v2;
            }
        }
        
        public dvec3 spt {
            get {
                return new dvec3(this.v0, this.v2, this.v1);
            }
            set {
                this.v0 = value.v0;
                this.v2 = value.v1;
                this.v1 = value.v2;
            }
        }
        
        public dvec3 xzz {
            get {
                return new dvec3(this.v0, this.v2, this.v2);
            }
        }
        
        public dvec3 rbb {
            get {
                return new dvec3(this.v0, this.v2, this.v2);
            }
        }
        
        public dvec3 spp {
            get {
                return new dvec3(this.v0, this.v2, this.v2);
            }
        }
        
        public dvec3 yxx {
            get {
                return new dvec3(this.v1, this.v0, this.v0);
            }
        }
        
        public dvec3 grr {
            get {
                return new dvec3(this.v1, this.v0, this.v0);
            }
        }
        
        public dvec3 tss {
            get {
                return new dvec3(this.v1, this.v0, this.v0);
            }
        }
        
        public dvec3 yxy {
            get {
                return new dvec3(this.v1, this.v0, this.v1);
            }
        }
        
        public dvec3 grg {
            get {
                return new dvec3(this.v1, this.v0, this.v1);
            }
        }
        
        public dvec3 tst {
            get {
                return new dvec3(this.v1, this.v0, this.v1);
            }
        }
        
        public dvec3 yxz {
            get {
                return new dvec3(this.v1, this.v0, this.v2);
            }
            set {
                this.v1 = value.v0;
                this.v0 = value.v1;
                this.v2 = value.v2;
            }
        }
        
        public dvec3 grb {
            get {
                return new dvec3(this.v1, this.v0, this.v2);
            }
            set {
                this.v1 = value.v0;
                this.v0 = value.v1;
                this.v2 = value.v2;
            }
        }
        
        public dvec3 tsp {
            get {
                return new dvec3(this.v1, this.v0, this.v2);
            }
            set {
                this.v1 = value.v0;
                this.v0 = value.v1;
                this.v2 = value.v2;
            }
        }
        
        public dvec3 yyx {
            get {
                return new dvec3(this.v1, this.v1, this.v0);
            }
        }
        
        public dvec3 ggr {
            get {
                return new dvec3(this.v1, this.v1, this.v0);
            }
        }
        
        public dvec3 tts {
            get {
                return new dvec3(this.v1, this.v1, this.v0);
            }
        }
        
        public dvec3 yyy {
            get {
                return new dvec3(this.v1, this.v1, this.v1);
            }
        }
        
        public dvec3 ggg {
            get {
                return new dvec3(this.v1, this.v1, this.v1);
            }
        }
        
        public dvec3 ttt {
            get {
                return new dvec3(this.v1, this.v1, this.v1);
            }
        }
        
        public dvec3 yyz {
            get {
                return new dvec3(this.v1, this.v1, this.v2);
            }
        }
        
        public dvec3 ggb {
            get {
                return new dvec3(this.v1, this.v1, this.v2);
            }
        }
        
        public dvec3 ttp {
            get {
                return new dvec3(this.v1, this.v1, this.v2);
            }
        }
        
        public dvec3 yzx {
            get {
                return new dvec3(this.v1, this.v2, this.v0);
            }
            set {
                this.v1 = value.v0;
                this.v2 = value.v1;
                this.v0 = value.v2;
            }
        }
        
        public dvec3 gbr {
            get {
                return new dvec3(this.v1, this.v2, this.v0);
            }
            set {
                this.v1 = value.v0;
                this.v2 = value.v1;
                this.v0 = value.v2;
            }
        }
        
        public dvec3 tps {
            get {
                return new dvec3(this.v1, this.v2, this.v0);
            }
            set {
                this.v1 = value.v0;
                this.v2 = value.v1;
                this.v0 = value.v2;
            }
        }
        
        public dvec3 yzy {
            get {
                return new dvec3(this.v1, this.v2, this.v1);
            }
        }
        
        public dvec3 gbg {
            get {
                return new dvec3(this.v1, this.v2, this.v1);
            }
        }
        
        public dvec3 tpt {
            get {
                return new dvec3(this.v1, this.v2, this.v1);
            }
        }
        
        public dvec3 yzz {
            get {
                return new dvec3(this.v1, this.v2, this.v2);
            }
        }
        
        public dvec3 gbb {
            get {
                return new dvec3(this.v1, this.v2, this.v2);
            }
        }
        
        public dvec3 tpp {
            get {
                return new dvec3(this.v1, this.v2, this.v2);
            }
        }
        
        public dvec3 zxx {
            get {
                return new dvec3(this.v2, this.v0, this.v0);
            }
        }
        
        public dvec3 brr {
            get {
                return new dvec3(this.v2, this.v0, this.v0);
            }
        }
        
        public dvec3 pss {
            get {
                return new dvec3(this.v2, this.v0, this.v0);
            }
        }
        
        public dvec3 zxy {
            get {
                return new dvec3(this.v2, this.v0, this.v1);
            }
            set {
                this.v2 = value.v0;
                this.v0 = value.v1;
                this.v1 = value.v2;
            }
        }
        
        public dvec3 brg {
            get {
                return new dvec3(this.v2, this.v0, this.v1);
            }
            set {
                this.v2 = value.v0;
                this.v0 = value.v1;
                this.v1 = value.v2;
            }
        }
        
        public dvec3 pst {
            get {
                return new dvec3(this.v2, this.v0, this.v1);
            }
            set {
                this.v2 = value.v0;
                this.v0 = value.v1;
                this.v1 = value.v2;
            }
        }
        
        public dvec3 zxz {
            get {
                return new dvec3(this.v2, this.v0, this.v2);
            }
        }
        
        public dvec3 brb {
            get {
                return new dvec3(this.v2, this.v0, this.v2);
            }
        }
        
        public dvec3 psp {
            get {
                return new dvec3(this.v2, this.v0, this.v2);
            }
        }
        
        public dvec3 zyx {
            get {
                return new dvec3(this.v2, this.v1, this.v0);
            }
            set {
                this.v2 = value.v0;
                this.v1 = value.v1;
                this.v0 = value.v2;
            }
        }
        
        public dvec3 bgr {
            get {
                return new dvec3(this.v2, this.v1, this.v0);
            }
            set {
                this.v2 = value.v0;
                this.v1 = value.v1;
                this.v0 = value.v2;
            }
        }
        
        public dvec3 pts {
            get {
                return new dvec3(this.v2, this.v1, this.v0);
            }
            set {
                this.v2 = value.v0;
                this.v1 = value.v1;
                this.v0 = value.v2;
            }
        }
        
        public dvec3 zyy {
            get {
                return new dvec3(this.v2, this.v1, this.v1);
            }
        }
        
        public dvec3 bgg {
            get {
                return new dvec3(this.v2, this.v1, this.v1);
            }
        }
        
        public dvec3 ptt {
            get {
                return new dvec3(this.v2, this.v1, this.v1);
            }
        }
        
        public dvec3 zyz {
            get {
                return new dvec3(this.v2, this.v1, this.v2);
            }
        }
        
        public dvec3 bgb {
            get {
                return new dvec3(this.v2, this.v1, this.v2);
            }
        }
        
        public dvec3 ptp {
            get {
                return new dvec3(this.v2, this.v1, this.v2);
            }
        }
        
        public dvec3 zzx {
            get {
                return new dvec3(this.v2, this.v2, this.v0);
            }
        }
        
        public dvec3 bbr {
            get {
                return new dvec3(this.v2, this.v2, this.v0);
            }
        }
        
        public dvec3 pps {
            get {
                return new dvec3(this.v2, this.v2, this.v0);
            }
        }
        
        public dvec3 zzy {
            get {
                return new dvec3(this.v2, this.v2, this.v1);
            }
        }
        
        public dvec3 bbg {
            get {
                return new dvec3(this.v2, this.v2, this.v1);
            }
        }
        
        public dvec3 ppt {
            get {
                return new dvec3(this.v2, this.v2, this.v1);
            }
        }
        
        public dvec3 zzz {
            get {
                return new dvec3(this.v2, this.v2, this.v2);
            }
        }
        
        public dvec3 bbb {
            get {
                return new dvec3(this.v2, this.v2, this.v2);
            }
        }
        
        public dvec3 ppp {
            get {
                return new dvec3(this.v2, this.v2, this.v2);
            }
        }
        
        public static dvec3 operator+(dvec3 v) {
            return new dvec3(+v.v0, +v.v1, +v.v2);
        }
        
        public static dvec3 operator-(dvec3 v) {
            return new dvec3(-v.v0, -v.v1, -v.v2);
        }
        
        public static dvec3 operator++(dvec3 v) {
            return new dvec3(++v.v0, ++v.v1, ++v.v2);
        }
        
        public static dvec3 operator--(dvec3 v) {
            return new dvec3(--v.v0, --v.v1, --v.v2);
        }
        
        public static dvec3 operator+(double s, dvec3 v2) {
            return new dvec3(s + v2.v0, s + v2.v1, s + v2.v2);
        }
        
        public static dvec3 operator-(double s, dvec3 v2) {
            return new dvec3(s - v2.v0, s - v2.v1, s - v2.v2);
        }
        
        public static dvec3 operator*(double s, dvec3 v2) {
            return new dvec3(s * v2.v0, s * v2.v1, s * v2.v2);
        }
        
        public static dvec3 operator/(double s, dvec3 v2) {
            return new dvec3(s / v2.v0, s / v2.v1, s / v2.v2);
        }
        
        public static dvec3 operator%(double s, dvec3 v2) {
            return new dvec3(s % v2.v0, s % v2.v1, s % v2.v2);
        }
        
        public static dvec3 operator+(dvec3 v1, double s) {
            return new dvec3(v1.v0 + s, v1.v1 + s, v1.v2 + s);
        }
        
        public static dvec3 operator-(dvec3 v1, double s) {
            return new dvec3(v1.v0 - s, v1.v1 - s, v1.v2 - s);
        }
        
        public static dvec3 operator*(dvec3 v1, double s) {
            return new dvec3(v1.v0 * s, v1.v1 * s, v1.v2 * s);
        }
        
        public static dvec3 operator/(dvec3 v1, double s) {
            return new dvec3(v1.v0 / s, v1.v1 / s, v1.v2 / s);
        }
        
        public static dvec3 operator%(dvec3 v1, double s) {
            return new dvec3(v1.v0 % s, v1.v1 % s, v1.v2 % s);
        }
        
        public static dvec3 operator+(dvec3 v1, dvec3 v2) {
            return new dvec3(v1.v0 + v2.v0, v1.v1 + v2.v1, v1.v2 + v2.v2);
        }
        
        public static dvec3 operator-(dvec3 v1, dvec3 v2) {
            return new dvec3(v1.v0 - v2.v0, v1.v1 - v2.v1, v1.v2 - v2.v2);
        }
        
        public static dvec3 operator*(dvec3 v1, dvec3 v2) {
            return new dvec3(v1.v0 * v2.v0, v1.v1 * v2.v1, v1.v2 * v2.v2);
        }
        
        public static dvec3 operator/(dvec3 v1, dvec3 v2) {
            return new dvec3(v1.v0 / v2.v0, v1.v1 / v2.v1, v1.v2 / v2.v2);
        }
        
        public static dvec3 operator%(dvec3 v1, dvec3 v2) {
            return new dvec3(v1.v0 % v2.v0, v1.v1 % v2.v1, v1.v2 % v2.v2);
        }
        
        public static bvec3 operator==(dvec3 v1, dvec3 v2) {
            return new bvec3(v1.v0 == v2.v0, v1.v1 == v2.v1, v1.v2 == v2.v2);
        }
        
        public static bvec3 operator>(dvec3 v1, dvec3 v2) {
            return new bvec3(v1.v0 > v2.v0, v1.v1 > v2.v1, v1.v2 > v2.v2);
        }
        
        public static bvec3 operator>=(dvec3 v1, dvec3 v2) {
            return new bvec3(v1.v0 >= v2.v0, v1.v1 >= v2.v1, v1.v2 >= v2.v2);
        }
        
        public static bvec3 operator<(dvec3 v1, dvec3 v2) {
            return new bvec3(v1.v0 < v2.v0, v1.v1 < v2.v1, v1.v2 < v2.v2);
        }
        
        public static bvec3 operator<=(dvec3 v1, dvec3 v2) {
            return new bvec3(v1.v0 <= v2.v0, v1.v1 <= v2.v1, v1.v2 <= v2.v2);
        }
        
        public static bvec3 operator!=(dvec3 v1, dvec3 v2) {
            return new bvec3(v1.v0 != v2.v0, v1.v1 != v2.v1, v1.v2 != v2.v2);
        }
        
        public override bool Equals(object obj) {
            return (obj is dvec3) ? bmath.All(this == (dvec3)obj) : false;
        }
        
        public override int GetHashCode() {
            return this.v0.GetHashCode() ^ this.v1.GetHashCode() ^ this.v2.GetHashCode();
        }
        
        public override string ToString() {
            return "dvec3(" + this.v0 + "," + this.v1 + "," + this.v2 + ")";
        }
    }
    
    [DataContract()]
    [StructLayout(LayoutKind.Sequential)]
    public struct dvec4 {
        
        [DataMember()]
        internal double v0;
        
        [DataMember()]
        internal double v1;
        
        [DataMember()]
        internal double v2;
        
        [DataMember()]
        internal double v3;
        
        public dvec4(double s) {
            this.v0 = s;
            this.v1 = s;
            this.v2 = s;
            this.v3 = s;
        }
        
        public dvec4(dvec4 v0) {
            this.v0 = v0.v0;
            this.v1 = v0.v1;
            this.v2 = v0.v2;
            this.v3 = v0.v3;
        }
        
        public dvec4(dvec2 v0, dvec2 v1) {
            this.v0 = v0.v0;
            this.v1 = v0.v1;
            this.v2 = v1.v0;
            this.v3 = v1.v1;
        }
        
        public dvec4(dvec3 v0, double v1) {
            this.v0 = v0.v0;
            this.v1 = v0.v1;
            this.v2 = v0.v2;
            this.v3 = v1;
        }
        
        public dvec4(double v0, dvec3 v1) {
            this.v0 = v0;
            this.v1 = v1.v0;
            this.v2 = v1.v1;
            this.v3 = v1.v2;
        }
        
        public dvec4(dvec2 v0, double v1, double v2) {
            this.v0 = v0.v0;
            this.v1 = v0.v1;
            this.v2 = v1;
            this.v3 = v2;
        }
        
        public dvec4(double v0, dvec2 v1, double v2) {
            this.v0 = v0;
            this.v1 = v1.v0;
            this.v2 = v1.v1;
            this.v3 = v2;
        }
        
        public dvec4(double v0, double v1, dvec2 v2) {
            this.v0 = v0;
            this.v1 = v1;
            this.v2 = v2.v0;
            this.v3 = v2.v1;
        }
        
        public dvec4(double v0, double v1, double v2, double v3) {
            this.v0 = v0;
            this.v1 = v1;
            this.v2 = v2;
            this.v3 = v3;
        }
        
        public dvec4(dvec2 s) {
            this.v0 = s.v0;
            this.v1 = s.v1;
            this.v2 = 0D;
            this.v3 = 0D;
        }
        
        public dvec4(dvec3 s) {
            this.v0 = s.v0;
            this.v1 = s.v1;
            this.v2 = s.v2;
            this.v3 = 0D;
        }
        
        public double this[int i0] {
            get {
                if (((i0 < 0) 
                            || (3 < i0))) {
                    throw new System.IndexOutOfRangeException("i0");
                }
                
                unsafe
                {
                    fixed (Double* v_ptr = &this.v0)
                    {
                        return *(v_ptr + (i0));
                    }
                };
            }
            set {
                if (((i0 < 0) 
                            || (3 < i0))) {
                    throw new System.IndexOutOfRangeException("i0");
                }
                
                unsafe
                {
                    fixed (Double* v_ptr = &this.v0)
                    {
                        *(v_ptr  + (i0)) = value;
                    }
                };
            }
        }
        
        public double x {
            get {
                return this.v0;
            }
            set {
                this.v0 = value;
            }
        }
        
        public double r {
            get {
                return this.v0;
            }
            set {
                this.v0 = value;
            }
        }
        
        public double s {
            get {
                return this.v0;
            }
            set {
                this.v0 = value;
            }
        }
        
        public double y {
            get {
                return this.v1;
            }
            set {
                this.v1 = value;
            }
        }
        
        public double g {
            get {
                return this.v1;
            }
            set {
                this.v1 = value;
            }
        }
        
        public double t {
            get {
                return this.v1;
            }
            set {
                this.v1 = value;
            }
        }
        
        public double z {
            get {
                return this.v2;
            }
            set {
                this.v2 = value;
            }
        }
        
        public double b {
            get {
                return this.v2;
            }
            set {
                this.v2 = value;
            }
        }
        
        public double p {
            get {
                return this.v2;
            }
            set {
                this.v2 = value;
            }
        }
        
        public double w {
            get {
                return this.v3;
            }
            set {
                this.v3 = value;
            }
        }
        
        public double a {
            get {
                return this.v3;
            }
            set {
                this.v3 = value;
            }
        }
        
        public double q {
            get {
                return this.v3;
            }
            set {
                this.v3 = value;
            }
        }
        
        public dvec2 xx {
            get {
                return new dvec2(this.v0, this.v0);
            }
        }
        
        public dvec2 rr {
            get {
                return new dvec2(this.v0, this.v0);
            }
        }
        
        public dvec2 ss {
            get {
                return new dvec2(this.v0, this.v0);
            }
        }
        
        public dvec2 xy {
            get {
                return new dvec2(this.v0, this.v1);
            }
            set {
                this.v0 = value.v0;
                this.v1 = value.v1;
            }
        }
        
        public dvec2 rg {
            get {
                return new dvec2(this.v0, this.v1);
            }
            set {
                this.v0 = value.v0;
                this.v1 = value.v1;
            }
        }
        
        public dvec2 st {
            get {
                return new dvec2(this.v0, this.v1);
            }
            set {
                this.v0 = value.v0;
                this.v1 = value.v1;
            }
        }
        
        public dvec2 xz {
            get {
                return new dvec2(this.v0, this.v2);
            }
            set {
                this.v0 = value.v0;
                this.v2 = value.v1;
            }
        }
        
        public dvec2 rb {
            get {
                return new dvec2(this.v0, this.v2);
            }
            set {
                this.v0 = value.v0;
                this.v2 = value.v1;
            }
        }
        
        public dvec2 sp {
            get {
                return new dvec2(this.v0, this.v2);
            }
            set {
                this.v0 = value.v0;
                this.v2 = value.v1;
            }
        }
        
        public dvec2 xw {
            get {
                return new dvec2(this.v0, this.v3);
            }
            set {
                this.v0 = value.v0;
                this.v3 = value.v1;
            }
        }
        
        public dvec2 ra {
            get {
                return new dvec2(this.v0, this.v3);
            }
            set {
                this.v0 = value.v0;
                this.v3 = value.v1;
            }
        }
        
        public dvec2 sq {
            get {
                return new dvec2(this.v0, this.v3);
            }
            set {
                this.v0 = value.v0;
                this.v3 = value.v1;
            }
        }
        
        public dvec2 yx {
            get {
                return new dvec2(this.v1, this.v0);
            }
            set {
                this.v1 = value.v0;
                this.v0 = value.v1;
            }
        }
        
        public dvec2 gr {
            get {
                return new dvec2(this.v1, this.v0);
            }
            set {
                this.v1 = value.v0;
                this.v0 = value.v1;
            }
        }
        
        public dvec2 ts {
            get {
                return new dvec2(this.v1, this.v0);
            }
            set {
                this.v1 = value.v0;
                this.v0 = value.v1;
            }
        }
        
        public dvec2 yy {
            get {
                return new dvec2(this.v1, this.v1);
            }
        }
        
        public dvec2 gg {
            get {
                return new dvec2(this.v1, this.v1);
            }
        }
        
        public dvec2 tt {
            get {
                return new dvec2(this.v1, this.v1);
            }
        }
        
        public dvec2 yz {
            get {
                return new dvec2(this.v1, this.v2);
            }
            set {
                this.v1 = value.v0;
                this.v2 = value.v1;
            }
        }
        
        public dvec2 gb {
            get {
                return new dvec2(this.v1, this.v2);
            }
            set {
                this.v1 = value.v0;
                this.v2 = value.v1;
            }
        }
        
        public dvec2 tp {
            get {
                return new dvec2(this.v1, this.v2);
            }
            set {
                this.v1 = value.v0;
                this.v2 = value.v1;
            }
        }
        
        public dvec2 yw {
            get {
                return new dvec2(this.v1, this.v3);
            }
            set {
                this.v1 = value.v0;
                this.v3 = value.v1;
            }
        }
        
        public dvec2 ga {
            get {
                return new dvec2(this.v1, this.v3);
            }
            set {
                this.v1 = value.v0;
                this.v3 = value.v1;
            }
        }
        
        public dvec2 tq {
            get {
                return new dvec2(this.v1, this.v3);
            }
            set {
                this.v1 = value.v0;
                this.v3 = value.v1;
            }
        }
        
        public dvec2 zx {
            get {
                return new dvec2(this.v2, this.v0);
            }
            set {
                this.v2 = value.v0;
                this.v0 = value.v1;
            }
        }
        
        public dvec2 br {
            get {
                return new dvec2(this.v2, this.v0);
            }
            set {
                this.v2 = value.v0;
                this.v0 = value.v1;
            }
        }
        
        public dvec2 ps {
            get {
                return new dvec2(this.v2, this.v0);
            }
            set {
                this.v2 = value.v0;
                this.v0 = value.v1;
            }
        }
        
        public dvec2 zy {
            get {
                return new dvec2(this.v2, this.v1);
            }
            set {
                this.v2 = value.v0;
                this.v1 = value.v1;
            }
        }
        
        public dvec2 bg {
            get {
                return new dvec2(this.v2, this.v1);
            }
            set {
                this.v2 = value.v0;
                this.v1 = value.v1;
            }
        }
        
        public dvec2 pt {
            get {
                return new dvec2(this.v2, this.v1);
            }
            set {
                this.v2 = value.v0;
                this.v1 = value.v1;
            }
        }
        
        public dvec2 zz {
            get {
                return new dvec2(this.v2, this.v2);
            }
        }
        
        public dvec2 bb {
            get {
                return new dvec2(this.v2, this.v2);
            }
        }
        
        public dvec2 pp {
            get {
                return new dvec2(this.v2, this.v2);
            }
        }
        
        public dvec2 zw {
            get {
                return new dvec2(this.v2, this.v3);
            }
            set {
                this.v2 = value.v0;
                this.v3 = value.v1;
            }
        }
        
        public dvec2 ba {
            get {
                return new dvec2(this.v2, this.v3);
            }
            set {
                this.v2 = value.v0;
                this.v3 = value.v1;
            }
        }
        
        public dvec2 pq {
            get {
                return new dvec2(this.v2, this.v3);
            }
            set {
                this.v2 = value.v0;
                this.v3 = value.v1;
            }
        }
        
        public dvec2 wx {
            get {
                return new dvec2(this.v3, this.v0);
            }
            set {
                this.v3 = value.v0;
                this.v0 = value.v1;
            }
        }
        
        public dvec2 ar {
            get {
                return new dvec2(this.v3, this.v0);
            }
            set {
                this.v3 = value.v0;
                this.v0 = value.v1;
            }
        }
        
        public dvec2 qs {
            get {
                return new dvec2(this.v3, this.v0);
            }
            set {
                this.v3 = value.v0;
                this.v0 = value.v1;
            }
        }
        
        public dvec2 wy {
            get {
                return new dvec2(this.v3, this.v1);
            }
            set {
                this.v3 = value.v0;
                this.v1 = value.v1;
            }
        }
        
        public dvec2 ag {
            get {
                return new dvec2(this.v3, this.v1);
            }
            set {
                this.v3 = value.v0;
                this.v1 = value.v1;
            }
        }
        
        public dvec2 qt {
            get {
                return new dvec2(this.v3, this.v1);
            }
            set {
                this.v3 = value.v0;
                this.v1 = value.v1;
            }
        }
        
        public dvec2 wz {
            get {
                return new dvec2(this.v3, this.v2);
            }
            set {
                this.v3 = value.v0;
                this.v2 = value.v1;
            }
        }
        
        public dvec2 ab {
            get {
                return new dvec2(this.v3, this.v2);
            }
            set {
                this.v3 = value.v0;
                this.v2 = value.v1;
            }
        }
        
        public dvec2 qp {
            get {
                return new dvec2(this.v3, this.v2);
            }
            set {
                this.v3 = value.v0;
                this.v2 = value.v1;
            }
        }
        
        public dvec2 ww {
            get {
                return new dvec2(this.v3, this.v3);
            }
        }
        
        public dvec2 aa {
            get {
                return new dvec2(this.v3, this.v3);
            }
        }
        
        public dvec2 qq {
            get {
                return new dvec2(this.v3, this.v3);
            }
        }
        
        public dvec3 xxx {
            get {
                return new dvec3(this.v0, this.v0, this.v0);
            }
        }
        
        public dvec3 rrr {
            get {
                return new dvec3(this.v0, this.v0, this.v0);
            }
        }
        
        public dvec3 sss {
            get {
                return new dvec3(this.v0, this.v0, this.v0);
            }
        }
        
        public dvec3 xxy {
            get {
                return new dvec3(this.v0, this.v0, this.v1);
            }
        }
        
        public dvec3 rrg {
            get {
                return new dvec3(this.v0, this.v0, this.v1);
            }
        }
        
        public dvec3 sst {
            get {
                return new dvec3(this.v0, this.v0, this.v1);
            }
        }
        
        public dvec3 xxz {
            get {
                return new dvec3(this.v0, this.v0, this.v2);
            }
        }
        
        public dvec3 rrb {
            get {
                return new dvec3(this.v0, this.v0, this.v2);
            }
        }
        
        public dvec3 ssp {
            get {
                return new dvec3(this.v0, this.v0, this.v2);
            }
        }
        
        public dvec3 xxw {
            get {
                return new dvec3(this.v0, this.v0, this.v3);
            }
        }
        
        public dvec3 rra {
            get {
                return new dvec3(this.v0, this.v0, this.v3);
            }
        }
        
        public dvec3 ssq {
            get {
                return new dvec3(this.v0, this.v0, this.v3);
            }
        }
        
        public dvec3 xyx {
            get {
                return new dvec3(this.v0, this.v1, this.v0);
            }
        }
        
        public dvec3 rgr {
            get {
                return new dvec3(this.v0, this.v1, this.v0);
            }
        }
        
        public dvec3 sts {
            get {
                return new dvec3(this.v0, this.v1, this.v0);
            }
        }
        
        public dvec3 xyy {
            get {
                return new dvec3(this.v0, this.v1, this.v1);
            }
        }
        
        public dvec3 rgg {
            get {
                return new dvec3(this.v0, this.v1, this.v1);
            }
        }
        
        public dvec3 stt {
            get {
                return new dvec3(this.v0, this.v1, this.v1);
            }
        }
        
        public dvec3 xyz {
            get {
                return new dvec3(this.v0, this.v1, this.v2);
            }
            set {
                this.v0 = value.v0;
                this.v1 = value.v1;
                this.v2 = value.v2;
            }
        }
        
        public dvec3 rgb {
            get {
                return new dvec3(this.v0, this.v1, this.v2);
            }
            set {
                this.v0 = value.v0;
                this.v1 = value.v1;
                this.v2 = value.v2;
            }
        }
        
        public dvec3 stp {
            get {
                return new dvec3(this.v0, this.v1, this.v2);
            }
            set {
                this.v0 = value.v0;
                this.v1 = value.v1;
                this.v2 = value.v2;
            }
        }
        
        public dvec3 xyw {
            get {
                return new dvec3(this.v0, this.v1, this.v3);
            }
            set {
                this.v0 = value.v0;
                this.v1 = value.v1;
                this.v3 = value.v2;
            }
        }
        
        public dvec3 rga {
            get {
                return new dvec3(this.v0, this.v1, this.v3);
            }
            set {
                this.v0 = value.v0;
                this.v1 = value.v1;
                this.v3 = value.v2;
            }
        }
        
        public dvec3 stq {
            get {
                return new dvec3(this.v0, this.v1, this.v3);
            }
            set {
                this.v0 = value.v0;
                this.v1 = value.v1;
                this.v3 = value.v2;
            }
        }
        
        public dvec3 xzx {
            get {
                return new dvec3(this.v0, this.v2, this.v0);
            }
        }
        
        public dvec3 rbr {
            get {
                return new dvec3(this.v0, this.v2, this.v0);
            }
        }
        
        public dvec3 sps {
            get {
                return new dvec3(this.v0, this.v2, this.v0);
            }
        }
        
        public dvec3 xzy {
            get {
                return new dvec3(this.v0, this.v2, this.v1);
            }
            set {
                this.v0 = value.v0;
                this.v2 = value.v1;
                this.v1 = value.v2;
            }
        }
        
        public dvec3 rbg {
            get {
                return new dvec3(this.v0, this.v2, this.v1);
            }
            set {
                this.v0 = value.v0;
                this.v2 = value.v1;
                this.v1 = value.v2;
            }
        }
        
        public dvec3 spt {
            get {
                return new dvec3(this.v0, this.v2, this.v1);
            }
            set {
                this.v0 = value.v0;
                this.v2 = value.v1;
                this.v1 = value.v2;
            }
        }
        
        public dvec3 xzz {
            get {
                return new dvec3(this.v0, this.v2, this.v2);
            }
        }
        
        public dvec3 rbb {
            get {
                return new dvec3(this.v0, this.v2, this.v2);
            }
        }
        
        public dvec3 spp {
            get {
                return new dvec3(this.v0, this.v2, this.v2);
            }
        }
        
        public dvec3 xzw {
            get {
                return new dvec3(this.v0, this.v2, this.v3);
            }
            set {
                this.v0 = value.v0;
                this.v2 = value.v1;
                this.v3 = value.v2;
            }
        }
        
        public dvec3 rba {
            get {
                return new dvec3(this.v0, this.v2, this.v3);
            }
            set {
                this.v0 = value.v0;
                this.v2 = value.v1;
                this.v3 = value.v2;
            }
        }
        
        public dvec3 spq {
            get {
                return new dvec3(this.v0, this.v2, this.v3);
            }
            set {
                this.v0 = value.v0;
                this.v2 = value.v1;
                this.v3 = value.v2;
            }
        }
        
        public dvec3 xwx {
            get {
                return new dvec3(this.v0, this.v3, this.v0);
            }
        }
        
        public dvec3 rar {
            get {
                return new dvec3(this.v0, this.v3, this.v0);
            }
        }
        
        public dvec3 sqs {
            get {
                return new dvec3(this.v0, this.v3, this.v0);
            }
        }
        
        public dvec3 xwy {
            get {
                return new dvec3(this.v0, this.v3, this.v1);
            }
            set {
                this.v0 = value.v0;
                this.v3 = value.v1;
                this.v1 = value.v2;
            }
        }
        
        public dvec3 rag {
            get {
                return new dvec3(this.v0, this.v3, this.v1);
            }
            set {
                this.v0 = value.v0;
                this.v3 = value.v1;
                this.v1 = value.v2;
            }
        }
        
        public dvec3 sqt {
            get {
                return new dvec3(this.v0, this.v3, this.v1);
            }
            set {
                this.v0 = value.v0;
                this.v3 = value.v1;
                this.v1 = value.v2;
            }
        }
        
        public dvec3 xwz {
            get {
                return new dvec3(this.v0, this.v3, this.v2);
            }
            set {
                this.v0 = value.v0;
                this.v3 = value.v1;
                this.v2 = value.v2;
            }
        }
        
        public dvec3 rab {
            get {
                return new dvec3(this.v0, this.v3, this.v2);
            }
            set {
                this.v0 = value.v0;
                this.v3 = value.v1;
                this.v2 = value.v2;
            }
        }
        
        public dvec3 sqp {
            get {
                return new dvec3(this.v0, this.v3, this.v2);
            }
            set {
                this.v0 = value.v0;
                this.v3 = value.v1;
                this.v2 = value.v2;
            }
        }
        
        public dvec3 xww {
            get {
                return new dvec3(this.v0, this.v3, this.v3);
            }
        }
        
        public dvec3 raa {
            get {
                return new dvec3(this.v0, this.v3, this.v3);
            }
        }
        
        public dvec3 sqq {
            get {
                return new dvec3(this.v0, this.v3, this.v3);
            }
        }
        
        public dvec3 yxx {
            get {
                return new dvec3(this.v1, this.v0, this.v0);
            }
        }
        
        public dvec3 grr {
            get {
                return new dvec3(this.v1, this.v0, this.v0);
            }
        }
        
        public dvec3 tss {
            get {
                return new dvec3(this.v1, this.v0, this.v0);
            }
        }
        
        public dvec3 yxy {
            get {
                return new dvec3(this.v1, this.v0, this.v1);
            }
        }
        
        public dvec3 grg {
            get {
                return new dvec3(this.v1, this.v0, this.v1);
            }
        }
        
        public dvec3 tst {
            get {
                return new dvec3(this.v1, this.v0, this.v1);
            }
        }
        
        public dvec3 yxz {
            get {
                return new dvec3(this.v1, this.v0, this.v2);
            }
            set {
                this.v1 = value.v0;
                this.v0 = value.v1;
                this.v2 = value.v2;
            }
        }
        
        public dvec3 grb {
            get {
                return new dvec3(this.v1, this.v0, this.v2);
            }
            set {
                this.v1 = value.v0;
                this.v0 = value.v1;
                this.v2 = value.v2;
            }
        }
        
        public dvec3 tsp {
            get {
                return new dvec3(this.v1, this.v0, this.v2);
            }
            set {
                this.v1 = value.v0;
                this.v0 = value.v1;
                this.v2 = value.v2;
            }
        }
        
        public dvec3 yxw {
            get {
                return new dvec3(this.v1, this.v0, this.v3);
            }
            set {
                this.v1 = value.v0;
                this.v0 = value.v1;
                this.v3 = value.v2;
            }
        }
        
        public dvec3 gra {
            get {
                return new dvec3(this.v1, this.v0, this.v3);
            }
            set {
                this.v1 = value.v0;
                this.v0 = value.v1;
                this.v3 = value.v2;
            }
        }
        
        public dvec3 tsq {
            get {
                return new dvec3(this.v1, this.v0, this.v3);
            }
            set {
                this.v1 = value.v0;
                this.v0 = value.v1;
                this.v3 = value.v2;
            }
        }
        
        public dvec3 yyx {
            get {
                return new dvec3(this.v1, this.v1, this.v0);
            }
        }
        
        public dvec3 ggr {
            get {
                return new dvec3(this.v1, this.v1, this.v0);
            }
        }
        
        public dvec3 tts {
            get {
                return new dvec3(this.v1, this.v1, this.v0);
            }
        }
        
        public dvec3 yyy {
            get {
                return new dvec3(this.v1, this.v1, this.v1);
            }
        }
        
        public dvec3 ggg {
            get {
                return new dvec3(this.v1, this.v1, this.v1);
            }
        }
        
        public dvec3 ttt {
            get {
                return new dvec3(this.v1, this.v1, this.v1);
            }
        }
        
        public dvec3 yyz {
            get {
                return new dvec3(this.v1, this.v1, this.v2);
            }
        }
        
        public dvec3 ggb {
            get {
                return new dvec3(this.v1, this.v1, this.v2);
            }
        }
        
        public dvec3 ttp {
            get {
                return new dvec3(this.v1, this.v1, this.v2);
            }
        }
        
        public dvec3 yyw {
            get {
                return new dvec3(this.v1, this.v1, this.v3);
            }
        }
        
        public dvec3 gga {
            get {
                return new dvec3(this.v1, this.v1, this.v3);
            }
        }
        
        public dvec3 ttq {
            get {
                return new dvec3(this.v1, this.v1, this.v3);
            }
        }
        
        public dvec3 yzx {
            get {
                return new dvec3(this.v1, this.v2, this.v0);
            }
            set {
                this.v1 = value.v0;
                this.v2 = value.v1;
                this.v0 = value.v2;
            }
        }
        
        public dvec3 gbr {
            get {
                return new dvec3(this.v1, this.v2, this.v0);
            }
            set {
                this.v1 = value.v0;
                this.v2 = value.v1;
                this.v0 = value.v2;
            }
        }
        
        public dvec3 tps {
            get {
                return new dvec3(this.v1, this.v2, this.v0);
            }
            set {
                this.v1 = value.v0;
                this.v2 = value.v1;
                this.v0 = value.v2;
            }
        }
        
        public dvec3 yzy {
            get {
                return new dvec3(this.v1, this.v2, this.v1);
            }
        }
        
        public dvec3 gbg {
            get {
                return new dvec3(this.v1, this.v2, this.v1);
            }
        }
        
        public dvec3 tpt {
            get {
                return new dvec3(this.v1, this.v2, this.v1);
            }
        }
        
        public dvec3 yzz {
            get {
                return new dvec3(this.v1, this.v2, this.v2);
            }
        }
        
        public dvec3 gbb {
            get {
                return new dvec3(this.v1, this.v2, this.v2);
            }
        }
        
        public dvec3 tpp {
            get {
                return new dvec3(this.v1, this.v2, this.v2);
            }
        }
        
        public dvec3 yzw {
            get {
                return new dvec3(this.v1, this.v2, this.v3);
            }
            set {
                this.v1 = value.v0;
                this.v2 = value.v1;
                this.v3 = value.v2;
            }
        }
        
        public dvec3 gba {
            get {
                return new dvec3(this.v1, this.v2, this.v3);
            }
            set {
                this.v1 = value.v0;
                this.v2 = value.v1;
                this.v3 = value.v2;
            }
        }
        
        public dvec3 tpq {
            get {
                return new dvec3(this.v1, this.v2, this.v3);
            }
            set {
                this.v1 = value.v0;
                this.v2 = value.v1;
                this.v3 = value.v2;
            }
        }
        
        public dvec3 ywx {
            get {
                return new dvec3(this.v1, this.v3, this.v0);
            }
            set {
                this.v1 = value.v0;
                this.v3 = value.v1;
                this.v0 = value.v2;
            }
        }
        
        public dvec3 gar {
            get {
                return new dvec3(this.v1, this.v3, this.v0);
            }
            set {
                this.v1 = value.v0;
                this.v3 = value.v1;
                this.v0 = value.v2;
            }
        }
        
        public dvec3 tqs {
            get {
                return new dvec3(this.v1, this.v3, this.v0);
            }
            set {
                this.v1 = value.v0;
                this.v3 = value.v1;
                this.v0 = value.v2;
            }
        }
        
        public dvec3 ywy {
            get {
                return new dvec3(this.v1, this.v3, this.v1);
            }
        }
        
        public dvec3 gag {
            get {
                return new dvec3(this.v1, this.v3, this.v1);
            }
        }
        
        public dvec3 tqt {
            get {
                return new dvec3(this.v1, this.v3, this.v1);
            }
        }
        
        public dvec3 ywz {
            get {
                return new dvec3(this.v1, this.v3, this.v2);
            }
            set {
                this.v1 = value.v0;
                this.v3 = value.v1;
                this.v2 = value.v2;
            }
        }
        
        public dvec3 gab {
            get {
                return new dvec3(this.v1, this.v3, this.v2);
            }
            set {
                this.v1 = value.v0;
                this.v3 = value.v1;
                this.v2 = value.v2;
            }
        }
        
        public dvec3 tqp {
            get {
                return new dvec3(this.v1, this.v3, this.v2);
            }
            set {
                this.v1 = value.v0;
                this.v3 = value.v1;
                this.v2 = value.v2;
            }
        }
        
        public dvec3 yww {
            get {
                return new dvec3(this.v1, this.v3, this.v3);
            }
        }
        
        public dvec3 gaa {
            get {
                return new dvec3(this.v1, this.v3, this.v3);
            }
        }
        
        public dvec3 tqq {
            get {
                return new dvec3(this.v1, this.v3, this.v3);
            }
        }
        
        public dvec3 zxx {
            get {
                return new dvec3(this.v2, this.v0, this.v0);
            }
        }
        
        public dvec3 brr {
            get {
                return new dvec3(this.v2, this.v0, this.v0);
            }
        }
        
        public dvec3 pss {
            get {
                return new dvec3(this.v2, this.v0, this.v0);
            }
        }
        
        public dvec3 zxy {
            get {
                return new dvec3(this.v2, this.v0, this.v1);
            }
            set {
                this.v2 = value.v0;
                this.v0 = value.v1;
                this.v1 = value.v2;
            }
        }
        
        public dvec3 brg {
            get {
                return new dvec3(this.v2, this.v0, this.v1);
            }
            set {
                this.v2 = value.v0;
                this.v0 = value.v1;
                this.v1 = value.v2;
            }
        }
        
        public dvec3 pst {
            get {
                return new dvec3(this.v2, this.v0, this.v1);
            }
            set {
                this.v2 = value.v0;
                this.v0 = value.v1;
                this.v1 = value.v2;
            }
        }
        
        public dvec3 zxz {
            get {
                return new dvec3(this.v2, this.v0, this.v2);
            }
        }
        
        public dvec3 brb {
            get {
                return new dvec3(this.v2, this.v0, this.v2);
            }
        }
        
        public dvec3 psp {
            get {
                return new dvec3(this.v2, this.v0, this.v2);
            }
        }
        
        public dvec3 zxw {
            get {
                return new dvec3(this.v2, this.v0, this.v3);
            }
            set {
                this.v2 = value.v0;
                this.v0 = value.v1;
                this.v3 = value.v2;
            }
        }
        
        public dvec3 bra {
            get {
                return new dvec3(this.v2, this.v0, this.v3);
            }
            set {
                this.v2 = value.v0;
                this.v0 = value.v1;
                this.v3 = value.v2;
            }
        }
        
        public dvec3 psq {
            get {
                return new dvec3(this.v2, this.v0, this.v3);
            }
            set {
                this.v2 = value.v0;
                this.v0 = value.v1;
                this.v3 = value.v2;
            }
        }
        
        public dvec3 zyx {
            get {
                return new dvec3(this.v2, this.v1, this.v0);
            }
            set {
                this.v2 = value.v0;
                this.v1 = value.v1;
                this.v0 = value.v2;
            }
        }
        
        public dvec3 bgr {
            get {
                return new dvec3(this.v2, this.v1, this.v0);
            }
            set {
                this.v2 = value.v0;
                this.v1 = value.v1;
                this.v0 = value.v2;
            }
        }
        
        public dvec3 pts {
            get {
                return new dvec3(this.v2, this.v1, this.v0);
            }
            set {
                this.v2 = value.v0;
                this.v1 = value.v1;
                this.v0 = value.v2;
            }
        }
        
        public dvec3 zyy {
            get {
                return new dvec3(this.v2, this.v1, this.v1);
            }
        }
        
        public dvec3 bgg {
            get {
                return new dvec3(this.v2, this.v1, this.v1);
            }
        }
        
        public dvec3 ptt {
            get {
                return new dvec3(this.v2, this.v1, this.v1);
            }
        }
        
        public dvec3 zyz {
            get {
                return new dvec3(this.v2, this.v1, this.v2);
            }
        }
        
        public dvec3 bgb {
            get {
                return new dvec3(this.v2, this.v1, this.v2);
            }
        }
        
        public dvec3 ptp {
            get {
                return new dvec3(this.v2, this.v1, this.v2);
            }
        }
        
        public dvec3 zyw {
            get {
                return new dvec3(this.v2, this.v1, this.v3);
            }
            set {
                this.v2 = value.v0;
                this.v1 = value.v1;
                this.v3 = value.v2;
            }
        }
        
        public dvec3 bga {
            get {
                return new dvec3(this.v2, this.v1, this.v3);
            }
            set {
                this.v2 = value.v0;
                this.v1 = value.v1;
                this.v3 = value.v2;
            }
        }
        
        public dvec3 ptq {
            get {
                return new dvec3(this.v2, this.v1, this.v3);
            }
            set {
                this.v2 = value.v0;
                this.v1 = value.v1;
                this.v3 = value.v2;
            }
        }
        
        public dvec3 zzx {
            get {
                return new dvec3(this.v2, this.v2, this.v0);
            }
        }
        
        public dvec3 bbr {
            get {
                return new dvec3(this.v2, this.v2, this.v0);
            }
        }
        
        public dvec3 pps {
            get {
                return new dvec3(this.v2, this.v2, this.v0);
            }
        }
        
        public dvec3 zzy {
            get {
                return new dvec3(this.v2, this.v2, this.v1);
            }
        }
        
        public dvec3 bbg {
            get {
                return new dvec3(this.v2, this.v2, this.v1);
            }
        }
        
        public dvec3 ppt {
            get {
                return new dvec3(this.v2, this.v2, this.v1);
            }
        }
        
        public dvec3 zzz {
            get {
                return new dvec3(this.v2, this.v2, this.v2);
            }
        }
        
        public dvec3 bbb {
            get {
                return new dvec3(this.v2, this.v2, this.v2);
            }
        }
        
        public dvec3 ppp {
            get {
                return new dvec3(this.v2, this.v2, this.v2);
            }
        }
        
        public dvec3 zzw {
            get {
                return new dvec3(this.v2, this.v2, this.v3);
            }
        }
        
        public dvec3 bba {
            get {
                return new dvec3(this.v2, this.v2, this.v3);
            }
        }
        
        public dvec3 ppq {
            get {
                return new dvec3(this.v2, this.v2, this.v3);
            }
        }
        
        public dvec3 zwx {
            get {
                return new dvec3(this.v2, this.v3, this.v0);
            }
            set {
                this.v2 = value.v0;
                this.v3 = value.v1;
                this.v0 = value.v2;
            }
        }
        
        public dvec3 bar {
            get {
                return new dvec3(this.v2, this.v3, this.v0);
            }
            set {
                this.v2 = value.v0;
                this.v3 = value.v1;
                this.v0 = value.v2;
            }
        }
        
        public dvec3 pqs {
            get {
                return new dvec3(this.v2, this.v3, this.v0);
            }
            set {
                this.v2 = value.v0;
                this.v3 = value.v1;
                this.v0 = value.v2;
            }
        }
        
        public dvec3 zwy {
            get {
                return new dvec3(this.v2, this.v3, this.v1);
            }
            set {
                this.v2 = value.v0;
                this.v3 = value.v1;
                this.v1 = value.v2;
            }
        }
        
        public dvec3 bag {
            get {
                return new dvec3(this.v2, this.v3, this.v1);
            }
            set {
                this.v2 = value.v0;
                this.v3 = value.v1;
                this.v1 = value.v2;
            }
        }
        
        public dvec3 pqt {
            get {
                return new dvec3(this.v2, this.v3, this.v1);
            }
            set {
                this.v2 = value.v0;
                this.v3 = value.v1;
                this.v1 = value.v2;
            }
        }
        
        public dvec3 zwz {
            get {
                return new dvec3(this.v2, this.v3, this.v2);
            }
        }
        
        public dvec3 bab {
            get {
                return new dvec3(this.v2, this.v3, this.v2);
            }
        }
        
        public dvec3 pqp {
            get {
                return new dvec3(this.v2, this.v3, this.v2);
            }
        }
        
        public dvec3 zww {
            get {
                return new dvec3(this.v2, this.v3, this.v3);
            }
        }
        
        public dvec3 baa {
            get {
                return new dvec3(this.v2, this.v3, this.v3);
            }
        }
        
        public dvec3 pqq {
            get {
                return new dvec3(this.v2, this.v3, this.v3);
            }
        }
        
        public dvec3 wxx {
            get {
                return new dvec3(this.v3, this.v0, this.v0);
            }
        }
        
        public dvec3 arr {
            get {
                return new dvec3(this.v3, this.v0, this.v0);
            }
        }
        
        public dvec3 qss {
            get {
                return new dvec3(this.v3, this.v0, this.v0);
            }
        }
        
        public dvec3 wxy {
            get {
                return new dvec3(this.v3, this.v0, this.v1);
            }
            set {
                this.v3 = value.v0;
                this.v0 = value.v1;
                this.v1 = value.v2;
            }
        }
        
        public dvec3 arg {
            get {
                return new dvec3(this.v3, this.v0, this.v1);
            }
            set {
                this.v3 = value.v0;
                this.v0 = value.v1;
                this.v1 = value.v2;
            }
        }
        
        public dvec3 qst {
            get {
                return new dvec3(this.v3, this.v0, this.v1);
            }
            set {
                this.v3 = value.v0;
                this.v0 = value.v1;
                this.v1 = value.v2;
            }
        }
        
        public dvec3 wxz {
            get {
                return new dvec3(this.v3, this.v0, this.v2);
            }
            set {
                this.v3 = value.v0;
                this.v0 = value.v1;
                this.v2 = value.v2;
            }
        }
        
        public dvec3 arb {
            get {
                return new dvec3(this.v3, this.v0, this.v2);
            }
            set {
                this.v3 = value.v0;
                this.v0 = value.v1;
                this.v2 = value.v2;
            }
        }
        
        public dvec3 qsp {
            get {
                return new dvec3(this.v3, this.v0, this.v2);
            }
            set {
                this.v3 = value.v0;
                this.v0 = value.v1;
                this.v2 = value.v2;
            }
        }
        
        public dvec3 wxw {
            get {
                return new dvec3(this.v3, this.v0, this.v3);
            }
        }
        
        public dvec3 ara {
            get {
                return new dvec3(this.v3, this.v0, this.v3);
            }
        }
        
        public dvec3 qsq {
            get {
                return new dvec3(this.v3, this.v0, this.v3);
            }
        }
        
        public dvec3 wyx {
            get {
                return new dvec3(this.v3, this.v1, this.v0);
            }
            set {
                this.v3 = value.v0;
                this.v1 = value.v1;
                this.v0 = value.v2;
            }
        }
        
        public dvec3 agr {
            get {
                return new dvec3(this.v3, this.v1, this.v0);
            }
            set {
                this.v3 = value.v0;
                this.v1 = value.v1;
                this.v0 = value.v2;
            }
        }
        
        public dvec3 qts {
            get {
                return new dvec3(this.v3, this.v1, this.v0);
            }
            set {
                this.v3 = value.v0;
                this.v1 = value.v1;
                this.v0 = value.v2;
            }
        }
        
        public dvec3 wyy {
            get {
                return new dvec3(this.v3, this.v1, this.v1);
            }
        }
        
        public dvec3 agg {
            get {
                return new dvec3(this.v3, this.v1, this.v1);
            }
        }
        
        public dvec3 qtt {
            get {
                return new dvec3(this.v3, this.v1, this.v1);
            }
        }
        
        public dvec3 wyz {
            get {
                return new dvec3(this.v3, this.v1, this.v2);
            }
            set {
                this.v3 = value.v0;
                this.v1 = value.v1;
                this.v2 = value.v2;
            }
        }
        
        public dvec3 agb {
            get {
                return new dvec3(this.v3, this.v1, this.v2);
            }
            set {
                this.v3 = value.v0;
                this.v1 = value.v1;
                this.v2 = value.v2;
            }
        }
        
        public dvec3 qtp {
            get {
                return new dvec3(this.v3, this.v1, this.v2);
            }
            set {
                this.v3 = value.v0;
                this.v1 = value.v1;
                this.v2 = value.v2;
            }
        }
        
        public dvec3 wyw {
            get {
                return new dvec3(this.v3, this.v1, this.v3);
            }
        }
        
        public dvec3 aga {
            get {
                return new dvec3(this.v3, this.v1, this.v3);
            }
        }
        
        public dvec3 qtq {
            get {
                return new dvec3(this.v3, this.v1, this.v3);
            }
        }
        
        public dvec3 wzx {
            get {
                return new dvec3(this.v3, this.v2, this.v0);
            }
            set {
                this.v3 = value.v0;
                this.v2 = value.v1;
                this.v0 = value.v2;
            }
        }
        
        public dvec3 abr {
            get {
                return new dvec3(this.v3, this.v2, this.v0);
            }
            set {
                this.v3 = value.v0;
                this.v2 = value.v1;
                this.v0 = value.v2;
            }
        }
        
        public dvec3 qps {
            get {
                return new dvec3(this.v3, this.v2, this.v0);
            }
            set {
                this.v3 = value.v0;
                this.v2 = value.v1;
                this.v0 = value.v2;
            }
        }
        
        public dvec3 wzy {
            get {
                return new dvec3(this.v3, this.v2, this.v1);
            }
            set {
                this.v3 = value.v0;
                this.v2 = value.v1;
                this.v1 = value.v2;
            }
        }
        
        public dvec3 abg {
            get {
                return new dvec3(this.v3, this.v2, this.v1);
            }
            set {
                this.v3 = value.v0;
                this.v2 = value.v1;
                this.v1 = value.v2;
            }
        }
        
        public dvec3 qpt {
            get {
                return new dvec3(this.v3, this.v2, this.v1);
            }
            set {
                this.v3 = value.v0;
                this.v2 = value.v1;
                this.v1 = value.v2;
            }
        }
        
        public dvec3 wzz {
            get {
                return new dvec3(this.v3, this.v2, this.v2);
            }
        }
        
        public dvec3 abb {
            get {
                return new dvec3(this.v3, this.v2, this.v2);
            }
        }
        
        public dvec3 qpp {
            get {
                return new dvec3(this.v3, this.v2, this.v2);
            }
        }
        
        public dvec3 wzw {
            get {
                return new dvec3(this.v3, this.v2, this.v3);
            }
        }
        
        public dvec3 aba {
            get {
                return new dvec3(this.v3, this.v2, this.v3);
            }
        }
        
        public dvec3 qpq {
            get {
                return new dvec3(this.v3, this.v2, this.v3);
            }
        }
        
        public dvec3 wwx {
            get {
                return new dvec3(this.v3, this.v3, this.v0);
            }
        }
        
        public dvec3 aar {
            get {
                return new dvec3(this.v3, this.v3, this.v0);
            }
        }
        
        public dvec3 qqs {
            get {
                return new dvec3(this.v3, this.v3, this.v0);
            }
        }
        
        public dvec3 wwy {
            get {
                return new dvec3(this.v3, this.v3, this.v1);
            }
        }
        
        public dvec3 aag {
            get {
                return new dvec3(this.v3, this.v3, this.v1);
            }
        }
        
        public dvec3 qqt {
            get {
                return new dvec3(this.v3, this.v3, this.v1);
            }
        }
        
        public dvec3 wwz {
            get {
                return new dvec3(this.v3, this.v3, this.v2);
            }
        }
        
        public dvec3 aab {
            get {
                return new dvec3(this.v3, this.v3, this.v2);
            }
        }
        
        public dvec3 qqp {
            get {
                return new dvec3(this.v3, this.v3, this.v2);
            }
        }
        
        public dvec3 www {
            get {
                return new dvec3(this.v3, this.v3, this.v3);
            }
        }
        
        public dvec3 aaa {
            get {
                return new dvec3(this.v3, this.v3, this.v3);
            }
        }
        
        public dvec3 qqq {
            get {
                return new dvec3(this.v3, this.v3, this.v3);
            }
        }
        
        public dvec4 xxxx {
            get {
                return new dvec4(this.v0, this.v0, this.v0, this.v0);
            }
        }
        
        public dvec4 rrrr {
            get {
                return new dvec4(this.v0, this.v0, this.v0, this.v0);
            }
        }
        
        public dvec4 ssss {
            get {
                return new dvec4(this.v0, this.v0, this.v0, this.v0);
            }
        }
        
        public dvec4 xxxy {
            get {
                return new dvec4(this.v0, this.v0, this.v0, this.v1);
            }
        }
        
        public dvec4 rrrg {
            get {
                return new dvec4(this.v0, this.v0, this.v0, this.v1);
            }
        }
        
        public dvec4 ssst {
            get {
                return new dvec4(this.v0, this.v0, this.v0, this.v1);
            }
        }
        
        public dvec4 xxxz {
            get {
                return new dvec4(this.v0, this.v0, this.v0, this.v2);
            }
        }
        
        public dvec4 rrrb {
            get {
                return new dvec4(this.v0, this.v0, this.v0, this.v2);
            }
        }
        
        public dvec4 sssp {
            get {
                return new dvec4(this.v0, this.v0, this.v0, this.v2);
            }
        }
        
        public dvec4 xxxw {
            get {
                return new dvec4(this.v0, this.v0, this.v0, this.v3);
            }
        }
        
        public dvec4 rrra {
            get {
                return new dvec4(this.v0, this.v0, this.v0, this.v3);
            }
        }
        
        public dvec4 sssq {
            get {
                return new dvec4(this.v0, this.v0, this.v0, this.v3);
            }
        }
        
        public dvec4 xxyx {
            get {
                return new dvec4(this.v0, this.v0, this.v1, this.v0);
            }
        }
        
        public dvec4 rrgr {
            get {
                return new dvec4(this.v0, this.v0, this.v1, this.v0);
            }
        }
        
        public dvec4 ssts {
            get {
                return new dvec4(this.v0, this.v0, this.v1, this.v0);
            }
        }
        
        public dvec4 xxyy {
            get {
                return new dvec4(this.v0, this.v0, this.v1, this.v1);
            }
        }
        
        public dvec4 rrgg {
            get {
                return new dvec4(this.v0, this.v0, this.v1, this.v1);
            }
        }
        
        public dvec4 sstt {
            get {
                return new dvec4(this.v0, this.v0, this.v1, this.v1);
            }
        }
        
        public dvec4 xxyz {
            get {
                return new dvec4(this.v0, this.v0, this.v1, this.v2);
            }
        }
        
        public dvec4 rrgb {
            get {
                return new dvec4(this.v0, this.v0, this.v1, this.v2);
            }
        }
        
        public dvec4 sstp {
            get {
                return new dvec4(this.v0, this.v0, this.v1, this.v2);
            }
        }
        
        public dvec4 xxyw {
            get {
                return new dvec4(this.v0, this.v0, this.v1, this.v3);
            }
        }
        
        public dvec4 rrga {
            get {
                return new dvec4(this.v0, this.v0, this.v1, this.v3);
            }
        }
        
        public dvec4 sstq {
            get {
                return new dvec4(this.v0, this.v0, this.v1, this.v3);
            }
        }
        
        public dvec4 xxzx {
            get {
                return new dvec4(this.v0, this.v0, this.v2, this.v0);
            }
        }
        
        public dvec4 rrbr {
            get {
                return new dvec4(this.v0, this.v0, this.v2, this.v0);
            }
        }
        
        public dvec4 ssps {
            get {
                return new dvec4(this.v0, this.v0, this.v2, this.v0);
            }
        }
        
        public dvec4 xxzy {
            get {
                return new dvec4(this.v0, this.v0, this.v2, this.v1);
            }
        }
        
        public dvec4 rrbg {
            get {
                return new dvec4(this.v0, this.v0, this.v2, this.v1);
            }
        }
        
        public dvec4 sspt {
            get {
                return new dvec4(this.v0, this.v0, this.v2, this.v1);
            }
        }
        
        public dvec4 xxzz {
            get {
                return new dvec4(this.v0, this.v0, this.v2, this.v2);
            }
        }
        
        public dvec4 rrbb {
            get {
                return new dvec4(this.v0, this.v0, this.v2, this.v2);
            }
        }
        
        public dvec4 sspp {
            get {
                return new dvec4(this.v0, this.v0, this.v2, this.v2);
            }
        }
        
        public dvec4 xxzw {
            get {
                return new dvec4(this.v0, this.v0, this.v2, this.v3);
            }
        }
        
        public dvec4 rrba {
            get {
                return new dvec4(this.v0, this.v0, this.v2, this.v3);
            }
        }
        
        public dvec4 sspq {
            get {
                return new dvec4(this.v0, this.v0, this.v2, this.v3);
            }
        }
        
        public dvec4 xxwx {
            get {
                return new dvec4(this.v0, this.v0, this.v3, this.v0);
            }
        }
        
        public dvec4 rrar {
            get {
                return new dvec4(this.v0, this.v0, this.v3, this.v0);
            }
        }
        
        public dvec4 ssqs {
            get {
                return new dvec4(this.v0, this.v0, this.v3, this.v0);
            }
        }
        
        public dvec4 xxwy {
            get {
                return new dvec4(this.v0, this.v0, this.v3, this.v1);
            }
        }
        
        public dvec4 rrag {
            get {
                return new dvec4(this.v0, this.v0, this.v3, this.v1);
            }
        }
        
        public dvec4 ssqt {
            get {
                return new dvec4(this.v0, this.v0, this.v3, this.v1);
            }
        }
        
        public dvec4 xxwz {
            get {
                return new dvec4(this.v0, this.v0, this.v3, this.v2);
            }
        }
        
        public dvec4 rrab {
            get {
                return new dvec4(this.v0, this.v0, this.v3, this.v2);
            }
        }
        
        public dvec4 ssqp {
            get {
                return new dvec4(this.v0, this.v0, this.v3, this.v2);
            }
        }
        
        public dvec4 xxww {
            get {
                return new dvec4(this.v0, this.v0, this.v3, this.v3);
            }
        }
        
        public dvec4 rraa {
            get {
                return new dvec4(this.v0, this.v0, this.v3, this.v3);
            }
        }
        
        public dvec4 ssqq {
            get {
                return new dvec4(this.v0, this.v0, this.v3, this.v3);
            }
        }
        
        public dvec4 xyxx {
            get {
                return new dvec4(this.v0, this.v1, this.v0, this.v0);
            }
        }
        
        public dvec4 rgrr {
            get {
                return new dvec4(this.v0, this.v1, this.v0, this.v0);
            }
        }
        
        public dvec4 stss {
            get {
                return new dvec4(this.v0, this.v1, this.v0, this.v0);
            }
        }
        
        public dvec4 xyxy {
            get {
                return new dvec4(this.v0, this.v1, this.v0, this.v1);
            }
        }
        
        public dvec4 rgrg {
            get {
                return new dvec4(this.v0, this.v1, this.v0, this.v1);
            }
        }
        
        public dvec4 stst {
            get {
                return new dvec4(this.v0, this.v1, this.v0, this.v1);
            }
        }
        
        public dvec4 xyxz {
            get {
                return new dvec4(this.v0, this.v1, this.v0, this.v2);
            }
        }
        
        public dvec4 rgrb {
            get {
                return new dvec4(this.v0, this.v1, this.v0, this.v2);
            }
        }
        
        public dvec4 stsp {
            get {
                return new dvec4(this.v0, this.v1, this.v0, this.v2);
            }
        }
        
        public dvec4 xyxw {
            get {
                return new dvec4(this.v0, this.v1, this.v0, this.v3);
            }
        }
        
        public dvec4 rgra {
            get {
                return new dvec4(this.v0, this.v1, this.v0, this.v3);
            }
        }
        
        public dvec4 stsq {
            get {
                return new dvec4(this.v0, this.v1, this.v0, this.v3);
            }
        }
        
        public dvec4 xyyx {
            get {
                return new dvec4(this.v0, this.v1, this.v1, this.v0);
            }
        }
        
        public dvec4 rggr {
            get {
                return new dvec4(this.v0, this.v1, this.v1, this.v0);
            }
        }
        
        public dvec4 stts {
            get {
                return new dvec4(this.v0, this.v1, this.v1, this.v0);
            }
        }
        
        public dvec4 xyyy {
            get {
                return new dvec4(this.v0, this.v1, this.v1, this.v1);
            }
        }
        
        public dvec4 rggg {
            get {
                return new dvec4(this.v0, this.v1, this.v1, this.v1);
            }
        }
        
        public dvec4 sttt {
            get {
                return new dvec4(this.v0, this.v1, this.v1, this.v1);
            }
        }
        
        public dvec4 xyyz {
            get {
                return new dvec4(this.v0, this.v1, this.v1, this.v2);
            }
        }
        
        public dvec4 rggb {
            get {
                return new dvec4(this.v0, this.v1, this.v1, this.v2);
            }
        }
        
        public dvec4 sttp {
            get {
                return new dvec4(this.v0, this.v1, this.v1, this.v2);
            }
        }
        
        public dvec4 xyyw {
            get {
                return new dvec4(this.v0, this.v1, this.v1, this.v3);
            }
        }
        
        public dvec4 rgga {
            get {
                return new dvec4(this.v0, this.v1, this.v1, this.v3);
            }
        }
        
        public dvec4 sttq {
            get {
                return new dvec4(this.v0, this.v1, this.v1, this.v3);
            }
        }
        
        public dvec4 xyzx {
            get {
                return new dvec4(this.v0, this.v1, this.v2, this.v0);
            }
        }
        
        public dvec4 rgbr {
            get {
                return new dvec4(this.v0, this.v1, this.v2, this.v0);
            }
        }
        
        public dvec4 stps {
            get {
                return new dvec4(this.v0, this.v1, this.v2, this.v0);
            }
        }
        
        public dvec4 xyzy {
            get {
                return new dvec4(this.v0, this.v1, this.v2, this.v1);
            }
        }
        
        public dvec4 rgbg {
            get {
                return new dvec4(this.v0, this.v1, this.v2, this.v1);
            }
        }
        
        public dvec4 stpt {
            get {
                return new dvec4(this.v0, this.v1, this.v2, this.v1);
            }
        }
        
        public dvec4 xyzz {
            get {
                return new dvec4(this.v0, this.v1, this.v2, this.v2);
            }
        }
        
        public dvec4 rgbb {
            get {
                return new dvec4(this.v0, this.v1, this.v2, this.v2);
            }
        }
        
        public dvec4 stpp {
            get {
                return new dvec4(this.v0, this.v1, this.v2, this.v2);
            }
        }
        
        public dvec4 xyzw {
            get {
                return new dvec4(this.v0, this.v1, this.v2, this.v3);
            }
            set {
                this.v0 = value.v0;
                this.v1 = value.v1;
                this.v2 = value.v2;
                this.v3 = value.v3;
            }
        }
        
        public dvec4 rgba {
            get {
                return new dvec4(this.v0, this.v1, this.v2, this.v3);
            }
            set {
                this.v0 = value.v0;
                this.v1 = value.v1;
                this.v2 = value.v2;
                this.v3 = value.v3;
            }
        }
        
        public dvec4 stpq {
            get {
                return new dvec4(this.v0, this.v1, this.v2, this.v3);
            }
            set {
                this.v0 = value.v0;
                this.v1 = value.v1;
                this.v2 = value.v2;
                this.v3 = value.v3;
            }
        }
        
        public dvec4 xywx {
            get {
                return new dvec4(this.v0, this.v1, this.v3, this.v0);
            }
        }
        
        public dvec4 rgar {
            get {
                return new dvec4(this.v0, this.v1, this.v3, this.v0);
            }
        }
        
        public dvec4 stqs {
            get {
                return new dvec4(this.v0, this.v1, this.v3, this.v0);
            }
        }
        
        public dvec4 xywy {
            get {
                return new dvec4(this.v0, this.v1, this.v3, this.v1);
            }
        }
        
        public dvec4 rgag {
            get {
                return new dvec4(this.v0, this.v1, this.v3, this.v1);
            }
        }
        
        public dvec4 stqt {
            get {
                return new dvec4(this.v0, this.v1, this.v3, this.v1);
            }
        }
        
        public dvec4 xywz {
            get {
                return new dvec4(this.v0, this.v1, this.v3, this.v2);
            }
            set {
                this.v0 = value.v0;
                this.v1 = value.v1;
                this.v3 = value.v2;
                this.v2 = value.v3;
            }
        }
        
        public dvec4 rgab {
            get {
                return new dvec4(this.v0, this.v1, this.v3, this.v2);
            }
            set {
                this.v0 = value.v0;
                this.v1 = value.v1;
                this.v3 = value.v2;
                this.v2 = value.v3;
            }
        }
        
        public dvec4 stqp {
            get {
                return new dvec4(this.v0, this.v1, this.v3, this.v2);
            }
            set {
                this.v0 = value.v0;
                this.v1 = value.v1;
                this.v3 = value.v2;
                this.v2 = value.v3;
            }
        }
        
        public dvec4 xyww {
            get {
                return new dvec4(this.v0, this.v1, this.v3, this.v3);
            }
        }
        
        public dvec4 rgaa {
            get {
                return new dvec4(this.v0, this.v1, this.v3, this.v3);
            }
        }
        
        public dvec4 stqq {
            get {
                return new dvec4(this.v0, this.v1, this.v3, this.v3);
            }
        }
        
        public dvec4 xzxx {
            get {
                return new dvec4(this.v0, this.v2, this.v0, this.v0);
            }
        }
        
        public dvec4 rbrr {
            get {
                return new dvec4(this.v0, this.v2, this.v0, this.v0);
            }
        }
        
        public dvec4 spss {
            get {
                return new dvec4(this.v0, this.v2, this.v0, this.v0);
            }
        }
        
        public dvec4 xzxy {
            get {
                return new dvec4(this.v0, this.v2, this.v0, this.v1);
            }
        }
        
        public dvec4 rbrg {
            get {
                return new dvec4(this.v0, this.v2, this.v0, this.v1);
            }
        }
        
        public dvec4 spst {
            get {
                return new dvec4(this.v0, this.v2, this.v0, this.v1);
            }
        }
        
        public dvec4 xzxz {
            get {
                return new dvec4(this.v0, this.v2, this.v0, this.v2);
            }
        }
        
        public dvec4 rbrb {
            get {
                return new dvec4(this.v0, this.v2, this.v0, this.v2);
            }
        }
        
        public dvec4 spsp {
            get {
                return new dvec4(this.v0, this.v2, this.v0, this.v2);
            }
        }
        
        public dvec4 xzxw {
            get {
                return new dvec4(this.v0, this.v2, this.v0, this.v3);
            }
        }
        
        public dvec4 rbra {
            get {
                return new dvec4(this.v0, this.v2, this.v0, this.v3);
            }
        }
        
        public dvec4 spsq {
            get {
                return new dvec4(this.v0, this.v2, this.v0, this.v3);
            }
        }
        
        public dvec4 xzyx {
            get {
                return new dvec4(this.v0, this.v2, this.v1, this.v0);
            }
        }
        
        public dvec4 rbgr {
            get {
                return new dvec4(this.v0, this.v2, this.v1, this.v0);
            }
        }
        
        public dvec4 spts {
            get {
                return new dvec4(this.v0, this.v2, this.v1, this.v0);
            }
        }
        
        public dvec4 xzyy {
            get {
                return new dvec4(this.v0, this.v2, this.v1, this.v1);
            }
        }
        
        public dvec4 rbgg {
            get {
                return new dvec4(this.v0, this.v2, this.v1, this.v1);
            }
        }
        
        public dvec4 sptt {
            get {
                return new dvec4(this.v0, this.v2, this.v1, this.v1);
            }
        }
        
        public dvec4 xzyz {
            get {
                return new dvec4(this.v0, this.v2, this.v1, this.v2);
            }
        }
        
        public dvec4 rbgb {
            get {
                return new dvec4(this.v0, this.v2, this.v1, this.v2);
            }
        }
        
        public dvec4 sptp {
            get {
                return new dvec4(this.v0, this.v2, this.v1, this.v2);
            }
        }
        
        public dvec4 xzyw {
            get {
                return new dvec4(this.v0, this.v2, this.v1, this.v3);
            }
            set {
                this.v0 = value.v0;
                this.v2 = value.v1;
                this.v1 = value.v2;
                this.v3 = value.v3;
            }
        }
        
        public dvec4 rbga {
            get {
                return new dvec4(this.v0, this.v2, this.v1, this.v3);
            }
            set {
                this.v0 = value.v0;
                this.v2 = value.v1;
                this.v1 = value.v2;
                this.v3 = value.v3;
            }
        }
        
        public dvec4 sptq {
            get {
                return new dvec4(this.v0, this.v2, this.v1, this.v3);
            }
            set {
                this.v0 = value.v0;
                this.v2 = value.v1;
                this.v1 = value.v2;
                this.v3 = value.v3;
            }
        }
        
        public dvec4 xzzx {
            get {
                return new dvec4(this.v0, this.v2, this.v2, this.v0);
            }
        }
        
        public dvec4 rbbr {
            get {
                return new dvec4(this.v0, this.v2, this.v2, this.v0);
            }
        }
        
        public dvec4 spps {
            get {
                return new dvec4(this.v0, this.v2, this.v2, this.v0);
            }
        }
        
        public dvec4 xzzy {
            get {
                return new dvec4(this.v0, this.v2, this.v2, this.v1);
            }
        }
        
        public dvec4 rbbg {
            get {
                return new dvec4(this.v0, this.v2, this.v2, this.v1);
            }
        }
        
        public dvec4 sppt {
            get {
                return new dvec4(this.v0, this.v2, this.v2, this.v1);
            }
        }
        
        public dvec4 xzzz {
            get {
                return new dvec4(this.v0, this.v2, this.v2, this.v2);
            }
        }
        
        public dvec4 rbbb {
            get {
                return new dvec4(this.v0, this.v2, this.v2, this.v2);
            }
        }
        
        public dvec4 sppp {
            get {
                return new dvec4(this.v0, this.v2, this.v2, this.v2);
            }
        }
        
        public dvec4 xzzw {
            get {
                return new dvec4(this.v0, this.v2, this.v2, this.v3);
            }
        }
        
        public dvec4 rbba {
            get {
                return new dvec4(this.v0, this.v2, this.v2, this.v3);
            }
        }
        
        public dvec4 sppq {
            get {
                return new dvec4(this.v0, this.v2, this.v2, this.v3);
            }
        }
        
        public dvec4 xzwx {
            get {
                return new dvec4(this.v0, this.v2, this.v3, this.v0);
            }
        }
        
        public dvec4 rbar {
            get {
                return new dvec4(this.v0, this.v2, this.v3, this.v0);
            }
        }
        
        public dvec4 spqs {
            get {
                return new dvec4(this.v0, this.v2, this.v3, this.v0);
            }
        }
        
        public dvec4 xzwy {
            get {
                return new dvec4(this.v0, this.v2, this.v3, this.v1);
            }
            set {
                this.v0 = value.v0;
                this.v2 = value.v1;
                this.v3 = value.v2;
                this.v1 = value.v3;
            }
        }
        
        public dvec4 rbag {
            get {
                return new dvec4(this.v0, this.v2, this.v3, this.v1);
            }
            set {
                this.v0 = value.v0;
                this.v2 = value.v1;
                this.v3 = value.v2;
                this.v1 = value.v3;
            }
        }
        
        public dvec4 spqt {
            get {
                return new dvec4(this.v0, this.v2, this.v3, this.v1);
            }
            set {
                this.v0 = value.v0;
                this.v2 = value.v1;
                this.v3 = value.v2;
                this.v1 = value.v3;
            }
        }
        
        public dvec4 xzwz {
            get {
                return new dvec4(this.v0, this.v2, this.v3, this.v2);
            }
        }
        
        public dvec4 rbab {
            get {
                return new dvec4(this.v0, this.v2, this.v3, this.v2);
            }
        }
        
        public dvec4 spqp {
            get {
                return new dvec4(this.v0, this.v2, this.v3, this.v2);
            }
        }
        
        public dvec4 xzww {
            get {
                return new dvec4(this.v0, this.v2, this.v3, this.v3);
            }
        }
        
        public dvec4 rbaa {
            get {
                return new dvec4(this.v0, this.v2, this.v3, this.v3);
            }
        }
        
        public dvec4 spqq {
            get {
                return new dvec4(this.v0, this.v2, this.v3, this.v3);
            }
        }
        
        public dvec4 xwxx {
            get {
                return new dvec4(this.v0, this.v3, this.v0, this.v0);
            }
        }
        
        public dvec4 rarr {
            get {
                return new dvec4(this.v0, this.v3, this.v0, this.v0);
            }
        }
        
        public dvec4 sqss {
            get {
                return new dvec4(this.v0, this.v3, this.v0, this.v0);
            }
        }
        
        public dvec4 xwxy {
            get {
                return new dvec4(this.v0, this.v3, this.v0, this.v1);
            }
        }
        
        public dvec4 rarg {
            get {
                return new dvec4(this.v0, this.v3, this.v0, this.v1);
            }
        }
        
        public dvec4 sqst {
            get {
                return new dvec4(this.v0, this.v3, this.v0, this.v1);
            }
        }
        
        public dvec4 xwxz {
            get {
                return new dvec4(this.v0, this.v3, this.v0, this.v2);
            }
        }
        
        public dvec4 rarb {
            get {
                return new dvec4(this.v0, this.v3, this.v0, this.v2);
            }
        }
        
        public dvec4 sqsp {
            get {
                return new dvec4(this.v0, this.v3, this.v0, this.v2);
            }
        }
        
        public dvec4 xwxw {
            get {
                return new dvec4(this.v0, this.v3, this.v0, this.v3);
            }
        }
        
        public dvec4 rara {
            get {
                return new dvec4(this.v0, this.v3, this.v0, this.v3);
            }
        }
        
        public dvec4 sqsq {
            get {
                return new dvec4(this.v0, this.v3, this.v0, this.v3);
            }
        }
        
        public dvec4 xwyx {
            get {
                return new dvec4(this.v0, this.v3, this.v1, this.v0);
            }
        }
        
        public dvec4 ragr {
            get {
                return new dvec4(this.v0, this.v3, this.v1, this.v0);
            }
        }
        
        public dvec4 sqts {
            get {
                return new dvec4(this.v0, this.v3, this.v1, this.v0);
            }
        }
        
        public dvec4 xwyy {
            get {
                return new dvec4(this.v0, this.v3, this.v1, this.v1);
            }
        }
        
        public dvec4 ragg {
            get {
                return new dvec4(this.v0, this.v3, this.v1, this.v1);
            }
        }
        
        public dvec4 sqtt {
            get {
                return new dvec4(this.v0, this.v3, this.v1, this.v1);
            }
        }
        
        public dvec4 xwyz {
            get {
                return new dvec4(this.v0, this.v3, this.v1, this.v2);
            }
            set {
                this.v0 = value.v0;
                this.v3 = value.v1;
                this.v1 = value.v2;
                this.v2 = value.v3;
            }
        }
        
        public dvec4 ragb {
            get {
                return new dvec4(this.v0, this.v3, this.v1, this.v2);
            }
            set {
                this.v0 = value.v0;
                this.v3 = value.v1;
                this.v1 = value.v2;
                this.v2 = value.v3;
            }
        }
        
        public dvec4 sqtp {
            get {
                return new dvec4(this.v0, this.v3, this.v1, this.v2);
            }
            set {
                this.v0 = value.v0;
                this.v3 = value.v1;
                this.v1 = value.v2;
                this.v2 = value.v3;
            }
        }
        
        public dvec4 xwyw {
            get {
                return new dvec4(this.v0, this.v3, this.v1, this.v3);
            }
        }
        
        public dvec4 raga {
            get {
                return new dvec4(this.v0, this.v3, this.v1, this.v3);
            }
        }
        
        public dvec4 sqtq {
            get {
                return new dvec4(this.v0, this.v3, this.v1, this.v3);
            }
        }
        
        public dvec4 xwzx {
            get {
                return new dvec4(this.v0, this.v3, this.v2, this.v0);
            }
        }
        
        public dvec4 rabr {
            get {
                return new dvec4(this.v0, this.v3, this.v2, this.v0);
            }
        }
        
        public dvec4 sqps {
            get {
                return new dvec4(this.v0, this.v3, this.v2, this.v0);
            }
        }
        
        public dvec4 xwzy {
            get {
                return new dvec4(this.v0, this.v3, this.v2, this.v1);
            }
            set {
                this.v0 = value.v0;
                this.v3 = value.v1;
                this.v2 = value.v2;
                this.v1 = value.v3;
            }
        }
        
        public dvec4 rabg {
            get {
                return new dvec4(this.v0, this.v3, this.v2, this.v1);
            }
            set {
                this.v0 = value.v0;
                this.v3 = value.v1;
                this.v2 = value.v2;
                this.v1 = value.v3;
            }
        }
        
        public dvec4 sqpt {
            get {
                return new dvec4(this.v0, this.v3, this.v2, this.v1);
            }
            set {
                this.v0 = value.v0;
                this.v3 = value.v1;
                this.v2 = value.v2;
                this.v1 = value.v3;
            }
        }
        
        public dvec4 xwzz {
            get {
                return new dvec4(this.v0, this.v3, this.v2, this.v2);
            }
        }
        
        public dvec4 rabb {
            get {
                return new dvec4(this.v0, this.v3, this.v2, this.v2);
            }
        }
        
        public dvec4 sqpp {
            get {
                return new dvec4(this.v0, this.v3, this.v2, this.v2);
            }
        }
        
        public dvec4 xwzw {
            get {
                return new dvec4(this.v0, this.v3, this.v2, this.v3);
            }
        }
        
        public dvec4 raba {
            get {
                return new dvec4(this.v0, this.v3, this.v2, this.v3);
            }
        }
        
        public dvec4 sqpq {
            get {
                return new dvec4(this.v0, this.v3, this.v2, this.v3);
            }
        }
        
        public dvec4 xwwx {
            get {
                return new dvec4(this.v0, this.v3, this.v3, this.v0);
            }
        }
        
        public dvec4 raar {
            get {
                return new dvec4(this.v0, this.v3, this.v3, this.v0);
            }
        }
        
        public dvec4 sqqs {
            get {
                return new dvec4(this.v0, this.v3, this.v3, this.v0);
            }
        }
        
        public dvec4 xwwy {
            get {
                return new dvec4(this.v0, this.v3, this.v3, this.v1);
            }
        }
        
        public dvec4 raag {
            get {
                return new dvec4(this.v0, this.v3, this.v3, this.v1);
            }
        }
        
        public dvec4 sqqt {
            get {
                return new dvec4(this.v0, this.v3, this.v3, this.v1);
            }
        }
        
        public dvec4 xwwz {
            get {
                return new dvec4(this.v0, this.v3, this.v3, this.v2);
            }
        }
        
        public dvec4 raab {
            get {
                return new dvec4(this.v0, this.v3, this.v3, this.v2);
            }
        }
        
        public dvec4 sqqp {
            get {
                return new dvec4(this.v0, this.v3, this.v3, this.v2);
            }
        }
        
        public dvec4 xwww {
            get {
                return new dvec4(this.v0, this.v3, this.v3, this.v3);
            }
        }
        
        public dvec4 raaa {
            get {
                return new dvec4(this.v0, this.v3, this.v3, this.v3);
            }
        }
        
        public dvec4 sqqq {
            get {
                return new dvec4(this.v0, this.v3, this.v3, this.v3);
            }
        }
        
        public dvec4 yxxx {
            get {
                return new dvec4(this.v1, this.v0, this.v0, this.v0);
            }
        }
        
        public dvec4 grrr {
            get {
                return new dvec4(this.v1, this.v0, this.v0, this.v0);
            }
        }
        
        public dvec4 tsss {
            get {
                return new dvec4(this.v1, this.v0, this.v0, this.v0);
            }
        }
        
        public dvec4 yxxy {
            get {
                return new dvec4(this.v1, this.v0, this.v0, this.v1);
            }
        }
        
        public dvec4 grrg {
            get {
                return new dvec4(this.v1, this.v0, this.v0, this.v1);
            }
        }
        
        public dvec4 tsst {
            get {
                return new dvec4(this.v1, this.v0, this.v0, this.v1);
            }
        }
        
        public dvec4 yxxz {
            get {
                return new dvec4(this.v1, this.v0, this.v0, this.v2);
            }
        }
        
        public dvec4 grrb {
            get {
                return new dvec4(this.v1, this.v0, this.v0, this.v2);
            }
        }
        
        public dvec4 tssp {
            get {
                return new dvec4(this.v1, this.v0, this.v0, this.v2);
            }
        }
        
        public dvec4 yxxw {
            get {
                return new dvec4(this.v1, this.v0, this.v0, this.v3);
            }
        }
        
        public dvec4 grra {
            get {
                return new dvec4(this.v1, this.v0, this.v0, this.v3);
            }
        }
        
        public dvec4 tssq {
            get {
                return new dvec4(this.v1, this.v0, this.v0, this.v3);
            }
        }
        
        public dvec4 yxyx {
            get {
                return new dvec4(this.v1, this.v0, this.v1, this.v0);
            }
        }
        
        public dvec4 grgr {
            get {
                return new dvec4(this.v1, this.v0, this.v1, this.v0);
            }
        }
        
        public dvec4 tsts {
            get {
                return new dvec4(this.v1, this.v0, this.v1, this.v0);
            }
        }
        
        public dvec4 yxyy {
            get {
                return new dvec4(this.v1, this.v0, this.v1, this.v1);
            }
        }
        
        public dvec4 grgg {
            get {
                return new dvec4(this.v1, this.v0, this.v1, this.v1);
            }
        }
        
        public dvec4 tstt {
            get {
                return new dvec4(this.v1, this.v0, this.v1, this.v1);
            }
        }
        
        public dvec4 yxyz {
            get {
                return new dvec4(this.v1, this.v0, this.v1, this.v2);
            }
        }
        
        public dvec4 grgb {
            get {
                return new dvec4(this.v1, this.v0, this.v1, this.v2);
            }
        }
        
        public dvec4 tstp {
            get {
                return new dvec4(this.v1, this.v0, this.v1, this.v2);
            }
        }
        
        public dvec4 yxyw {
            get {
                return new dvec4(this.v1, this.v0, this.v1, this.v3);
            }
        }
        
        public dvec4 grga {
            get {
                return new dvec4(this.v1, this.v0, this.v1, this.v3);
            }
        }
        
        public dvec4 tstq {
            get {
                return new dvec4(this.v1, this.v0, this.v1, this.v3);
            }
        }
        
        public dvec4 yxzx {
            get {
                return new dvec4(this.v1, this.v0, this.v2, this.v0);
            }
        }
        
        public dvec4 grbr {
            get {
                return new dvec4(this.v1, this.v0, this.v2, this.v0);
            }
        }
        
        public dvec4 tsps {
            get {
                return new dvec4(this.v1, this.v0, this.v2, this.v0);
            }
        }
        
        public dvec4 yxzy {
            get {
                return new dvec4(this.v1, this.v0, this.v2, this.v1);
            }
        }
        
        public dvec4 grbg {
            get {
                return new dvec4(this.v1, this.v0, this.v2, this.v1);
            }
        }
        
        public dvec4 tspt {
            get {
                return new dvec4(this.v1, this.v0, this.v2, this.v1);
            }
        }
        
        public dvec4 yxzz {
            get {
                return new dvec4(this.v1, this.v0, this.v2, this.v2);
            }
        }
        
        public dvec4 grbb {
            get {
                return new dvec4(this.v1, this.v0, this.v2, this.v2);
            }
        }
        
        public dvec4 tspp {
            get {
                return new dvec4(this.v1, this.v0, this.v2, this.v2);
            }
        }
        
        public dvec4 yxzw {
            get {
                return new dvec4(this.v1, this.v0, this.v2, this.v3);
            }
            set {
                this.v1 = value.v0;
                this.v0 = value.v1;
                this.v2 = value.v2;
                this.v3 = value.v3;
            }
        }
        
        public dvec4 grba {
            get {
                return new dvec4(this.v1, this.v0, this.v2, this.v3);
            }
            set {
                this.v1 = value.v0;
                this.v0 = value.v1;
                this.v2 = value.v2;
                this.v3 = value.v3;
            }
        }
        
        public dvec4 tspq {
            get {
                return new dvec4(this.v1, this.v0, this.v2, this.v3);
            }
            set {
                this.v1 = value.v0;
                this.v0 = value.v1;
                this.v2 = value.v2;
                this.v3 = value.v3;
            }
        }
        
        public dvec4 yxwx {
            get {
                return new dvec4(this.v1, this.v0, this.v3, this.v0);
            }
        }
        
        public dvec4 grar {
            get {
                return new dvec4(this.v1, this.v0, this.v3, this.v0);
            }
        }
        
        public dvec4 tsqs {
            get {
                return new dvec4(this.v1, this.v0, this.v3, this.v0);
            }
        }
        
        public dvec4 yxwy {
            get {
                return new dvec4(this.v1, this.v0, this.v3, this.v1);
            }
        }
        
        public dvec4 grag {
            get {
                return new dvec4(this.v1, this.v0, this.v3, this.v1);
            }
        }
        
        public dvec4 tsqt {
            get {
                return new dvec4(this.v1, this.v0, this.v3, this.v1);
            }
        }
        
        public dvec4 yxwz {
            get {
                return new dvec4(this.v1, this.v0, this.v3, this.v2);
            }
            set {
                this.v1 = value.v0;
                this.v0 = value.v1;
                this.v3 = value.v2;
                this.v2 = value.v3;
            }
        }
        
        public dvec4 grab {
            get {
                return new dvec4(this.v1, this.v0, this.v3, this.v2);
            }
            set {
                this.v1 = value.v0;
                this.v0 = value.v1;
                this.v3 = value.v2;
                this.v2 = value.v3;
            }
        }
        
        public dvec4 tsqp {
            get {
                return new dvec4(this.v1, this.v0, this.v3, this.v2);
            }
            set {
                this.v1 = value.v0;
                this.v0 = value.v1;
                this.v3 = value.v2;
                this.v2 = value.v3;
            }
        }
        
        public dvec4 yxww {
            get {
                return new dvec4(this.v1, this.v0, this.v3, this.v3);
            }
        }
        
        public dvec4 graa {
            get {
                return new dvec4(this.v1, this.v0, this.v3, this.v3);
            }
        }
        
        public dvec4 tsqq {
            get {
                return new dvec4(this.v1, this.v0, this.v3, this.v3);
            }
        }
        
        public dvec4 yyxx {
            get {
                return new dvec4(this.v1, this.v1, this.v0, this.v0);
            }
        }
        
        public dvec4 ggrr {
            get {
                return new dvec4(this.v1, this.v1, this.v0, this.v0);
            }
        }
        
        public dvec4 ttss {
            get {
                return new dvec4(this.v1, this.v1, this.v0, this.v0);
            }
        }
        
        public dvec4 yyxy {
            get {
                return new dvec4(this.v1, this.v1, this.v0, this.v1);
            }
        }
        
        public dvec4 ggrg {
            get {
                return new dvec4(this.v1, this.v1, this.v0, this.v1);
            }
        }
        
        public dvec4 ttst {
            get {
                return new dvec4(this.v1, this.v1, this.v0, this.v1);
            }
        }
        
        public dvec4 yyxz {
            get {
                return new dvec4(this.v1, this.v1, this.v0, this.v2);
            }
        }
        
        public dvec4 ggrb {
            get {
                return new dvec4(this.v1, this.v1, this.v0, this.v2);
            }
        }
        
        public dvec4 ttsp {
            get {
                return new dvec4(this.v1, this.v1, this.v0, this.v2);
            }
        }
        
        public dvec4 yyxw {
            get {
                return new dvec4(this.v1, this.v1, this.v0, this.v3);
            }
        }
        
        public dvec4 ggra {
            get {
                return new dvec4(this.v1, this.v1, this.v0, this.v3);
            }
        }
        
        public dvec4 ttsq {
            get {
                return new dvec4(this.v1, this.v1, this.v0, this.v3);
            }
        }
        
        public dvec4 yyyx {
            get {
                return new dvec4(this.v1, this.v1, this.v1, this.v0);
            }
        }
        
        public dvec4 gggr {
            get {
                return new dvec4(this.v1, this.v1, this.v1, this.v0);
            }
        }
        
        public dvec4 ttts {
            get {
                return new dvec4(this.v1, this.v1, this.v1, this.v0);
            }
        }
        
        public dvec4 yyyy {
            get {
                return new dvec4(this.v1, this.v1, this.v1, this.v1);
            }
        }
        
        public dvec4 gggg {
            get {
                return new dvec4(this.v1, this.v1, this.v1, this.v1);
            }
        }
        
        public dvec4 tttt {
            get {
                return new dvec4(this.v1, this.v1, this.v1, this.v1);
            }
        }
        
        public dvec4 yyyz {
            get {
                return new dvec4(this.v1, this.v1, this.v1, this.v2);
            }
        }
        
        public dvec4 gggb {
            get {
                return new dvec4(this.v1, this.v1, this.v1, this.v2);
            }
        }
        
        public dvec4 tttp {
            get {
                return new dvec4(this.v1, this.v1, this.v1, this.v2);
            }
        }
        
        public dvec4 yyyw {
            get {
                return new dvec4(this.v1, this.v1, this.v1, this.v3);
            }
        }
        
        public dvec4 ggga {
            get {
                return new dvec4(this.v1, this.v1, this.v1, this.v3);
            }
        }
        
        public dvec4 tttq {
            get {
                return new dvec4(this.v1, this.v1, this.v1, this.v3);
            }
        }
        
        public dvec4 yyzx {
            get {
                return new dvec4(this.v1, this.v1, this.v2, this.v0);
            }
        }
        
        public dvec4 ggbr {
            get {
                return new dvec4(this.v1, this.v1, this.v2, this.v0);
            }
        }
        
        public dvec4 ttps {
            get {
                return new dvec4(this.v1, this.v1, this.v2, this.v0);
            }
        }
        
        public dvec4 yyzy {
            get {
                return new dvec4(this.v1, this.v1, this.v2, this.v1);
            }
        }
        
        public dvec4 ggbg {
            get {
                return new dvec4(this.v1, this.v1, this.v2, this.v1);
            }
        }
        
        public dvec4 ttpt {
            get {
                return new dvec4(this.v1, this.v1, this.v2, this.v1);
            }
        }
        
        public dvec4 yyzz {
            get {
                return new dvec4(this.v1, this.v1, this.v2, this.v2);
            }
        }
        
        public dvec4 ggbb {
            get {
                return new dvec4(this.v1, this.v1, this.v2, this.v2);
            }
        }
        
        public dvec4 ttpp {
            get {
                return new dvec4(this.v1, this.v1, this.v2, this.v2);
            }
        }
        
        public dvec4 yyzw {
            get {
                return new dvec4(this.v1, this.v1, this.v2, this.v3);
            }
        }
        
        public dvec4 ggba {
            get {
                return new dvec4(this.v1, this.v1, this.v2, this.v3);
            }
        }
        
        public dvec4 ttpq {
            get {
                return new dvec4(this.v1, this.v1, this.v2, this.v3);
            }
        }
        
        public dvec4 yywx {
            get {
                return new dvec4(this.v1, this.v1, this.v3, this.v0);
            }
        }
        
        public dvec4 ggar {
            get {
                return new dvec4(this.v1, this.v1, this.v3, this.v0);
            }
        }
        
        public dvec4 ttqs {
            get {
                return new dvec4(this.v1, this.v1, this.v3, this.v0);
            }
        }
        
        public dvec4 yywy {
            get {
                return new dvec4(this.v1, this.v1, this.v3, this.v1);
            }
        }
        
        public dvec4 ggag {
            get {
                return new dvec4(this.v1, this.v1, this.v3, this.v1);
            }
        }
        
        public dvec4 ttqt {
            get {
                return new dvec4(this.v1, this.v1, this.v3, this.v1);
            }
        }
        
        public dvec4 yywz {
            get {
                return new dvec4(this.v1, this.v1, this.v3, this.v2);
            }
        }
        
        public dvec4 ggab {
            get {
                return new dvec4(this.v1, this.v1, this.v3, this.v2);
            }
        }
        
        public dvec4 ttqp {
            get {
                return new dvec4(this.v1, this.v1, this.v3, this.v2);
            }
        }
        
        public dvec4 yyww {
            get {
                return new dvec4(this.v1, this.v1, this.v3, this.v3);
            }
        }
        
        public dvec4 ggaa {
            get {
                return new dvec4(this.v1, this.v1, this.v3, this.v3);
            }
        }
        
        public dvec4 ttqq {
            get {
                return new dvec4(this.v1, this.v1, this.v3, this.v3);
            }
        }
        
        public dvec4 yzxx {
            get {
                return new dvec4(this.v1, this.v2, this.v0, this.v0);
            }
        }
        
        public dvec4 gbrr {
            get {
                return new dvec4(this.v1, this.v2, this.v0, this.v0);
            }
        }
        
        public dvec4 tpss {
            get {
                return new dvec4(this.v1, this.v2, this.v0, this.v0);
            }
        }
        
        public dvec4 yzxy {
            get {
                return new dvec4(this.v1, this.v2, this.v0, this.v1);
            }
        }
        
        public dvec4 gbrg {
            get {
                return new dvec4(this.v1, this.v2, this.v0, this.v1);
            }
        }
        
        public dvec4 tpst {
            get {
                return new dvec4(this.v1, this.v2, this.v0, this.v1);
            }
        }
        
        public dvec4 yzxz {
            get {
                return new dvec4(this.v1, this.v2, this.v0, this.v2);
            }
        }
        
        public dvec4 gbrb {
            get {
                return new dvec4(this.v1, this.v2, this.v0, this.v2);
            }
        }
        
        public dvec4 tpsp {
            get {
                return new dvec4(this.v1, this.v2, this.v0, this.v2);
            }
        }
        
        public dvec4 yzxw {
            get {
                return new dvec4(this.v1, this.v2, this.v0, this.v3);
            }
            set {
                this.v1 = value.v0;
                this.v2 = value.v1;
                this.v0 = value.v2;
                this.v3 = value.v3;
            }
        }
        
        public dvec4 gbra {
            get {
                return new dvec4(this.v1, this.v2, this.v0, this.v3);
            }
            set {
                this.v1 = value.v0;
                this.v2 = value.v1;
                this.v0 = value.v2;
                this.v3 = value.v3;
            }
        }
        
        public dvec4 tpsq {
            get {
                return new dvec4(this.v1, this.v2, this.v0, this.v3);
            }
            set {
                this.v1 = value.v0;
                this.v2 = value.v1;
                this.v0 = value.v2;
                this.v3 = value.v3;
            }
        }
        
        public dvec4 yzyx {
            get {
                return new dvec4(this.v1, this.v2, this.v1, this.v0);
            }
        }
        
        public dvec4 gbgr {
            get {
                return new dvec4(this.v1, this.v2, this.v1, this.v0);
            }
        }
        
        public dvec4 tpts {
            get {
                return new dvec4(this.v1, this.v2, this.v1, this.v0);
            }
        }
        
        public dvec4 yzyy {
            get {
                return new dvec4(this.v1, this.v2, this.v1, this.v1);
            }
        }
        
        public dvec4 gbgg {
            get {
                return new dvec4(this.v1, this.v2, this.v1, this.v1);
            }
        }
        
        public dvec4 tptt {
            get {
                return new dvec4(this.v1, this.v2, this.v1, this.v1);
            }
        }
        
        public dvec4 yzyz {
            get {
                return new dvec4(this.v1, this.v2, this.v1, this.v2);
            }
        }
        
        public dvec4 gbgb {
            get {
                return new dvec4(this.v1, this.v2, this.v1, this.v2);
            }
        }
        
        public dvec4 tptp {
            get {
                return new dvec4(this.v1, this.v2, this.v1, this.v2);
            }
        }
        
        public dvec4 yzyw {
            get {
                return new dvec4(this.v1, this.v2, this.v1, this.v3);
            }
        }
        
        public dvec4 gbga {
            get {
                return new dvec4(this.v1, this.v2, this.v1, this.v3);
            }
        }
        
        public dvec4 tptq {
            get {
                return new dvec4(this.v1, this.v2, this.v1, this.v3);
            }
        }
        
        public dvec4 yzzx {
            get {
                return new dvec4(this.v1, this.v2, this.v2, this.v0);
            }
        }
        
        public dvec4 gbbr {
            get {
                return new dvec4(this.v1, this.v2, this.v2, this.v0);
            }
        }
        
        public dvec4 tpps {
            get {
                return new dvec4(this.v1, this.v2, this.v2, this.v0);
            }
        }
        
        public dvec4 yzzy {
            get {
                return new dvec4(this.v1, this.v2, this.v2, this.v1);
            }
        }
        
        public dvec4 gbbg {
            get {
                return new dvec4(this.v1, this.v2, this.v2, this.v1);
            }
        }
        
        public dvec4 tppt {
            get {
                return new dvec4(this.v1, this.v2, this.v2, this.v1);
            }
        }
        
        public dvec4 yzzz {
            get {
                return new dvec4(this.v1, this.v2, this.v2, this.v2);
            }
        }
        
        public dvec4 gbbb {
            get {
                return new dvec4(this.v1, this.v2, this.v2, this.v2);
            }
        }
        
        public dvec4 tppp {
            get {
                return new dvec4(this.v1, this.v2, this.v2, this.v2);
            }
        }
        
        public dvec4 yzzw {
            get {
                return new dvec4(this.v1, this.v2, this.v2, this.v3);
            }
        }
        
        public dvec4 gbba {
            get {
                return new dvec4(this.v1, this.v2, this.v2, this.v3);
            }
        }
        
        public dvec4 tppq {
            get {
                return new dvec4(this.v1, this.v2, this.v2, this.v3);
            }
        }
        
        public dvec4 yzwx {
            get {
                return new dvec4(this.v1, this.v2, this.v3, this.v0);
            }
            set {
                this.v1 = value.v0;
                this.v2 = value.v1;
                this.v3 = value.v2;
                this.v0 = value.v3;
            }
        }
        
        public dvec4 gbar {
            get {
                return new dvec4(this.v1, this.v2, this.v3, this.v0);
            }
            set {
                this.v1 = value.v0;
                this.v2 = value.v1;
                this.v3 = value.v2;
                this.v0 = value.v3;
            }
        }
        
        public dvec4 tpqs {
            get {
                return new dvec4(this.v1, this.v2, this.v3, this.v0);
            }
            set {
                this.v1 = value.v0;
                this.v2 = value.v1;
                this.v3 = value.v2;
                this.v0 = value.v3;
            }
        }
        
        public dvec4 yzwy {
            get {
                return new dvec4(this.v1, this.v2, this.v3, this.v1);
            }
        }
        
        public dvec4 gbag {
            get {
                return new dvec4(this.v1, this.v2, this.v3, this.v1);
            }
        }
        
        public dvec4 tpqt {
            get {
                return new dvec4(this.v1, this.v2, this.v3, this.v1);
            }
        }
        
        public dvec4 yzwz {
            get {
                return new dvec4(this.v1, this.v2, this.v3, this.v2);
            }
        }
        
        public dvec4 gbab {
            get {
                return new dvec4(this.v1, this.v2, this.v3, this.v2);
            }
        }
        
        public dvec4 tpqp {
            get {
                return new dvec4(this.v1, this.v2, this.v3, this.v2);
            }
        }
        
        public dvec4 yzww {
            get {
                return new dvec4(this.v1, this.v2, this.v3, this.v3);
            }
        }
        
        public dvec4 gbaa {
            get {
                return new dvec4(this.v1, this.v2, this.v3, this.v3);
            }
        }
        
        public dvec4 tpqq {
            get {
                return new dvec4(this.v1, this.v2, this.v3, this.v3);
            }
        }
        
        public dvec4 ywxx {
            get {
                return new dvec4(this.v1, this.v3, this.v0, this.v0);
            }
        }
        
        public dvec4 garr {
            get {
                return new dvec4(this.v1, this.v3, this.v0, this.v0);
            }
        }
        
        public dvec4 tqss {
            get {
                return new dvec4(this.v1, this.v3, this.v0, this.v0);
            }
        }
        
        public dvec4 ywxy {
            get {
                return new dvec4(this.v1, this.v3, this.v0, this.v1);
            }
        }
        
        public dvec4 garg {
            get {
                return new dvec4(this.v1, this.v3, this.v0, this.v1);
            }
        }
        
        public dvec4 tqst {
            get {
                return new dvec4(this.v1, this.v3, this.v0, this.v1);
            }
        }
        
        public dvec4 ywxz {
            get {
                return new dvec4(this.v1, this.v3, this.v0, this.v2);
            }
            set {
                this.v1 = value.v0;
                this.v3 = value.v1;
                this.v0 = value.v2;
                this.v2 = value.v3;
            }
        }
        
        public dvec4 garb {
            get {
                return new dvec4(this.v1, this.v3, this.v0, this.v2);
            }
            set {
                this.v1 = value.v0;
                this.v3 = value.v1;
                this.v0 = value.v2;
                this.v2 = value.v3;
            }
        }
        
        public dvec4 tqsp {
            get {
                return new dvec4(this.v1, this.v3, this.v0, this.v2);
            }
            set {
                this.v1 = value.v0;
                this.v3 = value.v1;
                this.v0 = value.v2;
                this.v2 = value.v3;
            }
        }
        
        public dvec4 ywxw {
            get {
                return new dvec4(this.v1, this.v3, this.v0, this.v3);
            }
        }
        
        public dvec4 gara {
            get {
                return new dvec4(this.v1, this.v3, this.v0, this.v3);
            }
        }
        
        public dvec4 tqsq {
            get {
                return new dvec4(this.v1, this.v3, this.v0, this.v3);
            }
        }
        
        public dvec4 ywyx {
            get {
                return new dvec4(this.v1, this.v3, this.v1, this.v0);
            }
        }
        
        public dvec4 gagr {
            get {
                return new dvec4(this.v1, this.v3, this.v1, this.v0);
            }
        }
        
        public dvec4 tqts {
            get {
                return new dvec4(this.v1, this.v3, this.v1, this.v0);
            }
        }
        
        public dvec4 ywyy {
            get {
                return new dvec4(this.v1, this.v3, this.v1, this.v1);
            }
        }
        
        public dvec4 gagg {
            get {
                return new dvec4(this.v1, this.v3, this.v1, this.v1);
            }
        }
        
        public dvec4 tqtt {
            get {
                return new dvec4(this.v1, this.v3, this.v1, this.v1);
            }
        }
        
        public dvec4 ywyz {
            get {
                return new dvec4(this.v1, this.v3, this.v1, this.v2);
            }
        }
        
        public dvec4 gagb {
            get {
                return new dvec4(this.v1, this.v3, this.v1, this.v2);
            }
        }
        
        public dvec4 tqtp {
            get {
                return new dvec4(this.v1, this.v3, this.v1, this.v2);
            }
        }
        
        public dvec4 ywyw {
            get {
                return new dvec4(this.v1, this.v3, this.v1, this.v3);
            }
        }
        
        public dvec4 gaga {
            get {
                return new dvec4(this.v1, this.v3, this.v1, this.v3);
            }
        }
        
        public dvec4 tqtq {
            get {
                return new dvec4(this.v1, this.v3, this.v1, this.v3);
            }
        }
        
        public dvec4 ywzx {
            get {
                return new dvec4(this.v1, this.v3, this.v2, this.v0);
            }
            set {
                this.v1 = value.v0;
                this.v3 = value.v1;
                this.v2 = value.v2;
                this.v0 = value.v3;
            }
        }
        
        public dvec4 gabr {
            get {
                return new dvec4(this.v1, this.v3, this.v2, this.v0);
            }
            set {
                this.v1 = value.v0;
                this.v3 = value.v1;
                this.v2 = value.v2;
                this.v0 = value.v3;
            }
        }
        
        public dvec4 tqps {
            get {
                return new dvec4(this.v1, this.v3, this.v2, this.v0);
            }
            set {
                this.v1 = value.v0;
                this.v3 = value.v1;
                this.v2 = value.v2;
                this.v0 = value.v3;
            }
        }
        
        public dvec4 ywzy {
            get {
                return new dvec4(this.v1, this.v3, this.v2, this.v1);
            }
        }
        
        public dvec4 gabg {
            get {
                return new dvec4(this.v1, this.v3, this.v2, this.v1);
            }
        }
        
        public dvec4 tqpt {
            get {
                return new dvec4(this.v1, this.v3, this.v2, this.v1);
            }
        }
        
        public dvec4 ywzz {
            get {
                return new dvec4(this.v1, this.v3, this.v2, this.v2);
            }
        }
        
        public dvec4 gabb {
            get {
                return new dvec4(this.v1, this.v3, this.v2, this.v2);
            }
        }
        
        public dvec4 tqpp {
            get {
                return new dvec4(this.v1, this.v3, this.v2, this.v2);
            }
        }
        
        public dvec4 ywzw {
            get {
                return new dvec4(this.v1, this.v3, this.v2, this.v3);
            }
        }
        
        public dvec4 gaba {
            get {
                return new dvec4(this.v1, this.v3, this.v2, this.v3);
            }
        }
        
        public dvec4 tqpq {
            get {
                return new dvec4(this.v1, this.v3, this.v2, this.v3);
            }
        }
        
        public dvec4 ywwx {
            get {
                return new dvec4(this.v1, this.v3, this.v3, this.v0);
            }
        }
        
        public dvec4 gaar {
            get {
                return new dvec4(this.v1, this.v3, this.v3, this.v0);
            }
        }
        
        public dvec4 tqqs {
            get {
                return new dvec4(this.v1, this.v3, this.v3, this.v0);
            }
        }
        
        public dvec4 ywwy {
            get {
                return new dvec4(this.v1, this.v3, this.v3, this.v1);
            }
        }
        
        public dvec4 gaag {
            get {
                return new dvec4(this.v1, this.v3, this.v3, this.v1);
            }
        }
        
        public dvec4 tqqt {
            get {
                return new dvec4(this.v1, this.v3, this.v3, this.v1);
            }
        }
        
        public dvec4 ywwz {
            get {
                return new dvec4(this.v1, this.v3, this.v3, this.v2);
            }
        }
        
        public dvec4 gaab {
            get {
                return new dvec4(this.v1, this.v3, this.v3, this.v2);
            }
        }
        
        public dvec4 tqqp {
            get {
                return new dvec4(this.v1, this.v3, this.v3, this.v2);
            }
        }
        
        public dvec4 ywww {
            get {
                return new dvec4(this.v1, this.v3, this.v3, this.v3);
            }
        }
        
        public dvec4 gaaa {
            get {
                return new dvec4(this.v1, this.v3, this.v3, this.v3);
            }
        }
        
        public dvec4 tqqq {
            get {
                return new dvec4(this.v1, this.v3, this.v3, this.v3);
            }
        }
        
        public dvec4 zxxx {
            get {
                return new dvec4(this.v2, this.v0, this.v0, this.v0);
            }
        }
        
        public dvec4 brrr {
            get {
                return new dvec4(this.v2, this.v0, this.v0, this.v0);
            }
        }
        
        public dvec4 psss {
            get {
                return new dvec4(this.v2, this.v0, this.v0, this.v0);
            }
        }
        
        public dvec4 zxxy {
            get {
                return new dvec4(this.v2, this.v0, this.v0, this.v1);
            }
        }
        
        public dvec4 brrg {
            get {
                return new dvec4(this.v2, this.v0, this.v0, this.v1);
            }
        }
        
        public dvec4 psst {
            get {
                return new dvec4(this.v2, this.v0, this.v0, this.v1);
            }
        }
        
        public dvec4 zxxz {
            get {
                return new dvec4(this.v2, this.v0, this.v0, this.v2);
            }
        }
        
        public dvec4 brrb {
            get {
                return new dvec4(this.v2, this.v0, this.v0, this.v2);
            }
        }
        
        public dvec4 pssp {
            get {
                return new dvec4(this.v2, this.v0, this.v0, this.v2);
            }
        }
        
        public dvec4 zxxw {
            get {
                return new dvec4(this.v2, this.v0, this.v0, this.v3);
            }
        }
        
        public dvec4 brra {
            get {
                return new dvec4(this.v2, this.v0, this.v0, this.v3);
            }
        }
        
        public dvec4 pssq {
            get {
                return new dvec4(this.v2, this.v0, this.v0, this.v3);
            }
        }
        
        public dvec4 zxyx {
            get {
                return new dvec4(this.v2, this.v0, this.v1, this.v0);
            }
        }
        
        public dvec4 brgr {
            get {
                return new dvec4(this.v2, this.v0, this.v1, this.v0);
            }
        }
        
        public dvec4 psts {
            get {
                return new dvec4(this.v2, this.v0, this.v1, this.v0);
            }
        }
        
        public dvec4 zxyy {
            get {
                return new dvec4(this.v2, this.v0, this.v1, this.v1);
            }
        }
        
        public dvec4 brgg {
            get {
                return new dvec4(this.v2, this.v0, this.v1, this.v1);
            }
        }
        
        public dvec4 pstt {
            get {
                return new dvec4(this.v2, this.v0, this.v1, this.v1);
            }
        }
        
        public dvec4 zxyz {
            get {
                return new dvec4(this.v2, this.v0, this.v1, this.v2);
            }
        }
        
        public dvec4 brgb {
            get {
                return new dvec4(this.v2, this.v0, this.v1, this.v2);
            }
        }
        
        public dvec4 pstp {
            get {
                return new dvec4(this.v2, this.v0, this.v1, this.v2);
            }
        }
        
        public dvec4 zxyw {
            get {
                return new dvec4(this.v2, this.v0, this.v1, this.v3);
            }
            set {
                this.v2 = value.v0;
                this.v0 = value.v1;
                this.v1 = value.v2;
                this.v3 = value.v3;
            }
        }
        
        public dvec4 brga {
            get {
                return new dvec4(this.v2, this.v0, this.v1, this.v3);
            }
            set {
                this.v2 = value.v0;
                this.v0 = value.v1;
                this.v1 = value.v2;
                this.v3 = value.v3;
            }
        }
        
        public dvec4 pstq {
            get {
                return new dvec4(this.v2, this.v0, this.v1, this.v3);
            }
            set {
                this.v2 = value.v0;
                this.v0 = value.v1;
                this.v1 = value.v2;
                this.v3 = value.v3;
            }
        }
        
        public dvec4 zxzx {
            get {
                return new dvec4(this.v2, this.v0, this.v2, this.v0);
            }
        }
        
        public dvec4 brbr {
            get {
                return new dvec4(this.v2, this.v0, this.v2, this.v0);
            }
        }
        
        public dvec4 psps {
            get {
                return new dvec4(this.v2, this.v0, this.v2, this.v0);
            }
        }
        
        public dvec4 zxzy {
            get {
                return new dvec4(this.v2, this.v0, this.v2, this.v1);
            }
        }
        
        public dvec4 brbg {
            get {
                return new dvec4(this.v2, this.v0, this.v2, this.v1);
            }
        }
        
        public dvec4 pspt {
            get {
                return new dvec4(this.v2, this.v0, this.v2, this.v1);
            }
        }
        
        public dvec4 zxzz {
            get {
                return new dvec4(this.v2, this.v0, this.v2, this.v2);
            }
        }
        
        public dvec4 brbb {
            get {
                return new dvec4(this.v2, this.v0, this.v2, this.v2);
            }
        }
        
        public dvec4 pspp {
            get {
                return new dvec4(this.v2, this.v0, this.v2, this.v2);
            }
        }
        
        public dvec4 zxzw {
            get {
                return new dvec4(this.v2, this.v0, this.v2, this.v3);
            }
        }
        
        public dvec4 brba {
            get {
                return new dvec4(this.v2, this.v0, this.v2, this.v3);
            }
        }
        
        public dvec4 pspq {
            get {
                return new dvec4(this.v2, this.v0, this.v2, this.v3);
            }
        }
        
        public dvec4 zxwx {
            get {
                return new dvec4(this.v2, this.v0, this.v3, this.v0);
            }
        }
        
        public dvec4 brar {
            get {
                return new dvec4(this.v2, this.v0, this.v3, this.v0);
            }
        }
        
        public dvec4 psqs {
            get {
                return new dvec4(this.v2, this.v0, this.v3, this.v0);
            }
        }
        
        public dvec4 zxwy {
            get {
                return new dvec4(this.v2, this.v0, this.v3, this.v1);
            }
            set {
                this.v2 = value.v0;
                this.v0 = value.v1;
                this.v3 = value.v2;
                this.v1 = value.v3;
            }
        }
        
        public dvec4 brag {
            get {
                return new dvec4(this.v2, this.v0, this.v3, this.v1);
            }
            set {
                this.v2 = value.v0;
                this.v0 = value.v1;
                this.v3 = value.v2;
                this.v1 = value.v3;
            }
        }
        
        public dvec4 psqt {
            get {
                return new dvec4(this.v2, this.v0, this.v3, this.v1);
            }
            set {
                this.v2 = value.v0;
                this.v0 = value.v1;
                this.v3 = value.v2;
                this.v1 = value.v3;
            }
        }
        
        public dvec4 zxwz {
            get {
                return new dvec4(this.v2, this.v0, this.v3, this.v2);
            }
        }
        
        public dvec4 brab {
            get {
                return new dvec4(this.v2, this.v0, this.v3, this.v2);
            }
        }
        
        public dvec4 psqp {
            get {
                return new dvec4(this.v2, this.v0, this.v3, this.v2);
            }
        }
        
        public dvec4 zxww {
            get {
                return new dvec4(this.v2, this.v0, this.v3, this.v3);
            }
        }
        
        public dvec4 braa {
            get {
                return new dvec4(this.v2, this.v0, this.v3, this.v3);
            }
        }
        
        public dvec4 psqq {
            get {
                return new dvec4(this.v2, this.v0, this.v3, this.v3);
            }
        }
        
        public dvec4 zyxx {
            get {
                return new dvec4(this.v2, this.v1, this.v0, this.v0);
            }
        }
        
        public dvec4 bgrr {
            get {
                return new dvec4(this.v2, this.v1, this.v0, this.v0);
            }
        }
        
        public dvec4 ptss {
            get {
                return new dvec4(this.v2, this.v1, this.v0, this.v0);
            }
        }
        
        public dvec4 zyxy {
            get {
                return new dvec4(this.v2, this.v1, this.v0, this.v1);
            }
        }
        
        public dvec4 bgrg {
            get {
                return new dvec4(this.v2, this.v1, this.v0, this.v1);
            }
        }
        
        public dvec4 ptst {
            get {
                return new dvec4(this.v2, this.v1, this.v0, this.v1);
            }
        }
        
        public dvec4 zyxz {
            get {
                return new dvec4(this.v2, this.v1, this.v0, this.v2);
            }
        }
        
        public dvec4 bgrb {
            get {
                return new dvec4(this.v2, this.v1, this.v0, this.v2);
            }
        }
        
        public dvec4 ptsp {
            get {
                return new dvec4(this.v2, this.v1, this.v0, this.v2);
            }
        }
        
        public dvec4 zyxw {
            get {
                return new dvec4(this.v2, this.v1, this.v0, this.v3);
            }
            set {
                this.v2 = value.v0;
                this.v1 = value.v1;
                this.v0 = value.v2;
                this.v3 = value.v3;
            }
        }
        
        public dvec4 bgra {
            get {
                return new dvec4(this.v2, this.v1, this.v0, this.v3);
            }
            set {
                this.v2 = value.v0;
                this.v1 = value.v1;
                this.v0 = value.v2;
                this.v3 = value.v3;
            }
        }
        
        public dvec4 ptsq {
            get {
                return new dvec4(this.v2, this.v1, this.v0, this.v3);
            }
            set {
                this.v2 = value.v0;
                this.v1 = value.v1;
                this.v0 = value.v2;
                this.v3 = value.v3;
            }
        }
        
        public dvec4 zyyx {
            get {
                return new dvec4(this.v2, this.v1, this.v1, this.v0);
            }
        }
        
        public dvec4 bggr {
            get {
                return new dvec4(this.v2, this.v1, this.v1, this.v0);
            }
        }
        
        public dvec4 ptts {
            get {
                return new dvec4(this.v2, this.v1, this.v1, this.v0);
            }
        }
        
        public dvec4 zyyy {
            get {
                return new dvec4(this.v2, this.v1, this.v1, this.v1);
            }
        }
        
        public dvec4 bggg {
            get {
                return new dvec4(this.v2, this.v1, this.v1, this.v1);
            }
        }
        
        public dvec4 pttt {
            get {
                return new dvec4(this.v2, this.v1, this.v1, this.v1);
            }
        }
        
        public dvec4 zyyz {
            get {
                return new dvec4(this.v2, this.v1, this.v1, this.v2);
            }
        }
        
        public dvec4 bggb {
            get {
                return new dvec4(this.v2, this.v1, this.v1, this.v2);
            }
        }
        
        public dvec4 pttp {
            get {
                return new dvec4(this.v2, this.v1, this.v1, this.v2);
            }
        }
        
        public dvec4 zyyw {
            get {
                return new dvec4(this.v2, this.v1, this.v1, this.v3);
            }
        }
        
        public dvec4 bgga {
            get {
                return new dvec4(this.v2, this.v1, this.v1, this.v3);
            }
        }
        
        public dvec4 pttq {
            get {
                return new dvec4(this.v2, this.v1, this.v1, this.v3);
            }
        }
        
        public dvec4 zyzx {
            get {
                return new dvec4(this.v2, this.v1, this.v2, this.v0);
            }
        }
        
        public dvec4 bgbr {
            get {
                return new dvec4(this.v2, this.v1, this.v2, this.v0);
            }
        }
        
        public dvec4 ptps {
            get {
                return new dvec4(this.v2, this.v1, this.v2, this.v0);
            }
        }
        
        public dvec4 zyzy {
            get {
                return new dvec4(this.v2, this.v1, this.v2, this.v1);
            }
        }
        
        public dvec4 bgbg {
            get {
                return new dvec4(this.v2, this.v1, this.v2, this.v1);
            }
        }
        
        public dvec4 ptpt {
            get {
                return new dvec4(this.v2, this.v1, this.v2, this.v1);
            }
        }
        
        public dvec4 zyzz {
            get {
                return new dvec4(this.v2, this.v1, this.v2, this.v2);
            }
        }
        
        public dvec4 bgbb {
            get {
                return new dvec4(this.v2, this.v1, this.v2, this.v2);
            }
        }
        
        public dvec4 ptpp {
            get {
                return new dvec4(this.v2, this.v1, this.v2, this.v2);
            }
        }
        
        public dvec4 zyzw {
            get {
                return new dvec4(this.v2, this.v1, this.v2, this.v3);
            }
        }
        
        public dvec4 bgba {
            get {
                return new dvec4(this.v2, this.v1, this.v2, this.v3);
            }
        }
        
        public dvec4 ptpq {
            get {
                return new dvec4(this.v2, this.v1, this.v2, this.v3);
            }
        }
        
        public dvec4 zywx {
            get {
                return new dvec4(this.v2, this.v1, this.v3, this.v0);
            }
            set {
                this.v2 = value.v0;
                this.v1 = value.v1;
                this.v3 = value.v2;
                this.v0 = value.v3;
            }
        }
        
        public dvec4 bgar {
            get {
                return new dvec4(this.v2, this.v1, this.v3, this.v0);
            }
            set {
                this.v2 = value.v0;
                this.v1 = value.v1;
                this.v3 = value.v2;
                this.v0 = value.v3;
            }
        }
        
        public dvec4 ptqs {
            get {
                return new dvec4(this.v2, this.v1, this.v3, this.v0);
            }
            set {
                this.v2 = value.v0;
                this.v1 = value.v1;
                this.v3 = value.v2;
                this.v0 = value.v3;
            }
        }
        
        public dvec4 zywy {
            get {
                return new dvec4(this.v2, this.v1, this.v3, this.v1);
            }
        }
        
        public dvec4 bgag {
            get {
                return new dvec4(this.v2, this.v1, this.v3, this.v1);
            }
        }
        
        public dvec4 ptqt {
            get {
                return new dvec4(this.v2, this.v1, this.v3, this.v1);
            }
        }
        
        public dvec4 zywz {
            get {
                return new dvec4(this.v2, this.v1, this.v3, this.v2);
            }
        }
        
        public dvec4 bgab {
            get {
                return new dvec4(this.v2, this.v1, this.v3, this.v2);
            }
        }
        
        public dvec4 ptqp {
            get {
                return new dvec4(this.v2, this.v1, this.v3, this.v2);
            }
        }
        
        public dvec4 zyww {
            get {
                return new dvec4(this.v2, this.v1, this.v3, this.v3);
            }
        }
        
        public dvec4 bgaa {
            get {
                return new dvec4(this.v2, this.v1, this.v3, this.v3);
            }
        }
        
        public dvec4 ptqq {
            get {
                return new dvec4(this.v2, this.v1, this.v3, this.v3);
            }
        }
        
        public dvec4 zzxx {
            get {
                return new dvec4(this.v2, this.v2, this.v0, this.v0);
            }
        }
        
        public dvec4 bbrr {
            get {
                return new dvec4(this.v2, this.v2, this.v0, this.v0);
            }
        }
        
        public dvec4 ppss {
            get {
                return new dvec4(this.v2, this.v2, this.v0, this.v0);
            }
        }
        
        public dvec4 zzxy {
            get {
                return new dvec4(this.v2, this.v2, this.v0, this.v1);
            }
        }
        
        public dvec4 bbrg {
            get {
                return new dvec4(this.v2, this.v2, this.v0, this.v1);
            }
        }
        
        public dvec4 ppst {
            get {
                return new dvec4(this.v2, this.v2, this.v0, this.v1);
            }
        }
        
        public dvec4 zzxz {
            get {
                return new dvec4(this.v2, this.v2, this.v0, this.v2);
            }
        }
        
        public dvec4 bbrb {
            get {
                return new dvec4(this.v2, this.v2, this.v0, this.v2);
            }
        }
        
        public dvec4 ppsp {
            get {
                return new dvec4(this.v2, this.v2, this.v0, this.v2);
            }
        }
        
        public dvec4 zzxw {
            get {
                return new dvec4(this.v2, this.v2, this.v0, this.v3);
            }
        }
        
        public dvec4 bbra {
            get {
                return new dvec4(this.v2, this.v2, this.v0, this.v3);
            }
        }
        
        public dvec4 ppsq {
            get {
                return new dvec4(this.v2, this.v2, this.v0, this.v3);
            }
        }
        
        public dvec4 zzyx {
            get {
                return new dvec4(this.v2, this.v2, this.v1, this.v0);
            }
        }
        
        public dvec4 bbgr {
            get {
                return new dvec4(this.v2, this.v2, this.v1, this.v0);
            }
        }
        
        public dvec4 ppts {
            get {
                return new dvec4(this.v2, this.v2, this.v1, this.v0);
            }
        }
        
        public dvec4 zzyy {
            get {
                return new dvec4(this.v2, this.v2, this.v1, this.v1);
            }
        }
        
        public dvec4 bbgg {
            get {
                return new dvec4(this.v2, this.v2, this.v1, this.v1);
            }
        }
        
        public dvec4 pptt {
            get {
                return new dvec4(this.v2, this.v2, this.v1, this.v1);
            }
        }
        
        public dvec4 zzyz {
            get {
                return new dvec4(this.v2, this.v2, this.v1, this.v2);
            }
        }
        
        public dvec4 bbgb {
            get {
                return new dvec4(this.v2, this.v2, this.v1, this.v2);
            }
        }
        
        public dvec4 pptp {
            get {
                return new dvec4(this.v2, this.v2, this.v1, this.v2);
            }
        }
        
        public dvec4 zzyw {
            get {
                return new dvec4(this.v2, this.v2, this.v1, this.v3);
            }
        }
        
        public dvec4 bbga {
            get {
                return new dvec4(this.v2, this.v2, this.v1, this.v3);
            }
        }
        
        public dvec4 pptq {
            get {
                return new dvec4(this.v2, this.v2, this.v1, this.v3);
            }
        }
        
        public dvec4 zzzx {
            get {
                return new dvec4(this.v2, this.v2, this.v2, this.v0);
            }
        }
        
        public dvec4 bbbr {
            get {
                return new dvec4(this.v2, this.v2, this.v2, this.v0);
            }
        }
        
        public dvec4 ppps {
            get {
                return new dvec4(this.v2, this.v2, this.v2, this.v0);
            }
        }
        
        public dvec4 zzzy {
            get {
                return new dvec4(this.v2, this.v2, this.v2, this.v1);
            }
        }
        
        public dvec4 bbbg {
            get {
                return new dvec4(this.v2, this.v2, this.v2, this.v1);
            }
        }
        
        public dvec4 pppt {
            get {
                return new dvec4(this.v2, this.v2, this.v2, this.v1);
            }
        }
        
        public dvec4 zzzz {
            get {
                return new dvec4(this.v2, this.v2, this.v2, this.v2);
            }
        }
        
        public dvec4 bbbb {
            get {
                return new dvec4(this.v2, this.v2, this.v2, this.v2);
            }
        }
        
        public dvec4 pppp {
            get {
                return new dvec4(this.v2, this.v2, this.v2, this.v2);
            }
        }
        
        public dvec4 zzzw {
            get {
                return new dvec4(this.v2, this.v2, this.v2, this.v3);
            }
        }
        
        public dvec4 bbba {
            get {
                return new dvec4(this.v2, this.v2, this.v2, this.v3);
            }
        }
        
        public dvec4 pppq {
            get {
                return new dvec4(this.v2, this.v2, this.v2, this.v3);
            }
        }
        
        public dvec4 zzwx {
            get {
                return new dvec4(this.v2, this.v2, this.v3, this.v0);
            }
        }
        
        public dvec4 bbar {
            get {
                return new dvec4(this.v2, this.v2, this.v3, this.v0);
            }
        }
        
        public dvec4 ppqs {
            get {
                return new dvec4(this.v2, this.v2, this.v3, this.v0);
            }
        }
        
        public dvec4 zzwy {
            get {
                return new dvec4(this.v2, this.v2, this.v3, this.v1);
            }
        }
        
        public dvec4 bbag {
            get {
                return new dvec4(this.v2, this.v2, this.v3, this.v1);
            }
        }
        
        public dvec4 ppqt {
            get {
                return new dvec4(this.v2, this.v2, this.v3, this.v1);
            }
        }
        
        public dvec4 zzwz {
            get {
                return new dvec4(this.v2, this.v2, this.v3, this.v2);
            }
        }
        
        public dvec4 bbab {
            get {
                return new dvec4(this.v2, this.v2, this.v3, this.v2);
            }
        }
        
        public dvec4 ppqp {
            get {
                return new dvec4(this.v2, this.v2, this.v3, this.v2);
            }
        }
        
        public dvec4 zzww {
            get {
                return new dvec4(this.v2, this.v2, this.v3, this.v3);
            }
        }
        
        public dvec4 bbaa {
            get {
                return new dvec4(this.v2, this.v2, this.v3, this.v3);
            }
        }
        
        public dvec4 ppqq {
            get {
                return new dvec4(this.v2, this.v2, this.v3, this.v3);
            }
        }
        
        public dvec4 zwxx {
            get {
                return new dvec4(this.v2, this.v3, this.v0, this.v0);
            }
        }
        
        public dvec4 barr {
            get {
                return new dvec4(this.v2, this.v3, this.v0, this.v0);
            }
        }
        
        public dvec4 pqss {
            get {
                return new dvec4(this.v2, this.v3, this.v0, this.v0);
            }
        }
        
        public dvec4 zwxy {
            get {
                return new dvec4(this.v2, this.v3, this.v0, this.v1);
            }
            set {
                this.v2 = value.v0;
                this.v3 = value.v1;
                this.v0 = value.v2;
                this.v1 = value.v3;
            }
        }
        
        public dvec4 barg {
            get {
                return new dvec4(this.v2, this.v3, this.v0, this.v1);
            }
            set {
                this.v2 = value.v0;
                this.v3 = value.v1;
                this.v0 = value.v2;
                this.v1 = value.v3;
            }
        }
        
        public dvec4 pqst {
            get {
                return new dvec4(this.v2, this.v3, this.v0, this.v1);
            }
            set {
                this.v2 = value.v0;
                this.v3 = value.v1;
                this.v0 = value.v2;
                this.v1 = value.v3;
            }
        }
        
        public dvec4 zwxz {
            get {
                return new dvec4(this.v2, this.v3, this.v0, this.v2);
            }
        }
        
        public dvec4 barb {
            get {
                return new dvec4(this.v2, this.v3, this.v0, this.v2);
            }
        }
        
        public dvec4 pqsp {
            get {
                return new dvec4(this.v2, this.v3, this.v0, this.v2);
            }
        }
        
        public dvec4 zwxw {
            get {
                return new dvec4(this.v2, this.v3, this.v0, this.v3);
            }
        }
        
        public dvec4 bara {
            get {
                return new dvec4(this.v2, this.v3, this.v0, this.v3);
            }
        }
        
        public dvec4 pqsq {
            get {
                return new dvec4(this.v2, this.v3, this.v0, this.v3);
            }
        }
        
        public dvec4 zwyx {
            get {
                return new dvec4(this.v2, this.v3, this.v1, this.v0);
            }
            set {
                this.v2 = value.v0;
                this.v3 = value.v1;
                this.v1 = value.v2;
                this.v0 = value.v3;
            }
        }
        
        public dvec4 bagr {
            get {
                return new dvec4(this.v2, this.v3, this.v1, this.v0);
            }
            set {
                this.v2 = value.v0;
                this.v3 = value.v1;
                this.v1 = value.v2;
                this.v0 = value.v3;
            }
        }
        
        public dvec4 pqts {
            get {
                return new dvec4(this.v2, this.v3, this.v1, this.v0);
            }
            set {
                this.v2 = value.v0;
                this.v3 = value.v1;
                this.v1 = value.v2;
                this.v0 = value.v3;
            }
        }
        
        public dvec4 zwyy {
            get {
                return new dvec4(this.v2, this.v3, this.v1, this.v1);
            }
        }
        
        public dvec4 bagg {
            get {
                return new dvec4(this.v2, this.v3, this.v1, this.v1);
            }
        }
        
        public dvec4 pqtt {
            get {
                return new dvec4(this.v2, this.v3, this.v1, this.v1);
            }
        }
        
        public dvec4 zwyz {
            get {
                return new dvec4(this.v2, this.v3, this.v1, this.v2);
            }
        }
        
        public dvec4 bagb {
            get {
                return new dvec4(this.v2, this.v3, this.v1, this.v2);
            }
        }
        
        public dvec4 pqtp {
            get {
                return new dvec4(this.v2, this.v3, this.v1, this.v2);
            }
        }
        
        public dvec4 zwyw {
            get {
                return new dvec4(this.v2, this.v3, this.v1, this.v3);
            }
        }
        
        public dvec4 baga {
            get {
                return new dvec4(this.v2, this.v3, this.v1, this.v3);
            }
        }
        
        public dvec4 pqtq {
            get {
                return new dvec4(this.v2, this.v3, this.v1, this.v3);
            }
        }
        
        public dvec4 zwzx {
            get {
                return new dvec4(this.v2, this.v3, this.v2, this.v0);
            }
        }
        
        public dvec4 babr {
            get {
                return new dvec4(this.v2, this.v3, this.v2, this.v0);
            }
        }
        
        public dvec4 pqps {
            get {
                return new dvec4(this.v2, this.v3, this.v2, this.v0);
            }
        }
        
        public dvec4 zwzy {
            get {
                return new dvec4(this.v2, this.v3, this.v2, this.v1);
            }
        }
        
        public dvec4 babg {
            get {
                return new dvec4(this.v2, this.v3, this.v2, this.v1);
            }
        }
        
        public dvec4 pqpt {
            get {
                return new dvec4(this.v2, this.v3, this.v2, this.v1);
            }
        }
        
        public dvec4 zwzz {
            get {
                return new dvec4(this.v2, this.v3, this.v2, this.v2);
            }
        }
        
        public dvec4 babb {
            get {
                return new dvec4(this.v2, this.v3, this.v2, this.v2);
            }
        }
        
        public dvec4 pqpp {
            get {
                return new dvec4(this.v2, this.v3, this.v2, this.v2);
            }
        }
        
        public dvec4 zwzw {
            get {
                return new dvec4(this.v2, this.v3, this.v2, this.v3);
            }
        }
        
        public dvec4 baba {
            get {
                return new dvec4(this.v2, this.v3, this.v2, this.v3);
            }
        }
        
        public dvec4 pqpq {
            get {
                return new dvec4(this.v2, this.v3, this.v2, this.v3);
            }
        }
        
        public dvec4 zwwx {
            get {
                return new dvec4(this.v2, this.v3, this.v3, this.v0);
            }
        }
        
        public dvec4 baar {
            get {
                return new dvec4(this.v2, this.v3, this.v3, this.v0);
            }
        }
        
        public dvec4 pqqs {
            get {
                return new dvec4(this.v2, this.v3, this.v3, this.v0);
            }
        }
        
        public dvec4 zwwy {
            get {
                return new dvec4(this.v2, this.v3, this.v3, this.v1);
            }
        }
        
        public dvec4 baag {
            get {
                return new dvec4(this.v2, this.v3, this.v3, this.v1);
            }
        }
        
        public dvec4 pqqt {
            get {
                return new dvec4(this.v2, this.v3, this.v3, this.v1);
            }
        }
        
        public dvec4 zwwz {
            get {
                return new dvec4(this.v2, this.v3, this.v3, this.v2);
            }
        }
        
        public dvec4 baab {
            get {
                return new dvec4(this.v2, this.v3, this.v3, this.v2);
            }
        }
        
        public dvec4 pqqp {
            get {
                return new dvec4(this.v2, this.v3, this.v3, this.v2);
            }
        }
        
        public dvec4 zwww {
            get {
                return new dvec4(this.v2, this.v3, this.v3, this.v3);
            }
        }
        
        public dvec4 baaa {
            get {
                return new dvec4(this.v2, this.v3, this.v3, this.v3);
            }
        }
        
        public dvec4 pqqq {
            get {
                return new dvec4(this.v2, this.v3, this.v3, this.v3);
            }
        }
        
        public dvec4 wxxx {
            get {
                return new dvec4(this.v3, this.v0, this.v0, this.v0);
            }
        }
        
        public dvec4 arrr {
            get {
                return new dvec4(this.v3, this.v0, this.v0, this.v0);
            }
        }
        
        public dvec4 qsss {
            get {
                return new dvec4(this.v3, this.v0, this.v0, this.v0);
            }
        }
        
        public dvec4 wxxy {
            get {
                return new dvec4(this.v3, this.v0, this.v0, this.v1);
            }
        }
        
        public dvec4 arrg {
            get {
                return new dvec4(this.v3, this.v0, this.v0, this.v1);
            }
        }
        
        public dvec4 qsst {
            get {
                return new dvec4(this.v3, this.v0, this.v0, this.v1);
            }
        }
        
        public dvec4 wxxz {
            get {
                return new dvec4(this.v3, this.v0, this.v0, this.v2);
            }
        }
        
        public dvec4 arrb {
            get {
                return new dvec4(this.v3, this.v0, this.v0, this.v2);
            }
        }
        
        public dvec4 qssp {
            get {
                return new dvec4(this.v3, this.v0, this.v0, this.v2);
            }
        }
        
        public dvec4 wxxw {
            get {
                return new dvec4(this.v3, this.v0, this.v0, this.v3);
            }
        }
        
        public dvec4 arra {
            get {
                return new dvec4(this.v3, this.v0, this.v0, this.v3);
            }
        }
        
        public dvec4 qssq {
            get {
                return new dvec4(this.v3, this.v0, this.v0, this.v3);
            }
        }
        
        public dvec4 wxyx {
            get {
                return new dvec4(this.v3, this.v0, this.v1, this.v0);
            }
        }
        
        public dvec4 argr {
            get {
                return new dvec4(this.v3, this.v0, this.v1, this.v0);
            }
        }
        
        public dvec4 qsts {
            get {
                return new dvec4(this.v3, this.v0, this.v1, this.v0);
            }
        }
        
        public dvec4 wxyy {
            get {
                return new dvec4(this.v3, this.v0, this.v1, this.v1);
            }
        }
        
        public dvec4 argg {
            get {
                return new dvec4(this.v3, this.v0, this.v1, this.v1);
            }
        }
        
        public dvec4 qstt {
            get {
                return new dvec4(this.v3, this.v0, this.v1, this.v1);
            }
        }
        
        public dvec4 wxyz {
            get {
                return new dvec4(this.v3, this.v0, this.v1, this.v2);
            }
            set {
                this.v3 = value.v0;
                this.v0 = value.v1;
                this.v1 = value.v2;
                this.v2 = value.v3;
            }
        }
        
        public dvec4 argb {
            get {
                return new dvec4(this.v3, this.v0, this.v1, this.v2);
            }
            set {
                this.v3 = value.v0;
                this.v0 = value.v1;
                this.v1 = value.v2;
                this.v2 = value.v3;
            }
        }
        
        public dvec4 qstp {
            get {
                return new dvec4(this.v3, this.v0, this.v1, this.v2);
            }
            set {
                this.v3 = value.v0;
                this.v0 = value.v1;
                this.v1 = value.v2;
                this.v2 = value.v3;
            }
        }
        
        public dvec4 wxyw {
            get {
                return new dvec4(this.v3, this.v0, this.v1, this.v3);
            }
        }
        
        public dvec4 arga {
            get {
                return new dvec4(this.v3, this.v0, this.v1, this.v3);
            }
        }
        
        public dvec4 qstq {
            get {
                return new dvec4(this.v3, this.v0, this.v1, this.v3);
            }
        }
        
        public dvec4 wxzx {
            get {
                return new dvec4(this.v3, this.v0, this.v2, this.v0);
            }
        }
        
        public dvec4 arbr {
            get {
                return new dvec4(this.v3, this.v0, this.v2, this.v0);
            }
        }
        
        public dvec4 qsps {
            get {
                return new dvec4(this.v3, this.v0, this.v2, this.v0);
            }
        }
        
        public dvec4 wxzy {
            get {
                return new dvec4(this.v3, this.v0, this.v2, this.v1);
            }
            set {
                this.v3 = value.v0;
                this.v0 = value.v1;
                this.v2 = value.v2;
                this.v1 = value.v3;
            }
        }
        
        public dvec4 arbg {
            get {
                return new dvec4(this.v3, this.v0, this.v2, this.v1);
            }
            set {
                this.v3 = value.v0;
                this.v0 = value.v1;
                this.v2 = value.v2;
                this.v1 = value.v3;
            }
        }
        
        public dvec4 qspt {
            get {
                return new dvec4(this.v3, this.v0, this.v2, this.v1);
            }
            set {
                this.v3 = value.v0;
                this.v0 = value.v1;
                this.v2 = value.v2;
                this.v1 = value.v3;
            }
        }
        
        public dvec4 wxzz {
            get {
                return new dvec4(this.v3, this.v0, this.v2, this.v2);
            }
        }
        
        public dvec4 arbb {
            get {
                return new dvec4(this.v3, this.v0, this.v2, this.v2);
            }
        }
        
        public dvec4 qspp {
            get {
                return new dvec4(this.v3, this.v0, this.v2, this.v2);
            }
        }
        
        public dvec4 wxzw {
            get {
                return new dvec4(this.v3, this.v0, this.v2, this.v3);
            }
        }
        
        public dvec4 arba {
            get {
                return new dvec4(this.v3, this.v0, this.v2, this.v3);
            }
        }
        
        public dvec4 qspq {
            get {
                return new dvec4(this.v3, this.v0, this.v2, this.v3);
            }
        }
        
        public dvec4 wxwx {
            get {
                return new dvec4(this.v3, this.v0, this.v3, this.v0);
            }
        }
        
        public dvec4 arar {
            get {
                return new dvec4(this.v3, this.v0, this.v3, this.v0);
            }
        }
        
        public dvec4 qsqs {
            get {
                return new dvec4(this.v3, this.v0, this.v3, this.v0);
            }
        }
        
        public dvec4 wxwy {
            get {
                return new dvec4(this.v3, this.v0, this.v3, this.v1);
            }
        }
        
        public dvec4 arag {
            get {
                return new dvec4(this.v3, this.v0, this.v3, this.v1);
            }
        }
        
        public dvec4 qsqt {
            get {
                return new dvec4(this.v3, this.v0, this.v3, this.v1);
            }
        }
        
        public dvec4 wxwz {
            get {
                return new dvec4(this.v3, this.v0, this.v3, this.v2);
            }
        }
        
        public dvec4 arab {
            get {
                return new dvec4(this.v3, this.v0, this.v3, this.v2);
            }
        }
        
        public dvec4 qsqp {
            get {
                return new dvec4(this.v3, this.v0, this.v3, this.v2);
            }
        }
        
        public dvec4 wxww {
            get {
                return new dvec4(this.v3, this.v0, this.v3, this.v3);
            }
        }
        
        public dvec4 araa {
            get {
                return new dvec4(this.v3, this.v0, this.v3, this.v3);
            }
        }
        
        public dvec4 qsqq {
            get {
                return new dvec4(this.v3, this.v0, this.v3, this.v3);
            }
        }
        
        public dvec4 wyxx {
            get {
                return new dvec4(this.v3, this.v1, this.v0, this.v0);
            }
        }
        
        public dvec4 agrr {
            get {
                return new dvec4(this.v3, this.v1, this.v0, this.v0);
            }
        }
        
        public dvec4 qtss {
            get {
                return new dvec4(this.v3, this.v1, this.v0, this.v0);
            }
        }
        
        public dvec4 wyxy {
            get {
                return new dvec4(this.v3, this.v1, this.v0, this.v1);
            }
        }
        
        public dvec4 agrg {
            get {
                return new dvec4(this.v3, this.v1, this.v0, this.v1);
            }
        }
        
        public dvec4 qtst {
            get {
                return new dvec4(this.v3, this.v1, this.v0, this.v1);
            }
        }
        
        public dvec4 wyxz {
            get {
                return new dvec4(this.v3, this.v1, this.v0, this.v2);
            }
            set {
                this.v3 = value.v0;
                this.v1 = value.v1;
                this.v0 = value.v2;
                this.v2 = value.v3;
            }
        }
        
        public dvec4 agrb {
            get {
                return new dvec4(this.v3, this.v1, this.v0, this.v2);
            }
            set {
                this.v3 = value.v0;
                this.v1 = value.v1;
                this.v0 = value.v2;
                this.v2 = value.v3;
            }
        }
        
        public dvec4 qtsp {
            get {
                return new dvec4(this.v3, this.v1, this.v0, this.v2);
            }
            set {
                this.v3 = value.v0;
                this.v1 = value.v1;
                this.v0 = value.v2;
                this.v2 = value.v3;
            }
        }
        
        public dvec4 wyxw {
            get {
                return new dvec4(this.v3, this.v1, this.v0, this.v3);
            }
        }
        
        public dvec4 agra {
            get {
                return new dvec4(this.v3, this.v1, this.v0, this.v3);
            }
        }
        
        public dvec4 qtsq {
            get {
                return new dvec4(this.v3, this.v1, this.v0, this.v3);
            }
        }
        
        public dvec4 wyyx {
            get {
                return new dvec4(this.v3, this.v1, this.v1, this.v0);
            }
        }
        
        public dvec4 aggr {
            get {
                return new dvec4(this.v3, this.v1, this.v1, this.v0);
            }
        }
        
        public dvec4 qtts {
            get {
                return new dvec4(this.v3, this.v1, this.v1, this.v0);
            }
        }
        
        public dvec4 wyyy {
            get {
                return new dvec4(this.v3, this.v1, this.v1, this.v1);
            }
        }
        
        public dvec4 aggg {
            get {
                return new dvec4(this.v3, this.v1, this.v1, this.v1);
            }
        }
        
        public dvec4 qttt {
            get {
                return new dvec4(this.v3, this.v1, this.v1, this.v1);
            }
        }
        
        public dvec4 wyyz {
            get {
                return new dvec4(this.v3, this.v1, this.v1, this.v2);
            }
        }
        
        public dvec4 aggb {
            get {
                return new dvec4(this.v3, this.v1, this.v1, this.v2);
            }
        }
        
        public dvec4 qttp {
            get {
                return new dvec4(this.v3, this.v1, this.v1, this.v2);
            }
        }
        
        public dvec4 wyyw {
            get {
                return new dvec4(this.v3, this.v1, this.v1, this.v3);
            }
        }
        
        public dvec4 agga {
            get {
                return new dvec4(this.v3, this.v1, this.v1, this.v3);
            }
        }
        
        public dvec4 qttq {
            get {
                return new dvec4(this.v3, this.v1, this.v1, this.v3);
            }
        }
        
        public dvec4 wyzx {
            get {
                return new dvec4(this.v3, this.v1, this.v2, this.v0);
            }
            set {
                this.v3 = value.v0;
                this.v1 = value.v1;
                this.v2 = value.v2;
                this.v0 = value.v3;
            }
        }
        
        public dvec4 agbr {
            get {
                return new dvec4(this.v3, this.v1, this.v2, this.v0);
            }
            set {
                this.v3 = value.v0;
                this.v1 = value.v1;
                this.v2 = value.v2;
                this.v0 = value.v3;
            }
        }
        
        public dvec4 qtps {
            get {
                return new dvec4(this.v3, this.v1, this.v2, this.v0);
            }
            set {
                this.v3 = value.v0;
                this.v1 = value.v1;
                this.v2 = value.v2;
                this.v0 = value.v3;
            }
        }
        
        public dvec4 wyzy {
            get {
                return new dvec4(this.v3, this.v1, this.v2, this.v1);
            }
        }
        
        public dvec4 agbg {
            get {
                return new dvec4(this.v3, this.v1, this.v2, this.v1);
            }
        }
        
        public dvec4 qtpt {
            get {
                return new dvec4(this.v3, this.v1, this.v2, this.v1);
            }
        }
        
        public dvec4 wyzz {
            get {
                return new dvec4(this.v3, this.v1, this.v2, this.v2);
            }
        }
        
        public dvec4 agbb {
            get {
                return new dvec4(this.v3, this.v1, this.v2, this.v2);
            }
        }
        
        public dvec4 qtpp {
            get {
                return new dvec4(this.v3, this.v1, this.v2, this.v2);
            }
        }
        
        public dvec4 wyzw {
            get {
                return new dvec4(this.v3, this.v1, this.v2, this.v3);
            }
        }
        
        public dvec4 agba {
            get {
                return new dvec4(this.v3, this.v1, this.v2, this.v3);
            }
        }
        
        public dvec4 qtpq {
            get {
                return new dvec4(this.v3, this.v1, this.v2, this.v3);
            }
        }
        
        public dvec4 wywx {
            get {
                return new dvec4(this.v3, this.v1, this.v3, this.v0);
            }
        }
        
        public dvec4 agar {
            get {
                return new dvec4(this.v3, this.v1, this.v3, this.v0);
            }
        }
        
        public dvec4 qtqs {
            get {
                return new dvec4(this.v3, this.v1, this.v3, this.v0);
            }
        }
        
        public dvec4 wywy {
            get {
                return new dvec4(this.v3, this.v1, this.v3, this.v1);
            }
        }
        
        public dvec4 agag {
            get {
                return new dvec4(this.v3, this.v1, this.v3, this.v1);
            }
        }
        
        public dvec4 qtqt {
            get {
                return new dvec4(this.v3, this.v1, this.v3, this.v1);
            }
        }
        
        public dvec4 wywz {
            get {
                return new dvec4(this.v3, this.v1, this.v3, this.v2);
            }
        }
        
        public dvec4 agab {
            get {
                return new dvec4(this.v3, this.v1, this.v3, this.v2);
            }
        }
        
        public dvec4 qtqp {
            get {
                return new dvec4(this.v3, this.v1, this.v3, this.v2);
            }
        }
        
        public dvec4 wyww {
            get {
                return new dvec4(this.v3, this.v1, this.v3, this.v3);
            }
        }
        
        public dvec4 agaa {
            get {
                return new dvec4(this.v3, this.v1, this.v3, this.v3);
            }
        }
        
        public dvec4 qtqq {
            get {
                return new dvec4(this.v3, this.v1, this.v3, this.v3);
            }
        }
        
        public dvec4 wzxx {
            get {
                return new dvec4(this.v3, this.v2, this.v0, this.v0);
            }
        }
        
        public dvec4 abrr {
            get {
                return new dvec4(this.v3, this.v2, this.v0, this.v0);
            }
        }
        
        public dvec4 qpss {
            get {
                return new dvec4(this.v3, this.v2, this.v0, this.v0);
            }
        }
        
        public dvec4 wzxy {
            get {
                return new dvec4(this.v3, this.v2, this.v0, this.v1);
            }
            set {
                this.v3 = value.v0;
                this.v2 = value.v1;
                this.v0 = value.v2;
                this.v1 = value.v3;
            }
        }
        
        public dvec4 abrg {
            get {
                return new dvec4(this.v3, this.v2, this.v0, this.v1);
            }
            set {
                this.v3 = value.v0;
                this.v2 = value.v1;
                this.v0 = value.v2;
                this.v1 = value.v3;
            }
        }
        
        public dvec4 qpst {
            get {
                return new dvec4(this.v3, this.v2, this.v0, this.v1);
            }
            set {
                this.v3 = value.v0;
                this.v2 = value.v1;
                this.v0 = value.v2;
                this.v1 = value.v3;
            }
        }
        
        public dvec4 wzxz {
            get {
                return new dvec4(this.v3, this.v2, this.v0, this.v2);
            }
        }
        
        public dvec4 abrb {
            get {
                return new dvec4(this.v3, this.v2, this.v0, this.v2);
            }
        }
        
        public dvec4 qpsp {
            get {
                return new dvec4(this.v3, this.v2, this.v0, this.v2);
            }
        }
        
        public dvec4 wzxw {
            get {
                return new dvec4(this.v3, this.v2, this.v0, this.v3);
            }
        }
        
        public dvec4 abra {
            get {
                return new dvec4(this.v3, this.v2, this.v0, this.v3);
            }
        }
        
        public dvec4 qpsq {
            get {
                return new dvec4(this.v3, this.v2, this.v0, this.v3);
            }
        }
        
        public dvec4 wzyx {
            get {
                return new dvec4(this.v3, this.v2, this.v1, this.v0);
            }
            set {
                this.v3 = value.v0;
                this.v2 = value.v1;
                this.v1 = value.v2;
                this.v0 = value.v3;
            }
        }
        
        public dvec4 abgr {
            get {
                return new dvec4(this.v3, this.v2, this.v1, this.v0);
            }
            set {
                this.v3 = value.v0;
                this.v2 = value.v1;
                this.v1 = value.v2;
                this.v0 = value.v3;
            }
        }
        
        public dvec4 qpts {
            get {
                return new dvec4(this.v3, this.v2, this.v1, this.v0);
            }
            set {
                this.v3 = value.v0;
                this.v2 = value.v1;
                this.v1 = value.v2;
                this.v0 = value.v3;
            }
        }
        
        public dvec4 wzyy {
            get {
                return new dvec4(this.v3, this.v2, this.v1, this.v1);
            }
        }
        
        public dvec4 abgg {
            get {
                return new dvec4(this.v3, this.v2, this.v1, this.v1);
            }
        }
        
        public dvec4 qptt {
            get {
                return new dvec4(this.v3, this.v2, this.v1, this.v1);
            }
        }
        
        public dvec4 wzyz {
            get {
                return new dvec4(this.v3, this.v2, this.v1, this.v2);
            }
        }
        
        public dvec4 abgb {
            get {
                return new dvec4(this.v3, this.v2, this.v1, this.v2);
            }
        }
        
        public dvec4 qptp {
            get {
                return new dvec4(this.v3, this.v2, this.v1, this.v2);
            }
        }
        
        public dvec4 wzyw {
            get {
                return new dvec4(this.v3, this.v2, this.v1, this.v3);
            }
        }
        
        public dvec4 abga {
            get {
                return new dvec4(this.v3, this.v2, this.v1, this.v3);
            }
        }
        
        public dvec4 qptq {
            get {
                return new dvec4(this.v3, this.v2, this.v1, this.v3);
            }
        }
        
        public dvec4 wzzx {
            get {
                return new dvec4(this.v3, this.v2, this.v2, this.v0);
            }
        }
        
        public dvec4 abbr {
            get {
                return new dvec4(this.v3, this.v2, this.v2, this.v0);
            }
        }
        
        public dvec4 qpps {
            get {
                return new dvec4(this.v3, this.v2, this.v2, this.v0);
            }
        }
        
        public dvec4 wzzy {
            get {
                return new dvec4(this.v3, this.v2, this.v2, this.v1);
            }
        }
        
        public dvec4 abbg {
            get {
                return new dvec4(this.v3, this.v2, this.v2, this.v1);
            }
        }
        
        public dvec4 qppt {
            get {
                return new dvec4(this.v3, this.v2, this.v2, this.v1);
            }
        }
        
        public dvec4 wzzz {
            get {
                return new dvec4(this.v3, this.v2, this.v2, this.v2);
            }
        }
        
        public dvec4 abbb {
            get {
                return new dvec4(this.v3, this.v2, this.v2, this.v2);
            }
        }
        
        public dvec4 qppp {
            get {
                return new dvec4(this.v3, this.v2, this.v2, this.v2);
            }
        }
        
        public dvec4 wzzw {
            get {
                return new dvec4(this.v3, this.v2, this.v2, this.v3);
            }
        }
        
        public dvec4 abba {
            get {
                return new dvec4(this.v3, this.v2, this.v2, this.v3);
            }
        }
        
        public dvec4 qppq {
            get {
                return new dvec4(this.v3, this.v2, this.v2, this.v3);
            }
        }
        
        public dvec4 wzwx {
            get {
                return new dvec4(this.v3, this.v2, this.v3, this.v0);
            }
        }
        
        public dvec4 abar {
            get {
                return new dvec4(this.v3, this.v2, this.v3, this.v0);
            }
        }
        
        public dvec4 qpqs {
            get {
                return new dvec4(this.v3, this.v2, this.v3, this.v0);
            }
        }
        
        public dvec4 wzwy {
            get {
                return new dvec4(this.v3, this.v2, this.v3, this.v1);
            }
        }
        
        public dvec4 abag {
            get {
                return new dvec4(this.v3, this.v2, this.v3, this.v1);
            }
        }
        
        public dvec4 qpqt {
            get {
                return new dvec4(this.v3, this.v2, this.v3, this.v1);
            }
        }
        
        public dvec4 wzwz {
            get {
                return new dvec4(this.v3, this.v2, this.v3, this.v2);
            }
        }
        
        public dvec4 abab {
            get {
                return new dvec4(this.v3, this.v2, this.v3, this.v2);
            }
        }
        
        public dvec4 qpqp {
            get {
                return new dvec4(this.v3, this.v2, this.v3, this.v2);
            }
        }
        
        public dvec4 wzww {
            get {
                return new dvec4(this.v3, this.v2, this.v3, this.v3);
            }
        }
        
        public dvec4 abaa {
            get {
                return new dvec4(this.v3, this.v2, this.v3, this.v3);
            }
        }
        
        public dvec4 qpqq {
            get {
                return new dvec4(this.v3, this.v2, this.v3, this.v3);
            }
        }
        
        public dvec4 wwxx {
            get {
                return new dvec4(this.v3, this.v3, this.v0, this.v0);
            }
        }
        
        public dvec4 aarr {
            get {
                return new dvec4(this.v3, this.v3, this.v0, this.v0);
            }
        }
        
        public dvec4 qqss {
            get {
                return new dvec4(this.v3, this.v3, this.v0, this.v0);
            }
        }
        
        public dvec4 wwxy {
            get {
                return new dvec4(this.v3, this.v3, this.v0, this.v1);
            }
        }
        
        public dvec4 aarg {
            get {
                return new dvec4(this.v3, this.v3, this.v0, this.v1);
            }
        }
        
        public dvec4 qqst {
            get {
                return new dvec4(this.v3, this.v3, this.v0, this.v1);
            }
        }
        
        public dvec4 wwxz {
            get {
                return new dvec4(this.v3, this.v3, this.v0, this.v2);
            }
        }
        
        public dvec4 aarb {
            get {
                return new dvec4(this.v3, this.v3, this.v0, this.v2);
            }
        }
        
        public dvec4 qqsp {
            get {
                return new dvec4(this.v3, this.v3, this.v0, this.v2);
            }
        }
        
        public dvec4 wwxw {
            get {
                return new dvec4(this.v3, this.v3, this.v0, this.v3);
            }
        }
        
        public dvec4 aara {
            get {
                return new dvec4(this.v3, this.v3, this.v0, this.v3);
            }
        }
        
        public dvec4 qqsq {
            get {
                return new dvec4(this.v3, this.v3, this.v0, this.v3);
            }
        }
        
        public dvec4 wwyx {
            get {
                return new dvec4(this.v3, this.v3, this.v1, this.v0);
            }
        }
        
        public dvec4 aagr {
            get {
                return new dvec4(this.v3, this.v3, this.v1, this.v0);
            }
        }
        
        public dvec4 qqts {
            get {
                return new dvec4(this.v3, this.v3, this.v1, this.v0);
            }
        }
        
        public dvec4 wwyy {
            get {
                return new dvec4(this.v3, this.v3, this.v1, this.v1);
            }
        }
        
        public dvec4 aagg {
            get {
                return new dvec4(this.v3, this.v3, this.v1, this.v1);
            }
        }
        
        public dvec4 qqtt {
            get {
                return new dvec4(this.v3, this.v3, this.v1, this.v1);
            }
        }
        
        public dvec4 wwyz {
            get {
                return new dvec4(this.v3, this.v3, this.v1, this.v2);
            }
        }
        
        public dvec4 aagb {
            get {
                return new dvec4(this.v3, this.v3, this.v1, this.v2);
            }
        }
        
        public dvec4 qqtp {
            get {
                return new dvec4(this.v3, this.v3, this.v1, this.v2);
            }
        }
        
        public dvec4 wwyw {
            get {
                return new dvec4(this.v3, this.v3, this.v1, this.v3);
            }
        }
        
        public dvec4 aaga {
            get {
                return new dvec4(this.v3, this.v3, this.v1, this.v3);
            }
        }
        
        public dvec4 qqtq {
            get {
                return new dvec4(this.v3, this.v3, this.v1, this.v3);
            }
        }
        
        public dvec4 wwzx {
            get {
                return new dvec4(this.v3, this.v3, this.v2, this.v0);
            }
        }
        
        public dvec4 aabr {
            get {
                return new dvec4(this.v3, this.v3, this.v2, this.v0);
            }
        }
        
        public dvec4 qqps {
            get {
                return new dvec4(this.v3, this.v3, this.v2, this.v0);
            }
        }
        
        public dvec4 wwzy {
            get {
                return new dvec4(this.v3, this.v3, this.v2, this.v1);
            }
        }
        
        public dvec4 aabg {
            get {
                return new dvec4(this.v3, this.v3, this.v2, this.v1);
            }
        }
        
        public dvec4 qqpt {
            get {
                return new dvec4(this.v3, this.v3, this.v2, this.v1);
            }
        }
        
        public dvec4 wwzz {
            get {
                return new dvec4(this.v3, this.v3, this.v2, this.v2);
            }
        }
        
        public dvec4 aabb {
            get {
                return new dvec4(this.v3, this.v3, this.v2, this.v2);
            }
        }
        
        public dvec4 qqpp {
            get {
                return new dvec4(this.v3, this.v3, this.v2, this.v2);
            }
        }
        
        public dvec4 wwzw {
            get {
                return new dvec4(this.v3, this.v3, this.v2, this.v3);
            }
        }
        
        public dvec4 aaba {
            get {
                return new dvec4(this.v3, this.v3, this.v2, this.v3);
            }
        }
        
        public dvec4 qqpq {
            get {
                return new dvec4(this.v3, this.v3, this.v2, this.v3);
            }
        }
        
        public dvec4 wwwx {
            get {
                return new dvec4(this.v3, this.v3, this.v3, this.v0);
            }
        }
        
        public dvec4 aaar {
            get {
                return new dvec4(this.v3, this.v3, this.v3, this.v0);
            }
        }
        
        public dvec4 qqqs {
            get {
                return new dvec4(this.v3, this.v3, this.v3, this.v0);
            }
        }
        
        public dvec4 wwwy {
            get {
                return new dvec4(this.v3, this.v3, this.v3, this.v1);
            }
        }
        
        public dvec4 aaag {
            get {
                return new dvec4(this.v3, this.v3, this.v3, this.v1);
            }
        }
        
        public dvec4 qqqt {
            get {
                return new dvec4(this.v3, this.v3, this.v3, this.v1);
            }
        }
        
        public dvec4 wwwz {
            get {
                return new dvec4(this.v3, this.v3, this.v3, this.v2);
            }
        }
        
        public dvec4 aaab {
            get {
                return new dvec4(this.v3, this.v3, this.v3, this.v2);
            }
        }
        
        public dvec4 qqqp {
            get {
                return new dvec4(this.v3, this.v3, this.v3, this.v2);
            }
        }
        
        public dvec4 wwww {
            get {
                return new dvec4(this.v3, this.v3, this.v3, this.v3);
            }
        }
        
        public dvec4 aaaa {
            get {
                return new dvec4(this.v3, this.v3, this.v3, this.v3);
            }
        }
        
        public dvec4 qqqq {
            get {
                return new dvec4(this.v3, this.v3, this.v3, this.v3);
            }
        }
        
        public static dvec4 operator+(dvec4 v) {
            return new dvec4(+v.v0, +v.v1, +v.v2, +v.v3);
        }
        
        public static dvec4 operator-(dvec4 v) {
            return new dvec4(-v.v0, -v.v1, -v.v2, -v.v3);
        }
        
        public static dvec4 operator++(dvec4 v) {
            return new dvec4(++v.v0, ++v.v1, ++v.v2, ++v.v3);
        }
        
        public static dvec4 operator--(dvec4 v) {
            return new dvec4(--v.v0, --v.v1, --v.v2, --v.v3);
        }
        
        public static dvec4 operator+(double s, dvec4 v2) {
            return new dvec4(s + v2.v0, s + v2.v1, s + v2.v2, s + v2.v3);
        }
        
        public static dvec4 operator-(double s, dvec4 v2) {
            return new dvec4(s - v2.v0, s - v2.v1, s - v2.v2, s - v2.v3);
        }
        
        public static dvec4 operator*(double s, dvec4 v2) {
            return new dvec4(s * v2.v0, s * v2.v1, s * v2.v2, s * v2.v3);
        }
        
        public static dvec4 operator/(double s, dvec4 v2) {
            return new dvec4(s / v2.v0, s / v2.v1, s / v2.v2, s / v2.v3);
        }
        
        public static dvec4 operator%(double s, dvec4 v2) {
            return new dvec4(s % v2.v0, s % v2.v1, s % v2.v2, s % v2.v3);
        }
        
        public static dvec4 operator+(dvec4 v1, double s) {
            return new dvec4(v1.v0 + s, v1.v1 + s, v1.v2 + s, v1.v3 + s);
        }
        
        public static dvec4 operator-(dvec4 v1, double s) {
            return new dvec4(v1.v0 - s, v1.v1 - s, v1.v2 - s, v1.v3 - s);
        }
        
        public static dvec4 operator*(dvec4 v1, double s) {
            return new dvec4(v1.v0 * s, v1.v1 * s, v1.v2 * s, v1.v3 * s);
        }
        
        public static dvec4 operator/(dvec4 v1, double s) {
            return new dvec4(v1.v0 / s, v1.v1 / s, v1.v2 / s, v1.v3 / s);
        }
        
        public static dvec4 operator%(dvec4 v1, double s) {
            return new dvec4(v1.v0 % s, v1.v1 % s, v1.v2 % s, v1.v3 % s);
        }
        
        public static dvec4 operator+(dvec4 v1, dvec4 v2) {
            return new dvec4(v1.v0 + v2.v0, v1.v1 + v2.v1, v1.v2 + v2.v2, v1.v3 + v2.v3);
        }
        
        public static dvec4 operator-(dvec4 v1, dvec4 v2) {
            return new dvec4(v1.v0 - v2.v0, v1.v1 - v2.v1, v1.v2 - v2.v2, v1.v3 - v2.v3);
        }
        
        public static dvec4 operator*(dvec4 v1, dvec4 v2) {
            return new dvec4(v1.v0 * v2.v0, v1.v1 * v2.v1, v1.v2 * v2.v2, v1.v3 * v2.v3);
        }
        
        public static dvec4 operator/(dvec4 v1, dvec4 v2) {
            return new dvec4(v1.v0 / v2.v0, v1.v1 / v2.v1, v1.v2 / v2.v2, v1.v3 / v2.v3);
        }
        
        public static dvec4 operator%(dvec4 v1, dvec4 v2) {
            return new dvec4(v1.v0 % v2.v0, v1.v1 % v2.v1, v1.v2 % v2.v2, v1.v3 % v2.v3);
        }
        
        public static bvec4 operator==(dvec4 v1, dvec4 v2) {
            return new bvec4(v1.v0 == v2.v0, v1.v1 == v2.v1, v1.v2 == v2.v2, v1.v3 == v2.v3);
        }
        
        public static bvec4 operator>(dvec4 v1, dvec4 v2) {
            return new bvec4(v1.v0 > v2.v0, v1.v1 > v2.v1, v1.v2 > v2.v2, v1.v3 > v2.v3);
        }
        
        public static bvec4 operator>=(dvec4 v1, dvec4 v2) {
            return new bvec4(v1.v0 >= v2.v0, v1.v1 >= v2.v1, v1.v2 >= v2.v2, v1.v3 >= v2.v3);
        }
        
        public static bvec4 operator<(dvec4 v1, dvec4 v2) {
            return new bvec4(v1.v0 < v2.v0, v1.v1 < v2.v1, v1.v2 < v2.v2, v1.v3 < v2.v3);
        }
        
        public static bvec4 operator<=(dvec4 v1, dvec4 v2) {
            return new bvec4(v1.v0 <= v2.v0, v1.v1 <= v2.v1, v1.v2 <= v2.v2, v1.v3 <= v2.v3);
        }
        
        public static bvec4 operator!=(dvec4 v1, dvec4 v2) {
            return new bvec4(v1.v0 != v2.v0, v1.v1 != v2.v1, v1.v2 != v2.v2, v1.v3 != v2.v3);
        }
        
        public override bool Equals(object obj) {
            return (obj is dvec4) ? bmath.All(this == (dvec4)obj) : false;
        }
        
        public override int GetHashCode() {
            return this.v0.GetHashCode() ^ this.v1.GetHashCode() ^ this.v2.GetHashCode() ^ this.v3.GetHashCode();
        }
        
        public override string ToString() {
            return "dvec4(" + this.v0 + "," + this.v1 + "," + this.v2 + "," + this.v3 + ")";
        }
    }
}
