#region --- License ---
/*
Copyright 2010 Daniel Filonik

Permission is hereby granted, free of charge, to any person obtaining a copy of
this software and associated documentation files (the "Software"), to deal in
the Software without restriction, including without limitation the rights to
use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
of the Software, and to permit persons to whom the Software is furnished to do
so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all
copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
SOFTWARE.
*/
#endregion

namespace TKM {
    using System;
    using System.Runtime.InteropServices;
    using System.Runtime.Serialization;
    
    
    [DataContract()]
    [StructLayout(LayoutKind.Sequential)]
    public struct imat2 {
        
        [DataMember()]
        internal int m00;
        
        [DataMember()]
        internal int m01;
        
        [DataMember()]
        internal int m10;
        
        [DataMember()]
        internal int m11;
        
        public imat2(int s) {
            this.m00 = s;
            this.m01 = 0;
            this.m10 = 0;
            this.m11 = s;
        }
        
        public imat2(imat2 m00) {
            this.m00 = m00.m00;
            this.m01 = m00.m01;
            this.m10 = m00.m10;
            this.m11 = m00.m11;
        }
        
        public imat2(ivec2 m00, ivec2 m10) {
            this.m00 = m00.v0;
            this.m01 = m00.v1;
            this.m10 = m10.v0;
            this.m11 = m10.v1;
        }
        
        public imat2(int m00, int m01, int m10, int m11) {
            this.m00 = m00;
            this.m01 = m01;
            this.m10 = m10;
            this.m11 = m11;
        }
        
        public imat2(imat2x3 s) {
            this.m00 = s.m00;
            this.m01 = s.m01;
            this.m10 = s.m10;
            this.m11 = s.m11;
        }
        
        public imat2(imat2x4 s) {
            this.m00 = s.m00;
            this.m01 = s.m01;
            this.m10 = s.m10;
            this.m11 = s.m11;
        }
        
        public imat2(imat3x2 s) {
            this.m00 = s.m00;
            this.m01 = s.m01;
            this.m10 = s.m10;
            this.m11 = s.m11;
        }
        
        public imat2(imat3 s) {
            this.m00 = s.m00;
            this.m01 = s.m01;
            this.m10 = s.m10;
            this.m11 = s.m11;
        }
        
        public imat2(imat3x4 s) {
            this.m00 = s.m00;
            this.m01 = s.m01;
            this.m10 = s.m10;
            this.m11 = s.m11;
        }
        
        public imat2(imat4x2 s) {
            this.m00 = s.m00;
            this.m01 = s.m01;
            this.m10 = s.m10;
            this.m11 = s.m11;
        }
        
        public imat2(imat4x3 s) {
            this.m00 = s.m00;
            this.m01 = s.m01;
            this.m10 = s.m10;
            this.m11 = s.m11;
        }
        
        public imat2(imat4 s) {
            this.m00 = s.m00;
            this.m01 = s.m01;
            this.m10 = s.m10;
            this.m11 = s.m11;
        }
        
        public int this[int i0, int i1] {
            get {
                if (((i0 < 0) 
                            || (1 < i0))) {
                    throw new System.IndexOutOfRangeException("i0");
                }
                if (((i1 < 0) 
                            || (1 < i1))) {
                    throw new System.IndexOutOfRangeException("i1");
                }
                
                unsafe
                {
                    fixed (Int32* v_ptr = &this.m00)
                    {
                        return *(v_ptr + (i1 + i0 * 2));
                    }
                };
            }
            set {
                if (((i0 < 0) 
                            || (1 < i0))) {
                    throw new System.IndexOutOfRangeException("i0");
                }
                if (((i1 < 0) 
                            || (1 < i1))) {
                    throw new System.IndexOutOfRangeException("i1");
                }
                
                unsafe
                {
                    fixed (Int32* v_ptr = &this.m00)
                    {
                        *(v_ptr  + (i1 + i0 * 2)) = value;
                    }
                };
            }
        }
        
        public ivec2 this[int i0] {
            get {
                return new ivec2(this[i0, 0], this[i0, 1]);
            }
            set {
                this[i0, 0] = value[0];
                this[i0, 1] = value[1];
            }
        }
        
        public static imat2 operator+(imat2 v) {
            return new imat2(+v.m00, +v.m01, +v.m10, +v.m11);
        }
        
        public static imat2 operator-(imat2 v) {
            return new imat2(-v.m00, -v.m01, -v.m10, -v.m11);
        }
        
        public static imat2 operator~(imat2 v) {
            return new imat2(~v.m00, ~v.m01, ~v.m10, ~v.m11);
        }
        
        public static imat2 operator++(imat2 v) {
            return new imat2(++v.m00, ++v.m01, ++v.m10, ++v.m11);
        }
        
        public static imat2 operator--(imat2 v) {
            return new imat2(--v.m00, --v.m01, --v.m10, --v.m11);
        }
        
        public static imat2 operator+(int s, imat2 v2) {
            return new imat2(s + v2.m00, s + v2.m01, s + v2.m10, s + v2.m11);
        }
        
        public static imat2 operator-(int s, imat2 v2) {
            return new imat2(s - v2.m00, s - v2.m01, s - v2.m10, s - v2.m11);
        }
        
        public static imat2 operator*(int s, imat2 v2) {
            return new imat2(s * v2.m00, s * v2.m01, s * v2.m10, s * v2.m11);
        }
        
        public static imat2 operator/(int s, imat2 v2) {
            return new imat2(s / v2.m00, s / v2.m01, s / v2.m10, s / v2.m11);
        }
        
        public static imat2 operator%(int s, imat2 v2) {
            return new imat2(s % v2.m00, s % v2.m01, s % v2.m10, s % v2.m11);
        }
        
        public static imat2 operator&(int s, imat2 v2) {
            return new imat2(s & v2.m00, s & v2.m01, s & v2.m10, s & v2.m11);
        }
        
        public static imat2 operator|(int s, imat2 v2) {
            return new imat2(s | v2.m00, s | v2.m01, s | v2.m10, s | v2.m11);
        }
        
        public static imat2 operator^(int s, imat2 v2) {
            return new imat2(s ^ v2.m00, s ^ v2.m01, s ^ v2.m10, s ^ v2.m11);
        }
        
        public static imat2 operator+(imat2 v1, int s) {
            return new imat2(v1.m00 + s, v1.m01 + s, v1.m10 + s, v1.m11 + s);
        }
        
        public static imat2 operator-(imat2 v1, int s) {
            return new imat2(v1.m00 - s, v1.m01 - s, v1.m10 - s, v1.m11 - s);
        }
        
        public static imat2 operator*(imat2 v1, int s) {
            return new imat2(v1.m00 * s, v1.m01 * s, v1.m10 * s, v1.m11 * s);
        }
        
        public static imat2 operator/(imat2 v1, int s) {
            return new imat2(v1.m00 / s, v1.m01 / s, v1.m10 / s, v1.m11 / s);
        }
        
        public static imat2 operator%(imat2 v1, int s) {
            return new imat2(v1.m00 % s, v1.m01 % s, v1.m10 % s, v1.m11 % s);
        }
        
        public static imat2 operator&(imat2 v1, int s) {
            return new imat2(v1.m00 & s, v1.m01 & s, v1.m10 & s, v1.m11 & s);
        }
        
        public static imat2 operator|(imat2 v1, int s) {
            return new imat2(v1.m00 | s, v1.m01 | s, v1.m10 | s, v1.m11 | s);
        }
        
        public static imat2 operator^(imat2 v1, int s) {
            return new imat2(v1.m00 ^ s, v1.m01 ^ s, v1.m10 ^ s, v1.m11 ^ s);
        }
        
        public static imat2 operator+(imat2 v1, imat2 v2) {
            return new imat2(v1.m00 + v2.m00, v1.m01 + v2.m01, v1.m10 + v2.m10, v1.m11 + v2.m11);
        }
        
        public static imat2 operator-(imat2 v1, imat2 v2) {
            return new imat2(v1.m00 - v2.m00, v1.m01 - v2.m01, v1.m10 - v2.m10, v1.m11 - v2.m11);
        }
        
        public static imat2 operator/(imat2 v1, imat2 v2) {
            return new imat2(v1.m00 / v2.m00, v1.m01 / v2.m01, v1.m10 / v2.m10, v1.m11 / v2.m11);
        }
        
        public static imat2 operator%(imat2 v1, imat2 v2) {
            return new imat2(v1.m00 % v2.m00, v1.m01 % v2.m01, v1.m10 % v2.m10, v1.m11 % v2.m11);
        }
        
        public static imat2 operator&(imat2 v1, imat2 v2) {
            return new imat2(v1.m00 & v2.m00, v1.m01 & v2.m01, v1.m10 & v2.m10, v1.m11 & v2.m11);
        }
        
        public static imat2 operator|(imat2 v1, imat2 v2) {
            return new imat2(v1.m00 | v2.m00, v1.m01 | v2.m01, v1.m10 | v2.m10, v1.m11 | v2.m11);
        }
        
        public static imat2 operator^(imat2 v1, imat2 v2) {
            return new imat2(v1.m00 ^ v2.m00, v1.m01 ^ v2.m01, v1.m10 ^ v2.m10, v1.m11 ^ v2.m11);
        }
        
        public static bmat2 operator==(imat2 v1, imat2 v2) {
            return new bmat2(v1.m00 == v2.m00, v1.m01 == v2.m01, v1.m10 == v2.m10, v1.m11 == v2.m11);
        }
        
        public static bmat2 operator>(imat2 v1, imat2 v2) {
            return new bmat2(v1.m00 > v2.m00, v1.m01 > v2.m01, v1.m10 > v2.m10, v1.m11 > v2.m11);
        }
        
        public static bmat2 operator>=(imat2 v1, imat2 v2) {
            return new bmat2(v1.m00 >= v2.m00, v1.m01 >= v2.m01, v1.m10 >= v2.m10, v1.m11 >= v2.m11);
        }
        
        public static bmat2 operator<(imat2 v1, imat2 v2) {
            return new bmat2(v1.m00 < v2.m00, v1.m01 < v2.m01, v1.m10 < v2.m10, v1.m11 < v2.m11);
        }
        
        public static bmat2 operator<=(imat2 v1, imat2 v2) {
            return new bmat2(v1.m00 <= v2.m00, v1.m01 <= v2.m01, v1.m10 <= v2.m10, v1.m11 <= v2.m11);
        }
        
        public static bmat2 operator!=(imat2 v1, imat2 v2) {
            return new bmat2(v1.m00 != v2.m00, v1.m01 != v2.m01, v1.m10 != v2.m10, v1.m11 != v2.m11);
        }
        
        public static ivec2 operator*(ivec2 v0, imat2 m1) {
            return imath.InnerProduct1(ref v0, ref m1);
        }
        
        public static ivec2 operator*(imat2 m0, ivec2 v1) {
            return imath.InnerProduct1(ref m0, ref v1);
        }
        
        public static imat2 operator*(imat2 m0, imat2 m1) {
            return imath.InnerProduct1(ref m0, ref m1);
        }
        
        public static imat2x3 operator*(imat2 m0, imat2x3 m1) {
            return imath.InnerProduct1(ref m0, ref m1);
        }
        
        public static imat2x4 operator*(imat2 m0, imat2x4 m1) {
            return imath.InnerProduct1(ref m0, ref m1);
        }
        
        public override bool Equals(object obj) {
            return (obj is imat2) ? bmath.All(this == (imat2)obj) : false;
        }
        
        public override int GetHashCode() {
            return this.m00.GetHashCode() ^ this.m01.GetHashCode() ^ this.m10.GetHashCode() ^ this.m11.GetHashCode();
        }
        
        public override string ToString() {
            return "imat2(" + this.m00 + "," + this.m01 + "," + this.m10 + "," + this.m11 + ")";
        }
    }
    
    [DataContract()]
    [StructLayout(LayoutKind.Sequential)]
    public struct imat2x3 {
        
        [DataMember()]
        internal int m00;
        
        [DataMember()]
        internal int m01;
        
        [DataMember()]
        internal int m02;
        
        [DataMember()]
        internal int m10;
        
        [DataMember()]
        internal int m11;
        
        [DataMember()]
        internal int m12;
        
        public imat2x3(int s) {
            this.m00 = s;
            this.m01 = 0;
            this.m02 = 0;
            this.m10 = 0;
            this.m11 = s;
            this.m12 = 0;
        }
        
        public imat2x3(imat2x3 m00) {
            this.m00 = m00.m00;
            this.m01 = m00.m01;
            this.m02 = m00.m02;
            this.m10 = m00.m10;
            this.m11 = m00.m11;
            this.m12 = m00.m12;
        }
        
        public imat2x3(ivec3 m00, ivec3 m10) {
            this.m00 = m00.v0;
            this.m01 = m00.v1;
            this.m02 = m00.v2;
            this.m10 = m10.v0;
            this.m11 = m10.v1;
            this.m12 = m10.v2;
        }
        
        public imat2x3(ivec2 m00, int m01, ivec2 m10, int m11) {
            this.m00 = m00.v0;
            this.m01 = m00.v1;
            this.m02 = m01;
            this.m10 = m10.v0;
            this.m11 = m10.v1;
            this.m12 = m11;
        }
        
        public imat2x3(int m00, ivec2 m01, int m10, ivec2 m11) {
            this.m00 = m00;
            this.m01 = m01.v0;
            this.m02 = m01.v1;
            this.m10 = m10;
            this.m11 = m11.v0;
            this.m12 = m11.v1;
        }
        
        public imat2x3(int m00, int m01, int m02, int m10, int m11, int m12) {
            this.m00 = m00;
            this.m01 = m01;
            this.m02 = m02;
            this.m10 = m10;
            this.m11 = m11;
            this.m12 = m12;
        }
        
        public imat2x3(imat2 s) {
            this.m00 = s.m00;
            this.m01 = s.m01;
            this.m02 = 0;
            this.m10 = s.m10;
            this.m11 = s.m11;
            this.m12 = 0;
        }
        
        public imat2x3(imat2x4 s) {
            this.m00 = s.m00;
            this.m01 = s.m01;
            this.m02 = s.m02;
            this.m10 = s.m10;
            this.m11 = s.m11;
            this.m12 = s.m12;
        }
        
        public imat2x3(imat3x2 s) {
            this.m00 = s.m00;
            this.m01 = s.m01;
            this.m02 = 0;
            this.m10 = s.m10;
            this.m11 = s.m11;
            this.m12 = 0;
        }
        
        public imat2x3(imat3 s) {
            this.m00 = s.m00;
            this.m01 = s.m01;
            this.m02 = s.m02;
            this.m10 = s.m10;
            this.m11 = s.m11;
            this.m12 = s.m12;
        }
        
        public imat2x3(imat3x4 s) {
            this.m00 = s.m00;
            this.m01 = s.m01;
            this.m02 = s.m02;
            this.m10 = s.m10;
            this.m11 = s.m11;
            this.m12 = s.m12;
        }
        
        public imat2x3(imat4x2 s) {
            this.m00 = s.m00;
            this.m01 = s.m01;
            this.m02 = 0;
            this.m10 = s.m10;
            this.m11 = s.m11;
            this.m12 = 0;
        }
        
        public imat2x3(imat4x3 s) {
            this.m00 = s.m00;
            this.m01 = s.m01;
            this.m02 = s.m02;
            this.m10 = s.m10;
            this.m11 = s.m11;
            this.m12 = s.m12;
        }
        
        public imat2x3(imat4 s) {
            this.m00 = s.m00;
            this.m01 = s.m01;
            this.m02 = s.m02;
            this.m10 = s.m10;
            this.m11 = s.m11;
            this.m12 = s.m12;
        }
        
        public int this[int i0, int i1] {
            get {
                if (((i0 < 0) 
                            || (1 < i0))) {
                    throw new System.IndexOutOfRangeException("i0");
                }
                if (((i1 < 0) 
                            || (2 < i1))) {
                    throw new System.IndexOutOfRangeException("i1");
                }
                
                unsafe
                {
                    fixed (Int32* v_ptr = &this.m00)
                    {
                        return *(v_ptr + (i1 + i0 * 3));
                    }
                };
            }
            set {
                if (((i0 < 0) 
                            || (1 < i0))) {
                    throw new System.IndexOutOfRangeException("i0");
                }
                if (((i1 < 0) 
                            || (2 < i1))) {
                    throw new System.IndexOutOfRangeException("i1");
                }
                
                unsafe
                {
                    fixed (Int32* v_ptr = &this.m00)
                    {
                        *(v_ptr  + (i1 + i0 * 3)) = value;
                    }
                };
            }
        }
        
        public ivec3 this[int i0] {
            get {
                return new ivec3(this[i0, 0], this[i0, 1], this[i0, 2]);
            }
            set {
                this[i0, 0] = value[0];
                this[i0, 1] = value[1];
                this[i0, 2] = value[2];
            }
        }
        
        public static imat2x3 operator+(imat2x3 v) {
            return new imat2x3(+v.m00, +v.m01, +v.m02, +v.m10, +v.m11, +v.m12);
        }
        
        public static imat2x3 operator-(imat2x3 v) {
            return new imat2x3(-v.m00, -v.m01, -v.m02, -v.m10, -v.m11, -v.m12);
        }
        
        public static imat2x3 operator~(imat2x3 v) {
            return new imat2x3(~v.m00, ~v.m01, ~v.m02, ~v.m10, ~v.m11, ~v.m12);
        }
        
        public static imat2x3 operator++(imat2x3 v) {
            return new imat2x3(++v.m00, ++v.m01, ++v.m02, ++v.m10, ++v.m11, ++v.m12);
        }
        
        public static imat2x3 operator--(imat2x3 v) {
            return new imat2x3(--v.m00, --v.m01, --v.m02, --v.m10, --v.m11, --v.m12);
        }
        
        public static imat2x3 operator+(int s, imat2x3 v2) {
            return new imat2x3(s + v2.m00, s + v2.m01, s + v2.m02, s + v2.m10, s + v2.m11, s + v2.m12);
        }
        
        public static imat2x3 operator-(int s, imat2x3 v2) {
            return new imat2x3(s - v2.m00, s - v2.m01, s - v2.m02, s - v2.m10, s - v2.m11, s - v2.m12);
        }
        
        public static imat2x3 operator*(int s, imat2x3 v2) {
            return new imat2x3(s * v2.m00, s * v2.m01, s * v2.m02, s * v2.m10, s * v2.m11, s * v2.m12);
        }
        
        public static imat2x3 operator/(int s, imat2x3 v2) {
            return new imat2x3(s / v2.m00, s / v2.m01, s / v2.m02, s / v2.m10, s / v2.m11, s / v2.m12);
        }
        
        public static imat2x3 operator%(int s, imat2x3 v2) {
            return new imat2x3(s % v2.m00, s % v2.m01, s % v2.m02, s % v2.m10, s % v2.m11, s % v2.m12);
        }
        
        public static imat2x3 operator&(int s, imat2x3 v2) {
            return new imat2x3(s & v2.m00, s & v2.m01, s & v2.m02, s & v2.m10, s & v2.m11, s & v2.m12);
        }
        
        public static imat2x3 operator|(int s, imat2x3 v2) {
            return new imat2x3(s | v2.m00, s | v2.m01, s | v2.m02, s | v2.m10, s | v2.m11, s | v2.m12);
        }
        
        public static imat2x3 operator^(int s, imat2x3 v2) {
            return new imat2x3(s ^ v2.m00, s ^ v2.m01, s ^ v2.m02, s ^ v2.m10, s ^ v2.m11, s ^ v2.m12);
        }
        
        public static imat2x3 operator+(imat2x3 v1, int s) {
            return new imat2x3(v1.m00 + s, v1.m01 + s, v1.m02 + s, v1.m10 + s, v1.m11 + s, v1.m12 + s);
        }
        
        public static imat2x3 operator-(imat2x3 v1, int s) {
            return new imat2x3(v1.m00 - s, v1.m01 - s, v1.m02 - s, v1.m10 - s, v1.m11 - s, v1.m12 - s);
        }
        
        public static imat2x3 operator*(imat2x3 v1, int s) {
            return new imat2x3(v1.m00 * s, v1.m01 * s, v1.m02 * s, v1.m10 * s, v1.m11 * s, v1.m12 * s);
        }
        
        public static imat2x3 operator/(imat2x3 v1, int s) {
            return new imat2x3(v1.m00 / s, v1.m01 / s, v1.m02 / s, v1.m10 / s, v1.m11 / s, v1.m12 / s);
        }
        
        public static imat2x3 operator%(imat2x3 v1, int s) {
            return new imat2x3(v1.m00 % s, v1.m01 % s, v1.m02 % s, v1.m10 % s, v1.m11 % s, v1.m12 % s);
        }
        
        public static imat2x3 operator&(imat2x3 v1, int s) {
            return new imat2x3(v1.m00 & s, v1.m01 & s, v1.m02 & s, v1.m10 & s, v1.m11 & s, v1.m12 & s);
        }
        
        public static imat2x3 operator|(imat2x3 v1, int s) {
            return new imat2x3(v1.m00 | s, v1.m01 | s, v1.m02 | s, v1.m10 | s, v1.m11 | s, v1.m12 | s);
        }
        
        public static imat2x3 operator^(imat2x3 v1, int s) {
            return new imat2x3(v1.m00 ^ s, v1.m01 ^ s, v1.m02 ^ s, v1.m10 ^ s, v1.m11 ^ s, v1.m12 ^ s);
        }
        
        public static imat2x3 operator+(imat2x3 v1, imat2x3 v2) {
            return new imat2x3(v1.m00 + v2.m00, v1.m01 + v2.m01, v1.m02 + v2.m02, v1.m10 + v2.m10, v1.m11 + v2.m11, v1.m12 + v2.m12);
        }
        
        public static imat2x3 operator-(imat2x3 v1, imat2x3 v2) {
            return new imat2x3(v1.m00 - v2.m00, v1.m01 - v2.m01, v1.m02 - v2.m02, v1.m10 - v2.m10, v1.m11 - v2.m11, v1.m12 - v2.m12);
        }
        
        public static imat2x3 operator/(imat2x3 v1, imat2x3 v2) {
            return new imat2x3(v1.m00 / v2.m00, v1.m01 / v2.m01, v1.m02 / v2.m02, v1.m10 / v2.m10, v1.m11 / v2.m11, v1.m12 / v2.m12);
        }
        
        public static imat2x3 operator%(imat2x3 v1, imat2x3 v2) {
            return new imat2x3(v1.m00 % v2.m00, v1.m01 % v2.m01, v1.m02 % v2.m02, v1.m10 % v2.m10, v1.m11 % v2.m11, v1.m12 % v2.m12);
        }
        
        public static imat2x3 operator&(imat2x3 v1, imat2x3 v2) {
            return new imat2x3(v1.m00 & v2.m00, v1.m01 & v2.m01, v1.m02 & v2.m02, v1.m10 & v2.m10, v1.m11 & v2.m11, v1.m12 & v2.m12);
        }
        
        public static imat2x3 operator|(imat2x3 v1, imat2x3 v2) {
            return new imat2x3(v1.m00 | v2.m00, v1.m01 | v2.m01, v1.m02 | v2.m02, v1.m10 | v2.m10, v1.m11 | v2.m11, v1.m12 | v2.m12);
        }
        
        public static imat2x3 operator^(imat2x3 v1, imat2x3 v2) {
            return new imat2x3(v1.m00 ^ v2.m00, v1.m01 ^ v2.m01, v1.m02 ^ v2.m02, v1.m10 ^ v2.m10, v1.m11 ^ v2.m11, v1.m12 ^ v2.m12);
        }
        
        public static bmat2x3 operator==(imat2x3 v1, imat2x3 v2) {
            return new bmat2x3(v1.m00 == v2.m00, v1.m01 == v2.m01, v1.m02 == v2.m02, v1.m10 == v2.m10, v1.m11 == v2.m11, v1.m12 == v2.m12);
        }
        
        public static bmat2x3 operator>(imat2x3 v1, imat2x3 v2) {
            return new bmat2x3(v1.m00 > v2.m00, v1.m01 > v2.m01, v1.m02 > v2.m02, v1.m10 > v2.m10, v1.m11 > v2.m11, v1.m12 > v2.m12);
        }
        
        public static bmat2x3 operator>=(imat2x3 v1, imat2x3 v2) {
            return new bmat2x3(v1.m00 >= v2.m00, v1.m01 >= v2.m01, v1.m02 >= v2.m02, v1.m10 >= v2.m10, v1.m11 >= v2.m11, v1.m12 >= v2.m12);
        }
        
        public static bmat2x3 operator<(imat2x3 v1, imat2x3 v2) {
            return new bmat2x3(v1.m00 < v2.m00, v1.m01 < v2.m01, v1.m02 < v2.m02, v1.m10 < v2.m10, v1.m11 < v2.m11, v1.m12 < v2.m12);
        }
        
        public static bmat2x3 operator<=(imat2x3 v1, imat2x3 v2) {
            return new bmat2x3(v1.m00 <= v2.m00, v1.m01 <= v2.m01, v1.m02 <= v2.m02, v1.m10 <= v2.m10, v1.m11 <= v2.m11, v1.m12 <= v2.m12);
        }
        
        public static bmat2x3 operator!=(imat2x3 v1, imat2x3 v2) {
            return new bmat2x3(v1.m00 != v2.m00, v1.m01 != v2.m01, v1.m02 != v2.m02, v1.m10 != v2.m10, v1.m11 != v2.m11, v1.m12 != v2.m12);
        }
        
        public static ivec3 operator*(ivec2 v0, imat2x3 m1) {
            return imath.InnerProduct1(ref v0, ref m1);
        }
        
        public static ivec2 operator*(imat2x3 m0, ivec3 v1) {
            return imath.InnerProduct1(ref m0, ref v1);
        }
        
        public static imat2 operator*(imat2x3 m0, imat3x2 m1) {
            return imath.InnerProduct1(ref m0, ref m1);
        }
        
        public static imat2x3 operator*(imat2x3 m0, imat3 m1) {
            return imath.InnerProduct1(ref m0, ref m1);
        }
        
        public static imat2x4 operator*(imat2x3 m0, imat3x4 m1) {
            return imath.InnerProduct1(ref m0, ref m1);
        }
        
        public override bool Equals(object obj) {
            return (obj is imat2x3) ? bmath.All(this == (imat2x3)obj) : false;
        }
        
        public override int GetHashCode() {
            return this.m00.GetHashCode() ^ this.m01.GetHashCode() ^ this.m02.GetHashCode() ^ this.m10.GetHashCode() ^ this.m11.GetHashCode() ^ this.m12.GetHashCode();
        }
        
        public override string ToString() {
            return "imat2x3(" + this.m00 + "," + this.m01 + "," + this.m02 + "," + this.m10 + "," + this.m11 + "," + this.m12 + ")";
        }
    }
    
    [DataContract()]
    [StructLayout(LayoutKind.Sequential)]
    public struct imat2x4 {
        
        [DataMember()]
        internal int m00;
        
        [DataMember()]
        internal int m01;
        
        [DataMember()]
        internal int m02;
        
        [DataMember()]
        internal int m03;
        
        [DataMember()]
        internal int m10;
        
        [DataMember()]
        internal int m11;
        
        [DataMember()]
        internal int m12;
        
        [DataMember()]
        internal int m13;
        
        public imat2x4(int s) {
            this.m00 = s;
            this.m01 = 0;
            this.m02 = 0;
            this.m03 = 0;
            this.m10 = 0;
            this.m11 = s;
            this.m12 = 0;
            this.m13 = 0;
        }
        
        public imat2x4(imat2x4 m00) {
            this.m00 = m00.m00;
            this.m01 = m00.m01;
            this.m02 = m00.m02;
            this.m03 = m00.m03;
            this.m10 = m00.m10;
            this.m11 = m00.m11;
            this.m12 = m00.m12;
            this.m13 = m00.m13;
        }
        
        public imat2x4(imat2 m00, imat2 m01) {
            this.m00 = m00.m00;
            this.m01 = m00.m01;
            this.m02 = m01.m00;
            this.m03 = m01.m01;
            this.m10 = m00.m10;
            this.m11 = m00.m11;
            this.m12 = m01.m10;
            this.m13 = m01.m11;
        }
        
        public imat2x4(ivec4 m00, ivec4 m10) {
            this.m00 = m00.v0;
            this.m01 = m00.v1;
            this.m02 = m00.v2;
            this.m03 = m00.v3;
            this.m10 = m10.v0;
            this.m11 = m10.v1;
            this.m12 = m10.v2;
            this.m13 = m10.v3;
        }
        
        public imat2x4(ivec2 m00, ivec2 m01, ivec2 m10, ivec2 m11) {
            this.m00 = m00.v0;
            this.m01 = m00.v1;
            this.m02 = m01.v0;
            this.m03 = m01.v1;
            this.m10 = m10.v0;
            this.m11 = m10.v1;
            this.m12 = m11.v0;
            this.m13 = m11.v1;
        }
        
        public imat2x4(ivec3 m00, int m01, ivec3 m10, int m11) {
            this.m00 = m00.v0;
            this.m01 = m00.v1;
            this.m02 = m00.v2;
            this.m03 = m01;
            this.m10 = m10.v0;
            this.m11 = m10.v1;
            this.m12 = m10.v2;
            this.m13 = m11;
        }
        
        public imat2x4(int m00, ivec3 m01, int m10, ivec3 m11) {
            this.m00 = m00;
            this.m01 = m01.v0;
            this.m02 = m01.v1;
            this.m03 = m01.v2;
            this.m10 = m10;
            this.m11 = m11.v0;
            this.m12 = m11.v1;
            this.m13 = m11.v2;
        }
        
        public imat2x4(ivec2 m00, int m01, int m02, ivec2 m10, int m11, int m12) {
            this.m00 = m00.v0;
            this.m01 = m00.v1;
            this.m02 = m01;
            this.m03 = m02;
            this.m10 = m10.v0;
            this.m11 = m10.v1;
            this.m12 = m11;
            this.m13 = m12;
        }
        
        public imat2x4(int m00, ivec2 m01, int m02, int m10, ivec2 m11, int m12) {
            this.m00 = m00;
            this.m01 = m01.v0;
            this.m02 = m01.v1;
            this.m03 = m02;
            this.m10 = m10;
            this.m11 = m11.v0;
            this.m12 = m11.v1;
            this.m13 = m12;
        }
        
        public imat2x4(int m00, int m01, ivec2 m02, int m10, int m11, ivec2 m12) {
            this.m00 = m00;
            this.m01 = m01;
            this.m02 = m02.v0;
            this.m03 = m02.v1;
            this.m10 = m10;
            this.m11 = m11;
            this.m12 = m12.v0;
            this.m13 = m12.v1;
        }
        
        public imat2x4(int m00, int m01, int m02, int m03, int m10, int m11, int m12, int m13) {
            this.m00 = m00;
            this.m01 = m01;
            this.m02 = m02;
            this.m03 = m03;
            this.m10 = m10;
            this.m11 = m11;
            this.m12 = m12;
            this.m13 = m13;
        }
        
        public imat2x4(imat2 s) {
            this.m00 = s.m00;
            this.m01 = s.m01;
            this.m02 = 0;
            this.m03 = 0;
            this.m10 = s.m10;
            this.m11 = s.m11;
            this.m12 = 0;
            this.m13 = 0;
        }
        
        public imat2x4(imat2x3 s) {
            this.m00 = s.m00;
            this.m01 = s.m01;
            this.m02 = s.m02;
            this.m03 = 0;
            this.m10 = s.m10;
            this.m11 = s.m11;
            this.m12 = s.m12;
            this.m13 = 0;
        }
        
        public imat2x4(imat3x2 s) {
            this.m00 = s.m00;
            this.m01 = s.m01;
            this.m02 = 0;
            this.m03 = 0;
            this.m10 = s.m10;
            this.m11 = s.m11;
            this.m12 = 0;
            this.m13 = 0;
        }
        
        public imat2x4(imat3 s) {
            this.m00 = s.m00;
            this.m01 = s.m01;
            this.m02 = s.m02;
            this.m03 = 0;
            this.m10 = s.m10;
            this.m11 = s.m11;
            this.m12 = s.m12;
            this.m13 = 0;
        }
        
        public imat2x4(imat3x4 s) {
            this.m00 = s.m00;
            this.m01 = s.m01;
            this.m02 = s.m02;
            this.m03 = s.m03;
            this.m10 = s.m10;
            this.m11 = s.m11;
            this.m12 = s.m12;
            this.m13 = s.m13;
        }
        
        public imat2x4(imat4x2 s) {
            this.m00 = s.m00;
            this.m01 = s.m01;
            this.m02 = 0;
            this.m03 = 0;
            this.m10 = s.m10;
            this.m11 = s.m11;
            this.m12 = 0;
            this.m13 = 0;
        }
        
        public imat2x4(imat4x3 s) {
            this.m00 = s.m00;
            this.m01 = s.m01;
            this.m02 = s.m02;
            this.m03 = 0;
            this.m10 = s.m10;
            this.m11 = s.m11;
            this.m12 = s.m12;
            this.m13 = 0;
        }
        
        public imat2x4(imat4 s) {
            this.m00 = s.m00;
            this.m01 = s.m01;
            this.m02 = s.m02;
            this.m03 = s.m03;
            this.m10 = s.m10;
            this.m11 = s.m11;
            this.m12 = s.m12;
            this.m13 = s.m13;
        }
        
        public int this[int i0, int i1] {
            get {
                if (((i0 < 0) 
                            || (1 < i0))) {
                    throw new System.IndexOutOfRangeException("i0");
                }
                if (((i1 < 0) 
                            || (3 < i1))) {
                    throw new System.IndexOutOfRangeException("i1");
                }
                
                unsafe
                {
                    fixed (Int32* v_ptr = &this.m00)
                    {
                        return *(v_ptr + (i1 + i0 * 4));
                    }
                };
            }
            set {
                if (((i0 < 0) 
                            || (1 < i0))) {
                    throw new System.IndexOutOfRangeException("i0");
                }
                if (((i1 < 0) 
                            || (3 < i1))) {
                    throw new System.IndexOutOfRangeException("i1");
                }
                
                unsafe
                {
                    fixed (Int32* v_ptr = &this.m00)
                    {
                        *(v_ptr  + (i1 + i0 * 4)) = value;
                    }
                };
            }
        }
        
        public ivec4 this[int i0] {
            get {
                return new ivec4(this[i0, 0], this[i0, 1], this[i0, 2], this[i0, 3]);
            }
            set {
                this[i0, 0] = value[0];
                this[i0, 1] = value[1];
                this[i0, 2] = value[2];
                this[i0, 3] = value[3];
            }
        }
        
        public static imat2x4 operator+(imat2x4 v) {
            return new imat2x4(+v.m00, +v.m01, +v.m02, +v.m03, +v.m10, +v.m11, +v.m12, +v.m13);
        }
        
        public static imat2x4 operator-(imat2x4 v) {
            return new imat2x4(-v.m00, -v.m01, -v.m02, -v.m03, -v.m10, -v.m11, -v.m12, -v.m13);
        }
        
        public static imat2x4 operator~(imat2x4 v) {
            return new imat2x4(~v.m00, ~v.m01, ~v.m02, ~v.m03, ~v.m10, ~v.m11, ~v.m12, ~v.m13);
        }
        
        public static imat2x4 operator++(imat2x4 v) {
            return new imat2x4(++v.m00, ++v.m01, ++v.m02, ++v.m03, ++v.m10, ++v.m11, ++v.m12, ++v.m13);
        }
        
        public static imat2x4 operator--(imat2x4 v) {
            return new imat2x4(--v.m00, --v.m01, --v.m02, --v.m03, --v.m10, --v.m11, --v.m12, --v.m13);
        }
        
        public static imat2x4 operator+(int s, imat2x4 v2) {
            return new imat2x4(s + v2.m00, s + v2.m01, s + v2.m02, s + v2.m03, s + v2.m10, s + v2.m11, s + v2.m12, s + v2.m13);
        }
        
        public static imat2x4 operator-(int s, imat2x4 v2) {
            return new imat2x4(s - v2.m00, s - v2.m01, s - v2.m02, s - v2.m03, s - v2.m10, s - v2.m11, s - v2.m12, s - v2.m13);
        }
        
        public static imat2x4 operator*(int s, imat2x4 v2) {
            return new imat2x4(s * v2.m00, s * v2.m01, s * v2.m02, s * v2.m03, s * v2.m10, s * v2.m11, s * v2.m12, s * v2.m13);
        }
        
        public static imat2x4 operator/(int s, imat2x4 v2) {
            return new imat2x4(s / v2.m00, s / v2.m01, s / v2.m02, s / v2.m03, s / v2.m10, s / v2.m11, s / v2.m12, s / v2.m13);
        }
        
        public static imat2x4 operator%(int s, imat2x4 v2) {
            return new imat2x4(s % v2.m00, s % v2.m01, s % v2.m02, s % v2.m03, s % v2.m10, s % v2.m11, s % v2.m12, s % v2.m13);
        }
        
        public static imat2x4 operator&(int s, imat2x4 v2) {
            return new imat2x4(s & v2.m00, s & v2.m01, s & v2.m02, s & v2.m03, s & v2.m10, s & v2.m11, s & v2.m12, s & v2.m13);
        }
        
        public static imat2x4 operator|(int s, imat2x4 v2) {
            return new imat2x4(s | v2.m00, s | v2.m01, s | v2.m02, s | v2.m03, s | v2.m10, s | v2.m11, s | v2.m12, s | v2.m13);
        }
        
        public static imat2x4 operator^(int s, imat2x4 v2) {
            return new imat2x4(s ^ v2.m00, s ^ v2.m01, s ^ v2.m02, s ^ v2.m03, s ^ v2.m10, s ^ v2.m11, s ^ v2.m12, s ^ v2.m13);
        }
        
        public static imat2x4 operator+(imat2x4 v1, int s) {
            return new imat2x4(v1.m00 + s, v1.m01 + s, v1.m02 + s, v1.m03 + s, v1.m10 + s, v1.m11 + s, v1.m12 + s, v1.m13 + s);
        }
        
        public static imat2x4 operator-(imat2x4 v1, int s) {
            return new imat2x4(v1.m00 - s, v1.m01 - s, v1.m02 - s, v1.m03 - s, v1.m10 - s, v1.m11 - s, v1.m12 - s, v1.m13 - s);
        }
        
        public static imat2x4 operator*(imat2x4 v1, int s) {
            return new imat2x4(v1.m00 * s, v1.m01 * s, v1.m02 * s, v1.m03 * s, v1.m10 * s, v1.m11 * s, v1.m12 * s, v1.m13 * s);
        }
        
        public static imat2x4 operator/(imat2x4 v1, int s) {
            return new imat2x4(v1.m00 / s, v1.m01 / s, v1.m02 / s, v1.m03 / s, v1.m10 / s, v1.m11 / s, v1.m12 / s, v1.m13 / s);
        }
        
        public static imat2x4 operator%(imat2x4 v1, int s) {
            return new imat2x4(v1.m00 % s, v1.m01 % s, v1.m02 % s, v1.m03 % s, v1.m10 % s, v1.m11 % s, v1.m12 % s, v1.m13 % s);
        }
        
        public static imat2x4 operator&(imat2x4 v1, int s) {
            return new imat2x4(v1.m00 & s, v1.m01 & s, v1.m02 & s, v1.m03 & s, v1.m10 & s, v1.m11 & s, v1.m12 & s, v1.m13 & s);
        }
        
        public static imat2x4 operator|(imat2x4 v1, int s) {
            return new imat2x4(v1.m00 | s, v1.m01 | s, v1.m02 | s, v1.m03 | s, v1.m10 | s, v1.m11 | s, v1.m12 | s, v1.m13 | s);
        }
        
        public static imat2x4 operator^(imat2x4 v1, int s) {
            return new imat2x4(v1.m00 ^ s, v1.m01 ^ s, v1.m02 ^ s, v1.m03 ^ s, v1.m10 ^ s, v1.m11 ^ s, v1.m12 ^ s, v1.m13 ^ s);
        }
        
        public static imat2x4 operator+(imat2x4 v1, imat2x4 v2) {
            return new imat2x4(v1.m00 + v2.m00, v1.m01 + v2.m01, v1.m02 + v2.m02, v1.m03 + v2.m03, v1.m10 + v2.m10, v1.m11 + v2.m11, v1.m12 + v2.m12, v1.m13 + v2.m13);
        }
        
        public static imat2x4 operator-(imat2x4 v1, imat2x4 v2) {
            return new imat2x4(v1.m00 - v2.m00, v1.m01 - v2.m01, v1.m02 - v2.m02, v1.m03 - v2.m03, v1.m10 - v2.m10, v1.m11 - v2.m11, v1.m12 - v2.m12, v1.m13 - v2.m13);
        }
        
        public static imat2x4 operator/(imat2x4 v1, imat2x4 v2) {
            return new imat2x4(v1.m00 / v2.m00, v1.m01 / v2.m01, v1.m02 / v2.m02, v1.m03 / v2.m03, v1.m10 / v2.m10, v1.m11 / v2.m11, v1.m12 / v2.m12, v1.m13 / v2.m13);
        }
        
        public static imat2x4 operator%(imat2x4 v1, imat2x4 v2) {
            return new imat2x4(v1.m00 % v2.m00, v1.m01 % v2.m01, v1.m02 % v2.m02, v1.m03 % v2.m03, v1.m10 % v2.m10, v1.m11 % v2.m11, v1.m12 % v2.m12, v1.m13 % v2.m13);
        }
        
        public static imat2x4 operator&(imat2x4 v1, imat2x4 v2) {
            return new imat2x4(v1.m00 & v2.m00, v1.m01 & v2.m01, v1.m02 & v2.m02, v1.m03 & v2.m03, v1.m10 & v2.m10, v1.m11 & v2.m11, v1.m12 & v2.m12, v1.m13 & v2.m13);
        }
        
        public static imat2x4 operator|(imat2x4 v1, imat2x4 v2) {
            return new imat2x4(v1.m00 | v2.m00, v1.m01 | v2.m01, v1.m02 | v2.m02, v1.m03 | v2.m03, v1.m10 | v2.m10, v1.m11 | v2.m11, v1.m12 | v2.m12, v1.m13 | v2.m13);
        }
        
        public static imat2x4 operator^(imat2x4 v1, imat2x4 v2) {
            return new imat2x4(v1.m00 ^ v2.m00, v1.m01 ^ v2.m01, v1.m02 ^ v2.m02, v1.m03 ^ v2.m03, v1.m10 ^ v2.m10, v1.m11 ^ v2.m11, v1.m12 ^ v2.m12, v1.m13 ^ v2.m13);
        }
        
        public static bmat2x4 operator==(imat2x4 v1, imat2x4 v2) {
            return new bmat2x4(v1.m00 == v2.m00, v1.m01 == v2.m01, v1.m02 == v2.m02, v1.m03 == v2.m03, v1.m10 == v2.m10, v1.m11 == v2.m11, v1.m12 == v2.m12, v1.m13 == v2.m13);
        }
        
        public static bmat2x4 operator>(imat2x4 v1, imat2x4 v2) {
            return new bmat2x4(v1.m00 > v2.m00, v1.m01 > v2.m01, v1.m02 > v2.m02, v1.m03 > v2.m03, v1.m10 > v2.m10, v1.m11 > v2.m11, v1.m12 > v2.m12, v1.m13 > v2.m13);
        }
        
        public static bmat2x4 operator>=(imat2x4 v1, imat2x4 v2) {
            return new bmat2x4(v1.m00 >= v2.m00, v1.m01 >= v2.m01, v1.m02 >= v2.m02, v1.m03 >= v2.m03, v1.m10 >= v2.m10, v1.m11 >= v2.m11, v1.m12 >= v2.m12, v1.m13 >= v2.m13);
        }
        
        public static bmat2x4 operator<(imat2x4 v1, imat2x4 v2) {
            return new bmat2x4(v1.m00 < v2.m00, v1.m01 < v2.m01, v1.m02 < v2.m02, v1.m03 < v2.m03, v1.m10 < v2.m10, v1.m11 < v2.m11, v1.m12 < v2.m12, v1.m13 < v2.m13);
        }
        
        public static bmat2x4 operator<=(imat2x4 v1, imat2x4 v2) {
            return new bmat2x4(v1.m00 <= v2.m00, v1.m01 <= v2.m01, v1.m02 <= v2.m02, v1.m03 <= v2.m03, v1.m10 <= v2.m10, v1.m11 <= v2.m11, v1.m12 <= v2.m12, v1.m13 <= v2.m13);
        }
        
        public static bmat2x4 operator!=(imat2x4 v1, imat2x4 v2) {
            return new bmat2x4(v1.m00 != v2.m00, v1.m01 != v2.m01, v1.m02 != v2.m02, v1.m03 != v2.m03, v1.m10 != v2.m10, v1.m11 != v2.m11, v1.m12 != v2.m12, v1.m13 != v2.m13);
        }
        
        public static ivec4 operator*(ivec2 v0, imat2x4 m1) {
            return imath.InnerProduct1(ref v0, ref m1);
        }
        
        public static ivec2 operator*(imat2x4 m0, ivec4 v1) {
            return imath.InnerProduct1(ref m0, ref v1);
        }
        
        public static imat2 operator*(imat2x4 m0, imat4x2 m1) {
            return imath.InnerProduct1(ref m0, ref m1);
        }
        
        public static imat2x3 operator*(imat2x4 m0, imat4x3 m1) {
            return imath.InnerProduct1(ref m0, ref m1);
        }
        
        public static imat2x4 operator*(imat2x4 m0, imat4 m1) {
            return imath.InnerProduct1(ref m0, ref m1);
        }
        
        public override bool Equals(object obj) {
            return (obj is imat2x4) ? bmath.All(this == (imat2x4)obj) : false;
        }
        
        public override int GetHashCode() {
            return this.m00.GetHashCode() ^ this.m01.GetHashCode() ^ this.m02.GetHashCode() ^ this.m03.GetHashCode() ^ this.m10.GetHashCode() ^ this.m11.GetHashCode() ^ this.m12.GetHashCode() ^ this.m13.GetHashCode();
        }
        
        public override string ToString() {
            return "imat2x4(" + this.m00 + "," + this.m01 + "," + this.m02 + "," + this.m03 + "," + this.m10 + "," + this.m11 + "," + this.m12 + "," + this.m13 + ")";
        }
    }
    
    [DataContract()]
    [StructLayout(LayoutKind.Sequential)]
    public struct imat3x2 {
        
        [DataMember()]
        internal int m00;
        
        [DataMember()]
        internal int m01;
        
        [DataMember()]
        internal int m10;
        
        [DataMember()]
        internal int m11;
        
        [DataMember()]
        internal int m20;
        
        [DataMember()]
        internal int m21;
        
        public imat3x2(int s) {
            this.m00 = s;
            this.m01 = 0;
            this.m10 = 0;
            this.m11 = s;
            this.m20 = 0;
            this.m21 = 0;
        }
        
        public imat3x2(imat3x2 m00) {
            this.m00 = m00.m00;
            this.m01 = m00.m01;
            this.m10 = m00.m10;
            this.m11 = m00.m11;
            this.m20 = m00.m20;
            this.m21 = m00.m21;
        }
        
        public imat3x2(imat2 m00, ivec2 m10) {
            this.m00 = m00.m00;
            this.m01 = m00.m01;
            this.m10 = m00.m10;
            this.m11 = m00.m11;
            this.m20 = m10.v0;
            this.m21 = m10.v1;
        }
        
        public imat3x2(ivec2 m00, imat2 m10) {
            this.m00 = m00.v0;
            this.m01 = m00.v1;
            this.m10 = m10.m00;
            this.m11 = m10.m01;
            this.m20 = m10.m10;
            this.m21 = m10.m11;
        }
        
        public imat3x2(ivec2 m00, ivec2 m10, ivec2 m20) {
            this.m00 = m00.v0;
            this.m01 = m00.v1;
            this.m10 = m10.v0;
            this.m11 = m10.v1;
            this.m20 = m20.v0;
            this.m21 = m20.v1;
        }
        
        public imat3x2(int m00, int m01, int m10, int m11, int m20, int m21) {
            this.m00 = m00;
            this.m01 = m01;
            this.m10 = m10;
            this.m11 = m11;
            this.m20 = m20;
            this.m21 = m21;
        }
        
        public imat3x2(imat2 s) {
            this.m00 = s.m00;
            this.m01 = s.m01;
            this.m10 = s.m10;
            this.m11 = s.m11;
            this.m20 = 0;
            this.m21 = 0;
        }
        
        public imat3x2(imat2x3 s) {
            this.m00 = s.m00;
            this.m01 = s.m01;
            this.m10 = s.m10;
            this.m11 = s.m11;
            this.m20 = 0;
            this.m21 = 0;
        }
        
        public imat3x2(imat2x4 s) {
            this.m00 = s.m00;
            this.m01 = s.m01;
            this.m10 = s.m10;
            this.m11 = s.m11;
            this.m20 = 0;
            this.m21 = 0;
        }
        
        public imat3x2(imat3 s) {
            this.m00 = s.m00;
            this.m01 = s.m01;
            this.m10 = s.m10;
            this.m11 = s.m11;
            this.m20 = s.m20;
            this.m21 = s.m21;
        }
        
        public imat3x2(imat3x4 s) {
            this.m00 = s.m00;
            this.m01 = s.m01;
            this.m10 = s.m10;
            this.m11 = s.m11;
            this.m20 = s.m20;
            this.m21 = s.m21;
        }
        
        public imat3x2(imat4x2 s) {
            this.m00 = s.m00;
            this.m01 = s.m01;
            this.m10 = s.m10;
            this.m11 = s.m11;
            this.m20 = s.m20;
            this.m21 = s.m21;
        }
        
        public imat3x2(imat4x3 s) {
            this.m00 = s.m00;
            this.m01 = s.m01;
            this.m10 = s.m10;
            this.m11 = s.m11;
            this.m20 = s.m20;
            this.m21 = s.m21;
        }
        
        public imat3x2(imat4 s) {
            this.m00 = s.m00;
            this.m01 = s.m01;
            this.m10 = s.m10;
            this.m11 = s.m11;
            this.m20 = s.m20;
            this.m21 = s.m21;
        }
        
        public int this[int i0, int i1] {
            get {
                if (((i0 < 0) 
                            || (2 < i0))) {
                    throw new System.IndexOutOfRangeException("i0");
                }
                if (((i1 < 0) 
                            || (1 < i1))) {
                    throw new System.IndexOutOfRangeException("i1");
                }
                
                unsafe
                {
                    fixed (Int32* v_ptr = &this.m00)
                    {
                        return *(v_ptr + (i1 + i0 * 2));
                    }
                };
            }
            set {
                if (((i0 < 0) 
                            || (2 < i0))) {
                    throw new System.IndexOutOfRangeException("i0");
                }
                if (((i1 < 0) 
                            || (1 < i1))) {
                    throw new System.IndexOutOfRangeException("i1");
                }
                
                unsafe
                {
                    fixed (Int32* v_ptr = &this.m00)
                    {
                        *(v_ptr  + (i1 + i0 * 2)) = value;
                    }
                };
            }
        }
        
        public ivec2 this[int i0] {
            get {
                return new ivec2(this[i0, 0], this[i0, 1]);
            }
            set {
                this[i0, 0] = value[0];
                this[i0, 1] = value[1];
            }
        }
        
        public static imat3x2 operator+(imat3x2 v) {
            return new imat3x2(+v.m00, +v.m01, +v.m10, +v.m11, +v.m20, +v.m21);
        }
        
        public static imat3x2 operator-(imat3x2 v) {
            return new imat3x2(-v.m00, -v.m01, -v.m10, -v.m11, -v.m20, -v.m21);
        }
        
        public static imat3x2 operator~(imat3x2 v) {
            return new imat3x2(~v.m00, ~v.m01, ~v.m10, ~v.m11, ~v.m20, ~v.m21);
        }
        
        public static imat3x2 operator++(imat3x2 v) {
            return new imat3x2(++v.m00, ++v.m01, ++v.m10, ++v.m11, ++v.m20, ++v.m21);
        }
        
        public static imat3x2 operator--(imat3x2 v) {
            return new imat3x2(--v.m00, --v.m01, --v.m10, --v.m11, --v.m20, --v.m21);
        }
        
        public static imat3x2 operator+(int s, imat3x2 v2) {
            return new imat3x2(s + v2.m00, s + v2.m01, s + v2.m10, s + v2.m11, s + v2.m20, s + v2.m21);
        }
        
        public static imat3x2 operator-(int s, imat3x2 v2) {
            return new imat3x2(s - v2.m00, s - v2.m01, s - v2.m10, s - v2.m11, s - v2.m20, s - v2.m21);
        }
        
        public static imat3x2 operator*(int s, imat3x2 v2) {
            return new imat3x2(s * v2.m00, s * v2.m01, s * v2.m10, s * v2.m11, s * v2.m20, s * v2.m21);
        }
        
        public static imat3x2 operator/(int s, imat3x2 v2) {
            return new imat3x2(s / v2.m00, s / v2.m01, s / v2.m10, s / v2.m11, s / v2.m20, s / v2.m21);
        }
        
        public static imat3x2 operator%(int s, imat3x2 v2) {
            return new imat3x2(s % v2.m00, s % v2.m01, s % v2.m10, s % v2.m11, s % v2.m20, s % v2.m21);
        }
        
        public static imat3x2 operator&(int s, imat3x2 v2) {
            return new imat3x2(s & v2.m00, s & v2.m01, s & v2.m10, s & v2.m11, s & v2.m20, s & v2.m21);
        }
        
        public static imat3x2 operator|(int s, imat3x2 v2) {
            return new imat3x2(s | v2.m00, s | v2.m01, s | v2.m10, s | v2.m11, s | v2.m20, s | v2.m21);
        }
        
        public static imat3x2 operator^(int s, imat3x2 v2) {
            return new imat3x2(s ^ v2.m00, s ^ v2.m01, s ^ v2.m10, s ^ v2.m11, s ^ v2.m20, s ^ v2.m21);
        }
        
        public static imat3x2 operator+(imat3x2 v1, int s) {
            return new imat3x2(v1.m00 + s, v1.m01 + s, v1.m10 + s, v1.m11 + s, v1.m20 + s, v1.m21 + s);
        }
        
        public static imat3x2 operator-(imat3x2 v1, int s) {
            return new imat3x2(v1.m00 - s, v1.m01 - s, v1.m10 - s, v1.m11 - s, v1.m20 - s, v1.m21 - s);
        }
        
        public static imat3x2 operator*(imat3x2 v1, int s) {
            return new imat3x2(v1.m00 * s, v1.m01 * s, v1.m10 * s, v1.m11 * s, v1.m20 * s, v1.m21 * s);
        }
        
        public static imat3x2 operator/(imat3x2 v1, int s) {
            return new imat3x2(v1.m00 / s, v1.m01 / s, v1.m10 / s, v1.m11 / s, v1.m20 / s, v1.m21 / s);
        }
        
        public static imat3x2 operator%(imat3x2 v1, int s) {
            return new imat3x2(v1.m00 % s, v1.m01 % s, v1.m10 % s, v1.m11 % s, v1.m20 % s, v1.m21 % s);
        }
        
        public static imat3x2 operator&(imat3x2 v1, int s) {
            return new imat3x2(v1.m00 & s, v1.m01 & s, v1.m10 & s, v1.m11 & s, v1.m20 & s, v1.m21 & s);
        }
        
        public static imat3x2 operator|(imat3x2 v1, int s) {
            return new imat3x2(v1.m00 | s, v1.m01 | s, v1.m10 | s, v1.m11 | s, v1.m20 | s, v1.m21 | s);
        }
        
        public static imat3x2 operator^(imat3x2 v1, int s) {
            return new imat3x2(v1.m00 ^ s, v1.m01 ^ s, v1.m10 ^ s, v1.m11 ^ s, v1.m20 ^ s, v1.m21 ^ s);
        }
        
        public static imat3x2 operator+(imat3x2 v1, imat3x2 v2) {
            return new imat3x2(v1.m00 + v2.m00, v1.m01 + v2.m01, v1.m10 + v2.m10, v1.m11 + v2.m11, v1.m20 + v2.m20, v1.m21 + v2.m21);
        }
        
        public static imat3x2 operator-(imat3x2 v1, imat3x2 v2) {
            return new imat3x2(v1.m00 - v2.m00, v1.m01 - v2.m01, v1.m10 - v2.m10, v1.m11 - v2.m11, v1.m20 - v2.m20, v1.m21 - v2.m21);
        }
        
        public static imat3x2 operator/(imat3x2 v1, imat3x2 v2) {
            return new imat3x2(v1.m00 / v2.m00, v1.m01 / v2.m01, v1.m10 / v2.m10, v1.m11 / v2.m11, v1.m20 / v2.m20, v1.m21 / v2.m21);
        }
        
        public static imat3x2 operator%(imat3x2 v1, imat3x2 v2) {
            return new imat3x2(v1.m00 % v2.m00, v1.m01 % v2.m01, v1.m10 % v2.m10, v1.m11 % v2.m11, v1.m20 % v2.m20, v1.m21 % v2.m21);
        }
        
        public static imat3x2 operator&(imat3x2 v1, imat3x2 v2) {
            return new imat3x2(v1.m00 & v2.m00, v1.m01 & v2.m01, v1.m10 & v2.m10, v1.m11 & v2.m11, v1.m20 & v2.m20, v1.m21 & v2.m21);
        }
        
        public static imat3x2 operator|(imat3x2 v1, imat3x2 v2) {
            return new imat3x2(v1.m00 | v2.m00, v1.m01 | v2.m01, v1.m10 | v2.m10, v1.m11 | v2.m11, v1.m20 | v2.m20, v1.m21 | v2.m21);
        }
        
        public static imat3x2 operator^(imat3x2 v1, imat3x2 v2) {
            return new imat3x2(v1.m00 ^ v2.m00, v1.m01 ^ v2.m01, v1.m10 ^ v2.m10, v1.m11 ^ v2.m11, v1.m20 ^ v2.m20, v1.m21 ^ v2.m21);
        }
        
        public static bmat3x2 operator==(imat3x2 v1, imat3x2 v2) {
            return new bmat3x2(v1.m00 == v2.m00, v1.m01 == v2.m01, v1.m10 == v2.m10, v1.m11 == v2.m11, v1.m20 == v2.m20, v1.m21 == v2.m21);
        }
        
        public static bmat3x2 operator>(imat3x2 v1, imat3x2 v2) {
            return new bmat3x2(v1.m00 > v2.m00, v1.m01 > v2.m01, v1.m10 > v2.m10, v1.m11 > v2.m11, v1.m20 > v2.m20, v1.m21 > v2.m21);
        }
        
        public static bmat3x2 operator>=(imat3x2 v1, imat3x2 v2) {
            return new bmat3x2(v1.m00 >= v2.m00, v1.m01 >= v2.m01, v1.m10 >= v2.m10, v1.m11 >= v2.m11, v1.m20 >= v2.m20, v1.m21 >= v2.m21);
        }
        
        public static bmat3x2 operator<(imat3x2 v1, imat3x2 v2) {
            return new bmat3x2(v1.m00 < v2.m00, v1.m01 < v2.m01, v1.m10 < v2.m10, v1.m11 < v2.m11, v1.m20 < v2.m20, v1.m21 < v2.m21);
        }
        
        public static bmat3x2 operator<=(imat3x2 v1, imat3x2 v2) {
            return new bmat3x2(v1.m00 <= v2.m00, v1.m01 <= v2.m01, v1.m10 <= v2.m10, v1.m11 <= v2.m11, v1.m20 <= v2.m20, v1.m21 <= v2.m21);
        }
        
        public static bmat3x2 operator!=(imat3x2 v1, imat3x2 v2) {
            return new bmat3x2(v1.m00 != v2.m00, v1.m01 != v2.m01, v1.m10 != v2.m10, v1.m11 != v2.m11, v1.m20 != v2.m20, v1.m21 != v2.m21);
        }
        
        public static ivec2 operator*(ivec3 v0, imat3x2 m1) {
            return imath.InnerProduct1(ref v0, ref m1);
        }
        
        public static ivec3 operator*(imat3x2 m0, ivec2 v1) {
            return imath.InnerProduct1(ref m0, ref v1);
        }
        
        public static imat3x2 operator*(imat3x2 m0, imat2 m1) {
            return imath.InnerProduct1(ref m0, ref m1);
        }
        
        public static imat3 operator*(imat3x2 m0, imat2x3 m1) {
            return imath.InnerProduct1(ref m0, ref m1);
        }
        
        public static imat3x4 operator*(imat3x2 m0, imat2x4 m1) {
            return imath.InnerProduct1(ref m0, ref m1);
        }
        
        public override bool Equals(object obj) {
            return (obj is imat3x2) ? bmath.All(this == (imat3x2)obj) : false;
        }
        
        public override int GetHashCode() {
            return this.m00.GetHashCode() ^ this.m01.GetHashCode() ^ this.m10.GetHashCode() ^ this.m11.GetHashCode() ^ this.m20.GetHashCode() ^ this.m21.GetHashCode();
        }
        
        public override string ToString() {
            return "imat3x2(" + this.m00 + "," + this.m01 + "," + this.m10 + "," + this.m11 + "," + this.m20 + "," + this.m21 + ")";
        }
    }
    
    [DataContract()]
    [StructLayout(LayoutKind.Sequential)]
    public struct imat3 {
        
        [DataMember()]
        internal int m00;
        
        [DataMember()]
        internal int m01;
        
        [DataMember()]
        internal int m02;
        
        [DataMember()]
        internal int m10;
        
        [DataMember()]
        internal int m11;
        
        [DataMember()]
        internal int m12;
        
        [DataMember()]
        internal int m20;
        
        [DataMember()]
        internal int m21;
        
        [DataMember()]
        internal int m22;
        
        public imat3(int s) {
            this.m00 = s;
            this.m01 = 0;
            this.m02 = 0;
            this.m10 = 0;
            this.m11 = s;
            this.m12 = 0;
            this.m20 = 0;
            this.m21 = 0;
            this.m22 = s;
        }
        
        public imat3(imat3 m00) {
            this.m00 = m00.m00;
            this.m01 = m00.m01;
            this.m02 = m00.m02;
            this.m10 = m00.m10;
            this.m11 = m00.m11;
            this.m12 = m00.m12;
            this.m20 = m00.m20;
            this.m21 = m00.m21;
            this.m22 = m00.m22;
        }
        
        public imat3(imat2x3 m00, ivec3 m10) {
            this.m00 = m00.m00;
            this.m01 = m00.m01;
            this.m02 = m00.m02;
            this.m10 = m00.m10;
            this.m11 = m00.m11;
            this.m12 = m00.m12;
            this.m20 = m10.v0;
            this.m21 = m10.v1;
            this.m22 = m10.v2;
        }
        
        public imat3(ivec3 m00, imat2x3 m10) {
            this.m00 = m00.v0;
            this.m01 = m00.v1;
            this.m02 = m00.v2;
            this.m10 = m10.m00;
            this.m11 = m10.m01;
            this.m12 = m10.m02;
            this.m20 = m10.m10;
            this.m21 = m10.m11;
            this.m22 = m10.m12;
        }
        
        public imat3(ivec3 m00, ivec3 m10, ivec3 m20) {
            this.m00 = m00.v0;
            this.m01 = m00.v1;
            this.m02 = m00.v2;
            this.m10 = m10.v0;
            this.m11 = m10.v1;
            this.m12 = m10.v2;
            this.m20 = m20.v0;
            this.m21 = m20.v1;
            this.m22 = m20.v2;
        }
        
        public imat3(ivec2 m00, int m01, ivec2 m10, int m11, ivec2 m20, int m21) {
            this.m00 = m00.v0;
            this.m01 = m00.v1;
            this.m02 = m01;
            this.m10 = m10.v0;
            this.m11 = m10.v1;
            this.m12 = m11;
            this.m20 = m20.v0;
            this.m21 = m20.v1;
            this.m22 = m21;
        }
        
        public imat3(int m00, ivec2 m01, int m10, ivec2 m11, int m20, ivec2 m21) {
            this.m00 = m00;
            this.m01 = m01.v0;
            this.m02 = m01.v1;
            this.m10 = m10;
            this.m11 = m11.v0;
            this.m12 = m11.v1;
            this.m20 = m20;
            this.m21 = m21.v0;
            this.m22 = m21.v1;
        }
        
        public imat3(int m00, int m01, int m02, int m10, int m11, int m12, int m20, int m21, int m22) {
            this.m00 = m00;
            this.m01 = m01;
            this.m02 = m02;
            this.m10 = m10;
            this.m11 = m11;
            this.m12 = m12;
            this.m20 = m20;
            this.m21 = m21;
            this.m22 = m22;
        }
        
        public imat3(imat2 s) {
            this.m00 = s.m00;
            this.m01 = s.m01;
            this.m02 = 0;
            this.m10 = s.m10;
            this.m11 = s.m11;
            this.m12 = 0;
            this.m20 = 0;
            this.m21 = 0;
            this.m22 = 0;
        }
        
        public imat3(imat2x3 s) {
            this.m00 = s.m00;
            this.m01 = s.m01;
            this.m02 = s.m02;
            this.m10 = s.m10;
            this.m11 = s.m11;
            this.m12 = s.m12;
            this.m20 = 0;
            this.m21 = 0;
            this.m22 = 0;
        }
        
        public imat3(imat2x4 s) {
            this.m00 = s.m00;
            this.m01 = s.m01;
            this.m02 = s.m02;
            this.m10 = s.m10;
            this.m11 = s.m11;
            this.m12 = s.m12;
            this.m20 = 0;
            this.m21 = 0;
            this.m22 = 0;
        }
        
        public imat3(imat3x2 s) {
            this.m00 = s.m00;
            this.m01 = s.m01;
            this.m02 = 0;
            this.m10 = s.m10;
            this.m11 = s.m11;
            this.m12 = 0;
            this.m20 = s.m20;
            this.m21 = s.m21;
            this.m22 = 0;
        }
        
        public imat3(imat3x4 s) {
            this.m00 = s.m00;
            this.m01 = s.m01;
            this.m02 = s.m02;
            this.m10 = s.m10;
            this.m11 = s.m11;
            this.m12 = s.m12;
            this.m20 = s.m20;
            this.m21 = s.m21;
            this.m22 = s.m22;
        }
        
        public imat3(imat4x2 s) {
            this.m00 = s.m00;
            this.m01 = s.m01;
            this.m02 = 0;
            this.m10 = s.m10;
            this.m11 = s.m11;
            this.m12 = 0;
            this.m20 = s.m20;
            this.m21 = s.m21;
            this.m22 = 0;
        }
        
        public imat3(imat4x3 s) {
            this.m00 = s.m00;
            this.m01 = s.m01;
            this.m02 = s.m02;
            this.m10 = s.m10;
            this.m11 = s.m11;
            this.m12 = s.m12;
            this.m20 = s.m20;
            this.m21 = s.m21;
            this.m22 = s.m22;
        }
        
        public imat3(imat4 s) {
            this.m00 = s.m00;
            this.m01 = s.m01;
            this.m02 = s.m02;
            this.m10 = s.m10;
            this.m11 = s.m11;
            this.m12 = s.m12;
            this.m20 = s.m20;
            this.m21 = s.m21;
            this.m22 = s.m22;
        }
        
        public int this[int i0, int i1] {
            get {
                if (((i0 < 0) 
                            || (2 < i0))) {
                    throw new System.IndexOutOfRangeException("i0");
                }
                if (((i1 < 0) 
                            || (2 < i1))) {
                    throw new System.IndexOutOfRangeException("i1");
                }
                
                unsafe
                {
                    fixed (Int32* v_ptr = &this.m00)
                    {
                        return *(v_ptr + (i1 + i0 * 3));
                    }
                };
            }
            set {
                if (((i0 < 0) 
                            || (2 < i0))) {
                    throw new System.IndexOutOfRangeException("i0");
                }
                if (((i1 < 0) 
                            || (2 < i1))) {
                    throw new System.IndexOutOfRangeException("i1");
                }
                
                unsafe
                {
                    fixed (Int32* v_ptr = &this.m00)
                    {
                        *(v_ptr  + (i1 + i0 * 3)) = value;
                    }
                };
            }
        }
        
        public ivec3 this[int i0] {
            get {
                return new ivec3(this[i0, 0], this[i0, 1], this[i0, 2]);
            }
            set {
                this[i0, 0] = value[0];
                this[i0, 1] = value[1];
                this[i0, 2] = value[2];
            }
        }
        
        public static imat3 operator+(imat3 v) {
            return new imat3(+v.m00, +v.m01, +v.m02, +v.m10, +v.m11, +v.m12, +v.m20, +v.m21, +v.m22);
        }
        
        public static imat3 operator-(imat3 v) {
            return new imat3(-v.m00, -v.m01, -v.m02, -v.m10, -v.m11, -v.m12, -v.m20, -v.m21, -v.m22);
        }
        
        public static imat3 operator~(imat3 v) {
            return new imat3(~v.m00, ~v.m01, ~v.m02, ~v.m10, ~v.m11, ~v.m12, ~v.m20, ~v.m21, ~v.m22);
        }
        
        public static imat3 operator++(imat3 v) {
            return new imat3(++v.m00, ++v.m01, ++v.m02, ++v.m10, ++v.m11, ++v.m12, ++v.m20, ++v.m21, ++v.m22);
        }
        
        public static imat3 operator--(imat3 v) {
            return new imat3(--v.m00, --v.m01, --v.m02, --v.m10, --v.m11, --v.m12, --v.m20, --v.m21, --v.m22);
        }
        
        public static imat3 operator+(int s, imat3 v2) {
            return new imat3(s + v2.m00, s + v2.m01, s + v2.m02, s + v2.m10, s + v2.m11, s + v2.m12, s + v2.m20, s + v2.m21, s + v2.m22);
        }
        
        public static imat3 operator-(int s, imat3 v2) {
            return new imat3(s - v2.m00, s - v2.m01, s - v2.m02, s - v2.m10, s - v2.m11, s - v2.m12, s - v2.m20, s - v2.m21, s - v2.m22);
        }
        
        public static imat3 operator*(int s, imat3 v2) {
            return new imat3(s * v2.m00, s * v2.m01, s * v2.m02, s * v2.m10, s * v2.m11, s * v2.m12, s * v2.m20, s * v2.m21, s * v2.m22);
        }
        
        public static imat3 operator/(int s, imat3 v2) {
            return new imat3(s / v2.m00, s / v2.m01, s / v2.m02, s / v2.m10, s / v2.m11, s / v2.m12, s / v2.m20, s / v2.m21, s / v2.m22);
        }
        
        public static imat3 operator%(int s, imat3 v2) {
            return new imat3(s % v2.m00, s % v2.m01, s % v2.m02, s % v2.m10, s % v2.m11, s % v2.m12, s % v2.m20, s % v2.m21, s % v2.m22);
        }
        
        public static imat3 operator&(int s, imat3 v2) {
            return new imat3(s & v2.m00, s & v2.m01, s & v2.m02, s & v2.m10, s & v2.m11, s & v2.m12, s & v2.m20, s & v2.m21, s & v2.m22);
        }
        
        public static imat3 operator|(int s, imat3 v2) {
            return new imat3(s | v2.m00, s | v2.m01, s | v2.m02, s | v2.m10, s | v2.m11, s | v2.m12, s | v2.m20, s | v2.m21, s | v2.m22);
        }
        
        public static imat3 operator^(int s, imat3 v2) {
            return new imat3(s ^ v2.m00, s ^ v2.m01, s ^ v2.m02, s ^ v2.m10, s ^ v2.m11, s ^ v2.m12, s ^ v2.m20, s ^ v2.m21, s ^ v2.m22);
        }
        
        public static imat3 operator+(imat3 v1, int s) {
            return new imat3(v1.m00 + s, v1.m01 + s, v1.m02 + s, v1.m10 + s, v1.m11 + s, v1.m12 + s, v1.m20 + s, v1.m21 + s, v1.m22 + s);
        }
        
        public static imat3 operator-(imat3 v1, int s) {
            return new imat3(v1.m00 - s, v1.m01 - s, v1.m02 - s, v1.m10 - s, v1.m11 - s, v1.m12 - s, v1.m20 - s, v1.m21 - s, v1.m22 - s);
        }
        
        public static imat3 operator*(imat3 v1, int s) {
            return new imat3(v1.m00 * s, v1.m01 * s, v1.m02 * s, v1.m10 * s, v1.m11 * s, v1.m12 * s, v1.m20 * s, v1.m21 * s, v1.m22 * s);
        }
        
        public static imat3 operator/(imat3 v1, int s) {
            return new imat3(v1.m00 / s, v1.m01 / s, v1.m02 / s, v1.m10 / s, v1.m11 / s, v1.m12 / s, v1.m20 / s, v1.m21 / s, v1.m22 / s);
        }
        
        public static imat3 operator%(imat3 v1, int s) {
            return new imat3(v1.m00 % s, v1.m01 % s, v1.m02 % s, v1.m10 % s, v1.m11 % s, v1.m12 % s, v1.m20 % s, v1.m21 % s, v1.m22 % s);
        }
        
        public static imat3 operator&(imat3 v1, int s) {
            return new imat3(v1.m00 & s, v1.m01 & s, v1.m02 & s, v1.m10 & s, v1.m11 & s, v1.m12 & s, v1.m20 & s, v1.m21 & s, v1.m22 & s);
        }
        
        public static imat3 operator|(imat3 v1, int s) {
            return new imat3(v1.m00 | s, v1.m01 | s, v1.m02 | s, v1.m10 | s, v1.m11 | s, v1.m12 | s, v1.m20 | s, v1.m21 | s, v1.m22 | s);
        }
        
        public static imat3 operator^(imat3 v1, int s) {
            return new imat3(v1.m00 ^ s, v1.m01 ^ s, v1.m02 ^ s, v1.m10 ^ s, v1.m11 ^ s, v1.m12 ^ s, v1.m20 ^ s, v1.m21 ^ s, v1.m22 ^ s);
        }
        
        public static imat3 operator+(imat3 v1, imat3 v2) {
            return new imat3(v1.m00 + v2.m00, v1.m01 + v2.m01, v1.m02 + v2.m02, v1.m10 + v2.m10, v1.m11 + v2.m11, v1.m12 + v2.m12, v1.m20 + v2.m20, v1.m21 + v2.m21, v1.m22 + v2.m22);
        }
        
        public static imat3 operator-(imat3 v1, imat3 v2) {
            return new imat3(v1.m00 - v2.m00, v1.m01 - v2.m01, v1.m02 - v2.m02, v1.m10 - v2.m10, v1.m11 - v2.m11, v1.m12 - v2.m12, v1.m20 - v2.m20, v1.m21 - v2.m21, v1.m22 - v2.m22);
        }
        
        public static imat3 operator/(imat3 v1, imat3 v2) {
            return new imat3(v1.m00 / v2.m00, v1.m01 / v2.m01, v1.m02 / v2.m02, v1.m10 / v2.m10, v1.m11 / v2.m11, v1.m12 / v2.m12, v1.m20 / v2.m20, v1.m21 / v2.m21, v1.m22 / v2.m22);
        }
        
        public static imat3 operator%(imat3 v1, imat3 v2) {
            return new imat3(v1.m00 % v2.m00, v1.m01 % v2.m01, v1.m02 % v2.m02, v1.m10 % v2.m10, v1.m11 % v2.m11, v1.m12 % v2.m12, v1.m20 % v2.m20, v1.m21 % v2.m21, v1.m22 % v2.m22);
        }
        
        public static imat3 operator&(imat3 v1, imat3 v2) {
            return new imat3(v1.m00 & v2.m00, v1.m01 & v2.m01, v1.m02 & v2.m02, v1.m10 & v2.m10, v1.m11 & v2.m11, v1.m12 & v2.m12, v1.m20 & v2.m20, v1.m21 & v2.m21, v1.m22 & v2.m22);
        }
        
        public static imat3 operator|(imat3 v1, imat3 v2) {
            return new imat3(v1.m00 | v2.m00, v1.m01 | v2.m01, v1.m02 | v2.m02, v1.m10 | v2.m10, v1.m11 | v2.m11, v1.m12 | v2.m12, v1.m20 | v2.m20, v1.m21 | v2.m21, v1.m22 | v2.m22);
        }
        
        public static imat3 operator^(imat3 v1, imat3 v2) {
            return new imat3(v1.m00 ^ v2.m00, v1.m01 ^ v2.m01, v1.m02 ^ v2.m02, v1.m10 ^ v2.m10, v1.m11 ^ v2.m11, v1.m12 ^ v2.m12, v1.m20 ^ v2.m20, v1.m21 ^ v2.m21, v1.m22 ^ v2.m22);
        }
        
        public static bmat3 operator==(imat3 v1, imat3 v2) {
            return new bmat3(v1.m00 == v2.m00, v1.m01 == v2.m01, v1.m02 == v2.m02, v1.m10 == v2.m10, v1.m11 == v2.m11, v1.m12 == v2.m12, v1.m20 == v2.m20, v1.m21 == v2.m21, v1.m22 == v2.m22);
        }
        
        public static bmat3 operator>(imat3 v1, imat3 v2) {
            return new bmat3(v1.m00 > v2.m00, v1.m01 > v2.m01, v1.m02 > v2.m02, v1.m10 > v2.m10, v1.m11 > v2.m11, v1.m12 > v2.m12, v1.m20 > v2.m20, v1.m21 > v2.m21, v1.m22 > v2.m22);
        }
        
        public static bmat3 operator>=(imat3 v1, imat3 v2) {
            return new bmat3(v1.m00 >= v2.m00, v1.m01 >= v2.m01, v1.m02 >= v2.m02, v1.m10 >= v2.m10, v1.m11 >= v2.m11, v1.m12 >= v2.m12, v1.m20 >= v2.m20, v1.m21 >= v2.m21, v1.m22 >= v2.m22);
        }
        
        public static bmat3 operator<(imat3 v1, imat3 v2) {
            return new bmat3(v1.m00 < v2.m00, v1.m01 < v2.m01, v1.m02 < v2.m02, v1.m10 < v2.m10, v1.m11 < v2.m11, v1.m12 < v2.m12, v1.m20 < v2.m20, v1.m21 < v2.m21, v1.m22 < v2.m22);
        }
        
        public static bmat3 operator<=(imat3 v1, imat3 v2) {
            return new bmat3(v1.m00 <= v2.m00, v1.m01 <= v2.m01, v1.m02 <= v2.m02, v1.m10 <= v2.m10, v1.m11 <= v2.m11, v1.m12 <= v2.m12, v1.m20 <= v2.m20, v1.m21 <= v2.m21, v1.m22 <= v2.m22);
        }
        
        public static bmat3 operator!=(imat3 v1, imat3 v2) {
            return new bmat3(v1.m00 != v2.m00, v1.m01 != v2.m01, v1.m02 != v2.m02, v1.m10 != v2.m10, v1.m11 != v2.m11, v1.m12 != v2.m12, v1.m20 != v2.m20, v1.m21 != v2.m21, v1.m22 != v2.m22);
        }
        
        public static ivec3 operator*(ivec3 v0, imat3 m1) {
            return imath.InnerProduct1(ref v0, ref m1);
        }
        
        public static ivec3 operator*(imat3 m0, ivec3 v1) {
            return imath.InnerProduct1(ref m0, ref v1);
        }
        
        public static imat3x2 operator*(imat3 m0, imat3x2 m1) {
            return imath.InnerProduct1(ref m0, ref m1);
        }
        
        public static imat3 operator*(imat3 m0, imat3 m1) {
            return imath.InnerProduct1(ref m0, ref m1);
        }
        
        public static imat3x4 operator*(imat3 m0, imat3x4 m1) {
            return imath.InnerProduct1(ref m0, ref m1);
        }
        
        public override bool Equals(object obj) {
            return (obj is imat3) ? bmath.All(this == (imat3)obj) : false;
        }
        
        public override int GetHashCode() {
            return this.m00.GetHashCode() ^ this.m01.GetHashCode() ^ this.m02.GetHashCode() ^ this.m10.GetHashCode() ^ this.m11.GetHashCode() ^ this.m12.GetHashCode() ^ this.m20.GetHashCode() ^ this.m21.GetHashCode() ^ this.m22.GetHashCode();
        }
        
        public override string ToString() {
            return "imat3(" + this.m00 + "," + this.m01 + "," + this.m02 + "," + this.m10 + "," + this.m11 + "," + this.m12 + "," + this.m20 + "," + this.m21 + "," + this.m22 + ")";
        }
    }
    
    [DataContract()]
    [StructLayout(LayoutKind.Sequential)]
    public struct imat3x4 {
        
        [DataMember()]
        internal int m00;
        
        [DataMember()]
        internal int m01;
        
        [DataMember()]
        internal int m02;
        
        [DataMember()]
        internal int m03;
        
        [DataMember()]
        internal int m10;
        
        [DataMember()]
        internal int m11;
        
        [DataMember()]
        internal int m12;
        
        [DataMember()]
        internal int m13;
        
        [DataMember()]
        internal int m20;
        
        [DataMember()]
        internal int m21;
        
        [DataMember()]
        internal int m22;
        
        [DataMember()]
        internal int m23;
        
        public imat3x4(int s) {
            this.m00 = s;
            this.m01 = 0;
            this.m02 = 0;
            this.m03 = 0;
            this.m10 = 0;
            this.m11 = s;
            this.m12 = 0;
            this.m13 = 0;
            this.m20 = 0;
            this.m21 = 0;
            this.m22 = s;
            this.m23 = 0;
        }
        
        public imat3x4(imat3x4 m00) {
            this.m00 = m00.m00;
            this.m01 = m00.m01;
            this.m02 = m00.m02;
            this.m03 = m00.m03;
            this.m10 = m00.m10;
            this.m11 = m00.m11;
            this.m12 = m00.m12;
            this.m13 = m00.m13;
            this.m20 = m00.m20;
            this.m21 = m00.m21;
            this.m22 = m00.m22;
            this.m23 = m00.m23;
        }
        
        public imat3x4(imat3x2 m00, imat3x2 m01) {
            this.m00 = m00.m00;
            this.m01 = m00.m01;
            this.m02 = m01.m00;
            this.m03 = m01.m01;
            this.m10 = m00.m10;
            this.m11 = m00.m11;
            this.m12 = m01.m10;
            this.m13 = m01.m11;
            this.m20 = m00.m20;
            this.m21 = m00.m21;
            this.m22 = m01.m20;
            this.m23 = m01.m21;
        }
        
        public imat3x4(imat2x4 m00, ivec4 m10) {
            this.m00 = m00.m00;
            this.m01 = m00.m01;
            this.m02 = m00.m02;
            this.m03 = m00.m03;
            this.m10 = m00.m10;
            this.m11 = m00.m11;
            this.m12 = m00.m12;
            this.m13 = m00.m13;
            this.m20 = m10.v0;
            this.m21 = m10.v1;
            this.m22 = m10.v2;
            this.m23 = m10.v3;
        }
        
        public imat3x4(imat2 m00, imat2 m01, ivec2 m10, ivec2 m11) {
            this.m00 = m00.m00;
            this.m01 = m00.m01;
            this.m02 = m01.m00;
            this.m03 = m01.m01;
            this.m10 = m00.m10;
            this.m11 = m00.m11;
            this.m12 = m01.m10;
            this.m13 = m01.m11;
            this.m20 = m10.v0;
            this.m21 = m10.v1;
            this.m22 = m11.v0;
            this.m23 = m11.v1;
        }
        
        public imat3x4(ivec4 m00, imat2x4 m10) {
            this.m00 = m00.v0;
            this.m01 = m00.v1;
            this.m02 = m00.v2;
            this.m03 = m00.v3;
            this.m10 = m10.m00;
            this.m11 = m10.m01;
            this.m12 = m10.m02;
            this.m13 = m10.m03;
            this.m20 = m10.m10;
            this.m21 = m10.m11;
            this.m22 = m10.m12;
            this.m23 = m10.m13;
        }
        
        public imat3x4(ivec2 m00, ivec2 m01, imat2 m10, imat2 m11) {
            this.m00 = m00.v0;
            this.m01 = m00.v1;
            this.m02 = m01.v0;
            this.m03 = m01.v1;
            this.m10 = m10.m00;
            this.m11 = m10.m01;
            this.m12 = m11.m00;
            this.m13 = m11.m01;
            this.m20 = m10.m10;
            this.m21 = m10.m11;
            this.m22 = m11.m10;
            this.m23 = m11.m11;
        }
        
        public imat3x4(ivec4 m00, ivec4 m10, ivec4 m20) {
            this.m00 = m00.v0;
            this.m01 = m00.v1;
            this.m02 = m00.v2;
            this.m03 = m00.v3;
            this.m10 = m10.v0;
            this.m11 = m10.v1;
            this.m12 = m10.v2;
            this.m13 = m10.v3;
            this.m20 = m20.v0;
            this.m21 = m20.v1;
            this.m22 = m20.v2;
            this.m23 = m20.v3;
        }
        
        public imat3x4(ivec2 m00, ivec2 m01, ivec2 m10, ivec2 m11, ivec2 m20, ivec2 m21) {
            this.m00 = m00.v0;
            this.m01 = m00.v1;
            this.m02 = m01.v0;
            this.m03 = m01.v1;
            this.m10 = m10.v0;
            this.m11 = m10.v1;
            this.m12 = m11.v0;
            this.m13 = m11.v1;
            this.m20 = m20.v0;
            this.m21 = m20.v1;
            this.m22 = m21.v0;
            this.m23 = m21.v1;
        }
        
        public imat3x4(ivec3 m00, int m01, ivec3 m10, int m11, ivec3 m20, int m21) {
            this.m00 = m00.v0;
            this.m01 = m00.v1;
            this.m02 = m00.v2;
            this.m03 = m01;
            this.m10 = m10.v0;
            this.m11 = m10.v1;
            this.m12 = m10.v2;
            this.m13 = m11;
            this.m20 = m20.v0;
            this.m21 = m20.v1;
            this.m22 = m20.v2;
            this.m23 = m21;
        }
        
        public imat3x4(int m00, ivec3 m01, int m10, ivec3 m11, int m20, ivec3 m21) {
            this.m00 = m00;
            this.m01 = m01.v0;
            this.m02 = m01.v1;
            this.m03 = m01.v2;
            this.m10 = m10;
            this.m11 = m11.v0;
            this.m12 = m11.v1;
            this.m13 = m11.v2;
            this.m20 = m20;
            this.m21 = m21.v0;
            this.m22 = m21.v1;
            this.m23 = m21.v2;
        }
        
        public imat3x4(ivec2 m00, int m01, int m02, ivec2 m10, int m11, int m12, ivec2 m20, int m21, int m22) {
            this.m00 = m00.v0;
            this.m01 = m00.v1;
            this.m02 = m01;
            this.m03 = m02;
            this.m10 = m10.v0;
            this.m11 = m10.v1;
            this.m12 = m11;
            this.m13 = m12;
            this.m20 = m20.v0;
            this.m21 = m20.v1;
            this.m22 = m21;
            this.m23 = m22;
        }
        
        public imat3x4(int m00, ivec2 m01, int m02, int m10, ivec2 m11, int m12, int m20, ivec2 m21, int m22) {
            this.m00 = m00;
            this.m01 = m01.v0;
            this.m02 = m01.v1;
            this.m03 = m02;
            this.m10 = m10;
            this.m11 = m11.v0;
            this.m12 = m11.v1;
            this.m13 = m12;
            this.m20 = m20;
            this.m21 = m21.v0;
            this.m22 = m21.v1;
            this.m23 = m22;
        }
        
        public imat3x4(int m00, int m01, ivec2 m02, int m10, int m11, ivec2 m12, int m20, int m21, ivec2 m22) {
            this.m00 = m00;
            this.m01 = m01;
            this.m02 = m02.v0;
            this.m03 = m02.v1;
            this.m10 = m10;
            this.m11 = m11;
            this.m12 = m12.v0;
            this.m13 = m12.v1;
            this.m20 = m20;
            this.m21 = m21;
            this.m22 = m22.v0;
            this.m23 = m22.v1;
        }
        
        public imat3x4(int m00, int m01, int m02, int m03, int m10, int m11, int m12, int m13, int m20, int m21, int m22, int m23) {
            this.m00 = m00;
            this.m01 = m01;
            this.m02 = m02;
            this.m03 = m03;
            this.m10 = m10;
            this.m11 = m11;
            this.m12 = m12;
            this.m13 = m13;
            this.m20 = m20;
            this.m21 = m21;
            this.m22 = m22;
            this.m23 = m23;
        }
        
        public imat3x4(imat2 s) {
            this.m00 = s.m00;
            this.m01 = s.m01;
            this.m02 = 0;
            this.m03 = 0;
            this.m10 = s.m10;
            this.m11 = s.m11;
            this.m12 = 0;
            this.m13 = 0;
            this.m20 = 0;
            this.m21 = 0;
            this.m22 = 0;
            this.m23 = 0;
        }
        
        public imat3x4(imat2x3 s) {
            this.m00 = s.m00;
            this.m01 = s.m01;
            this.m02 = s.m02;
            this.m03 = 0;
            this.m10 = s.m10;
            this.m11 = s.m11;
            this.m12 = s.m12;
            this.m13 = 0;
            this.m20 = 0;
            this.m21 = 0;
            this.m22 = 0;
            this.m23 = 0;
        }
        
        public imat3x4(imat2x4 s) {
            this.m00 = s.m00;
            this.m01 = s.m01;
            this.m02 = s.m02;
            this.m03 = s.m03;
            this.m10 = s.m10;
            this.m11 = s.m11;
            this.m12 = s.m12;
            this.m13 = s.m13;
            this.m20 = 0;
            this.m21 = 0;
            this.m22 = 0;
            this.m23 = 0;
        }
        
        public imat3x4(imat3x2 s) {
            this.m00 = s.m00;
            this.m01 = s.m01;
            this.m02 = 0;
            this.m03 = 0;
            this.m10 = s.m10;
            this.m11 = s.m11;
            this.m12 = 0;
            this.m13 = 0;
            this.m20 = s.m20;
            this.m21 = s.m21;
            this.m22 = 0;
            this.m23 = 0;
        }
        
        public imat3x4(imat3 s) {
            this.m00 = s.m00;
            this.m01 = s.m01;
            this.m02 = s.m02;
            this.m03 = 0;
            this.m10 = s.m10;
            this.m11 = s.m11;
            this.m12 = s.m12;
            this.m13 = 0;
            this.m20 = s.m20;
            this.m21 = s.m21;
            this.m22 = s.m22;
            this.m23 = 0;
        }
        
        public imat3x4(imat4x2 s) {
            this.m00 = s.m00;
            this.m01 = s.m01;
            this.m02 = 0;
            this.m03 = 0;
            this.m10 = s.m10;
            this.m11 = s.m11;
            this.m12 = 0;
            this.m13 = 0;
            this.m20 = s.m20;
            this.m21 = s.m21;
            this.m22 = 0;
            this.m23 = 0;
        }
        
        public imat3x4(imat4x3 s) {
            this.m00 = s.m00;
            this.m01 = s.m01;
            this.m02 = s.m02;
            this.m03 = 0;
            this.m10 = s.m10;
            this.m11 = s.m11;
            this.m12 = s.m12;
            this.m13 = 0;
            this.m20 = s.m20;
            this.m21 = s.m21;
            this.m22 = s.m22;
            this.m23 = 0;
        }
        
        public imat3x4(imat4 s) {
            this.m00 = s.m00;
            this.m01 = s.m01;
            this.m02 = s.m02;
            this.m03 = s.m03;
            this.m10 = s.m10;
            this.m11 = s.m11;
            this.m12 = s.m12;
            this.m13 = s.m13;
            this.m20 = s.m20;
            this.m21 = s.m21;
            this.m22 = s.m22;
            this.m23 = s.m23;
        }
        
        public int this[int i0, int i1] {
            get {
                if (((i0 < 0) 
                            || (2 < i0))) {
                    throw new System.IndexOutOfRangeException("i0");
                }
                if (((i1 < 0) 
                            || (3 < i1))) {
                    throw new System.IndexOutOfRangeException("i1");
                }
                
                unsafe
                {
                    fixed (Int32* v_ptr = &this.m00)
                    {
                        return *(v_ptr + (i1 + i0 * 4));
                    }
                };
            }
            set {
                if (((i0 < 0) 
                            || (2 < i0))) {
                    throw new System.IndexOutOfRangeException("i0");
                }
                if (((i1 < 0) 
                            || (3 < i1))) {
                    throw new System.IndexOutOfRangeException("i1");
                }
                
                unsafe
                {
                    fixed (Int32* v_ptr = &this.m00)
                    {
                        *(v_ptr  + (i1 + i0 * 4)) = value;
                    }
                };
            }
        }
        
        public ivec4 this[int i0] {
            get {
                return new ivec4(this[i0, 0], this[i0, 1], this[i0, 2], this[i0, 3]);
            }
            set {
                this[i0, 0] = value[0];
                this[i0, 1] = value[1];
                this[i0, 2] = value[2];
                this[i0, 3] = value[3];
            }
        }
        
        public static imat3x4 operator+(imat3x4 v) {
            return new imat3x4(+v.m00, +v.m01, +v.m02, +v.m03, +v.m10, +v.m11, +v.m12, +v.m13, +v.m20, +v.m21, +v.m22, +v.m23);
        }
        
        public static imat3x4 operator-(imat3x4 v) {
            return new imat3x4(-v.m00, -v.m01, -v.m02, -v.m03, -v.m10, -v.m11, -v.m12, -v.m13, -v.m20, -v.m21, -v.m22, -v.m23);
        }
        
        public static imat3x4 operator~(imat3x4 v) {
            return new imat3x4(~v.m00, ~v.m01, ~v.m02, ~v.m03, ~v.m10, ~v.m11, ~v.m12, ~v.m13, ~v.m20, ~v.m21, ~v.m22, ~v.m23);
        }
        
        public static imat3x4 operator++(imat3x4 v) {
            return new imat3x4(++v.m00, ++v.m01, ++v.m02, ++v.m03, ++v.m10, ++v.m11, ++v.m12, ++v.m13, ++v.m20, ++v.m21, ++v.m22, ++v.m23);
        }
        
        public static imat3x4 operator--(imat3x4 v) {
            return new imat3x4(--v.m00, --v.m01, --v.m02, --v.m03, --v.m10, --v.m11, --v.m12, --v.m13, --v.m20, --v.m21, --v.m22, --v.m23);
        }
        
        public static imat3x4 operator+(int s, imat3x4 v2) {
            return new imat3x4(s + v2.m00, s + v2.m01, s + v2.m02, s + v2.m03, s + v2.m10, s + v2.m11, s + v2.m12, s + v2.m13, s + v2.m20, s + v2.m21, s + v2.m22, s + v2.m23);
        }
        
        public static imat3x4 operator-(int s, imat3x4 v2) {
            return new imat3x4(s - v2.m00, s - v2.m01, s - v2.m02, s - v2.m03, s - v2.m10, s - v2.m11, s - v2.m12, s - v2.m13, s - v2.m20, s - v2.m21, s - v2.m22, s - v2.m23);
        }
        
        public static imat3x4 operator*(int s, imat3x4 v2) {
            return new imat3x4(s * v2.m00, s * v2.m01, s * v2.m02, s * v2.m03, s * v2.m10, s * v2.m11, s * v2.m12, s * v2.m13, s * v2.m20, s * v2.m21, s * v2.m22, s * v2.m23);
        }
        
        public static imat3x4 operator/(int s, imat3x4 v2) {
            return new imat3x4(s / v2.m00, s / v2.m01, s / v2.m02, s / v2.m03, s / v2.m10, s / v2.m11, s / v2.m12, s / v2.m13, s / v2.m20, s / v2.m21, s / v2.m22, s / v2.m23);
        }
        
        public static imat3x4 operator%(int s, imat3x4 v2) {
            return new imat3x4(s % v2.m00, s % v2.m01, s % v2.m02, s % v2.m03, s % v2.m10, s % v2.m11, s % v2.m12, s % v2.m13, s % v2.m20, s % v2.m21, s % v2.m22, s % v2.m23);
        }
        
        public static imat3x4 operator&(int s, imat3x4 v2) {
            return new imat3x4(s & v2.m00, s & v2.m01, s & v2.m02, s & v2.m03, s & v2.m10, s & v2.m11, s & v2.m12, s & v2.m13, s & v2.m20, s & v2.m21, s & v2.m22, s & v2.m23);
        }
        
        public static imat3x4 operator|(int s, imat3x4 v2) {
            return new imat3x4(s | v2.m00, s | v2.m01, s | v2.m02, s | v2.m03, s | v2.m10, s | v2.m11, s | v2.m12, s | v2.m13, s | v2.m20, s | v2.m21, s | v2.m22, s | v2.m23);
        }
        
        public static imat3x4 operator^(int s, imat3x4 v2) {
            return new imat3x4(s ^ v2.m00, s ^ v2.m01, s ^ v2.m02, s ^ v2.m03, s ^ v2.m10, s ^ v2.m11, s ^ v2.m12, s ^ v2.m13, s ^ v2.m20, s ^ v2.m21, s ^ v2.m22, s ^ v2.m23);
        }
        
        public static imat3x4 operator+(imat3x4 v1, int s) {
            return new imat3x4(v1.m00 + s, v1.m01 + s, v1.m02 + s, v1.m03 + s, v1.m10 + s, v1.m11 + s, v1.m12 + s, v1.m13 + s, v1.m20 + s, v1.m21 + s, v1.m22 + s, v1.m23 + s);
        }
        
        public static imat3x4 operator-(imat3x4 v1, int s) {
            return new imat3x4(v1.m00 - s, v1.m01 - s, v1.m02 - s, v1.m03 - s, v1.m10 - s, v1.m11 - s, v1.m12 - s, v1.m13 - s, v1.m20 - s, v1.m21 - s, v1.m22 - s, v1.m23 - s);
        }
        
        public static imat3x4 operator*(imat3x4 v1, int s) {
            return new imat3x4(v1.m00 * s, v1.m01 * s, v1.m02 * s, v1.m03 * s, v1.m10 * s, v1.m11 * s, v1.m12 * s, v1.m13 * s, v1.m20 * s, v1.m21 * s, v1.m22 * s, v1.m23 * s);
        }
        
        public static imat3x4 operator/(imat3x4 v1, int s) {
            return new imat3x4(v1.m00 / s, v1.m01 / s, v1.m02 / s, v1.m03 / s, v1.m10 / s, v1.m11 / s, v1.m12 / s, v1.m13 / s, v1.m20 / s, v1.m21 / s, v1.m22 / s, v1.m23 / s);
        }
        
        public static imat3x4 operator%(imat3x4 v1, int s) {
            return new imat3x4(v1.m00 % s, v1.m01 % s, v1.m02 % s, v1.m03 % s, v1.m10 % s, v1.m11 % s, v1.m12 % s, v1.m13 % s, v1.m20 % s, v1.m21 % s, v1.m22 % s, v1.m23 % s);
        }
        
        public static imat3x4 operator&(imat3x4 v1, int s) {
            return new imat3x4(v1.m00 & s, v1.m01 & s, v1.m02 & s, v1.m03 & s, v1.m10 & s, v1.m11 & s, v1.m12 & s, v1.m13 & s, v1.m20 & s, v1.m21 & s, v1.m22 & s, v1.m23 & s);
        }
        
        public static imat3x4 operator|(imat3x4 v1, int s) {
            return new imat3x4(v1.m00 | s, v1.m01 | s, v1.m02 | s, v1.m03 | s, v1.m10 | s, v1.m11 | s, v1.m12 | s, v1.m13 | s, v1.m20 | s, v1.m21 | s, v1.m22 | s, v1.m23 | s);
        }
        
        public static imat3x4 operator^(imat3x4 v1, int s) {
            return new imat3x4(v1.m00 ^ s, v1.m01 ^ s, v1.m02 ^ s, v1.m03 ^ s, v1.m10 ^ s, v1.m11 ^ s, v1.m12 ^ s, v1.m13 ^ s, v1.m20 ^ s, v1.m21 ^ s, v1.m22 ^ s, v1.m23 ^ s);
        }
        
        public static imat3x4 operator+(imat3x4 v1, imat3x4 v2) {
            return new imat3x4(v1.m00 + v2.m00, v1.m01 + v2.m01, v1.m02 + v2.m02, v1.m03 + v2.m03, v1.m10 + v2.m10, v1.m11 + v2.m11, v1.m12 + v2.m12, v1.m13 + v2.m13, v1.m20 + v2.m20, v1.m21 + v2.m21, v1.m22 + v2.m22, v1.m23 + v2.m23);
        }
        
        public static imat3x4 operator-(imat3x4 v1, imat3x4 v2) {
            return new imat3x4(v1.m00 - v2.m00, v1.m01 - v2.m01, v1.m02 - v2.m02, v1.m03 - v2.m03, v1.m10 - v2.m10, v1.m11 - v2.m11, v1.m12 - v2.m12, v1.m13 - v2.m13, v1.m20 - v2.m20, v1.m21 - v2.m21, v1.m22 - v2.m22, v1.m23 - v2.m23);
        }
        
        public static imat3x4 operator/(imat3x4 v1, imat3x4 v2) {
            return new imat3x4(v1.m00 / v2.m00, v1.m01 / v2.m01, v1.m02 / v2.m02, v1.m03 / v2.m03, v1.m10 / v2.m10, v1.m11 / v2.m11, v1.m12 / v2.m12, v1.m13 / v2.m13, v1.m20 / v2.m20, v1.m21 / v2.m21, v1.m22 / v2.m22, v1.m23 / v2.m23);
        }
        
        public static imat3x4 operator%(imat3x4 v1, imat3x4 v2) {
            return new imat3x4(v1.m00 % v2.m00, v1.m01 % v2.m01, v1.m02 % v2.m02, v1.m03 % v2.m03, v1.m10 % v2.m10, v1.m11 % v2.m11, v1.m12 % v2.m12, v1.m13 % v2.m13, v1.m20 % v2.m20, v1.m21 % v2.m21, v1.m22 % v2.m22, v1.m23 % v2.m23);
        }
        
        public static imat3x4 operator&(imat3x4 v1, imat3x4 v2) {
            return new imat3x4(v1.m00 & v2.m00, v1.m01 & v2.m01, v1.m02 & v2.m02, v1.m03 & v2.m03, v1.m10 & v2.m10, v1.m11 & v2.m11, v1.m12 & v2.m12, v1.m13 & v2.m13, v1.m20 & v2.m20, v1.m21 & v2.m21, v1.m22 & v2.m22, v1.m23 & v2.m23);
        }
        
        public static imat3x4 operator|(imat3x4 v1, imat3x4 v2) {
            return new imat3x4(v1.m00 | v2.m00, v1.m01 | v2.m01, v1.m02 | v2.m02, v1.m03 | v2.m03, v1.m10 | v2.m10, v1.m11 | v2.m11, v1.m12 | v2.m12, v1.m13 | v2.m13, v1.m20 | v2.m20, v1.m21 | v2.m21, v1.m22 | v2.m22, v1.m23 | v2.m23);
        }
        
        public static imat3x4 operator^(imat3x4 v1, imat3x4 v2) {
            return new imat3x4(v1.m00 ^ v2.m00, v1.m01 ^ v2.m01, v1.m02 ^ v2.m02, v1.m03 ^ v2.m03, v1.m10 ^ v2.m10, v1.m11 ^ v2.m11, v1.m12 ^ v2.m12, v1.m13 ^ v2.m13, v1.m20 ^ v2.m20, v1.m21 ^ v2.m21, v1.m22 ^ v2.m22, v1.m23 ^ v2.m23);
        }
        
        public static bmat3x4 operator==(imat3x4 v1, imat3x4 v2) {
            return new bmat3x4(v1.m00 == v2.m00, v1.m01 == v2.m01, v1.m02 == v2.m02, v1.m03 == v2.m03, v1.m10 == v2.m10, v1.m11 == v2.m11, v1.m12 == v2.m12, v1.m13 == v2.m13, v1.m20 == v2.m20, v1.m21 == v2.m21, v1.m22 == v2.m22, v1.m23 == v2.m23);
        }
        
        public static bmat3x4 operator>(imat3x4 v1, imat3x4 v2) {
            return new bmat3x4(v1.m00 > v2.m00, v1.m01 > v2.m01, v1.m02 > v2.m02, v1.m03 > v2.m03, v1.m10 > v2.m10, v1.m11 > v2.m11, v1.m12 > v2.m12, v1.m13 > v2.m13, v1.m20 > v2.m20, v1.m21 > v2.m21, v1.m22 > v2.m22, v1.m23 > v2.m23);
        }
        
        public static bmat3x4 operator>=(imat3x4 v1, imat3x4 v2) {
            return new bmat3x4(v1.m00 >= v2.m00, v1.m01 >= v2.m01, v1.m02 >= v2.m02, v1.m03 >= v2.m03, v1.m10 >= v2.m10, v1.m11 >= v2.m11, v1.m12 >= v2.m12, v1.m13 >= v2.m13, v1.m20 >= v2.m20, v1.m21 >= v2.m21, v1.m22 >= v2.m22, v1.m23 >= v2.m23);
        }
        
        public static bmat3x4 operator<(imat3x4 v1, imat3x4 v2) {
            return new bmat3x4(v1.m00 < v2.m00, v1.m01 < v2.m01, v1.m02 < v2.m02, v1.m03 < v2.m03, v1.m10 < v2.m10, v1.m11 < v2.m11, v1.m12 < v2.m12, v1.m13 < v2.m13, v1.m20 < v2.m20, v1.m21 < v2.m21, v1.m22 < v2.m22, v1.m23 < v2.m23);
        }
        
        public static bmat3x4 operator<=(imat3x4 v1, imat3x4 v2) {
            return new bmat3x4(v1.m00 <= v2.m00, v1.m01 <= v2.m01, v1.m02 <= v2.m02, v1.m03 <= v2.m03, v1.m10 <= v2.m10, v1.m11 <= v2.m11, v1.m12 <= v2.m12, v1.m13 <= v2.m13, v1.m20 <= v2.m20, v1.m21 <= v2.m21, v1.m22 <= v2.m22, v1.m23 <= v2.m23);
        }
        
        public static bmat3x4 operator!=(imat3x4 v1, imat3x4 v2) {
            return new bmat3x4(v1.m00 != v2.m00, v1.m01 != v2.m01, v1.m02 != v2.m02, v1.m03 != v2.m03, v1.m10 != v2.m10, v1.m11 != v2.m11, v1.m12 != v2.m12, v1.m13 != v2.m13, v1.m20 != v2.m20, v1.m21 != v2.m21, v1.m22 != v2.m22, v1.m23 != v2.m23);
        }
        
        public static ivec4 operator*(ivec3 v0, imat3x4 m1) {
            return imath.InnerProduct1(ref v0, ref m1);
        }
        
        public static ivec3 operator*(imat3x4 m0, ivec4 v1) {
            return imath.InnerProduct1(ref m0, ref v1);
        }
        
        public static imat3x2 operator*(imat3x4 m0, imat4x2 m1) {
            return imath.InnerProduct1(ref m0, ref m1);
        }
        
        public static imat3 operator*(imat3x4 m0, imat4x3 m1) {
            return imath.InnerProduct1(ref m0, ref m1);
        }
        
        public static imat3x4 operator*(imat3x4 m0, imat4 m1) {
            return imath.InnerProduct1(ref m0, ref m1);
        }
        
        public override bool Equals(object obj) {
            return (obj is imat3x4) ? bmath.All(this == (imat3x4)obj) : false;
        }
        
        public override int GetHashCode() {
            return this.m00.GetHashCode() ^ this.m01.GetHashCode() ^ this.m02.GetHashCode() ^ this.m03.GetHashCode() ^ this.m10.GetHashCode() ^ this.m11.GetHashCode() ^ this.m12.GetHashCode() ^ this.m13.GetHashCode() ^ this.m20.GetHashCode() ^ this.m21.GetHashCode() ^ this.m22.GetHashCode() ^ this.m23.GetHashCode();
        }
        
        public override string ToString() {
            return "imat3x4(" + this.m00 + "," + this.m01 + "," + this.m02 + "," + this.m03 + "," + this.m10 + "," + this.m11 + "," + this.m12 + "," + this.m13 + "," + this.m20 + "," + this.m21 + "," + this.m22 + "," + this.m23 + ")";
        }
    }
    
    [DataContract()]
    [StructLayout(LayoutKind.Sequential)]
    public struct imat4x2 {
        
        [DataMember()]
        internal int m00;
        
        [DataMember()]
        internal int m01;
        
        [DataMember()]
        internal int m10;
        
        [DataMember()]
        internal int m11;
        
        [DataMember()]
        internal int m20;
        
        [DataMember()]
        internal int m21;
        
        [DataMember()]
        internal int m30;
        
        [DataMember()]
        internal int m31;
        
        public imat4x2(int s) {
            this.m00 = s;
            this.m01 = 0;
            this.m10 = 0;
            this.m11 = s;
            this.m20 = 0;
            this.m21 = 0;
            this.m30 = 0;
            this.m31 = 0;
        }
        
        public imat4x2(imat4x2 m00) {
            this.m00 = m00.m00;
            this.m01 = m00.m01;
            this.m10 = m00.m10;
            this.m11 = m00.m11;
            this.m20 = m00.m20;
            this.m21 = m00.m21;
            this.m30 = m00.m30;
            this.m31 = m00.m31;
        }
        
        public imat4x2(imat2 m00, imat2 m10) {
            this.m00 = m00.m00;
            this.m01 = m00.m01;
            this.m10 = m00.m10;
            this.m11 = m00.m11;
            this.m20 = m10.m00;
            this.m21 = m10.m01;
            this.m30 = m10.m10;
            this.m31 = m10.m11;
        }
        
        public imat4x2(imat3x2 m00, ivec2 m10) {
            this.m00 = m00.m00;
            this.m01 = m00.m01;
            this.m10 = m00.m10;
            this.m11 = m00.m11;
            this.m20 = m00.m20;
            this.m21 = m00.m21;
            this.m30 = m10.v0;
            this.m31 = m10.v1;
        }
        
        public imat4x2(ivec2 m00, imat3x2 m10) {
            this.m00 = m00.v0;
            this.m01 = m00.v1;
            this.m10 = m10.m00;
            this.m11 = m10.m01;
            this.m20 = m10.m10;
            this.m21 = m10.m11;
            this.m30 = m10.m20;
            this.m31 = m10.m21;
        }
        
        public imat4x2(imat2 m00, ivec2 m10, ivec2 m20) {
            this.m00 = m00.m00;
            this.m01 = m00.m01;
            this.m10 = m00.m10;
            this.m11 = m00.m11;
            this.m20 = m10.v0;
            this.m21 = m10.v1;
            this.m30 = m20.v0;
            this.m31 = m20.v1;
        }
        
        public imat4x2(ivec2 m00, imat2 m10, ivec2 m20) {
            this.m00 = m00.v0;
            this.m01 = m00.v1;
            this.m10 = m10.m00;
            this.m11 = m10.m01;
            this.m20 = m10.m10;
            this.m21 = m10.m11;
            this.m30 = m20.v0;
            this.m31 = m20.v1;
        }
        
        public imat4x2(ivec2 m00, ivec2 m10, imat2 m20) {
            this.m00 = m00.v0;
            this.m01 = m00.v1;
            this.m10 = m10.v0;
            this.m11 = m10.v1;
            this.m20 = m20.m00;
            this.m21 = m20.m01;
            this.m30 = m20.m10;
            this.m31 = m20.m11;
        }
        
        public imat4x2(ivec2 m00, ivec2 m10, ivec2 m20, ivec2 m30) {
            this.m00 = m00.v0;
            this.m01 = m00.v1;
            this.m10 = m10.v0;
            this.m11 = m10.v1;
            this.m20 = m20.v0;
            this.m21 = m20.v1;
            this.m30 = m30.v0;
            this.m31 = m30.v1;
        }
        
        public imat4x2(int m00, int m01, int m10, int m11, int m20, int m21, int m30, int m31) {
            this.m00 = m00;
            this.m01 = m01;
            this.m10 = m10;
            this.m11 = m11;
            this.m20 = m20;
            this.m21 = m21;
            this.m30 = m30;
            this.m31 = m31;
        }
        
        public imat4x2(imat2 s) {
            this.m00 = s.m00;
            this.m01 = s.m01;
            this.m10 = s.m10;
            this.m11 = s.m11;
            this.m20 = 0;
            this.m21 = 0;
            this.m30 = 0;
            this.m31 = 0;
        }
        
        public imat4x2(imat2x3 s) {
            this.m00 = s.m00;
            this.m01 = s.m01;
            this.m10 = s.m10;
            this.m11 = s.m11;
            this.m20 = 0;
            this.m21 = 0;
            this.m30 = 0;
            this.m31 = 0;
        }
        
        public imat4x2(imat2x4 s) {
            this.m00 = s.m00;
            this.m01 = s.m01;
            this.m10 = s.m10;
            this.m11 = s.m11;
            this.m20 = 0;
            this.m21 = 0;
            this.m30 = 0;
            this.m31 = 0;
        }
        
        public imat4x2(imat3x2 s) {
            this.m00 = s.m00;
            this.m01 = s.m01;
            this.m10 = s.m10;
            this.m11 = s.m11;
            this.m20 = s.m20;
            this.m21 = s.m21;
            this.m30 = 0;
            this.m31 = 0;
        }
        
        public imat4x2(imat3 s) {
            this.m00 = s.m00;
            this.m01 = s.m01;
            this.m10 = s.m10;
            this.m11 = s.m11;
            this.m20 = s.m20;
            this.m21 = s.m21;
            this.m30 = 0;
            this.m31 = 0;
        }
        
        public imat4x2(imat3x4 s) {
            this.m00 = s.m00;
            this.m01 = s.m01;
            this.m10 = s.m10;
            this.m11 = s.m11;
            this.m20 = s.m20;
            this.m21 = s.m21;
            this.m30 = 0;
            this.m31 = 0;
        }
        
        public imat4x2(imat4x3 s) {
            this.m00 = s.m00;
            this.m01 = s.m01;
            this.m10 = s.m10;
            this.m11 = s.m11;
            this.m20 = s.m20;
            this.m21 = s.m21;
            this.m30 = s.m30;
            this.m31 = s.m31;
        }
        
        public imat4x2(imat4 s) {
            this.m00 = s.m00;
            this.m01 = s.m01;
            this.m10 = s.m10;
            this.m11 = s.m11;
            this.m20 = s.m20;
            this.m21 = s.m21;
            this.m30 = s.m30;
            this.m31 = s.m31;
        }
        
        public int this[int i0, int i1] {
            get {
                if (((i0 < 0) 
                            || (3 < i0))) {
                    throw new System.IndexOutOfRangeException("i0");
                }
                if (((i1 < 0) 
                            || (1 < i1))) {
                    throw new System.IndexOutOfRangeException("i1");
                }
                
                unsafe
                {
                    fixed (Int32* v_ptr = &this.m00)
                    {
                        return *(v_ptr + (i1 + i0 * 2));
                    }
                };
            }
            set {
                if (((i0 < 0) 
                            || (3 < i0))) {
                    throw new System.IndexOutOfRangeException("i0");
                }
                if (((i1 < 0) 
                            || (1 < i1))) {
                    throw new System.IndexOutOfRangeException("i1");
                }
                
                unsafe
                {
                    fixed (Int32* v_ptr = &this.m00)
                    {
                        *(v_ptr  + (i1 + i0 * 2)) = value;
                    }
                };
            }
        }
        
        public ivec2 this[int i0] {
            get {
                return new ivec2(this[i0, 0], this[i0, 1]);
            }
            set {
                this[i0, 0] = value[0];
                this[i0, 1] = value[1];
            }
        }
        
        public static imat4x2 operator+(imat4x2 v) {
            return new imat4x2(+v.m00, +v.m01, +v.m10, +v.m11, +v.m20, +v.m21, +v.m30, +v.m31);
        }
        
        public static imat4x2 operator-(imat4x2 v) {
            return new imat4x2(-v.m00, -v.m01, -v.m10, -v.m11, -v.m20, -v.m21, -v.m30, -v.m31);
        }
        
        public static imat4x2 operator~(imat4x2 v) {
            return new imat4x2(~v.m00, ~v.m01, ~v.m10, ~v.m11, ~v.m20, ~v.m21, ~v.m30, ~v.m31);
        }
        
        public static imat4x2 operator++(imat4x2 v) {
            return new imat4x2(++v.m00, ++v.m01, ++v.m10, ++v.m11, ++v.m20, ++v.m21, ++v.m30, ++v.m31);
        }
        
        public static imat4x2 operator--(imat4x2 v) {
            return new imat4x2(--v.m00, --v.m01, --v.m10, --v.m11, --v.m20, --v.m21, --v.m30, --v.m31);
        }
        
        public static imat4x2 operator+(int s, imat4x2 v2) {
            return new imat4x2(s + v2.m00, s + v2.m01, s + v2.m10, s + v2.m11, s + v2.m20, s + v2.m21, s + v2.m30, s + v2.m31);
        }
        
        public static imat4x2 operator-(int s, imat4x2 v2) {
            return new imat4x2(s - v2.m00, s - v2.m01, s - v2.m10, s - v2.m11, s - v2.m20, s - v2.m21, s - v2.m30, s - v2.m31);
        }
        
        public static imat4x2 operator*(int s, imat4x2 v2) {
            return new imat4x2(s * v2.m00, s * v2.m01, s * v2.m10, s * v2.m11, s * v2.m20, s * v2.m21, s * v2.m30, s * v2.m31);
        }
        
        public static imat4x2 operator/(int s, imat4x2 v2) {
            return new imat4x2(s / v2.m00, s / v2.m01, s / v2.m10, s / v2.m11, s / v2.m20, s / v2.m21, s / v2.m30, s / v2.m31);
        }
        
        public static imat4x2 operator%(int s, imat4x2 v2) {
            return new imat4x2(s % v2.m00, s % v2.m01, s % v2.m10, s % v2.m11, s % v2.m20, s % v2.m21, s % v2.m30, s % v2.m31);
        }
        
        public static imat4x2 operator&(int s, imat4x2 v2) {
            return new imat4x2(s & v2.m00, s & v2.m01, s & v2.m10, s & v2.m11, s & v2.m20, s & v2.m21, s & v2.m30, s & v2.m31);
        }
        
        public static imat4x2 operator|(int s, imat4x2 v2) {
            return new imat4x2(s | v2.m00, s | v2.m01, s | v2.m10, s | v2.m11, s | v2.m20, s | v2.m21, s | v2.m30, s | v2.m31);
        }
        
        public static imat4x2 operator^(int s, imat4x2 v2) {
            return new imat4x2(s ^ v2.m00, s ^ v2.m01, s ^ v2.m10, s ^ v2.m11, s ^ v2.m20, s ^ v2.m21, s ^ v2.m30, s ^ v2.m31);
        }
        
        public static imat4x2 operator+(imat4x2 v1, int s) {
            return new imat4x2(v1.m00 + s, v1.m01 + s, v1.m10 + s, v1.m11 + s, v1.m20 + s, v1.m21 + s, v1.m30 + s, v1.m31 + s);
        }
        
        public static imat4x2 operator-(imat4x2 v1, int s) {
            return new imat4x2(v1.m00 - s, v1.m01 - s, v1.m10 - s, v1.m11 - s, v1.m20 - s, v1.m21 - s, v1.m30 - s, v1.m31 - s);
        }
        
        public static imat4x2 operator*(imat4x2 v1, int s) {
            return new imat4x2(v1.m00 * s, v1.m01 * s, v1.m10 * s, v1.m11 * s, v1.m20 * s, v1.m21 * s, v1.m30 * s, v1.m31 * s);
        }
        
        public static imat4x2 operator/(imat4x2 v1, int s) {
            return new imat4x2(v1.m00 / s, v1.m01 / s, v1.m10 / s, v1.m11 / s, v1.m20 / s, v1.m21 / s, v1.m30 / s, v1.m31 / s);
        }
        
        public static imat4x2 operator%(imat4x2 v1, int s) {
            return new imat4x2(v1.m00 % s, v1.m01 % s, v1.m10 % s, v1.m11 % s, v1.m20 % s, v1.m21 % s, v1.m30 % s, v1.m31 % s);
        }
        
        public static imat4x2 operator&(imat4x2 v1, int s) {
            return new imat4x2(v1.m00 & s, v1.m01 & s, v1.m10 & s, v1.m11 & s, v1.m20 & s, v1.m21 & s, v1.m30 & s, v1.m31 & s);
        }
        
        public static imat4x2 operator|(imat4x2 v1, int s) {
            return new imat4x2(v1.m00 | s, v1.m01 | s, v1.m10 | s, v1.m11 | s, v1.m20 | s, v1.m21 | s, v1.m30 | s, v1.m31 | s);
        }
        
        public static imat4x2 operator^(imat4x2 v1, int s) {
            return new imat4x2(v1.m00 ^ s, v1.m01 ^ s, v1.m10 ^ s, v1.m11 ^ s, v1.m20 ^ s, v1.m21 ^ s, v1.m30 ^ s, v1.m31 ^ s);
        }
        
        public static imat4x2 operator+(imat4x2 v1, imat4x2 v2) {
            return new imat4x2(v1.m00 + v2.m00, v1.m01 + v2.m01, v1.m10 + v2.m10, v1.m11 + v2.m11, v1.m20 + v2.m20, v1.m21 + v2.m21, v1.m30 + v2.m30, v1.m31 + v2.m31);
        }
        
        public static imat4x2 operator-(imat4x2 v1, imat4x2 v2) {
            return new imat4x2(v1.m00 - v2.m00, v1.m01 - v2.m01, v1.m10 - v2.m10, v1.m11 - v2.m11, v1.m20 - v2.m20, v1.m21 - v2.m21, v1.m30 - v2.m30, v1.m31 - v2.m31);
        }
        
        public static imat4x2 operator/(imat4x2 v1, imat4x2 v2) {
            return new imat4x2(v1.m00 / v2.m00, v1.m01 / v2.m01, v1.m10 / v2.m10, v1.m11 / v2.m11, v1.m20 / v2.m20, v1.m21 / v2.m21, v1.m30 / v2.m30, v1.m31 / v2.m31);
        }
        
        public static imat4x2 operator%(imat4x2 v1, imat4x2 v2) {
            return new imat4x2(v1.m00 % v2.m00, v1.m01 % v2.m01, v1.m10 % v2.m10, v1.m11 % v2.m11, v1.m20 % v2.m20, v1.m21 % v2.m21, v1.m30 % v2.m30, v1.m31 % v2.m31);
        }
        
        public static imat4x2 operator&(imat4x2 v1, imat4x2 v2) {
            return new imat4x2(v1.m00 & v2.m00, v1.m01 & v2.m01, v1.m10 & v2.m10, v1.m11 & v2.m11, v1.m20 & v2.m20, v1.m21 & v2.m21, v1.m30 & v2.m30, v1.m31 & v2.m31);
        }
        
        public static imat4x2 operator|(imat4x2 v1, imat4x2 v2) {
            return new imat4x2(v1.m00 | v2.m00, v1.m01 | v2.m01, v1.m10 | v2.m10, v1.m11 | v2.m11, v1.m20 | v2.m20, v1.m21 | v2.m21, v1.m30 | v2.m30, v1.m31 | v2.m31);
        }
        
        public static imat4x2 operator^(imat4x2 v1, imat4x2 v2) {
            return new imat4x2(v1.m00 ^ v2.m00, v1.m01 ^ v2.m01, v1.m10 ^ v2.m10, v1.m11 ^ v2.m11, v1.m20 ^ v2.m20, v1.m21 ^ v2.m21, v1.m30 ^ v2.m30, v1.m31 ^ v2.m31);
        }
        
        public static bmat4x2 operator==(imat4x2 v1, imat4x2 v2) {
            return new bmat4x2(v1.m00 == v2.m00, v1.m01 == v2.m01, v1.m10 == v2.m10, v1.m11 == v2.m11, v1.m20 == v2.m20, v1.m21 == v2.m21, v1.m30 == v2.m30, v1.m31 == v2.m31);
        }
        
        public static bmat4x2 operator>(imat4x2 v1, imat4x2 v2) {
            return new bmat4x2(v1.m00 > v2.m00, v1.m01 > v2.m01, v1.m10 > v2.m10, v1.m11 > v2.m11, v1.m20 > v2.m20, v1.m21 > v2.m21, v1.m30 > v2.m30, v1.m31 > v2.m31);
        }
        
        public static bmat4x2 operator>=(imat4x2 v1, imat4x2 v2) {
            return new bmat4x2(v1.m00 >= v2.m00, v1.m01 >= v2.m01, v1.m10 >= v2.m10, v1.m11 >= v2.m11, v1.m20 >= v2.m20, v1.m21 >= v2.m21, v1.m30 >= v2.m30, v1.m31 >= v2.m31);
        }
        
        public static bmat4x2 operator<(imat4x2 v1, imat4x2 v2) {
            return new bmat4x2(v1.m00 < v2.m00, v1.m01 < v2.m01, v1.m10 < v2.m10, v1.m11 < v2.m11, v1.m20 < v2.m20, v1.m21 < v2.m21, v1.m30 < v2.m30, v1.m31 < v2.m31);
        }
        
        public static bmat4x2 operator<=(imat4x2 v1, imat4x2 v2) {
            return new bmat4x2(v1.m00 <= v2.m00, v1.m01 <= v2.m01, v1.m10 <= v2.m10, v1.m11 <= v2.m11, v1.m20 <= v2.m20, v1.m21 <= v2.m21, v1.m30 <= v2.m30, v1.m31 <= v2.m31);
        }
        
        public static bmat4x2 operator!=(imat4x2 v1, imat4x2 v2) {
            return new bmat4x2(v1.m00 != v2.m00, v1.m01 != v2.m01, v1.m10 != v2.m10, v1.m11 != v2.m11, v1.m20 != v2.m20, v1.m21 != v2.m21, v1.m30 != v2.m30, v1.m31 != v2.m31);
        }
        
        public static ivec2 operator*(ivec4 v0, imat4x2 m1) {
            return imath.InnerProduct1(ref v0, ref m1);
        }
        
        public static ivec4 operator*(imat4x2 m0, ivec2 v1) {
            return imath.InnerProduct1(ref m0, ref v1);
        }
        
        public static imat4x2 operator*(imat4x2 m0, imat2 m1) {
            return imath.InnerProduct1(ref m0, ref m1);
        }
        
        public static imat4x3 operator*(imat4x2 m0, imat2x3 m1) {
            return imath.InnerProduct1(ref m0, ref m1);
        }
        
        public static imat4 operator*(imat4x2 m0, imat2x4 m1) {
            return imath.InnerProduct1(ref m0, ref m1);
        }
        
        public override bool Equals(object obj) {
            return (obj is imat4x2) ? bmath.All(this == (imat4x2)obj) : false;
        }
        
        public override int GetHashCode() {
            return this.m00.GetHashCode() ^ this.m01.GetHashCode() ^ this.m10.GetHashCode() ^ this.m11.GetHashCode() ^ this.m20.GetHashCode() ^ this.m21.GetHashCode() ^ this.m30.GetHashCode() ^ this.m31.GetHashCode();
        }
        
        public override string ToString() {
            return "imat4x2(" + this.m00 + "," + this.m01 + "," + this.m10 + "," + this.m11 + "," + this.m20 + "," + this.m21 + "," + this.m30 + "," + this.m31 + ")";
        }
    }
    
    [DataContract()]
    [StructLayout(LayoutKind.Sequential)]
    public struct imat4x3 {
        
        [DataMember()]
        internal int m00;
        
        [DataMember()]
        internal int m01;
        
        [DataMember()]
        internal int m02;
        
        [DataMember()]
        internal int m10;
        
        [DataMember()]
        internal int m11;
        
        [DataMember()]
        internal int m12;
        
        [DataMember()]
        internal int m20;
        
        [DataMember()]
        internal int m21;
        
        [DataMember()]
        internal int m22;
        
        [DataMember()]
        internal int m30;
        
        [DataMember()]
        internal int m31;
        
        [DataMember()]
        internal int m32;
        
        public imat4x3(int s) {
            this.m00 = s;
            this.m01 = 0;
            this.m02 = 0;
            this.m10 = 0;
            this.m11 = s;
            this.m12 = 0;
            this.m20 = 0;
            this.m21 = 0;
            this.m22 = s;
            this.m30 = 0;
            this.m31 = 0;
            this.m32 = 0;
        }
        
        public imat4x3(imat4x3 m00) {
            this.m00 = m00.m00;
            this.m01 = m00.m01;
            this.m02 = m00.m02;
            this.m10 = m00.m10;
            this.m11 = m00.m11;
            this.m12 = m00.m12;
            this.m20 = m00.m20;
            this.m21 = m00.m21;
            this.m22 = m00.m22;
            this.m30 = m00.m30;
            this.m31 = m00.m31;
            this.m32 = m00.m32;
        }
        
        public imat4x3(imat2x3 m00, imat2x3 m10) {
            this.m00 = m00.m00;
            this.m01 = m00.m01;
            this.m02 = m00.m02;
            this.m10 = m00.m10;
            this.m11 = m00.m11;
            this.m12 = m00.m12;
            this.m20 = m10.m00;
            this.m21 = m10.m01;
            this.m22 = m10.m02;
            this.m30 = m10.m10;
            this.m31 = m10.m11;
            this.m32 = m10.m12;
        }
        
        public imat4x3(imat3 m00, ivec3 m10) {
            this.m00 = m00.m00;
            this.m01 = m00.m01;
            this.m02 = m00.m02;
            this.m10 = m00.m10;
            this.m11 = m00.m11;
            this.m12 = m00.m12;
            this.m20 = m00.m20;
            this.m21 = m00.m21;
            this.m22 = m00.m22;
            this.m30 = m10.v0;
            this.m31 = m10.v1;
            this.m32 = m10.v2;
        }
        
        public imat4x3(ivec3 m00, imat3 m10) {
            this.m00 = m00.v0;
            this.m01 = m00.v1;
            this.m02 = m00.v2;
            this.m10 = m10.m00;
            this.m11 = m10.m01;
            this.m12 = m10.m02;
            this.m20 = m10.m10;
            this.m21 = m10.m11;
            this.m22 = m10.m12;
            this.m30 = m10.m20;
            this.m31 = m10.m21;
            this.m32 = m10.m22;
        }
        
        public imat4x3(imat2x3 m00, ivec3 m10, ivec3 m20) {
            this.m00 = m00.m00;
            this.m01 = m00.m01;
            this.m02 = m00.m02;
            this.m10 = m00.m10;
            this.m11 = m00.m11;
            this.m12 = m00.m12;
            this.m20 = m10.v0;
            this.m21 = m10.v1;
            this.m22 = m10.v2;
            this.m30 = m20.v0;
            this.m31 = m20.v1;
            this.m32 = m20.v2;
        }
        
        public imat4x3(ivec3 m00, imat2x3 m10, ivec3 m20) {
            this.m00 = m00.v0;
            this.m01 = m00.v1;
            this.m02 = m00.v2;
            this.m10 = m10.m00;
            this.m11 = m10.m01;
            this.m12 = m10.m02;
            this.m20 = m10.m10;
            this.m21 = m10.m11;
            this.m22 = m10.m12;
            this.m30 = m20.v0;
            this.m31 = m20.v1;
            this.m32 = m20.v2;
        }
        
        public imat4x3(ivec3 m00, ivec3 m10, imat2x3 m20) {
            this.m00 = m00.v0;
            this.m01 = m00.v1;
            this.m02 = m00.v2;
            this.m10 = m10.v0;
            this.m11 = m10.v1;
            this.m12 = m10.v2;
            this.m20 = m20.m00;
            this.m21 = m20.m01;
            this.m22 = m20.m02;
            this.m30 = m20.m10;
            this.m31 = m20.m11;
            this.m32 = m20.m12;
        }
        
        public imat4x3(ivec3 m00, ivec3 m10, ivec3 m20, ivec3 m30) {
            this.m00 = m00.v0;
            this.m01 = m00.v1;
            this.m02 = m00.v2;
            this.m10 = m10.v0;
            this.m11 = m10.v1;
            this.m12 = m10.v2;
            this.m20 = m20.v0;
            this.m21 = m20.v1;
            this.m22 = m20.v2;
            this.m30 = m30.v0;
            this.m31 = m30.v1;
            this.m32 = m30.v2;
        }
        
        public imat4x3(ivec2 m00, int m01, ivec2 m10, int m11, ivec2 m20, int m21, ivec2 m30, int m31) {
            this.m00 = m00.v0;
            this.m01 = m00.v1;
            this.m02 = m01;
            this.m10 = m10.v0;
            this.m11 = m10.v1;
            this.m12 = m11;
            this.m20 = m20.v0;
            this.m21 = m20.v1;
            this.m22 = m21;
            this.m30 = m30.v0;
            this.m31 = m30.v1;
            this.m32 = m31;
        }
        
        public imat4x3(int m00, ivec2 m01, int m10, ivec2 m11, int m20, ivec2 m21, int m30, ivec2 m31) {
            this.m00 = m00;
            this.m01 = m01.v0;
            this.m02 = m01.v1;
            this.m10 = m10;
            this.m11 = m11.v0;
            this.m12 = m11.v1;
            this.m20 = m20;
            this.m21 = m21.v0;
            this.m22 = m21.v1;
            this.m30 = m30;
            this.m31 = m31.v0;
            this.m32 = m31.v1;
        }
        
        public imat4x3(int m00, int m01, int m02, int m10, int m11, int m12, int m20, int m21, int m22, int m30, int m31, int m32) {
            this.m00 = m00;
            this.m01 = m01;
            this.m02 = m02;
            this.m10 = m10;
            this.m11 = m11;
            this.m12 = m12;
            this.m20 = m20;
            this.m21 = m21;
            this.m22 = m22;
            this.m30 = m30;
            this.m31 = m31;
            this.m32 = m32;
        }
        
        public imat4x3(imat2 s) {
            this.m00 = s.m00;
            this.m01 = s.m01;
            this.m02 = 0;
            this.m10 = s.m10;
            this.m11 = s.m11;
            this.m12 = 0;
            this.m20 = 0;
            this.m21 = 0;
            this.m22 = 0;
            this.m30 = 0;
            this.m31 = 0;
            this.m32 = 0;
        }
        
        public imat4x3(imat2x3 s) {
            this.m00 = s.m00;
            this.m01 = s.m01;
            this.m02 = s.m02;
            this.m10 = s.m10;
            this.m11 = s.m11;
            this.m12 = s.m12;
            this.m20 = 0;
            this.m21 = 0;
            this.m22 = 0;
            this.m30 = 0;
            this.m31 = 0;
            this.m32 = 0;
        }
        
        public imat4x3(imat2x4 s) {
            this.m00 = s.m00;
            this.m01 = s.m01;
            this.m02 = s.m02;
            this.m10 = s.m10;
            this.m11 = s.m11;
            this.m12 = s.m12;
            this.m20 = 0;
            this.m21 = 0;
            this.m22 = 0;
            this.m30 = 0;
            this.m31 = 0;
            this.m32 = 0;
        }
        
        public imat4x3(imat3x2 s) {
            this.m00 = s.m00;
            this.m01 = s.m01;
            this.m02 = 0;
            this.m10 = s.m10;
            this.m11 = s.m11;
            this.m12 = 0;
            this.m20 = s.m20;
            this.m21 = s.m21;
            this.m22 = 0;
            this.m30 = 0;
            this.m31 = 0;
            this.m32 = 0;
        }
        
        public imat4x3(imat3 s) {
            this.m00 = s.m00;
            this.m01 = s.m01;
            this.m02 = s.m02;
            this.m10 = s.m10;
            this.m11 = s.m11;
            this.m12 = s.m12;
            this.m20 = s.m20;
            this.m21 = s.m21;
            this.m22 = s.m22;
            this.m30 = 0;
            this.m31 = 0;
            this.m32 = 0;
        }
        
        public imat4x3(imat3x4 s) {
            this.m00 = s.m00;
            this.m01 = s.m01;
            this.m02 = s.m02;
            this.m10 = s.m10;
            this.m11 = s.m11;
            this.m12 = s.m12;
            this.m20 = s.m20;
            this.m21 = s.m21;
            this.m22 = s.m22;
            this.m30 = 0;
            this.m31 = 0;
            this.m32 = 0;
        }
        
        public imat4x3(imat4x2 s) {
            this.m00 = s.m00;
            this.m01 = s.m01;
            this.m02 = 0;
            this.m10 = s.m10;
            this.m11 = s.m11;
            this.m12 = 0;
            this.m20 = s.m20;
            this.m21 = s.m21;
            this.m22 = 0;
            this.m30 = s.m30;
            this.m31 = s.m31;
            this.m32 = 0;
        }
        
        public imat4x3(imat4 s) {
            this.m00 = s.m00;
            this.m01 = s.m01;
            this.m02 = s.m02;
            this.m10 = s.m10;
            this.m11 = s.m11;
            this.m12 = s.m12;
            this.m20 = s.m20;
            this.m21 = s.m21;
            this.m22 = s.m22;
            this.m30 = s.m30;
            this.m31 = s.m31;
            this.m32 = s.m32;
        }
        
        public int this[int i0, int i1] {
            get {
                if (((i0 < 0) 
                            || (3 < i0))) {
                    throw new System.IndexOutOfRangeException("i0");
                }
                if (((i1 < 0) 
                            || (2 < i1))) {
                    throw new System.IndexOutOfRangeException("i1");
                }
                
                unsafe
                {
                    fixed (Int32* v_ptr = &this.m00)
                    {
                        return *(v_ptr + (i1 + i0 * 3));
                    }
                };
            }
            set {
                if (((i0 < 0) 
                            || (3 < i0))) {
                    throw new System.IndexOutOfRangeException("i0");
                }
                if (((i1 < 0) 
                            || (2 < i1))) {
                    throw new System.IndexOutOfRangeException("i1");
                }
                
                unsafe
                {
                    fixed (Int32* v_ptr = &this.m00)
                    {
                        *(v_ptr  + (i1 + i0 * 3)) = value;
                    }
                };
            }
        }
        
        public ivec3 this[int i0] {
            get {
                return new ivec3(this[i0, 0], this[i0, 1], this[i0, 2]);
            }
            set {
                this[i0, 0] = value[0];
                this[i0, 1] = value[1];
                this[i0, 2] = value[2];
            }
        }
        
        public static imat4x3 operator+(imat4x3 v) {
            return new imat4x3(+v.m00, +v.m01, +v.m02, +v.m10, +v.m11, +v.m12, +v.m20, +v.m21, +v.m22, +v.m30, +v.m31, +v.m32);
        }
        
        public static imat4x3 operator-(imat4x3 v) {
            return new imat4x3(-v.m00, -v.m01, -v.m02, -v.m10, -v.m11, -v.m12, -v.m20, -v.m21, -v.m22, -v.m30, -v.m31, -v.m32);
        }
        
        public static imat4x3 operator~(imat4x3 v) {
            return new imat4x3(~v.m00, ~v.m01, ~v.m02, ~v.m10, ~v.m11, ~v.m12, ~v.m20, ~v.m21, ~v.m22, ~v.m30, ~v.m31, ~v.m32);
        }
        
        public static imat4x3 operator++(imat4x3 v) {
            return new imat4x3(++v.m00, ++v.m01, ++v.m02, ++v.m10, ++v.m11, ++v.m12, ++v.m20, ++v.m21, ++v.m22, ++v.m30, ++v.m31, ++v.m32);
        }
        
        public static imat4x3 operator--(imat4x3 v) {
            return new imat4x3(--v.m00, --v.m01, --v.m02, --v.m10, --v.m11, --v.m12, --v.m20, --v.m21, --v.m22, --v.m30, --v.m31, --v.m32);
        }
        
        public static imat4x3 operator+(int s, imat4x3 v2) {
            return new imat4x3(s + v2.m00, s + v2.m01, s + v2.m02, s + v2.m10, s + v2.m11, s + v2.m12, s + v2.m20, s + v2.m21, s + v2.m22, s + v2.m30, s + v2.m31, s + v2.m32);
        }
        
        public static imat4x3 operator-(int s, imat4x3 v2) {
            return new imat4x3(s - v2.m00, s - v2.m01, s - v2.m02, s - v2.m10, s - v2.m11, s - v2.m12, s - v2.m20, s - v2.m21, s - v2.m22, s - v2.m30, s - v2.m31, s - v2.m32);
        }
        
        public static imat4x3 operator*(int s, imat4x3 v2) {
            return new imat4x3(s * v2.m00, s * v2.m01, s * v2.m02, s * v2.m10, s * v2.m11, s * v2.m12, s * v2.m20, s * v2.m21, s * v2.m22, s * v2.m30, s * v2.m31, s * v2.m32);
        }
        
        public static imat4x3 operator/(int s, imat4x3 v2) {
            return new imat4x3(s / v2.m00, s / v2.m01, s / v2.m02, s / v2.m10, s / v2.m11, s / v2.m12, s / v2.m20, s / v2.m21, s / v2.m22, s / v2.m30, s / v2.m31, s / v2.m32);
        }
        
        public static imat4x3 operator%(int s, imat4x3 v2) {
            return new imat4x3(s % v2.m00, s % v2.m01, s % v2.m02, s % v2.m10, s % v2.m11, s % v2.m12, s % v2.m20, s % v2.m21, s % v2.m22, s % v2.m30, s % v2.m31, s % v2.m32);
        }
        
        public static imat4x3 operator&(int s, imat4x3 v2) {
            return new imat4x3(s & v2.m00, s & v2.m01, s & v2.m02, s & v2.m10, s & v2.m11, s & v2.m12, s & v2.m20, s & v2.m21, s & v2.m22, s & v2.m30, s & v2.m31, s & v2.m32);
        }
        
        public static imat4x3 operator|(int s, imat4x3 v2) {
            return new imat4x3(s | v2.m00, s | v2.m01, s | v2.m02, s | v2.m10, s | v2.m11, s | v2.m12, s | v2.m20, s | v2.m21, s | v2.m22, s | v2.m30, s | v2.m31, s | v2.m32);
        }
        
        public static imat4x3 operator^(int s, imat4x3 v2) {
            return new imat4x3(s ^ v2.m00, s ^ v2.m01, s ^ v2.m02, s ^ v2.m10, s ^ v2.m11, s ^ v2.m12, s ^ v2.m20, s ^ v2.m21, s ^ v2.m22, s ^ v2.m30, s ^ v2.m31, s ^ v2.m32);
        }
        
        public static imat4x3 operator+(imat4x3 v1, int s) {
            return new imat4x3(v1.m00 + s, v1.m01 + s, v1.m02 + s, v1.m10 + s, v1.m11 + s, v1.m12 + s, v1.m20 + s, v1.m21 + s, v1.m22 + s, v1.m30 + s, v1.m31 + s, v1.m32 + s);
        }
        
        public static imat4x3 operator-(imat4x3 v1, int s) {
            return new imat4x3(v1.m00 - s, v1.m01 - s, v1.m02 - s, v1.m10 - s, v1.m11 - s, v1.m12 - s, v1.m20 - s, v1.m21 - s, v1.m22 - s, v1.m30 - s, v1.m31 - s, v1.m32 - s);
        }
        
        public static imat4x3 operator*(imat4x3 v1, int s) {
            return new imat4x3(v1.m00 * s, v1.m01 * s, v1.m02 * s, v1.m10 * s, v1.m11 * s, v1.m12 * s, v1.m20 * s, v1.m21 * s, v1.m22 * s, v1.m30 * s, v1.m31 * s, v1.m32 * s);
        }
        
        public static imat4x3 operator/(imat4x3 v1, int s) {
            return new imat4x3(v1.m00 / s, v1.m01 / s, v1.m02 / s, v1.m10 / s, v1.m11 / s, v1.m12 / s, v1.m20 / s, v1.m21 / s, v1.m22 / s, v1.m30 / s, v1.m31 / s, v1.m32 / s);
        }
        
        public static imat4x3 operator%(imat4x3 v1, int s) {
            return new imat4x3(v1.m00 % s, v1.m01 % s, v1.m02 % s, v1.m10 % s, v1.m11 % s, v1.m12 % s, v1.m20 % s, v1.m21 % s, v1.m22 % s, v1.m30 % s, v1.m31 % s, v1.m32 % s);
        }
        
        public static imat4x3 operator&(imat4x3 v1, int s) {
            return new imat4x3(v1.m00 & s, v1.m01 & s, v1.m02 & s, v1.m10 & s, v1.m11 & s, v1.m12 & s, v1.m20 & s, v1.m21 & s, v1.m22 & s, v1.m30 & s, v1.m31 & s, v1.m32 & s);
        }
        
        public static imat4x3 operator|(imat4x3 v1, int s) {
            return new imat4x3(v1.m00 | s, v1.m01 | s, v1.m02 | s, v1.m10 | s, v1.m11 | s, v1.m12 | s, v1.m20 | s, v1.m21 | s, v1.m22 | s, v1.m30 | s, v1.m31 | s, v1.m32 | s);
        }
        
        public static imat4x3 operator^(imat4x3 v1, int s) {
            return new imat4x3(v1.m00 ^ s, v1.m01 ^ s, v1.m02 ^ s, v1.m10 ^ s, v1.m11 ^ s, v1.m12 ^ s, v1.m20 ^ s, v1.m21 ^ s, v1.m22 ^ s, v1.m30 ^ s, v1.m31 ^ s, v1.m32 ^ s);
        }
        
        public static imat4x3 operator+(imat4x3 v1, imat4x3 v2) {
            return new imat4x3(v1.m00 + v2.m00, v1.m01 + v2.m01, v1.m02 + v2.m02, v1.m10 + v2.m10, v1.m11 + v2.m11, v1.m12 + v2.m12, v1.m20 + v2.m20, v1.m21 + v2.m21, v1.m22 + v2.m22, v1.m30 + v2.m30, v1.m31 + v2.m31, v1.m32 + v2.m32);
        }
        
        public static imat4x3 operator-(imat4x3 v1, imat4x3 v2) {
            return new imat4x3(v1.m00 - v2.m00, v1.m01 - v2.m01, v1.m02 - v2.m02, v1.m10 - v2.m10, v1.m11 - v2.m11, v1.m12 - v2.m12, v1.m20 - v2.m20, v1.m21 - v2.m21, v1.m22 - v2.m22, v1.m30 - v2.m30, v1.m31 - v2.m31, v1.m32 - v2.m32);
        }
        
        public static imat4x3 operator/(imat4x3 v1, imat4x3 v2) {
            return new imat4x3(v1.m00 / v2.m00, v1.m01 / v2.m01, v1.m02 / v2.m02, v1.m10 / v2.m10, v1.m11 / v2.m11, v1.m12 / v2.m12, v1.m20 / v2.m20, v1.m21 / v2.m21, v1.m22 / v2.m22, v1.m30 / v2.m30, v1.m31 / v2.m31, v1.m32 / v2.m32);
        }
        
        public static imat4x3 operator%(imat4x3 v1, imat4x3 v2) {
            return new imat4x3(v1.m00 % v2.m00, v1.m01 % v2.m01, v1.m02 % v2.m02, v1.m10 % v2.m10, v1.m11 % v2.m11, v1.m12 % v2.m12, v1.m20 % v2.m20, v1.m21 % v2.m21, v1.m22 % v2.m22, v1.m30 % v2.m30, v1.m31 % v2.m31, v1.m32 % v2.m32);
        }
        
        public static imat4x3 operator&(imat4x3 v1, imat4x3 v2) {
            return new imat4x3(v1.m00 & v2.m00, v1.m01 & v2.m01, v1.m02 & v2.m02, v1.m10 & v2.m10, v1.m11 & v2.m11, v1.m12 & v2.m12, v1.m20 & v2.m20, v1.m21 & v2.m21, v1.m22 & v2.m22, v1.m30 & v2.m30, v1.m31 & v2.m31, v1.m32 & v2.m32);
        }
        
        public static imat4x3 operator|(imat4x3 v1, imat4x3 v2) {
            return new imat4x3(v1.m00 | v2.m00, v1.m01 | v2.m01, v1.m02 | v2.m02, v1.m10 | v2.m10, v1.m11 | v2.m11, v1.m12 | v2.m12, v1.m20 | v2.m20, v1.m21 | v2.m21, v1.m22 | v2.m22, v1.m30 | v2.m30, v1.m31 | v2.m31, v1.m32 | v2.m32);
        }
        
        public static imat4x3 operator^(imat4x3 v1, imat4x3 v2) {
            return new imat4x3(v1.m00 ^ v2.m00, v1.m01 ^ v2.m01, v1.m02 ^ v2.m02, v1.m10 ^ v2.m10, v1.m11 ^ v2.m11, v1.m12 ^ v2.m12, v1.m20 ^ v2.m20, v1.m21 ^ v2.m21, v1.m22 ^ v2.m22, v1.m30 ^ v2.m30, v1.m31 ^ v2.m31, v1.m32 ^ v2.m32);
        }
        
        public static bmat4x3 operator==(imat4x3 v1, imat4x3 v2) {
            return new bmat4x3(v1.m00 == v2.m00, v1.m01 == v2.m01, v1.m02 == v2.m02, v1.m10 == v2.m10, v1.m11 == v2.m11, v1.m12 == v2.m12, v1.m20 == v2.m20, v1.m21 == v2.m21, v1.m22 == v2.m22, v1.m30 == v2.m30, v1.m31 == v2.m31, v1.m32 == v2.m32);
        }
        
        public static bmat4x3 operator>(imat4x3 v1, imat4x3 v2) {
            return new bmat4x3(v1.m00 > v2.m00, v1.m01 > v2.m01, v1.m02 > v2.m02, v1.m10 > v2.m10, v1.m11 > v2.m11, v1.m12 > v2.m12, v1.m20 > v2.m20, v1.m21 > v2.m21, v1.m22 > v2.m22, v1.m30 > v2.m30, v1.m31 > v2.m31, v1.m32 > v2.m32);
        }
        
        public static bmat4x3 operator>=(imat4x3 v1, imat4x3 v2) {
            return new bmat4x3(v1.m00 >= v2.m00, v1.m01 >= v2.m01, v1.m02 >= v2.m02, v1.m10 >= v2.m10, v1.m11 >= v2.m11, v1.m12 >= v2.m12, v1.m20 >= v2.m20, v1.m21 >= v2.m21, v1.m22 >= v2.m22, v1.m30 >= v2.m30, v1.m31 >= v2.m31, v1.m32 >= v2.m32);
        }
        
        public static bmat4x3 operator<(imat4x3 v1, imat4x3 v2) {
            return new bmat4x3(v1.m00 < v2.m00, v1.m01 < v2.m01, v1.m02 < v2.m02, v1.m10 < v2.m10, v1.m11 < v2.m11, v1.m12 < v2.m12, v1.m20 < v2.m20, v1.m21 < v2.m21, v1.m22 < v2.m22, v1.m30 < v2.m30, v1.m31 < v2.m31, v1.m32 < v2.m32);
        }
        
        public static bmat4x3 operator<=(imat4x3 v1, imat4x3 v2) {
            return new bmat4x3(v1.m00 <= v2.m00, v1.m01 <= v2.m01, v1.m02 <= v2.m02, v1.m10 <= v2.m10, v1.m11 <= v2.m11, v1.m12 <= v2.m12, v1.m20 <= v2.m20, v1.m21 <= v2.m21, v1.m22 <= v2.m22, v1.m30 <= v2.m30, v1.m31 <= v2.m31, v1.m32 <= v2.m32);
        }
        
        public static bmat4x3 operator!=(imat4x3 v1, imat4x3 v2) {
            return new bmat4x3(v1.m00 != v2.m00, v1.m01 != v2.m01, v1.m02 != v2.m02, v1.m10 != v2.m10, v1.m11 != v2.m11, v1.m12 != v2.m12, v1.m20 != v2.m20, v1.m21 != v2.m21, v1.m22 != v2.m22, v1.m30 != v2.m30, v1.m31 != v2.m31, v1.m32 != v2.m32);
        }
        
        public static ivec3 operator*(ivec4 v0, imat4x3 m1) {
            return imath.InnerProduct1(ref v0, ref m1);
        }
        
        public static ivec4 operator*(imat4x3 m0, ivec3 v1) {
            return imath.InnerProduct1(ref m0, ref v1);
        }
        
        public static imat4x2 operator*(imat4x3 m0, imat3x2 m1) {
            return imath.InnerProduct1(ref m0, ref m1);
        }
        
        public static imat4x3 operator*(imat4x3 m0, imat3 m1) {
            return imath.InnerProduct1(ref m0, ref m1);
        }
        
        public static imat4 operator*(imat4x3 m0, imat3x4 m1) {
            return imath.InnerProduct1(ref m0, ref m1);
        }
        
        public override bool Equals(object obj) {
            return (obj is imat4x3) ? bmath.All(this == (imat4x3)obj) : false;
        }
        
        public override int GetHashCode() {
            return this.m00.GetHashCode() ^ this.m01.GetHashCode() ^ this.m02.GetHashCode() ^ this.m10.GetHashCode() ^ this.m11.GetHashCode() ^ this.m12.GetHashCode() ^ this.m20.GetHashCode() ^ this.m21.GetHashCode() ^ this.m22.GetHashCode() ^ this.m30.GetHashCode() ^ this.m31.GetHashCode() ^ this.m32.GetHashCode();
        }
        
        public override string ToString() {
            return "imat4x3(" + this.m00 + "," + this.m01 + "," + this.m02 + "," + this.m10 + "," + this.m11 + "," + this.m12 + "," + this.m20 + "," + this.m21 + "," + this.m22 + "," + this.m30 + "," + this.m31 + "," + this.m32 + ")";
        }
    }
    
    [DataContract()]
    [StructLayout(LayoutKind.Sequential)]
    public struct imat4 {
        
        [DataMember()]
        internal int m00;
        
        [DataMember()]
        internal int m01;
        
        [DataMember()]
        internal int m02;
        
        [DataMember()]
        internal int m03;
        
        [DataMember()]
        internal int m10;
        
        [DataMember()]
        internal int m11;
        
        [DataMember()]
        internal int m12;
        
        [DataMember()]
        internal int m13;
        
        [DataMember()]
        internal int m20;
        
        [DataMember()]
        internal int m21;
        
        [DataMember()]
        internal int m22;
        
        [DataMember()]
        internal int m23;
        
        [DataMember()]
        internal int m30;
        
        [DataMember()]
        internal int m31;
        
        [DataMember()]
        internal int m32;
        
        [DataMember()]
        internal int m33;
        
        public imat4(int s) {
            this.m00 = s;
            this.m01 = 0;
            this.m02 = 0;
            this.m03 = 0;
            this.m10 = 0;
            this.m11 = s;
            this.m12 = 0;
            this.m13 = 0;
            this.m20 = 0;
            this.m21 = 0;
            this.m22 = s;
            this.m23 = 0;
            this.m30 = 0;
            this.m31 = 0;
            this.m32 = 0;
            this.m33 = s;
        }
        
        public imat4(imat4 m00) {
            this.m00 = m00.m00;
            this.m01 = m00.m01;
            this.m02 = m00.m02;
            this.m03 = m00.m03;
            this.m10 = m00.m10;
            this.m11 = m00.m11;
            this.m12 = m00.m12;
            this.m13 = m00.m13;
            this.m20 = m00.m20;
            this.m21 = m00.m21;
            this.m22 = m00.m22;
            this.m23 = m00.m23;
            this.m30 = m00.m30;
            this.m31 = m00.m31;
            this.m32 = m00.m32;
            this.m33 = m00.m33;
        }
        
        public imat4(imat4x2 m00, imat4x2 m01) {
            this.m00 = m00.m00;
            this.m01 = m00.m01;
            this.m02 = m01.m00;
            this.m03 = m01.m01;
            this.m10 = m00.m10;
            this.m11 = m00.m11;
            this.m12 = m01.m10;
            this.m13 = m01.m11;
            this.m20 = m00.m20;
            this.m21 = m00.m21;
            this.m22 = m01.m20;
            this.m23 = m01.m21;
            this.m30 = m00.m30;
            this.m31 = m00.m31;
            this.m32 = m01.m30;
            this.m33 = m01.m31;
        }
        
        public imat4(imat2x4 m00, imat2x4 m10) {
            this.m00 = m00.m00;
            this.m01 = m00.m01;
            this.m02 = m00.m02;
            this.m03 = m00.m03;
            this.m10 = m00.m10;
            this.m11 = m00.m11;
            this.m12 = m00.m12;
            this.m13 = m00.m13;
            this.m20 = m10.m00;
            this.m21 = m10.m01;
            this.m22 = m10.m02;
            this.m23 = m10.m03;
            this.m30 = m10.m10;
            this.m31 = m10.m11;
            this.m32 = m10.m12;
            this.m33 = m10.m13;
        }
        
        public imat4(imat2 m00, imat2 m01, imat2 m10, imat2 m11) {
            this.m00 = m00.m00;
            this.m01 = m00.m01;
            this.m02 = m01.m00;
            this.m03 = m01.m01;
            this.m10 = m00.m10;
            this.m11 = m00.m11;
            this.m12 = m01.m10;
            this.m13 = m01.m11;
            this.m20 = m10.m00;
            this.m21 = m10.m01;
            this.m22 = m11.m00;
            this.m23 = m11.m01;
            this.m30 = m10.m10;
            this.m31 = m10.m11;
            this.m32 = m11.m10;
            this.m33 = m11.m11;
        }
        
        public imat4(imat3x4 m00, ivec4 m10) {
            this.m00 = m00.m00;
            this.m01 = m00.m01;
            this.m02 = m00.m02;
            this.m03 = m00.m03;
            this.m10 = m00.m10;
            this.m11 = m00.m11;
            this.m12 = m00.m12;
            this.m13 = m00.m13;
            this.m20 = m00.m20;
            this.m21 = m00.m21;
            this.m22 = m00.m22;
            this.m23 = m00.m23;
            this.m30 = m10.v0;
            this.m31 = m10.v1;
            this.m32 = m10.v2;
            this.m33 = m10.v3;
        }
        
        public imat4(imat3x2 m00, imat3x2 m01, ivec2 m10, ivec2 m11) {
            this.m00 = m00.m00;
            this.m01 = m00.m01;
            this.m02 = m01.m00;
            this.m03 = m01.m01;
            this.m10 = m00.m10;
            this.m11 = m00.m11;
            this.m12 = m01.m10;
            this.m13 = m01.m11;
            this.m20 = m00.m20;
            this.m21 = m00.m21;
            this.m22 = m01.m20;
            this.m23 = m01.m21;
            this.m30 = m10.v0;
            this.m31 = m10.v1;
            this.m32 = m11.v0;
            this.m33 = m11.v1;
        }
        
        public imat4(ivec4 m00, imat3x4 m10) {
            this.m00 = m00.v0;
            this.m01 = m00.v1;
            this.m02 = m00.v2;
            this.m03 = m00.v3;
            this.m10 = m10.m00;
            this.m11 = m10.m01;
            this.m12 = m10.m02;
            this.m13 = m10.m03;
            this.m20 = m10.m10;
            this.m21 = m10.m11;
            this.m22 = m10.m12;
            this.m23 = m10.m13;
            this.m30 = m10.m20;
            this.m31 = m10.m21;
            this.m32 = m10.m22;
            this.m33 = m10.m23;
        }
        
        public imat4(ivec2 m00, ivec2 m01, imat3x2 m10, imat3x2 m11) {
            this.m00 = m00.v0;
            this.m01 = m00.v1;
            this.m02 = m01.v0;
            this.m03 = m01.v1;
            this.m10 = m10.m00;
            this.m11 = m10.m01;
            this.m12 = m11.m00;
            this.m13 = m11.m01;
            this.m20 = m10.m10;
            this.m21 = m10.m11;
            this.m22 = m11.m10;
            this.m23 = m11.m11;
            this.m30 = m10.m20;
            this.m31 = m10.m21;
            this.m32 = m11.m20;
            this.m33 = m11.m21;
        }
        
        public imat4(imat2x4 m00, ivec4 m10, ivec4 m20) {
            this.m00 = m00.m00;
            this.m01 = m00.m01;
            this.m02 = m00.m02;
            this.m03 = m00.m03;
            this.m10 = m00.m10;
            this.m11 = m00.m11;
            this.m12 = m00.m12;
            this.m13 = m00.m13;
            this.m20 = m10.v0;
            this.m21 = m10.v1;
            this.m22 = m10.v2;
            this.m23 = m10.v3;
            this.m30 = m20.v0;
            this.m31 = m20.v1;
            this.m32 = m20.v2;
            this.m33 = m20.v3;
        }
        
        public imat4(imat2 m00, imat2 m01, ivec2 m10, ivec2 m11, ivec2 m20, ivec2 m21) {
            this.m00 = m00.m00;
            this.m01 = m00.m01;
            this.m02 = m01.m00;
            this.m03 = m01.m01;
            this.m10 = m00.m10;
            this.m11 = m00.m11;
            this.m12 = m01.m10;
            this.m13 = m01.m11;
            this.m20 = m10.v0;
            this.m21 = m10.v1;
            this.m22 = m11.v0;
            this.m23 = m11.v1;
            this.m30 = m20.v0;
            this.m31 = m20.v1;
            this.m32 = m21.v0;
            this.m33 = m21.v1;
        }
        
        public imat4(ivec4 m00, imat2x4 m10, ivec4 m20) {
            this.m00 = m00.v0;
            this.m01 = m00.v1;
            this.m02 = m00.v2;
            this.m03 = m00.v3;
            this.m10 = m10.m00;
            this.m11 = m10.m01;
            this.m12 = m10.m02;
            this.m13 = m10.m03;
            this.m20 = m10.m10;
            this.m21 = m10.m11;
            this.m22 = m10.m12;
            this.m23 = m10.m13;
            this.m30 = m20.v0;
            this.m31 = m20.v1;
            this.m32 = m20.v2;
            this.m33 = m20.v3;
        }
        
        public imat4(ivec2 m00, ivec2 m01, imat2 m10, imat2 m11, ivec2 m20, ivec2 m21) {
            this.m00 = m00.v0;
            this.m01 = m00.v1;
            this.m02 = m01.v0;
            this.m03 = m01.v1;
            this.m10 = m10.m00;
            this.m11 = m10.m01;
            this.m12 = m11.m00;
            this.m13 = m11.m01;
            this.m20 = m10.m10;
            this.m21 = m10.m11;
            this.m22 = m11.m10;
            this.m23 = m11.m11;
            this.m30 = m20.v0;
            this.m31 = m20.v1;
            this.m32 = m21.v0;
            this.m33 = m21.v1;
        }
        
        public imat4(ivec4 m00, ivec4 m10, imat2x4 m20) {
            this.m00 = m00.v0;
            this.m01 = m00.v1;
            this.m02 = m00.v2;
            this.m03 = m00.v3;
            this.m10 = m10.v0;
            this.m11 = m10.v1;
            this.m12 = m10.v2;
            this.m13 = m10.v3;
            this.m20 = m20.m00;
            this.m21 = m20.m01;
            this.m22 = m20.m02;
            this.m23 = m20.m03;
            this.m30 = m20.m10;
            this.m31 = m20.m11;
            this.m32 = m20.m12;
            this.m33 = m20.m13;
        }
        
        public imat4(ivec2 m00, ivec2 m01, ivec2 m10, ivec2 m11, imat2 m20, imat2 m21) {
            this.m00 = m00.v0;
            this.m01 = m00.v1;
            this.m02 = m01.v0;
            this.m03 = m01.v1;
            this.m10 = m10.v0;
            this.m11 = m10.v1;
            this.m12 = m11.v0;
            this.m13 = m11.v1;
            this.m20 = m20.m00;
            this.m21 = m20.m01;
            this.m22 = m21.m00;
            this.m23 = m21.m01;
            this.m30 = m20.m10;
            this.m31 = m20.m11;
            this.m32 = m21.m10;
            this.m33 = m21.m11;
        }
        
        public imat4(ivec4 m00, ivec4 m10, ivec4 m20, ivec4 m30) {
            this.m00 = m00.v0;
            this.m01 = m00.v1;
            this.m02 = m00.v2;
            this.m03 = m00.v3;
            this.m10 = m10.v0;
            this.m11 = m10.v1;
            this.m12 = m10.v2;
            this.m13 = m10.v3;
            this.m20 = m20.v0;
            this.m21 = m20.v1;
            this.m22 = m20.v2;
            this.m23 = m20.v3;
            this.m30 = m30.v0;
            this.m31 = m30.v1;
            this.m32 = m30.v2;
            this.m33 = m30.v3;
        }
        
        public imat4(ivec2 m00, ivec2 m01, ivec2 m10, ivec2 m11, ivec2 m20, ivec2 m21, ivec2 m30, ivec2 m31) {
            this.m00 = m00.v0;
            this.m01 = m00.v1;
            this.m02 = m01.v0;
            this.m03 = m01.v1;
            this.m10 = m10.v0;
            this.m11 = m10.v1;
            this.m12 = m11.v0;
            this.m13 = m11.v1;
            this.m20 = m20.v0;
            this.m21 = m20.v1;
            this.m22 = m21.v0;
            this.m23 = m21.v1;
            this.m30 = m30.v0;
            this.m31 = m30.v1;
            this.m32 = m31.v0;
            this.m33 = m31.v1;
        }
        
        public imat4(ivec3 m00, int m01, ivec3 m10, int m11, ivec3 m20, int m21, ivec3 m30, int m31) {
            this.m00 = m00.v0;
            this.m01 = m00.v1;
            this.m02 = m00.v2;
            this.m03 = m01;
            this.m10 = m10.v0;
            this.m11 = m10.v1;
            this.m12 = m10.v2;
            this.m13 = m11;
            this.m20 = m20.v0;
            this.m21 = m20.v1;
            this.m22 = m20.v2;
            this.m23 = m21;
            this.m30 = m30.v0;
            this.m31 = m30.v1;
            this.m32 = m30.v2;
            this.m33 = m31;
        }
        
        public imat4(int m00, ivec3 m01, int m10, ivec3 m11, int m20, ivec3 m21, int m30, ivec3 m31) {
            this.m00 = m00;
            this.m01 = m01.v0;
            this.m02 = m01.v1;
            this.m03 = m01.v2;
            this.m10 = m10;
            this.m11 = m11.v0;
            this.m12 = m11.v1;
            this.m13 = m11.v2;
            this.m20 = m20;
            this.m21 = m21.v0;
            this.m22 = m21.v1;
            this.m23 = m21.v2;
            this.m30 = m30;
            this.m31 = m31.v0;
            this.m32 = m31.v1;
            this.m33 = m31.v2;
        }
        
        public imat4(ivec2 m00, int m01, int m02, ivec2 m10, int m11, int m12, ivec2 m20, int m21, int m22, ivec2 m30, int m31, int m32) {
            this.m00 = m00.v0;
            this.m01 = m00.v1;
            this.m02 = m01;
            this.m03 = m02;
            this.m10 = m10.v0;
            this.m11 = m10.v1;
            this.m12 = m11;
            this.m13 = m12;
            this.m20 = m20.v0;
            this.m21 = m20.v1;
            this.m22 = m21;
            this.m23 = m22;
            this.m30 = m30.v0;
            this.m31 = m30.v1;
            this.m32 = m31;
            this.m33 = m32;
        }
        
        public imat4(int m00, ivec2 m01, int m02, int m10, ivec2 m11, int m12, int m20, ivec2 m21, int m22, int m30, ivec2 m31, int m32) {
            this.m00 = m00;
            this.m01 = m01.v0;
            this.m02 = m01.v1;
            this.m03 = m02;
            this.m10 = m10;
            this.m11 = m11.v0;
            this.m12 = m11.v1;
            this.m13 = m12;
            this.m20 = m20;
            this.m21 = m21.v0;
            this.m22 = m21.v1;
            this.m23 = m22;
            this.m30 = m30;
            this.m31 = m31.v0;
            this.m32 = m31.v1;
            this.m33 = m32;
        }
        
        public imat4(int m00, int m01, ivec2 m02, int m10, int m11, ivec2 m12, int m20, int m21, ivec2 m22, int m30, int m31, ivec2 m32) {
            this.m00 = m00;
            this.m01 = m01;
            this.m02 = m02.v0;
            this.m03 = m02.v1;
            this.m10 = m10;
            this.m11 = m11;
            this.m12 = m12.v0;
            this.m13 = m12.v1;
            this.m20 = m20;
            this.m21 = m21;
            this.m22 = m22.v0;
            this.m23 = m22.v1;
            this.m30 = m30;
            this.m31 = m31;
            this.m32 = m32.v0;
            this.m33 = m32.v1;
        }
        
        public imat4(
                    int m00, 
                    int m01, 
                    int m02, 
                    int m03, 
                    int m10, 
                    int m11, 
                    int m12, 
                    int m13, 
                    int m20, 
                    int m21, 
                    int m22, 
                    int m23, 
                    int m30, 
                    int m31, 
                    int m32, 
                    int m33) {
            this.m00 = m00;
            this.m01 = m01;
            this.m02 = m02;
            this.m03 = m03;
            this.m10 = m10;
            this.m11 = m11;
            this.m12 = m12;
            this.m13 = m13;
            this.m20 = m20;
            this.m21 = m21;
            this.m22 = m22;
            this.m23 = m23;
            this.m30 = m30;
            this.m31 = m31;
            this.m32 = m32;
            this.m33 = m33;
        }
        
        public imat4(imat2 s) {
            this.m00 = s.m00;
            this.m01 = s.m01;
            this.m02 = 0;
            this.m03 = 0;
            this.m10 = s.m10;
            this.m11 = s.m11;
            this.m12 = 0;
            this.m13 = 0;
            this.m20 = 0;
            this.m21 = 0;
            this.m22 = 0;
            this.m23 = 0;
            this.m30 = 0;
            this.m31 = 0;
            this.m32 = 0;
            this.m33 = 0;
        }
        
        public imat4(imat2x3 s) {
            this.m00 = s.m00;
            this.m01 = s.m01;
            this.m02 = s.m02;
            this.m03 = 0;
            this.m10 = s.m10;
            this.m11 = s.m11;
            this.m12 = s.m12;
            this.m13 = 0;
            this.m20 = 0;
            this.m21 = 0;
            this.m22 = 0;
            this.m23 = 0;
            this.m30 = 0;
            this.m31 = 0;
            this.m32 = 0;
            this.m33 = 0;
        }
        
        public imat4(imat2x4 s) {
            this.m00 = s.m00;
            this.m01 = s.m01;
            this.m02 = s.m02;
            this.m03 = s.m03;
            this.m10 = s.m10;
            this.m11 = s.m11;
            this.m12 = s.m12;
            this.m13 = s.m13;
            this.m20 = 0;
            this.m21 = 0;
            this.m22 = 0;
            this.m23 = 0;
            this.m30 = 0;
            this.m31 = 0;
            this.m32 = 0;
            this.m33 = 0;
        }
        
        public imat4(imat3x2 s) {
            this.m00 = s.m00;
            this.m01 = s.m01;
            this.m02 = 0;
            this.m03 = 0;
            this.m10 = s.m10;
            this.m11 = s.m11;
            this.m12 = 0;
            this.m13 = 0;
            this.m20 = s.m20;
            this.m21 = s.m21;
            this.m22 = 0;
            this.m23 = 0;
            this.m30 = 0;
            this.m31 = 0;
            this.m32 = 0;
            this.m33 = 0;
        }
        
        public imat4(imat3 s) {
            this.m00 = s.m00;
            this.m01 = s.m01;
            this.m02 = s.m02;
            this.m03 = 0;
            this.m10 = s.m10;
            this.m11 = s.m11;
            this.m12 = s.m12;
            this.m13 = 0;
            this.m20 = s.m20;
            this.m21 = s.m21;
            this.m22 = s.m22;
            this.m23 = 0;
            this.m30 = 0;
            this.m31 = 0;
            this.m32 = 0;
            this.m33 = 0;
        }
        
        public imat4(imat3x4 s) {
            this.m00 = s.m00;
            this.m01 = s.m01;
            this.m02 = s.m02;
            this.m03 = s.m03;
            this.m10 = s.m10;
            this.m11 = s.m11;
            this.m12 = s.m12;
            this.m13 = s.m13;
            this.m20 = s.m20;
            this.m21 = s.m21;
            this.m22 = s.m22;
            this.m23 = s.m23;
            this.m30 = 0;
            this.m31 = 0;
            this.m32 = 0;
            this.m33 = 0;
        }
        
        public imat4(imat4x2 s) {
            this.m00 = s.m00;
            this.m01 = s.m01;
            this.m02 = 0;
            this.m03 = 0;
            this.m10 = s.m10;
            this.m11 = s.m11;
            this.m12 = 0;
            this.m13 = 0;
            this.m20 = s.m20;
            this.m21 = s.m21;
            this.m22 = 0;
            this.m23 = 0;
            this.m30 = s.m30;
            this.m31 = s.m31;
            this.m32 = 0;
            this.m33 = 0;
        }
        
        public imat4(imat4x3 s) {
            this.m00 = s.m00;
            this.m01 = s.m01;
            this.m02 = s.m02;
            this.m03 = 0;
            this.m10 = s.m10;
            this.m11 = s.m11;
            this.m12 = s.m12;
            this.m13 = 0;
            this.m20 = s.m20;
            this.m21 = s.m21;
            this.m22 = s.m22;
            this.m23 = 0;
            this.m30 = s.m30;
            this.m31 = s.m31;
            this.m32 = s.m32;
            this.m33 = 0;
        }
        
        public int this[int i0, int i1] {
            get {
                if (((i0 < 0) 
                            || (3 < i0))) {
                    throw new System.IndexOutOfRangeException("i0");
                }
                if (((i1 < 0) 
                            || (3 < i1))) {
                    throw new System.IndexOutOfRangeException("i1");
                }
                
                unsafe
                {
                    fixed (Int32* v_ptr = &this.m00)
                    {
                        return *(v_ptr + (i1 + i0 * 4));
                    }
                };
            }
            set {
                if (((i0 < 0) 
                            || (3 < i0))) {
                    throw new System.IndexOutOfRangeException("i0");
                }
                if (((i1 < 0) 
                            || (3 < i1))) {
                    throw new System.IndexOutOfRangeException("i1");
                }
                
                unsafe
                {
                    fixed (Int32* v_ptr = &this.m00)
                    {
                        *(v_ptr  + (i1 + i0 * 4)) = value;
                    }
                };
            }
        }
        
        public ivec4 this[int i0] {
            get {
                return new ivec4(this[i0, 0], this[i0, 1], this[i0, 2], this[i0, 3]);
            }
            set {
                this[i0, 0] = value[0];
                this[i0, 1] = value[1];
                this[i0, 2] = value[2];
                this[i0, 3] = value[3];
            }
        }
        
        public static imat4 operator+(imat4 v) {
            return new imat4(+v.m00, +v.m01, +v.m02, +v.m03, +v.m10, +v.m11, +v.m12, +v.m13, +v.m20, +v.m21, +v.m22, +v.m23, +v.m30, +v.m31, +v.m32, +v.m33);
        }
        
        public static imat4 operator-(imat4 v) {
            return new imat4(-v.m00, -v.m01, -v.m02, -v.m03, -v.m10, -v.m11, -v.m12, -v.m13, -v.m20, -v.m21, -v.m22, -v.m23, -v.m30, -v.m31, -v.m32, -v.m33);
        }
        
        public static imat4 operator~(imat4 v) {
            return new imat4(~v.m00, ~v.m01, ~v.m02, ~v.m03, ~v.m10, ~v.m11, ~v.m12, ~v.m13, ~v.m20, ~v.m21, ~v.m22, ~v.m23, ~v.m30, ~v.m31, ~v.m32, ~v.m33);
        }
        
        public static imat4 operator++(imat4 v) {
            return new imat4(++v.m00, ++v.m01, ++v.m02, ++v.m03, ++v.m10, ++v.m11, ++v.m12, ++v.m13, ++v.m20, ++v.m21, ++v.m22, ++v.m23, ++v.m30, ++v.m31, ++v.m32, ++v.m33);
        }
        
        public static imat4 operator--(imat4 v) {
            return new imat4(--v.m00, --v.m01, --v.m02, --v.m03, --v.m10, --v.m11, --v.m12, --v.m13, --v.m20, --v.m21, --v.m22, --v.m23, --v.m30, --v.m31, --v.m32, --v.m33);
        }
        
        public static imat4 operator+(int s, imat4 v2) {
            return new imat4(s + v2.m00, s + v2.m01, s + v2.m02, s + v2.m03, s + v2.m10, s + v2.m11, s + v2.m12, s + v2.m13, s + v2.m20, s + v2.m21, s + v2.m22, s + v2.m23, s + v2.m30, s + v2.m31, s + v2.m32, s + v2.m33);
        }
        
        public static imat4 operator-(int s, imat4 v2) {
            return new imat4(s - v2.m00, s - v2.m01, s - v2.m02, s - v2.m03, s - v2.m10, s - v2.m11, s - v2.m12, s - v2.m13, s - v2.m20, s - v2.m21, s - v2.m22, s - v2.m23, s - v2.m30, s - v2.m31, s - v2.m32, s - v2.m33);
        }
        
        public static imat4 operator*(int s, imat4 v2) {
            return new imat4(s * v2.m00, s * v2.m01, s * v2.m02, s * v2.m03, s * v2.m10, s * v2.m11, s * v2.m12, s * v2.m13, s * v2.m20, s * v2.m21, s * v2.m22, s * v2.m23, s * v2.m30, s * v2.m31, s * v2.m32, s * v2.m33);
        }
        
        public static imat4 operator/(int s, imat4 v2) {
            return new imat4(s / v2.m00, s / v2.m01, s / v2.m02, s / v2.m03, s / v2.m10, s / v2.m11, s / v2.m12, s / v2.m13, s / v2.m20, s / v2.m21, s / v2.m22, s / v2.m23, s / v2.m30, s / v2.m31, s / v2.m32, s / v2.m33);
        }
        
        public static imat4 operator%(int s, imat4 v2) {
            return new imat4(s % v2.m00, s % v2.m01, s % v2.m02, s % v2.m03, s % v2.m10, s % v2.m11, s % v2.m12, s % v2.m13, s % v2.m20, s % v2.m21, s % v2.m22, s % v2.m23, s % v2.m30, s % v2.m31, s % v2.m32, s % v2.m33);
        }
        
        public static imat4 operator&(int s, imat4 v2) {
            return new imat4(s & v2.m00, s & v2.m01, s & v2.m02, s & v2.m03, s & v2.m10, s & v2.m11, s & v2.m12, s & v2.m13, s & v2.m20, s & v2.m21, s & v2.m22, s & v2.m23, s & v2.m30, s & v2.m31, s & v2.m32, s & v2.m33);
        }
        
        public static imat4 operator|(int s, imat4 v2) {
            return new imat4(s | v2.m00, s | v2.m01, s | v2.m02, s | v2.m03, s | v2.m10, s | v2.m11, s | v2.m12, s | v2.m13, s | v2.m20, s | v2.m21, s | v2.m22, s | v2.m23, s | v2.m30, s | v2.m31, s | v2.m32, s | v2.m33);
        }
        
        public static imat4 operator^(int s, imat4 v2) {
            return new imat4(s ^ v2.m00, s ^ v2.m01, s ^ v2.m02, s ^ v2.m03, s ^ v2.m10, s ^ v2.m11, s ^ v2.m12, s ^ v2.m13, s ^ v2.m20, s ^ v2.m21, s ^ v2.m22, s ^ v2.m23, s ^ v2.m30, s ^ v2.m31, s ^ v2.m32, s ^ v2.m33);
        }
        
        public static imat4 operator+(imat4 v1, int s) {
            return new imat4(v1.m00 + s, v1.m01 + s, v1.m02 + s, v1.m03 + s, v1.m10 + s, v1.m11 + s, v1.m12 + s, v1.m13 + s, v1.m20 + s, v1.m21 + s, v1.m22 + s, v1.m23 + s, v1.m30 + s, v1.m31 + s, v1.m32 + s, v1.m33 + s);
        }
        
        public static imat4 operator-(imat4 v1, int s) {
            return new imat4(v1.m00 - s, v1.m01 - s, v1.m02 - s, v1.m03 - s, v1.m10 - s, v1.m11 - s, v1.m12 - s, v1.m13 - s, v1.m20 - s, v1.m21 - s, v1.m22 - s, v1.m23 - s, v1.m30 - s, v1.m31 - s, v1.m32 - s, v1.m33 - s);
        }
        
        public static imat4 operator*(imat4 v1, int s) {
            return new imat4(v1.m00 * s, v1.m01 * s, v1.m02 * s, v1.m03 * s, v1.m10 * s, v1.m11 * s, v1.m12 * s, v1.m13 * s, v1.m20 * s, v1.m21 * s, v1.m22 * s, v1.m23 * s, v1.m30 * s, v1.m31 * s, v1.m32 * s, v1.m33 * s);
        }
        
        public static imat4 operator/(imat4 v1, int s) {
            return new imat4(v1.m00 / s, v1.m01 / s, v1.m02 / s, v1.m03 / s, v1.m10 / s, v1.m11 / s, v1.m12 / s, v1.m13 / s, v1.m20 / s, v1.m21 / s, v1.m22 / s, v1.m23 / s, v1.m30 / s, v1.m31 / s, v1.m32 / s, v1.m33 / s);
        }
        
        public static imat4 operator%(imat4 v1, int s) {
            return new imat4(v1.m00 % s, v1.m01 % s, v1.m02 % s, v1.m03 % s, v1.m10 % s, v1.m11 % s, v1.m12 % s, v1.m13 % s, v1.m20 % s, v1.m21 % s, v1.m22 % s, v1.m23 % s, v1.m30 % s, v1.m31 % s, v1.m32 % s, v1.m33 % s);
        }
        
        public static imat4 operator&(imat4 v1, int s) {
            return new imat4(v1.m00 & s, v1.m01 & s, v1.m02 & s, v1.m03 & s, v1.m10 & s, v1.m11 & s, v1.m12 & s, v1.m13 & s, v1.m20 & s, v1.m21 & s, v1.m22 & s, v1.m23 & s, v1.m30 & s, v1.m31 & s, v1.m32 & s, v1.m33 & s);
        }
        
        public static imat4 operator|(imat4 v1, int s) {
            return new imat4(v1.m00 | s, v1.m01 | s, v1.m02 | s, v1.m03 | s, v1.m10 | s, v1.m11 | s, v1.m12 | s, v1.m13 | s, v1.m20 | s, v1.m21 | s, v1.m22 | s, v1.m23 | s, v1.m30 | s, v1.m31 | s, v1.m32 | s, v1.m33 | s);
        }
        
        public static imat4 operator^(imat4 v1, int s) {
            return new imat4(v1.m00 ^ s, v1.m01 ^ s, v1.m02 ^ s, v1.m03 ^ s, v1.m10 ^ s, v1.m11 ^ s, v1.m12 ^ s, v1.m13 ^ s, v1.m20 ^ s, v1.m21 ^ s, v1.m22 ^ s, v1.m23 ^ s, v1.m30 ^ s, v1.m31 ^ s, v1.m32 ^ s, v1.m33 ^ s);
        }
        
        public static imat4 operator+(imat4 v1, imat4 v2) {
            return new imat4(v1.m00 + v2.m00, v1.m01 + v2.m01, v1.m02 + v2.m02, v1.m03 + v2.m03, v1.m10 + v2.m10, v1.m11 + v2.m11, v1.m12 + v2.m12, v1.m13 + v2.m13, v1.m20 + v2.m20, v1.m21 + v2.m21, v1.m22 + v2.m22, v1.m23 + v2.m23, v1.m30 + v2.m30, v1.m31 + v2.m31, v1.m32 + v2.m32, v1.m33 + v2.m33);
        }
        
        public static imat4 operator-(imat4 v1, imat4 v2) {
            return new imat4(v1.m00 - v2.m00, v1.m01 - v2.m01, v1.m02 - v2.m02, v1.m03 - v2.m03, v1.m10 - v2.m10, v1.m11 - v2.m11, v1.m12 - v2.m12, v1.m13 - v2.m13, v1.m20 - v2.m20, v1.m21 - v2.m21, v1.m22 - v2.m22, v1.m23 - v2.m23, v1.m30 - v2.m30, v1.m31 - v2.m31, v1.m32 - v2.m32, v1.m33 - v2.m33);
        }
        
        public static imat4 operator/(imat4 v1, imat4 v2) {
            return new imat4(v1.m00 / v2.m00, v1.m01 / v2.m01, v1.m02 / v2.m02, v1.m03 / v2.m03, v1.m10 / v2.m10, v1.m11 / v2.m11, v1.m12 / v2.m12, v1.m13 / v2.m13, v1.m20 / v2.m20, v1.m21 / v2.m21, v1.m22 / v2.m22, v1.m23 / v2.m23, v1.m30 / v2.m30, v1.m31 / v2.m31, v1.m32 / v2.m32, v1.m33 / v2.m33);
        }
        
        public static imat4 operator%(imat4 v1, imat4 v2) {
            return new imat4(v1.m00 % v2.m00, v1.m01 % v2.m01, v1.m02 % v2.m02, v1.m03 % v2.m03, v1.m10 % v2.m10, v1.m11 % v2.m11, v1.m12 % v2.m12, v1.m13 % v2.m13, v1.m20 % v2.m20, v1.m21 % v2.m21, v1.m22 % v2.m22, v1.m23 % v2.m23, v1.m30 % v2.m30, v1.m31 % v2.m31, v1.m32 % v2.m32, v1.m33 % v2.m33);
        }
        
        public static imat4 operator&(imat4 v1, imat4 v2) {
            return new imat4(v1.m00 & v2.m00, v1.m01 & v2.m01, v1.m02 & v2.m02, v1.m03 & v2.m03, v1.m10 & v2.m10, v1.m11 & v2.m11, v1.m12 & v2.m12, v1.m13 & v2.m13, v1.m20 & v2.m20, v1.m21 & v2.m21, v1.m22 & v2.m22, v1.m23 & v2.m23, v1.m30 & v2.m30, v1.m31 & v2.m31, v1.m32 & v2.m32, v1.m33 & v2.m33);
        }
        
        public static imat4 operator|(imat4 v1, imat4 v2) {
            return new imat4(v1.m00 | v2.m00, v1.m01 | v2.m01, v1.m02 | v2.m02, v1.m03 | v2.m03, v1.m10 | v2.m10, v1.m11 | v2.m11, v1.m12 | v2.m12, v1.m13 | v2.m13, v1.m20 | v2.m20, v1.m21 | v2.m21, v1.m22 | v2.m22, v1.m23 | v2.m23, v1.m30 | v2.m30, v1.m31 | v2.m31, v1.m32 | v2.m32, v1.m33 | v2.m33);
        }
        
        public static imat4 operator^(imat4 v1, imat4 v2) {
            return new imat4(v1.m00 ^ v2.m00, v1.m01 ^ v2.m01, v1.m02 ^ v2.m02, v1.m03 ^ v2.m03, v1.m10 ^ v2.m10, v1.m11 ^ v2.m11, v1.m12 ^ v2.m12, v1.m13 ^ v2.m13, v1.m20 ^ v2.m20, v1.m21 ^ v2.m21, v1.m22 ^ v2.m22, v1.m23 ^ v2.m23, v1.m30 ^ v2.m30, v1.m31 ^ v2.m31, v1.m32 ^ v2.m32, v1.m33 ^ v2.m33);
        }
        
        public static bmat4 operator==(imat4 v1, imat4 v2) {
            return new bmat4(v1.m00 == v2.m00, v1.m01 == v2.m01, v1.m02 == v2.m02, v1.m03 == v2.m03, v1.m10 == v2.m10, v1.m11 == v2.m11, v1.m12 == v2.m12, v1.m13 == v2.m13, v1.m20 == v2.m20, v1.m21 == v2.m21, v1.m22 == v2.m22, v1.m23 == v2.m23, v1.m30 == v2.m30, v1.m31 == v2.m31, v1.m32 == v2.m32, v1.m33 == v2.m33);
        }
        
        public static bmat4 operator>(imat4 v1, imat4 v2) {
            return new bmat4(v1.m00 > v2.m00, v1.m01 > v2.m01, v1.m02 > v2.m02, v1.m03 > v2.m03, v1.m10 > v2.m10, v1.m11 > v2.m11, v1.m12 > v2.m12, v1.m13 > v2.m13, v1.m20 > v2.m20, v1.m21 > v2.m21, v1.m22 > v2.m22, v1.m23 > v2.m23, v1.m30 > v2.m30, v1.m31 > v2.m31, v1.m32 > v2.m32, v1.m33 > v2.m33);
        }
        
        public static bmat4 operator>=(imat4 v1, imat4 v2) {
            return new bmat4(v1.m00 >= v2.m00, v1.m01 >= v2.m01, v1.m02 >= v2.m02, v1.m03 >= v2.m03, v1.m10 >= v2.m10, v1.m11 >= v2.m11, v1.m12 >= v2.m12, v1.m13 >= v2.m13, v1.m20 >= v2.m20, v1.m21 >= v2.m21, v1.m22 >= v2.m22, v1.m23 >= v2.m23, v1.m30 >= v2.m30, v1.m31 >= v2.m31, v1.m32 >= v2.m32, v1.m33 >= v2.m33);
        }
        
        public static bmat4 operator<(imat4 v1, imat4 v2) {
            return new bmat4(v1.m00 < v2.m00, v1.m01 < v2.m01, v1.m02 < v2.m02, v1.m03 < v2.m03, v1.m10 < v2.m10, v1.m11 < v2.m11, v1.m12 < v2.m12, v1.m13 < v2.m13, v1.m20 < v2.m20, v1.m21 < v2.m21, v1.m22 < v2.m22, v1.m23 < v2.m23, v1.m30 < v2.m30, v1.m31 < v2.m31, v1.m32 < v2.m32, v1.m33 < v2.m33);
        }
        
        public static bmat4 operator<=(imat4 v1, imat4 v2) {
            return new bmat4(v1.m00 <= v2.m00, v1.m01 <= v2.m01, v1.m02 <= v2.m02, v1.m03 <= v2.m03, v1.m10 <= v2.m10, v1.m11 <= v2.m11, v1.m12 <= v2.m12, v1.m13 <= v2.m13, v1.m20 <= v2.m20, v1.m21 <= v2.m21, v1.m22 <= v2.m22, v1.m23 <= v2.m23, v1.m30 <= v2.m30, v1.m31 <= v2.m31, v1.m32 <= v2.m32, v1.m33 <= v2.m33);
        }
        
        public static bmat4 operator!=(imat4 v1, imat4 v2) {
            return new bmat4(v1.m00 != v2.m00, v1.m01 != v2.m01, v1.m02 != v2.m02, v1.m03 != v2.m03, v1.m10 != v2.m10, v1.m11 != v2.m11, v1.m12 != v2.m12, v1.m13 != v2.m13, v1.m20 != v2.m20, v1.m21 != v2.m21, v1.m22 != v2.m22, v1.m23 != v2.m23, v1.m30 != v2.m30, v1.m31 != v2.m31, v1.m32 != v2.m32, v1.m33 != v2.m33);
        }
        
        public static ivec4 operator*(ivec4 v0, imat4 m1) {
            return imath.InnerProduct1(ref v0, ref m1);
        }
        
        public static ivec4 operator*(imat4 m0, ivec4 v1) {
            return imath.InnerProduct1(ref m0, ref v1);
        }
        
        public static imat4x2 operator*(imat4 m0, imat4x2 m1) {
            return imath.InnerProduct1(ref m0, ref m1);
        }
        
        public static imat4x3 operator*(imat4 m0, imat4x3 m1) {
            return imath.InnerProduct1(ref m0, ref m1);
        }
        
        public static imat4 operator*(imat4 m0, imat4 m1) {
            return imath.InnerProduct1(ref m0, ref m1);
        }
        
        public override bool Equals(object obj) {
            return (obj is imat4) ? bmath.All(this == (imat4)obj) : false;
        }
        
        public override int GetHashCode() {
            return this.m00.GetHashCode() ^ this.m01.GetHashCode() ^ this.m02.GetHashCode() ^ this.m03.GetHashCode() ^ this.m10.GetHashCode() ^ this.m11.GetHashCode() ^ this.m12.GetHashCode() ^ this.m13.GetHashCode() ^ this.m20.GetHashCode() ^ this.m21.GetHashCode() ^ this.m22.GetHashCode() ^ this.m23.GetHashCode() ^ this.m30.GetHashCode() ^ this.m31.GetHashCode() ^ this.m32.GetHashCode() ^ this.m33.GetHashCode();
        }
        
        public override string ToString() {
            return "imat4(" + this.m00 + "," + this.m01 + "," + this.m02 + "," + this.m03 + "," + this.m10 + "," + this.m11 + "," + this.m12 + "," + this.m13 + "," + this.m20 + "," + this.m21 + "," + this.m22 + "," + this.m23 + "," + this.m30 + "," + this.m31 + "," + this.m32 + "," + this.m33 + ")";
        }
    }
}
