#region --- License ---
/*
Copyright 2010 Daniel Filonik

Permission is hereby granted, free of charge, to any person obtaining a copy of
this software and associated documentation files (the "Software"), to deal in
the Software without restriction, including without limitation the rights to
use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
of the Software, and to permit persons to whom the Software is furnished to do
so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all
copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
SOFTWARE.
*/
#endregion

namespace TKM {
    using System;
    using System.Runtime.InteropServices;
    using System.Runtime.Serialization;
    
    
    [DataContract()]
    [StructLayout(LayoutKind.Sequential)]
    public struct ivec2 {
        
        [DataMember()]
        internal int v0;
        
        [DataMember()]
        internal int v1;
        
        public ivec2(int s) {
            this.v0 = s;
            this.v1 = s;
        }
        
        public ivec2(ivec2 v0) {
            this.v0 = v0.v0;
            this.v1 = v0.v1;
        }
        
        public ivec2(int v0, int v1) {
            this.v0 = v0;
            this.v1 = v1;
        }
        
        public ivec2(ivec3 s) {
            this.v0 = s.v0;
            this.v1 = s.v1;
        }
        
        public ivec2(ivec4 s) {
            this.v0 = s.v0;
            this.v1 = s.v1;
        }
        
        public int this[int i0] {
            get {
                if (((i0 < 0) 
                            || (1 < i0))) {
                    throw new System.IndexOutOfRangeException("i0");
                }
                
                unsafe
                {
                    fixed (Int32* v_ptr = &this.v0)
                    {
                        return *(v_ptr + (i0));
                    }
                };
            }
            set {
                if (((i0 < 0) 
                            || (1 < i0))) {
                    throw new System.IndexOutOfRangeException("i0");
                }
                
                unsafe
                {
                    fixed (Int32* v_ptr = &this.v0)
                    {
                        *(v_ptr  + (i0)) = value;
                    }
                };
            }
        }
        
        public int x {
            get {
                return this.v0;
            }
            set {
                this.v0 = value;
            }
        }
        
        public int r {
            get {
                return this.v0;
            }
            set {
                this.v0 = value;
            }
        }
        
        public int s {
            get {
                return this.v0;
            }
            set {
                this.v0 = value;
            }
        }
        
        public int y {
            get {
                return this.v1;
            }
            set {
                this.v1 = value;
            }
        }
        
        public int g {
            get {
                return this.v1;
            }
            set {
                this.v1 = value;
            }
        }
        
        public int t {
            get {
                return this.v1;
            }
            set {
                this.v1 = value;
            }
        }
        
        public ivec2 xx {
            get {
                return new ivec2(this.v0, this.v0);
            }
        }
        
        public ivec2 rr {
            get {
                return new ivec2(this.v0, this.v0);
            }
        }
        
        public ivec2 ss {
            get {
                return new ivec2(this.v0, this.v0);
            }
        }
        
        public ivec2 xy {
            get {
                return new ivec2(this.v0, this.v1);
            }
            set {
                this.v0 = value.v0;
                this.v1 = value.v1;
            }
        }
        
        public ivec2 rg {
            get {
                return new ivec2(this.v0, this.v1);
            }
            set {
                this.v0 = value.v0;
                this.v1 = value.v1;
            }
        }
        
        public ivec2 st {
            get {
                return new ivec2(this.v0, this.v1);
            }
            set {
                this.v0 = value.v0;
                this.v1 = value.v1;
            }
        }
        
        public ivec2 yx {
            get {
                return new ivec2(this.v1, this.v0);
            }
            set {
                this.v1 = value.v0;
                this.v0 = value.v1;
            }
        }
        
        public ivec2 gr {
            get {
                return new ivec2(this.v1, this.v0);
            }
            set {
                this.v1 = value.v0;
                this.v0 = value.v1;
            }
        }
        
        public ivec2 ts {
            get {
                return new ivec2(this.v1, this.v0);
            }
            set {
                this.v1 = value.v0;
                this.v0 = value.v1;
            }
        }
        
        public ivec2 yy {
            get {
                return new ivec2(this.v1, this.v1);
            }
        }
        
        public ivec2 gg {
            get {
                return new ivec2(this.v1, this.v1);
            }
        }
        
        public ivec2 tt {
            get {
                return new ivec2(this.v1, this.v1);
            }
        }
        
        public static ivec2 operator+(ivec2 v) {
            return new ivec2(+v.v0, +v.v1);
        }
        
        public static ivec2 operator-(ivec2 v) {
            return new ivec2(-v.v0, -v.v1);
        }
        
        public static ivec2 operator~(ivec2 v) {
            return new ivec2(~v.v0, ~v.v1);
        }
        
        public static ivec2 operator++(ivec2 v) {
            return new ivec2(++v.v0, ++v.v1);
        }
        
        public static ivec2 operator--(ivec2 v) {
            return new ivec2(--v.v0, --v.v1);
        }
        
        public static ivec2 operator+(int s, ivec2 v2) {
            return new ivec2(s + v2.v0, s + v2.v1);
        }
        
        public static ivec2 operator-(int s, ivec2 v2) {
            return new ivec2(s - v2.v0, s - v2.v1);
        }
        
        public static ivec2 operator*(int s, ivec2 v2) {
            return new ivec2(s * v2.v0, s * v2.v1);
        }
        
        public static ivec2 operator/(int s, ivec2 v2) {
            return new ivec2(s / v2.v0, s / v2.v1);
        }
        
        public static ivec2 operator%(int s, ivec2 v2) {
            return new ivec2(s % v2.v0, s % v2.v1);
        }
        
        public static ivec2 operator&(int s, ivec2 v2) {
            return new ivec2(s & v2.v0, s & v2.v1);
        }
        
        public static ivec2 operator|(int s, ivec2 v2) {
            return new ivec2(s | v2.v0, s | v2.v1);
        }
        
        public static ivec2 operator^(int s, ivec2 v2) {
            return new ivec2(s ^ v2.v0, s ^ v2.v1);
        }
        
        public static ivec2 operator+(ivec2 v1, int s) {
            return new ivec2(v1.v0 + s, v1.v1 + s);
        }
        
        public static ivec2 operator-(ivec2 v1, int s) {
            return new ivec2(v1.v0 - s, v1.v1 - s);
        }
        
        public static ivec2 operator*(ivec2 v1, int s) {
            return new ivec2(v1.v0 * s, v1.v1 * s);
        }
        
        public static ivec2 operator/(ivec2 v1, int s) {
            return new ivec2(v1.v0 / s, v1.v1 / s);
        }
        
        public static ivec2 operator%(ivec2 v1, int s) {
            return new ivec2(v1.v0 % s, v1.v1 % s);
        }
        
        public static ivec2 operator&(ivec2 v1, int s) {
            return new ivec2(v1.v0 & s, v1.v1 & s);
        }
        
        public static ivec2 operator|(ivec2 v1, int s) {
            return new ivec2(v1.v0 | s, v1.v1 | s);
        }
        
        public static ivec2 operator^(ivec2 v1, int s) {
            return new ivec2(v1.v0 ^ s, v1.v1 ^ s);
        }
        
        public static ivec2 operator+(ivec2 v1, ivec2 v2) {
            return new ivec2(v1.v0 + v2.v0, v1.v1 + v2.v1);
        }
        
        public static ivec2 operator-(ivec2 v1, ivec2 v2) {
            return new ivec2(v1.v0 - v2.v0, v1.v1 - v2.v1);
        }
        
        public static ivec2 operator*(ivec2 v1, ivec2 v2) {
            return new ivec2(v1.v0 * v2.v0, v1.v1 * v2.v1);
        }
        
        public static ivec2 operator/(ivec2 v1, ivec2 v2) {
            return new ivec2(v1.v0 / v2.v0, v1.v1 / v2.v1);
        }
        
        public static ivec2 operator%(ivec2 v1, ivec2 v2) {
            return new ivec2(v1.v0 % v2.v0, v1.v1 % v2.v1);
        }
        
        public static ivec2 operator&(ivec2 v1, ivec2 v2) {
            return new ivec2(v1.v0 & v2.v0, v1.v1 & v2.v1);
        }
        
        public static ivec2 operator|(ivec2 v1, ivec2 v2) {
            return new ivec2(v1.v0 | v2.v0, v1.v1 | v2.v1);
        }
        
        public static ivec2 operator^(ivec2 v1, ivec2 v2) {
            return new ivec2(v1.v0 ^ v2.v0, v1.v1 ^ v2.v1);
        }
        
        public static bvec2 operator==(ivec2 v1, ivec2 v2) {
            return new bvec2(v1.v0 == v2.v0, v1.v1 == v2.v1);
        }
        
        public static bvec2 operator>(ivec2 v1, ivec2 v2) {
            return new bvec2(v1.v0 > v2.v0, v1.v1 > v2.v1);
        }
        
        public static bvec2 operator>=(ivec2 v1, ivec2 v2) {
            return new bvec2(v1.v0 >= v2.v0, v1.v1 >= v2.v1);
        }
        
        public static bvec2 operator<(ivec2 v1, ivec2 v2) {
            return new bvec2(v1.v0 < v2.v0, v1.v1 < v2.v1);
        }
        
        public static bvec2 operator<=(ivec2 v1, ivec2 v2) {
            return new bvec2(v1.v0 <= v2.v0, v1.v1 <= v2.v1);
        }
        
        public static bvec2 operator!=(ivec2 v1, ivec2 v2) {
            return new bvec2(v1.v0 != v2.v0, v1.v1 != v2.v1);
        }
        
        public override bool Equals(object obj) {
            return (obj is ivec2) ? bmath.All(this == (ivec2)obj) : false;
        }
        
        public override int GetHashCode() {
            return this.v0.GetHashCode() ^ this.v1.GetHashCode();
        }
        
        public override string ToString() {
            return "ivec2(" + this.v0 + "," + this.v1 + ")";
        }
    }
    
    [DataContract()]
    [StructLayout(LayoutKind.Sequential)]
    public struct ivec3 {
        
        [DataMember()]
        internal int v0;
        
        [DataMember()]
        internal int v1;
        
        [DataMember()]
        internal int v2;
        
        public ivec3(int s) {
            this.v0 = s;
            this.v1 = s;
            this.v2 = s;
        }
        
        public ivec3(ivec3 v0) {
            this.v0 = v0.v0;
            this.v1 = v0.v1;
            this.v2 = v0.v2;
        }
        
        public ivec3(ivec2 v0, int v1) {
            this.v0 = v0.v0;
            this.v1 = v0.v1;
            this.v2 = v1;
        }
        
        public ivec3(int v0, ivec2 v1) {
            this.v0 = v0;
            this.v1 = v1.v0;
            this.v2 = v1.v1;
        }
        
        public ivec3(int v0, int v1, int v2) {
            this.v0 = v0;
            this.v1 = v1;
            this.v2 = v2;
        }
        
        public ivec3(ivec2 s) {
            this.v0 = s.v0;
            this.v1 = s.v1;
            this.v2 = 0;
        }
        
        public ivec3(ivec4 s) {
            this.v0 = s.v0;
            this.v1 = s.v1;
            this.v2 = s.v2;
        }
        
        public int this[int i0] {
            get {
                if (((i0 < 0) 
                            || (2 < i0))) {
                    throw new System.IndexOutOfRangeException("i0");
                }
                
                unsafe
                {
                    fixed (Int32* v_ptr = &this.v0)
                    {
                        return *(v_ptr + (i0));
                    }
                };
            }
            set {
                if (((i0 < 0) 
                            || (2 < i0))) {
                    throw new System.IndexOutOfRangeException("i0");
                }
                
                unsafe
                {
                    fixed (Int32* v_ptr = &this.v0)
                    {
                        *(v_ptr  + (i0)) = value;
                    }
                };
            }
        }
        
        public int x {
            get {
                return this.v0;
            }
            set {
                this.v0 = value;
            }
        }
        
        public int r {
            get {
                return this.v0;
            }
            set {
                this.v0 = value;
            }
        }
        
        public int s {
            get {
                return this.v0;
            }
            set {
                this.v0 = value;
            }
        }
        
        public int y {
            get {
                return this.v1;
            }
            set {
                this.v1 = value;
            }
        }
        
        public int g {
            get {
                return this.v1;
            }
            set {
                this.v1 = value;
            }
        }
        
        public int t {
            get {
                return this.v1;
            }
            set {
                this.v1 = value;
            }
        }
        
        public int z {
            get {
                return this.v2;
            }
            set {
                this.v2 = value;
            }
        }
        
        public int b {
            get {
                return this.v2;
            }
            set {
                this.v2 = value;
            }
        }
        
        public int p {
            get {
                return this.v2;
            }
            set {
                this.v2 = value;
            }
        }
        
        public ivec2 xx {
            get {
                return new ivec2(this.v0, this.v0);
            }
        }
        
        public ivec2 rr {
            get {
                return new ivec2(this.v0, this.v0);
            }
        }
        
        public ivec2 ss {
            get {
                return new ivec2(this.v0, this.v0);
            }
        }
        
        public ivec2 xy {
            get {
                return new ivec2(this.v0, this.v1);
            }
            set {
                this.v0 = value.v0;
                this.v1 = value.v1;
            }
        }
        
        public ivec2 rg {
            get {
                return new ivec2(this.v0, this.v1);
            }
            set {
                this.v0 = value.v0;
                this.v1 = value.v1;
            }
        }
        
        public ivec2 st {
            get {
                return new ivec2(this.v0, this.v1);
            }
            set {
                this.v0 = value.v0;
                this.v1 = value.v1;
            }
        }
        
        public ivec2 xz {
            get {
                return new ivec2(this.v0, this.v2);
            }
            set {
                this.v0 = value.v0;
                this.v2 = value.v1;
            }
        }
        
        public ivec2 rb {
            get {
                return new ivec2(this.v0, this.v2);
            }
            set {
                this.v0 = value.v0;
                this.v2 = value.v1;
            }
        }
        
        public ivec2 sp {
            get {
                return new ivec2(this.v0, this.v2);
            }
            set {
                this.v0 = value.v0;
                this.v2 = value.v1;
            }
        }
        
        public ivec2 yx {
            get {
                return new ivec2(this.v1, this.v0);
            }
            set {
                this.v1 = value.v0;
                this.v0 = value.v1;
            }
        }
        
        public ivec2 gr {
            get {
                return new ivec2(this.v1, this.v0);
            }
            set {
                this.v1 = value.v0;
                this.v0 = value.v1;
            }
        }
        
        public ivec2 ts {
            get {
                return new ivec2(this.v1, this.v0);
            }
            set {
                this.v1 = value.v0;
                this.v0 = value.v1;
            }
        }
        
        public ivec2 yy {
            get {
                return new ivec2(this.v1, this.v1);
            }
        }
        
        public ivec2 gg {
            get {
                return new ivec2(this.v1, this.v1);
            }
        }
        
        public ivec2 tt {
            get {
                return new ivec2(this.v1, this.v1);
            }
        }
        
        public ivec2 yz {
            get {
                return new ivec2(this.v1, this.v2);
            }
            set {
                this.v1 = value.v0;
                this.v2 = value.v1;
            }
        }
        
        public ivec2 gb {
            get {
                return new ivec2(this.v1, this.v2);
            }
            set {
                this.v1 = value.v0;
                this.v2 = value.v1;
            }
        }
        
        public ivec2 tp {
            get {
                return new ivec2(this.v1, this.v2);
            }
            set {
                this.v1 = value.v0;
                this.v2 = value.v1;
            }
        }
        
        public ivec2 zx {
            get {
                return new ivec2(this.v2, this.v0);
            }
            set {
                this.v2 = value.v0;
                this.v0 = value.v1;
            }
        }
        
        public ivec2 br {
            get {
                return new ivec2(this.v2, this.v0);
            }
            set {
                this.v2 = value.v0;
                this.v0 = value.v1;
            }
        }
        
        public ivec2 ps {
            get {
                return new ivec2(this.v2, this.v0);
            }
            set {
                this.v2 = value.v0;
                this.v0 = value.v1;
            }
        }
        
        public ivec2 zy {
            get {
                return new ivec2(this.v2, this.v1);
            }
            set {
                this.v2 = value.v0;
                this.v1 = value.v1;
            }
        }
        
        public ivec2 bg {
            get {
                return new ivec2(this.v2, this.v1);
            }
            set {
                this.v2 = value.v0;
                this.v1 = value.v1;
            }
        }
        
        public ivec2 pt {
            get {
                return new ivec2(this.v2, this.v1);
            }
            set {
                this.v2 = value.v0;
                this.v1 = value.v1;
            }
        }
        
        public ivec2 zz {
            get {
                return new ivec2(this.v2, this.v2);
            }
        }
        
        public ivec2 bb {
            get {
                return new ivec2(this.v2, this.v2);
            }
        }
        
        public ivec2 pp {
            get {
                return new ivec2(this.v2, this.v2);
            }
        }
        
        public ivec3 xxx {
            get {
                return new ivec3(this.v0, this.v0, this.v0);
            }
        }
        
        public ivec3 rrr {
            get {
                return new ivec3(this.v0, this.v0, this.v0);
            }
        }
        
        public ivec3 sss {
            get {
                return new ivec3(this.v0, this.v0, this.v0);
            }
        }
        
        public ivec3 xxy {
            get {
                return new ivec3(this.v0, this.v0, this.v1);
            }
        }
        
        public ivec3 rrg {
            get {
                return new ivec3(this.v0, this.v0, this.v1);
            }
        }
        
        public ivec3 sst {
            get {
                return new ivec3(this.v0, this.v0, this.v1);
            }
        }
        
        public ivec3 xxz {
            get {
                return new ivec3(this.v0, this.v0, this.v2);
            }
        }
        
        public ivec3 rrb {
            get {
                return new ivec3(this.v0, this.v0, this.v2);
            }
        }
        
        public ivec3 ssp {
            get {
                return new ivec3(this.v0, this.v0, this.v2);
            }
        }
        
        public ivec3 xyx {
            get {
                return new ivec3(this.v0, this.v1, this.v0);
            }
        }
        
        public ivec3 rgr {
            get {
                return new ivec3(this.v0, this.v1, this.v0);
            }
        }
        
        public ivec3 sts {
            get {
                return new ivec3(this.v0, this.v1, this.v0);
            }
        }
        
        public ivec3 xyy {
            get {
                return new ivec3(this.v0, this.v1, this.v1);
            }
        }
        
        public ivec3 rgg {
            get {
                return new ivec3(this.v0, this.v1, this.v1);
            }
        }
        
        public ivec3 stt {
            get {
                return new ivec3(this.v0, this.v1, this.v1);
            }
        }
        
        public ivec3 xyz {
            get {
                return new ivec3(this.v0, this.v1, this.v2);
            }
            set {
                this.v0 = value.v0;
                this.v1 = value.v1;
                this.v2 = value.v2;
            }
        }
        
        public ivec3 rgb {
            get {
                return new ivec3(this.v0, this.v1, this.v2);
            }
            set {
                this.v0 = value.v0;
                this.v1 = value.v1;
                this.v2 = value.v2;
            }
        }
        
        public ivec3 stp {
            get {
                return new ivec3(this.v0, this.v1, this.v2);
            }
            set {
                this.v0 = value.v0;
                this.v1 = value.v1;
                this.v2 = value.v2;
            }
        }
        
        public ivec3 xzx {
            get {
                return new ivec3(this.v0, this.v2, this.v0);
            }
        }
        
        public ivec3 rbr {
            get {
                return new ivec3(this.v0, this.v2, this.v0);
            }
        }
        
        public ivec3 sps {
            get {
                return new ivec3(this.v0, this.v2, this.v0);
            }
        }
        
        public ivec3 xzy {
            get {
                return new ivec3(this.v0, this.v2, this.v1);
            }
            set {
                this.v0 = value.v0;
                this.v2 = value.v1;
                this.v1 = value.v2;
            }
        }
        
        public ivec3 rbg {
            get {
                return new ivec3(this.v0, this.v2, this.v1);
            }
            set {
                this.v0 = value.v0;
                this.v2 = value.v1;
                this.v1 = value.v2;
            }
        }
        
        public ivec3 spt {
            get {
                return new ivec3(this.v0, this.v2, this.v1);
            }
            set {
                this.v0 = value.v0;
                this.v2 = value.v1;
                this.v1 = value.v2;
            }
        }
        
        public ivec3 xzz {
            get {
                return new ivec3(this.v0, this.v2, this.v2);
            }
        }
        
        public ivec3 rbb {
            get {
                return new ivec3(this.v0, this.v2, this.v2);
            }
        }
        
        public ivec3 spp {
            get {
                return new ivec3(this.v0, this.v2, this.v2);
            }
        }
        
        public ivec3 yxx {
            get {
                return new ivec3(this.v1, this.v0, this.v0);
            }
        }
        
        public ivec3 grr {
            get {
                return new ivec3(this.v1, this.v0, this.v0);
            }
        }
        
        public ivec3 tss {
            get {
                return new ivec3(this.v1, this.v0, this.v0);
            }
        }
        
        public ivec3 yxy {
            get {
                return new ivec3(this.v1, this.v0, this.v1);
            }
        }
        
        public ivec3 grg {
            get {
                return new ivec3(this.v1, this.v0, this.v1);
            }
        }
        
        public ivec3 tst {
            get {
                return new ivec3(this.v1, this.v0, this.v1);
            }
        }
        
        public ivec3 yxz {
            get {
                return new ivec3(this.v1, this.v0, this.v2);
            }
            set {
                this.v1 = value.v0;
                this.v0 = value.v1;
                this.v2 = value.v2;
            }
        }
        
        public ivec3 grb {
            get {
                return new ivec3(this.v1, this.v0, this.v2);
            }
            set {
                this.v1 = value.v0;
                this.v0 = value.v1;
                this.v2 = value.v2;
            }
        }
        
        public ivec3 tsp {
            get {
                return new ivec3(this.v1, this.v0, this.v2);
            }
            set {
                this.v1 = value.v0;
                this.v0 = value.v1;
                this.v2 = value.v2;
            }
        }
        
        public ivec3 yyx {
            get {
                return new ivec3(this.v1, this.v1, this.v0);
            }
        }
        
        public ivec3 ggr {
            get {
                return new ivec3(this.v1, this.v1, this.v0);
            }
        }
        
        public ivec3 tts {
            get {
                return new ivec3(this.v1, this.v1, this.v0);
            }
        }
        
        public ivec3 yyy {
            get {
                return new ivec3(this.v1, this.v1, this.v1);
            }
        }
        
        public ivec3 ggg {
            get {
                return new ivec3(this.v1, this.v1, this.v1);
            }
        }
        
        public ivec3 ttt {
            get {
                return new ivec3(this.v1, this.v1, this.v1);
            }
        }
        
        public ivec3 yyz {
            get {
                return new ivec3(this.v1, this.v1, this.v2);
            }
        }
        
        public ivec3 ggb {
            get {
                return new ivec3(this.v1, this.v1, this.v2);
            }
        }
        
        public ivec3 ttp {
            get {
                return new ivec3(this.v1, this.v1, this.v2);
            }
        }
        
        public ivec3 yzx {
            get {
                return new ivec3(this.v1, this.v2, this.v0);
            }
            set {
                this.v1 = value.v0;
                this.v2 = value.v1;
                this.v0 = value.v2;
            }
        }
        
        public ivec3 gbr {
            get {
                return new ivec3(this.v1, this.v2, this.v0);
            }
            set {
                this.v1 = value.v0;
                this.v2 = value.v1;
                this.v0 = value.v2;
            }
        }
        
        public ivec3 tps {
            get {
                return new ivec3(this.v1, this.v2, this.v0);
            }
            set {
                this.v1 = value.v0;
                this.v2 = value.v1;
                this.v0 = value.v2;
            }
        }
        
        public ivec3 yzy {
            get {
                return new ivec3(this.v1, this.v2, this.v1);
            }
        }
        
        public ivec3 gbg {
            get {
                return new ivec3(this.v1, this.v2, this.v1);
            }
        }
        
        public ivec3 tpt {
            get {
                return new ivec3(this.v1, this.v2, this.v1);
            }
        }
        
        public ivec3 yzz {
            get {
                return new ivec3(this.v1, this.v2, this.v2);
            }
        }
        
        public ivec3 gbb {
            get {
                return new ivec3(this.v1, this.v2, this.v2);
            }
        }
        
        public ivec3 tpp {
            get {
                return new ivec3(this.v1, this.v2, this.v2);
            }
        }
        
        public ivec3 zxx {
            get {
                return new ivec3(this.v2, this.v0, this.v0);
            }
        }
        
        public ivec3 brr {
            get {
                return new ivec3(this.v2, this.v0, this.v0);
            }
        }
        
        public ivec3 pss {
            get {
                return new ivec3(this.v2, this.v0, this.v0);
            }
        }
        
        public ivec3 zxy {
            get {
                return new ivec3(this.v2, this.v0, this.v1);
            }
            set {
                this.v2 = value.v0;
                this.v0 = value.v1;
                this.v1 = value.v2;
            }
        }
        
        public ivec3 brg {
            get {
                return new ivec3(this.v2, this.v0, this.v1);
            }
            set {
                this.v2 = value.v0;
                this.v0 = value.v1;
                this.v1 = value.v2;
            }
        }
        
        public ivec3 pst {
            get {
                return new ivec3(this.v2, this.v0, this.v1);
            }
            set {
                this.v2 = value.v0;
                this.v0 = value.v1;
                this.v1 = value.v2;
            }
        }
        
        public ivec3 zxz {
            get {
                return new ivec3(this.v2, this.v0, this.v2);
            }
        }
        
        public ivec3 brb {
            get {
                return new ivec3(this.v2, this.v0, this.v2);
            }
        }
        
        public ivec3 psp {
            get {
                return new ivec3(this.v2, this.v0, this.v2);
            }
        }
        
        public ivec3 zyx {
            get {
                return new ivec3(this.v2, this.v1, this.v0);
            }
            set {
                this.v2 = value.v0;
                this.v1 = value.v1;
                this.v0 = value.v2;
            }
        }
        
        public ivec3 bgr {
            get {
                return new ivec3(this.v2, this.v1, this.v0);
            }
            set {
                this.v2 = value.v0;
                this.v1 = value.v1;
                this.v0 = value.v2;
            }
        }
        
        public ivec3 pts {
            get {
                return new ivec3(this.v2, this.v1, this.v0);
            }
            set {
                this.v2 = value.v0;
                this.v1 = value.v1;
                this.v0 = value.v2;
            }
        }
        
        public ivec3 zyy {
            get {
                return new ivec3(this.v2, this.v1, this.v1);
            }
        }
        
        public ivec3 bgg {
            get {
                return new ivec3(this.v2, this.v1, this.v1);
            }
        }
        
        public ivec3 ptt {
            get {
                return new ivec3(this.v2, this.v1, this.v1);
            }
        }
        
        public ivec3 zyz {
            get {
                return new ivec3(this.v2, this.v1, this.v2);
            }
        }
        
        public ivec3 bgb {
            get {
                return new ivec3(this.v2, this.v1, this.v2);
            }
        }
        
        public ivec3 ptp {
            get {
                return new ivec3(this.v2, this.v1, this.v2);
            }
        }
        
        public ivec3 zzx {
            get {
                return new ivec3(this.v2, this.v2, this.v0);
            }
        }
        
        public ivec3 bbr {
            get {
                return new ivec3(this.v2, this.v2, this.v0);
            }
        }
        
        public ivec3 pps {
            get {
                return new ivec3(this.v2, this.v2, this.v0);
            }
        }
        
        public ivec3 zzy {
            get {
                return new ivec3(this.v2, this.v2, this.v1);
            }
        }
        
        public ivec3 bbg {
            get {
                return new ivec3(this.v2, this.v2, this.v1);
            }
        }
        
        public ivec3 ppt {
            get {
                return new ivec3(this.v2, this.v2, this.v1);
            }
        }
        
        public ivec3 zzz {
            get {
                return new ivec3(this.v2, this.v2, this.v2);
            }
        }
        
        public ivec3 bbb {
            get {
                return new ivec3(this.v2, this.v2, this.v2);
            }
        }
        
        public ivec3 ppp {
            get {
                return new ivec3(this.v2, this.v2, this.v2);
            }
        }
        
        public static ivec3 operator+(ivec3 v) {
            return new ivec3(+v.v0, +v.v1, +v.v2);
        }
        
        public static ivec3 operator-(ivec3 v) {
            return new ivec3(-v.v0, -v.v1, -v.v2);
        }
        
        public static ivec3 operator~(ivec3 v) {
            return new ivec3(~v.v0, ~v.v1, ~v.v2);
        }
        
        public static ivec3 operator++(ivec3 v) {
            return new ivec3(++v.v0, ++v.v1, ++v.v2);
        }
        
        public static ivec3 operator--(ivec3 v) {
            return new ivec3(--v.v0, --v.v1, --v.v2);
        }
        
        public static ivec3 operator+(int s, ivec3 v2) {
            return new ivec3(s + v2.v0, s + v2.v1, s + v2.v2);
        }
        
        public static ivec3 operator-(int s, ivec3 v2) {
            return new ivec3(s - v2.v0, s - v2.v1, s - v2.v2);
        }
        
        public static ivec3 operator*(int s, ivec3 v2) {
            return new ivec3(s * v2.v0, s * v2.v1, s * v2.v2);
        }
        
        public static ivec3 operator/(int s, ivec3 v2) {
            return new ivec3(s / v2.v0, s / v2.v1, s / v2.v2);
        }
        
        public static ivec3 operator%(int s, ivec3 v2) {
            return new ivec3(s % v2.v0, s % v2.v1, s % v2.v2);
        }
        
        public static ivec3 operator&(int s, ivec3 v2) {
            return new ivec3(s & v2.v0, s & v2.v1, s & v2.v2);
        }
        
        public static ivec3 operator|(int s, ivec3 v2) {
            return new ivec3(s | v2.v0, s | v2.v1, s | v2.v2);
        }
        
        public static ivec3 operator^(int s, ivec3 v2) {
            return new ivec3(s ^ v2.v0, s ^ v2.v1, s ^ v2.v2);
        }
        
        public static ivec3 operator+(ivec3 v1, int s) {
            return new ivec3(v1.v0 + s, v1.v1 + s, v1.v2 + s);
        }
        
        public static ivec3 operator-(ivec3 v1, int s) {
            return new ivec3(v1.v0 - s, v1.v1 - s, v1.v2 - s);
        }
        
        public static ivec3 operator*(ivec3 v1, int s) {
            return new ivec3(v1.v0 * s, v1.v1 * s, v1.v2 * s);
        }
        
        public static ivec3 operator/(ivec3 v1, int s) {
            return new ivec3(v1.v0 / s, v1.v1 / s, v1.v2 / s);
        }
        
        public static ivec3 operator%(ivec3 v1, int s) {
            return new ivec3(v1.v0 % s, v1.v1 % s, v1.v2 % s);
        }
        
        public static ivec3 operator&(ivec3 v1, int s) {
            return new ivec3(v1.v0 & s, v1.v1 & s, v1.v2 & s);
        }
        
        public static ivec3 operator|(ivec3 v1, int s) {
            return new ivec3(v1.v0 | s, v1.v1 | s, v1.v2 | s);
        }
        
        public static ivec3 operator^(ivec3 v1, int s) {
            return new ivec3(v1.v0 ^ s, v1.v1 ^ s, v1.v2 ^ s);
        }
        
        public static ivec3 operator+(ivec3 v1, ivec3 v2) {
            return new ivec3(v1.v0 + v2.v0, v1.v1 + v2.v1, v1.v2 + v2.v2);
        }
        
        public static ivec3 operator-(ivec3 v1, ivec3 v2) {
            return new ivec3(v1.v0 - v2.v0, v1.v1 - v2.v1, v1.v2 - v2.v2);
        }
        
        public static ivec3 operator*(ivec3 v1, ivec3 v2) {
            return new ivec3(v1.v0 * v2.v0, v1.v1 * v2.v1, v1.v2 * v2.v2);
        }
        
        public static ivec3 operator/(ivec3 v1, ivec3 v2) {
            return new ivec3(v1.v0 / v2.v0, v1.v1 / v2.v1, v1.v2 / v2.v2);
        }
        
        public static ivec3 operator%(ivec3 v1, ivec3 v2) {
            return new ivec3(v1.v0 % v2.v0, v1.v1 % v2.v1, v1.v2 % v2.v2);
        }
        
        public static ivec3 operator&(ivec3 v1, ivec3 v2) {
            return new ivec3(v1.v0 & v2.v0, v1.v1 & v2.v1, v1.v2 & v2.v2);
        }
        
        public static ivec3 operator|(ivec3 v1, ivec3 v2) {
            return new ivec3(v1.v0 | v2.v0, v1.v1 | v2.v1, v1.v2 | v2.v2);
        }
        
        public static ivec3 operator^(ivec3 v1, ivec3 v2) {
            return new ivec3(v1.v0 ^ v2.v0, v1.v1 ^ v2.v1, v1.v2 ^ v2.v2);
        }
        
        public static bvec3 operator==(ivec3 v1, ivec3 v2) {
            return new bvec3(v1.v0 == v2.v0, v1.v1 == v2.v1, v1.v2 == v2.v2);
        }
        
        public static bvec3 operator>(ivec3 v1, ivec3 v2) {
            return new bvec3(v1.v0 > v2.v0, v1.v1 > v2.v1, v1.v2 > v2.v2);
        }
        
        public static bvec3 operator>=(ivec3 v1, ivec3 v2) {
            return new bvec3(v1.v0 >= v2.v0, v1.v1 >= v2.v1, v1.v2 >= v2.v2);
        }
        
        public static bvec3 operator<(ivec3 v1, ivec3 v2) {
            return new bvec3(v1.v0 < v2.v0, v1.v1 < v2.v1, v1.v2 < v2.v2);
        }
        
        public static bvec3 operator<=(ivec3 v1, ivec3 v2) {
            return new bvec3(v1.v0 <= v2.v0, v1.v1 <= v2.v1, v1.v2 <= v2.v2);
        }
        
        public static bvec3 operator!=(ivec3 v1, ivec3 v2) {
            return new bvec3(v1.v0 != v2.v0, v1.v1 != v2.v1, v1.v2 != v2.v2);
        }
        
        public override bool Equals(object obj) {
            return (obj is ivec3) ? bmath.All(this == (ivec3)obj) : false;
        }
        
        public override int GetHashCode() {
            return this.v0.GetHashCode() ^ this.v1.GetHashCode() ^ this.v2.GetHashCode();
        }
        
        public override string ToString() {
            return "ivec3(" + this.v0 + "," + this.v1 + "," + this.v2 + ")";
        }
    }
    
    [DataContract()]
    [StructLayout(LayoutKind.Sequential)]
    public struct ivec4 {
        
        [DataMember()]
        internal int v0;
        
        [DataMember()]
        internal int v1;
        
        [DataMember()]
        internal int v2;
        
        [DataMember()]
        internal int v3;
        
        public ivec4(int s) {
            this.v0 = s;
            this.v1 = s;
            this.v2 = s;
            this.v3 = s;
        }
        
        public ivec4(ivec4 v0) {
            this.v0 = v0.v0;
            this.v1 = v0.v1;
            this.v2 = v0.v2;
            this.v3 = v0.v3;
        }
        
        public ivec4(ivec2 v0, ivec2 v1) {
            this.v0 = v0.v0;
            this.v1 = v0.v1;
            this.v2 = v1.v0;
            this.v3 = v1.v1;
        }
        
        public ivec4(ivec3 v0, int v1) {
            this.v0 = v0.v0;
            this.v1 = v0.v1;
            this.v2 = v0.v2;
            this.v3 = v1;
        }
        
        public ivec4(int v0, ivec3 v1) {
            this.v0 = v0;
            this.v1 = v1.v0;
            this.v2 = v1.v1;
            this.v3 = v1.v2;
        }
        
        public ivec4(ivec2 v0, int v1, int v2) {
            this.v0 = v0.v0;
            this.v1 = v0.v1;
            this.v2 = v1;
            this.v3 = v2;
        }
        
        public ivec4(int v0, ivec2 v1, int v2) {
            this.v0 = v0;
            this.v1 = v1.v0;
            this.v2 = v1.v1;
            this.v3 = v2;
        }
        
        public ivec4(int v0, int v1, ivec2 v2) {
            this.v0 = v0;
            this.v1 = v1;
            this.v2 = v2.v0;
            this.v3 = v2.v1;
        }
        
        public ivec4(int v0, int v1, int v2, int v3) {
            this.v0 = v0;
            this.v1 = v1;
            this.v2 = v2;
            this.v3 = v3;
        }
        
        public ivec4(ivec2 s) {
            this.v0 = s.v0;
            this.v1 = s.v1;
            this.v2 = 0;
            this.v3 = 0;
        }
        
        public ivec4(ivec3 s) {
            this.v0 = s.v0;
            this.v1 = s.v1;
            this.v2 = s.v2;
            this.v3 = 0;
        }
        
        public int this[int i0] {
            get {
                if (((i0 < 0) 
                            || (3 < i0))) {
                    throw new System.IndexOutOfRangeException("i0");
                }
                
                unsafe
                {
                    fixed (Int32* v_ptr = &this.v0)
                    {
                        return *(v_ptr + (i0));
                    }
                };
            }
            set {
                if (((i0 < 0) 
                            || (3 < i0))) {
                    throw new System.IndexOutOfRangeException("i0");
                }
                
                unsafe
                {
                    fixed (Int32* v_ptr = &this.v0)
                    {
                        *(v_ptr  + (i0)) = value;
                    }
                };
            }
        }
        
        public int x {
            get {
                return this.v0;
            }
            set {
                this.v0 = value;
            }
        }
        
        public int r {
            get {
                return this.v0;
            }
            set {
                this.v0 = value;
            }
        }
        
        public int s {
            get {
                return this.v0;
            }
            set {
                this.v0 = value;
            }
        }
        
        public int y {
            get {
                return this.v1;
            }
            set {
                this.v1 = value;
            }
        }
        
        public int g {
            get {
                return this.v1;
            }
            set {
                this.v1 = value;
            }
        }
        
        public int t {
            get {
                return this.v1;
            }
            set {
                this.v1 = value;
            }
        }
        
        public int z {
            get {
                return this.v2;
            }
            set {
                this.v2 = value;
            }
        }
        
        public int b {
            get {
                return this.v2;
            }
            set {
                this.v2 = value;
            }
        }
        
        public int p {
            get {
                return this.v2;
            }
            set {
                this.v2 = value;
            }
        }
        
        public int w {
            get {
                return this.v3;
            }
            set {
                this.v3 = value;
            }
        }
        
        public int a {
            get {
                return this.v3;
            }
            set {
                this.v3 = value;
            }
        }
        
        public int q {
            get {
                return this.v3;
            }
            set {
                this.v3 = value;
            }
        }
        
        public ivec2 xx {
            get {
                return new ivec2(this.v0, this.v0);
            }
        }
        
        public ivec2 rr {
            get {
                return new ivec2(this.v0, this.v0);
            }
        }
        
        public ivec2 ss {
            get {
                return new ivec2(this.v0, this.v0);
            }
        }
        
        public ivec2 xy {
            get {
                return new ivec2(this.v0, this.v1);
            }
            set {
                this.v0 = value.v0;
                this.v1 = value.v1;
            }
        }
        
        public ivec2 rg {
            get {
                return new ivec2(this.v0, this.v1);
            }
            set {
                this.v0 = value.v0;
                this.v1 = value.v1;
            }
        }
        
        public ivec2 st {
            get {
                return new ivec2(this.v0, this.v1);
            }
            set {
                this.v0 = value.v0;
                this.v1 = value.v1;
            }
        }
        
        public ivec2 xz {
            get {
                return new ivec2(this.v0, this.v2);
            }
            set {
                this.v0 = value.v0;
                this.v2 = value.v1;
            }
        }
        
        public ivec2 rb {
            get {
                return new ivec2(this.v0, this.v2);
            }
            set {
                this.v0 = value.v0;
                this.v2 = value.v1;
            }
        }
        
        public ivec2 sp {
            get {
                return new ivec2(this.v0, this.v2);
            }
            set {
                this.v0 = value.v0;
                this.v2 = value.v1;
            }
        }
        
        public ivec2 xw {
            get {
                return new ivec2(this.v0, this.v3);
            }
            set {
                this.v0 = value.v0;
                this.v3 = value.v1;
            }
        }
        
        public ivec2 ra {
            get {
                return new ivec2(this.v0, this.v3);
            }
            set {
                this.v0 = value.v0;
                this.v3 = value.v1;
            }
        }
        
        public ivec2 sq {
            get {
                return new ivec2(this.v0, this.v3);
            }
            set {
                this.v0 = value.v0;
                this.v3 = value.v1;
            }
        }
        
        public ivec2 yx {
            get {
                return new ivec2(this.v1, this.v0);
            }
            set {
                this.v1 = value.v0;
                this.v0 = value.v1;
            }
        }
        
        public ivec2 gr {
            get {
                return new ivec2(this.v1, this.v0);
            }
            set {
                this.v1 = value.v0;
                this.v0 = value.v1;
            }
        }
        
        public ivec2 ts {
            get {
                return new ivec2(this.v1, this.v0);
            }
            set {
                this.v1 = value.v0;
                this.v0 = value.v1;
            }
        }
        
        public ivec2 yy {
            get {
                return new ivec2(this.v1, this.v1);
            }
        }
        
        public ivec2 gg {
            get {
                return new ivec2(this.v1, this.v1);
            }
        }
        
        public ivec2 tt {
            get {
                return new ivec2(this.v1, this.v1);
            }
        }
        
        public ivec2 yz {
            get {
                return new ivec2(this.v1, this.v2);
            }
            set {
                this.v1 = value.v0;
                this.v2 = value.v1;
            }
        }
        
        public ivec2 gb {
            get {
                return new ivec2(this.v1, this.v2);
            }
            set {
                this.v1 = value.v0;
                this.v2 = value.v1;
            }
        }
        
        public ivec2 tp {
            get {
                return new ivec2(this.v1, this.v2);
            }
            set {
                this.v1 = value.v0;
                this.v2 = value.v1;
            }
        }
        
        public ivec2 yw {
            get {
                return new ivec2(this.v1, this.v3);
            }
            set {
                this.v1 = value.v0;
                this.v3 = value.v1;
            }
        }
        
        public ivec2 ga {
            get {
                return new ivec2(this.v1, this.v3);
            }
            set {
                this.v1 = value.v0;
                this.v3 = value.v1;
            }
        }
        
        public ivec2 tq {
            get {
                return new ivec2(this.v1, this.v3);
            }
            set {
                this.v1 = value.v0;
                this.v3 = value.v1;
            }
        }
        
        public ivec2 zx {
            get {
                return new ivec2(this.v2, this.v0);
            }
            set {
                this.v2 = value.v0;
                this.v0 = value.v1;
            }
        }
        
        public ivec2 br {
            get {
                return new ivec2(this.v2, this.v0);
            }
            set {
                this.v2 = value.v0;
                this.v0 = value.v1;
            }
        }
        
        public ivec2 ps {
            get {
                return new ivec2(this.v2, this.v0);
            }
            set {
                this.v2 = value.v0;
                this.v0 = value.v1;
            }
        }
        
        public ivec2 zy {
            get {
                return new ivec2(this.v2, this.v1);
            }
            set {
                this.v2 = value.v0;
                this.v1 = value.v1;
            }
        }
        
        public ivec2 bg {
            get {
                return new ivec2(this.v2, this.v1);
            }
            set {
                this.v2 = value.v0;
                this.v1 = value.v1;
            }
        }
        
        public ivec2 pt {
            get {
                return new ivec2(this.v2, this.v1);
            }
            set {
                this.v2 = value.v0;
                this.v1 = value.v1;
            }
        }
        
        public ivec2 zz {
            get {
                return new ivec2(this.v2, this.v2);
            }
        }
        
        public ivec2 bb {
            get {
                return new ivec2(this.v2, this.v2);
            }
        }
        
        public ivec2 pp {
            get {
                return new ivec2(this.v2, this.v2);
            }
        }
        
        public ivec2 zw {
            get {
                return new ivec2(this.v2, this.v3);
            }
            set {
                this.v2 = value.v0;
                this.v3 = value.v1;
            }
        }
        
        public ivec2 ba {
            get {
                return new ivec2(this.v2, this.v3);
            }
            set {
                this.v2 = value.v0;
                this.v3 = value.v1;
            }
        }
        
        public ivec2 pq {
            get {
                return new ivec2(this.v2, this.v3);
            }
            set {
                this.v2 = value.v0;
                this.v3 = value.v1;
            }
        }
        
        public ivec2 wx {
            get {
                return new ivec2(this.v3, this.v0);
            }
            set {
                this.v3 = value.v0;
                this.v0 = value.v1;
            }
        }
        
        public ivec2 ar {
            get {
                return new ivec2(this.v3, this.v0);
            }
            set {
                this.v3 = value.v0;
                this.v0 = value.v1;
            }
        }
        
        public ivec2 qs {
            get {
                return new ivec2(this.v3, this.v0);
            }
            set {
                this.v3 = value.v0;
                this.v0 = value.v1;
            }
        }
        
        public ivec2 wy {
            get {
                return new ivec2(this.v3, this.v1);
            }
            set {
                this.v3 = value.v0;
                this.v1 = value.v1;
            }
        }
        
        public ivec2 ag {
            get {
                return new ivec2(this.v3, this.v1);
            }
            set {
                this.v3 = value.v0;
                this.v1 = value.v1;
            }
        }
        
        public ivec2 qt {
            get {
                return new ivec2(this.v3, this.v1);
            }
            set {
                this.v3 = value.v0;
                this.v1 = value.v1;
            }
        }
        
        public ivec2 wz {
            get {
                return new ivec2(this.v3, this.v2);
            }
            set {
                this.v3 = value.v0;
                this.v2 = value.v1;
            }
        }
        
        public ivec2 ab {
            get {
                return new ivec2(this.v3, this.v2);
            }
            set {
                this.v3 = value.v0;
                this.v2 = value.v1;
            }
        }
        
        public ivec2 qp {
            get {
                return new ivec2(this.v3, this.v2);
            }
            set {
                this.v3 = value.v0;
                this.v2 = value.v1;
            }
        }
        
        public ivec2 ww {
            get {
                return new ivec2(this.v3, this.v3);
            }
        }
        
        public ivec2 aa {
            get {
                return new ivec2(this.v3, this.v3);
            }
        }
        
        public ivec2 qq {
            get {
                return new ivec2(this.v3, this.v3);
            }
        }
        
        public ivec3 xxx {
            get {
                return new ivec3(this.v0, this.v0, this.v0);
            }
        }
        
        public ivec3 rrr {
            get {
                return new ivec3(this.v0, this.v0, this.v0);
            }
        }
        
        public ivec3 sss {
            get {
                return new ivec3(this.v0, this.v0, this.v0);
            }
        }
        
        public ivec3 xxy {
            get {
                return new ivec3(this.v0, this.v0, this.v1);
            }
        }
        
        public ivec3 rrg {
            get {
                return new ivec3(this.v0, this.v0, this.v1);
            }
        }
        
        public ivec3 sst {
            get {
                return new ivec3(this.v0, this.v0, this.v1);
            }
        }
        
        public ivec3 xxz {
            get {
                return new ivec3(this.v0, this.v0, this.v2);
            }
        }
        
        public ivec3 rrb {
            get {
                return new ivec3(this.v0, this.v0, this.v2);
            }
        }
        
        public ivec3 ssp {
            get {
                return new ivec3(this.v0, this.v0, this.v2);
            }
        }
        
        public ivec3 xxw {
            get {
                return new ivec3(this.v0, this.v0, this.v3);
            }
        }
        
        public ivec3 rra {
            get {
                return new ivec3(this.v0, this.v0, this.v3);
            }
        }
        
        public ivec3 ssq {
            get {
                return new ivec3(this.v0, this.v0, this.v3);
            }
        }
        
        public ivec3 xyx {
            get {
                return new ivec3(this.v0, this.v1, this.v0);
            }
        }
        
        public ivec3 rgr {
            get {
                return new ivec3(this.v0, this.v1, this.v0);
            }
        }
        
        public ivec3 sts {
            get {
                return new ivec3(this.v0, this.v1, this.v0);
            }
        }
        
        public ivec3 xyy {
            get {
                return new ivec3(this.v0, this.v1, this.v1);
            }
        }
        
        public ivec3 rgg {
            get {
                return new ivec3(this.v0, this.v1, this.v1);
            }
        }
        
        public ivec3 stt {
            get {
                return new ivec3(this.v0, this.v1, this.v1);
            }
        }
        
        public ivec3 xyz {
            get {
                return new ivec3(this.v0, this.v1, this.v2);
            }
            set {
                this.v0 = value.v0;
                this.v1 = value.v1;
                this.v2 = value.v2;
            }
        }
        
        public ivec3 rgb {
            get {
                return new ivec3(this.v0, this.v1, this.v2);
            }
            set {
                this.v0 = value.v0;
                this.v1 = value.v1;
                this.v2 = value.v2;
            }
        }
        
        public ivec3 stp {
            get {
                return new ivec3(this.v0, this.v1, this.v2);
            }
            set {
                this.v0 = value.v0;
                this.v1 = value.v1;
                this.v2 = value.v2;
            }
        }
        
        public ivec3 xyw {
            get {
                return new ivec3(this.v0, this.v1, this.v3);
            }
            set {
                this.v0 = value.v0;
                this.v1 = value.v1;
                this.v3 = value.v2;
            }
        }
        
        public ivec3 rga {
            get {
                return new ivec3(this.v0, this.v1, this.v3);
            }
            set {
                this.v0 = value.v0;
                this.v1 = value.v1;
                this.v3 = value.v2;
            }
        }
        
        public ivec3 stq {
            get {
                return new ivec3(this.v0, this.v1, this.v3);
            }
            set {
                this.v0 = value.v0;
                this.v1 = value.v1;
                this.v3 = value.v2;
            }
        }
        
        public ivec3 xzx {
            get {
                return new ivec3(this.v0, this.v2, this.v0);
            }
        }
        
        public ivec3 rbr {
            get {
                return new ivec3(this.v0, this.v2, this.v0);
            }
        }
        
        public ivec3 sps {
            get {
                return new ivec3(this.v0, this.v2, this.v0);
            }
        }
        
        public ivec3 xzy {
            get {
                return new ivec3(this.v0, this.v2, this.v1);
            }
            set {
                this.v0 = value.v0;
                this.v2 = value.v1;
                this.v1 = value.v2;
            }
        }
        
        public ivec3 rbg {
            get {
                return new ivec3(this.v0, this.v2, this.v1);
            }
            set {
                this.v0 = value.v0;
                this.v2 = value.v1;
                this.v1 = value.v2;
            }
        }
        
        public ivec3 spt {
            get {
                return new ivec3(this.v0, this.v2, this.v1);
            }
            set {
                this.v0 = value.v0;
                this.v2 = value.v1;
                this.v1 = value.v2;
            }
        }
        
        public ivec3 xzz {
            get {
                return new ivec3(this.v0, this.v2, this.v2);
            }
        }
        
        public ivec3 rbb {
            get {
                return new ivec3(this.v0, this.v2, this.v2);
            }
        }
        
        public ivec3 spp {
            get {
                return new ivec3(this.v0, this.v2, this.v2);
            }
        }
        
        public ivec3 xzw {
            get {
                return new ivec3(this.v0, this.v2, this.v3);
            }
            set {
                this.v0 = value.v0;
                this.v2 = value.v1;
                this.v3 = value.v2;
            }
        }
        
        public ivec3 rba {
            get {
                return new ivec3(this.v0, this.v2, this.v3);
            }
            set {
                this.v0 = value.v0;
                this.v2 = value.v1;
                this.v3 = value.v2;
            }
        }
        
        public ivec3 spq {
            get {
                return new ivec3(this.v0, this.v2, this.v3);
            }
            set {
                this.v0 = value.v0;
                this.v2 = value.v1;
                this.v3 = value.v2;
            }
        }
        
        public ivec3 xwx {
            get {
                return new ivec3(this.v0, this.v3, this.v0);
            }
        }
        
        public ivec3 rar {
            get {
                return new ivec3(this.v0, this.v3, this.v0);
            }
        }
        
        public ivec3 sqs {
            get {
                return new ivec3(this.v0, this.v3, this.v0);
            }
        }
        
        public ivec3 xwy {
            get {
                return new ivec3(this.v0, this.v3, this.v1);
            }
            set {
                this.v0 = value.v0;
                this.v3 = value.v1;
                this.v1 = value.v2;
            }
        }
        
        public ivec3 rag {
            get {
                return new ivec3(this.v0, this.v3, this.v1);
            }
            set {
                this.v0 = value.v0;
                this.v3 = value.v1;
                this.v1 = value.v2;
            }
        }
        
        public ivec3 sqt {
            get {
                return new ivec3(this.v0, this.v3, this.v1);
            }
            set {
                this.v0 = value.v0;
                this.v3 = value.v1;
                this.v1 = value.v2;
            }
        }
        
        public ivec3 xwz {
            get {
                return new ivec3(this.v0, this.v3, this.v2);
            }
            set {
                this.v0 = value.v0;
                this.v3 = value.v1;
                this.v2 = value.v2;
            }
        }
        
        public ivec3 rab {
            get {
                return new ivec3(this.v0, this.v3, this.v2);
            }
            set {
                this.v0 = value.v0;
                this.v3 = value.v1;
                this.v2 = value.v2;
            }
        }
        
        public ivec3 sqp {
            get {
                return new ivec3(this.v0, this.v3, this.v2);
            }
            set {
                this.v0 = value.v0;
                this.v3 = value.v1;
                this.v2 = value.v2;
            }
        }
        
        public ivec3 xww {
            get {
                return new ivec3(this.v0, this.v3, this.v3);
            }
        }
        
        public ivec3 raa {
            get {
                return new ivec3(this.v0, this.v3, this.v3);
            }
        }
        
        public ivec3 sqq {
            get {
                return new ivec3(this.v0, this.v3, this.v3);
            }
        }
        
        public ivec3 yxx {
            get {
                return new ivec3(this.v1, this.v0, this.v0);
            }
        }
        
        public ivec3 grr {
            get {
                return new ivec3(this.v1, this.v0, this.v0);
            }
        }
        
        public ivec3 tss {
            get {
                return new ivec3(this.v1, this.v0, this.v0);
            }
        }
        
        public ivec3 yxy {
            get {
                return new ivec3(this.v1, this.v0, this.v1);
            }
        }
        
        public ivec3 grg {
            get {
                return new ivec3(this.v1, this.v0, this.v1);
            }
        }
        
        public ivec3 tst {
            get {
                return new ivec3(this.v1, this.v0, this.v1);
            }
        }
        
        public ivec3 yxz {
            get {
                return new ivec3(this.v1, this.v0, this.v2);
            }
            set {
                this.v1 = value.v0;
                this.v0 = value.v1;
                this.v2 = value.v2;
            }
        }
        
        public ivec3 grb {
            get {
                return new ivec3(this.v1, this.v0, this.v2);
            }
            set {
                this.v1 = value.v0;
                this.v0 = value.v1;
                this.v2 = value.v2;
            }
        }
        
        public ivec3 tsp {
            get {
                return new ivec3(this.v1, this.v0, this.v2);
            }
            set {
                this.v1 = value.v0;
                this.v0 = value.v1;
                this.v2 = value.v2;
            }
        }
        
        public ivec3 yxw {
            get {
                return new ivec3(this.v1, this.v0, this.v3);
            }
            set {
                this.v1 = value.v0;
                this.v0 = value.v1;
                this.v3 = value.v2;
            }
        }
        
        public ivec3 gra {
            get {
                return new ivec3(this.v1, this.v0, this.v3);
            }
            set {
                this.v1 = value.v0;
                this.v0 = value.v1;
                this.v3 = value.v2;
            }
        }
        
        public ivec3 tsq {
            get {
                return new ivec3(this.v1, this.v0, this.v3);
            }
            set {
                this.v1 = value.v0;
                this.v0 = value.v1;
                this.v3 = value.v2;
            }
        }
        
        public ivec3 yyx {
            get {
                return new ivec3(this.v1, this.v1, this.v0);
            }
        }
        
        public ivec3 ggr {
            get {
                return new ivec3(this.v1, this.v1, this.v0);
            }
        }
        
        public ivec3 tts {
            get {
                return new ivec3(this.v1, this.v1, this.v0);
            }
        }
        
        public ivec3 yyy {
            get {
                return new ivec3(this.v1, this.v1, this.v1);
            }
        }
        
        public ivec3 ggg {
            get {
                return new ivec3(this.v1, this.v1, this.v1);
            }
        }
        
        public ivec3 ttt {
            get {
                return new ivec3(this.v1, this.v1, this.v1);
            }
        }
        
        public ivec3 yyz {
            get {
                return new ivec3(this.v1, this.v1, this.v2);
            }
        }
        
        public ivec3 ggb {
            get {
                return new ivec3(this.v1, this.v1, this.v2);
            }
        }
        
        public ivec3 ttp {
            get {
                return new ivec3(this.v1, this.v1, this.v2);
            }
        }
        
        public ivec3 yyw {
            get {
                return new ivec3(this.v1, this.v1, this.v3);
            }
        }
        
        public ivec3 gga {
            get {
                return new ivec3(this.v1, this.v1, this.v3);
            }
        }
        
        public ivec3 ttq {
            get {
                return new ivec3(this.v1, this.v1, this.v3);
            }
        }
        
        public ivec3 yzx {
            get {
                return new ivec3(this.v1, this.v2, this.v0);
            }
            set {
                this.v1 = value.v0;
                this.v2 = value.v1;
                this.v0 = value.v2;
            }
        }
        
        public ivec3 gbr {
            get {
                return new ivec3(this.v1, this.v2, this.v0);
            }
            set {
                this.v1 = value.v0;
                this.v2 = value.v1;
                this.v0 = value.v2;
            }
        }
        
        public ivec3 tps {
            get {
                return new ivec3(this.v1, this.v2, this.v0);
            }
            set {
                this.v1 = value.v0;
                this.v2 = value.v1;
                this.v0 = value.v2;
            }
        }
        
        public ivec3 yzy {
            get {
                return new ivec3(this.v1, this.v2, this.v1);
            }
        }
        
        public ivec3 gbg {
            get {
                return new ivec3(this.v1, this.v2, this.v1);
            }
        }
        
        public ivec3 tpt {
            get {
                return new ivec3(this.v1, this.v2, this.v1);
            }
        }
        
        public ivec3 yzz {
            get {
                return new ivec3(this.v1, this.v2, this.v2);
            }
        }
        
        public ivec3 gbb {
            get {
                return new ivec3(this.v1, this.v2, this.v2);
            }
        }
        
        public ivec3 tpp {
            get {
                return new ivec3(this.v1, this.v2, this.v2);
            }
        }
        
        public ivec3 yzw {
            get {
                return new ivec3(this.v1, this.v2, this.v3);
            }
            set {
                this.v1 = value.v0;
                this.v2 = value.v1;
                this.v3 = value.v2;
            }
        }
        
        public ivec3 gba {
            get {
                return new ivec3(this.v1, this.v2, this.v3);
            }
            set {
                this.v1 = value.v0;
                this.v2 = value.v1;
                this.v3 = value.v2;
            }
        }
        
        public ivec3 tpq {
            get {
                return new ivec3(this.v1, this.v2, this.v3);
            }
            set {
                this.v1 = value.v0;
                this.v2 = value.v1;
                this.v3 = value.v2;
            }
        }
        
        public ivec3 ywx {
            get {
                return new ivec3(this.v1, this.v3, this.v0);
            }
            set {
                this.v1 = value.v0;
                this.v3 = value.v1;
                this.v0 = value.v2;
            }
        }
        
        public ivec3 gar {
            get {
                return new ivec3(this.v1, this.v3, this.v0);
            }
            set {
                this.v1 = value.v0;
                this.v3 = value.v1;
                this.v0 = value.v2;
            }
        }
        
        public ivec3 tqs {
            get {
                return new ivec3(this.v1, this.v3, this.v0);
            }
            set {
                this.v1 = value.v0;
                this.v3 = value.v1;
                this.v0 = value.v2;
            }
        }
        
        public ivec3 ywy {
            get {
                return new ivec3(this.v1, this.v3, this.v1);
            }
        }
        
        public ivec3 gag {
            get {
                return new ivec3(this.v1, this.v3, this.v1);
            }
        }
        
        public ivec3 tqt {
            get {
                return new ivec3(this.v1, this.v3, this.v1);
            }
        }
        
        public ivec3 ywz {
            get {
                return new ivec3(this.v1, this.v3, this.v2);
            }
            set {
                this.v1 = value.v0;
                this.v3 = value.v1;
                this.v2 = value.v2;
            }
        }
        
        public ivec3 gab {
            get {
                return new ivec3(this.v1, this.v3, this.v2);
            }
            set {
                this.v1 = value.v0;
                this.v3 = value.v1;
                this.v2 = value.v2;
            }
        }
        
        public ivec3 tqp {
            get {
                return new ivec3(this.v1, this.v3, this.v2);
            }
            set {
                this.v1 = value.v0;
                this.v3 = value.v1;
                this.v2 = value.v2;
            }
        }
        
        public ivec3 yww {
            get {
                return new ivec3(this.v1, this.v3, this.v3);
            }
        }
        
        public ivec3 gaa {
            get {
                return new ivec3(this.v1, this.v3, this.v3);
            }
        }
        
        public ivec3 tqq {
            get {
                return new ivec3(this.v1, this.v3, this.v3);
            }
        }
        
        public ivec3 zxx {
            get {
                return new ivec3(this.v2, this.v0, this.v0);
            }
        }
        
        public ivec3 brr {
            get {
                return new ivec3(this.v2, this.v0, this.v0);
            }
        }
        
        public ivec3 pss {
            get {
                return new ivec3(this.v2, this.v0, this.v0);
            }
        }
        
        public ivec3 zxy {
            get {
                return new ivec3(this.v2, this.v0, this.v1);
            }
            set {
                this.v2 = value.v0;
                this.v0 = value.v1;
                this.v1 = value.v2;
            }
        }
        
        public ivec3 brg {
            get {
                return new ivec3(this.v2, this.v0, this.v1);
            }
            set {
                this.v2 = value.v0;
                this.v0 = value.v1;
                this.v1 = value.v2;
            }
        }
        
        public ivec3 pst {
            get {
                return new ivec3(this.v2, this.v0, this.v1);
            }
            set {
                this.v2 = value.v0;
                this.v0 = value.v1;
                this.v1 = value.v2;
            }
        }
        
        public ivec3 zxz {
            get {
                return new ivec3(this.v2, this.v0, this.v2);
            }
        }
        
        public ivec3 brb {
            get {
                return new ivec3(this.v2, this.v0, this.v2);
            }
        }
        
        public ivec3 psp {
            get {
                return new ivec3(this.v2, this.v0, this.v2);
            }
        }
        
        public ivec3 zxw {
            get {
                return new ivec3(this.v2, this.v0, this.v3);
            }
            set {
                this.v2 = value.v0;
                this.v0 = value.v1;
                this.v3 = value.v2;
            }
        }
        
        public ivec3 bra {
            get {
                return new ivec3(this.v2, this.v0, this.v3);
            }
            set {
                this.v2 = value.v0;
                this.v0 = value.v1;
                this.v3 = value.v2;
            }
        }
        
        public ivec3 psq {
            get {
                return new ivec3(this.v2, this.v0, this.v3);
            }
            set {
                this.v2 = value.v0;
                this.v0 = value.v1;
                this.v3 = value.v2;
            }
        }
        
        public ivec3 zyx {
            get {
                return new ivec3(this.v2, this.v1, this.v0);
            }
            set {
                this.v2 = value.v0;
                this.v1 = value.v1;
                this.v0 = value.v2;
            }
        }
        
        public ivec3 bgr {
            get {
                return new ivec3(this.v2, this.v1, this.v0);
            }
            set {
                this.v2 = value.v0;
                this.v1 = value.v1;
                this.v0 = value.v2;
            }
        }
        
        public ivec3 pts {
            get {
                return new ivec3(this.v2, this.v1, this.v0);
            }
            set {
                this.v2 = value.v0;
                this.v1 = value.v1;
                this.v0 = value.v2;
            }
        }
        
        public ivec3 zyy {
            get {
                return new ivec3(this.v2, this.v1, this.v1);
            }
        }
        
        public ivec3 bgg {
            get {
                return new ivec3(this.v2, this.v1, this.v1);
            }
        }
        
        public ivec3 ptt {
            get {
                return new ivec3(this.v2, this.v1, this.v1);
            }
        }
        
        public ivec3 zyz {
            get {
                return new ivec3(this.v2, this.v1, this.v2);
            }
        }
        
        public ivec3 bgb {
            get {
                return new ivec3(this.v2, this.v1, this.v2);
            }
        }
        
        public ivec3 ptp {
            get {
                return new ivec3(this.v2, this.v1, this.v2);
            }
        }
        
        public ivec3 zyw {
            get {
                return new ivec3(this.v2, this.v1, this.v3);
            }
            set {
                this.v2 = value.v0;
                this.v1 = value.v1;
                this.v3 = value.v2;
            }
        }
        
        public ivec3 bga {
            get {
                return new ivec3(this.v2, this.v1, this.v3);
            }
            set {
                this.v2 = value.v0;
                this.v1 = value.v1;
                this.v3 = value.v2;
            }
        }
        
        public ivec3 ptq {
            get {
                return new ivec3(this.v2, this.v1, this.v3);
            }
            set {
                this.v2 = value.v0;
                this.v1 = value.v1;
                this.v3 = value.v2;
            }
        }
        
        public ivec3 zzx {
            get {
                return new ivec3(this.v2, this.v2, this.v0);
            }
        }
        
        public ivec3 bbr {
            get {
                return new ivec3(this.v2, this.v2, this.v0);
            }
        }
        
        public ivec3 pps {
            get {
                return new ivec3(this.v2, this.v2, this.v0);
            }
        }
        
        public ivec3 zzy {
            get {
                return new ivec3(this.v2, this.v2, this.v1);
            }
        }
        
        public ivec3 bbg {
            get {
                return new ivec3(this.v2, this.v2, this.v1);
            }
        }
        
        public ivec3 ppt {
            get {
                return new ivec3(this.v2, this.v2, this.v1);
            }
        }
        
        public ivec3 zzz {
            get {
                return new ivec3(this.v2, this.v2, this.v2);
            }
        }
        
        public ivec3 bbb {
            get {
                return new ivec3(this.v2, this.v2, this.v2);
            }
        }
        
        public ivec3 ppp {
            get {
                return new ivec3(this.v2, this.v2, this.v2);
            }
        }
        
        public ivec3 zzw {
            get {
                return new ivec3(this.v2, this.v2, this.v3);
            }
        }
        
        public ivec3 bba {
            get {
                return new ivec3(this.v2, this.v2, this.v3);
            }
        }
        
        public ivec3 ppq {
            get {
                return new ivec3(this.v2, this.v2, this.v3);
            }
        }
        
        public ivec3 zwx {
            get {
                return new ivec3(this.v2, this.v3, this.v0);
            }
            set {
                this.v2 = value.v0;
                this.v3 = value.v1;
                this.v0 = value.v2;
            }
        }
        
        public ivec3 bar {
            get {
                return new ivec3(this.v2, this.v3, this.v0);
            }
            set {
                this.v2 = value.v0;
                this.v3 = value.v1;
                this.v0 = value.v2;
            }
        }
        
        public ivec3 pqs {
            get {
                return new ivec3(this.v2, this.v3, this.v0);
            }
            set {
                this.v2 = value.v0;
                this.v3 = value.v1;
                this.v0 = value.v2;
            }
        }
        
        public ivec3 zwy {
            get {
                return new ivec3(this.v2, this.v3, this.v1);
            }
            set {
                this.v2 = value.v0;
                this.v3 = value.v1;
                this.v1 = value.v2;
            }
        }
        
        public ivec3 bag {
            get {
                return new ivec3(this.v2, this.v3, this.v1);
            }
            set {
                this.v2 = value.v0;
                this.v3 = value.v1;
                this.v1 = value.v2;
            }
        }
        
        public ivec3 pqt {
            get {
                return new ivec3(this.v2, this.v3, this.v1);
            }
            set {
                this.v2 = value.v0;
                this.v3 = value.v1;
                this.v1 = value.v2;
            }
        }
        
        public ivec3 zwz {
            get {
                return new ivec3(this.v2, this.v3, this.v2);
            }
        }
        
        public ivec3 bab {
            get {
                return new ivec3(this.v2, this.v3, this.v2);
            }
        }
        
        public ivec3 pqp {
            get {
                return new ivec3(this.v2, this.v3, this.v2);
            }
        }
        
        public ivec3 zww {
            get {
                return new ivec3(this.v2, this.v3, this.v3);
            }
        }
        
        public ivec3 baa {
            get {
                return new ivec3(this.v2, this.v3, this.v3);
            }
        }
        
        public ivec3 pqq {
            get {
                return new ivec3(this.v2, this.v3, this.v3);
            }
        }
        
        public ivec3 wxx {
            get {
                return new ivec3(this.v3, this.v0, this.v0);
            }
        }
        
        public ivec3 arr {
            get {
                return new ivec3(this.v3, this.v0, this.v0);
            }
        }
        
        public ivec3 qss {
            get {
                return new ivec3(this.v3, this.v0, this.v0);
            }
        }
        
        public ivec3 wxy {
            get {
                return new ivec3(this.v3, this.v0, this.v1);
            }
            set {
                this.v3 = value.v0;
                this.v0 = value.v1;
                this.v1 = value.v2;
            }
        }
        
        public ivec3 arg {
            get {
                return new ivec3(this.v3, this.v0, this.v1);
            }
            set {
                this.v3 = value.v0;
                this.v0 = value.v1;
                this.v1 = value.v2;
            }
        }
        
        public ivec3 qst {
            get {
                return new ivec3(this.v3, this.v0, this.v1);
            }
            set {
                this.v3 = value.v0;
                this.v0 = value.v1;
                this.v1 = value.v2;
            }
        }
        
        public ivec3 wxz {
            get {
                return new ivec3(this.v3, this.v0, this.v2);
            }
            set {
                this.v3 = value.v0;
                this.v0 = value.v1;
                this.v2 = value.v2;
            }
        }
        
        public ivec3 arb {
            get {
                return new ivec3(this.v3, this.v0, this.v2);
            }
            set {
                this.v3 = value.v0;
                this.v0 = value.v1;
                this.v2 = value.v2;
            }
        }
        
        public ivec3 qsp {
            get {
                return new ivec3(this.v3, this.v0, this.v2);
            }
            set {
                this.v3 = value.v0;
                this.v0 = value.v1;
                this.v2 = value.v2;
            }
        }
        
        public ivec3 wxw {
            get {
                return new ivec3(this.v3, this.v0, this.v3);
            }
        }
        
        public ivec3 ara {
            get {
                return new ivec3(this.v3, this.v0, this.v3);
            }
        }
        
        public ivec3 qsq {
            get {
                return new ivec3(this.v3, this.v0, this.v3);
            }
        }
        
        public ivec3 wyx {
            get {
                return new ivec3(this.v3, this.v1, this.v0);
            }
            set {
                this.v3 = value.v0;
                this.v1 = value.v1;
                this.v0 = value.v2;
            }
        }
        
        public ivec3 agr {
            get {
                return new ivec3(this.v3, this.v1, this.v0);
            }
            set {
                this.v3 = value.v0;
                this.v1 = value.v1;
                this.v0 = value.v2;
            }
        }
        
        public ivec3 qts {
            get {
                return new ivec3(this.v3, this.v1, this.v0);
            }
            set {
                this.v3 = value.v0;
                this.v1 = value.v1;
                this.v0 = value.v2;
            }
        }
        
        public ivec3 wyy {
            get {
                return new ivec3(this.v3, this.v1, this.v1);
            }
        }
        
        public ivec3 agg {
            get {
                return new ivec3(this.v3, this.v1, this.v1);
            }
        }
        
        public ivec3 qtt {
            get {
                return new ivec3(this.v3, this.v1, this.v1);
            }
        }
        
        public ivec3 wyz {
            get {
                return new ivec3(this.v3, this.v1, this.v2);
            }
            set {
                this.v3 = value.v0;
                this.v1 = value.v1;
                this.v2 = value.v2;
            }
        }
        
        public ivec3 agb {
            get {
                return new ivec3(this.v3, this.v1, this.v2);
            }
            set {
                this.v3 = value.v0;
                this.v1 = value.v1;
                this.v2 = value.v2;
            }
        }
        
        public ivec3 qtp {
            get {
                return new ivec3(this.v3, this.v1, this.v2);
            }
            set {
                this.v3 = value.v0;
                this.v1 = value.v1;
                this.v2 = value.v2;
            }
        }
        
        public ivec3 wyw {
            get {
                return new ivec3(this.v3, this.v1, this.v3);
            }
        }
        
        public ivec3 aga {
            get {
                return new ivec3(this.v3, this.v1, this.v3);
            }
        }
        
        public ivec3 qtq {
            get {
                return new ivec3(this.v3, this.v1, this.v3);
            }
        }
        
        public ivec3 wzx {
            get {
                return new ivec3(this.v3, this.v2, this.v0);
            }
            set {
                this.v3 = value.v0;
                this.v2 = value.v1;
                this.v0 = value.v2;
            }
        }
        
        public ivec3 abr {
            get {
                return new ivec3(this.v3, this.v2, this.v0);
            }
            set {
                this.v3 = value.v0;
                this.v2 = value.v1;
                this.v0 = value.v2;
            }
        }
        
        public ivec3 qps {
            get {
                return new ivec3(this.v3, this.v2, this.v0);
            }
            set {
                this.v3 = value.v0;
                this.v2 = value.v1;
                this.v0 = value.v2;
            }
        }
        
        public ivec3 wzy {
            get {
                return new ivec3(this.v3, this.v2, this.v1);
            }
            set {
                this.v3 = value.v0;
                this.v2 = value.v1;
                this.v1 = value.v2;
            }
        }
        
        public ivec3 abg {
            get {
                return new ivec3(this.v3, this.v2, this.v1);
            }
            set {
                this.v3 = value.v0;
                this.v2 = value.v1;
                this.v1 = value.v2;
            }
        }
        
        public ivec3 qpt {
            get {
                return new ivec3(this.v3, this.v2, this.v1);
            }
            set {
                this.v3 = value.v0;
                this.v2 = value.v1;
                this.v1 = value.v2;
            }
        }
        
        public ivec3 wzz {
            get {
                return new ivec3(this.v3, this.v2, this.v2);
            }
        }
        
        public ivec3 abb {
            get {
                return new ivec3(this.v3, this.v2, this.v2);
            }
        }
        
        public ivec3 qpp {
            get {
                return new ivec3(this.v3, this.v2, this.v2);
            }
        }
        
        public ivec3 wzw {
            get {
                return new ivec3(this.v3, this.v2, this.v3);
            }
        }
        
        public ivec3 aba {
            get {
                return new ivec3(this.v3, this.v2, this.v3);
            }
        }
        
        public ivec3 qpq {
            get {
                return new ivec3(this.v3, this.v2, this.v3);
            }
        }
        
        public ivec3 wwx {
            get {
                return new ivec3(this.v3, this.v3, this.v0);
            }
        }
        
        public ivec3 aar {
            get {
                return new ivec3(this.v3, this.v3, this.v0);
            }
        }
        
        public ivec3 qqs {
            get {
                return new ivec3(this.v3, this.v3, this.v0);
            }
        }
        
        public ivec3 wwy {
            get {
                return new ivec3(this.v3, this.v3, this.v1);
            }
        }
        
        public ivec3 aag {
            get {
                return new ivec3(this.v3, this.v3, this.v1);
            }
        }
        
        public ivec3 qqt {
            get {
                return new ivec3(this.v3, this.v3, this.v1);
            }
        }
        
        public ivec3 wwz {
            get {
                return new ivec3(this.v3, this.v3, this.v2);
            }
        }
        
        public ivec3 aab {
            get {
                return new ivec3(this.v3, this.v3, this.v2);
            }
        }
        
        public ivec3 qqp {
            get {
                return new ivec3(this.v3, this.v3, this.v2);
            }
        }
        
        public ivec3 www {
            get {
                return new ivec3(this.v3, this.v3, this.v3);
            }
        }
        
        public ivec3 aaa {
            get {
                return new ivec3(this.v3, this.v3, this.v3);
            }
        }
        
        public ivec3 qqq {
            get {
                return new ivec3(this.v3, this.v3, this.v3);
            }
        }
        
        public ivec4 xxxx {
            get {
                return new ivec4(this.v0, this.v0, this.v0, this.v0);
            }
        }
        
        public ivec4 rrrr {
            get {
                return new ivec4(this.v0, this.v0, this.v0, this.v0);
            }
        }
        
        public ivec4 ssss {
            get {
                return new ivec4(this.v0, this.v0, this.v0, this.v0);
            }
        }
        
        public ivec4 xxxy {
            get {
                return new ivec4(this.v0, this.v0, this.v0, this.v1);
            }
        }
        
        public ivec4 rrrg {
            get {
                return new ivec4(this.v0, this.v0, this.v0, this.v1);
            }
        }
        
        public ivec4 ssst {
            get {
                return new ivec4(this.v0, this.v0, this.v0, this.v1);
            }
        }
        
        public ivec4 xxxz {
            get {
                return new ivec4(this.v0, this.v0, this.v0, this.v2);
            }
        }
        
        public ivec4 rrrb {
            get {
                return new ivec4(this.v0, this.v0, this.v0, this.v2);
            }
        }
        
        public ivec4 sssp {
            get {
                return new ivec4(this.v0, this.v0, this.v0, this.v2);
            }
        }
        
        public ivec4 xxxw {
            get {
                return new ivec4(this.v0, this.v0, this.v0, this.v3);
            }
        }
        
        public ivec4 rrra {
            get {
                return new ivec4(this.v0, this.v0, this.v0, this.v3);
            }
        }
        
        public ivec4 sssq {
            get {
                return new ivec4(this.v0, this.v0, this.v0, this.v3);
            }
        }
        
        public ivec4 xxyx {
            get {
                return new ivec4(this.v0, this.v0, this.v1, this.v0);
            }
        }
        
        public ivec4 rrgr {
            get {
                return new ivec4(this.v0, this.v0, this.v1, this.v0);
            }
        }
        
        public ivec4 ssts {
            get {
                return new ivec4(this.v0, this.v0, this.v1, this.v0);
            }
        }
        
        public ivec4 xxyy {
            get {
                return new ivec4(this.v0, this.v0, this.v1, this.v1);
            }
        }
        
        public ivec4 rrgg {
            get {
                return new ivec4(this.v0, this.v0, this.v1, this.v1);
            }
        }
        
        public ivec4 sstt {
            get {
                return new ivec4(this.v0, this.v0, this.v1, this.v1);
            }
        }
        
        public ivec4 xxyz {
            get {
                return new ivec4(this.v0, this.v0, this.v1, this.v2);
            }
        }
        
        public ivec4 rrgb {
            get {
                return new ivec4(this.v0, this.v0, this.v1, this.v2);
            }
        }
        
        public ivec4 sstp {
            get {
                return new ivec4(this.v0, this.v0, this.v1, this.v2);
            }
        }
        
        public ivec4 xxyw {
            get {
                return new ivec4(this.v0, this.v0, this.v1, this.v3);
            }
        }
        
        public ivec4 rrga {
            get {
                return new ivec4(this.v0, this.v0, this.v1, this.v3);
            }
        }
        
        public ivec4 sstq {
            get {
                return new ivec4(this.v0, this.v0, this.v1, this.v3);
            }
        }
        
        public ivec4 xxzx {
            get {
                return new ivec4(this.v0, this.v0, this.v2, this.v0);
            }
        }
        
        public ivec4 rrbr {
            get {
                return new ivec4(this.v0, this.v0, this.v2, this.v0);
            }
        }
        
        public ivec4 ssps {
            get {
                return new ivec4(this.v0, this.v0, this.v2, this.v0);
            }
        }
        
        public ivec4 xxzy {
            get {
                return new ivec4(this.v0, this.v0, this.v2, this.v1);
            }
        }
        
        public ivec4 rrbg {
            get {
                return new ivec4(this.v0, this.v0, this.v2, this.v1);
            }
        }
        
        public ivec4 sspt {
            get {
                return new ivec4(this.v0, this.v0, this.v2, this.v1);
            }
        }
        
        public ivec4 xxzz {
            get {
                return new ivec4(this.v0, this.v0, this.v2, this.v2);
            }
        }
        
        public ivec4 rrbb {
            get {
                return new ivec4(this.v0, this.v0, this.v2, this.v2);
            }
        }
        
        public ivec4 sspp {
            get {
                return new ivec4(this.v0, this.v0, this.v2, this.v2);
            }
        }
        
        public ivec4 xxzw {
            get {
                return new ivec4(this.v0, this.v0, this.v2, this.v3);
            }
        }
        
        public ivec4 rrba {
            get {
                return new ivec4(this.v0, this.v0, this.v2, this.v3);
            }
        }
        
        public ivec4 sspq {
            get {
                return new ivec4(this.v0, this.v0, this.v2, this.v3);
            }
        }
        
        public ivec4 xxwx {
            get {
                return new ivec4(this.v0, this.v0, this.v3, this.v0);
            }
        }
        
        public ivec4 rrar {
            get {
                return new ivec4(this.v0, this.v0, this.v3, this.v0);
            }
        }
        
        public ivec4 ssqs {
            get {
                return new ivec4(this.v0, this.v0, this.v3, this.v0);
            }
        }
        
        public ivec4 xxwy {
            get {
                return new ivec4(this.v0, this.v0, this.v3, this.v1);
            }
        }
        
        public ivec4 rrag {
            get {
                return new ivec4(this.v0, this.v0, this.v3, this.v1);
            }
        }
        
        public ivec4 ssqt {
            get {
                return new ivec4(this.v0, this.v0, this.v3, this.v1);
            }
        }
        
        public ivec4 xxwz {
            get {
                return new ivec4(this.v0, this.v0, this.v3, this.v2);
            }
        }
        
        public ivec4 rrab {
            get {
                return new ivec4(this.v0, this.v0, this.v3, this.v2);
            }
        }
        
        public ivec4 ssqp {
            get {
                return new ivec4(this.v0, this.v0, this.v3, this.v2);
            }
        }
        
        public ivec4 xxww {
            get {
                return new ivec4(this.v0, this.v0, this.v3, this.v3);
            }
        }
        
        public ivec4 rraa {
            get {
                return new ivec4(this.v0, this.v0, this.v3, this.v3);
            }
        }
        
        public ivec4 ssqq {
            get {
                return new ivec4(this.v0, this.v0, this.v3, this.v3);
            }
        }
        
        public ivec4 xyxx {
            get {
                return new ivec4(this.v0, this.v1, this.v0, this.v0);
            }
        }
        
        public ivec4 rgrr {
            get {
                return new ivec4(this.v0, this.v1, this.v0, this.v0);
            }
        }
        
        public ivec4 stss {
            get {
                return new ivec4(this.v0, this.v1, this.v0, this.v0);
            }
        }
        
        public ivec4 xyxy {
            get {
                return new ivec4(this.v0, this.v1, this.v0, this.v1);
            }
        }
        
        public ivec4 rgrg {
            get {
                return new ivec4(this.v0, this.v1, this.v0, this.v1);
            }
        }
        
        public ivec4 stst {
            get {
                return new ivec4(this.v0, this.v1, this.v0, this.v1);
            }
        }
        
        public ivec4 xyxz {
            get {
                return new ivec4(this.v0, this.v1, this.v0, this.v2);
            }
        }
        
        public ivec4 rgrb {
            get {
                return new ivec4(this.v0, this.v1, this.v0, this.v2);
            }
        }
        
        public ivec4 stsp {
            get {
                return new ivec4(this.v0, this.v1, this.v0, this.v2);
            }
        }
        
        public ivec4 xyxw {
            get {
                return new ivec4(this.v0, this.v1, this.v0, this.v3);
            }
        }
        
        public ivec4 rgra {
            get {
                return new ivec4(this.v0, this.v1, this.v0, this.v3);
            }
        }
        
        public ivec4 stsq {
            get {
                return new ivec4(this.v0, this.v1, this.v0, this.v3);
            }
        }
        
        public ivec4 xyyx {
            get {
                return new ivec4(this.v0, this.v1, this.v1, this.v0);
            }
        }
        
        public ivec4 rggr {
            get {
                return new ivec4(this.v0, this.v1, this.v1, this.v0);
            }
        }
        
        public ivec4 stts {
            get {
                return new ivec4(this.v0, this.v1, this.v1, this.v0);
            }
        }
        
        public ivec4 xyyy {
            get {
                return new ivec4(this.v0, this.v1, this.v1, this.v1);
            }
        }
        
        public ivec4 rggg {
            get {
                return new ivec4(this.v0, this.v1, this.v1, this.v1);
            }
        }
        
        public ivec4 sttt {
            get {
                return new ivec4(this.v0, this.v1, this.v1, this.v1);
            }
        }
        
        public ivec4 xyyz {
            get {
                return new ivec4(this.v0, this.v1, this.v1, this.v2);
            }
        }
        
        public ivec4 rggb {
            get {
                return new ivec4(this.v0, this.v1, this.v1, this.v2);
            }
        }
        
        public ivec4 sttp {
            get {
                return new ivec4(this.v0, this.v1, this.v1, this.v2);
            }
        }
        
        public ivec4 xyyw {
            get {
                return new ivec4(this.v0, this.v1, this.v1, this.v3);
            }
        }
        
        public ivec4 rgga {
            get {
                return new ivec4(this.v0, this.v1, this.v1, this.v3);
            }
        }
        
        public ivec4 sttq {
            get {
                return new ivec4(this.v0, this.v1, this.v1, this.v3);
            }
        }
        
        public ivec4 xyzx {
            get {
                return new ivec4(this.v0, this.v1, this.v2, this.v0);
            }
        }
        
        public ivec4 rgbr {
            get {
                return new ivec4(this.v0, this.v1, this.v2, this.v0);
            }
        }
        
        public ivec4 stps {
            get {
                return new ivec4(this.v0, this.v1, this.v2, this.v0);
            }
        }
        
        public ivec4 xyzy {
            get {
                return new ivec4(this.v0, this.v1, this.v2, this.v1);
            }
        }
        
        public ivec4 rgbg {
            get {
                return new ivec4(this.v0, this.v1, this.v2, this.v1);
            }
        }
        
        public ivec4 stpt {
            get {
                return new ivec4(this.v0, this.v1, this.v2, this.v1);
            }
        }
        
        public ivec4 xyzz {
            get {
                return new ivec4(this.v0, this.v1, this.v2, this.v2);
            }
        }
        
        public ivec4 rgbb {
            get {
                return new ivec4(this.v0, this.v1, this.v2, this.v2);
            }
        }
        
        public ivec4 stpp {
            get {
                return new ivec4(this.v0, this.v1, this.v2, this.v2);
            }
        }
        
        public ivec4 xyzw {
            get {
                return new ivec4(this.v0, this.v1, this.v2, this.v3);
            }
            set {
                this.v0 = value.v0;
                this.v1 = value.v1;
                this.v2 = value.v2;
                this.v3 = value.v3;
            }
        }
        
        public ivec4 rgba {
            get {
                return new ivec4(this.v0, this.v1, this.v2, this.v3);
            }
            set {
                this.v0 = value.v0;
                this.v1 = value.v1;
                this.v2 = value.v2;
                this.v3 = value.v3;
            }
        }
        
        public ivec4 stpq {
            get {
                return new ivec4(this.v0, this.v1, this.v2, this.v3);
            }
            set {
                this.v0 = value.v0;
                this.v1 = value.v1;
                this.v2 = value.v2;
                this.v3 = value.v3;
            }
        }
        
        public ivec4 xywx {
            get {
                return new ivec4(this.v0, this.v1, this.v3, this.v0);
            }
        }
        
        public ivec4 rgar {
            get {
                return new ivec4(this.v0, this.v1, this.v3, this.v0);
            }
        }
        
        public ivec4 stqs {
            get {
                return new ivec4(this.v0, this.v1, this.v3, this.v0);
            }
        }
        
        public ivec4 xywy {
            get {
                return new ivec4(this.v0, this.v1, this.v3, this.v1);
            }
        }
        
        public ivec4 rgag {
            get {
                return new ivec4(this.v0, this.v1, this.v3, this.v1);
            }
        }
        
        public ivec4 stqt {
            get {
                return new ivec4(this.v0, this.v1, this.v3, this.v1);
            }
        }
        
        public ivec4 xywz {
            get {
                return new ivec4(this.v0, this.v1, this.v3, this.v2);
            }
            set {
                this.v0 = value.v0;
                this.v1 = value.v1;
                this.v3 = value.v2;
                this.v2 = value.v3;
            }
        }
        
        public ivec4 rgab {
            get {
                return new ivec4(this.v0, this.v1, this.v3, this.v2);
            }
            set {
                this.v0 = value.v0;
                this.v1 = value.v1;
                this.v3 = value.v2;
                this.v2 = value.v3;
            }
        }
        
        public ivec4 stqp {
            get {
                return new ivec4(this.v0, this.v1, this.v3, this.v2);
            }
            set {
                this.v0 = value.v0;
                this.v1 = value.v1;
                this.v3 = value.v2;
                this.v2 = value.v3;
            }
        }
        
        public ivec4 xyww {
            get {
                return new ivec4(this.v0, this.v1, this.v3, this.v3);
            }
        }
        
        public ivec4 rgaa {
            get {
                return new ivec4(this.v0, this.v1, this.v3, this.v3);
            }
        }
        
        public ivec4 stqq {
            get {
                return new ivec4(this.v0, this.v1, this.v3, this.v3);
            }
        }
        
        public ivec4 xzxx {
            get {
                return new ivec4(this.v0, this.v2, this.v0, this.v0);
            }
        }
        
        public ivec4 rbrr {
            get {
                return new ivec4(this.v0, this.v2, this.v0, this.v0);
            }
        }
        
        public ivec4 spss {
            get {
                return new ivec4(this.v0, this.v2, this.v0, this.v0);
            }
        }
        
        public ivec4 xzxy {
            get {
                return new ivec4(this.v0, this.v2, this.v0, this.v1);
            }
        }
        
        public ivec4 rbrg {
            get {
                return new ivec4(this.v0, this.v2, this.v0, this.v1);
            }
        }
        
        public ivec4 spst {
            get {
                return new ivec4(this.v0, this.v2, this.v0, this.v1);
            }
        }
        
        public ivec4 xzxz {
            get {
                return new ivec4(this.v0, this.v2, this.v0, this.v2);
            }
        }
        
        public ivec4 rbrb {
            get {
                return new ivec4(this.v0, this.v2, this.v0, this.v2);
            }
        }
        
        public ivec4 spsp {
            get {
                return new ivec4(this.v0, this.v2, this.v0, this.v2);
            }
        }
        
        public ivec4 xzxw {
            get {
                return new ivec4(this.v0, this.v2, this.v0, this.v3);
            }
        }
        
        public ivec4 rbra {
            get {
                return new ivec4(this.v0, this.v2, this.v0, this.v3);
            }
        }
        
        public ivec4 spsq {
            get {
                return new ivec4(this.v0, this.v2, this.v0, this.v3);
            }
        }
        
        public ivec4 xzyx {
            get {
                return new ivec4(this.v0, this.v2, this.v1, this.v0);
            }
        }
        
        public ivec4 rbgr {
            get {
                return new ivec4(this.v0, this.v2, this.v1, this.v0);
            }
        }
        
        public ivec4 spts {
            get {
                return new ivec4(this.v0, this.v2, this.v1, this.v0);
            }
        }
        
        public ivec4 xzyy {
            get {
                return new ivec4(this.v0, this.v2, this.v1, this.v1);
            }
        }
        
        public ivec4 rbgg {
            get {
                return new ivec4(this.v0, this.v2, this.v1, this.v1);
            }
        }
        
        public ivec4 sptt {
            get {
                return new ivec4(this.v0, this.v2, this.v1, this.v1);
            }
        }
        
        public ivec4 xzyz {
            get {
                return new ivec4(this.v0, this.v2, this.v1, this.v2);
            }
        }
        
        public ivec4 rbgb {
            get {
                return new ivec4(this.v0, this.v2, this.v1, this.v2);
            }
        }
        
        public ivec4 sptp {
            get {
                return new ivec4(this.v0, this.v2, this.v1, this.v2);
            }
        }
        
        public ivec4 xzyw {
            get {
                return new ivec4(this.v0, this.v2, this.v1, this.v3);
            }
            set {
                this.v0 = value.v0;
                this.v2 = value.v1;
                this.v1 = value.v2;
                this.v3 = value.v3;
            }
        }
        
        public ivec4 rbga {
            get {
                return new ivec4(this.v0, this.v2, this.v1, this.v3);
            }
            set {
                this.v0 = value.v0;
                this.v2 = value.v1;
                this.v1 = value.v2;
                this.v3 = value.v3;
            }
        }
        
        public ivec4 sptq {
            get {
                return new ivec4(this.v0, this.v2, this.v1, this.v3);
            }
            set {
                this.v0 = value.v0;
                this.v2 = value.v1;
                this.v1 = value.v2;
                this.v3 = value.v3;
            }
        }
        
        public ivec4 xzzx {
            get {
                return new ivec4(this.v0, this.v2, this.v2, this.v0);
            }
        }
        
        public ivec4 rbbr {
            get {
                return new ivec4(this.v0, this.v2, this.v2, this.v0);
            }
        }
        
        public ivec4 spps {
            get {
                return new ivec4(this.v0, this.v2, this.v2, this.v0);
            }
        }
        
        public ivec4 xzzy {
            get {
                return new ivec4(this.v0, this.v2, this.v2, this.v1);
            }
        }
        
        public ivec4 rbbg {
            get {
                return new ivec4(this.v0, this.v2, this.v2, this.v1);
            }
        }
        
        public ivec4 sppt {
            get {
                return new ivec4(this.v0, this.v2, this.v2, this.v1);
            }
        }
        
        public ivec4 xzzz {
            get {
                return new ivec4(this.v0, this.v2, this.v2, this.v2);
            }
        }
        
        public ivec4 rbbb {
            get {
                return new ivec4(this.v0, this.v2, this.v2, this.v2);
            }
        }
        
        public ivec4 sppp {
            get {
                return new ivec4(this.v0, this.v2, this.v2, this.v2);
            }
        }
        
        public ivec4 xzzw {
            get {
                return new ivec4(this.v0, this.v2, this.v2, this.v3);
            }
        }
        
        public ivec4 rbba {
            get {
                return new ivec4(this.v0, this.v2, this.v2, this.v3);
            }
        }
        
        public ivec4 sppq {
            get {
                return new ivec4(this.v0, this.v2, this.v2, this.v3);
            }
        }
        
        public ivec4 xzwx {
            get {
                return new ivec4(this.v0, this.v2, this.v3, this.v0);
            }
        }
        
        public ivec4 rbar {
            get {
                return new ivec4(this.v0, this.v2, this.v3, this.v0);
            }
        }
        
        public ivec4 spqs {
            get {
                return new ivec4(this.v0, this.v2, this.v3, this.v0);
            }
        }
        
        public ivec4 xzwy {
            get {
                return new ivec4(this.v0, this.v2, this.v3, this.v1);
            }
            set {
                this.v0 = value.v0;
                this.v2 = value.v1;
                this.v3 = value.v2;
                this.v1 = value.v3;
            }
        }
        
        public ivec4 rbag {
            get {
                return new ivec4(this.v0, this.v2, this.v3, this.v1);
            }
            set {
                this.v0 = value.v0;
                this.v2 = value.v1;
                this.v3 = value.v2;
                this.v1 = value.v3;
            }
        }
        
        public ivec4 spqt {
            get {
                return new ivec4(this.v0, this.v2, this.v3, this.v1);
            }
            set {
                this.v0 = value.v0;
                this.v2 = value.v1;
                this.v3 = value.v2;
                this.v1 = value.v3;
            }
        }
        
        public ivec4 xzwz {
            get {
                return new ivec4(this.v0, this.v2, this.v3, this.v2);
            }
        }
        
        public ivec4 rbab {
            get {
                return new ivec4(this.v0, this.v2, this.v3, this.v2);
            }
        }
        
        public ivec4 spqp {
            get {
                return new ivec4(this.v0, this.v2, this.v3, this.v2);
            }
        }
        
        public ivec4 xzww {
            get {
                return new ivec4(this.v0, this.v2, this.v3, this.v3);
            }
        }
        
        public ivec4 rbaa {
            get {
                return new ivec4(this.v0, this.v2, this.v3, this.v3);
            }
        }
        
        public ivec4 spqq {
            get {
                return new ivec4(this.v0, this.v2, this.v3, this.v3);
            }
        }
        
        public ivec4 xwxx {
            get {
                return new ivec4(this.v0, this.v3, this.v0, this.v0);
            }
        }
        
        public ivec4 rarr {
            get {
                return new ivec4(this.v0, this.v3, this.v0, this.v0);
            }
        }
        
        public ivec4 sqss {
            get {
                return new ivec4(this.v0, this.v3, this.v0, this.v0);
            }
        }
        
        public ivec4 xwxy {
            get {
                return new ivec4(this.v0, this.v3, this.v0, this.v1);
            }
        }
        
        public ivec4 rarg {
            get {
                return new ivec4(this.v0, this.v3, this.v0, this.v1);
            }
        }
        
        public ivec4 sqst {
            get {
                return new ivec4(this.v0, this.v3, this.v0, this.v1);
            }
        }
        
        public ivec4 xwxz {
            get {
                return new ivec4(this.v0, this.v3, this.v0, this.v2);
            }
        }
        
        public ivec4 rarb {
            get {
                return new ivec4(this.v0, this.v3, this.v0, this.v2);
            }
        }
        
        public ivec4 sqsp {
            get {
                return new ivec4(this.v0, this.v3, this.v0, this.v2);
            }
        }
        
        public ivec4 xwxw {
            get {
                return new ivec4(this.v0, this.v3, this.v0, this.v3);
            }
        }
        
        public ivec4 rara {
            get {
                return new ivec4(this.v0, this.v3, this.v0, this.v3);
            }
        }
        
        public ivec4 sqsq {
            get {
                return new ivec4(this.v0, this.v3, this.v0, this.v3);
            }
        }
        
        public ivec4 xwyx {
            get {
                return new ivec4(this.v0, this.v3, this.v1, this.v0);
            }
        }
        
        public ivec4 ragr {
            get {
                return new ivec4(this.v0, this.v3, this.v1, this.v0);
            }
        }
        
        public ivec4 sqts {
            get {
                return new ivec4(this.v0, this.v3, this.v1, this.v0);
            }
        }
        
        public ivec4 xwyy {
            get {
                return new ivec4(this.v0, this.v3, this.v1, this.v1);
            }
        }
        
        public ivec4 ragg {
            get {
                return new ivec4(this.v0, this.v3, this.v1, this.v1);
            }
        }
        
        public ivec4 sqtt {
            get {
                return new ivec4(this.v0, this.v3, this.v1, this.v1);
            }
        }
        
        public ivec4 xwyz {
            get {
                return new ivec4(this.v0, this.v3, this.v1, this.v2);
            }
            set {
                this.v0 = value.v0;
                this.v3 = value.v1;
                this.v1 = value.v2;
                this.v2 = value.v3;
            }
        }
        
        public ivec4 ragb {
            get {
                return new ivec4(this.v0, this.v3, this.v1, this.v2);
            }
            set {
                this.v0 = value.v0;
                this.v3 = value.v1;
                this.v1 = value.v2;
                this.v2 = value.v3;
            }
        }
        
        public ivec4 sqtp {
            get {
                return new ivec4(this.v0, this.v3, this.v1, this.v2);
            }
            set {
                this.v0 = value.v0;
                this.v3 = value.v1;
                this.v1 = value.v2;
                this.v2 = value.v3;
            }
        }
        
        public ivec4 xwyw {
            get {
                return new ivec4(this.v0, this.v3, this.v1, this.v3);
            }
        }
        
        public ivec4 raga {
            get {
                return new ivec4(this.v0, this.v3, this.v1, this.v3);
            }
        }
        
        public ivec4 sqtq {
            get {
                return new ivec4(this.v0, this.v3, this.v1, this.v3);
            }
        }
        
        public ivec4 xwzx {
            get {
                return new ivec4(this.v0, this.v3, this.v2, this.v0);
            }
        }
        
        public ivec4 rabr {
            get {
                return new ivec4(this.v0, this.v3, this.v2, this.v0);
            }
        }
        
        public ivec4 sqps {
            get {
                return new ivec4(this.v0, this.v3, this.v2, this.v0);
            }
        }
        
        public ivec4 xwzy {
            get {
                return new ivec4(this.v0, this.v3, this.v2, this.v1);
            }
            set {
                this.v0 = value.v0;
                this.v3 = value.v1;
                this.v2 = value.v2;
                this.v1 = value.v3;
            }
        }
        
        public ivec4 rabg {
            get {
                return new ivec4(this.v0, this.v3, this.v2, this.v1);
            }
            set {
                this.v0 = value.v0;
                this.v3 = value.v1;
                this.v2 = value.v2;
                this.v1 = value.v3;
            }
        }
        
        public ivec4 sqpt {
            get {
                return new ivec4(this.v0, this.v3, this.v2, this.v1);
            }
            set {
                this.v0 = value.v0;
                this.v3 = value.v1;
                this.v2 = value.v2;
                this.v1 = value.v3;
            }
        }
        
        public ivec4 xwzz {
            get {
                return new ivec4(this.v0, this.v3, this.v2, this.v2);
            }
        }
        
        public ivec4 rabb {
            get {
                return new ivec4(this.v0, this.v3, this.v2, this.v2);
            }
        }
        
        public ivec4 sqpp {
            get {
                return new ivec4(this.v0, this.v3, this.v2, this.v2);
            }
        }
        
        public ivec4 xwzw {
            get {
                return new ivec4(this.v0, this.v3, this.v2, this.v3);
            }
        }
        
        public ivec4 raba {
            get {
                return new ivec4(this.v0, this.v3, this.v2, this.v3);
            }
        }
        
        public ivec4 sqpq {
            get {
                return new ivec4(this.v0, this.v3, this.v2, this.v3);
            }
        }
        
        public ivec4 xwwx {
            get {
                return new ivec4(this.v0, this.v3, this.v3, this.v0);
            }
        }
        
        public ivec4 raar {
            get {
                return new ivec4(this.v0, this.v3, this.v3, this.v0);
            }
        }
        
        public ivec4 sqqs {
            get {
                return new ivec4(this.v0, this.v3, this.v3, this.v0);
            }
        }
        
        public ivec4 xwwy {
            get {
                return new ivec4(this.v0, this.v3, this.v3, this.v1);
            }
        }
        
        public ivec4 raag {
            get {
                return new ivec4(this.v0, this.v3, this.v3, this.v1);
            }
        }
        
        public ivec4 sqqt {
            get {
                return new ivec4(this.v0, this.v3, this.v3, this.v1);
            }
        }
        
        public ivec4 xwwz {
            get {
                return new ivec4(this.v0, this.v3, this.v3, this.v2);
            }
        }
        
        public ivec4 raab {
            get {
                return new ivec4(this.v0, this.v3, this.v3, this.v2);
            }
        }
        
        public ivec4 sqqp {
            get {
                return new ivec4(this.v0, this.v3, this.v3, this.v2);
            }
        }
        
        public ivec4 xwww {
            get {
                return new ivec4(this.v0, this.v3, this.v3, this.v3);
            }
        }
        
        public ivec4 raaa {
            get {
                return new ivec4(this.v0, this.v3, this.v3, this.v3);
            }
        }
        
        public ivec4 sqqq {
            get {
                return new ivec4(this.v0, this.v3, this.v3, this.v3);
            }
        }
        
        public ivec4 yxxx {
            get {
                return new ivec4(this.v1, this.v0, this.v0, this.v0);
            }
        }
        
        public ivec4 grrr {
            get {
                return new ivec4(this.v1, this.v0, this.v0, this.v0);
            }
        }
        
        public ivec4 tsss {
            get {
                return new ivec4(this.v1, this.v0, this.v0, this.v0);
            }
        }
        
        public ivec4 yxxy {
            get {
                return new ivec4(this.v1, this.v0, this.v0, this.v1);
            }
        }
        
        public ivec4 grrg {
            get {
                return new ivec4(this.v1, this.v0, this.v0, this.v1);
            }
        }
        
        public ivec4 tsst {
            get {
                return new ivec4(this.v1, this.v0, this.v0, this.v1);
            }
        }
        
        public ivec4 yxxz {
            get {
                return new ivec4(this.v1, this.v0, this.v0, this.v2);
            }
        }
        
        public ivec4 grrb {
            get {
                return new ivec4(this.v1, this.v0, this.v0, this.v2);
            }
        }
        
        public ivec4 tssp {
            get {
                return new ivec4(this.v1, this.v0, this.v0, this.v2);
            }
        }
        
        public ivec4 yxxw {
            get {
                return new ivec4(this.v1, this.v0, this.v0, this.v3);
            }
        }
        
        public ivec4 grra {
            get {
                return new ivec4(this.v1, this.v0, this.v0, this.v3);
            }
        }
        
        public ivec4 tssq {
            get {
                return new ivec4(this.v1, this.v0, this.v0, this.v3);
            }
        }
        
        public ivec4 yxyx {
            get {
                return new ivec4(this.v1, this.v0, this.v1, this.v0);
            }
        }
        
        public ivec4 grgr {
            get {
                return new ivec4(this.v1, this.v0, this.v1, this.v0);
            }
        }
        
        public ivec4 tsts {
            get {
                return new ivec4(this.v1, this.v0, this.v1, this.v0);
            }
        }
        
        public ivec4 yxyy {
            get {
                return new ivec4(this.v1, this.v0, this.v1, this.v1);
            }
        }
        
        public ivec4 grgg {
            get {
                return new ivec4(this.v1, this.v0, this.v1, this.v1);
            }
        }
        
        public ivec4 tstt {
            get {
                return new ivec4(this.v1, this.v0, this.v1, this.v1);
            }
        }
        
        public ivec4 yxyz {
            get {
                return new ivec4(this.v1, this.v0, this.v1, this.v2);
            }
        }
        
        public ivec4 grgb {
            get {
                return new ivec4(this.v1, this.v0, this.v1, this.v2);
            }
        }
        
        public ivec4 tstp {
            get {
                return new ivec4(this.v1, this.v0, this.v1, this.v2);
            }
        }
        
        public ivec4 yxyw {
            get {
                return new ivec4(this.v1, this.v0, this.v1, this.v3);
            }
        }
        
        public ivec4 grga {
            get {
                return new ivec4(this.v1, this.v0, this.v1, this.v3);
            }
        }
        
        public ivec4 tstq {
            get {
                return new ivec4(this.v1, this.v0, this.v1, this.v3);
            }
        }
        
        public ivec4 yxzx {
            get {
                return new ivec4(this.v1, this.v0, this.v2, this.v0);
            }
        }
        
        public ivec4 grbr {
            get {
                return new ivec4(this.v1, this.v0, this.v2, this.v0);
            }
        }
        
        public ivec4 tsps {
            get {
                return new ivec4(this.v1, this.v0, this.v2, this.v0);
            }
        }
        
        public ivec4 yxzy {
            get {
                return new ivec4(this.v1, this.v0, this.v2, this.v1);
            }
        }
        
        public ivec4 grbg {
            get {
                return new ivec4(this.v1, this.v0, this.v2, this.v1);
            }
        }
        
        public ivec4 tspt {
            get {
                return new ivec4(this.v1, this.v0, this.v2, this.v1);
            }
        }
        
        public ivec4 yxzz {
            get {
                return new ivec4(this.v1, this.v0, this.v2, this.v2);
            }
        }
        
        public ivec4 grbb {
            get {
                return new ivec4(this.v1, this.v0, this.v2, this.v2);
            }
        }
        
        public ivec4 tspp {
            get {
                return new ivec4(this.v1, this.v0, this.v2, this.v2);
            }
        }
        
        public ivec4 yxzw {
            get {
                return new ivec4(this.v1, this.v0, this.v2, this.v3);
            }
            set {
                this.v1 = value.v0;
                this.v0 = value.v1;
                this.v2 = value.v2;
                this.v3 = value.v3;
            }
        }
        
        public ivec4 grba {
            get {
                return new ivec4(this.v1, this.v0, this.v2, this.v3);
            }
            set {
                this.v1 = value.v0;
                this.v0 = value.v1;
                this.v2 = value.v2;
                this.v3 = value.v3;
            }
        }
        
        public ivec4 tspq {
            get {
                return new ivec4(this.v1, this.v0, this.v2, this.v3);
            }
            set {
                this.v1 = value.v0;
                this.v0 = value.v1;
                this.v2 = value.v2;
                this.v3 = value.v3;
            }
        }
        
        public ivec4 yxwx {
            get {
                return new ivec4(this.v1, this.v0, this.v3, this.v0);
            }
        }
        
        public ivec4 grar {
            get {
                return new ivec4(this.v1, this.v0, this.v3, this.v0);
            }
        }
        
        public ivec4 tsqs {
            get {
                return new ivec4(this.v1, this.v0, this.v3, this.v0);
            }
        }
        
        public ivec4 yxwy {
            get {
                return new ivec4(this.v1, this.v0, this.v3, this.v1);
            }
        }
        
        public ivec4 grag {
            get {
                return new ivec4(this.v1, this.v0, this.v3, this.v1);
            }
        }
        
        public ivec4 tsqt {
            get {
                return new ivec4(this.v1, this.v0, this.v3, this.v1);
            }
        }
        
        public ivec4 yxwz {
            get {
                return new ivec4(this.v1, this.v0, this.v3, this.v2);
            }
            set {
                this.v1 = value.v0;
                this.v0 = value.v1;
                this.v3 = value.v2;
                this.v2 = value.v3;
            }
        }
        
        public ivec4 grab {
            get {
                return new ivec4(this.v1, this.v0, this.v3, this.v2);
            }
            set {
                this.v1 = value.v0;
                this.v0 = value.v1;
                this.v3 = value.v2;
                this.v2 = value.v3;
            }
        }
        
        public ivec4 tsqp {
            get {
                return new ivec4(this.v1, this.v0, this.v3, this.v2);
            }
            set {
                this.v1 = value.v0;
                this.v0 = value.v1;
                this.v3 = value.v2;
                this.v2 = value.v3;
            }
        }
        
        public ivec4 yxww {
            get {
                return new ivec4(this.v1, this.v0, this.v3, this.v3);
            }
        }
        
        public ivec4 graa {
            get {
                return new ivec4(this.v1, this.v0, this.v3, this.v3);
            }
        }
        
        public ivec4 tsqq {
            get {
                return new ivec4(this.v1, this.v0, this.v3, this.v3);
            }
        }
        
        public ivec4 yyxx {
            get {
                return new ivec4(this.v1, this.v1, this.v0, this.v0);
            }
        }
        
        public ivec4 ggrr {
            get {
                return new ivec4(this.v1, this.v1, this.v0, this.v0);
            }
        }
        
        public ivec4 ttss {
            get {
                return new ivec4(this.v1, this.v1, this.v0, this.v0);
            }
        }
        
        public ivec4 yyxy {
            get {
                return new ivec4(this.v1, this.v1, this.v0, this.v1);
            }
        }
        
        public ivec4 ggrg {
            get {
                return new ivec4(this.v1, this.v1, this.v0, this.v1);
            }
        }
        
        public ivec4 ttst {
            get {
                return new ivec4(this.v1, this.v1, this.v0, this.v1);
            }
        }
        
        public ivec4 yyxz {
            get {
                return new ivec4(this.v1, this.v1, this.v0, this.v2);
            }
        }
        
        public ivec4 ggrb {
            get {
                return new ivec4(this.v1, this.v1, this.v0, this.v2);
            }
        }
        
        public ivec4 ttsp {
            get {
                return new ivec4(this.v1, this.v1, this.v0, this.v2);
            }
        }
        
        public ivec4 yyxw {
            get {
                return new ivec4(this.v1, this.v1, this.v0, this.v3);
            }
        }
        
        public ivec4 ggra {
            get {
                return new ivec4(this.v1, this.v1, this.v0, this.v3);
            }
        }
        
        public ivec4 ttsq {
            get {
                return new ivec4(this.v1, this.v1, this.v0, this.v3);
            }
        }
        
        public ivec4 yyyx {
            get {
                return new ivec4(this.v1, this.v1, this.v1, this.v0);
            }
        }
        
        public ivec4 gggr {
            get {
                return new ivec4(this.v1, this.v1, this.v1, this.v0);
            }
        }
        
        public ivec4 ttts {
            get {
                return new ivec4(this.v1, this.v1, this.v1, this.v0);
            }
        }
        
        public ivec4 yyyy {
            get {
                return new ivec4(this.v1, this.v1, this.v1, this.v1);
            }
        }
        
        public ivec4 gggg {
            get {
                return new ivec4(this.v1, this.v1, this.v1, this.v1);
            }
        }
        
        public ivec4 tttt {
            get {
                return new ivec4(this.v1, this.v1, this.v1, this.v1);
            }
        }
        
        public ivec4 yyyz {
            get {
                return new ivec4(this.v1, this.v1, this.v1, this.v2);
            }
        }
        
        public ivec4 gggb {
            get {
                return new ivec4(this.v1, this.v1, this.v1, this.v2);
            }
        }
        
        public ivec4 tttp {
            get {
                return new ivec4(this.v1, this.v1, this.v1, this.v2);
            }
        }
        
        public ivec4 yyyw {
            get {
                return new ivec4(this.v1, this.v1, this.v1, this.v3);
            }
        }
        
        public ivec4 ggga {
            get {
                return new ivec4(this.v1, this.v1, this.v1, this.v3);
            }
        }
        
        public ivec4 tttq {
            get {
                return new ivec4(this.v1, this.v1, this.v1, this.v3);
            }
        }
        
        public ivec4 yyzx {
            get {
                return new ivec4(this.v1, this.v1, this.v2, this.v0);
            }
        }
        
        public ivec4 ggbr {
            get {
                return new ivec4(this.v1, this.v1, this.v2, this.v0);
            }
        }
        
        public ivec4 ttps {
            get {
                return new ivec4(this.v1, this.v1, this.v2, this.v0);
            }
        }
        
        public ivec4 yyzy {
            get {
                return new ivec4(this.v1, this.v1, this.v2, this.v1);
            }
        }
        
        public ivec4 ggbg {
            get {
                return new ivec4(this.v1, this.v1, this.v2, this.v1);
            }
        }
        
        public ivec4 ttpt {
            get {
                return new ivec4(this.v1, this.v1, this.v2, this.v1);
            }
        }
        
        public ivec4 yyzz {
            get {
                return new ivec4(this.v1, this.v1, this.v2, this.v2);
            }
        }
        
        public ivec4 ggbb {
            get {
                return new ivec4(this.v1, this.v1, this.v2, this.v2);
            }
        }
        
        public ivec4 ttpp {
            get {
                return new ivec4(this.v1, this.v1, this.v2, this.v2);
            }
        }
        
        public ivec4 yyzw {
            get {
                return new ivec4(this.v1, this.v1, this.v2, this.v3);
            }
        }
        
        public ivec4 ggba {
            get {
                return new ivec4(this.v1, this.v1, this.v2, this.v3);
            }
        }
        
        public ivec4 ttpq {
            get {
                return new ivec4(this.v1, this.v1, this.v2, this.v3);
            }
        }
        
        public ivec4 yywx {
            get {
                return new ivec4(this.v1, this.v1, this.v3, this.v0);
            }
        }
        
        public ivec4 ggar {
            get {
                return new ivec4(this.v1, this.v1, this.v3, this.v0);
            }
        }
        
        public ivec4 ttqs {
            get {
                return new ivec4(this.v1, this.v1, this.v3, this.v0);
            }
        }
        
        public ivec4 yywy {
            get {
                return new ivec4(this.v1, this.v1, this.v3, this.v1);
            }
        }
        
        public ivec4 ggag {
            get {
                return new ivec4(this.v1, this.v1, this.v3, this.v1);
            }
        }
        
        public ivec4 ttqt {
            get {
                return new ivec4(this.v1, this.v1, this.v3, this.v1);
            }
        }
        
        public ivec4 yywz {
            get {
                return new ivec4(this.v1, this.v1, this.v3, this.v2);
            }
        }
        
        public ivec4 ggab {
            get {
                return new ivec4(this.v1, this.v1, this.v3, this.v2);
            }
        }
        
        public ivec4 ttqp {
            get {
                return new ivec4(this.v1, this.v1, this.v3, this.v2);
            }
        }
        
        public ivec4 yyww {
            get {
                return new ivec4(this.v1, this.v1, this.v3, this.v3);
            }
        }
        
        public ivec4 ggaa {
            get {
                return new ivec4(this.v1, this.v1, this.v3, this.v3);
            }
        }
        
        public ivec4 ttqq {
            get {
                return new ivec4(this.v1, this.v1, this.v3, this.v3);
            }
        }
        
        public ivec4 yzxx {
            get {
                return new ivec4(this.v1, this.v2, this.v0, this.v0);
            }
        }
        
        public ivec4 gbrr {
            get {
                return new ivec4(this.v1, this.v2, this.v0, this.v0);
            }
        }
        
        public ivec4 tpss {
            get {
                return new ivec4(this.v1, this.v2, this.v0, this.v0);
            }
        }
        
        public ivec4 yzxy {
            get {
                return new ivec4(this.v1, this.v2, this.v0, this.v1);
            }
        }
        
        public ivec4 gbrg {
            get {
                return new ivec4(this.v1, this.v2, this.v0, this.v1);
            }
        }
        
        public ivec4 tpst {
            get {
                return new ivec4(this.v1, this.v2, this.v0, this.v1);
            }
        }
        
        public ivec4 yzxz {
            get {
                return new ivec4(this.v1, this.v2, this.v0, this.v2);
            }
        }
        
        public ivec4 gbrb {
            get {
                return new ivec4(this.v1, this.v2, this.v0, this.v2);
            }
        }
        
        public ivec4 tpsp {
            get {
                return new ivec4(this.v1, this.v2, this.v0, this.v2);
            }
        }
        
        public ivec4 yzxw {
            get {
                return new ivec4(this.v1, this.v2, this.v0, this.v3);
            }
            set {
                this.v1 = value.v0;
                this.v2 = value.v1;
                this.v0 = value.v2;
                this.v3 = value.v3;
            }
        }
        
        public ivec4 gbra {
            get {
                return new ivec4(this.v1, this.v2, this.v0, this.v3);
            }
            set {
                this.v1 = value.v0;
                this.v2 = value.v1;
                this.v0 = value.v2;
                this.v3 = value.v3;
            }
        }
        
        public ivec4 tpsq {
            get {
                return new ivec4(this.v1, this.v2, this.v0, this.v3);
            }
            set {
                this.v1 = value.v0;
                this.v2 = value.v1;
                this.v0 = value.v2;
                this.v3 = value.v3;
            }
        }
        
        public ivec4 yzyx {
            get {
                return new ivec4(this.v1, this.v2, this.v1, this.v0);
            }
        }
        
        public ivec4 gbgr {
            get {
                return new ivec4(this.v1, this.v2, this.v1, this.v0);
            }
        }
        
        public ivec4 tpts {
            get {
                return new ivec4(this.v1, this.v2, this.v1, this.v0);
            }
        }
        
        public ivec4 yzyy {
            get {
                return new ivec4(this.v1, this.v2, this.v1, this.v1);
            }
        }
        
        public ivec4 gbgg {
            get {
                return new ivec4(this.v1, this.v2, this.v1, this.v1);
            }
        }
        
        public ivec4 tptt {
            get {
                return new ivec4(this.v1, this.v2, this.v1, this.v1);
            }
        }
        
        public ivec4 yzyz {
            get {
                return new ivec4(this.v1, this.v2, this.v1, this.v2);
            }
        }
        
        public ivec4 gbgb {
            get {
                return new ivec4(this.v1, this.v2, this.v1, this.v2);
            }
        }
        
        public ivec4 tptp {
            get {
                return new ivec4(this.v1, this.v2, this.v1, this.v2);
            }
        }
        
        public ivec4 yzyw {
            get {
                return new ivec4(this.v1, this.v2, this.v1, this.v3);
            }
        }
        
        public ivec4 gbga {
            get {
                return new ivec4(this.v1, this.v2, this.v1, this.v3);
            }
        }
        
        public ivec4 tptq {
            get {
                return new ivec4(this.v1, this.v2, this.v1, this.v3);
            }
        }
        
        public ivec4 yzzx {
            get {
                return new ivec4(this.v1, this.v2, this.v2, this.v0);
            }
        }
        
        public ivec4 gbbr {
            get {
                return new ivec4(this.v1, this.v2, this.v2, this.v0);
            }
        }
        
        public ivec4 tpps {
            get {
                return new ivec4(this.v1, this.v2, this.v2, this.v0);
            }
        }
        
        public ivec4 yzzy {
            get {
                return new ivec4(this.v1, this.v2, this.v2, this.v1);
            }
        }
        
        public ivec4 gbbg {
            get {
                return new ivec4(this.v1, this.v2, this.v2, this.v1);
            }
        }
        
        public ivec4 tppt {
            get {
                return new ivec4(this.v1, this.v2, this.v2, this.v1);
            }
        }
        
        public ivec4 yzzz {
            get {
                return new ivec4(this.v1, this.v2, this.v2, this.v2);
            }
        }
        
        public ivec4 gbbb {
            get {
                return new ivec4(this.v1, this.v2, this.v2, this.v2);
            }
        }
        
        public ivec4 tppp {
            get {
                return new ivec4(this.v1, this.v2, this.v2, this.v2);
            }
        }
        
        public ivec4 yzzw {
            get {
                return new ivec4(this.v1, this.v2, this.v2, this.v3);
            }
        }
        
        public ivec4 gbba {
            get {
                return new ivec4(this.v1, this.v2, this.v2, this.v3);
            }
        }
        
        public ivec4 tppq {
            get {
                return new ivec4(this.v1, this.v2, this.v2, this.v3);
            }
        }
        
        public ivec4 yzwx {
            get {
                return new ivec4(this.v1, this.v2, this.v3, this.v0);
            }
            set {
                this.v1 = value.v0;
                this.v2 = value.v1;
                this.v3 = value.v2;
                this.v0 = value.v3;
            }
        }
        
        public ivec4 gbar {
            get {
                return new ivec4(this.v1, this.v2, this.v3, this.v0);
            }
            set {
                this.v1 = value.v0;
                this.v2 = value.v1;
                this.v3 = value.v2;
                this.v0 = value.v3;
            }
        }
        
        public ivec4 tpqs {
            get {
                return new ivec4(this.v1, this.v2, this.v3, this.v0);
            }
            set {
                this.v1 = value.v0;
                this.v2 = value.v1;
                this.v3 = value.v2;
                this.v0 = value.v3;
            }
        }
        
        public ivec4 yzwy {
            get {
                return new ivec4(this.v1, this.v2, this.v3, this.v1);
            }
        }
        
        public ivec4 gbag {
            get {
                return new ivec4(this.v1, this.v2, this.v3, this.v1);
            }
        }
        
        public ivec4 tpqt {
            get {
                return new ivec4(this.v1, this.v2, this.v3, this.v1);
            }
        }
        
        public ivec4 yzwz {
            get {
                return new ivec4(this.v1, this.v2, this.v3, this.v2);
            }
        }
        
        public ivec4 gbab {
            get {
                return new ivec4(this.v1, this.v2, this.v3, this.v2);
            }
        }
        
        public ivec4 tpqp {
            get {
                return new ivec4(this.v1, this.v2, this.v3, this.v2);
            }
        }
        
        public ivec4 yzww {
            get {
                return new ivec4(this.v1, this.v2, this.v3, this.v3);
            }
        }
        
        public ivec4 gbaa {
            get {
                return new ivec4(this.v1, this.v2, this.v3, this.v3);
            }
        }
        
        public ivec4 tpqq {
            get {
                return new ivec4(this.v1, this.v2, this.v3, this.v3);
            }
        }
        
        public ivec4 ywxx {
            get {
                return new ivec4(this.v1, this.v3, this.v0, this.v0);
            }
        }
        
        public ivec4 garr {
            get {
                return new ivec4(this.v1, this.v3, this.v0, this.v0);
            }
        }
        
        public ivec4 tqss {
            get {
                return new ivec4(this.v1, this.v3, this.v0, this.v0);
            }
        }
        
        public ivec4 ywxy {
            get {
                return new ivec4(this.v1, this.v3, this.v0, this.v1);
            }
        }
        
        public ivec4 garg {
            get {
                return new ivec4(this.v1, this.v3, this.v0, this.v1);
            }
        }
        
        public ivec4 tqst {
            get {
                return new ivec4(this.v1, this.v3, this.v0, this.v1);
            }
        }
        
        public ivec4 ywxz {
            get {
                return new ivec4(this.v1, this.v3, this.v0, this.v2);
            }
            set {
                this.v1 = value.v0;
                this.v3 = value.v1;
                this.v0 = value.v2;
                this.v2 = value.v3;
            }
        }
        
        public ivec4 garb {
            get {
                return new ivec4(this.v1, this.v3, this.v0, this.v2);
            }
            set {
                this.v1 = value.v0;
                this.v3 = value.v1;
                this.v0 = value.v2;
                this.v2 = value.v3;
            }
        }
        
        public ivec4 tqsp {
            get {
                return new ivec4(this.v1, this.v3, this.v0, this.v2);
            }
            set {
                this.v1 = value.v0;
                this.v3 = value.v1;
                this.v0 = value.v2;
                this.v2 = value.v3;
            }
        }
        
        public ivec4 ywxw {
            get {
                return new ivec4(this.v1, this.v3, this.v0, this.v3);
            }
        }
        
        public ivec4 gara {
            get {
                return new ivec4(this.v1, this.v3, this.v0, this.v3);
            }
        }
        
        public ivec4 tqsq {
            get {
                return new ivec4(this.v1, this.v3, this.v0, this.v3);
            }
        }
        
        public ivec4 ywyx {
            get {
                return new ivec4(this.v1, this.v3, this.v1, this.v0);
            }
        }
        
        public ivec4 gagr {
            get {
                return new ivec4(this.v1, this.v3, this.v1, this.v0);
            }
        }
        
        public ivec4 tqts {
            get {
                return new ivec4(this.v1, this.v3, this.v1, this.v0);
            }
        }
        
        public ivec4 ywyy {
            get {
                return new ivec4(this.v1, this.v3, this.v1, this.v1);
            }
        }
        
        public ivec4 gagg {
            get {
                return new ivec4(this.v1, this.v3, this.v1, this.v1);
            }
        }
        
        public ivec4 tqtt {
            get {
                return new ivec4(this.v1, this.v3, this.v1, this.v1);
            }
        }
        
        public ivec4 ywyz {
            get {
                return new ivec4(this.v1, this.v3, this.v1, this.v2);
            }
        }
        
        public ivec4 gagb {
            get {
                return new ivec4(this.v1, this.v3, this.v1, this.v2);
            }
        }
        
        public ivec4 tqtp {
            get {
                return new ivec4(this.v1, this.v3, this.v1, this.v2);
            }
        }
        
        public ivec4 ywyw {
            get {
                return new ivec4(this.v1, this.v3, this.v1, this.v3);
            }
        }
        
        public ivec4 gaga {
            get {
                return new ivec4(this.v1, this.v3, this.v1, this.v3);
            }
        }
        
        public ivec4 tqtq {
            get {
                return new ivec4(this.v1, this.v3, this.v1, this.v3);
            }
        }
        
        public ivec4 ywzx {
            get {
                return new ivec4(this.v1, this.v3, this.v2, this.v0);
            }
            set {
                this.v1 = value.v0;
                this.v3 = value.v1;
                this.v2 = value.v2;
                this.v0 = value.v3;
            }
        }
        
        public ivec4 gabr {
            get {
                return new ivec4(this.v1, this.v3, this.v2, this.v0);
            }
            set {
                this.v1 = value.v0;
                this.v3 = value.v1;
                this.v2 = value.v2;
                this.v0 = value.v3;
            }
        }
        
        public ivec4 tqps {
            get {
                return new ivec4(this.v1, this.v3, this.v2, this.v0);
            }
            set {
                this.v1 = value.v0;
                this.v3 = value.v1;
                this.v2 = value.v2;
                this.v0 = value.v3;
            }
        }
        
        public ivec4 ywzy {
            get {
                return new ivec4(this.v1, this.v3, this.v2, this.v1);
            }
        }
        
        public ivec4 gabg {
            get {
                return new ivec4(this.v1, this.v3, this.v2, this.v1);
            }
        }
        
        public ivec4 tqpt {
            get {
                return new ivec4(this.v1, this.v3, this.v2, this.v1);
            }
        }
        
        public ivec4 ywzz {
            get {
                return new ivec4(this.v1, this.v3, this.v2, this.v2);
            }
        }
        
        public ivec4 gabb {
            get {
                return new ivec4(this.v1, this.v3, this.v2, this.v2);
            }
        }
        
        public ivec4 tqpp {
            get {
                return new ivec4(this.v1, this.v3, this.v2, this.v2);
            }
        }
        
        public ivec4 ywzw {
            get {
                return new ivec4(this.v1, this.v3, this.v2, this.v3);
            }
        }
        
        public ivec4 gaba {
            get {
                return new ivec4(this.v1, this.v3, this.v2, this.v3);
            }
        }
        
        public ivec4 tqpq {
            get {
                return new ivec4(this.v1, this.v3, this.v2, this.v3);
            }
        }
        
        public ivec4 ywwx {
            get {
                return new ivec4(this.v1, this.v3, this.v3, this.v0);
            }
        }
        
        public ivec4 gaar {
            get {
                return new ivec4(this.v1, this.v3, this.v3, this.v0);
            }
        }
        
        public ivec4 tqqs {
            get {
                return new ivec4(this.v1, this.v3, this.v3, this.v0);
            }
        }
        
        public ivec4 ywwy {
            get {
                return new ivec4(this.v1, this.v3, this.v3, this.v1);
            }
        }
        
        public ivec4 gaag {
            get {
                return new ivec4(this.v1, this.v3, this.v3, this.v1);
            }
        }
        
        public ivec4 tqqt {
            get {
                return new ivec4(this.v1, this.v3, this.v3, this.v1);
            }
        }
        
        public ivec4 ywwz {
            get {
                return new ivec4(this.v1, this.v3, this.v3, this.v2);
            }
        }
        
        public ivec4 gaab {
            get {
                return new ivec4(this.v1, this.v3, this.v3, this.v2);
            }
        }
        
        public ivec4 tqqp {
            get {
                return new ivec4(this.v1, this.v3, this.v3, this.v2);
            }
        }
        
        public ivec4 ywww {
            get {
                return new ivec4(this.v1, this.v3, this.v3, this.v3);
            }
        }
        
        public ivec4 gaaa {
            get {
                return new ivec4(this.v1, this.v3, this.v3, this.v3);
            }
        }
        
        public ivec4 tqqq {
            get {
                return new ivec4(this.v1, this.v3, this.v3, this.v3);
            }
        }
        
        public ivec4 zxxx {
            get {
                return new ivec4(this.v2, this.v0, this.v0, this.v0);
            }
        }
        
        public ivec4 brrr {
            get {
                return new ivec4(this.v2, this.v0, this.v0, this.v0);
            }
        }
        
        public ivec4 psss {
            get {
                return new ivec4(this.v2, this.v0, this.v0, this.v0);
            }
        }
        
        public ivec4 zxxy {
            get {
                return new ivec4(this.v2, this.v0, this.v0, this.v1);
            }
        }
        
        public ivec4 brrg {
            get {
                return new ivec4(this.v2, this.v0, this.v0, this.v1);
            }
        }
        
        public ivec4 psst {
            get {
                return new ivec4(this.v2, this.v0, this.v0, this.v1);
            }
        }
        
        public ivec4 zxxz {
            get {
                return new ivec4(this.v2, this.v0, this.v0, this.v2);
            }
        }
        
        public ivec4 brrb {
            get {
                return new ivec4(this.v2, this.v0, this.v0, this.v2);
            }
        }
        
        public ivec4 pssp {
            get {
                return new ivec4(this.v2, this.v0, this.v0, this.v2);
            }
        }
        
        public ivec4 zxxw {
            get {
                return new ivec4(this.v2, this.v0, this.v0, this.v3);
            }
        }
        
        public ivec4 brra {
            get {
                return new ivec4(this.v2, this.v0, this.v0, this.v3);
            }
        }
        
        public ivec4 pssq {
            get {
                return new ivec4(this.v2, this.v0, this.v0, this.v3);
            }
        }
        
        public ivec4 zxyx {
            get {
                return new ivec4(this.v2, this.v0, this.v1, this.v0);
            }
        }
        
        public ivec4 brgr {
            get {
                return new ivec4(this.v2, this.v0, this.v1, this.v0);
            }
        }
        
        public ivec4 psts {
            get {
                return new ivec4(this.v2, this.v0, this.v1, this.v0);
            }
        }
        
        public ivec4 zxyy {
            get {
                return new ivec4(this.v2, this.v0, this.v1, this.v1);
            }
        }
        
        public ivec4 brgg {
            get {
                return new ivec4(this.v2, this.v0, this.v1, this.v1);
            }
        }
        
        public ivec4 pstt {
            get {
                return new ivec4(this.v2, this.v0, this.v1, this.v1);
            }
        }
        
        public ivec4 zxyz {
            get {
                return new ivec4(this.v2, this.v0, this.v1, this.v2);
            }
        }
        
        public ivec4 brgb {
            get {
                return new ivec4(this.v2, this.v0, this.v1, this.v2);
            }
        }
        
        public ivec4 pstp {
            get {
                return new ivec4(this.v2, this.v0, this.v1, this.v2);
            }
        }
        
        public ivec4 zxyw {
            get {
                return new ivec4(this.v2, this.v0, this.v1, this.v3);
            }
            set {
                this.v2 = value.v0;
                this.v0 = value.v1;
                this.v1 = value.v2;
                this.v3 = value.v3;
            }
        }
        
        public ivec4 brga {
            get {
                return new ivec4(this.v2, this.v0, this.v1, this.v3);
            }
            set {
                this.v2 = value.v0;
                this.v0 = value.v1;
                this.v1 = value.v2;
                this.v3 = value.v3;
            }
        }
        
        public ivec4 pstq {
            get {
                return new ivec4(this.v2, this.v0, this.v1, this.v3);
            }
            set {
                this.v2 = value.v0;
                this.v0 = value.v1;
                this.v1 = value.v2;
                this.v3 = value.v3;
            }
        }
        
        public ivec4 zxzx {
            get {
                return new ivec4(this.v2, this.v0, this.v2, this.v0);
            }
        }
        
        public ivec4 brbr {
            get {
                return new ivec4(this.v2, this.v0, this.v2, this.v0);
            }
        }
        
        public ivec4 psps {
            get {
                return new ivec4(this.v2, this.v0, this.v2, this.v0);
            }
        }
        
        public ivec4 zxzy {
            get {
                return new ivec4(this.v2, this.v0, this.v2, this.v1);
            }
        }
        
        public ivec4 brbg {
            get {
                return new ivec4(this.v2, this.v0, this.v2, this.v1);
            }
        }
        
        public ivec4 pspt {
            get {
                return new ivec4(this.v2, this.v0, this.v2, this.v1);
            }
        }
        
        public ivec4 zxzz {
            get {
                return new ivec4(this.v2, this.v0, this.v2, this.v2);
            }
        }
        
        public ivec4 brbb {
            get {
                return new ivec4(this.v2, this.v0, this.v2, this.v2);
            }
        }
        
        public ivec4 pspp {
            get {
                return new ivec4(this.v2, this.v0, this.v2, this.v2);
            }
        }
        
        public ivec4 zxzw {
            get {
                return new ivec4(this.v2, this.v0, this.v2, this.v3);
            }
        }
        
        public ivec4 brba {
            get {
                return new ivec4(this.v2, this.v0, this.v2, this.v3);
            }
        }
        
        public ivec4 pspq {
            get {
                return new ivec4(this.v2, this.v0, this.v2, this.v3);
            }
        }
        
        public ivec4 zxwx {
            get {
                return new ivec4(this.v2, this.v0, this.v3, this.v0);
            }
        }
        
        public ivec4 brar {
            get {
                return new ivec4(this.v2, this.v0, this.v3, this.v0);
            }
        }
        
        public ivec4 psqs {
            get {
                return new ivec4(this.v2, this.v0, this.v3, this.v0);
            }
        }
        
        public ivec4 zxwy {
            get {
                return new ivec4(this.v2, this.v0, this.v3, this.v1);
            }
            set {
                this.v2 = value.v0;
                this.v0 = value.v1;
                this.v3 = value.v2;
                this.v1 = value.v3;
            }
        }
        
        public ivec4 brag {
            get {
                return new ivec4(this.v2, this.v0, this.v3, this.v1);
            }
            set {
                this.v2 = value.v0;
                this.v0 = value.v1;
                this.v3 = value.v2;
                this.v1 = value.v3;
            }
        }
        
        public ivec4 psqt {
            get {
                return new ivec4(this.v2, this.v0, this.v3, this.v1);
            }
            set {
                this.v2 = value.v0;
                this.v0 = value.v1;
                this.v3 = value.v2;
                this.v1 = value.v3;
            }
        }
        
        public ivec4 zxwz {
            get {
                return new ivec4(this.v2, this.v0, this.v3, this.v2);
            }
        }
        
        public ivec4 brab {
            get {
                return new ivec4(this.v2, this.v0, this.v3, this.v2);
            }
        }
        
        public ivec4 psqp {
            get {
                return new ivec4(this.v2, this.v0, this.v3, this.v2);
            }
        }
        
        public ivec4 zxww {
            get {
                return new ivec4(this.v2, this.v0, this.v3, this.v3);
            }
        }
        
        public ivec4 braa {
            get {
                return new ivec4(this.v2, this.v0, this.v3, this.v3);
            }
        }
        
        public ivec4 psqq {
            get {
                return new ivec4(this.v2, this.v0, this.v3, this.v3);
            }
        }
        
        public ivec4 zyxx {
            get {
                return new ivec4(this.v2, this.v1, this.v0, this.v0);
            }
        }
        
        public ivec4 bgrr {
            get {
                return new ivec4(this.v2, this.v1, this.v0, this.v0);
            }
        }
        
        public ivec4 ptss {
            get {
                return new ivec4(this.v2, this.v1, this.v0, this.v0);
            }
        }
        
        public ivec4 zyxy {
            get {
                return new ivec4(this.v2, this.v1, this.v0, this.v1);
            }
        }
        
        public ivec4 bgrg {
            get {
                return new ivec4(this.v2, this.v1, this.v0, this.v1);
            }
        }
        
        public ivec4 ptst {
            get {
                return new ivec4(this.v2, this.v1, this.v0, this.v1);
            }
        }
        
        public ivec4 zyxz {
            get {
                return new ivec4(this.v2, this.v1, this.v0, this.v2);
            }
        }
        
        public ivec4 bgrb {
            get {
                return new ivec4(this.v2, this.v1, this.v0, this.v2);
            }
        }
        
        public ivec4 ptsp {
            get {
                return new ivec4(this.v2, this.v1, this.v0, this.v2);
            }
        }
        
        public ivec4 zyxw {
            get {
                return new ivec4(this.v2, this.v1, this.v0, this.v3);
            }
            set {
                this.v2 = value.v0;
                this.v1 = value.v1;
                this.v0 = value.v2;
                this.v3 = value.v3;
            }
        }
        
        public ivec4 bgra {
            get {
                return new ivec4(this.v2, this.v1, this.v0, this.v3);
            }
            set {
                this.v2 = value.v0;
                this.v1 = value.v1;
                this.v0 = value.v2;
                this.v3 = value.v3;
            }
        }
        
        public ivec4 ptsq {
            get {
                return new ivec4(this.v2, this.v1, this.v0, this.v3);
            }
            set {
                this.v2 = value.v0;
                this.v1 = value.v1;
                this.v0 = value.v2;
                this.v3 = value.v3;
            }
        }
        
        public ivec4 zyyx {
            get {
                return new ivec4(this.v2, this.v1, this.v1, this.v0);
            }
        }
        
        public ivec4 bggr {
            get {
                return new ivec4(this.v2, this.v1, this.v1, this.v0);
            }
        }
        
        public ivec4 ptts {
            get {
                return new ivec4(this.v2, this.v1, this.v1, this.v0);
            }
        }
        
        public ivec4 zyyy {
            get {
                return new ivec4(this.v2, this.v1, this.v1, this.v1);
            }
        }
        
        public ivec4 bggg {
            get {
                return new ivec4(this.v2, this.v1, this.v1, this.v1);
            }
        }
        
        public ivec4 pttt {
            get {
                return new ivec4(this.v2, this.v1, this.v1, this.v1);
            }
        }
        
        public ivec4 zyyz {
            get {
                return new ivec4(this.v2, this.v1, this.v1, this.v2);
            }
        }
        
        public ivec4 bggb {
            get {
                return new ivec4(this.v2, this.v1, this.v1, this.v2);
            }
        }
        
        public ivec4 pttp {
            get {
                return new ivec4(this.v2, this.v1, this.v1, this.v2);
            }
        }
        
        public ivec4 zyyw {
            get {
                return new ivec4(this.v2, this.v1, this.v1, this.v3);
            }
        }
        
        public ivec4 bgga {
            get {
                return new ivec4(this.v2, this.v1, this.v1, this.v3);
            }
        }
        
        public ivec4 pttq {
            get {
                return new ivec4(this.v2, this.v1, this.v1, this.v3);
            }
        }
        
        public ivec4 zyzx {
            get {
                return new ivec4(this.v2, this.v1, this.v2, this.v0);
            }
        }
        
        public ivec4 bgbr {
            get {
                return new ivec4(this.v2, this.v1, this.v2, this.v0);
            }
        }
        
        public ivec4 ptps {
            get {
                return new ivec4(this.v2, this.v1, this.v2, this.v0);
            }
        }
        
        public ivec4 zyzy {
            get {
                return new ivec4(this.v2, this.v1, this.v2, this.v1);
            }
        }
        
        public ivec4 bgbg {
            get {
                return new ivec4(this.v2, this.v1, this.v2, this.v1);
            }
        }
        
        public ivec4 ptpt {
            get {
                return new ivec4(this.v2, this.v1, this.v2, this.v1);
            }
        }
        
        public ivec4 zyzz {
            get {
                return new ivec4(this.v2, this.v1, this.v2, this.v2);
            }
        }
        
        public ivec4 bgbb {
            get {
                return new ivec4(this.v2, this.v1, this.v2, this.v2);
            }
        }
        
        public ivec4 ptpp {
            get {
                return new ivec4(this.v2, this.v1, this.v2, this.v2);
            }
        }
        
        public ivec4 zyzw {
            get {
                return new ivec4(this.v2, this.v1, this.v2, this.v3);
            }
        }
        
        public ivec4 bgba {
            get {
                return new ivec4(this.v2, this.v1, this.v2, this.v3);
            }
        }
        
        public ivec4 ptpq {
            get {
                return new ivec4(this.v2, this.v1, this.v2, this.v3);
            }
        }
        
        public ivec4 zywx {
            get {
                return new ivec4(this.v2, this.v1, this.v3, this.v0);
            }
            set {
                this.v2 = value.v0;
                this.v1 = value.v1;
                this.v3 = value.v2;
                this.v0 = value.v3;
            }
        }
        
        public ivec4 bgar {
            get {
                return new ivec4(this.v2, this.v1, this.v3, this.v0);
            }
            set {
                this.v2 = value.v0;
                this.v1 = value.v1;
                this.v3 = value.v2;
                this.v0 = value.v3;
            }
        }
        
        public ivec4 ptqs {
            get {
                return new ivec4(this.v2, this.v1, this.v3, this.v0);
            }
            set {
                this.v2 = value.v0;
                this.v1 = value.v1;
                this.v3 = value.v2;
                this.v0 = value.v3;
            }
        }
        
        public ivec4 zywy {
            get {
                return new ivec4(this.v2, this.v1, this.v3, this.v1);
            }
        }
        
        public ivec4 bgag {
            get {
                return new ivec4(this.v2, this.v1, this.v3, this.v1);
            }
        }
        
        public ivec4 ptqt {
            get {
                return new ivec4(this.v2, this.v1, this.v3, this.v1);
            }
        }
        
        public ivec4 zywz {
            get {
                return new ivec4(this.v2, this.v1, this.v3, this.v2);
            }
        }
        
        public ivec4 bgab {
            get {
                return new ivec4(this.v2, this.v1, this.v3, this.v2);
            }
        }
        
        public ivec4 ptqp {
            get {
                return new ivec4(this.v2, this.v1, this.v3, this.v2);
            }
        }
        
        public ivec4 zyww {
            get {
                return new ivec4(this.v2, this.v1, this.v3, this.v3);
            }
        }
        
        public ivec4 bgaa {
            get {
                return new ivec4(this.v2, this.v1, this.v3, this.v3);
            }
        }
        
        public ivec4 ptqq {
            get {
                return new ivec4(this.v2, this.v1, this.v3, this.v3);
            }
        }
        
        public ivec4 zzxx {
            get {
                return new ivec4(this.v2, this.v2, this.v0, this.v0);
            }
        }
        
        public ivec4 bbrr {
            get {
                return new ivec4(this.v2, this.v2, this.v0, this.v0);
            }
        }
        
        public ivec4 ppss {
            get {
                return new ivec4(this.v2, this.v2, this.v0, this.v0);
            }
        }
        
        public ivec4 zzxy {
            get {
                return new ivec4(this.v2, this.v2, this.v0, this.v1);
            }
        }
        
        public ivec4 bbrg {
            get {
                return new ivec4(this.v2, this.v2, this.v0, this.v1);
            }
        }
        
        public ivec4 ppst {
            get {
                return new ivec4(this.v2, this.v2, this.v0, this.v1);
            }
        }
        
        public ivec4 zzxz {
            get {
                return new ivec4(this.v2, this.v2, this.v0, this.v2);
            }
        }
        
        public ivec4 bbrb {
            get {
                return new ivec4(this.v2, this.v2, this.v0, this.v2);
            }
        }
        
        public ivec4 ppsp {
            get {
                return new ivec4(this.v2, this.v2, this.v0, this.v2);
            }
        }
        
        public ivec4 zzxw {
            get {
                return new ivec4(this.v2, this.v2, this.v0, this.v3);
            }
        }
        
        public ivec4 bbra {
            get {
                return new ivec4(this.v2, this.v2, this.v0, this.v3);
            }
        }
        
        public ivec4 ppsq {
            get {
                return new ivec4(this.v2, this.v2, this.v0, this.v3);
            }
        }
        
        public ivec4 zzyx {
            get {
                return new ivec4(this.v2, this.v2, this.v1, this.v0);
            }
        }
        
        public ivec4 bbgr {
            get {
                return new ivec4(this.v2, this.v2, this.v1, this.v0);
            }
        }
        
        public ivec4 ppts {
            get {
                return new ivec4(this.v2, this.v2, this.v1, this.v0);
            }
        }
        
        public ivec4 zzyy {
            get {
                return new ivec4(this.v2, this.v2, this.v1, this.v1);
            }
        }
        
        public ivec4 bbgg {
            get {
                return new ivec4(this.v2, this.v2, this.v1, this.v1);
            }
        }
        
        public ivec4 pptt {
            get {
                return new ivec4(this.v2, this.v2, this.v1, this.v1);
            }
        }
        
        public ivec4 zzyz {
            get {
                return new ivec4(this.v2, this.v2, this.v1, this.v2);
            }
        }
        
        public ivec4 bbgb {
            get {
                return new ivec4(this.v2, this.v2, this.v1, this.v2);
            }
        }
        
        public ivec4 pptp {
            get {
                return new ivec4(this.v2, this.v2, this.v1, this.v2);
            }
        }
        
        public ivec4 zzyw {
            get {
                return new ivec4(this.v2, this.v2, this.v1, this.v3);
            }
        }
        
        public ivec4 bbga {
            get {
                return new ivec4(this.v2, this.v2, this.v1, this.v3);
            }
        }
        
        public ivec4 pptq {
            get {
                return new ivec4(this.v2, this.v2, this.v1, this.v3);
            }
        }
        
        public ivec4 zzzx {
            get {
                return new ivec4(this.v2, this.v2, this.v2, this.v0);
            }
        }
        
        public ivec4 bbbr {
            get {
                return new ivec4(this.v2, this.v2, this.v2, this.v0);
            }
        }
        
        public ivec4 ppps {
            get {
                return new ivec4(this.v2, this.v2, this.v2, this.v0);
            }
        }
        
        public ivec4 zzzy {
            get {
                return new ivec4(this.v2, this.v2, this.v2, this.v1);
            }
        }
        
        public ivec4 bbbg {
            get {
                return new ivec4(this.v2, this.v2, this.v2, this.v1);
            }
        }
        
        public ivec4 pppt {
            get {
                return new ivec4(this.v2, this.v2, this.v2, this.v1);
            }
        }
        
        public ivec4 zzzz {
            get {
                return new ivec4(this.v2, this.v2, this.v2, this.v2);
            }
        }
        
        public ivec4 bbbb {
            get {
                return new ivec4(this.v2, this.v2, this.v2, this.v2);
            }
        }
        
        public ivec4 pppp {
            get {
                return new ivec4(this.v2, this.v2, this.v2, this.v2);
            }
        }
        
        public ivec4 zzzw {
            get {
                return new ivec4(this.v2, this.v2, this.v2, this.v3);
            }
        }
        
        public ivec4 bbba {
            get {
                return new ivec4(this.v2, this.v2, this.v2, this.v3);
            }
        }
        
        public ivec4 pppq {
            get {
                return new ivec4(this.v2, this.v2, this.v2, this.v3);
            }
        }
        
        public ivec4 zzwx {
            get {
                return new ivec4(this.v2, this.v2, this.v3, this.v0);
            }
        }
        
        public ivec4 bbar {
            get {
                return new ivec4(this.v2, this.v2, this.v3, this.v0);
            }
        }
        
        public ivec4 ppqs {
            get {
                return new ivec4(this.v2, this.v2, this.v3, this.v0);
            }
        }
        
        public ivec4 zzwy {
            get {
                return new ivec4(this.v2, this.v2, this.v3, this.v1);
            }
        }
        
        public ivec4 bbag {
            get {
                return new ivec4(this.v2, this.v2, this.v3, this.v1);
            }
        }
        
        public ivec4 ppqt {
            get {
                return new ivec4(this.v2, this.v2, this.v3, this.v1);
            }
        }
        
        public ivec4 zzwz {
            get {
                return new ivec4(this.v2, this.v2, this.v3, this.v2);
            }
        }
        
        public ivec4 bbab {
            get {
                return new ivec4(this.v2, this.v2, this.v3, this.v2);
            }
        }
        
        public ivec4 ppqp {
            get {
                return new ivec4(this.v2, this.v2, this.v3, this.v2);
            }
        }
        
        public ivec4 zzww {
            get {
                return new ivec4(this.v2, this.v2, this.v3, this.v3);
            }
        }
        
        public ivec4 bbaa {
            get {
                return new ivec4(this.v2, this.v2, this.v3, this.v3);
            }
        }
        
        public ivec4 ppqq {
            get {
                return new ivec4(this.v2, this.v2, this.v3, this.v3);
            }
        }
        
        public ivec4 zwxx {
            get {
                return new ivec4(this.v2, this.v3, this.v0, this.v0);
            }
        }
        
        public ivec4 barr {
            get {
                return new ivec4(this.v2, this.v3, this.v0, this.v0);
            }
        }
        
        public ivec4 pqss {
            get {
                return new ivec4(this.v2, this.v3, this.v0, this.v0);
            }
        }
        
        public ivec4 zwxy {
            get {
                return new ivec4(this.v2, this.v3, this.v0, this.v1);
            }
            set {
                this.v2 = value.v0;
                this.v3 = value.v1;
                this.v0 = value.v2;
                this.v1 = value.v3;
            }
        }
        
        public ivec4 barg {
            get {
                return new ivec4(this.v2, this.v3, this.v0, this.v1);
            }
            set {
                this.v2 = value.v0;
                this.v3 = value.v1;
                this.v0 = value.v2;
                this.v1 = value.v3;
            }
        }
        
        public ivec4 pqst {
            get {
                return new ivec4(this.v2, this.v3, this.v0, this.v1);
            }
            set {
                this.v2 = value.v0;
                this.v3 = value.v1;
                this.v0 = value.v2;
                this.v1 = value.v3;
            }
        }
        
        public ivec4 zwxz {
            get {
                return new ivec4(this.v2, this.v3, this.v0, this.v2);
            }
        }
        
        public ivec4 barb {
            get {
                return new ivec4(this.v2, this.v3, this.v0, this.v2);
            }
        }
        
        public ivec4 pqsp {
            get {
                return new ivec4(this.v2, this.v3, this.v0, this.v2);
            }
        }
        
        public ivec4 zwxw {
            get {
                return new ivec4(this.v2, this.v3, this.v0, this.v3);
            }
        }
        
        public ivec4 bara {
            get {
                return new ivec4(this.v2, this.v3, this.v0, this.v3);
            }
        }
        
        public ivec4 pqsq {
            get {
                return new ivec4(this.v2, this.v3, this.v0, this.v3);
            }
        }
        
        public ivec4 zwyx {
            get {
                return new ivec4(this.v2, this.v3, this.v1, this.v0);
            }
            set {
                this.v2 = value.v0;
                this.v3 = value.v1;
                this.v1 = value.v2;
                this.v0 = value.v3;
            }
        }
        
        public ivec4 bagr {
            get {
                return new ivec4(this.v2, this.v3, this.v1, this.v0);
            }
            set {
                this.v2 = value.v0;
                this.v3 = value.v1;
                this.v1 = value.v2;
                this.v0 = value.v3;
            }
        }
        
        public ivec4 pqts {
            get {
                return new ivec4(this.v2, this.v3, this.v1, this.v0);
            }
            set {
                this.v2 = value.v0;
                this.v3 = value.v1;
                this.v1 = value.v2;
                this.v0 = value.v3;
            }
        }
        
        public ivec4 zwyy {
            get {
                return new ivec4(this.v2, this.v3, this.v1, this.v1);
            }
        }
        
        public ivec4 bagg {
            get {
                return new ivec4(this.v2, this.v3, this.v1, this.v1);
            }
        }
        
        public ivec4 pqtt {
            get {
                return new ivec4(this.v2, this.v3, this.v1, this.v1);
            }
        }
        
        public ivec4 zwyz {
            get {
                return new ivec4(this.v2, this.v3, this.v1, this.v2);
            }
        }
        
        public ivec4 bagb {
            get {
                return new ivec4(this.v2, this.v3, this.v1, this.v2);
            }
        }
        
        public ivec4 pqtp {
            get {
                return new ivec4(this.v2, this.v3, this.v1, this.v2);
            }
        }
        
        public ivec4 zwyw {
            get {
                return new ivec4(this.v2, this.v3, this.v1, this.v3);
            }
        }
        
        public ivec4 baga {
            get {
                return new ivec4(this.v2, this.v3, this.v1, this.v3);
            }
        }
        
        public ivec4 pqtq {
            get {
                return new ivec4(this.v2, this.v3, this.v1, this.v3);
            }
        }
        
        public ivec4 zwzx {
            get {
                return new ivec4(this.v2, this.v3, this.v2, this.v0);
            }
        }
        
        public ivec4 babr {
            get {
                return new ivec4(this.v2, this.v3, this.v2, this.v0);
            }
        }
        
        public ivec4 pqps {
            get {
                return new ivec4(this.v2, this.v3, this.v2, this.v0);
            }
        }
        
        public ivec4 zwzy {
            get {
                return new ivec4(this.v2, this.v3, this.v2, this.v1);
            }
        }
        
        public ivec4 babg {
            get {
                return new ivec4(this.v2, this.v3, this.v2, this.v1);
            }
        }
        
        public ivec4 pqpt {
            get {
                return new ivec4(this.v2, this.v3, this.v2, this.v1);
            }
        }
        
        public ivec4 zwzz {
            get {
                return new ivec4(this.v2, this.v3, this.v2, this.v2);
            }
        }
        
        public ivec4 babb {
            get {
                return new ivec4(this.v2, this.v3, this.v2, this.v2);
            }
        }
        
        public ivec4 pqpp {
            get {
                return new ivec4(this.v2, this.v3, this.v2, this.v2);
            }
        }
        
        public ivec4 zwzw {
            get {
                return new ivec4(this.v2, this.v3, this.v2, this.v3);
            }
        }
        
        public ivec4 baba {
            get {
                return new ivec4(this.v2, this.v3, this.v2, this.v3);
            }
        }
        
        public ivec4 pqpq {
            get {
                return new ivec4(this.v2, this.v3, this.v2, this.v3);
            }
        }
        
        public ivec4 zwwx {
            get {
                return new ivec4(this.v2, this.v3, this.v3, this.v0);
            }
        }
        
        public ivec4 baar {
            get {
                return new ivec4(this.v2, this.v3, this.v3, this.v0);
            }
        }
        
        public ivec4 pqqs {
            get {
                return new ivec4(this.v2, this.v3, this.v3, this.v0);
            }
        }
        
        public ivec4 zwwy {
            get {
                return new ivec4(this.v2, this.v3, this.v3, this.v1);
            }
        }
        
        public ivec4 baag {
            get {
                return new ivec4(this.v2, this.v3, this.v3, this.v1);
            }
        }
        
        public ivec4 pqqt {
            get {
                return new ivec4(this.v2, this.v3, this.v3, this.v1);
            }
        }
        
        public ivec4 zwwz {
            get {
                return new ivec4(this.v2, this.v3, this.v3, this.v2);
            }
        }
        
        public ivec4 baab {
            get {
                return new ivec4(this.v2, this.v3, this.v3, this.v2);
            }
        }
        
        public ivec4 pqqp {
            get {
                return new ivec4(this.v2, this.v3, this.v3, this.v2);
            }
        }
        
        public ivec4 zwww {
            get {
                return new ivec4(this.v2, this.v3, this.v3, this.v3);
            }
        }
        
        public ivec4 baaa {
            get {
                return new ivec4(this.v2, this.v3, this.v3, this.v3);
            }
        }
        
        public ivec4 pqqq {
            get {
                return new ivec4(this.v2, this.v3, this.v3, this.v3);
            }
        }
        
        public ivec4 wxxx {
            get {
                return new ivec4(this.v3, this.v0, this.v0, this.v0);
            }
        }
        
        public ivec4 arrr {
            get {
                return new ivec4(this.v3, this.v0, this.v0, this.v0);
            }
        }
        
        public ivec4 qsss {
            get {
                return new ivec4(this.v3, this.v0, this.v0, this.v0);
            }
        }
        
        public ivec4 wxxy {
            get {
                return new ivec4(this.v3, this.v0, this.v0, this.v1);
            }
        }
        
        public ivec4 arrg {
            get {
                return new ivec4(this.v3, this.v0, this.v0, this.v1);
            }
        }
        
        public ivec4 qsst {
            get {
                return new ivec4(this.v3, this.v0, this.v0, this.v1);
            }
        }
        
        public ivec4 wxxz {
            get {
                return new ivec4(this.v3, this.v0, this.v0, this.v2);
            }
        }
        
        public ivec4 arrb {
            get {
                return new ivec4(this.v3, this.v0, this.v0, this.v2);
            }
        }
        
        public ivec4 qssp {
            get {
                return new ivec4(this.v3, this.v0, this.v0, this.v2);
            }
        }
        
        public ivec4 wxxw {
            get {
                return new ivec4(this.v3, this.v0, this.v0, this.v3);
            }
        }
        
        public ivec4 arra {
            get {
                return new ivec4(this.v3, this.v0, this.v0, this.v3);
            }
        }
        
        public ivec4 qssq {
            get {
                return new ivec4(this.v3, this.v0, this.v0, this.v3);
            }
        }
        
        public ivec4 wxyx {
            get {
                return new ivec4(this.v3, this.v0, this.v1, this.v0);
            }
        }
        
        public ivec4 argr {
            get {
                return new ivec4(this.v3, this.v0, this.v1, this.v0);
            }
        }
        
        public ivec4 qsts {
            get {
                return new ivec4(this.v3, this.v0, this.v1, this.v0);
            }
        }
        
        public ivec4 wxyy {
            get {
                return new ivec4(this.v3, this.v0, this.v1, this.v1);
            }
        }
        
        public ivec4 argg {
            get {
                return new ivec4(this.v3, this.v0, this.v1, this.v1);
            }
        }
        
        public ivec4 qstt {
            get {
                return new ivec4(this.v3, this.v0, this.v1, this.v1);
            }
        }
        
        public ivec4 wxyz {
            get {
                return new ivec4(this.v3, this.v0, this.v1, this.v2);
            }
            set {
                this.v3 = value.v0;
                this.v0 = value.v1;
                this.v1 = value.v2;
                this.v2 = value.v3;
            }
        }
        
        public ivec4 argb {
            get {
                return new ivec4(this.v3, this.v0, this.v1, this.v2);
            }
            set {
                this.v3 = value.v0;
                this.v0 = value.v1;
                this.v1 = value.v2;
                this.v2 = value.v3;
            }
        }
        
        public ivec4 qstp {
            get {
                return new ivec4(this.v3, this.v0, this.v1, this.v2);
            }
            set {
                this.v3 = value.v0;
                this.v0 = value.v1;
                this.v1 = value.v2;
                this.v2 = value.v3;
            }
        }
        
        public ivec4 wxyw {
            get {
                return new ivec4(this.v3, this.v0, this.v1, this.v3);
            }
        }
        
        public ivec4 arga {
            get {
                return new ivec4(this.v3, this.v0, this.v1, this.v3);
            }
        }
        
        public ivec4 qstq {
            get {
                return new ivec4(this.v3, this.v0, this.v1, this.v3);
            }
        }
        
        public ivec4 wxzx {
            get {
                return new ivec4(this.v3, this.v0, this.v2, this.v0);
            }
        }
        
        public ivec4 arbr {
            get {
                return new ivec4(this.v3, this.v0, this.v2, this.v0);
            }
        }
        
        public ivec4 qsps {
            get {
                return new ivec4(this.v3, this.v0, this.v2, this.v0);
            }
        }
        
        public ivec4 wxzy {
            get {
                return new ivec4(this.v3, this.v0, this.v2, this.v1);
            }
            set {
                this.v3 = value.v0;
                this.v0 = value.v1;
                this.v2 = value.v2;
                this.v1 = value.v3;
            }
        }
        
        public ivec4 arbg {
            get {
                return new ivec4(this.v3, this.v0, this.v2, this.v1);
            }
            set {
                this.v3 = value.v0;
                this.v0 = value.v1;
                this.v2 = value.v2;
                this.v1 = value.v3;
            }
        }
        
        public ivec4 qspt {
            get {
                return new ivec4(this.v3, this.v0, this.v2, this.v1);
            }
            set {
                this.v3 = value.v0;
                this.v0 = value.v1;
                this.v2 = value.v2;
                this.v1 = value.v3;
            }
        }
        
        public ivec4 wxzz {
            get {
                return new ivec4(this.v3, this.v0, this.v2, this.v2);
            }
        }
        
        public ivec4 arbb {
            get {
                return new ivec4(this.v3, this.v0, this.v2, this.v2);
            }
        }
        
        public ivec4 qspp {
            get {
                return new ivec4(this.v3, this.v0, this.v2, this.v2);
            }
        }
        
        public ivec4 wxzw {
            get {
                return new ivec4(this.v3, this.v0, this.v2, this.v3);
            }
        }
        
        public ivec4 arba {
            get {
                return new ivec4(this.v3, this.v0, this.v2, this.v3);
            }
        }
        
        public ivec4 qspq {
            get {
                return new ivec4(this.v3, this.v0, this.v2, this.v3);
            }
        }
        
        public ivec4 wxwx {
            get {
                return new ivec4(this.v3, this.v0, this.v3, this.v0);
            }
        }
        
        public ivec4 arar {
            get {
                return new ivec4(this.v3, this.v0, this.v3, this.v0);
            }
        }
        
        public ivec4 qsqs {
            get {
                return new ivec4(this.v3, this.v0, this.v3, this.v0);
            }
        }
        
        public ivec4 wxwy {
            get {
                return new ivec4(this.v3, this.v0, this.v3, this.v1);
            }
        }
        
        public ivec4 arag {
            get {
                return new ivec4(this.v3, this.v0, this.v3, this.v1);
            }
        }
        
        public ivec4 qsqt {
            get {
                return new ivec4(this.v3, this.v0, this.v3, this.v1);
            }
        }
        
        public ivec4 wxwz {
            get {
                return new ivec4(this.v3, this.v0, this.v3, this.v2);
            }
        }
        
        public ivec4 arab {
            get {
                return new ivec4(this.v3, this.v0, this.v3, this.v2);
            }
        }
        
        public ivec4 qsqp {
            get {
                return new ivec4(this.v3, this.v0, this.v3, this.v2);
            }
        }
        
        public ivec4 wxww {
            get {
                return new ivec4(this.v3, this.v0, this.v3, this.v3);
            }
        }
        
        public ivec4 araa {
            get {
                return new ivec4(this.v3, this.v0, this.v3, this.v3);
            }
        }
        
        public ivec4 qsqq {
            get {
                return new ivec4(this.v3, this.v0, this.v3, this.v3);
            }
        }
        
        public ivec4 wyxx {
            get {
                return new ivec4(this.v3, this.v1, this.v0, this.v0);
            }
        }
        
        public ivec4 agrr {
            get {
                return new ivec4(this.v3, this.v1, this.v0, this.v0);
            }
        }
        
        public ivec4 qtss {
            get {
                return new ivec4(this.v3, this.v1, this.v0, this.v0);
            }
        }
        
        public ivec4 wyxy {
            get {
                return new ivec4(this.v3, this.v1, this.v0, this.v1);
            }
        }
        
        public ivec4 agrg {
            get {
                return new ivec4(this.v3, this.v1, this.v0, this.v1);
            }
        }
        
        public ivec4 qtst {
            get {
                return new ivec4(this.v3, this.v1, this.v0, this.v1);
            }
        }
        
        public ivec4 wyxz {
            get {
                return new ivec4(this.v3, this.v1, this.v0, this.v2);
            }
            set {
                this.v3 = value.v0;
                this.v1 = value.v1;
                this.v0 = value.v2;
                this.v2 = value.v3;
            }
        }
        
        public ivec4 agrb {
            get {
                return new ivec4(this.v3, this.v1, this.v0, this.v2);
            }
            set {
                this.v3 = value.v0;
                this.v1 = value.v1;
                this.v0 = value.v2;
                this.v2 = value.v3;
            }
        }
        
        public ivec4 qtsp {
            get {
                return new ivec4(this.v3, this.v1, this.v0, this.v2);
            }
            set {
                this.v3 = value.v0;
                this.v1 = value.v1;
                this.v0 = value.v2;
                this.v2 = value.v3;
            }
        }
        
        public ivec4 wyxw {
            get {
                return new ivec4(this.v3, this.v1, this.v0, this.v3);
            }
        }
        
        public ivec4 agra {
            get {
                return new ivec4(this.v3, this.v1, this.v0, this.v3);
            }
        }
        
        public ivec4 qtsq {
            get {
                return new ivec4(this.v3, this.v1, this.v0, this.v3);
            }
        }
        
        public ivec4 wyyx {
            get {
                return new ivec4(this.v3, this.v1, this.v1, this.v0);
            }
        }
        
        public ivec4 aggr {
            get {
                return new ivec4(this.v3, this.v1, this.v1, this.v0);
            }
        }
        
        public ivec4 qtts {
            get {
                return new ivec4(this.v3, this.v1, this.v1, this.v0);
            }
        }
        
        public ivec4 wyyy {
            get {
                return new ivec4(this.v3, this.v1, this.v1, this.v1);
            }
        }
        
        public ivec4 aggg {
            get {
                return new ivec4(this.v3, this.v1, this.v1, this.v1);
            }
        }
        
        public ivec4 qttt {
            get {
                return new ivec4(this.v3, this.v1, this.v1, this.v1);
            }
        }
        
        public ivec4 wyyz {
            get {
                return new ivec4(this.v3, this.v1, this.v1, this.v2);
            }
        }
        
        public ivec4 aggb {
            get {
                return new ivec4(this.v3, this.v1, this.v1, this.v2);
            }
        }
        
        public ivec4 qttp {
            get {
                return new ivec4(this.v3, this.v1, this.v1, this.v2);
            }
        }
        
        public ivec4 wyyw {
            get {
                return new ivec4(this.v3, this.v1, this.v1, this.v3);
            }
        }
        
        public ivec4 agga {
            get {
                return new ivec4(this.v3, this.v1, this.v1, this.v3);
            }
        }
        
        public ivec4 qttq {
            get {
                return new ivec4(this.v3, this.v1, this.v1, this.v3);
            }
        }
        
        public ivec4 wyzx {
            get {
                return new ivec4(this.v3, this.v1, this.v2, this.v0);
            }
            set {
                this.v3 = value.v0;
                this.v1 = value.v1;
                this.v2 = value.v2;
                this.v0 = value.v3;
            }
        }
        
        public ivec4 agbr {
            get {
                return new ivec4(this.v3, this.v1, this.v2, this.v0);
            }
            set {
                this.v3 = value.v0;
                this.v1 = value.v1;
                this.v2 = value.v2;
                this.v0 = value.v3;
            }
        }
        
        public ivec4 qtps {
            get {
                return new ivec4(this.v3, this.v1, this.v2, this.v0);
            }
            set {
                this.v3 = value.v0;
                this.v1 = value.v1;
                this.v2 = value.v2;
                this.v0 = value.v3;
            }
        }
        
        public ivec4 wyzy {
            get {
                return new ivec4(this.v3, this.v1, this.v2, this.v1);
            }
        }
        
        public ivec4 agbg {
            get {
                return new ivec4(this.v3, this.v1, this.v2, this.v1);
            }
        }
        
        public ivec4 qtpt {
            get {
                return new ivec4(this.v3, this.v1, this.v2, this.v1);
            }
        }
        
        public ivec4 wyzz {
            get {
                return new ivec4(this.v3, this.v1, this.v2, this.v2);
            }
        }
        
        public ivec4 agbb {
            get {
                return new ivec4(this.v3, this.v1, this.v2, this.v2);
            }
        }
        
        public ivec4 qtpp {
            get {
                return new ivec4(this.v3, this.v1, this.v2, this.v2);
            }
        }
        
        public ivec4 wyzw {
            get {
                return new ivec4(this.v3, this.v1, this.v2, this.v3);
            }
        }
        
        public ivec4 agba {
            get {
                return new ivec4(this.v3, this.v1, this.v2, this.v3);
            }
        }
        
        public ivec4 qtpq {
            get {
                return new ivec4(this.v3, this.v1, this.v2, this.v3);
            }
        }
        
        public ivec4 wywx {
            get {
                return new ivec4(this.v3, this.v1, this.v3, this.v0);
            }
        }
        
        public ivec4 agar {
            get {
                return new ivec4(this.v3, this.v1, this.v3, this.v0);
            }
        }
        
        public ivec4 qtqs {
            get {
                return new ivec4(this.v3, this.v1, this.v3, this.v0);
            }
        }
        
        public ivec4 wywy {
            get {
                return new ivec4(this.v3, this.v1, this.v3, this.v1);
            }
        }
        
        public ivec4 agag {
            get {
                return new ivec4(this.v3, this.v1, this.v3, this.v1);
            }
        }
        
        public ivec4 qtqt {
            get {
                return new ivec4(this.v3, this.v1, this.v3, this.v1);
            }
        }
        
        public ivec4 wywz {
            get {
                return new ivec4(this.v3, this.v1, this.v3, this.v2);
            }
        }
        
        public ivec4 agab {
            get {
                return new ivec4(this.v3, this.v1, this.v3, this.v2);
            }
        }
        
        public ivec4 qtqp {
            get {
                return new ivec4(this.v3, this.v1, this.v3, this.v2);
            }
        }
        
        public ivec4 wyww {
            get {
                return new ivec4(this.v3, this.v1, this.v3, this.v3);
            }
        }
        
        public ivec4 agaa {
            get {
                return new ivec4(this.v3, this.v1, this.v3, this.v3);
            }
        }
        
        public ivec4 qtqq {
            get {
                return new ivec4(this.v3, this.v1, this.v3, this.v3);
            }
        }
        
        public ivec4 wzxx {
            get {
                return new ivec4(this.v3, this.v2, this.v0, this.v0);
            }
        }
        
        public ivec4 abrr {
            get {
                return new ivec4(this.v3, this.v2, this.v0, this.v0);
            }
        }
        
        public ivec4 qpss {
            get {
                return new ivec4(this.v3, this.v2, this.v0, this.v0);
            }
        }
        
        public ivec4 wzxy {
            get {
                return new ivec4(this.v3, this.v2, this.v0, this.v1);
            }
            set {
                this.v3 = value.v0;
                this.v2 = value.v1;
                this.v0 = value.v2;
                this.v1 = value.v3;
            }
        }
        
        public ivec4 abrg {
            get {
                return new ivec4(this.v3, this.v2, this.v0, this.v1);
            }
            set {
                this.v3 = value.v0;
                this.v2 = value.v1;
                this.v0 = value.v2;
                this.v1 = value.v3;
            }
        }
        
        public ivec4 qpst {
            get {
                return new ivec4(this.v3, this.v2, this.v0, this.v1);
            }
            set {
                this.v3 = value.v0;
                this.v2 = value.v1;
                this.v0 = value.v2;
                this.v1 = value.v3;
            }
        }
        
        public ivec4 wzxz {
            get {
                return new ivec4(this.v3, this.v2, this.v0, this.v2);
            }
        }
        
        public ivec4 abrb {
            get {
                return new ivec4(this.v3, this.v2, this.v0, this.v2);
            }
        }
        
        public ivec4 qpsp {
            get {
                return new ivec4(this.v3, this.v2, this.v0, this.v2);
            }
        }
        
        public ivec4 wzxw {
            get {
                return new ivec4(this.v3, this.v2, this.v0, this.v3);
            }
        }
        
        public ivec4 abra {
            get {
                return new ivec4(this.v3, this.v2, this.v0, this.v3);
            }
        }
        
        public ivec4 qpsq {
            get {
                return new ivec4(this.v3, this.v2, this.v0, this.v3);
            }
        }
        
        public ivec4 wzyx {
            get {
                return new ivec4(this.v3, this.v2, this.v1, this.v0);
            }
            set {
                this.v3 = value.v0;
                this.v2 = value.v1;
                this.v1 = value.v2;
                this.v0 = value.v3;
            }
        }
        
        public ivec4 abgr {
            get {
                return new ivec4(this.v3, this.v2, this.v1, this.v0);
            }
            set {
                this.v3 = value.v0;
                this.v2 = value.v1;
                this.v1 = value.v2;
                this.v0 = value.v3;
            }
        }
        
        public ivec4 qpts {
            get {
                return new ivec4(this.v3, this.v2, this.v1, this.v0);
            }
            set {
                this.v3 = value.v0;
                this.v2 = value.v1;
                this.v1 = value.v2;
                this.v0 = value.v3;
            }
        }
        
        public ivec4 wzyy {
            get {
                return new ivec4(this.v3, this.v2, this.v1, this.v1);
            }
        }
        
        public ivec4 abgg {
            get {
                return new ivec4(this.v3, this.v2, this.v1, this.v1);
            }
        }
        
        public ivec4 qptt {
            get {
                return new ivec4(this.v3, this.v2, this.v1, this.v1);
            }
        }
        
        public ivec4 wzyz {
            get {
                return new ivec4(this.v3, this.v2, this.v1, this.v2);
            }
        }
        
        public ivec4 abgb {
            get {
                return new ivec4(this.v3, this.v2, this.v1, this.v2);
            }
        }
        
        public ivec4 qptp {
            get {
                return new ivec4(this.v3, this.v2, this.v1, this.v2);
            }
        }
        
        public ivec4 wzyw {
            get {
                return new ivec4(this.v3, this.v2, this.v1, this.v3);
            }
        }
        
        public ivec4 abga {
            get {
                return new ivec4(this.v3, this.v2, this.v1, this.v3);
            }
        }
        
        public ivec4 qptq {
            get {
                return new ivec4(this.v3, this.v2, this.v1, this.v3);
            }
        }
        
        public ivec4 wzzx {
            get {
                return new ivec4(this.v3, this.v2, this.v2, this.v0);
            }
        }
        
        public ivec4 abbr {
            get {
                return new ivec4(this.v3, this.v2, this.v2, this.v0);
            }
        }
        
        public ivec4 qpps {
            get {
                return new ivec4(this.v3, this.v2, this.v2, this.v0);
            }
        }
        
        public ivec4 wzzy {
            get {
                return new ivec4(this.v3, this.v2, this.v2, this.v1);
            }
        }
        
        public ivec4 abbg {
            get {
                return new ivec4(this.v3, this.v2, this.v2, this.v1);
            }
        }
        
        public ivec4 qppt {
            get {
                return new ivec4(this.v3, this.v2, this.v2, this.v1);
            }
        }
        
        public ivec4 wzzz {
            get {
                return new ivec4(this.v3, this.v2, this.v2, this.v2);
            }
        }
        
        public ivec4 abbb {
            get {
                return new ivec4(this.v3, this.v2, this.v2, this.v2);
            }
        }
        
        public ivec4 qppp {
            get {
                return new ivec4(this.v3, this.v2, this.v2, this.v2);
            }
        }
        
        public ivec4 wzzw {
            get {
                return new ivec4(this.v3, this.v2, this.v2, this.v3);
            }
        }
        
        public ivec4 abba {
            get {
                return new ivec4(this.v3, this.v2, this.v2, this.v3);
            }
        }
        
        public ivec4 qppq {
            get {
                return new ivec4(this.v3, this.v2, this.v2, this.v3);
            }
        }
        
        public ivec4 wzwx {
            get {
                return new ivec4(this.v3, this.v2, this.v3, this.v0);
            }
        }
        
        public ivec4 abar {
            get {
                return new ivec4(this.v3, this.v2, this.v3, this.v0);
            }
        }
        
        public ivec4 qpqs {
            get {
                return new ivec4(this.v3, this.v2, this.v3, this.v0);
            }
        }
        
        public ivec4 wzwy {
            get {
                return new ivec4(this.v3, this.v2, this.v3, this.v1);
            }
        }
        
        public ivec4 abag {
            get {
                return new ivec4(this.v3, this.v2, this.v3, this.v1);
            }
        }
        
        public ivec4 qpqt {
            get {
                return new ivec4(this.v3, this.v2, this.v3, this.v1);
            }
        }
        
        public ivec4 wzwz {
            get {
                return new ivec4(this.v3, this.v2, this.v3, this.v2);
            }
        }
        
        public ivec4 abab {
            get {
                return new ivec4(this.v3, this.v2, this.v3, this.v2);
            }
        }
        
        public ivec4 qpqp {
            get {
                return new ivec4(this.v3, this.v2, this.v3, this.v2);
            }
        }
        
        public ivec4 wzww {
            get {
                return new ivec4(this.v3, this.v2, this.v3, this.v3);
            }
        }
        
        public ivec4 abaa {
            get {
                return new ivec4(this.v3, this.v2, this.v3, this.v3);
            }
        }
        
        public ivec4 qpqq {
            get {
                return new ivec4(this.v3, this.v2, this.v3, this.v3);
            }
        }
        
        public ivec4 wwxx {
            get {
                return new ivec4(this.v3, this.v3, this.v0, this.v0);
            }
        }
        
        public ivec4 aarr {
            get {
                return new ivec4(this.v3, this.v3, this.v0, this.v0);
            }
        }
        
        public ivec4 qqss {
            get {
                return new ivec4(this.v3, this.v3, this.v0, this.v0);
            }
        }
        
        public ivec4 wwxy {
            get {
                return new ivec4(this.v3, this.v3, this.v0, this.v1);
            }
        }
        
        public ivec4 aarg {
            get {
                return new ivec4(this.v3, this.v3, this.v0, this.v1);
            }
        }
        
        public ivec4 qqst {
            get {
                return new ivec4(this.v3, this.v3, this.v0, this.v1);
            }
        }
        
        public ivec4 wwxz {
            get {
                return new ivec4(this.v3, this.v3, this.v0, this.v2);
            }
        }
        
        public ivec4 aarb {
            get {
                return new ivec4(this.v3, this.v3, this.v0, this.v2);
            }
        }
        
        public ivec4 qqsp {
            get {
                return new ivec4(this.v3, this.v3, this.v0, this.v2);
            }
        }
        
        public ivec4 wwxw {
            get {
                return new ivec4(this.v3, this.v3, this.v0, this.v3);
            }
        }
        
        public ivec4 aara {
            get {
                return new ivec4(this.v3, this.v3, this.v0, this.v3);
            }
        }
        
        public ivec4 qqsq {
            get {
                return new ivec4(this.v3, this.v3, this.v0, this.v3);
            }
        }
        
        public ivec4 wwyx {
            get {
                return new ivec4(this.v3, this.v3, this.v1, this.v0);
            }
        }
        
        public ivec4 aagr {
            get {
                return new ivec4(this.v3, this.v3, this.v1, this.v0);
            }
        }
        
        public ivec4 qqts {
            get {
                return new ivec4(this.v3, this.v3, this.v1, this.v0);
            }
        }
        
        public ivec4 wwyy {
            get {
                return new ivec4(this.v3, this.v3, this.v1, this.v1);
            }
        }
        
        public ivec4 aagg {
            get {
                return new ivec4(this.v3, this.v3, this.v1, this.v1);
            }
        }
        
        public ivec4 qqtt {
            get {
                return new ivec4(this.v3, this.v3, this.v1, this.v1);
            }
        }
        
        public ivec4 wwyz {
            get {
                return new ivec4(this.v3, this.v3, this.v1, this.v2);
            }
        }
        
        public ivec4 aagb {
            get {
                return new ivec4(this.v3, this.v3, this.v1, this.v2);
            }
        }
        
        public ivec4 qqtp {
            get {
                return new ivec4(this.v3, this.v3, this.v1, this.v2);
            }
        }
        
        public ivec4 wwyw {
            get {
                return new ivec4(this.v3, this.v3, this.v1, this.v3);
            }
        }
        
        public ivec4 aaga {
            get {
                return new ivec4(this.v3, this.v3, this.v1, this.v3);
            }
        }
        
        public ivec4 qqtq {
            get {
                return new ivec4(this.v3, this.v3, this.v1, this.v3);
            }
        }
        
        public ivec4 wwzx {
            get {
                return new ivec4(this.v3, this.v3, this.v2, this.v0);
            }
        }
        
        public ivec4 aabr {
            get {
                return new ivec4(this.v3, this.v3, this.v2, this.v0);
            }
        }
        
        public ivec4 qqps {
            get {
                return new ivec4(this.v3, this.v3, this.v2, this.v0);
            }
        }
        
        public ivec4 wwzy {
            get {
                return new ivec4(this.v3, this.v3, this.v2, this.v1);
            }
        }
        
        public ivec4 aabg {
            get {
                return new ivec4(this.v3, this.v3, this.v2, this.v1);
            }
        }
        
        public ivec4 qqpt {
            get {
                return new ivec4(this.v3, this.v3, this.v2, this.v1);
            }
        }
        
        public ivec4 wwzz {
            get {
                return new ivec4(this.v3, this.v3, this.v2, this.v2);
            }
        }
        
        public ivec4 aabb {
            get {
                return new ivec4(this.v3, this.v3, this.v2, this.v2);
            }
        }
        
        public ivec4 qqpp {
            get {
                return new ivec4(this.v3, this.v3, this.v2, this.v2);
            }
        }
        
        public ivec4 wwzw {
            get {
                return new ivec4(this.v3, this.v3, this.v2, this.v3);
            }
        }
        
        public ivec4 aaba {
            get {
                return new ivec4(this.v3, this.v3, this.v2, this.v3);
            }
        }
        
        public ivec4 qqpq {
            get {
                return new ivec4(this.v3, this.v3, this.v2, this.v3);
            }
        }
        
        public ivec4 wwwx {
            get {
                return new ivec4(this.v3, this.v3, this.v3, this.v0);
            }
        }
        
        public ivec4 aaar {
            get {
                return new ivec4(this.v3, this.v3, this.v3, this.v0);
            }
        }
        
        public ivec4 qqqs {
            get {
                return new ivec4(this.v3, this.v3, this.v3, this.v0);
            }
        }
        
        public ivec4 wwwy {
            get {
                return new ivec4(this.v3, this.v3, this.v3, this.v1);
            }
        }
        
        public ivec4 aaag {
            get {
                return new ivec4(this.v3, this.v3, this.v3, this.v1);
            }
        }
        
        public ivec4 qqqt {
            get {
                return new ivec4(this.v3, this.v3, this.v3, this.v1);
            }
        }
        
        public ivec4 wwwz {
            get {
                return new ivec4(this.v3, this.v3, this.v3, this.v2);
            }
        }
        
        public ivec4 aaab {
            get {
                return new ivec4(this.v3, this.v3, this.v3, this.v2);
            }
        }
        
        public ivec4 qqqp {
            get {
                return new ivec4(this.v3, this.v3, this.v3, this.v2);
            }
        }
        
        public ivec4 wwww {
            get {
                return new ivec4(this.v3, this.v3, this.v3, this.v3);
            }
        }
        
        public ivec4 aaaa {
            get {
                return new ivec4(this.v3, this.v3, this.v3, this.v3);
            }
        }
        
        public ivec4 qqqq {
            get {
                return new ivec4(this.v3, this.v3, this.v3, this.v3);
            }
        }
        
        public static ivec4 operator+(ivec4 v) {
            return new ivec4(+v.v0, +v.v1, +v.v2, +v.v3);
        }
        
        public static ivec4 operator-(ivec4 v) {
            return new ivec4(-v.v0, -v.v1, -v.v2, -v.v3);
        }
        
        public static ivec4 operator~(ivec4 v) {
            return new ivec4(~v.v0, ~v.v1, ~v.v2, ~v.v3);
        }
        
        public static ivec4 operator++(ivec4 v) {
            return new ivec4(++v.v0, ++v.v1, ++v.v2, ++v.v3);
        }
        
        public static ivec4 operator--(ivec4 v) {
            return new ivec4(--v.v0, --v.v1, --v.v2, --v.v3);
        }
        
        public static ivec4 operator+(int s, ivec4 v2) {
            return new ivec4(s + v2.v0, s + v2.v1, s + v2.v2, s + v2.v3);
        }
        
        public static ivec4 operator-(int s, ivec4 v2) {
            return new ivec4(s - v2.v0, s - v2.v1, s - v2.v2, s - v2.v3);
        }
        
        public static ivec4 operator*(int s, ivec4 v2) {
            return new ivec4(s * v2.v0, s * v2.v1, s * v2.v2, s * v2.v3);
        }
        
        public static ivec4 operator/(int s, ivec4 v2) {
            return new ivec4(s / v2.v0, s / v2.v1, s / v2.v2, s / v2.v3);
        }
        
        public static ivec4 operator%(int s, ivec4 v2) {
            return new ivec4(s % v2.v0, s % v2.v1, s % v2.v2, s % v2.v3);
        }
        
        public static ivec4 operator&(int s, ivec4 v2) {
            return new ivec4(s & v2.v0, s & v2.v1, s & v2.v2, s & v2.v3);
        }
        
        public static ivec4 operator|(int s, ivec4 v2) {
            return new ivec4(s | v2.v0, s | v2.v1, s | v2.v2, s | v2.v3);
        }
        
        public static ivec4 operator^(int s, ivec4 v2) {
            return new ivec4(s ^ v2.v0, s ^ v2.v1, s ^ v2.v2, s ^ v2.v3);
        }
        
        public static ivec4 operator+(ivec4 v1, int s) {
            return new ivec4(v1.v0 + s, v1.v1 + s, v1.v2 + s, v1.v3 + s);
        }
        
        public static ivec4 operator-(ivec4 v1, int s) {
            return new ivec4(v1.v0 - s, v1.v1 - s, v1.v2 - s, v1.v3 - s);
        }
        
        public static ivec4 operator*(ivec4 v1, int s) {
            return new ivec4(v1.v0 * s, v1.v1 * s, v1.v2 * s, v1.v3 * s);
        }
        
        public static ivec4 operator/(ivec4 v1, int s) {
            return new ivec4(v1.v0 / s, v1.v1 / s, v1.v2 / s, v1.v3 / s);
        }
        
        public static ivec4 operator%(ivec4 v1, int s) {
            return new ivec4(v1.v0 % s, v1.v1 % s, v1.v2 % s, v1.v3 % s);
        }
        
        public static ivec4 operator&(ivec4 v1, int s) {
            return new ivec4(v1.v0 & s, v1.v1 & s, v1.v2 & s, v1.v3 & s);
        }
        
        public static ivec4 operator|(ivec4 v1, int s) {
            return new ivec4(v1.v0 | s, v1.v1 | s, v1.v2 | s, v1.v3 | s);
        }
        
        public static ivec4 operator^(ivec4 v1, int s) {
            return new ivec4(v1.v0 ^ s, v1.v1 ^ s, v1.v2 ^ s, v1.v3 ^ s);
        }
        
        public static ivec4 operator+(ivec4 v1, ivec4 v2) {
            return new ivec4(v1.v0 + v2.v0, v1.v1 + v2.v1, v1.v2 + v2.v2, v1.v3 + v2.v3);
        }
        
        public static ivec4 operator-(ivec4 v1, ivec4 v2) {
            return new ivec4(v1.v0 - v2.v0, v1.v1 - v2.v1, v1.v2 - v2.v2, v1.v3 - v2.v3);
        }
        
        public static ivec4 operator*(ivec4 v1, ivec4 v2) {
            return new ivec4(v1.v0 * v2.v0, v1.v1 * v2.v1, v1.v2 * v2.v2, v1.v3 * v2.v3);
        }
        
        public static ivec4 operator/(ivec4 v1, ivec4 v2) {
            return new ivec4(v1.v0 / v2.v0, v1.v1 / v2.v1, v1.v2 / v2.v2, v1.v3 / v2.v3);
        }
        
        public static ivec4 operator%(ivec4 v1, ivec4 v2) {
            return new ivec4(v1.v0 % v2.v0, v1.v1 % v2.v1, v1.v2 % v2.v2, v1.v3 % v2.v3);
        }
        
        public static ivec4 operator&(ivec4 v1, ivec4 v2) {
            return new ivec4(v1.v0 & v2.v0, v1.v1 & v2.v1, v1.v2 & v2.v2, v1.v3 & v2.v3);
        }
        
        public static ivec4 operator|(ivec4 v1, ivec4 v2) {
            return new ivec4(v1.v0 | v2.v0, v1.v1 | v2.v1, v1.v2 | v2.v2, v1.v3 | v2.v3);
        }
        
        public static ivec4 operator^(ivec4 v1, ivec4 v2) {
            return new ivec4(v1.v0 ^ v2.v0, v1.v1 ^ v2.v1, v1.v2 ^ v2.v2, v1.v3 ^ v2.v3);
        }
        
        public static bvec4 operator==(ivec4 v1, ivec4 v2) {
            return new bvec4(v1.v0 == v2.v0, v1.v1 == v2.v1, v1.v2 == v2.v2, v1.v3 == v2.v3);
        }
        
        public static bvec4 operator>(ivec4 v1, ivec4 v2) {
            return new bvec4(v1.v0 > v2.v0, v1.v1 > v2.v1, v1.v2 > v2.v2, v1.v3 > v2.v3);
        }
        
        public static bvec4 operator>=(ivec4 v1, ivec4 v2) {
            return new bvec4(v1.v0 >= v2.v0, v1.v1 >= v2.v1, v1.v2 >= v2.v2, v1.v3 >= v2.v3);
        }
        
        public static bvec4 operator<(ivec4 v1, ivec4 v2) {
            return new bvec4(v1.v0 < v2.v0, v1.v1 < v2.v1, v1.v2 < v2.v2, v1.v3 < v2.v3);
        }
        
        public static bvec4 operator<=(ivec4 v1, ivec4 v2) {
            return new bvec4(v1.v0 <= v2.v0, v1.v1 <= v2.v1, v1.v2 <= v2.v2, v1.v3 <= v2.v3);
        }
        
        public static bvec4 operator!=(ivec4 v1, ivec4 v2) {
            return new bvec4(v1.v0 != v2.v0, v1.v1 != v2.v1, v1.v2 != v2.v2, v1.v3 != v2.v3);
        }
        
        public override bool Equals(object obj) {
            return (obj is ivec4) ? bmath.All(this == (ivec4)obj) : false;
        }
        
        public override int GetHashCode() {
            return this.v0.GetHashCode() ^ this.v1.GetHashCode() ^ this.v2.GetHashCode() ^ this.v3.GetHashCode();
        }
        
        public override string ToString() {
            return "ivec4(" + this.v0 + "," + this.v1 + "," + this.v2 + "," + this.v3 + ")";
        }
    }
}
