#region --- License ---
/*
Copyright 2010 Daniel Filonik

Permission is hereby granted, free of charge, to any person obtaining a copy of
this software and associated documentation files (the "Software"), to deal in
the Software without restriction, including without limitation the rights to
use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
of the Software, and to permit persons to whom the Software is furnished to do
so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all
copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
SOFTWARE.
*/
#endregion

namespace TKM {
    using System;
    using System.Runtime.InteropServices;
    using System.Runtime.Serialization;
    
    
    [DataContract()]
    [StructLayout(LayoutKind.Sequential)]
    public struct mat2 {
        
        [DataMember()]
        internal float m00;
        
        [DataMember()]
        internal float m01;
        
        [DataMember()]
        internal float m10;
        
        [DataMember()]
        internal float m11;
        
        public mat2(float s) {
            this.m00 = s;
            this.m01 = 0F;
            this.m10 = 0F;
            this.m11 = s;
        }
        
        public mat2(mat2 m00) {
            this.m00 = m00.m00;
            this.m01 = m00.m01;
            this.m10 = m00.m10;
            this.m11 = m00.m11;
        }
        
        public mat2(vec2 m00, vec2 m10) {
            this.m00 = m00.v0;
            this.m01 = m00.v1;
            this.m10 = m10.v0;
            this.m11 = m10.v1;
        }
        
        public mat2(float m00, float m01, float m10, float m11) {
            this.m00 = m00;
            this.m01 = m01;
            this.m10 = m10;
            this.m11 = m11;
        }
        
        public mat2(mat2x3 s) {
            this.m00 = s.m00;
            this.m01 = s.m01;
            this.m10 = s.m10;
            this.m11 = s.m11;
        }
        
        public mat2(mat2x4 s) {
            this.m00 = s.m00;
            this.m01 = s.m01;
            this.m10 = s.m10;
            this.m11 = s.m11;
        }
        
        public mat2(mat3x2 s) {
            this.m00 = s.m00;
            this.m01 = s.m01;
            this.m10 = s.m10;
            this.m11 = s.m11;
        }
        
        public mat2(mat3 s) {
            this.m00 = s.m00;
            this.m01 = s.m01;
            this.m10 = s.m10;
            this.m11 = s.m11;
        }
        
        public mat2(mat3x4 s) {
            this.m00 = s.m00;
            this.m01 = s.m01;
            this.m10 = s.m10;
            this.m11 = s.m11;
        }
        
        public mat2(mat4x2 s) {
            this.m00 = s.m00;
            this.m01 = s.m01;
            this.m10 = s.m10;
            this.m11 = s.m11;
        }
        
        public mat2(mat4x3 s) {
            this.m00 = s.m00;
            this.m01 = s.m01;
            this.m10 = s.m10;
            this.m11 = s.m11;
        }
        
        public mat2(mat4 s) {
            this.m00 = s.m00;
            this.m01 = s.m01;
            this.m10 = s.m10;
            this.m11 = s.m11;
        }
        
        public float this[int i0, int i1] {
            get {
                if (((i0 < 0) 
                            || (1 < i0))) {
                    throw new System.IndexOutOfRangeException("i0");
                }
                if (((i1 < 0) 
                            || (1 < i1))) {
                    throw new System.IndexOutOfRangeException("i1");
                }
                
                unsafe
                {
                    fixed (Single* v_ptr = &this.m00)
                    {
                        return *(v_ptr + (i1 + i0 * 2));
                    }
                };
            }
            set {
                if (((i0 < 0) 
                            || (1 < i0))) {
                    throw new System.IndexOutOfRangeException("i0");
                }
                if (((i1 < 0) 
                            || (1 < i1))) {
                    throw new System.IndexOutOfRangeException("i1");
                }
                
                unsafe
                {
                    fixed (Single* v_ptr = &this.m00)
                    {
                        *(v_ptr  + (i1 + i0 * 2)) = value;
                    }
                };
            }
        }
        
        public vec2 this[int i0] {
            get {
                return new vec2(this[i0, 0], this[i0, 1]);
            }
            set {
                this[i0, 0] = value[0];
                this[i0, 1] = value[1];
            }
        }
        
        public static mat2 operator+(mat2 v) {
            return new mat2(+v.m00, +v.m01, +v.m10, +v.m11);
        }
        
        public static mat2 operator-(mat2 v) {
            return new mat2(-v.m00, -v.m01, -v.m10, -v.m11);
        }
        
        public static mat2 operator++(mat2 v) {
            return new mat2(++v.m00, ++v.m01, ++v.m10, ++v.m11);
        }
        
        public static mat2 operator--(mat2 v) {
            return new mat2(--v.m00, --v.m01, --v.m10, --v.m11);
        }
        
        public static mat2 operator+(float s, mat2 v2) {
            return new mat2(s + v2.m00, s + v2.m01, s + v2.m10, s + v2.m11);
        }
        
        public static mat2 operator-(float s, mat2 v2) {
            return new mat2(s - v2.m00, s - v2.m01, s - v2.m10, s - v2.m11);
        }
        
        public static mat2 operator*(float s, mat2 v2) {
            return new mat2(s * v2.m00, s * v2.m01, s * v2.m10, s * v2.m11);
        }
        
        public static mat2 operator/(float s, mat2 v2) {
            return new mat2(s / v2.m00, s / v2.m01, s / v2.m10, s / v2.m11);
        }
        
        public static mat2 operator%(float s, mat2 v2) {
            return new mat2(s % v2.m00, s % v2.m01, s % v2.m10, s % v2.m11);
        }
        
        public static mat2 operator+(mat2 v1, float s) {
            return new mat2(v1.m00 + s, v1.m01 + s, v1.m10 + s, v1.m11 + s);
        }
        
        public static mat2 operator-(mat2 v1, float s) {
            return new mat2(v1.m00 - s, v1.m01 - s, v1.m10 - s, v1.m11 - s);
        }
        
        public static mat2 operator*(mat2 v1, float s) {
            return new mat2(v1.m00 * s, v1.m01 * s, v1.m10 * s, v1.m11 * s);
        }
        
        public static mat2 operator/(mat2 v1, float s) {
            return new mat2(v1.m00 / s, v1.m01 / s, v1.m10 / s, v1.m11 / s);
        }
        
        public static mat2 operator%(mat2 v1, float s) {
            return new mat2(v1.m00 % s, v1.m01 % s, v1.m10 % s, v1.m11 % s);
        }
        
        public static mat2 operator+(mat2 v1, mat2 v2) {
            return new mat2(v1.m00 + v2.m00, v1.m01 + v2.m01, v1.m10 + v2.m10, v1.m11 + v2.m11);
        }
        
        public static mat2 operator-(mat2 v1, mat2 v2) {
            return new mat2(v1.m00 - v2.m00, v1.m01 - v2.m01, v1.m10 - v2.m10, v1.m11 - v2.m11);
        }
        
        public static mat2 operator/(mat2 v1, mat2 v2) {
            return new mat2(v1.m00 / v2.m00, v1.m01 / v2.m01, v1.m10 / v2.m10, v1.m11 / v2.m11);
        }
        
        public static mat2 operator%(mat2 v1, mat2 v2) {
            return new mat2(v1.m00 % v2.m00, v1.m01 % v2.m01, v1.m10 % v2.m10, v1.m11 % v2.m11);
        }
        
        public static bmat2 operator==(mat2 v1, mat2 v2) {
            return new bmat2(v1.m00 == v2.m00, v1.m01 == v2.m01, v1.m10 == v2.m10, v1.m11 == v2.m11);
        }
        
        public static bmat2 operator>(mat2 v1, mat2 v2) {
            return new bmat2(v1.m00 > v2.m00, v1.m01 > v2.m01, v1.m10 > v2.m10, v1.m11 > v2.m11);
        }
        
        public static bmat2 operator>=(mat2 v1, mat2 v2) {
            return new bmat2(v1.m00 >= v2.m00, v1.m01 >= v2.m01, v1.m10 >= v2.m10, v1.m11 >= v2.m11);
        }
        
        public static bmat2 operator<(mat2 v1, mat2 v2) {
            return new bmat2(v1.m00 < v2.m00, v1.m01 < v2.m01, v1.m10 < v2.m10, v1.m11 < v2.m11);
        }
        
        public static bmat2 operator<=(mat2 v1, mat2 v2) {
            return new bmat2(v1.m00 <= v2.m00, v1.m01 <= v2.m01, v1.m10 <= v2.m10, v1.m11 <= v2.m11);
        }
        
        public static bmat2 operator!=(mat2 v1, mat2 v2) {
            return new bmat2(v1.m00 != v2.m00, v1.m01 != v2.m01, v1.m10 != v2.m10, v1.m11 != v2.m11);
        }
        
        public static vec2 operator*(vec2 v0, mat2 m1) {
            return math.InnerProduct1(ref v0, ref m1);
        }
        
        public static vec2 operator*(mat2 m0, vec2 v1) {
            return math.InnerProduct1(ref m0, ref v1);
        }
        
        public static mat2 operator*(mat2 m0, mat2 m1) {
            return math.InnerProduct1(ref m0, ref m1);
        }
        
        public static mat2x3 operator*(mat2 m0, mat2x3 m1) {
            return math.InnerProduct1(ref m0, ref m1);
        }
        
        public static mat2x4 operator*(mat2 m0, mat2x4 m1) {
            return math.InnerProduct1(ref m0, ref m1);
        }
        
        public override bool Equals(object obj) {
            return (obj is mat2) ? bmath.All(this == (mat2)obj) : false;
        }
        
        public override int GetHashCode() {
            return this.m00.GetHashCode() ^ this.m01.GetHashCode() ^ this.m10.GetHashCode() ^ this.m11.GetHashCode();
        }
        
        public override string ToString() {
            return "mat2(" + this.m00 + "," + this.m01 + "," + this.m10 + "," + this.m11 + ")";
        }
    }
    
    [DataContract()]
    [StructLayout(LayoutKind.Sequential)]
    public struct mat2x3 {
        
        [DataMember()]
        internal float m00;
        
        [DataMember()]
        internal float m01;
        
        [DataMember()]
        internal float m02;
        
        [DataMember()]
        internal float m10;
        
        [DataMember()]
        internal float m11;
        
        [DataMember()]
        internal float m12;
        
        public mat2x3(float s) {
            this.m00 = s;
            this.m01 = 0F;
            this.m02 = 0F;
            this.m10 = 0F;
            this.m11 = s;
            this.m12 = 0F;
        }
        
        public mat2x3(mat2x3 m00) {
            this.m00 = m00.m00;
            this.m01 = m00.m01;
            this.m02 = m00.m02;
            this.m10 = m00.m10;
            this.m11 = m00.m11;
            this.m12 = m00.m12;
        }
        
        public mat2x3(vec3 m00, vec3 m10) {
            this.m00 = m00.v0;
            this.m01 = m00.v1;
            this.m02 = m00.v2;
            this.m10 = m10.v0;
            this.m11 = m10.v1;
            this.m12 = m10.v2;
        }
        
        public mat2x3(vec2 m00, float m01, vec2 m10, float m11) {
            this.m00 = m00.v0;
            this.m01 = m00.v1;
            this.m02 = m01;
            this.m10 = m10.v0;
            this.m11 = m10.v1;
            this.m12 = m11;
        }
        
        public mat2x3(float m00, vec2 m01, float m10, vec2 m11) {
            this.m00 = m00;
            this.m01 = m01.v0;
            this.m02 = m01.v1;
            this.m10 = m10;
            this.m11 = m11.v0;
            this.m12 = m11.v1;
        }
        
        public mat2x3(float m00, float m01, float m02, float m10, float m11, float m12) {
            this.m00 = m00;
            this.m01 = m01;
            this.m02 = m02;
            this.m10 = m10;
            this.m11 = m11;
            this.m12 = m12;
        }
        
        public mat2x3(mat2 s) {
            this.m00 = s.m00;
            this.m01 = s.m01;
            this.m02 = 0F;
            this.m10 = s.m10;
            this.m11 = s.m11;
            this.m12 = 0F;
        }
        
        public mat2x3(mat2x4 s) {
            this.m00 = s.m00;
            this.m01 = s.m01;
            this.m02 = s.m02;
            this.m10 = s.m10;
            this.m11 = s.m11;
            this.m12 = s.m12;
        }
        
        public mat2x3(mat3x2 s) {
            this.m00 = s.m00;
            this.m01 = s.m01;
            this.m02 = 0F;
            this.m10 = s.m10;
            this.m11 = s.m11;
            this.m12 = 0F;
        }
        
        public mat2x3(mat3 s) {
            this.m00 = s.m00;
            this.m01 = s.m01;
            this.m02 = s.m02;
            this.m10 = s.m10;
            this.m11 = s.m11;
            this.m12 = s.m12;
        }
        
        public mat2x3(mat3x4 s) {
            this.m00 = s.m00;
            this.m01 = s.m01;
            this.m02 = s.m02;
            this.m10 = s.m10;
            this.m11 = s.m11;
            this.m12 = s.m12;
        }
        
        public mat2x3(mat4x2 s) {
            this.m00 = s.m00;
            this.m01 = s.m01;
            this.m02 = 0F;
            this.m10 = s.m10;
            this.m11 = s.m11;
            this.m12 = 0F;
        }
        
        public mat2x3(mat4x3 s) {
            this.m00 = s.m00;
            this.m01 = s.m01;
            this.m02 = s.m02;
            this.m10 = s.m10;
            this.m11 = s.m11;
            this.m12 = s.m12;
        }
        
        public mat2x3(mat4 s) {
            this.m00 = s.m00;
            this.m01 = s.m01;
            this.m02 = s.m02;
            this.m10 = s.m10;
            this.m11 = s.m11;
            this.m12 = s.m12;
        }
        
        public float this[int i0, int i1] {
            get {
                if (((i0 < 0) 
                            || (1 < i0))) {
                    throw new System.IndexOutOfRangeException("i0");
                }
                if (((i1 < 0) 
                            || (2 < i1))) {
                    throw new System.IndexOutOfRangeException("i1");
                }
                
                unsafe
                {
                    fixed (Single* v_ptr = &this.m00)
                    {
                        return *(v_ptr + (i1 + i0 * 3));
                    }
                };
            }
            set {
                if (((i0 < 0) 
                            || (1 < i0))) {
                    throw new System.IndexOutOfRangeException("i0");
                }
                if (((i1 < 0) 
                            || (2 < i1))) {
                    throw new System.IndexOutOfRangeException("i1");
                }
                
                unsafe
                {
                    fixed (Single* v_ptr = &this.m00)
                    {
                        *(v_ptr  + (i1 + i0 * 3)) = value;
                    }
                };
            }
        }
        
        public vec3 this[int i0] {
            get {
                return new vec3(this[i0, 0], this[i0, 1], this[i0, 2]);
            }
            set {
                this[i0, 0] = value[0];
                this[i0, 1] = value[1];
                this[i0, 2] = value[2];
            }
        }
        
        public static mat2x3 operator+(mat2x3 v) {
            return new mat2x3(+v.m00, +v.m01, +v.m02, +v.m10, +v.m11, +v.m12);
        }
        
        public static mat2x3 operator-(mat2x3 v) {
            return new mat2x3(-v.m00, -v.m01, -v.m02, -v.m10, -v.m11, -v.m12);
        }
        
        public static mat2x3 operator++(mat2x3 v) {
            return new mat2x3(++v.m00, ++v.m01, ++v.m02, ++v.m10, ++v.m11, ++v.m12);
        }
        
        public static mat2x3 operator--(mat2x3 v) {
            return new mat2x3(--v.m00, --v.m01, --v.m02, --v.m10, --v.m11, --v.m12);
        }
        
        public static mat2x3 operator+(float s, mat2x3 v2) {
            return new mat2x3(s + v2.m00, s + v2.m01, s + v2.m02, s + v2.m10, s + v2.m11, s + v2.m12);
        }
        
        public static mat2x3 operator-(float s, mat2x3 v2) {
            return new mat2x3(s - v2.m00, s - v2.m01, s - v2.m02, s - v2.m10, s - v2.m11, s - v2.m12);
        }
        
        public static mat2x3 operator*(float s, mat2x3 v2) {
            return new mat2x3(s * v2.m00, s * v2.m01, s * v2.m02, s * v2.m10, s * v2.m11, s * v2.m12);
        }
        
        public static mat2x3 operator/(float s, mat2x3 v2) {
            return new mat2x3(s / v2.m00, s / v2.m01, s / v2.m02, s / v2.m10, s / v2.m11, s / v2.m12);
        }
        
        public static mat2x3 operator%(float s, mat2x3 v2) {
            return new mat2x3(s % v2.m00, s % v2.m01, s % v2.m02, s % v2.m10, s % v2.m11, s % v2.m12);
        }
        
        public static mat2x3 operator+(mat2x3 v1, float s) {
            return new mat2x3(v1.m00 + s, v1.m01 + s, v1.m02 + s, v1.m10 + s, v1.m11 + s, v1.m12 + s);
        }
        
        public static mat2x3 operator-(mat2x3 v1, float s) {
            return new mat2x3(v1.m00 - s, v1.m01 - s, v1.m02 - s, v1.m10 - s, v1.m11 - s, v1.m12 - s);
        }
        
        public static mat2x3 operator*(mat2x3 v1, float s) {
            return new mat2x3(v1.m00 * s, v1.m01 * s, v1.m02 * s, v1.m10 * s, v1.m11 * s, v1.m12 * s);
        }
        
        public static mat2x3 operator/(mat2x3 v1, float s) {
            return new mat2x3(v1.m00 / s, v1.m01 / s, v1.m02 / s, v1.m10 / s, v1.m11 / s, v1.m12 / s);
        }
        
        public static mat2x3 operator%(mat2x3 v1, float s) {
            return new mat2x3(v1.m00 % s, v1.m01 % s, v1.m02 % s, v1.m10 % s, v1.m11 % s, v1.m12 % s);
        }
        
        public static mat2x3 operator+(mat2x3 v1, mat2x3 v2) {
            return new mat2x3(v1.m00 + v2.m00, v1.m01 + v2.m01, v1.m02 + v2.m02, v1.m10 + v2.m10, v1.m11 + v2.m11, v1.m12 + v2.m12);
        }
        
        public static mat2x3 operator-(mat2x3 v1, mat2x3 v2) {
            return new mat2x3(v1.m00 - v2.m00, v1.m01 - v2.m01, v1.m02 - v2.m02, v1.m10 - v2.m10, v1.m11 - v2.m11, v1.m12 - v2.m12);
        }
        
        public static mat2x3 operator/(mat2x3 v1, mat2x3 v2) {
            return new mat2x3(v1.m00 / v2.m00, v1.m01 / v2.m01, v1.m02 / v2.m02, v1.m10 / v2.m10, v1.m11 / v2.m11, v1.m12 / v2.m12);
        }
        
        public static mat2x3 operator%(mat2x3 v1, mat2x3 v2) {
            return new mat2x3(v1.m00 % v2.m00, v1.m01 % v2.m01, v1.m02 % v2.m02, v1.m10 % v2.m10, v1.m11 % v2.m11, v1.m12 % v2.m12);
        }
        
        public static bmat2x3 operator==(mat2x3 v1, mat2x3 v2) {
            return new bmat2x3(v1.m00 == v2.m00, v1.m01 == v2.m01, v1.m02 == v2.m02, v1.m10 == v2.m10, v1.m11 == v2.m11, v1.m12 == v2.m12);
        }
        
        public static bmat2x3 operator>(mat2x3 v1, mat2x3 v2) {
            return new bmat2x3(v1.m00 > v2.m00, v1.m01 > v2.m01, v1.m02 > v2.m02, v1.m10 > v2.m10, v1.m11 > v2.m11, v1.m12 > v2.m12);
        }
        
        public static bmat2x3 operator>=(mat2x3 v1, mat2x3 v2) {
            return new bmat2x3(v1.m00 >= v2.m00, v1.m01 >= v2.m01, v1.m02 >= v2.m02, v1.m10 >= v2.m10, v1.m11 >= v2.m11, v1.m12 >= v2.m12);
        }
        
        public static bmat2x3 operator<(mat2x3 v1, mat2x3 v2) {
            return new bmat2x3(v1.m00 < v2.m00, v1.m01 < v2.m01, v1.m02 < v2.m02, v1.m10 < v2.m10, v1.m11 < v2.m11, v1.m12 < v2.m12);
        }
        
        public static bmat2x3 operator<=(mat2x3 v1, mat2x3 v2) {
            return new bmat2x3(v1.m00 <= v2.m00, v1.m01 <= v2.m01, v1.m02 <= v2.m02, v1.m10 <= v2.m10, v1.m11 <= v2.m11, v1.m12 <= v2.m12);
        }
        
        public static bmat2x3 operator!=(mat2x3 v1, mat2x3 v2) {
            return new bmat2x3(v1.m00 != v2.m00, v1.m01 != v2.m01, v1.m02 != v2.m02, v1.m10 != v2.m10, v1.m11 != v2.m11, v1.m12 != v2.m12);
        }
        
        public static vec3 operator*(vec2 v0, mat2x3 m1) {
            return math.InnerProduct1(ref v0, ref m1);
        }
        
        public static vec2 operator*(mat2x3 m0, vec3 v1) {
            return math.InnerProduct1(ref m0, ref v1);
        }
        
        public static mat2 operator*(mat2x3 m0, mat3x2 m1) {
            return math.InnerProduct1(ref m0, ref m1);
        }
        
        public static mat2x3 operator*(mat2x3 m0, mat3 m1) {
            return math.InnerProduct1(ref m0, ref m1);
        }
        
        public static mat2x4 operator*(mat2x3 m0, mat3x4 m1) {
            return math.InnerProduct1(ref m0, ref m1);
        }
        
        public override bool Equals(object obj) {
            return (obj is mat2x3) ? bmath.All(this == (mat2x3)obj) : false;
        }
        
        public override int GetHashCode() {
            return this.m00.GetHashCode() ^ this.m01.GetHashCode() ^ this.m02.GetHashCode() ^ this.m10.GetHashCode() ^ this.m11.GetHashCode() ^ this.m12.GetHashCode();
        }
        
        public override string ToString() {
            return "mat2x3(" + this.m00 + "," + this.m01 + "," + this.m02 + "," + this.m10 + "," + this.m11 + "," + this.m12 + ")";
        }
    }
    
    [DataContract()]
    [StructLayout(LayoutKind.Sequential)]
    public struct mat2x4 {
        
        [DataMember()]
        internal float m00;
        
        [DataMember()]
        internal float m01;
        
        [DataMember()]
        internal float m02;
        
        [DataMember()]
        internal float m03;
        
        [DataMember()]
        internal float m10;
        
        [DataMember()]
        internal float m11;
        
        [DataMember()]
        internal float m12;
        
        [DataMember()]
        internal float m13;
        
        public mat2x4(float s) {
            this.m00 = s;
            this.m01 = 0F;
            this.m02 = 0F;
            this.m03 = 0F;
            this.m10 = 0F;
            this.m11 = s;
            this.m12 = 0F;
            this.m13 = 0F;
        }
        
        public mat2x4(mat2x4 m00) {
            this.m00 = m00.m00;
            this.m01 = m00.m01;
            this.m02 = m00.m02;
            this.m03 = m00.m03;
            this.m10 = m00.m10;
            this.m11 = m00.m11;
            this.m12 = m00.m12;
            this.m13 = m00.m13;
        }
        
        public mat2x4(mat2 m00, mat2 m01) {
            this.m00 = m00.m00;
            this.m01 = m00.m01;
            this.m02 = m01.m00;
            this.m03 = m01.m01;
            this.m10 = m00.m10;
            this.m11 = m00.m11;
            this.m12 = m01.m10;
            this.m13 = m01.m11;
        }
        
        public mat2x4(vec4 m00, vec4 m10) {
            this.m00 = m00.v0;
            this.m01 = m00.v1;
            this.m02 = m00.v2;
            this.m03 = m00.v3;
            this.m10 = m10.v0;
            this.m11 = m10.v1;
            this.m12 = m10.v2;
            this.m13 = m10.v3;
        }
        
        public mat2x4(vec2 m00, vec2 m01, vec2 m10, vec2 m11) {
            this.m00 = m00.v0;
            this.m01 = m00.v1;
            this.m02 = m01.v0;
            this.m03 = m01.v1;
            this.m10 = m10.v0;
            this.m11 = m10.v1;
            this.m12 = m11.v0;
            this.m13 = m11.v1;
        }
        
        public mat2x4(vec3 m00, float m01, vec3 m10, float m11) {
            this.m00 = m00.v0;
            this.m01 = m00.v1;
            this.m02 = m00.v2;
            this.m03 = m01;
            this.m10 = m10.v0;
            this.m11 = m10.v1;
            this.m12 = m10.v2;
            this.m13 = m11;
        }
        
        public mat2x4(float m00, vec3 m01, float m10, vec3 m11) {
            this.m00 = m00;
            this.m01 = m01.v0;
            this.m02 = m01.v1;
            this.m03 = m01.v2;
            this.m10 = m10;
            this.m11 = m11.v0;
            this.m12 = m11.v1;
            this.m13 = m11.v2;
        }
        
        public mat2x4(vec2 m00, float m01, float m02, vec2 m10, float m11, float m12) {
            this.m00 = m00.v0;
            this.m01 = m00.v1;
            this.m02 = m01;
            this.m03 = m02;
            this.m10 = m10.v0;
            this.m11 = m10.v1;
            this.m12 = m11;
            this.m13 = m12;
        }
        
        public mat2x4(float m00, vec2 m01, float m02, float m10, vec2 m11, float m12) {
            this.m00 = m00;
            this.m01 = m01.v0;
            this.m02 = m01.v1;
            this.m03 = m02;
            this.m10 = m10;
            this.m11 = m11.v0;
            this.m12 = m11.v1;
            this.m13 = m12;
        }
        
        public mat2x4(float m00, float m01, vec2 m02, float m10, float m11, vec2 m12) {
            this.m00 = m00;
            this.m01 = m01;
            this.m02 = m02.v0;
            this.m03 = m02.v1;
            this.m10 = m10;
            this.m11 = m11;
            this.m12 = m12.v0;
            this.m13 = m12.v1;
        }
        
        public mat2x4(float m00, float m01, float m02, float m03, float m10, float m11, float m12, float m13) {
            this.m00 = m00;
            this.m01 = m01;
            this.m02 = m02;
            this.m03 = m03;
            this.m10 = m10;
            this.m11 = m11;
            this.m12 = m12;
            this.m13 = m13;
        }
        
        public mat2x4(mat2 s) {
            this.m00 = s.m00;
            this.m01 = s.m01;
            this.m02 = 0F;
            this.m03 = 0F;
            this.m10 = s.m10;
            this.m11 = s.m11;
            this.m12 = 0F;
            this.m13 = 0F;
        }
        
        public mat2x4(mat2x3 s) {
            this.m00 = s.m00;
            this.m01 = s.m01;
            this.m02 = s.m02;
            this.m03 = 0F;
            this.m10 = s.m10;
            this.m11 = s.m11;
            this.m12 = s.m12;
            this.m13 = 0F;
        }
        
        public mat2x4(mat3x2 s) {
            this.m00 = s.m00;
            this.m01 = s.m01;
            this.m02 = 0F;
            this.m03 = 0F;
            this.m10 = s.m10;
            this.m11 = s.m11;
            this.m12 = 0F;
            this.m13 = 0F;
        }
        
        public mat2x4(mat3 s) {
            this.m00 = s.m00;
            this.m01 = s.m01;
            this.m02 = s.m02;
            this.m03 = 0F;
            this.m10 = s.m10;
            this.m11 = s.m11;
            this.m12 = s.m12;
            this.m13 = 0F;
        }
        
        public mat2x4(mat3x4 s) {
            this.m00 = s.m00;
            this.m01 = s.m01;
            this.m02 = s.m02;
            this.m03 = s.m03;
            this.m10 = s.m10;
            this.m11 = s.m11;
            this.m12 = s.m12;
            this.m13 = s.m13;
        }
        
        public mat2x4(mat4x2 s) {
            this.m00 = s.m00;
            this.m01 = s.m01;
            this.m02 = 0F;
            this.m03 = 0F;
            this.m10 = s.m10;
            this.m11 = s.m11;
            this.m12 = 0F;
            this.m13 = 0F;
        }
        
        public mat2x4(mat4x3 s) {
            this.m00 = s.m00;
            this.m01 = s.m01;
            this.m02 = s.m02;
            this.m03 = 0F;
            this.m10 = s.m10;
            this.m11 = s.m11;
            this.m12 = s.m12;
            this.m13 = 0F;
        }
        
        public mat2x4(mat4 s) {
            this.m00 = s.m00;
            this.m01 = s.m01;
            this.m02 = s.m02;
            this.m03 = s.m03;
            this.m10 = s.m10;
            this.m11 = s.m11;
            this.m12 = s.m12;
            this.m13 = s.m13;
        }
        
        public float this[int i0, int i1] {
            get {
                if (((i0 < 0) 
                            || (1 < i0))) {
                    throw new System.IndexOutOfRangeException("i0");
                }
                if (((i1 < 0) 
                            || (3 < i1))) {
                    throw new System.IndexOutOfRangeException("i1");
                }
                
                unsafe
                {
                    fixed (Single* v_ptr = &this.m00)
                    {
                        return *(v_ptr + (i1 + i0 * 4));
                    }
                };
            }
            set {
                if (((i0 < 0) 
                            || (1 < i0))) {
                    throw new System.IndexOutOfRangeException("i0");
                }
                if (((i1 < 0) 
                            || (3 < i1))) {
                    throw new System.IndexOutOfRangeException("i1");
                }
                
                unsafe
                {
                    fixed (Single* v_ptr = &this.m00)
                    {
                        *(v_ptr  + (i1 + i0 * 4)) = value;
                    }
                };
            }
        }
        
        public vec4 this[int i0] {
            get {
                return new vec4(this[i0, 0], this[i0, 1], this[i0, 2], this[i0, 3]);
            }
            set {
                this[i0, 0] = value[0];
                this[i0, 1] = value[1];
                this[i0, 2] = value[2];
                this[i0, 3] = value[3];
            }
        }
        
        public static mat2x4 operator+(mat2x4 v) {
            return new mat2x4(+v.m00, +v.m01, +v.m02, +v.m03, +v.m10, +v.m11, +v.m12, +v.m13);
        }
        
        public static mat2x4 operator-(mat2x4 v) {
            return new mat2x4(-v.m00, -v.m01, -v.m02, -v.m03, -v.m10, -v.m11, -v.m12, -v.m13);
        }
        
        public static mat2x4 operator++(mat2x4 v) {
            return new mat2x4(++v.m00, ++v.m01, ++v.m02, ++v.m03, ++v.m10, ++v.m11, ++v.m12, ++v.m13);
        }
        
        public static mat2x4 operator--(mat2x4 v) {
            return new mat2x4(--v.m00, --v.m01, --v.m02, --v.m03, --v.m10, --v.m11, --v.m12, --v.m13);
        }
        
        public static mat2x4 operator+(float s, mat2x4 v2) {
            return new mat2x4(s + v2.m00, s + v2.m01, s + v2.m02, s + v2.m03, s + v2.m10, s + v2.m11, s + v2.m12, s + v2.m13);
        }
        
        public static mat2x4 operator-(float s, mat2x4 v2) {
            return new mat2x4(s - v2.m00, s - v2.m01, s - v2.m02, s - v2.m03, s - v2.m10, s - v2.m11, s - v2.m12, s - v2.m13);
        }
        
        public static mat2x4 operator*(float s, mat2x4 v2) {
            return new mat2x4(s * v2.m00, s * v2.m01, s * v2.m02, s * v2.m03, s * v2.m10, s * v2.m11, s * v2.m12, s * v2.m13);
        }
        
        public static mat2x4 operator/(float s, mat2x4 v2) {
            return new mat2x4(s / v2.m00, s / v2.m01, s / v2.m02, s / v2.m03, s / v2.m10, s / v2.m11, s / v2.m12, s / v2.m13);
        }
        
        public static mat2x4 operator%(float s, mat2x4 v2) {
            return new mat2x4(s % v2.m00, s % v2.m01, s % v2.m02, s % v2.m03, s % v2.m10, s % v2.m11, s % v2.m12, s % v2.m13);
        }
        
        public static mat2x4 operator+(mat2x4 v1, float s) {
            return new mat2x4(v1.m00 + s, v1.m01 + s, v1.m02 + s, v1.m03 + s, v1.m10 + s, v1.m11 + s, v1.m12 + s, v1.m13 + s);
        }
        
        public static mat2x4 operator-(mat2x4 v1, float s) {
            return new mat2x4(v1.m00 - s, v1.m01 - s, v1.m02 - s, v1.m03 - s, v1.m10 - s, v1.m11 - s, v1.m12 - s, v1.m13 - s);
        }
        
        public static mat2x4 operator*(mat2x4 v1, float s) {
            return new mat2x4(v1.m00 * s, v1.m01 * s, v1.m02 * s, v1.m03 * s, v1.m10 * s, v1.m11 * s, v1.m12 * s, v1.m13 * s);
        }
        
        public static mat2x4 operator/(mat2x4 v1, float s) {
            return new mat2x4(v1.m00 / s, v1.m01 / s, v1.m02 / s, v1.m03 / s, v1.m10 / s, v1.m11 / s, v1.m12 / s, v1.m13 / s);
        }
        
        public static mat2x4 operator%(mat2x4 v1, float s) {
            return new mat2x4(v1.m00 % s, v1.m01 % s, v1.m02 % s, v1.m03 % s, v1.m10 % s, v1.m11 % s, v1.m12 % s, v1.m13 % s);
        }
        
        public static mat2x4 operator+(mat2x4 v1, mat2x4 v2) {
            return new mat2x4(v1.m00 + v2.m00, v1.m01 + v2.m01, v1.m02 + v2.m02, v1.m03 + v2.m03, v1.m10 + v2.m10, v1.m11 + v2.m11, v1.m12 + v2.m12, v1.m13 + v2.m13);
        }
        
        public static mat2x4 operator-(mat2x4 v1, mat2x4 v2) {
            return new mat2x4(v1.m00 - v2.m00, v1.m01 - v2.m01, v1.m02 - v2.m02, v1.m03 - v2.m03, v1.m10 - v2.m10, v1.m11 - v2.m11, v1.m12 - v2.m12, v1.m13 - v2.m13);
        }
        
        public static mat2x4 operator/(mat2x4 v1, mat2x4 v2) {
            return new mat2x4(v1.m00 / v2.m00, v1.m01 / v2.m01, v1.m02 / v2.m02, v1.m03 / v2.m03, v1.m10 / v2.m10, v1.m11 / v2.m11, v1.m12 / v2.m12, v1.m13 / v2.m13);
        }
        
        public static mat2x4 operator%(mat2x4 v1, mat2x4 v2) {
            return new mat2x4(v1.m00 % v2.m00, v1.m01 % v2.m01, v1.m02 % v2.m02, v1.m03 % v2.m03, v1.m10 % v2.m10, v1.m11 % v2.m11, v1.m12 % v2.m12, v1.m13 % v2.m13);
        }
        
        public static bmat2x4 operator==(mat2x4 v1, mat2x4 v2) {
            return new bmat2x4(v1.m00 == v2.m00, v1.m01 == v2.m01, v1.m02 == v2.m02, v1.m03 == v2.m03, v1.m10 == v2.m10, v1.m11 == v2.m11, v1.m12 == v2.m12, v1.m13 == v2.m13);
        }
        
        public static bmat2x4 operator>(mat2x4 v1, mat2x4 v2) {
            return new bmat2x4(v1.m00 > v2.m00, v1.m01 > v2.m01, v1.m02 > v2.m02, v1.m03 > v2.m03, v1.m10 > v2.m10, v1.m11 > v2.m11, v1.m12 > v2.m12, v1.m13 > v2.m13);
        }
        
        public static bmat2x4 operator>=(mat2x4 v1, mat2x4 v2) {
            return new bmat2x4(v1.m00 >= v2.m00, v1.m01 >= v2.m01, v1.m02 >= v2.m02, v1.m03 >= v2.m03, v1.m10 >= v2.m10, v1.m11 >= v2.m11, v1.m12 >= v2.m12, v1.m13 >= v2.m13);
        }
        
        public static bmat2x4 operator<(mat2x4 v1, mat2x4 v2) {
            return new bmat2x4(v1.m00 < v2.m00, v1.m01 < v2.m01, v1.m02 < v2.m02, v1.m03 < v2.m03, v1.m10 < v2.m10, v1.m11 < v2.m11, v1.m12 < v2.m12, v1.m13 < v2.m13);
        }
        
        public static bmat2x4 operator<=(mat2x4 v1, mat2x4 v2) {
            return new bmat2x4(v1.m00 <= v2.m00, v1.m01 <= v2.m01, v1.m02 <= v2.m02, v1.m03 <= v2.m03, v1.m10 <= v2.m10, v1.m11 <= v2.m11, v1.m12 <= v2.m12, v1.m13 <= v2.m13);
        }
        
        public static bmat2x4 operator!=(mat2x4 v1, mat2x4 v2) {
            return new bmat2x4(v1.m00 != v2.m00, v1.m01 != v2.m01, v1.m02 != v2.m02, v1.m03 != v2.m03, v1.m10 != v2.m10, v1.m11 != v2.m11, v1.m12 != v2.m12, v1.m13 != v2.m13);
        }
        
        public static vec4 operator*(vec2 v0, mat2x4 m1) {
            return math.InnerProduct1(ref v0, ref m1);
        }
        
        public static vec2 operator*(mat2x4 m0, vec4 v1) {
            return math.InnerProduct1(ref m0, ref v1);
        }
        
        public static mat2 operator*(mat2x4 m0, mat4x2 m1) {
            return math.InnerProduct1(ref m0, ref m1);
        }
        
        public static mat2x3 operator*(mat2x4 m0, mat4x3 m1) {
            return math.InnerProduct1(ref m0, ref m1);
        }
        
        public static mat2x4 operator*(mat2x4 m0, mat4 m1) {
            return math.InnerProduct1(ref m0, ref m1);
        }
        
        public override bool Equals(object obj) {
            return (obj is mat2x4) ? bmath.All(this == (mat2x4)obj) : false;
        }
        
        public override int GetHashCode() {
            return this.m00.GetHashCode() ^ this.m01.GetHashCode() ^ this.m02.GetHashCode() ^ this.m03.GetHashCode() ^ this.m10.GetHashCode() ^ this.m11.GetHashCode() ^ this.m12.GetHashCode() ^ this.m13.GetHashCode();
        }
        
        public override string ToString() {
            return "mat2x4(" + this.m00 + "," + this.m01 + "," + this.m02 + "," + this.m03 + "," + this.m10 + "," + this.m11 + "," + this.m12 + "," + this.m13 + ")";
        }
    }
    
    [DataContract()]
    [StructLayout(LayoutKind.Sequential)]
    public struct mat3x2 {
        
        [DataMember()]
        internal float m00;
        
        [DataMember()]
        internal float m01;
        
        [DataMember()]
        internal float m10;
        
        [DataMember()]
        internal float m11;
        
        [DataMember()]
        internal float m20;
        
        [DataMember()]
        internal float m21;
        
        public mat3x2(float s) {
            this.m00 = s;
            this.m01 = 0F;
            this.m10 = 0F;
            this.m11 = s;
            this.m20 = 0F;
            this.m21 = 0F;
        }
        
        public mat3x2(mat3x2 m00) {
            this.m00 = m00.m00;
            this.m01 = m00.m01;
            this.m10 = m00.m10;
            this.m11 = m00.m11;
            this.m20 = m00.m20;
            this.m21 = m00.m21;
        }
        
        public mat3x2(mat2 m00, vec2 m10) {
            this.m00 = m00.m00;
            this.m01 = m00.m01;
            this.m10 = m00.m10;
            this.m11 = m00.m11;
            this.m20 = m10.v0;
            this.m21 = m10.v1;
        }
        
        public mat3x2(vec2 m00, mat2 m10) {
            this.m00 = m00.v0;
            this.m01 = m00.v1;
            this.m10 = m10.m00;
            this.m11 = m10.m01;
            this.m20 = m10.m10;
            this.m21 = m10.m11;
        }
        
        public mat3x2(vec2 m00, vec2 m10, vec2 m20) {
            this.m00 = m00.v0;
            this.m01 = m00.v1;
            this.m10 = m10.v0;
            this.m11 = m10.v1;
            this.m20 = m20.v0;
            this.m21 = m20.v1;
        }
        
        public mat3x2(float m00, float m01, float m10, float m11, float m20, float m21) {
            this.m00 = m00;
            this.m01 = m01;
            this.m10 = m10;
            this.m11 = m11;
            this.m20 = m20;
            this.m21 = m21;
        }
        
        public mat3x2(mat2 s) {
            this.m00 = s.m00;
            this.m01 = s.m01;
            this.m10 = s.m10;
            this.m11 = s.m11;
            this.m20 = 0F;
            this.m21 = 0F;
        }
        
        public mat3x2(mat2x3 s) {
            this.m00 = s.m00;
            this.m01 = s.m01;
            this.m10 = s.m10;
            this.m11 = s.m11;
            this.m20 = 0F;
            this.m21 = 0F;
        }
        
        public mat3x2(mat2x4 s) {
            this.m00 = s.m00;
            this.m01 = s.m01;
            this.m10 = s.m10;
            this.m11 = s.m11;
            this.m20 = 0F;
            this.m21 = 0F;
        }
        
        public mat3x2(mat3 s) {
            this.m00 = s.m00;
            this.m01 = s.m01;
            this.m10 = s.m10;
            this.m11 = s.m11;
            this.m20 = s.m20;
            this.m21 = s.m21;
        }
        
        public mat3x2(mat3x4 s) {
            this.m00 = s.m00;
            this.m01 = s.m01;
            this.m10 = s.m10;
            this.m11 = s.m11;
            this.m20 = s.m20;
            this.m21 = s.m21;
        }
        
        public mat3x2(mat4x2 s) {
            this.m00 = s.m00;
            this.m01 = s.m01;
            this.m10 = s.m10;
            this.m11 = s.m11;
            this.m20 = s.m20;
            this.m21 = s.m21;
        }
        
        public mat3x2(mat4x3 s) {
            this.m00 = s.m00;
            this.m01 = s.m01;
            this.m10 = s.m10;
            this.m11 = s.m11;
            this.m20 = s.m20;
            this.m21 = s.m21;
        }
        
        public mat3x2(mat4 s) {
            this.m00 = s.m00;
            this.m01 = s.m01;
            this.m10 = s.m10;
            this.m11 = s.m11;
            this.m20 = s.m20;
            this.m21 = s.m21;
        }
        
        public float this[int i0, int i1] {
            get {
                if (((i0 < 0) 
                            || (2 < i0))) {
                    throw new System.IndexOutOfRangeException("i0");
                }
                if (((i1 < 0) 
                            || (1 < i1))) {
                    throw new System.IndexOutOfRangeException("i1");
                }
                
                unsafe
                {
                    fixed (Single* v_ptr = &this.m00)
                    {
                        return *(v_ptr + (i1 + i0 * 2));
                    }
                };
            }
            set {
                if (((i0 < 0) 
                            || (2 < i0))) {
                    throw new System.IndexOutOfRangeException("i0");
                }
                if (((i1 < 0) 
                            || (1 < i1))) {
                    throw new System.IndexOutOfRangeException("i1");
                }
                
                unsafe
                {
                    fixed (Single* v_ptr = &this.m00)
                    {
                        *(v_ptr  + (i1 + i0 * 2)) = value;
                    }
                };
            }
        }
        
        public vec2 this[int i0] {
            get {
                return new vec2(this[i0, 0], this[i0, 1]);
            }
            set {
                this[i0, 0] = value[0];
                this[i0, 1] = value[1];
            }
        }
        
        public static mat3x2 operator+(mat3x2 v) {
            return new mat3x2(+v.m00, +v.m01, +v.m10, +v.m11, +v.m20, +v.m21);
        }
        
        public static mat3x2 operator-(mat3x2 v) {
            return new mat3x2(-v.m00, -v.m01, -v.m10, -v.m11, -v.m20, -v.m21);
        }
        
        public static mat3x2 operator++(mat3x2 v) {
            return new mat3x2(++v.m00, ++v.m01, ++v.m10, ++v.m11, ++v.m20, ++v.m21);
        }
        
        public static mat3x2 operator--(mat3x2 v) {
            return new mat3x2(--v.m00, --v.m01, --v.m10, --v.m11, --v.m20, --v.m21);
        }
        
        public static mat3x2 operator+(float s, mat3x2 v2) {
            return new mat3x2(s + v2.m00, s + v2.m01, s + v2.m10, s + v2.m11, s + v2.m20, s + v2.m21);
        }
        
        public static mat3x2 operator-(float s, mat3x2 v2) {
            return new mat3x2(s - v2.m00, s - v2.m01, s - v2.m10, s - v2.m11, s - v2.m20, s - v2.m21);
        }
        
        public static mat3x2 operator*(float s, mat3x2 v2) {
            return new mat3x2(s * v2.m00, s * v2.m01, s * v2.m10, s * v2.m11, s * v2.m20, s * v2.m21);
        }
        
        public static mat3x2 operator/(float s, mat3x2 v2) {
            return new mat3x2(s / v2.m00, s / v2.m01, s / v2.m10, s / v2.m11, s / v2.m20, s / v2.m21);
        }
        
        public static mat3x2 operator%(float s, mat3x2 v2) {
            return new mat3x2(s % v2.m00, s % v2.m01, s % v2.m10, s % v2.m11, s % v2.m20, s % v2.m21);
        }
        
        public static mat3x2 operator+(mat3x2 v1, float s) {
            return new mat3x2(v1.m00 + s, v1.m01 + s, v1.m10 + s, v1.m11 + s, v1.m20 + s, v1.m21 + s);
        }
        
        public static mat3x2 operator-(mat3x2 v1, float s) {
            return new mat3x2(v1.m00 - s, v1.m01 - s, v1.m10 - s, v1.m11 - s, v1.m20 - s, v1.m21 - s);
        }
        
        public static mat3x2 operator*(mat3x2 v1, float s) {
            return new mat3x2(v1.m00 * s, v1.m01 * s, v1.m10 * s, v1.m11 * s, v1.m20 * s, v1.m21 * s);
        }
        
        public static mat3x2 operator/(mat3x2 v1, float s) {
            return new mat3x2(v1.m00 / s, v1.m01 / s, v1.m10 / s, v1.m11 / s, v1.m20 / s, v1.m21 / s);
        }
        
        public static mat3x2 operator%(mat3x2 v1, float s) {
            return new mat3x2(v1.m00 % s, v1.m01 % s, v1.m10 % s, v1.m11 % s, v1.m20 % s, v1.m21 % s);
        }
        
        public static mat3x2 operator+(mat3x2 v1, mat3x2 v2) {
            return new mat3x2(v1.m00 + v2.m00, v1.m01 + v2.m01, v1.m10 + v2.m10, v1.m11 + v2.m11, v1.m20 + v2.m20, v1.m21 + v2.m21);
        }
        
        public static mat3x2 operator-(mat3x2 v1, mat3x2 v2) {
            return new mat3x2(v1.m00 - v2.m00, v1.m01 - v2.m01, v1.m10 - v2.m10, v1.m11 - v2.m11, v1.m20 - v2.m20, v1.m21 - v2.m21);
        }
        
        public static mat3x2 operator/(mat3x2 v1, mat3x2 v2) {
            return new mat3x2(v1.m00 / v2.m00, v1.m01 / v2.m01, v1.m10 / v2.m10, v1.m11 / v2.m11, v1.m20 / v2.m20, v1.m21 / v2.m21);
        }
        
        public static mat3x2 operator%(mat3x2 v1, mat3x2 v2) {
            return new mat3x2(v1.m00 % v2.m00, v1.m01 % v2.m01, v1.m10 % v2.m10, v1.m11 % v2.m11, v1.m20 % v2.m20, v1.m21 % v2.m21);
        }
        
        public static bmat3x2 operator==(mat3x2 v1, mat3x2 v2) {
            return new bmat3x2(v1.m00 == v2.m00, v1.m01 == v2.m01, v1.m10 == v2.m10, v1.m11 == v2.m11, v1.m20 == v2.m20, v1.m21 == v2.m21);
        }
        
        public static bmat3x2 operator>(mat3x2 v1, mat3x2 v2) {
            return new bmat3x2(v1.m00 > v2.m00, v1.m01 > v2.m01, v1.m10 > v2.m10, v1.m11 > v2.m11, v1.m20 > v2.m20, v1.m21 > v2.m21);
        }
        
        public static bmat3x2 operator>=(mat3x2 v1, mat3x2 v2) {
            return new bmat3x2(v1.m00 >= v2.m00, v1.m01 >= v2.m01, v1.m10 >= v2.m10, v1.m11 >= v2.m11, v1.m20 >= v2.m20, v1.m21 >= v2.m21);
        }
        
        public static bmat3x2 operator<(mat3x2 v1, mat3x2 v2) {
            return new bmat3x2(v1.m00 < v2.m00, v1.m01 < v2.m01, v1.m10 < v2.m10, v1.m11 < v2.m11, v1.m20 < v2.m20, v1.m21 < v2.m21);
        }
        
        public static bmat3x2 operator<=(mat3x2 v1, mat3x2 v2) {
            return new bmat3x2(v1.m00 <= v2.m00, v1.m01 <= v2.m01, v1.m10 <= v2.m10, v1.m11 <= v2.m11, v1.m20 <= v2.m20, v1.m21 <= v2.m21);
        }
        
        public static bmat3x2 operator!=(mat3x2 v1, mat3x2 v2) {
            return new bmat3x2(v1.m00 != v2.m00, v1.m01 != v2.m01, v1.m10 != v2.m10, v1.m11 != v2.m11, v1.m20 != v2.m20, v1.m21 != v2.m21);
        }
        
        public static vec2 operator*(vec3 v0, mat3x2 m1) {
            return math.InnerProduct1(ref v0, ref m1);
        }
        
        public static vec3 operator*(mat3x2 m0, vec2 v1) {
            return math.InnerProduct1(ref m0, ref v1);
        }
        
        public static mat3x2 operator*(mat3x2 m0, mat2 m1) {
            return math.InnerProduct1(ref m0, ref m1);
        }
        
        public static mat3 operator*(mat3x2 m0, mat2x3 m1) {
            return math.InnerProduct1(ref m0, ref m1);
        }
        
        public static mat3x4 operator*(mat3x2 m0, mat2x4 m1) {
            return math.InnerProduct1(ref m0, ref m1);
        }
        
        public override bool Equals(object obj) {
            return (obj is mat3x2) ? bmath.All(this == (mat3x2)obj) : false;
        }
        
        public override int GetHashCode() {
            return this.m00.GetHashCode() ^ this.m01.GetHashCode() ^ this.m10.GetHashCode() ^ this.m11.GetHashCode() ^ this.m20.GetHashCode() ^ this.m21.GetHashCode();
        }
        
        public override string ToString() {
            return "mat3x2(" + this.m00 + "," + this.m01 + "," + this.m10 + "," + this.m11 + "," + this.m20 + "," + this.m21 + ")";
        }
    }
    
    [DataContract()]
    [StructLayout(LayoutKind.Sequential)]
    public struct mat3 {
        
        [DataMember()]
        internal float m00;
        
        [DataMember()]
        internal float m01;
        
        [DataMember()]
        internal float m02;
        
        [DataMember()]
        internal float m10;
        
        [DataMember()]
        internal float m11;
        
        [DataMember()]
        internal float m12;
        
        [DataMember()]
        internal float m20;
        
        [DataMember()]
        internal float m21;
        
        [DataMember()]
        internal float m22;
        
        public mat3(float s) {
            this.m00 = s;
            this.m01 = 0F;
            this.m02 = 0F;
            this.m10 = 0F;
            this.m11 = s;
            this.m12 = 0F;
            this.m20 = 0F;
            this.m21 = 0F;
            this.m22 = s;
        }
        
        public mat3(mat3 m00) {
            this.m00 = m00.m00;
            this.m01 = m00.m01;
            this.m02 = m00.m02;
            this.m10 = m00.m10;
            this.m11 = m00.m11;
            this.m12 = m00.m12;
            this.m20 = m00.m20;
            this.m21 = m00.m21;
            this.m22 = m00.m22;
        }
        
        public mat3(mat2x3 m00, vec3 m10) {
            this.m00 = m00.m00;
            this.m01 = m00.m01;
            this.m02 = m00.m02;
            this.m10 = m00.m10;
            this.m11 = m00.m11;
            this.m12 = m00.m12;
            this.m20 = m10.v0;
            this.m21 = m10.v1;
            this.m22 = m10.v2;
        }
        
        public mat3(vec3 m00, mat2x3 m10) {
            this.m00 = m00.v0;
            this.m01 = m00.v1;
            this.m02 = m00.v2;
            this.m10 = m10.m00;
            this.m11 = m10.m01;
            this.m12 = m10.m02;
            this.m20 = m10.m10;
            this.m21 = m10.m11;
            this.m22 = m10.m12;
        }
        
        public mat3(vec3 m00, vec3 m10, vec3 m20) {
            this.m00 = m00.v0;
            this.m01 = m00.v1;
            this.m02 = m00.v2;
            this.m10 = m10.v0;
            this.m11 = m10.v1;
            this.m12 = m10.v2;
            this.m20 = m20.v0;
            this.m21 = m20.v1;
            this.m22 = m20.v2;
        }
        
        public mat3(vec2 m00, float m01, vec2 m10, float m11, vec2 m20, float m21) {
            this.m00 = m00.v0;
            this.m01 = m00.v1;
            this.m02 = m01;
            this.m10 = m10.v0;
            this.m11 = m10.v1;
            this.m12 = m11;
            this.m20 = m20.v0;
            this.m21 = m20.v1;
            this.m22 = m21;
        }
        
        public mat3(float m00, vec2 m01, float m10, vec2 m11, float m20, vec2 m21) {
            this.m00 = m00;
            this.m01 = m01.v0;
            this.m02 = m01.v1;
            this.m10 = m10;
            this.m11 = m11.v0;
            this.m12 = m11.v1;
            this.m20 = m20;
            this.m21 = m21.v0;
            this.m22 = m21.v1;
        }
        
        public mat3(float m00, float m01, float m02, float m10, float m11, float m12, float m20, float m21, float m22) {
            this.m00 = m00;
            this.m01 = m01;
            this.m02 = m02;
            this.m10 = m10;
            this.m11 = m11;
            this.m12 = m12;
            this.m20 = m20;
            this.m21 = m21;
            this.m22 = m22;
        }
        
        public mat3(mat2 s) {
            this.m00 = s.m00;
            this.m01 = s.m01;
            this.m02 = 0F;
            this.m10 = s.m10;
            this.m11 = s.m11;
            this.m12 = 0F;
            this.m20 = 0F;
            this.m21 = 0F;
            this.m22 = 0F;
        }
        
        public mat3(mat2x3 s) {
            this.m00 = s.m00;
            this.m01 = s.m01;
            this.m02 = s.m02;
            this.m10 = s.m10;
            this.m11 = s.m11;
            this.m12 = s.m12;
            this.m20 = 0F;
            this.m21 = 0F;
            this.m22 = 0F;
        }
        
        public mat3(mat2x4 s) {
            this.m00 = s.m00;
            this.m01 = s.m01;
            this.m02 = s.m02;
            this.m10 = s.m10;
            this.m11 = s.m11;
            this.m12 = s.m12;
            this.m20 = 0F;
            this.m21 = 0F;
            this.m22 = 0F;
        }
        
        public mat3(mat3x2 s) {
            this.m00 = s.m00;
            this.m01 = s.m01;
            this.m02 = 0F;
            this.m10 = s.m10;
            this.m11 = s.m11;
            this.m12 = 0F;
            this.m20 = s.m20;
            this.m21 = s.m21;
            this.m22 = 0F;
        }
        
        public mat3(mat3x4 s) {
            this.m00 = s.m00;
            this.m01 = s.m01;
            this.m02 = s.m02;
            this.m10 = s.m10;
            this.m11 = s.m11;
            this.m12 = s.m12;
            this.m20 = s.m20;
            this.m21 = s.m21;
            this.m22 = s.m22;
        }
        
        public mat3(mat4x2 s) {
            this.m00 = s.m00;
            this.m01 = s.m01;
            this.m02 = 0F;
            this.m10 = s.m10;
            this.m11 = s.m11;
            this.m12 = 0F;
            this.m20 = s.m20;
            this.m21 = s.m21;
            this.m22 = 0F;
        }
        
        public mat3(mat4x3 s) {
            this.m00 = s.m00;
            this.m01 = s.m01;
            this.m02 = s.m02;
            this.m10 = s.m10;
            this.m11 = s.m11;
            this.m12 = s.m12;
            this.m20 = s.m20;
            this.m21 = s.m21;
            this.m22 = s.m22;
        }
        
        public mat3(mat4 s) {
            this.m00 = s.m00;
            this.m01 = s.m01;
            this.m02 = s.m02;
            this.m10 = s.m10;
            this.m11 = s.m11;
            this.m12 = s.m12;
            this.m20 = s.m20;
            this.m21 = s.m21;
            this.m22 = s.m22;
        }
        
        public float this[int i0, int i1] {
            get {
                if (((i0 < 0) 
                            || (2 < i0))) {
                    throw new System.IndexOutOfRangeException("i0");
                }
                if (((i1 < 0) 
                            || (2 < i1))) {
                    throw new System.IndexOutOfRangeException("i1");
                }
                
                unsafe
                {
                    fixed (Single* v_ptr = &this.m00)
                    {
                        return *(v_ptr + (i1 + i0 * 3));
                    }
                };
            }
            set {
                if (((i0 < 0) 
                            || (2 < i0))) {
                    throw new System.IndexOutOfRangeException("i0");
                }
                if (((i1 < 0) 
                            || (2 < i1))) {
                    throw new System.IndexOutOfRangeException("i1");
                }
                
                unsafe
                {
                    fixed (Single* v_ptr = &this.m00)
                    {
                        *(v_ptr  + (i1 + i0 * 3)) = value;
                    }
                };
            }
        }
        
        public vec3 this[int i0] {
            get {
                return new vec3(this[i0, 0], this[i0, 1], this[i0, 2]);
            }
            set {
                this[i0, 0] = value[0];
                this[i0, 1] = value[1];
                this[i0, 2] = value[2];
            }
        }
        
        public static mat3 operator+(mat3 v) {
            return new mat3(+v.m00, +v.m01, +v.m02, +v.m10, +v.m11, +v.m12, +v.m20, +v.m21, +v.m22);
        }
        
        public static mat3 operator-(mat3 v) {
            return new mat3(-v.m00, -v.m01, -v.m02, -v.m10, -v.m11, -v.m12, -v.m20, -v.m21, -v.m22);
        }
        
        public static mat3 operator++(mat3 v) {
            return new mat3(++v.m00, ++v.m01, ++v.m02, ++v.m10, ++v.m11, ++v.m12, ++v.m20, ++v.m21, ++v.m22);
        }
        
        public static mat3 operator--(mat3 v) {
            return new mat3(--v.m00, --v.m01, --v.m02, --v.m10, --v.m11, --v.m12, --v.m20, --v.m21, --v.m22);
        }
        
        public static mat3 operator+(float s, mat3 v2) {
            return new mat3(s + v2.m00, s + v2.m01, s + v2.m02, s + v2.m10, s + v2.m11, s + v2.m12, s + v2.m20, s + v2.m21, s + v2.m22);
        }
        
        public static mat3 operator-(float s, mat3 v2) {
            return new mat3(s - v2.m00, s - v2.m01, s - v2.m02, s - v2.m10, s - v2.m11, s - v2.m12, s - v2.m20, s - v2.m21, s - v2.m22);
        }
        
        public static mat3 operator*(float s, mat3 v2) {
            return new mat3(s * v2.m00, s * v2.m01, s * v2.m02, s * v2.m10, s * v2.m11, s * v2.m12, s * v2.m20, s * v2.m21, s * v2.m22);
        }
        
        public static mat3 operator/(float s, mat3 v2) {
            return new mat3(s / v2.m00, s / v2.m01, s / v2.m02, s / v2.m10, s / v2.m11, s / v2.m12, s / v2.m20, s / v2.m21, s / v2.m22);
        }
        
        public static mat3 operator%(float s, mat3 v2) {
            return new mat3(s % v2.m00, s % v2.m01, s % v2.m02, s % v2.m10, s % v2.m11, s % v2.m12, s % v2.m20, s % v2.m21, s % v2.m22);
        }
        
        public static mat3 operator+(mat3 v1, float s) {
            return new mat3(v1.m00 + s, v1.m01 + s, v1.m02 + s, v1.m10 + s, v1.m11 + s, v1.m12 + s, v1.m20 + s, v1.m21 + s, v1.m22 + s);
        }
        
        public static mat3 operator-(mat3 v1, float s) {
            return new mat3(v1.m00 - s, v1.m01 - s, v1.m02 - s, v1.m10 - s, v1.m11 - s, v1.m12 - s, v1.m20 - s, v1.m21 - s, v1.m22 - s);
        }
        
        public static mat3 operator*(mat3 v1, float s) {
            return new mat3(v1.m00 * s, v1.m01 * s, v1.m02 * s, v1.m10 * s, v1.m11 * s, v1.m12 * s, v1.m20 * s, v1.m21 * s, v1.m22 * s);
        }
        
        public static mat3 operator/(mat3 v1, float s) {
            return new mat3(v1.m00 / s, v1.m01 / s, v1.m02 / s, v1.m10 / s, v1.m11 / s, v1.m12 / s, v1.m20 / s, v1.m21 / s, v1.m22 / s);
        }
        
        public static mat3 operator%(mat3 v1, float s) {
            return new mat3(v1.m00 % s, v1.m01 % s, v1.m02 % s, v1.m10 % s, v1.m11 % s, v1.m12 % s, v1.m20 % s, v1.m21 % s, v1.m22 % s);
        }
        
        public static mat3 operator+(mat3 v1, mat3 v2) {
            return new mat3(v1.m00 + v2.m00, v1.m01 + v2.m01, v1.m02 + v2.m02, v1.m10 + v2.m10, v1.m11 + v2.m11, v1.m12 + v2.m12, v1.m20 + v2.m20, v1.m21 + v2.m21, v1.m22 + v2.m22);
        }
        
        public static mat3 operator-(mat3 v1, mat3 v2) {
            return new mat3(v1.m00 - v2.m00, v1.m01 - v2.m01, v1.m02 - v2.m02, v1.m10 - v2.m10, v1.m11 - v2.m11, v1.m12 - v2.m12, v1.m20 - v2.m20, v1.m21 - v2.m21, v1.m22 - v2.m22);
        }
        
        public static mat3 operator/(mat3 v1, mat3 v2) {
            return new mat3(v1.m00 / v2.m00, v1.m01 / v2.m01, v1.m02 / v2.m02, v1.m10 / v2.m10, v1.m11 / v2.m11, v1.m12 / v2.m12, v1.m20 / v2.m20, v1.m21 / v2.m21, v1.m22 / v2.m22);
        }
        
        public static mat3 operator%(mat3 v1, mat3 v2) {
            return new mat3(v1.m00 % v2.m00, v1.m01 % v2.m01, v1.m02 % v2.m02, v1.m10 % v2.m10, v1.m11 % v2.m11, v1.m12 % v2.m12, v1.m20 % v2.m20, v1.m21 % v2.m21, v1.m22 % v2.m22);
        }
        
        public static bmat3 operator==(mat3 v1, mat3 v2) {
            return new bmat3(v1.m00 == v2.m00, v1.m01 == v2.m01, v1.m02 == v2.m02, v1.m10 == v2.m10, v1.m11 == v2.m11, v1.m12 == v2.m12, v1.m20 == v2.m20, v1.m21 == v2.m21, v1.m22 == v2.m22);
        }
        
        public static bmat3 operator>(mat3 v1, mat3 v2) {
            return new bmat3(v1.m00 > v2.m00, v1.m01 > v2.m01, v1.m02 > v2.m02, v1.m10 > v2.m10, v1.m11 > v2.m11, v1.m12 > v2.m12, v1.m20 > v2.m20, v1.m21 > v2.m21, v1.m22 > v2.m22);
        }
        
        public static bmat3 operator>=(mat3 v1, mat3 v2) {
            return new bmat3(v1.m00 >= v2.m00, v1.m01 >= v2.m01, v1.m02 >= v2.m02, v1.m10 >= v2.m10, v1.m11 >= v2.m11, v1.m12 >= v2.m12, v1.m20 >= v2.m20, v1.m21 >= v2.m21, v1.m22 >= v2.m22);
        }
        
        public static bmat3 operator<(mat3 v1, mat3 v2) {
            return new bmat3(v1.m00 < v2.m00, v1.m01 < v2.m01, v1.m02 < v2.m02, v1.m10 < v2.m10, v1.m11 < v2.m11, v1.m12 < v2.m12, v1.m20 < v2.m20, v1.m21 < v2.m21, v1.m22 < v2.m22);
        }
        
        public static bmat3 operator<=(mat3 v1, mat3 v2) {
            return new bmat3(v1.m00 <= v2.m00, v1.m01 <= v2.m01, v1.m02 <= v2.m02, v1.m10 <= v2.m10, v1.m11 <= v2.m11, v1.m12 <= v2.m12, v1.m20 <= v2.m20, v1.m21 <= v2.m21, v1.m22 <= v2.m22);
        }
        
        public static bmat3 operator!=(mat3 v1, mat3 v2) {
            return new bmat3(v1.m00 != v2.m00, v1.m01 != v2.m01, v1.m02 != v2.m02, v1.m10 != v2.m10, v1.m11 != v2.m11, v1.m12 != v2.m12, v1.m20 != v2.m20, v1.m21 != v2.m21, v1.m22 != v2.m22);
        }
        
        public static vec3 operator*(vec3 v0, mat3 m1) {
            return math.InnerProduct1(ref v0, ref m1);
        }
        
        public static vec3 operator*(mat3 m0, vec3 v1) {
            return math.InnerProduct1(ref m0, ref v1);
        }
        
        public static mat3x2 operator*(mat3 m0, mat3x2 m1) {
            return math.InnerProduct1(ref m0, ref m1);
        }
        
        public static mat3 operator*(mat3 m0, mat3 m1) {
            return math.InnerProduct1(ref m0, ref m1);
        }
        
        public static mat3x4 operator*(mat3 m0, mat3x4 m1) {
            return math.InnerProduct1(ref m0, ref m1);
        }
        
        public override bool Equals(object obj) {
            return (obj is mat3) ? bmath.All(this == (mat3)obj) : false;
        }
        
        public override int GetHashCode() {
            return this.m00.GetHashCode() ^ this.m01.GetHashCode() ^ this.m02.GetHashCode() ^ this.m10.GetHashCode() ^ this.m11.GetHashCode() ^ this.m12.GetHashCode() ^ this.m20.GetHashCode() ^ this.m21.GetHashCode() ^ this.m22.GetHashCode();
        }
        
        public override string ToString() {
            return "mat3(" + this.m00 + "," + this.m01 + "," + this.m02 + "," + this.m10 + "," + this.m11 + "," + this.m12 + "," + this.m20 + "," + this.m21 + "," + this.m22 + ")";
        }
    }
    
    [DataContract()]
    [StructLayout(LayoutKind.Sequential)]
    public struct mat3x4 {
        
        [DataMember()]
        internal float m00;
        
        [DataMember()]
        internal float m01;
        
        [DataMember()]
        internal float m02;
        
        [DataMember()]
        internal float m03;
        
        [DataMember()]
        internal float m10;
        
        [DataMember()]
        internal float m11;
        
        [DataMember()]
        internal float m12;
        
        [DataMember()]
        internal float m13;
        
        [DataMember()]
        internal float m20;
        
        [DataMember()]
        internal float m21;
        
        [DataMember()]
        internal float m22;
        
        [DataMember()]
        internal float m23;
        
        public mat3x4(float s) {
            this.m00 = s;
            this.m01 = 0F;
            this.m02 = 0F;
            this.m03 = 0F;
            this.m10 = 0F;
            this.m11 = s;
            this.m12 = 0F;
            this.m13 = 0F;
            this.m20 = 0F;
            this.m21 = 0F;
            this.m22 = s;
            this.m23 = 0F;
        }
        
        public mat3x4(mat3x4 m00) {
            this.m00 = m00.m00;
            this.m01 = m00.m01;
            this.m02 = m00.m02;
            this.m03 = m00.m03;
            this.m10 = m00.m10;
            this.m11 = m00.m11;
            this.m12 = m00.m12;
            this.m13 = m00.m13;
            this.m20 = m00.m20;
            this.m21 = m00.m21;
            this.m22 = m00.m22;
            this.m23 = m00.m23;
        }
        
        public mat3x4(mat3x2 m00, mat3x2 m01) {
            this.m00 = m00.m00;
            this.m01 = m00.m01;
            this.m02 = m01.m00;
            this.m03 = m01.m01;
            this.m10 = m00.m10;
            this.m11 = m00.m11;
            this.m12 = m01.m10;
            this.m13 = m01.m11;
            this.m20 = m00.m20;
            this.m21 = m00.m21;
            this.m22 = m01.m20;
            this.m23 = m01.m21;
        }
        
        public mat3x4(mat2x4 m00, vec4 m10) {
            this.m00 = m00.m00;
            this.m01 = m00.m01;
            this.m02 = m00.m02;
            this.m03 = m00.m03;
            this.m10 = m00.m10;
            this.m11 = m00.m11;
            this.m12 = m00.m12;
            this.m13 = m00.m13;
            this.m20 = m10.v0;
            this.m21 = m10.v1;
            this.m22 = m10.v2;
            this.m23 = m10.v3;
        }
        
        public mat3x4(mat2 m00, mat2 m01, vec2 m10, vec2 m11) {
            this.m00 = m00.m00;
            this.m01 = m00.m01;
            this.m02 = m01.m00;
            this.m03 = m01.m01;
            this.m10 = m00.m10;
            this.m11 = m00.m11;
            this.m12 = m01.m10;
            this.m13 = m01.m11;
            this.m20 = m10.v0;
            this.m21 = m10.v1;
            this.m22 = m11.v0;
            this.m23 = m11.v1;
        }
        
        public mat3x4(vec4 m00, mat2x4 m10) {
            this.m00 = m00.v0;
            this.m01 = m00.v1;
            this.m02 = m00.v2;
            this.m03 = m00.v3;
            this.m10 = m10.m00;
            this.m11 = m10.m01;
            this.m12 = m10.m02;
            this.m13 = m10.m03;
            this.m20 = m10.m10;
            this.m21 = m10.m11;
            this.m22 = m10.m12;
            this.m23 = m10.m13;
        }
        
        public mat3x4(vec2 m00, vec2 m01, mat2 m10, mat2 m11) {
            this.m00 = m00.v0;
            this.m01 = m00.v1;
            this.m02 = m01.v0;
            this.m03 = m01.v1;
            this.m10 = m10.m00;
            this.m11 = m10.m01;
            this.m12 = m11.m00;
            this.m13 = m11.m01;
            this.m20 = m10.m10;
            this.m21 = m10.m11;
            this.m22 = m11.m10;
            this.m23 = m11.m11;
        }
        
        public mat3x4(vec4 m00, vec4 m10, vec4 m20) {
            this.m00 = m00.v0;
            this.m01 = m00.v1;
            this.m02 = m00.v2;
            this.m03 = m00.v3;
            this.m10 = m10.v0;
            this.m11 = m10.v1;
            this.m12 = m10.v2;
            this.m13 = m10.v3;
            this.m20 = m20.v0;
            this.m21 = m20.v1;
            this.m22 = m20.v2;
            this.m23 = m20.v3;
        }
        
        public mat3x4(vec2 m00, vec2 m01, vec2 m10, vec2 m11, vec2 m20, vec2 m21) {
            this.m00 = m00.v0;
            this.m01 = m00.v1;
            this.m02 = m01.v0;
            this.m03 = m01.v1;
            this.m10 = m10.v0;
            this.m11 = m10.v1;
            this.m12 = m11.v0;
            this.m13 = m11.v1;
            this.m20 = m20.v0;
            this.m21 = m20.v1;
            this.m22 = m21.v0;
            this.m23 = m21.v1;
        }
        
        public mat3x4(vec3 m00, float m01, vec3 m10, float m11, vec3 m20, float m21) {
            this.m00 = m00.v0;
            this.m01 = m00.v1;
            this.m02 = m00.v2;
            this.m03 = m01;
            this.m10 = m10.v0;
            this.m11 = m10.v1;
            this.m12 = m10.v2;
            this.m13 = m11;
            this.m20 = m20.v0;
            this.m21 = m20.v1;
            this.m22 = m20.v2;
            this.m23 = m21;
        }
        
        public mat3x4(float m00, vec3 m01, float m10, vec3 m11, float m20, vec3 m21) {
            this.m00 = m00;
            this.m01 = m01.v0;
            this.m02 = m01.v1;
            this.m03 = m01.v2;
            this.m10 = m10;
            this.m11 = m11.v0;
            this.m12 = m11.v1;
            this.m13 = m11.v2;
            this.m20 = m20;
            this.m21 = m21.v0;
            this.m22 = m21.v1;
            this.m23 = m21.v2;
        }
        
        public mat3x4(vec2 m00, float m01, float m02, vec2 m10, float m11, float m12, vec2 m20, float m21, float m22) {
            this.m00 = m00.v0;
            this.m01 = m00.v1;
            this.m02 = m01;
            this.m03 = m02;
            this.m10 = m10.v0;
            this.m11 = m10.v1;
            this.m12 = m11;
            this.m13 = m12;
            this.m20 = m20.v0;
            this.m21 = m20.v1;
            this.m22 = m21;
            this.m23 = m22;
        }
        
        public mat3x4(float m00, vec2 m01, float m02, float m10, vec2 m11, float m12, float m20, vec2 m21, float m22) {
            this.m00 = m00;
            this.m01 = m01.v0;
            this.m02 = m01.v1;
            this.m03 = m02;
            this.m10 = m10;
            this.m11 = m11.v0;
            this.m12 = m11.v1;
            this.m13 = m12;
            this.m20 = m20;
            this.m21 = m21.v0;
            this.m22 = m21.v1;
            this.m23 = m22;
        }
        
        public mat3x4(float m00, float m01, vec2 m02, float m10, float m11, vec2 m12, float m20, float m21, vec2 m22) {
            this.m00 = m00;
            this.m01 = m01;
            this.m02 = m02.v0;
            this.m03 = m02.v1;
            this.m10 = m10;
            this.m11 = m11;
            this.m12 = m12.v0;
            this.m13 = m12.v1;
            this.m20 = m20;
            this.m21 = m21;
            this.m22 = m22.v0;
            this.m23 = m22.v1;
        }
        
        public mat3x4(float m00, float m01, float m02, float m03, float m10, float m11, float m12, float m13, float m20, float m21, float m22, float m23) {
            this.m00 = m00;
            this.m01 = m01;
            this.m02 = m02;
            this.m03 = m03;
            this.m10 = m10;
            this.m11 = m11;
            this.m12 = m12;
            this.m13 = m13;
            this.m20 = m20;
            this.m21 = m21;
            this.m22 = m22;
            this.m23 = m23;
        }
        
        public mat3x4(mat2 s) {
            this.m00 = s.m00;
            this.m01 = s.m01;
            this.m02 = 0F;
            this.m03 = 0F;
            this.m10 = s.m10;
            this.m11 = s.m11;
            this.m12 = 0F;
            this.m13 = 0F;
            this.m20 = 0F;
            this.m21 = 0F;
            this.m22 = 0F;
            this.m23 = 0F;
        }
        
        public mat3x4(mat2x3 s) {
            this.m00 = s.m00;
            this.m01 = s.m01;
            this.m02 = s.m02;
            this.m03 = 0F;
            this.m10 = s.m10;
            this.m11 = s.m11;
            this.m12 = s.m12;
            this.m13 = 0F;
            this.m20 = 0F;
            this.m21 = 0F;
            this.m22 = 0F;
            this.m23 = 0F;
        }
        
        public mat3x4(mat2x4 s) {
            this.m00 = s.m00;
            this.m01 = s.m01;
            this.m02 = s.m02;
            this.m03 = s.m03;
            this.m10 = s.m10;
            this.m11 = s.m11;
            this.m12 = s.m12;
            this.m13 = s.m13;
            this.m20 = 0F;
            this.m21 = 0F;
            this.m22 = 0F;
            this.m23 = 0F;
        }
        
        public mat3x4(mat3x2 s) {
            this.m00 = s.m00;
            this.m01 = s.m01;
            this.m02 = 0F;
            this.m03 = 0F;
            this.m10 = s.m10;
            this.m11 = s.m11;
            this.m12 = 0F;
            this.m13 = 0F;
            this.m20 = s.m20;
            this.m21 = s.m21;
            this.m22 = 0F;
            this.m23 = 0F;
        }
        
        public mat3x4(mat3 s) {
            this.m00 = s.m00;
            this.m01 = s.m01;
            this.m02 = s.m02;
            this.m03 = 0F;
            this.m10 = s.m10;
            this.m11 = s.m11;
            this.m12 = s.m12;
            this.m13 = 0F;
            this.m20 = s.m20;
            this.m21 = s.m21;
            this.m22 = s.m22;
            this.m23 = 0F;
        }
        
        public mat3x4(mat4x2 s) {
            this.m00 = s.m00;
            this.m01 = s.m01;
            this.m02 = 0F;
            this.m03 = 0F;
            this.m10 = s.m10;
            this.m11 = s.m11;
            this.m12 = 0F;
            this.m13 = 0F;
            this.m20 = s.m20;
            this.m21 = s.m21;
            this.m22 = 0F;
            this.m23 = 0F;
        }
        
        public mat3x4(mat4x3 s) {
            this.m00 = s.m00;
            this.m01 = s.m01;
            this.m02 = s.m02;
            this.m03 = 0F;
            this.m10 = s.m10;
            this.m11 = s.m11;
            this.m12 = s.m12;
            this.m13 = 0F;
            this.m20 = s.m20;
            this.m21 = s.m21;
            this.m22 = s.m22;
            this.m23 = 0F;
        }
        
        public mat3x4(mat4 s) {
            this.m00 = s.m00;
            this.m01 = s.m01;
            this.m02 = s.m02;
            this.m03 = s.m03;
            this.m10 = s.m10;
            this.m11 = s.m11;
            this.m12 = s.m12;
            this.m13 = s.m13;
            this.m20 = s.m20;
            this.m21 = s.m21;
            this.m22 = s.m22;
            this.m23 = s.m23;
        }
        
        public float this[int i0, int i1] {
            get {
                if (((i0 < 0) 
                            || (2 < i0))) {
                    throw new System.IndexOutOfRangeException("i0");
                }
                if (((i1 < 0) 
                            || (3 < i1))) {
                    throw new System.IndexOutOfRangeException("i1");
                }
                
                unsafe
                {
                    fixed (Single* v_ptr = &this.m00)
                    {
                        return *(v_ptr + (i1 + i0 * 4));
                    }
                };
            }
            set {
                if (((i0 < 0) 
                            || (2 < i0))) {
                    throw new System.IndexOutOfRangeException("i0");
                }
                if (((i1 < 0) 
                            || (3 < i1))) {
                    throw new System.IndexOutOfRangeException("i1");
                }
                
                unsafe
                {
                    fixed (Single* v_ptr = &this.m00)
                    {
                        *(v_ptr  + (i1 + i0 * 4)) = value;
                    }
                };
            }
        }
        
        public vec4 this[int i0] {
            get {
                return new vec4(this[i0, 0], this[i0, 1], this[i0, 2], this[i0, 3]);
            }
            set {
                this[i0, 0] = value[0];
                this[i0, 1] = value[1];
                this[i0, 2] = value[2];
                this[i0, 3] = value[3];
            }
        }
        
        public static mat3x4 operator+(mat3x4 v) {
            return new mat3x4(+v.m00, +v.m01, +v.m02, +v.m03, +v.m10, +v.m11, +v.m12, +v.m13, +v.m20, +v.m21, +v.m22, +v.m23);
        }
        
        public static mat3x4 operator-(mat3x4 v) {
            return new mat3x4(-v.m00, -v.m01, -v.m02, -v.m03, -v.m10, -v.m11, -v.m12, -v.m13, -v.m20, -v.m21, -v.m22, -v.m23);
        }
        
        public static mat3x4 operator++(mat3x4 v) {
            return new mat3x4(++v.m00, ++v.m01, ++v.m02, ++v.m03, ++v.m10, ++v.m11, ++v.m12, ++v.m13, ++v.m20, ++v.m21, ++v.m22, ++v.m23);
        }
        
        public static mat3x4 operator--(mat3x4 v) {
            return new mat3x4(--v.m00, --v.m01, --v.m02, --v.m03, --v.m10, --v.m11, --v.m12, --v.m13, --v.m20, --v.m21, --v.m22, --v.m23);
        }
        
        public static mat3x4 operator+(float s, mat3x4 v2) {
            return new mat3x4(s + v2.m00, s + v2.m01, s + v2.m02, s + v2.m03, s + v2.m10, s + v2.m11, s + v2.m12, s + v2.m13, s + v2.m20, s + v2.m21, s + v2.m22, s + v2.m23);
        }
        
        public static mat3x4 operator-(float s, mat3x4 v2) {
            return new mat3x4(s - v2.m00, s - v2.m01, s - v2.m02, s - v2.m03, s - v2.m10, s - v2.m11, s - v2.m12, s - v2.m13, s - v2.m20, s - v2.m21, s - v2.m22, s - v2.m23);
        }
        
        public static mat3x4 operator*(float s, mat3x4 v2) {
            return new mat3x4(s * v2.m00, s * v2.m01, s * v2.m02, s * v2.m03, s * v2.m10, s * v2.m11, s * v2.m12, s * v2.m13, s * v2.m20, s * v2.m21, s * v2.m22, s * v2.m23);
        }
        
        public static mat3x4 operator/(float s, mat3x4 v2) {
            return new mat3x4(s / v2.m00, s / v2.m01, s / v2.m02, s / v2.m03, s / v2.m10, s / v2.m11, s / v2.m12, s / v2.m13, s / v2.m20, s / v2.m21, s / v2.m22, s / v2.m23);
        }
        
        public static mat3x4 operator%(float s, mat3x4 v2) {
            return new mat3x4(s % v2.m00, s % v2.m01, s % v2.m02, s % v2.m03, s % v2.m10, s % v2.m11, s % v2.m12, s % v2.m13, s % v2.m20, s % v2.m21, s % v2.m22, s % v2.m23);
        }
        
        public static mat3x4 operator+(mat3x4 v1, float s) {
            return new mat3x4(v1.m00 + s, v1.m01 + s, v1.m02 + s, v1.m03 + s, v1.m10 + s, v1.m11 + s, v1.m12 + s, v1.m13 + s, v1.m20 + s, v1.m21 + s, v1.m22 + s, v1.m23 + s);
        }
        
        public static mat3x4 operator-(mat3x4 v1, float s) {
            return new mat3x4(v1.m00 - s, v1.m01 - s, v1.m02 - s, v1.m03 - s, v1.m10 - s, v1.m11 - s, v1.m12 - s, v1.m13 - s, v1.m20 - s, v1.m21 - s, v1.m22 - s, v1.m23 - s);
        }
        
        public static mat3x4 operator*(mat3x4 v1, float s) {
            return new mat3x4(v1.m00 * s, v1.m01 * s, v1.m02 * s, v1.m03 * s, v1.m10 * s, v1.m11 * s, v1.m12 * s, v1.m13 * s, v1.m20 * s, v1.m21 * s, v1.m22 * s, v1.m23 * s);
        }
        
        public static mat3x4 operator/(mat3x4 v1, float s) {
            return new mat3x4(v1.m00 / s, v1.m01 / s, v1.m02 / s, v1.m03 / s, v1.m10 / s, v1.m11 / s, v1.m12 / s, v1.m13 / s, v1.m20 / s, v1.m21 / s, v1.m22 / s, v1.m23 / s);
        }
        
        public static mat3x4 operator%(mat3x4 v1, float s) {
            return new mat3x4(v1.m00 % s, v1.m01 % s, v1.m02 % s, v1.m03 % s, v1.m10 % s, v1.m11 % s, v1.m12 % s, v1.m13 % s, v1.m20 % s, v1.m21 % s, v1.m22 % s, v1.m23 % s);
        }
        
        public static mat3x4 operator+(mat3x4 v1, mat3x4 v2) {
            return new mat3x4(v1.m00 + v2.m00, v1.m01 + v2.m01, v1.m02 + v2.m02, v1.m03 + v2.m03, v1.m10 + v2.m10, v1.m11 + v2.m11, v1.m12 + v2.m12, v1.m13 + v2.m13, v1.m20 + v2.m20, v1.m21 + v2.m21, v1.m22 + v2.m22, v1.m23 + v2.m23);
        }
        
        public static mat3x4 operator-(mat3x4 v1, mat3x4 v2) {
            return new mat3x4(v1.m00 - v2.m00, v1.m01 - v2.m01, v1.m02 - v2.m02, v1.m03 - v2.m03, v1.m10 - v2.m10, v1.m11 - v2.m11, v1.m12 - v2.m12, v1.m13 - v2.m13, v1.m20 - v2.m20, v1.m21 - v2.m21, v1.m22 - v2.m22, v1.m23 - v2.m23);
        }
        
        public static mat3x4 operator/(mat3x4 v1, mat3x4 v2) {
            return new mat3x4(v1.m00 / v2.m00, v1.m01 / v2.m01, v1.m02 / v2.m02, v1.m03 / v2.m03, v1.m10 / v2.m10, v1.m11 / v2.m11, v1.m12 / v2.m12, v1.m13 / v2.m13, v1.m20 / v2.m20, v1.m21 / v2.m21, v1.m22 / v2.m22, v1.m23 / v2.m23);
        }
        
        public static mat3x4 operator%(mat3x4 v1, mat3x4 v2) {
            return new mat3x4(v1.m00 % v2.m00, v1.m01 % v2.m01, v1.m02 % v2.m02, v1.m03 % v2.m03, v1.m10 % v2.m10, v1.m11 % v2.m11, v1.m12 % v2.m12, v1.m13 % v2.m13, v1.m20 % v2.m20, v1.m21 % v2.m21, v1.m22 % v2.m22, v1.m23 % v2.m23);
        }
        
        public static bmat3x4 operator==(mat3x4 v1, mat3x4 v2) {
            return new bmat3x4(v1.m00 == v2.m00, v1.m01 == v2.m01, v1.m02 == v2.m02, v1.m03 == v2.m03, v1.m10 == v2.m10, v1.m11 == v2.m11, v1.m12 == v2.m12, v1.m13 == v2.m13, v1.m20 == v2.m20, v1.m21 == v2.m21, v1.m22 == v2.m22, v1.m23 == v2.m23);
        }
        
        public static bmat3x4 operator>(mat3x4 v1, mat3x4 v2) {
            return new bmat3x4(v1.m00 > v2.m00, v1.m01 > v2.m01, v1.m02 > v2.m02, v1.m03 > v2.m03, v1.m10 > v2.m10, v1.m11 > v2.m11, v1.m12 > v2.m12, v1.m13 > v2.m13, v1.m20 > v2.m20, v1.m21 > v2.m21, v1.m22 > v2.m22, v1.m23 > v2.m23);
        }
        
        public static bmat3x4 operator>=(mat3x4 v1, mat3x4 v2) {
            return new bmat3x4(v1.m00 >= v2.m00, v1.m01 >= v2.m01, v1.m02 >= v2.m02, v1.m03 >= v2.m03, v1.m10 >= v2.m10, v1.m11 >= v2.m11, v1.m12 >= v2.m12, v1.m13 >= v2.m13, v1.m20 >= v2.m20, v1.m21 >= v2.m21, v1.m22 >= v2.m22, v1.m23 >= v2.m23);
        }
        
        public static bmat3x4 operator<(mat3x4 v1, mat3x4 v2) {
            return new bmat3x4(v1.m00 < v2.m00, v1.m01 < v2.m01, v1.m02 < v2.m02, v1.m03 < v2.m03, v1.m10 < v2.m10, v1.m11 < v2.m11, v1.m12 < v2.m12, v1.m13 < v2.m13, v1.m20 < v2.m20, v1.m21 < v2.m21, v1.m22 < v2.m22, v1.m23 < v2.m23);
        }
        
        public static bmat3x4 operator<=(mat3x4 v1, mat3x4 v2) {
            return new bmat3x4(v1.m00 <= v2.m00, v1.m01 <= v2.m01, v1.m02 <= v2.m02, v1.m03 <= v2.m03, v1.m10 <= v2.m10, v1.m11 <= v2.m11, v1.m12 <= v2.m12, v1.m13 <= v2.m13, v1.m20 <= v2.m20, v1.m21 <= v2.m21, v1.m22 <= v2.m22, v1.m23 <= v2.m23);
        }
        
        public static bmat3x4 operator!=(mat3x4 v1, mat3x4 v2) {
            return new bmat3x4(v1.m00 != v2.m00, v1.m01 != v2.m01, v1.m02 != v2.m02, v1.m03 != v2.m03, v1.m10 != v2.m10, v1.m11 != v2.m11, v1.m12 != v2.m12, v1.m13 != v2.m13, v1.m20 != v2.m20, v1.m21 != v2.m21, v1.m22 != v2.m22, v1.m23 != v2.m23);
        }
        
        public static vec4 operator*(vec3 v0, mat3x4 m1) {
            return math.InnerProduct1(ref v0, ref m1);
        }
        
        public static vec3 operator*(mat3x4 m0, vec4 v1) {
            return math.InnerProduct1(ref m0, ref v1);
        }
        
        public static mat3x2 operator*(mat3x4 m0, mat4x2 m1) {
            return math.InnerProduct1(ref m0, ref m1);
        }
        
        public static mat3 operator*(mat3x4 m0, mat4x3 m1) {
            return math.InnerProduct1(ref m0, ref m1);
        }
        
        public static mat3x4 operator*(mat3x4 m0, mat4 m1) {
            return math.InnerProduct1(ref m0, ref m1);
        }
        
        public override bool Equals(object obj) {
            return (obj is mat3x4) ? bmath.All(this == (mat3x4)obj) : false;
        }
        
        public override int GetHashCode() {
            return this.m00.GetHashCode() ^ this.m01.GetHashCode() ^ this.m02.GetHashCode() ^ this.m03.GetHashCode() ^ this.m10.GetHashCode() ^ this.m11.GetHashCode() ^ this.m12.GetHashCode() ^ this.m13.GetHashCode() ^ this.m20.GetHashCode() ^ this.m21.GetHashCode() ^ this.m22.GetHashCode() ^ this.m23.GetHashCode();
        }
        
        public override string ToString() {
            return "mat3x4(" + this.m00 + "," + this.m01 + "," + this.m02 + "," + this.m03 + "," + this.m10 + "," + this.m11 + "," + this.m12 + "," + this.m13 + "," + this.m20 + "," + this.m21 + "," + this.m22 + "," + this.m23 + ")";
        }
    }
    
    [DataContract()]
    [StructLayout(LayoutKind.Sequential)]
    public struct mat4x2 {
        
        [DataMember()]
        internal float m00;
        
        [DataMember()]
        internal float m01;
        
        [DataMember()]
        internal float m10;
        
        [DataMember()]
        internal float m11;
        
        [DataMember()]
        internal float m20;
        
        [DataMember()]
        internal float m21;
        
        [DataMember()]
        internal float m30;
        
        [DataMember()]
        internal float m31;
        
        public mat4x2(float s) {
            this.m00 = s;
            this.m01 = 0F;
            this.m10 = 0F;
            this.m11 = s;
            this.m20 = 0F;
            this.m21 = 0F;
            this.m30 = 0F;
            this.m31 = 0F;
        }
        
        public mat4x2(mat4x2 m00) {
            this.m00 = m00.m00;
            this.m01 = m00.m01;
            this.m10 = m00.m10;
            this.m11 = m00.m11;
            this.m20 = m00.m20;
            this.m21 = m00.m21;
            this.m30 = m00.m30;
            this.m31 = m00.m31;
        }
        
        public mat4x2(mat2 m00, mat2 m10) {
            this.m00 = m00.m00;
            this.m01 = m00.m01;
            this.m10 = m00.m10;
            this.m11 = m00.m11;
            this.m20 = m10.m00;
            this.m21 = m10.m01;
            this.m30 = m10.m10;
            this.m31 = m10.m11;
        }
        
        public mat4x2(mat3x2 m00, vec2 m10) {
            this.m00 = m00.m00;
            this.m01 = m00.m01;
            this.m10 = m00.m10;
            this.m11 = m00.m11;
            this.m20 = m00.m20;
            this.m21 = m00.m21;
            this.m30 = m10.v0;
            this.m31 = m10.v1;
        }
        
        public mat4x2(vec2 m00, mat3x2 m10) {
            this.m00 = m00.v0;
            this.m01 = m00.v1;
            this.m10 = m10.m00;
            this.m11 = m10.m01;
            this.m20 = m10.m10;
            this.m21 = m10.m11;
            this.m30 = m10.m20;
            this.m31 = m10.m21;
        }
        
        public mat4x2(mat2 m00, vec2 m10, vec2 m20) {
            this.m00 = m00.m00;
            this.m01 = m00.m01;
            this.m10 = m00.m10;
            this.m11 = m00.m11;
            this.m20 = m10.v0;
            this.m21 = m10.v1;
            this.m30 = m20.v0;
            this.m31 = m20.v1;
        }
        
        public mat4x2(vec2 m00, mat2 m10, vec2 m20) {
            this.m00 = m00.v0;
            this.m01 = m00.v1;
            this.m10 = m10.m00;
            this.m11 = m10.m01;
            this.m20 = m10.m10;
            this.m21 = m10.m11;
            this.m30 = m20.v0;
            this.m31 = m20.v1;
        }
        
        public mat4x2(vec2 m00, vec2 m10, mat2 m20) {
            this.m00 = m00.v0;
            this.m01 = m00.v1;
            this.m10 = m10.v0;
            this.m11 = m10.v1;
            this.m20 = m20.m00;
            this.m21 = m20.m01;
            this.m30 = m20.m10;
            this.m31 = m20.m11;
        }
        
        public mat4x2(vec2 m00, vec2 m10, vec2 m20, vec2 m30) {
            this.m00 = m00.v0;
            this.m01 = m00.v1;
            this.m10 = m10.v0;
            this.m11 = m10.v1;
            this.m20 = m20.v0;
            this.m21 = m20.v1;
            this.m30 = m30.v0;
            this.m31 = m30.v1;
        }
        
        public mat4x2(float m00, float m01, float m10, float m11, float m20, float m21, float m30, float m31) {
            this.m00 = m00;
            this.m01 = m01;
            this.m10 = m10;
            this.m11 = m11;
            this.m20 = m20;
            this.m21 = m21;
            this.m30 = m30;
            this.m31 = m31;
        }
        
        public mat4x2(mat2 s) {
            this.m00 = s.m00;
            this.m01 = s.m01;
            this.m10 = s.m10;
            this.m11 = s.m11;
            this.m20 = 0F;
            this.m21 = 0F;
            this.m30 = 0F;
            this.m31 = 0F;
        }
        
        public mat4x2(mat2x3 s) {
            this.m00 = s.m00;
            this.m01 = s.m01;
            this.m10 = s.m10;
            this.m11 = s.m11;
            this.m20 = 0F;
            this.m21 = 0F;
            this.m30 = 0F;
            this.m31 = 0F;
        }
        
        public mat4x2(mat2x4 s) {
            this.m00 = s.m00;
            this.m01 = s.m01;
            this.m10 = s.m10;
            this.m11 = s.m11;
            this.m20 = 0F;
            this.m21 = 0F;
            this.m30 = 0F;
            this.m31 = 0F;
        }
        
        public mat4x2(mat3x2 s) {
            this.m00 = s.m00;
            this.m01 = s.m01;
            this.m10 = s.m10;
            this.m11 = s.m11;
            this.m20 = s.m20;
            this.m21 = s.m21;
            this.m30 = 0F;
            this.m31 = 0F;
        }
        
        public mat4x2(mat3 s) {
            this.m00 = s.m00;
            this.m01 = s.m01;
            this.m10 = s.m10;
            this.m11 = s.m11;
            this.m20 = s.m20;
            this.m21 = s.m21;
            this.m30 = 0F;
            this.m31 = 0F;
        }
        
        public mat4x2(mat3x4 s) {
            this.m00 = s.m00;
            this.m01 = s.m01;
            this.m10 = s.m10;
            this.m11 = s.m11;
            this.m20 = s.m20;
            this.m21 = s.m21;
            this.m30 = 0F;
            this.m31 = 0F;
        }
        
        public mat4x2(mat4x3 s) {
            this.m00 = s.m00;
            this.m01 = s.m01;
            this.m10 = s.m10;
            this.m11 = s.m11;
            this.m20 = s.m20;
            this.m21 = s.m21;
            this.m30 = s.m30;
            this.m31 = s.m31;
        }
        
        public mat4x2(mat4 s) {
            this.m00 = s.m00;
            this.m01 = s.m01;
            this.m10 = s.m10;
            this.m11 = s.m11;
            this.m20 = s.m20;
            this.m21 = s.m21;
            this.m30 = s.m30;
            this.m31 = s.m31;
        }
        
        public float this[int i0, int i1] {
            get {
                if (((i0 < 0) 
                            || (3 < i0))) {
                    throw new System.IndexOutOfRangeException("i0");
                }
                if (((i1 < 0) 
                            || (1 < i1))) {
                    throw new System.IndexOutOfRangeException("i1");
                }
                
                unsafe
                {
                    fixed (Single* v_ptr = &this.m00)
                    {
                        return *(v_ptr + (i1 + i0 * 2));
                    }
                };
            }
            set {
                if (((i0 < 0) 
                            || (3 < i0))) {
                    throw new System.IndexOutOfRangeException("i0");
                }
                if (((i1 < 0) 
                            || (1 < i1))) {
                    throw new System.IndexOutOfRangeException("i1");
                }
                
                unsafe
                {
                    fixed (Single* v_ptr = &this.m00)
                    {
                        *(v_ptr  + (i1 + i0 * 2)) = value;
                    }
                };
            }
        }
        
        public vec2 this[int i0] {
            get {
                return new vec2(this[i0, 0], this[i0, 1]);
            }
            set {
                this[i0, 0] = value[0];
                this[i0, 1] = value[1];
            }
        }
        
        public static mat4x2 operator+(mat4x2 v) {
            return new mat4x2(+v.m00, +v.m01, +v.m10, +v.m11, +v.m20, +v.m21, +v.m30, +v.m31);
        }
        
        public static mat4x2 operator-(mat4x2 v) {
            return new mat4x2(-v.m00, -v.m01, -v.m10, -v.m11, -v.m20, -v.m21, -v.m30, -v.m31);
        }
        
        public static mat4x2 operator++(mat4x2 v) {
            return new mat4x2(++v.m00, ++v.m01, ++v.m10, ++v.m11, ++v.m20, ++v.m21, ++v.m30, ++v.m31);
        }
        
        public static mat4x2 operator--(mat4x2 v) {
            return new mat4x2(--v.m00, --v.m01, --v.m10, --v.m11, --v.m20, --v.m21, --v.m30, --v.m31);
        }
        
        public static mat4x2 operator+(float s, mat4x2 v2) {
            return new mat4x2(s + v2.m00, s + v2.m01, s + v2.m10, s + v2.m11, s + v2.m20, s + v2.m21, s + v2.m30, s + v2.m31);
        }
        
        public static mat4x2 operator-(float s, mat4x2 v2) {
            return new mat4x2(s - v2.m00, s - v2.m01, s - v2.m10, s - v2.m11, s - v2.m20, s - v2.m21, s - v2.m30, s - v2.m31);
        }
        
        public static mat4x2 operator*(float s, mat4x2 v2) {
            return new mat4x2(s * v2.m00, s * v2.m01, s * v2.m10, s * v2.m11, s * v2.m20, s * v2.m21, s * v2.m30, s * v2.m31);
        }
        
        public static mat4x2 operator/(float s, mat4x2 v2) {
            return new mat4x2(s / v2.m00, s / v2.m01, s / v2.m10, s / v2.m11, s / v2.m20, s / v2.m21, s / v2.m30, s / v2.m31);
        }
        
        public static mat4x2 operator%(float s, mat4x2 v2) {
            return new mat4x2(s % v2.m00, s % v2.m01, s % v2.m10, s % v2.m11, s % v2.m20, s % v2.m21, s % v2.m30, s % v2.m31);
        }
        
        public static mat4x2 operator+(mat4x2 v1, float s) {
            return new mat4x2(v1.m00 + s, v1.m01 + s, v1.m10 + s, v1.m11 + s, v1.m20 + s, v1.m21 + s, v1.m30 + s, v1.m31 + s);
        }
        
        public static mat4x2 operator-(mat4x2 v1, float s) {
            return new mat4x2(v1.m00 - s, v1.m01 - s, v1.m10 - s, v1.m11 - s, v1.m20 - s, v1.m21 - s, v1.m30 - s, v1.m31 - s);
        }
        
        public static mat4x2 operator*(mat4x2 v1, float s) {
            return new mat4x2(v1.m00 * s, v1.m01 * s, v1.m10 * s, v1.m11 * s, v1.m20 * s, v1.m21 * s, v1.m30 * s, v1.m31 * s);
        }
        
        public static mat4x2 operator/(mat4x2 v1, float s) {
            return new mat4x2(v1.m00 / s, v1.m01 / s, v1.m10 / s, v1.m11 / s, v1.m20 / s, v1.m21 / s, v1.m30 / s, v1.m31 / s);
        }
        
        public static mat4x2 operator%(mat4x2 v1, float s) {
            return new mat4x2(v1.m00 % s, v1.m01 % s, v1.m10 % s, v1.m11 % s, v1.m20 % s, v1.m21 % s, v1.m30 % s, v1.m31 % s);
        }
        
        public static mat4x2 operator+(mat4x2 v1, mat4x2 v2) {
            return new mat4x2(v1.m00 + v2.m00, v1.m01 + v2.m01, v1.m10 + v2.m10, v1.m11 + v2.m11, v1.m20 + v2.m20, v1.m21 + v2.m21, v1.m30 + v2.m30, v1.m31 + v2.m31);
        }
        
        public static mat4x2 operator-(mat4x2 v1, mat4x2 v2) {
            return new mat4x2(v1.m00 - v2.m00, v1.m01 - v2.m01, v1.m10 - v2.m10, v1.m11 - v2.m11, v1.m20 - v2.m20, v1.m21 - v2.m21, v1.m30 - v2.m30, v1.m31 - v2.m31);
        }
        
        public static mat4x2 operator/(mat4x2 v1, mat4x2 v2) {
            return new mat4x2(v1.m00 / v2.m00, v1.m01 / v2.m01, v1.m10 / v2.m10, v1.m11 / v2.m11, v1.m20 / v2.m20, v1.m21 / v2.m21, v1.m30 / v2.m30, v1.m31 / v2.m31);
        }
        
        public static mat4x2 operator%(mat4x2 v1, mat4x2 v2) {
            return new mat4x2(v1.m00 % v2.m00, v1.m01 % v2.m01, v1.m10 % v2.m10, v1.m11 % v2.m11, v1.m20 % v2.m20, v1.m21 % v2.m21, v1.m30 % v2.m30, v1.m31 % v2.m31);
        }
        
        public static bmat4x2 operator==(mat4x2 v1, mat4x2 v2) {
            return new bmat4x2(v1.m00 == v2.m00, v1.m01 == v2.m01, v1.m10 == v2.m10, v1.m11 == v2.m11, v1.m20 == v2.m20, v1.m21 == v2.m21, v1.m30 == v2.m30, v1.m31 == v2.m31);
        }
        
        public static bmat4x2 operator>(mat4x2 v1, mat4x2 v2) {
            return new bmat4x2(v1.m00 > v2.m00, v1.m01 > v2.m01, v1.m10 > v2.m10, v1.m11 > v2.m11, v1.m20 > v2.m20, v1.m21 > v2.m21, v1.m30 > v2.m30, v1.m31 > v2.m31);
        }
        
        public static bmat4x2 operator>=(mat4x2 v1, mat4x2 v2) {
            return new bmat4x2(v1.m00 >= v2.m00, v1.m01 >= v2.m01, v1.m10 >= v2.m10, v1.m11 >= v2.m11, v1.m20 >= v2.m20, v1.m21 >= v2.m21, v1.m30 >= v2.m30, v1.m31 >= v2.m31);
        }
        
        public static bmat4x2 operator<(mat4x2 v1, mat4x2 v2) {
            return new bmat4x2(v1.m00 < v2.m00, v1.m01 < v2.m01, v1.m10 < v2.m10, v1.m11 < v2.m11, v1.m20 < v2.m20, v1.m21 < v2.m21, v1.m30 < v2.m30, v1.m31 < v2.m31);
        }
        
        public static bmat4x2 operator<=(mat4x2 v1, mat4x2 v2) {
            return new bmat4x2(v1.m00 <= v2.m00, v1.m01 <= v2.m01, v1.m10 <= v2.m10, v1.m11 <= v2.m11, v1.m20 <= v2.m20, v1.m21 <= v2.m21, v1.m30 <= v2.m30, v1.m31 <= v2.m31);
        }
        
        public static bmat4x2 operator!=(mat4x2 v1, mat4x2 v2) {
            return new bmat4x2(v1.m00 != v2.m00, v1.m01 != v2.m01, v1.m10 != v2.m10, v1.m11 != v2.m11, v1.m20 != v2.m20, v1.m21 != v2.m21, v1.m30 != v2.m30, v1.m31 != v2.m31);
        }
        
        public static vec2 operator*(vec4 v0, mat4x2 m1) {
            return math.InnerProduct1(ref v0, ref m1);
        }
        
        public static vec4 operator*(mat4x2 m0, vec2 v1) {
            return math.InnerProduct1(ref m0, ref v1);
        }
        
        public static mat4x2 operator*(mat4x2 m0, mat2 m1) {
            return math.InnerProduct1(ref m0, ref m1);
        }
        
        public static mat4x3 operator*(mat4x2 m0, mat2x3 m1) {
            return math.InnerProduct1(ref m0, ref m1);
        }
        
        public static mat4 operator*(mat4x2 m0, mat2x4 m1) {
            return math.InnerProduct1(ref m0, ref m1);
        }
        
        public override bool Equals(object obj) {
            return (obj is mat4x2) ? bmath.All(this == (mat4x2)obj) : false;
        }
        
        public override int GetHashCode() {
            return this.m00.GetHashCode() ^ this.m01.GetHashCode() ^ this.m10.GetHashCode() ^ this.m11.GetHashCode() ^ this.m20.GetHashCode() ^ this.m21.GetHashCode() ^ this.m30.GetHashCode() ^ this.m31.GetHashCode();
        }
        
        public override string ToString() {
            return "mat4x2(" + this.m00 + "," + this.m01 + "," + this.m10 + "," + this.m11 + "," + this.m20 + "," + this.m21 + "," + this.m30 + "," + this.m31 + ")";
        }
    }
    
    [DataContract()]
    [StructLayout(LayoutKind.Sequential)]
    public struct mat4x3 {
        
        [DataMember()]
        internal float m00;
        
        [DataMember()]
        internal float m01;
        
        [DataMember()]
        internal float m02;
        
        [DataMember()]
        internal float m10;
        
        [DataMember()]
        internal float m11;
        
        [DataMember()]
        internal float m12;
        
        [DataMember()]
        internal float m20;
        
        [DataMember()]
        internal float m21;
        
        [DataMember()]
        internal float m22;
        
        [DataMember()]
        internal float m30;
        
        [DataMember()]
        internal float m31;
        
        [DataMember()]
        internal float m32;
        
        public mat4x3(float s) {
            this.m00 = s;
            this.m01 = 0F;
            this.m02 = 0F;
            this.m10 = 0F;
            this.m11 = s;
            this.m12 = 0F;
            this.m20 = 0F;
            this.m21 = 0F;
            this.m22 = s;
            this.m30 = 0F;
            this.m31 = 0F;
            this.m32 = 0F;
        }
        
        public mat4x3(mat4x3 m00) {
            this.m00 = m00.m00;
            this.m01 = m00.m01;
            this.m02 = m00.m02;
            this.m10 = m00.m10;
            this.m11 = m00.m11;
            this.m12 = m00.m12;
            this.m20 = m00.m20;
            this.m21 = m00.m21;
            this.m22 = m00.m22;
            this.m30 = m00.m30;
            this.m31 = m00.m31;
            this.m32 = m00.m32;
        }
        
        public mat4x3(mat2x3 m00, mat2x3 m10) {
            this.m00 = m00.m00;
            this.m01 = m00.m01;
            this.m02 = m00.m02;
            this.m10 = m00.m10;
            this.m11 = m00.m11;
            this.m12 = m00.m12;
            this.m20 = m10.m00;
            this.m21 = m10.m01;
            this.m22 = m10.m02;
            this.m30 = m10.m10;
            this.m31 = m10.m11;
            this.m32 = m10.m12;
        }
        
        public mat4x3(mat3 m00, vec3 m10) {
            this.m00 = m00.m00;
            this.m01 = m00.m01;
            this.m02 = m00.m02;
            this.m10 = m00.m10;
            this.m11 = m00.m11;
            this.m12 = m00.m12;
            this.m20 = m00.m20;
            this.m21 = m00.m21;
            this.m22 = m00.m22;
            this.m30 = m10.v0;
            this.m31 = m10.v1;
            this.m32 = m10.v2;
        }
        
        public mat4x3(vec3 m00, mat3 m10) {
            this.m00 = m00.v0;
            this.m01 = m00.v1;
            this.m02 = m00.v2;
            this.m10 = m10.m00;
            this.m11 = m10.m01;
            this.m12 = m10.m02;
            this.m20 = m10.m10;
            this.m21 = m10.m11;
            this.m22 = m10.m12;
            this.m30 = m10.m20;
            this.m31 = m10.m21;
            this.m32 = m10.m22;
        }
        
        public mat4x3(mat2x3 m00, vec3 m10, vec3 m20) {
            this.m00 = m00.m00;
            this.m01 = m00.m01;
            this.m02 = m00.m02;
            this.m10 = m00.m10;
            this.m11 = m00.m11;
            this.m12 = m00.m12;
            this.m20 = m10.v0;
            this.m21 = m10.v1;
            this.m22 = m10.v2;
            this.m30 = m20.v0;
            this.m31 = m20.v1;
            this.m32 = m20.v2;
        }
        
        public mat4x3(vec3 m00, mat2x3 m10, vec3 m20) {
            this.m00 = m00.v0;
            this.m01 = m00.v1;
            this.m02 = m00.v2;
            this.m10 = m10.m00;
            this.m11 = m10.m01;
            this.m12 = m10.m02;
            this.m20 = m10.m10;
            this.m21 = m10.m11;
            this.m22 = m10.m12;
            this.m30 = m20.v0;
            this.m31 = m20.v1;
            this.m32 = m20.v2;
        }
        
        public mat4x3(vec3 m00, vec3 m10, mat2x3 m20) {
            this.m00 = m00.v0;
            this.m01 = m00.v1;
            this.m02 = m00.v2;
            this.m10 = m10.v0;
            this.m11 = m10.v1;
            this.m12 = m10.v2;
            this.m20 = m20.m00;
            this.m21 = m20.m01;
            this.m22 = m20.m02;
            this.m30 = m20.m10;
            this.m31 = m20.m11;
            this.m32 = m20.m12;
        }
        
        public mat4x3(vec3 m00, vec3 m10, vec3 m20, vec3 m30) {
            this.m00 = m00.v0;
            this.m01 = m00.v1;
            this.m02 = m00.v2;
            this.m10 = m10.v0;
            this.m11 = m10.v1;
            this.m12 = m10.v2;
            this.m20 = m20.v0;
            this.m21 = m20.v1;
            this.m22 = m20.v2;
            this.m30 = m30.v0;
            this.m31 = m30.v1;
            this.m32 = m30.v2;
        }
        
        public mat4x3(vec2 m00, float m01, vec2 m10, float m11, vec2 m20, float m21, vec2 m30, float m31) {
            this.m00 = m00.v0;
            this.m01 = m00.v1;
            this.m02 = m01;
            this.m10 = m10.v0;
            this.m11 = m10.v1;
            this.m12 = m11;
            this.m20 = m20.v0;
            this.m21 = m20.v1;
            this.m22 = m21;
            this.m30 = m30.v0;
            this.m31 = m30.v1;
            this.m32 = m31;
        }
        
        public mat4x3(float m00, vec2 m01, float m10, vec2 m11, float m20, vec2 m21, float m30, vec2 m31) {
            this.m00 = m00;
            this.m01 = m01.v0;
            this.m02 = m01.v1;
            this.m10 = m10;
            this.m11 = m11.v0;
            this.m12 = m11.v1;
            this.m20 = m20;
            this.m21 = m21.v0;
            this.m22 = m21.v1;
            this.m30 = m30;
            this.m31 = m31.v0;
            this.m32 = m31.v1;
        }
        
        public mat4x3(float m00, float m01, float m02, float m10, float m11, float m12, float m20, float m21, float m22, float m30, float m31, float m32) {
            this.m00 = m00;
            this.m01 = m01;
            this.m02 = m02;
            this.m10 = m10;
            this.m11 = m11;
            this.m12 = m12;
            this.m20 = m20;
            this.m21 = m21;
            this.m22 = m22;
            this.m30 = m30;
            this.m31 = m31;
            this.m32 = m32;
        }
        
        public mat4x3(mat2 s) {
            this.m00 = s.m00;
            this.m01 = s.m01;
            this.m02 = 0F;
            this.m10 = s.m10;
            this.m11 = s.m11;
            this.m12 = 0F;
            this.m20 = 0F;
            this.m21 = 0F;
            this.m22 = 0F;
            this.m30 = 0F;
            this.m31 = 0F;
            this.m32 = 0F;
        }
        
        public mat4x3(mat2x3 s) {
            this.m00 = s.m00;
            this.m01 = s.m01;
            this.m02 = s.m02;
            this.m10 = s.m10;
            this.m11 = s.m11;
            this.m12 = s.m12;
            this.m20 = 0F;
            this.m21 = 0F;
            this.m22 = 0F;
            this.m30 = 0F;
            this.m31 = 0F;
            this.m32 = 0F;
        }
        
        public mat4x3(mat2x4 s) {
            this.m00 = s.m00;
            this.m01 = s.m01;
            this.m02 = s.m02;
            this.m10 = s.m10;
            this.m11 = s.m11;
            this.m12 = s.m12;
            this.m20 = 0F;
            this.m21 = 0F;
            this.m22 = 0F;
            this.m30 = 0F;
            this.m31 = 0F;
            this.m32 = 0F;
        }
        
        public mat4x3(mat3x2 s) {
            this.m00 = s.m00;
            this.m01 = s.m01;
            this.m02 = 0F;
            this.m10 = s.m10;
            this.m11 = s.m11;
            this.m12 = 0F;
            this.m20 = s.m20;
            this.m21 = s.m21;
            this.m22 = 0F;
            this.m30 = 0F;
            this.m31 = 0F;
            this.m32 = 0F;
        }
        
        public mat4x3(mat3 s) {
            this.m00 = s.m00;
            this.m01 = s.m01;
            this.m02 = s.m02;
            this.m10 = s.m10;
            this.m11 = s.m11;
            this.m12 = s.m12;
            this.m20 = s.m20;
            this.m21 = s.m21;
            this.m22 = s.m22;
            this.m30 = 0F;
            this.m31 = 0F;
            this.m32 = 0F;
        }
        
        public mat4x3(mat3x4 s) {
            this.m00 = s.m00;
            this.m01 = s.m01;
            this.m02 = s.m02;
            this.m10 = s.m10;
            this.m11 = s.m11;
            this.m12 = s.m12;
            this.m20 = s.m20;
            this.m21 = s.m21;
            this.m22 = s.m22;
            this.m30 = 0F;
            this.m31 = 0F;
            this.m32 = 0F;
        }
        
        public mat4x3(mat4x2 s) {
            this.m00 = s.m00;
            this.m01 = s.m01;
            this.m02 = 0F;
            this.m10 = s.m10;
            this.m11 = s.m11;
            this.m12 = 0F;
            this.m20 = s.m20;
            this.m21 = s.m21;
            this.m22 = 0F;
            this.m30 = s.m30;
            this.m31 = s.m31;
            this.m32 = 0F;
        }
        
        public mat4x3(mat4 s) {
            this.m00 = s.m00;
            this.m01 = s.m01;
            this.m02 = s.m02;
            this.m10 = s.m10;
            this.m11 = s.m11;
            this.m12 = s.m12;
            this.m20 = s.m20;
            this.m21 = s.m21;
            this.m22 = s.m22;
            this.m30 = s.m30;
            this.m31 = s.m31;
            this.m32 = s.m32;
        }
        
        public float this[int i0, int i1] {
            get {
                if (((i0 < 0) 
                            || (3 < i0))) {
                    throw new System.IndexOutOfRangeException("i0");
                }
                if (((i1 < 0) 
                            || (2 < i1))) {
                    throw new System.IndexOutOfRangeException("i1");
                }
                
                unsafe
                {
                    fixed (Single* v_ptr = &this.m00)
                    {
                        return *(v_ptr + (i1 + i0 * 3));
                    }
                };
            }
            set {
                if (((i0 < 0) 
                            || (3 < i0))) {
                    throw new System.IndexOutOfRangeException("i0");
                }
                if (((i1 < 0) 
                            || (2 < i1))) {
                    throw new System.IndexOutOfRangeException("i1");
                }
                
                unsafe
                {
                    fixed (Single* v_ptr = &this.m00)
                    {
                        *(v_ptr  + (i1 + i0 * 3)) = value;
                    }
                };
            }
        }
        
        public vec3 this[int i0] {
            get {
                return new vec3(this[i0, 0], this[i0, 1], this[i0, 2]);
            }
            set {
                this[i0, 0] = value[0];
                this[i0, 1] = value[1];
                this[i0, 2] = value[2];
            }
        }
        
        public static mat4x3 operator+(mat4x3 v) {
            return new mat4x3(+v.m00, +v.m01, +v.m02, +v.m10, +v.m11, +v.m12, +v.m20, +v.m21, +v.m22, +v.m30, +v.m31, +v.m32);
        }
        
        public static mat4x3 operator-(mat4x3 v) {
            return new mat4x3(-v.m00, -v.m01, -v.m02, -v.m10, -v.m11, -v.m12, -v.m20, -v.m21, -v.m22, -v.m30, -v.m31, -v.m32);
        }
        
        public static mat4x3 operator++(mat4x3 v) {
            return new mat4x3(++v.m00, ++v.m01, ++v.m02, ++v.m10, ++v.m11, ++v.m12, ++v.m20, ++v.m21, ++v.m22, ++v.m30, ++v.m31, ++v.m32);
        }
        
        public static mat4x3 operator--(mat4x3 v) {
            return new mat4x3(--v.m00, --v.m01, --v.m02, --v.m10, --v.m11, --v.m12, --v.m20, --v.m21, --v.m22, --v.m30, --v.m31, --v.m32);
        }
        
        public static mat4x3 operator+(float s, mat4x3 v2) {
            return new mat4x3(s + v2.m00, s + v2.m01, s + v2.m02, s + v2.m10, s + v2.m11, s + v2.m12, s + v2.m20, s + v2.m21, s + v2.m22, s + v2.m30, s + v2.m31, s + v2.m32);
        }
        
        public static mat4x3 operator-(float s, mat4x3 v2) {
            return new mat4x3(s - v2.m00, s - v2.m01, s - v2.m02, s - v2.m10, s - v2.m11, s - v2.m12, s - v2.m20, s - v2.m21, s - v2.m22, s - v2.m30, s - v2.m31, s - v2.m32);
        }
        
        public static mat4x3 operator*(float s, mat4x3 v2) {
            return new mat4x3(s * v2.m00, s * v2.m01, s * v2.m02, s * v2.m10, s * v2.m11, s * v2.m12, s * v2.m20, s * v2.m21, s * v2.m22, s * v2.m30, s * v2.m31, s * v2.m32);
        }
        
        public static mat4x3 operator/(float s, mat4x3 v2) {
            return new mat4x3(s / v2.m00, s / v2.m01, s / v2.m02, s / v2.m10, s / v2.m11, s / v2.m12, s / v2.m20, s / v2.m21, s / v2.m22, s / v2.m30, s / v2.m31, s / v2.m32);
        }
        
        public static mat4x3 operator%(float s, mat4x3 v2) {
            return new mat4x3(s % v2.m00, s % v2.m01, s % v2.m02, s % v2.m10, s % v2.m11, s % v2.m12, s % v2.m20, s % v2.m21, s % v2.m22, s % v2.m30, s % v2.m31, s % v2.m32);
        }
        
        public static mat4x3 operator+(mat4x3 v1, float s) {
            return new mat4x3(v1.m00 + s, v1.m01 + s, v1.m02 + s, v1.m10 + s, v1.m11 + s, v1.m12 + s, v1.m20 + s, v1.m21 + s, v1.m22 + s, v1.m30 + s, v1.m31 + s, v1.m32 + s);
        }
        
        public static mat4x3 operator-(mat4x3 v1, float s) {
            return new mat4x3(v1.m00 - s, v1.m01 - s, v1.m02 - s, v1.m10 - s, v1.m11 - s, v1.m12 - s, v1.m20 - s, v1.m21 - s, v1.m22 - s, v1.m30 - s, v1.m31 - s, v1.m32 - s);
        }
        
        public static mat4x3 operator*(mat4x3 v1, float s) {
            return new mat4x3(v1.m00 * s, v1.m01 * s, v1.m02 * s, v1.m10 * s, v1.m11 * s, v1.m12 * s, v1.m20 * s, v1.m21 * s, v1.m22 * s, v1.m30 * s, v1.m31 * s, v1.m32 * s);
        }
        
        public static mat4x3 operator/(mat4x3 v1, float s) {
            return new mat4x3(v1.m00 / s, v1.m01 / s, v1.m02 / s, v1.m10 / s, v1.m11 / s, v1.m12 / s, v1.m20 / s, v1.m21 / s, v1.m22 / s, v1.m30 / s, v1.m31 / s, v1.m32 / s);
        }
        
        public static mat4x3 operator%(mat4x3 v1, float s) {
            return new mat4x3(v1.m00 % s, v1.m01 % s, v1.m02 % s, v1.m10 % s, v1.m11 % s, v1.m12 % s, v1.m20 % s, v1.m21 % s, v1.m22 % s, v1.m30 % s, v1.m31 % s, v1.m32 % s);
        }
        
        public static mat4x3 operator+(mat4x3 v1, mat4x3 v2) {
            return new mat4x3(v1.m00 + v2.m00, v1.m01 + v2.m01, v1.m02 + v2.m02, v1.m10 + v2.m10, v1.m11 + v2.m11, v1.m12 + v2.m12, v1.m20 + v2.m20, v1.m21 + v2.m21, v1.m22 + v2.m22, v1.m30 + v2.m30, v1.m31 + v2.m31, v1.m32 + v2.m32);
        }
        
        public static mat4x3 operator-(mat4x3 v1, mat4x3 v2) {
            return new mat4x3(v1.m00 - v2.m00, v1.m01 - v2.m01, v1.m02 - v2.m02, v1.m10 - v2.m10, v1.m11 - v2.m11, v1.m12 - v2.m12, v1.m20 - v2.m20, v1.m21 - v2.m21, v1.m22 - v2.m22, v1.m30 - v2.m30, v1.m31 - v2.m31, v1.m32 - v2.m32);
        }
        
        public static mat4x3 operator/(mat4x3 v1, mat4x3 v2) {
            return new mat4x3(v1.m00 / v2.m00, v1.m01 / v2.m01, v1.m02 / v2.m02, v1.m10 / v2.m10, v1.m11 / v2.m11, v1.m12 / v2.m12, v1.m20 / v2.m20, v1.m21 / v2.m21, v1.m22 / v2.m22, v1.m30 / v2.m30, v1.m31 / v2.m31, v1.m32 / v2.m32);
        }
        
        public static mat4x3 operator%(mat4x3 v1, mat4x3 v2) {
            return new mat4x3(v1.m00 % v2.m00, v1.m01 % v2.m01, v1.m02 % v2.m02, v1.m10 % v2.m10, v1.m11 % v2.m11, v1.m12 % v2.m12, v1.m20 % v2.m20, v1.m21 % v2.m21, v1.m22 % v2.m22, v1.m30 % v2.m30, v1.m31 % v2.m31, v1.m32 % v2.m32);
        }
        
        public static bmat4x3 operator==(mat4x3 v1, mat4x3 v2) {
            return new bmat4x3(v1.m00 == v2.m00, v1.m01 == v2.m01, v1.m02 == v2.m02, v1.m10 == v2.m10, v1.m11 == v2.m11, v1.m12 == v2.m12, v1.m20 == v2.m20, v1.m21 == v2.m21, v1.m22 == v2.m22, v1.m30 == v2.m30, v1.m31 == v2.m31, v1.m32 == v2.m32);
        }
        
        public static bmat4x3 operator>(mat4x3 v1, mat4x3 v2) {
            return new bmat4x3(v1.m00 > v2.m00, v1.m01 > v2.m01, v1.m02 > v2.m02, v1.m10 > v2.m10, v1.m11 > v2.m11, v1.m12 > v2.m12, v1.m20 > v2.m20, v1.m21 > v2.m21, v1.m22 > v2.m22, v1.m30 > v2.m30, v1.m31 > v2.m31, v1.m32 > v2.m32);
        }
        
        public static bmat4x3 operator>=(mat4x3 v1, mat4x3 v2) {
            return new bmat4x3(v1.m00 >= v2.m00, v1.m01 >= v2.m01, v1.m02 >= v2.m02, v1.m10 >= v2.m10, v1.m11 >= v2.m11, v1.m12 >= v2.m12, v1.m20 >= v2.m20, v1.m21 >= v2.m21, v1.m22 >= v2.m22, v1.m30 >= v2.m30, v1.m31 >= v2.m31, v1.m32 >= v2.m32);
        }
        
        public static bmat4x3 operator<(mat4x3 v1, mat4x3 v2) {
            return new bmat4x3(v1.m00 < v2.m00, v1.m01 < v2.m01, v1.m02 < v2.m02, v1.m10 < v2.m10, v1.m11 < v2.m11, v1.m12 < v2.m12, v1.m20 < v2.m20, v1.m21 < v2.m21, v1.m22 < v2.m22, v1.m30 < v2.m30, v1.m31 < v2.m31, v1.m32 < v2.m32);
        }
        
        public static bmat4x3 operator<=(mat4x3 v1, mat4x3 v2) {
            return new bmat4x3(v1.m00 <= v2.m00, v1.m01 <= v2.m01, v1.m02 <= v2.m02, v1.m10 <= v2.m10, v1.m11 <= v2.m11, v1.m12 <= v2.m12, v1.m20 <= v2.m20, v1.m21 <= v2.m21, v1.m22 <= v2.m22, v1.m30 <= v2.m30, v1.m31 <= v2.m31, v1.m32 <= v2.m32);
        }
        
        public static bmat4x3 operator!=(mat4x3 v1, mat4x3 v2) {
            return new bmat4x3(v1.m00 != v2.m00, v1.m01 != v2.m01, v1.m02 != v2.m02, v1.m10 != v2.m10, v1.m11 != v2.m11, v1.m12 != v2.m12, v1.m20 != v2.m20, v1.m21 != v2.m21, v1.m22 != v2.m22, v1.m30 != v2.m30, v1.m31 != v2.m31, v1.m32 != v2.m32);
        }
        
        public static vec3 operator*(vec4 v0, mat4x3 m1) {
            return math.InnerProduct1(ref v0, ref m1);
        }
        
        public static vec4 operator*(mat4x3 m0, vec3 v1) {
            return math.InnerProduct1(ref m0, ref v1);
        }
        
        public static mat4x2 operator*(mat4x3 m0, mat3x2 m1) {
            return math.InnerProduct1(ref m0, ref m1);
        }
        
        public static mat4x3 operator*(mat4x3 m0, mat3 m1) {
            return math.InnerProduct1(ref m0, ref m1);
        }
        
        public static mat4 operator*(mat4x3 m0, mat3x4 m1) {
            return math.InnerProduct1(ref m0, ref m1);
        }
        
        public override bool Equals(object obj) {
            return (obj is mat4x3) ? bmath.All(this == (mat4x3)obj) : false;
        }
        
        public override int GetHashCode() {
            return this.m00.GetHashCode() ^ this.m01.GetHashCode() ^ this.m02.GetHashCode() ^ this.m10.GetHashCode() ^ this.m11.GetHashCode() ^ this.m12.GetHashCode() ^ this.m20.GetHashCode() ^ this.m21.GetHashCode() ^ this.m22.GetHashCode() ^ this.m30.GetHashCode() ^ this.m31.GetHashCode() ^ this.m32.GetHashCode();
        }
        
        public override string ToString() {
            return "mat4x3(" + this.m00 + "," + this.m01 + "," + this.m02 + "," + this.m10 + "," + this.m11 + "," + this.m12 + "," + this.m20 + "," + this.m21 + "," + this.m22 + "," + this.m30 + "," + this.m31 + "," + this.m32 + ")";
        }
    }
    
    [DataContract()]
    [StructLayout(LayoutKind.Sequential)]
    public struct mat4 {
        
        [DataMember()]
        internal float m00;
        
        [DataMember()]
        internal float m01;
        
        [DataMember()]
        internal float m02;
        
        [DataMember()]
        internal float m03;
        
        [DataMember()]
        internal float m10;
        
        [DataMember()]
        internal float m11;
        
        [DataMember()]
        internal float m12;
        
        [DataMember()]
        internal float m13;
        
        [DataMember()]
        internal float m20;
        
        [DataMember()]
        internal float m21;
        
        [DataMember()]
        internal float m22;
        
        [DataMember()]
        internal float m23;
        
        [DataMember()]
        internal float m30;
        
        [DataMember()]
        internal float m31;
        
        [DataMember()]
        internal float m32;
        
        [DataMember()]
        internal float m33;
        
        public mat4(float s) {
            this.m00 = s;
            this.m01 = 0F;
            this.m02 = 0F;
            this.m03 = 0F;
            this.m10 = 0F;
            this.m11 = s;
            this.m12 = 0F;
            this.m13 = 0F;
            this.m20 = 0F;
            this.m21 = 0F;
            this.m22 = s;
            this.m23 = 0F;
            this.m30 = 0F;
            this.m31 = 0F;
            this.m32 = 0F;
            this.m33 = s;
        }
        
        public mat4(mat4 m00) {
            this.m00 = m00.m00;
            this.m01 = m00.m01;
            this.m02 = m00.m02;
            this.m03 = m00.m03;
            this.m10 = m00.m10;
            this.m11 = m00.m11;
            this.m12 = m00.m12;
            this.m13 = m00.m13;
            this.m20 = m00.m20;
            this.m21 = m00.m21;
            this.m22 = m00.m22;
            this.m23 = m00.m23;
            this.m30 = m00.m30;
            this.m31 = m00.m31;
            this.m32 = m00.m32;
            this.m33 = m00.m33;
        }
        
        public mat4(mat4x2 m00, mat4x2 m01) {
            this.m00 = m00.m00;
            this.m01 = m00.m01;
            this.m02 = m01.m00;
            this.m03 = m01.m01;
            this.m10 = m00.m10;
            this.m11 = m00.m11;
            this.m12 = m01.m10;
            this.m13 = m01.m11;
            this.m20 = m00.m20;
            this.m21 = m00.m21;
            this.m22 = m01.m20;
            this.m23 = m01.m21;
            this.m30 = m00.m30;
            this.m31 = m00.m31;
            this.m32 = m01.m30;
            this.m33 = m01.m31;
        }
        
        public mat4(mat2x4 m00, mat2x4 m10) {
            this.m00 = m00.m00;
            this.m01 = m00.m01;
            this.m02 = m00.m02;
            this.m03 = m00.m03;
            this.m10 = m00.m10;
            this.m11 = m00.m11;
            this.m12 = m00.m12;
            this.m13 = m00.m13;
            this.m20 = m10.m00;
            this.m21 = m10.m01;
            this.m22 = m10.m02;
            this.m23 = m10.m03;
            this.m30 = m10.m10;
            this.m31 = m10.m11;
            this.m32 = m10.m12;
            this.m33 = m10.m13;
        }
        
        public mat4(mat2 m00, mat2 m01, mat2 m10, mat2 m11) {
            this.m00 = m00.m00;
            this.m01 = m00.m01;
            this.m02 = m01.m00;
            this.m03 = m01.m01;
            this.m10 = m00.m10;
            this.m11 = m00.m11;
            this.m12 = m01.m10;
            this.m13 = m01.m11;
            this.m20 = m10.m00;
            this.m21 = m10.m01;
            this.m22 = m11.m00;
            this.m23 = m11.m01;
            this.m30 = m10.m10;
            this.m31 = m10.m11;
            this.m32 = m11.m10;
            this.m33 = m11.m11;
        }
        
        public mat4(mat3x4 m00, vec4 m10) {
            this.m00 = m00.m00;
            this.m01 = m00.m01;
            this.m02 = m00.m02;
            this.m03 = m00.m03;
            this.m10 = m00.m10;
            this.m11 = m00.m11;
            this.m12 = m00.m12;
            this.m13 = m00.m13;
            this.m20 = m00.m20;
            this.m21 = m00.m21;
            this.m22 = m00.m22;
            this.m23 = m00.m23;
            this.m30 = m10.v0;
            this.m31 = m10.v1;
            this.m32 = m10.v2;
            this.m33 = m10.v3;
        }
        
        public mat4(mat3x2 m00, mat3x2 m01, vec2 m10, vec2 m11) {
            this.m00 = m00.m00;
            this.m01 = m00.m01;
            this.m02 = m01.m00;
            this.m03 = m01.m01;
            this.m10 = m00.m10;
            this.m11 = m00.m11;
            this.m12 = m01.m10;
            this.m13 = m01.m11;
            this.m20 = m00.m20;
            this.m21 = m00.m21;
            this.m22 = m01.m20;
            this.m23 = m01.m21;
            this.m30 = m10.v0;
            this.m31 = m10.v1;
            this.m32 = m11.v0;
            this.m33 = m11.v1;
        }
        
        public mat4(vec4 m00, mat3x4 m10) {
            this.m00 = m00.v0;
            this.m01 = m00.v1;
            this.m02 = m00.v2;
            this.m03 = m00.v3;
            this.m10 = m10.m00;
            this.m11 = m10.m01;
            this.m12 = m10.m02;
            this.m13 = m10.m03;
            this.m20 = m10.m10;
            this.m21 = m10.m11;
            this.m22 = m10.m12;
            this.m23 = m10.m13;
            this.m30 = m10.m20;
            this.m31 = m10.m21;
            this.m32 = m10.m22;
            this.m33 = m10.m23;
        }
        
        public mat4(vec2 m00, vec2 m01, mat3x2 m10, mat3x2 m11) {
            this.m00 = m00.v0;
            this.m01 = m00.v1;
            this.m02 = m01.v0;
            this.m03 = m01.v1;
            this.m10 = m10.m00;
            this.m11 = m10.m01;
            this.m12 = m11.m00;
            this.m13 = m11.m01;
            this.m20 = m10.m10;
            this.m21 = m10.m11;
            this.m22 = m11.m10;
            this.m23 = m11.m11;
            this.m30 = m10.m20;
            this.m31 = m10.m21;
            this.m32 = m11.m20;
            this.m33 = m11.m21;
        }
        
        public mat4(mat2x4 m00, vec4 m10, vec4 m20) {
            this.m00 = m00.m00;
            this.m01 = m00.m01;
            this.m02 = m00.m02;
            this.m03 = m00.m03;
            this.m10 = m00.m10;
            this.m11 = m00.m11;
            this.m12 = m00.m12;
            this.m13 = m00.m13;
            this.m20 = m10.v0;
            this.m21 = m10.v1;
            this.m22 = m10.v2;
            this.m23 = m10.v3;
            this.m30 = m20.v0;
            this.m31 = m20.v1;
            this.m32 = m20.v2;
            this.m33 = m20.v3;
        }
        
        public mat4(mat2 m00, mat2 m01, vec2 m10, vec2 m11, vec2 m20, vec2 m21) {
            this.m00 = m00.m00;
            this.m01 = m00.m01;
            this.m02 = m01.m00;
            this.m03 = m01.m01;
            this.m10 = m00.m10;
            this.m11 = m00.m11;
            this.m12 = m01.m10;
            this.m13 = m01.m11;
            this.m20 = m10.v0;
            this.m21 = m10.v1;
            this.m22 = m11.v0;
            this.m23 = m11.v1;
            this.m30 = m20.v0;
            this.m31 = m20.v1;
            this.m32 = m21.v0;
            this.m33 = m21.v1;
        }
        
        public mat4(vec4 m00, mat2x4 m10, vec4 m20) {
            this.m00 = m00.v0;
            this.m01 = m00.v1;
            this.m02 = m00.v2;
            this.m03 = m00.v3;
            this.m10 = m10.m00;
            this.m11 = m10.m01;
            this.m12 = m10.m02;
            this.m13 = m10.m03;
            this.m20 = m10.m10;
            this.m21 = m10.m11;
            this.m22 = m10.m12;
            this.m23 = m10.m13;
            this.m30 = m20.v0;
            this.m31 = m20.v1;
            this.m32 = m20.v2;
            this.m33 = m20.v3;
        }
        
        public mat4(vec2 m00, vec2 m01, mat2 m10, mat2 m11, vec2 m20, vec2 m21) {
            this.m00 = m00.v0;
            this.m01 = m00.v1;
            this.m02 = m01.v0;
            this.m03 = m01.v1;
            this.m10 = m10.m00;
            this.m11 = m10.m01;
            this.m12 = m11.m00;
            this.m13 = m11.m01;
            this.m20 = m10.m10;
            this.m21 = m10.m11;
            this.m22 = m11.m10;
            this.m23 = m11.m11;
            this.m30 = m20.v0;
            this.m31 = m20.v1;
            this.m32 = m21.v0;
            this.m33 = m21.v1;
        }
        
        public mat4(vec4 m00, vec4 m10, mat2x4 m20) {
            this.m00 = m00.v0;
            this.m01 = m00.v1;
            this.m02 = m00.v2;
            this.m03 = m00.v3;
            this.m10 = m10.v0;
            this.m11 = m10.v1;
            this.m12 = m10.v2;
            this.m13 = m10.v3;
            this.m20 = m20.m00;
            this.m21 = m20.m01;
            this.m22 = m20.m02;
            this.m23 = m20.m03;
            this.m30 = m20.m10;
            this.m31 = m20.m11;
            this.m32 = m20.m12;
            this.m33 = m20.m13;
        }
        
        public mat4(vec2 m00, vec2 m01, vec2 m10, vec2 m11, mat2 m20, mat2 m21) {
            this.m00 = m00.v0;
            this.m01 = m00.v1;
            this.m02 = m01.v0;
            this.m03 = m01.v1;
            this.m10 = m10.v0;
            this.m11 = m10.v1;
            this.m12 = m11.v0;
            this.m13 = m11.v1;
            this.m20 = m20.m00;
            this.m21 = m20.m01;
            this.m22 = m21.m00;
            this.m23 = m21.m01;
            this.m30 = m20.m10;
            this.m31 = m20.m11;
            this.m32 = m21.m10;
            this.m33 = m21.m11;
        }
        
        public mat4(vec4 m00, vec4 m10, vec4 m20, vec4 m30) {
            this.m00 = m00.v0;
            this.m01 = m00.v1;
            this.m02 = m00.v2;
            this.m03 = m00.v3;
            this.m10 = m10.v0;
            this.m11 = m10.v1;
            this.m12 = m10.v2;
            this.m13 = m10.v3;
            this.m20 = m20.v0;
            this.m21 = m20.v1;
            this.m22 = m20.v2;
            this.m23 = m20.v3;
            this.m30 = m30.v0;
            this.m31 = m30.v1;
            this.m32 = m30.v2;
            this.m33 = m30.v3;
        }
        
        public mat4(vec2 m00, vec2 m01, vec2 m10, vec2 m11, vec2 m20, vec2 m21, vec2 m30, vec2 m31) {
            this.m00 = m00.v0;
            this.m01 = m00.v1;
            this.m02 = m01.v0;
            this.m03 = m01.v1;
            this.m10 = m10.v0;
            this.m11 = m10.v1;
            this.m12 = m11.v0;
            this.m13 = m11.v1;
            this.m20 = m20.v0;
            this.m21 = m20.v1;
            this.m22 = m21.v0;
            this.m23 = m21.v1;
            this.m30 = m30.v0;
            this.m31 = m30.v1;
            this.m32 = m31.v0;
            this.m33 = m31.v1;
        }
        
        public mat4(vec3 m00, float m01, vec3 m10, float m11, vec3 m20, float m21, vec3 m30, float m31) {
            this.m00 = m00.v0;
            this.m01 = m00.v1;
            this.m02 = m00.v2;
            this.m03 = m01;
            this.m10 = m10.v0;
            this.m11 = m10.v1;
            this.m12 = m10.v2;
            this.m13 = m11;
            this.m20 = m20.v0;
            this.m21 = m20.v1;
            this.m22 = m20.v2;
            this.m23 = m21;
            this.m30 = m30.v0;
            this.m31 = m30.v1;
            this.m32 = m30.v2;
            this.m33 = m31;
        }
        
        public mat4(float m00, vec3 m01, float m10, vec3 m11, float m20, vec3 m21, float m30, vec3 m31) {
            this.m00 = m00;
            this.m01 = m01.v0;
            this.m02 = m01.v1;
            this.m03 = m01.v2;
            this.m10 = m10;
            this.m11 = m11.v0;
            this.m12 = m11.v1;
            this.m13 = m11.v2;
            this.m20 = m20;
            this.m21 = m21.v0;
            this.m22 = m21.v1;
            this.m23 = m21.v2;
            this.m30 = m30;
            this.m31 = m31.v0;
            this.m32 = m31.v1;
            this.m33 = m31.v2;
        }
        
        public mat4(vec2 m00, float m01, float m02, vec2 m10, float m11, float m12, vec2 m20, float m21, float m22, vec2 m30, float m31, float m32) {
            this.m00 = m00.v0;
            this.m01 = m00.v1;
            this.m02 = m01;
            this.m03 = m02;
            this.m10 = m10.v0;
            this.m11 = m10.v1;
            this.m12 = m11;
            this.m13 = m12;
            this.m20 = m20.v0;
            this.m21 = m20.v1;
            this.m22 = m21;
            this.m23 = m22;
            this.m30 = m30.v0;
            this.m31 = m30.v1;
            this.m32 = m31;
            this.m33 = m32;
        }
        
        public mat4(float m00, vec2 m01, float m02, float m10, vec2 m11, float m12, float m20, vec2 m21, float m22, float m30, vec2 m31, float m32) {
            this.m00 = m00;
            this.m01 = m01.v0;
            this.m02 = m01.v1;
            this.m03 = m02;
            this.m10 = m10;
            this.m11 = m11.v0;
            this.m12 = m11.v1;
            this.m13 = m12;
            this.m20 = m20;
            this.m21 = m21.v0;
            this.m22 = m21.v1;
            this.m23 = m22;
            this.m30 = m30;
            this.m31 = m31.v0;
            this.m32 = m31.v1;
            this.m33 = m32;
        }
        
        public mat4(float m00, float m01, vec2 m02, float m10, float m11, vec2 m12, float m20, float m21, vec2 m22, float m30, float m31, vec2 m32) {
            this.m00 = m00;
            this.m01 = m01;
            this.m02 = m02.v0;
            this.m03 = m02.v1;
            this.m10 = m10;
            this.m11 = m11;
            this.m12 = m12.v0;
            this.m13 = m12.v1;
            this.m20 = m20;
            this.m21 = m21;
            this.m22 = m22.v0;
            this.m23 = m22.v1;
            this.m30 = m30;
            this.m31 = m31;
            this.m32 = m32.v0;
            this.m33 = m32.v1;
        }
        
        public mat4(
                    float m00, 
                    float m01, 
                    float m02, 
                    float m03, 
                    float m10, 
                    float m11, 
                    float m12, 
                    float m13, 
                    float m20, 
                    float m21, 
                    float m22, 
                    float m23, 
                    float m30, 
                    float m31, 
                    float m32, 
                    float m33) {
            this.m00 = m00;
            this.m01 = m01;
            this.m02 = m02;
            this.m03 = m03;
            this.m10 = m10;
            this.m11 = m11;
            this.m12 = m12;
            this.m13 = m13;
            this.m20 = m20;
            this.m21 = m21;
            this.m22 = m22;
            this.m23 = m23;
            this.m30 = m30;
            this.m31 = m31;
            this.m32 = m32;
            this.m33 = m33;
        }
        
        public mat4(mat2 s) {
            this.m00 = s.m00;
            this.m01 = s.m01;
            this.m02 = 0F;
            this.m03 = 0F;
            this.m10 = s.m10;
            this.m11 = s.m11;
            this.m12 = 0F;
            this.m13 = 0F;
            this.m20 = 0F;
            this.m21 = 0F;
            this.m22 = 0F;
            this.m23 = 0F;
            this.m30 = 0F;
            this.m31 = 0F;
            this.m32 = 0F;
            this.m33 = 0F;
        }
        
        public mat4(mat2x3 s) {
            this.m00 = s.m00;
            this.m01 = s.m01;
            this.m02 = s.m02;
            this.m03 = 0F;
            this.m10 = s.m10;
            this.m11 = s.m11;
            this.m12 = s.m12;
            this.m13 = 0F;
            this.m20 = 0F;
            this.m21 = 0F;
            this.m22 = 0F;
            this.m23 = 0F;
            this.m30 = 0F;
            this.m31 = 0F;
            this.m32 = 0F;
            this.m33 = 0F;
        }
        
        public mat4(mat2x4 s) {
            this.m00 = s.m00;
            this.m01 = s.m01;
            this.m02 = s.m02;
            this.m03 = s.m03;
            this.m10 = s.m10;
            this.m11 = s.m11;
            this.m12 = s.m12;
            this.m13 = s.m13;
            this.m20 = 0F;
            this.m21 = 0F;
            this.m22 = 0F;
            this.m23 = 0F;
            this.m30 = 0F;
            this.m31 = 0F;
            this.m32 = 0F;
            this.m33 = 0F;
        }
        
        public mat4(mat3x2 s) {
            this.m00 = s.m00;
            this.m01 = s.m01;
            this.m02 = 0F;
            this.m03 = 0F;
            this.m10 = s.m10;
            this.m11 = s.m11;
            this.m12 = 0F;
            this.m13 = 0F;
            this.m20 = s.m20;
            this.m21 = s.m21;
            this.m22 = 0F;
            this.m23 = 0F;
            this.m30 = 0F;
            this.m31 = 0F;
            this.m32 = 0F;
            this.m33 = 0F;
        }
        
        public mat4(mat3 s) {
            this.m00 = s.m00;
            this.m01 = s.m01;
            this.m02 = s.m02;
            this.m03 = 0F;
            this.m10 = s.m10;
            this.m11 = s.m11;
            this.m12 = s.m12;
            this.m13 = 0F;
            this.m20 = s.m20;
            this.m21 = s.m21;
            this.m22 = s.m22;
            this.m23 = 0F;
            this.m30 = 0F;
            this.m31 = 0F;
            this.m32 = 0F;
            this.m33 = 0F;
        }
        
        public mat4(mat3x4 s) {
            this.m00 = s.m00;
            this.m01 = s.m01;
            this.m02 = s.m02;
            this.m03 = s.m03;
            this.m10 = s.m10;
            this.m11 = s.m11;
            this.m12 = s.m12;
            this.m13 = s.m13;
            this.m20 = s.m20;
            this.m21 = s.m21;
            this.m22 = s.m22;
            this.m23 = s.m23;
            this.m30 = 0F;
            this.m31 = 0F;
            this.m32 = 0F;
            this.m33 = 0F;
        }
        
        public mat4(mat4x2 s) {
            this.m00 = s.m00;
            this.m01 = s.m01;
            this.m02 = 0F;
            this.m03 = 0F;
            this.m10 = s.m10;
            this.m11 = s.m11;
            this.m12 = 0F;
            this.m13 = 0F;
            this.m20 = s.m20;
            this.m21 = s.m21;
            this.m22 = 0F;
            this.m23 = 0F;
            this.m30 = s.m30;
            this.m31 = s.m31;
            this.m32 = 0F;
            this.m33 = 0F;
        }
        
        public mat4(mat4x3 s) {
            this.m00 = s.m00;
            this.m01 = s.m01;
            this.m02 = s.m02;
            this.m03 = 0F;
            this.m10 = s.m10;
            this.m11 = s.m11;
            this.m12 = s.m12;
            this.m13 = 0F;
            this.m20 = s.m20;
            this.m21 = s.m21;
            this.m22 = s.m22;
            this.m23 = 0F;
            this.m30 = s.m30;
            this.m31 = s.m31;
            this.m32 = s.m32;
            this.m33 = 0F;
        }
        
        public float this[int i0, int i1] {
            get {
                if (((i0 < 0) 
                            || (3 < i0))) {
                    throw new System.IndexOutOfRangeException("i0");
                }
                if (((i1 < 0) 
                            || (3 < i1))) {
                    throw new System.IndexOutOfRangeException("i1");
                }
                
                unsafe
                {
                    fixed (Single* v_ptr = &this.m00)
                    {
                        return *(v_ptr + (i1 + i0 * 4));
                    }
                };
            }
            set {
                if (((i0 < 0) 
                            || (3 < i0))) {
                    throw new System.IndexOutOfRangeException("i0");
                }
                if (((i1 < 0) 
                            || (3 < i1))) {
                    throw new System.IndexOutOfRangeException("i1");
                }
                
                unsafe
                {
                    fixed (Single* v_ptr = &this.m00)
                    {
                        *(v_ptr  + (i1 + i0 * 4)) = value;
                    }
                };
            }
        }
        
        public vec4 this[int i0] {
            get {
                return new vec4(this[i0, 0], this[i0, 1], this[i0, 2], this[i0, 3]);
            }
            set {
                this[i0, 0] = value[0];
                this[i0, 1] = value[1];
                this[i0, 2] = value[2];
                this[i0, 3] = value[3];
            }
        }
        
        public static mat4 operator+(mat4 v) {
            return new mat4(+v.m00, +v.m01, +v.m02, +v.m03, +v.m10, +v.m11, +v.m12, +v.m13, +v.m20, +v.m21, +v.m22, +v.m23, +v.m30, +v.m31, +v.m32, +v.m33);
        }
        
        public static mat4 operator-(mat4 v) {
            return new mat4(-v.m00, -v.m01, -v.m02, -v.m03, -v.m10, -v.m11, -v.m12, -v.m13, -v.m20, -v.m21, -v.m22, -v.m23, -v.m30, -v.m31, -v.m32, -v.m33);
        }
        
        public static mat4 operator++(mat4 v) {
            return new mat4(++v.m00, ++v.m01, ++v.m02, ++v.m03, ++v.m10, ++v.m11, ++v.m12, ++v.m13, ++v.m20, ++v.m21, ++v.m22, ++v.m23, ++v.m30, ++v.m31, ++v.m32, ++v.m33);
        }
        
        public static mat4 operator--(mat4 v) {
            return new mat4(--v.m00, --v.m01, --v.m02, --v.m03, --v.m10, --v.m11, --v.m12, --v.m13, --v.m20, --v.m21, --v.m22, --v.m23, --v.m30, --v.m31, --v.m32, --v.m33);
        }
        
        public static mat4 operator+(float s, mat4 v2) {
            return new mat4(s + v2.m00, s + v2.m01, s + v2.m02, s + v2.m03, s + v2.m10, s + v2.m11, s + v2.m12, s + v2.m13, s + v2.m20, s + v2.m21, s + v2.m22, s + v2.m23, s + v2.m30, s + v2.m31, s + v2.m32, s + v2.m33);
        }
        
        public static mat4 operator-(float s, mat4 v2) {
            return new mat4(s - v2.m00, s - v2.m01, s - v2.m02, s - v2.m03, s - v2.m10, s - v2.m11, s - v2.m12, s - v2.m13, s - v2.m20, s - v2.m21, s - v2.m22, s - v2.m23, s - v2.m30, s - v2.m31, s - v2.m32, s - v2.m33);
        }
        
        public static mat4 operator*(float s, mat4 v2) {
            return new mat4(s * v2.m00, s * v2.m01, s * v2.m02, s * v2.m03, s * v2.m10, s * v2.m11, s * v2.m12, s * v2.m13, s * v2.m20, s * v2.m21, s * v2.m22, s * v2.m23, s * v2.m30, s * v2.m31, s * v2.m32, s * v2.m33);
        }
        
        public static mat4 operator/(float s, mat4 v2) {
            return new mat4(s / v2.m00, s / v2.m01, s / v2.m02, s / v2.m03, s / v2.m10, s / v2.m11, s / v2.m12, s / v2.m13, s / v2.m20, s / v2.m21, s / v2.m22, s / v2.m23, s / v2.m30, s / v2.m31, s / v2.m32, s / v2.m33);
        }
        
        public static mat4 operator%(float s, mat4 v2) {
            return new mat4(s % v2.m00, s % v2.m01, s % v2.m02, s % v2.m03, s % v2.m10, s % v2.m11, s % v2.m12, s % v2.m13, s % v2.m20, s % v2.m21, s % v2.m22, s % v2.m23, s % v2.m30, s % v2.m31, s % v2.m32, s % v2.m33);
        }
        
        public static mat4 operator+(mat4 v1, float s) {
            return new mat4(v1.m00 + s, v1.m01 + s, v1.m02 + s, v1.m03 + s, v1.m10 + s, v1.m11 + s, v1.m12 + s, v1.m13 + s, v1.m20 + s, v1.m21 + s, v1.m22 + s, v1.m23 + s, v1.m30 + s, v1.m31 + s, v1.m32 + s, v1.m33 + s);
        }
        
        public static mat4 operator-(mat4 v1, float s) {
            return new mat4(v1.m00 - s, v1.m01 - s, v1.m02 - s, v1.m03 - s, v1.m10 - s, v1.m11 - s, v1.m12 - s, v1.m13 - s, v1.m20 - s, v1.m21 - s, v1.m22 - s, v1.m23 - s, v1.m30 - s, v1.m31 - s, v1.m32 - s, v1.m33 - s);
        }
        
        public static mat4 operator*(mat4 v1, float s) {
            return new mat4(v1.m00 * s, v1.m01 * s, v1.m02 * s, v1.m03 * s, v1.m10 * s, v1.m11 * s, v1.m12 * s, v1.m13 * s, v1.m20 * s, v1.m21 * s, v1.m22 * s, v1.m23 * s, v1.m30 * s, v1.m31 * s, v1.m32 * s, v1.m33 * s);
        }
        
        public static mat4 operator/(mat4 v1, float s) {
            return new mat4(v1.m00 / s, v1.m01 / s, v1.m02 / s, v1.m03 / s, v1.m10 / s, v1.m11 / s, v1.m12 / s, v1.m13 / s, v1.m20 / s, v1.m21 / s, v1.m22 / s, v1.m23 / s, v1.m30 / s, v1.m31 / s, v1.m32 / s, v1.m33 / s);
        }
        
        public static mat4 operator%(mat4 v1, float s) {
            return new mat4(v1.m00 % s, v1.m01 % s, v1.m02 % s, v1.m03 % s, v1.m10 % s, v1.m11 % s, v1.m12 % s, v1.m13 % s, v1.m20 % s, v1.m21 % s, v1.m22 % s, v1.m23 % s, v1.m30 % s, v1.m31 % s, v1.m32 % s, v1.m33 % s);
        }
        
        public static mat4 operator+(mat4 v1, mat4 v2) {
            return new mat4(v1.m00 + v2.m00, v1.m01 + v2.m01, v1.m02 + v2.m02, v1.m03 + v2.m03, v1.m10 + v2.m10, v1.m11 + v2.m11, v1.m12 + v2.m12, v1.m13 + v2.m13, v1.m20 + v2.m20, v1.m21 + v2.m21, v1.m22 + v2.m22, v1.m23 + v2.m23, v1.m30 + v2.m30, v1.m31 + v2.m31, v1.m32 + v2.m32, v1.m33 + v2.m33);
        }
        
        public static mat4 operator-(mat4 v1, mat4 v2) {
            return new mat4(v1.m00 - v2.m00, v1.m01 - v2.m01, v1.m02 - v2.m02, v1.m03 - v2.m03, v1.m10 - v2.m10, v1.m11 - v2.m11, v1.m12 - v2.m12, v1.m13 - v2.m13, v1.m20 - v2.m20, v1.m21 - v2.m21, v1.m22 - v2.m22, v1.m23 - v2.m23, v1.m30 - v2.m30, v1.m31 - v2.m31, v1.m32 - v2.m32, v1.m33 - v2.m33);
        }
        
        public static mat4 operator/(mat4 v1, mat4 v2) {
            return new mat4(v1.m00 / v2.m00, v1.m01 / v2.m01, v1.m02 / v2.m02, v1.m03 / v2.m03, v1.m10 / v2.m10, v1.m11 / v2.m11, v1.m12 / v2.m12, v1.m13 / v2.m13, v1.m20 / v2.m20, v1.m21 / v2.m21, v1.m22 / v2.m22, v1.m23 / v2.m23, v1.m30 / v2.m30, v1.m31 / v2.m31, v1.m32 / v2.m32, v1.m33 / v2.m33);
        }
        
        public static mat4 operator%(mat4 v1, mat4 v2) {
            return new mat4(v1.m00 % v2.m00, v1.m01 % v2.m01, v1.m02 % v2.m02, v1.m03 % v2.m03, v1.m10 % v2.m10, v1.m11 % v2.m11, v1.m12 % v2.m12, v1.m13 % v2.m13, v1.m20 % v2.m20, v1.m21 % v2.m21, v1.m22 % v2.m22, v1.m23 % v2.m23, v1.m30 % v2.m30, v1.m31 % v2.m31, v1.m32 % v2.m32, v1.m33 % v2.m33);
        }
        
        public static bmat4 operator==(mat4 v1, mat4 v2) {
            return new bmat4(v1.m00 == v2.m00, v1.m01 == v2.m01, v1.m02 == v2.m02, v1.m03 == v2.m03, v1.m10 == v2.m10, v1.m11 == v2.m11, v1.m12 == v2.m12, v1.m13 == v2.m13, v1.m20 == v2.m20, v1.m21 == v2.m21, v1.m22 == v2.m22, v1.m23 == v2.m23, v1.m30 == v2.m30, v1.m31 == v2.m31, v1.m32 == v2.m32, v1.m33 == v2.m33);
        }
        
        public static bmat4 operator>(mat4 v1, mat4 v2) {
            return new bmat4(v1.m00 > v2.m00, v1.m01 > v2.m01, v1.m02 > v2.m02, v1.m03 > v2.m03, v1.m10 > v2.m10, v1.m11 > v2.m11, v1.m12 > v2.m12, v1.m13 > v2.m13, v1.m20 > v2.m20, v1.m21 > v2.m21, v1.m22 > v2.m22, v1.m23 > v2.m23, v1.m30 > v2.m30, v1.m31 > v2.m31, v1.m32 > v2.m32, v1.m33 > v2.m33);
        }
        
        public static bmat4 operator>=(mat4 v1, mat4 v2) {
            return new bmat4(v1.m00 >= v2.m00, v1.m01 >= v2.m01, v1.m02 >= v2.m02, v1.m03 >= v2.m03, v1.m10 >= v2.m10, v1.m11 >= v2.m11, v1.m12 >= v2.m12, v1.m13 >= v2.m13, v1.m20 >= v2.m20, v1.m21 >= v2.m21, v1.m22 >= v2.m22, v1.m23 >= v2.m23, v1.m30 >= v2.m30, v1.m31 >= v2.m31, v1.m32 >= v2.m32, v1.m33 >= v2.m33);
        }
        
        public static bmat4 operator<(mat4 v1, mat4 v2) {
            return new bmat4(v1.m00 < v2.m00, v1.m01 < v2.m01, v1.m02 < v2.m02, v1.m03 < v2.m03, v1.m10 < v2.m10, v1.m11 < v2.m11, v1.m12 < v2.m12, v1.m13 < v2.m13, v1.m20 < v2.m20, v1.m21 < v2.m21, v1.m22 < v2.m22, v1.m23 < v2.m23, v1.m30 < v2.m30, v1.m31 < v2.m31, v1.m32 < v2.m32, v1.m33 < v2.m33);
        }
        
        public static bmat4 operator<=(mat4 v1, mat4 v2) {
            return new bmat4(v1.m00 <= v2.m00, v1.m01 <= v2.m01, v1.m02 <= v2.m02, v1.m03 <= v2.m03, v1.m10 <= v2.m10, v1.m11 <= v2.m11, v1.m12 <= v2.m12, v1.m13 <= v2.m13, v1.m20 <= v2.m20, v1.m21 <= v2.m21, v1.m22 <= v2.m22, v1.m23 <= v2.m23, v1.m30 <= v2.m30, v1.m31 <= v2.m31, v1.m32 <= v2.m32, v1.m33 <= v2.m33);
        }
        
        public static bmat4 operator!=(mat4 v1, mat4 v2) {
            return new bmat4(v1.m00 != v2.m00, v1.m01 != v2.m01, v1.m02 != v2.m02, v1.m03 != v2.m03, v1.m10 != v2.m10, v1.m11 != v2.m11, v1.m12 != v2.m12, v1.m13 != v2.m13, v1.m20 != v2.m20, v1.m21 != v2.m21, v1.m22 != v2.m22, v1.m23 != v2.m23, v1.m30 != v2.m30, v1.m31 != v2.m31, v1.m32 != v2.m32, v1.m33 != v2.m33);
        }
        
        public static vec4 operator*(vec4 v0, mat4 m1) {
            return math.InnerProduct1(ref v0, ref m1);
        }
        
        public static vec4 operator*(mat4 m0, vec4 v1) {
            return math.InnerProduct1(ref m0, ref v1);
        }
        
        public static mat4x2 operator*(mat4 m0, mat4x2 m1) {
            return math.InnerProduct1(ref m0, ref m1);
        }
        
        public static mat4x3 operator*(mat4 m0, mat4x3 m1) {
            return math.InnerProduct1(ref m0, ref m1);
        }
        
        public static mat4 operator*(mat4 m0, mat4 m1) {
            return math.InnerProduct1(ref m0, ref m1);
        }
        
        public override bool Equals(object obj) {
            return (obj is mat4) ? bmath.All(this == (mat4)obj) : false;
        }
        
        public override int GetHashCode() {
            return this.m00.GetHashCode() ^ this.m01.GetHashCode() ^ this.m02.GetHashCode() ^ this.m03.GetHashCode() ^ this.m10.GetHashCode() ^ this.m11.GetHashCode() ^ this.m12.GetHashCode() ^ this.m13.GetHashCode() ^ this.m20.GetHashCode() ^ this.m21.GetHashCode() ^ this.m22.GetHashCode() ^ this.m23.GetHashCode() ^ this.m30.GetHashCode() ^ this.m31.GetHashCode() ^ this.m32.GetHashCode() ^ this.m33.GetHashCode();
        }
        
        public override string ToString() {
            return "mat4(" + this.m00 + "," + this.m01 + "," + this.m02 + "," + this.m03 + "," + this.m10 + "," + this.m11 + "," + this.m12 + "," + this.m13 + "," + this.m20 + "," + this.m21 + "," + this.m22 + "," + this.m23 + "," + this.m30 + "," + this.m31 + "," + this.m32 + "," + this.m33 + ")";
        }
    }
}
