#region --- License ---
/*
Copyright 2010 Daniel Filonik

Permission is hereby granted, free of charge, to any person obtaining a copy of
this software and associated documentation files (the "Software"), to deal in
the Software without restriction, including without limitation the rights to
use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
of the Software, and to permit persons to whom the Software is furnished to do
so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all
copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
SOFTWARE.
*/
#endregion

namespace TKM {
    using System;
    using System.Runtime.InteropServices;
    using System.Runtime.Serialization;
    
    
    [DataContract()]
    [StructLayout(LayoutKind.Sequential)]
    public struct umat2 {
        
        [DataMember()]
        internal uint m00;
        
        [DataMember()]
        internal uint m01;
        
        [DataMember()]
        internal uint m10;
        
        [DataMember()]
        internal uint m11;
        
        public umat2(uint s) {
            this.m00 = s;
            this.m01 = 0u;
            this.m10 = 0u;
            this.m11 = s;
        }
        
        public umat2(umat2 m00) {
            this.m00 = m00.m00;
            this.m01 = m00.m01;
            this.m10 = m00.m10;
            this.m11 = m00.m11;
        }
        
        public umat2(uvec2 m00, uvec2 m10) {
            this.m00 = m00.v0;
            this.m01 = m00.v1;
            this.m10 = m10.v0;
            this.m11 = m10.v1;
        }
        
        public umat2(uint m00, uint m01, uint m10, uint m11) {
            this.m00 = m00;
            this.m01 = m01;
            this.m10 = m10;
            this.m11 = m11;
        }
        
        public umat2(umat2x3 s) {
            this.m00 = s.m00;
            this.m01 = s.m01;
            this.m10 = s.m10;
            this.m11 = s.m11;
        }
        
        public umat2(umat2x4 s) {
            this.m00 = s.m00;
            this.m01 = s.m01;
            this.m10 = s.m10;
            this.m11 = s.m11;
        }
        
        public umat2(umat3x2 s) {
            this.m00 = s.m00;
            this.m01 = s.m01;
            this.m10 = s.m10;
            this.m11 = s.m11;
        }
        
        public umat2(umat3 s) {
            this.m00 = s.m00;
            this.m01 = s.m01;
            this.m10 = s.m10;
            this.m11 = s.m11;
        }
        
        public umat2(umat3x4 s) {
            this.m00 = s.m00;
            this.m01 = s.m01;
            this.m10 = s.m10;
            this.m11 = s.m11;
        }
        
        public umat2(umat4x2 s) {
            this.m00 = s.m00;
            this.m01 = s.m01;
            this.m10 = s.m10;
            this.m11 = s.m11;
        }
        
        public umat2(umat4x3 s) {
            this.m00 = s.m00;
            this.m01 = s.m01;
            this.m10 = s.m10;
            this.m11 = s.m11;
        }
        
        public umat2(umat4 s) {
            this.m00 = s.m00;
            this.m01 = s.m01;
            this.m10 = s.m10;
            this.m11 = s.m11;
        }
        
        public uint this[int i0, int i1] {
            get {
                if (((i0 < 0) 
                            || (1 < i0))) {
                    throw new System.IndexOutOfRangeException("i0");
                }
                if (((i1 < 0) 
                            || (1 < i1))) {
                    throw new System.IndexOutOfRangeException("i1");
                }
                
                unsafe
                {
                    fixed (UInt32* v_ptr = &this.m00)
                    {
                        return *(v_ptr + (i1 + i0 * 2));
                    }
                };
            }
            set {
                if (((i0 < 0) 
                            || (1 < i0))) {
                    throw new System.IndexOutOfRangeException("i0");
                }
                if (((i1 < 0) 
                            || (1 < i1))) {
                    throw new System.IndexOutOfRangeException("i1");
                }
                
                unsafe
                {
                    fixed (UInt32* v_ptr = &this.m00)
                    {
                        *(v_ptr  + (i1 + i0 * 2)) = value;
                    }
                };
            }
        }
        
        public uvec2 this[int i0] {
            get {
                return new uvec2(this[i0, 0], this[i0, 1]);
            }
            set {
                this[i0, 0] = value[0];
                this[i0, 1] = value[1];
            }
        }
        
        public static umat2 operator+(umat2 v) {
            return new umat2(+v.m00, +v.m01, +v.m10, +v.m11);
        }
        
        public static umat2 operator~(umat2 v) {
            return new umat2(~v.m00, ~v.m01, ~v.m10, ~v.m11);
        }
        
        public static umat2 operator++(umat2 v) {
            return new umat2(++v.m00, ++v.m01, ++v.m10, ++v.m11);
        }
        
        public static umat2 operator--(umat2 v) {
            return new umat2(--v.m00, --v.m01, --v.m10, --v.m11);
        }
        
        public static umat2 operator+(uint s, umat2 v2) {
            return new umat2(s + v2.m00, s + v2.m01, s + v2.m10, s + v2.m11);
        }
        
        public static umat2 operator-(uint s, umat2 v2) {
            return new umat2(s - v2.m00, s - v2.m01, s - v2.m10, s - v2.m11);
        }
        
        public static umat2 operator*(uint s, umat2 v2) {
            return new umat2(s * v2.m00, s * v2.m01, s * v2.m10, s * v2.m11);
        }
        
        public static umat2 operator/(uint s, umat2 v2) {
            return new umat2(s / v2.m00, s / v2.m01, s / v2.m10, s / v2.m11);
        }
        
        public static umat2 operator%(uint s, umat2 v2) {
            return new umat2(s % v2.m00, s % v2.m01, s % v2.m10, s % v2.m11);
        }
        
        public static umat2 operator&(uint s, umat2 v2) {
            return new umat2(s & v2.m00, s & v2.m01, s & v2.m10, s & v2.m11);
        }
        
        public static umat2 operator|(uint s, umat2 v2) {
            return new umat2(s | v2.m00, s | v2.m01, s | v2.m10, s | v2.m11);
        }
        
        public static umat2 operator^(uint s, umat2 v2) {
            return new umat2(s ^ v2.m00, s ^ v2.m01, s ^ v2.m10, s ^ v2.m11);
        }
        
        public static umat2 operator+(umat2 v1, uint s) {
            return new umat2(v1.m00 + s, v1.m01 + s, v1.m10 + s, v1.m11 + s);
        }
        
        public static umat2 operator-(umat2 v1, uint s) {
            return new umat2(v1.m00 - s, v1.m01 - s, v1.m10 - s, v1.m11 - s);
        }
        
        public static umat2 operator*(umat2 v1, uint s) {
            return new umat2(v1.m00 * s, v1.m01 * s, v1.m10 * s, v1.m11 * s);
        }
        
        public static umat2 operator/(umat2 v1, uint s) {
            return new umat2(v1.m00 / s, v1.m01 / s, v1.m10 / s, v1.m11 / s);
        }
        
        public static umat2 operator%(umat2 v1, uint s) {
            return new umat2(v1.m00 % s, v1.m01 % s, v1.m10 % s, v1.m11 % s);
        }
        
        public static umat2 operator&(umat2 v1, uint s) {
            return new umat2(v1.m00 & s, v1.m01 & s, v1.m10 & s, v1.m11 & s);
        }
        
        public static umat2 operator|(umat2 v1, uint s) {
            return new umat2(v1.m00 | s, v1.m01 | s, v1.m10 | s, v1.m11 | s);
        }
        
        public static umat2 operator^(umat2 v1, uint s) {
            return new umat2(v1.m00 ^ s, v1.m01 ^ s, v1.m10 ^ s, v1.m11 ^ s);
        }
        
        public static umat2 operator+(umat2 v1, umat2 v2) {
            return new umat2(v1.m00 + v2.m00, v1.m01 + v2.m01, v1.m10 + v2.m10, v1.m11 + v2.m11);
        }
        
        public static umat2 operator-(umat2 v1, umat2 v2) {
            return new umat2(v1.m00 - v2.m00, v1.m01 - v2.m01, v1.m10 - v2.m10, v1.m11 - v2.m11);
        }
        
        public static umat2 operator/(umat2 v1, umat2 v2) {
            return new umat2(v1.m00 / v2.m00, v1.m01 / v2.m01, v1.m10 / v2.m10, v1.m11 / v2.m11);
        }
        
        public static umat2 operator%(umat2 v1, umat2 v2) {
            return new umat2(v1.m00 % v2.m00, v1.m01 % v2.m01, v1.m10 % v2.m10, v1.m11 % v2.m11);
        }
        
        public static umat2 operator&(umat2 v1, umat2 v2) {
            return new umat2(v1.m00 & v2.m00, v1.m01 & v2.m01, v1.m10 & v2.m10, v1.m11 & v2.m11);
        }
        
        public static umat2 operator|(umat2 v1, umat2 v2) {
            return new umat2(v1.m00 | v2.m00, v1.m01 | v2.m01, v1.m10 | v2.m10, v1.m11 | v2.m11);
        }
        
        public static umat2 operator^(umat2 v1, umat2 v2) {
            return new umat2(v1.m00 ^ v2.m00, v1.m01 ^ v2.m01, v1.m10 ^ v2.m10, v1.m11 ^ v2.m11);
        }
        
        public static bmat2 operator==(umat2 v1, umat2 v2) {
            return new bmat2(v1.m00 == v2.m00, v1.m01 == v2.m01, v1.m10 == v2.m10, v1.m11 == v2.m11);
        }
        
        public static bmat2 operator>(umat2 v1, umat2 v2) {
            return new bmat2(v1.m00 > v2.m00, v1.m01 > v2.m01, v1.m10 > v2.m10, v1.m11 > v2.m11);
        }
        
        public static bmat2 operator>=(umat2 v1, umat2 v2) {
            return new bmat2(v1.m00 >= v2.m00, v1.m01 >= v2.m01, v1.m10 >= v2.m10, v1.m11 >= v2.m11);
        }
        
        public static bmat2 operator<(umat2 v1, umat2 v2) {
            return new bmat2(v1.m00 < v2.m00, v1.m01 < v2.m01, v1.m10 < v2.m10, v1.m11 < v2.m11);
        }
        
        public static bmat2 operator<=(umat2 v1, umat2 v2) {
            return new bmat2(v1.m00 <= v2.m00, v1.m01 <= v2.m01, v1.m10 <= v2.m10, v1.m11 <= v2.m11);
        }
        
        public static bmat2 operator!=(umat2 v1, umat2 v2) {
            return new bmat2(v1.m00 != v2.m00, v1.m01 != v2.m01, v1.m10 != v2.m10, v1.m11 != v2.m11);
        }
        
        public static uvec2 operator*(uvec2 v0, umat2 m1) {
            return umath.InnerProduct1(ref v0, ref m1);
        }
        
        public static uvec2 operator*(umat2 m0, uvec2 v1) {
            return umath.InnerProduct1(ref m0, ref v1);
        }
        
        public static umat2 operator*(umat2 m0, umat2 m1) {
            return umath.InnerProduct1(ref m0, ref m1);
        }
        
        public static umat2x3 operator*(umat2 m0, umat2x3 m1) {
            return umath.InnerProduct1(ref m0, ref m1);
        }
        
        public static umat2x4 operator*(umat2 m0, umat2x4 m1) {
            return umath.InnerProduct1(ref m0, ref m1);
        }
        
        public override bool Equals(object obj) {
            return (obj is umat2) ? bmath.All(this == (umat2)obj) : false;
        }
        
        public override int GetHashCode() {
            return this.m00.GetHashCode() ^ this.m01.GetHashCode() ^ this.m10.GetHashCode() ^ this.m11.GetHashCode();
        }
        
        public override string ToString() {
            return "umat2(" + this.m00 + "," + this.m01 + "," + this.m10 + "," + this.m11 + ")";
        }
    }
    
    [DataContract()]
    [StructLayout(LayoutKind.Sequential)]
    public struct umat2x3 {
        
        [DataMember()]
        internal uint m00;
        
        [DataMember()]
        internal uint m01;
        
        [DataMember()]
        internal uint m02;
        
        [DataMember()]
        internal uint m10;
        
        [DataMember()]
        internal uint m11;
        
        [DataMember()]
        internal uint m12;
        
        public umat2x3(uint s) {
            this.m00 = s;
            this.m01 = 0u;
            this.m02 = 0u;
            this.m10 = 0u;
            this.m11 = s;
            this.m12 = 0u;
        }
        
        public umat2x3(umat2x3 m00) {
            this.m00 = m00.m00;
            this.m01 = m00.m01;
            this.m02 = m00.m02;
            this.m10 = m00.m10;
            this.m11 = m00.m11;
            this.m12 = m00.m12;
        }
        
        public umat2x3(uvec3 m00, uvec3 m10) {
            this.m00 = m00.v0;
            this.m01 = m00.v1;
            this.m02 = m00.v2;
            this.m10 = m10.v0;
            this.m11 = m10.v1;
            this.m12 = m10.v2;
        }
        
        public umat2x3(uvec2 m00, uint m01, uvec2 m10, uint m11) {
            this.m00 = m00.v0;
            this.m01 = m00.v1;
            this.m02 = m01;
            this.m10 = m10.v0;
            this.m11 = m10.v1;
            this.m12 = m11;
        }
        
        public umat2x3(uint m00, uvec2 m01, uint m10, uvec2 m11) {
            this.m00 = m00;
            this.m01 = m01.v0;
            this.m02 = m01.v1;
            this.m10 = m10;
            this.m11 = m11.v0;
            this.m12 = m11.v1;
        }
        
        public umat2x3(uint m00, uint m01, uint m02, uint m10, uint m11, uint m12) {
            this.m00 = m00;
            this.m01 = m01;
            this.m02 = m02;
            this.m10 = m10;
            this.m11 = m11;
            this.m12 = m12;
        }
        
        public umat2x3(umat2 s) {
            this.m00 = s.m00;
            this.m01 = s.m01;
            this.m02 = 0u;
            this.m10 = s.m10;
            this.m11 = s.m11;
            this.m12 = 0u;
        }
        
        public umat2x3(umat2x4 s) {
            this.m00 = s.m00;
            this.m01 = s.m01;
            this.m02 = s.m02;
            this.m10 = s.m10;
            this.m11 = s.m11;
            this.m12 = s.m12;
        }
        
        public umat2x3(umat3x2 s) {
            this.m00 = s.m00;
            this.m01 = s.m01;
            this.m02 = 0u;
            this.m10 = s.m10;
            this.m11 = s.m11;
            this.m12 = 0u;
        }
        
        public umat2x3(umat3 s) {
            this.m00 = s.m00;
            this.m01 = s.m01;
            this.m02 = s.m02;
            this.m10 = s.m10;
            this.m11 = s.m11;
            this.m12 = s.m12;
        }
        
        public umat2x3(umat3x4 s) {
            this.m00 = s.m00;
            this.m01 = s.m01;
            this.m02 = s.m02;
            this.m10 = s.m10;
            this.m11 = s.m11;
            this.m12 = s.m12;
        }
        
        public umat2x3(umat4x2 s) {
            this.m00 = s.m00;
            this.m01 = s.m01;
            this.m02 = 0u;
            this.m10 = s.m10;
            this.m11 = s.m11;
            this.m12 = 0u;
        }
        
        public umat2x3(umat4x3 s) {
            this.m00 = s.m00;
            this.m01 = s.m01;
            this.m02 = s.m02;
            this.m10 = s.m10;
            this.m11 = s.m11;
            this.m12 = s.m12;
        }
        
        public umat2x3(umat4 s) {
            this.m00 = s.m00;
            this.m01 = s.m01;
            this.m02 = s.m02;
            this.m10 = s.m10;
            this.m11 = s.m11;
            this.m12 = s.m12;
        }
        
        public uint this[int i0, int i1] {
            get {
                if (((i0 < 0) 
                            || (1 < i0))) {
                    throw new System.IndexOutOfRangeException("i0");
                }
                if (((i1 < 0) 
                            || (2 < i1))) {
                    throw new System.IndexOutOfRangeException("i1");
                }
                
                unsafe
                {
                    fixed (UInt32* v_ptr = &this.m00)
                    {
                        return *(v_ptr + (i1 + i0 * 3));
                    }
                };
            }
            set {
                if (((i0 < 0) 
                            || (1 < i0))) {
                    throw new System.IndexOutOfRangeException("i0");
                }
                if (((i1 < 0) 
                            || (2 < i1))) {
                    throw new System.IndexOutOfRangeException("i1");
                }
                
                unsafe
                {
                    fixed (UInt32* v_ptr = &this.m00)
                    {
                        *(v_ptr  + (i1 + i0 * 3)) = value;
                    }
                };
            }
        }
        
        public uvec3 this[int i0] {
            get {
                return new uvec3(this[i0, 0], this[i0, 1], this[i0, 2]);
            }
            set {
                this[i0, 0] = value[0];
                this[i0, 1] = value[1];
                this[i0, 2] = value[2];
            }
        }
        
        public static umat2x3 operator+(umat2x3 v) {
            return new umat2x3(+v.m00, +v.m01, +v.m02, +v.m10, +v.m11, +v.m12);
        }
        
        public static umat2x3 operator~(umat2x3 v) {
            return new umat2x3(~v.m00, ~v.m01, ~v.m02, ~v.m10, ~v.m11, ~v.m12);
        }
        
        public static umat2x3 operator++(umat2x3 v) {
            return new umat2x3(++v.m00, ++v.m01, ++v.m02, ++v.m10, ++v.m11, ++v.m12);
        }
        
        public static umat2x3 operator--(umat2x3 v) {
            return new umat2x3(--v.m00, --v.m01, --v.m02, --v.m10, --v.m11, --v.m12);
        }
        
        public static umat2x3 operator+(uint s, umat2x3 v2) {
            return new umat2x3(s + v2.m00, s + v2.m01, s + v2.m02, s + v2.m10, s + v2.m11, s + v2.m12);
        }
        
        public static umat2x3 operator-(uint s, umat2x3 v2) {
            return new umat2x3(s - v2.m00, s - v2.m01, s - v2.m02, s - v2.m10, s - v2.m11, s - v2.m12);
        }
        
        public static umat2x3 operator*(uint s, umat2x3 v2) {
            return new umat2x3(s * v2.m00, s * v2.m01, s * v2.m02, s * v2.m10, s * v2.m11, s * v2.m12);
        }
        
        public static umat2x3 operator/(uint s, umat2x3 v2) {
            return new umat2x3(s / v2.m00, s / v2.m01, s / v2.m02, s / v2.m10, s / v2.m11, s / v2.m12);
        }
        
        public static umat2x3 operator%(uint s, umat2x3 v2) {
            return new umat2x3(s % v2.m00, s % v2.m01, s % v2.m02, s % v2.m10, s % v2.m11, s % v2.m12);
        }
        
        public static umat2x3 operator&(uint s, umat2x3 v2) {
            return new umat2x3(s & v2.m00, s & v2.m01, s & v2.m02, s & v2.m10, s & v2.m11, s & v2.m12);
        }
        
        public static umat2x3 operator|(uint s, umat2x3 v2) {
            return new umat2x3(s | v2.m00, s | v2.m01, s | v2.m02, s | v2.m10, s | v2.m11, s | v2.m12);
        }
        
        public static umat2x3 operator^(uint s, umat2x3 v2) {
            return new umat2x3(s ^ v2.m00, s ^ v2.m01, s ^ v2.m02, s ^ v2.m10, s ^ v2.m11, s ^ v2.m12);
        }
        
        public static umat2x3 operator+(umat2x3 v1, uint s) {
            return new umat2x3(v1.m00 + s, v1.m01 + s, v1.m02 + s, v1.m10 + s, v1.m11 + s, v1.m12 + s);
        }
        
        public static umat2x3 operator-(umat2x3 v1, uint s) {
            return new umat2x3(v1.m00 - s, v1.m01 - s, v1.m02 - s, v1.m10 - s, v1.m11 - s, v1.m12 - s);
        }
        
        public static umat2x3 operator*(umat2x3 v1, uint s) {
            return new umat2x3(v1.m00 * s, v1.m01 * s, v1.m02 * s, v1.m10 * s, v1.m11 * s, v1.m12 * s);
        }
        
        public static umat2x3 operator/(umat2x3 v1, uint s) {
            return new umat2x3(v1.m00 / s, v1.m01 / s, v1.m02 / s, v1.m10 / s, v1.m11 / s, v1.m12 / s);
        }
        
        public static umat2x3 operator%(umat2x3 v1, uint s) {
            return new umat2x3(v1.m00 % s, v1.m01 % s, v1.m02 % s, v1.m10 % s, v1.m11 % s, v1.m12 % s);
        }
        
        public static umat2x3 operator&(umat2x3 v1, uint s) {
            return new umat2x3(v1.m00 & s, v1.m01 & s, v1.m02 & s, v1.m10 & s, v1.m11 & s, v1.m12 & s);
        }
        
        public static umat2x3 operator|(umat2x3 v1, uint s) {
            return new umat2x3(v1.m00 | s, v1.m01 | s, v1.m02 | s, v1.m10 | s, v1.m11 | s, v1.m12 | s);
        }
        
        public static umat2x3 operator^(umat2x3 v1, uint s) {
            return new umat2x3(v1.m00 ^ s, v1.m01 ^ s, v1.m02 ^ s, v1.m10 ^ s, v1.m11 ^ s, v1.m12 ^ s);
        }
        
        public static umat2x3 operator+(umat2x3 v1, umat2x3 v2) {
            return new umat2x3(v1.m00 + v2.m00, v1.m01 + v2.m01, v1.m02 + v2.m02, v1.m10 + v2.m10, v1.m11 + v2.m11, v1.m12 + v2.m12);
        }
        
        public static umat2x3 operator-(umat2x3 v1, umat2x3 v2) {
            return new umat2x3(v1.m00 - v2.m00, v1.m01 - v2.m01, v1.m02 - v2.m02, v1.m10 - v2.m10, v1.m11 - v2.m11, v1.m12 - v2.m12);
        }
        
        public static umat2x3 operator/(umat2x3 v1, umat2x3 v2) {
            return new umat2x3(v1.m00 / v2.m00, v1.m01 / v2.m01, v1.m02 / v2.m02, v1.m10 / v2.m10, v1.m11 / v2.m11, v1.m12 / v2.m12);
        }
        
        public static umat2x3 operator%(umat2x3 v1, umat2x3 v2) {
            return new umat2x3(v1.m00 % v2.m00, v1.m01 % v2.m01, v1.m02 % v2.m02, v1.m10 % v2.m10, v1.m11 % v2.m11, v1.m12 % v2.m12);
        }
        
        public static umat2x3 operator&(umat2x3 v1, umat2x3 v2) {
            return new umat2x3(v1.m00 & v2.m00, v1.m01 & v2.m01, v1.m02 & v2.m02, v1.m10 & v2.m10, v1.m11 & v2.m11, v1.m12 & v2.m12);
        }
        
        public static umat2x3 operator|(umat2x3 v1, umat2x3 v2) {
            return new umat2x3(v1.m00 | v2.m00, v1.m01 | v2.m01, v1.m02 | v2.m02, v1.m10 | v2.m10, v1.m11 | v2.m11, v1.m12 | v2.m12);
        }
        
        public static umat2x3 operator^(umat2x3 v1, umat2x3 v2) {
            return new umat2x3(v1.m00 ^ v2.m00, v1.m01 ^ v2.m01, v1.m02 ^ v2.m02, v1.m10 ^ v2.m10, v1.m11 ^ v2.m11, v1.m12 ^ v2.m12);
        }
        
        public static bmat2x3 operator==(umat2x3 v1, umat2x3 v2) {
            return new bmat2x3(v1.m00 == v2.m00, v1.m01 == v2.m01, v1.m02 == v2.m02, v1.m10 == v2.m10, v1.m11 == v2.m11, v1.m12 == v2.m12);
        }
        
        public static bmat2x3 operator>(umat2x3 v1, umat2x3 v2) {
            return new bmat2x3(v1.m00 > v2.m00, v1.m01 > v2.m01, v1.m02 > v2.m02, v1.m10 > v2.m10, v1.m11 > v2.m11, v1.m12 > v2.m12);
        }
        
        public static bmat2x3 operator>=(umat2x3 v1, umat2x3 v2) {
            return new bmat2x3(v1.m00 >= v2.m00, v1.m01 >= v2.m01, v1.m02 >= v2.m02, v1.m10 >= v2.m10, v1.m11 >= v2.m11, v1.m12 >= v2.m12);
        }
        
        public static bmat2x3 operator<(umat2x3 v1, umat2x3 v2) {
            return new bmat2x3(v1.m00 < v2.m00, v1.m01 < v2.m01, v1.m02 < v2.m02, v1.m10 < v2.m10, v1.m11 < v2.m11, v1.m12 < v2.m12);
        }
        
        public static bmat2x3 operator<=(umat2x3 v1, umat2x3 v2) {
            return new bmat2x3(v1.m00 <= v2.m00, v1.m01 <= v2.m01, v1.m02 <= v2.m02, v1.m10 <= v2.m10, v1.m11 <= v2.m11, v1.m12 <= v2.m12);
        }
        
        public static bmat2x3 operator!=(umat2x3 v1, umat2x3 v2) {
            return new bmat2x3(v1.m00 != v2.m00, v1.m01 != v2.m01, v1.m02 != v2.m02, v1.m10 != v2.m10, v1.m11 != v2.m11, v1.m12 != v2.m12);
        }
        
        public static uvec3 operator*(uvec2 v0, umat2x3 m1) {
            return umath.InnerProduct1(ref v0, ref m1);
        }
        
        public static uvec2 operator*(umat2x3 m0, uvec3 v1) {
            return umath.InnerProduct1(ref m0, ref v1);
        }
        
        public static umat2 operator*(umat2x3 m0, umat3x2 m1) {
            return umath.InnerProduct1(ref m0, ref m1);
        }
        
        public static umat2x3 operator*(umat2x3 m0, umat3 m1) {
            return umath.InnerProduct1(ref m0, ref m1);
        }
        
        public static umat2x4 operator*(umat2x3 m0, umat3x4 m1) {
            return umath.InnerProduct1(ref m0, ref m1);
        }
        
        public override bool Equals(object obj) {
            return (obj is umat2x3) ? bmath.All(this == (umat2x3)obj) : false;
        }
        
        public override int GetHashCode() {
            return this.m00.GetHashCode() ^ this.m01.GetHashCode() ^ this.m02.GetHashCode() ^ this.m10.GetHashCode() ^ this.m11.GetHashCode() ^ this.m12.GetHashCode();
        }
        
        public override string ToString() {
            return "umat2x3(" + this.m00 + "," + this.m01 + "," + this.m02 + "," + this.m10 + "," + this.m11 + "," + this.m12 + ")";
        }
    }
    
    [DataContract()]
    [StructLayout(LayoutKind.Sequential)]
    public struct umat2x4 {
        
        [DataMember()]
        internal uint m00;
        
        [DataMember()]
        internal uint m01;
        
        [DataMember()]
        internal uint m02;
        
        [DataMember()]
        internal uint m03;
        
        [DataMember()]
        internal uint m10;
        
        [DataMember()]
        internal uint m11;
        
        [DataMember()]
        internal uint m12;
        
        [DataMember()]
        internal uint m13;
        
        public umat2x4(uint s) {
            this.m00 = s;
            this.m01 = 0u;
            this.m02 = 0u;
            this.m03 = 0u;
            this.m10 = 0u;
            this.m11 = s;
            this.m12 = 0u;
            this.m13 = 0u;
        }
        
        public umat2x4(umat2x4 m00) {
            this.m00 = m00.m00;
            this.m01 = m00.m01;
            this.m02 = m00.m02;
            this.m03 = m00.m03;
            this.m10 = m00.m10;
            this.m11 = m00.m11;
            this.m12 = m00.m12;
            this.m13 = m00.m13;
        }
        
        public umat2x4(umat2 m00, umat2 m01) {
            this.m00 = m00.m00;
            this.m01 = m00.m01;
            this.m02 = m01.m00;
            this.m03 = m01.m01;
            this.m10 = m00.m10;
            this.m11 = m00.m11;
            this.m12 = m01.m10;
            this.m13 = m01.m11;
        }
        
        public umat2x4(uvec4 m00, uvec4 m10) {
            this.m00 = m00.v0;
            this.m01 = m00.v1;
            this.m02 = m00.v2;
            this.m03 = m00.v3;
            this.m10 = m10.v0;
            this.m11 = m10.v1;
            this.m12 = m10.v2;
            this.m13 = m10.v3;
        }
        
        public umat2x4(uvec2 m00, uvec2 m01, uvec2 m10, uvec2 m11) {
            this.m00 = m00.v0;
            this.m01 = m00.v1;
            this.m02 = m01.v0;
            this.m03 = m01.v1;
            this.m10 = m10.v0;
            this.m11 = m10.v1;
            this.m12 = m11.v0;
            this.m13 = m11.v1;
        }
        
        public umat2x4(uvec3 m00, uint m01, uvec3 m10, uint m11) {
            this.m00 = m00.v0;
            this.m01 = m00.v1;
            this.m02 = m00.v2;
            this.m03 = m01;
            this.m10 = m10.v0;
            this.m11 = m10.v1;
            this.m12 = m10.v2;
            this.m13 = m11;
        }
        
        public umat2x4(uint m00, uvec3 m01, uint m10, uvec3 m11) {
            this.m00 = m00;
            this.m01 = m01.v0;
            this.m02 = m01.v1;
            this.m03 = m01.v2;
            this.m10 = m10;
            this.m11 = m11.v0;
            this.m12 = m11.v1;
            this.m13 = m11.v2;
        }
        
        public umat2x4(uvec2 m00, uint m01, uint m02, uvec2 m10, uint m11, uint m12) {
            this.m00 = m00.v0;
            this.m01 = m00.v1;
            this.m02 = m01;
            this.m03 = m02;
            this.m10 = m10.v0;
            this.m11 = m10.v1;
            this.m12 = m11;
            this.m13 = m12;
        }
        
        public umat2x4(uint m00, uvec2 m01, uint m02, uint m10, uvec2 m11, uint m12) {
            this.m00 = m00;
            this.m01 = m01.v0;
            this.m02 = m01.v1;
            this.m03 = m02;
            this.m10 = m10;
            this.m11 = m11.v0;
            this.m12 = m11.v1;
            this.m13 = m12;
        }
        
        public umat2x4(uint m00, uint m01, uvec2 m02, uint m10, uint m11, uvec2 m12) {
            this.m00 = m00;
            this.m01 = m01;
            this.m02 = m02.v0;
            this.m03 = m02.v1;
            this.m10 = m10;
            this.m11 = m11;
            this.m12 = m12.v0;
            this.m13 = m12.v1;
        }
        
        public umat2x4(uint m00, uint m01, uint m02, uint m03, uint m10, uint m11, uint m12, uint m13) {
            this.m00 = m00;
            this.m01 = m01;
            this.m02 = m02;
            this.m03 = m03;
            this.m10 = m10;
            this.m11 = m11;
            this.m12 = m12;
            this.m13 = m13;
        }
        
        public umat2x4(umat2 s) {
            this.m00 = s.m00;
            this.m01 = s.m01;
            this.m02 = 0u;
            this.m03 = 0u;
            this.m10 = s.m10;
            this.m11 = s.m11;
            this.m12 = 0u;
            this.m13 = 0u;
        }
        
        public umat2x4(umat2x3 s) {
            this.m00 = s.m00;
            this.m01 = s.m01;
            this.m02 = s.m02;
            this.m03 = 0u;
            this.m10 = s.m10;
            this.m11 = s.m11;
            this.m12 = s.m12;
            this.m13 = 0u;
        }
        
        public umat2x4(umat3x2 s) {
            this.m00 = s.m00;
            this.m01 = s.m01;
            this.m02 = 0u;
            this.m03 = 0u;
            this.m10 = s.m10;
            this.m11 = s.m11;
            this.m12 = 0u;
            this.m13 = 0u;
        }
        
        public umat2x4(umat3 s) {
            this.m00 = s.m00;
            this.m01 = s.m01;
            this.m02 = s.m02;
            this.m03 = 0u;
            this.m10 = s.m10;
            this.m11 = s.m11;
            this.m12 = s.m12;
            this.m13 = 0u;
        }
        
        public umat2x4(umat3x4 s) {
            this.m00 = s.m00;
            this.m01 = s.m01;
            this.m02 = s.m02;
            this.m03 = s.m03;
            this.m10 = s.m10;
            this.m11 = s.m11;
            this.m12 = s.m12;
            this.m13 = s.m13;
        }
        
        public umat2x4(umat4x2 s) {
            this.m00 = s.m00;
            this.m01 = s.m01;
            this.m02 = 0u;
            this.m03 = 0u;
            this.m10 = s.m10;
            this.m11 = s.m11;
            this.m12 = 0u;
            this.m13 = 0u;
        }
        
        public umat2x4(umat4x3 s) {
            this.m00 = s.m00;
            this.m01 = s.m01;
            this.m02 = s.m02;
            this.m03 = 0u;
            this.m10 = s.m10;
            this.m11 = s.m11;
            this.m12 = s.m12;
            this.m13 = 0u;
        }
        
        public umat2x4(umat4 s) {
            this.m00 = s.m00;
            this.m01 = s.m01;
            this.m02 = s.m02;
            this.m03 = s.m03;
            this.m10 = s.m10;
            this.m11 = s.m11;
            this.m12 = s.m12;
            this.m13 = s.m13;
        }
        
        public uint this[int i0, int i1] {
            get {
                if (((i0 < 0) 
                            || (1 < i0))) {
                    throw new System.IndexOutOfRangeException("i0");
                }
                if (((i1 < 0) 
                            || (3 < i1))) {
                    throw new System.IndexOutOfRangeException("i1");
                }
                
                unsafe
                {
                    fixed (UInt32* v_ptr = &this.m00)
                    {
                        return *(v_ptr + (i1 + i0 * 4));
                    }
                };
            }
            set {
                if (((i0 < 0) 
                            || (1 < i0))) {
                    throw new System.IndexOutOfRangeException("i0");
                }
                if (((i1 < 0) 
                            || (3 < i1))) {
                    throw new System.IndexOutOfRangeException("i1");
                }
                
                unsafe
                {
                    fixed (UInt32* v_ptr = &this.m00)
                    {
                        *(v_ptr  + (i1 + i0 * 4)) = value;
                    }
                };
            }
        }
        
        public uvec4 this[int i0] {
            get {
                return new uvec4(this[i0, 0], this[i0, 1], this[i0, 2], this[i0, 3]);
            }
            set {
                this[i0, 0] = value[0];
                this[i0, 1] = value[1];
                this[i0, 2] = value[2];
                this[i0, 3] = value[3];
            }
        }
        
        public static umat2x4 operator+(umat2x4 v) {
            return new umat2x4(+v.m00, +v.m01, +v.m02, +v.m03, +v.m10, +v.m11, +v.m12, +v.m13);
        }
        
        public static umat2x4 operator~(umat2x4 v) {
            return new umat2x4(~v.m00, ~v.m01, ~v.m02, ~v.m03, ~v.m10, ~v.m11, ~v.m12, ~v.m13);
        }
        
        public static umat2x4 operator++(umat2x4 v) {
            return new umat2x4(++v.m00, ++v.m01, ++v.m02, ++v.m03, ++v.m10, ++v.m11, ++v.m12, ++v.m13);
        }
        
        public static umat2x4 operator--(umat2x4 v) {
            return new umat2x4(--v.m00, --v.m01, --v.m02, --v.m03, --v.m10, --v.m11, --v.m12, --v.m13);
        }
        
        public static umat2x4 operator+(uint s, umat2x4 v2) {
            return new umat2x4(s + v2.m00, s + v2.m01, s + v2.m02, s + v2.m03, s + v2.m10, s + v2.m11, s + v2.m12, s + v2.m13);
        }
        
        public static umat2x4 operator-(uint s, umat2x4 v2) {
            return new umat2x4(s - v2.m00, s - v2.m01, s - v2.m02, s - v2.m03, s - v2.m10, s - v2.m11, s - v2.m12, s - v2.m13);
        }
        
        public static umat2x4 operator*(uint s, umat2x4 v2) {
            return new umat2x4(s * v2.m00, s * v2.m01, s * v2.m02, s * v2.m03, s * v2.m10, s * v2.m11, s * v2.m12, s * v2.m13);
        }
        
        public static umat2x4 operator/(uint s, umat2x4 v2) {
            return new umat2x4(s / v2.m00, s / v2.m01, s / v2.m02, s / v2.m03, s / v2.m10, s / v2.m11, s / v2.m12, s / v2.m13);
        }
        
        public static umat2x4 operator%(uint s, umat2x4 v2) {
            return new umat2x4(s % v2.m00, s % v2.m01, s % v2.m02, s % v2.m03, s % v2.m10, s % v2.m11, s % v2.m12, s % v2.m13);
        }
        
        public static umat2x4 operator&(uint s, umat2x4 v2) {
            return new umat2x4(s & v2.m00, s & v2.m01, s & v2.m02, s & v2.m03, s & v2.m10, s & v2.m11, s & v2.m12, s & v2.m13);
        }
        
        public static umat2x4 operator|(uint s, umat2x4 v2) {
            return new umat2x4(s | v2.m00, s | v2.m01, s | v2.m02, s | v2.m03, s | v2.m10, s | v2.m11, s | v2.m12, s | v2.m13);
        }
        
        public static umat2x4 operator^(uint s, umat2x4 v2) {
            return new umat2x4(s ^ v2.m00, s ^ v2.m01, s ^ v2.m02, s ^ v2.m03, s ^ v2.m10, s ^ v2.m11, s ^ v2.m12, s ^ v2.m13);
        }
        
        public static umat2x4 operator+(umat2x4 v1, uint s) {
            return new umat2x4(v1.m00 + s, v1.m01 + s, v1.m02 + s, v1.m03 + s, v1.m10 + s, v1.m11 + s, v1.m12 + s, v1.m13 + s);
        }
        
        public static umat2x4 operator-(umat2x4 v1, uint s) {
            return new umat2x4(v1.m00 - s, v1.m01 - s, v1.m02 - s, v1.m03 - s, v1.m10 - s, v1.m11 - s, v1.m12 - s, v1.m13 - s);
        }
        
        public static umat2x4 operator*(umat2x4 v1, uint s) {
            return new umat2x4(v1.m00 * s, v1.m01 * s, v1.m02 * s, v1.m03 * s, v1.m10 * s, v1.m11 * s, v1.m12 * s, v1.m13 * s);
        }
        
        public static umat2x4 operator/(umat2x4 v1, uint s) {
            return new umat2x4(v1.m00 / s, v1.m01 / s, v1.m02 / s, v1.m03 / s, v1.m10 / s, v1.m11 / s, v1.m12 / s, v1.m13 / s);
        }
        
        public static umat2x4 operator%(umat2x4 v1, uint s) {
            return new umat2x4(v1.m00 % s, v1.m01 % s, v1.m02 % s, v1.m03 % s, v1.m10 % s, v1.m11 % s, v1.m12 % s, v1.m13 % s);
        }
        
        public static umat2x4 operator&(umat2x4 v1, uint s) {
            return new umat2x4(v1.m00 & s, v1.m01 & s, v1.m02 & s, v1.m03 & s, v1.m10 & s, v1.m11 & s, v1.m12 & s, v1.m13 & s);
        }
        
        public static umat2x4 operator|(umat2x4 v1, uint s) {
            return new umat2x4(v1.m00 | s, v1.m01 | s, v1.m02 | s, v1.m03 | s, v1.m10 | s, v1.m11 | s, v1.m12 | s, v1.m13 | s);
        }
        
        public static umat2x4 operator^(umat2x4 v1, uint s) {
            return new umat2x4(v1.m00 ^ s, v1.m01 ^ s, v1.m02 ^ s, v1.m03 ^ s, v1.m10 ^ s, v1.m11 ^ s, v1.m12 ^ s, v1.m13 ^ s);
        }
        
        public static umat2x4 operator+(umat2x4 v1, umat2x4 v2) {
            return new umat2x4(v1.m00 + v2.m00, v1.m01 + v2.m01, v1.m02 + v2.m02, v1.m03 + v2.m03, v1.m10 + v2.m10, v1.m11 + v2.m11, v1.m12 + v2.m12, v1.m13 + v2.m13);
        }
        
        public static umat2x4 operator-(umat2x4 v1, umat2x4 v2) {
            return new umat2x4(v1.m00 - v2.m00, v1.m01 - v2.m01, v1.m02 - v2.m02, v1.m03 - v2.m03, v1.m10 - v2.m10, v1.m11 - v2.m11, v1.m12 - v2.m12, v1.m13 - v2.m13);
        }
        
        public static umat2x4 operator/(umat2x4 v1, umat2x4 v2) {
            return new umat2x4(v1.m00 / v2.m00, v1.m01 / v2.m01, v1.m02 / v2.m02, v1.m03 / v2.m03, v1.m10 / v2.m10, v1.m11 / v2.m11, v1.m12 / v2.m12, v1.m13 / v2.m13);
        }
        
        public static umat2x4 operator%(umat2x4 v1, umat2x4 v2) {
            return new umat2x4(v1.m00 % v2.m00, v1.m01 % v2.m01, v1.m02 % v2.m02, v1.m03 % v2.m03, v1.m10 % v2.m10, v1.m11 % v2.m11, v1.m12 % v2.m12, v1.m13 % v2.m13);
        }
        
        public static umat2x4 operator&(umat2x4 v1, umat2x4 v2) {
            return new umat2x4(v1.m00 & v2.m00, v1.m01 & v2.m01, v1.m02 & v2.m02, v1.m03 & v2.m03, v1.m10 & v2.m10, v1.m11 & v2.m11, v1.m12 & v2.m12, v1.m13 & v2.m13);
        }
        
        public static umat2x4 operator|(umat2x4 v1, umat2x4 v2) {
            return new umat2x4(v1.m00 | v2.m00, v1.m01 | v2.m01, v1.m02 | v2.m02, v1.m03 | v2.m03, v1.m10 | v2.m10, v1.m11 | v2.m11, v1.m12 | v2.m12, v1.m13 | v2.m13);
        }
        
        public static umat2x4 operator^(umat2x4 v1, umat2x4 v2) {
            return new umat2x4(v1.m00 ^ v2.m00, v1.m01 ^ v2.m01, v1.m02 ^ v2.m02, v1.m03 ^ v2.m03, v1.m10 ^ v2.m10, v1.m11 ^ v2.m11, v1.m12 ^ v2.m12, v1.m13 ^ v2.m13);
        }
        
        public static bmat2x4 operator==(umat2x4 v1, umat2x4 v2) {
            return new bmat2x4(v1.m00 == v2.m00, v1.m01 == v2.m01, v1.m02 == v2.m02, v1.m03 == v2.m03, v1.m10 == v2.m10, v1.m11 == v2.m11, v1.m12 == v2.m12, v1.m13 == v2.m13);
        }
        
        public static bmat2x4 operator>(umat2x4 v1, umat2x4 v2) {
            return new bmat2x4(v1.m00 > v2.m00, v1.m01 > v2.m01, v1.m02 > v2.m02, v1.m03 > v2.m03, v1.m10 > v2.m10, v1.m11 > v2.m11, v1.m12 > v2.m12, v1.m13 > v2.m13);
        }
        
        public static bmat2x4 operator>=(umat2x4 v1, umat2x4 v2) {
            return new bmat2x4(v1.m00 >= v2.m00, v1.m01 >= v2.m01, v1.m02 >= v2.m02, v1.m03 >= v2.m03, v1.m10 >= v2.m10, v1.m11 >= v2.m11, v1.m12 >= v2.m12, v1.m13 >= v2.m13);
        }
        
        public static bmat2x4 operator<(umat2x4 v1, umat2x4 v2) {
            return new bmat2x4(v1.m00 < v2.m00, v1.m01 < v2.m01, v1.m02 < v2.m02, v1.m03 < v2.m03, v1.m10 < v2.m10, v1.m11 < v2.m11, v1.m12 < v2.m12, v1.m13 < v2.m13);
        }
        
        public static bmat2x4 operator<=(umat2x4 v1, umat2x4 v2) {
            return new bmat2x4(v1.m00 <= v2.m00, v1.m01 <= v2.m01, v1.m02 <= v2.m02, v1.m03 <= v2.m03, v1.m10 <= v2.m10, v1.m11 <= v2.m11, v1.m12 <= v2.m12, v1.m13 <= v2.m13);
        }
        
        public static bmat2x4 operator!=(umat2x4 v1, umat2x4 v2) {
            return new bmat2x4(v1.m00 != v2.m00, v1.m01 != v2.m01, v1.m02 != v2.m02, v1.m03 != v2.m03, v1.m10 != v2.m10, v1.m11 != v2.m11, v1.m12 != v2.m12, v1.m13 != v2.m13);
        }
        
        public static uvec4 operator*(uvec2 v0, umat2x4 m1) {
            return umath.InnerProduct1(ref v0, ref m1);
        }
        
        public static uvec2 operator*(umat2x4 m0, uvec4 v1) {
            return umath.InnerProduct1(ref m0, ref v1);
        }
        
        public static umat2 operator*(umat2x4 m0, umat4x2 m1) {
            return umath.InnerProduct1(ref m0, ref m1);
        }
        
        public static umat2x3 operator*(umat2x4 m0, umat4x3 m1) {
            return umath.InnerProduct1(ref m0, ref m1);
        }
        
        public static umat2x4 operator*(umat2x4 m0, umat4 m1) {
            return umath.InnerProduct1(ref m0, ref m1);
        }
        
        public override bool Equals(object obj) {
            return (obj is umat2x4) ? bmath.All(this == (umat2x4)obj) : false;
        }
        
        public override int GetHashCode() {
            return this.m00.GetHashCode() ^ this.m01.GetHashCode() ^ this.m02.GetHashCode() ^ this.m03.GetHashCode() ^ this.m10.GetHashCode() ^ this.m11.GetHashCode() ^ this.m12.GetHashCode() ^ this.m13.GetHashCode();
        }
        
        public override string ToString() {
            return "umat2x4(" + this.m00 + "," + this.m01 + "," + this.m02 + "," + this.m03 + "," + this.m10 + "," + this.m11 + "," + this.m12 + "," + this.m13 + ")";
        }
    }
    
    [DataContract()]
    [StructLayout(LayoutKind.Sequential)]
    public struct umat3x2 {
        
        [DataMember()]
        internal uint m00;
        
        [DataMember()]
        internal uint m01;
        
        [DataMember()]
        internal uint m10;
        
        [DataMember()]
        internal uint m11;
        
        [DataMember()]
        internal uint m20;
        
        [DataMember()]
        internal uint m21;
        
        public umat3x2(uint s) {
            this.m00 = s;
            this.m01 = 0u;
            this.m10 = 0u;
            this.m11 = s;
            this.m20 = 0u;
            this.m21 = 0u;
        }
        
        public umat3x2(umat3x2 m00) {
            this.m00 = m00.m00;
            this.m01 = m00.m01;
            this.m10 = m00.m10;
            this.m11 = m00.m11;
            this.m20 = m00.m20;
            this.m21 = m00.m21;
        }
        
        public umat3x2(umat2 m00, uvec2 m10) {
            this.m00 = m00.m00;
            this.m01 = m00.m01;
            this.m10 = m00.m10;
            this.m11 = m00.m11;
            this.m20 = m10.v0;
            this.m21 = m10.v1;
        }
        
        public umat3x2(uvec2 m00, umat2 m10) {
            this.m00 = m00.v0;
            this.m01 = m00.v1;
            this.m10 = m10.m00;
            this.m11 = m10.m01;
            this.m20 = m10.m10;
            this.m21 = m10.m11;
        }
        
        public umat3x2(uvec2 m00, uvec2 m10, uvec2 m20) {
            this.m00 = m00.v0;
            this.m01 = m00.v1;
            this.m10 = m10.v0;
            this.m11 = m10.v1;
            this.m20 = m20.v0;
            this.m21 = m20.v1;
        }
        
        public umat3x2(uint m00, uint m01, uint m10, uint m11, uint m20, uint m21) {
            this.m00 = m00;
            this.m01 = m01;
            this.m10 = m10;
            this.m11 = m11;
            this.m20 = m20;
            this.m21 = m21;
        }
        
        public umat3x2(umat2 s) {
            this.m00 = s.m00;
            this.m01 = s.m01;
            this.m10 = s.m10;
            this.m11 = s.m11;
            this.m20 = 0u;
            this.m21 = 0u;
        }
        
        public umat3x2(umat2x3 s) {
            this.m00 = s.m00;
            this.m01 = s.m01;
            this.m10 = s.m10;
            this.m11 = s.m11;
            this.m20 = 0u;
            this.m21 = 0u;
        }
        
        public umat3x2(umat2x4 s) {
            this.m00 = s.m00;
            this.m01 = s.m01;
            this.m10 = s.m10;
            this.m11 = s.m11;
            this.m20 = 0u;
            this.m21 = 0u;
        }
        
        public umat3x2(umat3 s) {
            this.m00 = s.m00;
            this.m01 = s.m01;
            this.m10 = s.m10;
            this.m11 = s.m11;
            this.m20 = s.m20;
            this.m21 = s.m21;
        }
        
        public umat3x2(umat3x4 s) {
            this.m00 = s.m00;
            this.m01 = s.m01;
            this.m10 = s.m10;
            this.m11 = s.m11;
            this.m20 = s.m20;
            this.m21 = s.m21;
        }
        
        public umat3x2(umat4x2 s) {
            this.m00 = s.m00;
            this.m01 = s.m01;
            this.m10 = s.m10;
            this.m11 = s.m11;
            this.m20 = s.m20;
            this.m21 = s.m21;
        }
        
        public umat3x2(umat4x3 s) {
            this.m00 = s.m00;
            this.m01 = s.m01;
            this.m10 = s.m10;
            this.m11 = s.m11;
            this.m20 = s.m20;
            this.m21 = s.m21;
        }
        
        public umat3x2(umat4 s) {
            this.m00 = s.m00;
            this.m01 = s.m01;
            this.m10 = s.m10;
            this.m11 = s.m11;
            this.m20 = s.m20;
            this.m21 = s.m21;
        }
        
        public uint this[int i0, int i1] {
            get {
                if (((i0 < 0) 
                            || (2 < i0))) {
                    throw new System.IndexOutOfRangeException("i0");
                }
                if (((i1 < 0) 
                            || (1 < i1))) {
                    throw new System.IndexOutOfRangeException("i1");
                }
                
                unsafe
                {
                    fixed (UInt32* v_ptr = &this.m00)
                    {
                        return *(v_ptr + (i1 + i0 * 2));
                    }
                };
            }
            set {
                if (((i0 < 0) 
                            || (2 < i0))) {
                    throw new System.IndexOutOfRangeException("i0");
                }
                if (((i1 < 0) 
                            || (1 < i1))) {
                    throw new System.IndexOutOfRangeException("i1");
                }
                
                unsafe
                {
                    fixed (UInt32* v_ptr = &this.m00)
                    {
                        *(v_ptr  + (i1 + i0 * 2)) = value;
                    }
                };
            }
        }
        
        public uvec2 this[int i0] {
            get {
                return new uvec2(this[i0, 0], this[i0, 1]);
            }
            set {
                this[i0, 0] = value[0];
                this[i0, 1] = value[1];
            }
        }
        
        public static umat3x2 operator+(umat3x2 v) {
            return new umat3x2(+v.m00, +v.m01, +v.m10, +v.m11, +v.m20, +v.m21);
        }
        
        public static umat3x2 operator~(umat3x2 v) {
            return new umat3x2(~v.m00, ~v.m01, ~v.m10, ~v.m11, ~v.m20, ~v.m21);
        }
        
        public static umat3x2 operator++(umat3x2 v) {
            return new umat3x2(++v.m00, ++v.m01, ++v.m10, ++v.m11, ++v.m20, ++v.m21);
        }
        
        public static umat3x2 operator--(umat3x2 v) {
            return new umat3x2(--v.m00, --v.m01, --v.m10, --v.m11, --v.m20, --v.m21);
        }
        
        public static umat3x2 operator+(uint s, umat3x2 v2) {
            return new umat3x2(s + v2.m00, s + v2.m01, s + v2.m10, s + v2.m11, s + v2.m20, s + v2.m21);
        }
        
        public static umat3x2 operator-(uint s, umat3x2 v2) {
            return new umat3x2(s - v2.m00, s - v2.m01, s - v2.m10, s - v2.m11, s - v2.m20, s - v2.m21);
        }
        
        public static umat3x2 operator*(uint s, umat3x2 v2) {
            return new umat3x2(s * v2.m00, s * v2.m01, s * v2.m10, s * v2.m11, s * v2.m20, s * v2.m21);
        }
        
        public static umat3x2 operator/(uint s, umat3x2 v2) {
            return new umat3x2(s / v2.m00, s / v2.m01, s / v2.m10, s / v2.m11, s / v2.m20, s / v2.m21);
        }
        
        public static umat3x2 operator%(uint s, umat3x2 v2) {
            return new umat3x2(s % v2.m00, s % v2.m01, s % v2.m10, s % v2.m11, s % v2.m20, s % v2.m21);
        }
        
        public static umat3x2 operator&(uint s, umat3x2 v2) {
            return new umat3x2(s & v2.m00, s & v2.m01, s & v2.m10, s & v2.m11, s & v2.m20, s & v2.m21);
        }
        
        public static umat3x2 operator|(uint s, umat3x2 v2) {
            return new umat3x2(s | v2.m00, s | v2.m01, s | v2.m10, s | v2.m11, s | v2.m20, s | v2.m21);
        }
        
        public static umat3x2 operator^(uint s, umat3x2 v2) {
            return new umat3x2(s ^ v2.m00, s ^ v2.m01, s ^ v2.m10, s ^ v2.m11, s ^ v2.m20, s ^ v2.m21);
        }
        
        public static umat3x2 operator+(umat3x2 v1, uint s) {
            return new umat3x2(v1.m00 + s, v1.m01 + s, v1.m10 + s, v1.m11 + s, v1.m20 + s, v1.m21 + s);
        }
        
        public static umat3x2 operator-(umat3x2 v1, uint s) {
            return new umat3x2(v1.m00 - s, v1.m01 - s, v1.m10 - s, v1.m11 - s, v1.m20 - s, v1.m21 - s);
        }
        
        public static umat3x2 operator*(umat3x2 v1, uint s) {
            return new umat3x2(v1.m00 * s, v1.m01 * s, v1.m10 * s, v1.m11 * s, v1.m20 * s, v1.m21 * s);
        }
        
        public static umat3x2 operator/(umat3x2 v1, uint s) {
            return new umat3x2(v1.m00 / s, v1.m01 / s, v1.m10 / s, v1.m11 / s, v1.m20 / s, v1.m21 / s);
        }
        
        public static umat3x2 operator%(umat3x2 v1, uint s) {
            return new umat3x2(v1.m00 % s, v1.m01 % s, v1.m10 % s, v1.m11 % s, v1.m20 % s, v1.m21 % s);
        }
        
        public static umat3x2 operator&(umat3x2 v1, uint s) {
            return new umat3x2(v1.m00 & s, v1.m01 & s, v1.m10 & s, v1.m11 & s, v1.m20 & s, v1.m21 & s);
        }
        
        public static umat3x2 operator|(umat3x2 v1, uint s) {
            return new umat3x2(v1.m00 | s, v1.m01 | s, v1.m10 | s, v1.m11 | s, v1.m20 | s, v1.m21 | s);
        }
        
        public static umat3x2 operator^(umat3x2 v1, uint s) {
            return new umat3x2(v1.m00 ^ s, v1.m01 ^ s, v1.m10 ^ s, v1.m11 ^ s, v1.m20 ^ s, v1.m21 ^ s);
        }
        
        public static umat3x2 operator+(umat3x2 v1, umat3x2 v2) {
            return new umat3x2(v1.m00 + v2.m00, v1.m01 + v2.m01, v1.m10 + v2.m10, v1.m11 + v2.m11, v1.m20 + v2.m20, v1.m21 + v2.m21);
        }
        
        public static umat3x2 operator-(umat3x2 v1, umat3x2 v2) {
            return new umat3x2(v1.m00 - v2.m00, v1.m01 - v2.m01, v1.m10 - v2.m10, v1.m11 - v2.m11, v1.m20 - v2.m20, v1.m21 - v2.m21);
        }
        
        public static umat3x2 operator/(umat3x2 v1, umat3x2 v2) {
            return new umat3x2(v1.m00 / v2.m00, v1.m01 / v2.m01, v1.m10 / v2.m10, v1.m11 / v2.m11, v1.m20 / v2.m20, v1.m21 / v2.m21);
        }
        
        public static umat3x2 operator%(umat3x2 v1, umat3x2 v2) {
            return new umat3x2(v1.m00 % v2.m00, v1.m01 % v2.m01, v1.m10 % v2.m10, v1.m11 % v2.m11, v1.m20 % v2.m20, v1.m21 % v2.m21);
        }
        
        public static umat3x2 operator&(umat3x2 v1, umat3x2 v2) {
            return new umat3x2(v1.m00 & v2.m00, v1.m01 & v2.m01, v1.m10 & v2.m10, v1.m11 & v2.m11, v1.m20 & v2.m20, v1.m21 & v2.m21);
        }
        
        public static umat3x2 operator|(umat3x2 v1, umat3x2 v2) {
            return new umat3x2(v1.m00 | v2.m00, v1.m01 | v2.m01, v1.m10 | v2.m10, v1.m11 | v2.m11, v1.m20 | v2.m20, v1.m21 | v2.m21);
        }
        
        public static umat3x2 operator^(umat3x2 v1, umat3x2 v2) {
            return new umat3x2(v1.m00 ^ v2.m00, v1.m01 ^ v2.m01, v1.m10 ^ v2.m10, v1.m11 ^ v2.m11, v1.m20 ^ v2.m20, v1.m21 ^ v2.m21);
        }
        
        public static bmat3x2 operator==(umat3x2 v1, umat3x2 v2) {
            return new bmat3x2(v1.m00 == v2.m00, v1.m01 == v2.m01, v1.m10 == v2.m10, v1.m11 == v2.m11, v1.m20 == v2.m20, v1.m21 == v2.m21);
        }
        
        public static bmat3x2 operator>(umat3x2 v1, umat3x2 v2) {
            return new bmat3x2(v1.m00 > v2.m00, v1.m01 > v2.m01, v1.m10 > v2.m10, v1.m11 > v2.m11, v1.m20 > v2.m20, v1.m21 > v2.m21);
        }
        
        public static bmat3x2 operator>=(umat3x2 v1, umat3x2 v2) {
            return new bmat3x2(v1.m00 >= v2.m00, v1.m01 >= v2.m01, v1.m10 >= v2.m10, v1.m11 >= v2.m11, v1.m20 >= v2.m20, v1.m21 >= v2.m21);
        }
        
        public static bmat3x2 operator<(umat3x2 v1, umat3x2 v2) {
            return new bmat3x2(v1.m00 < v2.m00, v1.m01 < v2.m01, v1.m10 < v2.m10, v1.m11 < v2.m11, v1.m20 < v2.m20, v1.m21 < v2.m21);
        }
        
        public static bmat3x2 operator<=(umat3x2 v1, umat3x2 v2) {
            return new bmat3x2(v1.m00 <= v2.m00, v1.m01 <= v2.m01, v1.m10 <= v2.m10, v1.m11 <= v2.m11, v1.m20 <= v2.m20, v1.m21 <= v2.m21);
        }
        
        public static bmat3x2 operator!=(umat3x2 v1, umat3x2 v2) {
            return new bmat3x2(v1.m00 != v2.m00, v1.m01 != v2.m01, v1.m10 != v2.m10, v1.m11 != v2.m11, v1.m20 != v2.m20, v1.m21 != v2.m21);
        }
        
        public static uvec2 operator*(uvec3 v0, umat3x2 m1) {
            return umath.InnerProduct1(ref v0, ref m1);
        }
        
        public static uvec3 operator*(umat3x2 m0, uvec2 v1) {
            return umath.InnerProduct1(ref m0, ref v1);
        }
        
        public static umat3x2 operator*(umat3x2 m0, umat2 m1) {
            return umath.InnerProduct1(ref m0, ref m1);
        }
        
        public static umat3 operator*(umat3x2 m0, umat2x3 m1) {
            return umath.InnerProduct1(ref m0, ref m1);
        }
        
        public static umat3x4 operator*(umat3x2 m0, umat2x4 m1) {
            return umath.InnerProduct1(ref m0, ref m1);
        }
        
        public override bool Equals(object obj) {
            return (obj is umat3x2) ? bmath.All(this == (umat3x2)obj) : false;
        }
        
        public override int GetHashCode() {
            return this.m00.GetHashCode() ^ this.m01.GetHashCode() ^ this.m10.GetHashCode() ^ this.m11.GetHashCode() ^ this.m20.GetHashCode() ^ this.m21.GetHashCode();
        }
        
        public override string ToString() {
            return "umat3x2(" + this.m00 + "," + this.m01 + "," + this.m10 + "," + this.m11 + "," + this.m20 + "," + this.m21 + ")";
        }
    }
    
    [DataContract()]
    [StructLayout(LayoutKind.Sequential)]
    public struct umat3 {
        
        [DataMember()]
        internal uint m00;
        
        [DataMember()]
        internal uint m01;
        
        [DataMember()]
        internal uint m02;
        
        [DataMember()]
        internal uint m10;
        
        [DataMember()]
        internal uint m11;
        
        [DataMember()]
        internal uint m12;
        
        [DataMember()]
        internal uint m20;
        
        [DataMember()]
        internal uint m21;
        
        [DataMember()]
        internal uint m22;
        
        public umat3(uint s) {
            this.m00 = s;
            this.m01 = 0u;
            this.m02 = 0u;
            this.m10 = 0u;
            this.m11 = s;
            this.m12 = 0u;
            this.m20 = 0u;
            this.m21 = 0u;
            this.m22 = s;
        }
        
        public umat3(umat3 m00) {
            this.m00 = m00.m00;
            this.m01 = m00.m01;
            this.m02 = m00.m02;
            this.m10 = m00.m10;
            this.m11 = m00.m11;
            this.m12 = m00.m12;
            this.m20 = m00.m20;
            this.m21 = m00.m21;
            this.m22 = m00.m22;
        }
        
        public umat3(umat2x3 m00, uvec3 m10) {
            this.m00 = m00.m00;
            this.m01 = m00.m01;
            this.m02 = m00.m02;
            this.m10 = m00.m10;
            this.m11 = m00.m11;
            this.m12 = m00.m12;
            this.m20 = m10.v0;
            this.m21 = m10.v1;
            this.m22 = m10.v2;
        }
        
        public umat3(uvec3 m00, umat2x3 m10) {
            this.m00 = m00.v0;
            this.m01 = m00.v1;
            this.m02 = m00.v2;
            this.m10 = m10.m00;
            this.m11 = m10.m01;
            this.m12 = m10.m02;
            this.m20 = m10.m10;
            this.m21 = m10.m11;
            this.m22 = m10.m12;
        }
        
        public umat3(uvec3 m00, uvec3 m10, uvec3 m20) {
            this.m00 = m00.v0;
            this.m01 = m00.v1;
            this.m02 = m00.v2;
            this.m10 = m10.v0;
            this.m11 = m10.v1;
            this.m12 = m10.v2;
            this.m20 = m20.v0;
            this.m21 = m20.v1;
            this.m22 = m20.v2;
        }
        
        public umat3(uvec2 m00, uint m01, uvec2 m10, uint m11, uvec2 m20, uint m21) {
            this.m00 = m00.v0;
            this.m01 = m00.v1;
            this.m02 = m01;
            this.m10 = m10.v0;
            this.m11 = m10.v1;
            this.m12 = m11;
            this.m20 = m20.v0;
            this.m21 = m20.v1;
            this.m22 = m21;
        }
        
        public umat3(uint m00, uvec2 m01, uint m10, uvec2 m11, uint m20, uvec2 m21) {
            this.m00 = m00;
            this.m01 = m01.v0;
            this.m02 = m01.v1;
            this.m10 = m10;
            this.m11 = m11.v0;
            this.m12 = m11.v1;
            this.m20 = m20;
            this.m21 = m21.v0;
            this.m22 = m21.v1;
        }
        
        public umat3(uint m00, uint m01, uint m02, uint m10, uint m11, uint m12, uint m20, uint m21, uint m22) {
            this.m00 = m00;
            this.m01 = m01;
            this.m02 = m02;
            this.m10 = m10;
            this.m11 = m11;
            this.m12 = m12;
            this.m20 = m20;
            this.m21 = m21;
            this.m22 = m22;
        }
        
        public umat3(umat2 s) {
            this.m00 = s.m00;
            this.m01 = s.m01;
            this.m02 = 0u;
            this.m10 = s.m10;
            this.m11 = s.m11;
            this.m12 = 0u;
            this.m20 = 0u;
            this.m21 = 0u;
            this.m22 = 0u;
        }
        
        public umat3(umat2x3 s) {
            this.m00 = s.m00;
            this.m01 = s.m01;
            this.m02 = s.m02;
            this.m10 = s.m10;
            this.m11 = s.m11;
            this.m12 = s.m12;
            this.m20 = 0u;
            this.m21 = 0u;
            this.m22 = 0u;
        }
        
        public umat3(umat2x4 s) {
            this.m00 = s.m00;
            this.m01 = s.m01;
            this.m02 = s.m02;
            this.m10 = s.m10;
            this.m11 = s.m11;
            this.m12 = s.m12;
            this.m20 = 0u;
            this.m21 = 0u;
            this.m22 = 0u;
        }
        
        public umat3(umat3x2 s) {
            this.m00 = s.m00;
            this.m01 = s.m01;
            this.m02 = 0u;
            this.m10 = s.m10;
            this.m11 = s.m11;
            this.m12 = 0u;
            this.m20 = s.m20;
            this.m21 = s.m21;
            this.m22 = 0u;
        }
        
        public umat3(umat3x4 s) {
            this.m00 = s.m00;
            this.m01 = s.m01;
            this.m02 = s.m02;
            this.m10 = s.m10;
            this.m11 = s.m11;
            this.m12 = s.m12;
            this.m20 = s.m20;
            this.m21 = s.m21;
            this.m22 = s.m22;
        }
        
        public umat3(umat4x2 s) {
            this.m00 = s.m00;
            this.m01 = s.m01;
            this.m02 = 0u;
            this.m10 = s.m10;
            this.m11 = s.m11;
            this.m12 = 0u;
            this.m20 = s.m20;
            this.m21 = s.m21;
            this.m22 = 0u;
        }
        
        public umat3(umat4x3 s) {
            this.m00 = s.m00;
            this.m01 = s.m01;
            this.m02 = s.m02;
            this.m10 = s.m10;
            this.m11 = s.m11;
            this.m12 = s.m12;
            this.m20 = s.m20;
            this.m21 = s.m21;
            this.m22 = s.m22;
        }
        
        public umat3(umat4 s) {
            this.m00 = s.m00;
            this.m01 = s.m01;
            this.m02 = s.m02;
            this.m10 = s.m10;
            this.m11 = s.m11;
            this.m12 = s.m12;
            this.m20 = s.m20;
            this.m21 = s.m21;
            this.m22 = s.m22;
        }
        
        public uint this[int i0, int i1] {
            get {
                if (((i0 < 0) 
                            || (2 < i0))) {
                    throw new System.IndexOutOfRangeException("i0");
                }
                if (((i1 < 0) 
                            || (2 < i1))) {
                    throw new System.IndexOutOfRangeException("i1");
                }
                
                unsafe
                {
                    fixed (UInt32* v_ptr = &this.m00)
                    {
                        return *(v_ptr + (i1 + i0 * 3));
                    }
                };
            }
            set {
                if (((i0 < 0) 
                            || (2 < i0))) {
                    throw new System.IndexOutOfRangeException("i0");
                }
                if (((i1 < 0) 
                            || (2 < i1))) {
                    throw new System.IndexOutOfRangeException("i1");
                }
                
                unsafe
                {
                    fixed (UInt32* v_ptr = &this.m00)
                    {
                        *(v_ptr  + (i1 + i0 * 3)) = value;
                    }
                };
            }
        }
        
        public uvec3 this[int i0] {
            get {
                return new uvec3(this[i0, 0], this[i0, 1], this[i0, 2]);
            }
            set {
                this[i0, 0] = value[0];
                this[i0, 1] = value[1];
                this[i0, 2] = value[2];
            }
        }
        
        public static umat3 operator+(umat3 v) {
            return new umat3(+v.m00, +v.m01, +v.m02, +v.m10, +v.m11, +v.m12, +v.m20, +v.m21, +v.m22);
        }
        
        public static umat3 operator~(umat3 v) {
            return new umat3(~v.m00, ~v.m01, ~v.m02, ~v.m10, ~v.m11, ~v.m12, ~v.m20, ~v.m21, ~v.m22);
        }
        
        public static umat3 operator++(umat3 v) {
            return new umat3(++v.m00, ++v.m01, ++v.m02, ++v.m10, ++v.m11, ++v.m12, ++v.m20, ++v.m21, ++v.m22);
        }
        
        public static umat3 operator--(umat3 v) {
            return new umat3(--v.m00, --v.m01, --v.m02, --v.m10, --v.m11, --v.m12, --v.m20, --v.m21, --v.m22);
        }
        
        public static umat3 operator+(uint s, umat3 v2) {
            return new umat3(s + v2.m00, s + v2.m01, s + v2.m02, s + v2.m10, s + v2.m11, s + v2.m12, s + v2.m20, s + v2.m21, s + v2.m22);
        }
        
        public static umat3 operator-(uint s, umat3 v2) {
            return new umat3(s - v2.m00, s - v2.m01, s - v2.m02, s - v2.m10, s - v2.m11, s - v2.m12, s - v2.m20, s - v2.m21, s - v2.m22);
        }
        
        public static umat3 operator*(uint s, umat3 v2) {
            return new umat3(s * v2.m00, s * v2.m01, s * v2.m02, s * v2.m10, s * v2.m11, s * v2.m12, s * v2.m20, s * v2.m21, s * v2.m22);
        }
        
        public static umat3 operator/(uint s, umat3 v2) {
            return new umat3(s / v2.m00, s / v2.m01, s / v2.m02, s / v2.m10, s / v2.m11, s / v2.m12, s / v2.m20, s / v2.m21, s / v2.m22);
        }
        
        public static umat3 operator%(uint s, umat3 v2) {
            return new umat3(s % v2.m00, s % v2.m01, s % v2.m02, s % v2.m10, s % v2.m11, s % v2.m12, s % v2.m20, s % v2.m21, s % v2.m22);
        }
        
        public static umat3 operator&(uint s, umat3 v2) {
            return new umat3(s & v2.m00, s & v2.m01, s & v2.m02, s & v2.m10, s & v2.m11, s & v2.m12, s & v2.m20, s & v2.m21, s & v2.m22);
        }
        
        public static umat3 operator|(uint s, umat3 v2) {
            return new umat3(s | v2.m00, s | v2.m01, s | v2.m02, s | v2.m10, s | v2.m11, s | v2.m12, s | v2.m20, s | v2.m21, s | v2.m22);
        }
        
        public static umat3 operator^(uint s, umat3 v2) {
            return new umat3(s ^ v2.m00, s ^ v2.m01, s ^ v2.m02, s ^ v2.m10, s ^ v2.m11, s ^ v2.m12, s ^ v2.m20, s ^ v2.m21, s ^ v2.m22);
        }
        
        public static umat3 operator+(umat3 v1, uint s) {
            return new umat3(v1.m00 + s, v1.m01 + s, v1.m02 + s, v1.m10 + s, v1.m11 + s, v1.m12 + s, v1.m20 + s, v1.m21 + s, v1.m22 + s);
        }
        
        public static umat3 operator-(umat3 v1, uint s) {
            return new umat3(v1.m00 - s, v1.m01 - s, v1.m02 - s, v1.m10 - s, v1.m11 - s, v1.m12 - s, v1.m20 - s, v1.m21 - s, v1.m22 - s);
        }
        
        public static umat3 operator*(umat3 v1, uint s) {
            return new umat3(v1.m00 * s, v1.m01 * s, v1.m02 * s, v1.m10 * s, v1.m11 * s, v1.m12 * s, v1.m20 * s, v1.m21 * s, v1.m22 * s);
        }
        
        public static umat3 operator/(umat3 v1, uint s) {
            return new umat3(v1.m00 / s, v1.m01 / s, v1.m02 / s, v1.m10 / s, v1.m11 / s, v1.m12 / s, v1.m20 / s, v1.m21 / s, v1.m22 / s);
        }
        
        public static umat3 operator%(umat3 v1, uint s) {
            return new umat3(v1.m00 % s, v1.m01 % s, v1.m02 % s, v1.m10 % s, v1.m11 % s, v1.m12 % s, v1.m20 % s, v1.m21 % s, v1.m22 % s);
        }
        
        public static umat3 operator&(umat3 v1, uint s) {
            return new umat3(v1.m00 & s, v1.m01 & s, v1.m02 & s, v1.m10 & s, v1.m11 & s, v1.m12 & s, v1.m20 & s, v1.m21 & s, v1.m22 & s);
        }
        
        public static umat3 operator|(umat3 v1, uint s) {
            return new umat3(v1.m00 | s, v1.m01 | s, v1.m02 | s, v1.m10 | s, v1.m11 | s, v1.m12 | s, v1.m20 | s, v1.m21 | s, v1.m22 | s);
        }
        
        public static umat3 operator^(umat3 v1, uint s) {
            return new umat3(v1.m00 ^ s, v1.m01 ^ s, v1.m02 ^ s, v1.m10 ^ s, v1.m11 ^ s, v1.m12 ^ s, v1.m20 ^ s, v1.m21 ^ s, v1.m22 ^ s);
        }
        
        public static umat3 operator+(umat3 v1, umat3 v2) {
            return new umat3(v1.m00 + v2.m00, v1.m01 + v2.m01, v1.m02 + v2.m02, v1.m10 + v2.m10, v1.m11 + v2.m11, v1.m12 + v2.m12, v1.m20 + v2.m20, v1.m21 + v2.m21, v1.m22 + v2.m22);
        }
        
        public static umat3 operator-(umat3 v1, umat3 v2) {
            return new umat3(v1.m00 - v2.m00, v1.m01 - v2.m01, v1.m02 - v2.m02, v1.m10 - v2.m10, v1.m11 - v2.m11, v1.m12 - v2.m12, v1.m20 - v2.m20, v1.m21 - v2.m21, v1.m22 - v2.m22);
        }
        
        public static umat3 operator/(umat3 v1, umat3 v2) {
            return new umat3(v1.m00 / v2.m00, v1.m01 / v2.m01, v1.m02 / v2.m02, v1.m10 / v2.m10, v1.m11 / v2.m11, v1.m12 / v2.m12, v1.m20 / v2.m20, v1.m21 / v2.m21, v1.m22 / v2.m22);
        }
        
        public static umat3 operator%(umat3 v1, umat3 v2) {
            return new umat3(v1.m00 % v2.m00, v1.m01 % v2.m01, v1.m02 % v2.m02, v1.m10 % v2.m10, v1.m11 % v2.m11, v1.m12 % v2.m12, v1.m20 % v2.m20, v1.m21 % v2.m21, v1.m22 % v2.m22);
        }
        
        public static umat3 operator&(umat3 v1, umat3 v2) {
            return new umat3(v1.m00 & v2.m00, v1.m01 & v2.m01, v1.m02 & v2.m02, v1.m10 & v2.m10, v1.m11 & v2.m11, v1.m12 & v2.m12, v1.m20 & v2.m20, v1.m21 & v2.m21, v1.m22 & v2.m22);
        }
        
        public static umat3 operator|(umat3 v1, umat3 v2) {
            return new umat3(v1.m00 | v2.m00, v1.m01 | v2.m01, v1.m02 | v2.m02, v1.m10 | v2.m10, v1.m11 | v2.m11, v1.m12 | v2.m12, v1.m20 | v2.m20, v1.m21 | v2.m21, v1.m22 | v2.m22);
        }
        
        public static umat3 operator^(umat3 v1, umat3 v2) {
            return new umat3(v1.m00 ^ v2.m00, v1.m01 ^ v2.m01, v1.m02 ^ v2.m02, v1.m10 ^ v2.m10, v1.m11 ^ v2.m11, v1.m12 ^ v2.m12, v1.m20 ^ v2.m20, v1.m21 ^ v2.m21, v1.m22 ^ v2.m22);
        }
        
        public static bmat3 operator==(umat3 v1, umat3 v2) {
            return new bmat3(v1.m00 == v2.m00, v1.m01 == v2.m01, v1.m02 == v2.m02, v1.m10 == v2.m10, v1.m11 == v2.m11, v1.m12 == v2.m12, v1.m20 == v2.m20, v1.m21 == v2.m21, v1.m22 == v2.m22);
        }
        
        public static bmat3 operator>(umat3 v1, umat3 v2) {
            return new bmat3(v1.m00 > v2.m00, v1.m01 > v2.m01, v1.m02 > v2.m02, v1.m10 > v2.m10, v1.m11 > v2.m11, v1.m12 > v2.m12, v1.m20 > v2.m20, v1.m21 > v2.m21, v1.m22 > v2.m22);
        }
        
        public static bmat3 operator>=(umat3 v1, umat3 v2) {
            return new bmat3(v1.m00 >= v2.m00, v1.m01 >= v2.m01, v1.m02 >= v2.m02, v1.m10 >= v2.m10, v1.m11 >= v2.m11, v1.m12 >= v2.m12, v1.m20 >= v2.m20, v1.m21 >= v2.m21, v1.m22 >= v2.m22);
        }
        
        public static bmat3 operator<(umat3 v1, umat3 v2) {
            return new bmat3(v1.m00 < v2.m00, v1.m01 < v2.m01, v1.m02 < v2.m02, v1.m10 < v2.m10, v1.m11 < v2.m11, v1.m12 < v2.m12, v1.m20 < v2.m20, v1.m21 < v2.m21, v1.m22 < v2.m22);
        }
        
        public static bmat3 operator<=(umat3 v1, umat3 v2) {
            return new bmat3(v1.m00 <= v2.m00, v1.m01 <= v2.m01, v1.m02 <= v2.m02, v1.m10 <= v2.m10, v1.m11 <= v2.m11, v1.m12 <= v2.m12, v1.m20 <= v2.m20, v1.m21 <= v2.m21, v1.m22 <= v2.m22);
        }
        
        public static bmat3 operator!=(umat3 v1, umat3 v2) {
            return new bmat3(v1.m00 != v2.m00, v1.m01 != v2.m01, v1.m02 != v2.m02, v1.m10 != v2.m10, v1.m11 != v2.m11, v1.m12 != v2.m12, v1.m20 != v2.m20, v1.m21 != v2.m21, v1.m22 != v2.m22);
        }
        
        public static uvec3 operator*(uvec3 v0, umat3 m1) {
            return umath.InnerProduct1(ref v0, ref m1);
        }
        
        public static uvec3 operator*(umat3 m0, uvec3 v1) {
            return umath.InnerProduct1(ref m0, ref v1);
        }
        
        public static umat3x2 operator*(umat3 m0, umat3x2 m1) {
            return umath.InnerProduct1(ref m0, ref m1);
        }
        
        public static umat3 operator*(umat3 m0, umat3 m1) {
            return umath.InnerProduct1(ref m0, ref m1);
        }
        
        public static umat3x4 operator*(umat3 m0, umat3x4 m1) {
            return umath.InnerProduct1(ref m0, ref m1);
        }
        
        public override bool Equals(object obj) {
            return (obj is umat3) ? bmath.All(this == (umat3)obj) : false;
        }
        
        public override int GetHashCode() {
            return this.m00.GetHashCode() ^ this.m01.GetHashCode() ^ this.m02.GetHashCode() ^ this.m10.GetHashCode() ^ this.m11.GetHashCode() ^ this.m12.GetHashCode() ^ this.m20.GetHashCode() ^ this.m21.GetHashCode() ^ this.m22.GetHashCode();
        }
        
        public override string ToString() {
            return "umat3(" + this.m00 + "," + this.m01 + "," + this.m02 + "," + this.m10 + "," + this.m11 + "," + this.m12 + "," + this.m20 + "," + this.m21 + "," + this.m22 + ")";
        }
    }
    
    [DataContract()]
    [StructLayout(LayoutKind.Sequential)]
    public struct umat3x4 {
        
        [DataMember()]
        internal uint m00;
        
        [DataMember()]
        internal uint m01;
        
        [DataMember()]
        internal uint m02;
        
        [DataMember()]
        internal uint m03;
        
        [DataMember()]
        internal uint m10;
        
        [DataMember()]
        internal uint m11;
        
        [DataMember()]
        internal uint m12;
        
        [DataMember()]
        internal uint m13;
        
        [DataMember()]
        internal uint m20;
        
        [DataMember()]
        internal uint m21;
        
        [DataMember()]
        internal uint m22;
        
        [DataMember()]
        internal uint m23;
        
        public umat3x4(uint s) {
            this.m00 = s;
            this.m01 = 0u;
            this.m02 = 0u;
            this.m03 = 0u;
            this.m10 = 0u;
            this.m11 = s;
            this.m12 = 0u;
            this.m13 = 0u;
            this.m20 = 0u;
            this.m21 = 0u;
            this.m22 = s;
            this.m23 = 0u;
        }
        
        public umat3x4(umat3x4 m00) {
            this.m00 = m00.m00;
            this.m01 = m00.m01;
            this.m02 = m00.m02;
            this.m03 = m00.m03;
            this.m10 = m00.m10;
            this.m11 = m00.m11;
            this.m12 = m00.m12;
            this.m13 = m00.m13;
            this.m20 = m00.m20;
            this.m21 = m00.m21;
            this.m22 = m00.m22;
            this.m23 = m00.m23;
        }
        
        public umat3x4(umat3x2 m00, umat3x2 m01) {
            this.m00 = m00.m00;
            this.m01 = m00.m01;
            this.m02 = m01.m00;
            this.m03 = m01.m01;
            this.m10 = m00.m10;
            this.m11 = m00.m11;
            this.m12 = m01.m10;
            this.m13 = m01.m11;
            this.m20 = m00.m20;
            this.m21 = m00.m21;
            this.m22 = m01.m20;
            this.m23 = m01.m21;
        }
        
        public umat3x4(umat2x4 m00, uvec4 m10) {
            this.m00 = m00.m00;
            this.m01 = m00.m01;
            this.m02 = m00.m02;
            this.m03 = m00.m03;
            this.m10 = m00.m10;
            this.m11 = m00.m11;
            this.m12 = m00.m12;
            this.m13 = m00.m13;
            this.m20 = m10.v0;
            this.m21 = m10.v1;
            this.m22 = m10.v2;
            this.m23 = m10.v3;
        }
        
        public umat3x4(umat2 m00, umat2 m01, uvec2 m10, uvec2 m11) {
            this.m00 = m00.m00;
            this.m01 = m00.m01;
            this.m02 = m01.m00;
            this.m03 = m01.m01;
            this.m10 = m00.m10;
            this.m11 = m00.m11;
            this.m12 = m01.m10;
            this.m13 = m01.m11;
            this.m20 = m10.v0;
            this.m21 = m10.v1;
            this.m22 = m11.v0;
            this.m23 = m11.v1;
        }
        
        public umat3x4(uvec4 m00, umat2x4 m10) {
            this.m00 = m00.v0;
            this.m01 = m00.v1;
            this.m02 = m00.v2;
            this.m03 = m00.v3;
            this.m10 = m10.m00;
            this.m11 = m10.m01;
            this.m12 = m10.m02;
            this.m13 = m10.m03;
            this.m20 = m10.m10;
            this.m21 = m10.m11;
            this.m22 = m10.m12;
            this.m23 = m10.m13;
        }
        
        public umat3x4(uvec2 m00, uvec2 m01, umat2 m10, umat2 m11) {
            this.m00 = m00.v0;
            this.m01 = m00.v1;
            this.m02 = m01.v0;
            this.m03 = m01.v1;
            this.m10 = m10.m00;
            this.m11 = m10.m01;
            this.m12 = m11.m00;
            this.m13 = m11.m01;
            this.m20 = m10.m10;
            this.m21 = m10.m11;
            this.m22 = m11.m10;
            this.m23 = m11.m11;
        }
        
        public umat3x4(uvec4 m00, uvec4 m10, uvec4 m20) {
            this.m00 = m00.v0;
            this.m01 = m00.v1;
            this.m02 = m00.v2;
            this.m03 = m00.v3;
            this.m10 = m10.v0;
            this.m11 = m10.v1;
            this.m12 = m10.v2;
            this.m13 = m10.v3;
            this.m20 = m20.v0;
            this.m21 = m20.v1;
            this.m22 = m20.v2;
            this.m23 = m20.v3;
        }
        
        public umat3x4(uvec2 m00, uvec2 m01, uvec2 m10, uvec2 m11, uvec2 m20, uvec2 m21) {
            this.m00 = m00.v0;
            this.m01 = m00.v1;
            this.m02 = m01.v0;
            this.m03 = m01.v1;
            this.m10 = m10.v0;
            this.m11 = m10.v1;
            this.m12 = m11.v0;
            this.m13 = m11.v1;
            this.m20 = m20.v0;
            this.m21 = m20.v1;
            this.m22 = m21.v0;
            this.m23 = m21.v1;
        }
        
        public umat3x4(uvec3 m00, uint m01, uvec3 m10, uint m11, uvec3 m20, uint m21) {
            this.m00 = m00.v0;
            this.m01 = m00.v1;
            this.m02 = m00.v2;
            this.m03 = m01;
            this.m10 = m10.v0;
            this.m11 = m10.v1;
            this.m12 = m10.v2;
            this.m13 = m11;
            this.m20 = m20.v0;
            this.m21 = m20.v1;
            this.m22 = m20.v2;
            this.m23 = m21;
        }
        
        public umat3x4(uint m00, uvec3 m01, uint m10, uvec3 m11, uint m20, uvec3 m21) {
            this.m00 = m00;
            this.m01 = m01.v0;
            this.m02 = m01.v1;
            this.m03 = m01.v2;
            this.m10 = m10;
            this.m11 = m11.v0;
            this.m12 = m11.v1;
            this.m13 = m11.v2;
            this.m20 = m20;
            this.m21 = m21.v0;
            this.m22 = m21.v1;
            this.m23 = m21.v2;
        }
        
        public umat3x4(uvec2 m00, uint m01, uint m02, uvec2 m10, uint m11, uint m12, uvec2 m20, uint m21, uint m22) {
            this.m00 = m00.v0;
            this.m01 = m00.v1;
            this.m02 = m01;
            this.m03 = m02;
            this.m10 = m10.v0;
            this.m11 = m10.v1;
            this.m12 = m11;
            this.m13 = m12;
            this.m20 = m20.v0;
            this.m21 = m20.v1;
            this.m22 = m21;
            this.m23 = m22;
        }
        
        public umat3x4(uint m00, uvec2 m01, uint m02, uint m10, uvec2 m11, uint m12, uint m20, uvec2 m21, uint m22) {
            this.m00 = m00;
            this.m01 = m01.v0;
            this.m02 = m01.v1;
            this.m03 = m02;
            this.m10 = m10;
            this.m11 = m11.v0;
            this.m12 = m11.v1;
            this.m13 = m12;
            this.m20 = m20;
            this.m21 = m21.v0;
            this.m22 = m21.v1;
            this.m23 = m22;
        }
        
        public umat3x4(uint m00, uint m01, uvec2 m02, uint m10, uint m11, uvec2 m12, uint m20, uint m21, uvec2 m22) {
            this.m00 = m00;
            this.m01 = m01;
            this.m02 = m02.v0;
            this.m03 = m02.v1;
            this.m10 = m10;
            this.m11 = m11;
            this.m12 = m12.v0;
            this.m13 = m12.v1;
            this.m20 = m20;
            this.m21 = m21;
            this.m22 = m22.v0;
            this.m23 = m22.v1;
        }
        
        public umat3x4(uint m00, uint m01, uint m02, uint m03, uint m10, uint m11, uint m12, uint m13, uint m20, uint m21, uint m22, uint m23) {
            this.m00 = m00;
            this.m01 = m01;
            this.m02 = m02;
            this.m03 = m03;
            this.m10 = m10;
            this.m11 = m11;
            this.m12 = m12;
            this.m13 = m13;
            this.m20 = m20;
            this.m21 = m21;
            this.m22 = m22;
            this.m23 = m23;
        }
        
        public umat3x4(umat2 s) {
            this.m00 = s.m00;
            this.m01 = s.m01;
            this.m02 = 0u;
            this.m03 = 0u;
            this.m10 = s.m10;
            this.m11 = s.m11;
            this.m12 = 0u;
            this.m13 = 0u;
            this.m20 = 0u;
            this.m21 = 0u;
            this.m22 = 0u;
            this.m23 = 0u;
        }
        
        public umat3x4(umat2x3 s) {
            this.m00 = s.m00;
            this.m01 = s.m01;
            this.m02 = s.m02;
            this.m03 = 0u;
            this.m10 = s.m10;
            this.m11 = s.m11;
            this.m12 = s.m12;
            this.m13 = 0u;
            this.m20 = 0u;
            this.m21 = 0u;
            this.m22 = 0u;
            this.m23 = 0u;
        }
        
        public umat3x4(umat2x4 s) {
            this.m00 = s.m00;
            this.m01 = s.m01;
            this.m02 = s.m02;
            this.m03 = s.m03;
            this.m10 = s.m10;
            this.m11 = s.m11;
            this.m12 = s.m12;
            this.m13 = s.m13;
            this.m20 = 0u;
            this.m21 = 0u;
            this.m22 = 0u;
            this.m23 = 0u;
        }
        
        public umat3x4(umat3x2 s) {
            this.m00 = s.m00;
            this.m01 = s.m01;
            this.m02 = 0u;
            this.m03 = 0u;
            this.m10 = s.m10;
            this.m11 = s.m11;
            this.m12 = 0u;
            this.m13 = 0u;
            this.m20 = s.m20;
            this.m21 = s.m21;
            this.m22 = 0u;
            this.m23 = 0u;
        }
        
        public umat3x4(umat3 s) {
            this.m00 = s.m00;
            this.m01 = s.m01;
            this.m02 = s.m02;
            this.m03 = 0u;
            this.m10 = s.m10;
            this.m11 = s.m11;
            this.m12 = s.m12;
            this.m13 = 0u;
            this.m20 = s.m20;
            this.m21 = s.m21;
            this.m22 = s.m22;
            this.m23 = 0u;
        }
        
        public umat3x4(umat4x2 s) {
            this.m00 = s.m00;
            this.m01 = s.m01;
            this.m02 = 0u;
            this.m03 = 0u;
            this.m10 = s.m10;
            this.m11 = s.m11;
            this.m12 = 0u;
            this.m13 = 0u;
            this.m20 = s.m20;
            this.m21 = s.m21;
            this.m22 = 0u;
            this.m23 = 0u;
        }
        
        public umat3x4(umat4x3 s) {
            this.m00 = s.m00;
            this.m01 = s.m01;
            this.m02 = s.m02;
            this.m03 = 0u;
            this.m10 = s.m10;
            this.m11 = s.m11;
            this.m12 = s.m12;
            this.m13 = 0u;
            this.m20 = s.m20;
            this.m21 = s.m21;
            this.m22 = s.m22;
            this.m23 = 0u;
        }
        
        public umat3x4(umat4 s) {
            this.m00 = s.m00;
            this.m01 = s.m01;
            this.m02 = s.m02;
            this.m03 = s.m03;
            this.m10 = s.m10;
            this.m11 = s.m11;
            this.m12 = s.m12;
            this.m13 = s.m13;
            this.m20 = s.m20;
            this.m21 = s.m21;
            this.m22 = s.m22;
            this.m23 = s.m23;
        }
        
        public uint this[int i0, int i1] {
            get {
                if (((i0 < 0) 
                            || (2 < i0))) {
                    throw new System.IndexOutOfRangeException("i0");
                }
                if (((i1 < 0) 
                            || (3 < i1))) {
                    throw new System.IndexOutOfRangeException("i1");
                }
                
                unsafe
                {
                    fixed (UInt32* v_ptr = &this.m00)
                    {
                        return *(v_ptr + (i1 + i0 * 4));
                    }
                };
            }
            set {
                if (((i0 < 0) 
                            || (2 < i0))) {
                    throw new System.IndexOutOfRangeException("i0");
                }
                if (((i1 < 0) 
                            || (3 < i1))) {
                    throw new System.IndexOutOfRangeException("i1");
                }
                
                unsafe
                {
                    fixed (UInt32* v_ptr = &this.m00)
                    {
                        *(v_ptr  + (i1 + i0 * 4)) = value;
                    }
                };
            }
        }
        
        public uvec4 this[int i0] {
            get {
                return new uvec4(this[i0, 0], this[i0, 1], this[i0, 2], this[i0, 3]);
            }
            set {
                this[i0, 0] = value[0];
                this[i0, 1] = value[1];
                this[i0, 2] = value[2];
                this[i0, 3] = value[3];
            }
        }
        
        public static umat3x4 operator+(umat3x4 v) {
            return new umat3x4(+v.m00, +v.m01, +v.m02, +v.m03, +v.m10, +v.m11, +v.m12, +v.m13, +v.m20, +v.m21, +v.m22, +v.m23);
        }
        
        public static umat3x4 operator~(umat3x4 v) {
            return new umat3x4(~v.m00, ~v.m01, ~v.m02, ~v.m03, ~v.m10, ~v.m11, ~v.m12, ~v.m13, ~v.m20, ~v.m21, ~v.m22, ~v.m23);
        }
        
        public static umat3x4 operator++(umat3x4 v) {
            return new umat3x4(++v.m00, ++v.m01, ++v.m02, ++v.m03, ++v.m10, ++v.m11, ++v.m12, ++v.m13, ++v.m20, ++v.m21, ++v.m22, ++v.m23);
        }
        
        public static umat3x4 operator--(umat3x4 v) {
            return new umat3x4(--v.m00, --v.m01, --v.m02, --v.m03, --v.m10, --v.m11, --v.m12, --v.m13, --v.m20, --v.m21, --v.m22, --v.m23);
        }
        
        public static umat3x4 operator+(uint s, umat3x4 v2) {
            return new umat3x4(s + v2.m00, s + v2.m01, s + v2.m02, s + v2.m03, s + v2.m10, s + v2.m11, s + v2.m12, s + v2.m13, s + v2.m20, s + v2.m21, s + v2.m22, s + v2.m23);
        }
        
        public static umat3x4 operator-(uint s, umat3x4 v2) {
            return new umat3x4(s - v2.m00, s - v2.m01, s - v2.m02, s - v2.m03, s - v2.m10, s - v2.m11, s - v2.m12, s - v2.m13, s - v2.m20, s - v2.m21, s - v2.m22, s - v2.m23);
        }
        
        public static umat3x4 operator*(uint s, umat3x4 v2) {
            return new umat3x4(s * v2.m00, s * v2.m01, s * v2.m02, s * v2.m03, s * v2.m10, s * v2.m11, s * v2.m12, s * v2.m13, s * v2.m20, s * v2.m21, s * v2.m22, s * v2.m23);
        }
        
        public static umat3x4 operator/(uint s, umat3x4 v2) {
            return new umat3x4(s / v2.m00, s / v2.m01, s / v2.m02, s / v2.m03, s / v2.m10, s / v2.m11, s / v2.m12, s / v2.m13, s / v2.m20, s / v2.m21, s / v2.m22, s / v2.m23);
        }
        
        public static umat3x4 operator%(uint s, umat3x4 v2) {
            return new umat3x4(s % v2.m00, s % v2.m01, s % v2.m02, s % v2.m03, s % v2.m10, s % v2.m11, s % v2.m12, s % v2.m13, s % v2.m20, s % v2.m21, s % v2.m22, s % v2.m23);
        }
        
        public static umat3x4 operator&(uint s, umat3x4 v2) {
            return new umat3x4(s & v2.m00, s & v2.m01, s & v2.m02, s & v2.m03, s & v2.m10, s & v2.m11, s & v2.m12, s & v2.m13, s & v2.m20, s & v2.m21, s & v2.m22, s & v2.m23);
        }
        
        public static umat3x4 operator|(uint s, umat3x4 v2) {
            return new umat3x4(s | v2.m00, s | v2.m01, s | v2.m02, s | v2.m03, s | v2.m10, s | v2.m11, s | v2.m12, s | v2.m13, s | v2.m20, s | v2.m21, s | v2.m22, s | v2.m23);
        }
        
        public static umat3x4 operator^(uint s, umat3x4 v2) {
            return new umat3x4(s ^ v2.m00, s ^ v2.m01, s ^ v2.m02, s ^ v2.m03, s ^ v2.m10, s ^ v2.m11, s ^ v2.m12, s ^ v2.m13, s ^ v2.m20, s ^ v2.m21, s ^ v2.m22, s ^ v2.m23);
        }
        
        public static umat3x4 operator+(umat3x4 v1, uint s) {
            return new umat3x4(v1.m00 + s, v1.m01 + s, v1.m02 + s, v1.m03 + s, v1.m10 + s, v1.m11 + s, v1.m12 + s, v1.m13 + s, v1.m20 + s, v1.m21 + s, v1.m22 + s, v1.m23 + s);
        }
        
        public static umat3x4 operator-(umat3x4 v1, uint s) {
            return new umat3x4(v1.m00 - s, v1.m01 - s, v1.m02 - s, v1.m03 - s, v1.m10 - s, v1.m11 - s, v1.m12 - s, v1.m13 - s, v1.m20 - s, v1.m21 - s, v1.m22 - s, v1.m23 - s);
        }
        
        public static umat3x4 operator*(umat3x4 v1, uint s) {
            return new umat3x4(v1.m00 * s, v1.m01 * s, v1.m02 * s, v1.m03 * s, v1.m10 * s, v1.m11 * s, v1.m12 * s, v1.m13 * s, v1.m20 * s, v1.m21 * s, v1.m22 * s, v1.m23 * s);
        }
        
        public static umat3x4 operator/(umat3x4 v1, uint s) {
            return new umat3x4(v1.m00 / s, v1.m01 / s, v1.m02 / s, v1.m03 / s, v1.m10 / s, v1.m11 / s, v1.m12 / s, v1.m13 / s, v1.m20 / s, v1.m21 / s, v1.m22 / s, v1.m23 / s);
        }
        
        public static umat3x4 operator%(umat3x4 v1, uint s) {
            return new umat3x4(v1.m00 % s, v1.m01 % s, v1.m02 % s, v1.m03 % s, v1.m10 % s, v1.m11 % s, v1.m12 % s, v1.m13 % s, v1.m20 % s, v1.m21 % s, v1.m22 % s, v1.m23 % s);
        }
        
        public static umat3x4 operator&(umat3x4 v1, uint s) {
            return new umat3x4(v1.m00 & s, v1.m01 & s, v1.m02 & s, v1.m03 & s, v1.m10 & s, v1.m11 & s, v1.m12 & s, v1.m13 & s, v1.m20 & s, v1.m21 & s, v1.m22 & s, v1.m23 & s);
        }
        
        public static umat3x4 operator|(umat3x4 v1, uint s) {
            return new umat3x4(v1.m00 | s, v1.m01 | s, v1.m02 | s, v1.m03 | s, v1.m10 | s, v1.m11 | s, v1.m12 | s, v1.m13 | s, v1.m20 | s, v1.m21 | s, v1.m22 | s, v1.m23 | s);
        }
        
        public static umat3x4 operator^(umat3x4 v1, uint s) {
            return new umat3x4(v1.m00 ^ s, v1.m01 ^ s, v1.m02 ^ s, v1.m03 ^ s, v1.m10 ^ s, v1.m11 ^ s, v1.m12 ^ s, v1.m13 ^ s, v1.m20 ^ s, v1.m21 ^ s, v1.m22 ^ s, v1.m23 ^ s);
        }
        
        public static umat3x4 operator+(umat3x4 v1, umat3x4 v2) {
            return new umat3x4(v1.m00 + v2.m00, v1.m01 + v2.m01, v1.m02 + v2.m02, v1.m03 + v2.m03, v1.m10 + v2.m10, v1.m11 + v2.m11, v1.m12 + v2.m12, v1.m13 + v2.m13, v1.m20 + v2.m20, v1.m21 + v2.m21, v1.m22 + v2.m22, v1.m23 + v2.m23);
        }
        
        public static umat3x4 operator-(umat3x4 v1, umat3x4 v2) {
            return new umat3x4(v1.m00 - v2.m00, v1.m01 - v2.m01, v1.m02 - v2.m02, v1.m03 - v2.m03, v1.m10 - v2.m10, v1.m11 - v2.m11, v1.m12 - v2.m12, v1.m13 - v2.m13, v1.m20 - v2.m20, v1.m21 - v2.m21, v1.m22 - v2.m22, v1.m23 - v2.m23);
        }
        
        public static umat3x4 operator/(umat3x4 v1, umat3x4 v2) {
            return new umat3x4(v1.m00 / v2.m00, v1.m01 / v2.m01, v1.m02 / v2.m02, v1.m03 / v2.m03, v1.m10 / v2.m10, v1.m11 / v2.m11, v1.m12 / v2.m12, v1.m13 / v2.m13, v1.m20 / v2.m20, v1.m21 / v2.m21, v1.m22 / v2.m22, v1.m23 / v2.m23);
        }
        
        public static umat3x4 operator%(umat3x4 v1, umat3x4 v2) {
            return new umat3x4(v1.m00 % v2.m00, v1.m01 % v2.m01, v1.m02 % v2.m02, v1.m03 % v2.m03, v1.m10 % v2.m10, v1.m11 % v2.m11, v1.m12 % v2.m12, v1.m13 % v2.m13, v1.m20 % v2.m20, v1.m21 % v2.m21, v1.m22 % v2.m22, v1.m23 % v2.m23);
        }
        
        public static umat3x4 operator&(umat3x4 v1, umat3x4 v2) {
            return new umat3x4(v1.m00 & v2.m00, v1.m01 & v2.m01, v1.m02 & v2.m02, v1.m03 & v2.m03, v1.m10 & v2.m10, v1.m11 & v2.m11, v1.m12 & v2.m12, v1.m13 & v2.m13, v1.m20 & v2.m20, v1.m21 & v2.m21, v1.m22 & v2.m22, v1.m23 & v2.m23);
        }
        
        public static umat3x4 operator|(umat3x4 v1, umat3x4 v2) {
            return new umat3x4(v1.m00 | v2.m00, v1.m01 | v2.m01, v1.m02 | v2.m02, v1.m03 | v2.m03, v1.m10 | v2.m10, v1.m11 | v2.m11, v1.m12 | v2.m12, v1.m13 | v2.m13, v1.m20 | v2.m20, v1.m21 | v2.m21, v1.m22 | v2.m22, v1.m23 | v2.m23);
        }
        
        public static umat3x4 operator^(umat3x4 v1, umat3x4 v2) {
            return new umat3x4(v1.m00 ^ v2.m00, v1.m01 ^ v2.m01, v1.m02 ^ v2.m02, v1.m03 ^ v2.m03, v1.m10 ^ v2.m10, v1.m11 ^ v2.m11, v1.m12 ^ v2.m12, v1.m13 ^ v2.m13, v1.m20 ^ v2.m20, v1.m21 ^ v2.m21, v1.m22 ^ v2.m22, v1.m23 ^ v2.m23);
        }
        
        public static bmat3x4 operator==(umat3x4 v1, umat3x4 v2) {
            return new bmat3x4(v1.m00 == v2.m00, v1.m01 == v2.m01, v1.m02 == v2.m02, v1.m03 == v2.m03, v1.m10 == v2.m10, v1.m11 == v2.m11, v1.m12 == v2.m12, v1.m13 == v2.m13, v1.m20 == v2.m20, v1.m21 == v2.m21, v1.m22 == v2.m22, v1.m23 == v2.m23);
        }
        
        public static bmat3x4 operator>(umat3x4 v1, umat3x4 v2) {
            return new bmat3x4(v1.m00 > v2.m00, v1.m01 > v2.m01, v1.m02 > v2.m02, v1.m03 > v2.m03, v1.m10 > v2.m10, v1.m11 > v2.m11, v1.m12 > v2.m12, v1.m13 > v2.m13, v1.m20 > v2.m20, v1.m21 > v2.m21, v1.m22 > v2.m22, v1.m23 > v2.m23);
        }
        
        public static bmat3x4 operator>=(umat3x4 v1, umat3x4 v2) {
            return new bmat3x4(v1.m00 >= v2.m00, v1.m01 >= v2.m01, v1.m02 >= v2.m02, v1.m03 >= v2.m03, v1.m10 >= v2.m10, v1.m11 >= v2.m11, v1.m12 >= v2.m12, v1.m13 >= v2.m13, v1.m20 >= v2.m20, v1.m21 >= v2.m21, v1.m22 >= v2.m22, v1.m23 >= v2.m23);
        }
        
        public static bmat3x4 operator<(umat3x4 v1, umat3x4 v2) {
            return new bmat3x4(v1.m00 < v2.m00, v1.m01 < v2.m01, v1.m02 < v2.m02, v1.m03 < v2.m03, v1.m10 < v2.m10, v1.m11 < v2.m11, v1.m12 < v2.m12, v1.m13 < v2.m13, v1.m20 < v2.m20, v1.m21 < v2.m21, v1.m22 < v2.m22, v1.m23 < v2.m23);
        }
        
        public static bmat3x4 operator<=(umat3x4 v1, umat3x4 v2) {
            return new bmat3x4(v1.m00 <= v2.m00, v1.m01 <= v2.m01, v1.m02 <= v2.m02, v1.m03 <= v2.m03, v1.m10 <= v2.m10, v1.m11 <= v2.m11, v1.m12 <= v2.m12, v1.m13 <= v2.m13, v1.m20 <= v2.m20, v1.m21 <= v2.m21, v1.m22 <= v2.m22, v1.m23 <= v2.m23);
        }
        
        public static bmat3x4 operator!=(umat3x4 v1, umat3x4 v2) {
            return new bmat3x4(v1.m00 != v2.m00, v1.m01 != v2.m01, v1.m02 != v2.m02, v1.m03 != v2.m03, v1.m10 != v2.m10, v1.m11 != v2.m11, v1.m12 != v2.m12, v1.m13 != v2.m13, v1.m20 != v2.m20, v1.m21 != v2.m21, v1.m22 != v2.m22, v1.m23 != v2.m23);
        }
        
        public static uvec4 operator*(uvec3 v0, umat3x4 m1) {
            return umath.InnerProduct1(ref v0, ref m1);
        }
        
        public static uvec3 operator*(umat3x4 m0, uvec4 v1) {
            return umath.InnerProduct1(ref m0, ref v1);
        }
        
        public static umat3x2 operator*(umat3x4 m0, umat4x2 m1) {
            return umath.InnerProduct1(ref m0, ref m1);
        }
        
        public static umat3 operator*(umat3x4 m0, umat4x3 m1) {
            return umath.InnerProduct1(ref m0, ref m1);
        }
        
        public static umat3x4 operator*(umat3x4 m0, umat4 m1) {
            return umath.InnerProduct1(ref m0, ref m1);
        }
        
        public override bool Equals(object obj) {
            return (obj is umat3x4) ? bmath.All(this == (umat3x4)obj) : false;
        }
        
        public override int GetHashCode() {
            return this.m00.GetHashCode() ^ this.m01.GetHashCode() ^ this.m02.GetHashCode() ^ this.m03.GetHashCode() ^ this.m10.GetHashCode() ^ this.m11.GetHashCode() ^ this.m12.GetHashCode() ^ this.m13.GetHashCode() ^ this.m20.GetHashCode() ^ this.m21.GetHashCode() ^ this.m22.GetHashCode() ^ this.m23.GetHashCode();
        }
        
        public override string ToString() {
            return "umat3x4(" + this.m00 + "," + this.m01 + "," + this.m02 + "," + this.m03 + "," + this.m10 + "," + this.m11 + "," + this.m12 + "," + this.m13 + "," + this.m20 + "," + this.m21 + "," + this.m22 + "," + this.m23 + ")";
        }
    }
    
    [DataContract()]
    [StructLayout(LayoutKind.Sequential)]
    public struct umat4x2 {
        
        [DataMember()]
        internal uint m00;
        
        [DataMember()]
        internal uint m01;
        
        [DataMember()]
        internal uint m10;
        
        [DataMember()]
        internal uint m11;
        
        [DataMember()]
        internal uint m20;
        
        [DataMember()]
        internal uint m21;
        
        [DataMember()]
        internal uint m30;
        
        [DataMember()]
        internal uint m31;
        
        public umat4x2(uint s) {
            this.m00 = s;
            this.m01 = 0u;
            this.m10 = 0u;
            this.m11 = s;
            this.m20 = 0u;
            this.m21 = 0u;
            this.m30 = 0u;
            this.m31 = 0u;
        }
        
        public umat4x2(umat4x2 m00) {
            this.m00 = m00.m00;
            this.m01 = m00.m01;
            this.m10 = m00.m10;
            this.m11 = m00.m11;
            this.m20 = m00.m20;
            this.m21 = m00.m21;
            this.m30 = m00.m30;
            this.m31 = m00.m31;
        }
        
        public umat4x2(umat2 m00, umat2 m10) {
            this.m00 = m00.m00;
            this.m01 = m00.m01;
            this.m10 = m00.m10;
            this.m11 = m00.m11;
            this.m20 = m10.m00;
            this.m21 = m10.m01;
            this.m30 = m10.m10;
            this.m31 = m10.m11;
        }
        
        public umat4x2(umat3x2 m00, uvec2 m10) {
            this.m00 = m00.m00;
            this.m01 = m00.m01;
            this.m10 = m00.m10;
            this.m11 = m00.m11;
            this.m20 = m00.m20;
            this.m21 = m00.m21;
            this.m30 = m10.v0;
            this.m31 = m10.v1;
        }
        
        public umat4x2(uvec2 m00, umat3x2 m10) {
            this.m00 = m00.v0;
            this.m01 = m00.v1;
            this.m10 = m10.m00;
            this.m11 = m10.m01;
            this.m20 = m10.m10;
            this.m21 = m10.m11;
            this.m30 = m10.m20;
            this.m31 = m10.m21;
        }
        
        public umat4x2(umat2 m00, uvec2 m10, uvec2 m20) {
            this.m00 = m00.m00;
            this.m01 = m00.m01;
            this.m10 = m00.m10;
            this.m11 = m00.m11;
            this.m20 = m10.v0;
            this.m21 = m10.v1;
            this.m30 = m20.v0;
            this.m31 = m20.v1;
        }
        
        public umat4x2(uvec2 m00, umat2 m10, uvec2 m20) {
            this.m00 = m00.v0;
            this.m01 = m00.v1;
            this.m10 = m10.m00;
            this.m11 = m10.m01;
            this.m20 = m10.m10;
            this.m21 = m10.m11;
            this.m30 = m20.v0;
            this.m31 = m20.v1;
        }
        
        public umat4x2(uvec2 m00, uvec2 m10, umat2 m20) {
            this.m00 = m00.v0;
            this.m01 = m00.v1;
            this.m10 = m10.v0;
            this.m11 = m10.v1;
            this.m20 = m20.m00;
            this.m21 = m20.m01;
            this.m30 = m20.m10;
            this.m31 = m20.m11;
        }
        
        public umat4x2(uvec2 m00, uvec2 m10, uvec2 m20, uvec2 m30) {
            this.m00 = m00.v0;
            this.m01 = m00.v1;
            this.m10 = m10.v0;
            this.m11 = m10.v1;
            this.m20 = m20.v0;
            this.m21 = m20.v1;
            this.m30 = m30.v0;
            this.m31 = m30.v1;
        }
        
        public umat4x2(uint m00, uint m01, uint m10, uint m11, uint m20, uint m21, uint m30, uint m31) {
            this.m00 = m00;
            this.m01 = m01;
            this.m10 = m10;
            this.m11 = m11;
            this.m20 = m20;
            this.m21 = m21;
            this.m30 = m30;
            this.m31 = m31;
        }
        
        public umat4x2(umat2 s) {
            this.m00 = s.m00;
            this.m01 = s.m01;
            this.m10 = s.m10;
            this.m11 = s.m11;
            this.m20 = 0u;
            this.m21 = 0u;
            this.m30 = 0u;
            this.m31 = 0u;
        }
        
        public umat4x2(umat2x3 s) {
            this.m00 = s.m00;
            this.m01 = s.m01;
            this.m10 = s.m10;
            this.m11 = s.m11;
            this.m20 = 0u;
            this.m21 = 0u;
            this.m30 = 0u;
            this.m31 = 0u;
        }
        
        public umat4x2(umat2x4 s) {
            this.m00 = s.m00;
            this.m01 = s.m01;
            this.m10 = s.m10;
            this.m11 = s.m11;
            this.m20 = 0u;
            this.m21 = 0u;
            this.m30 = 0u;
            this.m31 = 0u;
        }
        
        public umat4x2(umat3x2 s) {
            this.m00 = s.m00;
            this.m01 = s.m01;
            this.m10 = s.m10;
            this.m11 = s.m11;
            this.m20 = s.m20;
            this.m21 = s.m21;
            this.m30 = 0u;
            this.m31 = 0u;
        }
        
        public umat4x2(umat3 s) {
            this.m00 = s.m00;
            this.m01 = s.m01;
            this.m10 = s.m10;
            this.m11 = s.m11;
            this.m20 = s.m20;
            this.m21 = s.m21;
            this.m30 = 0u;
            this.m31 = 0u;
        }
        
        public umat4x2(umat3x4 s) {
            this.m00 = s.m00;
            this.m01 = s.m01;
            this.m10 = s.m10;
            this.m11 = s.m11;
            this.m20 = s.m20;
            this.m21 = s.m21;
            this.m30 = 0u;
            this.m31 = 0u;
        }
        
        public umat4x2(umat4x3 s) {
            this.m00 = s.m00;
            this.m01 = s.m01;
            this.m10 = s.m10;
            this.m11 = s.m11;
            this.m20 = s.m20;
            this.m21 = s.m21;
            this.m30 = s.m30;
            this.m31 = s.m31;
        }
        
        public umat4x2(umat4 s) {
            this.m00 = s.m00;
            this.m01 = s.m01;
            this.m10 = s.m10;
            this.m11 = s.m11;
            this.m20 = s.m20;
            this.m21 = s.m21;
            this.m30 = s.m30;
            this.m31 = s.m31;
        }
        
        public uint this[int i0, int i1] {
            get {
                if (((i0 < 0) 
                            || (3 < i0))) {
                    throw new System.IndexOutOfRangeException("i0");
                }
                if (((i1 < 0) 
                            || (1 < i1))) {
                    throw new System.IndexOutOfRangeException("i1");
                }
                
                unsafe
                {
                    fixed (UInt32* v_ptr = &this.m00)
                    {
                        return *(v_ptr + (i1 + i0 * 2));
                    }
                };
            }
            set {
                if (((i0 < 0) 
                            || (3 < i0))) {
                    throw new System.IndexOutOfRangeException("i0");
                }
                if (((i1 < 0) 
                            || (1 < i1))) {
                    throw new System.IndexOutOfRangeException("i1");
                }
                
                unsafe
                {
                    fixed (UInt32* v_ptr = &this.m00)
                    {
                        *(v_ptr  + (i1 + i0 * 2)) = value;
                    }
                };
            }
        }
        
        public uvec2 this[int i0] {
            get {
                return new uvec2(this[i0, 0], this[i0, 1]);
            }
            set {
                this[i0, 0] = value[0];
                this[i0, 1] = value[1];
            }
        }
        
        public static umat4x2 operator+(umat4x2 v) {
            return new umat4x2(+v.m00, +v.m01, +v.m10, +v.m11, +v.m20, +v.m21, +v.m30, +v.m31);
        }
        
        public static umat4x2 operator~(umat4x2 v) {
            return new umat4x2(~v.m00, ~v.m01, ~v.m10, ~v.m11, ~v.m20, ~v.m21, ~v.m30, ~v.m31);
        }
        
        public static umat4x2 operator++(umat4x2 v) {
            return new umat4x2(++v.m00, ++v.m01, ++v.m10, ++v.m11, ++v.m20, ++v.m21, ++v.m30, ++v.m31);
        }
        
        public static umat4x2 operator--(umat4x2 v) {
            return new umat4x2(--v.m00, --v.m01, --v.m10, --v.m11, --v.m20, --v.m21, --v.m30, --v.m31);
        }
        
        public static umat4x2 operator+(uint s, umat4x2 v2) {
            return new umat4x2(s + v2.m00, s + v2.m01, s + v2.m10, s + v2.m11, s + v2.m20, s + v2.m21, s + v2.m30, s + v2.m31);
        }
        
        public static umat4x2 operator-(uint s, umat4x2 v2) {
            return new umat4x2(s - v2.m00, s - v2.m01, s - v2.m10, s - v2.m11, s - v2.m20, s - v2.m21, s - v2.m30, s - v2.m31);
        }
        
        public static umat4x2 operator*(uint s, umat4x2 v2) {
            return new umat4x2(s * v2.m00, s * v2.m01, s * v2.m10, s * v2.m11, s * v2.m20, s * v2.m21, s * v2.m30, s * v2.m31);
        }
        
        public static umat4x2 operator/(uint s, umat4x2 v2) {
            return new umat4x2(s / v2.m00, s / v2.m01, s / v2.m10, s / v2.m11, s / v2.m20, s / v2.m21, s / v2.m30, s / v2.m31);
        }
        
        public static umat4x2 operator%(uint s, umat4x2 v2) {
            return new umat4x2(s % v2.m00, s % v2.m01, s % v2.m10, s % v2.m11, s % v2.m20, s % v2.m21, s % v2.m30, s % v2.m31);
        }
        
        public static umat4x2 operator&(uint s, umat4x2 v2) {
            return new umat4x2(s & v2.m00, s & v2.m01, s & v2.m10, s & v2.m11, s & v2.m20, s & v2.m21, s & v2.m30, s & v2.m31);
        }
        
        public static umat4x2 operator|(uint s, umat4x2 v2) {
            return new umat4x2(s | v2.m00, s | v2.m01, s | v2.m10, s | v2.m11, s | v2.m20, s | v2.m21, s | v2.m30, s | v2.m31);
        }
        
        public static umat4x2 operator^(uint s, umat4x2 v2) {
            return new umat4x2(s ^ v2.m00, s ^ v2.m01, s ^ v2.m10, s ^ v2.m11, s ^ v2.m20, s ^ v2.m21, s ^ v2.m30, s ^ v2.m31);
        }
        
        public static umat4x2 operator+(umat4x2 v1, uint s) {
            return new umat4x2(v1.m00 + s, v1.m01 + s, v1.m10 + s, v1.m11 + s, v1.m20 + s, v1.m21 + s, v1.m30 + s, v1.m31 + s);
        }
        
        public static umat4x2 operator-(umat4x2 v1, uint s) {
            return new umat4x2(v1.m00 - s, v1.m01 - s, v1.m10 - s, v1.m11 - s, v1.m20 - s, v1.m21 - s, v1.m30 - s, v1.m31 - s);
        }
        
        public static umat4x2 operator*(umat4x2 v1, uint s) {
            return new umat4x2(v1.m00 * s, v1.m01 * s, v1.m10 * s, v1.m11 * s, v1.m20 * s, v1.m21 * s, v1.m30 * s, v1.m31 * s);
        }
        
        public static umat4x2 operator/(umat4x2 v1, uint s) {
            return new umat4x2(v1.m00 / s, v1.m01 / s, v1.m10 / s, v1.m11 / s, v1.m20 / s, v1.m21 / s, v1.m30 / s, v1.m31 / s);
        }
        
        public static umat4x2 operator%(umat4x2 v1, uint s) {
            return new umat4x2(v1.m00 % s, v1.m01 % s, v1.m10 % s, v1.m11 % s, v1.m20 % s, v1.m21 % s, v1.m30 % s, v1.m31 % s);
        }
        
        public static umat4x2 operator&(umat4x2 v1, uint s) {
            return new umat4x2(v1.m00 & s, v1.m01 & s, v1.m10 & s, v1.m11 & s, v1.m20 & s, v1.m21 & s, v1.m30 & s, v1.m31 & s);
        }
        
        public static umat4x2 operator|(umat4x2 v1, uint s) {
            return new umat4x2(v1.m00 | s, v1.m01 | s, v1.m10 | s, v1.m11 | s, v1.m20 | s, v1.m21 | s, v1.m30 | s, v1.m31 | s);
        }
        
        public static umat4x2 operator^(umat4x2 v1, uint s) {
            return new umat4x2(v1.m00 ^ s, v1.m01 ^ s, v1.m10 ^ s, v1.m11 ^ s, v1.m20 ^ s, v1.m21 ^ s, v1.m30 ^ s, v1.m31 ^ s);
        }
        
        public static umat4x2 operator+(umat4x2 v1, umat4x2 v2) {
            return new umat4x2(v1.m00 + v2.m00, v1.m01 + v2.m01, v1.m10 + v2.m10, v1.m11 + v2.m11, v1.m20 + v2.m20, v1.m21 + v2.m21, v1.m30 + v2.m30, v1.m31 + v2.m31);
        }
        
        public static umat4x2 operator-(umat4x2 v1, umat4x2 v2) {
            return new umat4x2(v1.m00 - v2.m00, v1.m01 - v2.m01, v1.m10 - v2.m10, v1.m11 - v2.m11, v1.m20 - v2.m20, v1.m21 - v2.m21, v1.m30 - v2.m30, v1.m31 - v2.m31);
        }
        
        public static umat4x2 operator/(umat4x2 v1, umat4x2 v2) {
            return new umat4x2(v1.m00 / v2.m00, v1.m01 / v2.m01, v1.m10 / v2.m10, v1.m11 / v2.m11, v1.m20 / v2.m20, v1.m21 / v2.m21, v1.m30 / v2.m30, v1.m31 / v2.m31);
        }
        
        public static umat4x2 operator%(umat4x2 v1, umat4x2 v2) {
            return new umat4x2(v1.m00 % v2.m00, v1.m01 % v2.m01, v1.m10 % v2.m10, v1.m11 % v2.m11, v1.m20 % v2.m20, v1.m21 % v2.m21, v1.m30 % v2.m30, v1.m31 % v2.m31);
        }
        
        public static umat4x2 operator&(umat4x2 v1, umat4x2 v2) {
            return new umat4x2(v1.m00 & v2.m00, v1.m01 & v2.m01, v1.m10 & v2.m10, v1.m11 & v2.m11, v1.m20 & v2.m20, v1.m21 & v2.m21, v1.m30 & v2.m30, v1.m31 & v2.m31);
        }
        
        public static umat4x2 operator|(umat4x2 v1, umat4x2 v2) {
            return new umat4x2(v1.m00 | v2.m00, v1.m01 | v2.m01, v1.m10 | v2.m10, v1.m11 | v2.m11, v1.m20 | v2.m20, v1.m21 | v2.m21, v1.m30 | v2.m30, v1.m31 | v2.m31);
        }
        
        public static umat4x2 operator^(umat4x2 v1, umat4x2 v2) {
            return new umat4x2(v1.m00 ^ v2.m00, v1.m01 ^ v2.m01, v1.m10 ^ v2.m10, v1.m11 ^ v2.m11, v1.m20 ^ v2.m20, v1.m21 ^ v2.m21, v1.m30 ^ v2.m30, v1.m31 ^ v2.m31);
        }
        
        public static bmat4x2 operator==(umat4x2 v1, umat4x2 v2) {
            return new bmat4x2(v1.m00 == v2.m00, v1.m01 == v2.m01, v1.m10 == v2.m10, v1.m11 == v2.m11, v1.m20 == v2.m20, v1.m21 == v2.m21, v1.m30 == v2.m30, v1.m31 == v2.m31);
        }
        
        public static bmat4x2 operator>(umat4x2 v1, umat4x2 v2) {
            return new bmat4x2(v1.m00 > v2.m00, v1.m01 > v2.m01, v1.m10 > v2.m10, v1.m11 > v2.m11, v1.m20 > v2.m20, v1.m21 > v2.m21, v1.m30 > v2.m30, v1.m31 > v2.m31);
        }
        
        public static bmat4x2 operator>=(umat4x2 v1, umat4x2 v2) {
            return new bmat4x2(v1.m00 >= v2.m00, v1.m01 >= v2.m01, v1.m10 >= v2.m10, v1.m11 >= v2.m11, v1.m20 >= v2.m20, v1.m21 >= v2.m21, v1.m30 >= v2.m30, v1.m31 >= v2.m31);
        }
        
        public static bmat4x2 operator<(umat4x2 v1, umat4x2 v2) {
            return new bmat4x2(v1.m00 < v2.m00, v1.m01 < v2.m01, v1.m10 < v2.m10, v1.m11 < v2.m11, v1.m20 < v2.m20, v1.m21 < v2.m21, v1.m30 < v2.m30, v1.m31 < v2.m31);
        }
        
        public static bmat4x2 operator<=(umat4x2 v1, umat4x2 v2) {
            return new bmat4x2(v1.m00 <= v2.m00, v1.m01 <= v2.m01, v1.m10 <= v2.m10, v1.m11 <= v2.m11, v1.m20 <= v2.m20, v1.m21 <= v2.m21, v1.m30 <= v2.m30, v1.m31 <= v2.m31);
        }
        
        public static bmat4x2 operator!=(umat4x2 v1, umat4x2 v2) {
            return new bmat4x2(v1.m00 != v2.m00, v1.m01 != v2.m01, v1.m10 != v2.m10, v1.m11 != v2.m11, v1.m20 != v2.m20, v1.m21 != v2.m21, v1.m30 != v2.m30, v1.m31 != v2.m31);
        }
        
        public static uvec2 operator*(uvec4 v0, umat4x2 m1) {
            return umath.InnerProduct1(ref v0, ref m1);
        }
        
        public static uvec4 operator*(umat4x2 m0, uvec2 v1) {
            return umath.InnerProduct1(ref m0, ref v1);
        }
        
        public static umat4x2 operator*(umat4x2 m0, umat2 m1) {
            return umath.InnerProduct1(ref m0, ref m1);
        }
        
        public static umat4x3 operator*(umat4x2 m0, umat2x3 m1) {
            return umath.InnerProduct1(ref m0, ref m1);
        }
        
        public static umat4 operator*(umat4x2 m0, umat2x4 m1) {
            return umath.InnerProduct1(ref m0, ref m1);
        }
        
        public override bool Equals(object obj) {
            return (obj is umat4x2) ? bmath.All(this == (umat4x2)obj) : false;
        }
        
        public override int GetHashCode() {
            return this.m00.GetHashCode() ^ this.m01.GetHashCode() ^ this.m10.GetHashCode() ^ this.m11.GetHashCode() ^ this.m20.GetHashCode() ^ this.m21.GetHashCode() ^ this.m30.GetHashCode() ^ this.m31.GetHashCode();
        }
        
        public override string ToString() {
            return "umat4x2(" + this.m00 + "," + this.m01 + "," + this.m10 + "," + this.m11 + "," + this.m20 + "," + this.m21 + "," + this.m30 + "," + this.m31 + ")";
        }
    }
    
    [DataContract()]
    [StructLayout(LayoutKind.Sequential)]
    public struct umat4x3 {
        
        [DataMember()]
        internal uint m00;
        
        [DataMember()]
        internal uint m01;
        
        [DataMember()]
        internal uint m02;
        
        [DataMember()]
        internal uint m10;
        
        [DataMember()]
        internal uint m11;
        
        [DataMember()]
        internal uint m12;
        
        [DataMember()]
        internal uint m20;
        
        [DataMember()]
        internal uint m21;
        
        [DataMember()]
        internal uint m22;
        
        [DataMember()]
        internal uint m30;
        
        [DataMember()]
        internal uint m31;
        
        [DataMember()]
        internal uint m32;
        
        public umat4x3(uint s) {
            this.m00 = s;
            this.m01 = 0u;
            this.m02 = 0u;
            this.m10 = 0u;
            this.m11 = s;
            this.m12 = 0u;
            this.m20 = 0u;
            this.m21 = 0u;
            this.m22 = s;
            this.m30 = 0u;
            this.m31 = 0u;
            this.m32 = 0u;
        }
        
        public umat4x3(umat4x3 m00) {
            this.m00 = m00.m00;
            this.m01 = m00.m01;
            this.m02 = m00.m02;
            this.m10 = m00.m10;
            this.m11 = m00.m11;
            this.m12 = m00.m12;
            this.m20 = m00.m20;
            this.m21 = m00.m21;
            this.m22 = m00.m22;
            this.m30 = m00.m30;
            this.m31 = m00.m31;
            this.m32 = m00.m32;
        }
        
        public umat4x3(umat2x3 m00, umat2x3 m10) {
            this.m00 = m00.m00;
            this.m01 = m00.m01;
            this.m02 = m00.m02;
            this.m10 = m00.m10;
            this.m11 = m00.m11;
            this.m12 = m00.m12;
            this.m20 = m10.m00;
            this.m21 = m10.m01;
            this.m22 = m10.m02;
            this.m30 = m10.m10;
            this.m31 = m10.m11;
            this.m32 = m10.m12;
        }
        
        public umat4x3(umat3 m00, uvec3 m10) {
            this.m00 = m00.m00;
            this.m01 = m00.m01;
            this.m02 = m00.m02;
            this.m10 = m00.m10;
            this.m11 = m00.m11;
            this.m12 = m00.m12;
            this.m20 = m00.m20;
            this.m21 = m00.m21;
            this.m22 = m00.m22;
            this.m30 = m10.v0;
            this.m31 = m10.v1;
            this.m32 = m10.v2;
        }
        
        public umat4x3(uvec3 m00, umat3 m10) {
            this.m00 = m00.v0;
            this.m01 = m00.v1;
            this.m02 = m00.v2;
            this.m10 = m10.m00;
            this.m11 = m10.m01;
            this.m12 = m10.m02;
            this.m20 = m10.m10;
            this.m21 = m10.m11;
            this.m22 = m10.m12;
            this.m30 = m10.m20;
            this.m31 = m10.m21;
            this.m32 = m10.m22;
        }
        
        public umat4x3(umat2x3 m00, uvec3 m10, uvec3 m20) {
            this.m00 = m00.m00;
            this.m01 = m00.m01;
            this.m02 = m00.m02;
            this.m10 = m00.m10;
            this.m11 = m00.m11;
            this.m12 = m00.m12;
            this.m20 = m10.v0;
            this.m21 = m10.v1;
            this.m22 = m10.v2;
            this.m30 = m20.v0;
            this.m31 = m20.v1;
            this.m32 = m20.v2;
        }
        
        public umat4x3(uvec3 m00, umat2x3 m10, uvec3 m20) {
            this.m00 = m00.v0;
            this.m01 = m00.v1;
            this.m02 = m00.v2;
            this.m10 = m10.m00;
            this.m11 = m10.m01;
            this.m12 = m10.m02;
            this.m20 = m10.m10;
            this.m21 = m10.m11;
            this.m22 = m10.m12;
            this.m30 = m20.v0;
            this.m31 = m20.v1;
            this.m32 = m20.v2;
        }
        
        public umat4x3(uvec3 m00, uvec3 m10, umat2x3 m20) {
            this.m00 = m00.v0;
            this.m01 = m00.v1;
            this.m02 = m00.v2;
            this.m10 = m10.v0;
            this.m11 = m10.v1;
            this.m12 = m10.v2;
            this.m20 = m20.m00;
            this.m21 = m20.m01;
            this.m22 = m20.m02;
            this.m30 = m20.m10;
            this.m31 = m20.m11;
            this.m32 = m20.m12;
        }
        
        public umat4x3(uvec3 m00, uvec3 m10, uvec3 m20, uvec3 m30) {
            this.m00 = m00.v0;
            this.m01 = m00.v1;
            this.m02 = m00.v2;
            this.m10 = m10.v0;
            this.m11 = m10.v1;
            this.m12 = m10.v2;
            this.m20 = m20.v0;
            this.m21 = m20.v1;
            this.m22 = m20.v2;
            this.m30 = m30.v0;
            this.m31 = m30.v1;
            this.m32 = m30.v2;
        }
        
        public umat4x3(uvec2 m00, uint m01, uvec2 m10, uint m11, uvec2 m20, uint m21, uvec2 m30, uint m31) {
            this.m00 = m00.v0;
            this.m01 = m00.v1;
            this.m02 = m01;
            this.m10 = m10.v0;
            this.m11 = m10.v1;
            this.m12 = m11;
            this.m20 = m20.v0;
            this.m21 = m20.v1;
            this.m22 = m21;
            this.m30 = m30.v0;
            this.m31 = m30.v1;
            this.m32 = m31;
        }
        
        public umat4x3(uint m00, uvec2 m01, uint m10, uvec2 m11, uint m20, uvec2 m21, uint m30, uvec2 m31) {
            this.m00 = m00;
            this.m01 = m01.v0;
            this.m02 = m01.v1;
            this.m10 = m10;
            this.m11 = m11.v0;
            this.m12 = m11.v1;
            this.m20 = m20;
            this.m21 = m21.v0;
            this.m22 = m21.v1;
            this.m30 = m30;
            this.m31 = m31.v0;
            this.m32 = m31.v1;
        }
        
        public umat4x3(uint m00, uint m01, uint m02, uint m10, uint m11, uint m12, uint m20, uint m21, uint m22, uint m30, uint m31, uint m32) {
            this.m00 = m00;
            this.m01 = m01;
            this.m02 = m02;
            this.m10 = m10;
            this.m11 = m11;
            this.m12 = m12;
            this.m20 = m20;
            this.m21 = m21;
            this.m22 = m22;
            this.m30 = m30;
            this.m31 = m31;
            this.m32 = m32;
        }
        
        public umat4x3(umat2 s) {
            this.m00 = s.m00;
            this.m01 = s.m01;
            this.m02 = 0u;
            this.m10 = s.m10;
            this.m11 = s.m11;
            this.m12 = 0u;
            this.m20 = 0u;
            this.m21 = 0u;
            this.m22 = 0u;
            this.m30 = 0u;
            this.m31 = 0u;
            this.m32 = 0u;
        }
        
        public umat4x3(umat2x3 s) {
            this.m00 = s.m00;
            this.m01 = s.m01;
            this.m02 = s.m02;
            this.m10 = s.m10;
            this.m11 = s.m11;
            this.m12 = s.m12;
            this.m20 = 0u;
            this.m21 = 0u;
            this.m22 = 0u;
            this.m30 = 0u;
            this.m31 = 0u;
            this.m32 = 0u;
        }
        
        public umat4x3(umat2x4 s) {
            this.m00 = s.m00;
            this.m01 = s.m01;
            this.m02 = s.m02;
            this.m10 = s.m10;
            this.m11 = s.m11;
            this.m12 = s.m12;
            this.m20 = 0u;
            this.m21 = 0u;
            this.m22 = 0u;
            this.m30 = 0u;
            this.m31 = 0u;
            this.m32 = 0u;
        }
        
        public umat4x3(umat3x2 s) {
            this.m00 = s.m00;
            this.m01 = s.m01;
            this.m02 = 0u;
            this.m10 = s.m10;
            this.m11 = s.m11;
            this.m12 = 0u;
            this.m20 = s.m20;
            this.m21 = s.m21;
            this.m22 = 0u;
            this.m30 = 0u;
            this.m31 = 0u;
            this.m32 = 0u;
        }
        
        public umat4x3(umat3 s) {
            this.m00 = s.m00;
            this.m01 = s.m01;
            this.m02 = s.m02;
            this.m10 = s.m10;
            this.m11 = s.m11;
            this.m12 = s.m12;
            this.m20 = s.m20;
            this.m21 = s.m21;
            this.m22 = s.m22;
            this.m30 = 0u;
            this.m31 = 0u;
            this.m32 = 0u;
        }
        
        public umat4x3(umat3x4 s) {
            this.m00 = s.m00;
            this.m01 = s.m01;
            this.m02 = s.m02;
            this.m10 = s.m10;
            this.m11 = s.m11;
            this.m12 = s.m12;
            this.m20 = s.m20;
            this.m21 = s.m21;
            this.m22 = s.m22;
            this.m30 = 0u;
            this.m31 = 0u;
            this.m32 = 0u;
        }
        
        public umat4x3(umat4x2 s) {
            this.m00 = s.m00;
            this.m01 = s.m01;
            this.m02 = 0u;
            this.m10 = s.m10;
            this.m11 = s.m11;
            this.m12 = 0u;
            this.m20 = s.m20;
            this.m21 = s.m21;
            this.m22 = 0u;
            this.m30 = s.m30;
            this.m31 = s.m31;
            this.m32 = 0u;
        }
        
        public umat4x3(umat4 s) {
            this.m00 = s.m00;
            this.m01 = s.m01;
            this.m02 = s.m02;
            this.m10 = s.m10;
            this.m11 = s.m11;
            this.m12 = s.m12;
            this.m20 = s.m20;
            this.m21 = s.m21;
            this.m22 = s.m22;
            this.m30 = s.m30;
            this.m31 = s.m31;
            this.m32 = s.m32;
        }
        
        public uint this[int i0, int i1] {
            get {
                if (((i0 < 0) 
                            || (3 < i0))) {
                    throw new System.IndexOutOfRangeException("i0");
                }
                if (((i1 < 0) 
                            || (2 < i1))) {
                    throw new System.IndexOutOfRangeException("i1");
                }
                
                unsafe
                {
                    fixed (UInt32* v_ptr = &this.m00)
                    {
                        return *(v_ptr + (i1 + i0 * 3));
                    }
                };
            }
            set {
                if (((i0 < 0) 
                            || (3 < i0))) {
                    throw new System.IndexOutOfRangeException("i0");
                }
                if (((i1 < 0) 
                            || (2 < i1))) {
                    throw new System.IndexOutOfRangeException("i1");
                }
                
                unsafe
                {
                    fixed (UInt32* v_ptr = &this.m00)
                    {
                        *(v_ptr  + (i1 + i0 * 3)) = value;
                    }
                };
            }
        }
        
        public uvec3 this[int i0] {
            get {
                return new uvec3(this[i0, 0], this[i0, 1], this[i0, 2]);
            }
            set {
                this[i0, 0] = value[0];
                this[i0, 1] = value[1];
                this[i0, 2] = value[2];
            }
        }
        
        public static umat4x3 operator+(umat4x3 v) {
            return new umat4x3(+v.m00, +v.m01, +v.m02, +v.m10, +v.m11, +v.m12, +v.m20, +v.m21, +v.m22, +v.m30, +v.m31, +v.m32);
        }
        
        public static umat4x3 operator~(umat4x3 v) {
            return new umat4x3(~v.m00, ~v.m01, ~v.m02, ~v.m10, ~v.m11, ~v.m12, ~v.m20, ~v.m21, ~v.m22, ~v.m30, ~v.m31, ~v.m32);
        }
        
        public static umat4x3 operator++(umat4x3 v) {
            return new umat4x3(++v.m00, ++v.m01, ++v.m02, ++v.m10, ++v.m11, ++v.m12, ++v.m20, ++v.m21, ++v.m22, ++v.m30, ++v.m31, ++v.m32);
        }
        
        public static umat4x3 operator--(umat4x3 v) {
            return new umat4x3(--v.m00, --v.m01, --v.m02, --v.m10, --v.m11, --v.m12, --v.m20, --v.m21, --v.m22, --v.m30, --v.m31, --v.m32);
        }
        
        public static umat4x3 operator+(uint s, umat4x3 v2) {
            return new umat4x3(s + v2.m00, s + v2.m01, s + v2.m02, s + v2.m10, s + v2.m11, s + v2.m12, s + v2.m20, s + v2.m21, s + v2.m22, s + v2.m30, s + v2.m31, s + v2.m32);
        }
        
        public static umat4x3 operator-(uint s, umat4x3 v2) {
            return new umat4x3(s - v2.m00, s - v2.m01, s - v2.m02, s - v2.m10, s - v2.m11, s - v2.m12, s - v2.m20, s - v2.m21, s - v2.m22, s - v2.m30, s - v2.m31, s - v2.m32);
        }
        
        public static umat4x3 operator*(uint s, umat4x3 v2) {
            return new umat4x3(s * v2.m00, s * v2.m01, s * v2.m02, s * v2.m10, s * v2.m11, s * v2.m12, s * v2.m20, s * v2.m21, s * v2.m22, s * v2.m30, s * v2.m31, s * v2.m32);
        }
        
        public static umat4x3 operator/(uint s, umat4x3 v2) {
            return new umat4x3(s / v2.m00, s / v2.m01, s / v2.m02, s / v2.m10, s / v2.m11, s / v2.m12, s / v2.m20, s / v2.m21, s / v2.m22, s / v2.m30, s / v2.m31, s / v2.m32);
        }
        
        public static umat4x3 operator%(uint s, umat4x3 v2) {
            return new umat4x3(s % v2.m00, s % v2.m01, s % v2.m02, s % v2.m10, s % v2.m11, s % v2.m12, s % v2.m20, s % v2.m21, s % v2.m22, s % v2.m30, s % v2.m31, s % v2.m32);
        }
        
        public static umat4x3 operator&(uint s, umat4x3 v2) {
            return new umat4x3(s & v2.m00, s & v2.m01, s & v2.m02, s & v2.m10, s & v2.m11, s & v2.m12, s & v2.m20, s & v2.m21, s & v2.m22, s & v2.m30, s & v2.m31, s & v2.m32);
        }
        
        public static umat4x3 operator|(uint s, umat4x3 v2) {
            return new umat4x3(s | v2.m00, s | v2.m01, s | v2.m02, s | v2.m10, s | v2.m11, s | v2.m12, s | v2.m20, s | v2.m21, s | v2.m22, s | v2.m30, s | v2.m31, s | v2.m32);
        }
        
        public static umat4x3 operator^(uint s, umat4x3 v2) {
            return new umat4x3(s ^ v2.m00, s ^ v2.m01, s ^ v2.m02, s ^ v2.m10, s ^ v2.m11, s ^ v2.m12, s ^ v2.m20, s ^ v2.m21, s ^ v2.m22, s ^ v2.m30, s ^ v2.m31, s ^ v2.m32);
        }
        
        public static umat4x3 operator+(umat4x3 v1, uint s) {
            return new umat4x3(v1.m00 + s, v1.m01 + s, v1.m02 + s, v1.m10 + s, v1.m11 + s, v1.m12 + s, v1.m20 + s, v1.m21 + s, v1.m22 + s, v1.m30 + s, v1.m31 + s, v1.m32 + s);
        }
        
        public static umat4x3 operator-(umat4x3 v1, uint s) {
            return new umat4x3(v1.m00 - s, v1.m01 - s, v1.m02 - s, v1.m10 - s, v1.m11 - s, v1.m12 - s, v1.m20 - s, v1.m21 - s, v1.m22 - s, v1.m30 - s, v1.m31 - s, v1.m32 - s);
        }
        
        public static umat4x3 operator*(umat4x3 v1, uint s) {
            return new umat4x3(v1.m00 * s, v1.m01 * s, v1.m02 * s, v1.m10 * s, v1.m11 * s, v1.m12 * s, v1.m20 * s, v1.m21 * s, v1.m22 * s, v1.m30 * s, v1.m31 * s, v1.m32 * s);
        }
        
        public static umat4x3 operator/(umat4x3 v1, uint s) {
            return new umat4x3(v1.m00 / s, v1.m01 / s, v1.m02 / s, v1.m10 / s, v1.m11 / s, v1.m12 / s, v1.m20 / s, v1.m21 / s, v1.m22 / s, v1.m30 / s, v1.m31 / s, v1.m32 / s);
        }
        
        public static umat4x3 operator%(umat4x3 v1, uint s) {
            return new umat4x3(v1.m00 % s, v1.m01 % s, v1.m02 % s, v1.m10 % s, v1.m11 % s, v1.m12 % s, v1.m20 % s, v1.m21 % s, v1.m22 % s, v1.m30 % s, v1.m31 % s, v1.m32 % s);
        }
        
        public static umat4x3 operator&(umat4x3 v1, uint s) {
            return new umat4x3(v1.m00 & s, v1.m01 & s, v1.m02 & s, v1.m10 & s, v1.m11 & s, v1.m12 & s, v1.m20 & s, v1.m21 & s, v1.m22 & s, v1.m30 & s, v1.m31 & s, v1.m32 & s);
        }
        
        public static umat4x3 operator|(umat4x3 v1, uint s) {
            return new umat4x3(v1.m00 | s, v1.m01 | s, v1.m02 | s, v1.m10 | s, v1.m11 | s, v1.m12 | s, v1.m20 | s, v1.m21 | s, v1.m22 | s, v1.m30 | s, v1.m31 | s, v1.m32 | s);
        }
        
        public static umat4x3 operator^(umat4x3 v1, uint s) {
            return new umat4x3(v1.m00 ^ s, v1.m01 ^ s, v1.m02 ^ s, v1.m10 ^ s, v1.m11 ^ s, v1.m12 ^ s, v1.m20 ^ s, v1.m21 ^ s, v1.m22 ^ s, v1.m30 ^ s, v1.m31 ^ s, v1.m32 ^ s);
        }
        
        public static umat4x3 operator+(umat4x3 v1, umat4x3 v2) {
            return new umat4x3(v1.m00 + v2.m00, v1.m01 + v2.m01, v1.m02 + v2.m02, v1.m10 + v2.m10, v1.m11 + v2.m11, v1.m12 + v2.m12, v1.m20 + v2.m20, v1.m21 + v2.m21, v1.m22 + v2.m22, v1.m30 + v2.m30, v1.m31 + v2.m31, v1.m32 + v2.m32);
        }
        
        public static umat4x3 operator-(umat4x3 v1, umat4x3 v2) {
            return new umat4x3(v1.m00 - v2.m00, v1.m01 - v2.m01, v1.m02 - v2.m02, v1.m10 - v2.m10, v1.m11 - v2.m11, v1.m12 - v2.m12, v1.m20 - v2.m20, v1.m21 - v2.m21, v1.m22 - v2.m22, v1.m30 - v2.m30, v1.m31 - v2.m31, v1.m32 - v2.m32);
        }
        
        public static umat4x3 operator/(umat4x3 v1, umat4x3 v2) {
            return new umat4x3(v1.m00 / v2.m00, v1.m01 / v2.m01, v1.m02 / v2.m02, v1.m10 / v2.m10, v1.m11 / v2.m11, v1.m12 / v2.m12, v1.m20 / v2.m20, v1.m21 / v2.m21, v1.m22 / v2.m22, v1.m30 / v2.m30, v1.m31 / v2.m31, v1.m32 / v2.m32);
        }
        
        public static umat4x3 operator%(umat4x3 v1, umat4x3 v2) {
            return new umat4x3(v1.m00 % v2.m00, v1.m01 % v2.m01, v1.m02 % v2.m02, v1.m10 % v2.m10, v1.m11 % v2.m11, v1.m12 % v2.m12, v1.m20 % v2.m20, v1.m21 % v2.m21, v1.m22 % v2.m22, v1.m30 % v2.m30, v1.m31 % v2.m31, v1.m32 % v2.m32);
        }
        
        public static umat4x3 operator&(umat4x3 v1, umat4x3 v2) {
            return new umat4x3(v1.m00 & v2.m00, v1.m01 & v2.m01, v1.m02 & v2.m02, v1.m10 & v2.m10, v1.m11 & v2.m11, v1.m12 & v2.m12, v1.m20 & v2.m20, v1.m21 & v2.m21, v1.m22 & v2.m22, v1.m30 & v2.m30, v1.m31 & v2.m31, v1.m32 & v2.m32);
        }
        
        public static umat4x3 operator|(umat4x3 v1, umat4x3 v2) {
            return new umat4x3(v1.m00 | v2.m00, v1.m01 | v2.m01, v1.m02 | v2.m02, v1.m10 | v2.m10, v1.m11 | v2.m11, v1.m12 | v2.m12, v1.m20 | v2.m20, v1.m21 | v2.m21, v1.m22 | v2.m22, v1.m30 | v2.m30, v1.m31 | v2.m31, v1.m32 | v2.m32);
        }
        
        public static umat4x3 operator^(umat4x3 v1, umat4x3 v2) {
            return new umat4x3(v1.m00 ^ v2.m00, v1.m01 ^ v2.m01, v1.m02 ^ v2.m02, v1.m10 ^ v2.m10, v1.m11 ^ v2.m11, v1.m12 ^ v2.m12, v1.m20 ^ v2.m20, v1.m21 ^ v2.m21, v1.m22 ^ v2.m22, v1.m30 ^ v2.m30, v1.m31 ^ v2.m31, v1.m32 ^ v2.m32);
        }
        
        public static bmat4x3 operator==(umat4x3 v1, umat4x3 v2) {
            return new bmat4x3(v1.m00 == v2.m00, v1.m01 == v2.m01, v1.m02 == v2.m02, v1.m10 == v2.m10, v1.m11 == v2.m11, v1.m12 == v2.m12, v1.m20 == v2.m20, v1.m21 == v2.m21, v1.m22 == v2.m22, v1.m30 == v2.m30, v1.m31 == v2.m31, v1.m32 == v2.m32);
        }
        
        public static bmat4x3 operator>(umat4x3 v1, umat4x3 v2) {
            return new bmat4x3(v1.m00 > v2.m00, v1.m01 > v2.m01, v1.m02 > v2.m02, v1.m10 > v2.m10, v1.m11 > v2.m11, v1.m12 > v2.m12, v1.m20 > v2.m20, v1.m21 > v2.m21, v1.m22 > v2.m22, v1.m30 > v2.m30, v1.m31 > v2.m31, v1.m32 > v2.m32);
        }
        
        public static bmat4x3 operator>=(umat4x3 v1, umat4x3 v2) {
            return new bmat4x3(v1.m00 >= v2.m00, v1.m01 >= v2.m01, v1.m02 >= v2.m02, v1.m10 >= v2.m10, v1.m11 >= v2.m11, v1.m12 >= v2.m12, v1.m20 >= v2.m20, v1.m21 >= v2.m21, v1.m22 >= v2.m22, v1.m30 >= v2.m30, v1.m31 >= v2.m31, v1.m32 >= v2.m32);
        }
        
        public static bmat4x3 operator<(umat4x3 v1, umat4x3 v2) {
            return new bmat4x3(v1.m00 < v2.m00, v1.m01 < v2.m01, v1.m02 < v2.m02, v1.m10 < v2.m10, v1.m11 < v2.m11, v1.m12 < v2.m12, v1.m20 < v2.m20, v1.m21 < v2.m21, v1.m22 < v2.m22, v1.m30 < v2.m30, v1.m31 < v2.m31, v1.m32 < v2.m32);
        }
        
        public static bmat4x3 operator<=(umat4x3 v1, umat4x3 v2) {
            return new bmat4x3(v1.m00 <= v2.m00, v1.m01 <= v2.m01, v1.m02 <= v2.m02, v1.m10 <= v2.m10, v1.m11 <= v2.m11, v1.m12 <= v2.m12, v1.m20 <= v2.m20, v1.m21 <= v2.m21, v1.m22 <= v2.m22, v1.m30 <= v2.m30, v1.m31 <= v2.m31, v1.m32 <= v2.m32);
        }
        
        public static bmat4x3 operator!=(umat4x3 v1, umat4x3 v2) {
            return new bmat4x3(v1.m00 != v2.m00, v1.m01 != v2.m01, v1.m02 != v2.m02, v1.m10 != v2.m10, v1.m11 != v2.m11, v1.m12 != v2.m12, v1.m20 != v2.m20, v1.m21 != v2.m21, v1.m22 != v2.m22, v1.m30 != v2.m30, v1.m31 != v2.m31, v1.m32 != v2.m32);
        }
        
        public static uvec3 operator*(uvec4 v0, umat4x3 m1) {
            return umath.InnerProduct1(ref v0, ref m1);
        }
        
        public static uvec4 operator*(umat4x3 m0, uvec3 v1) {
            return umath.InnerProduct1(ref m0, ref v1);
        }
        
        public static umat4x2 operator*(umat4x3 m0, umat3x2 m1) {
            return umath.InnerProduct1(ref m0, ref m1);
        }
        
        public static umat4x3 operator*(umat4x3 m0, umat3 m1) {
            return umath.InnerProduct1(ref m0, ref m1);
        }
        
        public static umat4 operator*(umat4x3 m0, umat3x4 m1) {
            return umath.InnerProduct1(ref m0, ref m1);
        }
        
        public override bool Equals(object obj) {
            return (obj is umat4x3) ? bmath.All(this == (umat4x3)obj) : false;
        }
        
        public override int GetHashCode() {
            return this.m00.GetHashCode() ^ this.m01.GetHashCode() ^ this.m02.GetHashCode() ^ this.m10.GetHashCode() ^ this.m11.GetHashCode() ^ this.m12.GetHashCode() ^ this.m20.GetHashCode() ^ this.m21.GetHashCode() ^ this.m22.GetHashCode() ^ this.m30.GetHashCode() ^ this.m31.GetHashCode() ^ this.m32.GetHashCode();
        }
        
        public override string ToString() {
            return "umat4x3(" + this.m00 + "," + this.m01 + "," + this.m02 + "," + this.m10 + "," + this.m11 + "," + this.m12 + "," + this.m20 + "," + this.m21 + "," + this.m22 + "," + this.m30 + "," + this.m31 + "," + this.m32 + ")";
        }
    }
    
    [DataContract()]
    [StructLayout(LayoutKind.Sequential)]
    public struct umat4 {
        
        [DataMember()]
        internal uint m00;
        
        [DataMember()]
        internal uint m01;
        
        [DataMember()]
        internal uint m02;
        
        [DataMember()]
        internal uint m03;
        
        [DataMember()]
        internal uint m10;
        
        [DataMember()]
        internal uint m11;
        
        [DataMember()]
        internal uint m12;
        
        [DataMember()]
        internal uint m13;
        
        [DataMember()]
        internal uint m20;
        
        [DataMember()]
        internal uint m21;
        
        [DataMember()]
        internal uint m22;
        
        [DataMember()]
        internal uint m23;
        
        [DataMember()]
        internal uint m30;
        
        [DataMember()]
        internal uint m31;
        
        [DataMember()]
        internal uint m32;
        
        [DataMember()]
        internal uint m33;
        
        public umat4(uint s) {
            this.m00 = s;
            this.m01 = 0u;
            this.m02 = 0u;
            this.m03 = 0u;
            this.m10 = 0u;
            this.m11 = s;
            this.m12 = 0u;
            this.m13 = 0u;
            this.m20 = 0u;
            this.m21 = 0u;
            this.m22 = s;
            this.m23 = 0u;
            this.m30 = 0u;
            this.m31 = 0u;
            this.m32 = 0u;
            this.m33 = s;
        }
        
        public umat4(umat4 m00) {
            this.m00 = m00.m00;
            this.m01 = m00.m01;
            this.m02 = m00.m02;
            this.m03 = m00.m03;
            this.m10 = m00.m10;
            this.m11 = m00.m11;
            this.m12 = m00.m12;
            this.m13 = m00.m13;
            this.m20 = m00.m20;
            this.m21 = m00.m21;
            this.m22 = m00.m22;
            this.m23 = m00.m23;
            this.m30 = m00.m30;
            this.m31 = m00.m31;
            this.m32 = m00.m32;
            this.m33 = m00.m33;
        }
        
        public umat4(umat4x2 m00, umat4x2 m01) {
            this.m00 = m00.m00;
            this.m01 = m00.m01;
            this.m02 = m01.m00;
            this.m03 = m01.m01;
            this.m10 = m00.m10;
            this.m11 = m00.m11;
            this.m12 = m01.m10;
            this.m13 = m01.m11;
            this.m20 = m00.m20;
            this.m21 = m00.m21;
            this.m22 = m01.m20;
            this.m23 = m01.m21;
            this.m30 = m00.m30;
            this.m31 = m00.m31;
            this.m32 = m01.m30;
            this.m33 = m01.m31;
        }
        
        public umat4(umat2x4 m00, umat2x4 m10) {
            this.m00 = m00.m00;
            this.m01 = m00.m01;
            this.m02 = m00.m02;
            this.m03 = m00.m03;
            this.m10 = m00.m10;
            this.m11 = m00.m11;
            this.m12 = m00.m12;
            this.m13 = m00.m13;
            this.m20 = m10.m00;
            this.m21 = m10.m01;
            this.m22 = m10.m02;
            this.m23 = m10.m03;
            this.m30 = m10.m10;
            this.m31 = m10.m11;
            this.m32 = m10.m12;
            this.m33 = m10.m13;
        }
        
        public umat4(umat2 m00, umat2 m01, umat2 m10, umat2 m11) {
            this.m00 = m00.m00;
            this.m01 = m00.m01;
            this.m02 = m01.m00;
            this.m03 = m01.m01;
            this.m10 = m00.m10;
            this.m11 = m00.m11;
            this.m12 = m01.m10;
            this.m13 = m01.m11;
            this.m20 = m10.m00;
            this.m21 = m10.m01;
            this.m22 = m11.m00;
            this.m23 = m11.m01;
            this.m30 = m10.m10;
            this.m31 = m10.m11;
            this.m32 = m11.m10;
            this.m33 = m11.m11;
        }
        
        public umat4(umat3x4 m00, uvec4 m10) {
            this.m00 = m00.m00;
            this.m01 = m00.m01;
            this.m02 = m00.m02;
            this.m03 = m00.m03;
            this.m10 = m00.m10;
            this.m11 = m00.m11;
            this.m12 = m00.m12;
            this.m13 = m00.m13;
            this.m20 = m00.m20;
            this.m21 = m00.m21;
            this.m22 = m00.m22;
            this.m23 = m00.m23;
            this.m30 = m10.v0;
            this.m31 = m10.v1;
            this.m32 = m10.v2;
            this.m33 = m10.v3;
        }
        
        public umat4(umat3x2 m00, umat3x2 m01, uvec2 m10, uvec2 m11) {
            this.m00 = m00.m00;
            this.m01 = m00.m01;
            this.m02 = m01.m00;
            this.m03 = m01.m01;
            this.m10 = m00.m10;
            this.m11 = m00.m11;
            this.m12 = m01.m10;
            this.m13 = m01.m11;
            this.m20 = m00.m20;
            this.m21 = m00.m21;
            this.m22 = m01.m20;
            this.m23 = m01.m21;
            this.m30 = m10.v0;
            this.m31 = m10.v1;
            this.m32 = m11.v0;
            this.m33 = m11.v1;
        }
        
        public umat4(uvec4 m00, umat3x4 m10) {
            this.m00 = m00.v0;
            this.m01 = m00.v1;
            this.m02 = m00.v2;
            this.m03 = m00.v3;
            this.m10 = m10.m00;
            this.m11 = m10.m01;
            this.m12 = m10.m02;
            this.m13 = m10.m03;
            this.m20 = m10.m10;
            this.m21 = m10.m11;
            this.m22 = m10.m12;
            this.m23 = m10.m13;
            this.m30 = m10.m20;
            this.m31 = m10.m21;
            this.m32 = m10.m22;
            this.m33 = m10.m23;
        }
        
        public umat4(uvec2 m00, uvec2 m01, umat3x2 m10, umat3x2 m11) {
            this.m00 = m00.v0;
            this.m01 = m00.v1;
            this.m02 = m01.v0;
            this.m03 = m01.v1;
            this.m10 = m10.m00;
            this.m11 = m10.m01;
            this.m12 = m11.m00;
            this.m13 = m11.m01;
            this.m20 = m10.m10;
            this.m21 = m10.m11;
            this.m22 = m11.m10;
            this.m23 = m11.m11;
            this.m30 = m10.m20;
            this.m31 = m10.m21;
            this.m32 = m11.m20;
            this.m33 = m11.m21;
        }
        
        public umat4(umat2x4 m00, uvec4 m10, uvec4 m20) {
            this.m00 = m00.m00;
            this.m01 = m00.m01;
            this.m02 = m00.m02;
            this.m03 = m00.m03;
            this.m10 = m00.m10;
            this.m11 = m00.m11;
            this.m12 = m00.m12;
            this.m13 = m00.m13;
            this.m20 = m10.v0;
            this.m21 = m10.v1;
            this.m22 = m10.v2;
            this.m23 = m10.v3;
            this.m30 = m20.v0;
            this.m31 = m20.v1;
            this.m32 = m20.v2;
            this.m33 = m20.v3;
        }
        
        public umat4(umat2 m00, umat2 m01, uvec2 m10, uvec2 m11, uvec2 m20, uvec2 m21) {
            this.m00 = m00.m00;
            this.m01 = m00.m01;
            this.m02 = m01.m00;
            this.m03 = m01.m01;
            this.m10 = m00.m10;
            this.m11 = m00.m11;
            this.m12 = m01.m10;
            this.m13 = m01.m11;
            this.m20 = m10.v0;
            this.m21 = m10.v1;
            this.m22 = m11.v0;
            this.m23 = m11.v1;
            this.m30 = m20.v0;
            this.m31 = m20.v1;
            this.m32 = m21.v0;
            this.m33 = m21.v1;
        }
        
        public umat4(uvec4 m00, umat2x4 m10, uvec4 m20) {
            this.m00 = m00.v0;
            this.m01 = m00.v1;
            this.m02 = m00.v2;
            this.m03 = m00.v3;
            this.m10 = m10.m00;
            this.m11 = m10.m01;
            this.m12 = m10.m02;
            this.m13 = m10.m03;
            this.m20 = m10.m10;
            this.m21 = m10.m11;
            this.m22 = m10.m12;
            this.m23 = m10.m13;
            this.m30 = m20.v0;
            this.m31 = m20.v1;
            this.m32 = m20.v2;
            this.m33 = m20.v3;
        }
        
        public umat4(uvec2 m00, uvec2 m01, umat2 m10, umat2 m11, uvec2 m20, uvec2 m21) {
            this.m00 = m00.v0;
            this.m01 = m00.v1;
            this.m02 = m01.v0;
            this.m03 = m01.v1;
            this.m10 = m10.m00;
            this.m11 = m10.m01;
            this.m12 = m11.m00;
            this.m13 = m11.m01;
            this.m20 = m10.m10;
            this.m21 = m10.m11;
            this.m22 = m11.m10;
            this.m23 = m11.m11;
            this.m30 = m20.v0;
            this.m31 = m20.v1;
            this.m32 = m21.v0;
            this.m33 = m21.v1;
        }
        
        public umat4(uvec4 m00, uvec4 m10, umat2x4 m20) {
            this.m00 = m00.v0;
            this.m01 = m00.v1;
            this.m02 = m00.v2;
            this.m03 = m00.v3;
            this.m10 = m10.v0;
            this.m11 = m10.v1;
            this.m12 = m10.v2;
            this.m13 = m10.v3;
            this.m20 = m20.m00;
            this.m21 = m20.m01;
            this.m22 = m20.m02;
            this.m23 = m20.m03;
            this.m30 = m20.m10;
            this.m31 = m20.m11;
            this.m32 = m20.m12;
            this.m33 = m20.m13;
        }
        
        public umat4(uvec2 m00, uvec2 m01, uvec2 m10, uvec2 m11, umat2 m20, umat2 m21) {
            this.m00 = m00.v0;
            this.m01 = m00.v1;
            this.m02 = m01.v0;
            this.m03 = m01.v1;
            this.m10 = m10.v0;
            this.m11 = m10.v1;
            this.m12 = m11.v0;
            this.m13 = m11.v1;
            this.m20 = m20.m00;
            this.m21 = m20.m01;
            this.m22 = m21.m00;
            this.m23 = m21.m01;
            this.m30 = m20.m10;
            this.m31 = m20.m11;
            this.m32 = m21.m10;
            this.m33 = m21.m11;
        }
        
        public umat4(uvec4 m00, uvec4 m10, uvec4 m20, uvec4 m30) {
            this.m00 = m00.v0;
            this.m01 = m00.v1;
            this.m02 = m00.v2;
            this.m03 = m00.v3;
            this.m10 = m10.v0;
            this.m11 = m10.v1;
            this.m12 = m10.v2;
            this.m13 = m10.v3;
            this.m20 = m20.v0;
            this.m21 = m20.v1;
            this.m22 = m20.v2;
            this.m23 = m20.v3;
            this.m30 = m30.v0;
            this.m31 = m30.v1;
            this.m32 = m30.v2;
            this.m33 = m30.v3;
        }
        
        public umat4(uvec2 m00, uvec2 m01, uvec2 m10, uvec2 m11, uvec2 m20, uvec2 m21, uvec2 m30, uvec2 m31) {
            this.m00 = m00.v0;
            this.m01 = m00.v1;
            this.m02 = m01.v0;
            this.m03 = m01.v1;
            this.m10 = m10.v0;
            this.m11 = m10.v1;
            this.m12 = m11.v0;
            this.m13 = m11.v1;
            this.m20 = m20.v0;
            this.m21 = m20.v1;
            this.m22 = m21.v0;
            this.m23 = m21.v1;
            this.m30 = m30.v0;
            this.m31 = m30.v1;
            this.m32 = m31.v0;
            this.m33 = m31.v1;
        }
        
        public umat4(uvec3 m00, uint m01, uvec3 m10, uint m11, uvec3 m20, uint m21, uvec3 m30, uint m31) {
            this.m00 = m00.v0;
            this.m01 = m00.v1;
            this.m02 = m00.v2;
            this.m03 = m01;
            this.m10 = m10.v0;
            this.m11 = m10.v1;
            this.m12 = m10.v2;
            this.m13 = m11;
            this.m20 = m20.v0;
            this.m21 = m20.v1;
            this.m22 = m20.v2;
            this.m23 = m21;
            this.m30 = m30.v0;
            this.m31 = m30.v1;
            this.m32 = m30.v2;
            this.m33 = m31;
        }
        
        public umat4(uint m00, uvec3 m01, uint m10, uvec3 m11, uint m20, uvec3 m21, uint m30, uvec3 m31) {
            this.m00 = m00;
            this.m01 = m01.v0;
            this.m02 = m01.v1;
            this.m03 = m01.v2;
            this.m10 = m10;
            this.m11 = m11.v0;
            this.m12 = m11.v1;
            this.m13 = m11.v2;
            this.m20 = m20;
            this.m21 = m21.v0;
            this.m22 = m21.v1;
            this.m23 = m21.v2;
            this.m30 = m30;
            this.m31 = m31.v0;
            this.m32 = m31.v1;
            this.m33 = m31.v2;
        }
        
        public umat4(uvec2 m00, uint m01, uint m02, uvec2 m10, uint m11, uint m12, uvec2 m20, uint m21, uint m22, uvec2 m30, uint m31, uint m32) {
            this.m00 = m00.v0;
            this.m01 = m00.v1;
            this.m02 = m01;
            this.m03 = m02;
            this.m10 = m10.v0;
            this.m11 = m10.v1;
            this.m12 = m11;
            this.m13 = m12;
            this.m20 = m20.v0;
            this.m21 = m20.v1;
            this.m22 = m21;
            this.m23 = m22;
            this.m30 = m30.v0;
            this.m31 = m30.v1;
            this.m32 = m31;
            this.m33 = m32;
        }
        
        public umat4(uint m00, uvec2 m01, uint m02, uint m10, uvec2 m11, uint m12, uint m20, uvec2 m21, uint m22, uint m30, uvec2 m31, uint m32) {
            this.m00 = m00;
            this.m01 = m01.v0;
            this.m02 = m01.v1;
            this.m03 = m02;
            this.m10 = m10;
            this.m11 = m11.v0;
            this.m12 = m11.v1;
            this.m13 = m12;
            this.m20 = m20;
            this.m21 = m21.v0;
            this.m22 = m21.v1;
            this.m23 = m22;
            this.m30 = m30;
            this.m31 = m31.v0;
            this.m32 = m31.v1;
            this.m33 = m32;
        }
        
        public umat4(uint m00, uint m01, uvec2 m02, uint m10, uint m11, uvec2 m12, uint m20, uint m21, uvec2 m22, uint m30, uint m31, uvec2 m32) {
            this.m00 = m00;
            this.m01 = m01;
            this.m02 = m02.v0;
            this.m03 = m02.v1;
            this.m10 = m10;
            this.m11 = m11;
            this.m12 = m12.v0;
            this.m13 = m12.v1;
            this.m20 = m20;
            this.m21 = m21;
            this.m22 = m22.v0;
            this.m23 = m22.v1;
            this.m30 = m30;
            this.m31 = m31;
            this.m32 = m32.v0;
            this.m33 = m32.v1;
        }
        
        public umat4(
                    uint m00, 
                    uint m01, 
                    uint m02, 
                    uint m03, 
                    uint m10, 
                    uint m11, 
                    uint m12, 
                    uint m13, 
                    uint m20, 
                    uint m21, 
                    uint m22, 
                    uint m23, 
                    uint m30, 
                    uint m31, 
                    uint m32, 
                    uint m33) {
            this.m00 = m00;
            this.m01 = m01;
            this.m02 = m02;
            this.m03 = m03;
            this.m10 = m10;
            this.m11 = m11;
            this.m12 = m12;
            this.m13 = m13;
            this.m20 = m20;
            this.m21 = m21;
            this.m22 = m22;
            this.m23 = m23;
            this.m30 = m30;
            this.m31 = m31;
            this.m32 = m32;
            this.m33 = m33;
        }
        
        public umat4(umat2 s) {
            this.m00 = s.m00;
            this.m01 = s.m01;
            this.m02 = 0u;
            this.m03 = 0u;
            this.m10 = s.m10;
            this.m11 = s.m11;
            this.m12 = 0u;
            this.m13 = 0u;
            this.m20 = 0u;
            this.m21 = 0u;
            this.m22 = 0u;
            this.m23 = 0u;
            this.m30 = 0u;
            this.m31 = 0u;
            this.m32 = 0u;
            this.m33 = 0u;
        }
        
        public umat4(umat2x3 s) {
            this.m00 = s.m00;
            this.m01 = s.m01;
            this.m02 = s.m02;
            this.m03 = 0u;
            this.m10 = s.m10;
            this.m11 = s.m11;
            this.m12 = s.m12;
            this.m13 = 0u;
            this.m20 = 0u;
            this.m21 = 0u;
            this.m22 = 0u;
            this.m23 = 0u;
            this.m30 = 0u;
            this.m31 = 0u;
            this.m32 = 0u;
            this.m33 = 0u;
        }
        
        public umat4(umat2x4 s) {
            this.m00 = s.m00;
            this.m01 = s.m01;
            this.m02 = s.m02;
            this.m03 = s.m03;
            this.m10 = s.m10;
            this.m11 = s.m11;
            this.m12 = s.m12;
            this.m13 = s.m13;
            this.m20 = 0u;
            this.m21 = 0u;
            this.m22 = 0u;
            this.m23 = 0u;
            this.m30 = 0u;
            this.m31 = 0u;
            this.m32 = 0u;
            this.m33 = 0u;
        }
        
        public umat4(umat3x2 s) {
            this.m00 = s.m00;
            this.m01 = s.m01;
            this.m02 = 0u;
            this.m03 = 0u;
            this.m10 = s.m10;
            this.m11 = s.m11;
            this.m12 = 0u;
            this.m13 = 0u;
            this.m20 = s.m20;
            this.m21 = s.m21;
            this.m22 = 0u;
            this.m23 = 0u;
            this.m30 = 0u;
            this.m31 = 0u;
            this.m32 = 0u;
            this.m33 = 0u;
        }
        
        public umat4(umat3 s) {
            this.m00 = s.m00;
            this.m01 = s.m01;
            this.m02 = s.m02;
            this.m03 = 0u;
            this.m10 = s.m10;
            this.m11 = s.m11;
            this.m12 = s.m12;
            this.m13 = 0u;
            this.m20 = s.m20;
            this.m21 = s.m21;
            this.m22 = s.m22;
            this.m23 = 0u;
            this.m30 = 0u;
            this.m31 = 0u;
            this.m32 = 0u;
            this.m33 = 0u;
        }
        
        public umat4(umat3x4 s) {
            this.m00 = s.m00;
            this.m01 = s.m01;
            this.m02 = s.m02;
            this.m03 = s.m03;
            this.m10 = s.m10;
            this.m11 = s.m11;
            this.m12 = s.m12;
            this.m13 = s.m13;
            this.m20 = s.m20;
            this.m21 = s.m21;
            this.m22 = s.m22;
            this.m23 = s.m23;
            this.m30 = 0u;
            this.m31 = 0u;
            this.m32 = 0u;
            this.m33 = 0u;
        }
        
        public umat4(umat4x2 s) {
            this.m00 = s.m00;
            this.m01 = s.m01;
            this.m02 = 0u;
            this.m03 = 0u;
            this.m10 = s.m10;
            this.m11 = s.m11;
            this.m12 = 0u;
            this.m13 = 0u;
            this.m20 = s.m20;
            this.m21 = s.m21;
            this.m22 = 0u;
            this.m23 = 0u;
            this.m30 = s.m30;
            this.m31 = s.m31;
            this.m32 = 0u;
            this.m33 = 0u;
        }
        
        public umat4(umat4x3 s) {
            this.m00 = s.m00;
            this.m01 = s.m01;
            this.m02 = s.m02;
            this.m03 = 0u;
            this.m10 = s.m10;
            this.m11 = s.m11;
            this.m12 = s.m12;
            this.m13 = 0u;
            this.m20 = s.m20;
            this.m21 = s.m21;
            this.m22 = s.m22;
            this.m23 = 0u;
            this.m30 = s.m30;
            this.m31 = s.m31;
            this.m32 = s.m32;
            this.m33 = 0u;
        }
        
        public uint this[int i0, int i1] {
            get {
                if (((i0 < 0) 
                            || (3 < i0))) {
                    throw new System.IndexOutOfRangeException("i0");
                }
                if (((i1 < 0) 
                            || (3 < i1))) {
                    throw new System.IndexOutOfRangeException("i1");
                }
                
                unsafe
                {
                    fixed (UInt32* v_ptr = &this.m00)
                    {
                        return *(v_ptr + (i1 + i0 * 4));
                    }
                };
            }
            set {
                if (((i0 < 0) 
                            || (3 < i0))) {
                    throw new System.IndexOutOfRangeException("i0");
                }
                if (((i1 < 0) 
                            || (3 < i1))) {
                    throw new System.IndexOutOfRangeException("i1");
                }
                
                unsafe
                {
                    fixed (UInt32* v_ptr = &this.m00)
                    {
                        *(v_ptr  + (i1 + i0 * 4)) = value;
                    }
                };
            }
        }
        
        public uvec4 this[int i0] {
            get {
                return new uvec4(this[i0, 0], this[i0, 1], this[i0, 2], this[i0, 3]);
            }
            set {
                this[i0, 0] = value[0];
                this[i0, 1] = value[1];
                this[i0, 2] = value[2];
                this[i0, 3] = value[3];
            }
        }
        
        public static umat4 operator+(umat4 v) {
            return new umat4(+v.m00, +v.m01, +v.m02, +v.m03, +v.m10, +v.m11, +v.m12, +v.m13, +v.m20, +v.m21, +v.m22, +v.m23, +v.m30, +v.m31, +v.m32, +v.m33);
        }
        
        public static umat4 operator~(umat4 v) {
            return new umat4(~v.m00, ~v.m01, ~v.m02, ~v.m03, ~v.m10, ~v.m11, ~v.m12, ~v.m13, ~v.m20, ~v.m21, ~v.m22, ~v.m23, ~v.m30, ~v.m31, ~v.m32, ~v.m33);
        }
        
        public static umat4 operator++(umat4 v) {
            return new umat4(++v.m00, ++v.m01, ++v.m02, ++v.m03, ++v.m10, ++v.m11, ++v.m12, ++v.m13, ++v.m20, ++v.m21, ++v.m22, ++v.m23, ++v.m30, ++v.m31, ++v.m32, ++v.m33);
        }
        
        public static umat4 operator--(umat4 v) {
            return new umat4(--v.m00, --v.m01, --v.m02, --v.m03, --v.m10, --v.m11, --v.m12, --v.m13, --v.m20, --v.m21, --v.m22, --v.m23, --v.m30, --v.m31, --v.m32, --v.m33);
        }
        
        public static umat4 operator+(uint s, umat4 v2) {
            return new umat4(s + v2.m00, s + v2.m01, s + v2.m02, s + v2.m03, s + v2.m10, s + v2.m11, s + v2.m12, s + v2.m13, s + v2.m20, s + v2.m21, s + v2.m22, s + v2.m23, s + v2.m30, s + v2.m31, s + v2.m32, s + v2.m33);
        }
        
        public static umat4 operator-(uint s, umat4 v2) {
            return new umat4(s - v2.m00, s - v2.m01, s - v2.m02, s - v2.m03, s - v2.m10, s - v2.m11, s - v2.m12, s - v2.m13, s - v2.m20, s - v2.m21, s - v2.m22, s - v2.m23, s - v2.m30, s - v2.m31, s - v2.m32, s - v2.m33);
        }
        
        public static umat4 operator*(uint s, umat4 v2) {
            return new umat4(s * v2.m00, s * v2.m01, s * v2.m02, s * v2.m03, s * v2.m10, s * v2.m11, s * v2.m12, s * v2.m13, s * v2.m20, s * v2.m21, s * v2.m22, s * v2.m23, s * v2.m30, s * v2.m31, s * v2.m32, s * v2.m33);
        }
        
        public static umat4 operator/(uint s, umat4 v2) {
            return new umat4(s / v2.m00, s / v2.m01, s / v2.m02, s / v2.m03, s / v2.m10, s / v2.m11, s / v2.m12, s / v2.m13, s / v2.m20, s / v2.m21, s / v2.m22, s / v2.m23, s / v2.m30, s / v2.m31, s / v2.m32, s / v2.m33);
        }
        
        public static umat4 operator%(uint s, umat4 v2) {
            return new umat4(s % v2.m00, s % v2.m01, s % v2.m02, s % v2.m03, s % v2.m10, s % v2.m11, s % v2.m12, s % v2.m13, s % v2.m20, s % v2.m21, s % v2.m22, s % v2.m23, s % v2.m30, s % v2.m31, s % v2.m32, s % v2.m33);
        }
        
        public static umat4 operator&(uint s, umat4 v2) {
            return new umat4(s & v2.m00, s & v2.m01, s & v2.m02, s & v2.m03, s & v2.m10, s & v2.m11, s & v2.m12, s & v2.m13, s & v2.m20, s & v2.m21, s & v2.m22, s & v2.m23, s & v2.m30, s & v2.m31, s & v2.m32, s & v2.m33);
        }
        
        public static umat4 operator|(uint s, umat4 v2) {
            return new umat4(s | v2.m00, s | v2.m01, s | v2.m02, s | v2.m03, s | v2.m10, s | v2.m11, s | v2.m12, s | v2.m13, s | v2.m20, s | v2.m21, s | v2.m22, s | v2.m23, s | v2.m30, s | v2.m31, s | v2.m32, s | v2.m33);
        }
        
        public static umat4 operator^(uint s, umat4 v2) {
            return new umat4(s ^ v2.m00, s ^ v2.m01, s ^ v2.m02, s ^ v2.m03, s ^ v2.m10, s ^ v2.m11, s ^ v2.m12, s ^ v2.m13, s ^ v2.m20, s ^ v2.m21, s ^ v2.m22, s ^ v2.m23, s ^ v2.m30, s ^ v2.m31, s ^ v2.m32, s ^ v2.m33);
        }
        
        public static umat4 operator+(umat4 v1, uint s) {
            return new umat4(v1.m00 + s, v1.m01 + s, v1.m02 + s, v1.m03 + s, v1.m10 + s, v1.m11 + s, v1.m12 + s, v1.m13 + s, v1.m20 + s, v1.m21 + s, v1.m22 + s, v1.m23 + s, v1.m30 + s, v1.m31 + s, v1.m32 + s, v1.m33 + s);
        }
        
        public static umat4 operator-(umat4 v1, uint s) {
            return new umat4(v1.m00 - s, v1.m01 - s, v1.m02 - s, v1.m03 - s, v1.m10 - s, v1.m11 - s, v1.m12 - s, v1.m13 - s, v1.m20 - s, v1.m21 - s, v1.m22 - s, v1.m23 - s, v1.m30 - s, v1.m31 - s, v1.m32 - s, v1.m33 - s);
        }
        
        public static umat4 operator*(umat4 v1, uint s) {
            return new umat4(v1.m00 * s, v1.m01 * s, v1.m02 * s, v1.m03 * s, v1.m10 * s, v1.m11 * s, v1.m12 * s, v1.m13 * s, v1.m20 * s, v1.m21 * s, v1.m22 * s, v1.m23 * s, v1.m30 * s, v1.m31 * s, v1.m32 * s, v1.m33 * s);
        }
        
        public static umat4 operator/(umat4 v1, uint s) {
            return new umat4(v1.m00 / s, v1.m01 / s, v1.m02 / s, v1.m03 / s, v1.m10 / s, v1.m11 / s, v1.m12 / s, v1.m13 / s, v1.m20 / s, v1.m21 / s, v1.m22 / s, v1.m23 / s, v1.m30 / s, v1.m31 / s, v1.m32 / s, v1.m33 / s);
        }
        
        public static umat4 operator%(umat4 v1, uint s) {
            return new umat4(v1.m00 % s, v1.m01 % s, v1.m02 % s, v1.m03 % s, v1.m10 % s, v1.m11 % s, v1.m12 % s, v1.m13 % s, v1.m20 % s, v1.m21 % s, v1.m22 % s, v1.m23 % s, v1.m30 % s, v1.m31 % s, v1.m32 % s, v1.m33 % s);
        }
        
        public static umat4 operator&(umat4 v1, uint s) {
            return new umat4(v1.m00 & s, v1.m01 & s, v1.m02 & s, v1.m03 & s, v1.m10 & s, v1.m11 & s, v1.m12 & s, v1.m13 & s, v1.m20 & s, v1.m21 & s, v1.m22 & s, v1.m23 & s, v1.m30 & s, v1.m31 & s, v1.m32 & s, v1.m33 & s);
        }
        
        public static umat4 operator|(umat4 v1, uint s) {
            return new umat4(v1.m00 | s, v1.m01 | s, v1.m02 | s, v1.m03 | s, v1.m10 | s, v1.m11 | s, v1.m12 | s, v1.m13 | s, v1.m20 | s, v1.m21 | s, v1.m22 | s, v1.m23 | s, v1.m30 | s, v1.m31 | s, v1.m32 | s, v1.m33 | s);
        }
        
        public static umat4 operator^(umat4 v1, uint s) {
            return new umat4(v1.m00 ^ s, v1.m01 ^ s, v1.m02 ^ s, v1.m03 ^ s, v1.m10 ^ s, v1.m11 ^ s, v1.m12 ^ s, v1.m13 ^ s, v1.m20 ^ s, v1.m21 ^ s, v1.m22 ^ s, v1.m23 ^ s, v1.m30 ^ s, v1.m31 ^ s, v1.m32 ^ s, v1.m33 ^ s);
        }
        
        public static umat4 operator+(umat4 v1, umat4 v2) {
            return new umat4(v1.m00 + v2.m00, v1.m01 + v2.m01, v1.m02 + v2.m02, v1.m03 + v2.m03, v1.m10 + v2.m10, v1.m11 + v2.m11, v1.m12 + v2.m12, v1.m13 + v2.m13, v1.m20 + v2.m20, v1.m21 + v2.m21, v1.m22 + v2.m22, v1.m23 + v2.m23, v1.m30 + v2.m30, v1.m31 + v2.m31, v1.m32 + v2.m32, v1.m33 + v2.m33);
        }
        
        public static umat4 operator-(umat4 v1, umat4 v2) {
            return new umat4(v1.m00 - v2.m00, v1.m01 - v2.m01, v1.m02 - v2.m02, v1.m03 - v2.m03, v1.m10 - v2.m10, v1.m11 - v2.m11, v1.m12 - v2.m12, v1.m13 - v2.m13, v1.m20 - v2.m20, v1.m21 - v2.m21, v1.m22 - v2.m22, v1.m23 - v2.m23, v1.m30 - v2.m30, v1.m31 - v2.m31, v1.m32 - v2.m32, v1.m33 - v2.m33);
        }
        
        public static umat4 operator/(umat4 v1, umat4 v2) {
            return new umat4(v1.m00 / v2.m00, v1.m01 / v2.m01, v1.m02 / v2.m02, v1.m03 / v2.m03, v1.m10 / v2.m10, v1.m11 / v2.m11, v1.m12 / v2.m12, v1.m13 / v2.m13, v1.m20 / v2.m20, v1.m21 / v2.m21, v1.m22 / v2.m22, v1.m23 / v2.m23, v1.m30 / v2.m30, v1.m31 / v2.m31, v1.m32 / v2.m32, v1.m33 / v2.m33);
        }
        
        public static umat4 operator%(umat4 v1, umat4 v2) {
            return new umat4(v1.m00 % v2.m00, v1.m01 % v2.m01, v1.m02 % v2.m02, v1.m03 % v2.m03, v1.m10 % v2.m10, v1.m11 % v2.m11, v1.m12 % v2.m12, v1.m13 % v2.m13, v1.m20 % v2.m20, v1.m21 % v2.m21, v1.m22 % v2.m22, v1.m23 % v2.m23, v1.m30 % v2.m30, v1.m31 % v2.m31, v1.m32 % v2.m32, v1.m33 % v2.m33);
        }
        
        public static umat4 operator&(umat4 v1, umat4 v2) {
            return new umat4(v1.m00 & v2.m00, v1.m01 & v2.m01, v1.m02 & v2.m02, v1.m03 & v2.m03, v1.m10 & v2.m10, v1.m11 & v2.m11, v1.m12 & v2.m12, v1.m13 & v2.m13, v1.m20 & v2.m20, v1.m21 & v2.m21, v1.m22 & v2.m22, v1.m23 & v2.m23, v1.m30 & v2.m30, v1.m31 & v2.m31, v1.m32 & v2.m32, v1.m33 & v2.m33);
        }
        
        public static umat4 operator|(umat4 v1, umat4 v2) {
            return new umat4(v1.m00 | v2.m00, v1.m01 | v2.m01, v1.m02 | v2.m02, v1.m03 | v2.m03, v1.m10 | v2.m10, v1.m11 | v2.m11, v1.m12 | v2.m12, v1.m13 | v2.m13, v1.m20 | v2.m20, v1.m21 | v2.m21, v1.m22 | v2.m22, v1.m23 | v2.m23, v1.m30 | v2.m30, v1.m31 | v2.m31, v1.m32 | v2.m32, v1.m33 | v2.m33);
        }
        
        public static umat4 operator^(umat4 v1, umat4 v2) {
            return new umat4(v1.m00 ^ v2.m00, v1.m01 ^ v2.m01, v1.m02 ^ v2.m02, v1.m03 ^ v2.m03, v1.m10 ^ v2.m10, v1.m11 ^ v2.m11, v1.m12 ^ v2.m12, v1.m13 ^ v2.m13, v1.m20 ^ v2.m20, v1.m21 ^ v2.m21, v1.m22 ^ v2.m22, v1.m23 ^ v2.m23, v1.m30 ^ v2.m30, v1.m31 ^ v2.m31, v1.m32 ^ v2.m32, v1.m33 ^ v2.m33);
        }
        
        public static bmat4 operator==(umat4 v1, umat4 v2) {
            return new bmat4(v1.m00 == v2.m00, v1.m01 == v2.m01, v1.m02 == v2.m02, v1.m03 == v2.m03, v1.m10 == v2.m10, v1.m11 == v2.m11, v1.m12 == v2.m12, v1.m13 == v2.m13, v1.m20 == v2.m20, v1.m21 == v2.m21, v1.m22 == v2.m22, v1.m23 == v2.m23, v1.m30 == v2.m30, v1.m31 == v2.m31, v1.m32 == v2.m32, v1.m33 == v2.m33);
        }
        
        public static bmat4 operator>(umat4 v1, umat4 v2) {
            return new bmat4(v1.m00 > v2.m00, v1.m01 > v2.m01, v1.m02 > v2.m02, v1.m03 > v2.m03, v1.m10 > v2.m10, v1.m11 > v2.m11, v1.m12 > v2.m12, v1.m13 > v2.m13, v1.m20 > v2.m20, v1.m21 > v2.m21, v1.m22 > v2.m22, v1.m23 > v2.m23, v1.m30 > v2.m30, v1.m31 > v2.m31, v1.m32 > v2.m32, v1.m33 > v2.m33);
        }
        
        public static bmat4 operator>=(umat4 v1, umat4 v2) {
            return new bmat4(v1.m00 >= v2.m00, v1.m01 >= v2.m01, v1.m02 >= v2.m02, v1.m03 >= v2.m03, v1.m10 >= v2.m10, v1.m11 >= v2.m11, v1.m12 >= v2.m12, v1.m13 >= v2.m13, v1.m20 >= v2.m20, v1.m21 >= v2.m21, v1.m22 >= v2.m22, v1.m23 >= v2.m23, v1.m30 >= v2.m30, v1.m31 >= v2.m31, v1.m32 >= v2.m32, v1.m33 >= v2.m33);
        }
        
        public static bmat4 operator<(umat4 v1, umat4 v2) {
            return new bmat4(v1.m00 < v2.m00, v1.m01 < v2.m01, v1.m02 < v2.m02, v1.m03 < v2.m03, v1.m10 < v2.m10, v1.m11 < v2.m11, v1.m12 < v2.m12, v1.m13 < v2.m13, v1.m20 < v2.m20, v1.m21 < v2.m21, v1.m22 < v2.m22, v1.m23 < v2.m23, v1.m30 < v2.m30, v1.m31 < v2.m31, v1.m32 < v2.m32, v1.m33 < v2.m33);
        }
        
        public static bmat4 operator<=(umat4 v1, umat4 v2) {
            return new bmat4(v1.m00 <= v2.m00, v1.m01 <= v2.m01, v1.m02 <= v2.m02, v1.m03 <= v2.m03, v1.m10 <= v2.m10, v1.m11 <= v2.m11, v1.m12 <= v2.m12, v1.m13 <= v2.m13, v1.m20 <= v2.m20, v1.m21 <= v2.m21, v1.m22 <= v2.m22, v1.m23 <= v2.m23, v1.m30 <= v2.m30, v1.m31 <= v2.m31, v1.m32 <= v2.m32, v1.m33 <= v2.m33);
        }
        
        public static bmat4 operator!=(umat4 v1, umat4 v2) {
            return new bmat4(v1.m00 != v2.m00, v1.m01 != v2.m01, v1.m02 != v2.m02, v1.m03 != v2.m03, v1.m10 != v2.m10, v1.m11 != v2.m11, v1.m12 != v2.m12, v1.m13 != v2.m13, v1.m20 != v2.m20, v1.m21 != v2.m21, v1.m22 != v2.m22, v1.m23 != v2.m23, v1.m30 != v2.m30, v1.m31 != v2.m31, v1.m32 != v2.m32, v1.m33 != v2.m33);
        }
        
        public static uvec4 operator*(uvec4 v0, umat4 m1) {
            return umath.InnerProduct1(ref v0, ref m1);
        }
        
        public static uvec4 operator*(umat4 m0, uvec4 v1) {
            return umath.InnerProduct1(ref m0, ref v1);
        }
        
        public static umat4x2 operator*(umat4 m0, umat4x2 m1) {
            return umath.InnerProduct1(ref m0, ref m1);
        }
        
        public static umat4x3 operator*(umat4 m0, umat4x3 m1) {
            return umath.InnerProduct1(ref m0, ref m1);
        }
        
        public static umat4 operator*(umat4 m0, umat4 m1) {
            return umath.InnerProduct1(ref m0, ref m1);
        }
        
        public override bool Equals(object obj) {
            return (obj is umat4) ? bmath.All(this == (umat4)obj) : false;
        }
        
        public override int GetHashCode() {
            return this.m00.GetHashCode() ^ this.m01.GetHashCode() ^ this.m02.GetHashCode() ^ this.m03.GetHashCode() ^ this.m10.GetHashCode() ^ this.m11.GetHashCode() ^ this.m12.GetHashCode() ^ this.m13.GetHashCode() ^ this.m20.GetHashCode() ^ this.m21.GetHashCode() ^ this.m22.GetHashCode() ^ this.m23.GetHashCode() ^ this.m30.GetHashCode() ^ this.m31.GetHashCode() ^ this.m32.GetHashCode() ^ this.m33.GetHashCode();
        }
        
        public override string ToString() {
            return "umat4(" + this.m00 + "," + this.m01 + "," + this.m02 + "," + this.m03 + "," + this.m10 + "," + this.m11 + "," + this.m12 + "," + this.m13 + "," + this.m20 + "," + this.m21 + "," + this.m22 + "," + this.m23 + "," + this.m30 + "," + this.m31 + "," + this.m32 + "," + this.m33 + ")";
        }
    }
}
