#region --- License ---
/*
Copyright 2010 Daniel Filonik

Permission is hereby granted, free of charge, to any person obtaining a copy of
this software and associated documentation files (the "Software"), to deal in
the Software without restriction, including without limitation the rights to
use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
of the Software, and to permit persons to whom the Software is furnished to do
so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all
copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
SOFTWARE.
*/
#endregion

namespace TKM {
    using System;
    using System.Runtime.InteropServices;
    using System.Runtime.Serialization;
    
    
    [DataContract()]
    [StructLayout(LayoutKind.Sequential)]
    public struct uvec2 {
        
        [DataMember()]
        internal uint v0;
        
        [DataMember()]
        internal uint v1;
        
        public uvec2(uint s) {
            this.v0 = s;
            this.v1 = s;
        }
        
        public uvec2(uvec2 v0) {
            this.v0 = v0.v0;
            this.v1 = v0.v1;
        }
        
        public uvec2(uint v0, uint v1) {
            this.v0 = v0;
            this.v1 = v1;
        }
        
        public uvec2(uvec3 s) {
            this.v0 = s.v0;
            this.v1 = s.v1;
        }
        
        public uvec2(uvec4 s) {
            this.v0 = s.v0;
            this.v1 = s.v1;
        }
        
        public uint this[int i0] {
            get {
                if (((i0 < 0) 
                            || (1 < i0))) {
                    throw new System.IndexOutOfRangeException("i0");
                }
                
                unsafe
                {
                    fixed (UInt32* v_ptr = &this.v0)
                    {
                        return *(v_ptr + (i0));
                    }
                };
            }
            set {
                if (((i0 < 0) 
                            || (1 < i0))) {
                    throw new System.IndexOutOfRangeException("i0");
                }
                
                unsafe
                {
                    fixed (UInt32* v_ptr = &this.v0)
                    {
                        *(v_ptr  + (i0)) = value;
                    }
                };
            }
        }
        
        public uint x {
            get {
                return this.v0;
            }
            set {
                this.v0 = value;
            }
        }
        
        public uint r {
            get {
                return this.v0;
            }
            set {
                this.v0 = value;
            }
        }
        
        public uint s {
            get {
                return this.v0;
            }
            set {
                this.v0 = value;
            }
        }
        
        public uint y {
            get {
                return this.v1;
            }
            set {
                this.v1 = value;
            }
        }
        
        public uint g {
            get {
                return this.v1;
            }
            set {
                this.v1 = value;
            }
        }
        
        public uint t {
            get {
                return this.v1;
            }
            set {
                this.v1 = value;
            }
        }
        
        public uvec2 xx {
            get {
                return new uvec2(this.v0, this.v0);
            }
        }
        
        public uvec2 rr {
            get {
                return new uvec2(this.v0, this.v0);
            }
        }
        
        public uvec2 ss {
            get {
                return new uvec2(this.v0, this.v0);
            }
        }
        
        public uvec2 xy {
            get {
                return new uvec2(this.v0, this.v1);
            }
            set {
                this.v0 = value.v0;
                this.v1 = value.v1;
            }
        }
        
        public uvec2 rg {
            get {
                return new uvec2(this.v0, this.v1);
            }
            set {
                this.v0 = value.v0;
                this.v1 = value.v1;
            }
        }
        
        public uvec2 st {
            get {
                return new uvec2(this.v0, this.v1);
            }
            set {
                this.v0 = value.v0;
                this.v1 = value.v1;
            }
        }
        
        public uvec2 yx {
            get {
                return new uvec2(this.v1, this.v0);
            }
            set {
                this.v1 = value.v0;
                this.v0 = value.v1;
            }
        }
        
        public uvec2 gr {
            get {
                return new uvec2(this.v1, this.v0);
            }
            set {
                this.v1 = value.v0;
                this.v0 = value.v1;
            }
        }
        
        public uvec2 ts {
            get {
                return new uvec2(this.v1, this.v0);
            }
            set {
                this.v1 = value.v0;
                this.v0 = value.v1;
            }
        }
        
        public uvec2 yy {
            get {
                return new uvec2(this.v1, this.v1);
            }
        }
        
        public uvec2 gg {
            get {
                return new uvec2(this.v1, this.v1);
            }
        }
        
        public uvec2 tt {
            get {
                return new uvec2(this.v1, this.v1);
            }
        }
        
        public static uvec2 operator+(uvec2 v) {
            return new uvec2(+v.v0, +v.v1);
        }
        
        public static uvec2 operator~(uvec2 v) {
            return new uvec2(~v.v0, ~v.v1);
        }
        
        public static uvec2 operator++(uvec2 v) {
            return new uvec2(++v.v0, ++v.v1);
        }
        
        public static uvec2 operator--(uvec2 v) {
            return new uvec2(--v.v0, --v.v1);
        }
        
        public static uvec2 operator+(uint s, uvec2 v2) {
            return new uvec2(s + v2.v0, s + v2.v1);
        }
        
        public static uvec2 operator-(uint s, uvec2 v2) {
            return new uvec2(s - v2.v0, s - v2.v1);
        }
        
        public static uvec2 operator*(uint s, uvec2 v2) {
            return new uvec2(s * v2.v0, s * v2.v1);
        }
        
        public static uvec2 operator/(uint s, uvec2 v2) {
            return new uvec2(s / v2.v0, s / v2.v1);
        }
        
        public static uvec2 operator%(uint s, uvec2 v2) {
            return new uvec2(s % v2.v0, s % v2.v1);
        }
        
        public static uvec2 operator&(uint s, uvec2 v2) {
            return new uvec2(s & v2.v0, s & v2.v1);
        }
        
        public static uvec2 operator|(uint s, uvec2 v2) {
            return new uvec2(s | v2.v0, s | v2.v1);
        }
        
        public static uvec2 operator^(uint s, uvec2 v2) {
            return new uvec2(s ^ v2.v0, s ^ v2.v1);
        }
        
        public static uvec2 operator+(uvec2 v1, uint s) {
            return new uvec2(v1.v0 + s, v1.v1 + s);
        }
        
        public static uvec2 operator-(uvec2 v1, uint s) {
            return new uvec2(v1.v0 - s, v1.v1 - s);
        }
        
        public static uvec2 operator*(uvec2 v1, uint s) {
            return new uvec2(v1.v0 * s, v1.v1 * s);
        }
        
        public static uvec2 operator/(uvec2 v1, uint s) {
            return new uvec2(v1.v0 / s, v1.v1 / s);
        }
        
        public static uvec2 operator%(uvec2 v1, uint s) {
            return new uvec2(v1.v0 % s, v1.v1 % s);
        }
        
        public static uvec2 operator&(uvec2 v1, uint s) {
            return new uvec2(v1.v0 & s, v1.v1 & s);
        }
        
        public static uvec2 operator|(uvec2 v1, uint s) {
            return new uvec2(v1.v0 | s, v1.v1 | s);
        }
        
        public static uvec2 operator^(uvec2 v1, uint s) {
            return new uvec2(v1.v0 ^ s, v1.v1 ^ s);
        }
        
        public static uvec2 operator+(uvec2 v1, uvec2 v2) {
            return new uvec2(v1.v0 + v2.v0, v1.v1 + v2.v1);
        }
        
        public static uvec2 operator-(uvec2 v1, uvec2 v2) {
            return new uvec2(v1.v0 - v2.v0, v1.v1 - v2.v1);
        }
        
        public static uvec2 operator*(uvec2 v1, uvec2 v2) {
            return new uvec2(v1.v0 * v2.v0, v1.v1 * v2.v1);
        }
        
        public static uvec2 operator/(uvec2 v1, uvec2 v2) {
            return new uvec2(v1.v0 / v2.v0, v1.v1 / v2.v1);
        }
        
        public static uvec2 operator%(uvec2 v1, uvec2 v2) {
            return new uvec2(v1.v0 % v2.v0, v1.v1 % v2.v1);
        }
        
        public static uvec2 operator&(uvec2 v1, uvec2 v2) {
            return new uvec2(v1.v0 & v2.v0, v1.v1 & v2.v1);
        }
        
        public static uvec2 operator|(uvec2 v1, uvec2 v2) {
            return new uvec2(v1.v0 | v2.v0, v1.v1 | v2.v1);
        }
        
        public static uvec2 operator^(uvec2 v1, uvec2 v2) {
            return new uvec2(v1.v0 ^ v2.v0, v1.v1 ^ v2.v1);
        }
        
        public static bvec2 operator==(uvec2 v1, uvec2 v2) {
            return new bvec2(v1.v0 == v2.v0, v1.v1 == v2.v1);
        }
        
        public static bvec2 operator>(uvec2 v1, uvec2 v2) {
            return new bvec2(v1.v0 > v2.v0, v1.v1 > v2.v1);
        }
        
        public static bvec2 operator>=(uvec2 v1, uvec2 v2) {
            return new bvec2(v1.v0 >= v2.v0, v1.v1 >= v2.v1);
        }
        
        public static bvec2 operator<(uvec2 v1, uvec2 v2) {
            return new bvec2(v1.v0 < v2.v0, v1.v1 < v2.v1);
        }
        
        public static bvec2 operator<=(uvec2 v1, uvec2 v2) {
            return new bvec2(v1.v0 <= v2.v0, v1.v1 <= v2.v1);
        }
        
        public static bvec2 operator!=(uvec2 v1, uvec2 v2) {
            return new bvec2(v1.v0 != v2.v0, v1.v1 != v2.v1);
        }
        
        public override bool Equals(object obj) {
            return (obj is uvec2) ? bmath.All(this == (uvec2)obj) : false;
        }
        
        public override int GetHashCode() {
            return this.v0.GetHashCode() ^ this.v1.GetHashCode();
        }
        
        public override string ToString() {
            return "uvec2(" + this.v0 + "," + this.v1 + ")";
        }
    }
    
    [DataContract()]
    [StructLayout(LayoutKind.Sequential)]
    public struct uvec3 {
        
        [DataMember()]
        internal uint v0;
        
        [DataMember()]
        internal uint v1;
        
        [DataMember()]
        internal uint v2;
        
        public uvec3(uint s) {
            this.v0 = s;
            this.v1 = s;
            this.v2 = s;
        }
        
        public uvec3(uvec3 v0) {
            this.v0 = v0.v0;
            this.v1 = v0.v1;
            this.v2 = v0.v2;
        }
        
        public uvec3(uvec2 v0, uint v1) {
            this.v0 = v0.v0;
            this.v1 = v0.v1;
            this.v2 = v1;
        }
        
        public uvec3(uint v0, uvec2 v1) {
            this.v0 = v0;
            this.v1 = v1.v0;
            this.v2 = v1.v1;
        }
        
        public uvec3(uint v0, uint v1, uint v2) {
            this.v0 = v0;
            this.v1 = v1;
            this.v2 = v2;
        }
        
        public uvec3(uvec2 s) {
            this.v0 = s.v0;
            this.v1 = s.v1;
            this.v2 = 0u;
        }
        
        public uvec3(uvec4 s) {
            this.v0 = s.v0;
            this.v1 = s.v1;
            this.v2 = s.v2;
        }
        
        public uint this[int i0] {
            get {
                if (((i0 < 0) 
                            || (2 < i0))) {
                    throw new System.IndexOutOfRangeException("i0");
                }
                
                unsafe
                {
                    fixed (UInt32* v_ptr = &this.v0)
                    {
                        return *(v_ptr + (i0));
                    }
                };
            }
            set {
                if (((i0 < 0) 
                            || (2 < i0))) {
                    throw new System.IndexOutOfRangeException("i0");
                }
                
                unsafe
                {
                    fixed (UInt32* v_ptr = &this.v0)
                    {
                        *(v_ptr  + (i0)) = value;
                    }
                };
            }
        }
        
        public uint x {
            get {
                return this.v0;
            }
            set {
                this.v0 = value;
            }
        }
        
        public uint r {
            get {
                return this.v0;
            }
            set {
                this.v0 = value;
            }
        }
        
        public uint s {
            get {
                return this.v0;
            }
            set {
                this.v0 = value;
            }
        }
        
        public uint y {
            get {
                return this.v1;
            }
            set {
                this.v1 = value;
            }
        }
        
        public uint g {
            get {
                return this.v1;
            }
            set {
                this.v1 = value;
            }
        }
        
        public uint t {
            get {
                return this.v1;
            }
            set {
                this.v1 = value;
            }
        }
        
        public uint z {
            get {
                return this.v2;
            }
            set {
                this.v2 = value;
            }
        }
        
        public uint b {
            get {
                return this.v2;
            }
            set {
                this.v2 = value;
            }
        }
        
        public uint p {
            get {
                return this.v2;
            }
            set {
                this.v2 = value;
            }
        }
        
        public uvec2 xx {
            get {
                return new uvec2(this.v0, this.v0);
            }
        }
        
        public uvec2 rr {
            get {
                return new uvec2(this.v0, this.v0);
            }
        }
        
        public uvec2 ss {
            get {
                return new uvec2(this.v0, this.v0);
            }
        }
        
        public uvec2 xy {
            get {
                return new uvec2(this.v0, this.v1);
            }
            set {
                this.v0 = value.v0;
                this.v1 = value.v1;
            }
        }
        
        public uvec2 rg {
            get {
                return new uvec2(this.v0, this.v1);
            }
            set {
                this.v0 = value.v0;
                this.v1 = value.v1;
            }
        }
        
        public uvec2 st {
            get {
                return new uvec2(this.v0, this.v1);
            }
            set {
                this.v0 = value.v0;
                this.v1 = value.v1;
            }
        }
        
        public uvec2 xz {
            get {
                return new uvec2(this.v0, this.v2);
            }
            set {
                this.v0 = value.v0;
                this.v2 = value.v1;
            }
        }
        
        public uvec2 rb {
            get {
                return new uvec2(this.v0, this.v2);
            }
            set {
                this.v0 = value.v0;
                this.v2 = value.v1;
            }
        }
        
        public uvec2 sp {
            get {
                return new uvec2(this.v0, this.v2);
            }
            set {
                this.v0 = value.v0;
                this.v2 = value.v1;
            }
        }
        
        public uvec2 yx {
            get {
                return new uvec2(this.v1, this.v0);
            }
            set {
                this.v1 = value.v0;
                this.v0 = value.v1;
            }
        }
        
        public uvec2 gr {
            get {
                return new uvec2(this.v1, this.v0);
            }
            set {
                this.v1 = value.v0;
                this.v0 = value.v1;
            }
        }
        
        public uvec2 ts {
            get {
                return new uvec2(this.v1, this.v0);
            }
            set {
                this.v1 = value.v0;
                this.v0 = value.v1;
            }
        }
        
        public uvec2 yy {
            get {
                return new uvec2(this.v1, this.v1);
            }
        }
        
        public uvec2 gg {
            get {
                return new uvec2(this.v1, this.v1);
            }
        }
        
        public uvec2 tt {
            get {
                return new uvec2(this.v1, this.v1);
            }
        }
        
        public uvec2 yz {
            get {
                return new uvec2(this.v1, this.v2);
            }
            set {
                this.v1 = value.v0;
                this.v2 = value.v1;
            }
        }
        
        public uvec2 gb {
            get {
                return new uvec2(this.v1, this.v2);
            }
            set {
                this.v1 = value.v0;
                this.v2 = value.v1;
            }
        }
        
        public uvec2 tp {
            get {
                return new uvec2(this.v1, this.v2);
            }
            set {
                this.v1 = value.v0;
                this.v2 = value.v1;
            }
        }
        
        public uvec2 zx {
            get {
                return new uvec2(this.v2, this.v0);
            }
            set {
                this.v2 = value.v0;
                this.v0 = value.v1;
            }
        }
        
        public uvec2 br {
            get {
                return new uvec2(this.v2, this.v0);
            }
            set {
                this.v2 = value.v0;
                this.v0 = value.v1;
            }
        }
        
        public uvec2 ps {
            get {
                return new uvec2(this.v2, this.v0);
            }
            set {
                this.v2 = value.v0;
                this.v0 = value.v1;
            }
        }
        
        public uvec2 zy {
            get {
                return new uvec2(this.v2, this.v1);
            }
            set {
                this.v2 = value.v0;
                this.v1 = value.v1;
            }
        }
        
        public uvec2 bg {
            get {
                return new uvec2(this.v2, this.v1);
            }
            set {
                this.v2 = value.v0;
                this.v1 = value.v1;
            }
        }
        
        public uvec2 pt {
            get {
                return new uvec2(this.v2, this.v1);
            }
            set {
                this.v2 = value.v0;
                this.v1 = value.v1;
            }
        }
        
        public uvec2 zz {
            get {
                return new uvec2(this.v2, this.v2);
            }
        }
        
        public uvec2 bb {
            get {
                return new uvec2(this.v2, this.v2);
            }
        }
        
        public uvec2 pp {
            get {
                return new uvec2(this.v2, this.v2);
            }
        }
        
        public uvec3 xxx {
            get {
                return new uvec3(this.v0, this.v0, this.v0);
            }
        }
        
        public uvec3 rrr {
            get {
                return new uvec3(this.v0, this.v0, this.v0);
            }
        }
        
        public uvec3 sss {
            get {
                return new uvec3(this.v0, this.v0, this.v0);
            }
        }
        
        public uvec3 xxy {
            get {
                return new uvec3(this.v0, this.v0, this.v1);
            }
        }
        
        public uvec3 rrg {
            get {
                return new uvec3(this.v0, this.v0, this.v1);
            }
        }
        
        public uvec3 sst {
            get {
                return new uvec3(this.v0, this.v0, this.v1);
            }
        }
        
        public uvec3 xxz {
            get {
                return new uvec3(this.v0, this.v0, this.v2);
            }
        }
        
        public uvec3 rrb {
            get {
                return new uvec3(this.v0, this.v0, this.v2);
            }
        }
        
        public uvec3 ssp {
            get {
                return new uvec3(this.v0, this.v0, this.v2);
            }
        }
        
        public uvec3 xyx {
            get {
                return new uvec3(this.v0, this.v1, this.v0);
            }
        }
        
        public uvec3 rgr {
            get {
                return new uvec3(this.v0, this.v1, this.v0);
            }
        }
        
        public uvec3 sts {
            get {
                return new uvec3(this.v0, this.v1, this.v0);
            }
        }
        
        public uvec3 xyy {
            get {
                return new uvec3(this.v0, this.v1, this.v1);
            }
        }
        
        public uvec3 rgg {
            get {
                return new uvec3(this.v0, this.v1, this.v1);
            }
        }
        
        public uvec3 stt {
            get {
                return new uvec3(this.v0, this.v1, this.v1);
            }
        }
        
        public uvec3 xyz {
            get {
                return new uvec3(this.v0, this.v1, this.v2);
            }
            set {
                this.v0 = value.v0;
                this.v1 = value.v1;
                this.v2 = value.v2;
            }
        }
        
        public uvec3 rgb {
            get {
                return new uvec3(this.v0, this.v1, this.v2);
            }
            set {
                this.v0 = value.v0;
                this.v1 = value.v1;
                this.v2 = value.v2;
            }
        }
        
        public uvec3 stp {
            get {
                return new uvec3(this.v0, this.v1, this.v2);
            }
            set {
                this.v0 = value.v0;
                this.v1 = value.v1;
                this.v2 = value.v2;
            }
        }
        
        public uvec3 xzx {
            get {
                return new uvec3(this.v0, this.v2, this.v0);
            }
        }
        
        public uvec3 rbr {
            get {
                return new uvec3(this.v0, this.v2, this.v0);
            }
        }
        
        public uvec3 sps {
            get {
                return new uvec3(this.v0, this.v2, this.v0);
            }
        }
        
        public uvec3 xzy {
            get {
                return new uvec3(this.v0, this.v2, this.v1);
            }
            set {
                this.v0 = value.v0;
                this.v2 = value.v1;
                this.v1 = value.v2;
            }
        }
        
        public uvec3 rbg {
            get {
                return new uvec3(this.v0, this.v2, this.v1);
            }
            set {
                this.v0 = value.v0;
                this.v2 = value.v1;
                this.v1 = value.v2;
            }
        }
        
        public uvec3 spt {
            get {
                return new uvec3(this.v0, this.v2, this.v1);
            }
            set {
                this.v0 = value.v0;
                this.v2 = value.v1;
                this.v1 = value.v2;
            }
        }
        
        public uvec3 xzz {
            get {
                return new uvec3(this.v0, this.v2, this.v2);
            }
        }
        
        public uvec3 rbb {
            get {
                return new uvec3(this.v0, this.v2, this.v2);
            }
        }
        
        public uvec3 spp {
            get {
                return new uvec3(this.v0, this.v2, this.v2);
            }
        }
        
        public uvec3 yxx {
            get {
                return new uvec3(this.v1, this.v0, this.v0);
            }
        }
        
        public uvec3 grr {
            get {
                return new uvec3(this.v1, this.v0, this.v0);
            }
        }
        
        public uvec3 tss {
            get {
                return new uvec3(this.v1, this.v0, this.v0);
            }
        }
        
        public uvec3 yxy {
            get {
                return new uvec3(this.v1, this.v0, this.v1);
            }
        }
        
        public uvec3 grg {
            get {
                return new uvec3(this.v1, this.v0, this.v1);
            }
        }
        
        public uvec3 tst {
            get {
                return new uvec3(this.v1, this.v0, this.v1);
            }
        }
        
        public uvec3 yxz {
            get {
                return new uvec3(this.v1, this.v0, this.v2);
            }
            set {
                this.v1 = value.v0;
                this.v0 = value.v1;
                this.v2 = value.v2;
            }
        }
        
        public uvec3 grb {
            get {
                return new uvec3(this.v1, this.v0, this.v2);
            }
            set {
                this.v1 = value.v0;
                this.v0 = value.v1;
                this.v2 = value.v2;
            }
        }
        
        public uvec3 tsp {
            get {
                return new uvec3(this.v1, this.v0, this.v2);
            }
            set {
                this.v1 = value.v0;
                this.v0 = value.v1;
                this.v2 = value.v2;
            }
        }
        
        public uvec3 yyx {
            get {
                return new uvec3(this.v1, this.v1, this.v0);
            }
        }
        
        public uvec3 ggr {
            get {
                return new uvec3(this.v1, this.v1, this.v0);
            }
        }
        
        public uvec3 tts {
            get {
                return new uvec3(this.v1, this.v1, this.v0);
            }
        }
        
        public uvec3 yyy {
            get {
                return new uvec3(this.v1, this.v1, this.v1);
            }
        }
        
        public uvec3 ggg {
            get {
                return new uvec3(this.v1, this.v1, this.v1);
            }
        }
        
        public uvec3 ttt {
            get {
                return new uvec3(this.v1, this.v1, this.v1);
            }
        }
        
        public uvec3 yyz {
            get {
                return new uvec3(this.v1, this.v1, this.v2);
            }
        }
        
        public uvec3 ggb {
            get {
                return new uvec3(this.v1, this.v1, this.v2);
            }
        }
        
        public uvec3 ttp {
            get {
                return new uvec3(this.v1, this.v1, this.v2);
            }
        }
        
        public uvec3 yzx {
            get {
                return new uvec3(this.v1, this.v2, this.v0);
            }
            set {
                this.v1 = value.v0;
                this.v2 = value.v1;
                this.v0 = value.v2;
            }
        }
        
        public uvec3 gbr {
            get {
                return new uvec3(this.v1, this.v2, this.v0);
            }
            set {
                this.v1 = value.v0;
                this.v2 = value.v1;
                this.v0 = value.v2;
            }
        }
        
        public uvec3 tps {
            get {
                return new uvec3(this.v1, this.v2, this.v0);
            }
            set {
                this.v1 = value.v0;
                this.v2 = value.v1;
                this.v0 = value.v2;
            }
        }
        
        public uvec3 yzy {
            get {
                return new uvec3(this.v1, this.v2, this.v1);
            }
        }
        
        public uvec3 gbg {
            get {
                return new uvec3(this.v1, this.v2, this.v1);
            }
        }
        
        public uvec3 tpt {
            get {
                return new uvec3(this.v1, this.v2, this.v1);
            }
        }
        
        public uvec3 yzz {
            get {
                return new uvec3(this.v1, this.v2, this.v2);
            }
        }
        
        public uvec3 gbb {
            get {
                return new uvec3(this.v1, this.v2, this.v2);
            }
        }
        
        public uvec3 tpp {
            get {
                return new uvec3(this.v1, this.v2, this.v2);
            }
        }
        
        public uvec3 zxx {
            get {
                return new uvec3(this.v2, this.v0, this.v0);
            }
        }
        
        public uvec3 brr {
            get {
                return new uvec3(this.v2, this.v0, this.v0);
            }
        }
        
        public uvec3 pss {
            get {
                return new uvec3(this.v2, this.v0, this.v0);
            }
        }
        
        public uvec3 zxy {
            get {
                return new uvec3(this.v2, this.v0, this.v1);
            }
            set {
                this.v2 = value.v0;
                this.v0 = value.v1;
                this.v1 = value.v2;
            }
        }
        
        public uvec3 brg {
            get {
                return new uvec3(this.v2, this.v0, this.v1);
            }
            set {
                this.v2 = value.v0;
                this.v0 = value.v1;
                this.v1 = value.v2;
            }
        }
        
        public uvec3 pst {
            get {
                return new uvec3(this.v2, this.v0, this.v1);
            }
            set {
                this.v2 = value.v0;
                this.v0 = value.v1;
                this.v1 = value.v2;
            }
        }
        
        public uvec3 zxz {
            get {
                return new uvec3(this.v2, this.v0, this.v2);
            }
        }
        
        public uvec3 brb {
            get {
                return new uvec3(this.v2, this.v0, this.v2);
            }
        }
        
        public uvec3 psp {
            get {
                return new uvec3(this.v2, this.v0, this.v2);
            }
        }
        
        public uvec3 zyx {
            get {
                return new uvec3(this.v2, this.v1, this.v0);
            }
            set {
                this.v2 = value.v0;
                this.v1 = value.v1;
                this.v0 = value.v2;
            }
        }
        
        public uvec3 bgr {
            get {
                return new uvec3(this.v2, this.v1, this.v0);
            }
            set {
                this.v2 = value.v0;
                this.v1 = value.v1;
                this.v0 = value.v2;
            }
        }
        
        public uvec3 pts {
            get {
                return new uvec3(this.v2, this.v1, this.v0);
            }
            set {
                this.v2 = value.v0;
                this.v1 = value.v1;
                this.v0 = value.v2;
            }
        }
        
        public uvec3 zyy {
            get {
                return new uvec3(this.v2, this.v1, this.v1);
            }
        }
        
        public uvec3 bgg {
            get {
                return new uvec3(this.v2, this.v1, this.v1);
            }
        }
        
        public uvec3 ptt {
            get {
                return new uvec3(this.v2, this.v1, this.v1);
            }
        }
        
        public uvec3 zyz {
            get {
                return new uvec3(this.v2, this.v1, this.v2);
            }
        }
        
        public uvec3 bgb {
            get {
                return new uvec3(this.v2, this.v1, this.v2);
            }
        }
        
        public uvec3 ptp {
            get {
                return new uvec3(this.v2, this.v1, this.v2);
            }
        }
        
        public uvec3 zzx {
            get {
                return new uvec3(this.v2, this.v2, this.v0);
            }
        }
        
        public uvec3 bbr {
            get {
                return new uvec3(this.v2, this.v2, this.v0);
            }
        }
        
        public uvec3 pps {
            get {
                return new uvec3(this.v2, this.v2, this.v0);
            }
        }
        
        public uvec3 zzy {
            get {
                return new uvec3(this.v2, this.v2, this.v1);
            }
        }
        
        public uvec3 bbg {
            get {
                return new uvec3(this.v2, this.v2, this.v1);
            }
        }
        
        public uvec3 ppt {
            get {
                return new uvec3(this.v2, this.v2, this.v1);
            }
        }
        
        public uvec3 zzz {
            get {
                return new uvec3(this.v2, this.v2, this.v2);
            }
        }
        
        public uvec3 bbb {
            get {
                return new uvec3(this.v2, this.v2, this.v2);
            }
        }
        
        public uvec3 ppp {
            get {
                return new uvec3(this.v2, this.v2, this.v2);
            }
        }
        
        public static uvec3 operator+(uvec3 v) {
            return new uvec3(+v.v0, +v.v1, +v.v2);
        }
        
        public static uvec3 operator~(uvec3 v) {
            return new uvec3(~v.v0, ~v.v1, ~v.v2);
        }
        
        public static uvec3 operator++(uvec3 v) {
            return new uvec3(++v.v0, ++v.v1, ++v.v2);
        }
        
        public static uvec3 operator--(uvec3 v) {
            return new uvec3(--v.v0, --v.v1, --v.v2);
        }
        
        public static uvec3 operator+(uint s, uvec3 v2) {
            return new uvec3(s + v2.v0, s + v2.v1, s + v2.v2);
        }
        
        public static uvec3 operator-(uint s, uvec3 v2) {
            return new uvec3(s - v2.v0, s - v2.v1, s - v2.v2);
        }
        
        public static uvec3 operator*(uint s, uvec3 v2) {
            return new uvec3(s * v2.v0, s * v2.v1, s * v2.v2);
        }
        
        public static uvec3 operator/(uint s, uvec3 v2) {
            return new uvec3(s / v2.v0, s / v2.v1, s / v2.v2);
        }
        
        public static uvec3 operator%(uint s, uvec3 v2) {
            return new uvec3(s % v2.v0, s % v2.v1, s % v2.v2);
        }
        
        public static uvec3 operator&(uint s, uvec3 v2) {
            return new uvec3(s & v2.v0, s & v2.v1, s & v2.v2);
        }
        
        public static uvec3 operator|(uint s, uvec3 v2) {
            return new uvec3(s | v2.v0, s | v2.v1, s | v2.v2);
        }
        
        public static uvec3 operator^(uint s, uvec3 v2) {
            return new uvec3(s ^ v2.v0, s ^ v2.v1, s ^ v2.v2);
        }
        
        public static uvec3 operator+(uvec3 v1, uint s) {
            return new uvec3(v1.v0 + s, v1.v1 + s, v1.v2 + s);
        }
        
        public static uvec3 operator-(uvec3 v1, uint s) {
            return new uvec3(v1.v0 - s, v1.v1 - s, v1.v2 - s);
        }
        
        public static uvec3 operator*(uvec3 v1, uint s) {
            return new uvec3(v1.v0 * s, v1.v1 * s, v1.v2 * s);
        }
        
        public static uvec3 operator/(uvec3 v1, uint s) {
            return new uvec3(v1.v0 / s, v1.v1 / s, v1.v2 / s);
        }
        
        public static uvec3 operator%(uvec3 v1, uint s) {
            return new uvec3(v1.v0 % s, v1.v1 % s, v1.v2 % s);
        }
        
        public static uvec3 operator&(uvec3 v1, uint s) {
            return new uvec3(v1.v0 & s, v1.v1 & s, v1.v2 & s);
        }
        
        public static uvec3 operator|(uvec3 v1, uint s) {
            return new uvec3(v1.v0 | s, v1.v1 | s, v1.v2 | s);
        }
        
        public static uvec3 operator^(uvec3 v1, uint s) {
            return new uvec3(v1.v0 ^ s, v1.v1 ^ s, v1.v2 ^ s);
        }
        
        public static uvec3 operator+(uvec3 v1, uvec3 v2) {
            return new uvec3(v1.v0 + v2.v0, v1.v1 + v2.v1, v1.v2 + v2.v2);
        }
        
        public static uvec3 operator-(uvec3 v1, uvec3 v2) {
            return new uvec3(v1.v0 - v2.v0, v1.v1 - v2.v1, v1.v2 - v2.v2);
        }
        
        public static uvec3 operator*(uvec3 v1, uvec3 v2) {
            return new uvec3(v1.v0 * v2.v0, v1.v1 * v2.v1, v1.v2 * v2.v2);
        }
        
        public static uvec3 operator/(uvec3 v1, uvec3 v2) {
            return new uvec3(v1.v0 / v2.v0, v1.v1 / v2.v1, v1.v2 / v2.v2);
        }
        
        public static uvec3 operator%(uvec3 v1, uvec3 v2) {
            return new uvec3(v1.v0 % v2.v0, v1.v1 % v2.v1, v1.v2 % v2.v2);
        }
        
        public static uvec3 operator&(uvec3 v1, uvec3 v2) {
            return new uvec3(v1.v0 & v2.v0, v1.v1 & v2.v1, v1.v2 & v2.v2);
        }
        
        public static uvec3 operator|(uvec3 v1, uvec3 v2) {
            return new uvec3(v1.v0 | v2.v0, v1.v1 | v2.v1, v1.v2 | v2.v2);
        }
        
        public static uvec3 operator^(uvec3 v1, uvec3 v2) {
            return new uvec3(v1.v0 ^ v2.v0, v1.v1 ^ v2.v1, v1.v2 ^ v2.v2);
        }
        
        public static bvec3 operator==(uvec3 v1, uvec3 v2) {
            return new bvec3(v1.v0 == v2.v0, v1.v1 == v2.v1, v1.v2 == v2.v2);
        }
        
        public static bvec3 operator>(uvec3 v1, uvec3 v2) {
            return new bvec3(v1.v0 > v2.v0, v1.v1 > v2.v1, v1.v2 > v2.v2);
        }
        
        public static bvec3 operator>=(uvec3 v1, uvec3 v2) {
            return new bvec3(v1.v0 >= v2.v0, v1.v1 >= v2.v1, v1.v2 >= v2.v2);
        }
        
        public static bvec3 operator<(uvec3 v1, uvec3 v2) {
            return new bvec3(v1.v0 < v2.v0, v1.v1 < v2.v1, v1.v2 < v2.v2);
        }
        
        public static bvec3 operator<=(uvec3 v1, uvec3 v2) {
            return new bvec3(v1.v0 <= v2.v0, v1.v1 <= v2.v1, v1.v2 <= v2.v2);
        }
        
        public static bvec3 operator!=(uvec3 v1, uvec3 v2) {
            return new bvec3(v1.v0 != v2.v0, v1.v1 != v2.v1, v1.v2 != v2.v2);
        }
        
        public override bool Equals(object obj) {
            return (obj is uvec3) ? bmath.All(this == (uvec3)obj) : false;
        }
        
        public override int GetHashCode() {
            return this.v0.GetHashCode() ^ this.v1.GetHashCode() ^ this.v2.GetHashCode();
        }
        
        public override string ToString() {
            return "uvec3(" + this.v0 + "," + this.v1 + "," + this.v2 + ")";
        }
    }
    
    [DataContract()]
    [StructLayout(LayoutKind.Sequential)]
    public struct uvec4 {
        
        [DataMember()]
        internal uint v0;
        
        [DataMember()]
        internal uint v1;
        
        [DataMember()]
        internal uint v2;
        
        [DataMember()]
        internal uint v3;
        
        public uvec4(uint s) {
            this.v0 = s;
            this.v1 = s;
            this.v2 = s;
            this.v3 = s;
        }
        
        public uvec4(uvec4 v0) {
            this.v0 = v0.v0;
            this.v1 = v0.v1;
            this.v2 = v0.v2;
            this.v3 = v0.v3;
        }
        
        public uvec4(uvec2 v0, uvec2 v1) {
            this.v0 = v0.v0;
            this.v1 = v0.v1;
            this.v2 = v1.v0;
            this.v3 = v1.v1;
        }
        
        public uvec4(uvec3 v0, uint v1) {
            this.v0 = v0.v0;
            this.v1 = v0.v1;
            this.v2 = v0.v2;
            this.v3 = v1;
        }
        
        public uvec4(uint v0, uvec3 v1) {
            this.v0 = v0;
            this.v1 = v1.v0;
            this.v2 = v1.v1;
            this.v3 = v1.v2;
        }
        
        public uvec4(uvec2 v0, uint v1, uint v2) {
            this.v0 = v0.v0;
            this.v1 = v0.v1;
            this.v2 = v1;
            this.v3 = v2;
        }
        
        public uvec4(uint v0, uvec2 v1, uint v2) {
            this.v0 = v0;
            this.v1 = v1.v0;
            this.v2 = v1.v1;
            this.v3 = v2;
        }
        
        public uvec4(uint v0, uint v1, uvec2 v2) {
            this.v0 = v0;
            this.v1 = v1;
            this.v2 = v2.v0;
            this.v3 = v2.v1;
        }
        
        public uvec4(uint v0, uint v1, uint v2, uint v3) {
            this.v0 = v0;
            this.v1 = v1;
            this.v2 = v2;
            this.v3 = v3;
        }
        
        public uvec4(uvec2 s) {
            this.v0 = s.v0;
            this.v1 = s.v1;
            this.v2 = 0u;
            this.v3 = 0u;
        }
        
        public uvec4(uvec3 s) {
            this.v0 = s.v0;
            this.v1 = s.v1;
            this.v2 = s.v2;
            this.v3 = 0u;
        }
        
        public uint this[int i0] {
            get {
                if (((i0 < 0) 
                            || (3 < i0))) {
                    throw new System.IndexOutOfRangeException("i0");
                }
                
                unsafe
                {
                    fixed (UInt32* v_ptr = &this.v0)
                    {
                        return *(v_ptr + (i0));
                    }
                };
            }
            set {
                if (((i0 < 0) 
                            || (3 < i0))) {
                    throw new System.IndexOutOfRangeException("i0");
                }
                
                unsafe
                {
                    fixed (UInt32* v_ptr = &this.v0)
                    {
                        *(v_ptr  + (i0)) = value;
                    }
                };
            }
        }
        
        public uint x {
            get {
                return this.v0;
            }
            set {
                this.v0 = value;
            }
        }
        
        public uint r {
            get {
                return this.v0;
            }
            set {
                this.v0 = value;
            }
        }
        
        public uint s {
            get {
                return this.v0;
            }
            set {
                this.v0 = value;
            }
        }
        
        public uint y {
            get {
                return this.v1;
            }
            set {
                this.v1 = value;
            }
        }
        
        public uint g {
            get {
                return this.v1;
            }
            set {
                this.v1 = value;
            }
        }
        
        public uint t {
            get {
                return this.v1;
            }
            set {
                this.v1 = value;
            }
        }
        
        public uint z {
            get {
                return this.v2;
            }
            set {
                this.v2 = value;
            }
        }
        
        public uint b {
            get {
                return this.v2;
            }
            set {
                this.v2 = value;
            }
        }
        
        public uint p {
            get {
                return this.v2;
            }
            set {
                this.v2 = value;
            }
        }
        
        public uint w {
            get {
                return this.v3;
            }
            set {
                this.v3 = value;
            }
        }
        
        public uint a {
            get {
                return this.v3;
            }
            set {
                this.v3 = value;
            }
        }
        
        public uint q {
            get {
                return this.v3;
            }
            set {
                this.v3 = value;
            }
        }
        
        public uvec2 xx {
            get {
                return new uvec2(this.v0, this.v0);
            }
        }
        
        public uvec2 rr {
            get {
                return new uvec2(this.v0, this.v0);
            }
        }
        
        public uvec2 ss {
            get {
                return new uvec2(this.v0, this.v0);
            }
        }
        
        public uvec2 xy {
            get {
                return new uvec2(this.v0, this.v1);
            }
            set {
                this.v0 = value.v0;
                this.v1 = value.v1;
            }
        }
        
        public uvec2 rg {
            get {
                return new uvec2(this.v0, this.v1);
            }
            set {
                this.v0 = value.v0;
                this.v1 = value.v1;
            }
        }
        
        public uvec2 st {
            get {
                return new uvec2(this.v0, this.v1);
            }
            set {
                this.v0 = value.v0;
                this.v1 = value.v1;
            }
        }
        
        public uvec2 xz {
            get {
                return new uvec2(this.v0, this.v2);
            }
            set {
                this.v0 = value.v0;
                this.v2 = value.v1;
            }
        }
        
        public uvec2 rb {
            get {
                return new uvec2(this.v0, this.v2);
            }
            set {
                this.v0 = value.v0;
                this.v2 = value.v1;
            }
        }
        
        public uvec2 sp {
            get {
                return new uvec2(this.v0, this.v2);
            }
            set {
                this.v0 = value.v0;
                this.v2 = value.v1;
            }
        }
        
        public uvec2 xw {
            get {
                return new uvec2(this.v0, this.v3);
            }
            set {
                this.v0 = value.v0;
                this.v3 = value.v1;
            }
        }
        
        public uvec2 ra {
            get {
                return new uvec2(this.v0, this.v3);
            }
            set {
                this.v0 = value.v0;
                this.v3 = value.v1;
            }
        }
        
        public uvec2 sq {
            get {
                return new uvec2(this.v0, this.v3);
            }
            set {
                this.v0 = value.v0;
                this.v3 = value.v1;
            }
        }
        
        public uvec2 yx {
            get {
                return new uvec2(this.v1, this.v0);
            }
            set {
                this.v1 = value.v0;
                this.v0 = value.v1;
            }
        }
        
        public uvec2 gr {
            get {
                return new uvec2(this.v1, this.v0);
            }
            set {
                this.v1 = value.v0;
                this.v0 = value.v1;
            }
        }
        
        public uvec2 ts {
            get {
                return new uvec2(this.v1, this.v0);
            }
            set {
                this.v1 = value.v0;
                this.v0 = value.v1;
            }
        }
        
        public uvec2 yy {
            get {
                return new uvec2(this.v1, this.v1);
            }
        }
        
        public uvec2 gg {
            get {
                return new uvec2(this.v1, this.v1);
            }
        }
        
        public uvec2 tt {
            get {
                return new uvec2(this.v1, this.v1);
            }
        }
        
        public uvec2 yz {
            get {
                return new uvec2(this.v1, this.v2);
            }
            set {
                this.v1 = value.v0;
                this.v2 = value.v1;
            }
        }
        
        public uvec2 gb {
            get {
                return new uvec2(this.v1, this.v2);
            }
            set {
                this.v1 = value.v0;
                this.v2 = value.v1;
            }
        }
        
        public uvec2 tp {
            get {
                return new uvec2(this.v1, this.v2);
            }
            set {
                this.v1 = value.v0;
                this.v2 = value.v1;
            }
        }
        
        public uvec2 yw {
            get {
                return new uvec2(this.v1, this.v3);
            }
            set {
                this.v1 = value.v0;
                this.v3 = value.v1;
            }
        }
        
        public uvec2 ga {
            get {
                return new uvec2(this.v1, this.v3);
            }
            set {
                this.v1 = value.v0;
                this.v3 = value.v1;
            }
        }
        
        public uvec2 tq {
            get {
                return new uvec2(this.v1, this.v3);
            }
            set {
                this.v1 = value.v0;
                this.v3 = value.v1;
            }
        }
        
        public uvec2 zx {
            get {
                return new uvec2(this.v2, this.v0);
            }
            set {
                this.v2 = value.v0;
                this.v0 = value.v1;
            }
        }
        
        public uvec2 br {
            get {
                return new uvec2(this.v2, this.v0);
            }
            set {
                this.v2 = value.v0;
                this.v0 = value.v1;
            }
        }
        
        public uvec2 ps {
            get {
                return new uvec2(this.v2, this.v0);
            }
            set {
                this.v2 = value.v0;
                this.v0 = value.v1;
            }
        }
        
        public uvec2 zy {
            get {
                return new uvec2(this.v2, this.v1);
            }
            set {
                this.v2 = value.v0;
                this.v1 = value.v1;
            }
        }
        
        public uvec2 bg {
            get {
                return new uvec2(this.v2, this.v1);
            }
            set {
                this.v2 = value.v0;
                this.v1 = value.v1;
            }
        }
        
        public uvec2 pt {
            get {
                return new uvec2(this.v2, this.v1);
            }
            set {
                this.v2 = value.v0;
                this.v1 = value.v1;
            }
        }
        
        public uvec2 zz {
            get {
                return new uvec2(this.v2, this.v2);
            }
        }
        
        public uvec2 bb {
            get {
                return new uvec2(this.v2, this.v2);
            }
        }
        
        public uvec2 pp {
            get {
                return new uvec2(this.v2, this.v2);
            }
        }
        
        public uvec2 zw {
            get {
                return new uvec2(this.v2, this.v3);
            }
            set {
                this.v2 = value.v0;
                this.v3 = value.v1;
            }
        }
        
        public uvec2 ba {
            get {
                return new uvec2(this.v2, this.v3);
            }
            set {
                this.v2 = value.v0;
                this.v3 = value.v1;
            }
        }
        
        public uvec2 pq {
            get {
                return new uvec2(this.v2, this.v3);
            }
            set {
                this.v2 = value.v0;
                this.v3 = value.v1;
            }
        }
        
        public uvec2 wx {
            get {
                return new uvec2(this.v3, this.v0);
            }
            set {
                this.v3 = value.v0;
                this.v0 = value.v1;
            }
        }
        
        public uvec2 ar {
            get {
                return new uvec2(this.v3, this.v0);
            }
            set {
                this.v3 = value.v0;
                this.v0 = value.v1;
            }
        }
        
        public uvec2 qs {
            get {
                return new uvec2(this.v3, this.v0);
            }
            set {
                this.v3 = value.v0;
                this.v0 = value.v1;
            }
        }
        
        public uvec2 wy {
            get {
                return new uvec2(this.v3, this.v1);
            }
            set {
                this.v3 = value.v0;
                this.v1 = value.v1;
            }
        }
        
        public uvec2 ag {
            get {
                return new uvec2(this.v3, this.v1);
            }
            set {
                this.v3 = value.v0;
                this.v1 = value.v1;
            }
        }
        
        public uvec2 qt {
            get {
                return new uvec2(this.v3, this.v1);
            }
            set {
                this.v3 = value.v0;
                this.v1 = value.v1;
            }
        }
        
        public uvec2 wz {
            get {
                return new uvec2(this.v3, this.v2);
            }
            set {
                this.v3 = value.v0;
                this.v2 = value.v1;
            }
        }
        
        public uvec2 ab {
            get {
                return new uvec2(this.v3, this.v2);
            }
            set {
                this.v3 = value.v0;
                this.v2 = value.v1;
            }
        }
        
        public uvec2 qp {
            get {
                return new uvec2(this.v3, this.v2);
            }
            set {
                this.v3 = value.v0;
                this.v2 = value.v1;
            }
        }
        
        public uvec2 ww {
            get {
                return new uvec2(this.v3, this.v3);
            }
        }
        
        public uvec2 aa {
            get {
                return new uvec2(this.v3, this.v3);
            }
        }
        
        public uvec2 qq {
            get {
                return new uvec2(this.v3, this.v3);
            }
        }
        
        public uvec3 xxx {
            get {
                return new uvec3(this.v0, this.v0, this.v0);
            }
        }
        
        public uvec3 rrr {
            get {
                return new uvec3(this.v0, this.v0, this.v0);
            }
        }
        
        public uvec3 sss {
            get {
                return new uvec3(this.v0, this.v0, this.v0);
            }
        }
        
        public uvec3 xxy {
            get {
                return new uvec3(this.v0, this.v0, this.v1);
            }
        }
        
        public uvec3 rrg {
            get {
                return new uvec3(this.v0, this.v0, this.v1);
            }
        }
        
        public uvec3 sst {
            get {
                return new uvec3(this.v0, this.v0, this.v1);
            }
        }
        
        public uvec3 xxz {
            get {
                return new uvec3(this.v0, this.v0, this.v2);
            }
        }
        
        public uvec3 rrb {
            get {
                return new uvec3(this.v0, this.v0, this.v2);
            }
        }
        
        public uvec3 ssp {
            get {
                return new uvec3(this.v0, this.v0, this.v2);
            }
        }
        
        public uvec3 xxw {
            get {
                return new uvec3(this.v0, this.v0, this.v3);
            }
        }
        
        public uvec3 rra {
            get {
                return new uvec3(this.v0, this.v0, this.v3);
            }
        }
        
        public uvec3 ssq {
            get {
                return new uvec3(this.v0, this.v0, this.v3);
            }
        }
        
        public uvec3 xyx {
            get {
                return new uvec3(this.v0, this.v1, this.v0);
            }
        }
        
        public uvec3 rgr {
            get {
                return new uvec3(this.v0, this.v1, this.v0);
            }
        }
        
        public uvec3 sts {
            get {
                return new uvec3(this.v0, this.v1, this.v0);
            }
        }
        
        public uvec3 xyy {
            get {
                return new uvec3(this.v0, this.v1, this.v1);
            }
        }
        
        public uvec3 rgg {
            get {
                return new uvec3(this.v0, this.v1, this.v1);
            }
        }
        
        public uvec3 stt {
            get {
                return new uvec3(this.v0, this.v1, this.v1);
            }
        }
        
        public uvec3 xyz {
            get {
                return new uvec3(this.v0, this.v1, this.v2);
            }
            set {
                this.v0 = value.v0;
                this.v1 = value.v1;
                this.v2 = value.v2;
            }
        }
        
        public uvec3 rgb {
            get {
                return new uvec3(this.v0, this.v1, this.v2);
            }
            set {
                this.v0 = value.v0;
                this.v1 = value.v1;
                this.v2 = value.v2;
            }
        }
        
        public uvec3 stp {
            get {
                return new uvec3(this.v0, this.v1, this.v2);
            }
            set {
                this.v0 = value.v0;
                this.v1 = value.v1;
                this.v2 = value.v2;
            }
        }
        
        public uvec3 xyw {
            get {
                return new uvec3(this.v0, this.v1, this.v3);
            }
            set {
                this.v0 = value.v0;
                this.v1 = value.v1;
                this.v3 = value.v2;
            }
        }
        
        public uvec3 rga {
            get {
                return new uvec3(this.v0, this.v1, this.v3);
            }
            set {
                this.v0 = value.v0;
                this.v1 = value.v1;
                this.v3 = value.v2;
            }
        }
        
        public uvec3 stq {
            get {
                return new uvec3(this.v0, this.v1, this.v3);
            }
            set {
                this.v0 = value.v0;
                this.v1 = value.v1;
                this.v3 = value.v2;
            }
        }
        
        public uvec3 xzx {
            get {
                return new uvec3(this.v0, this.v2, this.v0);
            }
        }
        
        public uvec3 rbr {
            get {
                return new uvec3(this.v0, this.v2, this.v0);
            }
        }
        
        public uvec3 sps {
            get {
                return new uvec3(this.v0, this.v2, this.v0);
            }
        }
        
        public uvec3 xzy {
            get {
                return new uvec3(this.v0, this.v2, this.v1);
            }
            set {
                this.v0 = value.v0;
                this.v2 = value.v1;
                this.v1 = value.v2;
            }
        }
        
        public uvec3 rbg {
            get {
                return new uvec3(this.v0, this.v2, this.v1);
            }
            set {
                this.v0 = value.v0;
                this.v2 = value.v1;
                this.v1 = value.v2;
            }
        }
        
        public uvec3 spt {
            get {
                return new uvec3(this.v0, this.v2, this.v1);
            }
            set {
                this.v0 = value.v0;
                this.v2 = value.v1;
                this.v1 = value.v2;
            }
        }
        
        public uvec3 xzz {
            get {
                return new uvec3(this.v0, this.v2, this.v2);
            }
        }
        
        public uvec3 rbb {
            get {
                return new uvec3(this.v0, this.v2, this.v2);
            }
        }
        
        public uvec3 spp {
            get {
                return new uvec3(this.v0, this.v2, this.v2);
            }
        }
        
        public uvec3 xzw {
            get {
                return new uvec3(this.v0, this.v2, this.v3);
            }
            set {
                this.v0 = value.v0;
                this.v2 = value.v1;
                this.v3 = value.v2;
            }
        }
        
        public uvec3 rba {
            get {
                return new uvec3(this.v0, this.v2, this.v3);
            }
            set {
                this.v0 = value.v0;
                this.v2 = value.v1;
                this.v3 = value.v2;
            }
        }
        
        public uvec3 spq {
            get {
                return new uvec3(this.v0, this.v2, this.v3);
            }
            set {
                this.v0 = value.v0;
                this.v2 = value.v1;
                this.v3 = value.v2;
            }
        }
        
        public uvec3 xwx {
            get {
                return new uvec3(this.v0, this.v3, this.v0);
            }
        }
        
        public uvec3 rar {
            get {
                return new uvec3(this.v0, this.v3, this.v0);
            }
        }
        
        public uvec3 sqs {
            get {
                return new uvec3(this.v0, this.v3, this.v0);
            }
        }
        
        public uvec3 xwy {
            get {
                return new uvec3(this.v0, this.v3, this.v1);
            }
            set {
                this.v0 = value.v0;
                this.v3 = value.v1;
                this.v1 = value.v2;
            }
        }
        
        public uvec3 rag {
            get {
                return new uvec3(this.v0, this.v3, this.v1);
            }
            set {
                this.v0 = value.v0;
                this.v3 = value.v1;
                this.v1 = value.v2;
            }
        }
        
        public uvec3 sqt {
            get {
                return new uvec3(this.v0, this.v3, this.v1);
            }
            set {
                this.v0 = value.v0;
                this.v3 = value.v1;
                this.v1 = value.v2;
            }
        }
        
        public uvec3 xwz {
            get {
                return new uvec3(this.v0, this.v3, this.v2);
            }
            set {
                this.v0 = value.v0;
                this.v3 = value.v1;
                this.v2 = value.v2;
            }
        }
        
        public uvec3 rab {
            get {
                return new uvec3(this.v0, this.v3, this.v2);
            }
            set {
                this.v0 = value.v0;
                this.v3 = value.v1;
                this.v2 = value.v2;
            }
        }
        
        public uvec3 sqp {
            get {
                return new uvec3(this.v0, this.v3, this.v2);
            }
            set {
                this.v0 = value.v0;
                this.v3 = value.v1;
                this.v2 = value.v2;
            }
        }
        
        public uvec3 xww {
            get {
                return new uvec3(this.v0, this.v3, this.v3);
            }
        }
        
        public uvec3 raa {
            get {
                return new uvec3(this.v0, this.v3, this.v3);
            }
        }
        
        public uvec3 sqq {
            get {
                return new uvec3(this.v0, this.v3, this.v3);
            }
        }
        
        public uvec3 yxx {
            get {
                return new uvec3(this.v1, this.v0, this.v0);
            }
        }
        
        public uvec3 grr {
            get {
                return new uvec3(this.v1, this.v0, this.v0);
            }
        }
        
        public uvec3 tss {
            get {
                return new uvec3(this.v1, this.v0, this.v0);
            }
        }
        
        public uvec3 yxy {
            get {
                return new uvec3(this.v1, this.v0, this.v1);
            }
        }
        
        public uvec3 grg {
            get {
                return new uvec3(this.v1, this.v0, this.v1);
            }
        }
        
        public uvec3 tst {
            get {
                return new uvec3(this.v1, this.v0, this.v1);
            }
        }
        
        public uvec3 yxz {
            get {
                return new uvec3(this.v1, this.v0, this.v2);
            }
            set {
                this.v1 = value.v0;
                this.v0 = value.v1;
                this.v2 = value.v2;
            }
        }
        
        public uvec3 grb {
            get {
                return new uvec3(this.v1, this.v0, this.v2);
            }
            set {
                this.v1 = value.v0;
                this.v0 = value.v1;
                this.v2 = value.v2;
            }
        }
        
        public uvec3 tsp {
            get {
                return new uvec3(this.v1, this.v0, this.v2);
            }
            set {
                this.v1 = value.v0;
                this.v0 = value.v1;
                this.v2 = value.v2;
            }
        }
        
        public uvec3 yxw {
            get {
                return new uvec3(this.v1, this.v0, this.v3);
            }
            set {
                this.v1 = value.v0;
                this.v0 = value.v1;
                this.v3 = value.v2;
            }
        }
        
        public uvec3 gra {
            get {
                return new uvec3(this.v1, this.v0, this.v3);
            }
            set {
                this.v1 = value.v0;
                this.v0 = value.v1;
                this.v3 = value.v2;
            }
        }
        
        public uvec3 tsq {
            get {
                return new uvec3(this.v1, this.v0, this.v3);
            }
            set {
                this.v1 = value.v0;
                this.v0 = value.v1;
                this.v3 = value.v2;
            }
        }
        
        public uvec3 yyx {
            get {
                return new uvec3(this.v1, this.v1, this.v0);
            }
        }
        
        public uvec3 ggr {
            get {
                return new uvec3(this.v1, this.v1, this.v0);
            }
        }
        
        public uvec3 tts {
            get {
                return new uvec3(this.v1, this.v1, this.v0);
            }
        }
        
        public uvec3 yyy {
            get {
                return new uvec3(this.v1, this.v1, this.v1);
            }
        }
        
        public uvec3 ggg {
            get {
                return new uvec3(this.v1, this.v1, this.v1);
            }
        }
        
        public uvec3 ttt {
            get {
                return new uvec3(this.v1, this.v1, this.v1);
            }
        }
        
        public uvec3 yyz {
            get {
                return new uvec3(this.v1, this.v1, this.v2);
            }
        }
        
        public uvec3 ggb {
            get {
                return new uvec3(this.v1, this.v1, this.v2);
            }
        }
        
        public uvec3 ttp {
            get {
                return new uvec3(this.v1, this.v1, this.v2);
            }
        }
        
        public uvec3 yyw {
            get {
                return new uvec3(this.v1, this.v1, this.v3);
            }
        }
        
        public uvec3 gga {
            get {
                return new uvec3(this.v1, this.v1, this.v3);
            }
        }
        
        public uvec3 ttq {
            get {
                return new uvec3(this.v1, this.v1, this.v3);
            }
        }
        
        public uvec3 yzx {
            get {
                return new uvec3(this.v1, this.v2, this.v0);
            }
            set {
                this.v1 = value.v0;
                this.v2 = value.v1;
                this.v0 = value.v2;
            }
        }
        
        public uvec3 gbr {
            get {
                return new uvec3(this.v1, this.v2, this.v0);
            }
            set {
                this.v1 = value.v0;
                this.v2 = value.v1;
                this.v0 = value.v2;
            }
        }
        
        public uvec3 tps {
            get {
                return new uvec3(this.v1, this.v2, this.v0);
            }
            set {
                this.v1 = value.v0;
                this.v2 = value.v1;
                this.v0 = value.v2;
            }
        }
        
        public uvec3 yzy {
            get {
                return new uvec3(this.v1, this.v2, this.v1);
            }
        }
        
        public uvec3 gbg {
            get {
                return new uvec3(this.v1, this.v2, this.v1);
            }
        }
        
        public uvec3 tpt {
            get {
                return new uvec3(this.v1, this.v2, this.v1);
            }
        }
        
        public uvec3 yzz {
            get {
                return new uvec3(this.v1, this.v2, this.v2);
            }
        }
        
        public uvec3 gbb {
            get {
                return new uvec3(this.v1, this.v2, this.v2);
            }
        }
        
        public uvec3 tpp {
            get {
                return new uvec3(this.v1, this.v2, this.v2);
            }
        }
        
        public uvec3 yzw {
            get {
                return new uvec3(this.v1, this.v2, this.v3);
            }
            set {
                this.v1 = value.v0;
                this.v2 = value.v1;
                this.v3 = value.v2;
            }
        }
        
        public uvec3 gba {
            get {
                return new uvec3(this.v1, this.v2, this.v3);
            }
            set {
                this.v1 = value.v0;
                this.v2 = value.v1;
                this.v3 = value.v2;
            }
        }
        
        public uvec3 tpq {
            get {
                return new uvec3(this.v1, this.v2, this.v3);
            }
            set {
                this.v1 = value.v0;
                this.v2 = value.v1;
                this.v3 = value.v2;
            }
        }
        
        public uvec3 ywx {
            get {
                return new uvec3(this.v1, this.v3, this.v0);
            }
            set {
                this.v1 = value.v0;
                this.v3 = value.v1;
                this.v0 = value.v2;
            }
        }
        
        public uvec3 gar {
            get {
                return new uvec3(this.v1, this.v3, this.v0);
            }
            set {
                this.v1 = value.v0;
                this.v3 = value.v1;
                this.v0 = value.v2;
            }
        }
        
        public uvec3 tqs {
            get {
                return new uvec3(this.v1, this.v3, this.v0);
            }
            set {
                this.v1 = value.v0;
                this.v3 = value.v1;
                this.v0 = value.v2;
            }
        }
        
        public uvec3 ywy {
            get {
                return new uvec3(this.v1, this.v3, this.v1);
            }
        }
        
        public uvec3 gag {
            get {
                return new uvec3(this.v1, this.v3, this.v1);
            }
        }
        
        public uvec3 tqt {
            get {
                return new uvec3(this.v1, this.v3, this.v1);
            }
        }
        
        public uvec3 ywz {
            get {
                return new uvec3(this.v1, this.v3, this.v2);
            }
            set {
                this.v1 = value.v0;
                this.v3 = value.v1;
                this.v2 = value.v2;
            }
        }
        
        public uvec3 gab {
            get {
                return new uvec3(this.v1, this.v3, this.v2);
            }
            set {
                this.v1 = value.v0;
                this.v3 = value.v1;
                this.v2 = value.v2;
            }
        }
        
        public uvec3 tqp {
            get {
                return new uvec3(this.v1, this.v3, this.v2);
            }
            set {
                this.v1 = value.v0;
                this.v3 = value.v1;
                this.v2 = value.v2;
            }
        }
        
        public uvec3 yww {
            get {
                return new uvec3(this.v1, this.v3, this.v3);
            }
        }
        
        public uvec3 gaa {
            get {
                return new uvec3(this.v1, this.v3, this.v3);
            }
        }
        
        public uvec3 tqq {
            get {
                return new uvec3(this.v1, this.v3, this.v3);
            }
        }
        
        public uvec3 zxx {
            get {
                return new uvec3(this.v2, this.v0, this.v0);
            }
        }
        
        public uvec3 brr {
            get {
                return new uvec3(this.v2, this.v0, this.v0);
            }
        }
        
        public uvec3 pss {
            get {
                return new uvec3(this.v2, this.v0, this.v0);
            }
        }
        
        public uvec3 zxy {
            get {
                return new uvec3(this.v2, this.v0, this.v1);
            }
            set {
                this.v2 = value.v0;
                this.v0 = value.v1;
                this.v1 = value.v2;
            }
        }
        
        public uvec3 brg {
            get {
                return new uvec3(this.v2, this.v0, this.v1);
            }
            set {
                this.v2 = value.v0;
                this.v0 = value.v1;
                this.v1 = value.v2;
            }
        }
        
        public uvec3 pst {
            get {
                return new uvec3(this.v2, this.v0, this.v1);
            }
            set {
                this.v2 = value.v0;
                this.v0 = value.v1;
                this.v1 = value.v2;
            }
        }
        
        public uvec3 zxz {
            get {
                return new uvec3(this.v2, this.v0, this.v2);
            }
        }
        
        public uvec3 brb {
            get {
                return new uvec3(this.v2, this.v0, this.v2);
            }
        }
        
        public uvec3 psp {
            get {
                return new uvec3(this.v2, this.v0, this.v2);
            }
        }
        
        public uvec3 zxw {
            get {
                return new uvec3(this.v2, this.v0, this.v3);
            }
            set {
                this.v2 = value.v0;
                this.v0 = value.v1;
                this.v3 = value.v2;
            }
        }
        
        public uvec3 bra {
            get {
                return new uvec3(this.v2, this.v0, this.v3);
            }
            set {
                this.v2 = value.v0;
                this.v0 = value.v1;
                this.v3 = value.v2;
            }
        }
        
        public uvec3 psq {
            get {
                return new uvec3(this.v2, this.v0, this.v3);
            }
            set {
                this.v2 = value.v0;
                this.v0 = value.v1;
                this.v3 = value.v2;
            }
        }
        
        public uvec3 zyx {
            get {
                return new uvec3(this.v2, this.v1, this.v0);
            }
            set {
                this.v2 = value.v0;
                this.v1 = value.v1;
                this.v0 = value.v2;
            }
        }
        
        public uvec3 bgr {
            get {
                return new uvec3(this.v2, this.v1, this.v0);
            }
            set {
                this.v2 = value.v0;
                this.v1 = value.v1;
                this.v0 = value.v2;
            }
        }
        
        public uvec3 pts {
            get {
                return new uvec3(this.v2, this.v1, this.v0);
            }
            set {
                this.v2 = value.v0;
                this.v1 = value.v1;
                this.v0 = value.v2;
            }
        }
        
        public uvec3 zyy {
            get {
                return new uvec3(this.v2, this.v1, this.v1);
            }
        }
        
        public uvec3 bgg {
            get {
                return new uvec3(this.v2, this.v1, this.v1);
            }
        }
        
        public uvec3 ptt {
            get {
                return new uvec3(this.v2, this.v1, this.v1);
            }
        }
        
        public uvec3 zyz {
            get {
                return new uvec3(this.v2, this.v1, this.v2);
            }
        }
        
        public uvec3 bgb {
            get {
                return new uvec3(this.v2, this.v1, this.v2);
            }
        }
        
        public uvec3 ptp {
            get {
                return new uvec3(this.v2, this.v1, this.v2);
            }
        }
        
        public uvec3 zyw {
            get {
                return new uvec3(this.v2, this.v1, this.v3);
            }
            set {
                this.v2 = value.v0;
                this.v1 = value.v1;
                this.v3 = value.v2;
            }
        }
        
        public uvec3 bga {
            get {
                return new uvec3(this.v2, this.v1, this.v3);
            }
            set {
                this.v2 = value.v0;
                this.v1 = value.v1;
                this.v3 = value.v2;
            }
        }
        
        public uvec3 ptq {
            get {
                return new uvec3(this.v2, this.v1, this.v3);
            }
            set {
                this.v2 = value.v0;
                this.v1 = value.v1;
                this.v3 = value.v2;
            }
        }
        
        public uvec3 zzx {
            get {
                return new uvec3(this.v2, this.v2, this.v0);
            }
        }
        
        public uvec3 bbr {
            get {
                return new uvec3(this.v2, this.v2, this.v0);
            }
        }
        
        public uvec3 pps {
            get {
                return new uvec3(this.v2, this.v2, this.v0);
            }
        }
        
        public uvec3 zzy {
            get {
                return new uvec3(this.v2, this.v2, this.v1);
            }
        }
        
        public uvec3 bbg {
            get {
                return new uvec3(this.v2, this.v2, this.v1);
            }
        }
        
        public uvec3 ppt {
            get {
                return new uvec3(this.v2, this.v2, this.v1);
            }
        }
        
        public uvec3 zzz {
            get {
                return new uvec3(this.v2, this.v2, this.v2);
            }
        }
        
        public uvec3 bbb {
            get {
                return new uvec3(this.v2, this.v2, this.v2);
            }
        }
        
        public uvec3 ppp {
            get {
                return new uvec3(this.v2, this.v2, this.v2);
            }
        }
        
        public uvec3 zzw {
            get {
                return new uvec3(this.v2, this.v2, this.v3);
            }
        }
        
        public uvec3 bba {
            get {
                return new uvec3(this.v2, this.v2, this.v3);
            }
        }
        
        public uvec3 ppq {
            get {
                return new uvec3(this.v2, this.v2, this.v3);
            }
        }
        
        public uvec3 zwx {
            get {
                return new uvec3(this.v2, this.v3, this.v0);
            }
            set {
                this.v2 = value.v0;
                this.v3 = value.v1;
                this.v0 = value.v2;
            }
        }
        
        public uvec3 bar {
            get {
                return new uvec3(this.v2, this.v3, this.v0);
            }
            set {
                this.v2 = value.v0;
                this.v3 = value.v1;
                this.v0 = value.v2;
            }
        }
        
        public uvec3 pqs {
            get {
                return new uvec3(this.v2, this.v3, this.v0);
            }
            set {
                this.v2 = value.v0;
                this.v3 = value.v1;
                this.v0 = value.v2;
            }
        }
        
        public uvec3 zwy {
            get {
                return new uvec3(this.v2, this.v3, this.v1);
            }
            set {
                this.v2 = value.v0;
                this.v3 = value.v1;
                this.v1 = value.v2;
            }
        }
        
        public uvec3 bag {
            get {
                return new uvec3(this.v2, this.v3, this.v1);
            }
            set {
                this.v2 = value.v0;
                this.v3 = value.v1;
                this.v1 = value.v2;
            }
        }
        
        public uvec3 pqt {
            get {
                return new uvec3(this.v2, this.v3, this.v1);
            }
            set {
                this.v2 = value.v0;
                this.v3 = value.v1;
                this.v1 = value.v2;
            }
        }
        
        public uvec3 zwz {
            get {
                return new uvec3(this.v2, this.v3, this.v2);
            }
        }
        
        public uvec3 bab {
            get {
                return new uvec3(this.v2, this.v3, this.v2);
            }
        }
        
        public uvec3 pqp {
            get {
                return new uvec3(this.v2, this.v3, this.v2);
            }
        }
        
        public uvec3 zww {
            get {
                return new uvec3(this.v2, this.v3, this.v3);
            }
        }
        
        public uvec3 baa {
            get {
                return new uvec3(this.v2, this.v3, this.v3);
            }
        }
        
        public uvec3 pqq {
            get {
                return new uvec3(this.v2, this.v3, this.v3);
            }
        }
        
        public uvec3 wxx {
            get {
                return new uvec3(this.v3, this.v0, this.v0);
            }
        }
        
        public uvec3 arr {
            get {
                return new uvec3(this.v3, this.v0, this.v0);
            }
        }
        
        public uvec3 qss {
            get {
                return new uvec3(this.v3, this.v0, this.v0);
            }
        }
        
        public uvec3 wxy {
            get {
                return new uvec3(this.v3, this.v0, this.v1);
            }
            set {
                this.v3 = value.v0;
                this.v0 = value.v1;
                this.v1 = value.v2;
            }
        }
        
        public uvec3 arg {
            get {
                return new uvec3(this.v3, this.v0, this.v1);
            }
            set {
                this.v3 = value.v0;
                this.v0 = value.v1;
                this.v1 = value.v2;
            }
        }
        
        public uvec3 qst {
            get {
                return new uvec3(this.v3, this.v0, this.v1);
            }
            set {
                this.v3 = value.v0;
                this.v0 = value.v1;
                this.v1 = value.v2;
            }
        }
        
        public uvec3 wxz {
            get {
                return new uvec3(this.v3, this.v0, this.v2);
            }
            set {
                this.v3 = value.v0;
                this.v0 = value.v1;
                this.v2 = value.v2;
            }
        }
        
        public uvec3 arb {
            get {
                return new uvec3(this.v3, this.v0, this.v2);
            }
            set {
                this.v3 = value.v0;
                this.v0 = value.v1;
                this.v2 = value.v2;
            }
        }
        
        public uvec3 qsp {
            get {
                return new uvec3(this.v3, this.v0, this.v2);
            }
            set {
                this.v3 = value.v0;
                this.v0 = value.v1;
                this.v2 = value.v2;
            }
        }
        
        public uvec3 wxw {
            get {
                return new uvec3(this.v3, this.v0, this.v3);
            }
        }
        
        public uvec3 ara {
            get {
                return new uvec3(this.v3, this.v0, this.v3);
            }
        }
        
        public uvec3 qsq {
            get {
                return new uvec3(this.v3, this.v0, this.v3);
            }
        }
        
        public uvec3 wyx {
            get {
                return new uvec3(this.v3, this.v1, this.v0);
            }
            set {
                this.v3 = value.v0;
                this.v1 = value.v1;
                this.v0 = value.v2;
            }
        }
        
        public uvec3 agr {
            get {
                return new uvec3(this.v3, this.v1, this.v0);
            }
            set {
                this.v3 = value.v0;
                this.v1 = value.v1;
                this.v0 = value.v2;
            }
        }
        
        public uvec3 qts {
            get {
                return new uvec3(this.v3, this.v1, this.v0);
            }
            set {
                this.v3 = value.v0;
                this.v1 = value.v1;
                this.v0 = value.v2;
            }
        }
        
        public uvec3 wyy {
            get {
                return new uvec3(this.v3, this.v1, this.v1);
            }
        }
        
        public uvec3 agg {
            get {
                return new uvec3(this.v3, this.v1, this.v1);
            }
        }
        
        public uvec3 qtt {
            get {
                return new uvec3(this.v3, this.v1, this.v1);
            }
        }
        
        public uvec3 wyz {
            get {
                return new uvec3(this.v3, this.v1, this.v2);
            }
            set {
                this.v3 = value.v0;
                this.v1 = value.v1;
                this.v2 = value.v2;
            }
        }
        
        public uvec3 agb {
            get {
                return new uvec3(this.v3, this.v1, this.v2);
            }
            set {
                this.v3 = value.v0;
                this.v1 = value.v1;
                this.v2 = value.v2;
            }
        }
        
        public uvec3 qtp {
            get {
                return new uvec3(this.v3, this.v1, this.v2);
            }
            set {
                this.v3 = value.v0;
                this.v1 = value.v1;
                this.v2 = value.v2;
            }
        }
        
        public uvec3 wyw {
            get {
                return new uvec3(this.v3, this.v1, this.v3);
            }
        }
        
        public uvec3 aga {
            get {
                return new uvec3(this.v3, this.v1, this.v3);
            }
        }
        
        public uvec3 qtq {
            get {
                return new uvec3(this.v3, this.v1, this.v3);
            }
        }
        
        public uvec3 wzx {
            get {
                return new uvec3(this.v3, this.v2, this.v0);
            }
            set {
                this.v3 = value.v0;
                this.v2 = value.v1;
                this.v0 = value.v2;
            }
        }
        
        public uvec3 abr {
            get {
                return new uvec3(this.v3, this.v2, this.v0);
            }
            set {
                this.v3 = value.v0;
                this.v2 = value.v1;
                this.v0 = value.v2;
            }
        }
        
        public uvec3 qps {
            get {
                return new uvec3(this.v3, this.v2, this.v0);
            }
            set {
                this.v3 = value.v0;
                this.v2 = value.v1;
                this.v0 = value.v2;
            }
        }
        
        public uvec3 wzy {
            get {
                return new uvec3(this.v3, this.v2, this.v1);
            }
            set {
                this.v3 = value.v0;
                this.v2 = value.v1;
                this.v1 = value.v2;
            }
        }
        
        public uvec3 abg {
            get {
                return new uvec3(this.v3, this.v2, this.v1);
            }
            set {
                this.v3 = value.v0;
                this.v2 = value.v1;
                this.v1 = value.v2;
            }
        }
        
        public uvec3 qpt {
            get {
                return new uvec3(this.v3, this.v2, this.v1);
            }
            set {
                this.v3 = value.v0;
                this.v2 = value.v1;
                this.v1 = value.v2;
            }
        }
        
        public uvec3 wzz {
            get {
                return new uvec3(this.v3, this.v2, this.v2);
            }
        }
        
        public uvec3 abb {
            get {
                return new uvec3(this.v3, this.v2, this.v2);
            }
        }
        
        public uvec3 qpp {
            get {
                return new uvec3(this.v3, this.v2, this.v2);
            }
        }
        
        public uvec3 wzw {
            get {
                return new uvec3(this.v3, this.v2, this.v3);
            }
        }
        
        public uvec3 aba {
            get {
                return new uvec3(this.v3, this.v2, this.v3);
            }
        }
        
        public uvec3 qpq {
            get {
                return new uvec3(this.v3, this.v2, this.v3);
            }
        }
        
        public uvec3 wwx {
            get {
                return new uvec3(this.v3, this.v3, this.v0);
            }
        }
        
        public uvec3 aar {
            get {
                return new uvec3(this.v3, this.v3, this.v0);
            }
        }
        
        public uvec3 qqs {
            get {
                return new uvec3(this.v3, this.v3, this.v0);
            }
        }
        
        public uvec3 wwy {
            get {
                return new uvec3(this.v3, this.v3, this.v1);
            }
        }
        
        public uvec3 aag {
            get {
                return new uvec3(this.v3, this.v3, this.v1);
            }
        }
        
        public uvec3 qqt {
            get {
                return new uvec3(this.v3, this.v3, this.v1);
            }
        }
        
        public uvec3 wwz {
            get {
                return new uvec3(this.v3, this.v3, this.v2);
            }
        }
        
        public uvec3 aab {
            get {
                return new uvec3(this.v3, this.v3, this.v2);
            }
        }
        
        public uvec3 qqp {
            get {
                return new uvec3(this.v3, this.v3, this.v2);
            }
        }
        
        public uvec3 www {
            get {
                return new uvec3(this.v3, this.v3, this.v3);
            }
        }
        
        public uvec3 aaa {
            get {
                return new uvec3(this.v3, this.v3, this.v3);
            }
        }
        
        public uvec3 qqq {
            get {
                return new uvec3(this.v3, this.v3, this.v3);
            }
        }
        
        public uvec4 xxxx {
            get {
                return new uvec4(this.v0, this.v0, this.v0, this.v0);
            }
        }
        
        public uvec4 rrrr {
            get {
                return new uvec4(this.v0, this.v0, this.v0, this.v0);
            }
        }
        
        public uvec4 ssss {
            get {
                return new uvec4(this.v0, this.v0, this.v0, this.v0);
            }
        }
        
        public uvec4 xxxy {
            get {
                return new uvec4(this.v0, this.v0, this.v0, this.v1);
            }
        }
        
        public uvec4 rrrg {
            get {
                return new uvec4(this.v0, this.v0, this.v0, this.v1);
            }
        }
        
        public uvec4 ssst {
            get {
                return new uvec4(this.v0, this.v0, this.v0, this.v1);
            }
        }
        
        public uvec4 xxxz {
            get {
                return new uvec4(this.v0, this.v0, this.v0, this.v2);
            }
        }
        
        public uvec4 rrrb {
            get {
                return new uvec4(this.v0, this.v0, this.v0, this.v2);
            }
        }
        
        public uvec4 sssp {
            get {
                return new uvec4(this.v0, this.v0, this.v0, this.v2);
            }
        }
        
        public uvec4 xxxw {
            get {
                return new uvec4(this.v0, this.v0, this.v0, this.v3);
            }
        }
        
        public uvec4 rrra {
            get {
                return new uvec4(this.v0, this.v0, this.v0, this.v3);
            }
        }
        
        public uvec4 sssq {
            get {
                return new uvec4(this.v0, this.v0, this.v0, this.v3);
            }
        }
        
        public uvec4 xxyx {
            get {
                return new uvec4(this.v0, this.v0, this.v1, this.v0);
            }
        }
        
        public uvec4 rrgr {
            get {
                return new uvec4(this.v0, this.v0, this.v1, this.v0);
            }
        }
        
        public uvec4 ssts {
            get {
                return new uvec4(this.v0, this.v0, this.v1, this.v0);
            }
        }
        
        public uvec4 xxyy {
            get {
                return new uvec4(this.v0, this.v0, this.v1, this.v1);
            }
        }
        
        public uvec4 rrgg {
            get {
                return new uvec4(this.v0, this.v0, this.v1, this.v1);
            }
        }
        
        public uvec4 sstt {
            get {
                return new uvec4(this.v0, this.v0, this.v1, this.v1);
            }
        }
        
        public uvec4 xxyz {
            get {
                return new uvec4(this.v0, this.v0, this.v1, this.v2);
            }
        }
        
        public uvec4 rrgb {
            get {
                return new uvec4(this.v0, this.v0, this.v1, this.v2);
            }
        }
        
        public uvec4 sstp {
            get {
                return new uvec4(this.v0, this.v0, this.v1, this.v2);
            }
        }
        
        public uvec4 xxyw {
            get {
                return new uvec4(this.v0, this.v0, this.v1, this.v3);
            }
        }
        
        public uvec4 rrga {
            get {
                return new uvec4(this.v0, this.v0, this.v1, this.v3);
            }
        }
        
        public uvec4 sstq {
            get {
                return new uvec4(this.v0, this.v0, this.v1, this.v3);
            }
        }
        
        public uvec4 xxzx {
            get {
                return new uvec4(this.v0, this.v0, this.v2, this.v0);
            }
        }
        
        public uvec4 rrbr {
            get {
                return new uvec4(this.v0, this.v0, this.v2, this.v0);
            }
        }
        
        public uvec4 ssps {
            get {
                return new uvec4(this.v0, this.v0, this.v2, this.v0);
            }
        }
        
        public uvec4 xxzy {
            get {
                return new uvec4(this.v0, this.v0, this.v2, this.v1);
            }
        }
        
        public uvec4 rrbg {
            get {
                return new uvec4(this.v0, this.v0, this.v2, this.v1);
            }
        }
        
        public uvec4 sspt {
            get {
                return new uvec4(this.v0, this.v0, this.v2, this.v1);
            }
        }
        
        public uvec4 xxzz {
            get {
                return new uvec4(this.v0, this.v0, this.v2, this.v2);
            }
        }
        
        public uvec4 rrbb {
            get {
                return new uvec4(this.v0, this.v0, this.v2, this.v2);
            }
        }
        
        public uvec4 sspp {
            get {
                return new uvec4(this.v0, this.v0, this.v2, this.v2);
            }
        }
        
        public uvec4 xxzw {
            get {
                return new uvec4(this.v0, this.v0, this.v2, this.v3);
            }
        }
        
        public uvec4 rrba {
            get {
                return new uvec4(this.v0, this.v0, this.v2, this.v3);
            }
        }
        
        public uvec4 sspq {
            get {
                return new uvec4(this.v0, this.v0, this.v2, this.v3);
            }
        }
        
        public uvec4 xxwx {
            get {
                return new uvec4(this.v0, this.v0, this.v3, this.v0);
            }
        }
        
        public uvec4 rrar {
            get {
                return new uvec4(this.v0, this.v0, this.v3, this.v0);
            }
        }
        
        public uvec4 ssqs {
            get {
                return new uvec4(this.v0, this.v0, this.v3, this.v0);
            }
        }
        
        public uvec4 xxwy {
            get {
                return new uvec4(this.v0, this.v0, this.v3, this.v1);
            }
        }
        
        public uvec4 rrag {
            get {
                return new uvec4(this.v0, this.v0, this.v3, this.v1);
            }
        }
        
        public uvec4 ssqt {
            get {
                return new uvec4(this.v0, this.v0, this.v3, this.v1);
            }
        }
        
        public uvec4 xxwz {
            get {
                return new uvec4(this.v0, this.v0, this.v3, this.v2);
            }
        }
        
        public uvec4 rrab {
            get {
                return new uvec4(this.v0, this.v0, this.v3, this.v2);
            }
        }
        
        public uvec4 ssqp {
            get {
                return new uvec4(this.v0, this.v0, this.v3, this.v2);
            }
        }
        
        public uvec4 xxww {
            get {
                return new uvec4(this.v0, this.v0, this.v3, this.v3);
            }
        }
        
        public uvec4 rraa {
            get {
                return new uvec4(this.v0, this.v0, this.v3, this.v3);
            }
        }
        
        public uvec4 ssqq {
            get {
                return new uvec4(this.v0, this.v0, this.v3, this.v3);
            }
        }
        
        public uvec4 xyxx {
            get {
                return new uvec4(this.v0, this.v1, this.v0, this.v0);
            }
        }
        
        public uvec4 rgrr {
            get {
                return new uvec4(this.v0, this.v1, this.v0, this.v0);
            }
        }
        
        public uvec4 stss {
            get {
                return new uvec4(this.v0, this.v1, this.v0, this.v0);
            }
        }
        
        public uvec4 xyxy {
            get {
                return new uvec4(this.v0, this.v1, this.v0, this.v1);
            }
        }
        
        public uvec4 rgrg {
            get {
                return new uvec4(this.v0, this.v1, this.v0, this.v1);
            }
        }
        
        public uvec4 stst {
            get {
                return new uvec4(this.v0, this.v1, this.v0, this.v1);
            }
        }
        
        public uvec4 xyxz {
            get {
                return new uvec4(this.v0, this.v1, this.v0, this.v2);
            }
        }
        
        public uvec4 rgrb {
            get {
                return new uvec4(this.v0, this.v1, this.v0, this.v2);
            }
        }
        
        public uvec4 stsp {
            get {
                return new uvec4(this.v0, this.v1, this.v0, this.v2);
            }
        }
        
        public uvec4 xyxw {
            get {
                return new uvec4(this.v0, this.v1, this.v0, this.v3);
            }
        }
        
        public uvec4 rgra {
            get {
                return new uvec4(this.v0, this.v1, this.v0, this.v3);
            }
        }
        
        public uvec4 stsq {
            get {
                return new uvec4(this.v0, this.v1, this.v0, this.v3);
            }
        }
        
        public uvec4 xyyx {
            get {
                return new uvec4(this.v0, this.v1, this.v1, this.v0);
            }
        }
        
        public uvec4 rggr {
            get {
                return new uvec4(this.v0, this.v1, this.v1, this.v0);
            }
        }
        
        public uvec4 stts {
            get {
                return new uvec4(this.v0, this.v1, this.v1, this.v0);
            }
        }
        
        public uvec4 xyyy {
            get {
                return new uvec4(this.v0, this.v1, this.v1, this.v1);
            }
        }
        
        public uvec4 rggg {
            get {
                return new uvec4(this.v0, this.v1, this.v1, this.v1);
            }
        }
        
        public uvec4 sttt {
            get {
                return new uvec4(this.v0, this.v1, this.v1, this.v1);
            }
        }
        
        public uvec4 xyyz {
            get {
                return new uvec4(this.v0, this.v1, this.v1, this.v2);
            }
        }
        
        public uvec4 rggb {
            get {
                return new uvec4(this.v0, this.v1, this.v1, this.v2);
            }
        }
        
        public uvec4 sttp {
            get {
                return new uvec4(this.v0, this.v1, this.v1, this.v2);
            }
        }
        
        public uvec4 xyyw {
            get {
                return new uvec4(this.v0, this.v1, this.v1, this.v3);
            }
        }
        
        public uvec4 rgga {
            get {
                return new uvec4(this.v0, this.v1, this.v1, this.v3);
            }
        }
        
        public uvec4 sttq {
            get {
                return new uvec4(this.v0, this.v1, this.v1, this.v3);
            }
        }
        
        public uvec4 xyzx {
            get {
                return new uvec4(this.v0, this.v1, this.v2, this.v0);
            }
        }
        
        public uvec4 rgbr {
            get {
                return new uvec4(this.v0, this.v1, this.v2, this.v0);
            }
        }
        
        public uvec4 stps {
            get {
                return new uvec4(this.v0, this.v1, this.v2, this.v0);
            }
        }
        
        public uvec4 xyzy {
            get {
                return new uvec4(this.v0, this.v1, this.v2, this.v1);
            }
        }
        
        public uvec4 rgbg {
            get {
                return new uvec4(this.v0, this.v1, this.v2, this.v1);
            }
        }
        
        public uvec4 stpt {
            get {
                return new uvec4(this.v0, this.v1, this.v2, this.v1);
            }
        }
        
        public uvec4 xyzz {
            get {
                return new uvec4(this.v0, this.v1, this.v2, this.v2);
            }
        }
        
        public uvec4 rgbb {
            get {
                return new uvec4(this.v0, this.v1, this.v2, this.v2);
            }
        }
        
        public uvec4 stpp {
            get {
                return new uvec4(this.v0, this.v1, this.v2, this.v2);
            }
        }
        
        public uvec4 xyzw {
            get {
                return new uvec4(this.v0, this.v1, this.v2, this.v3);
            }
            set {
                this.v0 = value.v0;
                this.v1 = value.v1;
                this.v2 = value.v2;
                this.v3 = value.v3;
            }
        }
        
        public uvec4 rgba {
            get {
                return new uvec4(this.v0, this.v1, this.v2, this.v3);
            }
            set {
                this.v0 = value.v0;
                this.v1 = value.v1;
                this.v2 = value.v2;
                this.v3 = value.v3;
            }
        }
        
        public uvec4 stpq {
            get {
                return new uvec4(this.v0, this.v1, this.v2, this.v3);
            }
            set {
                this.v0 = value.v0;
                this.v1 = value.v1;
                this.v2 = value.v2;
                this.v3 = value.v3;
            }
        }
        
        public uvec4 xywx {
            get {
                return new uvec4(this.v0, this.v1, this.v3, this.v0);
            }
        }
        
        public uvec4 rgar {
            get {
                return new uvec4(this.v0, this.v1, this.v3, this.v0);
            }
        }
        
        public uvec4 stqs {
            get {
                return new uvec4(this.v0, this.v1, this.v3, this.v0);
            }
        }
        
        public uvec4 xywy {
            get {
                return new uvec4(this.v0, this.v1, this.v3, this.v1);
            }
        }
        
        public uvec4 rgag {
            get {
                return new uvec4(this.v0, this.v1, this.v3, this.v1);
            }
        }
        
        public uvec4 stqt {
            get {
                return new uvec4(this.v0, this.v1, this.v3, this.v1);
            }
        }
        
        public uvec4 xywz {
            get {
                return new uvec4(this.v0, this.v1, this.v3, this.v2);
            }
            set {
                this.v0 = value.v0;
                this.v1 = value.v1;
                this.v3 = value.v2;
                this.v2 = value.v3;
            }
        }
        
        public uvec4 rgab {
            get {
                return new uvec4(this.v0, this.v1, this.v3, this.v2);
            }
            set {
                this.v0 = value.v0;
                this.v1 = value.v1;
                this.v3 = value.v2;
                this.v2 = value.v3;
            }
        }
        
        public uvec4 stqp {
            get {
                return new uvec4(this.v0, this.v1, this.v3, this.v2);
            }
            set {
                this.v0 = value.v0;
                this.v1 = value.v1;
                this.v3 = value.v2;
                this.v2 = value.v3;
            }
        }
        
        public uvec4 xyww {
            get {
                return new uvec4(this.v0, this.v1, this.v3, this.v3);
            }
        }
        
        public uvec4 rgaa {
            get {
                return new uvec4(this.v0, this.v1, this.v3, this.v3);
            }
        }
        
        public uvec4 stqq {
            get {
                return new uvec4(this.v0, this.v1, this.v3, this.v3);
            }
        }
        
        public uvec4 xzxx {
            get {
                return new uvec4(this.v0, this.v2, this.v0, this.v0);
            }
        }
        
        public uvec4 rbrr {
            get {
                return new uvec4(this.v0, this.v2, this.v0, this.v0);
            }
        }
        
        public uvec4 spss {
            get {
                return new uvec4(this.v0, this.v2, this.v0, this.v0);
            }
        }
        
        public uvec4 xzxy {
            get {
                return new uvec4(this.v0, this.v2, this.v0, this.v1);
            }
        }
        
        public uvec4 rbrg {
            get {
                return new uvec4(this.v0, this.v2, this.v0, this.v1);
            }
        }
        
        public uvec4 spst {
            get {
                return new uvec4(this.v0, this.v2, this.v0, this.v1);
            }
        }
        
        public uvec4 xzxz {
            get {
                return new uvec4(this.v0, this.v2, this.v0, this.v2);
            }
        }
        
        public uvec4 rbrb {
            get {
                return new uvec4(this.v0, this.v2, this.v0, this.v2);
            }
        }
        
        public uvec4 spsp {
            get {
                return new uvec4(this.v0, this.v2, this.v0, this.v2);
            }
        }
        
        public uvec4 xzxw {
            get {
                return new uvec4(this.v0, this.v2, this.v0, this.v3);
            }
        }
        
        public uvec4 rbra {
            get {
                return new uvec4(this.v0, this.v2, this.v0, this.v3);
            }
        }
        
        public uvec4 spsq {
            get {
                return new uvec4(this.v0, this.v2, this.v0, this.v3);
            }
        }
        
        public uvec4 xzyx {
            get {
                return new uvec4(this.v0, this.v2, this.v1, this.v0);
            }
        }
        
        public uvec4 rbgr {
            get {
                return new uvec4(this.v0, this.v2, this.v1, this.v0);
            }
        }
        
        public uvec4 spts {
            get {
                return new uvec4(this.v0, this.v2, this.v1, this.v0);
            }
        }
        
        public uvec4 xzyy {
            get {
                return new uvec4(this.v0, this.v2, this.v1, this.v1);
            }
        }
        
        public uvec4 rbgg {
            get {
                return new uvec4(this.v0, this.v2, this.v1, this.v1);
            }
        }
        
        public uvec4 sptt {
            get {
                return new uvec4(this.v0, this.v2, this.v1, this.v1);
            }
        }
        
        public uvec4 xzyz {
            get {
                return new uvec4(this.v0, this.v2, this.v1, this.v2);
            }
        }
        
        public uvec4 rbgb {
            get {
                return new uvec4(this.v0, this.v2, this.v1, this.v2);
            }
        }
        
        public uvec4 sptp {
            get {
                return new uvec4(this.v0, this.v2, this.v1, this.v2);
            }
        }
        
        public uvec4 xzyw {
            get {
                return new uvec4(this.v0, this.v2, this.v1, this.v3);
            }
            set {
                this.v0 = value.v0;
                this.v2 = value.v1;
                this.v1 = value.v2;
                this.v3 = value.v3;
            }
        }
        
        public uvec4 rbga {
            get {
                return new uvec4(this.v0, this.v2, this.v1, this.v3);
            }
            set {
                this.v0 = value.v0;
                this.v2 = value.v1;
                this.v1 = value.v2;
                this.v3 = value.v3;
            }
        }
        
        public uvec4 sptq {
            get {
                return new uvec4(this.v0, this.v2, this.v1, this.v3);
            }
            set {
                this.v0 = value.v0;
                this.v2 = value.v1;
                this.v1 = value.v2;
                this.v3 = value.v3;
            }
        }
        
        public uvec4 xzzx {
            get {
                return new uvec4(this.v0, this.v2, this.v2, this.v0);
            }
        }
        
        public uvec4 rbbr {
            get {
                return new uvec4(this.v0, this.v2, this.v2, this.v0);
            }
        }
        
        public uvec4 spps {
            get {
                return new uvec4(this.v0, this.v2, this.v2, this.v0);
            }
        }
        
        public uvec4 xzzy {
            get {
                return new uvec4(this.v0, this.v2, this.v2, this.v1);
            }
        }
        
        public uvec4 rbbg {
            get {
                return new uvec4(this.v0, this.v2, this.v2, this.v1);
            }
        }
        
        public uvec4 sppt {
            get {
                return new uvec4(this.v0, this.v2, this.v2, this.v1);
            }
        }
        
        public uvec4 xzzz {
            get {
                return new uvec4(this.v0, this.v2, this.v2, this.v2);
            }
        }
        
        public uvec4 rbbb {
            get {
                return new uvec4(this.v0, this.v2, this.v2, this.v2);
            }
        }
        
        public uvec4 sppp {
            get {
                return new uvec4(this.v0, this.v2, this.v2, this.v2);
            }
        }
        
        public uvec4 xzzw {
            get {
                return new uvec4(this.v0, this.v2, this.v2, this.v3);
            }
        }
        
        public uvec4 rbba {
            get {
                return new uvec4(this.v0, this.v2, this.v2, this.v3);
            }
        }
        
        public uvec4 sppq {
            get {
                return new uvec4(this.v0, this.v2, this.v2, this.v3);
            }
        }
        
        public uvec4 xzwx {
            get {
                return new uvec4(this.v0, this.v2, this.v3, this.v0);
            }
        }
        
        public uvec4 rbar {
            get {
                return new uvec4(this.v0, this.v2, this.v3, this.v0);
            }
        }
        
        public uvec4 spqs {
            get {
                return new uvec4(this.v0, this.v2, this.v3, this.v0);
            }
        }
        
        public uvec4 xzwy {
            get {
                return new uvec4(this.v0, this.v2, this.v3, this.v1);
            }
            set {
                this.v0 = value.v0;
                this.v2 = value.v1;
                this.v3 = value.v2;
                this.v1 = value.v3;
            }
        }
        
        public uvec4 rbag {
            get {
                return new uvec4(this.v0, this.v2, this.v3, this.v1);
            }
            set {
                this.v0 = value.v0;
                this.v2 = value.v1;
                this.v3 = value.v2;
                this.v1 = value.v3;
            }
        }
        
        public uvec4 spqt {
            get {
                return new uvec4(this.v0, this.v2, this.v3, this.v1);
            }
            set {
                this.v0 = value.v0;
                this.v2 = value.v1;
                this.v3 = value.v2;
                this.v1 = value.v3;
            }
        }
        
        public uvec4 xzwz {
            get {
                return new uvec4(this.v0, this.v2, this.v3, this.v2);
            }
        }
        
        public uvec4 rbab {
            get {
                return new uvec4(this.v0, this.v2, this.v3, this.v2);
            }
        }
        
        public uvec4 spqp {
            get {
                return new uvec4(this.v0, this.v2, this.v3, this.v2);
            }
        }
        
        public uvec4 xzww {
            get {
                return new uvec4(this.v0, this.v2, this.v3, this.v3);
            }
        }
        
        public uvec4 rbaa {
            get {
                return new uvec4(this.v0, this.v2, this.v3, this.v3);
            }
        }
        
        public uvec4 spqq {
            get {
                return new uvec4(this.v0, this.v2, this.v3, this.v3);
            }
        }
        
        public uvec4 xwxx {
            get {
                return new uvec4(this.v0, this.v3, this.v0, this.v0);
            }
        }
        
        public uvec4 rarr {
            get {
                return new uvec4(this.v0, this.v3, this.v0, this.v0);
            }
        }
        
        public uvec4 sqss {
            get {
                return new uvec4(this.v0, this.v3, this.v0, this.v0);
            }
        }
        
        public uvec4 xwxy {
            get {
                return new uvec4(this.v0, this.v3, this.v0, this.v1);
            }
        }
        
        public uvec4 rarg {
            get {
                return new uvec4(this.v0, this.v3, this.v0, this.v1);
            }
        }
        
        public uvec4 sqst {
            get {
                return new uvec4(this.v0, this.v3, this.v0, this.v1);
            }
        }
        
        public uvec4 xwxz {
            get {
                return new uvec4(this.v0, this.v3, this.v0, this.v2);
            }
        }
        
        public uvec4 rarb {
            get {
                return new uvec4(this.v0, this.v3, this.v0, this.v2);
            }
        }
        
        public uvec4 sqsp {
            get {
                return new uvec4(this.v0, this.v3, this.v0, this.v2);
            }
        }
        
        public uvec4 xwxw {
            get {
                return new uvec4(this.v0, this.v3, this.v0, this.v3);
            }
        }
        
        public uvec4 rara {
            get {
                return new uvec4(this.v0, this.v3, this.v0, this.v3);
            }
        }
        
        public uvec4 sqsq {
            get {
                return new uvec4(this.v0, this.v3, this.v0, this.v3);
            }
        }
        
        public uvec4 xwyx {
            get {
                return new uvec4(this.v0, this.v3, this.v1, this.v0);
            }
        }
        
        public uvec4 ragr {
            get {
                return new uvec4(this.v0, this.v3, this.v1, this.v0);
            }
        }
        
        public uvec4 sqts {
            get {
                return new uvec4(this.v0, this.v3, this.v1, this.v0);
            }
        }
        
        public uvec4 xwyy {
            get {
                return new uvec4(this.v0, this.v3, this.v1, this.v1);
            }
        }
        
        public uvec4 ragg {
            get {
                return new uvec4(this.v0, this.v3, this.v1, this.v1);
            }
        }
        
        public uvec4 sqtt {
            get {
                return new uvec4(this.v0, this.v3, this.v1, this.v1);
            }
        }
        
        public uvec4 xwyz {
            get {
                return new uvec4(this.v0, this.v3, this.v1, this.v2);
            }
            set {
                this.v0 = value.v0;
                this.v3 = value.v1;
                this.v1 = value.v2;
                this.v2 = value.v3;
            }
        }
        
        public uvec4 ragb {
            get {
                return new uvec4(this.v0, this.v3, this.v1, this.v2);
            }
            set {
                this.v0 = value.v0;
                this.v3 = value.v1;
                this.v1 = value.v2;
                this.v2 = value.v3;
            }
        }
        
        public uvec4 sqtp {
            get {
                return new uvec4(this.v0, this.v3, this.v1, this.v2);
            }
            set {
                this.v0 = value.v0;
                this.v3 = value.v1;
                this.v1 = value.v2;
                this.v2 = value.v3;
            }
        }
        
        public uvec4 xwyw {
            get {
                return new uvec4(this.v0, this.v3, this.v1, this.v3);
            }
        }
        
        public uvec4 raga {
            get {
                return new uvec4(this.v0, this.v3, this.v1, this.v3);
            }
        }
        
        public uvec4 sqtq {
            get {
                return new uvec4(this.v0, this.v3, this.v1, this.v3);
            }
        }
        
        public uvec4 xwzx {
            get {
                return new uvec4(this.v0, this.v3, this.v2, this.v0);
            }
        }
        
        public uvec4 rabr {
            get {
                return new uvec4(this.v0, this.v3, this.v2, this.v0);
            }
        }
        
        public uvec4 sqps {
            get {
                return new uvec4(this.v0, this.v3, this.v2, this.v0);
            }
        }
        
        public uvec4 xwzy {
            get {
                return new uvec4(this.v0, this.v3, this.v2, this.v1);
            }
            set {
                this.v0 = value.v0;
                this.v3 = value.v1;
                this.v2 = value.v2;
                this.v1 = value.v3;
            }
        }
        
        public uvec4 rabg {
            get {
                return new uvec4(this.v0, this.v3, this.v2, this.v1);
            }
            set {
                this.v0 = value.v0;
                this.v3 = value.v1;
                this.v2 = value.v2;
                this.v1 = value.v3;
            }
        }
        
        public uvec4 sqpt {
            get {
                return new uvec4(this.v0, this.v3, this.v2, this.v1);
            }
            set {
                this.v0 = value.v0;
                this.v3 = value.v1;
                this.v2 = value.v2;
                this.v1 = value.v3;
            }
        }
        
        public uvec4 xwzz {
            get {
                return new uvec4(this.v0, this.v3, this.v2, this.v2);
            }
        }
        
        public uvec4 rabb {
            get {
                return new uvec4(this.v0, this.v3, this.v2, this.v2);
            }
        }
        
        public uvec4 sqpp {
            get {
                return new uvec4(this.v0, this.v3, this.v2, this.v2);
            }
        }
        
        public uvec4 xwzw {
            get {
                return new uvec4(this.v0, this.v3, this.v2, this.v3);
            }
        }
        
        public uvec4 raba {
            get {
                return new uvec4(this.v0, this.v3, this.v2, this.v3);
            }
        }
        
        public uvec4 sqpq {
            get {
                return new uvec4(this.v0, this.v3, this.v2, this.v3);
            }
        }
        
        public uvec4 xwwx {
            get {
                return new uvec4(this.v0, this.v3, this.v3, this.v0);
            }
        }
        
        public uvec4 raar {
            get {
                return new uvec4(this.v0, this.v3, this.v3, this.v0);
            }
        }
        
        public uvec4 sqqs {
            get {
                return new uvec4(this.v0, this.v3, this.v3, this.v0);
            }
        }
        
        public uvec4 xwwy {
            get {
                return new uvec4(this.v0, this.v3, this.v3, this.v1);
            }
        }
        
        public uvec4 raag {
            get {
                return new uvec4(this.v0, this.v3, this.v3, this.v1);
            }
        }
        
        public uvec4 sqqt {
            get {
                return new uvec4(this.v0, this.v3, this.v3, this.v1);
            }
        }
        
        public uvec4 xwwz {
            get {
                return new uvec4(this.v0, this.v3, this.v3, this.v2);
            }
        }
        
        public uvec4 raab {
            get {
                return new uvec4(this.v0, this.v3, this.v3, this.v2);
            }
        }
        
        public uvec4 sqqp {
            get {
                return new uvec4(this.v0, this.v3, this.v3, this.v2);
            }
        }
        
        public uvec4 xwww {
            get {
                return new uvec4(this.v0, this.v3, this.v3, this.v3);
            }
        }
        
        public uvec4 raaa {
            get {
                return new uvec4(this.v0, this.v3, this.v3, this.v3);
            }
        }
        
        public uvec4 sqqq {
            get {
                return new uvec4(this.v0, this.v3, this.v3, this.v3);
            }
        }
        
        public uvec4 yxxx {
            get {
                return new uvec4(this.v1, this.v0, this.v0, this.v0);
            }
        }
        
        public uvec4 grrr {
            get {
                return new uvec4(this.v1, this.v0, this.v0, this.v0);
            }
        }
        
        public uvec4 tsss {
            get {
                return new uvec4(this.v1, this.v0, this.v0, this.v0);
            }
        }
        
        public uvec4 yxxy {
            get {
                return new uvec4(this.v1, this.v0, this.v0, this.v1);
            }
        }
        
        public uvec4 grrg {
            get {
                return new uvec4(this.v1, this.v0, this.v0, this.v1);
            }
        }
        
        public uvec4 tsst {
            get {
                return new uvec4(this.v1, this.v0, this.v0, this.v1);
            }
        }
        
        public uvec4 yxxz {
            get {
                return new uvec4(this.v1, this.v0, this.v0, this.v2);
            }
        }
        
        public uvec4 grrb {
            get {
                return new uvec4(this.v1, this.v0, this.v0, this.v2);
            }
        }
        
        public uvec4 tssp {
            get {
                return new uvec4(this.v1, this.v0, this.v0, this.v2);
            }
        }
        
        public uvec4 yxxw {
            get {
                return new uvec4(this.v1, this.v0, this.v0, this.v3);
            }
        }
        
        public uvec4 grra {
            get {
                return new uvec4(this.v1, this.v0, this.v0, this.v3);
            }
        }
        
        public uvec4 tssq {
            get {
                return new uvec4(this.v1, this.v0, this.v0, this.v3);
            }
        }
        
        public uvec4 yxyx {
            get {
                return new uvec4(this.v1, this.v0, this.v1, this.v0);
            }
        }
        
        public uvec4 grgr {
            get {
                return new uvec4(this.v1, this.v0, this.v1, this.v0);
            }
        }
        
        public uvec4 tsts {
            get {
                return new uvec4(this.v1, this.v0, this.v1, this.v0);
            }
        }
        
        public uvec4 yxyy {
            get {
                return new uvec4(this.v1, this.v0, this.v1, this.v1);
            }
        }
        
        public uvec4 grgg {
            get {
                return new uvec4(this.v1, this.v0, this.v1, this.v1);
            }
        }
        
        public uvec4 tstt {
            get {
                return new uvec4(this.v1, this.v0, this.v1, this.v1);
            }
        }
        
        public uvec4 yxyz {
            get {
                return new uvec4(this.v1, this.v0, this.v1, this.v2);
            }
        }
        
        public uvec4 grgb {
            get {
                return new uvec4(this.v1, this.v0, this.v1, this.v2);
            }
        }
        
        public uvec4 tstp {
            get {
                return new uvec4(this.v1, this.v0, this.v1, this.v2);
            }
        }
        
        public uvec4 yxyw {
            get {
                return new uvec4(this.v1, this.v0, this.v1, this.v3);
            }
        }
        
        public uvec4 grga {
            get {
                return new uvec4(this.v1, this.v0, this.v1, this.v3);
            }
        }
        
        public uvec4 tstq {
            get {
                return new uvec4(this.v1, this.v0, this.v1, this.v3);
            }
        }
        
        public uvec4 yxzx {
            get {
                return new uvec4(this.v1, this.v0, this.v2, this.v0);
            }
        }
        
        public uvec4 grbr {
            get {
                return new uvec4(this.v1, this.v0, this.v2, this.v0);
            }
        }
        
        public uvec4 tsps {
            get {
                return new uvec4(this.v1, this.v0, this.v2, this.v0);
            }
        }
        
        public uvec4 yxzy {
            get {
                return new uvec4(this.v1, this.v0, this.v2, this.v1);
            }
        }
        
        public uvec4 grbg {
            get {
                return new uvec4(this.v1, this.v0, this.v2, this.v1);
            }
        }
        
        public uvec4 tspt {
            get {
                return new uvec4(this.v1, this.v0, this.v2, this.v1);
            }
        }
        
        public uvec4 yxzz {
            get {
                return new uvec4(this.v1, this.v0, this.v2, this.v2);
            }
        }
        
        public uvec4 grbb {
            get {
                return new uvec4(this.v1, this.v0, this.v2, this.v2);
            }
        }
        
        public uvec4 tspp {
            get {
                return new uvec4(this.v1, this.v0, this.v2, this.v2);
            }
        }
        
        public uvec4 yxzw {
            get {
                return new uvec4(this.v1, this.v0, this.v2, this.v3);
            }
            set {
                this.v1 = value.v0;
                this.v0 = value.v1;
                this.v2 = value.v2;
                this.v3 = value.v3;
            }
        }
        
        public uvec4 grba {
            get {
                return new uvec4(this.v1, this.v0, this.v2, this.v3);
            }
            set {
                this.v1 = value.v0;
                this.v0 = value.v1;
                this.v2 = value.v2;
                this.v3 = value.v3;
            }
        }
        
        public uvec4 tspq {
            get {
                return new uvec4(this.v1, this.v0, this.v2, this.v3);
            }
            set {
                this.v1 = value.v0;
                this.v0 = value.v1;
                this.v2 = value.v2;
                this.v3 = value.v3;
            }
        }
        
        public uvec4 yxwx {
            get {
                return new uvec4(this.v1, this.v0, this.v3, this.v0);
            }
        }
        
        public uvec4 grar {
            get {
                return new uvec4(this.v1, this.v0, this.v3, this.v0);
            }
        }
        
        public uvec4 tsqs {
            get {
                return new uvec4(this.v1, this.v0, this.v3, this.v0);
            }
        }
        
        public uvec4 yxwy {
            get {
                return new uvec4(this.v1, this.v0, this.v3, this.v1);
            }
        }
        
        public uvec4 grag {
            get {
                return new uvec4(this.v1, this.v0, this.v3, this.v1);
            }
        }
        
        public uvec4 tsqt {
            get {
                return new uvec4(this.v1, this.v0, this.v3, this.v1);
            }
        }
        
        public uvec4 yxwz {
            get {
                return new uvec4(this.v1, this.v0, this.v3, this.v2);
            }
            set {
                this.v1 = value.v0;
                this.v0 = value.v1;
                this.v3 = value.v2;
                this.v2 = value.v3;
            }
        }
        
        public uvec4 grab {
            get {
                return new uvec4(this.v1, this.v0, this.v3, this.v2);
            }
            set {
                this.v1 = value.v0;
                this.v0 = value.v1;
                this.v3 = value.v2;
                this.v2 = value.v3;
            }
        }
        
        public uvec4 tsqp {
            get {
                return new uvec4(this.v1, this.v0, this.v3, this.v2);
            }
            set {
                this.v1 = value.v0;
                this.v0 = value.v1;
                this.v3 = value.v2;
                this.v2 = value.v3;
            }
        }
        
        public uvec4 yxww {
            get {
                return new uvec4(this.v1, this.v0, this.v3, this.v3);
            }
        }
        
        public uvec4 graa {
            get {
                return new uvec4(this.v1, this.v0, this.v3, this.v3);
            }
        }
        
        public uvec4 tsqq {
            get {
                return new uvec4(this.v1, this.v0, this.v3, this.v3);
            }
        }
        
        public uvec4 yyxx {
            get {
                return new uvec4(this.v1, this.v1, this.v0, this.v0);
            }
        }
        
        public uvec4 ggrr {
            get {
                return new uvec4(this.v1, this.v1, this.v0, this.v0);
            }
        }
        
        public uvec4 ttss {
            get {
                return new uvec4(this.v1, this.v1, this.v0, this.v0);
            }
        }
        
        public uvec4 yyxy {
            get {
                return new uvec4(this.v1, this.v1, this.v0, this.v1);
            }
        }
        
        public uvec4 ggrg {
            get {
                return new uvec4(this.v1, this.v1, this.v0, this.v1);
            }
        }
        
        public uvec4 ttst {
            get {
                return new uvec4(this.v1, this.v1, this.v0, this.v1);
            }
        }
        
        public uvec4 yyxz {
            get {
                return new uvec4(this.v1, this.v1, this.v0, this.v2);
            }
        }
        
        public uvec4 ggrb {
            get {
                return new uvec4(this.v1, this.v1, this.v0, this.v2);
            }
        }
        
        public uvec4 ttsp {
            get {
                return new uvec4(this.v1, this.v1, this.v0, this.v2);
            }
        }
        
        public uvec4 yyxw {
            get {
                return new uvec4(this.v1, this.v1, this.v0, this.v3);
            }
        }
        
        public uvec4 ggra {
            get {
                return new uvec4(this.v1, this.v1, this.v0, this.v3);
            }
        }
        
        public uvec4 ttsq {
            get {
                return new uvec4(this.v1, this.v1, this.v0, this.v3);
            }
        }
        
        public uvec4 yyyx {
            get {
                return new uvec4(this.v1, this.v1, this.v1, this.v0);
            }
        }
        
        public uvec4 gggr {
            get {
                return new uvec4(this.v1, this.v1, this.v1, this.v0);
            }
        }
        
        public uvec4 ttts {
            get {
                return new uvec4(this.v1, this.v1, this.v1, this.v0);
            }
        }
        
        public uvec4 yyyy {
            get {
                return new uvec4(this.v1, this.v1, this.v1, this.v1);
            }
        }
        
        public uvec4 gggg {
            get {
                return new uvec4(this.v1, this.v1, this.v1, this.v1);
            }
        }
        
        public uvec4 tttt {
            get {
                return new uvec4(this.v1, this.v1, this.v1, this.v1);
            }
        }
        
        public uvec4 yyyz {
            get {
                return new uvec4(this.v1, this.v1, this.v1, this.v2);
            }
        }
        
        public uvec4 gggb {
            get {
                return new uvec4(this.v1, this.v1, this.v1, this.v2);
            }
        }
        
        public uvec4 tttp {
            get {
                return new uvec4(this.v1, this.v1, this.v1, this.v2);
            }
        }
        
        public uvec4 yyyw {
            get {
                return new uvec4(this.v1, this.v1, this.v1, this.v3);
            }
        }
        
        public uvec4 ggga {
            get {
                return new uvec4(this.v1, this.v1, this.v1, this.v3);
            }
        }
        
        public uvec4 tttq {
            get {
                return new uvec4(this.v1, this.v1, this.v1, this.v3);
            }
        }
        
        public uvec4 yyzx {
            get {
                return new uvec4(this.v1, this.v1, this.v2, this.v0);
            }
        }
        
        public uvec4 ggbr {
            get {
                return new uvec4(this.v1, this.v1, this.v2, this.v0);
            }
        }
        
        public uvec4 ttps {
            get {
                return new uvec4(this.v1, this.v1, this.v2, this.v0);
            }
        }
        
        public uvec4 yyzy {
            get {
                return new uvec4(this.v1, this.v1, this.v2, this.v1);
            }
        }
        
        public uvec4 ggbg {
            get {
                return new uvec4(this.v1, this.v1, this.v2, this.v1);
            }
        }
        
        public uvec4 ttpt {
            get {
                return new uvec4(this.v1, this.v1, this.v2, this.v1);
            }
        }
        
        public uvec4 yyzz {
            get {
                return new uvec4(this.v1, this.v1, this.v2, this.v2);
            }
        }
        
        public uvec4 ggbb {
            get {
                return new uvec4(this.v1, this.v1, this.v2, this.v2);
            }
        }
        
        public uvec4 ttpp {
            get {
                return new uvec4(this.v1, this.v1, this.v2, this.v2);
            }
        }
        
        public uvec4 yyzw {
            get {
                return new uvec4(this.v1, this.v1, this.v2, this.v3);
            }
        }
        
        public uvec4 ggba {
            get {
                return new uvec4(this.v1, this.v1, this.v2, this.v3);
            }
        }
        
        public uvec4 ttpq {
            get {
                return new uvec4(this.v1, this.v1, this.v2, this.v3);
            }
        }
        
        public uvec4 yywx {
            get {
                return new uvec4(this.v1, this.v1, this.v3, this.v0);
            }
        }
        
        public uvec4 ggar {
            get {
                return new uvec4(this.v1, this.v1, this.v3, this.v0);
            }
        }
        
        public uvec4 ttqs {
            get {
                return new uvec4(this.v1, this.v1, this.v3, this.v0);
            }
        }
        
        public uvec4 yywy {
            get {
                return new uvec4(this.v1, this.v1, this.v3, this.v1);
            }
        }
        
        public uvec4 ggag {
            get {
                return new uvec4(this.v1, this.v1, this.v3, this.v1);
            }
        }
        
        public uvec4 ttqt {
            get {
                return new uvec4(this.v1, this.v1, this.v3, this.v1);
            }
        }
        
        public uvec4 yywz {
            get {
                return new uvec4(this.v1, this.v1, this.v3, this.v2);
            }
        }
        
        public uvec4 ggab {
            get {
                return new uvec4(this.v1, this.v1, this.v3, this.v2);
            }
        }
        
        public uvec4 ttqp {
            get {
                return new uvec4(this.v1, this.v1, this.v3, this.v2);
            }
        }
        
        public uvec4 yyww {
            get {
                return new uvec4(this.v1, this.v1, this.v3, this.v3);
            }
        }
        
        public uvec4 ggaa {
            get {
                return new uvec4(this.v1, this.v1, this.v3, this.v3);
            }
        }
        
        public uvec4 ttqq {
            get {
                return new uvec4(this.v1, this.v1, this.v3, this.v3);
            }
        }
        
        public uvec4 yzxx {
            get {
                return new uvec4(this.v1, this.v2, this.v0, this.v0);
            }
        }
        
        public uvec4 gbrr {
            get {
                return new uvec4(this.v1, this.v2, this.v0, this.v0);
            }
        }
        
        public uvec4 tpss {
            get {
                return new uvec4(this.v1, this.v2, this.v0, this.v0);
            }
        }
        
        public uvec4 yzxy {
            get {
                return new uvec4(this.v1, this.v2, this.v0, this.v1);
            }
        }
        
        public uvec4 gbrg {
            get {
                return new uvec4(this.v1, this.v2, this.v0, this.v1);
            }
        }
        
        public uvec4 tpst {
            get {
                return new uvec4(this.v1, this.v2, this.v0, this.v1);
            }
        }
        
        public uvec4 yzxz {
            get {
                return new uvec4(this.v1, this.v2, this.v0, this.v2);
            }
        }
        
        public uvec4 gbrb {
            get {
                return new uvec4(this.v1, this.v2, this.v0, this.v2);
            }
        }
        
        public uvec4 tpsp {
            get {
                return new uvec4(this.v1, this.v2, this.v0, this.v2);
            }
        }
        
        public uvec4 yzxw {
            get {
                return new uvec4(this.v1, this.v2, this.v0, this.v3);
            }
            set {
                this.v1 = value.v0;
                this.v2 = value.v1;
                this.v0 = value.v2;
                this.v3 = value.v3;
            }
        }
        
        public uvec4 gbra {
            get {
                return new uvec4(this.v1, this.v2, this.v0, this.v3);
            }
            set {
                this.v1 = value.v0;
                this.v2 = value.v1;
                this.v0 = value.v2;
                this.v3 = value.v3;
            }
        }
        
        public uvec4 tpsq {
            get {
                return new uvec4(this.v1, this.v2, this.v0, this.v3);
            }
            set {
                this.v1 = value.v0;
                this.v2 = value.v1;
                this.v0 = value.v2;
                this.v3 = value.v3;
            }
        }
        
        public uvec4 yzyx {
            get {
                return new uvec4(this.v1, this.v2, this.v1, this.v0);
            }
        }
        
        public uvec4 gbgr {
            get {
                return new uvec4(this.v1, this.v2, this.v1, this.v0);
            }
        }
        
        public uvec4 tpts {
            get {
                return new uvec4(this.v1, this.v2, this.v1, this.v0);
            }
        }
        
        public uvec4 yzyy {
            get {
                return new uvec4(this.v1, this.v2, this.v1, this.v1);
            }
        }
        
        public uvec4 gbgg {
            get {
                return new uvec4(this.v1, this.v2, this.v1, this.v1);
            }
        }
        
        public uvec4 tptt {
            get {
                return new uvec4(this.v1, this.v2, this.v1, this.v1);
            }
        }
        
        public uvec4 yzyz {
            get {
                return new uvec4(this.v1, this.v2, this.v1, this.v2);
            }
        }
        
        public uvec4 gbgb {
            get {
                return new uvec4(this.v1, this.v2, this.v1, this.v2);
            }
        }
        
        public uvec4 tptp {
            get {
                return new uvec4(this.v1, this.v2, this.v1, this.v2);
            }
        }
        
        public uvec4 yzyw {
            get {
                return new uvec4(this.v1, this.v2, this.v1, this.v3);
            }
        }
        
        public uvec4 gbga {
            get {
                return new uvec4(this.v1, this.v2, this.v1, this.v3);
            }
        }
        
        public uvec4 tptq {
            get {
                return new uvec4(this.v1, this.v2, this.v1, this.v3);
            }
        }
        
        public uvec4 yzzx {
            get {
                return new uvec4(this.v1, this.v2, this.v2, this.v0);
            }
        }
        
        public uvec4 gbbr {
            get {
                return new uvec4(this.v1, this.v2, this.v2, this.v0);
            }
        }
        
        public uvec4 tpps {
            get {
                return new uvec4(this.v1, this.v2, this.v2, this.v0);
            }
        }
        
        public uvec4 yzzy {
            get {
                return new uvec4(this.v1, this.v2, this.v2, this.v1);
            }
        }
        
        public uvec4 gbbg {
            get {
                return new uvec4(this.v1, this.v2, this.v2, this.v1);
            }
        }
        
        public uvec4 tppt {
            get {
                return new uvec4(this.v1, this.v2, this.v2, this.v1);
            }
        }
        
        public uvec4 yzzz {
            get {
                return new uvec4(this.v1, this.v2, this.v2, this.v2);
            }
        }
        
        public uvec4 gbbb {
            get {
                return new uvec4(this.v1, this.v2, this.v2, this.v2);
            }
        }
        
        public uvec4 tppp {
            get {
                return new uvec4(this.v1, this.v2, this.v2, this.v2);
            }
        }
        
        public uvec4 yzzw {
            get {
                return new uvec4(this.v1, this.v2, this.v2, this.v3);
            }
        }
        
        public uvec4 gbba {
            get {
                return new uvec4(this.v1, this.v2, this.v2, this.v3);
            }
        }
        
        public uvec4 tppq {
            get {
                return new uvec4(this.v1, this.v2, this.v2, this.v3);
            }
        }
        
        public uvec4 yzwx {
            get {
                return new uvec4(this.v1, this.v2, this.v3, this.v0);
            }
            set {
                this.v1 = value.v0;
                this.v2 = value.v1;
                this.v3 = value.v2;
                this.v0 = value.v3;
            }
        }
        
        public uvec4 gbar {
            get {
                return new uvec4(this.v1, this.v2, this.v3, this.v0);
            }
            set {
                this.v1 = value.v0;
                this.v2 = value.v1;
                this.v3 = value.v2;
                this.v0 = value.v3;
            }
        }
        
        public uvec4 tpqs {
            get {
                return new uvec4(this.v1, this.v2, this.v3, this.v0);
            }
            set {
                this.v1 = value.v0;
                this.v2 = value.v1;
                this.v3 = value.v2;
                this.v0 = value.v3;
            }
        }
        
        public uvec4 yzwy {
            get {
                return new uvec4(this.v1, this.v2, this.v3, this.v1);
            }
        }
        
        public uvec4 gbag {
            get {
                return new uvec4(this.v1, this.v2, this.v3, this.v1);
            }
        }
        
        public uvec4 tpqt {
            get {
                return new uvec4(this.v1, this.v2, this.v3, this.v1);
            }
        }
        
        public uvec4 yzwz {
            get {
                return new uvec4(this.v1, this.v2, this.v3, this.v2);
            }
        }
        
        public uvec4 gbab {
            get {
                return new uvec4(this.v1, this.v2, this.v3, this.v2);
            }
        }
        
        public uvec4 tpqp {
            get {
                return new uvec4(this.v1, this.v2, this.v3, this.v2);
            }
        }
        
        public uvec4 yzww {
            get {
                return new uvec4(this.v1, this.v2, this.v3, this.v3);
            }
        }
        
        public uvec4 gbaa {
            get {
                return new uvec4(this.v1, this.v2, this.v3, this.v3);
            }
        }
        
        public uvec4 tpqq {
            get {
                return new uvec4(this.v1, this.v2, this.v3, this.v3);
            }
        }
        
        public uvec4 ywxx {
            get {
                return new uvec4(this.v1, this.v3, this.v0, this.v0);
            }
        }
        
        public uvec4 garr {
            get {
                return new uvec4(this.v1, this.v3, this.v0, this.v0);
            }
        }
        
        public uvec4 tqss {
            get {
                return new uvec4(this.v1, this.v3, this.v0, this.v0);
            }
        }
        
        public uvec4 ywxy {
            get {
                return new uvec4(this.v1, this.v3, this.v0, this.v1);
            }
        }
        
        public uvec4 garg {
            get {
                return new uvec4(this.v1, this.v3, this.v0, this.v1);
            }
        }
        
        public uvec4 tqst {
            get {
                return new uvec4(this.v1, this.v3, this.v0, this.v1);
            }
        }
        
        public uvec4 ywxz {
            get {
                return new uvec4(this.v1, this.v3, this.v0, this.v2);
            }
            set {
                this.v1 = value.v0;
                this.v3 = value.v1;
                this.v0 = value.v2;
                this.v2 = value.v3;
            }
        }
        
        public uvec4 garb {
            get {
                return new uvec4(this.v1, this.v3, this.v0, this.v2);
            }
            set {
                this.v1 = value.v0;
                this.v3 = value.v1;
                this.v0 = value.v2;
                this.v2 = value.v3;
            }
        }
        
        public uvec4 tqsp {
            get {
                return new uvec4(this.v1, this.v3, this.v0, this.v2);
            }
            set {
                this.v1 = value.v0;
                this.v3 = value.v1;
                this.v0 = value.v2;
                this.v2 = value.v3;
            }
        }
        
        public uvec4 ywxw {
            get {
                return new uvec4(this.v1, this.v3, this.v0, this.v3);
            }
        }
        
        public uvec4 gara {
            get {
                return new uvec4(this.v1, this.v3, this.v0, this.v3);
            }
        }
        
        public uvec4 tqsq {
            get {
                return new uvec4(this.v1, this.v3, this.v0, this.v3);
            }
        }
        
        public uvec4 ywyx {
            get {
                return new uvec4(this.v1, this.v3, this.v1, this.v0);
            }
        }
        
        public uvec4 gagr {
            get {
                return new uvec4(this.v1, this.v3, this.v1, this.v0);
            }
        }
        
        public uvec4 tqts {
            get {
                return new uvec4(this.v1, this.v3, this.v1, this.v0);
            }
        }
        
        public uvec4 ywyy {
            get {
                return new uvec4(this.v1, this.v3, this.v1, this.v1);
            }
        }
        
        public uvec4 gagg {
            get {
                return new uvec4(this.v1, this.v3, this.v1, this.v1);
            }
        }
        
        public uvec4 tqtt {
            get {
                return new uvec4(this.v1, this.v3, this.v1, this.v1);
            }
        }
        
        public uvec4 ywyz {
            get {
                return new uvec4(this.v1, this.v3, this.v1, this.v2);
            }
        }
        
        public uvec4 gagb {
            get {
                return new uvec4(this.v1, this.v3, this.v1, this.v2);
            }
        }
        
        public uvec4 tqtp {
            get {
                return new uvec4(this.v1, this.v3, this.v1, this.v2);
            }
        }
        
        public uvec4 ywyw {
            get {
                return new uvec4(this.v1, this.v3, this.v1, this.v3);
            }
        }
        
        public uvec4 gaga {
            get {
                return new uvec4(this.v1, this.v3, this.v1, this.v3);
            }
        }
        
        public uvec4 tqtq {
            get {
                return new uvec4(this.v1, this.v3, this.v1, this.v3);
            }
        }
        
        public uvec4 ywzx {
            get {
                return new uvec4(this.v1, this.v3, this.v2, this.v0);
            }
            set {
                this.v1 = value.v0;
                this.v3 = value.v1;
                this.v2 = value.v2;
                this.v0 = value.v3;
            }
        }
        
        public uvec4 gabr {
            get {
                return new uvec4(this.v1, this.v3, this.v2, this.v0);
            }
            set {
                this.v1 = value.v0;
                this.v3 = value.v1;
                this.v2 = value.v2;
                this.v0 = value.v3;
            }
        }
        
        public uvec4 tqps {
            get {
                return new uvec4(this.v1, this.v3, this.v2, this.v0);
            }
            set {
                this.v1 = value.v0;
                this.v3 = value.v1;
                this.v2 = value.v2;
                this.v0 = value.v3;
            }
        }
        
        public uvec4 ywzy {
            get {
                return new uvec4(this.v1, this.v3, this.v2, this.v1);
            }
        }
        
        public uvec4 gabg {
            get {
                return new uvec4(this.v1, this.v3, this.v2, this.v1);
            }
        }
        
        public uvec4 tqpt {
            get {
                return new uvec4(this.v1, this.v3, this.v2, this.v1);
            }
        }
        
        public uvec4 ywzz {
            get {
                return new uvec4(this.v1, this.v3, this.v2, this.v2);
            }
        }
        
        public uvec4 gabb {
            get {
                return new uvec4(this.v1, this.v3, this.v2, this.v2);
            }
        }
        
        public uvec4 tqpp {
            get {
                return new uvec4(this.v1, this.v3, this.v2, this.v2);
            }
        }
        
        public uvec4 ywzw {
            get {
                return new uvec4(this.v1, this.v3, this.v2, this.v3);
            }
        }
        
        public uvec4 gaba {
            get {
                return new uvec4(this.v1, this.v3, this.v2, this.v3);
            }
        }
        
        public uvec4 tqpq {
            get {
                return new uvec4(this.v1, this.v3, this.v2, this.v3);
            }
        }
        
        public uvec4 ywwx {
            get {
                return new uvec4(this.v1, this.v3, this.v3, this.v0);
            }
        }
        
        public uvec4 gaar {
            get {
                return new uvec4(this.v1, this.v3, this.v3, this.v0);
            }
        }
        
        public uvec4 tqqs {
            get {
                return new uvec4(this.v1, this.v3, this.v3, this.v0);
            }
        }
        
        public uvec4 ywwy {
            get {
                return new uvec4(this.v1, this.v3, this.v3, this.v1);
            }
        }
        
        public uvec4 gaag {
            get {
                return new uvec4(this.v1, this.v3, this.v3, this.v1);
            }
        }
        
        public uvec4 tqqt {
            get {
                return new uvec4(this.v1, this.v3, this.v3, this.v1);
            }
        }
        
        public uvec4 ywwz {
            get {
                return new uvec4(this.v1, this.v3, this.v3, this.v2);
            }
        }
        
        public uvec4 gaab {
            get {
                return new uvec4(this.v1, this.v3, this.v3, this.v2);
            }
        }
        
        public uvec4 tqqp {
            get {
                return new uvec4(this.v1, this.v3, this.v3, this.v2);
            }
        }
        
        public uvec4 ywww {
            get {
                return new uvec4(this.v1, this.v3, this.v3, this.v3);
            }
        }
        
        public uvec4 gaaa {
            get {
                return new uvec4(this.v1, this.v3, this.v3, this.v3);
            }
        }
        
        public uvec4 tqqq {
            get {
                return new uvec4(this.v1, this.v3, this.v3, this.v3);
            }
        }
        
        public uvec4 zxxx {
            get {
                return new uvec4(this.v2, this.v0, this.v0, this.v0);
            }
        }
        
        public uvec4 brrr {
            get {
                return new uvec4(this.v2, this.v0, this.v0, this.v0);
            }
        }
        
        public uvec4 psss {
            get {
                return new uvec4(this.v2, this.v0, this.v0, this.v0);
            }
        }
        
        public uvec4 zxxy {
            get {
                return new uvec4(this.v2, this.v0, this.v0, this.v1);
            }
        }
        
        public uvec4 brrg {
            get {
                return new uvec4(this.v2, this.v0, this.v0, this.v1);
            }
        }
        
        public uvec4 psst {
            get {
                return new uvec4(this.v2, this.v0, this.v0, this.v1);
            }
        }
        
        public uvec4 zxxz {
            get {
                return new uvec4(this.v2, this.v0, this.v0, this.v2);
            }
        }
        
        public uvec4 brrb {
            get {
                return new uvec4(this.v2, this.v0, this.v0, this.v2);
            }
        }
        
        public uvec4 pssp {
            get {
                return new uvec4(this.v2, this.v0, this.v0, this.v2);
            }
        }
        
        public uvec4 zxxw {
            get {
                return new uvec4(this.v2, this.v0, this.v0, this.v3);
            }
        }
        
        public uvec4 brra {
            get {
                return new uvec4(this.v2, this.v0, this.v0, this.v3);
            }
        }
        
        public uvec4 pssq {
            get {
                return new uvec4(this.v2, this.v0, this.v0, this.v3);
            }
        }
        
        public uvec4 zxyx {
            get {
                return new uvec4(this.v2, this.v0, this.v1, this.v0);
            }
        }
        
        public uvec4 brgr {
            get {
                return new uvec4(this.v2, this.v0, this.v1, this.v0);
            }
        }
        
        public uvec4 psts {
            get {
                return new uvec4(this.v2, this.v0, this.v1, this.v0);
            }
        }
        
        public uvec4 zxyy {
            get {
                return new uvec4(this.v2, this.v0, this.v1, this.v1);
            }
        }
        
        public uvec4 brgg {
            get {
                return new uvec4(this.v2, this.v0, this.v1, this.v1);
            }
        }
        
        public uvec4 pstt {
            get {
                return new uvec4(this.v2, this.v0, this.v1, this.v1);
            }
        }
        
        public uvec4 zxyz {
            get {
                return new uvec4(this.v2, this.v0, this.v1, this.v2);
            }
        }
        
        public uvec4 brgb {
            get {
                return new uvec4(this.v2, this.v0, this.v1, this.v2);
            }
        }
        
        public uvec4 pstp {
            get {
                return new uvec4(this.v2, this.v0, this.v1, this.v2);
            }
        }
        
        public uvec4 zxyw {
            get {
                return new uvec4(this.v2, this.v0, this.v1, this.v3);
            }
            set {
                this.v2 = value.v0;
                this.v0 = value.v1;
                this.v1 = value.v2;
                this.v3 = value.v3;
            }
        }
        
        public uvec4 brga {
            get {
                return new uvec4(this.v2, this.v0, this.v1, this.v3);
            }
            set {
                this.v2 = value.v0;
                this.v0 = value.v1;
                this.v1 = value.v2;
                this.v3 = value.v3;
            }
        }
        
        public uvec4 pstq {
            get {
                return new uvec4(this.v2, this.v0, this.v1, this.v3);
            }
            set {
                this.v2 = value.v0;
                this.v0 = value.v1;
                this.v1 = value.v2;
                this.v3 = value.v3;
            }
        }
        
        public uvec4 zxzx {
            get {
                return new uvec4(this.v2, this.v0, this.v2, this.v0);
            }
        }
        
        public uvec4 brbr {
            get {
                return new uvec4(this.v2, this.v0, this.v2, this.v0);
            }
        }
        
        public uvec4 psps {
            get {
                return new uvec4(this.v2, this.v0, this.v2, this.v0);
            }
        }
        
        public uvec4 zxzy {
            get {
                return new uvec4(this.v2, this.v0, this.v2, this.v1);
            }
        }
        
        public uvec4 brbg {
            get {
                return new uvec4(this.v2, this.v0, this.v2, this.v1);
            }
        }
        
        public uvec4 pspt {
            get {
                return new uvec4(this.v2, this.v0, this.v2, this.v1);
            }
        }
        
        public uvec4 zxzz {
            get {
                return new uvec4(this.v2, this.v0, this.v2, this.v2);
            }
        }
        
        public uvec4 brbb {
            get {
                return new uvec4(this.v2, this.v0, this.v2, this.v2);
            }
        }
        
        public uvec4 pspp {
            get {
                return new uvec4(this.v2, this.v0, this.v2, this.v2);
            }
        }
        
        public uvec4 zxzw {
            get {
                return new uvec4(this.v2, this.v0, this.v2, this.v3);
            }
        }
        
        public uvec4 brba {
            get {
                return new uvec4(this.v2, this.v0, this.v2, this.v3);
            }
        }
        
        public uvec4 pspq {
            get {
                return new uvec4(this.v2, this.v0, this.v2, this.v3);
            }
        }
        
        public uvec4 zxwx {
            get {
                return new uvec4(this.v2, this.v0, this.v3, this.v0);
            }
        }
        
        public uvec4 brar {
            get {
                return new uvec4(this.v2, this.v0, this.v3, this.v0);
            }
        }
        
        public uvec4 psqs {
            get {
                return new uvec4(this.v2, this.v0, this.v3, this.v0);
            }
        }
        
        public uvec4 zxwy {
            get {
                return new uvec4(this.v2, this.v0, this.v3, this.v1);
            }
            set {
                this.v2 = value.v0;
                this.v0 = value.v1;
                this.v3 = value.v2;
                this.v1 = value.v3;
            }
        }
        
        public uvec4 brag {
            get {
                return new uvec4(this.v2, this.v0, this.v3, this.v1);
            }
            set {
                this.v2 = value.v0;
                this.v0 = value.v1;
                this.v3 = value.v2;
                this.v1 = value.v3;
            }
        }
        
        public uvec4 psqt {
            get {
                return new uvec4(this.v2, this.v0, this.v3, this.v1);
            }
            set {
                this.v2 = value.v0;
                this.v0 = value.v1;
                this.v3 = value.v2;
                this.v1 = value.v3;
            }
        }
        
        public uvec4 zxwz {
            get {
                return new uvec4(this.v2, this.v0, this.v3, this.v2);
            }
        }
        
        public uvec4 brab {
            get {
                return new uvec4(this.v2, this.v0, this.v3, this.v2);
            }
        }
        
        public uvec4 psqp {
            get {
                return new uvec4(this.v2, this.v0, this.v3, this.v2);
            }
        }
        
        public uvec4 zxww {
            get {
                return new uvec4(this.v2, this.v0, this.v3, this.v3);
            }
        }
        
        public uvec4 braa {
            get {
                return new uvec4(this.v2, this.v0, this.v3, this.v3);
            }
        }
        
        public uvec4 psqq {
            get {
                return new uvec4(this.v2, this.v0, this.v3, this.v3);
            }
        }
        
        public uvec4 zyxx {
            get {
                return new uvec4(this.v2, this.v1, this.v0, this.v0);
            }
        }
        
        public uvec4 bgrr {
            get {
                return new uvec4(this.v2, this.v1, this.v0, this.v0);
            }
        }
        
        public uvec4 ptss {
            get {
                return new uvec4(this.v2, this.v1, this.v0, this.v0);
            }
        }
        
        public uvec4 zyxy {
            get {
                return new uvec4(this.v2, this.v1, this.v0, this.v1);
            }
        }
        
        public uvec4 bgrg {
            get {
                return new uvec4(this.v2, this.v1, this.v0, this.v1);
            }
        }
        
        public uvec4 ptst {
            get {
                return new uvec4(this.v2, this.v1, this.v0, this.v1);
            }
        }
        
        public uvec4 zyxz {
            get {
                return new uvec4(this.v2, this.v1, this.v0, this.v2);
            }
        }
        
        public uvec4 bgrb {
            get {
                return new uvec4(this.v2, this.v1, this.v0, this.v2);
            }
        }
        
        public uvec4 ptsp {
            get {
                return new uvec4(this.v2, this.v1, this.v0, this.v2);
            }
        }
        
        public uvec4 zyxw {
            get {
                return new uvec4(this.v2, this.v1, this.v0, this.v3);
            }
            set {
                this.v2 = value.v0;
                this.v1 = value.v1;
                this.v0 = value.v2;
                this.v3 = value.v3;
            }
        }
        
        public uvec4 bgra {
            get {
                return new uvec4(this.v2, this.v1, this.v0, this.v3);
            }
            set {
                this.v2 = value.v0;
                this.v1 = value.v1;
                this.v0 = value.v2;
                this.v3 = value.v3;
            }
        }
        
        public uvec4 ptsq {
            get {
                return new uvec4(this.v2, this.v1, this.v0, this.v3);
            }
            set {
                this.v2 = value.v0;
                this.v1 = value.v1;
                this.v0 = value.v2;
                this.v3 = value.v3;
            }
        }
        
        public uvec4 zyyx {
            get {
                return new uvec4(this.v2, this.v1, this.v1, this.v0);
            }
        }
        
        public uvec4 bggr {
            get {
                return new uvec4(this.v2, this.v1, this.v1, this.v0);
            }
        }
        
        public uvec4 ptts {
            get {
                return new uvec4(this.v2, this.v1, this.v1, this.v0);
            }
        }
        
        public uvec4 zyyy {
            get {
                return new uvec4(this.v2, this.v1, this.v1, this.v1);
            }
        }
        
        public uvec4 bggg {
            get {
                return new uvec4(this.v2, this.v1, this.v1, this.v1);
            }
        }
        
        public uvec4 pttt {
            get {
                return new uvec4(this.v2, this.v1, this.v1, this.v1);
            }
        }
        
        public uvec4 zyyz {
            get {
                return new uvec4(this.v2, this.v1, this.v1, this.v2);
            }
        }
        
        public uvec4 bggb {
            get {
                return new uvec4(this.v2, this.v1, this.v1, this.v2);
            }
        }
        
        public uvec4 pttp {
            get {
                return new uvec4(this.v2, this.v1, this.v1, this.v2);
            }
        }
        
        public uvec4 zyyw {
            get {
                return new uvec4(this.v2, this.v1, this.v1, this.v3);
            }
        }
        
        public uvec4 bgga {
            get {
                return new uvec4(this.v2, this.v1, this.v1, this.v3);
            }
        }
        
        public uvec4 pttq {
            get {
                return new uvec4(this.v2, this.v1, this.v1, this.v3);
            }
        }
        
        public uvec4 zyzx {
            get {
                return new uvec4(this.v2, this.v1, this.v2, this.v0);
            }
        }
        
        public uvec4 bgbr {
            get {
                return new uvec4(this.v2, this.v1, this.v2, this.v0);
            }
        }
        
        public uvec4 ptps {
            get {
                return new uvec4(this.v2, this.v1, this.v2, this.v0);
            }
        }
        
        public uvec4 zyzy {
            get {
                return new uvec4(this.v2, this.v1, this.v2, this.v1);
            }
        }
        
        public uvec4 bgbg {
            get {
                return new uvec4(this.v2, this.v1, this.v2, this.v1);
            }
        }
        
        public uvec4 ptpt {
            get {
                return new uvec4(this.v2, this.v1, this.v2, this.v1);
            }
        }
        
        public uvec4 zyzz {
            get {
                return new uvec4(this.v2, this.v1, this.v2, this.v2);
            }
        }
        
        public uvec4 bgbb {
            get {
                return new uvec4(this.v2, this.v1, this.v2, this.v2);
            }
        }
        
        public uvec4 ptpp {
            get {
                return new uvec4(this.v2, this.v1, this.v2, this.v2);
            }
        }
        
        public uvec4 zyzw {
            get {
                return new uvec4(this.v2, this.v1, this.v2, this.v3);
            }
        }
        
        public uvec4 bgba {
            get {
                return new uvec4(this.v2, this.v1, this.v2, this.v3);
            }
        }
        
        public uvec4 ptpq {
            get {
                return new uvec4(this.v2, this.v1, this.v2, this.v3);
            }
        }
        
        public uvec4 zywx {
            get {
                return new uvec4(this.v2, this.v1, this.v3, this.v0);
            }
            set {
                this.v2 = value.v0;
                this.v1 = value.v1;
                this.v3 = value.v2;
                this.v0 = value.v3;
            }
        }
        
        public uvec4 bgar {
            get {
                return new uvec4(this.v2, this.v1, this.v3, this.v0);
            }
            set {
                this.v2 = value.v0;
                this.v1 = value.v1;
                this.v3 = value.v2;
                this.v0 = value.v3;
            }
        }
        
        public uvec4 ptqs {
            get {
                return new uvec4(this.v2, this.v1, this.v3, this.v0);
            }
            set {
                this.v2 = value.v0;
                this.v1 = value.v1;
                this.v3 = value.v2;
                this.v0 = value.v3;
            }
        }
        
        public uvec4 zywy {
            get {
                return new uvec4(this.v2, this.v1, this.v3, this.v1);
            }
        }
        
        public uvec4 bgag {
            get {
                return new uvec4(this.v2, this.v1, this.v3, this.v1);
            }
        }
        
        public uvec4 ptqt {
            get {
                return new uvec4(this.v2, this.v1, this.v3, this.v1);
            }
        }
        
        public uvec4 zywz {
            get {
                return new uvec4(this.v2, this.v1, this.v3, this.v2);
            }
        }
        
        public uvec4 bgab {
            get {
                return new uvec4(this.v2, this.v1, this.v3, this.v2);
            }
        }
        
        public uvec4 ptqp {
            get {
                return new uvec4(this.v2, this.v1, this.v3, this.v2);
            }
        }
        
        public uvec4 zyww {
            get {
                return new uvec4(this.v2, this.v1, this.v3, this.v3);
            }
        }
        
        public uvec4 bgaa {
            get {
                return new uvec4(this.v2, this.v1, this.v3, this.v3);
            }
        }
        
        public uvec4 ptqq {
            get {
                return new uvec4(this.v2, this.v1, this.v3, this.v3);
            }
        }
        
        public uvec4 zzxx {
            get {
                return new uvec4(this.v2, this.v2, this.v0, this.v0);
            }
        }
        
        public uvec4 bbrr {
            get {
                return new uvec4(this.v2, this.v2, this.v0, this.v0);
            }
        }
        
        public uvec4 ppss {
            get {
                return new uvec4(this.v2, this.v2, this.v0, this.v0);
            }
        }
        
        public uvec4 zzxy {
            get {
                return new uvec4(this.v2, this.v2, this.v0, this.v1);
            }
        }
        
        public uvec4 bbrg {
            get {
                return new uvec4(this.v2, this.v2, this.v0, this.v1);
            }
        }
        
        public uvec4 ppst {
            get {
                return new uvec4(this.v2, this.v2, this.v0, this.v1);
            }
        }
        
        public uvec4 zzxz {
            get {
                return new uvec4(this.v2, this.v2, this.v0, this.v2);
            }
        }
        
        public uvec4 bbrb {
            get {
                return new uvec4(this.v2, this.v2, this.v0, this.v2);
            }
        }
        
        public uvec4 ppsp {
            get {
                return new uvec4(this.v2, this.v2, this.v0, this.v2);
            }
        }
        
        public uvec4 zzxw {
            get {
                return new uvec4(this.v2, this.v2, this.v0, this.v3);
            }
        }
        
        public uvec4 bbra {
            get {
                return new uvec4(this.v2, this.v2, this.v0, this.v3);
            }
        }
        
        public uvec4 ppsq {
            get {
                return new uvec4(this.v2, this.v2, this.v0, this.v3);
            }
        }
        
        public uvec4 zzyx {
            get {
                return new uvec4(this.v2, this.v2, this.v1, this.v0);
            }
        }
        
        public uvec4 bbgr {
            get {
                return new uvec4(this.v2, this.v2, this.v1, this.v0);
            }
        }
        
        public uvec4 ppts {
            get {
                return new uvec4(this.v2, this.v2, this.v1, this.v0);
            }
        }
        
        public uvec4 zzyy {
            get {
                return new uvec4(this.v2, this.v2, this.v1, this.v1);
            }
        }
        
        public uvec4 bbgg {
            get {
                return new uvec4(this.v2, this.v2, this.v1, this.v1);
            }
        }
        
        public uvec4 pptt {
            get {
                return new uvec4(this.v2, this.v2, this.v1, this.v1);
            }
        }
        
        public uvec4 zzyz {
            get {
                return new uvec4(this.v2, this.v2, this.v1, this.v2);
            }
        }
        
        public uvec4 bbgb {
            get {
                return new uvec4(this.v2, this.v2, this.v1, this.v2);
            }
        }
        
        public uvec4 pptp {
            get {
                return new uvec4(this.v2, this.v2, this.v1, this.v2);
            }
        }
        
        public uvec4 zzyw {
            get {
                return new uvec4(this.v2, this.v2, this.v1, this.v3);
            }
        }
        
        public uvec4 bbga {
            get {
                return new uvec4(this.v2, this.v2, this.v1, this.v3);
            }
        }
        
        public uvec4 pptq {
            get {
                return new uvec4(this.v2, this.v2, this.v1, this.v3);
            }
        }
        
        public uvec4 zzzx {
            get {
                return new uvec4(this.v2, this.v2, this.v2, this.v0);
            }
        }
        
        public uvec4 bbbr {
            get {
                return new uvec4(this.v2, this.v2, this.v2, this.v0);
            }
        }
        
        public uvec4 ppps {
            get {
                return new uvec4(this.v2, this.v2, this.v2, this.v0);
            }
        }
        
        public uvec4 zzzy {
            get {
                return new uvec4(this.v2, this.v2, this.v2, this.v1);
            }
        }
        
        public uvec4 bbbg {
            get {
                return new uvec4(this.v2, this.v2, this.v2, this.v1);
            }
        }
        
        public uvec4 pppt {
            get {
                return new uvec4(this.v2, this.v2, this.v2, this.v1);
            }
        }
        
        public uvec4 zzzz {
            get {
                return new uvec4(this.v2, this.v2, this.v2, this.v2);
            }
        }
        
        public uvec4 bbbb {
            get {
                return new uvec4(this.v2, this.v2, this.v2, this.v2);
            }
        }
        
        public uvec4 pppp {
            get {
                return new uvec4(this.v2, this.v2, this.v2, this.v2);
            }
        }
        
        public uvec4 zzzw {
            get {
                return new uvec4(this.v2, this.v2, this.v2, this.v3);
            }
        }
        
        public uvec4 bbba {
            get {
                return new uvec4(this.v2, this.v2, this.v2, this.v3);
            }
        }
        
        public uvec4 pppq {
            get {
                return new uvec4(this.v2, this.v2, this.v2, this.v3);
            }
        }
        
        public uvec4 zzwx {
            get {
                return new uvec4(this.v2, this.v2, this.v3, this.v0);
            }
        }
        
        public uvec4 bbar {
            get {
                return new uvec4(this.v2, this.v2, this.v3, this.v0);
            }
        }
        
        public uvec4 ppqs {
            get {
                return new uvec4(this.v2, this.v2, this.v3, this.v0);
            }
        }
        
        public uvec4 zzwy {
            get {
                return new uvec4(this.v2, this.v2, this.v3, this.v1);
            }
        }
        
        public uvec4 bbag {
            get {
                return new uvec4(this.v2, this.v2, this.v3, this.v1);
            }
        }
        
        public uvec4 ppqt {
            get {
                return new uvec4(this.v2, this.v2, this.v3, this.v1);
            }
        }
        
        public uvec4 zzwz {
            get {
                return new uvec4(this.v2, this.v2, this.v3, this.v2);
            }
        }
        
        public uvec4 bbab {
            get {
                return new uvec4(this.v2, this.v2, this.v3, this.v2);
            }
        }
        
        public uvec4 ppqp {
            get {
                return new uvec4(this.v2, this.v2, this.v3, this.v2);
            }
        }
        
        public uvec4 zzww {
            get {
                return new uvec4(this.v2, this.v2, this.v3, this.v3);
            }
        }
        
        public uvec4 bbaa {
            get {
                return new uvec4(this.v2, this.v2, this.v3, this.v3);
            }
        }
        
        public uvec4 ppqq {
            get {
                return new uvec4(this.v2, this.v2, this.v3, this.v3);
            }
        }
        
        public uvec4 zwxx {
            get {
                return new uvec4(this.v2, this.v3, this.v0, this.v0);
            }
        }
        
        public uvec4 barr {
            get {
                return new uvec4(this.v2, this.v3, this.v0, this.v0);
            }
        }
        
        public uvec4 pqss {
            get {
                return new uvec4(this.v2, this.v3, this.v0, this.v0);
            }
        }
        
        public uvec4 zwxy {
            get {
                return new uvec4(this.v2, this.v3, this.v0, this.v1);
            }
            set {
                this.v2 = value.v0;
                this.v3 = value.v1;
                this.v0 = value.v2;
                this.v1 = value.v3;
            }
        }
        
        public uvec4 barg {
            get {
                return new uvec4(this.v2, this.v3, this.v0, this.v1);
            }
            set {
                this.v2 = value.v0;
                this.v3 = value.v1;
                this.v0 = value.v2;
                this.v1 = value.v3;
            }
        }
        
        public uvec4 pqst {
            get {
                return new uvec4(this.v2, this.v3, this.v0, this.v1);
            }
            set {
                this.v2 = value.v0;
                this.v3 = value.v1;
                this.v0 = value.v2;
                this.v1 = value.v3;
            }
        }
        
        public uvec4 zwxz {
            get {
                return new uvec4(this.v2, this.v3, this.v0, this.v2);
            }
        }
        
        public uvec4 barb {
            get {
                return new uvec4(this.v2, this.v3, this.v0, this.v2);
            }
        }
        
        public uvec4 pqsp {
            get {
                return new uvec4(this.v2, this.v3, this.v0, this.v2);
            }
        }
        
        public uvec4 zwxw {
            get {
                return new uvec4(this.v2, this.v3, this.v0, this.v3);
            }
        }
        
        public uvec4 bara {
            get {
                return new uvec4(this.v2, this.v3, this.v0, this.v3);
            }
        }
        
        public uvec4 pqsq {
            get {
                return new uvec4(this.v2, this.v3, this.v0, this.v3);
            }
        }
        
        public uvec4 zwyx {
            get {
                return new uvec4(this.v2, this.v3, this.v1, this.v0);
            }
            set {
                this.v2 = value.v0;
                this.v3 = value.v1;
                this.v1 = value.v2;
                this.v0 = value.v3;
            }
        }
        
        public uvec4 bagr {
            get {
                return new uvec4(this.v2, this.v3, this.v1, this.v0);
            }
            set {
                this.v2 = value.v0;
                this.v3 = value.v1;
                this.v1 = value.v2;
                this.v0 = value.v3;
            }
        }
        
        public uvec4 pqts {
            get {
                return new uvec4(this.v2, this.v3, this.v1, this.v0);
            }
            set {
                this.v2 = value.v0;
                this.v3 = value.v1;
                this.v1 = value.v2;
                this.v0 = value.v3;
            }
        }
        
        public uvec4 zwyy {
            get {
                return new uvec4(this.v2, this.v3, this.v1, this.v1);
            }
        }
        
        public uvec4 bagg {
            get {
                return new uvec4(this.v2, this.v3, this.v1, this.v1);
            }
        }
        
        public uvec4 pqtt {
            get {
                return new uvec4(this.v2, this.v3, this.v1, this.v1);
            }
        }
        
        public uvec4 zwyz {
            get {
                return new uvec4(this.v2, this.v3, this.v1, this.v2);
            }
        }
        
        public uvec4 bagb {
            get {
                return new uvec4(this.v2, this.v3, this.v1, this.v2);
            }
        }
        
        public uvec4 pqtp {
            get {
                return new uvec4(this.v2, this.v3, this.v1, this.v2);
            }
        }
        
        public uvec4 zwyw {
            get {
                return new uvec4(this.v2, this.v3, this.v1, this.v3);
            }
        }
        
        public uvec4 baga {
            get {
                return new uvec4(this.v2, this.v3, this.v1, this.v3);
            }
        }
        
        public uvec4 pqtq {
            get {
                return new uvec4(this.v2, this.v3, this.v1, this.v3);
            }
        }
        
        public uvec4 zwzx {
            get {
                return new uvec4(this.v2, this.v3, this.v2, this.v0);
            }
        }
        
        public uvec4 babr {
            get {
                return new uvec4(this.v2, this.v3, this.v2, this.v0);
            }
        }
        
        public uvec4 pqps {
            get {
                return new uvec4(this.v2, this.v3, this.v2, this.v0);
            }
        }
        
        public uvec4 zwzy {
            get {
                return new uvec4(this.v2, this.v3, this.v2, this.v1);
            }
        }
        
        public uvec4 babg {
            get {
                return new uvec4(this.v2, this.v3, this.v2, this.v1);
            }
        }
        
        public uvec4 pqpt {
            get {
                return new uvec4(this.v2, this.v3, this.v2, this.v1);
            }
        }
        
        public uvec4 zwzz {
            get {
                return new uvec4(this.v2, this.v3, this.v2, this.v2);
            }
        }
        
        public uvec4 babb {
            get {
                return new uvec4(this.v2, this.v3, this.v2, this.v2);
            }
        }
        
        public uvec4 pqpp {
            get {
                return new uvec4(this.v2, this.v3, this.v2, this.v2);
            }
        }
        
        public uvec4 zwzw {
            get {
                return new uvec4(this.v2, this.v3, this.v2, this.v3);
            }
        }
        
        public uvec4 baba {
            get {
                return new uvec4(this.v2, this.v3, this.v2, this.v3);
            }
        }
        
        public uvec4 pqpq {
            get {
                return new uvec4(this.v2, this.v3, this.v2, this.v3);
            }
        }
        
        public uvec4 zwwx {
            get {
                return new uvec4(this.v2, this.v3, this.v3, this.v0);
            }
        }
        
        public uvec4 baar {
            get {
                return new uvec4(this.v2, this.v3, this.v3, this.v0);
            }
        }
        
        public uvec4 pqqs {
            get {
                return new uvec4(this.v2, this.v3, this.v3, this.v0);
            }
        }
        
        public uvec4 zwwy {
            get {
                return new uvec4(this.v2, this.v3, this.v3, this.v1);
            }
        }
        
        public uvec4 baag {
            get {
                return new uvec4(this.v2, this.v3, this.v3, this.v1);
            }
        }
        
        public uvec4 pqqt {
            get {
                return new uvec4(this.v2, this.v3, this.v3, this.v1);
            }
        }
        
        public uvec4 zwwz {
            get {
                return new uvec4(this.v2, this.v3, this.v3, this.v2);
            }
        }
        
        public uvec4 baab {
            get {
                return new uvec4(this.v2, this.v3, this.v3, this.v2);
            }
        }
        
        public uvec4 pqqp {
            get {
                return new uvec4(this.v2, this.v3, this.v3, this.v2);
            }
        }
        
        public uvec4 zwww {
            get {
                return new uvec4(this.v2, this.v3, this.v3, this.v3);
            }
        }
        
        public uvec4 baaa {
            get {
                return new uvec4(this.v2, this.v3, this.v3, this.v3);
            }
        }
        
        public uvec4 pqqq {
            get {
                return new uvec4(this.v2, this.v3, this.v3, this.v3);
            }
        }
        
        public uvec4 wxxx {
            get {
                return new uvec4(this.v3, this.v0, this.v0, this.v0);
            }
        }
        
        public uvec4 arrr {
            get {
                return new uvec4(this.v3, this.v0, this.v0, this.v0);
            }
        }
        
        public uvec4 qsss {
            get {
                return new uvec4(this.v3, this.v0, this.v0, this.v0);
            }
        }
        
        public uvec4 wxxy {
            get {
                return new uvec4(this.v3, this.v0, this.v0, this.v1);
            }
        }
        
        public uvec4 arrg {
            get {
                return new uvec4(this.v3, this.v0, this.v0, this.v1);
            }
        }
        
        public uvec4 qsst {
            get {
                return new uvec4(this.v3, this.v0, this.v0, this.v1);
            }
        }
        
        public uvec4 wxxz {
            get {
                return new uvec4(this.v3, this.v0, this.v0, this.v2);
            }
        }
        
        public uvec4 arrb {
            get {
                return new uvec4(this.v3, this.v0, this.v0, this.v2);
            }
        }
        
        public uvec4 qssp {
            get {
                return new uvec4(this.v3, this.v0, this.v0, this.v2);
            }
        }
        
        public uvec4 wxxw {
            get {
                return new uvec4(this.v3, this.v0, this.v0, this.v3);
            }
        }
        
        public uvec4 arra {
            get {
                return new uvec4(this.v3, this.v0, this.v0, this.v3);
            }
        }
        
        public uvec4 qssq {
            get {
                return new uvec4(this.v3, this.v0, this.v0, this.v3);
            }
        }
        
        public uvec4 wxyx {
            get {
                return new uvec4(this.v3, this.v0, this.v1, this.v0);
            }
        }
        
        public uvec4 argr {
            get {
                return new uvec4(this.v3, this.v0, this.v1, this.v0);
            }
        }
        
        public uvec4 qsts {
            get {
                return new uvec4(this.v3, this.v0, this.v1, this.v0);
            }
        }
        
        public uvec4 wxyy {
            get {
                return new uvec4(this.v3, this.v0, this.v1, this.v1);
            }
        }
        
        public uvec4 argg {
            get {
                return new uvec4(this.v3, this.v0, this.v1, this.v1);
            }
        }
        
        public uvec4 qstt {
            get {
                return new uvec4(this.v3, this.v0, this.v1, this.v1);
            }
        }
        
        public uvec4 wxyz {
            get {
                return new uvec4(this.v3, this.v0, this.v1, this.v2);
            }
            set {
                this.v3 = value.v0;
                this.v0 = value.v1;
                this.v1 = value.v2;
                this.v2 = value.v3;
            }
        }
        
        public uvec4 argb {
            get {
                return new uvec4(this.v3, this.v0, this.v1, this.v2);
            }
            set {
                this.v3 = value.v0;
                this.v0 = value.v1;
                this.v1 = value.v2;
                this.v2 = value.v3;
            }
        }
        
        public uvec4 qstp {
            get {
                return new uvec4(this.v3, this.v0, this.v1, this.v2);
            }
            set {
                this.v3 = value.v0;
                this.v0 = value.v1;
                this.v1 = value.v2;
                this.v2 = value.v3;
            }
        }
        
        public uvec4 wxyw {
            get {
                return new uvec4(this.v3, this.v0, this.v1, this.v3);
            }
        }
        
        public uvec4 arga {
            get {
                return new uvec4(this.v3, this.v0, this.v1, this.v3);
            }
        }
        
        public uvec4 qstq {
            get {
                return new uvec4(this.v3, this.v0, this.v1, this.v3);
            }
        }
        
        public uvec4 wxzx {
            get {
                return new uvec4(this.v3, this.v0, this.v2, this.v0);
            }
        }
        
        public uvec4 arbr {
            get {
                return new uvec4(this.v3, this.v0, this.v2, this.v0);
            }
        }
        
        public uvec4 qsps {
            get {
                return new uvec4(this.v3, this.v0, this.v2, this.v0);
            }
        }
        
        public uvec4 wxzy {
            get {
                return new uvec4(this.v3, this.v0, this.v2, this.v1);
            }
            set {
                this.v3 = value.v0;
                this.v0 = value.v1;
                this.v2 = value.v2;
                this.v1 = value.v3;
            }
        }
        
        public uvec4 arbg {
            get {
                return new uvec4(this.v3, this.v0, this.v2, this.v1);
            }
            set {
                this.v3 = value.v0;
                this.v0 = value.v1;
                this.v2 = value.v2;
                this.v1 = value.v3;
            }
        }
        
        public uvec4 qspt {
            get {
                return new uvec4(this.v3, this.v0, this.v2, this.v1);
            }
            set {
                this.v3 = value.v0;
                this.v0 = value.v1;
                this.v2 = value.v2;
                this.v1 = value.v3;
            }
        }
        
        public uvec4 wxzz {
            get {
                return new uvec4(this.v3, this.v0, this.v2, this.v2);
            }
        }
        
        public uvec4 arbb {
            get {
                return new uvec4(this.v3, this.v0, this.v2, this.v2);
            }
        }
        
        public uvec4 qspp {
            get {
                return new uvec4(this.v3, this.v0, this.v2, this.v2);
            }
        }
        
        public uvec4 wxzw {
            get {
                return new uvec4(this.v3, this.v0, this.v2, this.v3);
            }
        }
        
        public uvec4 arba {
            get {
                return new uvec4(this.v3, this.v0, this.v2, this.v3);
            }
        }
        
        public uvec4 qspq {
            get {
                return new uvec4(this.v3, this.v0, this.v2, this.v3);
            }
        }
        
        public uvec4 wxwx {
            get {
                return new uvec4(this.v3, this.v0, this.v3, this.v0);
            }
        }
        
        public uvec4 arar {
            get {
                return new uvec4(this.v3, this.v0, this.v3, this.v0);
            }
        }
        
        public uvec4 qsqs {
            get {
                return new uvec4(this.v3, this.v0, this.v3, this.v0);
            }
        }
        
        public uvec4 wxwy {
            get {
                return new uvec4(this.v3, this.v0, this.v3, this.v1);
            }
        }
        
        public uvec4 arag {
            get {
                return new uvec4(this.v3, this.v0, this.v3, this.v1);
            }
        }
        
        public uvec4 qsqt {
            get {
                return new uvec4(this.v3, this.v0, this.v3, this.v1);
            }
        }
        
        public uvec4 wxwz {
            get {
                return new uvec4(this.v3, this.v0, this.v3, this.v2);
            }
        }
        
        public uvec4 arab {
            get {
                return new uvec4(this.v3, this.v0, this.v3, this.v2);
            }
        }
        
        public uvec4 qsqp {
            get {
                return new uvec4(this.v3, this.v0, this.v3, this.v2);
            }
        }
        
        public uvec4 wxww {
            get {
                return new uvec4(this.v3, this.v0, this.v3, this.v3);
            }
        }
        
        public uvec4 araa {
            get {
                return new uvec4(this.v3, this.v0, this.v3, this.v3);
            }
        }
        
        public uvec4 qsqq {
            get {
                return new uvec4(this.v3, this.v0, this.v3, this.v3);
            }
        }
        
        public uvec4 wyxx {
            get {
                return new uvec4(this.v3, this.v1, this.v0, this.v0);
            }
        }
        
        public uvec4 agrr {
            get {
                return new uvec4(this.v3, this.v1, this.v0, this.v0);
            }
        }
        
        public uvec4 qtss {
            get {
                return new uvec4(this.v3, this.v1, this.v0, this.v0);
            }
        }
        
        public uvec4 wyxy {
            get {
                return new uvec4(this.v3, this.v1, this.v0, this.v1);
            }
        }
        
        public uvec4 agrg {
            get {
                return new uvec4(this.v3, this.v1, this.v0, this.v1);
            }
        }
        
        public uvec4 qtst {
            get {
                return new uvec4(this.v3, this.v1, this.v0, this.v1);
            }
        }
        
        public uvec4 wyxz {
            get {
                return new uvec4(this.v3, this.v1, this.v0, this.v2);
            }
            set {
                this.v3 = value.v0;
                this.v1 = value.v1;
                this.v0 = value.v2;
                this.v2 = value.v3;
            }
        }
        
        public uvec4 agrb {
            get {
                return new uvec4(this.v3, this.v1, this.v0, this.v2);
            }
            set {
                this.v3 = value.v0;
                this.v1 = value.v1;
                this.v0 = value.v2;
                this.v2 = value.v3;
            }
        }
        
        public uvec4 qtsp {
            get {
                return new uvec4(this.v3, this.v1, this.v0, this.v2);
            }
            set {
                this.v3 = value.v0;
                this.v1 = value.v1;
                this.v0 = value.v2;
                this.v2 = value.v3;
            }
        }
        
        public uvec4 wyxw {
            get {
                return new uvec4(this.v3, this.v1, this.v0, this.v3);
            }
        }
        
        public uvec4 agra {
            get {
                return new uvec4(this.v3, this.v1, this.v0, this.v3);
            }
        }
        
        public uvec4 qtsq {
            get {
                return new uvec4(this.v3, this.v1, this.v0, this.v3);
            }
        }
        
        public uvec4 wyyx {
            get {
                return new uvec4(this.v3, this.v1, this.v1, this.v0);
            }
        }
        
        public uvec4 aggr {
            get {
                return new uvec4(this.v3, this.v1, this.v1, this.v0);
            }
        }
        
        public uvec4 qtts {
            get {
                return new uvec4(this.v3, this.v1, this.v1, this.v0);
            }
        }
        
        public uvec4 wyyy {
            get {
                return new uvec4(this.v3, this.v1, this.v1, this.v1);
            }
        }
        
        public uvec4 aggg {
            get {
                return new uvec4(this.v3, this.v1, this.v1, this.v1);
            }
        }
        
        public uvec4 qttt {
            get {
                return new uvec4(this.v3, this.v1, this.v1, this.v1);
            }
        }
        
        public uvec4 wyyz {
            get {
                return new uvec4(this.v3, this.v1, this.v1, this.v2);
            }
        }
        
        public uvec4 aggb {
            get {
                return new uvec4(this.v3, this.v1, this.v1, this.v2);
            }
        }
        
        public uvec4 qttp {
            get {
                return new uvec4(this.v3, this.v1, this.v1, this.v2);
            }
        }
        
        public uvec4 wyyw {
            get {
                return new uvec4(this.v3, this.v1, this.v1, this.v3);
            }
        }
        
        public uvec4 agga {
            get {
                return new uvec4(this.v3, this.v1, this.v1, this.v3);
            }
        }
        
        public uvec4 qttq {
            get {
                return new uvec4(this.v3, this.v1, this.v1, this.v3);
            }
        }
        
        public uvec4 wyzx {
            get {
                return new uvec4(this.v3, this.v1, this.v2, this.v0);
            }
            set {
                this.v3 = value.v0;
                this.v1 = value.v1;
                this.v2 = value.v2;
                this.v0 = value.v3;
            }
        }
        
        public uvec4 agbr {
            get {
                return new uvec4(this.v3, this.v1, this.v2, this.v0);
            }
            set {
                this.v3 = value.v0;
                this.v1 = value.v1;
                this.v2 = value.v2;
                this.v0 = value.v3;
            }
        }
        
        public uvec4 qtps {
            get {
                return new uvec4(this.v3, this.v1, this.v2, this.v0);
            }
            set {
                this.v3 = value.v0;
                this.v1 = value.v1;
                this.v2 = value.v2;
                this.v0 = value.v3;
            }
        }
        
        public uvec4 wyzy {
            get {
                return new uvec4(this.v3, this.v1, this.v2, this.v1);
            }
        }
        
        public uvec4 agbg {
            get {
                return new uvec4(this.v3, this.v1, this.v2, this.v1);
            }
        }
        
        public uvec4 qtpt {
            get {
                return new uvec4(this.v3, this.v1, this.v2, this.v1);
            }
        }
        
        public uvec4 wyzz {
            get {
                return new uvec4(this.v3, this.v1, this.v2, this.v2);
            }
        }
        
        public uvec4 agbb {
            get {
                return new uvec4(this.v3, this.v1, this.v2, this.v2);
            }
        }
        
        public uvec4 qtpp {
            get {
                return new uvec4(this.v3, this.v1, this.v2, this.v2);
            }
        }
        
        public uvec4 wyzw {
            get {
                return new uvec4(this.v3, this.v1, this.v2, this.v3);
            }
        }
        
        public uvec4 agba {
            get {
                return new uvec4(this.v3, this.v1, this.v2, this.v3);
            }
        }
        
        public uvec4 qtpq {
            get {
                return new uvec4(this.v3, this.v1, this.v2, this.v3);
            }
        }
        
        public uvec4 wywx {
            get {
                return new uvec4(this.v3, this.v1, this.v3, this.v0);
            }
        }
        
        public uvec4 agar {
            get {
                return new uvec4(this.v3, this.v1, this.v3, this.v0);
            }
        }
        
        public uvec4 qtqs {
            get {
                return new uvec4(this.v3, this.v1, this.v3, this.v0);
            }
        }
        
        public uvec4 wywy {
            get {
                return new uvec4(this.v3, this.v1, this.v3, this.v1);
            }
        }
        
        public uvec4 agag {
            get {
                return new uvec4(this.v3, this.v1, this.v3, this.v1);
            }
        }
        
        public uvec4 qtqt {
            get {
                return new uvec4(this.v3, this.v1, this.v3, this.v1);
            }
        }
        
        public uvec4 wywz {
            get {
                return new uvec4(this.v3, this.v1, this.v3, this.v2);
            }
        }
        
        public uvec4 agab {
            get {
                return new uvec4(this.v3, this.v1, this.v3, this.v2);
            }
        }
        
        public uvec4 qtqp {
            get {
                return new uvec4(this.v3, this.v1, this.v3, this.v2);
            }
        }
        
        public uvec4 wyww {
            get {
                return new uvec4(this.v3, this.v1, this.v3, this.v3);
            }
        }
        
        public uvec4 agaa {
            get {
                return new uvec4(this.v3, this.v1, this.v3, this.v3);
            }
        }
        
        public uvec4 qtqq {
            get {
                return new uvec4(this.v3, this.v1, this.v3, this.v3);
            }
        }
        
        public uvec4 wzxx {
            get {
                return new uvec4(this.v3, this.v2, this.v0, this.v0);
            }
        }
        
        public uvec4 abrr {
            get {
                return new uvec4(this.v3, this.v2, this.v0, this.v0);
            }
        }
        
        public uvec4 qpss {
            get {
                return new uvec4(this.v3, this.v2, this.v0, this.v0);
            }
        }
        
        public uvec4 wzxy {
            get {
                return new uvec4(this.v3, this.v2, this.v0, this.v1);
            }
            set {
                this.v3 = value.v0;
                this.v2 = value.v1;
                this.v0 = value.v2;
                this.v1 = value.v3;
            }
        }
        
        public uvec4 abrg {
            get {
                return new uvec4(this.v3, this.v2, this.v0, this.v1);
            }
            set {
                this.v3 = value.v0;
                this.v2 = value.v1;
                this.v0 = value.v2;
                this.v1 = value.v3;
            }
        }
        
        public uvec4 qpst {
            get {
                return new uvec4(this.v3, this.v2, this.v0, this.v1);
            }
            set {
                this.v3 = value.v0;
                this.v2 = value.v1;
                this.v0 = value.v2;
                this.v1 = value.v3;
            }
        }
        
        public uvec4 wzxz {
            get {
                return new uvec4(this.v3, this.v2, this.v0, this.v2);
            }
        }
        
        public uvec4 abrb {
            get {
                return new uvec4(this.v3, this.v2, this.v0, this.v2);
            }
        }
        
        public uvec4 qpsp {
            get {
                return new uvec4(this.v3, this.v2, this.v0, this.v2);
            }
        }
        
        public uvec4 wzxw {
            get {
                return new uvec4(this.v3, this.v2, this.v0, this.v3);
            }
        }
        
        public uvec4 abra {
            get {
                return new uvec4(this.v3, this.v2, this.v0, this.v3);
            }
        }
        
        public uvec4 qpsq {
            get {
                return new uvec4(this.v3, this.v2, this.v0, this.v3);
            }
        }
        
        public uvec4 wzyx {
            get {
                return new uvec4(this.v3, this.v2, this.v1, this.v0);
            }
            set {
                this.v3 = value.v0;
                this.v2 = value.v1;
                this.v1 = value.v2;
                this.v0 = value.v3;
            }
        }
        
        public uvec4 abgr {
            get {
                return new uvec4(this.v3, this.v2, this.v1, this.v0);
            }
            set {
                this.v3 = value.v0;
                this.v2 = value.v1;
                this.v1 = value.v2;
                this.v0 = value.v3;
            }
        }
        
        public uvec4 qpts {
            get {
                return new uvec4(this.v3, this.v2, this.v1, this.v0);
            }
            set {
                this.v3 = value.v0;
                this.v2 = value.v1;
                this.v1 = value.v2;
                this.v0 = value.v3;
            }
        }
        
        public uvec4 wzyy {
            get {
                return new uvec4(this.v3, this.v2, this.v1, this.v1);
            }
        }
        
        public uvec4 abgg {
            get {
                return new uvec4(this.v3, this.v2, this.v1, this.v1);
            }
        }
        
        public uvec4 qptt {
            get {
                return new uvec4(this.v3, this.v2, this.v1, this.v1);
            }
        }
        
        public uvec4 wzyz {
            get {
                return new uvec4(this.v3, this.v2, this.v1, this.v2);
            }
        }
        
        public uvec4 abgb {
            get {
                return new uvec4(this.v3, this.v2, this.v1, this.v2);
            }
        }
        
        public uvec4 qptp {
            get {
                return new uvec4(this.v3, this.v2, this.v1, this.v2);
            }
        }
        
        public uvec4 wzyw {
            get {
                return new uvec4(this.v3, this.v2, this.v1, this.v3);
            }
        }
        
        public uvec4 abga {
            get {
                return new uvec4(this.v3, this.v2, this.v1, this.v3);
            }
        }
        
        public uvec4 qptq {
            get {
                return new uvec4(this.v3, this.v2, this.v1, this.v3);
            }
        }
        
        public uvec4 wzzx {
            get {
                return new uvec4(this.v3, this.v2, this.v2, this.v0);
            }
        }
        
        public uvec4 abbr {
            get {
                return new uvec4(this.v3, this.v2, this.v2, this.v0);
            }
        }
        
        public uvec4 qpps {
            get {
                return new uvec4(this.v3, this.v2, this.v2, this.v0);
            }
        }
        
        public uvec4 wzzy {
            get {
                return new uvec4(this.v3, this.v2, this.v2, this.v1);
            }
        }
        
        public uvec4 abbg {
            get {
                return new uvec4(this.v3, this.v2, this.v2, this.v1);
            }
        }
        
        public uvec4 qppt {
            get {
                return new uvec4(this.v3, this.v2, this.v2, this.v1);
            }
        }
        
        public uvec4 wzzz {
            get {
                return new uvec4(this.v3, this.v2, this.v2, this.v2);
            }
        }
        
        public uvec4 abbb {
            get {
                return new uvec4(this.v3, this.v2, this.v2, this.v2);
            }
        }
        
        public uvec4 qppp {
            get {
                return new uvec4(this.v3, this.v2, this.v2, this.v2);
            }
        }
        
        public uvec4 wzzw {
            get {
                return new uvec4(this.v3, this.v2, this.v2, this.v3);
            }
        }
        
        public uvec4 abba {
            get {
                return new uvec4(this.v3, this.v2, this.v2, this.v3);
            }
        }
        
        public uvec4 qppq {
            get {
                return new uvec4(this.v3, this.v2, this.v2, this.v3);
            }
        }
        
        public uvec4 wzwx {
            get {
                return new uvec4(this.v3, this.v2, this.v3, this.v0);
            }
        }
        
        public uvec4 abar {
            get {
                return new uvec4(this.v3, this.v2, this.v3, this.v0);
            }
        }
        
        public uvec4 qpqs {
            get {
                return new uvec4(this.v3, this.v2, this.v3, this.v0);
            }
        }
        
        public uvec4 wzwy {
            get {
                return new uvec4(this.v3, this.v2, this.v3, this.v1);
            }
        }
        
        public uvec4 abag {
            get {
                return new uvec4(this.v3, this.v2, this.v3, this.v1);
            }
        }
        
        public uvec4 qpqt {
            get {
                return new uvec4(this.v3, this.v2, this.v3, this.v1);
            }
        }
        
        public uvec4 wzwz {
            get {
                return new uvec4(this.v3, this.v2, this.v3, this.v2);
            }
        }
        
        public uvec4 abab {
            get {
                return new uvec4(this.v3, this.v2, this.v3, this.v2);
            }
        }
        
        public uvec4 qpqp {
            get {
                return new uvec4(this.v3, this.v2, this.v3, this.v2);
            }
        }
        
        public uvec4 wzww {
            get {
                return new uvec4(this.v3, this.v2, this.v3, this.v3);
            }
        }
        
        public uvec4 abaa {
            get {
                return new uvec4(this.v3, this.v2, this.v3, this.v3);
            }
        }
        
        public uvec4 qpqq {
            get {
                return new uvec4(this.v3, this.v2, this.v3, this.v3);
            }
        }
        
        public uvec4 wwxx {
            get {
                return new uvec4(this.v3, this.v3, this.v0, this.v0);
            }
        }
        
        public uvec4 aarr {
            get {
                return new uvec4(this.v3, this.v3, this.v0, this.v0);
            }
        }
        
        public uvec4 qqss {
            get {
                return new uvec4(this.v3, this.v3, this.v0, this.v0);
            }
        }
        
        public uvec4 wwxy {
            get {
                return new uvec4(this.v3, this.v3, this.v0, this.v1);
            }
        }
        
        public uvec4 aarg {
            get {
                return new uvec4(this.v3, this.v3, this.v0, this.v1);
            }
        }
        
        public uvec4 qqst {
            get {
                return new uvec4(this.v3, this.v3, this.v0, this.v1);
            }
        }
        
        public uvec4 wwxz {
            get {
                return new uvec4(this.v3, this.v3, this.v0, this.v2);
            }
        }
        
        public uvec4 aarb {
            get {
                return new uvec4(this.v3, this.v3, this.v0, this.v2);
            }
        }
        
        public uvec4 qqsp {
            get {
                return new uvec4(this.v3, this.v3, this.v0, this.v2);
            }
        }
        
        public uvec4 wwxw {
            get {
                return new uvec4(this.v3, this.v3, this.v0, this.v3);
            }
        }
        
        public uvec4 aara {
            get {
                return new uvec4(this.v3, this.v3, this.v0, this.v3);
            }
        }
        
        public uvec4 qqsq {
            get {
                return new uvec4(this.v3, this.v3, this.v0, this.v3);
            }
        }
        
        public uvec4 wwyx {
            get {
                return new uvec4(this.v3, this.v3, this.v1, this.v0);
            }
        }
        
        public uvec4 aagr {
            get {
                return new uvec4(this.v3, this.v3, this.v1, this.v0);
            }
        }
        
        public uvec4 qqts {
            get {
                return new uvec4(this.v3, this.v3, this.v1, this.v0);
            }
        }
        
        public uvec4 wwyy {
            get {
                return new uvec4(this.v3, this.v3, this.v1, this.v1);
            }
        }
        
        public uvec4 aagg {
            get {
                return new uvec4(this.v3, this.v3, this.v1, this.v1);
            }
        }
        
        public uvec4 qqtt {
            get {
                return new uvec4(this.v3, this.v3, this.v1, this.v1);
            }
        }
        
        public uvec4 wwyz {
            get {
                return new uvec4(this.v3, this.v3, this.v1, this.v2);
            }
        }
        
        public uvec4 aagb {
            get {
                return new uvec4(this.v3, this.v3, this.v1, this.v2);
            }
        }
        
        public uvec4 qqtp {
            get {
                return new uvec4(this.v3, this.v3, this.v1, this.v2);
            }
        }
        
        public uvec4 wwyw {
            get {
                return new uvec4(this.v3, this.v3, this.v1, this.v3);
            }
        }
        
        public uvec4 aaga {
            get {
                return new uvec4(this.v3, this.v3, this.v1, this.v3);
            }
        }
        
        public uvec4 qqtq {
            get {
                return new uvec4(this.v3, this.v3, this.v1, this.v3);
            }
        }
        
        public uvec4 wwzx {
            get {
                return new uvec4(this.v3, this.v3, this.v2, this.v0);
            }
        }
        
        public uvec4 aabr {
            get {
                return new uvec4(this.v3, this.v3, this.v2, this.v0);
            }
        }
        
        public uvec4 qqps {
            get {
                return new uvec4(this.v3, this.v3, this.v2, this.v0);
            }
        }
        
        public uvec4 wwzy {
            get {
                return new uvec4(this.v3, this.v3, this.v2, this.v1);
            }
        }
        
        public uvec4 aabg {
            get {
                return new uvec4(this.v3, this.v3, this.v2, this.v1);
            }
        }
        
        public uvec4 qqpt {
            get {
                return new uvec4(this.v3, this.v3, this.v2, this.v1);
            }
        }
        
        public uvec4 wwzz {
            get {
                return new uvec4(this.v3, this.v3, this.v2, this.v2);
            }
        }
        
        public uvec4 aabb {
            get {
                return new uvec4(this.v3, this.v3, this.v2, this.v2);
            }
        }
        
        public uvec4 qqpp {
            get {
                return new uvec4(this.v3, this.v3, this.v2, this.v2);
            }
        }
        
        public uvec4 wwzw {
            get {
                return new uvec4(this.v3, this.v3, this.v2, this.v3);
            }
        }
        
        public uvec4 aaba {
            get {
                return new uvec4(this.v3, this.v3, this.v2, this.v3);
            }
        }
        
        public uvec4 qqpq {
            get {
                return new uvec4(this.v3, this.v3, this.v2, this.v3);
            }
        }
        
        public uvec4 wwwx {
            get {
                return new uvec4(this.v3, this.v3, this.v3, this.v0);
            }
        }
        
        public uvec4 aaar {
            get {
                return new uvec4(this.v3, this.v3, this.v3, this.v0);
            }
        }
        
        public uvec4 qqqs {
            get {
                return new uvec4(this.v3, this.v3, this.v3, this.v0);
            }
        }
        
        public uvec4 wwwy {
            get {
                return new uvec4(this.v3, this.v3, this.v3, this.v1);
            }
        }
        
        public uvec4 aaag {
            get {
                return new uvec4(this.v3, this.v3, this.v3, this.v1);
            }
        }
        
        public uvec4 qqqt {
            get {
                return new uvec4(this.v3, this.v3, this.v3, this.v1);
            }
        }
        
        public uvec4 wwwz {
            get {
                return new uvec4(this.v3, this.v3, this.v3, this.v2);
            }
        }
        
        public uvec4 aaab {
            get {
                return new uvec4(this.v3, this.v3, this.v3, this.v2);
            }
        }
        
        public uvec4 qqqp {
            get {
                return new uvec4(this.v3, this.v3, this.v3, this.v2);
            }
        }
        
        public uvec4 wwww {
            get {
                return new uvec4(this.v3, this.v3, this.v3, this.v3);
            }
        }
        
        public uvec4 aaaa {
            get {
                return new uvec4(this.v3, this.v3, this.v3, this.v3);
            }
        }
        
        public uvec4 qqqq {
            get {
                return new uvec4(this.v3, this.v3, this.v3, this.v3);
            }
        }
        
        public static uvec4 operator+(uvec4 v) {
            return new uvec4(+v.v0, +v.v1, +v.v2, +v.v3);
        }
        
        public static uvec4 operator~(uvec4 v) {
            return new uvec4(~v.v0, ~v.v1, ~v.v2, ~v.v3);
        }
        
        public static uvec4 operator++(uvec4 v) {
            return new uvec4(++v.v0, ++v.v1, ++v.v2, ++v.v3);
        }
        
        public static uvec4 operator--(uvec4 v) {
            return new uvec4(--v.v0, --v.v1, --v.v2, --v.v3);
        }
        
        public static uvec4 operator+(uint s, uvec4 v2) {
            return new uvec4(s + v2.v0, s + v2.v1, s + v2.v2, s + v2.v3);
        }
        
        public static uvec4 operator-(uint s, uvec4 v2) {
            return new uvec4(s - v2.v0, s - v2.v1, s - v2.v2, s - v2.v3);
        }
        
        public static uvec4 operator*(uint s, uvec4 v2) {
            return new uvec4(s * v2.v0, s * v2.v1, s * v2.v2, s * v2.v3);
        }
        
        public static uvec4 operator/(uint s, uvec4 v2) {
            return new uvec4(s / v2.v0, s / v2.v1, s / v2.v2, s / v2.v3);
        }
        
        public static uvec4 operator%(uint s, uvec4 v2) {
            return new uvec4(s % v2.v0, s % v2.v1, s % v2.v2, s % v2.v3);
        }
        
        public static uvec4 operator&(uint s, uvec4 v2) {
            return new uvec4(s & v2.v0, s & v2.v1, s & v2.v2, s & v2.v3);
        }
        
        public static uvec4 operator|(uint s, uvec4 v2) {
            return new uvec4(s | v2.v0, s | v2.v1, s | v2.v2, s | v2.v3);
        }
        
        public static uvec4 operator^(uint s, uvec4 v2) {
            return new uvec4(s ^ v2.v0, s ^ v2.v1, s ^ v2.v2, s ^ v2.v3);
        }
        
        public static uvec4 operator+(uvec4 v1, uint s) {
            return new uvec4(v1.v0 + s, v1.v1 + s, v1.v2 + s, v1.v3 + s);
        }
        
        public static uvec4 operator-(uvec4 v1, uint s) {
            return new uvec4(v1.v0 - s, v1.v1 - s, v1.v2 - s, v1.v3 - s);
        }
        
        public static uvec4 operator*(uvec4 v1, uint s) {
            return new uvec4(v1.v0 * s, v1.v1 * s, v1.v2 * s, v1.v3 * s);
        }
        
        public static uvec4 operator/(uvec4 v1, uint s) {
            return new uvec4(v1.v0 / s, v1.v1 / s, v1.v2 / s, v1.v3 / s);
        }
        
        public static uvec4 operator%(uvec4 v1, uint s) {
            return new uvec4(v1.v0 % s, v1.v1 % s, v1.v2 % s, v1.v3 % s);
        }
        
        public static uvec4 operator&(uvec4 v1, uint s) {
            return new uvec4(v1.v0 & s, v1.v1 & s, v1.v2 & s, v1.v3 & s);
        }
        
        public static uvec4 operator|(uvec4 v1, uint s) {
            return new uvec4(v1.v0 | s, v1.v1 | s, v1.v2 | s, v1.v3 | s);
        }
        
        public static uvec4 operator^(uvec4 v1, uint s) {
            return new uvec4(v1.v0 ^ s, v1.v1 ^ s, v1.v2 ^ s, v1.v3 ^ s);
        }
        
        public static uvec4 operator+(uvec4 v1, uvec4 v2) {
            return new uvec4(v1.v0 + v2.v0, v1.v1 + v2.v1, v1.v2 + v2.v2, v1.v3 + v2.v3);
        }
        
        public static uvec4 operator-(uvec4 v1, uvec4 v2) {
            return new uvec4(v1.v0 - v2.v0, v1.v1 - v2.v1, v1.v2 - v2.v2, v1.v3 - v2.v3);
        }
        
        public static uvec4 operator*(uvec4 v1, uvec4 v2) {
            return new uvec4(v1.v0 * v2.v0, v1.v1 * v2.v1, v1.v2 * v2.v2, v1.v3 * v2.v3);
        }
        
        public static uvec4 operator/(uvec4 v1, uvec4 v2) {
            return new uvec4(v1.v0 / v2.v0, v1.v1 / v2.v1, v1.v2 / v2.v2, v1.v3 / v2.v3);
        }
        
        public static uvec4 operator%(uvec4 v1, uvec4 v2) {
            return new uvec4(v1.v0 % v2.v0, v1.v1 % v2.v1, v1.v2 % v2.v2, v1.v3 % v2.v3);
        }
        
        public static uvec4 operator&(uvec4 v1, uvec4 v2) {
            return new uvec4(v1.v0 & v2.v0, v1.v1 & v2.v1, v1.v2 & v2.v2, v1.v3 & v2.v3);
        }
        
        public static uvec4 operator|(uvec4 v1, uvec4 v2) {
            return new uvec4(v1.v0 | v2.v0, v1.v1 | v2.v1, v1.v2 | v2.v2, v1.v3 | v2.v3);
        }
        
        public static uvec4 operator^(uvec4 v1, uvec4 v2) {
            return new uvec4(v1.v0 ^ v2.v0, v1.v1 ^ v2.v1, v1.v2 ^ v2.v2, v1.v3 ^ v2.v3);
        }
        
        public static bvec4 operator==(uvec4 v1, uvec4 v2) {
            return new bvec4(v1.v0 == v2.v0, v1.v1 == v2.v1, v1.v2 == v2.v2, v1.v3 == v2.v3);
        }
        
        public static bvec4 operator>(uvec4 v1, uvec4 v2) {
            return new bvec4(v1.v0 > v2.v0, v1.v1 > v2.v1, v1.v2 > v2.v2, v1.v3 > v2.v3);
        }
        
        public static bvec4 operator>=(uvec4 v1, uvec4 v2) {
            return new bvec4(v1.v0 >= v2.v0, v1.v1 >= v2.v1, v1.v2 >= v2.v2, v1.v3 >= v2.v3);
        }
        
        public static bvec4 operator<(uvec4 v1, uvec4 v2) {
            return new bvec4(v1.v0 < v2.v0, v1.v1 < v2.v1, v1.v2 < v2.v2, v1.v3 < v2.v3);
        }
        
        public static bvec4 operator<=(uvec4 v1, uvec4 v2) {
            return new bvec4(v1.v0 <= v2.v0, v1.v1 <= v2.v1, v1.v2 <= v2.v2, v1.v3 <= v2.v3);
        }
        
        public static bvec4 operator!=(uvec4 v1, uvec4 v2) {
            return new bvec4(v1.v0 != v2.v0, v1.v1 != v2.v1, v1.v2 != v2.v2, v1.v3 != v2.v3);
        }
        
        public override bool Equals(object obj) {
            return (obj is uvec4) ? bmath.All(this == (uvec4)obj) : false;
        }
        
        public override int GetHashCode() {
            return this.v0.GetHashCode() ^ this.v1.GetHashCode() ^ this.v2.GetHashCode() ^ this.v3.GetHashCode();
        }
        
        public override string ToString() {
            return "uvec4(" + this.v0 + "," + this.v1 + "," + this.v2 + "," + this.v3 + ")";
        }
    }
}
