#region --- License ---
/*
Copyright 2010 Daniel Filonik

Permission is hereby granted, free of charge, to any person obtaining a copy of
this software and associated documentation files (the "Software"), to deal in
the Software without restriction, including without limitation the rights to
use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
of the Software, and to permit persons to whom the Software is furnished to do
so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all
copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
SOFTWARE.
*/
#endregion

namespace TKM {
    using System;
    using System.Runtime.InteropServices;
    using System.Runtime.Serialization;
    
    
    [DataContract()]
    [StructLayout(LayoutKind.Sequential)]
    public struct vec2 {
        
        [DataMember()]
        internal float v0;
        
        [DataMember()]
        internal float v1;
        
        public vec2(float s) {
            this.v0 = s;
            this.v1 = s;
        }
        
        public vec2(vec2 v0) {
            this.v0 = v0.v0;
            this.v1 = v0.v1;
        }
        
        public vec2(float v0, float v1) {
            this.v0 = v0;
            this.v1 = v1;
        }
        
        public vec2(vec3 s) {
            this.v0 = s.v0;
            this.v1 = s.v1;
        }
        
        public vec2(vec4 s) {
            this.v0 = s.v0;
            this.v1 = s.v1;
        }
        
        public float this[int i0] {
            get {
                if (((i0 < 0) 
                            || (1 < i0))) {
                    throw new System.IndexOutOfRangeException("i0");
                }
                
                unsafe
                {
                    fixed (Single* v_ptr = &this.v0)
                    {
                        return *(v_ptr + (i0));
                    }
                };
            }
            set {
                if (((i0 < 0) 
                            || (1 < i0))) {
                    throw new System.IndexOutOfRangeException("i0");
                }
                
                unsafe
                {
                    fixed (Single* v_ptr = &this.v0)
                    {
                        *(v_ptr  + (i0)) = value;
                    }
                };
            }
        }
        
        public float x {
            get {
                return this.v0;
            }
            set {
                this.v0 = value;
            }
        }
        
        public float r {
            get {
                return this.v0;
            }
            set {
                this.v0 = value;
            }
        }
        
        public float s {
            get {
                return this.v0;
            }
            set {
                this.v0 = value;
            }
        }
        
        public float y {
            get {
                return this.v1;
            }
            set {
                this.v1 = value;
            }
        }
        
        public float g {
            get {
                return this.v1;
            }
            set {
                this.v1 = value;
            }
        }
        
        public float t {
            get {
                return this.v1;
            }
            set {
                this.v1 = value;
            }
        }
        
        public vec2 xx {
            get {
                return new vec2(this.v0, this.v0);
            }
        }
        
        public vec2 rr {
            get {
                return new vec2(this.v0, this.v0);
            }
        }
        
        public vec2 ss {
            get {
                return new vec2(this.v0, this.v0);
            }
        }
        
        public vec2 xy {
            get {
                return new vec2(this.v0, this.v1);
            }
            set {
                this.v0 = value.v0;
                this.v1 = value.v1;
            }
        }
        
        public vec2 rg {
            get {
                return new vec2(this.v0, this.v1);
            }
            set {
                this.v0 = value.v0;
                this.v1 = value.v1;
            }
        }
        
        public vec2 st {
            get {
                return new vec2(this.v0, this.v1);
            }
            set {
                this.v0 = value.v0;
                this.v1 = value.v1;
            }
        }
        
        public vec2 yx {
            get {
                return new vec2(this.v1, this.v0);
            }
            set {
                this.v1 = value.v0;
                this.v0 = value.v1;
            }
        }
        
        public vec2 gr {
            get {
                return new vec2(this.v1, this.v0);
            }
            set {
                this.v1 = value.v0;
                this.v0 = value.v1;
            }
        }
        
        public vec2 ts {
            get {
                return new vec2(this.v1, this.v0);
            }
            set {
                this.v1 = value.v0;
                this.v0 = value.v1;
            }
        }
        
        public vec2 yy {
            get {
                return new vec2(this.v1, this.v1);
            }
        }
        
        public vec2 gg {
            get {
                return new vec2(this.v1, this.v1);
            }
        }
        
        public vec2 tt {
            get {
                return new vec2(this.v1, this.v1);
            }
        }
        
        public static vec2 operator+(vec2 v) {
            return new vec2(+v.v0, +v.v1);
        }
        
        public static vec2 operator-(vec2 v) {
            return new vec2(-v.v0, -v.v1);
        }
        
        public static vec2 operator++(vec2 v) {
            return new vec2(++v.v0, ++v.v1);
        }
        
        public static vec2 operator--(vec2 v) {
            return new vec2(--v.v0, --v.v1);
        }
        
        public static vec2 operator+(float s, vec2 v2) {
            return new vec2(s + v2.v0, s + v2.v1);
        }
        
        public static vec2 operator-(float s, vec2 v2) {
            return new vec2(s - v2.v0, s - v2.v1);
        }
        
        public static vec2 operator*(float s, vec2 v2) {
            return new vec2(s * v2.v0, s * v2.v1);
        }
        
        public static vec2 operator/(float s, vec2 v2) {
            return new vec2(s / v2.v0, s / v2.v1);
        }
        
        public static vec2 operator%(float s, vec2 v2) {
            return new vec2(s % v2.v0, s % v2.v1);
        }
        
        public static vec2 operator+(vec2 v1, float s) {
            return new vec2(v1.v0 + s, v1.v1 + s);
        }
        
        public static vec2 operator-(vec2 v1, float s) {
            return new vec2(v1.v0 - s, v1.v1 - s);
        }
        
        public static vec2 operator*(vec2 v1, float s) {
            return new vec2(v1.v0 * s, v1.v1 * s);
        }
        
        public static vec2 operator/(vec2 v1, float s) {
            return new vec2(v1.v0 / s, v1.v1 / s);
        }
        
        public static vec2 operator%(vec2 v1, float s) {
            return new vec2(v1.v0 % s, v1.v1 % s);
        }
        
        public static vec2 operator+(vec2 v1, vec2 v2) {
            return new vec2(v1.v0 + v2.v0, v1.v1 + v2.v1);
        }
        
        public static vec2 operator-(vec2 v1, vec2 v2) {
            return new vec2(v1.v0 - v2.v0, v1.v1 - v2.v1);
        }
        
        public static vec2 operator*(vec2 v1, vec2 v2) {
            return new vec2(v1.v0 * v2.v0, v1.v1 * v2.v1);
        }
        
        public static vec2 operator/(vec2 v1, vec2 v2) {
            return new vec2(v1.v0 / v2.v0, v1.v1 / v2.v1);
        }
        
        public static vec2 operator%(vec2 v1, vec2 v2) {
            return new vec2(v1.v0 % v2.v0, v1.v1 % v2.v1);
        }
        
        public static bvec2 operator==(vec2 v1, vec2 v2) {
            return new bvec2(v1.v0 == v2.v0, v1.v1 == v2.v1);
        }
        
        public static bvec2 operator>(vec2 v1, vec2 v2) {
            return new bvec2(v1.v0 > v2.v0, v1.v1 > v2.v1);
        }
        
        public static bvec2 operator>=(vec2 v1, vec2 v2) {
            return new bvec2(v1.v0 >= v2.v0, v1.v1 >= v2.v1);
        }
        
        public static bvec2 operator<(vec2 v1, vec2 v2) {
            return new bvec2(v1.v0 < v2.v0, v1.v1 < v2.v1);
        }
        
        public static bvec2 operator<=(vec2 v1, vec2 v2) {
            return new bvec2(v1.v0 <= v2.v0, v1.v1 <= v2.v1);
        }
        
        public static bvec2 operator!=(vec2 v1, vec2 v2) {
            return new bvec2(v1.v0 != v2.v0, v1.v1 != v2.v1);
        }
        
        public override bool Equals(object obj) {
            return (obj is vec2) ? bmath.All(this == (vec2)obj) : false;
        }
        
        public override int GetHashCode() {
            return this.v0.GetHashCode() ^ this.v1.GetHashCode();
        }
        
        public override string ToString() {
            return "vec2(" + this.v0 + "," + this.v1 + ")";
        }
    }
    
    [DataContract()]
    [StructLayout(LayoutKind.Sequential)]
    public struct vec3 {
        
        [DataMember()]
        internal float v0;
        
        [DataMember()]
        internal float v1;
        
        [DataMember()]
        internal float v2;
        
        public vec3(float s) {
            this.v0 = s;
            this.v1 = s;
            this.v2 = s;
        }
        
        public vec3(vec3 v0) {
            this.v0 = v0.v0;
            this.v1 = v0.v1;
            this.v2 = v0.v2;
        }
        
        public vec3(vec2 v0, float v1) {
            this.v0 = v0.v0;
            this.v1 = v0.v1;
            this.v2 = v1;
        }
        
        public vec3(float v0, vec2 v1) {
            this.v0 = v0;
            this.v1 = v1.v0;
            this.v2 = v1.v1;
        }
        
        public vec3(float v0, float v1, float v2) {
            this.v0 = v0;
            this.v1 = v1;
            this.v2 = v2;
        }
        
        public vec3(vec2 s) {
            this.v0 = s.v0;
            this.v1 = s.v1;
            this.v2 = 0F;
        }
        
        public vec3(vec4 s) {
            this.v0 = s.v0;
            this.v1 = s.v1;
            this.v2 = s.v2;
        }
        
        public float this[int i0] {
            get {
                if (((i0 < 0) 
                            || (2 < i0))) {
                    throw new System.IndexOutOfRangeException("i0");
                }
                
                unsafe
                {
                    fixed (Single* v_ptr = &this.v0)
                    {
                        return *(v_ptr + (i0));
                    }
                };
            }
            set {
                if (((i0 < 0) 
                            || (2 < i0))) {
                    throw new System.IndexOutOfRangeException("i0");
                }
                
                unsafe
                {
                    fixed (Single* v_ptr = &this.v0)
                    {
                        *(v_ptr  + (i0)) = value;
                    }
                };
            }
        }
        
        public float x {
            get {
                return this.v0;
            }
            set {
                this.v0 = value;
            }
        }
        
        public float r {
            get {
                return this.v0;
            }
            set {
                this.v0 = value;
            }
        }
        
        public float s {
            get {
                return this.v0;
            }
            set {
                this.v0 = value;
            }
        }
        
        public float y {
            get {
                return this.v1;
            }
            set {
                this.v1 = value;
            }
        }
        
        public float g {
            get {
                return this.v1;
            }
            set {
                this.v1 = value;
            }
        }
        
        public float t {
            get {
                return this.v1;
            }
            set {
                this.v1 = value;
            }
        }
        
        public float z {
            get {
                return this.v2;
            }
            set {
                this.v2 = value;
            }
        }
        
        public float b {
            get {
                return this.v2;
            }
            set {
                this.v2 = value;
            }
        }
        
        public float p {
            get {
                return this.v2;
            }
            set {
                this.v2 = value;
            }
        }
        
        public vec2 xx {
            get {
                return new vec2(this.v0, this.v0);
            }
        }
        
        public vec2 rr {
            get {
                return new vec2(this.v0, this.v0);
            }
        }
        
        public vec2 ss {
            get {
                return new vec2(this.v0, this.v0);
            }
        }
        
        public vec2 xy {
            get {
                return new vec2(this.v0, this.v1);
            }
            set {
                this.v0 = value.v0;
                this.v1 = value.v1;
            }
        }
        
        public vec2 rg {
            get {
                return new vec2(this.v0, this.v1);
            }
            set {
                this.v0 = value.v0;
                this.v1 = value.v1;
            }
        }
        
        public vec2 st {
            get {
                return new vec2(this.v0, this.v1);
            }
            set {
                this.v0 = value.v0;
                this.v1 = value.v1;
            }
        }
        
        public vec2 xz {
            get {
                return new vec2(this.v0, this.v2);
            }
            set {
                this.v0 = value.v0;
                this.v2 = value.v1;
            }
        }
        
        public vec2 rb {
            get {
                return new vec2(this.v0, this.v2);
            }
            set {
                this.v0 = value.v0;
                this.v2 = value.v1;
            }
        }
        
        public vec2 sp {
            get {
                return new vec2(this.v0, this.v2);
            }
            set {
                this.v0 = value.v0;
                this.v2 = value.v1;
            }
        }
        
        public vec2 yx {
            get {
                return new vec2(this.v1, this.v0);
            }
            set {
                this.v1 = value.v0;
                this.v0 = value.v1;
            }
        }
        
        public vec2 gr {
            get {
                return new vec2(this.v1, this.v0);
            }
            set {
                this.v1 = value.v0;
                this.v0 = value.v1;
            }
        }
        
        public vec2 ts {
            get {
                return new vec2(this.v1, this.v0);
            }
            set {
                this.v1 = value.v0;
                this.v0 = value.v1;
            }
        }
        
        public vec2 yy {
            get {
                return new vec2(this.v1, this.v1);
            }
        }
        
        public vec2 gg {
            get {
                return new vec2(this.v1, this.v1);
            }
        }
        
        public vec2 tt {
            get {
                return new vec2(this.v1, this.v1);
            }
        }
        
        public vec2 yz {
            get {
                return new vec2(this.v1, this.v2);
            }
            set {
                this.v1 = value.v0;
                this.v2 = value.v1;
            }
        }
        
        public vec2 gb {
            get {
                return new vec2(this.v1, this.v2);
            }
            set {
                this.v1 = value.v0;
                this.v2 = value.v1;
            }
        }
        
        public vec2 tp {
            get {
                return new vec2(this.v1, this.v2);
            }
            set {
                this.v1 = value.v0;
                this.v2 = value.v1;
            }
        }
        
        public vec2 zx {
            get {
                return new vec2(this.v2, this.v0);
            }
            set {
                this.v2 = value.v0;
                this.v0 = value.v1;
            }
        }
        
        public vec2 br {
            get {
                return new vec2(this.v2, this.v0);
            }
            set {
                this.v2 = value.v0;
                this.v0 = value.v1;
            }
        }
        
        public vec2 ps {
            get {
                return new vec2(this.v2, this.v0);
            }
            set {
                this.v2 = value.v0;
                this.v0 = value.v1;
            }
        }
        
        public vec2 zy {
            get {
                return new vec2(this.v2, this.v1);
            }
            set {
                this.v2 = value.v0;
                this.v1 = value.v1;
            }
        }
        
        public vec2 bg {
            get {
                return new vec2(this.v2, this.v1);
            }
            set {
                this.v2 = value.v0;
                this.v1 = value.v1;
            }
        }
        
        public vec2 pt {
            get {
                return new vec2(this.v2, this.v1);
            }
            set {
                this.v2 = value.v0;
                this.v1 = value.v1;
            }
        }
        
        public vec2 zz {
            get {
                return new vec2(this.v2, this.v2);
            }
        }
        
        public vec2 bb {
            get {
                return new vec2(this.v2, this.v2);
            }
        }
        
        public vec2 pp {
            get {
                return new vec2(this.v2, this.v2);
            }
        }
        
        public vec3 xxx {
            get {
                return new vec3(this.v0, this.v0, this.v0);
            }
        }
        
        public vec3 rrr {
            get {
                return new vec3(this.v0, this.v0, this.v0);
            }
        }
        
        public vec3 sss {
            get {
                return new vec3(this.v0, this.v0, this.v0);
            }
        }
        
        public vec3 xxy {
            get {
                return new vec3(this.v0, this.v0, this.v1);
            }
        }
        
        public vec3 rrg {
            get {
                return new vec3(this.v0, this.v0, this.v1);
            }
        }
        
        public vec3 sst {
            get {
                return new vec3(this.v0, this.v0, this.v1);
            }
        }
        
        public vec3 xxz {
            get {
                return new vec3(this.v0, this.v0, this.v2);
            }
        }
        
        public vec3 rrb {
            get {
                return new vec3(this.v0, this.v0, this.v2);
            }
        }
        
        public vec3 ssp {
            get {
                return new vec3(this.v0, this.v0, this.v2);
            }
        }
        
        public vec3 xyx {
            get {
                return new vec3(this.v0, this.v1, this.v0);
            }
        }
        
        public vec3 rgr {
            get {
                return new vec3(this.v0, this.v1, this.v0);
            }
        }
        
        public vec3 sts {
            get {
                return new vec3(this.v0, this.v1, this.v0);
            }
        }
        
        public vec3 xyy {
            get {
                return new vec3(this.v0, this.v1, this.v1);
            }
        }
        
        public vec3 rgg {
            get {
                return new vec3(this.v0, this.v1, this.v1);
            }
        }
        
        public vec3 stt {
            get {
                return new vec3(this.v0, this.v1, this.v1);
            }
        }
        
        public vec3 xyz {
            get {
                return new vec3(this.v0, this.v1, this.v2);
            }
            set {
                this.v0 = value.v0;
                this.v1 = value.v1;
                this.v2 = value.v2;
            }
        }
        
        public vec3 rgb {
            get {
                return new vec3(this.v0, this.v1, this.v2);
            }
            set {
                this.v0 = value.v0;
                this.v1 = value.v1;
                this.v2 = value.v2;
            }
        }
        
        public vec3 stp {
            get {
                return new vec3(this.v0, this.v1, this.v2);
            }
            set {
                this.v0 = value.v0;
                this.v1 = value.v1;
                this.v2 = value.v2;
            }
        }
        
        public vec3 xzx {
            get {
                return new vec3(this.v0, this.v2, this.v0);
            }
        }
        
        public vec3 rbr {
            get {
                return new vec3(this.v0, this.v2, this.v0);
            }
        }
        
        public vec3 sps {
            get {
                return new vec3(this.v0, this.v2, this.v0);
            }
        }
        
        public vec3 xzy {
            get {
                return new vec3(this.v0, this.v2, this.v1);
            }
            set {
                this.v0 = value.v0;
                this.v2 = value.v1;
                this.v1 = value.v2;
            }
        }
        
        public vec3 rbg {
            get {
                return new vec3(this.v0, this.v2, this.v1);
            }
            set {
                this.v0 = value.v0;
                this.v2 = value.v1;
                this.v1 = value.v2;
            }
        }
        
        public vec3 spt {
            get {
                return new vec3(this.v0, this.v2, this.v1);
            }
            set {
                this.v0 = value.v0;
                this.v2 = value.v1;
                this.v1 = value.v2;
            }
        }
        
        public vec3 xzz {
            get {
                return new vec3(this.v0, this.v2, this.v2);
            }
        }
        
        public vec3 rbb {
            get {
                return new vec3(this.v0, this.v2, this.v2);
            }
        }
        
        public vec3 spp {
            get {
                return new vec3(this.v0, this.v2, this.v2);
            }
        }
        
        public vec3 yxx {
            get {
                return new vec3(this.v1, this.v0, this.v0);
            }
        }
        
        public vec3 grr {
            get {
                return new vec3(this.v1, this.v0, this.v0);
            }
        }
        
        public vec3 tss {
            get {
                return new vec3(this.v1, this.v0, this.v0);
            }
        }
        
        public vec3 yxy {
            get {
                return new vec3(this.v1, this.v0, this.v1);
            }
        }
        
        public vec3 grg {
            get {
                return new vec3(this.v1, this.v0, this.v1);
            }
        }
        
        public vec3 tst {
            get {
                return new vec3(this.v1, this.v0, this.v1);
            }
        }
        
        public vec3 yxz {
            get {
                return new vec3(this.v1, this.v0, this.v2);
            }
            set {
                this.v1 = value.v0;
                this.v0 = value.v1;
                this.v2 = value.v2;
            }
        }
        
        public vec3 grb {
            get {
                return new vec3(this.v1, this.v0, this.v2);
            }
            set {
                this.v1 = value.v0;
                this.v0 = value.v1;
                this.v2 = value.v2;
            }
        }
        
        public vec3 tsp {
            get {
                return new vec3(this.v1, this.v0, this.v2);
            }
            set {
                this.v1 = value.v0;
                this.v0 = value.v1;
                this.v2 = value.v2;
            }
        }
        
        public vec3 yyx {
            get {
                return new vec3(this.v1, this.v1, this.v0);
            }
        }
        
        public vec3 ggr {
            get {
                return new vec3(this.v1, this.v1, this.v0);
            }
        }
        
        public vec3 tts {
            get {
                return new vec3(this.v1, this.v1, this.v0);
            }
        }
        
        public vec3 yyy {
            get {
                return new vec3(this.v1, this.v1, this.v1);
            }
        }
        
        public vec3 ggg {
            get {
                return new vec3(this.v1, this.v1, this.v1);
            }
        }
        
        public vec3 ttt {
            get {
                return new vec3(this.v1, this.v1, this.v1);
            }
        }
        
        public vec3 yyz {
            get {
                return new vec3(this.v1, this.v1, this.v2);
            }
        }
        
        public vec3 ggb {
            get {
                return new vec3(this.v1, this.v1, this.v2);
            }
        }
        
        public vec3 ttp {
            get {
                return new vec3(this.v1, this.v1, this.v2);
            }
        }
        
        public vec3 yzx {
            get {
                return new vec3(this.v1, this.v2, this.v0);
            }
            set {
                this.v1 = value.v0;
                this.v2 = value.v1;
                this.v0 = value.v2;
            }
        }
        
        public vec3 gbr {
            get {
                return new vec3(this.v1, this.v2, this.v0);
            }
            set {
                this.v1 = value.v0;
                this.v2 = value.v1;
                this.v0 = value.v2;
            }
        }
        
        public vec3 tps {
            get {
                return new vec3(this.v1, this.v2, this.v0);
            }
            set {
                this.v1 = value.v0;
                this.v2 = value.v1;
                this.v0 = value.v2;
            }
        }
        
        public vec3 yzy {
            get {
                return new vec3(this.v1, this.v2, this.v1);
            }
        }
        
        public vec3 gbg {
            get {
                return new vec3(this.v1, this.v2, this.v1);
            }
        }
        
        public vec3 tpt {
            get {
                return new vec3(this.v1, this.v2, this.v1);
            }
        }
        
        public vec3 yzz {
            get {
                return new vec3(this.v1, this.v2, this.v2);
            }
        }
        
        public vec3 gbb {
            get {
                return new vec3(this.v1, this.v2, this.v2);
            }
        }
        
        public vec3 tpp {
            get {
                return new vec3(this.v1, this.v2, this.v2);
            }
        }
        
        public vec3 zxx {
            get {
                return new vec3(this.v2, this.v0, this.v0);
            }
        }
        
        public vec3 brr {
            get {
                return new vec3(this.v2, this.v0, this.v0);
            }
        }
        
        public vec3 pss {
            get {
                return new vec3(this.v2, this.v0, this.v0);
            }
        }
        
        public vec3 zxy {
            get {
                return new vec3(this.v2, this.v0, this.v1);
            }
            set {
                this.v2 = value.v0;
                this.v0 = value.v1;
                this.v1 = value.v2;
            }
        }
        
        public vec3 brg {
            get {
                return new vec3(this.v2, this.v0, this.v1);
            }
            set {
                this.v2 = value.v0;
                this.v0 = value.v1;
                this.v1 = value.v2;
            }
        }
        
        public vec3 pst {
            get {
                return new vec3(this.v2, this.v0, this.v1);
            }
            set {
                this.v2 = value.v0;
                this.v0 = value.v1;
                this.v1 = value.v2;
            }
        }
        
        public vec3 zxz {
            get {
                return new vec3(this.v2, this.v0, this.v2);
            }
        }
        
        public vec3 brb {
            get {
                return new vec3(this.v2, this.v0, this.v2);
            }
        }
        
        public vec3 psp {
            get {
                return new vec3(this.v2, this.v0, this.v2);
            }
        }
        
        public vec3 zyx {
            get {
                return new vec3(this.v2, this.v1, this.v0);
            }
            set {
                this.v2 = value.v0;
                this.v1 = value.v1;
                this.v0 = value.v2;
            }
        }
        
        public vec3 bgr {
            get {
                return new vec3(this.v2, this.v1, this.v0);
            }
            set {
                this.v2 = value.v0;
                this.v1 = value.v1;
                this.v0 = value.v2;
            }
        }
        
        public vec3 pts {
            get {
                return new vec3(this.v2, this.v1, this.v0);
            }
            set {
                this.v2 = value.v0;
                this.v1 = value.v1;
                this.v0 = value.v2;
            }
        }
        
        public vec3 zyy {
            get {
                return new vec3(this.v2, this.v1, this.v1);
            }
        }
        
        public vec3 bgg {
            get {
                return new vec3(this.v2, this.v1, this.v1);
            }
        }
        
        public vec3 ptt {
            get {
                return new vec3(this.v2, this.v1, this.v1);
            }
        }
        
        public vec3 zyz {
            get {
                return new vec3(this.v2, this.v1, this.v2);
            }
        }
        
        public vec3 bgb {
            get {
                return new vec3(this.v2, this.v1, this.v2);
            }
        }
        
        public vec3 ptp {
            get {
                return new vec3(this.v2, this.v1, this.v2);
            }
        }
        
        public vec3 zzx {
            get {
                return new vec3(this.v2, this.v2, this.v0);
            }
        }
        
        public vec3 bbr {
            get {
                return new vec3(this.v2, this.v2, this.v0);
            }
        }
        
        public vec3 pps {
            get {
                return new vec3(this.v2, this.v2, this.v0);
            }
        }
        
        public vec3 zzy {
            get {
                return new vec3(this.v2, this.v2, this.v1);
            }
        }
        
        public vec3 bbg {
            get {
                return new vec3(this.v2, this.v2, this.v1);
            }
        }
        
        public vec3 ppt {
            get {
                return new vec3(this.v2, this.v2, this.v1);
            }
        }
        
        public vec3 zzz {
            get {
                return new vec3(this.v2, this.v2, this.v2);
            }
        }
        
        public vec3 bbb {
            get {
                return new vec3(this.v2, this.v2, this.v2);
            }
        }
        
        public vec3 ppp {
            get {
                return new vec3(this.v2, this.v2, this.v2);
            }
        }
        
        public static vec3 operator+(vec3 v) {
            return new vec3(+v.v0, +v.v1, +v.v2);
        }
        
        public static vec3 operator-(vec3 v) {
            return new vec3(-v.v0, -v.v1, -v.v2);
        }
        
        public static vec3 operator++(vec3 v) {
            return new vec3(++v.v0, ++v.v1, ++v.v2);
        }
        
        public static vec3 operator--(vec3 v) {
            return new vec3(--v.v0, --v.v1, --v.v2);
        }
        
        public static vec3 operator+(float s, vec3 v2) {
            return new vec3(s + v2.v0, s + v2.v1, s + v2.v2);
        }
        
        public static vec3 operator-(float s, vec3 v2) {
            return new vec3(s - v2.v0, s - v2.v1, s - v2.v2);
        }
        
        public static vec3 operator*(float s, vec3 v2) {
            return new vec3(s * v2.v0, s * v2.v1, s * v2.v2);
        }
        
        public static vec3 operator/(float s, vec3 v2) {
            return new vec3(s / v2.v0, s / v2.v1, s / v2.v2);
        }
        
        public static vec3 operator%(float s, vec3 v2) {
            return new vec3(s % v2.v0, s % v2.v1, s % v2.v2);
        }
        
        public static vec3 operator+(vec3 v1, float s) {
            return new vec3(v1.v0 + s, v1.v1 + s, v1.v2 + s);
        }
        
        public static vec3 operator-(vec3 v1, float s) {
            return new vec3(v1.v0 - s, v1.v1 - s, v1.v2 - s);
        }
        
        public static vec3 operator*(vec3 v1, float s) {
            return new vec3(v1.v0 * s, v1.v1 * s, v1.v2 * s);
        }
        
        public static vec3 operator/(vec3 v1, float s) {
            return new vec3(v1.v0 / s, v1.v1 / s, v1.v2 / s);
        }
        
        public static vec3 operator%(vec3 v1, float s) {
            return new vec3(v1.v0 % s, v1.v1 % s, v1.v2 % s);
        }
        
        public static vec3 operator+(vec3 v1, vec3 v2) {
            return new vec3(v1.v0 + v2.v0, v1.v1 + v2.v1, v1.v2 + v2.v2);
        }
        
        public static vec3 operator-(vec3 v1, vec3 v2) {
            return new vec3(v1.v0 - v2.v0, v1.v1 - v2.v1, v1.v2 - v2.v2);
        }
        
        public static vec3 operator*(vec3 v1, vec3 v2) {
            return new vec3(v1.v0 * v2.v0, v1.v1 * v2.v1, v1.v2 * v2.v2);
        }
        
        public static vec3 operator/(vec3 v1, vec3 v2) {
            return new vec3(v1.v0 / v2.v0, v1.v1 / v2.v1, v1.v2 / v2.v2);
        }
        
        public static vec3 operator%(vec3 v1, vec3 v2) {
            return new vec3(v1.v0 % v2.v0, v1.v1 % v2.v1, v1.v2 % v2.v2);
        }
        
        public static bvec3 operator==(vec3 v1, vec3 v2) {
            return new bvec3(v1.v0 == v2.v0, v1.v1 == v2.v1, v1.v2 == v2.v2);
        }
        
        public static bvec3 operator>(vec3 v1, vec3 v2) {
            return new bvec3(v1.v0 > v2.v0, v1.v1 > v2.v1, v1.v2 > v2.v2);
        }
        
        public static bvec3 operator>=(vec3 v1, vec3 v2) {
            return new bvec3(v1.v0 >= v2.v0, v1.v1 >= v2.v1, v1.v2 >= v2.v2);
        }
        
        public static bvec3 operator<(vec3 v1, vec3 v2) {
            return new bvec3(v1.v0 < v2.v0, v1.v1 < v2.v1, v1.v2 < v2.v2);
        }
        
        public static bvec3 operator<=(vec3 v1, vec3 v2) {
            return new bvec3(v1.v0 <= v2.v0, v1.v1 <= v2.v1, v1.v2 <= v2.v2);
        }
        
        public static bvec3 operator!=(vec3 v1, vec3 v2) {
            return new bvec3(v1.v0 != v2.v0, v1.v1 != v2.v1, v1.v2 != v2.v2);
        }
        
        public override bool Equals(object obj) {
            return (obj is vec3) ? bmath.All(this == (vec3)obj) : false;
        }
        
        public override int GetHashCode() {
            return this.v0.GetHashCode() ^ this.v1.GetHashCode() ^ this.v2.GetHashCode();
        }
        
        public override string ToString() {
            return "vec3(" + this.v0 + "," + this.v1 + "," + this.v2 + ")";
        }
    }
    
    [DataContract()]
    [StructLayout(LayoutKind.Sequential)]
    public struct vec4 {
        
        [DataMember()]
        internal float v0;
        
        [DataMember()]
        internal float v1;
        
        [DataMember()]
        internal float v2;
        
        [DataMember()]
        internal float v3;
        
        public vec4(float s) {
            this.v0 = s;
            this.v1 = s;
            this.v2 = s;
            this.v3 = s;
        }
        
        public vec4(vec4 v0) {
            this.v0 = v0.v0;
            this.v1 = v0.v1;
            this.v2 = v0.v2;
            this.v3 = v0.v3;
        }
        
        public vec4(vec2 v0, vec2 v1) {
            this.v0 = v0.v0;
            this.v1 = v0.v1;
            this.v2 = v1.v0;
            this.v3 = v1.v1;
        }
        
        public vec4(vec3 v0, float v1) {
            this.v0 = v0.v0;
            this.v1 = v0.v1;
            this.v2 = v0.v2;
            this.v3 = v1;
        }
        
        public vec4(float v0, vec3 v1) {
            this.v0 = v0;
            this.v1 = v1.v0;
            this.v2 = v1.v1;
            this.v3 = v1.v2;
        }
        
        public vec4(vec2 v0, float v1, float v2) {
            this.v0 = v0.v0;
            this.v1 = v0.v1;
            this.v2 = v1;
            this.v3 = v2;
        }
        
        public vec4(float v0, vec2 v1, float v2) {
            this.v0 = v0;
            this.v1 = v1.v0;
            this.v2 = v1.v1;
            this.v3 = v2;
        }
        
        public vec4(float v0, float v1, vec2 v2) {
            this.v0 = v0;
            this.v1 = v1;
            this.v2 = v2.v0;
            this.v3 = v2.v1;
        }
        
        public vec4(float v0, float v1, float v2, float v3) {
            this.v0 = v0;
            this.v1 = v1;
            this.v2 = v2;
            this.v3 = v3;
        }
        
        public vec4(vec2 s) {
            this.v0 = s.v0;
            this.v1 = s.v1;
            this.v2 = 0F;
            this.v3 = 0F;
        }
        
        public vec4(vec3 s) {
            this.v0 = s.v0;
            this.v1 = s.v1;
            this.v2 = s.v2;
            this.v3 = 0F;
        }
        
        public float this[int i0] {
            get {
                if (((i0 < 0) 
                            || (3 < i0))) {
                    throw new System.IndexOutOfRangeException("i0");
                }
                
                unsafe
                {
                    fixed (Single* v_ptr = &this.v0)
                    {
                        return *(v_ptr + (i0));
                    }
                };
            }
            set {
                if (((i0 < 0) 
                            || (3 < i0))) {
                    throw new System.IndexOutOfRangeException("i0");
                }
                
                unsafe
                {
                    fixed (Single* v_ptr = &this.v0)
                    {
                        *(v_ptr  + (i0)) = value;
                    }
                };
            }
        }
        
        public float x {
            get {
                return this.v0;
            }
            set {
                this.v0 = value;
            }
        }
        
        public float r {
            get {
                return this.v0;
            }
            set {
                this.v0 = value;
            }
        }
        
        public float s {
            get {
                return this.v0;
            }
            set {
                this.v0 = value;
            }
        }
        
        public float y {
            get {
                return this.v1;
            }
            set {
                this.v1 = value;
            }
        }
        
        public float g {
            get {
                return this.v1;
            }
            set {
                this.v1 = value;
            }
        }
        
        public float t {
            get {
                return this.v1;
            }
            set {
                this.v1 = value;
            }
        }
        
        public float z {
            get {
                return this.v2;
            }
            set {
                this.v2 = value;
            }
        }
        
        public float b {
            get {
                return this.v2;
            }
            set {
                this.v2 = value;
            }
        }
        
        public float p {
            get {
                return this.v2;
            }
            set {
                this.v2 = value;
            }
        }
        
        public float w {
            get {
                return this.v3;
            }
            set {
                this.v3 = value;
            }
        }
        
        public float a {
            get {
                return this.v3;
            }
            set {
                this.v3 = value;
            }
        }
        
        public float q {
            get {
                return this.v3;
            }
            set {
                this.v3 = value;
            }
        }
        
        public vec2 xx {
            get {
                return new vec2(this.v0, this.v0);
            }
        }
        
        public vec2 rr {
            get {
                return new vec2(this.v0, this.v0);
            }
        }
        
        public vec2 ss {
            get {
                return new vec2(this.v0, this.v0);
            }
        }
        
        public vec2 xy {
            get {
                return new vec2(this.v0, this.v1);
            }
            set {
                this.v0 = value.v0;
                this.v1 = value.v1;
            }
        }
        
        public vec2 rg {
            get {
                return new vec2(this.v0, this.v1);
            }
            set {
                this.v0 = value.v0;
                this.v1 = value.v1;
            }
        }
        
        public vec2 st {
            get {
                return new vec2(this.v0, this.v1);
            }
            set {
                this.v0 = value.v0;
                this.v1 = value.v1;
            }
        }
        
        public vec2 xz {
            get {
                return new vec2(this.v0, this.v2);
            }
            set {
                this.v0 = value.v0;
                this.v2 = value.v1;
            }
        }
        
        public vec2 rb {
            get {
                return new vec2(this.v0, this.v2);
            }
            set {
                this.v0 = value.v0;
                this.v2 = value.v1;
            }
        }
        
        public vec2 sp {
            get {
                return new vec2(this.v0, this.v2);
            }
            set {
                this.v0 = value.v0;
                this.v2 = value.v1;
            }
        }
        
        public vec2 xw {
            get {
                return new vec2(this.v0, this.v3);
            }
            set {
                this.v0 = value.v0;
                this.v3 = value.v1;
            }
        }
        
        public vec2 ra {
            get {
                return new vec2(this.v0, this.v3);
            }
            set {
                this.v0 = value.v0;
                this.v3 = value.v1;
            }
        }
        
        public vec2 sq {
            get {
                return new vec2(this.v0, this.v3);
            }
            set {
                this.v0 = value.v0;
                this.v3 = value.v1;
            }
        }
        
        public vec2 yx {
            get {
                return new vec2(this.v1, this.v0);
            }
            set {
                this.v1 = value.v0;
                this.v0 = value.v1;
            }
        }
        
        public vec2 gr {
            get {
                return new vec2(this.v1, this.v0);
            }
            set {
                this.v1 = value.v0;
                this.v0 = value.v1;
            }
        }
        
        public vec2 ts {
            get {
                return new vec2(this.v1, this.v0);
            }
            set {
                this.v1 = value.v0;
                this.v0 = value.v1;
            }
        }
        
        public vec2 yy {
            get {
                return new vec2(this.v1, this.v1);
            }
        }
        
        public vec2 gg {
            get {
                return new vec2(this.v1, this.v1);
            }
        }
        
        public vec2 tt {
            get {
                return new vec2(this.v1, this.v1);
            }
        }
        
        public vec2 yz {
            get {
                return new vec2(this.v1, this.v2);
            }
            set {
                this.v1 = value.v0;
                this.v2 = value.v1;
            }
        }
        
        public vec2 gb {
            get {
                return new vec2(this.v1, this.v2);
            }
            set {
                this.v1 = value.v0;
                this.v2 = value.v1;
            }
        }
        
        public vec2 tp {
            get {
                return new vec2(this.v1, this.v2);
            }
            set {
                this.v1 = value.v0;
                this.v2 = value.v1;
            }
        }
        
        public vec2 yw {
            get {
                return new vec2(this.v1, this.v3);
            }
            set {
                this.v1 = value.v0;
                this.v3 = value.v1;
            }
        }
        
        public vec2 ga {
            get {
                return new vec2(this.v1, this.v3);
            }
            set {
                this.v1 = value.v0;
                this.v3 = value.v1;
            }
        }
        
        public vec2 tq {
            get {
                return new vec2(this.v1, this.v3);
            }
            set {
                this.v1 = value.v0;
                this.v3 = value.v1;
            }
        }
        
        public vec2 zx {
            get {
                return new vec2(this.v2, this.v0);
            }
            set {
                this.v2 = value.v0;
                this.v0 = value.v1;
            }
        }
        
        public vec2 br {
            get {
                return new vec2(this.v2, this.v0);
            }
            set {
                this.v2 = value.v0;
                this.v0 = value.v1;
            }
        }
        
        public vec2 ps {
            get {
                return new vec2(this.v2, this.v0);
            }
            set {
                this.v2 = value.v0;
                this.v0 = value.v1;
            }
        }
        
        public vec2 zy {
            get {
                return new vec2(this.v2, this.v1);
            }
            set {
                this.v2 = value.v0;
                this.v1 = value.v1;
            }
        }
        
        public vec2 bg {
            get {
                return new vec2(this.v2, this.v1);
            }
            set {
                this.v2 = value.v0;
                this.v1 = value.v1;
            }
        }
        
        public vec2 pt {
            get {
                return new vec2(this.v2, this.v1);
            }
            set {
                this.v2 = value.v0;
                this.v1 = value.v1;
            }
        }
        
        public vec2 zz {
            get {
                return new vec2(this.v2, this.v2);
            }
        }
        
        public vec2 bb {
            get {
                return new vec2(this.v2, this.v2);
            }
        }
        
        public vec2 pp {
            get {
                return new vec2(this.v2, this.v2);
            }
        }
        
        public vec2 zw {
            get {
                return new vec2(this.v2, this.v3);
            }
            set {
                this.v2 = value.v0;
                this.v3 = value.v1;
            }
        }
        
        public vec2 ba {
            get {
                return new vec2(this.v2, this.v3);
            }
            set {
                this.v2 = value.v0;
                this.v3 = value.v1;
            }
        }
        
        public vec2 pq {
            get {
                return new vec2(this.v2, this.v3);
            }
            set {
                this.v2 = value.v0;
                this.v3 = value.v1;
            }
        }
        
        public vec2 wx {
            get {
                return new vec2(this.v3, this.v0);
            }
            set {
                this.v3 = value.v0;
                this.v0 = value.v1;
            }
        }
        
        public vec2 ar {
            get {
                return new vec2(this.v3, this.v0);
            }
            set {
                this.v3 = value.v0;
                this.v0 = value.v1;
            }
        }
        
        public vec2 qs {
            get {
                return new vec2(this.v3, this.v0);
            }
            set {
                this.v3 = value.v0;
                this.v0 = value.v1;
            }
        }
        
        public vec2 wy {
            get {
                return new vec2(this.v3, this.v1);
            }
            set {
                this.v3 = value.v0;
                this.v1 = value.v1;
            }
        }
        
        public vec2 ag {
            get {
                return new vec2(this.v3, this.v1);
            }
            set {
                this.v3 = value.v0;
                this.v1 = value.v1;
            }
        }
        
        public vec2 qt {
            get {
                return new vec2(this.v3, this.v1);
            }
            set {
                this.v3 = value.v0;
                this.v1 = value.v1;
            }
        }
        
        public vec2 wz {
            get {
                return new vec2(this.v3, this.v2);
            }
            set {
                this.v3 = value.v0;
                this.v2 = value.v1;
            }
        }
        
        public vec2 ab {
            get {
                return new vec2(this.v3, this.v2);
            }
            set {
                this.v3 = value.v0;
                this.v2 = value.v1;
            }
        }
        
        public vec2 qp {
            get {
                return new vec2(this.v3, this.v2);
            }
            set {
                this.v3 = value.v0;
                this.v2 = value.v1;
            }
        }
        
        public vec2 ww {
            get {
                return new vec2(this.v3, this.v3);
            }
        }
        
        public vec2 aa {
            get {
                return new vec2(this.v3, this.v3);
            }
        }
        
        public vec2 qq {
            get {
                return new vec2(this.v3, this.v3);
            }
        }
        
        public vec3 xxx {
            get {
                return new vec3(this.v0, this.v0, this.v0);
            }
        }
        
        public vec3 rrr {
            get {
                return new vec3(this.v0, this.v0, this.v0);
            }
        }
        
        public vec3 sss {
            get {
                return new vec3(this.v0, this.v0, this.v0);
            }
        }
        
        public vec3 xxy {
            get {
                return new vec3(this.v0, this.v0, this.v1);
            }
        }
        
        public vec3 rrg {
            get {
                return new vec3(this.v0, this.v0, this.v1);
            }
        }
        
        public vec3 sst {
            get {
                return new vec3(this.v0, this.v0, this.v1);
            }
        }
        
        public vec3 xxz {
            get {
                return new vec3(this.v0, this.v0, this.v2);
            }
        }
        
        public vec3 rrb {
            get {
                return new vec3(this.v0, this.v0, this.v2);
            }
        }
        
        public vec3 ssp {
            get {
                return new vec3(this.v0, this.v0, this.v2);
            }
        }
        
        public vec3 xxw {
            get {
                return new vec3(this.v0, this.v0, this.v3);
            }
        }
        
        public vec3 rra {
            get {
                return new vec3(this.v0, this.v0, this.v3);
            }
        }
        
        public vec3 ssq {
            get {
                return new vec3(this.v0, this.v0, this.v3);
            }
        }
        
        public vec3 xyx {
            get {
                return new vec3(this.v0, this.v1, this.v0);
            }
        }
        
        public vec3 rgr {
            get {
                return new vec3(this.v0, this.v1, this.v0);
            }
        }
        
        public vec3 sts {
            get {
                return new vec3(this.v0, this.v1, this.v0);
            }
        }
        
        public vec3 xyy {
            get {
                return new vec3(this.v0, this.v1, this.v1);
            }
        }
        
        public vec3 rgg {
            get {
                return new vec3(this.v0, this.v1, this.v1);
            }
        }
        
        public vec3 stt {
            get {
                return new vec3(this.v0, this.v1, this.v1);
            }
        }
        
        public vec3 xyz {
            get {
                return new vec3(this.v0, this.v1, this.v2);
            }
            set {
                this.v0 = value.v0;
                this.v1 = value.v1;
                this.v2 = value.v2;
            }
        }
        
        public vec3 rgb {
            get {
                return new vec3(this.v0, this.v1, this.v2);
            }
            set {
                this.v0 = value.v0;
                this.v1 = value.v1;
                this.v2 = value.v2;
            }
        }
        
        public vec3 stp {
            get {
                return new vec3(this.v0, this.v1, this.v2);
            }
            set {
                this.v0 = value.v0;
                this.v1 = value.v1;
                this.v2 = value.v2;
            }
        }
        
        public vec3 xyw {
            get {
                return new vec3(this.v0, this.v1, this.v3);
            }
            set {
                this.v0 = value.v0;
                this.v1 = value.v1;
                this.v3 = value.v2;
            }
        }
        
        public vec3 rga {
            get {
                return new vec3(this.v0, this.v1, this.v3);
            }
            set {
                this.v0 = value.v0;
                this.v1 = value.v1;
                this.v3 = value.v2;
            }
        }
        
        public vec3 stq {
            get {
                return new vec3(this.v0, this.v1, this.v3);
            }
            set {
                this.v0 = value.v0;
                this.v1 = value.v1;
                this.v3 = value.v2;
            }
        }
        
        public vec3 xzx {
            get {
                return new vec3(this.v0, this.v2, this.v0);
            }
        }
        
        public vec3 rbr {
            get {
                return new vec3(this.v0, this.v2, this.v0);
            }
        }
        
        public vec3 sps {
            get {
                return new vec3(this.v0, this.v2, this.v0);
            }
        }
        
        public vec3 xzy {
            get {
                return new vec3(this.v0, this.v2, this.v1);
            }
            set {
                this.v0 = value.v0;
                this.v2 = value.v1;
                this.v1 = value.v2;
            }
        }
        
        public vec3 rbg {
            get {
                return new vec3(this.v0, this.v2, this.v1);
            }
            set {
                this.v0 = value.v0;
                this.v2 = value.v1;
                this.v1 = value.v2;
            }
        }
        
        public vec3 spt {
            get {
                return new vec3(this.v0, this.v2, this.v1);
            }
            set {
                this.v0 = value.v0;
                this.v2 = value.v1;
                this.v1 = value.v2;
            }
        }
        
        public vec3 xzz {
            get {
                return new vec3(this.v0, this.v2, this.v2);
            }
        }
        
        public vec3 rbb {
            get {
                return new vec3(this.v0, this.v2, this.v2);
            }
        }
        
        public vec3 spp {
            get {
                return new vec3(this.v0, this.v2, this.v2);
            }
        }
        
        public vec3 xzw {
            get {
                return new vec3(this.v0, this.v2, this.v3);
            }
            set {
                this.v0 = value.v0;
                this.v2 = value.v1;
                this.v3 = value.v2;
            }
        }
        
        public vec3 rba {
            get {
                return new vec3(this.v0, this.v2, this.v3);
            }
            set {
                this.v0 = value.v0;
                this.v2 = value.v1;
                this.v3 = value.v2;
            }
        }
        
        public vec3 spq {
            get {
                return new vec3(this.v0, this.v2, this.v3);
            }
            set {
                this.v0 = value.v0;
                this.v2 = value.v1;
                this.v3 = value.v2;
            }
        }
        
        public vec3 xwx {
            get {
                return new vec3(this.v0, this.v3, this.v0);
            }
        }
        
        public vec3 rar {
            get {
                return new vec3(this.v0, this.v3, this.v0);
            }
        }
        
        public vec3 sqs {
            get {
                return new vec3(this.v0, this.v3, this.v0);
            }
        }
        
        public vec3 xwy {
            get {
                return new vec3(this.v0, this.v3, this.v1);
            }
            set {
                this.v0 = value.v0;
                this.v3 = value.v1;
                this.v1 = value.v2;
            }
        }
        
        public vec3 rag {
            get {
                return new vec3(this.v0, this.v3, this.v1);
            }
            set {
                this.v0 = value.v0;
                this.v3 = value.v1;
                this.v1 = value.v2;
            }
        }
        
        public vec3 sqt {
            get {
                return new vec3(this.v0, this.v3, this.v1);
            }
            set {
                this.v0 = value.v0;
                this.v3 = value.v1;
                this.v1 = value.v2;
            }
        }
        
        public vec3 xwz {
            get {
                return new vec3(this.v0, this.v3, this.v2);
            }
            set {
                this.v0 = value.v0;
                this.v3 = value.v1;
                this.v2 = value.v2;
            }
        }
        
        public vec3 rab {
            get {
                return new vec3(this.v0, this.v3, this.v2);
            }
            set {
                this.v0 = value.v0;
                this.v3 = value.v1;
                this.v2 = value.v2;
            }
        }
        
        public vec3 sqp {
            get {
                return new vec3(this.v0, this.v3, this.v2);
            }
            set {
                this.v0 = value.v0;
                this.v3 = value.v1;
                this.v2 = value.v2;
            }
        }
        
        public vec3 xww {
            get {
                return new vec3(this.v0, this.v3, this.v3);
            }
        }
        
        public vec3 raa {
            get {
                return new vec3(this.v0, this.v3, this.v3);
            }
        }
        
        public vec3 sqq {
            get {
                return new vec3(this.v0, this.v3, this.v3);
            }
        }
        
        public vec3 yxx {
            get {
                return new vec3(this.v1, this.v0, this.v0);
            }
        }
        
        public vec3 grr {
            get {
                return new vec3(this.v1, this.v0, this.v0);
            }
        }
        
        public vec3 tss {
            get {
                return new vec3(this.v1, this.v0, this.v0);
            }
        }
        
        public vec3 yxy {
            get {
                return new vec3(this.v1, this.v0, this.v1);
            }
        }
        
        public vec3 grg {
            get {
                return new vec3(this.v1, this.v0, this.v1);
            }
        }
        
        public vec3 tst {
            get {
                return new vec3(this.v1, this.v0, this.v1);
            }
        }
        
        public vec3 yxz {
            get {
                return new vec3(this.v1, this.v0, this.v2);
            }
            set {
                this.v1 = value.v0;
                this.v0 = value.v1;
                this.v2 = value.v2;
            }
        }
        
        public vec3 grb {
            get {
                return new vec3(this.v1, this.v0, this.v2);
            }
            set {
                this.v1 = value.v0;
                this.v0 = value.v1;
                this.v2 = value.v2;
            }
        }
        
        public vec3 tsp {
            get {
                return new vec3(this.v1, this.v0, this.v2);
            }
            set {
                this.v1 = value.v0;
                this.v0 = value.v1;
                this.v2 = value.v2;
            }
        }
        
        public vec3 yxw {
            get {
                return new vec3(this.v1, this.v0, this.v3);
            }
            set {
                this.v1 = value.v0;
                this.v0 = value.v1;
                this.v3 = value.v2;
            }
        }
        
        public vec3 gra {
            get {
                return new vec3(this.v1, this.v0, this.v3);
            }
            set {
                this.v1 = value.v0;
                this.v0 = value.v1;
                this.v3 = value.v2;
            }
        }
        
        public vec3 tsq {
            get {
                return new vec3(this.v1, this.v0, this.v3);
            }
            set {
                this.v1 = value.v0;
                this.v0 = value.v1;
                this.v3 = value.v2;
            }
        }
        
        public vec3 yyx {
            get {
                return new vec3(this.v1, this.v1, this.v0);
            }
        }
        
        public vec3 ggr {
            get {
                return new vec3(this.v1, this.v1, this.v0);
            }
        }
        
        public vec3 tts {
            get {
                return new vec3(this.v1, this.v1, this.v0);
            }
        }
        
        public vec3 yyy {
            get {
                return new vec3(this.v1, this.v1, this.v1);
            }
        }
        
        public vec3 ggg {
            get {
                return new vec3(this.v1, this.v1, this.v1);
            }
        }
        
        public vec3 ttt {
            get {
                return new vec3(this.v1, this.v1, this.v1);
            }
        }
        
        public vec3 yyz {
            get {
                return new vec3(this.v1, this.v1, this.v2);
            }
        }
        
        public vec3 ggb {
            get {
                return new vec3(this.v1, this.v1, this.v2);
            }
        }
        
        public vec3 ttp {
            get {
                return new vec3(this.v1, this.v1, this.v2);
            }
        }
        
        public vec3 yyw {
            get {
                return new vec3(this.v1, this.v1, this.v3);
            }
        }
        
        public vec3 gga {
            get {
                return new vec3(this.v1, this.v1, this.v3);
            }
        }
        
        public vec3 ttq {
            get {
                return new vec3(this.v1, this.v1, this.v3);
            }
        }
        
        public vec3 yzx {
            get {
                return new vec3(this.v1, this.v2, this.v0);
            }
            set {
                this.v1 = value.v0;
                this.v2 = value.v1;
                this.v0 = value.v2;
            }
        }
        
        public vec3 gbr {
            get {
                return new vec3(this.v1, this.v2, this.v0);
            }
            set {
                this.v1 = value.v0;
                this.v2 = value.v1;
                this.v0 = value.v2;
            }
        }
        
        public vec3 tps {
            get {
                return new vec3(this.v1, this.v2, this.v0);
            }
            set {
                this.v1 = value.v0;
                this.v2 = value.v1;
                this.v0 = value.v2;
            }
        }
        
        public vec3 yzy {
            get {
                return new vec3(this.v1, this.v2, this.v1);
            }
        }
        
        public vec3 gbg {
            get {
                return new vec3(this.v1, this.v2, this.v1);
            }
        }
        
        public vec3 tpt {
            get {
                return new vec3(this.v1, this.v2, this.v1);
            }
        }
        
        public vec3 yzz {
            get {
                return new vec3(this.v1, this.v2, this.v2);
            }
        }
        
        public vec3 gbb {
            get {
                return new vec3(this.v1, this.v2, this.v2);
            }
        }
        
        public vec3 tpp {
            get {
                return new vec3(this.v1, this.v2, this.v2);
            }
        }
        
        public vec3 yzw {
            get {
                return new vec3(this.v1, this.v2, this.v3);
            }
            set {
                this.v1 = value.v0;
                this.v2 = value.v1;
                this.v3 = value.v2;
            }
        }
        
        public vec3 gba {
            get {
                return new vec3(this.v1, this.v2, this.v3);
            }
            set {
                this.v1 = value.v0;
                this.v2 = value.v1;
                this.v3 = value.v2;
            }
        }
        
        public vec3 tpq {
            get {
                return new vec3(this.v1, this.v2, this.v3);
            }
            set {
                this.v1 = value.v0;
                this.v2 = value.v1;
                this.v3 = value.v2;
            }
        }
        
        public vec3 ywx {
            get {
                return new vec3(this.v1, this.v3, this.v0);
            }
            set {
                this.v1 = value.v0;
                this.v3 = value.v1;
                this.v0 = value.v2;
            }
        }
        
        public vec3 gar {
            get {
                return new vec3(this.v1, this.v3, this.v0);
            }
            set {
                this.v1 = value.v0;
                this.v3 = value.v1;
                this.v0 = value.v2;
            }
        }
        
        public vec3 tqs {
            get {
                return new vec3(this.v1, this.v3, this.v0);
            }
            set {
                this.v1 = value.v0;
                this.v3 = value.v1;
                this.v0 = value.v2;
            }
        }
        
        public vec3 ywy {
            get {
                return new vec3(this.v1, this.v3, this.v1);
            }
        }
        
        public vec3 gag {
            get {
                return new vec3(this.v1, this.v3, this.v1);
            }
        }
        
        public vec3 tqt {
            get {
                return new vec3(this.v1, this.v3, this.v1);
            }
        }
        
        public vec3 ywz {
            get {
                return new vec3(this.v1, this.v3, this.v2);
            }
            set {
                this.v1 = value.v0;
                this.v3 = value.v1;
                this.v2 = value.v2;
            }
        }
        
        public vec3 gab {
            get {
                return new vec3(this.v1, this.v3, this.v2);
            }
            set {
                this.v1 = value.v0;
                this.v3 = value.v1;
                this.v2 = value.v2;
            }
        }
        
        public vec3 tqp {
            get {
                return new vec3(this.v1, this.v3, this.v2);
            }
            set {
                this.v1 = value.v0;
                this.v3 = value.v1;
                this.v2 = value.v2;
            }
        }
        
        public vec3 yww {
            get {
                return new vec3(this.v1, this.v3, this.v3);
            }
        }
        
        public vec3 gaa {
            get {
                return new vec3(this.v1, this.v3, this.v3);
            }
        }
        
        public vec3 tqq {
            get {
                return new vec3(this.v1, this.v3, this.v3);
            }
        }
        
        public vec3 zxx {
            get {
                return new vec3(this.v2, this.v0, this.v0);
            }
        }
        
        public vec3 brr {
            get {
                return new vec3(this.v2, this.v0, this.v0);
            }
        }
        
        public vec3 pss {
            get {
                return new vec3(this.v2, this.v0, this.v0);
            }
        }
        
        public vec3 zxy {
            get {
                return new vec3(this.v2, this.v0, this.v1);
            }
            set {
                this.v2 = value.v0;
                this.v0 = value.v1;
                this.v1 = value.v2;
            }
        }
        
        public vec3 brg {
            get {
                return new vec3(this.v2, this.v0, this.v1);
            }
            set {
                this.v2 = value.v0;
                this.v0 = value.v1;
                this.v1 = value.v2;
            }
        }
        
        public vec3 pst {
            get {
                return new vec3(this.v2, this.v0, this.v1);
            }
            set {
                this.v2 = value.v0;
                this.v0 = value.v1;
                this.v1 = value.v2;
            }
        }
        
        public vec3 zxz {
            get {
                return new vec3(this.v2, this.v0, this.v2);
            }
        }
        
        public vec3 brb {
            get {
                return new vec3(this.v2, this.v0, this.v2);
            }
        }
        
        public vec3 psp {
            get {
                return new vec3(this.v2, this.v0, this.v2);
            }
        }
        
        public vec3 zxw {
            get {
                return new vec3(this.v2, this.v0, this.v3);
            }
            set {
                this.v2 = value.v0;
                this.v0 = value.v1;
                this.v3 = value.v2;
            }
        }
        
        public vec3 bra {
            get {
                return new vec3(this.v2, this.v0, this.v3);
            }
            set {
                this.v2 = value.v0;
                this.v0 = value.v1;
                this.v3 = value.v2;
            }
        }
        
        public vec3 psq {
            get {
                return new vec3(this.v2, this.v0, this.v3);
            }
            set {
                this.v2 = value.v0;
                this.v0 = value.v1;
                this.v3 = value.v2;
            }
        }
        
        public vec3 zyx {
            get {
                return new vec3(this.v2, this.v1, this.v0);
            }
            set {
                this.v2 = value.v0;
                this.v1 = value.v1;
                this.v0 = value.v2;
            }
        }
        
        public vec3 bgr {
            get {
                return new vec3(this.v2, this.v1, this.v0);
            }
            set {
                this.v2 = value.v0;
                this.v1 = value.v1;
                this.v0 = value.v2;
            }
        }
        
        public vec3 pts {
            get {
                return new vec3(this.v2, this.v1, this.v0);
            }
            set {
                this.v2 = value.v0;
                this.v1 = value.v1;
                this.v0 = value.v2;
            }
        }
        
        public vec3 zyy {
            get {
                return new vec3(this.v2, this.v1, this.v1);
            }
        }
        
        public vec3 bgg {
            get {
                return new vec3(this.v2, this.v1, this.v1);
            }
        }
        
        public vec3 ptt {
            get {
                return new vec3(this.v2, this.v1, this.v1);
            }
        }
        
        public vec3 zyz {
            get {
                return new vec3(this.v2, this.v1, this.v2);
            }
        }
        
        public vec3 bgb {
            get {
                return new vec3(this.v2, this.v1, this.v2);
            }
        }
        
        public vec3 ptp {
            get {
                return new vec3(this.v2, this.v1, this.v2);
            }
        }
        
        public vec3 zyw {
            get {
                return new vec3(this.v2, this.v1, this.v3);
            }
            set {
                this.v2 = value.v0;
                this.v1 = value.v1;
                this.v3 = value.v2;
            }
        }
        
        public vec3 bga {
            get {
                return new vec3(this.v2, this.v1, this.v3);
            }
            set {
                this.v2 = value.v0;
                this.v1 = value.v1;
                this.v3 = value.v2;
            }
        }
        
        public vec3 ptq {
            get {
                return new vec3(this.v2, this.v1, this.v3);
            }
            set {
                this.v2 = value.v0;
                this.v1 = value.v1;
                this.v3 = value.v2;
            }
        }
        
        public vec3 zzx {
            get {
                return new vec3(this.v2, this.v2, this.v0);
            }
        }
        
        public vec3 bbr {
            get {
                return new vec3(this.v2, this.v2, this.v0);
            }
        }
        
        public vec3 pps {
            get {
                return new vec3(this.v2, this.v2, this.v0);
            }
        }
        
        public vec3 zzy {
            get {
                return new vec3(this.v2, this.v2, this.v1);
            }
        }
        
        public vec3 bbg {
            get {
                return new vec3(this.v2, this.v2, this.v1);
            }
        }
        
        public vec3 ppt {
            get {
                return new vec3(this.v2, this.v2, this.v1);
            }
        }
        
        public vec3 zzz {
            get {
                return new vec3(this.v2, this.v2, this.v2);
            }
        }
        
        public vec3 bbb {
            get {
                return new vec3(this.v2, this.v2, this.v2);
            }
        }
        
        public vec3 ppp {
            get {
                return new vec3(this.v2, this.v2, this.v2);
            }
        }
        
        public vec3 zzw {
            get {
                return new vec3(this.v2, this.v2, this.v3);
            }
        }
        
        public vec3 bba {
            get {
                return new vec3(this.v2, this.v2, this.v3);
            }
        }
        
        public vec3 ppq {
            get {
                return new vec3(this.v2, this.v2, this.v3);
            }
        }
        
        public vec3 zwx {
            get {
                return new vec3(this.v2, this.v3, this.v0);
            }
            set {
                this.v2 = value.v0;
                this.v3 = value.v1;
                this.v0 = value.v2;
            }
        }
        
        public vec3 bar {
            get {
                return new vec3(this.v2, this.v3, this.v0);
            }
            set {
                this.v2 = value.v0;
                this.v3 = value.v1;
                this.v0 = value.v2;
            }
        }
        
        public vec3 pqs {
            get {
                return new vec3(this.v2, this.v3, this.v0);
            }
            set {
                this.v2 = value.v0;
                this.v3 = value.v1;
                this.v0 = value.v2;
            }
        }
        
        public vec3 zwy {
            get {
                return new vec3(this.v2, this.v3, this.v1);
            }
            set {
                this.v2 = value.v0;
                this.v3 = value.v1;
                this.v1 = value.v2;
            }
        }
        
        public vec3 bag {
            get {
                return new vec3(this.v2, this.v3, this.v1);
            }
            set {
                this.v2 = value.v0;
                this.v3 = value.v1;
                this.v1 = value.v2;
            }
        }
        
        public vec3 pqt {
            get {
                return new vec3(this.v2, this.v3, this.v1);
            }
            set {
                this.v2 = value.v0;
                this.v3 = value.v1;
                this.v1 = value.v2;
            }
        }
        
        public vec3 zwz {
            get {
                return new vec3(this.v2, this.v3, this.v2);
            }
        }
        
        public vec3 bab {
            get {
                return new vec3(this.v2, this.v3, this.v2);
            }
        }
        
        public vec3 pqp {
            get {
                return new vec3(this.v2, this.v3, this.v2);
            }
        }
        
        public vec3 zww {
            get {
                return new vec3(this.v2, this.v3, this.v3);
            }
        }
        
        public vec3 baa {
            get {
                return new vec3(this.v2, this.v3, this.v3);
            }
        }
        
        public vec3 pqq {
            get {
                return new vec3(this.v2, this.v3, this.v3);
            }
        }
        
        public vec3 wxx {
            get {
                return new vec3(this.v3, this.v0, this.v0);
            }
        }
        
        public vec3 arr {
            get {
                return new vec3(this.v3, this.v0, this.v0);
            }
        }
        
        public vec3 qss {
            get {
                return new vec3(this.v3, this.v0, this.v0);
            }
        }
        
        public vec3 wxy {
            get {
                return new vec3(this.v3, this.v0, this.v1);
            }
            set {
                this.v3 = value.v0;
                this.v0 = value.v1;
                this.v1 = value.v2;
            }
        }
        
        public vec3 arg {
            get {
                return new vec3(this.v3, this.v0, this.v1);
            }
            set {
                this.v3 = value.v0;
                this.v0 = value.v1;
                this.v1 = value.v2;
            }
        }
        
        public vec3 qst {
            get {
                return new vec3(this.v3, this.v0, this.v1);
            }
            set {
                this.v3 = value.v0;
                this.v0 = value.v1;
                this.v1 = value.v2;
            }
        }
        
        public vec3 wxz {
            get {
                return new vec3(this.v3, this.v0, this.v2);
            }
            set {
                this.v3 = value.v0;
                this.v0 = value.v1;
                this.v2 = value.v2;
            }
        }
        
        public vec3 arb {
            get {
                return new vec3(this.v3, this.v0, this.v2);
            }
            set {
                this.v3 = value.v0;
                this.v0 = value.v1;
                this.v2 = value.v2;
            }
        }
        
        public vec3 qsp {
            get {
                return new vec3(this.v3, this.v0, this.v2);
            }
            set {
                this.v3 = value.v0;
                this.v0 = value.v1;
                this.v2 = value.v2;
            }
        }
        
        public vec3 wxw {
            get {
                return new vec3(this.v3, this.v0, this.v3);
            }
        }
        
        public vec3 ara {
            get {
                return new vec3(this.v3, this.v0, this.v3);
            }
        }
        
        public vec3 qsq {
            get {
                return new vec3(this.v3, this.v0, this.v3);
            }
        }
        
        public vec3 wyx {
            get {
                return new vec3(this.v3, this.v1, this.v0);
            }
            set {
                this.v3 = value.v0;
                this.v1 = value.v1;
                this.v0 = value.v2;
            }
        }
        
        public vec3 agr {
            get {
                return new vec3(this.v3, this.v1, this.v0);
            }
            set {
                this.v3 = value.v0;
                this.v1 = value.v1;
                this.v0 = value.v2;
            }
        }
        
        public vec3 qts {
            get {
                return new vec3(this.v3, this.v1, this.v0);
            }
            set {
                this.v3 = value.v0;
                this.v1 = value.v1;
                this.v0 = value.v2;
            }
        }
        
        public vec3 wyy {
            get {
                return new vec3(this.v3, this.v1, this.v1);
            }
        }
        
        public vec3 agg {
            get {
                return new vec3(this.v3, this.v1, this.v1);
            }
        }
        
        public vec3 qtt {
            get {
                return new vec3(this.v3, this.v1, this.v1);
            }
        }
        
        public vec3 wyz {
            get {
                return new vec3(this.v3, this.v1, this.v2);
            }
            set {
                this.v3 = value.v0;
                this.v1 = value.v1;
                this.v2 = value.v2;
            }
        }
        
        public vec3 agb {
            get {
                return new vec3(this.v3, this.v1, this.v2);
            }
            set {
                this.v3 = value.v0;
                this.v1 = value.v1;
                this.v2 = value.v2;
            }
        }
        
        public vec3 qtp {
            get {
                return new vec3(this.v3, this.v1, this.v2);
            }
            set {
                this.v3 = value.v0;
                this.v1 = value.v1;
                this.v2 = value.v2;
            }
        }
        
        public vec3 wyw {
            get {
                return new vec3(this.v3, this.v1, this.v3);
            }
        }
        
        public vec3 aga {
            get {
                return new vec3(this.v3, this.v1, this.v3);
            }
        }
        
        public vec3 qtq {
            get {
                return new vec3(this.v3, this.v1, this.v3);
            }
        }
        
        public vec3 wzx {
            get {
                return new vec3(this.v3, this.v2, this.v0);
            }
            set {
                this.v3 = value.v0;
                this.v2 = value.v1;
                this.v0 = value.v2;
            }
        }
        
        public vec3 abr {
            get {
                return new vec3(this.v3, this.v2, this.v0);
            }
            set {
                this.v3 = value.v0;
                this.v2 = value.v1;
                this.v0 = value.v2;
            }
        }
        
        public vec3 qps {
            get {
                return new vec3(this.v3, this.v2, this.v0);
            }
            set {
                this.v3 = value.v0;
                this.v2 = value.v1;
                this.v0 = value.v2;
            }
        }
        
        public vec3 wzy {
            get {
                return new vec3(this.v3, this.v2, this.v1);
            }
            set {
                this.v3 = value.v0;
                this.v2 = value.v1;
                this.v1 = value.v2;
            }
        }
        
        public vec3 abg {
            get {
                return new vec3(this.v3, this.v2, this.v1);
            }
            set {
                this.v3 = value.v0;
                this.v2 = value.v1;
                this.v1 = value.v2;
            }
        }
        
        public vec3 qpt {
            get {
                return new vec3(this.v3, this.v2, this.v1);
            }
            set {
                this.v3 = value.v0;
                this.v2 = value.v1;
                this.v1 = value.v2;
            }
        }
        
        public vec3 wzz {
            get {
                return new vec3(this.v3, this.v2, this.v2);
            }
        }
        
        public vec3 abb {
            get {
                return new vec3(this.v3, this.v2, this.v2);
            }
        }
        
        public vec3 qpp {
            get {
                return new vec3(this.v3, this.v2, this.v2);
            }
        }
        
        public vec3 wzw {
            get {
                return new vec3(this.v3, this.v2, this.v3);
            }
        }
        
        public vec3 aba {
            get {
                return new vec3(this.v3, this.v2, this.v3);
            }
        }
        
        public vec3 qpq {
            get {
                return new vec3(this.v3, this.v2, this.v3);
            }
        }
        
        public vec3 wwx {
            get {
                return new vec3(this.v3, this.v3, this.v0);
            }
        }
        
        public vec3 aar {
            get {
                return new vec3(this.v3, this.v3, this.v0);
            }
        }
        
        public vec3 qqs {
            get {
                return new vec3(this.v3, this.v3, this.v0);
            }
        }
        
        public vec3 wwy {
            get {
                return new vec3(this.v3, this.v3, this.v1);
            }
        }
        
        public vec3 aag {
            get {
                return new vec3(this.v3, this.v3, this.v1);
            }
        }
        
        public vec3 qqt {
            get {
                return new vec3(this.v3, this.v3, this.v1);
            }
        }
        
        public vec3 wwz {
            get {
                return new vec3(this.v3, this.v3, this.v2);
            }
        }
        
        public vec3 aab {
            get {
                return new vec3(this.v3, this.v3, this.v2);
            }
        }
        
        public vec3 qqp {
            get {
                return new vec3(this.v3, this.v3, this.v2);
            }
        }
        
        public vec3 www {
            get {
                return new vec3(this.v3, this.v3, this.v3);
            }
        }
        
        public vec3 aaa {
            get {
                return new vec3(this.v3, this.v3, this.v3);
            }
        }
        
        public vec3 qqq {
            get {
                return new vec3(this.v3, this.v3, this.v3);
            }
        }
        
        public vec4 xxxx {
            get {
                return new vec4(this.v0, this.v0, this.v0, this.v0);
            }
        }
        
        public vec4 rrrr {
            get {
                return new vec4(this.v0, this.v0, this.v0, this.v0);
            }
        }
        
        public vec4 ssss {
            get {
                return new vec4(this.v0, this.v0, this.v0, this.v0);
            }
        }
        
        public vec4 xxxy {
            get {
                return new vec4(this.v0, this.v0, this.v0, this.v1);
            }
        }
        
        public vec4 rrrg {
            get {
                return new vec4(this.v0, this.v0, this.v0, this.v1);
            }
        }
        
        public vec4 ssst {
            get {
                return new vec4(this.v0, this.v0, this.v0, this.v1);
            }
        }
        
        public vec4 xxxz {
            get {
                return new vec4(this.v0, this.v0, this.v0, this.v2);
            }
        }
        
        public vec4 rrrb {
            get {
                return new vec4(this.v0, this.v0, this.v0, this.v2);
            }
        }
        
        public vec4 sssp {
            get {
                return new vec4(this.v0, this.v0, this.v0, this.v2);
            }
        }
        
        public vec4 xxxw {
            get {
                return new vec4(this.v0, this.v0, this.v0, this.v3);
            }
        }
        
        public vec4 rrra {
            get {
                return new vec4(this.v0, this.v0, this.v0, this.v3);
            }
        }
        
        public vec4 sssq {
            get {
                return new vec4(this.v0, this.v0, this.v0, this.v3);
            }
        }
        
        public vec4 xxyx {
            get {
                return new vec4(this.v0, this.v0, this.v1, this.v0);
            }
        }
        
        public vec4 rrgr {
            get {
                return new vec4(this.v0, this.v0, this.v1, this.v0);
            }
        }
        
        public vec4 ssts {
            get {
                return new vec4(this.v0, this.v0, this.v1, this.v0);
            }
        }
        
        public vec4 xxyy {
            get {
                return new vec4(this.v0, this.v0, this.v1, this.v1);
            }
        }
        
        public vec4 rrgg {
            get {
                return new vec4(this.v0, this.v0, this.v1, this.v1);
            }
        }
        
        public vec4 sstt {
            get {
                return new vec4(this.v0, this.v0, this.v1, this.v1);
            }
        }
        
        public vec4 xxyz {
            get {
                return new vec4(this.v0, this.v0, this.v1, this.v2);
            }
        }
        
        public vec4 rrgb {
            get {
                return new vec4(this.v0, this.v0, this.v1, this.v2);
            }
        }
        
        public vec4 sstp {
            get {
                return new vec4(this.v0, this.v0, this.v1, this.v2);
            }
        }
        
        public vec4 xxyw {
            get {
                return new vec4(this.v0, this.v0, this.v1, this.v3);
            }
        }
        
        public vec4 rrga {
            get {
                return new vec4(this.v0, this.v0, this.v1, this.v3);
            }
        }
        
        public vec4 sstq {
            get {
                return new vec4(this.v0, this.v0, this.v1, this.v3);
            }
        }
        
        public vec4 xxzx {
            get {
                return new vec4(this.v0, this.v0, this.v2, this.v0);
            }
        }
        
        public vec4 rrbr {
            get {
                return new vec4(this.v0, this.v0, this.v2, this.v0);
            }
        }
        
        public vec4 ssps {
            get {
                return new vec4(this.v0, this.v0, this.v2, this.v0);
            }
        }
        
        public vec4 xxzy {
            get {
                return new vec4(this.v0, this.v0, this.v2, this.v1);
            }
        }
        
        public vec4 rrbg {
            get {
                return new vec4(this.v0, this.v0, this.v2, this.v1);
            }
        }
        
        public vec4 sspt {
            get {
                return new vec4(this.v0, this.v0, this.v2, this.v1);
            }
        }
        
        public vec4 xxzz {
            get {
                return new vec4(this.v0, this.v0, this.v2, this.v2);
            }
        }
        
        public vec4 rrbb {
            get {
                return new vec4(this.v0, this.v0, this.v2, this.v2);
            }
        }
        
        public vec4 sspp {
            get {
                return new vec4(this.v0, this.v0, this.v2, this.v2);
            }
        }
        
        public vec4 xxzw {
            get {
                return new vec4(this.v0, this.v0, this.v2, this.v3);
            }
        }
        
        public vec4 rrba {
            get {
                return new vec4(this.v0, this.v0, this.v2, this.v3);
            }
        }
        
        public vec4 sspq {
            get {
                return new vec4(this.v0, this.v0, this.v2, this.v3);
            }
        }
        
        public vec4 xxwx {
            get {
                return new vec4(this.v0, this.v0, this.v3, this.v0);
            }
        }
        
        public vec4 rrar {
            get {
                return new vec4(this.v0, this.v0, this.v3, this.v0);
            }
        }
        
        public vec4 ssqs {
            get {
                return new vec4(this.v0, this.v0, this.v3, this.v0);
            }
        }
        
        public vec4 xxwy {
            get {
                return new vec4(this.v0, this.v0, this.v3, this.v1);
            }
        }
        
        public vec4 rrag {
            get {
                return new vec4(this.v0, this.v0, this.v3, this.v1);
            }
        }
        
        public vec4 ssqt {
            get {
                return new vec4(this.v0, this.v0, this.v3, this.v1);
            }
        }
        
        public vec4 xxwz {
            get {
                return new vec4(this.v0, this.v0, this.v3, this.v2);
            }
        }
        
        public vec4 rrab {
            get {
                return new vec4(this.v0, this.v0, this.v3, this.v2);
            }
        }
        
        public vec4 ssqp {
            get {
                return new vec4(this.v0, this.v0, this.v3, this.v2);
            }
        }
        
        public vec4 xxww {
            get {
                return new vec4(this.v0, this.v0, this.v3, this.v3);
            }
        }
        
        public vec4 rraa {
            get {
                return new vec4(this.v0, this.v0, this.v3, this.v3);
            }
        }
        
        public vec4 ssqq {
            get {
                return new vec4(this.v0, this.v0, this.v3, this.v3);
            }
        }
        
        public vec4 xyxx {
            get {
                return new vec4(this.v0, this.v1, this.v0, this.v0);
            }
        }
        
        public vec4 rgrr {
            get {
                return new vec4(this.v0, this.v1, this.v0, this.v0);
            }
        }
        
        public vec4 stss {
            get {
                return new vec4(this.v0, this.v1, this.v0, this.v0);
            }
        }
        
        public vec4 xyxy {
            get {
                return new vec4(this.v0, this.v1, this.v0, this.v1);
            }
        }
        
        public vec4 rgrg {
            get {
                return new vec4(this.v0, this.v1, this.v0, this.v1);
            }
        }
        
        public vec4 stst {
            get {
                return new vec4(this.v0, this.v1, this.v0, this.v1);
            }
        }
        
        public vec4 xyxz {
            get {
                return new vec4(this.v0, this.v1, this.v0, this.v2);
            }
        }
        
        public vec4 rgrb {
            get {
                return new vec4(this.v0, this.v1, this.v0, this.v2);
            }
        }
        
        public vec4 stsp {
            get {
                return new vec4(this.v0, this.v1, this.v0, this.v2);
            }
        }
        
        public vec4 xyxw {
            get {
                return new vec4(this.v0, this.v1, this.v0, this.v3);
            }
        }
        
        public vec4 rgra {
            get {
                return new vec4(this.v0, this.v1, this.v0, this.v3);
            }
        }
        
        public vec4 stsq {
            get {
                return new vec4(this.v0, this.v1, this.v0, this.v3);
            }
        }
        
        public vec4 xyyx {
            get {
                return new vec4(this.v0, this.v1, this.v1, this.v0);
            }
        }
        
        public vec4 rggr {
            get {
                return new vec4(this.v0, this.v1, this.v1, this.v0);
            }
        }
        
        public vec4 stts {
            get {
                return new vec4(this.v0, this.v1, this.v1, this.v0);
            }
        }
        
        public vec4 xyyy {
            get {
                return new vec4(this.v0, this.v1, this.v1, this.v1);
            }
        }
        
        public vec4 rggg {
            get {
                return new vec4(this.v0, this.v1, this.v1, this.v1);
            }
        }
        
        public vec4 sttt {
            get {
                return new vec4(this.v0, this.v1, this.v1, this.v1);
            }
        }
        
        public vec4 xyyz {
            get {
                return new vec4(this.v0, this.v1, this.v1, this.v2);
            }
        }
        
        public vec4 rggb {
            get {
                return new vec4(this.v0, this.v1, this.v1, this.v2);
            }
        }
        
        public vec4 sttp {
            get {
                return new vec4(this.v0, this.v1, this.v1, this.v2);
            }
        }
        
        public vec4 xyyw {
            get {
                return new vec4(this.v0, this.v1, this.v1, this.v3);
            }
        }
        
        public vec4 rgga {
            get {
                return new vec4(this.v0, this.v1, this.v1, this.v3);
            }
        }
        
        public vec4 sttq {
            get {
                return new vec4(this.v0, this.v1, this.v1, this.v3);
            }
        }
        
        public vec4 xyzx {
            get {
                return new vec4(this.v0, this.v1, this.v2, this.v0);
            }
        }
        
        public vec4 rgbr {
            get {
                return new vec4(this.v0, this.v1, this.v2, this.v0);
            }
        }
        
        public vec4 stps {
            get {
                return new vec4(this.v0, this.v1, this.v2, this.v0);
            }
        }
        
        public vec4 xyzy {
            get {
                return new vec4(this.v0, this.v1, this.v2, this.v1);
            }
        }
        
        public vec4 rgbg {
            get {
                return new vec4(this.v0, this.v1, this.v2, this.v1);
            }
        }
        
        public vec4 stpt {
            get {
                return new vec4(this.v0, this.v1, this.v2, this.v1);
            }
        }
        
        public vec4 xyzz {
            get {
                return new vec4(this.v0, this.v1, this.v2, this.v2);
            }
        }
        
        public vec4 rgbb {
            get {
                return new vec4(this.v0, this.v1, this.v2, this.v2);
            }
        }
        
        public vec4 stpp {
            get {
                return new vec4(this.v0, this.v1, this.v2, this.v2);
            }
        }
        
        public vec4 xyzw {
            get {
                return new vec4(this.v0, this.v1, this.v2, this.v3);
            }
            set {
                this.v0 = value.v0;
                this.v1 = value.v1;
                this.v2 = value.v2;
                this.v3 = value.v3;
            }
        }
        
        public vec4 rgba {
            get {
                return new vec4(this.v0, this.v1, this.v2, this.v3);
            }
            set {
                this.v0 = value.v0;
                this.v1 = value.v1;
                this.v2 = value.v2;
                this.v3 = value.v3;
            }
        }
        
        public vec4 stpq {
            get {
                return new vec4(this.v0, this.v1, this.v2, this.v3);
            }
            set {
                this.v0 = value.v0;
                this.v1 = value.v1;
                this.v2 = value.v2;
                this.v3 = value.v3;
            }
        }
        
        public vec4 xywx {
            get {
                return new vec4(this.v0, this.v1, this.v3, this.v0);
            }
        }
        
        public vec4 rgar {
            get {
                return new vec4(this.v0, this.v1, this.v3, this.v0);
            }
        }
        
        public vec4 stqs {
            get {
                return new vec4(this.v0, this.v1, this.v3, this.v0);
            }
        }
        
        public vec4 xywy {
            get {
                return new vec4(this.v0, this.v1, this.v3, this.v1);
            }
        }
        
        public vec4 rgag {
            get {
                return new vec4(this.v0, this.v1, this.v3, this.v1);
            }
        }
        
        public vec4 stqt {
            get {
                return new vec4(this.v0, this.v1, this.v3, this.v1);
            }
        }
        
        public vec4 xywz {
            get {
                return new vec4(this.v0, this.v1, this.v3, this.v2);
            }
            set {
                this.v0 = value.v0;
                this.v1 = value.v1;
                this.v3 = value.v2;
                this.v2 = value.v3;
            }
        }
        
        public vec4 rgab {
            get {
                return new vec4(this.v0, this.v1, this.v3, this.v2);
            }
            set {
                this.v0 = value.v0;
                this.v1 = value.v1;
                this.v3 = value.v2;
                this.v2 = value.v3;
            }
        }
        
        public vec4 stqp {
            get {
                return new vec4(this.v0, this.v1, this.v3, this.v2);
            }
            set {
                this.v0 = value.v0;
                this.v1 = value.v1;
                this.v3 = value.v2;
                this.v2 = value.v3;
            }
        }
        
        public vec4 xyww {
            get {
                return new vec4(this.v0, this.v1, this.v3, this.v3);
            }
        }
        
        public vec4 rgaa {
            get {
                return new vec4(this.v0, this.v1, this.v3, this.v3);
            }
        }
        
        public vec4 stqq {
            get {
                return new vec4(this.v0, this.v1, this.v3, this.v3);
            }
        }
        
        public vec4 xzxx {
            get {
                return new vec4(this.v0, this.v2, this.v0, this.v0);
            }
        }
        
        public vec4 rbrr {
            get {
                return new vec4(this.v0, this.v2, this.v0, this.v0);
            }
        }
        
        public vec4 spss {
            get {
                return new vec4(this.v0, this.v2, this.v0, this.v0);
            }
        }
        
        public vec4 xzxy {
            get {
                return new vec4(this.v0, this.v2, this.v0, this.v1);
            }
        }
        
        public vec4 rbrg {
            get {
                return new vec4(this.v0, this.v2, this.v0, this.v1);
            }
        }
        
        public vec4 spst {
            get {
                return new vec4(this.v0, this.v2, this.v0, this.v1);
            }
        }
        
        public vec4 xzxz {
            get {
                return new vec4(this.v0, this.v2, this.v0, this.v2);
            }
        }
        
        public vec4 rbrb {
            get {
                return new vec4(this.v0, this.v2, this.v0, this.v2);
            }
        }
        
        public vec4 spsp {
            get {
                return new vec4(this.v0, this.v2, this.v0, this.v2);
            }
        }
        
        public vec4 xzxw {
            get {
                return new vec4(this.v0, this.v2, this.v0, this.v3);
            }
        }
        
        public vec4 rbra {
            get {
                return new vec4(this.v0, this.v2, this.v0, this.v3);
            }
        }
        
        public vec4 spsq {
            get {
                return new vec4(this.v0, this.v2, this.v0, this.v3);
            }
        }
        
        public vec4 xzyx {
            get {
                return new vec4(this.v0, this.v2, this.v1, this.v0);
            }
        }
        
        public vec4 rbgr {
            get {
                return new vec4(this.v0, this.v2, this.v1, this.v0);
            }
        }
        
        public vec4 spts {
            get {
                return new vec4(this.v0, this.v2, this.v1, this.v0);
            }
        }
        
        public vec4 xzyy {
            get {
                return new vec4(this.v0, this.v2, this.v1, this.v1);
            }
        }
        
        public vec4 rbgg {
            get {
                return new vec4(this.v0, this.v2, this.v1, this.v1);
            }
        }
        
        public vec4 sptt {
            get {
                return new vec4(this.v0, this.v2, this.v1, this.v1);
            }
        }
        
        public vec4 xzyz {
            get {
                return new vec4(this.v0, this.v2, this.v1, this.v2);
            }
        }
        
        public vec4 rbgb {
            get {
                return new vec4(this.v0, this.v2, this.v1, this.v2);
            }
        }
        
        public vec4 sptp {
            get {
                return new vec4(this.v0, this.v2, this.v1, this.v2);
            }
        }
        
        public vec4 xzyw {
            get {
                return new vec4(this.v0, this.v2, this.v1, this.v3);
            }
            set {
                this.v0 = value.v0;
                this.v2 = value.v1;
                this.v1 = value.v2;
                this.v3 = value.v3;
            }
        }
        
        public vec4 rbga {
            get {
                return new vec4(this.v0, this.v2, this.v1, this.v3);
            }
            set {
                this.v0 = value.v0;
                this.v2 = value.v1;
                this.v1 = value.v2;
                this.v3 = value.v3;
            }
        }
        
        public vec4 sptq {
            get {
                return new vec4(this.v0, this.v2, this.v1, this.v3);
            }
            set {
                this.v0 = value.v0;
                this.v2 = value.v1;
                this.v1 = value.v2;
                this.v3 = value.v3;
            }
        }
        
        public vec4 xzzx {
            get {
                return new vec4(this.v0, this.v2, this.v2, this.v0);
            }
        }
        
        public vec4 rbbr {
            get {
                return new vec4(this.v0, this.v2, this.v2, this.v0);
            }
        }
        
        public vec4 spps {
            get {
                return new vec4(this.v0, this.v2, this.v2, this.v0);
            }
        }
        
        public vec4 xzzy {
            get {
                return new vec4(this.v0, this.v2, this.v2, this.v1);
            }
        }
        
        public vec4 rbbg {
            get {
                return new vec4(this.v0, this.v2, this.v2, this.v1);
            }
        }
        
        public vec4 sppt {
            get {
                return new vec4(this.v0, this.v2, this.v2, this.v1);
            }
        }
        
        public vec4 xzzz {
            get {
                return new vec4(this.v0, this.v2, this.v2, this.v2);
            }
        }
        
        public vec4 rbbb {
            get {
                return new vec4(this.v0, this.v2, this.v2, this.v2);
            }
        }
        
        public vec4 sppp {
            get {
                return new vec4(this.v0, this.v2, this.v2, this.v2);
            }
        }
        
        public vec4 xzzw {
            get {
                return new vec4(this.v0, this.v2, this.v2, this.v3);
            }
        }
        
        public vec4 rbba {
            get {
                return new vec4(this.v0, this.v2, this.v2, this.v3);
            }
        }
        
        public vec4 sppq {
            get {
                return new vec4(this.v0, this.v2, this.v2, this.v3);
            }
        }
        
        public vec4 xzwx {
            get {
                return new vec4(this.v0, this.v2, this.v3, this.v0);
            }
        }
        
        public vec4 rbar {
            get {
                return new vec4(this.v0, this.v2, this.v3, this.v0);
            }
        }
        
        public vec4 spqs {
            get {
                return new vec4(this.v0, this.v2, this.v3, this.v0);
            }
        }
        
        public vec4 xzwy {
            get {
                return new vec4(this.v0, this.v2, this.v3, this.v1);
            }
            set {
                this.v0 = value.v0;
                this.v2 = value.v1;
                this.v3 = value.v2;
                this.v1 = value.v3;
            }
        }
        
        public vec4 rbag {
            get {
                return new vec4(this.v0, this.v2, this.v3, this.v1);
            }
            set {
                this.v0 = value.v0;
                this.v2 = value.v1;
                this.v3 = value.v2;
                this.v1 = value.v3;
            }
        }
        
        public vec4 spqt {
            get {
                return new vec4(this.v0, this.v2, this.v3, this.v1);
            }
            set {
                this.v0 = value.v0;
                this.v2 = value.v1;
                this.v3 = value.v2;
                this.v1 = value.v3;
            }
        }
        
        public vec4 xzwz {
            get {
                return new vec4(this.v0, this.v2, this.v3, this.v2);
            }
        }
        
        public vec4 rbab {
            get {
                return new vec4(this.v0, this.v2, this.v3, this.v2);
            }
        }
        
        public vec4 spqp {
            get {
                return new vec4(this.v0, this.v2, this.v3, this.v2);
            }
        }
        
        public vec4 xzww {
            get {
                return new vec4(this.v0, this.v2, this.v3, this.v3);
            }
        }
        
        public vec4 rbaa {
            get {
                return new vec4(this.v0, this.v2, this.v3, this.v3);
            }
        }
        
        public vec4 spqq {
            get {
                return new vec4(this.v0, this.v2, this.v3, this.v3);
            }
        }
        
        public vec4 xwxx {
            get {
                return new vec4(this.v0, this.v3, this.v0, this.v0);
            }
        }
        
        public vec4 rarr {
            get {
                return new vec4(this.v0, this.v3, this.v0, this.v0);
            }
        }
        
        public vec4 sqss {
            get {
                return new vec4(this.v0, this.v3, this.v0, this.v0);
            }
        }
        
        public vec4 xwxy {
            get {
                return new vec4(this.v0, this.v3, this.v0, this.v1);
            }
        }
        
        public vec4 rarg {
            get {
                return new vec4(this.v0, this.v3, this.v0, this.v1);
            }
        }
        
        public vec4 sqst {
            get {
                return new vec4(this.v0, this.v3, this.v0, this.v1);
            }
        }
        
        public vec4 xwxz {
            get {
                return new vec4(this.v0, this.v3, this.v0, this.v2);
            }
        }
        
        public vec4 rarb {
            get {
                return new vec4(this.v0, this.v3, this.v0, this.v2);
            }
        }
        
        public vec4 sqsp {
            get {
                return new vec4(this.v0, this.v3, this.v0, this.v2);
            }
        }
        
        public vec4 xwxw {
            get {
                return new vec4(this.v0, this.v3, this.v0, this.v3);
            }
        }
        
        public vec4 rara {
            get {
                return new vec4(this.v0, this.v3, this.v0, this.v3);
            }
        }
        
        public vec4 sqsq {
            get {
                return new vec4(this.v0, this.v3, this.v0, this.v3);
            }
        }
        
        public vec4 xwyx {
            get {
                return new vec4(this.v0, this.v3, this.v1, this.v0);
            }
        }
        
        public vec4 ragr {
            get {
                return new vec4(this.v0, this.v3, this.v1, this.v0);
            }
        }
        
        public vec4 sqts {
            get {
                return new vec4(this.v0, this.v3, this.v1, this.v0);
            }
        }
        
        public vec4 xwyy {
            get {
                return new vec4(this.v0, this.v3, this.v1, this.v1);
            }
        }
        
        public vec4 ragg {
            get {
                return new vec4(this.v0, this.v3, this.v1, this.v1);
            }
        }
        
        public vec4 sqtt {
            get {
                return new vec4(this.v0, this.v3, this.v1, this.v1);
            }
        }
        
        public vec4 xwyz {
            get {
                return new vec4(this.v0, this.v3, this.v1, this.v2);
            }
            set {
                this.v0 = value.v0;
                this.v3 = value.v1;
                this.v1 = value.v2;
                this.v2 = value.v3;
            }
        }
        
        public vec4 ragb {
            get {
                return new vec4(this.v0, this.v3, this.v1, this.v2);
            }
            set {
                this.v0 = value.v0;
                this.v3 = value.v1;
                this.v1 = value.v2;
                this.v2 = value.v3;
            }
        }
        
        public vec4 sqtp {
            get {
                return new vec4(this.v0, this.v3, this.v1, this.v2);
            }
            set {
                this.v0 = value.v0;
                this.v3 = value.v1;
                this.v1 = value.v2;
                this.v2 = value.v3;
            }
        }
        
        public vec4 xwyw {
            get {
                return new vec4(this.v0, this.v3, this.v1, this.v3);
            }
        }
        
        public vec4 raga {
            get {
                return new vec4(this.v0, this.v3, this.v1, this.v3);
            }
        }
        
        public vec4 sqtq {
            get {
                return new vec4(this.v0, this.v3, this.v1, this.v3);
            }
        }
        
        public vec4 xwzx {
            get {
                return new vec4(this.v0, this.v3, this.v2, this.v0);
            }
        }
        
        public vec4 rabr {
            get {
                return new vec4(this.v0, this.v3, this.v2, this.v0);
            }
        }
        
        public vec4 sqps {
            get {
                return new vec4(this.v0, this.v3, this.v2, this.v0);
            }
        }
        
        public vec4 xwzy {
            get {
                return new vec4(this.v0, this.v3, this.v2, this.v1);
            }
            set {
                this.v0 = value.v0;
                this.v3 = value.v1;
                this.v2 = value.v2;
                this.v1 = value.v3;
            }
        }
        
        public vec4 rabg {
            get {
                return new vec4(this.v0, this.v3, this.v2, this.v1);
            }
            set {
                this.v0 = value.v0;
                this.v3 = value.v1;
                this.v2 = value.v2;
                this.v1 = value.v3;
            }
        }
        
        public vec4 sqpt {
            get {
                return new vec4(this.v0, this.v3, this.v2, this.v1);
            }
            set {
                this.v0 = value.v0;
                this.v3 = value.v1;
                this.v2 = value.v2;
                this.v1 = value.v3;
            }
        }
        
        public vec4 xwzz {
            get {
                return new vec4(this.v0, this.v3, this.v2, this.v2);
            }
        }
        
        public vec4 rabb {
            get {
                return new vec4(this.v0, this.v3, this.v2, this.v2);
            }
        }
        
        public vec4 sqpp {
            get {
                return new vec4(this.v0, this.v3, this.v2, this.v2);
            }
        }
        
        public vec4 xwzw {
            get {
                return new vec4(this.v0, this.v3, this.v2, this.v3);
            }
        }
        
        public vec4 raba {
            get {
                return new vec4(this.v0, this.v3, this.v2, this.v3);
            }
        }
        
        public vec4 sqpq {
            get {
                return new vec4(this.v0, this.v3, this.v2, this.v3);
            }
        }
        
        public vec4 xwwx {
            get {
                return new vec4(this.v0, this.v3, this.v3, this.v0);
            }
        }
        
        public vec4 raar {
            get {
                return new vec4(this.v0, this.v3, this.v3, this.v0);
            }
        }
        
        public vec4 sqqs {
            get {
                return new vec4(this.v0, this.v3, this.v3, this.v0);
            }
        }
        
        public vec4 xwwy {
            get {
                return new vec4(this.v0, this.v3, this.v3, this.v1);
            }
        }
        
        public vec4 raag {
            get {
                return new vec4(this.v0, this.v3, this.v3, this.v1);
            }
        }
        
        public vec4 sqqt {
            get {
                return new vec4(this.v0, this.v3, this.v3, this.v1);
            }
        }
        
        public vec4 xwwz {
            get {
                return new vec4(this.v0, this.v3, this.v3, this.v2);
            }
        }
        
        public vec4 raab {
            get {
                return new vec4(this.v0, this.v3, this.v3, this.v2);
            }
        }
        
        public vec4 sqqp {
            get {
                return new vec4(this.v0, this.v3, this.v3, this.v2);
            }
        }
        
        public vec4 xwww {
            get {
                return new vec4(this.v0, this.v3, this.v3, this.v3);
            }
        }
        
        public vec4 raaa {
            get {
                return new vec4(this.v0, this.v3, this.v3, this.v3);
            }
        }
        
        public vec4 sqqq {
            get {
                return new vec4(this.v0, this.v3, this.v3, this.v3);
            }
        }
        
        public vec4 yxxx {
            get {
                return new vec4(this.v1, this.v0, this.v0, this.v0);
            }
        }
        
        public vec4 grrr {
            get {
                return new vec4(this.v1, this.v0, this.v0, this.v0);
            }
        }
        
        public vec4 tsss {
            get {
                return new vec4(this.v1, this.v0, this.v0, this.v0);
            }
        }
        
        public vec4 yxxy {
            get {
                return new vec4(this.v1, this.v0, this.v0, this.v1);
            }
        }
        
        public vec4 grrg {
            get {
                return new vec4(this.v1, this.v0, this.v0, this.v1);
            }
        }
        
        public vec4 tsst {
            get {
                return new vec4(this.v1, this.v0, this.v0, this.v1);
            }
        }
        
        public vec4 yxxz {
            get {
                return new vec4(this.v1, this.v0, this.v0, this.v2);
            }
        }
        
        public vec4 grrb {
            get {
                return new vec4(this.v1, this.v0, this.v0, this.v2);
            }
        }
        
        public vec4 tssp {
            get {
                return new vec4(this.v1, this.v0, this.v0, this.v2);
            }
        }
        
        public vec4 yxxw {
            get {
                return new vec4(this.v1, this.v0, this.v0, this.v3);
            }
        }
        
        public vec4 grra {
            get {
                return new vec4(this.v1, this.v0, this.v0, this.v3);
            }
        }
        
        public vec4 tssq {
            get {
                return new vec4(this.v1, this.v0, this.v0, this.v3);
            }
        }
        
        public vec4 yxyx {
            get {
                return new vec4(this.v1, this.v0, this.v1, this.v0);
            }
        }
        
        public vec4 grgr {
            get {
                return new vec4(this.v1, this.v0, this.v1, this.v0);
            }
        }
        
        public vec4 tsts {
            get {
                return new vec4(this.v1, this.v0, this.v1, this.v0);
            }
        }
        
        public vec4 yxyy {
            get {
                return new vec4(this.v1, this.v0, this.v1, this.v1);
            }
        }
        
        public vec4 grgg {
            get {
                return new vec4(this.v1, this.v0, this.v1, this.v1);
            }
        }
        
        public vec4 tstt {
            get {
                return new vec4(this.v1, this.v0, this.v1, this.v1);
            }
        }
        
        public vec4 yxyz {
            get {
                return new vec4(this.v1, this.v0, this.v1, this.v2);
            }
        }
        
        public vec4 grgb {
            get {
                return new vec4(this.v1, this.v0, this.v1, this.v2);
            }
        }
        
        public vec4 tstp {
            get {
                return new vec4(this.v1, this.v0, this.v1, this.v2);
            }
        }
        
        public vec4 yxyw {
            get {
                return new vec4(this.v1, this.v0, this.v1, this.v3);
            }
        }
        
        public vec4 grga {
            get {
                return new vec4(this.v1, this.v0, this.v1, this.v3);
            }
        }
        
        public vec4 tstq {
            get {
                return new vec4(this.v1, this.v0, this.v1, this.v3);
            }
        }
        
        public vec4 yxzx {
            get {
                return new vec4(this.v1, this.v0, this.v2, this.v0);
            }
        }
        
        public vec4 grbr {
            get {
                return new vec4(this.v1, this.v0, this.v2, this.v0);
            }
        }
        
        public vec4 tsps {
            get {
                return new vec4(this.v1, this.v0, this.v2, this.v0);
            }
        }
        
        public vec4 yxzy {
            get {
                return new vec4(this.v1, this.v0, this.v2, this.v1);
            }
        }
        
        public vec4 grbg {
            get {
                return new vec4(this.v1, this.v0, this.v2, this.v1);
            }
        }
        
        public vec4 tspt {
            get {
                return new vec4(this.v1, this.v0, this.v2, this.v1);
            }
        }
        
        public vec4 yxzz {
            get {
                return new vec4(this.v1, this.v0, this.v2, this.v2);
            }
        }
        
        public vec4 grbb {
            get {
                return new vec4(this.v1, this.v0, this.v2, this.v2);
            }
        }
        
        public vec4 tspp {
            get {
                return new vec4(this.v1, this.v0, this.v2, this.v2);
            }
        }
        
        public vec4 yxzw {
            get {
                return new vec4(this.v1, this.v0, this.v2, this.v3);
            }
            set {
                this.v1 = value.v0;
                this.v0 = value.v1;
                this.v2 = value.v2;
                this.v3 = value.v3;
            }
        }
        
        public vec4 grba {
            get {
                return new vec4(this.v1, this.v0, this.v2, this.v3);
            }
            set {
                this.v1 = value.v0;
                this.v0 = value.v1;
                this.v2 = value.v2;
                this.v3 = value.v3;
            }
        }
        
        public vec4 tspq {
            get {
                return new vec4(this.v1, this.v0, this.v2, this.v3);
            }
            set {
                this.v1 = value.v0;
                this.v0 = value.v1;
                this.v2 = value.v2;
                this.v3 = value.v3;
            }
        }
        
        public vec4 yxwx {
            get {
                return new vec4(this.v1, this.v0, this.v3, this.v0);
            }
        }
        
        public vec4 grar {
            get {
                return new vec4(this.v1, this.v0, this.v3, this.v0);
            }
        }
        
        public vec4 tsqs {
            get {
                return new vec4(this.v1, this.v0, this.v3, this.v0);
            }
        }
        
        public vec4 yxwy {
            get {
                return new vec4(this.v1, this.v0, this.v3, this.v1);
            }
        }
        
        public vec4 grag {
            get {
                return new vec4(this.v1, this.v0, this.v3, this.v1);
            }
        }
        
        public vec4 tsqt {
            get {
                return new vec4(this.v1, this.v0, this.v3, this.v1);
            }
        }
        
        public vec4 yxwz {
            get {
                return new vec4(this.v1, this.v0, this.v3, this.v2);
            }
            set {
                this.v1 = value.v0;
                this.v0 = value.v1;
                this.v3 = value.v2;
                this.v2 = value.v3;
            }
        }
        
        public vec4 grab {
            get {
                return new vec4(this.v1, this.v0, this.v3, this.v2);
            }
            set {
                this.v1 = value.v0;
                this.v0 = value.v1;
                this.v3 = value.v2;
                this.v2 = value.v3;
            }
        }
        
        public vec4 tsqp {
            get {
                return new vec4(this.v1, this.v0, this.v3, this.v2);
            }
            set {
                this.v1 = value.v0;
                this.v0 = value.v1;
                this.v3 = value.v2;
                this.v2 = value.v3;
            }
        }
        
        public vec4 yxww {
            get {
                return new vec4(this.v1, this.v0, this.v3, this.v3);
            }
        }
        
        public vec4 graa {
            get {
                return new vec4(this.v1, this.v0, this.v3, this.v3);
            }
        }
        
        public vec4 tsqq {
            get {
                return new vec4(this.v1, this.v0, this.v3, this.v3);
            }
        }
        
        public vec4 yyxx {
            get {
                return new vec4(this.v1, this.v1, this.v0, this.v0);
            }
        }
        
        public vec4 ggrr {
            get {
                return new vec4(this.v1, this.v1, this.v0, this.v0);
            }
        }
        
        public vec4 ttss {
            get {
                return new vec4(this.v1, this.v1, this.v0, this.v0);
            }
        }
        
        public vec4 yyxy {
            get {
                return new vec4(this.v1, this.v1, this.v0, this.v1);
            }
        }
        
        public vec4 ggrg {
            get {
                return new vec4(this.v1, this.v1, this.v0, this.v1);
            }
        }
        
        public vec4 ttst {
            get {
                return new vec4(this.v1, this.v1, this.v0, this.v1);
            }
        }
        
        public vec4 yyxz {
            get {
                return new vec4(this.v1, this.v1, this.v0, this.v2);
            }
        }
        
        public vec4 ggrb {
            get {
                return new vec4(this.v1, this.v1, this.v0, this.v2);
            }
        }
        
        public vec4 ttsp {
            get {
                return new vec4(this.v1, this.v1, this.v0, this.v2);
            }
        }
        
        public vec4 yyxw {
            get {
                return new vec4(this.v1, this.v1, this.v0, this.v3);
            }
        }
        
        public vec4 ggra {
            get {
                return new vec4(this.v1, this.v1, this.v0, this.v3);
            }
        }
        
        public vec4 ttsq {
            get {
                return new vec4(this.v1, this.v1, this.v0, this.v3);
            }
        }
        
        public vec4 yyyx {
            get {
                return new vec4(this.v1, this.v1, this.v1, this.v0);
            }
        }
        
        public vec4 gggr {
            get {
                return new vec4(this.v1, this.v1, this.v1, this.v0);
            }
        }
        
        public vec4 ttts {
            get {
                return new vec4(this.v1, this.v1, this.v1, this.v0);
            }
        }
        
        public vec4 yyyy {
            get {
                return new vec4(this.v1, this.v1, this.v1, this.v1);
            }
        }
        
        public vec4 gggg {
            get {
                return new vec4(this.v1, this.v1, this.v1, this.v1);
            }
        }
        
        public vec4 tttt {
            get {
                return new vec4(this.v1, this.v1, this.v1, this.v1);
            }
        }
        
        public vec4 yyyz {
            get {
                return new vec4(this.v1, this.v1, this.v1, this.v2);
            }
        }
        
        public vec4 gggb {
            get {
                return new vec4(this.v1, this.v1, this.v1, this.v2);
            }
        }
        
        public vec4 tttp {
            get {
                return new vec4(this.v1, this.v1, this.v1, this.v2);
            }
        }
        
        public vec4 yyyw {
            get {
                return new vec4(this.v1, this.v1, this.v1, this.v3);
            }
        }
        
        public vec4 ggga {
            get {
                return new vec4(this.v1, this.v1, this.v1, this.v3);
            }
        }
        
        public vec4 tttq {
            get {
                return new vec4(this.v1, this.v1, this.v1, this.v3);
            }
        }
        
        public vec4 yyzx {
            get {
                return new vec4(this.v1, this.v1, this.v2, this.v0);
            }
        }
        
        public vec4 ggbr {
            get {
                return new vec4(this.v1, this.v1, this.v2, this.v0);
            }
        }
        
        public vec4 ttps {
            get {
                return new vec4(this.v1, this.v1, this.v2, this.v0);
            }
        }
        
        public vec4 yyzy {
            get {
                return new vec4(this.v1, this.v1, this.v2, this.v1);
            }
        }
        
        public vec4 ggbg {
            get {
                return new vec4(this.v1, this.v1, this.v2, this.v1);
            }
        }
        
        public vec4 ttpt {
            get {
                return new vec4(this.v1, this.v1, this.v2, this.v1);
            }
        }
        
        public vec4 yyzz {
            get {
                return new vec4(this.v1, this.v1, this.v2, this.v2);
            }
        }
        
        public vec4 ggbb {
            get {
                return new vec4(this.v1, this.v1, this.v2, this.v2);
            }
        }
        
        public vec4 ttpp {
            get {
                return new vec4(this.v1, this.v1, this.v2, this.v2);
            }
        }
        
        public vec4 yyzw {
            get {
                return new vec4(this.v1, this.v1, this.v2, this.v3);
            }
        }
        
        public vec4 ggba {
            get {
                return new vec4(this.v1, this.v1, this.v2, this.v3);
            }
        }
        
        public vec4 ttpq {
            get {
                return new vec4(this.v1, this.v1, this.v2, this.v3);
            }
        }
        
        public vec4 yywx {
            get {
                return new vec4(this.v1, this.v1, this.v3, this.v0);
            }
        }
        
        public vec4 ggar {
            get {
                return new vec4(this.v1, this.v1, this.v3, this.v0);
            }
        }
        
        public vec4 ttqs {
            get {
                return new vec4(this.v1, this.v1, this.v3, this.v0);
            }
        }
        
        public vec4 yywy {
            get {
                return new vec4(this.v1, this.v1, this.v3, this.v1);
            }
        }
        
        public vec4 ggag {
            get {
                return new vec4(this.v1, this.v1, this.v3, this.v1);
            }
        }
        
        public vec4 ttqt {
            get {
                return new vec4(this.v1, this.v1, this.v3, this.v1);
            }
        }
        
        public vec4 yywz {
            get {
                return new vec4(this.v1, this.v1, this.v3, this.v2);
            }
        }
        
        public vec4 ggab {
            get {
                return new vec4(this.v1, this.v1, this.v3, this.v2);
            }
        }
        
        public vec4 ttqp {
            get {
                return new vec4(this.v1, this.v1, this.v3, this.v2);
            }
        }
        
        public vec4 yyww {
            get {
                return new vec4(this.v1, this.v1, this.v3, this.v3);
            }
        }
        
        public vec4 ggaa {
            get {
                return new vec4(this.v1, this.v1, this.v3, this.v3);
            }
        }
        
        public vec4 ttqq {
            get {
                return new vec4(this.v1, this.v1, this.v3, this.v3);
            }
        }
        
        public vec4 yzxx {
            get {
                return new vec4(this.v1, this.v2, this.v0, this.v0);
            }
        }
        
        public vec4 gbrr {
            get {
                return new vec4(this.v1, this.v2, this.v0, this.v0);
            }
        }
        
        public vec4 tpss {
            get {
                return new vec4(this.v1, this.v2, this.v0, this.v0);
            }
        }
        
        public vec4 yzxy {
            get {
                return new vec4(this.v1, this.v2, this.v0, this.v1);
            }
        }
        
        public vec4 gbrg {
            get {
                return new vec4(this.v1, this.v2, this.v0, this.v1);
            }
        }
        
        public vec4 tpst {
            get {
                return new vec4(this.v1, this.v2, this.v0, this.v1);
            }
        }
        
        public vec4 yzxz {
            get {
                return new vec4(this.v1, this.v2, this.v0, this.v2);
            }
        }
        
        public vec4 gbrb {
            get {
                return new vec4(this.v1, this.v2, this.v0, this.v2);
            }
        }
        
        public vec4 tpsp {
            get {
                return new vec4(this.v1, this.v2, this.v0, this.v2);
            }
        }
        
        public vec4 yzxw {
            get {
                return new vec4(this.v1, this.v2, this.v0, this.v3);
            }
            set {
                this.v1 = value.v0;
                this.v2 = value.v1;
                this.v0 = value.v2;
                this.v3 = value.v3;
            }
        }
        
        public vec4 gbra {
            get {
                return new vec4(this.v1, this.v2, this.v0, this.v3);
            }
            set {
                this.v1 = value.v0;
                this.v2 = value.v1;
                this.v0 = value.v2;
                this.v3 = value.v3;
            }
        }
        
        public vec4 tpsq {
            get {
                return new vec4(this.v1, this.v2, this.v0, this.v3);
            }
            set {
                this.v1 = value.v0;
                this.v2 = value.v1;
                this.v0 = value.v2;
                this.v3 = value.v3;
            }
        }
        
        public vec4 yzyx {
            get {
                return new vec4(this.v1, this.v2, this.v1, this.v0);
            }
        }
        
        public vec4 gbgr {
            get {
                return new vec4(this.v1, this.v2, this.v1, this.v0);
            }
        }
        
        public vec4 tpts {
            get {
                return new vec4(this.v1, this.v2, this.v1, this.v0);
            }
        }
        
        public vec4 yzyy {
            get {
                return new vec4(this.v1, this.v2, this.v1, this.v1);
            }
        }
        
        public vec4 gbgg {
            get {
                return new vec4(this.v1, this.v2, this.v1, this.v1);
            }
        }
        
        public vec4 tptt {
            get {
                return new vec4(this.v1, this.v2, this.v1, this.v1);
            }
        }
        
        public vec4 yzyz {
            get {
                return new vec4(this.v1, this.v2, this.v1, this.v2);
            }
        }
        
        public vec4 gbgb {
            get {
                return new vec4(this.v1, this.v2, this.v1, this.v2);
            }
        }
        
        public vec4 tptp {
            get {
                return new vec4(this.v1, this.v2, this.v1, this.v2);
            }
        }
        
        public vec4 yzyw {
            get {
                return new vec4(this.v1, this.v2, this.v1, this.v3);
            }
        }
        
        public vec4 gbga {
            get {
                return new vec4(this.v1, this.v2, this.v1, this.v3);
            }
        }
        
        public vec4 tptq {
            get {
                return new vec4(this.v1, this.v2, this.v1, this.v3);
            }
        }
        
        public vec4 yzzx {
            get {
                return new vec4(this.v1, this.v2, this.v2, this.v0);
            }
        }
        
        public vec4 gbbr {
            get {
                return new vec4(this.v1, this.v2, this.v2, this.v0);
            }
        }
        
        public vec4 tpps {
            get {
                return new vec4(this.v1, this.v2, this.v2, this.v0);
            }
        }
        
        public vec4 yzzy {
            get {
                return new vec4(this.v1, this.v2, this.v2, this.v1);
            }
        }
        
        public vec4 gbbg {
            get {
                return new vec4(this.v1, this.v2, this.v2, this.v1);
            }
        }
        
        public vec4 tppt {
            get {
                return new vec4(this.v1, this.v2, this.v2, this.v1);
            }
        }
        
        public vec4 yzzz {
            get {
                return new vec4(this.v1, this.v2, this.v2, this.v2);
            }
        }
        
        public vec4 gbbb {
            get {
                return new vec4(this.v1, this.v2, this.v2, this.v2);
            }
        }
        
        public vec4 tppp {
            get {
                return new vec4(this.v1, this.v2, this.v2, this.v2);
            }
        }
        
        public vec4 yzzw {
            get {
                return new vec4(this.v1, this.v2, this.v2, this.v3);
            }
        }
        
        public vec4 gbba {
            get {
                return new vec4(this.v1, this.v2, this.v2, this.v3);
            }
        }
        
        public vec4 tppq {
            get {
                return new vec4(this.v1, this.v2, this.v2, this.v3);
            }
        }
        
        public vec4 yzwx {
            get {
                return new vec4(this.v1, this.v2, this.v3, this.v0);
            }
            set {
                this.v1 = value.v0;
                this.v2 = value.v1;
                this.v3 = value.v2;
                this.v0 = value.v3;
            }
        }
        
        public vec4 gbar {
            get {
                return new vec4(this.v1, this.v2, this.v3, this.v0);
            }
            set {
                this.v1 = value.v0;
                this.v2 = value.v1;
                this.v3 = value.v2;
                this.v0 = value.v3;
            }
        }
        
        public vec4 tpqs {
            get {
                return new vec4(this.v1, this.v2, this.v3, this.v0);
            }
            set {
                this.v1 = value.v0;
                this.v2 = value.v1;
                this.v3 = value.v2;
                this.v0 = value.v3;
            }
        }
        
        public vec4 yzwy {
            get {
                return new vec4(this.v1, this.v2, this.v3, this.v1);
            }
        }
        
        public vec4 gbag {
            get {
                return new vec4(this.v1, this.v2, this.v3, this.v1);
            }
        }
        
        public vec4 tpqt {
            get {
                return new vec4(this.v1, this.v2, this.v3, this.v1);
            }
        }
        
        public vec4 yzwz {
            get {
                return new vec4(this.v1, this.v2, this.v3, this.v2);
            }
        }
        
        public vec4 gbab {
            get {
                return new vec4(this.v1, this.v2, this.v3, this.v2);
            }
        }
        
        public vec4 tpqp {
            get {
                return new vec4(this.v1, this.v2, this.v3, this.v2);
            }
        }
        
        public vec4 yzww {
            get {
                return new vec4(this.v1, this.v2, this.v3, this.v3);
            }
        }
        
        public vec4 gbaa {
            get {
                return new vec4(this.v1, this.v2, this.v3, this.v3);
            }
        }
        
        public vec4 tpqq {
            get {
                return new vec4(this.v1, this.v2, this.v3, this.v3);
            }
        }
        
        public vec4 ywxx {
            get {
                return new vec4(this.v1, this.v3, this.v0, this.v0);
            }
        }
        
        public vec4 garr {
            get {
                return new vec4(this.v1, this.v3, this.v0, this.v0);
            }
        }
        
        public vec4 tqss {
            get {
                return new vec4(this.v1, this.v3, this.v0, this.v0);
            }
        }
        
        public vec4 ywxy {
            get {
                return new vec4(this.v1, this.v3, this.v0, this.v1);
            }
        }
        
        public vec4 garg {
            get {
                return new vec4(this.v1, this.v3, this.v0, this.v1);
            }
        }
        
        public vec4 tqst {
            get {
                return new vec4(this.v1, this.v3, this.v0, this.v1);
            }
        }
        
        public vec4 ywxz {
            get {
                return new vec4(this.v1, this.v3, this.v0, this.v2);
            }
            set {
                this.v1 = value.v0;
                this.v3 = value.v1;
                this.v0 = value.v2;
                this.v2 = value.v3;
            }
        }
        
        public vec4 garb {
            get {
                return new vec4(this.v1, this.v3, this.v0, this.v2);
            }
            set {
                this.v1 = value.v0;
                this.v3 = value.v1;
                this.v0 = value.v2;
                this.v2 = value.v3;
            }
        }
        
        public vec4 tqsp {
            get {
                return new vec4(this.v1, this.v3, this.v0, this.v2);
            }
            set {
                this.v1 = value.v0;
                this.v3 = value.v1;
                this.v0 = value.v2;
                this.v2 = value.v3;
            }
        }
        
        public vec4 ywxw {
            get {
                return new vec4(this.v1, this.v3, this.v0, this.v3);
            }
        }
        
        public vec4 gara {
            get {
                return new vec4(this.v1, this.v3, this.v0, this.v3);
            }
        }
        
        public vec4 tqsq {
            get {
                return new vec4(this.v1, this.v3, this.v0, this.v3);
            }
        }
        
        public vec4 ywyx {
            get {
                return new vec4(this.v1, this.v3, this.v1, this.v0);
            }
        }
        
        public vec4 gagr {
            get {
                return new vec4(this.v1, this.v3, this.v1, this.v0);
            }
        }
        
        public vec4 tqts {
            get {
                return new vec4(this.v1, this.v3, this.v1, this.v0);
            }
        }
        
        public vec4 ywyy {
            get {
                return new vec4(this.v1, this.v3, this.v1, this.v1);
            }
        }
        
        public vec4 gagg {
            get {
                return new vec4(this.v1, this.v3, this.v1, this.v1);
            }
        }
        
        public vec4 tqtt {
            get {
                return new vec4(this.v1, this.v3, this.v1, this.v1);
            }
        }
        
        public vec4 ywyz {
            get {
                return new vec4(this.v1, this.v3, this.v1, this.v2);
            }
        }
        
        public vec4 gagb {
            get {
                return new vec4(this.v1, this.v3, this.v1, this.v2);
            }
        }
        
        public vec4 tqtp {
            get {
                return new vec4(this.v1, this.v3, this.v1, this.v2);
            }
        }
        
        public vec4 ywyw {
            get {
                return new vec4(this.v1, this.v3, this.v1, this.v3);
            }
        }
        
        public vec4 gaga {
            get {
                return new vec4(this.v1, this.v3, this.v1, this.v3);
            }
        }
        
        public vec4 tqtq {
            get {
                return new vec4(this.v1, this.v3, this.v1, this.v3);
            }
        }
        
        public vec4 ywzx {
            get {
                return new vec4(this.v1, this.v3, this.v2, this.v0);
            }
            set {
                this.v1 = value.v0;
                this.v3 = value.v1;
                this.v2 = value.v2;
                this.v0 = value.v3;
            }
        }
        
        public vec4 gabr {
            get {
                return new vec4(this.v1, this.v3, this.v2, this.v0);
            }
            set {
                this.v1 = value.v0;
                this.v3 = value.v1;
                this.v2 = value.v2;
                this.v0 = value.v3;
            }
        }
        
        public vec4 tqps {
            get {
                return new vec4(this.v1, this.v3, this.v2, this.v0);
            }
            set {
                this.v1 = value.v0;
                this.v3 = value.v1;
                this.v2 = value.v2;
                this.v0 = value.v3;
            }
        }
        
        public vec4 ywzy {
            get {
                return new vec4(this.v1, this.v3, this.v2, this.v1);
            }
        }
        
        public vec4 gabg {
            get {
                return new vec4(this.v1, this.v3, this.v2, this.v1);
            }
        }
        
        public vec4 tqpt {
            get {
                return new vec4(this.v1, this.v3, this.v2, this.v1);
            }
        }
        
        public vec4 ywzz {
            get {
                return new vec4(this.v1, this.v3, this.v2, this.v2);
            }
        }
        
        public vec4 gabb {
            get {
                return new vec4(this.v1, this.v3, this.v2, this.v2);
            }
        }
        
        public vec4 tqpp {
            get {
                return new vec4(this.v1, this.v3, this.v2, this.v2);
            }
        }
        
        public vec4 ywzw {
            get {
                return new vec4(this.v1, this.v3, this.v2, this.v3);
            }
        }
        
        public vec4 gaba {
            get {
                return new vec4(this.v1, this.v3, this.v2, this.v3);
            }
        }
        
        public vec4 tqpq {
            get {
                return new vec4(this.v1, this.v3, this.v2, this.v3);
            }
        }
        
        public vec4 ywwx {
            get {
                return new vec4(this.v1, this.v3, this.v3, this.v0);
            }
        }
        
        public vec4 gaar {
            get {
                return new vec4(this.v1, this.v3, this.v3, this.v0);
            }
        }
        
        public vec4 tqqs {
            get {
                return new vec4(this.v1, this.v3, this.v3, this.v0);
            }
        }
        
        public vec4 ywwy {
            get {
                return new vec4(this.v1, this.v3, this.v3, this.v1);
            }
        }
        
        public vec4 gaag {
            get {
                return new vec4(this.v1, this.v3, this.v3, this.v1);
            }
        }
        
        public vec4 tqqt {
            get {
                return new vec4(this.v1, this.v3, this.v3, this.v1);
            }
        }
        
        public vec4 ywwz {
            get {
                return new vec4(this.v1, this.v3, this.v3, this.v2);
            }
        }
        
        public vec4 gaab {
            get {
                return new vec4(this.v1, this.v3, this.v3, this.v2);
            }
        }
        
        public vec4 tqqp {
            get {
                return new vec4(this.v1, this.v3, this.v3, this.v2);
            }
        }
        
        public vec4 ywww {
            get {
                return new vec4(this.v1, this.v3, this.v3, this.v3);
            }
        }
        
        public vec4 gaaa {
            get {
                return new vec4(this.v1, this.v3, this.v3, this.v3);
            }
        }
        
        public vec4 tqqq {
            get {
                return new vec4(this.v1, this.v3, this.v3, this.v3);
            }
        }
        
        public vec4 zxxx {
            get {
                return new vec4(this.v2, this.v0, this.v0, this.v0);
            }
        }
        
        public vec4 brrr {
            get {
                return new vec4(this.v2, this.v0, this.v0, this.v0);
            }
        }
        
        public vec4 psss {
            get {
                return new vec4(this.v2, this.v0, this.v0, this.v0);
            }
        }
        
        public vec4 zxxy {
            get {
                return new vec4(this.v2, this.v0, this.v0, this.v1);
            }
        }
        
        public vec4 brrg {
            get {
                return new vec4(this.v2, this.v0, this.v0, this.v1);
            }
        }
        
        public vec4 psst {
            get {
                return new vec4(this.v2, this.v0, this.v0, this.v1);
            }
        }
        
        public vec4 zxxz {
            get {
                return new vec4(this.v2, this.v0, this.v0, this.v2);
            }
        }
        
        public vec4 brrb {
            get {
                return new vec4(this.v2, this.v0, this.v0, this.v2);
            }
        }
        
        public vec4 pssp {
            get {
                return new vec4(this.v2, this.v0, this.v0, this.v2);
            }
        }
        
        public vec4 zxxw {
            get {
                return new vec4(this.v2, this.v0, this.v0, this.v3);
            }
        }
        
        public vec4 brra {
            get {
                return new vec4(this.v2, this.v0, this.v0, this.v3);
            }
        }
        
        public vec4 pssq {
            get {
                return new vec4(this.v2, this.v0, this.v0, this.v3);
            }
        }
        
        public vec4 zxyx {
            get {
                return new vec4(this.v2, this.v0, this.v1, this.v0);
            }
        }
        
        public vec4 brgr {
            get {
                return new vec4(this.v2, this.v0, this.v1, this.v0);
            }
        }
        
        public vec4 psts {
            get {
                return new vec4(this.v2, this.v0, this.v1, this.v0);
            }
        }
        
        public vec4 zxyy {
            get {
                return new vec4(this.v2, this.v0, this.v1, this.v1);
            }
        }
        
        public vec4 brgg {
            get {
                return new vec4(this.v2, this.v0, this.v1, this.v1);
            }
        }
        
        public vec4 pstt {
            get {
                return new vec4(this.v2, this.v0, this.v1, this.v1);
            }
        }
        
        public vec4 zxyz {
            get {
                return new vec4(this.v2, this.v0, this.v1, this.v2);
            }
        }
        
        public vec4 brgb {
            get {
                return new vec4(this.v2, this.v0, this.v1, this.v2);
            }
        }
        
        public vec4 pstp {
            get {
                return new vec4(this.v2, this.v0, this.v1, this.v2);
            }
        }
        
        public vec4 zxyw {
            get {
                return new vec4(this.v2, this.v0, this.v1, this.v3);
            }
            set {
                this.v2 = value.v0;
                this.v0 = value.v1;
                this.v1 = value.v2;
                this.v3 = value.v3;
            }
        }
        
        public vec4 brga {
            get {
                return new vec4(this.v2, this.v0, this.v1, this.v3);
            }
            set {
                this.v2 = value.v0;
                this.v0 = value.v1;
                this.v1 = value.v2;
                this.v3 = value.v3;
            }
        }
        
        public vec4 pstq {
            get {
                return new vec4(this.v2, this.v0, this.v1, this.v3);
            }
            set {
                this.v2 = value.v0;
                this.v0 = value.v1;
                this.v1 = value.v2;
                this.v3 = value.v3;
            }
        }
        
        public vec4 zxzx {
            get {
                return new vec4(this.v2, this.v0, this.v2, this.v0);
            }
        }
        
        public vec4 brbr {
            get {
                return new vec4(this.v2, this.v0, this.v2, this.v0);
            }
        }
        
        public vec4 psps {
            get {
                return new vec4(this.v2, this.v0, this.v2, this.v0);
            }
        }
        
        public vec4 zxzy {
            get {
                return new vec4(this.v2, this.v0, this.v2, this.v1);
            }
        }
        
        public vec4 brbg {
            get {
                return new vec4(this.v2, this.v0, this.v2, this.v1);
            }
        }
        
        public vec4 pspt {
            get {
                return new vec4(this.v2, this.v0, this.v2, this.v1);
            }
        }
        
        public vec4 zxzz {
            get {
                return new vec4(this.v2, this.v0, this.v2, this.v2);
            }
        }
        
        public vec4 brbb {
            get {
                return new vec4(this.v2, this.v0, this.v2, this.v2);
            }
        }
        
        public vec4 pspp {
            get {
                return new vec4(this.v2, this.v0, this.v2, this.v2);
            }
        }
        
        public vec4 zxzw {
            get {
                return new vec4(this.v2, this.v0, this.v2, this.v3);
            }
        }
        
        public vec4 brba {
            get {
                return new vec4(this.v2, this.v0, this.v2, this.v3);
            }
        }
        
        public vec4 pspq {
            get {
                return new vec4(this.v2, this.v0, this.v2, this.v3);
            }
        }
        
        public vec4 zxwx {
            get {
                return new vec4(this.v2, this.v0, this.v3, this.v0);
            }
        }
        
        public vec4 brar {
            get {
                return new vec4(this.v2, this.v0, this.v3, this.v0);
            }
        }
        
        public vec4 psqs {
            get {
                return new vec4(this.v2, this.v0, this.v3, this.v0);
            }
        }
        
        public vec4 zxwy {
            get {
                return new vec4(this.v2, this.v0, this.v3, this.v1);
            }
            set {
                this.v2 = value.v0;
                this.v0 = value.v1;
                this.v3 = value.v2;
                this.v1 = value.v3;
            }
        }
        
        public vec4 brag {
            get {
                return new vec4(this.v2, this.v0, this.v3, this.v1);
            }
            set {
                this.v2 = value.v0;
                this.v0 = value.v1;
                this.v3 = value.v2;
                this.v1 = value.v3;
            }
        }
        
        public vec4 psqt {
            get {
                return new vec4(this.v2, this.v0, this.v3, this.v1);
            }
            set {
                this.v2 = value.v0;
                this.v0 = value.v1;
                this.v3 = value.v2;
                this.v1 = value.v3;
            }
        }
        
        public vec4 zxwz {
            get {
                return new vec4(this.v2, this.v0, this.v3, this.v2);
            }
        }
        
        public vec4 brab {
            get {
                return new vec4(this.v2, this.v0, this.v3, this.v2);
            }
        }
        
        public vec4 psqp {
            get {
                return new vec4(this.v2, this.v0, this.v3, this.v2);
            }
        }
        
        public vec4 zxww {
            get {
                return new vec4(this.v2, this.v0, this.v3, this.v3);
            }
        }
        
        public vec4 braa {
            get {
                return new vec4(this.v2, this.v0, this.v3, this.v3);
            }
        }
        
        public vec4 psqq {
            get {
                return new vec4(this.v2, this.v0, this.v3, this.v3);
            }
        }
        
        public vec4 zyxx {
            get {
                return new vec4(this.v2, this.v1, this.v0, this.v0);
            }
        }
        
        public vec4 bgrr {
            get {
                return new vec4(this.v2, this.v1, this.v0, this.v0);
            }
        }
        
        public vec4 ptss {
            get {
                return new vec4(this.v2, this.v1, this.v0, this.v0);
            }
        }
        
        public vec4 zyxy {
            get {
                return new vec4(this.v2, this.v1, this.v0, this.v1);
            }
        }
        
        public vec4 bgrg {
            get {
                return new vec4(this.v2, this.v1, this.v0, this.v1);
            }
        }
        
        public vec4 ptst {
            get {
                return new vec4(this.v2, this.v1, this.v0, this.v1);
            }
        }
        
        public vec4 zyxz {
            get {
                return new vec4(this.v2, this.v1, this.v0, this.v2);
            }
        }
        
        public vec4 bgrb {
            get {
                return new vec4(this.v2, this.v1, this.v0, this.v2);
            }
        }
        
        public vec4 ptsp {
            get {
                return new vec4(this.v2, this.v1, this.v0, this.v2);
            }
        }
        
        public vec4 zyxw {
            get {
                return new vec4(this.v2, this.v1, this.v0, this.v3);
            }
            set {
                this.v2 = value.v0;
                this.v1 = value.v1;
                this.v0 = value.v2;
                this.v3 = value.v3;
            }
        }
        
        public vec4 bgra {
            get {
                return new vec4(this.v2, this.v1, this.v0, this.v3);
            }
            set {
                this.v2 = value.v0;
                this.v1 = value.v1;
                this.v0 = value.v2;
                this.v3 = value.v3;
            }
        }
        
        public vec4 ptsq {
            get {
                return new vec4(this.v2, this.v1, this.v0, this.v3);
            }
            set {
                this.v2 = value.v0;
                this.v1 = value.v1;
                this.v0 = value.v2;
                this.v3 = value.v3;
            }
        }
        
        public vec4 zyyx {
            get {
                return new vec4(this.v2, this.v1, this.v1, this.v0);
            }
        }
        
        public vec4 bggr {
            get {
                return new vec4(this.v2, this.v1, this.v1, this.v0);
            }
        }
        
        public vec4 ptts {
            get {
                return new vec4(this.v2, this.v1, this.v1, this.v0);
            }
        }
        
        public vec4 zyyy {
            get {
                return new vec4(this.v2, this.v1, this.v1, this.v1);
            }
        }
        
        public vec4 bggg {
            get {
                return new vec4(this.v2, this.v1, this.v1, this.v1);
            }
        }
        
        public vec4 pttt {
            get {
                return new vec4(this.v2, this.v1, this.v1, this.v1);
            }
        }
        
        public vec4 zyyz {
            get {
                return new vec4(this.v2, this.v1, this.v1, this.v2);
            }
        }
        
        public vec4 bggb {
            get {
                return new vec4(this.v2, this.v1, this.v1, this.v2);
            }
        }
        
        public vec4 pttp {
            get {
                return new vec4(this.v2, this.v1, this.v1, this.v2);
            }
        }
        
        public vec4 zyyw {
            get {
                return new vec4(this.v2, this.v1, this.v1, this.v3);
            }
        }
        
        public vec4 bgga {
            get {
                return new vec4(this.v2, this.v1, this.v1, this.v3);
            }
        }
        
        public vec4 pttq {
            get {
                return new vec4(this.v2, this.v1, this.v1, this.v3);
            }
        }
        
        public vec4 zyzx {
            get {
                return new vec4(this.v2, this.v1, this.v2, this.v0);
            }
        }
        
        public vec4 bgbr {
            get {
                return new vec4(this.v2, this.v1, this.v2, this.v0);
            }
        }
        
        public vec4 ptps {
            get {
                return new vec4(this.v2, this.v1, this.v2, this.v0);
            }
        }
        
        public vec4 zyzy {
            get {
                return new vec4(this.v2, this.v1, this.v2, this.v1);
            }
        }
        
        public vec4 bgbg {
            get {
                return new vec4(this.v2, this.v1, this.v2, this.v1);
            }
        }
        
        public vec4 ptpt {
            get {
                return new vec4(this.v2, this.v1, this.v2, this.v1);
            }
        }
        
        public vec4 zyzz {
            get {
                return new vec4(this.v2, this.v1, this.v2, this.v2);
            }
        }
        
        public vec4 bgbb {
            get {
                return new vec4(this.v2, this.v1, this.v2, this.v2);
            }
        }
        
        public vec4 ptpp {
            get {
                return new vec4(this.v2, this.v1, this.v2, this.v2);
            }
        }
        
        public vec4 zyzw {
            get {
                return new vec4(this.v2, this.v1, this.v2, this.v3);
            }
        }
        
        public vec4 bgba {
            get {
                return new vec4(this.v2, this.v1, this.v2, this.v3);
            }
        }
        
        public vec4 ptpq {
            get {
                return new vec4(this.v2, this.v1, this.v2, this.v3);
            }
        }
        
        public vec4 zywx {
            get {
                return new vec4(this.v2, this.v1, this.v3, this.v0);
            }
            set {
                this.v2 = value.v0;
                this.v1 = value.v1;
                this.v3 = value.v2;
                this.v0 = value.v3;
            }
        }
        
        public vec4 bgar {
            get {
                return new vec4(this.v2, this.v1, this.v3, this.v0);
            }
            set {
                this.v2 = value.v0;
                this.v1 = value.v1;
                this.v3 = value.v2;
                this.v0 = value.v3;
            }
        }
        
        public vec4 ptqs {
            get {
                return new vec4(this.v2, this.v1, this.v3, this.v0);
            }
            set {
                this.v2 = value.v0;
                this.v1 = value.v1;
                this.v3 = value.v2;
                this.v0 = value.v3;
            }
        }
        
        public vec4 zywy {
            get {
                return new vec4(this.v2, this.v1, this.v3, this.v1);
            }
        }
        
        public vec4 bgag {
            get {
                return new vec4(this.v2, this.v1, this.v3, this.v1);
            }
        }
        
        public vec4 ptqt {
            get {
                return new vec4(this.v2, this.v1, this.v3, this.v1);
            }
        }
        
        public vec4 zywz {
            get {
                return new vec4(this.v2, this.v1, this.v3, this.v2);
            }
        }
        
        public vec4 bgab {
            get {
                return new vec4(this.v2, this.v1, this.v3, this.v2);
            }
        }
        
        public vec4 ptqp {
            get {
                return new vec4(this.v2, this.v1, this.v3, this.v2);
            }
        }
        
        public vec4 zyww {
            get {
                return new vec4(this.v2, this.v1, this.v3, this.v3);
            }
        }
        
        public vec4 bgaa {
            get {
                return new vec4(this.v2, this.v1, this.v3, this.v3);
            }
        }
        
        public vec4 ptqq {
            get {
                return new vec4(this.v2, this.v1, this.v3, this.v3);
            }
        }
        
        public vec4 zzxx {
            get {
                return new vec4(this.v2, this.v2, this.v0, this.v0);
            }
        }
        
        public vec4 bbrr {
            get {
                return new vec4(this.v2, this.v2, this.v0, this.v0);
            }
        }
        
        public vec4 ppss {
            get {
                return new vec4(this.v2, this.v2, this.v0, this.v0);
            }
        }
        
        public vec4 zzxy {
            get {
                return new vec4(this.v2, this.v2, this.v0, this.v1);
            }
        }
        
        public vec4 bbrg {
            get {
                return new vec4(this.v2, this.v2, this.v0, this.v1);
            }
        }
        
        public vec4 ppst {
            get {
                return new vec4(this.v2, this.v2, this.v0, this.v1);
            }
        }
        
        public vec4 zzxz {
            get {
                return new vec4(this.v2, this.v2, this.v0, this.v2);
            }
        }
        
        public vec4 bbrb {
            get {
                return new vec4(this.v2, this.v2, this.v0, this.v2);
            }
        }
        
        public vec4 ppsp {
            get {
                return new vec4(this.v2, this.v2, this.v0, this.v2);
            }
        }
        
        public vec4 zzxw {
            get {
                return new vec4(this.v2, this.v2, this.v0, this.v3);
            }
        }
        
        public vec4 bbra {
            get {
                return new vec4(this.v2, this.v2, this.v0, this.v3);
            }
        }
        
        public vec4 ppsq {
            get {
                return new vec4(this.v2, this.v2, this.v0, this.v3);
            }
        }
        
        public vec4 zzyx {
            get {
                return new vec4(this.v2, this.v2, this.v1, this.v0);
            }
        }
        
        public vec4 bbgr {
            get {
                return new vec4(this.v2, this.v2, this.v1, this.v0);
            }
        }
        
        public vec4 ppts {
            get {
                return new vec4(this.v2, this.v2, this.v1, this.v0);
            }
        }
        
        public vec4 zzyy {
            get {
                return new vec4(this.v2, this.v2, this.v1, this.v1);
            }
        }
        
        public vec4 bbgg {
            get {
                return new vec4(this.v2, this.v2, this.v1, this.v1);
            }
        }
        
        public vec4 pptt {
            get {
                return new vec4(this.v2, this.v2, this.v1, this.v1);
            }
        }
        
        public vec4 zzyz {
            get {
                return new vec4(this.v2, this.v2, this.v1, this.v2);
            }
        }
        
        public vec4 bbgb {
            get {
                return new vec4(this.v2, this.v2, this.v1, this.v2);
            }
        }
        
        public vec4 pptp {
            get {
                return new vec4(this.v2, this.v2, this.v1, this.v2);
            }
        }
        
        public vec4 zzyw {
            get {
                return new vec4(this.v2, this.v2, this.v1, this.v3);
            }
        }
        
        public vec4 bbga {
            get {
                return new vec4(this.v2, this.v2, this.v1, this.v3);
            }
        }
        
        public vec4 pptq {
            get {
                return new vec4(this.v2, this.v2, this.v1, this.v3);
            }
        }
        
        public vec4 zzzx {
            get {
                return new vec4(this.v2, this.v2, this.v2, this.v0);
            }
        }
        
        public vec4 bbbr {
            get {
                return new vec4(this.v2, this.v2, this.v2, this.v0);
            }
        }
        
        public vec4 ppps {
            get {
                return new vec4(this.v2, this.v2, this.v2, this.v0);
            }
        }
        
        public vec4 zzzy {
            get {
                return new vec4(this.v2, this.v2, this.v2, this.v1);
            }
        }
        
        public vec4 bbbg {
            get {
                return new vec4(this.v2, this.v2, this.v2, this.v1);
            }
        }
        
        public vec4 pppt {
            get {
                return new vec4(this.v2, this.v2, this.v2, this.v1);
            }
        }
        
        public vec4 zzzz {
            get {
                return new vec4(this.v2, this.v2, this.v2, this.v2);
            }
        }
        
        public vec4 bbbb {
            get {
                return new vec4(this.v2, this.v2, this.v2, this.v2);
            }
        }
        
        public vec4 pppp {
            get {
                return new vec4(this.v2, this.v2, this.v2, this.v2);
            }
        }
        
        public vec4 zzzw {
            get {
                return new vec4(this.v2, this.v2, this.v2, this.v3);
            }
        }
        
        public vec4 bbba {
            get {
                return new vec4(this.v2, this.v2, this.v2, this.v3);
            }
        }
        
        public vec4 pppq {
            get {
                return new vec4(this.v2, this.v2, this.v2, this.v3);
            }
        }
        
        public vec4 zzwx {
            get {
                return new vec4(this.v2, this.v2, this.v3, this.v0);
            }
        }
        
        public vec4 bbar {
            get {
                return new vec4(this.v2, this.v2, this.v3, this.v0);
            }
        }
        
        public vec4 ppqs {
            get {
                return new vec4(this.v2, this.v2, this.v3, this.v0);
            }
        }
        
        public vec4 zzwy {
            get {
                return new vec4(this.v2, this.v2, this.v3, this.v1);
            }
        }
        
        public vec4 bbag {
            get {
                return new vec4(this.v2, this.v2, this.v3, this.v1);
            }
        }
        
        public vec4 ppqt {
            get {
                return new vec4(this.v2, this.v2, this.v3, this.v1);
            }
        }
        
        public vec4 zzwz {
            get {
                return new vec4(this.v2, this.v2, this.v3, this.v2);
            }
        }
        
        public vec4 bbab {
            get {
                return new vec4(this.v2, this.v2, this.v3, this.v2);
            }
        }
        
        public vec4 ppqp {
            get {
                return new vec4(this.v2, this.v2, this.v3, this.v2);
            }
        }
        
        public vec4 zzww {
            get {
                return new vec4(this.v2, this.v2, this.v3, this.v3);
            }
        }
        
        public vec4 bbaa {
            get {
                return new vec4(this.v2, this.v2, this.v3, this.v3);
            }
        }
        
        public vec4 ppqq {
            get {
                return new vec4(this.v2, this.v2, this.v3, this.v3);
            }
        }
        
        public vec4 zwxx {
            get {
                return new vec4(this.v2, this.v3, this.v0, this.v0);
            }
        }
        
        public vec4 barr {
            get {
                return new vec4(this.v2, this.v3, this.v0, this.v0);
            }
        }
        
        public vec4 pqss {
            get {
                return new vec4(this.v2, this.v3, this.v0, this.v0);
            }
        }
        
        public vec4 zwxy {
            get {
                return new vec4(this.v2, this.v3, this.v0, this.v1);
            }
            set {
                this.v2 = value.v0;
                this.v3 = value.v1;
                this.v0 = value.v2;
                this.v1 = value.v3;
            }
        }
        
        public vec4 barg {
            get {
                return new vec4(this.v2, this.v3, this.v0, this.v1);
            }
            set {
                this.v2 = value.v0;
                this.v3 = value.v1;
                this.v0 = value.v2;
                this.v1 = value.v3;
            }
        }
        
        public vec4 pqst {
            get {
                return new vec4(this.v2, this.v3, this.v0, this.v1);
            }
            set {
                this.v2 = value.v0;
                this.v3 = value.v1;
                this.v0 = value.v2;
                this.v1 = value.v3;
            }
        }
        
        public vec4 zwxz {
            get {
                return new vec4(this.v2, this.v3, this.v0, this.v2);
            }
        }
        
        public vec4 barb {
            get {
                return new vec4(this.v2, this.v3, this.v0, this.v2);
            }
        }
        
        public vec4 pqsp {
            get {
                return new vec4(this.v2, this.v3, this.v0, this.v2);
            }
        }
        
        public vec4 zwxw {
            get {
                return new vec4(this.v2, this.v3, this.v0, this.v3);
            }
        }
        
        public vec4 bara {
            get {
                return new vec4(this.v2, this.v3, this.v0, this.v3);
            }
        }
        
        public vec4 pqsq {
            get {
                return new vec4(this.v2, this.v3, this.v0, this.v3);
            }
        }
        
        public vec4 zwyx {
            get {
                return new vec4(this.v2, this.v3, this.v1, this.v0);
            }
            set {
                this.v2 = value.v0;
                this.v3 = value.v1;
                this.v1 = value.v2;
                this.v0 = value.v3;
            }
        }
        
        public vec4 bagr {
            get {
                return new vec4(this.v2, this.v3, this.v1, this.v0);
            }
            set {
                this.v2 = value.v0;
                this.v3 = value.v1;
                this.v1 = value.v2;
                this.v0 = value.v3;
            }
        }
        
        public vec4 pqts {
            get {
                return new vec4(this.v2, this.v3, this.v1, this.v0);
            }
            set {
                this.v2 = value.v0;
                this.v3 = value.v1;
                this.v1 = value.v2;
                this.v0 = value.v3;
            }
        }
        
        public vec4 zwyy {
            get {
                return new vec4(this.v2, this.v3, this.v1, this.v1);
            }
        }
        
        public vec4 bagg {
            get {
                return new vec4(this.v2, this.v3, this.v1, this.v1);
            }
        }
        
        public vec4 pqtt {
            get {
                return new vec4(this.v2, this.v3, this.v1, this.v1);
            }
        }
        
        public vec4 zwyz {
            get {
                return new vec4(this.v2, this.v3, this.v1, this.v2);
            }
        }
        
        public vec4 bagb {
            get {
                return new vec4(this.v2, this.v3, this.v1, this.v2);
            }
        }
        
        public vec4 pqtp {
            get {
                return new vec4(this.v2, this.v3, this.v1, this.v2);
            }
        }
        
        public vec4 zwyw {
            get {
                return new vec4(this.v2, this.v3, this.v1, this.v3);
            }
        }
        
        public vec4 baga {
            get {
                return new vec4(this.v2, this.v3, this.v1, this.v3);
            }
        }
        
        public vec4 pqtq {
            get {
                return new vec4(this.v2, this.v3, this.v1, this.v3);
            }
        }
        
        public vec4 zwzx {
            get {
                return new vec4(this.v2, this.v3, this.v2, this.v0);
            }
        }
        
        public vec4 babr {
            get {
                return new vec4(this.v2, this.v3, this.v2, this.v0);
            }
        }
        
        public vec4 pqps {
            get {
                return new vec4(this.v2, this.v3, this.v2, this.v0);
            }
        }
        
        public vec4 zwzy {
            get {
                return new vec4(this.v2, this.v3, this.v2, this.v1);
            }
        }
        
        public vec4 babg {
            get {
                return new vec4(this.v2, this.v3, this.v2, this.v1);
            }
        }
        
        public vec4 pqpt {
            get {
                return new vec4(this.v2, this.v3, this.v2, this.v1);
            }
        }
        
        public vec4 zwzz {
            get {
                return new vec4(this.v2, this.v3, this.v2, this.v2);
            }
        }
        
        public vec4 babb {
            get {
                return new vec4(this.v2, this.v3, this.v2, this.v2);
            }
        }
        
        public vec4 pqpp {
            get {
                return new vec4(this.v2, this.v3, this.v2, this.v2);
            }
        }
        
        public vec4 zwzw {
            get {
                return new vec4(this.v2, this.v3, this.v2, this.v3);
            }
        }
        
        public vec4 baba {
            get {
                return new vec4(this.v2, this.v3, this.v2, this.v3);
            }
        }
        
        public vec4 pqpq {
            get {
                return new vec4(this.v2, this.v3, this.v2, this.v3);
            }
        }
        
        public vec4 zwwx {
            get {
                return new vec4(this.v2, this.v3, this.v3, this.v0);
            }
        }
        
        public vec4 baar {
            get {
                return new vec4(this.v2, this.v3, this.v3, this.v0);
            }
        }
        
        public vec4 pqqs {
            get {
                return new vec4(this.v2, this.v3, this.v3, this.v0);
            }
        }
        
        public vec4 zwwy {
            get {
                return new vec4(this.v2, this.v3, this.v3, this.v1);
            }
        }
        
        public vec4 baag {
            get {
                return new vec4(this.v2, this.v3, this.v3, this.v1);
            }
        }
        
        public vec4 pqqt {
            get {
                return new vec4(this.v2, this.v3, this.v3, this.v1);
            }
        }
        
        public vec4 zwwz {
            get {
                return new vec4(this.v2, this.v3, this.v3, this.v2);
            }
        }
        
        public vec4 baab {
            get {
                return new vec4(this.v2, this.v3, this.v3, this.v2);
            }
        }
        
        public vec4 pqqp {
            get {
                return new vec4(this.v2, this.v3, this.v3, this.v2);
            }
        }
        
        public vec4 zwww {
            get {
                return new vec4(this.v2, this.v3, this.v3, this.v3);
            }
        }
        
        public vec4 baaa {
            get {
                return new vec4(this.v2, this.v3, this.v3, this.v3);
            }
        }
        
        public vec4 pqqq {
            get {
                return new vec4(this.v2, this.v3, this.v3, this.v3);
            }
        }
        
        public vec4 wxxx {
            get {
                return new vec4(this.v3, this.v0, this.v0, this.v0);
            }
        }
        
        public vec4 arrr {
            get {
                return new vec4(this.v3, this.v0, this.v0, this.v0);
            }
        }
        
        public vec4 qsss {
            get {
                return new vec4(this.v3, this.v0, this.v0, this.v0);
            }
        }
        
        public vec4 wxxy {
            get {
                return new vec4(this.v3, this.v0, this.v0, this.v1);
            }
        }
        
        public vec4 arrg {
            get {
                return new vec4(this.v3, this.v0, this.v0, this.v1);
            }
        }
        
        public vec4 qsst {
            get {
                return new vec4(this.v3, this.v0, this.v0, this.v1);
            }
        }
        
        public vec4 wxxz {
            get {
                return new vec4(this.v3, this.v0, this.v0, this.v2);
            }
        }
        
        public vec4 arrb {
            get {
                return new vec4(this.v3, this.v0, this.v0, this.v2);
            }
        }
        
        public vec4 qssp {
            get {
                return new vec4(this.v3, this.v0, this.v0, this.v2);
            }
        }
        
        public vec4 wxxw {
            get {
                return new vec4(this.v3, this.v0, this.v0, this.v3);
            }
        }
        
        public vec4 arra {
            get {
                return new vec4(this.v3, this.v0, this.v0, this.v3);
            }
        }
        
        public vec4 qssq {
            get {
                return new vec4(this.v3, this.v0, this.v0, this.v3);
            }
        }
        
        public vec4 wxyx {
            get {
                return new vec4(this.v3, this.v0, this.v1, this.v0);
            }
        }
        
        public vec4 argr {
            get {
                return new vec4(this.v3, this.v0, this.v1, this.v0);
            }
        }
        
        public vec4 qsts {
            get {
                return new vec4(this.v3, this.v0, this.v1, this.v0);
            }
        }
        
        public vec4 wxyy {
            get {
                return new vec4(this.v3, this.v0, this.v1, this.v1);
            }
        }
        
        public vec4 argg {
            get {
                return new vec4(this.v3, this.v0, this.v1, this.v1);
            }
        }
        
        public vec4 qstt {
            get {
                return new vec4(this.v3, this.v0, this.v1, this.v1);
            }
        }
        
        public vec4 wxyz {
            get {
                return new vec4(this.v3, this.v0, this.v1, this.v2);
            }
            set {
                this.v3 = value.v0;
                this.v0 = value.v1;
                this.v1 = value.v2;
                this.v2 = value.v3;
            }
        }
        
        public vec4 argb {
            get {
                return new vec4(this.v3, this.v0, this.v1, this.v2);
            }
            set {
                this.v3 = value.v0;
                this.v0 = value.v1;
                this.v1 = value.v2;
                this.v2 = value.v3;
            }
        }
        
        public vec4 qstp {
            get {
                return new vec4(this.v3, this.v0, this.v1, this.v2);
            }
            set {
                this.v3 = value.v0;
                this.v0 = value.v1;
                this.v1 = value.v2;
                this.v2 = value.v3;
            }
        }
        
        public vec4 wxyw {
            get {
                return new vec4(this.v3, this.v0, this.v1, this.v3);
            }
        }
        
        public vec4 arga {
            get {
                return new vec4(this.v3, this.v0, this.v1, this.v3);
            }
        }
        
        public vec4 qstq {
            get {
                return new vec4(this.v3, this.v0, this.v1, this.v3);
            }
        }
        
        public vec4 wxzx {
            get {
                return new vec4(this.v3, this.v0, this.v2, this.v0);
            }
        }
        
        public vec4 arbr {
            get {
                return new vec4(this.v3, this.v0, this.v2, this.v0);
            }
        }
        
        public vec4 qsps {
            get {
                return new vec4(this.v3, this.v0, this.v2, this.v0);
            }
        }
        
        public vec4 wxzy {
            get {
                return new vec4(this.v3, this.v0, this.v2, this.v1);
            }
            set {
                this.v3 = value.v0;
                this.v0 = value.v1;
                this.v2 = value.v2;
                this.v1 = value.v3;
            }
        }
        
        public vec4 arbg {
            get {
                return new vec4(this.v3, this.v0, this.v2, this.v1);
            }
            set {
                this.v3 = value.v0;
                this.v0 = value.v1;
                this.v2 = value.v2;
                this.v1 = value.v3;
            }
        }
        
        public vec4 qspt {
            get {
                return new vec4(this.v3, this.v0, this.v2, this.v1);
            }
            set {
                this.v3 = value.v0;
                this.v0 = value.v1;
                this.v2 = value.v2;
                this.v1 = value.v3;
            }
        }
        
        public vec4 wxzz {
            get {
                return new vec4(this.v3, this.v0, this.v2, this.v2);
            }
        }
        
        public vec4 arbb {
            get {
                return new vec4(this.v3, this.v0, this.v2, this.v2);
            }
        }
        
        public vec4 qspp {
            get {
                return new vec4(this.v3, this.v0, this.v2, this.v2);
            }
        }
        
        public vec4 wxzw {
            get {
                return new vec4(this.v3, this.v0, this.v2, this.v3);
            }
        }
        
        public vec4 arba {
            get {
                return new vec4(this.v3, this.v0, this.v2, this.v3);
            }
        }
        
        public vec4 qspq {
            get {
                return new vec4(this.v3, this.v0, this.v2, this.v3);
            }
        }
        
        public vec4 wxwx {
            get {
                return new vec4(this.v3, this.v0, this.v3, this.v0);
            }
        }
        
        public vec4 arar {
            get {
                return new vec4(this.v3, this.v0, this.v3, this.v0);
            }
        }
        
        public vec4 qsqs {
            get {
                return new vec4(this.v3, this.v0, this.v3, this.v0);
            }
        }
        
        public vec4 wxwy {
            get {
                return new vec4(this.v3, this.v0, this.v3, this.v1);
            }
        }
        
        public vec4 arag {
            get {
                return new vec4(this.v3, this.v0, this.v3, this.v1);
            }
        }
        
        public vec4 qsqt {
            get {
                return new vec4(this.v3, this.v0, this.v3, this.v1);
            }
        }
        
        public vec4 wxwz {
            get {
                return new vec4(this.v3, this.v0, this.v3, this.v2);
            }
        }
        
        public vec4 arab {
            get {
                return new vec4(this.v3, this.v0, this.v3, this.v2);
            }
        }
        
        public vec4 qsqp {
            get {
                return new vec4(this.v3, this.v0, this.v3, this.v2);
            }
        }
        
        public vec4 wxww {
            get {
                return new vec4(this.v3, this.v0, this.v3, this.v3);
            }
        }
        
        public vec4 araa {
            get {
                return new vec4(this.v3, this.v0, this.v3, this.v3);
            }
        }
        
        public vec4 qsqq {
            get {
                return new vec4(this.v3, this.v0, this.v3, this.v3);
            }
        }
        
        public vec4 wyxx {
            get {
                return new vec4(this.v3, this.v1, this.v0, this.v0);
            }
        }
        
        public vec4 agrr {
            get {
                return new vec4(this.v3, this.v1, this.v0, this.v0);
            }
        }
        
        public vec4 qtss {
            get {
                return new vec4(this.v3, this.v1, this.v0, this.v0);
            }
        }
        
        public vec4 wyxy {
            get {
                return new vec4(this.v3, this.v1, this.v0, this.v1);
            }
        }
        
        public vec4 agrg {
            get {
                return new vec4(this.v3, this.v1, this.v0, this.v1);
            }
        }
        
        public vec4 qtst {
            get {
                return new vec4(this.v3, this.v1, this.v0, this.v1);
            }
        }
        
        public vec4 wyxz {
            get {
                return new vec4(this.v3, this.v1, this.v0, this.v2);
            }
            set {
                this.v3 = value.v0;
                this.v1 = value.v1;
                this.v0 = value.v2;
                this.v2 = value.v3;
            }
        }
        
        public vec4 agrb {
            get {
                return new vec4(this.v3, this.v1, this.v0, this.v2);
            }
            set {
                this.v3 = value.v0;
                this.v1 = value.v1;
                this.v0 = value.v2;
                this.v2 = value.v3;
            }
        }
        
        public vec4 qtsp {
            get {
                return new vec4(this.v3, this.v1, this.v0, this.v2);
            }
            set {
                this.v3 = value.v0;
                this.v1 = value.v1;
                this.v0 = value.v2;
                this.v2 = value.v3;
            }
        }
        
        public vec4 wyxw {
            get {
                return new vec4(this.v3, this.v1, this.v0, this.v3);
            }
        }
        
        public vec4 agra {
            get {
                return new vec4(this.v3, this.v1, this.v0, this.v3);
            }
        }
        
        public vec4 qtsq {
            get {
                return new vec4(this.v3, this.v1, this.v0, this.v3);
            }
        }
        
        public vec4 wyyx {
            get {
                return new vec4(this.v3, this.v1, this.v1, this.v0);
            }
        }
        
        public vec4 aggr {
            get {
                return new vec4(this.v3, this.v1, this.v1, this.v0);
            }
        }
        
        public vec4 qtts {
            get {
                return new vec4(this.v3, this.v1, this.v1, this.v0);
            }
        }
        
        public vec4 wyyy {
            get {
                return new vec4(this.v3, this.v1, this.v1, this.v1);
            }
        }
        
        public vec4 aggg {
            get {
                return new vec4(this.v3, this.v1, this.v1, this.v1);
            }
        }
        
        public vec4 qttt {
            get {
                return new vec4(this.v3, this.v1, this.v1, this.v1);
            }
        }
        
        public vec4 wyyz {
            get {
                return new vec4(this.v3, this.v1, this.v1, this.v2);
            }
        }
        
        public vec4 aggb {
            get {
                return new vec4(this.v3, this.v1, this.v1, this.v2);
            }
        }
        
        public vec4 qttp {
            get {
                return new vec4(this.v3, this.v1, this.v1, this.v2);
            }
        }
        
        public vec4 wyyw {
            get {
                return new vec4(this.v3, this.v1, this.v1, this.v3);
            }
        }
        
        public vec4 agga {
            get {
                return new vec4(this.v3, this.v1, this.v1, this.v3);
            }
        }
        
        public vec4 qttq {
            get {
                return new vec4(this.v3, this.v1, this.v1, this.v3);
            }
        }
        
        public vec4 wyzx {
            get {
                return new vec4(this.v3, this.v1, this.v2, this.v0);
            }
            set {
                this.v3 = value.v0;
                this.v1 = value.v1;
                this.v2 = value.v2;
                this.v0 = value.v3;
            }
        }
        
        public vec4 agbr {
            get {
                return new vec4(this.v3, this.v1, this.v2, this.v0);
            }
            set {
                this.v3 = value.v0;
                this.v1 = value.v1;
                this.v2 = value.v2;
                this.v0 = value.v3;
            }
        }
        
        public vec4 qtps {
            get {
                return new vec4(this.v3, this.v1, this.v2, this.v0);
            }
            set {
                this.v3 = value.v0;
                this.v1 = value.v1;
                this.v2 = value.v2;
                this.v0 = value.v3;
            }
        }
        
        public vec4 wyzy {
            get {
                return new vec4(this.v3, this.v1, this.v2, this.v1);
            }
        }
        
        public vec4 agbg {
            get {
                return new vec4(this.v3, this.v1, this.v2, this.v1);
            }
        }
        
        public vec4 qtpt {
            get {
                return new vec4(this.v3, this.v1, this.v2, this.v1);
            }
        }
        
        public vec4 wyzz {
            get {
                return new vec4(this.v3, this.v1, this.v2, this.v2);
            }
        }
        
        public vec4 agbb {
            get {
                return new vec4(this.v3, this.v1, this.v2, this.v2);
            }
        }
        
        public vec4 qtpp {
            get {
                return new vec4(this.v3, this.v1, this.v2, this.v2);
            }
        }
        
        public vec4 wyzw {
            get {
                return new vec4(this.v3, this.v1, this.v2, this.v3);
            }
        }
        
        public vec4 agba {
            get {
                return new vec4(this.v3, this.v1, this.v2, this.v3);
            }
        }
        
        public vec4 qtpq {
            get {
                return new vec4(this.v3, this.v1, this.v2, this.v3);
            }
        }
        
        public vec4 wywx {
            get {
                return new vec4(this.v3, this.v1, this.v3, this.v0);
            }
        }
        
        public vec4 agar {
            get {
                return new vec4(this.v3, this.v1, this.v3, this.v0);
            }
        }
        
        public vec4 qtqs {
            get {
                return new vec4(this.v3, this.v1, this.v3, this.v0);
            }
        }
        
        public vec4 wywy {
            get {
                return new vec4(this.v3, this.v1, this.v3, this.v1);
            }
        }
        
        public vec4 agag {
            get {
                return new vec4(this.v3, this.v1, this.v3, this.v1);
            }
        }
        
        public vec4 qtqt {
            get {
                return new vec4(this.v3, this.v1, this.v3, this.v1);
            }
        }
        
        public vec4 wywz {
            get {
                return new vec4(this.v3, this.v1, this.v3, this.v2);
            }
        }
        
        public vec4 agab {
            get {
                return new vec4(this.v3, this.v1, this.v3, this.v2);
            }
        }
        
        public vec4 qtqp {
            get {
                return new vec4(this.v3, this.v1, this.v3, this.v2);
            }
        }
        
        public vec4 wyww {
            get {
                return new vec4(this.v3, this.v1, this.v3, this.v3);
            }
        }
        
        public vec4 agaa {
            get {
                return new vec4(this.v3, this.v1, this.v3, this.v3);
            }
        }
        
        public vec4 qtqq {
            get {
                return new vec4(this.v3, this.v1, this.v3, this.v3);
            }
        }
        
        public vec4 wzxx {
            get {
                return new vec4(this.v3, this.v2, this.v0, this.v0);
            }
        }
        
        public vec4 abrr {
            get {
                return new vec4(this.v3, this.v2, this.v0, this.v0);
            }
        }
        
        public vec4 qpss {
            get {
                return new vec4(this.v3, this.v2, this.v0, this.v0);
            }
        }
        
        public vec4 wzxy {
            get {
                return new vec4(this.v3, this.v2, this.v0, this.v1);
            }
            set {
                this.v3 = value.v0;
                this.v2 = value.v1;
                this.v0 = value.v2;
                this.v1 = value.v3;
            }
        }
        
        public vec4 abrg {
            get {
                return new vec4(this.v3, this.v2, this.v0, this.v1);
            }
            set {
                this.v3 = value.v0;
                this.v2 = value.v1;
                this.v0 = value.v2;
                this.v1 = value.v3;
            }
        }
        
        public vec4 qpst {
            get {
                return new vec4(this.v3, this.v2, this.v0, this.v1);
            }
            set {
                this.v3 = value.v0;
                this.v2 = value.v1;
                this.v0 = value.v2;
                this.v1 = value.v3;
            }
        }
        
        public vec4 wzxz {
            get {
                return new vec4(this.v3, this.v2, this.v0, this.v2);
            }
        }
        
        public vec4 abrb {
            get {
                return new vec4(this.v3, this.v2, this.v0, this.v2);
            }
        }
        
        public vec4 qpsp {
            get {
                return new vec4(this.v3, this.v2, this.v0, this.v2);
            }
        }
        
        public vec4 wzxw {
            get {
                return new vec4(this.v3, this.v2, this.v0, this.v3);
            }
        }
        
        public vec4 abra {
            get {
                return new vec4(this.v3, this.v2, this.v0, this.v3);
            }
        }
        
        public vec4 qpsq {
            get {
                return new vec4(this.v3, this.v2, this.v0, this.v3);
            }
        }
        
        public vec4 wzyx {
            get {
                return new vec4(this.v3, this.v2, this.v1, this.v0);
            }
            set {
                this.v3 = value.v0;
                this.v2 = value.v1;
                this.v1 = value.v2;
                this.v0 = value.v3;
            }
        }
        
        public vec4 abgr {
            get {
                return new vec4(this.v3, this.v2, this.v1, this.v0);
            }
            set {
                this.v3 = value.v0;
                this.v2 = value.v1;
                this.v1 = value.v2;
                this.v0 = value.v3;
            }
        }
        
        public vec4 qpts {
            get {
                return new vec4(this.v3, this.v2, this.v1, this.v0);
            }
            set {
                this.v3 = value.v0;
                this.v2 = value.v1;
                this.v1 = value.v2;
                this.v0 = value.v3;
            }
        }
        
        public vec4 wzyy {
            get {
                return new vec4(this.v3, this.v2, this.v1, this.v1);
            }
        }
        
        public vec4 abgg {
            get {
                return new vec4(this.v3, this.v2, this.v1, this.v1);
            }
        }
        
        public vec4 qptt {
            get {
                return new vec4(this.v3, this.v2, this.v1, this.v1);
            }
        }
        
        public vec4 wzyz {
            get {
                return new vec4(this.v3, this.v2, this.v1, this.v2);
            }
        }
        
        public vec4 abgb {
            get {
                return new vec4(this.v3, this.v2, this.v1, this.v2);
            }
        }
        
        public vec4 qptp {
            get {
                return new vec4(this.v3, this.v2, this.v1, this.v2);
            }
        }
        
        public vec4 wzyw {
            get {
                return new vec4(this.v3, this.v2, this.v1, this.v3);
            }
        }
        
        public vec4 abga {
            get {
                return new vec4(this.v3, this.v2, this.v1, this.v3);
            }
        }
        
        public vec4 qptq {
            get {
                return new vec4(this.v3, this.v2, this.v1, this.v3);
            }
        }
        
        public vec4 wzzx {
            get {
                return new vec4(this.v3, this.v2, this.v2, this.v0);
            }
        }
        
        public vec4 abbr {
            get {
                return new vec4(this.v3, this.v2, this.v2, this.v0);
            }
        }
        
        public vec4 qpps {
            get {
                return new vec4(this.v3, this.v2, this.v2, this.v0);
            }
        }
        
        public vec4 wzzy {
            get {
                return new vec4(this.v3, this.v2, this.v2, this.v1);
            }
        }
        
        public vec4 abbg {
            get {
                return new vec4(this.v3, this.v2, this.v2, this.v1);
            }
        }
        
        public vec4 qppt {
            get {
                return new vec4(this.v3, this.v2, this.v2, this.v1);
            }
        }
        
        public vec4 wzzz {
            get {
                return new vec4(this.v3, this.v2, this.v2, this.v2);
            }
        }
        
        public vec4 abbb {
            get {
                return new vec4(this.v3, this.v2, this.v2, this.v2);
            }
        }
        
        public vec4 qppp {
            get {
                return new vec4(this.v3, this.v2, this.v2, this.v2);
            }
        }
        
        public vec4 wzzw {
            get {
                return new vec4(this.v3, this.v2, this.v2, this.v3);
            }
        }
        
        public vec4 abba {
            get {
                return new vec4(this.v3, this.v2, this.v2, this.v3);
            }
        }
        
        public vec4 qppq {
            get {
                return new vec4(this.v3, this.v2, this.v2, this.v3);
            }
        }
        
        public vec4 wzwx {
            get {
                return new vec4(this.v3, this.v2, this.v3, this.v0);
            }
        }
        
        public vec4 abar {
            get {
                return new vec4(this.v3, this.v2, this.v3, this.v0);
            }
        }
        
        public vec4 qpqs {
            get {
                return new vec4(this.v3, this.v2, this.v3, this.v0);
            }
        }
        
        public vec4 wzwy {
            get {
                return new vec4(this.v3, this.v2, this.v3, this.v1);
            }
        }
        
        public vec4 abag {
            get {
                return new vec4(this.v3, this.v2, this.v3, this.v1);
            }
        }
        
        public vec4 qpqt {
            get {
                return new vec4(this.v3, this.v2, this.v3, this.v1);
            }
        }
        
        public vec4 wzwz {
            get {
                return new vec4(this.v3, this.v2, this.v3, this.v2);
            }
        }
        
        public vec4 abab {
            get {
                return new vec4(this.v3, this.v2, this.v3, this.v2);
            }
        }
        
        public vec4 qpqp {
            get {
                return new vec4(this.v3, this.v2, this.v3, this.v2);
            }
        }
        
        public vec4 wzww {
            get {
                return new vec4(this.v3, this.v2, this.v3, this.v3);
            }
        }
        
        public vec4 abaa {
            get {
                return new vec4(this.v3, this.v2, this.v3, this.v3);
            }
        }
        
        public vec4 qpqq {
            get {
                return new vec4(this.v3, this.v2, this.v3, this.v3);
            }
        }
        
        public vec4 wwxx {
            get {
                return new vec4(this.v3, this.v3, this.v0, this.v0);
            }
        }
        
        public vec4 aarr {
            get {
                return new vec4(this.v3, this.v3, this.v0, this.v0);
            }
        }
        
        public vec4 qqss {
            get {
                return new vec4(this.v3, this.v3, this.v0, this.v0);
            }
        }
        
        public vec4 wwxy {
            get {
                return new vec4(this.v3, this.v3, this.v0, this.v1);
            }
        }
        
        public vec4 aarg {
            get {
                return new vec4(this.v3, this.v3, this.v0, this.v1);
            }
        }
        
        public vec4 qqst {
            get {
                return new vec4(this.v3, this.v3, this.v0, this.v1);
            }
        }
        
        public vec4 wwxz {
            get {
                return new vec4(this.v3, this.v3, this.v0, this.v2);
            }
        }
        
        public vec4 aarb {
            get {
                return new vec4(this.v3, this.v3, this.v0, this.v2);
            }
        }
        
        public vec4 qqsp {
            get {
                return new vec4(this.v3, this.v3, this.v0, this.v2);
            }
        }
        
        public vec4 wwxw {
            get {
                return new vec4(this.v3, this.v3, this.v0, this.v3);
            }
        }
        
        public vec4 aara {
            get {
                return new vec4(this.v3, this.v3, this.v0, this.v3);
            }
        }
        
        public vec4 qqsq {
            get {
                return new vec4(this.v3, this.v3, this.v0, this.v3);
            }
        }
        
        public vec4 wwyx {
            get {
                return new vec4(this.v3, this.v3, this.v1, this.v0);
            }
        }
        
        public vec4 aagr {
            get {
                return new vec4(this.v3, this.v3, this.v1, this.v0);
            }
        }
        
        public vec4 qqts {
            get {
                return new vec4(this.v3, this.v3, this.v1, this.v0);
            }
        }
        
        public vec4 wwyy {
            get {
                return new vec4(this.v3, this.v3, this.v1, this.v1);
            }
        }
        
        public vec4 aagg {
            get {
                return new vec4(this.v3, this.v3, this.v1, this.v1);
            }
        }
        
        public vec4 qqtt {
            get {
                return new vec4(this.v3, this.v3, this.v1, this.v1);
            }
        }
        
        public vec4 wwyz {
            get {
                return new vec4(this.v3, this.v3, this.v1, this.v2);
            }
        }
        
        public vec4 aagb {
            get {
                return new vec4(this.v3, this.v3, this.v1, this.v2);
            }
        }
        
        public vec4 qqtp {
            get {
                return new vec4(this.v3, this.v3, this.v1, this.v2);
            }
        }
        
        public vec4 wwyw {
            get {
                return new vec4(this.v3, this.v3, this.v1, this.v3);
            }
        }
        
        public vec4 aaga {
            get {
                return new vec4(this.v3, this.v3, this.v1, this.v3);
            }
        }
        
        public vec4 qqtq {
            get {
                return new vec4(this.v3, this.v3, this.v1, this.v3);
            }
        }
        
        public vec4 wwzx {
            get {
                return new vec4(this.v3, this.v3, this.v2, this.v0);
            }
        }
        
        public vec4 aabr {
            get {
                return new vec4(this.v3, this.v3, this.v2, this.v0);
            }
        }
        
        public vec4 qqps {
            get {
                return new vec4(this.v3, this.v3, this.v2, this.v0);
            }
        }
        
        public vec4 wwzy {
            get {
                return new vec4(this.v3, this.v3, this.v2, this.v1);
            }
        }
        
        public vec4 aabg {
            get {
                return new vec4(this.v3, this.v3, this.v2, this.v1);
            }
        }
        
        public vec4 qqpt {
            get {
                return new vec4(this.v3, this.v3, this.v2, this.v1);
            }
        }
        
        public vec4 wwzz {
            get {
                return new vec4(this.v3, this.v3, this.v2, this.v2);
            }
        }
        
        public vec4 aabb {
            get {
                return new vec4(this.v3, this.v3, this.v2, this.v2);
            }
        }
        
        public vec4 qqpp {
            get {
                return new vec4(this.v3, this.v3, this.v2, this.v2);
            }
        }
        
        public vec4 wwzw {
            get {
                return new vec4(this.v3, this.v3, this.v2, this.v3);
            }
        }
        
        public vec4 aaba {
            get {
                return new vec4(this.v3, this.v3, this.v2, this.v3);
            }
        }
        
        public vec4 qqpq {
            get {
                return new vec4(this.v3, this.v3, this.v2, this.v3);
            }
        }
        
        public vec4 wwwx {
            get {
                return new vec4(this.v3, this.v3, this.v3, this.v0);
            }
        }
        
        public vec4 aaar {
            get {
                return new vec4(this.v3, this.v3, this.v3, this.v0);
            }
        }
        
        public vec4 qqqs {
            get {
                return new vec4(this.v3, this.v3, this.v3, this.v0);
            }
        }
        
        public vec4 wwwy {
            get {
                return new vec4(this.v3, this.v3, this.v3, this.v1);
            }
        }
        
        public vec4 aaag {
            get {
                return new vec4(this.v3, this.v3, this.v3, this.v1);
            }
        }
        
        public vec4 qqqt {
            get {
                return new vec4(this.v3, this.v3, this.v3, this.v1);
            }
        }
        
        public vec4 wwwz {
            get {
                return new vec4(this.v3, this.v3, this.v3, this.v2);
            }
        }
        
        public vec4 aaab {
            get {
                return new vec4(this.v3, this.v3, this.v3, this.v2);
            }
        }
        
        public vec4 qqqp {
            get {
                return new vec4(this.v3, this.v3, this.v3, this.v2);
            }
        }
        
        public vec4 wwww {
            get {
                return new vec4(this.v3, this.v3, this.v3, this.v3);
            }
        }
        
        public vec4 aaaa {
            get {
                return new vec4(this.v3, this.v3, this.v3, this.v3);
            }
        }
        
        public vec4 qqqq {
            get {
                return new vec4(this.v3, this.v3, this.v3, this.v3);
            }
        }
        
        public static vec4 operator+(vec4 v) {
            return new vec4(+v.v0, +v.v1, +v.v2, +v.v3);
        }
        
        public static vec4 operator-(vec4 v) {
            return new vec4(-v.v0, -v.v1, -v.v2, -v.v3);
        }
        
        public static vec4 operator++(vec4 v) {
            return new vec4(++v.v0, ++v.v1, ++v.v2, ++v.v3);
        }
        
        public static vec4 operator--(vec4 v) {
            return new vec4(--v.v0, --v.v1, --v.v2, --v.v3);
        }
        
        public static vec4 operator+(float s, vec4 v2) {
            return new vec4(s + v2.v0, s + v2.v1, s + v2.v2, s + v2.v3);
        }
        
        public static vec4 operator-(float s, vec4 v2) {
            return new vec4(s - v2.v0, s - v2.v1, s - v2.v2, s - v2.v3);
        }
        
        public static vec4 operator*(float s, vec4 v2) {
            return new vec4(s * v2.v0, s * v2.v1, s * v2.v2, s * v2.v3);
        }
        
        public static vec4 operator/(float s, vec4 v2) {
            return new vec4(s / v2.v0, s / v2.v1, s / v2.v2, s / v2.v3);
        }
        
        public static vec4 operator%(float s, vec4 v2) {
            return new vec4(s % v2.v0, s % v2.v1, s % v2.v2, s % v2.v3);
        }
        
        public static vec4 operator+(vec4 v1, float s) {
            return new vec4(v1.v0 + s, v1.v1 + s, v1.v2 + s, v1.v3 + s);
        }
        
        public static vec4 operator-(vec4 v1, float s) {
            return new vec4(v1.v0 - s, v1.v1 - s, v1.v2 - s, v1.v3 - s);
        }
        
        public static vec4 operator*(vec4 v1, float s) {
            return new vec4(v1.v0 * s, v1.v1 * s, v1.v2 * s, v1.v3 * s);
        }
        
        public static vec4 operator/(vec4 v1, float s) {
            return new vec4(v1.v0 / s, v1.v1 / s, v1.v2 / s, v1.v3 / s);
        }
        
        public static vec4 operator%(vec4 v1, float s) {
            return new vec4(v1.v0 % s, v1.v1 % s, v1.v2 % s, v1.v3 % s);
        }
        
        public static vec4 operator+(vec4 v1, vec4 v2) {
            return new vec4(v1.v0 + v2.v0, v1.v1 + v2.v1, v1.v2 + v2.v2, v1.v3 + v2.v3);
        }
        
        public static vec4 operator-(vec4 v1, vec4 v2) {
            return new vec4(v1.v0 - v2.v0, v1.v1 - v2.v1, v1.v2 - v2.v2, v1.v3 - v2.v3);
        }
        
        public static vec4 operator*(vec4 v1, vec4 v2) {
            return new vec4(v1.v0 * v2.v0, v1.v1 * v2.v1, v1.v2 * v2.v2, v1.v3 * v2.v3);
        }
        
        public static vec4 operator/(vec4 v1, vec4 v2) {
            return new vec4(v1.v0 / v2.v0, v1.v1 / v2.v1, v1.v2 / v2.v2, v1.v3 / v2.v3);
        }
        
        public static vec4 operator%(vec4 v1, vec4 v2) {
            return new vec4(v1.v0 % v2.v0, v1.v1 % v2.v1, v1.v2 % v2.v2, v1.v3 % v2.v3);
        }
        
        public static bvec4 operator==(vec4 v1, vec4 v2) {
            return new bvec4(v1.v0 == v2.v0, v1.v1 == v2.v1, v1.v2 == v2.v2, v1.v3 == v2.v3);
        }
        
        public static bvec4 operator>(vec4 v1, vec4 v2) {
            return new bvec4(v1.v0 > v2.v0, v1.v1 > v2.v1, v1.v2 > v2.v2, v1.v3 > v2.v3);
        }
        
        public static bvec4 operator>=(vec4 v1, vec4 v2) {
            return new bvec4(v1.v0 >= v2.v0, v1.v1 >= v2.v1, v1.v2 >= v2.v2, v1.v3 >= v2.v3);
        }
        
        public static bvec4 operator<(vec4 v1, vec4 v2) {
            return new bvec4(v1.v0 < v2.v0, v1.v1 < v2.v1, v1.v2 < v2.v2, v1.v3 < v2.v3);
        }
        
        public static bvec4 operator<=(vec4 v1, vec4 v2) {
            return new bvec4(v1.v0 <= v2.v0, v1.v1 <= v2.v1, v1.v2 <= v2.v2, v1.v3 <= v2.v3);
        }
        
        public static bvec4 operator!=(vec4 v1, vec4 v2) {
            return new bvec4(v1.v0 != v2.v0, v1.v1 != v2.v1, v1.v2 != v2.v2, v1.v3 != v2.v3);
        }
        
        public override bool Equals(object obj) {
            return (obj is vec4) ? bmath.All(this == (vec4)obj) : false;
        }
        
        public override int GetHashCode() {
            return this.v0.GetHashCode() ^ this.v1.GetHashCode() ^ this.v2.GetHashCode() ^ this.v3.GetHashCode();
        }
        
        public override string ToString() {
            return "vec4(" + this.v0 + "," + this.v1 + "," + this.v2 + "," + this.v3 + ")";
        }
    }
}
